/*
 * Copyright 2006-2008 The FLWOR Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include "stdafx.h"

#include "util/unicode_categories.h"

namespace zorba {
namespace unicode {

  
//Unicode codepoint categories, as from http://www.fileformat.info/info/unicode/category/index.htm

static const unsigned char codepoints_categories[] = {
UNICODE_Cc, //U+0000
UNICODE_Cc, //U+0001
UNICODE_Cc, //U+0002
UNICODE_Cc, //U+0003
UNICODE_Cc, //U+0004
UNICODE_Cc, //U+0005
UNICODE_Cc, //U+0006
UNICODE_Cc, //U+0007
UNICODE_Cc, //U+0008
UNICODE_Cc, //U+0009
UNICODE_Cc, //U+000A
UNICODE_Cc, //U+000B
UNICODE_Cc, //U+000C
UNICODE_Cc, //U+000D
UNICODE_Cc, //U+000E
UNICODE_Cc, //U+000F
UNICODE_Cc, //U+0010
UNICODE_Cc, //U+0011
UNICODE_Cc, //U+0012
UNICODE_Cc, //U+0013
UNICODE_Cc, //U+0014
UNICODE_Cc, //U+0015
UNICODE_Cc, //U+0016
UNICODE_Cc, //U+0017
UNICODE_Cc, //U+0018
UNICODE_Cc, //U+0019
UNICODE_Cc, //U+001A
UNICODE_Cc, //U+001B
UNICODE_Cc, //U+001C
UNICODE_Cc, //U+001D
UNICODE_Cc, //U+001E
UNICODE_Cc, //U+001F
UNICODE_Zs, //U+0020
UNICODE_Po, //U+0021
UNICODE_Po, //U+0022
UNICODE_Po, //U+0023
UNICODE_Sc, //U+0024
UNICODE_Po, //U+0025
UNICODE_Po, //U+0026
UNICODE_Po, //U+0027
UNICODE_Ps, //U+0028
UNICODE_Pe, //U+0029
UNICODE_Po, //U+002A
UNICODE_Sm, //U+002B
UNICODE_Po, //U+002C
UNICODE_Pd, //U+002D
UNICODE_Po, //U+002E
UNICODE_Po, //U+002F
UNICODE_Nd, //U+0030
UNICODE_Nd, //U+0031
UNICODE_Nd, //U+0032
UNICODE_Nd, //U+0033
UNICODE_Nd, //U+0034
UNICODE_Nd, //U+0035
UNICODE_Nd, //U+0036
UNICODE_Nd, //U+0037
UNICODE_Nd, //U+0038
UNICODE_Nd, //U+0039
UNICODE_Po, //U+003A
UNICODE_Po, //U+003B
UNICODE_Sm, //U+003C
UNICODE_Sm, //U+003D
UNICODE_Sm, //U+003E
UNICODE_Po, //U+003F
UNICODE_Po, //U+0040
UNICODE_Lu, //U+0041
UNICODE_Lu, //U+0042
UNICODE_Lu, //U+0043
UNICODE_Lu, //U+0044
UNICODE_Lu, //U+0045
UNICODE_Lu, //U+0046
UNICODE_Lu, //U+0047
UNICODE_Lu, //U+0048
UNICODE_Lu, //U+0049
UNICODE_Lu, //U+004A
UNICODE_Lu, //U+004B
UNICODE_Lu, //U+004C
UNICODE_Lu, //U+004D
UNICODE_Lu, //U+004E
UNICODE_Lu, //U+004F
UNICODE_Lu, //U+0050
UNICODE_Lu, //U+0051
UNICODE_Lu, //U+0052
UNICODE_Lu, //U+0053
UNICODE_Lu, //U+0054
UNICODE_Lu, //U+0055
UNICODE_Lu, //U+0056
UNICODE_Lu, //U+0057
UNICODE_Lu, //U+0058
UNICODE_Lu, //U+0059
UNICODE_Lu, //U+005A
UNICODE_Ps, //U+005B
UNICODE_Po, //U+005C
UNICODE_Pe, //U+005D
UNICODE_Sk, //U+005E
UNICODE_Pc, //U+005F
UNICODE_Sk, //U+0060
UNICODE_Ll, //U+0061
UNICODE_Ll, //U+0062
UNICODE_Ll, //U+0063
UNICODE_Ll, //U+0064
UNICODE_Ll, //U+0065
UNICODE_Ll, //U+0066
UNICODE_Ll, //U+0067
UNICODE_Ll, //U+0068
UNICODE_Ll, //U+0069
UNICODE_Ll, //U+006A
UNICODE_Ll, //U+006B
UNICODE_Ll, //U+006C
UNICODE_Ll, //U+006D
UNICODE_Ll, //U+006E
UNICODE_Ll, //U+006F
UNICODE_Ll, //U+0070
UNICODE_Ll, //U+0071
UNICODE_Ll, //U+0072
UNICODE_Ll, //U+0073
UNICODE_Ll, //U+0074
UNICODE_Ll, //U+0075
UNICODE_Ll, //U+0076
UNICODE_Ll, //U+0077
UNICODE_Ll, //U+0078
UNICODE_Ll, //U+0079
UNICODE_Ll, //U+007A
UNICODE_Ps, //U+007B
UNICODE_Sm, //U+007C
UNICODE_Pe, //U+007D
UNICODE_Sm, //U+007E
UNICODE_Cc, //U+007F
UNICODE_Cc, //U+0080
UNICODE_Cc, //U+0081
UNICODE_Cc, //U+0082
UNICODE_Cc, //U+0083
UNICODE_Cc, //U+0084
UNICODE_Cc, //U+0085
UNICODE_Cc, //U+0086
UNICODE_Cc, //U+0087
UNICODE_Cc, //U+0088
UNICODE_Cc, //U+0089
UNICODE_Cc, //U+008A
UNICODE_Cc, //U+008B
UNICODE_Cc, //U+008C
UNICODE_Cc, //U+008D
UNICODE_Cc, //U+008E
UNICODE_Cc, //U+008F
UNICODE_Cc, //U+0090
UNICODE_Cc, //U+0091
UNICODE_Cc, //U+0092
UNICODE_Cc, //U+0093
UNICODE_Cc, //U+0094
UNICODE_Cc, //U+0095
UNICODE_Cc, //U+0096
UNICODE_Cc, //U+0097
UNICODE_Cc, //U+0098
UNICODE_Cc, //U+0099
UNICODE_Cc, //U+009A
UNICODE_Cc, //U+009B
UNICODE_Cc, //U+009C
UNICODE_Cc, //U+009D
UNICODE_Cc, //U+009E
UNICODE_Cc, //U+009F
UNICODE_Zs, //U+00A0
UNICODE_Po, //U+00A1
UNICODE_Sc, //U+00A2
UNICODE_Sc, //U+00A3
UNICODE_Sc, //U+00A4
UNICODE_Sc, //U+00A5
UNICODE_So, //U+00A6
UNICODE_So, //U+00A7
UNICODE_Sk, //U+00A8
UNICODE_So, //U+00A9
UNICODE_Ll, //U+00AA
UNICODE_Pi, //U+00AB
UNICODE_Sm, //U+00AC
UNICODE_Cf, //U+00AD
UNICODE_So, //U+00AE
UNICODE_Sk, //U+00AF
UNICODE_So, //U+00B0
UNICODE_Sm, //U+00B1
UNICODE_No, //U+00B2
UNICODE_No, //U+00B3
UNICODE_Sk, //U+00B4
UNICODE_Ll, //U+00B5
UNICODE_So, //U+00B6
UNICODE_Po, //U+00B7
UNICODE_Sk, //U+00B8
UNICODE_No, //U+00B9
UNICODE_Ll, //U+00BA
UNICODE_Pf, //U+00BB
UNICODE_No, //U+00BC
UNICODE_No, //U+00BD
UNICODE_No, //U+00BE
UNICODE_Po, //U+00BF
UNICODE_Lu, //U+00C0
UNICODE_Lu, //U+00C1
UNICODE_Lu, //U+00C2
UNICODE_Lu, //U+00C3
UNICODE_Lu, //U+00C4
UNICODE_Lu, //U+00C5
UNICODE_Lu, //U+00C6
UNICODE_Lu, //U+00C7
UNICODE_Lu, //U+00C8
UNICODE_Lu, //U+00C9
UNICODE_Lu, //U+00CA
UNICODE_Lu, //U+00CB
UNICODE_Lu, //U+00CC
UNICODE_Lu, //U+00CD
UNICODE_Lu, //U+00CE
UNICODE_Lu, //U+00CF
UNICODE_Lu, //U+00D0
UNICODE_Lu, //U+00D1
UNICODE_Lu, //U+00D2
UNICODE_Lu, //U+00D3
UNICODE_Lu, //U+00D4
UNICODE_Lu, //U+00D5
UNICODE_Lu, //U+00D6
UNICODE_Sm, //U+00D7
UNICODE_Lu, //U+00D8
UNICODE_Lu, //U+00D9
UNICODE_Lu, //U+00DA
UNICODE_Lu, //U+00DB
UNICODE_Lu, //U+00DC
UNICODE_Lu, //U+00DD
UNICODE_Lu, //U+00DE
UNICODE_Ll, //U+00DF
UNICODE_Ll, //U+00E0
UNICODE_Ll, //U+00E1
UNICODE_Ll, //U+00E2
UNICODE_Ll, //U+00E3
UNICODE_Ll, //U+00E4
UNICODE_Ll, //U+00E5
UNICODE_Ll, //U+00E6
UNICODE_Ll, //U+00E7
UNICODE_Ll, //U+00E8
UNICODE_Ll, //U+00E9
UNICODE_Ll, //U+00EA
UNICODE_Ll, //U+00EB
UNICODE_Ll, //U+00EC
UNICODE_Ll, //U+00ED
UNICODE_Ll, //U+00EE
UNICODE_Ll, //U+00EF
UNICODE_Ll, //U+00F0
UNICODE_Ll, //U+00F1
UNICODE_Ll, //U+00F2
UNICODE_Ll, //U+00F3
UNICODE_Ll, //U+00F4
UNICODE_Ll, //U+00F5
UNICODE_Ll, //U+00F6
UNICODE_Sm, //U+00F7
UNICODE_Ll, //U+00F8
UNICODE_Ll, //U+00F9
UNICODE_Ll, //U+00FA
UNICODE_Ll, //U+00FB
UNICODE_Ll, //U+00FC
UNICODE_Ll, //U+00FD
UNICODE_Ll, //U+00FE
UNICODE_Ll, //U+00FF
UNICODE_Lu, //U+0100
UNICODE_Ll, //U+0101
UNICODE_Lu, //U+0102
UNICODE_Ll, //U+0103
UNICODE_Lu, //U+0104
UNICODE_Ll, //U+0105
UNICODE_Lu, //U+0106
UNICODE_Ll, //U+0107
UNICODE_Lu, //U+0108
UNICODE_Ll, //U+0109
UNICODE_Lu, //U+010A
UNICODE_Ll, //U+010B
UNICODE_Lu, //U+010C
UNICODE_Ll, //U+010D
UNICODE_Lu, //U+010E
UNICODE_Ll, //U+010F
UNICODE_Lu, //U+0110
UNICODE_Ll, //U+0111
UNICODE_Lu, //U+0112
UNICODE_Ll, //U+0113
UNICODE_Lu, //U+0114
UNICODE_Ll, //U+0115
UNICODE_Lu, //U+0116
UNICODE_Ll, //U+0117
UNICODE_Lu, //U+0118
UNICODE_Ll, //U+0119
UNICODE_Lu, //U+011A
UNICODE_Ll, //U+011B
UNICODE_Lu, //U+011C
UNICODE_Ll, //U+011D
UNICODE_Lu, //U+011E
UNICODE_Ll, //U+011F
UNICODE_Lu, //U+0120
UNICODE_Ll, //U+0121
UNICODE_Lu, //U+0122
UNICODE_Ll, //U+0123
UNICODE_Lu, //U+0124
UNICODE_Ll, //U+0125
UNICODE_Lu, //U+0126
UNICODE_Ll, //U+0127
UNICODE_Lu, //U+0128
UNICODE_Ll, //U+0129
UNICODE_Lu, //U+012A
UNICODE_Ll, //U+012B
UNICODE_Lu, //U+012C
UNICODE_Ll, //U+012D
UNICODE_Lu, //U+012E
UNICODE_Ll, //U+012F
UNICODE_Lu, //U+0130
UNICODE_Ll, //U+0131
UNICODE_Lu, //U+0132
UNICODE_Ll, //U+0133
UNICODE_Lu, //U+0134
UNICODE_Ll, //U+0135
UNICODE_Lu, //U+0136
UNICODE_Ll, //U+0137
UNICODE_Ll, //U+0138
UNICODE_Lu, //U+0139
UNICODE_Ll, //U+013A
UNICODE_Lu, //U+013B
UNICODE_Ll, //U+013C
UNICODE_Lu, //U+013D
UNICODE_Ll, //U+013E
UNICODE_Lu, //U+013F
UNICODE_Ll, //U+0140
UNICODE_Lu, //U+0141
UNICODE_Ll, //U+0142
UNICODE_Lu, //U+0143
UNICODE_Ll, //U+0144
UNICODE_Lu, //U+0145
UNICODE_Ll, //U+0146
UNICODE_Lu, //U+0147
UNICODE_Ll, //U+0148
UNICODE_Ll, //U+0149
UNICODE_Lu, //U+014A
UNICODE_Ll, //U+014B
UNICODE_Lu, //U+014C
UNICODE_Ll, //U+014D
UNICODE_Lu, //U+014E
UNICODE_Ll, //U+014F
UNICODE_Lu, //U+0150
UNICODE_Ll, //U+0151
UNICODE_Lu, //U+0152
UNICODE_Ll, //U+0153
UNICODE_Lu, //U+0154
UNICODE_Ll, //U+0155
UNICODE_Lu, //U+0156
UNICODE_Ll, //U+0157
UNICODE_Lu, //U+0158
UNICODE_Ll, //U+0159
UNICODE_Lu, //U+015A
UNICODE_Ll, //U+015B
UNICODE_Lu, //U+015C
UNICODE_Ll, //U+015D
UNICODE_Lu, //U+015E
UNICODE_Ll, //U+015F
UNICODE_Lu, //U+0160
UNICODE_Ll, //U+0161
UNICODE_Lu, //U+0162
UNICODE_Ll, //U+0163
UNICODE_Lu, //U+0164
UNICODE_Ll, //U+0165
UNICODE_Lu, //U+0166
UNICODE_Ll, //U+0167
UNICODE_Lu, //U+0168
UNICODE_Ll, //U+0169
UNICODE_Lu, //U+016A
UNICODE_Ll, //U+016B
UNICODE_Lu, //U+016C
UNICODE_Ll, //U+016D
UNICODE_Lu, //U+016E
UNICODE_Ll, //U+016F
UNICODE_Lu, //U+0170
UNICODE_Ll, //U+0171
UNICODE_Lu, //U+0172
UNICODE_Ll, //U+0173
UNICODE_Lu, //U+0174
UNICODE_Ll, //U+0175
UNICODE_Lu, //U+0176
UNICODE_Ll, //U+0177
UNICODE_Lu, //U+0178
UNICODE_Lu, //U+0179
UNICODE_Ll, //U+017A
UNICODE_Lu, //U+017B
UNICODE_Ll, //U+017C
UNICODE_Lu, //U+017D
UNICODE_Ll, //U+017E
UNICODE_Ll, //U+017F
UNICODE_Ll, //U+0180
UNICODE_Lu, //U+0181
UNICODE_Lu, //U+0182
UNICODE_Ll, //U+0183
UNICODE_Lu, //U+0184
UNICODE_Ll, //U+0185
UNICODE_Lu, //U+0186
UNICODE_Lu, //U+0187
UNICODE_Ll, //U+0188
UNICODE_Lu, //U+0189
UNICODE_Lu, //U+018A
UNICODE_Lu, //U+018B
UNICODE_Ll, //U+018C
UNICODE_Ll, //U+018D
UNICODE_Lu, //U+018E
UNICODE_Lu, //U+018F
UNICODE_Lu, //U+0190
UNICODE_Lu, //U+0191
UNICODE_Ll, //U+0192
UNICODE_Lu, //U+0193
UNICODE_Lu, //U+0194
UNICODE_Ll, //U+0195
UNICODE_Lu, //U+0196
UNICODE_Lu, //U+0197
UNICODE_Lu, //U+0198
UNICODE_Ll, //U+0199
UNICODE_Ll, //U+019A
UNICODE_Ll, //U+019B
UNICODE_Lu, //U+019C
UNICODE_Lu, //U+019D
UNICODE_Ll, //U+019E
UNICODE_Lu, //U+019F
UNICODE_Lu, //U+01A0
UNICODE_Ll, //U+01A1
UNICODE_Lu, //U+01A2
UNICODE_Ll, //U+01A3
UNICODE_Lu, //U+01A4
UNICODE_Ll, //U+01A5
UNICODE_Lu, //U+01A6
UNICODE_Lu, //U+01A7
UNICODE_Ll, //U+01A8
UNICODE_Lu, //U+01A9
UNICODE_Ll, //U+01AA
UNICODE_Ll, //U+01AB
UNICODE_Lu, //U+01AC
UNICODE_Ll, //U+01AD
UNICODE_Lu, //U+01AE
UNICODE_Lu, //U+01AF
UNICODE_Ll, //U+01B0
UNICODE_Lu, //U+01B1
UNICODE_Lu, //U+01B2
UNICODE_Lu, //U+01B3
UNICODE_Ll, //U+01B4
UNICODE_Lu, //U+01B5
UNICODE_Ll, //U+01B6
UNICODE_Lu, //U+01B7
UNICODE_Lu, //U+01B8
UNICODE_Ll, //U+01B9
UNICODE_Ll, //U+01BA
UNICODE_Lo, //U+01BB
UNICODE_Lu, //U+01BC
UNICODE_Ll, //U+01BD
UNICODE_Ll, //U+01BE
UNICODE_Ll, //U+01BF
UNICODE_Lo, //U+01C0
UNICODE_Lo, //U+01C1
UNICODE_Lo, //U+01C2
UNICODE_Lo, //U+01C3
UNICODE_Lu, //U+01C4
UNICODE_Lt, //U+01C5
UNICODE_Ll, //U+01C6
UNICODE_Lu, //U+01C7
UNICODE_Lt, //U+01C8
UNICODE_Ll, //U+01C9
UNICODE_Lu, //U+01CA
UNICODE_Lt, //U+01CB
UNICODE_Ll, //U+01CC
UNICODE_Lu, //U+01CD
UNICODE_Ll, //U+01CE
UNICODE_Lu, //U+01CF
UNICODE_Ll, //U+01D0
UNICODE_Lu, //U+01D1
UNICODE_Ll, //U+01D2
UNICODE_Lu, //U+01D3
UNICODE_Ll, //U+01D4
UNICODE_Lu, //U+01D5
UNICODE_Ll, //U+01D6
UNICODE_Lu, //U+01D7
UNICODE_Ll, //U+01D8
UNICODE_Lu, //U+01D9
UNICODE_Ll, //U+01DA
UNICODE_Lu, //U+01DB
UNICODE_Ll, //U+01DC
UNICODE_Ll, //U+01DD
UNICODE_Lu, //U+01DE
UNICODE_Ll, //U+01DF
UNICODE_Lu, //U+01E0
UNICODE_Ll, //U+01E1
UNICODE_Lu, //U+01E2
UNICODE_Ll, //U+01E3
UNICODE_Lu, //U+01E4
UNICODE_Ll, //U+01E5
UNICODE_Lu, //U+01E6
UNICODE_Ll, //U+01E7
UNICODE_Lu, //U+01E8
UNICODE_Ll, //U+01E9
UNICODE_Lu, //U+01EA
UNICODE_Ll, //U+01EB
UNICODE_Lu, //U+01EC
UNICODE_Ll, //U+01ED
UNICODE_Lu, //U+01EE
UNICODE_Ll, //U+01EF
UNICODE_Ll, //U+01F0
UNICODE_Lu, //U+01F1
UNICODE_Lt, //U+01F2
UNICODE_Ll, //U+01F3
UNICODE_Lu, //U+01F4
UNICODE_Ll, //U+01F5
UNICODE_Lu, //U+01F6
UNICODE_Lu, //U+01F7
UNICODE_Lu, //U+01F8
UNICODE_Ll, //U+01F9
UNICODE_Lu, //U+01FA
UNICODE_Ll, //U+01FB
UNICODE_Lu, //U+01FC
UNICODE_Ll, //U+01FD
UNICODE_Lu, //U+01FE
UNICODE_Ll, //U+01FF
UNICODE_Lu, //U+0200
UNICODE_Ll, //U+0201
UNICODE_Lu, //U+0202
UNICODE_Ll, //U+0203
UNICODE_Lu, //U+0204
UNICODE_Ll, //U+0205
UNICODE_Lu, //U+0206
UNICODE_Ll, //U+0207
UNICODE_Lu, //U+0208
UNICODE_Ll, //U+0209
UNICODE_Lu, //U+020A
UNICODE_Ll, //U+020B
UNICODE_Lu, //U+020C
UNICODE_Ll, //U+020D
UNICODE_Lu, //U+020E
UNICODE_Ll, //U+020F
UNICODE_Lu, //U+0210
UNICODE_Ll, //U+0211
UNICODE_Lu, //U+0212
UNICODE_Ll, //U+0213
UNICODE_Lu, //U+0214
UNICODE_Ll, //U+0215
UNICODE_Lu, //U+0216
UNICODE_Ll, //U+0217
UNICODE_Lu, //U+0218
UNICODE_Ll, //U+0219
UNICODE_Lu, //U+021A
UNICODE_Ll, //U+021B
UNICODE_Lu, //U+021C
UNICODE_Ll, //U+021D
UNICODE_Lu, //U+021E
UNICODE_Ll, //U+021F
UNICODE_Lu, //U+0220
UNICODE_Ll, //U+0221
UNICODE_Lu, //U+0222
UNICODE_Ll, //U+0223
UNICODE_Lu, //U+0224
UNICODE_Ll, //U+0225
UNICODE_Lu, //U+0226
UNICODE_Ll, //U+0227
UNICODE_Lu, //U+0228
UNICODE_Ll, //U+0229
UNICODE_Lu, //U+022A
UNICODE_Ll, //U+022B
UNICODE_Lu, //U+022C
UNICODE_Ll, //U+022D
UNICODE_Lu, //U+022E
UNICODE_Ll, //U+022F
UNICODE_Lu, //U+0230
UNICODE_Ll, //U+0231
UNICODE_Lu, //U+0232
UNICODE_Ll, //U+0233
UNICODE_Ll, //U+0234
UNICODE_Ll, //U+0235
UNICODE_Ll, //U+0236
UNICODE_Ll, //U+0237
UNICODE_Ll, //U+0238
UNICODE_Ll, //U+0239
UNICODE_Lu, //U+023A
UNICODE_Lu, //U+023B
UNICODE_Ll, //U+023C
UNICODE_Lu, //U+023D
UNICODE_Lu, //U+023E
UNICODE_Ll, //U+023F
UNICODE_Ll, //U+0240
UNICODE_Lu, //U+0241
UNICODE_Ll, //U+0242
UNICODE_Lu, //U+0243
UNICODE_Lu, //U+0244
UNICODE_Lu, //U+0245
UNICODE_Lu, //U+0246
UNICODE_Ll, //U+0247
UNICODE_Lu, //U+0248
UNICODE_Ll, //U+0249
UNICODE_Lu, //U+024A
UNICODE_Ll, //U+024B
UNICODE_Lu, //U+024C
UNICODE_Ll, //U+024D
UNICODE_Lu, //U+024E
UNICODE_Ll, //U+024F
UNICODE_Ll, //U+0250
UNICODE_Ll, //U+0251
UNICODE_Ll, //U+0252
UNICODE_Ll, //U+0253
UNICODE_Ll, //U+0254
UNICODE_Ll, //U+0255
UNICODE_Ll, //U+0256
UNICODE_Ll, //U+0257
UNICODE_Ll, //U+0258
UNICODE_Ll, //U+0259
UNICODE_Ll, //U+025A
UNICODE_Ll, //U+025B
UNICODE_Ll, //U+025C
UNICODE_Ll, //U+025D
UNICODE_Ll, //U+025E
UNICODE_Ll, //U+025F
UNICODE_Ll, //U+0260
UNICODE_Ll, //U+0261
UNICODE_Ll, //U+0262
UNICODE_Ll, //U+0263
UNICODE_Ll, //U+0264
UNICODE_Ll, //U+0265
UNICODE_Ll, //U+0266
UNICODE_Ll, //U+0267
UNICODE_Ll, //U+0268
UNICODE_Ll, //U+0269
UNICODE_Ll, //U+026A
UNICODE_Ll, //U+026B
UNICODE_Ll, //U+026C
UNICODE_Ll, //U+026D
UNICODE_Ll, //U+026E
UNICODE_Ll, //U+026F
UNICODE_Ll, //U+0270
UNICODE_Ll, //U+0271
UNICODE_Ll, //U+0272
UNICODE_Ll, //U+0273
UNICODE_Ll, //U+0274
UNICODE_Ll, //U+0275
UNICODE_Ll, //U+0276
UNICODE_Ll, //U+0277
UNICODE_Ll, //U+0278
UNICODE_Ll, //U+0279
UNICODE_Ll, //U+027A
UNICODE_Ll, //U+027B
UNICODE_Ll, //U+027C
UNICODE_Ll, //U+027D
UNICODE_Ll, //U+027E
UNICODE_Ll, //U+027F
UNICODE_Ll, //U+0280
UNICODE_Ll, //U+0281
UNICODE_Ll, //U+0282
UNICODE_Ll, //U+0283
UNICODE_Ll, //U+0284
UNICODE_Ll, //U+0285
UNICODE_Ll, //U+0286
UNICODE_Ll, //U+0287
UNICODE_Ll, //U+0288
UNICODE_Ll, //U+0289
UNICODE_Ll, //U+028A
UNICODE_Ll, //U+028B
UNICODE_Ll, //U+028C
UNICODE_Ll, //U+028D
UNICODE_Ll, //U+028E
UNICODE_Ll, //U+028F
UNICODE_Ll, //U+0290
UNICODE_Ll, //U+0291
UNICODE_Ll, //U+0292
UNICODE_Ll, //U+0293
UNICODE_Lo, //U+0294
UNICODE_Ll, //U+0295
UNICODE_Ll, //U+0296
UNICODE_Ll, //U+0297
UNICODE_Ll, //U+0298
UNICODE_Ll, //U+0299
UNICODE_Ll, //U+029A
UNICODE_Ll, //U+029B
UNICODE_Ll, //U+029C
UNICODE_Ll, //U+029D
UNICODE_Ll, //U+029E
UNICODE_Ll, //U+029F
UNICODE_Ll, //U+02A0
UNICODE_Ll, //U+02A1
UNICODE_Ll, //U+02A2
UNICODE_Ll, //U+02A3
UNICODE_Ll, //U+02A4
UNICODE_Ll, //U+02A5
UNICODE_Ll, //U+02A6
UNICODE_Ll, //U+02A7
UNICODE_Ll, //U+02A8
UNICODE_Ll, //U+02A9
UNICODE_Ll, //U+02AA
UNICODE_Ll, //U+02AB
UNICODE_Ll, //U+02AC
UNICODE_Ll, //U+02AD
UNICODE_Ll, //U+02AE
UNICODE_Ll, //U+02AF
UNICODE_Lm, //U+02B0
UNICODE_Lm, //U+02B1
UNICODE_Lm, //U+02B2
UNICODE_Lm, //U+02B3
UNICODE_Lm, //U+02B4
UNICODE_Lm, //U+02B5
UNICODE_Lm, //U+02B6
UNICODE_Lm, //U+02B7
UNICODE_Lm, //U+02B8
UNICODE_Lm, //U+02B9
UNICODE_Lm, //U+02BA
UNICODE_Lm, //U+02BB
UNICODE_Lm, //U+02BC
UNICODE_Lm, //U+02BD
UNICODE_Lm, //U+02BE
UNICODE_Lm, //U+02BF
UNICODE_Lm, //U+02C0
UNICODE_Lm, //U+02C1
UNICODE_Sk, //U+02C2
UNICODE_Sk, //U+02C3
UNICODE_Sk, //U+02C4
UNICODE_Sk, //U+02C5
UNICODE_Lm, //U+02C6
UNICODE_Lm, //U+02C7
UNICODE_Lm, //U+02C8
UNICODE_Lm, //U+02C9
UNICODE_Lm, //U+02CA
UNICODE_Lm, //U+02CB
UNICODE_Lm, //U+02CC
UNICODE_Lm, //U+02CD
UNICODE_Lm, //U+02CE
UNICODE_Lm, //U+02CF
UNICODE_Lm, //U+02D0
UNICODE_Lm, //U+02D1
UNICODE_Sk, //U+02D2
UNICODE_Sk, //U+02D3
UNICODE_Sk, //U+02D4
UNICODE_Sk, //U+02D5
UNICODE_Sk, //U+02D6
UNICODE_Sk, //U+02D7
UNICODE_Sk, //U+02D8
UNICODE_Sk, //U+02D9
UNICODE_Sk, //U+02DA
UNICODE_Sk, //U+02DB
UNICODE_Sk, //U+02DC
UNICODE_Sk, //U+02DD
UNICODE_Sk, //U+02DE
UNICODE_Sk, //U+02DF
UNICODE_Lm, //U+02E0
UNICODE_Lm, //U+02E1
UNICODE_Lm, //U+02E2
UNICODE_Lm, //U+02E3
UNICODE_Lm, //U+02E4
UNICODE_Sk, //U+02E5
UNICODE_Sk, //U+02E6
UNICODE_Sk, //U+02E7
UNICODE_Sk, //U+02E8
UNICODE_Sk, //U+02E9
UNICODE_Sk, //U+02EA
UNICODE_Sk, //U+02EB
UNICODE_Lm, //U+02EC
UNICODE_Sk, //U+02ED
UNICODE_Lm, //U+02EE
UNICODE_Sk, //U+02EF
UNICODE_Sk, //U+02F0
UNICODE_Sk, //U+02F1
UNICODE_Sk, //U+02F2
UNICODE_Sk, //U+02F3
UNICODE_Sk, //U+02F4
UNICODE_Sk, //U+02F5
UNICODE_Sk, //U+02F6
UNICODE_Sk, //U+02F7
UNICODE_Sk, //U+02F8
UNICODE_Sk, //U+02F9
UNICODE_Sk, //U+02FA
UNICODE_Sk, //U+02FB
UNICODE_Sk, //U+02FC
UNICODE_Sk, //U+02FD
UNICODE_Sk, //U+02FE
UNICODE_Sk, //U+02FF
UNICODE_Mn, //U+0300
UNICODE_Mn, //U+0301
UNICODE_Mn, //U+0302
UNICODE_Mn, //U+0303
UNICODE_Mn, //U+0304
UNICODE_Mn, //U+0305
UNICODE_Mn, //U+0306
UNICODE_Mn, //U+0307
UNICODE_Mn, //U+0308
UNICODE_Mn, //U+0309
UNICODE_Mn, //U+030A
UNICODE_Mn, //U+030B
UNICODE_Mn, //U+030C
UNICODE_Mn, //U+030D
UNICODE_Mn, //U+030E
UNICODE_Mn, //U+030F
UNICODE_Mn, //U+0310
UNICODE_Mn, //U+0311
UNICODE_Mn, //U+0312
UNICODE_Mn, //U+0313
UNICODE_Mn, //U+0314
UNICODE_Mn, //U+0315
UNICODE_Mn, //U+0316
UNICODE_Mn, //U+0317
UNICODE_Mn, //U+0318
UNICODE_Mn, //U+0319
UNICODE_Mn, //U+031A
UNICODE_Mn, //U+031B
UNICODE_Mn, //U+031C
UNICODE_Mn, //U+031D
UNICODE_Mn, //U+031E
UNICODE_Mn, //U+031F
UNICODE_Mn, //U+0320
UNICODE_Mn, //U+0321
UNICODE_Mn, //U+0322
UNICODE_Mn, //U+0323
UNICODE_Mn, //U+0324
UNICODE_Mn, //U+0325
UNICODE_Mn, //U+0326
UNICODE_Mn, //U+0327
UNICODE_Mn, //U+0328
UNICODE_Mn, //U+0329
UNICODE_Mn, //U+032A
UNICODE_Mn, //U+032B
UNICODE_Mn, //U+032C
UNICODE_Mn, //U+032D
UNICODE_Mn, //U+032E
UNICODE_Mn, //U+032F
UNICODE_Mn, //U+0330
UNICODE_Mn, //U+0331
UNICODE_Mn, //U+0332
UNICODE_Mn, //U+0333
UNICODE_Mn, //U+0334
UNICODE_Mn, //U+0335
UNICODE_Mn, //U+0336
UNICODE_Mn, //U+0337
UNICODE_Mn, //U+0338
UNICODE_Mn, //U+0339
UNICODE_Mn, //U+033A
UNICODE_Mn, //U+033B
UNICODE_Mn, //U+033C
UNICODE_Mn, //U+033D
UNICODE_Mn, //U+033E
UNICODE_Mn, //U+033F
UNICODE_Mn, //U+0340
UNICODE_Mn, //U+0341
UNICODE_Mn, //U+0342
UNICODE_Mn, //U+0343
UNICODE_Mn, //U+0344
UNICODE_Mn, //U+0345
UNICODE_Mn, //U+0346
UNICODE_Mn, //U+0347
UNICODE_Mn, //U+0348
UNICODE_Mn, //U+0349
UNICODE_Mn, //U+034A
UNICODE_Mn, //U+034B
UNICODE_Mn, //U+034C
UNICODE_Mn, //U+034D
UNICODE_Mn, //U+034E
UNICODE_Mn, //U+034F
UNICODE_Mn, //U+0350
UNICODE_Mn, //U+0351
UNICODE_Mn, //U+0352
UNICODE_Mn, //U+0353
UNICODE_Mn, //U+0354
UNICODE_Mn, //U+0355
UNICODE_Mn, //U+0356
UNICODE_Mn, //U+0357
UNICODE_Mn, //U+0358
UNICODE_Mn, //U+0359
UNICODE_Mn, //U+035A
UNICODE_Mn, //U+035B
UNICODE_Mn, //U+035C
UNICODE_Mn, //U+035D
UNICODE_Mn, //U+035E
UNICODE_Mn, //U+035F
UNICODE_Mn, //U+0360
UNICODE_Mn, //U+0361
UNICODE_Mn, //U+0362
UNICODE_Mn, //U+0363
UNICODE_Mn, //U+0364
UNICODE_Mn, //U+0365
UNICODE_Mn, //U+0366
UNICODE_Mn, //U+0367
UNICODE_Mn, //U+0368
UNICODE_Mn, //U+0369
UNICODE_Mn, //U+036A
UNICODE_Mn, //U+036B
UNICODE_Mn, //U+036C
UNICODE_Mn, //U+036D
UNICODE_Mn, //U+036E
UNICODE_Mn, //U+036F
UNICODE_Lu, //U+0370
UNICODE_Ll, //U+0371
UNICODE_Lu, //U+0372
UNICODE_Ll, //U+0373
UNICODE_Lm, //U+0374
UNICODE_Sk, //U+0375
UNICODE_Lu, //U+0376
UNICODE_Ll, //U+0377
0, //U+0378
0, //U+0379
UNICODE_Lm, //U+037A
UNICODE_Ll, //U+037B
UNICODE_Ll, //U+037C
UNICODE_Ll, //U+037D
UNICODE_Po, //U+037E
0, //U+037F
0, //U+0380
0, //U+0381
0, //U+0382
0, //U+0383
UNICODE_Sk, //U+0384
UNICODE_Sk, //U+0385
UNICODE_Lu, //U+0386
UNICODE_Po, //U+0387
UNICODE_Lu, //U+0388
UNICODE_Lu, //U+0389
UNICODE_Lu, //U+038A
0, //U+038B
UNICODE_Lu, //U+038C
0, //U+038D
UNICODE_Lu, //U+038E
UNICODE_Lu, //U+038F
UNICODE_Ll, //U+0390
UNICODE_Lu, //U+0391
UNICODE_Lu, //U+0392
UNICODE_Lu, //U+0393
UNICODE_Lu, //U+0394
UNICODE_Lu, //U+0395
UNICODE_Lu, //U+0396
UNICODE_Lu, //U+0397
UNICODE_Lu, //U+0398
UNICODE_Lu, //U+0399
UNICODE_Lu, //U+039A
UNICODE_Lu, //U+039B
UNICODE_Lu, //U+039C
UNICODE_Lu, //U+039D
UNICODE_Lu, //U+039E
UNICODE_Lu, //U+039F
UNICODE_Lu, //U+03A0
UNICODE_Lu, //U+03A1
0, //U+03A2
UNICODE_Lu, //U+03A3
UNICODE_Lu, //U+03A4
UNICODE_Lu, //U+03A5
UNICODE_Lu, //U+03A6
UNICODE_Lu, //U+03A7
UNICODE_Lu, //U+03A8
UNICODE_Lu, //U+03A9
UNICODE_Lu, //U+03AA
UNICODE_Lu, //U+03AB
UNICODE_Ll, //U+03AC
UNICODE_Ll, //U+03AD
UNICODE_Ll, //U+03AE
UNICODE_Ll, //U+03AF
UNICODE_Ll, //U+03B0
UNICODE_Ll, //U+03B1
UNICODE_Ll, //U+03B2
UNICODE_Ll, //U+03B3
UNICODE_Ll, //U+03B4
UNICODE_Ll, //U+03B5
UNICODE_Ll, //U+03B6
UNICODE_Ll, //U+03B7
UNICODE_Ll, //U+03B8
UNICODE_Ll, //U+03B9
UNICODE_Ll, //U+03BA
UNICODE_Ll, //U+03BB
UNICODE_Ll, //U+03BC
UNICODE_Ll, //U+03BD
UNICODE_Ll, //U+03BE
UNICODE_Ll, //U+03BF
UNICODE_Ll, //U+03C0
UNICODE_Ll, //U+03C1
UNICODE_Ll, //U+03C2
UNICODE_Ll, //U+03C3
UNICODE_Ll, //U+03C4
UNICODE_Ll, //U+03C5
UNICODE_Ll, //U+03C6
UNICODE_Ll, //U+03C7
UNICODE_Ll, //U+03C8
UNICODE_Ll, //U+03C9
UNICODE_Ll, //U+03CA
UNICODE_Ll, //U+03CB
UNICODE_Ll, //U+03CC
UNICODE_Ll, //U+03CD
UNICODE_Ll, //U+03CE
UNICODE_Lu, //U+03CF
UNICODE_Ll, //U+03D0
UNICODE_Ll, //U+03D1
UNICODE_Lu, //U+03D2
UNICODE_Lu, //U+03D3
UNICODE_Lu, //U+03D4
UNICODE_Ll, //U+03D5
UNICODE_Ll, //U+03D6
UNICODE_Ll, //U+03D7
UNICODE_Lu, //U+03D8
UNICODE_Ll, //U+03D9
UNICODE_Lu, //U+03DA
UNICODE_Ll, //U+03DB
UNICODE_Lu, //U+03DC
UNICODE_Ll, //U+03DD
UNICODE_Lu, //U+03DE
UNICODE_Ll, //U+03DF
UNICODE_Lu, //U+03E0
UNICODE_Ll, //U+03E1
UNICODE_Lu, //U+03E2
UNICODE_Ll, //U+03E3
UNICODE_Lu, //U+03E4
UNICODE_Ll, //U+03E5
UNICODE_Lu, //U+03E6
UNICODE_Ll, //U+03E7
UNICODE_Lu, //U+03E8
UNICODE_Ll, //U+03E9
UNICODE_Lu, //U+03EA
UNICODE_Ll, //U+03EB
UNICODE_Lu, //U+03EC
UNICODE_Ll, //U+03ED
UNICODE_Lu, //U+03EE
UNICODE_Ll, //U+03EF
UNICODE_Ll, //U+03F0
UNICODE_Ll, //U+03F1
UNICODE_Ll, //U+03F2
UNICODE_Ll, //U+03F3
UNICODE_Lu, //U+03F4
UNICODE_Ll, //U+03F5
UNICODE_Sm, //U+03F6
UNICODE_Lu, //U+03F7
UNICODE_Ll, //U+03F8
UNICODE_Lu, //U+03F9
UNICODE_Lu, //U+03FA
UNICODE_Ll, //U+03FB
UNICODE_Ll, //U+03FC
UNICODE_Lu, //U+03FD
UNICODE_Lu, //U+03FE
UNICODE_Lu, //U+03FF
UNICODE_Lu, //U+0400
UNICODE_Lu, //U+0401
UNICODE_Lu, //U+0402
UNICODE_Lu, //U+0403
UNICODE_Lu, //U+0404
UNICODE_Lu, //U+0405
UNICODE_Lu, //U+0406
UNICODE_Lu, //U+0407
UNICODE_Lu, //U+0408
UNICODE_Lu, //U+0409
UNICODE_Lu, //U+040A
UNICODE_Lu, //U+040B
UNICODE_Lu, //U+040C
UNICODE_Lu, //U+040D
UNICODE_Lu, //U+040E
UNICODE_Lu, //U+040F
UNICODE_Lu, //U+0410
UNICODE_Lu, //U+0411
UNICODE_Lu, //U+0412
UNICODE_Lu, //U+0413
UNICODE_Lu, //U+0414
UNICODE_Lu, //U+0415
UNICODE_Lu, //U+0416
UNICODE_Lu, //U+0417
UNICODE_Lu, //U+0418
UNICODE_Lu, //U+0419
UNICODE_Lu, //U+041A
UNICODE_Lu, //U+041B
UNICODE_Lu, //U+041C
UNICODE_Lu, //U+041D
UNICODE_Lu, //U+041E
UNICODE_Lu, //U+041F
UNICODE_Lu, //U+0420
UNICODE_Lu, //U+0421
UNICODE_Lu, //U+0422
UNICODE_Lu, //U+0423
UNICODE_Lu, //U+0424
UNICODE_Lu, //U+0425
UNICODE_Lu, //U+0426
UNICODE_Lu, //U+0427
UNICODE_Lu, //U+0428
UNICODE_Lu, //U+0429
UNICODE_Lu, //U+042A
UNICODE_Lu, //U+042B
UNICODE_Lu, //U+042C
UNICODE_Lu, //U+042D
UNICODE_Lu, //U+042E
UNICODE_Lu, //U+042F
UNICODE_Ll, //U+0430
UNICODE_Ll, //U+0431
UNICODE_Ll, //U+0432
UNICODE_Ll, //U+0433
UNICODE_Ll, //U+0434
UNICODE_Ll, //U+0435
UNICODE_Ll, //U+0436
UNICODE_Ll, //U+0437
UNICODE_Ll, //U+0438
UNICODE_Ll, //U+0439
UNICODE_Ll, //U+043A
UNICODE_Ll, //U+043B
UNICODE_Ll, //U+043C
UNICODE_Ll, //U+043D
UNICODE_Ll, //U+043E
UNICODE_Ll, //U+043F
UNICODE_Ll, //U+0440
UNICODE_Ll, //U+0441
UNICODE_Ll, //U+0442
UNICODE_Ll, //U+0443
UNICODE_Ll, //U+0444
UNICODE_Ll, //U+0445
UNICODE_Ll, //U+0446
UNICODE_Ll, //U+0447
UNICODE_Ll, //U+0448
UNICODE_Ll, //U+0449
UNICODE_Ll, //U+044A
UNICODE_Ll, //U+044B
UNICODE_Ll, //U+044C
UNICODE_Ll, //U+044D
UNICODE_Ll, //U+044E
UNICODE_Ll, //U+044F
UNICODE_Ll, //U+0450
UNICODE_Ll, //U+0451
UNICODE_Ll, //U+0452
UNICODE_Ll, //U+0453
UNICODE_Ll, //U+0454
UNICODE_Ll, //U+0455
UNICODE_Ll, //U+0456
UNICODE_Ll, //U+0457
UNICODE_Ll, //U+0458
UNICODE_Ll, //U+0459
UNICODE_Ll, //U+045A
UNICODE_Ll, //U+045B
UNICODE_Ll, //U+045C
UNICODE_Ll, //U+045D
UNICODE_Ll, //U+045E
UNICODE_Ll, //U+045F
UNICODE_Lu, //U+0460
UNICODE_Ll, //U+0461
UNICODE_Lu, //U+0462
UNICODE_Ll, //U+0463
UNICODE_Lu, //U+0464
UNICODE_Ll, //U+0465
UNICODE_Lu, //U+0466
UNICODE_Ll, //U+0467
UNICODE_Lu, //U+0468
UNICODE_Ll, //U+0469
UNICODE_Lu, //U+046A
UNICODE_Ll, //U+046B
UNICODE_Lu, //U+046C
UNICODE_Ll, //U+046D
UNICODE_Lu, //U+046E
UNICODE_Ll, //U+046F
UNICODE_Lu, //U+0470
UNICODE_Ll, //U+0471
UNICODE_Lu, //U+0472
UNICODE_Ll, //U+0473
UNICODE_Lu, //U+0474
UNICODE_Ll, //U+0475
UNICODE_Lu, //U+0476
UNICODE_Ll, //U+0477
UNICODE_Lu, //U+0478
UNICODE_Ll, //U+0479
UNICODE_Lu, //U+047A
UNICODE_Ll, //U+047B
UNICODE_Lu, //U+047C
UNICODE_Ll, //U+047D
UNICODE_Lu, //U+047E
UNICODE_Ll, //U+047F
UNICODE_Lu, //U+0480
UNICODE_Ll, //U+0481
UNICODE_So, //U+0482
UNICODE_Mn, //U+0483
UNICODE_Mn, //U+0484
UNICODE_Mn, //U+0485
UNICODE_Mn, //U+0486
UNICODE_Mn, //U+0487
UNICODE_Me, //U+0488
UNICODE_Me, //U+0489
UNICODE_Lu, //U+048A
UNICODE_Ll, //U+048B
UNICODE_Lu, //U+048C
UNICODE_Ll, //U+048D
UNICODE_Lu, //U+048E
UNICODE_Ll, //U+048F
UNICODE_Lu, //U+0490
UNICODE_Ll, //U+0491
UNICODE_Lu, //U+0492
UNICODE_Ll, //U+0493
UNICODE_Lu, //U+0494
UNICODE_Ll, //U+0495
UNICODE_Lu, //U+0496
UNICODE_Ll, //U+0497
UNICODE_Lu, //U+0498
UNICODE_Ll, //U+0499
UNICODE_Lu, //U+049A
UNICODE_Ll, //U+049B
UNICODE_Lu, //U+049C
UNICODE_Ll, //U+049D
UNICODE_Lu, //U+049E
UNICODE_Ll, //U+049F
UNICODE_Lu, //U+04A0
UNICODE_Ll, //U+04A1
UNICODE_Lu, //U+04A2
UNICODE_Ll, //U+04A3
UNICODE_Lu, //U+04A4
UNICODE_Ll, //U+04A5
UNICODE_Lu, //U+04A6
UNICODE_Ll, //U+04A7
UNICODE_Lu, //U+04A8
UNICODE_Ll, //U+04A9
UNICODE_Lu, //U+04AA
UNICODE_Ll, //U+04AB
UNICODE_Lu, //U+04AC
UNICODE_Ll, //U+04AD
UNICODE_Lu, //U+04AE
UNICODE_Ll, //U+04AF
UNICODE_Lu, //U+04B0
UNICODE_Ll, //U+04B1
UNICODE_Lu, //U+04B2
UNICODE_Ll, //U+04B3
UNICODE_Lu, //U+04B4
UNICODE_Ll, //U+04B5
UNICODE_Lu, //U+04B6
UNICODE_Ll, //U+04B7
UNICODE_Lu, //U+04B8
UNICODE_Ll, //U+04B9
UNICODE_Lu, //U+04BA
UNICODE_Ll, //U+04BB
UNICODE_Lu, //U+04BC
UNICODE_Ll, //U+04BD
UNICODE_Lu, //U+04BE
UNICODE_Ll, //U+04BF
UNICODE_Lu, //U+04C0
UNICODE_Lu, //U+04C1
UNICODE_Ll, //U+04C2
UNICODE_Lu, //U+04C3
UNICODE_Ll, //U+04C4
UNICODE_Lu, //U+04C5
UNICODE_Ll, //U+04C6
UNICODE_Lu, //U+04C7
UNICODE_Ll, //U+04C8
UNICODE_Lu, //U+04C9
UNICODE_Ll, //U+04CA
UNICODE_Lu, //U+04CB
UNICODE_Ll, //U+04CC
UNICODE_Lu, //U+04CD
UNICODE_Ll, //U+04CE
UNICODE_Ll, //U+04CF
UNICODE_Lu, //U+04D0
UNICODE_Ll, //U+04D1
UNICODE_Lu, //U+04D2
UNICODE_Ll, //U+04D3
UNICODE_Lu, //U+04D4
UNICODE_Ll, //U+04D5
UNICODE_Lu, //U+04D6
UNICODE_Ll, //U+04D7
UNICODE_Lu, //U+04D8
UNICODE_Ll, //U+04D9
UNICODE_Lu, //U+04DA
UNICODE_Ll, //U+04DB
UNICODE_Lu, //U+04DC
UNICODE_Ll, //U+04DD
UNICODE_Lu, //U+04DE
UNICODE_Ll, //U+04DF
UNICODE_Lu, //U+04E0
UNICODE_Ll, //U+04E1
UNICODE_Lu, //U+04E2
UNICODE_Ll, //U+04E3
UNICODE_Lu, //U+04E4
UNICODE_Ll, //U+04E5
UNICODE_Lu, //U+04E6
UNICODE_Ll, //U+04E7
UNICODE_Lu, //U+04E8
UNICODE_Ll, //U+04E9
UNICODE_Lu, //U+04EA
UNICODE_Ll, //U+04EB
UNICODE_Lu, //U+04EC
UNICODE_Ll, //U+04ED
UNICODE_Lu, //U+04EE
UNICODE_Ll, //U+04EF
UNICODE_Lu, //U+04F0
UNICODE_Ll, //U+04F1
UNICODE_Lu, //U+04F2
UNICODE_Ll, //U+04F3
UNICODE_Lu, //U+04F4
UNICODE_Ll, //U+04F5
UNICODE_Lu, //U+04F6
UNICODE_Ll, //U+04F7
UNICODE_Lu, //U+04F8
UNICODE_Ll, //U+04F9
UNICODE_Lu, //U+04FA
UNICODE_Ll, //U+04FB
UNICODE_Lu, //U+04FC
UNICODE_Ll, //U+04FD
UNICODE_Lu, //U+04FE
UNICODE_Ll, //U+04FF
UNICODE_Lu, //U+0500
UNICODE_Ll, //U+0501
UNICODE_Lu, //U+0502
UNICODE_Ll, //U+0503
UNICODE_Lu, //U+0504
UNICODE_Ll, //U+0505
UNICODE_Lu, //U+0506
UNICODE_Ll, //U+0507
UNICODE_Lu, //U+0508
UNICODE_Ll, //U+0509
UNICODE_Lu, //U+050A
UNICODE_Ll, //U+050B
UNICODE_Lu, //U+050C
UNICODE_Ll, //U+050D
UNICODE_Lu, //U+050E
UNICODE_Ll, //U+050F
UNICODE_Lu, //U+0510
UNICODE_Ll, //U+0511
UNICODE_Lu, //U+0512
UNICODE_Ll, //U+0513
UNICODE_Lu, //U+0514
UNICODE_Ll, //U+0515
UNICODE_Lu, //U+0516
UNICODE_Ll, //U+0517
UNICODE_Lu, //U+0518
UNICODE_Ll, //U+0519
UNICODE_Lu, //U+051A
UNICODE_Ll, //U+051B
UNICODE_Lu, //U+051C
UNICODE_Ll, //U+051D
UNICODE_Lu, //U+051E
UNICODE_Ll, //U+051F
UNICODE_Lu, //U+0520
UNICODE_Ll, //U+0521
UNICODE_Lu, //U+0522
UNICODE_Ll, //U+0523
UNICODE_Lu, //U+0524
UNICODE_Ll, //U+0525
UNICODE_Lu, //U+0526
UNICODE_Ll, //U+0527
0, //U+0528
0, //U+0529
0, //U+052A
0, //U+052B
0, //U+052C
0, //U+052D
0, //U+052E
0, //U+052F
0, //U+0530
UNICODE_Lu, //U+0531
UNICODE_Lu, //U+0532
UNICODE_Lu, //U+0533
UNICODE_Lu, //U+0534
UNICODE_Lu, //U+0535
UNICODE_Lu, //U+0536
UNICODE_Lu, //U+0537
UNICODE_Lu, //U+0538
UNICODE_Lu, //U+0539
UNICODE_Lu, //U+053A
UNICODE_Lu, //U+053B
UNICODE_Lu, //U+053C
UNICODE_Lu, //U+053D
UNICODE_Lu, //U+053E
UNICODE_Lu, //U+053F
UNICODE_Lu, //U+0540
UNICODE_Lu, //U+0541
UNICODE_Lu, //U+0542
UNICODE_Lu, //U+0543
UNICODE_Lu, //U+0544
UNICODE_Lu, //U+0545
UNICODE_Lu, //U+0546
UNICODE_Lu, //U+0547
UNICODE_Lu, //U+0548
UNICODE_Lu, //U+0549
UNICODE_Lu, //U+054A
UNICODE_Lu, //U+054B
UNICODE_Lu, //U+054C
UNICODE_Lu, //U+054D
UNICODE_Lu, //U+054E
UNICODE_Lu, //U+054F
UNICODE_Lu, //U+0550
UNICODE_Lu, //U+0551
UNICODE_Lu, //U+0552
UNICODE_Lu, //U+0553
UNICODE_Lu, //U+0554
UNICODE_Lu, //U+0555
UNICODE_Lu, //U+0556
0, //U+0557
0, //U+0558
UNICODE_Lm, //U+0559
UNICODE_Po, //U+055A
UNICODE_Po, //U+055B
UNICODE_Po, //U+055C
UNICODE_Po, //U+055D
UNICODE_Po, //U+055E
UNICODE_Po, //U+055F
0, //U+0560
UNICODE_Ll, //U+0561
UNICODE_Ll, //U+0562
UNICODE_Ll, //U+0563
UNICODE_Ll, //U+0564
UNICODE_Ll, //U+0565
UNICODE_Ll, //U+0566
UNICODE_Ll, //U+0567
UNICODE_Ll, //U+0568
UNICODE_Ll, //U+0569
UNICODE_Ll, //U+056A
UNICODE_Ll, //U+056B
UNICODE_Ll, //U+056C
UNICODE_Ll, //U+056D
UNICODE_Ll, //U+056E
UNICODE_Ll, //U+056F
UNICODE_Ll, //U+0570
UNICODE_Ll, //U+0571
UNICODE_Ll, //U+0572
UNICODE_Ll, //U+0573
UNICODE_Ll, //U+0574
UNICODE_Ll, //U+0575
UNICODE_Ll, //U+0576
UNICODE_Ll, //U+0577
UNICODE_Ll, //U+0578
UNICODE_Ll, //U+0579
UNICODE_Ll, //U+057A
UNICODE_Ll, //U+057B
UNICODE_Ll, //U+057C
UNICODE_Ll, //U+057D
UNICODE_Ll, //U+057E
UNICODE_Ll, //U+057F
UNICODE_Ll, //U+0580
UNICODE_Ll, //U+0581
UNICODE_Ll, //U+0582
UNICODE_Ll, //U+0583
UNICODE_Ll, //U+0584
UNICODE_Ll, //U+0585
UNICODE_Ll, //U+0586
UNICODE_Ll, //U+0587
0, //U+0588
UNICODE_Po, //U+0589
UNICODE_Pd, //U+058A
0, //U+058B
0, //U+058C
0, //U+058D
0, //U+058E
0, //U+058F
0, //U+0590
UNICODE_Mn, //U+0591
UNICODE_Mn, //U+0592
UNICODE_Mn, //U+0593
UNICODE_Mn, //U+0594
UNICODE_Mn, //U+0595
UNICODE_Mn, //U+0596
UNICODE_Mn, //U+0597
UNICODE_Mn, //U+0598
UNICODE_Mn, //U+0599
UNICODE_Mn, //U+059A
UNICODE_Mn, //U+059B
UNICODE_Mn, //U+059C
UNICODE_Mn, //U+059D
UNICODE_Mn, //U+059E
UNICODE_Mn, //U+059F
UNICODE_Mn, //U+05A0
UNICODE_Mn, //U+05A1
UNICODE_Mn, //U+05A2
UNICODE_Mn, //U+05A3
UNICODE_Mn, //U+05A4
UNICODE_Mn, //U+05A5
UNICODE_Mn, //U+05A6
UNICODE_Mn, //U+05A7
UNICODE_Mn, //U+05A8
UNICODE_Mn, //U+05A9
UNICODE_Mn, //U+05AA
UNICODE_Mn, //U+05AB
UNICODE_Mn, //U+05AC
UNICODE_Mn, //U+05AD
UNICODE_Mn, //U+05AE
UNICODE_Mn, //U+05AF
UNICODE_Mn, //U+05B0
UNICODE_Mn, //U+05B1
UNICODE_Mn, //U+05B2
UNICODE_Mn, //U+05B3
UNICODE_Mn, //U+05B4
UNICODE_Mn, //U+05B5
UNICODE_Mn, //U+05B6
UNICODE_Mn, //U+05B7
UNICODE_Mn, //U+05B8
UNICODE_Mn, //U+05B9
UNICODE_Mn, //U+05BA
UNICODE_Mn, //U+05BB
UNICODE_Mn, //U+05BC
UNICODE_Mn, //U+05BD
UNICODE_Pd, //U+05BE
UNICODE_Mn, //U+05BF
UNICODE_Po, //U+05C0
UNICODE_Mn, //U+05C1
UNICODE_Mn, //U+05C2
UNICODE_Po, //U+05C3
UNICODE_Mn, //U+05C4
UNICODE_Mn, //U+05C5
UNICODE_Po, //U+05C6
UNICODE_Mn, //U+05C7
0, //U+05C8
0, //U+05C9
0, //U+05CA
0, //U+05CB
0, //U+05CC
0, //U+05CD
0, //U+05CE
0, //U+05CF
UNICODE_Lo, //U+05D0
UNICODE_Lo, //U+05D1
UNICODE_Lo, //U+05D2
UNICODE_Lo, //U+05D3
UNICODE_Lo, //U+05D4
UNICODE_Lo, //U+05D5
UNICODE_Lo, //U+05D6
UNICODE_Lo, //U+05D7
UNICODE_Lo, //U+05D8
UNICODE_Lo, //U+05D9
UNICODE_Lo, //U+05DA
UNICODE_Lo, //U+05DB
UNICODE_Lo, //U+05DC
UNICODE_Lo, //U+05DD
UNICODE_Lo, //U+05DE
UNICODE_Lo, //U+05DF
UNICODE_Lo, //U+05E0
UNICODE_Lo, //U+05E1
UNICODE_Lo, //U+05E2
UNICODE_Lo, //U+05E3
UNICODE_Lo, //U+05E4
UNICODE_Lo, //U+05E5
UNICODE_Lo, //U+05E6
UNICODE_Lo, //U+05E7
UNICODE_Lo, //U+05E8
UNICODE_Lo, //U+05E9
UNICODE_Lo, //U+05EA
0, //U+05EB
0, //U+05EC
0, //U+05ED
0, //U+05EE
0, //U+05EF
UNICODE_Lo, //U+05F0
UNICODE_Lo, //U+05F1
UNICODE_Lo, //U+05F2
UNICODE_Po, //U+05F3
UNICODE_Po, //U+05F4
0, //U+05F5
0, //U+05F6
0, //U+05F7
0, //U+05F8
0, //U+05F9
0, //U+05FA
0, //U+05FB
0, //U+05FC
0, //U+05FD
0, //U+05FE
0, //U+05FF
UNICODE_Cf, //U+0600
UNICODE_Cf, //U+0601
UNICODE_Cf, //U+0602
UNICODE_Cf, //U+0603
0, //U+0604
0, //U+0605
UNICODE_Sm, //U+0606
UNICODE_Sm, //U+0607
UNICODE_Sm, //U+0608
UNICODE_Po, //U+0609
UNICODE_Po, //U+060A
UNICODE_Sc, //U+060B
UNICODE_Po, //U+060C
UNICODE_Po, //U+060D
UNICODE_So, //U+060E
UNICODE_So, //U+060F
UNICODE_Mn, //U+0610
UNICODE_Mn, //U+0611
UNICODE_Mn, //U+0612
UNICODE_Mn, //U+0613
UNICODE_Mn, //U+0614
UNICODE_Mn, //U+0615
UNICODE_Mn, //U+0616
UNICODE_Mn, //U+0617
UNICODE_Mn, //U+0618
UNICODE_Mn, //U+0619
UNICODE_Mn, //U+061A
UNICODE_Po, //U+061B
0, //U+061C
0, //U+061D
UNICODE_Po, //U+061E
UNICODE_Po, //U+061F
UNICODE_Lo, //U+0620
UNICODE_Lo, //U+0621
UNICODE_Lo, //U+0622
UNICODE_Lo, //U+0623
UNICODE_Lo, //U+0624
UNICODE_Lo, //U+0625
UNICODE_Lo, //U+0626
UNICODE_Lo, //U+0627
UNICODE_Lo, //U+0628
UNICODE_Lo, //U+0629
UNICODE_Lo, //U+062A
UNICODE_Lo, //U+062B
UNICODE_Lo, //U+062C
UNICODE_Lo, //U+062D
UNICODE_Lo, //U+062E
UNICODE_Lo, //U+062F
UNICODE_Lo, //U+0630
UNICODE_Lo, //U+0631
UNICODE_Lo, //U+0632
UNICODE_Lo, //U+0633
UNICODE_Lo, //U+0634
UNICODE_Lo, //U+0635
UNICODE_Lo, //U+0636
UNICODE_Lo, //U+0637
UNICODE_Lo, //U+0638
UNICODE_Lo, //U+0639
UNICODE_Lo, //U+063A
UNICODE_Lo, //U+063B
UNICODE_Lo, //U+063C
UNICODE_Lo, //U+063D
UNICODE_Lo, //U+063E
UNICODE_Lo, //U+063F
UNICODE_Lm, //U+0640
UNICODE_Lo, //U+0641
UNICODE_Lo, //U+0642
UNICODE_Lo, //U+0643
UNICODE_Lo, //U+0644
UNICODE_Lo, //U+0645
UNICODE_Lo, //U+0646
UNICODE_Lo, //U+0647
UNICODE_Lo, //U+0648
UNICODE_Lo, //U+0649
UNICODE_Lo, //U+064A
UNICODE_Mn, //U+064B
UNICODE_Mn, //U+064C
UNICODE_Mn, //U+064D
UNICODE_Mn, //U+064E
UNICODE_Mn, //U+064F
UNICODE_Mn, //U+0650
UNICODE_Mn, //U+0651
UNICODE_Mn, //U+0652
UNICODE_Mn, //U+0653
UNICODE_Mn, //U+0654
UNICODE_Mn, //U+0655
UNICODE_Mn, //U+0656
UNICODE_Mn, //U+0657
UNICODE_Mn, //U+0658
UNICODE_Mn, //U+0659
UNICODE_Mn, //U+065A
UNICODE_Mn, //U+065B
UNICODE_Mn, //U+065C
UNICODE_Mn, //U+065D
UNICODE_Mn, //U+065E
UNICODE_Mn, //U+065F
UNICODE_Nd, //U+0660
UNICODE_Nd, //U+0661
UNICODE_Nd, //U+0662
UNICODE_Nd, //U+0663
UNICODE_Nd, //U+0664
UNICODE_Nd, //U+0665
UNICODE_Nd, //U+0666
UNICODE_Nd, //U+0667
UNICODE_Nd, //U+0668
UNICODE_Nd, //U+0669
UNICODE_Po, //U+066A
UNICODE_Po, //U+066B
UNICODE_Po, //U+066C
UNICODE_Po, //U+066D
UNICODE_Lo, //U+066E
UNICODE_Lo, //U+066F
UNICODE_Mn, //U+0670
UNICODE_Lo, //U+0671
UNICODE_Lo, //U+0672
UNICODE_Lo, //U+0673
UNICODE_Lo, //U+0674
UNICODE_Lo, //U+0675
UNICODE_Lo, //U+0676
UNICODE_Lo, //U+0677
UNICODE_Lo, //U+0678
UNICODE_Lo, //U+0679
UNICODE_Lo, //U+067A
UNICODE_Lo, //U+067B
UNICODE_Lo, //U+067C
UNICODE_Lo, //U+067D
UNICODE_Lo, //U+067E
UNICODE_Lo, //U+067F
UNICODE_Lo, //U+0680
UNICODE_Lo, //U+0681
UNICODE_Lo, //U+0682
UNICODE_Lo, //U+0683
UNICODE_Lo, //U+0684
UNICODE_Lo, //U+0685
UNICODE_Lo, //U+0686
UNICODE_Lo, //U+0687
UNICODE_Lo, //U+0688
UNICODE_Lo, //U+0689
UNICODE_Lo, //U+068A
UNICODE_Lo, //U+068B
UNICODE_Lo, //U+068C
UNICODE_Lo, //U+068D
UNICODE_Lo, //U+068E
UNICODE_Lo, //U+068F
UNICODE_Lo, //U+0690
UNICODE_Lo, //U+0691
UNICODE_Lo, //U+0692
UNICODE_Lo, //U+0693
UNICODE_Lo, //U+0694
UNICODE_Lo, //U+0695
UNICODE_Lo, //U+0696
UNICODE_Lo, //U+0697
UNICODE_Lo, //U+0698
UNICODE_Lo, //U+0699
UNICODE_Lo, //U+069A
UNICODE_Lo, //U+069B
UNICODE_Lo, //U+069C
UNICODE_Lo, //U+069D
UNICODE_Lo, //U+069E
UNICODE_Lo, //U+069F
UNICODE_Lo, //U+06A0
UNICODE_Lo, //U+06A1
UNICODE_Lo, //U+06A2
UNICODE_Lo, //U+06A3
UNICODE_Lo, //U+06A4
UNICODE_Lo, //U+06A5
UNICODE_Lo, //U+06A6
UNICODE_Lo, //U+06A7
UNICODE_Lo, //U+06A8
UNICODE_Lo, //U+06A9
UNICODE_Lo, //U+06AA
UNICODE_Lo, //U+06AB
UNICODE_Lo, //U+06AC
UNICODE_Lo, //U+06AD
UNICODE_Lo, //U+06AE
UNICODE_Lo, //U+06AF
UNICODE_Lo, //U+06B0
UNICODE_Lo, //U+06B1
UNICODE_Lo, //U+06B2
UNICODE_Lo, //U+06B3
UNICODE_Lo, //U+06B4
UNICODE_Lo, //U+06B5
UNICODE_Lo, //U+06B6
UNICODE_Lo, //U+06B7
UNICODE_Lo, //U+06B8
UNICODE_Lo, //U+06B9
UNICODE_Lo, //U+06BA
UNICODE_Lo, //U+06BB
UNICODE_Lo, //U+06BC
UNICODE_Lo, //U+06BD
UNICODE_Lo, //U+06BE
UNICODE_Lo, //U+06BF
UNICODE_Lo, //U+06C0
UNICODE_Lo, //U+06C1
UNICODE_Lo, //U+06C2
UNICODE_Lo, //U+06C3
UNICODE_Lo, //U+06C4
UNICODE_Lo, //U+06C5
UNICODE_Lo, //U+06C6
UNICODE_Lo, //U+06C7
UNICODE_Lo, //U+06C8
UNICODE_Lo, //U+06C9
UNICODE_Lo, //U+06CA
UNICODE_Lo, //U+06CB
UNICODE_Lo, //U+06CC
UNICODE_Lo, //U+06CD
UNICODE_Lo, //U+06CE
UNICODE_Lo, //U+06CF
UNICODE_Lo, //U+06D0
UNICODE_Lo, //U+06D1
UNICODE_Lo, //U+06D2
UNICODE_Lo, //U+06D3
UNICODE_Po, //U+06D4
UNICODE_Lo, //U+06D5
UNICODE_Mn, //U+06D6
UNICODE_Mn, //U+06D7
UNICODE_Mn, //U+06D8
UNICODE_Mn, //U+06D9
UNICODE_Mn, //U+06DA
UNICODE_Mn, //U+06DB
UNICODE_Mn, //U+06DC
UNICODE_Cf, //U+06DD
UNICODE_So, //U+06DE
UNICODE_Mn, //U+06DF
UNICODE_Mn, //U+06E0
UNICODE_Mn, //U+06E1
UNICODE_Mn, //U+06E2
UNICODE_Mn, //U+06E3
UNICODE_Mn, //U+06E4
UNICODE_Lm, //U+06E5
UNICODE_Lm, //U+06E6
UNICODE_Mn, //U+06E7
UNICODE_Mn, //U+06E8
UNICODE_So, //U+06E9
UNICODE_Mn, //U+06EA
UNICODE_Mn, //U+06EB
UNICODE_Mn, //U+06EC
UNICODE_Mn, //U+06ED
UNICODE_Lo, //U+06EE
UNICODE_Lo, //U+06EF
UNICODE_Nd, //U+06F0
UNICODE_Nd, //U+06F1
UNICODE_Nd, //U+06F2
UNICODE_Nd, //U+06F3
UNICODE_Nd, //U+06F4
UNICODE_Nd, //U+06F5
UNICODE_Nd, //U+06F6
UNICODE_Nd, //U+06F7
UNICODE_Nd, //U+06F8
UNICODE_Nd, //U+06F9
UNICODE_Lo, //U+06FA
UNICODE_Lo, //U+06FB
UNICODE_Lo, //U+06FC
UNICODE_So, //U+06FD
UNICODE_So, //U+06FE
UNICODE_Lo, //U+06FF
UNICODE_Po, //U+0700
UNICODE_Po, //U+0701
UNICODE_Po, //U+0702
UNICODE_Po, //U+0703
UNICODE_Po, //U+0704
UNICODE_Po, //U+0705
UNICODE_Po, //U+0706
UNICODE_Po, //U+0707
UNICODE_Po, //U+0708
UNICODE_Po, //U+0709
UNICODE_Po, //U+070A
UNICODE_Po, //U+070B
UNICODE_Po, //U+070C
UNICODE_Po, //U+070D
0, //U+070E
UNICODE_Cf, //U+070F
UNICODE_Lo, //U+0710
UNICODE_Mn, //U+0711
UNICODE_Lo, //U+0712
UNICODE_Lo, //U+0713
UNICODE_Lo, //U+0714
UNICODE_Lo, //U+0715
UNICODE_Lo, //U+0716
UNICODE_Lo, //U+0717
UNICODE_Lo, //U+0718
UNICODE_Lo, //U+0719
UNICODE_Lo, //U+071A
UNICODE_Lo, //U+071B
UNICODE_Lo, //U+071C
UNICODE_Lo, //U+071D
UNICODE_Lo, //U+071E
UNICODE_Lo, //U+071F
UNICODE_Lo, //U+0720
UNICODE_Lo, //U+0721
UNICODE_Lo, //U+0722
UNICODE_Lo, //U+0723
UNICODE_Lo, //U+0724
UNICODE_Lo, //U+0725
UNICODE_Lo, //U+0726
UNICODE_Lo, //U+0727
UNICODE_Lo, //U+0728
UNICODE_Lo, //U+0729
UNICODE_Lo, //U+072A
UNICODE_Lo, //U+072B
UNICODE_Lo, //U+072C
UNICODE_Lo, //U+072D
UNICODE_Lo, //U+072E
UNICODE_Lo, //U+072F
UNICODE_Mn, //U+0730
UNICODE_Mn, //U+0731
UNICODE_Mn, //U+0732
UNICODE_Mn, //U+0733
UNICODE_Mn, //U+0734
UNICODE_Mn, //U+0735
UNICODE_Mn, //U+0736
UNICODE_Mn, //U+0737
UNICODE_Mn, //U+0738
UNICODE_Mn, //U+0739
UNICODE_Mn, //U+073A
UNICODE_Mn, //U+073B
UNICODE_Mn, //U+073C
UNICODE_Mn, //U+073D
UNICODE_Mn, //U+073E
UNICODE_Mn, //U+073F
UNICODE_Mn, //U+0740
UNICODE_Mn, //U+0741
UNICODE_Mn, //U+0742
UNICODE_Mn, //U+0743
UNICODE_Mn, //U+0744
UNICODE_Mn, //U+0745
UNICODE_Mn, //U+0746
UNICODE_Mn, //U+0747
UNICODE_Mn, //U+0748
UNICODE_Mn, //U+0749
UNICODE_Mn, //U+074A
0, //U+074B
0, //U+074C
UNICODE_Lo, //U+074D
UNICODE_Lo, //U+074E
UNICODE_Lo, //U+074F
UNICODE_Lo, //U+0750
UNICODE_Lo, //U+0751
UNICODE_Lo, //U+0752
UNICODE_Lo, //U+0753
UNICODE_Lo, //U+0754
UNICODE_Lo, //U+0755
UNICODE_Lo, //U+0756
UNICODE_Lo, //U+0757
UNICODE_Lo, //U+0758
UNICODE_Lo, //U+0759
UNICODE_Lo, //U+075A
UNICODE_Lo, //U+075B
UNICODE_Lo, //U+075C
UNICODE_Lo, //U+075D
UNICODE_Lo, //U+075E
UNICODE_Lo, //U+075F
UNICODE_Lo, //U+0760
UNICODE_Lo, //U+0761
UNICODE_Lo, //U+0762
UNICODE_Lo, //U+0763
UNICODE_Lo, //U+0764
UNICODE_Lo, //U+0765
UNICODE_Lo, //U+0766
UNICODE_Lo, //U+0767
UNICODE_Lo, //U+0768
UNICODE_Lo, //U+0769
UNICODE_Lo, //U+076A
UNICODE_Lo, //U+076B
UNICODE_Lo, //U+076C
UNICODE_Lo, //U+076D
UNICODE_Lo, //U+076E
UNICODE_Lo, //U+076F
UNICODE_Lo, //U+0770
UNICODE_Lo, //U+0771
UNICODE_Lo, //U+0772
UNICODE_Lo, //U+0773
UNICODE_Lo, //U+0774
UNICODE_Lo, //U+0775
UNICODE_Lo, //U+0776
UNICODE_Lo, //U+0777
UNICODE_Lo, //U+0778
UNICODE_Lo, //U+0779
UNICODE_Lo, //U+077A
UNICODE_Lo, //U+077B
UNICODE_Lo, //U+077C
UNICODE_Lo, //U+077D
UNICODE_Lo, //U+077E
UNICODE_Lo, //U+077F
UNICODE_Lo, //U+0780
UNICODE_Lo, //U+0781
UNICODE_Lo, //U+0782
UNICODE_Lo, //U+0783
UNICODE_Lo, //U+0784
UNICODE_Lo, //U+0785
UNICODE_Lo, //U+0786
UNICODE_Lo, //U+0787
UNICODE_Lo, //U+0788
UNICODE_Lo, //U+0789
UNICODE_Lo, //U+078A
UNICODE_Lo, //U+078B
UNICODE_Lo, //U+078C
UNICODE_Lo, //U+078D
UNICODE_Lo, //U+078E
UNICODE_Lo, //U+078F
UNICODE_Lo, //U+0790
UNICODE_Lo, //U+0791
UNICODE_Lo, //U+0792
UNICODE_Lo, //U+0793
UNICODE_Lo, //U+0794
UNICODE_Lo, //U+0795
UNICODE_Lo, //U+0796
UNICODE_Lo, //U+0797
UNICODE_Lo, //U+0798
UNICODE_Lo, //U+0799
UNICODE_Lo, //U+079A
UNICODE_Lo, //U+079B
UNICODE_Lo, //U+079C
UNICODE_Lo, //U+079D
UNICODE_Lo, //U+079E
UNICODE_Lo, //U+079F
UNICODE_Lo, //U+07A0
UNICODE_Lo, //U+07A1
UNICODE_Lo, //U+07A2
UNICODE_Lo, //U+07A3
UNICODE_Lo, //U+07A4
UNICODE_Lo, //U+07A5
UNICODE_Mn, //U+07A6
UNICODE_Mn, //U+07A7
UNICODE_Mn, //U+07A8
UNICODE_Mn, //U+07A9
UNICODE_Mn, //U+07AA
UNICODE_Mn, //U+07AB
UNICODE_Mn, //U+07AC
UNICODE_Mn, //U+07AD
UNICODE_Mn, //U+07AE
UNICODE_Mn, //U+07AF
UNICODE_Mn, //U+07B0
UNICODE_Lo, //U+07B1
0, //U+07B2
0, //U+07B3
0, //U+07B4
0, //U+07B5
0, //U+07B6
0, //U+07B7
0, //U+07B8
0, //U+07B9
0, //U+07BA
0, //U+07BB
0, //U+07BC
0, //U+07BD
0, //U+07BE
0, //U+07BF
UNICODE_Nd, //U+07C0
UNICODE_Nd, //U+07C1
UNICODE_Nd, //U+07C2
UNICODE_Nd, //U+07C3
UNICODE_Nd, //U+07C4
UNICODE_Nd, //U+07C5
UNICODE_Nd, //U+07C6
UNICODE_Nd, //U+07C7
UNICODE_Nd, //U+07C8
UNICODE_Nd, //U+07C9
UNICODE_Lo, //U+07CA
UNICODE_Lo, //U+07CB
UNICODE_Lo, //U+07CC
UNICODE_Lo, //U+07CD
UNICODE_Lo, //U+07CE
UNICODE_Lo, //U+07CF
UNICODE_Lo, //U+07D0
UNICODE_Lo, //U+07D1
UNICODE_Lo, //U+07D2
UNICODE_Lo, //U+07D3
UNICODE_Lo, //U+07D4
UNICODE_Lo, //U+07D5
UNICODE_Lo, //U+07D6
UNICODE_Lo, //U+07D7
UNICODE_Lo, //U+07D8
UNICODE_Lo, //U+07D9
UNICODE_Lo, //U+07DA
UNICODE_Lo, //U+07DB
UNICODE_Lo, //U+07DC
UNICODE_Lo, //U+07DD
UNICODE_Lo, //U+07DE
UNICODE_Lo, //U+07DF
UNICODE_Lo, //U+07E0
UNICODE_Lo, //U+07E1
UNICODE_Lo, //U+07E2
UNICODE_Lo, //U+07E3
UNICODE_Lo, //U+07E4
UNICODE_Lo, //U+07E5
UNICODE_Lo, //U+07E6
UNICODE_Lo, //U+07E7
UNICODE_Lo, //U+07E8
UNICODE_Lo, //U+07E9
UNICODE_Lo, //U+07EA
UNICODE_Mn, //U+07EB
UNICODE_Mn, //U+07EC
UNICODE_Mn, //U+07ED
UNICODE_Mn, //U+07EE
UNICODE_Mn, //U+07EF
UNICODE_Mn, //U+07F0
UNICODE_Mn, //U+07F1
UNICODE_Mn, //U+07F2
UNICODE_Mn, //U+07F3
UNICODE_Lm, //U+07F4
UNICODE_Lm, //U+07F5
UNICODE_So, //U+07F6
UNICODE_Po, //U+07F7
UNICODE_Po, //U+07F8
UNICODE_Po, //U+07F9
UNICODE_Lm, //U+07FA
0, //U+07FB
0, //U+07FC
0, //U+07FD
0, //U+07FE
0, //U+07FF
UNICODE_Lo, //U+0800
UNICODE_Lo, //U+0801
UNICODE_Lo, //U+0802
UNICODE_Lo, //U+0803
UNICODE_Lo, //U+0804
UNICODE_Lo, //U+0805
UNICODE_Lo, //U+0806
UNICODE_Lo, //U+0807
UNICODE_Lo, //U+0808
UNICODE_Lo, //U+0809
UNICODE_Lo, //U+080A
UNICODE_Lo, //U+080B
UNICODE_Lo, //U+080C
UNICODE_Lo, //U+080D
UNICODE_Lo, //U+080E
UNICODE_Lo, //U+080F
UNICODE_Lo, //U+0810
UNICODE_Lo, //U+0811
UNICODE_Lo, //U+0812
UNICODE_Lo, //U+0813
UNICODE_Lo, //U+0814
UNICODE_Lo, //U+0815
UNICODE_Mn, //U+0816
UNICODE_Mn, //U+0817
UNICODE_Mn, //U+0818
UNICODE_Mn, //U+0819
UNICODE_Lm, //U+081A
UNICODE_Mn, //U+081B
UNICODE_Mn, //U+081C
UNICODE_Mn, //U+081D
UNICODE_Mn, //U+081E
UNICODE_Mn, //U+081F
UNICODE_Mn, //U+0820
UNICODE_Mn, //U+0821
UNICODE_Mn, //U+0822
UNICODE_Mn, //U+0823
UNICODE_Lm, //U+0824
UNICODE_Mn, //U+0825
UNICODE_Mn, //U+0826
UNICODE_Mn, //U+0827
UNICODE_Lm, //U+0828
UNICODE_Mn, //U+0829
UNICODE_Mn, //U+082A
UNICODE_Mn, //U+082B
UNICODE_Mn, //U+082C
UNICODE_Mn, //U+082D
0, //U+082E
0, //U+082F
UNICODE_Po, //U+0830
UNICODE_Po, //U+0831
UNICODE_Po, //U+0832
UNICODE_Po, //U+0833
UNICODE_Po, //U+0834
UNICODE_Po, //U+0835
UNICODE_Po, //U+0836
UNICODE_Po, //U+0837
UNICODE_Po, //U+0838
UNICODE_Po, //U+0839
UNICODE_Po, //U+083A
UNICODE_Po, //U+083B
UNICODE_Po, //U+083C
UNICODE_Po, //U+083D
UNICODE_Po, //U+083E
0, //U+083F
UNICODE_Lo, //U+0840
UNICODE_Lo, //U+0841
UNICODE_Lo, //U+0842
UNICODE_Lo, //U+0843
UNICODE_Lo, //U+0844
UNICODE_Lo, //U+0845
UNICODE_Lo, //U+0846
UNICODE_Lo, //U+0847
UNICODE_Lo, //U+0848
UNICODE_Lo, //U+0849
UNICODE_Lo, //U+084A
UNICODE_Lo, //U+084B
UNICODE_Lo, //U+084C
UNICODE_Lo, //U+084D
UNICODE_Lo, //U+084E
UNICODE_Lo, //U+084F
UNICODE_Lo, //U+0850
UNICODE_Lo, //U+0851
UNICODE_Lo, //U+0852
UNICODE_Lo, //U+0853
UNICODE_Lo, //U+0854
UNICODE_Lo, //U+0855
UNICODE_Lo, //U+0856
UNICODE_Lo, //U+0857
UNICODE_Lo, //U+0858
UNICODE_Mn, //U+0859
UNICODE_Mn, //U+085A
UNICODE_Mn, //U+085B
0, //U+085C
0, //U+085D
UNICODE_Po, //U+085E
0, //U+085F
0, //U+0860
0, //U+0861
0, //U+0862
0, //U+0863
0, //U+0864
0, //U+0865
0, //U+0866
0, //U+0867
0, //U+0868
0, //U+0869
0, //U+086A
0, //U+086B
0, //U+086C
0, //U+086D
0, //U+086E
0, //U+086F
0, //U+0870
0, //U+0871
0, //U+0872
0, //U+0873
0, //U+0874
0, //U+0875
0, //U+0876
0, //U+0877
0, //U+0878
0, //U+0879
0, //U+087A
0, //U+087B
0, //U+087C
0, //U+087D
0, //U+087E
0, //U+087F
0, //U+0880
0, //U+0881
0, //U+0882
0, //U+0883
0, //U+0884
0, //U+0885
0, //U+0886
0, //U+0887
0, //U+0888
0, //U+0889
0, //U+088A
0, //U+088B
0, //U+088C
0, //U+088D
0, //U+088E
0, //U+088F
0, //U+0890
0, //U+0891
0, //U+0892
0, //U+0893
0, //U+0894
0, //U+0895
0, //U+0896
0, //U+0897
0, //U+0898
0, //U+0899
0, //U+089A
0, //U+089B
0, //U+089C
0, //U+089D
0, //U+089E
0, //U+089F
0, //U+08A0
0, //U+08A1
0, //U+08A2
0, //U+08A3
0, //U+08A4
0, //U+08A5
0, //U+08A6
0, //U+08A7
0, //U+08A8
0, //U+08A9
0, //U+08AA
0, //U+08AB
0, //U+08AC
0, //U+08AD
0, //U+08AE
0, //U+08AF
0, //U+08B0
0, //U+08B1
0, //U+08B2
0, //U+08B3
0, //U+08B4
0, //U+08B5
0, //U+08B6
0, //U+08B7
0, //U+08B8
0, //U+08B9
0, //U+08BA
0, //U+08BB
0, //U+08BC
0, //U+08BD
0, //U+08BE
0, //U+08BF
0, //U+08C0
0, //U+08C1
0, //U+08C2
0, //U+08C3
0, //U+08C4
0, //U+08C5
0, //U+08C6
0, //U+08C7
0, //U+08C8
0, //U+08C9
0, //U+08CA
0, //U+08CB
0, //U+08CC
0, //U+08CD
0, //U+08CE
0, //U+08CF
0, //U+08D0
0, //U+08D1
0, //U+08D2
0, //U+08D3
0, //U+08D4
0, //U+08D5
0, //U+08D6
0, //U+08D7
0, //U+08D8
0, //U+08D9
0, //U+08DA
0, //U+08DB
0, //U+08DC
0, //U+08DD
0, //U+08DE
0, //U+08DF
0, //U+08E0
0, //U+08E1
0, //U+08E2
0, //U+08E3
0, //U+08E4
0, //U+08E5
0, //U+08E6
0, //U+08E7
0, //U+08E8
0, //U+08E9
0, //U+08EA
0, //U+08EB
0, //U+08EC
0, //U+08ED
0, //U+08EE
0, //U+08EF
0, //U+08F0
0, //U+08F1
0, //U+08F2
0, //U+08F3
0, //U+08F4
0, //U+08F5
0, //U+08F6
0, //U+08F7
0, //U+08F8
0, //U+08F9
0, //U+08FA
0, //U+08FB
0, //U+08FC
0, //U+08FD
0, //U+08FE
0, //U+08FF
UNICODE_Mn, //U+0900
UNICODE_Mn, //U+0901
UNICODE_Mn, //U+0902
UNICODE_Mc, //U+0903
UNICODE_Lo, //U+0904
UNICODE_Lo, //U+0905
UNICODE_Lo, //U+0906
UNICODE_Lo, //U+0907
UNICODE_Lo, //U+0908
UNICODE_Lo, //U+0909
UNICODE_Lo, //U+090A
UNICODE_Lo, //U+090B
UNICODE_Lo, //U+090C
UNICODE_Lo, //U+090D
UNICODE_Lo, //U+090E
UNICODE_Lo, //U+090F
UNICODE_Lo, //U+0910
UNICODE_Lo, //U+0911
UNICODE_Lo, //U+0912
UNICODE_Lo, //U+0913
UNICODE_Lo, //U+0914
UNICODE_Lo, //U+0915
UNICODE_Lo, //U+0916
UNICODE_Lo, //U+0917
UNICODE_Lo, //U+0918
UNICODE_Lo, //U+0919
UNICODE_Lo, //U+091A
UNICODE_Lo, //U+091B
UNICODE_Lo, //U+091C
UNICODE_Lo, //U+091D
UNICODE_Lo, //U+091E
UNICODE_Lo, //U+091F
UNICODE_Lo, //U+0920
UNICODE_Lo, //U+0921
UNICODE_Lo, //U+0922
UNICODE_Lo, //U+0923
UNICODE_Lo, //U+0924
UNICODE_Lo, //U+0925
UNICODE_Lo, //U+0926
UNICODE_Lo, //U+0927
UNICODE_Lo, //U+0928
UNICODE_Lo, //U+0929
UNICODE_Lo, //U+092A
UNICODE_Lo, //U+092B
UNICODE_Lo, //U+092C
UNICODE_Lo, //U+092D
UNICODE_Lo, //U+092E
UNICODE_Lo, //U+092F
UNICODE_Lo, //U+0930
UNICODE_Lo, //U+0931
UNICODE_Lo, //U+0932
UNICODE_Lo, //U+0933
UNICODE_Lo, //U+0934
UNICODE_Lo, //U+0935
UNICODE_Lo, //U+0936
UNICODE_Lo, //U+0937
UNICODE_Lo, //U+0938
UNICODE_Lo, //U+0939
UNICODE_Mn, //U+093A
UNICODE_Mc, //U+093B
UNICODE_Mn, //U+093C
UNICODE_Lo, //U+093D
UNICODE_Mc, //U+093E
UNICODE_Mc, //U+093F
UNICODE_Mc, //U+0940
UNICODE_Mn, //U+0941
UNICODE_Mn, //U+0942
UNICODE_Mn, //U+0943
UNICODE_Mn, //U+0944
UNICODE_Mn, //U+0945
UNICODE_Mn, //U+0946
UNICODE_Mn, //U+0947
UNICODE_Mn, //U+0948
UNICODE_Mc, //U+0949
UNICODE_Mc, //U+094A
UNICODE_Mc, //U+094B
UNICODE_Mc, //U+094C
UNICODE_Mn, //U+094D
UNICODE_Mc, //U+094E
UNICODE_Mc, //U+094F
UNICODE_Lo, //U+0950
UNICODE_Mn, //U+0951
UNICODE_Mn, //U+0952
UNICODE_Mn, //U+0953
UNICODE_Mn, //U+0954
UNICODE_Mn, //U+0955
UNICODE_Mn, //U+0956
UNICODE_Mn, //U+0957
UNICODE_Lo, //U+0958
UNICODE_Lo, //U+0959
UNICODE_Lo, //U+095A
UNICODE_Lo, //U+095B
UNICODE_Lo, //U+095C
UNICODE_Lo, //U+095D
UNICODE_Lo, //U+095E
UNICODE_Lo, //U+095F
UNICODE_Lo, //U+0960
UNICODE_Lo, //U+0961
UNICODE_Mn, //U+0962
UNICODE_Mn, //U+0963
UNICODE_Po, //U+0964
UNICODE_Po, //U+0965
UNICODE_Nd, //U+0966
UNICODE_Nd, //U+0967
UNICODE_Nd, //U+0968
UNICODE_Nd, //U+0969
UNICODE_Nd, //U+096A
UNICODE_Nd, //U+096B
UNICODE_Nd, //U+096C
UNICODE_Nd, //U+096D
UNICODE_Nd, //U+096E
UNICODE_Nd, //U+096F
UNICODE_Po, //U+0970
UNICODE_Lm, //U+0971
UNICODE_Lo, //U+0972
UNICODE_Lo, //U+0973
UNICODE_Lo, //U+0974
UNICODE_Lo, //U+0975
UNICODE_Lo, //U+0976
UNICODE_Lo, //U+0977
0, //U+0978
UNICODE_Lo, //U+0979
UNICODE_Lo, //U+097A
UNICODE_Lo, //U+097B
UNICODE_Lo, //U+097C
UNICODE_Lo, //U+097D
UNICODE_Lo, //U+097E
UNICODE_Lo, //U+097F
0, //U+0980
UNICODE_Mn, //U+0981
UNICODE_Mc, //U+0982
UNICODE_Mc, //U+0983
0, //U+0984
UNICODE_Lo, //U+0985
UNICODE_Lo, //U+0986
UNICODE_Lo, //U+0987
UNICODE_Lo, //U+0988
UNICODE_Lo, //U+0989
UNICODE_Lo, //U+098A
UNICODE_Lo, //U+098B
UNICODE_Lo, //U+098C
0, //U+098D
0, //U+098E
UNICODE_Lo, //U+098F
UNICODE_Lo, //U+0990
0, //U+0991
0, //U+0992
UNICODE_Lo, //U+0993
UNICODE_Lo, //U+0994
UNICODE_Lo, //U+0995
UNICODE_Lo, //U+0996
UNICODE_Lo, //U+0997
UNICODE_Lo, //U+0998
UNICODE_Lo, //U+0999
UNICODE_Lo, //U+099A
UNICODE_Lo, //U+099B
UNICODE_Lo, //U+099C
UNICODE_Lo, //U+099D
UNICODE_Lo, //U+099E
UNICODE_Lo, //U+099F
UNICODE_Lo, //U+09A0
UNICODE_Lo, //U+09A1
UNICODE_Lo, //U+09A2
UNICODE_Lo, //U+09A3
UNICODE_Lo, //U+09A4
UNICODE_Lo, //U+09A5
UNICODE_Lo, //U+09A6
UNICODE_Lo, //U+09A7
UNICODE_Lo, //U+09A8
0, //U+09A9
UNICODE_Lo, //U+09AA
UNICODE_Lo, //U+09AB
UNICODE_Lo, //U+09AC
UNICODE_Lo, //U+09AD
UNICODE_Lo, //U+09AE
UNICODE_Lo, //U+09AF
UNICODE_Lo, //U+09B0
0, //U+09B1
UNICODE_Lo, //U+09B2
0, //U+09B3
0, //U+09B4
0, //U+09B5
UNICODE_Lo, //U+09B6
UNICODE_Lo, //U+09B7
UNICODE_Lo, //U+09B8
UNICODE_Lo, //U+09B9
0, //U+09BA
0, //U+09BB
UNICODE_Mn, //U+09BC
UNICODE_Lo, //U+09BD
UNICODE_Mc, //U+09BE
UNICODE_Mc, //U+09BF
UNICODE_Mc, //U+09C0
UNICODE_Mn, //U+09C1
UNICODE_Mn, //U+09C2
UNICODE_Mn, //U+09C3
UNICODE_Mn, //U+09C4
0, //U+09C5
0, //U+09C6
UNICODE_Mc, //U+09C7
UNICODE_Mc, //U+09C8
0, //U+09C9
0, //U+09CA
UNICODE_Mc, //U+09CB
UNICODE_Mc, //U+09CC
UNICODE_Mn, //U+09CD
UNICODE_Lo, //U+09CE
0, //U+09CF
0, //U+09D0
0, //U+09D1
0, //U+09D2
0, //U+09D3
0, //U+09D4
0, //U+09D5
0, //U+09D6
UNICODE_Mc, //U+09D7
0, //U+09D8
0, //U+09D9
0, //U+09DA
0, //U+09DB
UNICODE_Lo, //U+09DC
UNICODE_Lo, //U+09DD
0, //U+09DE
UNICODE_Lo, //U+09DF
UNICODE_Lo, //U+09E0
UNICODE_Lo, //U+09E1
UNICODE_Mn, //U+09E2
UNICODE_Mn, //U+09E3
0, //U+09E4
0, //U+09E5
UNICODE_Nd, //U+09E6
UNICODE_Nd, //U+09E7
UNICODE_Nd, //U+09E8
UNICODE_Nd, //U+09E9
UNICODE_Nd, //U+09EA
UNICODE_Nd, //U+09EB
UNICODE_Nd, //U+09EC
UNICODE_Nd, //U+09ED
UNICODE_Nd, //U+09EE
UNICODE_Nd, //U+09EF
UNICODE_Lo, //U+09F0
UNICODE_Lo, //U+09F1
UNICODE_Sc, //U+09F2
UNICODE_Sc, //U+09F3
UNICODE_No, //U+09F4
UNICODE_No, //U+09F5
UNICODE_No, //U+09F6
UNICODE_No, //U+09F7
UNICODE_No, //U+09F8
UNICODE_No, //U+09F9
UNICODE_So, //U+09FA
UNICODE_Sc, //U+09FB
0, //U+09FC
0, //U+09FD
0, //U+09FE
0, //U+09FF
0, //U+0A00
UNICODE_Mn, //U+0A01
UNICODE_Mn, //U+0A02
UNICODE_Mc, //U+0A03
0, //U+0A04
UNICODE_Lo, //U+0A05
UNICODE_Lo, //U+0A06
UNICODE_Lo, //U+0A07
UNICODE_Lo, //U+0A08
UNICODE_Lo, //U+0A09
UNICODE_Lo, //U+0A0A
0, //U+0A0B
0, //U+0A0C
0, //U+0A0D
0, //U+0A0E
UNICODE_Lo, //U+0A0F
UNICODE_Lo, //U+0A10
0, //U+0A11
0, //U+0A12
UNICODE_Lo, //U+0A13
UNICODE_Lo, //U+0A14
UNICODE_Lo, //U+0A15
UNICODE_Lo, //U+0A16
UNICODE_Lo, //U+0A17
UNICODE_Lo, //U+0A18
UNICODE_Lo, //U+0A19
UNICODE_Lo, //U+0A1A
UNICODE_Lo, //U+0A1B
UNICODE_Lo, //U+0A1C
UNICODE_Lo, //U+0A1D
UNICODE_Lo, //U+0A1E
UNICODE_Lo, //U+0A1F
UNICODE_Lo, //U+0A20
UNICODE_Lo, //U+0A21
UNICODE_Lo, //U+0A22
UNICODE_Lo, //U+0A23
UNICODE_Lo, //U+0A24
UNICODE_Lo, //U+0A25
UNICODE_Lo, //U+0A26
UNICODE_Lo, //U+0A27
UNICODE_Lo, //U+0A28
0, //U+0A29
UNICODE_Lo, //U+0A2A
UNICODE_Lo, //U+0A2B
UNICODE_Lo, //U+0A2C
UNICODE_Lo, //U+0A2D
UNICODE_Lo, //U+0A2E
UNICODE_Lo, //U+0A2F
UNICODE_Lo, //U+0A30
0, //U+0A31
UNICODE_Lo, //U+0A32
UNICODE_Lo, //U+0A33
0, //U+0A34
UNICODE_Lo, //U+0A35
UNICODE_Lo, //U+0A36
0, //U+0A37
UNICODE_Lo, //U+0A38
UNICODE_Lo, //U+0A39
0, //U+0A3A
0, //U+0A3B
UNICODE_Mn, //U+0A3C
0, //U+0A3D
UNICODE_Mc, //U+0A3E
UNICODE_Mc, //U+0A3F
UNICODE_Mc, //U+0A40
UNICODE_Mn, //U+0A41
UNICODE_Mn, //U+0A42
0, //U+0A43
0, //U+0A44
0, //U+0A45
0, //U+0A46
UNICODE_Mn, //U+0A47
UNICODE_Mn, //U+0A48
0, //U+0A49
0, //U+0A4A
UNICODE_Mn, //U+0A4B
UNICODE_Mn, //U+0A4C
UNICODE_Mn, //U+0A4D
0, //U+0A4E
0, //U+0A4F
0, //U+0A50
UNICODE_Mn, //U+0A51
0, //U+0A52
0, //U+0A53
0, //U+0A54
0, //U+0A55
0, //U+0A56
0, //U+0A57
0, //U+0A58
UNICODE_Lo, //U+0A59
UNICODE_Lo, //U+0A5A
UNICODE_Lo, //U+0A5B
UNICODE_Lo, //U+0A5C
0, //U+0A5D
UNICODE_Lo, //U+0A5E
0, //U+0A5F
0, //U+0A60
0, //U+0A61
0, //U+0A62
0, //U+0A63
0, //U+0A64
0, //U+0A65
UNICODE_Nd, //U+0A66
UNICODE_Nd, //U+0A67
UNICODE_Nd, //U+0A68
UNICODE_Nd, //U+0A69
UNICODE_Nd, //U+0A6A
UNICODE_Nd, //U+0A6B
UNICODE_Nd, //U+0A6C
UNICODE_Nd, //U+0A6D
UNICODE_Nd, //U+0A6E
UNICODE_Nd, //U+0A6F
UNICODE_Mn, //U+0A70
UNICODE_Mn, //U+0A71
UNICODE_Lo, //U+0A72
UNICODE_Lo, //U+0A73
UNICODE_Lo, //U+0A74
UNICODE_Mn, //U+0A75
0, //U+0A76
0, //U+0A77
0, //U+0A78
0, //U+0A79
0, //U+0A7A
0, //U+0A7B
0, //U+0A7C
0, //U+0A7D
0, //U+0A7E
0, //U+0A7F
0, //U+0A80
UNICODE_Mn, //U+0A81
UNICODE_Mn, //U+0A82
UNICODE_Mc, //U+0A83
0, //U+0A84
UNICODE_Lo, //U+0A85
UNICODE_Lo, //U+0A86
UNICODE_Lo, //U+0A87
UNICODE_Lo, //U+0A88
UNICODE_Lo, //U+0A89
UNICODE_Lo, //U+0A8A
UNICODE_Lo, //U+0A8B
UNICODE_Lo, //U+0A8C
UNICODE_Lo, //U+0A8D
0, //U+0A8E
UNICODE_Lo, //U+0A8F
UNICODE_Lo, //U+0A90
UNICODE_Lo, //U+0A91
0, //U+0A92
UNICODE_Lo, //U+0A93
UNICODE_Lo, //U+0A94
UNICODE_Lo, //U+0A95
UNICODE_Lo, //U+0A96
UNICODE_Lo, //U+0A97
UNICODE_Lo, //U+0A98
UNICODE_Lo, //U+0A99
UNICODE_Lo, //U+0A9A
UNICODE_Lo, //U+0A9B
UNICODE_Lo, //U+0A9C
UNICODE_Lo, //U+0A9D
UNICODE_Lo, //U+0A9E
UNICODE_Lo, //U+0A9F
UNICODE_Lo, //U+0AA0
UNICODE_Lo, //U+0AA1
UNICODE_Lo, //U+0AA2
UNICODE_Lo, //U+0AA3
UNICODE_Lo, //U+0AA4
UNICODE_Lo, //U+0AA5
UNICODE_Lo, //U+0AA6
UNICODE_Lo, //U+0AA7
UNICODE_Lo, //U+0AA8
0, //U+0AA9
UNICODE_Lo, //U+0AAA
UNICODE_Lo, //U+0AAB
UNICODE_Lo, //U+0AAC
UNICODE_Lo, //U+0AAD
UNICODE_Lo, //U+0AAE
UNICODE_Lo, //U+0AAF
UNICODE_Lo, //U+0AB0
0, //U+0AB1
UNICODE_Lo, //U+0AB2
UNICODE_Lo, //U+0AB3
0, //U+0AB4
UNICODE_Lo, //U+0AB5
UNICODE_Lo, //U+0AB6
UNICODE_Lo, //U+0AB7
UNICODE_Lo, //U+0AB8
UNICODE_Lo, //U+0AB9
0, //U+0ABA
0, //U+0ABB
UNICODE_Mn, //U+0ABC
UNICODE_Lo, //U+0ABD
UNICODE_Mc, //U+0ABE
UNICODE_Mc, //U+0ABF
UNICODE_Mc, //U+0AC0
UNICODE_Mn, //U+0AC1
UNICODE_Mn, //U+0AC2
UNICODE_Mn, //U+0AC3
UNICODE_Mn, //U+0AC4
UNICODE_Mn, //U+0AC5
0, //U+0AC6
UNICODE_Mn, //U+0AC7
UNICODE_Mn, //U+0AC8
UNICODE_Mc, //U+0AC9
0, //U+0ACA
UNICODE_Mc, //U+0ACB
UNICODE_Mc, //U+0ACC
UNICODE_Mn, //U+0ACD
0, //U+0ACE
0, //U+0ACF
UNICODE_Lo, //U+0AD0
0, //U+0AD1
0, //U+0AD2
0, //U+0AD3
0, //U+0AD4
0, //U+0AD5
0, //U+0AD6
0, //U+0AD7
0, //U+0AD8
0, //U+0AD9
0, //U+0ADA
0, //U+0ADB
0, //U+0ADC
0, //U+0ADD
0, //U+0ADE
0, //U+0ADF
UNICODE_Lo, //U+0AE0
UNICODE_Lo, //U+0AE1
UNICODE_Mn, //U+0AE2
UNICODE_Mn, //U+0AE3
0, //U+0AE4
0, //U+0AE5
UNICODE_Nd, //U+0AE6
UNICODE_Nd, //U+0AE7
UNICODE_Nd, //U+0AE8
UNICODE_Nd, //U+0AE9
UNICODE_Nd, //U+0AEA
UNICODE_Nd, //U+0AEB
UNICODE_Nd, //U+0AEC
UNICODE_Nd, //U+0AED
UNICODE_Nd, //U+0AEE
UNICODE_Nd, //U+0AEF
0, //U+0AF0
UNICODE_Sc, //U+0AF1
0, //U+0AF2
0, //U+0AF3
0, //U+0AF4
0, //U+0AF5
0, //U+0AF6
0, //U+0AF7
0, //U+0AF8
0, //U+0AF9
0, //U+0AFA
0, //U+0AFB
0, //U+0AFC
0, //U+0AFD
0, //U+0AFE
0, //U+0AFF
0, //U+0B00
UNICODE_Mn, //U+0B01
UNICODE_Mc, //U+0B02
UNICODE_Mc, //U+0B03
0, //U+0B04
UNICODE_Lo, //U+0B05
UNICODE_Lo, //U+0B06
UNICODE_Lo, //U+0B07
UNICODE_Lo, //U+0B08
UNICODE_Lo, //U+0B09
UNICODE_Lo, //U+0B0A
UNICODE_Lo, //U+0B0B
UNICODE_Lo, //U+0B0C
0, //U+0B0D
0, //U+0B0E
UNICODE_Lo, //U+0B0F
UNICODE_Lo, //U+0B10
0, //U+0B11
0, //U+0B12
UNICODE_Lo, //U+0B13
UNICODE_Lo, //U+0B14
UNICODE_Lo, //U+0B15
UNICODE_Lo, //U+0B16
UNICODE_Lo, //U+0B17
UNICODE_Lo, //U+0B18
UNICODE_Lo, //U+0B19
UNICODE_Lo, //U+0B1A
UNICODE_Lo, //U+0B1B
UNICODE_Lo, //U+0B1C
UNICODE_Lo, //U+0B1D
UNICODE_Lo, //U+0B1E
UNICODE_Lo, //U+0B1F
UNICODE_Lo, //U+0B20
UNICODE_Lo, //U+0B21
UNICODE_Lo, //U+0B22
UNICODE_Lo, //U+0B23
UNICODE_Lo, //U+0B24
UNICODE_Lo, //U+0B25
UNICODE_Lo, //U+0B26
UNICODE_Lo, //U+0B27
UNICODE_Lo, //U+0B28
0, //U+0B29
UNICODE_Lo, //U+0B2A
UNICODE_Lo, //U+0B2B
UNICODE_Lo, //U+0B2C
UNICODE_Lo, //U+0B2D
UNICODE_Lo, //U+0B2E
UNICODE_Lo, //U+0B2F
UNICODE_Lo, //U+0B30
0, //U+0B31
UNICODE_Lo, //U+0B32
UNICODE_Lo, //U+0B33
0, //U+0B34
UNICODE_Lo, //U+0B35
UNICODE_Lo, //U+0B36
UNICODE_Lo, //U+0B37
UNICODE_Lo, //U+0B38
UNICODE_Lo, //U+0B39
0, //U+0B3A
0, //U+0B3B
UNICODE_Mn, //U+0B3C
UNICODE_Lo, //U+0B3D
UNICODE_Mc, //U+0B3E
UNICODE_Mn, //U+0B3F
UNICODE_Mc, //U+0B40
UNICODE_Mn, //U+0B41
UNICODE_Mn, //U+0B42
UNICODE_Mn, //U+0B43
UNICODE_Mn, //U+0B44
0, //U+0B45
0, //U+0B46
UNICODE_Mc, //U+0B47
UNICODE_Mc, //U+0B48
0, //U+0B49
0, //U+0B4A
UNICODE_Mc, //U+0B4B
UNICODE_Mc, //U+0B4C
UNICODE_Mn, //U+0B4D
0, //U+0B4E
0, //U+0B4F
0, //U+0B50
0, //U+0B51
0, //U+0B52
0, //U+0B53
0, //U+0B54
0, //U+0B55
UNICODE_Mn, //U+0B56
UNICODE_Mc, //U+0B57
0, //U+0B58
0, //U+0B59
0, //U+0B5A
0, //U+0B5B
UNICODE_Lo, //U+0B5C
UNICODE_Lo, //U+0B5D
0, //U+0B5E
UNICODE_Lo, //U+0B5F
UNICODE_Lo, //U+0B60
UNICODE_Lo, //U+0B61
UNICODE_Mn, //U+0B62
UNICODE_Mn, //U+0B63
0, //U+0B64
0, //U+0B65
UNICODE_Nd, //U+0B66
UNICODE_Nd, //U+0B67
UNICODE_Nd, //U+0B68
UNICODE_Nd, //U+0B69
UNICODE_Nd, //U+0B6A
UNICODE_Nd, //U+0B6B
UNICODE_Nd, //U+0B6C
UNICODE_Nd, //U+0B6D
UNICODE_Nd, //U+0B6E
UNICODE_Nd, //U+0B6F
UNICODE_So, //U+0B70
UNICODE_Lo, //U+0B71
UNICODE_No, //U+0B72
UNICODE_No, //U+0B73
UNICODE_No, //U+0B74
UNICODE_No, //U+0B75
UNICODE_No, //U+0B76
UNICODE_No, //U+0B77
0, //U+0B78
0, //U+0B79
0, //U+0B7A
0, //U+0B7B
0, //U+0B7C
0, //U+0B7D
0, //U+0B7E
0, //U+0B7F
0, //U+0B80
0, //U+0B81
UNICODE_Mn, //U+0B82
UNICODE_Lo, //U+0B83
0, //U+0B84
UNICODE_Lo, //U+0B85
UNICODE_Lo, //U+0B86
UNICODE_Lo, //U+0B87
UNICODE_Lo, //U+0B88
UNICODE_Lo, //U+0B89
UNICODE_Lo, //U+0B8A
0, //U+0B8B
0, //U+0B8C
0, //U+0B8D
UNICODE_Lo, //U+0B8E
UNICODE_Lo, //U+0B8F
UNICODE_Lo, //U+0B90
0, //U+0B91
UNICODE_Lo, //U+0B92
UNICODE_Lo, //U+0B93
UNICODE_Lo, //U+0B94
UNICODE_Lo, //U+0B95
0, //U+0B96
0, //U+0B97
0, //U+0B98
UNICODE_Lo, //U+0B99
UNICODE_Lo, //U+0B9A
0, //U+0B9B
UNICODE_Lo, //U+0B9C
0, //U+0B9D
UNICODE_Lo, //U+0B9E
UNICODE_Lo, //U+0B9F
0, //U+0BA0
0, //U+0BA1
0, //U+0BA2
UNICODE_Lo, //U+0BA3
UNICODE_Lo, //U+0BA4
0, //U+0BA5
0, //U+0BA6
0, //U+0BA7
UNICODE_Lo, //U+0BA8
UNICODE_Lo, //U+0BA9
UNICODE_Lo, //U+0BAA
0, //U+0BAB
0, //U+0BAC
0, //U+0BAD
UNICODE_Lo, //U+0BAE
UNICODE_Lo, //U+0BAF
UNICODE_Lo, //U+0BB0
UNICODE_Lo, //U+0BB1
UNICODE_Lo, //U+0BB2
UNICODE_Lo, //U+0BB3
UNICODE_Lo, //U+0BB4
UNICODE_Lo, //U+0BB5
UNICODE_Lo, //U+0BB6
UNICODE_Lo, //U+0BB7
UNICODE_Lo, //U+0BB8
UNICODE_Lo, //U+0BB9
0, //U+0BBA
0, //U+0BBB
0, //U+0BBC
0, //U+0BBD
UNICODE_Mc, //U+0BBE
UNICODE_Mc, //U+0BBF
UNICODE_Mn, //U+0BC0
UNICODE_Mc, //U+0BC1
UNICODE_Mc, //U+0BC2
0, //U+0BC3
0, //U+0BC4
0, //U+0BC5
UNICODE_Mc, //U+0BC6
UNICODE_Mc, //U+0BC7
UNICODE_Mc, //U+0BC8
0, //U+0BC9
UNICODE_Mc, //U+0BCA
UNICODE_Mc, //U+0BCB
UNICODE_Mc, //U+0BCC
UNICODE_Mn, //U+0BCD
0, //U+0BCE
0, //U+0BCF
UNICODE_Lo, //U+0BD0
0, //U+0BD1
0, //U+0BD2
0, //U+0BD3
0, //U+0BD4
0, //U+0BD5
0, //U+0BD6
UNICODE_Mc, //U+0BD7
0, //U+0BD8
0, //U+0BD9
0, //U+0BDA
0, //U+0BDB
0, //U+0BDC
0, //U+0BDD
0, //U+0BDE
0, //U+0BDF
0, //U+0BE0
0, //U+0BE1
0, //U+0BE2
0, //U+0BE3
0, //U+0BE4
0, //U+0BE5
UNICODE_Nd, //U+0BE6
UNICODE_Nd, //U+0BE7
UNICODE_Nd, //U+0BE8
UNICODE_Nd, //U+0BE9
UNICODE_Nd, //U+0BEA
UNICODE_Nd, //U+0BEB
UNICODE_Nd, //U+0BEC
UNICODE_Nd, //U+0BED
UNICODE_Nd, //U+0BEE
UNICODE_Nd, //U+0BEF
UNICODE_No, //U+0BF0
UNICODE_No, //U+0BF1
UNICODE_No, //U+0BF2
UNICODE_So, //U+0BF3
UNICODE_So, //U+0BF4
UNICODE_So, //U+0BF5
UNICODE_So, //U+0BF6
UNICODE_So, //U+0BF7
UNICODE_So, //U+0BF8
UNICODE_Sc, //U+0BF9
UNICODE_So, //U+0BFA
0, //U+0BFB
0, //U+0BFC
0, //U+0BFD
0, //U+0BFE
0, //U+0BFF
0, //U+0C00
UNICODE_Mc, //U+0C01
UNICODE_Mc, //U+0C02
UNICODE_Mc, //U+0C03
0, //U+0C04
UNICODE_Lo, //U+0C05
UNICODE_Lo, //U+0C06
UNICODE_Lo, //U+0C07
UNICODE_Lo, //U+0C08
UNICODE_Lo, //U+0C09
UNICODE_Lo, //U+0C0A
UNICODE_Lo, //U+0C0B
UNICODE_Lo, //U+0C0C
0, //U+0C0D
UNICODE_Lo, //U+0C0E
UNICODE_Lo, //U+0C0F
UNICODE_Lo, //U+0C10
0, //U+0C11
UNICODE_Lo, //U+0C12
UNICODE_Lo, //U+0C13
UNICODE_Lo, //U+0C14
UNICODE_Lo, //U+0C15
UNICODE_Lo, //U+0C16
UNICODE_Lo, //U+0C17
UNICODE_Lo, //U+0C18
UNICODE_Lo, //U+0C19
UNICODE_Lo, //U+0C1A
UNICODE_Lo, //U+0C1B
UNICODE_Lo, //U+0C1C
UNICODE_Lo, //U+0C1D
UNICODE_Lo, //U+0C1E
UNICODE_Lo, //U+0C1F
UNICODE_Lo, //U+0C20
UNICODE_Lo, //U+0C21
UNICODE_Lo, //U+0C22
UNICODE_Lo, //U+0C23
UNICODE_Lo, //U+0C24
UNICODE_Lo, //U+0C25
UNICODE_Lo, //U+0C26
UNICODE_Lo, //U+0C27
UNICODE_Lo, //U+0C28
0, //U+0C29
UNICODE_Lo, //U+0C2A
UNICODE_Lo, //U+0C2B
UNICODE_Lo, //U+0C2C
UNICODE_Lo, //U+0C2D
UNICODE_Lo, //U+0C2E
UNICODE_Lo, //U+0C2F
UNICODE_Lo, //U+0C30
UNICODE_Lo, //U+0C31
UNICODE_Lo, //U+0C32
UNICODE_Lo, //U+0C33
0, //U+0C34
UNICODE_Lo, //U+0C35
UNICODE_Lo, //U+0C36
UNICODE_Lo, //U+0C37
UNICODE_Lo, //U+0C38
UNICODE_Lo, //U+0C39
0, //U+0C3A
0, //U+0C3B
0, //U+0C3C
UNICODE_Lo, //U+0C3D
UNICODE_Mn, //U+0C3E
UNICODE_Mn, //U+0C3F
UNICODE_Mn, //U+0C40
UNICODE_Mc, //U+0C41
UNICODE_Mc, //U+0C42
UNICODE_Mc, //U+0C43
UNICODE_Mc, //U+0C44
0, //U+0C45
UNICODE_Mn, //U+0C46
UNICODE_Mn, //U+0C47
UNICODE_Mn, //U+0C48
0, //U+0C49
UNICODE_Mn, //U+0C4A
UNICODE_Mn, //U+0C4B
UNICODE_Mn, //U+0C4C
UNICODE_Mn, //U+0C4D
0, //U+0C4E
0, //U+0C4F
0, //U+0C50
0, //U+0C51
0, //U+0C52
0, //U+0C53
0, //U+0C54
UNICODE_Mn, //U+0C55
UNICODE_Mn, //U+0C56
0, //U+0C57
UNICODE_Lo, //U+0C58
UNICODE_Lo, //U+0C59
0, //U+0C5A
0, //U+0C5B
0, //U+0C5C
0, //U+0C5D
0, //U+0C5E
0, //U+0C5F
UNICODE_Lo, //U+0C60
UNICODE_Lo, //U+0C61
UNICODE_Mn, //U+0C62
UNICODE_Mn, //U+0C63
0, //U+0C64
0, //U+0C65
UNICODE_Nd, //U+0C66
UNICODE_Nd, //U+0C67
UNICODE_Nd, //U+0C68
UNICODE_Nd, //U+0C69
UNICODE_Nd, //U+0C6A
UNICODE_Nd, //U+0C6B
UNICODE_Nd, //U+0C6C
UNICODE_Nd, //U+0C6D
UNICODE_Nd, //U+0C6E
UNICODE_Nd, //U+0C6F
0, //U+0C70
0, //U+0C71
0, //U+0C72
0, //U+0C73
0, //U+0C74
0, //U+0C75
0, //U+0C76
0, //U+0C77
UNICODE_No, //U+0C78
UNICODE_No, //U+0C79
UNICODE_No, //U+0C7A
UNICODE_No, //U+0C7B
UNICODE_No, //U+0C7C
UNICODE_No, //U+0C7D
UNICODE_No, //U+0C7E
UNICODE_So, //U+0C7F
0, //U+0C80
0, //U+0C81
UNICODE_Mc, //U+0C82
UNICODE_Mc, //U+0C83
0, //U+0C84
UNICODE_Lo, //U+0C85
UNICODE_Lo, //U+0C86
UNICODE_Lo, //U+0C87
UNICODE_Lo, //U+0C88
UNICODE_Lo, //U+0C89
UNICODE_Lo, //U+0C8A
UNICODE_Lo, //U+0C8B
UNICODE_Lo, //U+0C8C
0, //U+0C8D
UNICODE_Lo, //U+0C8E
UNICODE_Lo, //U+0C8F
UNICODE_Lo, //U+0C90
0, //U+0C91
UNICODE_Lo, //U+0C92
UNICODE_Lo, //U+0C93
UNICODE_Lo, //U+0C94
UNICODE_Lo, //U+0C95
UNICODE_Lo, //U+0C96
UNICODE_Lo, //U+0C97
UNICODE_Lo, //U+0C98
UNICODE_Lo, //U+0C99
UNICODE_Lo, //U+0C9A
UNICODE_Lo, //U+0C9B
UNICODE_Lo, //U+0C9C
UNICODE_Lo, //U+0C9D
UNICODE_Lo, //U+0C9E
UNICODE_Lo, //U+0C9F
UNICODE_Lo, //U+0CA0
UNICODE_Lo, //U+0CA1
UNICODE_Lo, //U+0CA2
UNICODE_Lo, //U+0CA3
UNICODE_Lo, //U+0CA4
UNICODE_Lo, //U+0CA5
UNICODE_Lo, //U+0CA6
UNICODE_Lo, //U+0CA7
UNICODE_Lo, //U+0CA8
0, //U+0CA9
UNICODE_Lo, //U+0CAA
UNICODE_Lo, //U+0CAB
UNICODE_Lo, //U+0CAC
UNICODE_Lo, //U+0CAD
UNICODE_Lo, //U+0CAE
UNICODE_Lo, //U+0CAF
UNICODE_Lo, //U+0CB0
UNICODE_Lo, //U+0CB1
UNICODE_Lo, //U+0CB2
UNICODE_Lo, //U+0CB3
0, //U+0CB4
UNICODE_Lo, //U+0CB5
UNICODE_Lo, //U+0CB6
UNICODE_Lo, //U+0CB7
UNICODE_Lo, //U+0CB8
UNICODE_Lo, //U+0CB9
0, //U+0CBA
0, //U+0CBB
UNICODE_Mn, //U+0CBC
UNICODE_Lo, //U+0CBD
UNICODE_Mc, //U+0CBE
UNICODE_Mn, //U+0CBF
UNICODE_Mc, //U+0CC0
UNICODE_Mc, //U+0CC1
UNICODE_Mc, //U+0CC2
UNICODE_Mc, //U+0CC3
UNICODE_Mc, //U+0CC4
0, //U+0CC5
UNICODE_Mn, //U+0CC6
UNICODE_Mc, //U+0CC7
UNICODE_Mc, //U+0CC8
0, //U+0CC9
UNICODE_Mc, //U+0CCA
UNICODE_Mc, //U+0CCB
UNICODE_Mn, //U+0CCC
UNICODE_Mn, //U+0CCD
0, //U+0CCE
0, //U+0CCF
0, //U+0CD0
0, //U+0CD1
0, //U+0CD2
0, //U+0CD3
0, //U+0CD4
UNICODE_Mc, //U+0CD5
UNICODE_Mc, //U+0CD6
0, //U+0CD7
0, //U+0CD8
0, //U+0CD9
0, //U+0CDA
0, //U+0CDB
0, //U+0CDC
0, //U+0CDD
UNICODE_Lo, //U+0CDE
0, //U+0CDF
UNICODE_Lo, //U+0CE0
UNICODE_Lo, //U+0CE1
UNICODE_Mn, //U+0CE2
UNICODE_Mn, //U+0CE3
0, //U+0CE4
0, //U+0CE5
UNICODE_Nd, //U+0CE6
UNICODE_Nd, //U+0CE7
UNICODE_Nd, //U+0CE8
UNICODE_Nd, //U+0CE9
UNICODE_Nd, //U+0CEA
UNICODE_Nd, //U+0CEB
UNICODE_Nd, //U+0CEC
UNICODE_Nd, //U+0CED
UNICODE_Nd, //U+0CEE
UNICODE_Nd, //U+0CEF
0, //U+0CF0
UNICODE_Lo, //U+0CF1
UNICODE_Lo, //U+0CF2
0, //U+0CF3
0, //U+0CF4
0, //U+0CF5
0, //U+0CF6
0, //U+0CF7
0, //U+0CF8
0, //U+0CF9
0, //U+0CFA
0, //U+0CFB
0, //U+0CFC
0, //U+0CFD
0, //U+0CFE
0, //U+0CFF
0, //U+0D00
0, //U+0D01
UNICODE_Mc, //U+0D02
UNICODE_Mc, //U+0D03
0, //U+0D04
UNICODE_Lo, //U+0D05
UNICODE_Lo, //U+0D06
UNICODE_Lo, //U+0D07
UNICODE_Lo, //U+0D08
UNICODE_Lo, //U+0D09
UNICODE_Lo, //U+0D0A
UNICODE_Lo, //U+0D0B
UNICODE_Lo, //U+0D0C
0, //U+0D0D
UNICODE_Lo, //U+0D0E
UNICODE_Lo, //U+0D0F
UNICODE_Lo, //U+0D10
0, //U+0D11
UNICODE_Lo, //U+0D12
UNICODE_Lo, //U+0D13
UNICODE_Lo, //U+0D14
UNICODE_Lo, //U+0D15
UNICODE_Lo, //U+0D16
UNICODE_Lo, //U+0D17
UNICODE_Lo, //U+0D18
UNICODE_Lo, //U+0D19
UNICODE_Lo, //U+0D1A
UNICODE_Lo, //U+0D1B
UNICODE_Lo, //U+0D1C
UNICODE_Lo, //U+0D1D
UNICODE_Lo, //U+0D1E
UNICODE_Lo, //U+0D1F
UNICODE_Lo, //U+0D20
UNICODE_Lo, //U+0D21
UNICODE_Lo, //U+0D22
UNICODE_Lo, //U+0D23
UNICODE_Lo, //U+0D24
UNICODE_Lo, //U+0D25
UNICODE_Lo, //U+0D26
UNICODE_Lo, //U+0D27
UNICODE_Lo, //U+0D28
UNICODE_Lo, //U+0D29
UNICODE_Lo, //U+0D2A
UNICODE_Lo, //U+0D2B
UNICODE_Lo, //U+0D2C
UNICODE_Lo, //U+0D2D
UNICODE_Lo, //U+0D2E
UNICODE_Lo, //U+0D2F
UNICODE_Lo, //U+0D30
UNICODE_Lo, //U+0D31
UNICODE_Lo, //U+0D32
UNICODE_Lo, //U+0D33
UNICODE_Lo, //U+0D34
UNICODE_Lo, //U+0D35
UNICODE_Lo, //U+0D36
UNICODE_Lo, //U+0D37
UNICODE_Lo, //U+0D38
UNICODE_Lo, //U+0D39
UNICODE_Lo, //U+0D3A
0, //U+0D3B
0, //U+0D3C
UNICODE_Lo, //U+0D3D
UNICODE_Mc, //U+0D3E
UNICODE_Mc, //U+0D3F
UNICODE_Mc, //U+0D40
UNICODE_Mn, //U+0D41
UNICODE_Mn, //U+0D42
UNICODE_Mn, //U+0D43
UNICODE_Mn, //U+0D44
0, //U+0D45
UNICODE_Mc, //U+0D46
UNICODE_Mc, //U+0D47
UNICODE_Mc, //U+0D48
0, //U+0D49
UNICODE_Mc, //U+0D4A
UNICODE_Mc, //U+0D4B
UNICODE_Mc, //U+0D4C
UNICODE_Mn, //U+0D4D
UNICODE_Lo, //U+0D4E
0, //U+0D4F
0, //U+0D50
0, //U+0D51
0, //U+0D52
0, //U+0D53
0, //U+0D54
0, //U+0D55
0, //U+0D56
UNICODE_Mc, //U+0D57
0, //U+0D58
0, //U+0D59
0, //U+0D5A
0, //U+0D5B
0, //U+0D5C
0, //U+0D5D
0, //U+0D5E
0, //U+0D5F
UNICODE_Lo, //U+0D60
UNICODE_Lo, //U+0D61
UNICODE_Mn, //U+0D62
UNICODE_Mn, //U+0D63
0, //U+0D64
0, //U+0D65
UNICODE_Nd, //U+0D66
UNICODE_Nd, //U+0D67
UNICODE_Nd, //U+0D68
UNICODE_Nd, //U+0D69
UNICODE_Nd, //U+0D6A
UNICODE_Nd, //U+0D6B
UNICODE_Nd, //U+0D6C
UNICODE_Nd, //U+0D6D
UNICODE_Nd, //U+0D6E
UNICODE_Nd, //U+0D6F
UNICODE_No, //U+0D70
UNICODE_No, //U+0D71
UNICODE_No, //U+0D72
UNICODE_No, //U+0D73
UNICODE_No, //U+0D74
UNICODE_No, //U+0D75
0, //U+0D76
0, //U+0D77
0, //U+0D78
UNICODE_So, //U+0D79
UNICODE_Lo, //U+0D7A
UNICODE_Lo, //U+0D7B
UNICODE_Lo, //U+0D7C
UNICODE_Lo, //U+0D7D
UNICODE_Lo, //U+0D7E
UNICODE_Lo, //U+0D7F
0, //U+0D80
0, //U+0D81
UNICODE_Mc, //U+0D82
UNICODE_Mc, //U+0D83
0, //U+0D84
UNICODE_Lo, //U+0D85
UNICODE_Lo, //U+0D86
UNICODE_Lo, //U+0D87
UNICODE_Lo, //U+0D88
UNICODE_Lo, //U+0D89
UNICODE_Lo, //U+0D8A
UNICODE_Lo, //U+0D8B
UNICODE_Lo, //U+0D8C
UNICODE_Lo, //U+0D8D
UNICODE_Lo, //U+0D8E
UNICODE_Lo, //U+0D8F
UNICODE_Lo, //U+0D90
UNICODE_Lo, //U+0D91
UNICODE_Lo, //U+0D92
UNICODE_Lo, //U+0D93
UNICODE_Lo, //U+0D94
UNICODE_Lo, //U+0D95
UNICODE_Lo, //U+0D96
0, //U+0D97
0, //U+0D98
0, //U+0D99
UNICODE_Lo, //U+0D9A
UNICODE_Lo, //U+0D9B
UNICODE_Lo, //U+0D9C
UNICODE_Lo, //U+0D9D
UNICODE_Lo, //U+0D9E
UNICODE_Lo, //U+0D9F
UNICODE_Lo, //U+0DA0
UNICODE_Lo, //U+0DA1
UNICODE_Lo, //U+0DA2
UNICODE_Lo, //U+0DA3
UNICODE_Lo, //U+0DA4
UNICODE_Lo, //U+0DA5
UNICODE_Lo, //U+0DA6
UNICODE_Lo, //U+0DA7
UNICODE_Lo, //U+0DA8
UNICODE_Lo, //U+0DA9
UNICODE_Lo, //U+0DAA
UNICODE_Lo, //U+0DAB
UNICODE_Lo, //U+0DAC
UNICODE_Lo, //U+0DAD
UNICODE_Lo, //U+0DAE
UNICODE_Lo, //U+0DAF
UNICODE_Lo, //U+0DB0
UNICODE_Lo, //U+0DB1
0, //U+0DB2
UNICODE_Lo, //U+0DB3
UNICODE_Lo, //U+0DB4
UNICODE_Lo, //U+0DB5
UNICODE_Lo, //U+0DB6
UNICODE_Lo, //U+0DB7
UNICODE_Lo, //U+0DB8
UNICODE_Lo, //U+0DB9
UNICODE_Lo, //U+0DBA
UNICODE_Lo, //U+0DBB
0, //U+0DBC
UNICODE_Lo, //U+0DBD
0, //U+0DBE
0, //U+0DBF
UNICODE_Lo, //U+0DC0
UNICODE_Lo, //U+0DC1
UNICODE_Lo, //U+0DC2
UNICODE_Lo, //U+0DC3
UNICODE_Lo, //U+0DC4
UNICODE_Lo, //U+0DC5
UNICODE_Lo, //U+0DC6
0, //U+0DC7
0, //U+0DC8
0, //U+0DC9
UNICODE_Mn, //U+0DCA
0, //U+0DCB
0, //U+0DCC
0, //U+0DCD
0, //U+0DCE
UNICODE_Mc, //U+0DCF
UNICODE_Mc, //U+0DD0
UNICODE_Mc, //U+0DD1
UNICODE_Mn, //U+0DD2
UNICODE_Mn, //U+0DD3
UNICODE_Mn, //U+0DD4
0, //U+0DD5
UNICODE_Mn, //U+0DD6
0, //U+0DD7
UNICODE_Mc, //U+0DD8
UNICODE_Mc, //U+0DD9
UNICODE_Mc, //U+0DDA
UNICODE_Mc, //U+0DDB
UNICODE_Mc, //U+0DDC
UNICODE_Mc, //U+0DDD
UNICODE_Mc, //U+0DDE
UNICODE_Mc, //U+0DDF
0, //U+0DE0
0, //U+0DE1
0, //U+0DE2
0, //U+0DE3
0, //U+0DE4
0, //U+0DE5
0, //U+0DE6
0, //U+0DE7
0, //U+0DE8
0, //U+0DE9
0, //U+0DEA
0, //U+0DEB
0, //U+0DEC
0, //U+0DED
0, //U+0DEE
0, //U+0DEF
0, //U+0DF0
0, //U+0DF1
UNICODE_Mc, //U+0DF2
UNICODE_Mc, //U+0DF3
UNICODE_Po, //U+0DF4
0, //U+0DF5
0, //U+0DF6
0, //U+0DF7
0, //U+0DF8
0, //U+0DF9
0, //U+0DFA
0, //U+0DFB
0, //U+0DFC
0, //U+0DFD
0, //U+0DFE
0, //U+0DFF
0, //U+0E00
UNICODE_Lo, //U+0E01
UNICODE_Lo, //U+0E02
UNICODE_Lo, //U+0E03
UNICODE_Lo, //U+0E04
UNICODE_Lo, //U+0E05
UNICODE_Lo, //U+0E06
UNICODE_Lo, //U+0E07
UNICODE_Lo, //U+0E08
UNICODE_Lo, //U+0E09
UNICODE_Lo, //U+0E0A
UNICODE_Lo, //U+0E0B
UNICODE_Lo, //U+0E0C
UNICODE_Lo, //U+0E0D
UNICODE_Lo, //U+0E0E
UNICODE_Lo, //U+0E0F
UNICODE_Lo, //U+0E10
UNICODE_Lo, //U+0E11
UNICODE_Lo, //U+0E12
UNICODE_Lo, //U+0E13
UNICODE_Lo, //U+0E14
UNICODE_Lo, //U+0E15
UNICODE_Lo, //U+0E16
UNICODE_Lo, //U+0E17
UNICODE_Lo, //U+0E18
UNICODE_Lo, //U+0E19
UNICODE_Lo, //U+0E1A
UNICODE_Lo, //U+0E1B
UNICODE_Lo, //U+0E1C
UNICODE_Lo, //U+0E1D
UNICODE_Lo, //U+0E1E
UNICODE_Lo, //U+0E1F
UNICODE_Lo, //U+0E20
UNICODE_Lo, //U+0E21
UNICODE_Lo, //U+0E22
UNICODE_Lo, //U+0E23
UNICODE_Lo, //U+0E24
UNICODE_Lo, //U+0E25
UNICODE_Lo, //U+0E26
UNICODE_Lo, //U+0E27
UNICODE_Lo, //U+0E28
UNICODE_Lo, //U+0E29
UNICODE_Lo, //U+0E2A
UNICODE_Lo, //U+0E2B
UNICODE_Lo, //U+0E2C
UNICODE_Lo, //U+0E2D
UNICODE_Lo, //U+0E2E
UNICODE_Lo, //U+0E2F
UNICODE_Lo, //U+0E30
UNICODE_Mn, //U+0E31
UNICODE_Lo, //U+0E32
UNICODE_Lo, //U+0E33
UNICODE_Mn, //U+0E34
UNICODE_Mn, //U+0E35
UNICODE_Mn, //U+0E36
UNICODE_Mn, //U+0E37
UNICODE_Mn, //U+0E38
UNICODE_Mn, //U+0E39
UNICODE_Mn, //U+0E3A
0, //U+0E3B
0, //U+0E3C
0, //U+0E3D
0, //U+0E3E
UNICODE_Sc, //U+0E3F
UNICODE_Lo, //U+0E40
UNICODE_Lo, //U+0E41
UNICODE_Lo, //U+0E42
UNICODE_Lo, //U+0E43
UNICODE_Lo, //U+0E44
UNICODE_Lo, //U+0E45
UNICODE_Lm, //U+0E46
UNICODE_Mn, //U+0E47
UNICODE_Mn, //U+0E48
UNICODE_Mn, //U+0E49
UNICODE_Mn, //U+0E4A
UNICODE_Mn, //U+0E4B
UNICODE_Mn, //U+0E4C
UNICODE_Mn, //U+0E4D
UNICODE_Mn, //U+0E4E
UNICODE_Po, //U+0E4F
UNICODE_Nd, //U+0E50
UNICODE_Nd, //U+0E51
UNICODE_Nd, //U+0E52
UNICODE_Nd, //U+0E53
UNICODE_Nd, //U+0E54
UNICODE_Nd, //U+0E55
UNICODE_Nd, //U+0E56
UNICODE_Nd, //U+0E57
UNICODE_Nd, //U+0E58
UNICODE_Nd, //U+0E59
UNICODE_Po, //U+0E5A
UNICODE_Po, //U+0E5B
0, //U+0E5C
0, //U+0E5D
0, //U+0E5E
0, //U+0E5F
0, //U+0E60
0, //U+0E61
0, //U+0E62
0, //U+0E63
0, //U+0E64
0, //U+0E65
0, //U+0E66
0, //U+0E67
0, //U+0E68
0, //U+0E69
0, //U+0E6A
0, //U+0E6B
0, //U+0E6C
0, //U+0E6D
0, //U+0E6E
0, //U+0E6F
0, //U+0E70
0, //U+0E71
0, //U+0E72
0, //U+0E73
0, //U+0E74
0, //U+0E75
0, //U+0E76
0, //U+0E77
0, //U+0E78
0, //U+0E79
0, //U+0E7A
0, //U+0E7B
0, //U+0E7C
0, //U+0E7D
0, //U+0E7E
0, //U+0E7F
0, //U+0E80
UNICODE_Lo, //U+0E81
UNICODE_Lo, //U+0E82
0, //U+0E83
UNICODE_Lo, //U+0E84
0, //U+0E85
0, //U+0E86
UNICODE_Lo, //U+0E87
UNICODE_Lo, //U+0E88
0, //U+0E89
UNICODE_Lo, //U+0E8A
0, //U+0E8B
0, //U+0E8C
UNICODE_Lo, //U+0E8D
0, //U+0E8E
0, //U+0E8F
0, //U+0E90
0, //U+0E91
0, //U+0E92
0, //U+0E93
UNICODE_Lo, //U+0E94
UNICODE_Lo, //U+0E95
UNICODE_Lo, //U+0E96
UNICODE_Lo, //U+0E97
0, //U+0E98
UNICODE_Lo, //U+0E99
UNICODE_Lo, //U+0E9A
UNICODE_Lo, //U+0E9B
UNICODE_Lo, //U+0E9C
UNICODE_Lo, //U+0E9D
UNICODE_Lo, //U+0E9E
UNICODE_Lo, //U+0E9F
0, //U+0EA0
UNICODE_Lo, //U+0EA1
UNICODE_Lo, //U+0EA2
UNICODE_Lo, //U+0EA3
0, //U+0EA4
UNICODE_Lo, //U+0EA5
0, //U+0EA6
UNICODE_Lo, //U+0EA7
0, //U+0EA8
0, //U+0EA9
UNICODE_Lo, //U+0EAA
UNICODE_Lo, //U+0EAB
0, //U+0EAC
UNICODE_Lo, //U+0EAD
UNICODE_Lo, //U+0EAE
UNICODE_Lo, //U+0EAF
UNICODE_Lo, //U+0EB0
UNICODE_Mn, //U+0EB1
UNICODE_Lo, //U+0EB2
UNICODE_Lo, //U+0EB3
UNICODE_Mn, //U+0EB4
UNICODE_Mn, //U+0EB5
UNICODE_Mn, //U+0EB6
UNICODE_Mn, //U+0EB7
UNICODE_Mn, //U+0EB8
UNICODE_Mn, //U+0EB9
0, //U+0EBA
UNICODE_Mn, //U+0EBB
UNICODE_Mn, //U+0EBC
UNICODE_Lo, //U+0EBD
0, //U+0EBE
0, //U+0EBF
UNICODE_Lo, //U+0EC0
UNICODE_Lo, //U+0EC1
UNICODE_Lo, //U+0EC2
UNICODE_Lo, //U+0EC3
UNICODE_Lo, //U+0EC4
0, //U+0EC5
UNICODE_Lm, //U+0EC6
0, //U+0EC7
UNICODE_Mn, //U+0EC8
UNICODE_Mn, //U+0EC9
UNICODE_Mn, //U+0ECA
UNICODE_Mn, //U+0ECB
UNICODE_Mn, //U+0ECC
UNICODE_Mn, //U+0ECD
0, //U+0ECE
0, //U+0ECF
UNICODE_Nd, //U+0ED0
UNICODE_Nd, //U+0ED1
UNICODE_Nd, //U+0ED2
UNICODE_Nd, //U+0ED3
UNICODE_Nd, //U+0ED4
UNICODE_Nd, //U+0ED5
UNICODE_Nd, //U+0ED6
UNICODE_Nd, //U+0ED7
UNICODE_Nd, //U+0ED8
UNICODE_Nd, //U+0ED9
0, //U+0EDA
0, //U+0EDB
UNICODE_Lo, //U+0EDC
UNICODE_Lo, //U+0EDD
0, //U+0EDE
0, //U+0EDF
0, //U+0EE0
0, //U+0EE1
0, //U+0EE2
0, //U+0EE3
0, //U+0EE4
0, //U+0EE5
0, //U+0EE6
0, //U+0EE7
0, //U+0EE8
0, //U+0EE9
0, //U+0EEA
0, //U+0EEB
0, //U+0EEC
0, //U+0EED
0, //U+0EEE
0, //U+0EEF
0, //U+0EF0
0, //U+0EF1
0, //U+0EF2
0, //U+0EF3
0, //U+0EF4
0, //U+0EF5
0, //U+0EF6
0, //U+0EF7
0, //U+0EF8
0, //U+0EF9
0, //U+0EFA
0, //U+0EFB
0, //U+0EFC
0, //U+0EFD
0, //U+0EFE
0, //U+0EFF
UNICODE_Lo, //U+0F00
UNICODE_So, //U+0F01
UNICODE_So, //U+0F02
UNICODE_So, //U+0F03
UNICODE_Po, //U+0F04
UNICODE_Po, //U+0F05
UNICODE_Po, //U+0F06
UNICODE_Po, //U+0F07
UNICODE_Po, //U+0F08
UNICODE_Po, //U+0F09
UNICODE_Po, //U+0F0A
UNICODE_Po, //U+0F0B
UNICODE_Po, //U+0F0C
UNICODE_Po, //U+0F0D
UNICODE_Po, //U+0F0E
UNICODE_Po, //U+0F0F
UNICODE_Po, //U+0F10
UNICODE_Po, //U+0F11
UNICODE_Po, //U+0F12
UNICODE_So, //U+0F13
UNICODE_So, //U+0F14
UNICODE_So, //U+0F15
UNICODE_So, //U+0F16
UNICODE_So, //U+0F17
UNICODE_Mn, //U+0F18
UNICODE_Mn, //U+0F19
UNICODE_So, //U+0F1A
UNICODE_So, //U+0F1B
UNICODE_So, //U+0F1C
UNICODE_So, //U+0F1D
UNICODE_So, //U+0F1E
UNICODE_So, //U+0F1F
UNICODE_Nd, //U+0F20
UNICODE_Nd, //U+0F21
UNICODE_Nd, //U+0F22
UNICODE_Nd, //U+0F23
UNICODE_Nd, //U+0F24
UNICODE_Nd, //U+0F25
UNICODE_Nd, //U+0F26
UNICODE_Nd, //U+0F27
UNICODE_Nd, //U+0F28
UNICODE_Nd, //U+0F29
UNICODE_No, //U+0F2A
UNICODE_No, //U+0F2B
UNICODE_No, //U+0F2C
UNICODE_No, //U+0F2D
UNICODE_No, //U+0F2E
UNICODE_No, //U+0F2F
UNICODE_No, //U+0F30
UNICODE_No, //U+0F31
UNICODE_No, //U+0F32
UNICODE_No, //U+0F33
UNICODE_So, //U+0F34
UNICODE_Mn, //U+0F35
UNICODE_So, //U+0F36
UNICODE_Mn, //U+0F37
UNICODE_So, //U+0F38
UNICODE_Mn, //U+0F39
UNICODE_Ps, //U+0F3A
UNICODE_Pe, //U+0F3B
UNICODE_Ps, //U+0F3C
UNICODE_Pe, //U+0F3D
UNICODE_Mc, //U+0F3E
UNICODE_Mc, //U+0F3F
UNICODE_Lo, //U+0F40
UNICODE_Lo, //U+0F41
UNICODE_Lo, //U+0F42
UNICODE_Lo, //U+0F43
UNICODE_Lo, //U+0F44
UNICODE_Lo, //U+0F45
UNICODE_Lo, //U+0F46
UNICODE_Lo, //U+0F47
0, //U+0F48
UNICODE_Lo, //U+0F49
UNICODE_Lo, //U+0F4A
UNICODE_Lo, //U+0F4B
UNICODE_Lo, //U+0F4C
UNICODE_Lo, //U+0F4D
UNICODE_Lo, //U+0F4E
UNICODE_Lo, //U+0F4F
UNICODE_Lo, //U+0F50
UNICODE_Lo, //U+0F51
UNICODE_Lo, //U+0F52
UNICODE_Lo, //U+0F53
UNICODE_Lo, //U+0F54
UNICODE_Lo, //U+0F55
UNICODE_Lo, //U+0F56
UNICODE_Lo, //U+0F57
UNICODE_Lo, //U+0F58
UNICODE_Lo, //U+0F59
UNICODE_Lo, //U+0F5A
UNICODE_Lo, //U+0F5B
UNICODE_Lo, //U+0F5C
UNICODE_Lo, //U+0F5D
UNICODE_Lo, //U+0F5E
UNICODE_Lo, //U+0F5F
UNICODE_Lo, //U+0F60
UNICODE_Lo, //U+0F61
UNICODE_Lo, //U+0F62
UNICODE_Lo, //U+0F63
UNICODE_Lo, //U+0F64
UNICODE_Lo, //U+0F65
UNICODE_Lo, //U+0F66
UNICODE_Lo, //U+0F67
UNICODE_Lo, //U+0F68
UNICODE_Lo, //U+0F69
UNICODE_Lo, //U+0F6A
UNICODE_Lo, //U+0F6B
UNICODE_Lo, //U+0F6C
0, //U+0F6D
0, //U+0F6E
0, //U+0F6F
0, //U+0F70
UNICODE_Mn, //U+0F71
UNICODE_Mn, //U+0F72
UNICODE_Mn, //U+0F73
UNICODE_Mn, //U+0F74
UNICODE_Mn, //U+0F75
UNICODE_Mn, //U+0F76
UNICODE_Mn, //U+0F77
UNICODE_Mn, //U+0F78
UNICODE_Mn, //U+0F79
UNICODE_Mn, //U+0F7A
UNICODE_Mn, //U+0F7B
UNICODE_Mn, //U+0F7C
UNICODE_Mn, //U+0F7D
UNICODE_Mn, //U+0F7E
UNICODE_Mc, //U+0F7F
UNICODE_Mn, //U+0F80
UNICODE_Mn, //U+0F81
UNICODE_Mn, //U+0F82
UNICODE_Mn, //U+0F83
UNICODE_Mn, //U+0F84
UNICODE_Po, //U+0F85
UNICODE_Mn, //U+0F86
UNICODE_Mn, //U+0F87
UNICODE_Lo, //U+0F88
UNICODE_Lo, //U+0F89
UNICODE_Lo, //U+0F8A
UNICODE_Lo, //U+0F8B
UNICODE_Lo, //U+0F8C
UNICODE_Mn, //U+0F8D
UNICODE_Mn, //U+0F8E
UNICODE_Mn, //U+0F8F
UNICODE_Mn, //U+0F90
UNICODE_Mn, //U+0F91
UNICODE_Mn, //U+0F92
UNICODE_Mn, //U+0F93
UNICODE_Mn, //U+0F94
UNICODE_Mn, //U+0F95
UNICODE_Mn, //U+0F96
UNICODE_Mn, //U+0F97
0, //U+0F98
UNICODE_Mn, //U+0F99
UNICODE_Mn, //U+0F9A
UNICODE_Mn, //U+0F9B
UNICODE_Mn, //U+0F9C
UNICODE_Mn, //U+0F9D
UNICODE_Mn, //U+0F9E
UNICODE_Mn, //U+0F9F
UNICODE_Mn, //U+0FA0
UNICODE_Mn, //U+0FA1
UNICODE_Mn, //U+0FA2
UNICODE_Mn, //U+0FA3
UNICODE_Mn, //U+0FA4
UNICODE_Mn, //U+0FA5
UNICODE_Mn, //U+0FA6
UNICODE_Mn, //U+0FA7
UNICODE_Mn, //U+0FA8
UNICODE_Mn, //U+0FA9
UNICODE_Mn, //U+0FAA
UNICODE_Mn, //U+0FAB
UNICODE_Mn, //U+0FAC
UNICODE_Mn, //U+0FAD
UNICODE_Mn, //U+0FAE
UNICODE_Mn, //U+0FAF
UNICODE_Mn, //U+0FB0
UNICODE_Mn, //U+0FB1
UNICODE_Mn, //U+0FB2
UNICODE_Mn, //U+0FB3
UNICODE_Mn, //U+0FB4
UNICODE_Mn, //U+0FB5
UNICODE_Mn, //U+0FB6
UNICODE_Mn, //U+0FB7
UNICODE_Mn, //U+0FB8
UNICODE_Mn, //U+0FB9
UNICODE_Mn, //U+0FBA
UNICODE_Mn, //U+0FBB
UNICODE_Mn, //U+0FBC
0, //U+0FBD
UNICODE_So, //U+0FBE
UNICODE_So, //U+0FBF
UNICODE_So, //U+0FC0
UNICODE_So, //U+0FC1
UNICODE_So, //U+0FC2
UNICODE_So, //U+0FC3
UNICODE_So, //U+0FC4
UNICODE_So, //U+0FC5
UNICODE_Mn, //U+0FC6
UNICODE_So, //U+0FC7
UNICODE_So, //U+0FC8
UNICODE_So, //U+0FC9
UNICODE_So, //U+0FCA
UNICODE_So, //U+0FCB
UNICODE_So, //U+0FCC
0, //U+0FCD
UNICODE_So, //U+0FCE
UNICODE_So, //U+0FCF
UNICODE_Po, //U+0FD0
UNICODE_Po, //U+0FD1
UNICODE_Po, //U+0FD2
UNICODE_Po, //U+0FD3
UNICODE_Po, //U+0FD4
UNICODE_So, //U+0FD5
UNICODE_So, //U+0FD6
UNICODE_So, //U+0FD7
UNICODE_So, //U+0FD8
UNICODE_Po, //U+0FD9
UNICODE_Po, //U+0FDA
0, //U+0FDB
0, //U+0FDC
0, //U+0FDD
0, //U+0FDE
0, //U+0FDF
0, //U+0FE0
0, //U+0FE1
0, //U+0FE2
0, //U+0FE3
0, //U+0FE4
0, //U+0FE5
0, //U+0FE6
0, //U+0FE7
0, //U+0FE8
0, //U+0FE9
0, //U+0FEA
0, //U+0FEB
0, //U+0FEC
0, //U+0FED
0, //U+0FEE
0, //U+0FEF
0, //U+0FF0
0, //U+0FF1
0, //U+0FF2
0, //U+0FF3
0, //U+0FF4
0, //U+0FF5
0, //U+0FF6
0, //U+0FF7
0, //U+0FF8
0, //U+0FF9
0, //U+0FFA
0, //U+0FFB
0, //U+0FFC
0, //U+0FFD
0, //U+0FFE
0, //U+0FFF
UNICODE_Lo, //U+1000
UNICODE_Lo, //U+1001
UNICODE_Lo, //U+1002
UNICODE_Lo, //U+1003
UNICODE_Lo, //U+1004
UNICODE_Lo, //U+1005
UNICODE_Lo, //U+1006
UNICODE_Lo, //U+1007
UNICODE_Lo, //U+1008
UNICODE_Lo, //U+1009
UNICODE_Lo, //U+100A
UNICODE_Lo, //U+100B
UNICODE_Lo, //U+100C
UNICODE_Lo, //U+100D
UNICODE_Lo, //U+100E
UNICODE_Lo, //U+100F
UNICODE_Lo, //U+1010
UNICODE_Lo, //U+1011
UNICODE_Lo, //U+1012
UNICODE_Lo, //U+1013
UNICODE_Lo, //U+1014
UNICODE_Lo, //U+1015
UNICODE_Lo, //U+1016
UNICODE_Lo, //U+1017
UNICODE_Lo, //U+1018
UNICODE_Lo, //U+1019
UNICODE_Lo, //U+101A
UNICODE_Lo, //U+101B
UNICODE_Lo, //U+101C
UNICODE_Lo, //U+101D
UNICODE_Lo, //U+101E
UNICODE_Lo, //U+101F
UNICODE_Lo, //U+1020
UNICODE_Lo, //U+1021
UNICODE_Lo, //U+1022
UNICODE_Lo, //U+1023
UNICODE_Lo, //U+1024
UNICODE_Lo, //U+1025
UNICODE_Lo, //U+1026
UNICODE_Lo, //U+1027
UNICODE_Lo, //U+1028
UNICODE_Lo, //U+1029
UNICODE_Lo, //U+102A
UNICODE_Mc, //U+102B
UNICODE_Mc, //U+102C
UNICODE_Mn, //U+102D
UNICODE_Mn, //U+102E
UNICODE_Mn, //U+102F
UNICODE_Mn, //U+1030
UNICODE_Mc, //U+1031
UNICODE_Mn, //U+1032
UNICODE_Mn, //U+1033
UNICODE_Mn, //U+1034
UNICODE_Mn, //U+1035
UNICODE_Mn, //U+1036
UNICODE_Mn, //U+1037
UNICODE_Mc, //U+1038
UNICODE_Mn, //U+1039
UNICODE_Mn, //U+103A
UNICODE_Mc, //U+103B
UNICODE_Mc, //U+103C
UNICODE_Mn, //U+103D
UNICODE_Mn, //U+103E
UNICODE_Lo, //U+103F
UNICODE_Nd, //U+1040
UNICODE_Nd, //U+1041
UNICODE_Nd, //U+1042
UNICODE_Nd, //U+1043
UNICODE_Nd, //U+1044
UNICODE_Nd, //U+1045
UNICODE_Nd, //U+1046
UNICODE_Nd, //U+1047
UNICODE_Nd, //U+1048
UNICODE_Nd, //U+1049
UNICODE_Po, //U+104A
UNICODE_Po, //U+104B
UNICODE_Po, //U+104C
UNICODE_Po, //U+104D
UNICODE_Po, //U+104E
UNICODE_Po, //U+104F
UNICODE_Lo, //U+1050
UNICODE_Lo, //U+1051
UNICODE_Lo, //U+1052
UNICODE_Lo, //U+1053
UNICODE_Lo, //U+1054
UNICODE_Lo, //U+1055
UNICODE_Mc, //U+1056
UNICODE_Mc, //U+1057
UNICODE_Mn, //U+1058
UNICODE_Mn, //U+1059
UNICODE_Lo, //U+105A
UNICODE_Lo, //U+105B
UNICODE_Lo, //U+105C
UNICODE_Lo, //U+105D
UNICODE_Mn, //U+105E
UNICODE_Mn, //U+105F
UNICODE_Mn, //U+1060
UNICODE_Lo, //U+1061
UNICODE_Mc, //U+1062
UNICODE_Mc, //U+1063
UNICODE_Mc, //U+1064
UNICODE_Lo, //U+1065
UNICODE_Lo, //U+1066
UNICODE_Mc, //U+1067
UNICODE_Mc, //U+1068
UNICODE_Mc, //U+1069
UNICODE_Mc, //U+106A
UNICODE_Mc, //U+106B
UNICODE_Mc, //U+106C
UNICODE_Mc, //U+106D
UNICODE_Lo, //U+106E
UNICODE_Lo, //U+106F
UNICODE_Lo, //U+1070
UNICODE_Mn, //U+1071
UNICODE_Mn, //U+1072
UNICODE_Mn, //U+1073
UNICODE_Mn, //U+1074
UNICODE_Lo, //U+1075
UNICODE_Lo, //U+1076
UNICODE_Lo, //U+1077
UNICODE_Lo, //U+1078
UNICODE_Lo, //U+1079
UNICODE_Lo, //U+107A
UNICODE_Lo, //U+107B
UNICODE_Lo, //U+107C
UNICODE_Lo, //U+107D
UNICODE_Lo, //U+107E
UNICODE_Lo, //U+107F
UNICODE_Lo, //U+1080
UNICODE_Lo, //U+1081
UNICODE_Mn, //U+1082
UNICODE_Mc, //U+1083
UNICODE_Mc, //U+1084
UNICODE_Mn, //U+1085
UNICODE_Mn, //U+1086
UNICODE_Mc, //U+1087
UNICODE_Mc, //U+1088
UNICODE_Mc, //U+1089
UNICODE_Mc, //U+108A
UNICODE_Mc, //U+108B
UNICODE_Mc, //U+108C
UNICODE_Mn, //U+108D
UNICODE_Lo, //U+108E
UNICODE_Mc, //U+108F
UNICODE_Nd, //U+1090
UNICODE_Nd, //U+1091
UNICODE_Nd, //U+1092
UNICODE_Nd, //U+1093
UNICODE_Nd, //U+1094
UNICODE_Nd, //U+1095
UNICODE_Nd, //U+1096
UNICODE_Nd, //U+1097
UNICODE_Nd, //U+1098
UNICODE_Nd, //U+1099
UNICODE_Mc, //U+109A
UNICODE_Mc, //U+109B
UNICODE_Mc, //U+109C
UNICODE_Mn, //U+109D
UNICODE_So, //U+109E
UNICODE_So, //U+109F
UNICODE_Lu, //U+10A0
UNICODE_Lu, //U+10A1
UNICODE_Lu, //U+10A2
UNICODE_Lu, //U+10A3
UNICODE_Lu, //U+10A4
UNICODE_Lu, //U+10A5
UNICODE_Lu, //U+10A6
UNICODE_Lu, //U+10A7
UNICODE_Lu, //U+10A8
UNICODE_Lu, //U+10A9
UNICODE_Lu, //U+10AA
UNICODE_Lu, //U+10AB
UNICODE_Lu, //U+10AC
UNICODE_Lu, //U+10AD
UNICODE_Lu, //U+10AE
UNICODE_Lu, //U+10AF
UNICODE_Lu, //U+10B0
UNICODE_Lu, //U+10B1
UNICODE_Lu, //U+10B2
UNICODE_Lu, //U+10B3
UNICODE_Lu, //U+10B4
UNICODE_Lu, //U+10B5
UNICODE_Lu, //U+10B6
UNICODE_Lu, //U+10B7
UNICODE_Lu, //U+10B8
UNICODE_Lu, //U+10B9
UNICODE_Lu, //U+10BA
UNICODE_Lu, //U+10BB
UNICODE_Lu, //U+10BC
UNICODE_Lu, //U+10BD
UNICODE_Lu, //U+10BE
UNICODE_Lu, //U+10BF
UNICODE_Lu, //U+10C0
UNICODE_Lu, //U+10C1
UNICODE_Lu, //U+10C2
UNICODE_Lu, //U+10C3
UNICODE_Lu, //U+10C4
UNICODE_Lu, //U+10C5
0, //U+10C6
0, //U+10C7
0, //U+10C8
0, //U+10C9
0, //U+10CA
0, //U+10CB
0, //U+10CC
0, //U+10CD
0, //U+10CE
0, //U+10CF
UNICODE_Lo, //U+10D0
UNICODE_Lo, //U+10D1
UNICODE_Lo, //U+10D2
UNICODE_Lo, //U+10D3
UNICODE_Lo, //U+10D4
UNICODE_Lo, //U+10D5
UNICODE_Lo, //U+10D6
UNICODE_Lo, //U+10D7
UNICODE_Lo, //U+10D8
UNICODE_Lo, //U+10D9
UNICODE_Lo, //U+10DA
UNICODE_Lo, //U+10DB
UNICODE_Lo, //U+10DC
UNICODE_Lo, //U+10DD
UNICODE_Lo, //U+10DE
UNICODE_Lo, //U+10DF
UNICODE_Lo, //U+10E0
UNICODE_Lo, //U+10E1
UNICODE_Lo, //U+10E2
UNICODE_Lo, //U+10E3
UNICODE_Lo, //U+10E4
UNICODE_Lo, //U+10E5
UNICODE_Lo, //U+10E6
UNICODE_Lo, //U+10E7
UNICODE_Lo, //U+10E8
UNICODE_Lo, //U+10E9
UNICODE_Lo, //U+10EA
UNICODE_Lo, //U+10EB
UNICODE_Lo, //U+10EC
UNICODE_Lo, //U+10ED
UNICODE_Lo, //U+10EE
UNICODE_Lo, //U+10EF
UNICODE_Lo, //U+10F0
UNICODE_Lo, //U+10F1
UNICODE_Lo, //U+10F2
UNICODE_Lo, //U+10F3
UNICODE_Lo, //U+10F4
UNICODE_Lo, //U+10F5
UNICODE_Lo, //U+10F6
UNICODE_Lo, //U+10F7
UNICODE_Lo, //U+10F8
UNICODE_Lo, //U+10F9
UNICODE_Lo, //U+10FA
UNICODE_Po, //U+10FB
UNICODE_Lm, //U+10FC
0, //U+10FD
0, //U+10FE
0, //U+10FF
UNICODE_Lo, //U+1100
UNICODE_Lo, //U+1101
UNICODE_Lo, //U+1102
UNICODE_Lo, //U+1103
UNICODE_Lo, //U+1104
UNICODE_Lo, //U+1105
UNICODE_Lo, //U+1106
UNICODE_Lo, //U+1107
UNICODE_Lo, //U+1108
UNICODE_Lo, //U+1109
UNICODE_Lo, //U+110A
UNICODE_Lo, //U+110B
UNICODE_Lo, //U+110C
UNICODE_Lo, //U+110D
UNICODE_Lo, //U+110E
UNICODE_Lo, //U+110F
UNICODE_Lo, //U+1110
UNICODE_Lo, //U+1111
UNICODE_Lo, //U+1112
UNICODE_Lo, //U+1113
UNICODE_Lo, //U+1114
UNICODE_Lo, //U+1115
UNICODE_Lo, //U+1116
UNICODE_Lo, //U+1117
UNICODE_Lo, //U+1118
UNICODE_Lo, //U+1119
UNICODE_Lo, //U+111A
UNICODE_Lo, //U+111B
UNICODE_Lo, //U+111C
UNICODE_Lo, //U+111D
UNICODE_Lo, //U+111E
UNICODE_Lo, //U+111F
UNICODE_Lo, //U+1120
UNICODE_Lo, //U+1121
UNICODE_Lo, //U+1122
UNICODE_Lo, //U+1123
UNICODE_Lo, //U+1124
UNICODE_Lo, //U+1125
UNICODE_Lo, //U+1126
UNICODE_Lo, //U+1127
UNICODE_Lo, //U+1128
UNICODE_Lo, //U+1129
UNICODE_Lo, //U+112A
UNICODE_Lo, //U+112B
UNICODE_Lo, //U+112C
UNICODE_Lo, //U+112D
UNICODE_Lo, //U+112E
UNICODE_Lo, //U+112F
UNICODE_Lo, //U+1130
UNICODE_Lo, //U+1131
UNICODE_Lo, //U+1132
UNICODE_Lo, //U+1133
UNICODE_Lo, //U+1134
UNICODE_Lo, //U+1135
UNICODE_Lo, //U+1136
UNICODE_Lo, //U+1137
UNICODE_Lo, //U+1138
UNICODE_Lo, //U+1139
UNICODE_Lo, //U+113A
UNICODE_Lo, //U+113B
UNICODE_Lo, //U+113C
UNICODE_Lo, //U+113D
UNICODE_Lo, //U+113E
UNICODE_Lo, //U+113F
UNICODE_Lo, //U+1140
UNICODE_Lo, //U+1141
UNICODE_Lo, //U+1142
UNICODE_Lo, //U+1143
UNICODE_Lo, //U+1144
UNICODE_Lo, //U+1145
UNICODE_Lo, //U+1146
UNICODE_Lo, //U+1147
UNICODE_Lo, //U+1148
UNICODE_Lo, //U+1149
UNICODE_Lo, //U+114A
UNICODE_Lo, //U+114B
UNICODE_Lo, //U+114C
UNICODE_Lo, //U+114D
UNICODE_Lo, //U+114E
UNICODE_Lo, //U+114F
UNICODE_Lo, //U+1150
UNICODE_Lo, //U+1151
UNICODE_Lo, //U+1152
UNICODE_Lo, //U+1153
UNICODE_Lo, //U+1154
UNICODE_Lo, //U+1155
UNICODE_Lo, //U+1156
UNICODE_Lo, //U+1157
UNICODE_Lo, //U+1158
UNICODE_Lo, //U+1159
UNICODE_Lo, //U+115A
UNICODE_Lo, //U+115B
UNICODE_Lo, //U+115C
UNICODE_Lo, //U+115D
UNICODE_Lo, //U+115E
UNICODE_Lo, //U+115F
UNICODE_Lo, //U+1160
UNICODE_Lo, //U+1161
UNICODE_Lo, //U+1162
UNICODE_Lo, //U+1163
UNICODE_Lo, //U+1164
UNICODE_Lo, //U+1165
UNICODE_Lo, //U+1166
UNICODE_Lo, //U+1167
UNICODE_Lo, //U+1168
UNICODE_Lo, //U+1169
UNICODE_Lo, //U+116A
UNICODE_Lo, //U+116B
UNICODE_Lo, //U+116C
UNICODE_Lo, //U+116D
UNICODE_Lo, //U+116E
UNICODE_Lo, //U+116F
UNICODE_Lo, //U+1170
UNICODE_Lo, //U+1171
UNICODE_Lo, //U+1172
UNICODE_Lo, //U+1173
UNICODE_Lo, //U+1174
UNICODE_Lo, //U+1175
UNICODE_Lo, //U+1176
UNICODE_Lo, //U+1177
UNICODE_Lo, //U+1178
UNICODE_Lo, //U+1179
UNICODE_Lo, //U+117A
UNICODE_Lo, //U+117B
UNICODE_Lo, //U+117C
UNICODE_Lo, //U+117D
UNICODE_Lo, //U+117E
UNICODE_Lo, //U+117F
UNICODE_Lo, //U+1180
UNICODE_Lo, //U+1181
UNICODE_Lo, //U+1182
UNICODE_Lo, //U+1183
UNICODE_Lo, //U+1184
UNICODE_Lo, //U+1185
UNICODE_Lo, //U+1186
UNICODE_Lo, //U+1187
UNICODE_Lo, //U+1188
UNICODE_Lo, //U+1189
UNICODE_Lo, //U+118A
UNICODE_Lo, //U+118B
UNICODE_Lo, //U+118C
UNICODE_Lo, //U+118D
UNICODE_Lo, //U+118E
UNICODE_Lo, //U+118F
UNICODE_Lo, //U+1190
UNICODE_Lo, //U+1191
UNICODE_Lo, //U+1192
UNICODE_Lo, //U+1193
UNICODE_Lo, //U+1194
UNICODE_Lo, //U+1195
UNICODE_Lo, //U+1196
UNICODE_Lo, //U+1197
UNICODE_Lo, //U+1198
UNICODE_Lo, //U+1199
UNICODE_Lo, //U+119A
UNICODE_Lo, //U+119B
UNICODE_Lo, //U+119C
UNICODE_Lo, //U+119D
UNICODE_Lo, //U+119E
UNICODE_Lo, //U+119F
UNICODE_Lo, //U+11A0
UNICODE_Lo, //U+11A1
UNICODE_Lo, //U+11A2
UNICODE_Lo, //U+11A3
UNICODE_Lo, //U+11A4
UNICODE_Lo, //U+11A5
UNICODE_Lo, //U+11A6
UNICODE_Lo, //U+11A7
UNICODE_Lo, //U+11A8
UNICODE_Lo, //U+11A9
UNICODE_Lo, //U+11AA
UNICODE_Lo, //U+11AB
UNICODE_Lo, //U+11AC
UNICODE_Lo, //U+11AD
UNICODE_Lo, //U+11AE
UNICODE_Lo, //U+11AF
UNICODE_Lo, //U+11B0
UNICODE_Lo, //U+11B1
UNICODE_Lo, //U+11B2
UNICODE_Lo, //U+11B3
UNICODE_Lo, //U+11B4
UNICODE_Lo, //U+11B5
UNICODE_Lo, //U+11B6
UNICODE_Lo, //U+11B7
UNICODE_Lo, //U+11B8
UNICODE_Lo, //U+11B9
UNICODE_Lo, //U+11BA
UNICODE_Lo, //U+11BB
UNICODE_Lo, //U+11BC
UNICODE_Lo, //U+11BD
UNICODE_Lo, //U+11BE
UNICODE_Lo, //U+11BF
UNICODE_Lo, //U+11C0
UNICODE_Lo, //U+11C1
UNICODE_Lo, //U+11C2
UNICODE_Lo, //U+11C3
UNICODE_Lo, //U+11C4
UNICODE_Lo, //U+11C5
UNICODE_Lo, //U+11C6
UNICODE_Lo, //U+11C7
UNICODE_Lo, //U+11C8
UNICODE_Lo, //U+11C9
UNICODE_Lo, //U+11CA
UNICODE_Lo, //U+11CB
UNICODE_Lo, //U+11CC
UNICODE_Lo, //U+11CD
UNICODE_Lo, //U+11CE
UNICODE_Lo, //U+11CF
UNICODE_Lo, //U+11D0
UNICODE_Lo, //U+11D1
UNICODE_Lo, //U+11D2
UNICODE_Lo, //U+11D3
UNICODE_Lo, //U+11D4
UNICODE_Lo, //U+11D5
UNICODE_Lo, //U+11D6
UNICODE_Lo, //U+11D7
UNICODE_Lo, //U+11D8
UNICODE_Lo, //U+11D9
UNICODE_Lo, //U+11DA
UNICODE_Lo, //U+11DB
UNICODE_Lo, //U+11DC
UNICODE_Lo, //U+11DD
UNICODE_Lo, //U+11DE
UNICODE_Lo, //U+11DF
UNICODE_Lo, //U+11E0
UNICODE_Lo, //U+11E1
UNICODE_Lo, //U+11E2
UNICODE_Lo, //U+11E3
UNICODE_Lo, //U+11E4
UNICODE_Lo, //U+11E5
UNICODE_Lo, //U+11E6
UNICODE_Lo, //U+11E7
UNICODE_Lo, //U+11E8
UNICODE_Lo, //U+11E9
UNICODE_Lo, //U+11EA
UNICODE_Lo, //U+11EB
UNICODE_Lo, //U+11EC
UNICODE_Lo, //U+11ED
UNICODE_Lo, //U+11EE
UNICODE_Lo, //U+11EF
UNICODE_Lo, //U+11F0
UNICODE_Lo, //U+11F1
UNICODE_Lo, //U+11F2
UNICODE_Lo, //U+11F3
UNICODE_Lo, //U+11F4
UNICODE_Lo, //U+11F5
UNICODE_Lo, //U+11F6
UNICODE_Lo, //U+11F7
UNICODE_Lo, //U+11F8
UNICODE_Lo, //U+11F9
UNICODE_Lo, //U+11FA
UNICODE_Lo, //U+11FB
UNICODE_Lo, //U+11FC
UNICODE_Lo, //U+11FD
UNICODE_Lo, //U+11FE
UNICODE_Lo, //U+11FF
UNICODE_Lo, //U+1200
UNICODE_Lo, //U+1201
UNICODE_Lo, //U+1202
UNICODE_Lo, //U+1203
UNICODE_Lo, //U+1204
UNICODE_Lo, //U+1205
UNICODE_Lo, //U+1206
UNICODE_Lo, //U+1207
UNICODE_Lo, //U+1208
UNICODE_Lo, //U+1209
UNICODE_Lo, //U+120A
UNICODE_Lo, //U+120B
UNICODE_Lo, //U+120C
UNICODE_Lo, //U+120D
UNICODE_Lo, //U+120E
UNICODE_Lo, //U+120F
UNICODE_Lo, //U+1210
UNICODE_Lo, //U+1211
UNICODE_Lo, //U+1212
UNICODE_Lo, //U+1213
UNICODE_Lo, //U+1214
UNICODE_Lo, //U+1215
UNICODE_Lo, //U+1216
UNICODE_Lo, //U+1217
UNICODE_Lo, //U+1218
UNICODE_Lo, //U+1219
UNICODE_Lo, //U+121A
UNICODE_Lo, //U+121B
UNICODE_Lo, //U+121C
UNICODE_Lo, //U+121D
UNICODE_Lo, //U+121E
UNICODE_Lo, //U+121F
UNICODE_Lo, //U+1220
UNICODE_Lo, //U+1221
UNICODE_Lo, //U+1222
UNICODE_Lo, //U+1223
UNICODE_Lo, //U+1224
UNICODE_Lo, //U+1225
UNICODE_Lo, //U+1226
UNICODE_Lo, //U+1227
UNICODE_Lo, //U+1228
UNICODE_Lo, //U+1229
UNICODE_Lo, //U+122A
UNICODE_Lo, //U+122B
UNICODE_Lo, //U+122C
UNICODE_Lo, //U+122D
UNICODE_Lo, //U+122E
UNICODE_Lo, //U+122F
UNICODE_Lo, //U+1230
UNICODE_Lo, //U+1231
UNICODE_Lo, //U+1232
UNICODE_Lo, //U+1233
UNICODE_Lo, //U+1234
UNICODE_Lo, //U+1235
UNICODE_Lo, //U+1236
UNICODE_Lo, //U+1237
UNICODE_Lo, //U+1238
UNICODE_Lo, //U+1239
UNICODE_Lo, //U+123A
UNICODE_Lo, //U+123B
UNICODE_Lo, //U+123C
UNICODE_Lo, //U+123D
UNICODE_Lo, //U+123E
UNICODE_Lo, //U+123F
UNICODE_Lo, //U+1240
UNICODE_Lo, //U+1241
UNICODE_Lo, //U+1242
UNICODE_Lo, //U+1243
UNICODE_Lo, //U+1244
UNICODE_Lo, //U+1245
UNICODE_Lo, //U+1246
UNICODE_Lo, //U+1247
UNICODE_Lo, //U+1248
0, //U+1249
UNICODE_Lo, //U+124A
UNICODE_Lo, //U+124B
UNICODE_Lo, //U+124C
UNICODE_Lo, //U+124D
0, //U+124E
0, //U+124F
UNICODE_Lo, //U+1250
UNICODE_Lo, //U+1251
UNICODE_Lo, //U+1252
UNICODE_Lo, //U+1253
UNICODE_Lo, //U+1254
UNICODE_Lo, //U+1255
UNICODE_Lo, //U+1256
0, //U+1257
UNICODE_Lo, //U+1258
0, //U+1259
UNICODE_Lo, //U+125A
UNICODE_Lo, //U+125B
UNICODE_Lo, //U+125C
UNICODE_Lo, //U+125D
0, //U+125E
0, //U+125F
UNICODE_Lo, //U+1260
UNICODE_Lo, //U+1261
UNICODE_Lo, //U+1262
UNICODE_Lo, //U+1263
UNICODE_Lo, //U+1264
UNICODE_Lo, //U+1265
UNICODE_Lo, //U+1266
UNICODE_Lo, //U+1267
UNICODE_Lo, //U+1268
UNICODE_Lo, //U+1269
UNICODE_Lo, //U+126A
UNICODE_Lo, //U+126B
UNICODE_Lo, //U+126C
UNICODE_Lo, //U+126D
UNICODE_Lo, //U+126E
UNICODE_Lo, //U+126F
UNICODE_Lo, //U+1270
UNICODE_Lo, //U+1271
UNICODE_Lo, //U+1272
UNICODE_Lo, //U+1273
UNICODE_Lo, //U+1274
UNICODE_Lo, //U+1275
UNICODE_Lo, //U+1276
UNICODE_Lo, //U+1277
UNICODE_Lo, //U+1278
UNICODE_Lo, //U+1279
UNICODE_Lo, //U+127A
UNICODE_Lo, //U+127B
UNICODE_Lo, //U+127C
UNICODE_Lo, //U+127D
UNICODE_Lo, //U+127E
UNICODE_Lo, //U+127F
UNICODE_Lo, //U+1280
UNICODE_Lo, //U+1281
UNICODE_Lo, //U+1282
UNICODE_Lo, //U+1283
UNICODE_Lo, //U+1284
UNICODE_Lo, //U+1285
UNICODE_Lo, //U+1286
UNICODE_Lo, //U+1287
UNICODE_Lo, //U+1288
0, //U+1289
UNICODE_Lo, //U+128A
UNICODE_Lo, //U+128B
UNICODE_Lo, //U+128C
UNICODE_Lo, //U+128D
0, //U+128E
0, //U+128F
UNICODE_Lo, //U+1290
UNICODE_Lo, //U+1291
UNICODE_Lo, //U+1292
UNICODE_Lo, //U+1293
UNICODE_Lo, //U+1294
UNICODE_Lo, //U+1295
UNICODE_Lo, //U+1296
UNICODE_Lo, //U+1297
UNICODE_Lo, //U+1298
UNICODE_Lo, //U+1299
UNICODE_Lo, //U+129A
UNICODE_Lo, //U+129B
UNICODE_Lo, //U+129C
UNICODE_Lo, //U+129D
UNICODE_Lo, //U+129E
UNICODE_Lo, //U+129F
UNICODE_Lo, //U+12A0
UNICODE_Lo, //U+12A1
UNICODE_Lo, //U+12A2
UNICODE_Lo, //U+12A3
UNICODE_Lo, //U+12A4
UNICODE_Lo, //U+12A5
UNICODE_Lo, //U+12A6
UNICODE_Lo, //U+12A7
UNICODE_Lo, //U+12A8
UNICODE_Lo, //U+12A9
UNICODE_Lo, //U+12AA
UNICODE_Lo, //U+12AB
UNICODE_Lo, //U+12AC
UNICODE_Lo, //U+12AD
UNICODE_Lo, //U+12AE
UNICODE_Lo, //U+12AF
UNICODE_Lo, //U+12B0
0, //U+12B1
UNICODE_Lo, //U+12B2
UNICODE_Lo, //U+12B3
UNICODE_Lo, //U+12B4
UNICODE_Lo, //U+12B5
0, //U+12B6
0, //U+12B7
UNICODE_Lo, //U+12B8
UNICODE_Lo, //U+12B9
UNICODE_Lo, //U+12BA
UNICODE_Lo, //U+12BB
UNICODE_Lo, //U+12BC
UNICODE_Lo, //U+12BD
UNICODE_Lo, //U+12BE
0, //U+12BF
UNICODE_Lo, //U+12C0
0, //U+12C1
UNICODE_Lo, //U+12C2
UNICODE_Lo, //U+12C3
UNICODE_Lo, //U+12C4
UNICODE_Lo, //U+12C5
0, //U+12C6
0, //U+12C7
UNICODE_Lo, //U+12C8
UNICODE_Lo, //U+12C9
UNICODE_Lo, //U+12CA
UNICODE_Lo, //U+12CB
UNICODE_Lo, //U+12CC
UNICODE_Lo, //U+12CD
UNICODE_Lo, //U+12CE
UNICODE_Lo, //U+12CF
UNICODE_Lo, //U+12D0
UNICODE_Lo, //U+12D1
UNICODE_Lo, //U+12D2
UNICODE_Lo, //U+12D3
UNICODE_Lo, //U+12D4
UNICODE_Lo, //U+12D5
UNICODE_Lo, //U+12D6
0, //U+12D7
UNICODE_Lo, //U+12D8
UNICODE_Lo, //U+12D9
UNICODE_Lo, //U+12DA
UNICODE_Lo, //U+12DB
UNICODE_Lo, //U+12DC
UNICODE_Lo, //U+12DD
UNICODE_Lo, //U+12DE
UNICODE_Lo, //U+12DF
UNICODE_Lo, //U+12E0
UNICODE_Lo, //U+12E1
UNICODE_Lo, //U+12E2
UNICODE_Lo, //U+12E3
UNICODE_Lo, //U+12E4
UNICODE_Lo, //U+12E5
UNICODE_Lo, //U+12E6
UNICODE_Lo, //U+12E7
UNICODE_Lo, //U+12E8
UNICODE_Lo, //U+12E9
UNICODE_Lo, //U+12EA
UNICODE_Lo, //U+12EB
UNICODE_Lo, //U+12EC
UNICODE_Lo, //U+12ED
UNICODE_Lo, //U+12EE
UNICODE_Lo, //U+12EF
UNICODE_Lo, //U+12F0
UNICODE_Lo, //U+12F1
UNICODE_Lo, //U+12F2
UNICODE_Lo, //U+12F3
UNICODE_Lo, //U+12F4
UNICODE_Lo, //U+12F5
UNICODE_Lo, //U+12F6
UNICODE_Lo, //U+12F7
UNICODE_Lo, //U+12F8
UNICODE_Lo, //U+12F9
UNICODE_Lo, //U+12FA
UNICODE_Lo, //U+12FB
UNICODE_Lo, //U+12FC
UNICODE_Lo, //U+12FD
UNICODE_Lo, //U+12FE
UNICODE_Lo, //U+12FF
UNICODE_Lo, //U+1300
UNICODE_Lo, //U+1301
UNICODE_Lo, //U+1302
UNICODE_Lo, //U+1303
UNICODE_Lo, //U+1304
UNICODE_Lo, //U+1305
UNICODE_Lo, //U+1306
UNICODE_Lo, //U+1307
UNICODE_Lo, //U+1308
UNICODE_Lo, //U+1309
UNICODE_Lo, //U+130A
UNICODE_Lo, //U+130B
UNICODE_Lo, //U+130C
UNICODE_Lo, //U+130D
UNICODE_Lo, //U+130E
UNICODE_Lo, //U+130F
UNICODE_Lo, //U+1310
0, //U+1311
UNICODE_Lo, //U+1312
UNICODE_Lo, //U+1313
UNICODE_Lo, //U+1314
UNICODE_Lo, //U+1315
0, //U+1316
0, //U+1317
UNICODE_Lo, //U+1318
UNICODE_Lo, //U+1319
UNICODE_Lo, //U+131A
UNICODE_Lo, //U+131B
UNICODE_Lo, //U+131C
UNICODE_Lo, //U+131D
UNICODE_Lo, //U+131E
UNICODE_Lo, //U+131F
UNICODE_Lo, //U+1320
UNICODE_Lo, //U+1321
UNICODE_Lo, //U+1322
UNICODE_Lo, //U+1323
UNICODE_Lo, //U+1324
UNICODE_Lo, //U+1325
UNICODE_Lo, //U+1326
UNICODE_Lo, //U+1327
UNICODE_Lo, //U+1328
UNICODE_Lo, //U+1329
UNICODE_Lo, //U+132A
UNICODE_Lo, //U+132B
UNICODE_Lo, //U+132C
UNICODE_Lo, //U+132D
UNICODE_Lo, //U+132E
UNICODE_Lo, //U+132F
UNICODE_Lo, //U+1330
UNICODE_Lo, //U+1331
UNICODE_Lo, //U+1332
UNICODE_Lo, //U+1333
UNICODE_Lo, //U+1334
UNICODE_Lo, //U+1335
UNICODE_Lo, //U+1336
UNICODE_Lo, //U+1337
UNICODE_Lo, //U+1338
UNICODE_Lo, //U+1339
UNICODE_Lo, //U+133A
UNICODE_Lo, //U+133B
UNICODE_Lo, //U+133C
UNICODE_Lo, //U+133D
UNICODE_Lo, //U+133E
UNICODE_Lo, //U+133F
UNICODE_Lo, //U+1340
UNICODE_Lo, //U+1341
UNICODE_Lo, //U+1342
UNICODE_Lo, //U+1343
UNICODE_Lo, //U+1344
UNICODE_Lo, //U+1345
UNICODE_Lo, //U+1346
UNICODE_Lo, //U+1347
UNICODE_Lo, //U+1348
UNICODE_Lo, //U+1349
UNICODE_Lo, //U+134A
UNICODE_Lo, //U+134B
UNICODE_Lo, //U+134C
UNICODE_Lo, //U+134D
UNICODE_Lo, //U+134E
UNICODE_Lo, //U+134F
UNICODE_Lo, //U+1350
UNICODE_Lo, //U+1351
UNICODE_Lo, //U+1352
UNICODE_Lo, //U+1353
UNICODE_Lo, //U+1354
UNICODE_Lo, //U+1355
UNICODE_Lo, //U+1356
UNICODE_Lo, //U+1357
UNICODE_Lo, //U+1358
UNICODE_Lo, //U+1359
UNICODE_Lo, //U+135A
0, //U+135B
0, //U+135C
UNICODE_Mn, //U+135D
UNICODE_Mn, //U+135E
UNICODE_Mn, //U+135F
UNICODE_So, //U+1360
UNICODE_Po, //U+1361
UNICODE_Po, //U+1362
UNICODE_Po, //U+1363
UNICODE_Po, //U+1364
UNICODE_Po, //U+1365
UNICODE_Po, //U+1366
UNICODE_Po, //U+1367
UNICODE_Po, //U+1368
UNICODE_No, //U+1369
UNICODE_No, //U+136A
UNICODE_No, //U+136B
UNICODE_No, //U+136C
UNICODE_No, //U+136D
UNICODE_No, //U+136E
UNICODE_No, //U+136F
UNICODE_No, //U+1370
UNICODE_No, //U+1371
UNICODE_No, //U+1372
UNICODE_No, //U+1373
UNICODE_No, //U+1374
UNICODE_No, //U+1375
UNICODE_No, //U+1376
UNICODE_No, //U+1377
UNICODE_No, //U+1378
UNICODE_No, //U+1379
UNICODE_No, //U+137A
UNICODE_No, //U+137B
UNICODE_No, //U+137C
0, //U+137D
0, //U+137E
0, //U+137F
UNICODE_Lo, //U+1380
UNICODE_Lo, //U+1381
UNICODE_Lo, //U+1382
UNICODE_Lo, //U+1383
UNICODE_Lo, //U+1384
UNICODE_Lo, //U+1385
UNICODE_Lo, //U+1386
UNICODE_Lo, //U+1387
UNICODE_Lo, //U+1388
UNICODE_Lo, //U+1389
UNICODE_Lo, //U+138A
UNICODE_Lo, //U+138B
UNICODE_Lo, //U+138C
UNICODE_Lo, //U+138D
UNICODE_Lo, //U+138E
UNICODE_Lo, //U+138F
UNICODE_So, //U+1390
UNICODE_So, //U+1391
UNICODE_So, //U+1392
UNICODE_So, //U+1393
UNICODE_So, //U+1394
UNICODE_So, //U+1395
UNICODE_So, //U+1396
UNICODE_So, //U+1397
UNICODE_So, //U+1398
UNICODE_So, //U+1399
0, //U+139A
0, //U+139B
0, //U+139C
0, //U+139D
0, //U+139E
0, //U+139F
UNICODE_Lo, //U+13A0
UNICODE_Lo, //U+13A1
UNICODE_Lo, //U+13A2
UNICODE_Lo, //U+13A3
UNICODE_Lo, //U+13A4
UNICODE_Lo, //U+13A5
UNICODE_Lo, //U+13A6
UNICODE_Lo, //U+13A7
UNICODE_Lo, //U+13A8
UNICODE_Lo, //U+13A9
UNICODE_Lo, //U+13AA
UNICODE_Lo, //U+13AB
UNICODE_Lo, //U+13AC
UNICODE_Lo, //U+13AD
UNICODE_Lo, //U+13AE
UNICODE_Lo, //U+13AF
UNICODE_Lo, //U+13B0
UNICODE_Lo, //U+13B1
UNICODE_Lo, //U+13B2
UNICODE_Lo, //U+13B3
UNICODE_Lo, //U+13B4
UNICODE_Lo, //U+13B5
UNICODE_Lo, //U+13B6
UNICODE_Lo, //U+13B7
UNICODE_Lo, //U+13B8
UNICODE_Lo, //U+13B9
UNICODE_Lo, //U+13BA
UNICODE_Lo, //U+13BB
UNICODE_Lo, //U+13BC
UNICODE_Lo, //U+13BD
UNICODE_Lo, //U+13BE
UNICODE_Lo, //U+13BF
UNICODE_Lo, //U+13C0
UNICODE_Lo, //U+13C1
UNICODE_Lo, //U+13C2
UNICODE_Lo, //U+13C3
UNICODE_Lo, //U+13C4
UNICODE_Lo, //U+13C5
UNICODE_Lo, //U+13C6
UNICODE_Lo, //U+13C7
UNICODE_Lo, //U+13C8
UNICODE_Lo, //U+13C9
UNICODE_Lo, //U+13CA
UNICODE_Lo, //U+13CB
UNICODE_Lo, //U+13CC
UNICODE_Lo, //U+13CD
UNICODE_Lo, //U+13CE
UNICODE_Lo, //U+13CF
UNICODE_Lo, //U+13D0
UNICODE_Lo, //U+13D1
UNICODE_Lo, //U+13D2
UNICODE_Lo, //U+13D3
UNICODE_Lo, //U+13D4
UNICODE_Lo, //U+13D5
UNICODE_Lo, //U+13D6
UNICODE_Lo, //U+13D7
UNICODE_Lo, //U+13D8
UNICODE_Lo, //U+13D9
UNICODE_Lo, //U+13DA
UNICODE_Lo, //U+13DB
UNICODE_Lo, //U+13DC
UNICODE_Lo, //U+13DD
UNICODE_Lo, //U+13DE
UNICODE_Lo, //U+13DF
UNICODE_Lo, //U+13E0
UNICODE_Lo, //U+13E1
UNICODE_Lo, //U+13E2
UNICODE_Lo, //U+13E3
UNICODE_Lo, //U+13E4
UNICODE_Lo, //U+13E5
UNICODE_Lo, //U+13E6
UNICODE_Lo, //U+13E7
UNICODE_Lo, //U+13E8
UNICODE_Lo, //U+13E9
UNICODE_Lo, //U+13EA
UNICODE_Lo, //U+13EB
UNICODE_Lo, //U+13EC
UNICODE_Lo, //U+13ED
UNICODE_Lo, //U+13EE
UNICODE_Lo, //U+13EF
UNICODE_Lo, //U+13F0
UNICODE_Lo, //U+13F1
UNICODE_Lo, //U+13F2
UNICODE_Lo, //U+13F3
UNICODE_Lo, //U+13F4
0, //U+13F5
0, //U+13F6
0, //U+13F7
0, //U+13F8
0, //U+13F9
0, //U+13FA
0, //U+13FB
0, //U+13FC
0, //U+13FD
0, //U+13FE
0, //U+13FF
UNICODE_Pd, //U+1400
UNICODE_Lo, //U+1401
UNICODE_Lo, //U+1402
UNICODE_Lo, //U+1403
UNICODE_Lo, //U+1404
UNICODE_Lo, //U+1405
UNICODE_Lo, //U+1406
UNICODE_Lo, //U+1407
UNICODE_Lo, //U+1408
UNICODE_Lo, //U+1409
UNICODE_Lo, //U+140A
UNICODE_Lo, //U+140B
UNICODE_Lo, //U+140C
UNICODE_Lo, //U+140D
UNICODE_Lo, //U+140E
UNICODE_Lo, //U+140F
UNICODE_Lo, //U+1410
UNICODE_Lo, //U+1411
UNICODE_Lo, //U+1412
UNICODE_Lo, //U+1413
UNICODE_Lo, //U+1414
UNICODE_Lo, //U+1415
UNICODE_Lo, //U+1416
UNICODE_Lo, //U+1417
UNICODE_Lo, //U+1418
UNICODE_Lo, //U+1419
UNICODE_Lo, //U+141A
UNICODE_Lo, //U+141B
UNICODE_Lo, //U+141C
UNICODE_Lo, //U+141D
UNICODE_Lo, //U+141E
UNICODE_Lo, //U+141F
UNICODE_Lo, //U+1420
UNICODE_Lo, //U+1421
UNICODE_Lo, //U+1422
UNICODE_Lo, //U+1423
UNICODE_Lo, //U+1424
UNICODE_Lo, //U+1425
UNICODE_Lo, //U+1426
UNICODE_Lo, //U+1427
UNICODE_Lo, //U+1428
UNICODE_Lo, //U+1429
UNICODE_Lo, //U+142A
UNICODE_Lo, //U+142B
UNICODE_Lo, //U+142C
UNICODE_Lo, //U+142D
UNICODE_Lo, //U+142E
UNICODE_Lo, //U+142F
UNICODE_Lo, //U+1430
UNICODE_Lo, //U+1431
UNICODE_Lo, //U+1432
UNICODE_Lo, //U+1433
UNICODE_Lo, //U+1434
UNICODE_Lo, //U+1435
UNICODE_Lo, //U+1436
UNICODE_Lo, //U+1437
UNICODE_Lo, //U+1438
UNICODE_Lo, //U+1439
UNICODE_Lo, //U+143A
UNICODE_Lo, //U+143B
UNICODE_Lo, //U+143C
UNICODE_Lo, //U+143D
UNICODE_Lo, //U+143E
UNICODE_Lo, //U+143F
UNICODE_Lo, //U+1440
UNICODE_Lo, //U+1441
UNICODE_Lo, //U+1442
UNICODE_Lo, //U+1443
UNICODE_Lo, //U+1444
UNICODE_Lo, //U+1445
UNICODE_Lo, //U+1446
UNICODE_Lo, //U+1447
UNICODE_Lo, //U+1448
UNICODE_Lo, //U+1449
UNICODE_Lo, //U+144A
UNICODE_Lo, //U+144B
UNICODE_Lo, //U+144C
UNICODE_Lo, //U+144D
UNICODE_Lo, //U+144E
UNICODE_Lo, //U+144F
UNICODE_Lo, //U+1450
UNICODE_Lo, //U+1451
UNICODE_Lo, //U+1452
UNICODE_Lo, //U+1453
UNICODE_Lo, //U+1454
UNICODE_Lo, //U+1455
UNICODE_Lo, //U+1456
UNICODE_Lo, //U+1457
UNICODE_Lo, //U+1458
UNICODE_Lo, //U+1459
UNICODE_Lo, //U+145A
UNICODE_Lo, //U+145B
UNICODE_Lo, //U+145C
UNICODE_Lo, //U+145D
UNICODE_Lo, //U+145E
UNICODE_Lo, //U+145F
UNICODE_Lo, //U+1460
UNICODE_Lo, //U+1461
UNICODE_Lo, //U+1462
UNICODE_Lo, //U+1463
UNICODE_Lo, //U+1464
UNICODE_Lo, //U+1465
UNICODE_Lo, //U+1466
UNICODE_Lo, //U+1467
UNICODE_Lo, //U+1468
UNICODE_Lo, //U+1469
UNICODE_Lo, //U+146A
UNICODE_Lo, //U+146B
UNICODE_Lo, //U+146C
UNICODE_Lo, //U+146D
UNICODE_Lo, //U+146E
UNICODE_Lo, //U+146F
UNICODE_Lo, //U+1470
UNICODE_Lo, //U+1471
UNICODE_Lo, //U+1472
UNICODE_Lo, //U+1473
UNICODE_Lo, //U+1474
UNICODE_Lo, //U+1475
UNICODE_Lo, //U+1476
UNICODE_Lo, //U+1477
UNICODE_Lo, //U+1478
UNICODE_Lo, //U+1479
UNICODE_Lo, //U+147A
UNICODE_Lo, //U+147B
UNICODE_Lo, //U+147C
UNICODE_Lo, //U+147D
UNICODE_Lo, //U+147E
UNICODE_Lo, //U+147F
UNICODE_Lo, //U+1480
UNICODE_Lo, //U+1481
UNICODE_Lo, //U+1482
UNICODE_Lo, //U+1483
UNICODE_Lo, //U+1484
UNICODE_Lo, //U+1485
UNICODE_Lo, //U+1486
UNICODE_Lo, //U+1487
UNICODE_Lo, //U+1488
UNICODE_Lo, //U+1489
UNICODE_Lo, //U+148A
UNICODE_Lo, //U+148B
UNICODE_Lo, //U+148C
UNICODE_Lo, //U+148D
UNICODE_Lo, //U+148E
UNICODE_Lo, //U+148F
UNICODE_Lo, //U+1490
UNICODE_Lo, //U+1491
UNICODE_Lo, //U+1492
UNICODE_Lo, //U+1493
UNICODE_Lo, //U+1494
UNICODE_Lo, //U+1495
UNICODE_Lo, //U+1496
UNICODE_Lo, //U+1497
UNICODE_Lo, //U+1498
UNICODE_Lo, //U+1499
UNICODE_Lo, //U+149A
UNICODE_Lo, //U+149B
UNICODE_Lo, //U+149C
UNICODE_Lo, //U+149D
UNICODE_Lo, //U+149E
UNICODE_Lo, //U+149F
UNICODE_Lo, //U+14A0
UNICODE_Lo, //U+14A1
UNICODE_Lo, //U+14A2
UNICODE_Lo, //U+14A3
UNICODE_Lo, //U+14A4
UNICODE_Lo, //U+14A5
UNICODE_Lo, //U+14A6
UNICODE_Lo, //U+14A7
UNICODE_Lo, //U+14A8
UNICODE_Lo, //U+14A9
UNICODE_Lo, //U+14AA
UNICODE_Lo, //U+14AB
UNICODE_Lo, //U+14AC
UNICODE_Lo, //U+14AD
UNICODE_Lo, //U+14AE
UNICODE_Lo, //U+14AF
UNICODE_Lo, //U+14B0
UNICODE_Lo, //U+14B1
UNICODE_Lo, //U+14B2
UNICODE_Lo, //U+14B3
UNICODE_Lo, //U+14B4
UNICODE_Lo, //U+14B5
UNICODE_Lo, //U+14B6
UNICODE_Lo, //U+14B7
UNICODE_Lo, //U+14B8
UNICODE_Lo, //U+14B9
UNICODE_Lo, //U+14BA
UNICODE_Lo, //U+14BB
UNICODE_Lo, //U+14BC
UNICODE_Lo, //U+14BD
UNICODE_Lo, //U+14BE
UNICODE_Lo, //U+14BF
UNICODE_Lo, //U+14C0
UNICODE_Lo, //U+14C1
UNICODE_Lo, //U+14C2
UNICODE_Lo, //U+14C3
UNICODE_Lo, //U+14C4
UNICODE_Lo, //U+14C5
UNICODE_Lo, //U+14C6
UNICODE_Lo, //U+14C7
UNICODE_Lo, //U+14C8
UNICODE_Lo, //U+14C9
UNICODE_Lo, //U+14CA
UNICODE_Lo, //U+14CB
UNICODE_Lo, //U+14CC
UNICODE_Lo, //U+14CD
UNICODE_Lo, //U+14CE
UNICODE_Lo, //U+14CF
UNICODE_Lo, //U+14D0
UNICODE_Lo, //U+14D1
UNICODE_Lo, //U+14D2
UNICODE_Lo, //U+14D3
UNICODE_Lo, //U+14D4
UNICODE_Lo, //U+14D5
UNICODE_Lo, //U+14D6
UNICODE_Lo, //U+14D7
UNICODE_Lo, //U+14D8
UNICODE_Lo, //U+14D9
UNICODE_Lo, //U+14DA
UNICODE_Lo, //U+14DB
UNICODE_Lo, //U+14DC
UNICODE_Lo, //U+14DD
UNICODE_Lo, //U+14DE
UNICODE_Lo, //U+14DF
UNICODE_Lo, //U+14E0
UNICODE_Lo, //U+14E1
UNICODE_Lo, //U+14E2
UNICODE_Lo, //U+14E3
UNICODE_Lo, //U+14E4
UNICODE_Lo, //U+14E5
UNICODE_Lo, //U+14E6
UNICODE_Lo, //U+14E7
UNICODE_Lo, //U+14E8
UNICODE_Lo, //U+14E9
UNICODE_Lo, //U+14EA
UNICODE_Lo, //U+14EB
UNICODE_Lo, //U+14EC
UNICODE_Lo, //U+14ED
UNICODE_Lo, //U+14EE
UNICODE_Lo, //U+14EF
UNICODE_Lo, //U+14F0
UNICODE_Lo, //U+14F1
UNICODE_Lo, //U+14F2
UNICODE_Lo, //U+14F3
UNICODE_Lo, //U+14F4
UNICODE_Lo, //U+14F5
UNICODE_Lo, //U+14F6
UNICODE_Lo, //U+14F7
UNICODE_Lo, //U+14F8
UNICODE_Lo, //U+14F9
UNICODE_Lo, //U+14FA
UNICODE_Lo, //U+14FB
UNICODE_Lo, //U+14FC
UNICODE_Lo, //U+14FD
UNICODE_Lo, //U+14FE
UNICODE_Lo, //U+14FF
UNICODE_Lo, //U+1500
UNICODE_Lo, //U+1501
UNICODE_Lo, //U+1502
UNICODE_Lo, //U+1503
UNICODE_Lo, //U+1504
UNICODE_Lo, //U+1505
UNICODE_Lo, //U+1506
UNICODE_Lo, //U+1507
UNICODE_Lo, //U+1508
UNICODE_Lo, //U+1509
UNICODE_Lo, //U+150A
UNICODE_Lo, //U+150B
UNICODE_Lo, //U+150C
UNICODE_Lo, //U+150D
UNICODE_Lo, //U+150E
UNICODE_Lo, //U+150F
UNICODE_Lo, //U+1510
UNICODE_Lo, //U+1511
UNICODE_Lo, //U+1512
UNICODE_Lo, //U+1513
UNICODE_Lo, //U+1514
UNICODE_Lo, //U+1515
UNICODE_Lo, //U+1516
UNICODE_Lo, //U+1517
UNICODE_Lo, //U+1518
UNICODE_Lo, //U+1519
UNICODE_Lo, //U+151A
UNICODE_Lo, //U+151B
UNICODE_Lo, //U+151C
UNICODE_Lo, //U+151D
UNICODE_Lo, //U+151E
UNICODE_Lo, //U+151F
UNICODE_Lo, //U+1520
UNICODE_Lo, //U+1521
UNICODE_Lo, //U+1522
UNICODE_Lo, //U+1523
UNICODE_Lo, //U+1524
UNICODE_Lo, //U+1525
UNICODE_Lo, //U+1526
UNICODE_Lo, //U+1527
UNICODE_Lo, //U+1528
UNICODE_Lo, //U+1529
UNICODE_Lo, //U+152A
UNICODE_Lo, //U+152B
UNICODE_Lo, //U+152C
UNICODE_Lo, //U+152D
UNICODE_Lo, //U+152E
UNICODE_Lo, //U+152F
UNICODE_Lo, //U+1530
UNICODE_Lo, //U+1531
UNICODE_Lo, //U+1532
UNICODE_Lo, //U+1533
UNICODE_Lo, //U+1534
UNICODE_Lo, //U+1535
UNICODE_Lo, //U+1536
UNICODE_Lo, //U+1537
UNICODE_Lo, //U+1538
UNICODE_Lo, //U+1539
UNICODE_Lo, //U+153A
UNICODE_Lo, //U+153B
UNICODE_Lo, //U+153C
UNICODE_Lo, //U+153D
UNICODE_Lo, //U+153E
UNICODE_Lo, //U+153F
UNICODE_Lo, //U+1540
UNICODE_Lo, //U+1541
UNICODE_Lo, //U+1542
UNICODE_Lo, //U+1543
UNICODE_Lo, //U+1544
UNICODE_Lo, //U+1545
UNICODE_Lo, //U+1546
UNICODE_Lo, //U+1547
UNICODE_Lo, //U+1548
UNICODE_Lo, //U+1549
UNICODE_Lo, //U+154A
UNICODE_Lo, //U+154B
UNICODE_Lo, //U+154C
UNICODE_Lo, //U+154D
UNICODE_Lo, //U+154E
UNICODE_Lo, //U+154F
UNICODE_Lo, //U+1550
UNICODE_Lo, //U+1551
UNICODE_Lo, //U+1552
UNICODE_Lo, //U+1553
UNICODE_Lo, //U+1554
UNICODE_Lo, //U+1555
UNICODE_Lo, //U+1556
UNICODE_Lo, //U+1557
UNICODE_Lo, //U+1558
UNICODE_Lo, //U+1559
UNICODE_Lo, //U+155A
UNICODE_Lo, //U+155B
UNICODE_Lo, //U+155C
UNICODE_Lo, //U+155D
UNICODE_Lo, //U+155E
UNICODE_Lo, //U+155F
UNICODE_Lo, //U+1560
UNICODE_Lo, //U+1561
UNICODE_Lo, //U+1562
UNICODE_Lo, //U+1563
UNICODE_Lo, //U+1564
UNICODE_Lo, //U+1565
UNICODE_Lo, //U+1566
UNICODE_Lo, //U+1567
UNICODE_Lo, //U+1568
UNICODE_Lo, //U+1569
UNICODE_Lo, //U+156A
UNICODE_Lo, //U+156B
UNICODE_Lo, //U+156C
UNICODE_Lo, //U+156D
UNICODE_Lo, //U+156E
UNICODE_Lo, //U+156F
UNICODE_Lo, //U+1570
UNICODE_Lo, //U+1571
UNICODE_Lo, //U+1572
UNICODE_Lo, //U+1573
UNICODE_Lo, //U+1574
UNICODE_Lo, //U+1575
UNICODE_Lo, //U+1576
UNICODE_Lo, //U+1577
UNICODE_Lo, //U+1578
UNICODE_Lo, //U+1579
UNICODE_Lo, //U+157A
UNICODE_Lo, //U+157B
UNICODE_Lo, //U+157C
UNICODE_Lo, //U+157D
UNICODE_Lo, //U+157E
UNICODE_Lo, //U+157F
UNICODE_Lo, //U+1580
UNICODE_Lo, //U+1581
UNICODE_Lo, //U+1582
UNICODE_Lo, //U+1583
UNICODE_Lo, //U+1584
UNICODE_Lo, //U+1585
UNICODE_Lo, //U+1586
UNICODE_Lo, //U+1587
UNICODE_Lo, //U+1588
UNICODE_Lo, //U+1589
UNICODE_Lo, //U+158A
UNICODE_Lo, //U+158B
UNICODE_Lo, //U+158C
UNICODE_Lo, //U+158D
UNICODE_Lo, //U+158E
UNICODE_Lo, //U+158F
UNICODE_Lo, //U+1590
UNICODE_Lo, //U+1591
UNICODE_Lo, //U+1592
UNICODE_Lo, //U+1593
UNICODE_Lo, //U+1594
UNICODE_Lo, //U+1595
UNICODE_Lo, //U+1596
UNICODE_Lo, //U+1597
UNICODE_Lo, //U+1598
UNICODE_Lo, //U+1599
UNICODE_Lo, //U+159A
UNICODE_Lo, //U+159B
UNICODE_Lo, //U+159C
UNICODE_Lo, //U+159D
UNICODE_Lo, //U+159E
UNICODE_Lo, //U+159F
UNICODE_Lo, //U+15A0
UNICODE_Lo, //U+15A1
UNICODE_Lo, //U+15A2
UNICODE_Lo, //U+15A3
UNICODE_Lo, //U+15A4
UNICODE_Lo, //U+15A5
UNICODE_Lo, //U+15A6
UNICODE_Lo, //U+15A7
UNICODE_Lo, //U+15A8
UNICODE_Lo, //U+15A9
UNICODE_Lo, //U+15AA
UNICODE_Lo, //U+15AB
UNICODE_Lo, //U+15AC
UNICODE_Lo, //U+15AD
UNICODE_Lo, //U+15AE
UNICODE_Lo, //U+15AF
UNICODE_Lo, //U+15B0
UNICODE_Lo, //U+15B1
UNICODE_Lo, //U+15B2
UNICODE_Lo, //U+15B3
UNICODE_Lo, //U+15B4
UNICODE_Lo, //U+15B5
UNICODE_Lo, //U+15B6
UNICODE_Lo, //U+15B7
UNICODE_Lo, //U+15B8
UNICODE_Lo, //U+15B9
UNICODE_Lo, //U+15BA
UNICODE_Lo, //U+15BB
UNICODE_Lo, //U+15BC
UNICODE_Lo, //U+15BD
UNICODE_Lo, //U+15BE
UNICODE_Lo, //U+15BF
UNICODE_Lo, //U+15C0
UNICODE_Lo, //U+15C1
UNICODE_Lo, //U+15C2
UNICODE_Lo, //U+15C3
UNICODE_Lo, //U+15C4
UNICODE_Lo, //U+15C5
UNICODE_Lo, //U+15C6
UNICODE_Lo, //U+15C7
UNICODE_Lo, //U+15C8
UNICODE_Lo, //U+15C9
UNICODE_Lo, //U+15CA
UNICODE_Lo, //U+15CB
UNICODE_Lo, //U+15CC
UNICODE_Lo, //U+15CD
UNICODE_Lo, //U+15CE
UNICODE_Lo, //U+15CF
UNICODE_Lo, //U+15D0
UNICODE_Lo, //U+15D1
UNICODE_Lo, //U+15D2
UNICODE_Lo, //U+15D3
UNICODE_Lo, //U+15D4
UNICODE_Lo, //U+15D5
UNICODE_Lo, //U+15D6
UNICODE_Lo, //U+15D7
UNICODE_Lo, //U+15D8
UNICODE_Lo, //U+15D9
UNICODE_Lo, //U+15DA
UNICODE_Lo, //U+15DB
UNICODE_Lo, //U+15DC
UNICODE_Lo, //U+15DD
UNICODE_Lo, //U+15DE
UNICODE_Lo, //U+15DF
UNICODE_Lo, //U+15E0
UNICODE_Lo, //U+15E1
UNICODE_Lo, //U+15E2
UNICODE_Lo, //U+15E3
UNICODE_Lo, //U+15E4
UNICODE_Lo, //U+15E5
UNICODE_Lo, //U+15E6
UNICODE_Lo, //U+15E7
UNICODE_Lo, //U+15E8
UNICODE_Lo, //U+15E9
UNICODE_Lo, //U+15EA
UNICODE_Lo, //U+15EB
UNICODE_Lo, //U+15EC
UNICODE_Lo, //U+15ED
UNICODE_Lo, //U+15EE
UNICODE_Lo, //U+15EF
UNICODE_Lo, //U+15F0
UNICODE_Lo, //U+15F1
UNICODE_Lo, //U+15F2
UNICODE_Lo, //U+15F3
UNICODE_Lo, //U+15F4
UNICODE_Lo, //U+15F5
UNICODE_Lo, //U+15F6
UNICODE_Lo, //U+15F7
UNICODE_Lo, //U+15F8
UNICODE_Lo, //U+15F9
UNICODE_Lo, //U+15FA
UNICODE_Lo, //U+15FB
UNICODE_Lo, //U+15FC
UNICODE_Lo, //U+15FD
UNICODE_Lo, //U+15FE
UNICODE_Lo, //U+15FF
UNICODE_Lo, //U+1600
UNICODE_Lo, //U+1601
UNICODE_Lo, //U+1602
UNICODE_Lo, //U+1603
UNICODE_Lo, //U+1604
UNICODE_Lo, //U+1605
UNICODE_Lo, //U+1606
UNICODE_Lo, //U+1607
UNICODE_Lo, //U+1608
UNICODE_Lo, //U+1609
UNICODE_Lo, //U+160A
UNICODE_Lo, //U+160B
UNICODE_Lo, //U+160C
UNICODE_Lo, //U+160D
UNICODE_Lo, //U+160E
UNICODE_Lo, //U+160F
UNICODE_Lo, //U+1610
UNICODE_Lo, //U+1611
UNICODE_Lo, //U+1612
UNICODE_Lo, //U+1613
UNICODE_Lo, //U+1614
UNICODE_Lo, //U+1615
UNICODE_Lo, //U+1616
UNICODE_Lo, //U+1617
UNICODE_Lo, //U+1618
UNICODE_Lo, //U+1619
UNICODE_Lo, //U+161A
UNICODE_Lo, //U+161B
UNICODE_Lo, //U+161C
UNICODE_Lo, //U+161D
UNICODE_Lo, //U+161E
UNICODE_Lo, //U+161F
UNICODE_Lo, //U+1620
UNICODE_Lo, //U+1621
UNICODE_Lo, //U+1622
UNICODE_Lo, //U+1623
UNICODE_Lo, //U+1624
UNICODE_Lo, //U+1625
UNICODE_Lo, //U+1626
UNICODE_Lo, //U+1627
UNICODE_Lo, //U+1628
UNICODE_Lo, //U+1629
UNICODE_Lo, //U+162A
UNICODE_Lo, //U+162B
UNICODE_Lo, //U+162C
UNICODE_Lo, //U+162D
UNICODE_Lo, //U+162E
UNICODE_Lo, //U+162F
UNICODE_Lo, //U+1630
UNICODE_Lo, //U+1631
UNICODE_Lo, //U+1632
UNICODE_Lo, //U+1633
UNICODE_Lo, //U+1634
UNICODE_Lo, //U+1635
UNICODE_Lo, //U+1636
UNICODE_Lo, //U+1637
UNICODE_Lo, //U+1638
UNICODE_Lo, //U+1639
UNICODE_Lo, //U+163A
UNICODE_Lo, //U+163B
UNICODE_Lo, //U+163C
UNICODE_Lo, //U+163D
UNICODE_Lo, //U+163E
UNICODE_Lo, //U+163F
UNICODE_Lo, //U+1640
UNICODE_Lo, //U+1641
UNICODE_Lo, //U+1642
UNICODE_Lo, //U+1643
UNICODE_Lo, //U+1644
UNICODE_Lo, //U+1645
UNICODE_Lo, //U+1646
UNICODE_Lo, //U+1647
UNICODE_Lo, //U+1648
UNICODE_Lo, //U+1649
UNICODE_Lo, //U+164A
UNICODE_Lo, //U+164B
UNICODE_Lo, //U+164C
UNICODE_Lo, //U+164D
UNICODE_Lo, //U+164E
UNICODE_Lo, //U+164F
UNICODE_Lo, //U+1650
UNICODE_Lo, //U+1651
UNICODE_Lo, //U+1652
UNICODE_Lo, //U+1653
UNICODE_Lo, //U+1654
UNICODE_Lo, //U+1655
UNICODE_Lo, //U+1656
UNICODE_Lo, //U+1657
UNICODE_Lo, //U+1658
UNICODE_Lo, //U+1659
UNICODE_Lo, //U+165A
UNICODE_Lo, //U+165B
UNICODE_Lo, //U+165C
UNICODE_Lo, //U+165D
UNICODE_Lo, //U+165E
UNICODE_Lo, //U+165F
UNICODE_Lo, //U+1660
UNICODE_Lo, //U+1661
UNICODE_Lo, //U+1662
UNICODE_Lo, //U+1663
UNICODE_Lo, //U+1664
UNICODE_Lo, //U+1665
UNICODE_Lo, //U+1666
UNICODE_Lo, //U+1667
UNICODE_Lo, //U+1668
UNICODE_Lo, //U+1669
UNICODE_Lo, //U+166A
UNICODE_Lo, //U+166B
UNICODE_Lo, //U+166C
UNICODE_Po, //U+166D
UNICODE_Po, //U+166E
UNICODE_Lo, //U+166F
UNICODE_Lo, //U+1670
UNICODE_Lo, //U+1671
UNICODE_Lo, //U+1672
UNICODE_Lo, //U+1673
UNICODE_Lo, //U+1674
UNICODE_Lo, //U+1675
UNICODE_Lo, //U+1676
UNICODE_Lo, //U+1677
UNICODE_Lo, //U+1678
UNICODE_Lo, //U+1679
UNICODE_Lo, //U+167A
UNICODE_Lo, //U+167B
UNICODE_Lo, //U+167C
UNICODE_Lo, //U+167D
UNICODE_Lo, //U+167E
UNICODE_Lo, //U+167F
UNICODE_Zs, //U+1680
UNICODE_Lo, //U+1681
UNICODE_Lo, //U+1682
UNICODE_Lo, //U+1683
UNICODE_Lo, //U+1684
UNICODE_Lo, //U+1685
UNICODE_Lo, //U+1686
UNICODE_Lo, //U+1687
UNICODE_Lo, //U+1688
UNICODE_Lo, //U+1689
UNICODE_Lo, //U+168A
UNICODE_Lo, //U+168B
UNICODE_Lo, //U+168C
UNICODE_Lo, //U+168D
UNICODE_Lo, //U+168E
UNICODE_Lo, //U+168F
UNICODE_Lo, //U+1690
UNICODE_Lo, //U+1691
UNICODE_Lo, //U+1692
UNICODE_Lo, //U+1693
UNICODE_Lo, //U+1694
UNICODE_Lo, //U+1695
UNICODE_Lo, //U+1696
UNICODE_Lo, //U+1697
UNICODE_Lo, //U+1698
UNICODE_Lo, //U+1699
UNICODE_Lo, //U+169A
UNICODE_Ps, //U+169B
UNICODE_Pe, //U+169C
0, //U+169D
0, //U+169E
0, //U+169F
UNICODE_Lo, //U+16A0
UNICODE_Lo, //U+16A1
UNICODE_Lo, //U+16A2
UNICODE_Lo, //U+16A3
UNICODE_Lo, //U+16A4
UNICODE_Lo, //U+16A5
UNICODE_Lo, //U+16A6
UNICODE_Lo, //U+16A7
UNICODE_Lo, //U+16A8
UNICODE_Lo, //U+16A9
UNICODE_Lo, //U+16AA
UNICODE_Lo, //U+16AB
UNICODE_Lo, //U+16AC
UNICODE_Lo, //U+16AD
UNICODE_Lo, //U+16AE
UNICODE_Lo, //U+16AF
UNICODE_Lo, //U+16B0
UNICODE_Lo, //U+16B1
UNICODE_Lo, //U+16B2
UNICODE_Lo, //U+16B3
UNICODE_Lo, //U+16B4
UNICODE_Lo, //U+16B5
UNICODE_Lo, //U+16B6
UNICODE_Lo, //U+16B7
UNICODE_Lo, //U+16B8
UNICODE_Lo, //U+16B9
UNICODE_Lo, //U+16BA
UNICODE_Lo, //U+16BB
UNICODE_Lo, //U+16BC
UNICODE_Lo, //U+16BD
UNICODE_Lo, //U+16BE
UNICODE_Lo, //U+16BF
UNICODE_Lo, //U+16C0
UNICODE_Lo, //U+16C1
UNICODE_Lo, //U+16C2
UNICODE_Lo, //U+16C3
UNICODE_Lo, //U+16C4
UNICODE_Lo, //U+16C5
UNICODE_Lo, //U+16C6
UNICODE_Lo, //U+16C7
UNICODE_Lo, //U+16C8
UNICODE_Lo, //U+16C9
UNICODE_Lo, //U+16CA
UNICODE_Lo, //U+16CB
UNICODE_Lo, //U+16CC
UNICODE_Lo, //U+16CD
UNICODE_Lo, //U+16CE
UNICODE_Lo, //U+16CF
UNICODE_Lo, //U+16D0
UNICODE_Lo, //U+16D1
UNICODE_Lo, //U+16D2
UNICODE_Lo, //U+16D3
UNICODE_Lo, //U+16D4
UNICODE_Lo, //U+16D5
UNICODE_Lo, //U+16D6
UNICODE_Lo, //U+16D7
UNICODE_Lo, //U+16D8
UNICODE_Lo, //U+16D9
UNICODE_Lo, //U+16DA
UNICODE_Lo, //U+16DB
UNICODE_Lo, //U+16DC
UNICODE_Lo, //U+16DD
UNICODE_Lo, //U+16DE
UNICODE_Lo, //U+16DF
UNICODE_Lo, //U+16E0
UNICODE_Lo, //U+16E1
UNICODE_Lo, //U+16E2
UNICODE_Lo, //U+16E3
UNICODE_Lo, //U+16E4
UNICODE_Lo, //U+16E5
UNICODE_Lo, //U+16E6
UNICODE_Lo, //U+16E7
UNICODE_Lo, //U+16E8
UNICODE_Lo, //U+16E9
UNICODE_Lo, //U+16EA
UNICODE_Po, //U+16EB
UNICODE_Po, //U+16EC
UNICODE_Po, //U+16ED
UNICODE_Nl, //U+16EE
UNICODE_Nl, //U+16EF
UNICODE_Nl, //U+16F0
0, //U+16F1
0, //U+16F2
0, //U+16F3
0, //U+16F4
0, //U+16F5
0, //U+16F6
0, //U+16F7
0, //U+16F8
0, //U+16F9
0, //U+16FA
0, //U+16FB
0, //U+16FC
0, //U+16FD
0, //U+16FE
0, //U+16FF
UNICODE_Lo, //U+1700
UNICODE_Lo, //U+1701
UNICODE_Lo, //U+1702
UNICODE_Lo, //U+1703
UNICODE_Lo, //U+1704
UNICODE_Lo, //U+1705
UNICODE_Lo, //U+1706
UNICODE_Lo, //U+1707
UNICODE_Lo, //U+1708
UNICODE_Lo, //U+1709
UNICODE_Lo, //U+170A
UNICODE_Lo, //U+170B
UNICODE_Lo, //U+170C
0, //U+170D
UNICODE_Lo, //U+170E
UNICODE_Lo, //U+170F
UNICODE_Lo, //U+1710
UNICODE_Lo, //U+1711
UNICODE_Mn, //U+1712
UNICODE_Mn, //U+1713
UNICODE_Mn, //U+1714
0, //U+1715
0, //U+1716
0, //U+1717
0, //U+1718
0, //U+1719
0, //U+171A
0, //U+171B
0, //U+171C
0, //U+171D
0, //U+171E
0, //U+171F
UNICODE_Lo, //U+1720
UNICODE_Lo, //U+1721
UNICODE_Lo, //U+1722
UNICODE_Lo, //U+1723
UNICODE_Lo, //U+1724
UNICODE_Lo, //U+1725
UNICODE_Lo, //U+1726
UNICODE_Lo, //U+1727
UNICODE_Lo, //U+1728
UNICODE_Lo, //U+1729
UNICODE_Lo, //U+172A
UNICODE_Lo, //U+172B
UNICODE_Lo, //U+172C
UNICODE_Lo, //U+172D
UNICODE_Lo, //U+172E
UNICODE_Lo, //U+172F
UNICODE_Lo, //U+1730
UNICODE_Lo, //U+1731
UNICODE_Mn, //U+1732
UNICODE_Mn, //U+1733
UNICODE_Mn, //U+1734
UNICODE_Po, //U+1735
UNICODE_Po, //U+1736
0, //U+1737
0, //U+1738
0, //U+1739
0, //U+173A
0, //U+173B
0, //U+173C
0, //U+173D
0, //U+173E
0, //U+173F
UNICODE_Lo, //U+1740
UNICODE_Lo, //U+1741
UNICODE_Lo, //U+1742
UNICODE_Lo, //U+1743
UNICODE_Lo, //U+1744
UNICODE_Lo, //U+1745
UNICODE_Lo, //U+1746
UNICODE_Lo, //U+1747
UNICODE_Lo, //U+1748
UNICODE_Lo, //U+1749
UNICODE_Lo, //U+174A
UNICODE_Lo, //U+174B
UNICODE_Lo, //U+174C
UNICODE_Lo, //U+174D
UNICODE_Lo, //U+174E
UNICODE_Lo, //U+174F
UNICODE_Lo, //U+1750
UNICODE_Lo, //U+1751
UNICODE_Mn, //U+1752
UNICODE_Mn, //U+1753
0, //U+1754
0, //U+1755
0, //U+1756
0, //U+1757
0, //U+1758
0, //U+1759
0, //U+175A
0, //U+175B
0, //U+175C
0, //U+175D
0, //U+175E
0, //U+175F
UNICODE_Lo, //U+1760
UNICODE_Lo, //U+1761
UNICODE_Lo, //U+1762
UNICODE_Lo, //U+1763
UNICODE_Lo, //U+1764
UNICODE_Lo, //U+1765
UNICODE_Lo, //U+1766
UNICODE_Lo, //U+1767
UNICODE_Lo, //U+1768
UNICODE_Lo, //U+1769
UNICODE_Lo, //U+176A
UNICODE_Lo, //U+176B
UNICODE_Lo, //U+176C
0, //U+176D
UNICODE_Lo, //U+176E
UNICODE_Lo, //U+176F
UNICODE_Lo, //U+1770
0, //U+1771
UNICODE_Mn, //U+1772
UNICODE_Mn, //U+1773
0, //U+1774
0, //U+1775
0, //U+1776
0, //U+1777
0, //U+1778
0, //U+1779
0, //U+177A
0, //U+177B
0, //U+177C
0, //U+177D
0, //U+177E
0, //U+177F
UNICODE_Lo, //U+1780
UNICODE_Lo, //U+1781
UNICODE_Lo, //U+1782
UNICODE_Lo, //U+1783
UNICODE_Lo, //U+1784
UNICODE_Lo, //U+1785
UNICODE_Lo, //U+1786
UNICODE_Lo, //U+1787
UNICODE_Lo, //U+1788
UNICODE_Lo, //U+1789
UNICODE_Lo, //U+178A
UNICODE_Lo, //U+178B
UNICODE_Lo, //U+178C
UNICODE_Lo, //U+178D
UNICODE_Lo, //U+178E
UNICODE_Lo, //U+178F
UNICODE_Lo, //U+1790
UNICODE_Lo, //U+1791
UNICODE_Lo, //U+1792
UNICODE_Lo, //U+1793
UNICODE_Lo, //U+1794
UNICODE_Lo, //U+1795
UNICODE_Lo, //U+1796
UNICODE_Lo, //U+1797
UNICODE_Lo, //U+1798
UNICODE_Lo, //U+1799
UNICODE_Lo, //U+179A
UNICODE_Lo, //U+179B
UNICODE_Lo, //U+179C
UNICODE_Lo, //U+179D
UNICODE_Lo, //U+179E
UNICODE_Lo, //U+179F
UNICODE_Lo, //U+17A0
UNICODE_Lo, //U+17A1
UNICODE_Lo, //U+17A2
UNICODE_Lo, //U+17A3
UNICODE_Lo, //U+17A4
UNICODE_Lo, //U+17A5
UNICODE_Lo, //U+17A6
UNICODE_Lo, //U+17A7
UNICODE_Lo, //U+17A8
UNICODE_Lo, //U+17A9
UNICODE_Lo, //U+17AA
UNICODE_Lo, //U+17AB
UNICODE_Lo, //U+17AC
UNICODE_Lo, //U+17AD
UNICODE_Lo, //U+17AE
UNICODE_Lo, //U+17AF
UNICODE_Lo, //U+17B0
UNICODE_Lo, //U+17B1
UNICODE_Lo, //U+17B2
UNICODE_Lo, //U+17B3
UNICODE_Cf, //U+17B4
UNICODE_Cf, //U+17B5
UNICODE_Mc, //U+17B6
UNICODE_Mn, //U+17B7
UNICODE_Mn, //U+17B8
UNICODE_Mn, //U+17B9
UNICODE_Mn, //U+17BA
UNICODE_Mn, //U+17BB
UNICODE_Mn, //U+17BC
UNICODE_Mn, //U+17BD
UNICODE_Mc, //U+17BE
UNICODE_Mc, //U+17BF
UNICODE_Mc, //U+17C0
UNICODE_Mc, //U+17C1
UNICODE_Mc, //U+17C2
UNICODE_Mc, //U+17C3
UNICODE_Mc, //U+17C4
UNICODE_Mc, //U+17C5
UNICODE_Mn, //U+17C6
UNICODE_Mc, //U+17C7
UNICODE_Mc, //U+17C8
UNICODE_Mn, //U+17C9
UNICODE_Mn, //U+17CA
UNICODE_Mn, //U+17CB
UNICODE_Mn, //U+17CC
UNICODE_Mn, //U+17CD
UNICODE_Mn, //U+17CE
UNICODE_Mn, //U+17CF
UNICODE_Mn, //U+17D0
UNICODE_Mn, //U+17D1
UNICODE_Mn, //U+17D2
UNICODE_Mn, //U+17D3
UNICODE_Po, //U+17D4
UNICODE_Po, //U+17D5
UNICODE_Po, //U+17D6
UNICODE_Lm, //U+17D7
UNICODE_Po, //U+17D8
UNICODE_Po, //U+17D9
UNICODE_Po, //U+17DA
UNICODE_Sc, //U+17DB
UNICODE_Lo, //U+17DC
UNICODE_Mn, //U+17DD
0, //U+17DE
0, //U+17DF
UNICODE_Nd, //U+17E0
UNICODE_Nd, //U+17E1
UNICODE_Nd, //U+17E2
UNICODE_Nd, //U+17E3
UNICODE_Nd, //U+17E4
UNICODE_Nd, //U+17E5
UNICODE_Nd, //U+17E6
UNICODE_Nd, //U+17E7
UNICODE_Nd, //U+17E8
UNICODE_Nd, //U+17E9
0, //U+17EA
0, //U+17EB
0, //U+17EC
0, //U+17ED
0, //U+17EE
0, //U+17EF
UNICODE_No, //U+17F0
UNICODE_No, //U+17F1
UNICODE_No, //U+17F2
UNICODE_No, //U+17F3
UNICODE_No, //U+17F4
UNICODE_No, //U+17F5
UNICODE_No, //U+17F6
UNICODE_No, //U+17F7
UNICODE_No, //U+17F8
UNICODE_No, //U+17F9
0, //U+17FA
0, //U+17FB
0, //U+17FC
0, //U+17FD
0, //U+17FE
0, //U+17FF
UNICODE_Po, //U+1800
UNICODE_Po, //U+1801
UNICODE_Po, //U+1802
UNICODE_Po, //U+1803
UNICODE_Po, //U+1804
UNICODE_Po, //U+1805
UNICODE_Pd, //U+1806
UNICODE_Po, //U+1807
UNICODE_Po, //U+1808
UNICODE_Po, //U+1809
UNICODE_Po, //U+180A
UNICODE_Mn, //U+180B
UNICODE_Mn, //U+180C
UNICODE_Mn, //U+180D
UNICODE_Zs, //U+180E
0, //U+180F
UNICODE_Nd, //U+1810
UNICODE_Nd, //U+1811
UNICODE_Nd, //U+1812
UNICODE_Nd, //U+1813
UNICODE_Nd, //U+1814
UNICODE_Nd, //U+1815
UNICODE_Nd, //U+1816
UNICODE_Nd, //U+1817
UNICODE_Nd, //U+1818
UNICODE_Nd, //U+1819
0, //U+181A
0, //U+181B
0, //U+181C
0, //U+181D
0, //U+181E
0, //U+181F
UNICODE_Lo, //U+1820
UNICODE_Lo, //U+1821
UNICODE_Lo, //U+1822
UNICODE_Lo, //U+1823
UNICODE_Lo, //U+1824
UNICODE_Lo, //U+1825
UNICODE_Lo, //U+1826
UNICODE_Lo, //U+1827
UNICODE_Lo, //U+1828
UNICODE_Lo, //U+1829
UNICODE_Lo, //U+182A
UNICODE_Lo, //U+182B
UNICODE_Lo, //U+182C
UNICODE_Lo, //U+182D
UNICODE_Lo, //U+182E
UNICODE_Lo, //U+182F
UNICODE_Lo, //U+1830
UNICODE_Lo, //U+1831
UNICODE_Lo, //U+1832
UNICODE_Lo, //U+1833
UNICODE_Lo, //U+1834
UNICODE_Lo, //U+1835
UNICODE_Lo, //U+1836
UNICODE_Lo, //U+1837
UNICODE_Lo, //U+1838
UNICODE_Lo, //U+1839
UNICODE_Lo, //U+183A
UNICODE_Lo, //U+183B
UNICODE_Lo, //U+183C
UNICODE_Lo, //U+183D
UNICODE_Lo, //U+183E
UNICODE_Lo, //U+183F
UNICODE_Lo, //U+1840
UNICODE_Lo, //U+1841
UNICODE_Lo, //U+1842
UNICODE_Lm, //U+1843
UNICODE_Lo, //U+1844
UNICODE_Lo, //U+1845
UNICODE_Lo, //U+1846
UNICODE_Lo, //U+1847
UNICODE_Lo, //U+1848
UNICODE_Lo, //U+1849
UNICODE_Lo, //U+184A
UNICODE_Lo, //U+184B
UNICODE_Lo, //U+184C
UNICODE_Lo, //U+184D
UNICODE_Lo, //U+184E
UNICODE_Lo, //U+184F
UNICODE_Lo, //U+1850
UNICODE_Lo, //U+1851
UNICODE_Lo, //U+1852
UNICODE_Lo, //U+1853
UNICODE_Lo, //U+1854
UNICODE_Lo, //U+1855
UNICODE_Lo, //U+1856
UNICODE_Lo, //U+1857
UNICODE_Lo, //U+1858
UNICODE_Lo, //U+1859
UNICODE_Lo, //U+185A
UNICODE_Lo, //U+185B
UNICODE_Lo, //U+185C
UNICODE_Lo, //U+185D
UNICODE_Lo, //U+185E
UNICODE_Lo, //U+185F
UNICODE_Lo, //U+1860
UNICODE_Lo, //U+1861
UNICODE_Lo, //U+1862
UNICODE_Lo, //U+1863
UNICODE_Lo, //U+1864
UNICODE_Lo, //U+1865
UNICODE_Lo, //U+1866
UNICODE_Lo, //U+1867
UNICODE_Lo, //U+1868
UNICODE_Lo, //U+1869
UNICODE_Lo, //U+186A
UNICODE_Lo, //U+186B
UNICODE_Lo, //U+186C
UNICODE_Lo, //U+186D
UNICODE_Lo, //U+186E
UNICODE_Lo, //U+186F
UNICODE_Lo, //U+1870
UNICODE_Lo, //U+1871
UNICODE_Lo, //U+1872
UNICODE_Lo, //U+1873
UNICODE_Lo, //U+1874
UNICODE_Lo, //U+1875
UNICODE_Lo, //U+1876
UNICODE_Lo, //U+1877
0, //U+1878
0, //U+1879
0, //U+187A
0, //U+187B
0, //U+187C
0, //U+187D
0, //U+187E
0, //U+187F
UNICODE_Lo, //U+1880
UNICODE_Lo, //U+1881
UNICODE_Lo, //U+1882
UNICODE_Lo, //U+1883
UNICODE_Lo, //U+1884
UNICODE_Lo, //U+1885
UNICODE_Lo, //U+1886
UNICODE_Lo, //U+1887
UNICODE_Lo, //U+1888
UNICODE_Lo, //U+1889
UNICODE_Lo, //U+188A
UNICODE_Lo, //U+188B
UNICODE_Lo, //U+188C
UNICODE_Lo, //U+188D
UNICODE_Lo, //U+188E
UNICODE_Lo, //U+188F
UNICODE_Lo, //U+1890
UNICODE_Lo, //U+1891
UNICODE_Lo, //U+1892
UNICODE_Lo, //U+1893
UNICODE_Lo, //U+1894
UNICODE_Lo, //U+1895
UNICODE_Lo, //U+1896
UNICODE_Lo, //U+1897
UNICODE_Lo, //U+1898
UNICODE_Lo, //U+1899
UNICODE_Lo, //U+189A
UNICODE_Lo, //U+189B
UNICODE_Lo, //U+189C
UNICODE_Lo, //U+189D
UNICODE_Lo, //U+189E
UNICODE_Lo, //U+189F
UNICODE_Lo, //U+18A0
UNICODE_Lo, //U+18A1
UNICODE_Lo, //U+18A2
UNICODE_Lo, //U+18A3
UNICODE_Lo, //U+18A4
UNICODE_Lo, //U+18A5
UNICODE_Lo, //U+18A6
UNICODE_Lo, //U+18A7
UNICODE_Lo, //U+18A8
UNICODE_Mn, //U+18A9
UNICODE_Lo, //U+18AA
0, //U+18AB
0, //U+18AC
0, //U+18AD
0, //U+18AE
0, //U+18AF
UNICODE_Lo, //U+18B0
UNICODE_Lo, //U+18B1
UNICODE_Lo, //U+18B2
UNICODE_Lo, //U+18B3
UNICODE_Lo, //U+18B4
UNICODE_Lo, //U+18B5
UNICODE_Lo, //U+18B6
UNICODE_Lo, //U+18B7
UNICODE_Lo, //U+18B8
UNICODE_Lo, //U+18B9
UNICODE_Lo, //U+18BA
UNICODE_Lo, //U+18BB
UNICODE_Lo, //U+18BC
UNICODE_Lo, //U+18BD
UNICODE_Lo, //U+18BE
UNICODE_Lo, //U+18BF
UNICODE_Lo, //U+18C0
UNICODE_Lo, //U+18C1
UNICODE_Lo, //U+18C2
UNICODE_Lo, //U+18C3
UNICODE_Lo, //U+18C4
UNICODE_Lo, //U+18C5
UNICODE_Lo, //U+18C6
UNICODE_Lo, //U+18C7
UNICODE_Lo, //U+18C8
UNICODE_Lo, //U+18C9
UNICODE_Lo, //U+18CA
UNICODE_Lo, //U+18CB
UNICODE_Lo, //U+18CC
UNICODE_Lo, //U+18CD
UNICODE_Lo, //U+18CE
UNICODE_Lo, //U+18CF
UNICODE_Lo, //U+18D0
UNICODE_Lo, //U+18D1
UNICODE_Lo, //U+18D2
UNICODE_Lo, //U+18D3
UNICODE_Lo, //U+18D4
UNICODE_Lo, //U+18D5
UNICODE_Lo, //U+18D6
UNICODE_Lo, //U+18D7
UNICODE_Lo, //U+18D8
UNICODE_Lo, //U+18D9
UNICODE_Lo, //U+18DA
UNICODE_Lo, //U+18DB
UNICODE_Lo, //U+18DC
UNICODE_Lo, //U+18DD
UNICODE_Lo, //U+18DE
UNICODE_Lo, //U+18DF
UNICODE_Lo, //U+18E0
UNICODE_Lo, //U+18E1
UNICODE_Lo, //U+18E2
UNICODE_Lo, //U+18E3
UNICODE_Lo, //U+18E4
UNICODE_Lo, //U+18E5
UNICODE_Lo, //U+18E6
UNICODE_Lo, //U+18E7
UNICODE_Lo, //U+18E8
UNICODE_Lo, //U+18E9
UNICODE_Lo, //U+18EA
UNICODE_Lo, //U+18EB
UNICODE_Lo, //U+18EC
UNICODE_Lo, //U+18ED
UNICODE_Lo, //U+18EE
UNICODE_Lo, //U+18EF
UNICODE_Lo, //U+18F0
UNICODE_Lo, //U+18F1
UNICODE_Lo, //U+18F2
UNICODE_Lo, //U+18F3
UNICODE_Lo, //U+18F4
UNICODE_Lo, //U+18F5
0, //U+18F6
0, //U+18F7
0, //U+18F8
0, //U+18F9
0, //U+18FA
0, //U+18FB
0, //U+18FC
0, //U+18FD
0, //U+18FE
0, //U+18FF
UNICODE_Lo, //U+1900
UNICODE_Lo, //U+1901
UNICODE_Lo, //U+1902
UNICODE_Lo, //U+1903
UNICODE_Lo, //U+1904
UNICODE_Lo, //U+1905
UNICODE_Lo, //U+1906
UNICODE_Lo, //U+1907
UNICODE_Lo, //U+1908
UNICODE_Lo, //U+1909
UNICODE_Lo, //U+190A
UNICODE_Lo, //U+190B
UNICODE_Lo, //U+190C
UNICODE_Lo, //U+190D
UNICODE_Lo, //U+190E
UNICODE_Lo, //U+190F
UNICODE_Lo, //U+1910
UNICODE_Lo, //U+1911
UNICODE_Lo, //U+1912
UNICODE_Lo, //U+1913
UNICODE_Lo, //U+1914
UNICODE_Lo, //U+1915
UNICODE_Lo, //U+1916
UNICODE_Lo, //U+1917
UNICODE_Lo, //U+1918
UNICODE_Lo, //U+1919
UNICODE_Lo, //U+191A
UNICODE_Lo, //U+191B
UNICODE_Lo, //U+191C
0, //U+191D
0, //U+191E
0, //U+191F
UNICODE_Mn, //U+1920
UNICODE_Mn, //U+1921
UNICODE_Mn, //U+1922
UNICODE_Mc, //U+1923
UNICODE_Mc, //U+1924
UNICODE_Mc, //U+1925
UNICODE_Mc, //U+1926
UNICODE_Mn, //U+1927
UNICODE_Mn, //U+1928
UNICODE_Mc, //U+1929
UNICODE_Mc, //U+192A
UNICODE_Mc, //U+192B
0, //U+192C
0, //U+192D
0, //U+192E
0, //U+192F
UNICODE_Mc, //U+1930
UNICODE_Mc, //U+1931
UNICODE_Mn, //U+1932
UNICODE_Mc, //U+1933
UNICODE_Mc, //U+1934
UNICODE_Mc, //U+1935
UNICODE_Mc, //U+1936
UNICODE_Mc, //U+1937
UNICODE_Mc, //U+1938
UNICODE_Mn, //U+1939
UNICODE_Mn, //U+193A
UNICODE_Mn, //U+193B
0, //U+193C
0, //U+193D
0, //U+193E
0, //U+193F
UNICODE_So, //U+1940
0, //U+1941
0, //U+1942
0, //U+1943
UNICODE_Po, //U+1944
UNICODE_Po, //U+1945
UNICODE_Nd, //U+1946
UNICODE_Nd, //U+1947
UNICODE_Nd, //U+1948
UNICODE_Nd, //U+1949
UNICODE_Nd, //U+194A
UNICODE_Nd, //U+194B
UNICODE_Nd, //U+194C
UNICODE_Nd, //U+194D
UNICODE_Nd, //U+194E
UNICODE_Nd, //U+194F
UNICODE_Lo, //U+1950
UNICODE_Lo, //U+1951
UNICODE_Lo, //U+1952
UNICODE_Lo, //U+1953
UNICODE_Lo, //U+1954
UNICODE_Lo, //U+1955
UNICODE_Lo, //U+1956
UNICODE_Lo, //U+1957
UNICODE_Lo, //U+1958
UNICODE_Lo, //U+1959
UNICODE_Lo, //U+195A
UNICODE_Lo, //U+195B
UNICODE_Lo, //U+195C
UNICODE_Lo, //U+195D
UNICODE_Lo, //U+195E
UNICODE_Lo, //U+195F
UNICODE_Lo, //U+1960
UNICODE_Lo, //U+1961
UNICODE_Lo, //U+1962
UNICODE_Lo, //U+1963
UNICODE_Lo, //U+1964
UNICODE_Lo, //U+1965
UNICODE_Lo, //U+1966
UNICODE_Lo, //U+1967
UNICODE_Lo, //U+1968
UNICODE_Lo, //U+1969
UNICODE_Lo, //U+196A
UNICODE_Lo, //U+196B
UNICODE_Lo, //U+196C
UNICODE_Lo, //U+196D
0, //U+196E
0, //U+196F
UNICODE_Lo, //U+1970
UNICODE_Lo, //U+1971
UNICODE_Lo, //U+1972
UNICODE_Lo, //U+1973
UNICODE_Lo, //U+1974
0, //U+1975
0, //U+1976
0, //U+1977
0, //U+1978
0, //U+1979
0, //U+197A
0, //U+197B
0, //U+197C
0, //U+197D
0, //U+197E
0, //U+197F
UNICODE_Lo, //U+1980
UNICODE_Lo, //U+1981
UNICODE_Lo, //U+1982
UNICODE_Lo, //U+1983
UNICODE_Lo, //U+1984
UNICODE_Lo, //U+1985
UNICODE_Lo, //U+1986
UNICODE_Lo, //U+1987
UNICODE_Lo, //U+1988
UNICODE_Lo, //U+1989
UNICODE_Lo, //U+198A
UNICODE_Lo, //U+198B
UNICODE_Lo, //U+198C
UNICODE_Lo, //U+198D
UNICODE_Lo, //U+198E
UNICODE_Lo, //U+198F
UNICODE_Lo, //U+1990
UNICODE_Lo, //U+1991
UNICODE_Lo, //U+1992
UNICODE_Lo, //U+1993
UNICODE_Lo, //U+1994
UNICODE_Lo, //U+1995
UNICODE_Lo, //U+1996
UNICODE_Lo, //U+1997
UNICODE_Lo, //U+1998
UNICODE_Lo, //U+1999
UNICODE_Lo, //U+199A
UNICODE_Lo, //U+199B
UNICODE_Lo, //U+199C
UNICODE_Lo, //U+199D
UNICODE_Lo, //U+199E
UNICODE_Lo, //U+199F
UNICODE_Lo, //U+19A0
UNICODE_Lo, //U+19A1
UNICODE_Lo, //U+19A2
UNICODE_Lo, //U+19A3
UNICODE_Lo, //U+19A4
UNICODE_Lo, //U+19A5
UNICODE_Lo, //U+19A6
UNICODE_Lo, //U+19A7
UNICODE_Lo, //U+19A8
UNICODE_Lo, //U+19A9
UNICODE_Lo, //U+19AA
UNICODE_Lo, //U+19AB
0, //U+19AC
0, //U+19AD
0, //U+19AE
0, //U+19AF
UNICODE_Mc, //U+19B0
UNICODE_Mc, //U+19B1
UNICODE_Mc, //U+19B2
UNICODE_Mc, //U+19B3
UNICODE_Mc, //U+19B4
UNICODE_Mc, //U+19B5
UNICODE_Mc, //U+19B6
UNICODE_Mc, //U+19B7
UNICODE_Mc, //U+19B8
UNICODE_Mc, //U+19B9
UNICODE_Mc, //U+19BA
UNICODE_Mc, //U+19BB
UNICODE_Mc, //U+19BC
UNICODE_Mc, //U+19BD
UNICODE_Mc, //U+19BE
UNICODE_Mc, //U+19BF
UNICODE_Mc, //U+19C0
UNICODE_Lo, //U+19C1
UNICODE_Lo, //U+19C2
UNICODE_Lo, //U+19C3
UNICODE_Lo, //U+19C4
UNICODE_Lo, //U+19C5
UNICODE_Lo, //U+19C6
UNICODE_Lo, //U+19C7
UNICODE_Mc, //U+19C8
UNICODE_Mc, //U+19C9
0, //U+19CA
0, //U+19CB
0, //U+19CC
0, //U+19CD
0, //U+19CE
0, //U+19CF
UNICODE_Nd, //U+19D0
UNICODE_Nd, //U+19D1
UNICODE_Nd, //U+19D2
UNICODE_Nd, //U+19D3
UNICODE_Nd, //U+19D4
UNICODE_Nd, //U+19D5
UNICODE_Nd, //U+19D6
UNICODE_Nd, //U+19D7
UNICODE_Nd, //U+19D8
UNICODE_Nd, //U+19D9
UNICODE_No, //U+19DA
0, //U+19DB
0, //U+19DC
0, //U+19DD
UNICODE_So, //U+19DE
UNICODE_So, //U+19DF
UNICODE_So, //U+19E0
UNICODE_So, //U+19E1
UNICODE_So, //U+19E2
UNICODE_So, //U+19E3
UNICODE_So, //U+19E4
UNICODE_So, //U+19E5
UNICODE_So, //U+19E6
UNICODE_So, //U+19E7
UNICODE_So, //U+19E8
UNICODE_So, //U+19E9
UNICODE_So, //U+19EA
UNICODE_So, //U+19EB
UNICODE_So, //U+19EC
UNICODE_So, //U+19ED
UNICODE_So, //U+19EE
UNICODE_So, //U+19EF
UNICODE_So, //U+19F0
UNICODE_So, //U+19F1
UNICODE_So, //U+19F2
UNICODE_So, //U+19F3
UNICODE_So, //U+19F4
UNICODE_So, //U+19F5
UNICODE_So, //U+19F6
UNICODE_So, //U+19F7
UNICODE_So, //U+19F8
UNICODE_So, //U+19F9
UNICODE_So, //U+19FA
UNICODE_So, //U+19FB
UNICODE_So, //U+19FC
UNICODE_So, //U+19FD
UNICODE_So, //U+19FE
UNICODE_So, //U+19FF
UNICODE_Lo, //U+1A00
UNICODE_Lo, //U+1A01
UNICODE_Lo, //U+1A02
UNICODE_Lo, //U+1A03
UNICODE_Lo, //U+1A04
UNICODE_Lo, //U+1A05
UNICODE_Lo, //U+1A06
UNICODE_Lo, //U+1A07
UNICODE_Lo, //U+1A08
UNICODE_Lo, //U+1A09
UNICODE_Lo, //U+1A0A
UNICODE_Lo, //U+1A0B
UNICODE_Lo, //U+1A0C
UNICODE_Lo, //U+1A0D
UNICODE_Lo, //U+1A0E
UNICODE_Lo, //U+1A0F
UNICODE_Lo, //U+1A10
UNICODE_Lo, //U+1A11
UNICODE_Lo, //U+1A12
UNICODE_Lo, //U+1A13
UNICODE_Lo, //U+1A14
UNICODE_Lo, //U+1A15
UNICODE_Lo, //U+1A16
UNICODE_Mn, //U+1A17
UNICODE_Mn, //U+1A18
UNICODE_Mc, //U+1A19
UNICODE_Mc, //U+1A1A
UNICODE_Mc, //U+1A1B
0, //U+1A1C
0, //U+1A1D
UNICODE_Po, //U+1A1E
UNICODE_Po, //U+1A1F
UNICODE_Lo, //U+1A20
UNICODE_Lo, //U+1A21
UNICODE_Lo, //U+1A22
UNICODE_Lo, //U+1A23
UNICODE_Lo, //U+1A24
UNICODE_Lo, //U+1A25
UNICODE_Lo, //U+1A26
UNICODE_Lo, //U+1A27
UNICODE_Lo, //U+1A28
UNICODE_Lo, //U+1A29
UNICODE_Lo, //U+1A2A
UNICODE_Lo, //U+1A2B
UNICODE_Lo, //U+1A2C
UNICODE_Lo, //U+1A2D
UNICODE_Lo, //U+1A2E
UNICODE_Lo, //U+1A2F
UNICODE_Lo, //U+1A30
UNICODE_Lo, //U+1A31
UNICODE_Lo, //U+1A32
UNICODE_Lo, //U+1A33
UNICODE_Lo, //U+1A34
UNICODE_Lo, //U+1A35
UNICODE_Lo, //U+1A36
UNICODE_Lo, //U+1A37
UNICODE_Lo, //U+1A38
UNICODE_Lo, //U+1A39
UNICODE_Lo, //U+1A3A
UNICODE_Lo, //U+1A3B
UNICODE_Lo, //U+1A3C
UNICODE_Lo, //U+1A3D
UNICODE_Lo, //U+1A3E
UNICODE_Lo, //U+1A3F
UNICODE_Lo, //U+1A40
UNICODE_Lo, //U+1A41
UNICODE_Lo, //U+1A42
UNICODE_Lo, //U+1A43
UNICODE_Lo, //U+1A44
UNICODE_Lo, //U+1A45
UNICODE_Lo, //U+1A46
UNICODE_Lo, //U+1A47
UNICODE_Lo, //U+1A48
UNICODE_Lo, //U+1A49
UNICODE_Lo, //U+1A4A
UNICODE_Lo, //U+1A4B
UNICODE_Lo, //U+1A4C
UNICODE_Lo, //U+1A4D
UNICODE_Lo, //U+1A4E
UNICODE_Lo, //U+1A4F
UNICODE_Lo, //U+1A50
UNICODE_Lo, //U+1A51
UNICODE_Lo, //U+1A52
UNICODE_Lo, //U+1A53
UNICODE_Lo, //U+1A54
UNICODE_Mc, //U+1A55
UNICODE_Mn, //U+1A56
UNICODE_Mc, //U+1A57
UNICODE_Mn, //U+1A58
UNICODE_Mn, //U+1A59
UNICODE_Mn, //U+1A5A
UNICODE_Mn, //U+1A5B
UNICODE_Mn, //U+1A5C
UNICODE_Mn, //U+1A5D
UNICODE_Mn, //U+1A5E
0, //U+1A5F
UNICODE_Mn, //U+1A60
UNICODE_Mc, //U+1A61
UNICODE_Mn, //U+1A62
UNICODE_Mc, //U+1A63
UNICODE_Mc, //U+1A64
UNICODE_Mn, //U+1A65
UNICODE_Mn, //U+1A66
UNICODE_Mn, //U+1A67
UNICODE_Mn, //U+1A68
UNICODE_Mn, //U+1A69
UNICODE_Mn, //U+1A6A
UNICODE_Mn, //U+1A6B
UNICODE_Mn, //U+1A6C
UNICODE_Mc, //U+1A6D
UNICODE_Mc, //U+1A6E
UNICODE_Mc, //U+1A6F
UNICODE_Mc, //U+1A70
UNICODE_Mc, //U+1A71
UNICODE_Mc, //U+1A72
UNICODE_Mn, //U+1A73
UNICODE_Mn, //U+1A74
UNICODE_Mn, //U+1A75
UNICODE_Mn, //U+1A76
UNICODE_Mn, //U+1A77
UNICODE_Mn, //U+1A78
UNICODE_Mn, //U+1A79
UNICODE_Mn, //U+1A7A
UNICODE_Mn, //U+1A7B
UNICODE_Mn, //U+1A7C
0, //U+1A7D
0, //U+1A7E
UNICODE_Mn, //U+1A7F
UNICODE_Nd, //U+1A80
UNICODE_Nd, //U+1A81
UNICODE_Nd, //U+1A82
UNICODE_Nd, //U+1A83
UNICODE_Nd, //U+1A84
UNICODE_Nd, //U+1A85
UNICODE_Nd, //U+1A86
UNICODE_Nd, //U+1A87
UNICODE_Nd, //U+1A88
UNICODE_Nd, //U+1A89
0, //U+1A8A
0, //U+1A8B
0, //U+1A8C
0, //U+1A8D
0, //U+1A8E
0, //U+1A8F
UNICODE_Nd, //U+1A90
UNICODE_Nd, //U+1A91
UNICODE_Nd, //U+1A92
UNICODE_Nd, //U+1A93
UNICODE_Nd, //U+1A94
UNICODE_Nd, //U+1A95
UNICODE_Nd, //U+1A96
UNICODE_Nd, //U+1A97
UNICODE_Nd, //U+1A98
UNICODE_Nd, //U+1A99
0, //U+1A9A
0, //U+1A9B
0, //U+1A9C
0, //U+1A9D
0, //U+1A9E
0, //U+1A9F
UNICODE_Po, //U+1AA0
UNICODE_Po, //U+1AA1
UNICODE_Po, //U+1AA2
UNICODE_Po, //U+1AA3
UNICODE_Po, //U+1AA4
UNICODE_Po, //U+1AA5
UNICODE_Po, //U+1AA6
UNICODE_Lm, //U+1AA7
UNICODE_Po, //U+1AA8
UNICODE_Po, //U+1AA9
UNICODE_Po, //U+1AAA
UNICODE_Po, //U+1AAB
UNICODE_Po, //U+1AAC
UNICODE_Po, //U+1AAD
0, //U+1AAE
0, //U+1AAF
0, //U+1AB0
0, //U+1AB1
0, //U+1AB2
0, //U+1AB3
0, //U+1AB4
0, //U+1AB5
0, //U+1AB6
0, //U+1AB7
0, //U+1AB8
0, //U+1AB9
0, //U+1ABA
0, //U+1ABB
0, //U+1ABC
0, //U+1ABD
0, //U+1ABE
0, //U+1ABF
0, //U+1AC0
0, //U+1AC1
0, //U+1AC2
0, //U+1AC3
0, //U+1AC4
0, //U+1AC5
0, //U+1AC6
0, //U+1AC7
0, //U+1AC8
0, //U+1AC9
0, //U+1ACA
0, //U+1ACB
0, //U+1ACC
0, //U+1ACD
0, //U+1ACE
0, //U+1ACF
0, //U+1AD0
0, //U+1AD1
0, //U+1AD2
0, //U+1AD3
0, //U+1AD4
0, //U+1AD5
0, //U+1AD6
0, //U+1AD7
0, //U+1AD8
0, //U+1AD9
0, //U+1ADA
0, //U+1ADB
0, //U+1ADC
0, //U+1ADD
0, //U+1ADE
0, //U+1ADF
0, //U+1AE0
0, //U+1AE1
0, //U+1AE2
0, //U+1AE3
0, //U+1AE4
0, //U+1AE5
0, //U+1AE6
0, //U+1AE7
0, //U+1AE8
0, //U+1AE9
0, //U+1AEA
0, //U+1AEB
0, //U+1AEC
0, //U+1AED
0, //U+1AEE
0, //U+1AEF
0, //U+1AF0
0, //U+1AF1
0, //U+1AF2
0, //U+1AF3
0, //U+1AF4
0, //U+1AF5
0, //U+1AF6
0, //U+1AF7
0, //U+1AF8
0, //U+1AF9
0, //U+1AFA
0, //U+1AFB
0, //U+1AFC
0, //U+1AFD
0, //U+1AFE
0, //U+1AFF
UNICODE_Mn, //U+1B00
UNICODE_Mn, //U+1B01
UNICODE_Mn, //U+1B02
UNICODE_Mn, //U+1B03
UNICODE_Mc, //U+1B04
UNICODE_Lo, //U+1B05
UNICODE_Lo, //U+1B06
UNICODE_Lo, //U+1B07
UNICODE_Lo, //U+1B08
UNICODE_Lo, //U+1B09
UNICODE_Lo, //U+1B0A
UNICODE_Lo, //U+1B0B
UNICODE_Lo, //U+1B0C
UNICODE_Lo, //U+1B0D
UNICODE_Lo, //U+1B0E
UNICODE_Lo, //U+1B0F
UNICODE_Lo, //U+1B10
UNICODE_Lo, //U+1B11
UNICODE_Lo, //U+1B12
UNICODE_Lo, //U+1B13
UNICODE_Lo, //U+1B14
UNICODE_Lo, //U+1B15
UNICODE_Lo, //U+1B16
UNICODE_Lo, //U+1B17
UNICODE_Lo, //U+1B18
UNICODE_Lo, //U+1B19
UNICODE_Lo, //U+1B1A
UNICODE_Lo, //U+1B1B
UNICODE_Lo, //U+1B1C
UNICODE_Lo, //U+1B1D
UNICODE_Lo, //U+1B1E
UNICODE_Lo, //U+1B1F
UNICODE_Lo, //U+1B20
UNICODE_Lo, //U+1B21
UNICODE_Lo, //U+1B22
UNICODE_Lo, //U+1B23
UNICODE_Lo, //U+1B24
UNICODE_Lo, //U+1B25
UNICODE_Lo, //U+1B26
UNICODE_Lo, //U+1B27
UNICODE_Lo, //U+1B28
UNICODE_Lo, //U+1B29
UNICODE_Lo, //U+1B2A
UNICODE_Lo, //U+1B2B
UNICODE_Lo, //U+1B2C
UNICODE_Lo, //U+1B2D
UNICODE_Lo, //U+1B2E
UNICODE_Lo, //U+1B2F
UNICODE_Lo, //U+1B30
UNICODE_Lo, //U+1B31
UNICODE_Lo, //U+1B32
UNICODE_Lo, //U+1B33
UNICODE_Mn, //U+1B34
UNICODE_Mc, //U+1B35
UNICODE_Mn, //U+1B36
UNICODE_Mn, //U+1B37
UNICODE_Mn, //U+1B38
UNICODE_Mn, //U+1B39
UNICODE_Mn, //U+1B3A
UNICODE_Mc, //U+1B3B
UNICODE_Mn, //U+1B3C
UNICODE_Mc, //U+1B3D
UNICODE_Mc, //U+1B3E
UNICODE_Mc, //U+1B3F
UNICODE_Mc, //U+1B40
UNICODE_Mc, //U+1B41
UNICODE_Mn, //U+1B42
UNICODE_Mc, //U+1B43
UNICODE_Mc, //U+1B44
UNICODE_Lo, //U+1B45
UNICODE_Lo, //U+1B46
UNICODE_Lo, //U+1B47
UNICODE_Lo, //U+1B48
UNICODE_Lo, //U+1B49
UNICODE_Lo, //U+1B4A
UNICODE_Lo, //U+1B4B
0, //U+1B4C
0, //U+1B4D
0, //U+1B4E
0, //U+1B4F
UNICODE_Nd, //U+1B50
UNICODE_Nd, //U+1B51
UNICODE_Nd, //U+1B52
UNICODE_Nd, //U+1B53
UNICODE_Nd, //U+1B54
UNICODE_Nd, //U+1B55
UNICODE_Nd, //U+1B56
UNICODE_Nd, //U+1B57
UNICODE_Nd, //U+1B58
UNICODE_Nd, //U+1B59
UNICODE_Po, //U+1B5A
UNICODE_Po, //U+1B5B
UNICODE_Po, //U+1B5C
UNICODE_Po, //U+1B5D
UNICODE_Po, //U+1B5E
UNICODE_Po, //U+1B5F
UNICODE_Po, //U+1B60
UNICODE_So, //U+1B61
UNICODE_So, //U+1B62
UNICODE_So, //U+1B63
UNICODE_So, //U+1B64
UNICODE_So, //U+1B65
UNICODE_So, //U+1B66
UNICODE_So, //U+1B67
UNICODE_So, //U+1B68
UNICODE_So, //U+1B69
UNICODE_So, //U+1B6A
UNICODE_Mn, //U+1B6B
UNICODE_Mn, //U+1B6C
UNICODE_Mn, //U+1B6D
UNICODE_Mn, //U+1B6E
UNICODE_Mn, //U+1B6F
UNICODE_Mn, //U+1B70
UNICODE_Mn, //U+1B71
UNICODE_Mn, //U+1B72
UNICODE_Mn, //U+1B73
UNICODE_So, //U+1B74
UNICODE_So, //U+1B75
UNICODE_So, //U+1B76
UNICODE_So, //U+1B77
UNICODE_So, //U+1B78
UNICODE_So, //U+1B79
UNICODE_So, //U+1B7A
UNICODE_So, //U+1B7B
UNICODE_So, //U+1B7C
0, //U+1B7D
0, //U+1B7E
0, //U+1B7F
UNICODE_Mn, //U+1B80
UNICODE_Mn, //U+1B81
UNICODE_Mc, //U+1B82
UNICODE_Lo, //U+1B83
UNICODE_Lo, //U+1B84
UNICODE_Lo, //U+1B85
UNICODE_Lo, //U+1B86
UNICODE_Lo, //U+1B87
UNICODE_Lo, //U+1B88
UNICODE_Lo, //U+1B89
UNICODE_Lo, //U+1B8A
UNICODE_Lo, //U+1B8B
UNICODE_Lo, //U+1B8C
UNICODE_Lo, //U+1B8D
UNICODE_Lo, //U+1B8E
UNICODE_Lo, //U+1B8F
UNICODE_Lo, //U+1B90
UNICODE_Lo, //U+1B91
UNICODE_Lo, //U+1B92
UNICODE_Lo, //U+1B93
UNICODE_Lo, //U+1B94
UNICODE_Lo, //U+1B95
UNICODE_Lo, //U+1B96
UNICODE_Lo, //U+1B97
UNICODE_Lo, //U+1B98
UNICODE_Lo, //U+1B99
UNICODE_Lo, //U+1B9A
UNICODE_Lo, //U+1B9B
UNICODE_Lo, //U+1B9C
UNICODE_Lo, //U+1B9D
UNICODE_Lo, //U+1B9E
UNICODE_Lo, //U+1B9F
UNICODE_Lo, //U+1BA0
UNICODE_Mc, //U+1BA1
UNICODE_Mn, //U+1BA2
UNICODE_Mn, //U+1BA3
UNICODE_Mn, //U+1BA4
UNICODE_Mn, //U+1BA5
UNICODE_Mc, //U+1BA6
UNICODE_Mc, //U+1BA7
UNICODE_Mn, //U+1BA8
UNICODE_Mn, //U+1BA9
UNICODE_Mc, //U+1BAA
0, //U+1BAB
0, //U+1BAC
0, //U+1BAD
UNICODE_Lo, //U+1BAE
UNICODE_Lo, //U+1BAF
UNICODE_Nd, //U+1BB0
UNICODE_Nd, //U+1BB1
UNICODE_Nd, //U+1BB2
UNICODE_Nd, //U+1BB3
UNICODE_Nd, //U+1BB4
UNICODE_Nd, //U+1BB5
UNICODE_Nd, //U+1BB6
UNICODE_Nd, //U+1BB7
UNICODE_Nd, //U+1BB8
UNICODE_Nd, //U+1BB9
0, //U+1BBA
0, //U+1BBB
0, //U+1BBC
0, //U+1BBD
0, //U+1BBE
0, //U+1BBF
UNICODE_Lo, //U+1BC0
UNICODE_Lo, //U+1BC1
UNICODE_Lo, //U+1BC2
UNICODE_Lo, //U+1BC3
UNICODE_Lo, //U+1BC4
UNICODE_Lo, //U+1BC5
UNICODE_Lo, //U+1BC6
UNICODE_Lo, //U+1BC7
UNICODE_Lo, //U+1BC8
UNICODE_Lo, //U+1BC9
UNICODE_Lo, //U+1BCA
UNICODE_Lo, //U+1BCB
UNICODE_Lo, //U+1BCC
UNICODE_Lo, //U+1BCD
UNICODE_Lo, //U+1BCE
UNICODE_Lo, //U+1BCF
UNICODE_Lo, //U+1BD0
UNICODE_Lo, //U+1BD1
UNICODE_Lo, //U+1BD2
UNICODE_Lo, //U+1BD3
UNICODE_Lo, //U+1BD4
UNICODE_Lo, //U+1BD5
UNICODE_Lo, //U+1BD6
UNICODE_Lo, //U+1BD7
UNICODE_Lo, //U+1BD8
UNICODE_Lo, //U+1BD9
UNICODE_Lo, //U+1BDA
UNICODE_Lo, //U+1BDB
UNICODE_Lo, //U+1BDC
UNICODE_Lo, //U+1BDD
UNICODE_Lo, //U+1BDE
UNICODE_Lo, //U+1BDF
UNICODE_Lo, //U+1BE0
UNICODE_Lo, //U+1BE1
UNICODE_Lo, //U+1BE2
UNICODE_Lo, //U+1BE3
UNICODE_Lo, //U+1BE4
UNICODE_Lo, //U+1BE5
UNICODE_Mn, //U+1BE6
UNICODE_Mc, //U+1BE7
UNICODE_Mn, //U+1BE8
UNICODE_Mn, //U+1BE9
UNICODE_Mc, //U+1BEA
UNICODE_Mc, //U+1BEB
UNICODE_Mc, //U+1BEC
UNICODE_Mn, //U+1BED
UNICODE_Mc, //U+1BEE
UNICODE_Mn, //U+1BEF
UNICODE_Mn, //U+1BF0
UNICODE_Mn, //U+1BF1
UNICODE_Mc, //U+1BF2
UNICODE_Mc, //U+1BF3
0, //U+1BF4
0, //U+1BF5
0, //U+1BF6
0, //U+1BF7
0, //U+1BF8
0, //U+1BF9
0, //U+1BFA
0, //U+1BFB
UNICODE_Po, //U+1BFC
UNICODE_Po, //U+1BFD
UNICODE_Po, //U+1BFE
UNICODE_Po, //U+1BFF
UNICODE_Lo, //U+1C00
UNICODE_Lo, //U+1C01
UNICODE_Lo, //U+1C02
UNICODE_Lo, //U+1C03
UNICODE_Lo, //U+1C04
UNICODE_Lo, //U+1C05
UNICODE_Lo, //U+1C06
UNICODE_Lo, //U+1C07
UNICODE_Lo, //U+1C08
UNICODE_Lo, //U+1C09
UNICODE_Lo, //U+1C0A
UNICODE_Lo, //U+1C0B
UNICODE_Lo, //U+1C0C
UNICODE_Lo, //U+1C0D
UNICODE_Lo, //U+1C0E
UNICODE_Lo, //U+1C0F
UNICODE_Lo, //U+1C10
UNICODE_Lo, //U+1C11
UNICODE_Lo, //U+1C12
UNICODE_Lo, //U+1C13
UNICODE_Lo, //U+1C14
UNICODE_Lo, //U+1C15
UNICODE_Lo, //U+1C16
UNICODE_Lo, //U+1C17
UNICODE_Lo, //U+1C18
UNICODE_Lo, //U+1C19
UNICODE_Lo, //U+1C1A
UNICODE_Lo, //U+1C1B
UNICODE_Lo, //U+1C1C
UNICODE_Lo, //U+1C1D
UNICODE_Lo, //U+1C1E
UNICODE_Lo, //U+1C1F
UNICODE_Lo, //U+1C20
UNICODE_Lo, //U+1C21
UNICODE_Lo, //U+1C22
UNICODE_Lo, //U+1C23
UNICODE_Mc, //U+1C24
UNICODE_Mc, //U+1C25
UNICODE_Mc, //U+1C26
UNICODE_Mc, //U+1C27
UNICODE_Mc, //U+1C28
UNICODE_Mc, //U+1C29
UNICODE_Mc, //U+1C2A
UNICODE_Mc, //U+1C2B
UNICODE_Mn, //U+1C2C
UNICODE_Mn, //U+1C2D
UNICODE_Mn, //U+1C2E
UNICODE_Mn, //U+1C2F
UNICODE_Mn, //U+1C30
UNICODE_Mn, //U+1C31
UNICODE_Mn, //U+1C32
UNICODE_Mn, //U+1C33
UNICODE_Mc, //U+1C34
UNICODE_Mc, //U+1C35
UNICODE_Mn, //U+1C36
UNICODE_Mn, //U+1C37
0, //U+1C38
0, //U+1C39
0, //U+1C3A
UNICODE_Po, //U+1C3B
UNICODE_Po, //U+1C3C
UNICODE_Po, //U+1C3D
UNICODE_Po, //U+1C3E
UNICODE_Po, //U+1C3F
UNICODE_Nd, //U+1C40
UNICODE_Nd, //U+1C41
UNICODE_Nd, //U+1C42
UNICODE_Nd, //U+1C43
UNICODE_Nd, //U+1C44
UNICODE_Nd, //U+1C45
UNICODE_Nd, //U+1C46
UNICODE_Nd, //U+1C47
UNICODE_Nd, //U+1C48
UNICODE_Nd, //U+1C49
0, //U+1C4A
0, //U+1C4B
0, //U+1C4C
UNICODE_Lo, //U+1C4D
UNICODE_Lo, //U+1C4E
UNICODE_Lo, //U+1C4F
UNICODE_Nd, //U+1C50
UNICODE_Nd, //U+1C51
UNICODE_Nd, //U+1C52
UNICODE_Nd, //U+1C53
UNICODE_Nd, //U+1C54
UNICODE_Nd, //U+1C55
UNICODE_Nd, //U+1C56
UNICODE_Nd, //U+1C57
UNICODE_Nd, //U+1C58
UNICODE_Nd, //U+1C59
UNICODE_Lo, //U+1C5A
UNICODE_Lo, //U+1C5B
UNICODE_Lo, //U+1C5C
UNICODE_Lo, //U+1C5D
UNICODE_Lo, //U+1C5E
UNICODE_Lo, //U+1C5F
UNICODE_Lo, //U+1C60
UNICODE_Lo, //U+1C61
UNICODE_Lo, //U+1C62
UNICODE_Lo, //U+1C63
UNICODE_Lo, //U+1C64
UNICODE_Lo, //U+1C65
UNICODE_Lo, //U+1C66
UNICODE_Lo, //U+1C67
UNICODE_Lo, //U+1C68
UNICODE_Lo, //U+1C69
UNICODE_Lo, //U+1C6A
UNICODE_Lo, //U+1C6B
UNICODE_Lo, //U+1C6C
UNICODE_Lo, //U+1C6D
UNICODE_Lo, //U+1C6E
UNICODE_Lo, //U+1C6F
UNICODE_Lo, //U+1C70
UNICODE_Lo, //U+1C71
UNICODE_Lo, //U+1C72
UNICODE_Lo, //U+1C73
UNICODE_Lo, //U+1C74
UNICODE_Lo, //U+1C75
UNICODE_Lo, //U+1C76
UNICODE_Lo, //U+1C77
UNICODE_Lm, //U+1C78
UNICODE_Lm, //U+1C79
UNICODE_Lm, //U+1C7A
UNICODE_Lm, //U+1C7B
UNICODE_Lm, //U+1C7C
UNICODE_Lm, //U+1C7D
UNICODE_Po, //U+1C7E
UNICODE_Po, //U+1C7F
0, //U+1C80
0, //U+1C81
0, //U+1C82
0, //U+1C83
0, //U+1C84
0, //U+1C85
0, //U+1C86
0, //U+1C87
0, //U+1C88
0, //U+1C89
0, //U+1C8A
0, //U+1C8B
0, //U+1C8C
0, //U+1C8D
0, //U+1C8E
0, //U+1C8F
0, //U+1C90
0, //U+1C91
0, //U+1C92
0, //U+1C93
0, //U+1C94
0, //U+1C95
0, //U+1C96
0, //U+1C97
0, //U+1C98
0, //U+1C99
0, //U+1C9A
0, //U+1C9B
0, //U+1C9C
0, //U+1C9D
0, //U+1C9E
0, //U+1C9F
0, //U+1CA0
0, //U+1CA1
0, //U+1CA2
0, //U+1CA3
0, //U+1CA4
0, //U+1CA5
0, //U+1CA6
0, //U+1CA7
0, //U+1CA8
0, //U+1CA9
0, //U+1CAA
0, //U+1CAB
0, //U+1CAC
0, //U+1CAD
0, //U+1CAE
0, //U+1CAF
0, //U+1CB0
0, //U+1CB1
0, //U+1CB2
0, //U+1CB3
0, //U+1CB4
0, //U+1CB5
0, //U+1CB6
0, //U+1CB7
0, //U+1CB8
0, //U+1CB9
0, //U+1CBA
0, //U+1CBB
0, //U+1CBC
0, //U+1CBD
0, //U+1CBE
0, //U+1CBF
0, //U+1CC0
0, //U+1CC1
0, //U+1CC2
0, //U+1CC3
0, //U+1CC4
0, //U+1CC5
0, //U+1CC6
0, //U+1CC7
0, //U+1CC8
0, //U+1CC9
0, //U+1CCA
0, //U+1CCB
0, //U+1CCC
0, //U+1CCD
0, //U+1CCE
0, //U+1CCF
UNICODE_Mn, //U+1CD0
UNICODE_Mn, //U+1CD1
UNICODE_Mn, //U+1CD2
UNICODE_Po, //U+1CD3
UNICODE_Mn, //U+1CD4
UNICODE_Mn, //U+1CD5
UNICODE_Mn, //U+1CD6
UNICODE_Mn, //U+1CD7
UNICODE_Mn, //U+1CD8
UNICODE_Mn, //U+1CD9
UNICODE_Mn, //U+1CDA
UNICODE_Mn, //U+1CDB
UNICODE_Mn, //U+1CDC
UNICODE_Mn, //U+1CDD
UNICODE_Mn, //U+1CDE
UNICODE_Mn, //U+1CDF
UNICODE_Mn, //U+1CE0
UNICODE_Mc, //U+1CE1
UNICODE_Mn, //U+1CE2
UNICODE_Mn, //U+1CE3
UNICODE_Mn, //U+1CE4
UNICODE_Mn, //U+1CE5
UNICODE_Mn, //U+1CE6
UNICODE_Mn, //U+1CE7
UNICODE_Mn, //U+1CE8
UNICODE_Lo, //U+1CE9
UNICODE_Lo, //U+1CEA
UNICODE_Lo, //U+1CEB
UNICODE_Lo, //U+1CEC
UNICODE_Mn, //U+1CED
UNICODE_Lo, //U+1CEE
UNICODE_Lo, //U+1CEF
UNICODE_Lo, //U+1CF0
UNICODE_Lo, //U+1CF1
UNICODE_Mc, //U+1CF2
0, //U+1CF3
0, //U+1CF4
0, //U+1CF5
0, //U+1CF6
0, //U+1CF7
0, //U+1CF8
0, //U+1CF9
0, //U+1CFA
0, //U+1CFB
0, //U+1CFC
0, //U+1CFD
0, //U+1CFE
0, //U+1CFF
UNICODE_Ll, //U+1D00
UNICODE_Ll, //U+1D01
UNICODE_Ll, //U+1D02
UNICODE_Ll, //U+1D03
UNICODE_Ll, //U+1D04
UNICODE_Ll, //U+1D05
UNICODE_Ll, //U+1D06
UNICODE_Ll, //U+1D07
UNICODE_Ll, //U+1D08
UNICODE_Ll, //U+1D09
UNICODE_Ll, //U+1D0A
UNICODE_Ll, //U+1D0B
UNICODE_Ll, //U+1D0C
UNICODE_Ll, //U+1D0D
UNICODE_Ll, //U+1D0E
UNICODE_Ll, //U+1D0F
UNICODE_Ll, //U+1D10
UNICODE_Ll, //U+1D11
UNICODE_Ll, //U+1D12
UNICODE_Ll, //U+1D13
UNICODE_Ll, //U+1D14
UNICODE_Ll, //U+1D15
UNICODE_Ll, //U+1D16
UNICODE_Ll, //U+1D17
UNICODE_Ll, //U+1D18
UNICODE_Ll, //U+1D19
UNICODE_Ll, //U+1D1A
UNICODE_Ll, //U+1D1B
UNICODE_Ll, //U+1D1C
UNICODE_Ll, //U+1D1D
UNICODE_Ll, //U+1D1E
UNICODE_Ll, //U+1D1F
UNICODE_Ll, //U+1D20
UNICODE_Ll, //U+1D21
UNICODE_Ll, //U+1D22
UNICODE_Ll, //U+1D23
UNICODE_Ll, //U+1D24
UNICODE_Ll, //U+1D25
UNICODE_Ll, //U+1D26
UNICODE_Ll, //U+1D27
UNICODE_Ll, //U+1D28
UNICODE_Ll, //U+1D29
UNICODE_Ll, //U+1D2A
UNICODE_Ll, //U+1D2B
UNICODE_Lm, //U+1D2C
UNICODE_Lm, //U+1D2D
UNICODE_Lm, //U+1D2E
UNICODE_Lm, //U+1D2F
UNICODE_Lm, //U+1D30
UNICODE_Lm, //U+1D31
UNICODE_Lm, //U+1D32
UNICODE_Lm, //U+1D33
UNICODE_Lm, //U+1D34
UNICODE_Lm, //U+1D35
UNICODE_Lm, //U+1D36
UNICODE_Lm, //U+1D37
UNICODE_Lm, //U+1D38
UNICODE_Lm, //U+1D39
UNICODE_Lm, //U+1D3A
UNICODE_Lm, //U+1D3B
UNICODE_Lm, //U+1D3C
UNICODE_Lm, //U+1D3D
UNICODE_Lm, //U+1D3E
UNICODE_Lm, //U+1D3F
UNICODE_Lm, //U+1D40
UNICODE_Lm, //U+1D41
UNICODE_Lm, //U+1D42
UNICODE_Lm, //U+1D43
UNICODE_Lm, //U+1D44
UNICODE_Lm, //U+1D45
UNICODE_Lm, //U+1D46
UNICODE_Lm, //U+1D47
UNICODE_Lm, //U+1D48
UNICODE_Lm, //U+1D49
UNICODE_Lm, //U+1D4A
UNICODE_Lm, //U+1D4B
UNICODE_Lm, //U+1D4C
UNICODE_Lm, //U+1D4D
UNICODE_Lm, //U+1D4E
UNICODE_Lm, //U+1D4F
UNICODE_Lm, //U+1D50
UNICODE_Lm, //U+1D51
UNICODE_Lm, //U+1D52
UNICODE_Lm, //U+1D53
UNICODE_Lm, //U+1D54
UNICODE_Lm, //U+1D55
UNICODE_Lm, //U+1D56
UNICODE_Lm, //U+1D57
UNICODE_Lm, //U+1D58
UNICODE_Lm, //U+1D59
UNICODE_Lm, //U+1D5A
UNICODE_Lm, //U+1D5B
UNICODE_Lm, //U+1D5C
UNICODE_Lm, //U+1D5D
UNICODE_Lm, //U+1D5E
UNICODE_Lm, //U+1D5F
UNICODE_Lm, //U+1D60
UNICODE_Lm, //U+1D61
UNICODE_Ll, //U+1D62
UNICODE_Ll, //U+1D63
UNICODE_Ll, //U+1D64
UNICODE_Ll, //U+1D65
UNICODE_Ll, //U+1D66
UNICODE_Ll, //U+1D67
UNICODE_Ll, //U+1D68
UNICODE_Ll, //U+1D69
UNICODE_Ll, //U+1D6A
UNICODE_Ll, //U+1D6B
UNICODE_Ll, //U+1D6C
UNICODE_Ll, //U+1D6D
UNICODE_Ll, //U+1D6E
UNICODE_Ll, //U+1D6F
UNICODE_Ll, //U+1D70
UNICODE_Ll, //U+1D71
UNICODE_Ll, //U+1D72
UNICODE_Ll, //U+1D73
UNICODE_Ll, //U+1D74
UNICODE_Ll, //U+1D75
UNICODE_Ll, //U+1D76
UNICODE_Ll, //U+1D77
UNICODE_Lm, //U+1D78
UNICODE_Ll, //U+1D79
UNICODE_Ll, //U+1D7A
UNICODE_Ll, //U+1D7B
UNICODE_Ll, //U+1D7C
UNICODE_Ll, //U+1D7D
UNICODE_Ll, //U+1D7E
UNICODE_Ll, //U+1D7F
UNICODE_Ll, //U+1D80
UNICODE_Ll, //U+1D81
UNICODE_Ll, //U+1D82
UNICODE_Ll, //U+1D83
UNICODE_Ll, //U+1D84
UNICODE_Ll, //U+1D85
UNICODE_Ll, //U+1D86
UNICODE_Ll, //U+1D87
UNICODE_Ll, //U+1D88
UNICODE_Ll, //U+1D89
UNICODE_Ll, //U+1D8A
UNICODE_Ll, //U+1D8B
UNICODE_Ll, //U+1D8C
UNICODE_Ll, //U+1D8D
UNICODE_Ll, //U+1D8E
UNICODE_Ll, //U+1D8F
UNICODE_Ll, //U+1D90
UNICODE_Ll, //U+1D91
UNICODE_Ll, //U+1D92
UNICODE_Ll, //U+1D93
UNICODE_Ll, //U+1D94
UNICODE_Ll, //U+1D95
UNICODE_Ll, //U+1D96
UNICODE_Ll, //U+1D97
UNICODE_Ll, //U+1D98
UNICODE_Ll, //U+1D99
UNICODE_Ll, //U+1D9A
UNICODE_Lm, //U+1D9B
UNICODE_Lm, //U+1D9C
UNICODE_Lm, //U+1D9D
UNICODE_Lm, //U+1D9E
UNICODE_Lm, //U+1D9F
UNICODE_Lm, //U+1DA0
UNICODE_Lm, //U+1DA1
UNICODE_Lm, //U+1DA2
UNICODE_Lm, //U+1DA3
UNICODE_Lm, //U+1DA4
UNICODE_Lm, //U+1DA5
UNICODE_Lm, //U+1DA6
UNICODE_Lm, //U+1DA7
UNICODE_Lm, //U+1DA8
UNICODE_Lm, //U+1DA9
UNICODE_Lm, //U+1DAA
UNICODE_Lm, //U+1DAB
UNICODE_Lm, //U+1DAC
UNICODE_Lm, //U+1DAD
UNICODE_Lm, //U+1DAE
UNICODE_Lm, //U+1DAF
UNICODE_Lm, //U+1DB0
UNICODE_Lm, //U+1DB1
UNICODE_Lm, //U+1DB2
UNICODE_Lm, //U+1DB3
UNICODE_Lm, //U+1DB4
UNICODE_Lm, //U+1DB5
UNICODE_Lm, //U+1DB6
UNICODE_Lm, //U+1DB7
UNICODE_Lm, //U+1DB8
UNICODE_Lm, //U+1DB9
UNICODE_Lm, //U+1DBA
UNICODE_Lm, //U+1DBB
UNICODE_Lm, //U+1DBC
UNICODE_Lm, //U+1DBD
UNICODE_Lm, //U+1DBE
UNICODE_Lm, //U+1DBF
UNICODE_Mn, //U+1DC0
UNICODE_Mn, //U+1DC1
UNICODE_Mn, //U+1DC2
UNICODE_Mn, //U+1DC3
UNICODE_Mn, //U+1DC4
UNICODE_Mn, //U+1DC5
UNICODE_Mn, //U+1DC6
UNICODE_Mn, //U+1DC7
UNICODE_Mn, //U+1DC8
UNICODE_Mn, //U+1DC9
UNICODE_Mn, //U+1DCA
UNICODE_Mn, //U+1DCB
UNICODE_Mn, //U+1DCC
UNICODE_Mn, //U+1DCD
UNICODE_Mn, //U+1DCE
UNICODE_Mn, //U+1DCF
UNICODE_Mn, //U+1DD0
UNICODE_Mn, //U+1DD1
UNICODE_Mn, //U+1DD2
UNICODE_Mn, //U+1DD3
UNICODE_Mn, //U+1DD4
UNICODE_Mn, //U+1DD5
UNICODE_Mn, //U+1DD6
UNICODE_Mn, //U+1DD7
UNICODE_Mn, //U+1DD8
UNICODE_Mn, //U+1DD9
UNICODE_Mn, //U+1DDA
UNICODE_Mn, //U+1DDB
UNICODE_Mn, //U+1DDC
UNICODE_Mn, //U+1DDD
UNICODE_Mn, //U+1DDE
UNICODE_Mn, //U+1DDF
UNICODE_Mn, //U+1DE0
UNICODE_Mn, //U+1DE1
UNICODE_Mn, //U+1DE2
UNICODE_Mn, //U+1DE3
UNICODE_Mn, //U+1DE4
UNICODE_Mn, //U+1DE5
UNICODE_Mn, //U+1DE6
0, //U+1DE7
0, //U+1DE8
0, //U+1DE9
0, //U+1DEA
0, //U+1DEB
0, //U+1DEC
0, //U+1DED
0, //U+1DEE
0, //U+1DEF
0, //U+1DF0
0, //U+1DF1
0, //U+1DF2
0, //U+1DF3
0, //U+1DF4
0, //U+1DF5
0, //U+1DF6
0, //U+1DF7
0, //U+1DF8
0, //U+1DF9
0, //U+1DFA
0, //U+1DFB
UNICODE_Mn, //U+1DFC
UNICODE_Mn, //U+1DFD
UNICODE_Mn, //U+1DFE
UNICODE_Mn, //U+1DFF
UNICODE_Lu, //U+1E00
UNICODE_Ll, //U+1E01
UNICODE_Lu, //U+1E02
UNICODE_Ll, //U+1E03
UNICODE_Lu, //U+1E04
UNICODE_Ll, //U+1E05
UNICODE_Lu, //U+1E06
UNICODE_Ll, //U+1E07
UNICODE_Lu, //U+1E08
UNICODE_Ll, //U+1E09
UNICODE_Lu, //U+1E0A
UNICODE_Ll, //U+1E0B
UNICODE_Lu, //U+1E0C
UNICODE_Ll, //U+1E0D
UNICODE_Lu, //U+1E0E
UNICODE_Ll, //U+1E0F
UNICODE_Lu, //U+1E10
UNICODE_Ll, //U+1E11
UNICODE_Lu, //U+1E12
UNICODE_Ll, //U+1E13
UNICODE_Lu, //U+1E14
UNICODE_Ll, //U+1E15
UNICODE_Lu, //U+1E16
UNICODE_Ll, //U+1E17
UNICODE_Lu, //U+1E18
UNICODE_Ll, //U+1E19
UNICODE_Lu, //U+1E1A
UNICODE_Ll, //U+1E1B
UNICODE_Lu, //U+1E1C
UNICODE_Ll, //U+1E1D
UNICODE_Lu, //U+1E1E
UNICODE_Ll, //U+1E1F
UNICODE_Lu, //U+1E20
UNICODE_Ll, //U+1E21
UNICODE_Lu, //U+1E22
UNICODE_Ll, //U+1E23
UNICODE_Lu, //U+1E24
UNICODE_Ll, //U+1E25
UNICODE_Lu, //U+1E26
UNICODE_Ll, //U+1E27
UNICODE_Lu, //U+1E28
UNICODE_Ll, //U+1E29
UNICODE_Lu, //U+1E2A
UNICODE_Ll, //U+1E2B
UNICODE_Lu, //U+1E2C
UNICODE_Ll, //U+1E2D
UNICODE_Lu, //U+1E2E
UNICODE_Ll, //U+1E2F
UNICODE_Lu, //U+1E30
UNICODE_Ll, //U+1E31
UNICODE_Lu, //U+1E32
UNICODE_Ll, //U+1E33
UNICODE_Lu, //U+1E34
UNICODE_Ll, //U+1E35
UNICODE_Lu, //U+1E36
UNICODE_Ll, //U+1E37
UNICODE_Lu, //U+1E38
UNICODE_Ll, //U+1E39
UNICODE_Lu, //U+1E3A
UNICODE_Ll, //U+1E3B
UNICODE_Lu, //U+1E3C
UNICODE_Ll, //U+1E3D
UNICODE_Lu, //U+1E3E
UNICODE_Ll, //U+1E3F
UNICODE_Lu, //U+1E40
UNICODE_Ll, //U+1E41
UNICODE_Lu, //U+1E42
UNICODE_Ll, //U+1E43
UNICODE_Lu, //U+1E44
UNICODE_Ll, //U+1E45
UNICODE_Lu, //U+1E46
UNICODE_Ll, //U+1E47
UNICODE_Lu, //U+1E48
UNICODE_Ll, //U+1E49
UNICODE_Lu, //U+1E4A
UNICODE_Ll, //U+1E4B
UNICODE_Lu, //U+1E4C
UNICODE_Ll, //U+1E4D
UNICODE_Lu, //U+1E4E
UNICODE_Ll, //U+1E4F
UNICODE_Lu, //U+1E50
UNICODE_Ll, //U+1E51
UNICODE_Lu, //U+1E52
UNICODE_Ll, //U+1E53
UNICODE_Lu, //U+1E54
UNICODE_Ll, //U+1E55
UNICODE_Lu, //U+1E56
UNICODE_Ll, //U+1E57
UNICODE_Lu, //U+1E58
UNICODE_Ll, //U+1E59
UNICODE_Lu, //U+1E5A
UNICODE_Ll, //U+1E5B
UNICODE_Lu, //U+1E5C
UNICODE_Ll, //U+1E5D
UNICODE_Lu, //U+1E5E
UNICODE_Ll, //U+1E5F
UNICODE_Lu, //U+1E60
UNICODE_Ll, //U+1E61
UNICODE_Lu, //U+1E62
UNICODE_Ll, //U+1E63
UNICODE_Lu, //U+1E64
UNICODE_Ll, //U+1E65
UNICODE_Lu, //U+1E66
UNICODE_Ll, //U+1E67
UNICODE_Lu, //U+1E68
UNICODE_Ll, //U+1E69
UNICODE_Lu, //U+1E6A
UNICODE_Ll, //U+1E6B
UNICODE_Lu, //U+1E6C
UNICODE_Ll, //U+1E6D
UNICODE_Lu, //U+1E6E
UNICODE_Ll, //U+1E6F
UNICODE_Lu, //U+1E70
UNICODE_Ll, //U+1E71
UNICODE_Lu, //U+1E72
UNICODE_Ll, //U+1E73
UNICODE_Lu, //U+1E74
UNICODE_Ll, //U+1E75
UNICODE_Lu, //U+1E76
UNICODE_Ll, //U+1E77
UNICODE_Lu, //U+1E78
UNICODE_Ll, //U+1E79
UNICODE_Lu, //U+1E7A
UNICODE_Ll, //U+1E7B
UNICODE_Lu, //U+1E7C
UNICODE_Ll, //U+1E7D
UNICODE_Lu, //U+1E7E
UNICODE_Ll, //U+1E7F
UNICODE_Lu, //U+1E80
UNICODE_Ll, //U+1E81
UNICODE_Lu, //U+1E82
UNICODE_Ll, //U+1E83
UNICODE_Lu, //U+1E84
UNICODE_Ll, //U+1E85
UNICODE_Lu, //U+1E86
UNICODE_Ll, //U+1E87
UNICODE_Lu, //U+1E88
UNICODE_Ll, //U+1E89
UNICODE_Lu, //U+1E8A
UNICODE_Ll, //U+1E8B
UNICODE_Lu, //U+1E8C
UNICODE_Ll, //U+1E8D
UNICODE_Lu, //U+1E8E
UNICODE_Ll, //U+1E8F
UNICODE_Lu, //U+1E90
UNICODE_Ll, //U+1E91
UNICODE_Lu, //U+1E92
UNICODE_Ll, //U+1E93
UNICODE_Lu, //U+1E94
UNICODE_Ll, //U+1E95
UNICODE_Ll, //U+1E96
UNICODE_Ll, //U+1E97
UNICODE_Ll, //U+1E98
UNICODE_Ll, //U+1E99
UNICODE_Ll, //U+1E9A
UNICODE_Ll, //U+1E9B
UNICODE_Ll, //U+1E9C
UNICODE_Ll, //U+1E9D
UNICODE_Lu, //U+1E9E
UNICODE_Ll, //U+1E9F
UNICODE_Lu, //U+1EA0
UNICODE_Ll, //U+1EA1
UNICODE_Lu, //U+1EA2
UNICODE_Ll, //U+1EA3
UNICODE_Lu, //U+1EA4
UNICODE_Ll, //U+1EA5
UNICODE_Lu, //U+1EA6
UNICODE_Ll, //U+1EA7
UNICODE_Lu, //U+1EA8
UNICODE_Ll, //U+1EA9
UNICODE_Lu, //U+1EAA
UNICODE_Ll, //U+1EAB
UNICODE_Lu, //U+1EAC
UNICODE_Ll, //U+1EAD
UNICODE_Lu, //U+1EAE
UNICODE_Ll, //U+1EAF
UNICODE_Lu, //U+1EB0
UNICODE_Ll, //U+1EB1
UNICODE_Lu, //U+1EB2
UNICODE_Ll, //U+1EB3
UNICODE_Lu, //U+1EB4
UNICODE_Ll, //U+1EB5
UNICODE_Lu, //U+1EB6
UNICODE_Ll, //U+1EB7
UNICODE_Lu, //U+1EB8
UNICODE_Ll, //U+1EB9
UNICODE_Lu, //U+1EBA
UNICODE_Ll, //U+1EBB
UNICODE_Lu, //U+1EBC
UNICODE_Ll, //U+1EBD
UNICODE_Lu, //U+1EBE
UNICODE_Ll, //U+1EBF
UNICODE_Lu, //U+1EC0
UNICODE_Ll, //U+1EC1
UNICODE_Lu, //U+1EC2
UNICODE_Ll, //U+1EC3
UNICODE_Lu, //U+1EC4
UNICODE_Ll, //U+1EC5
UNICODE_Lu, //U+1EC6
UNICODE_Ll, //U+1EC7
UNICODE_Lu, //U+1EC8
UNICODE_Ll, //U+1EC9
UNICODE_Lu, //U+1ECA
UNICODE_Ll, //U+1ECB
UNICODE_Lu, //U+1ECC
UNICODE_Ll, //U+1ECD
UNICODE_Lu, //U+1ECE
UNICODE_Ll, //U+1ECF
UNICODE_Lu, //U+1ED0
UNICODE_Ll, //U+1ED1
UNICODE_Lu, //U+1ED2
UNICODE_Ll, //U+1ED3
UNICODE_Lu, //U+1ED4
UNICODE_Ll, //U+1ED5
UNICODE_Lu, //U+1ED6
UNICODE_Ll, //U+1ED7
UNICODE_Lu, //U+1ED8
UNICODE_Ll, //U+1ED9
UNICODE_Lu, //U+1EDA
UNICODE_Ll, //U+1EDB
UNICODE_Lu, //U+1EDC
UNICODE_Ll, //U+1EDD
UNICODE_Lu, //U+1EDE
UNICODE_Ll, //U+1EDF
UNICODE_Lu, //U+1EE0
UNICODE_Ll, //U+1EE1
UNICODE_Lu, //U+1EE2
UNICODE_Ll, //U+1EE3
UNICODE_Lu, //U+1EE4
UNICODE_Ll, //U+1EE5
UNICODE_Lu, //U+1EE6
UNICODE_Ll, //U+1EE7
UNICODE_Lu, //U+1EE8
UNICODE_Ll, //U+1EE9
UNICODE_Lu, //U+1EEA
UNICODE_Ll, //U+1EEB
UNICODE_Lu, //U+1EEC
UNICODE_Ll, //U+1EED
UNICODE_Lu, //U+1EEE
UNICODE_Ll, //U+1EEF
UNICODE_Lu, //U+1EF0
UNICODE_Ll, //U+1EF1
UNICODE_Lu, //U+1EF2
UNICODE_Ll, //U+1EF3
UNICODE_Lu, //U+1EF4
UNICODE_Ll, //U+1EF5
UNICODE_Lu, //U+1EF6
UNICODE_Ll, //U+1EF7
UNICODE_Lu, //U+1EF8
UNICODE_Ll, //U+1EF9
UNICODE_Lu, //U+1EFA
UNICODE_Ll, //U+1EFB
UNICODE_Lu, //U+1EFC
UNICODE_Ll, //U+1EFD
UNICODE_Lu, //U+1EFE
UNICODE_Ll, //U+1EFF
UNICODE_Ll, //U+1F00
UNICODE_Ll, //U+1F01
UNICODE_Ll, //U+1F02
UNICODE_Ll, //U+1F03
UNICODE_Ll, //U+1F04
UNICODE_Ll, //U+1F05
UNICODE_Ll, //U+1F06
UNICODE_Ll, //U+1F07
UNICODE_Lu, //U+1F08
UNICODE_Lu, //U+1F09
UNICODE_Lu, //U+1F0A
UNICODE_Lu, //U+1F0B
UNICODE_Lu, //U+1F0C
UNICODE_Lu, //U+1F0D
UNICODE_Lu, //U+1F0E
UNICODE_Lu, //U+1F0F
UNICODE_Ll, //U+1F10
UNICODE_Ll, //U+1F11
UNICODE_Ll, //U+1F12
UNICODE_Ll, //U+1F13
UNICODE_Ll, //U+1F14
UNICODE_Ll, //U+1F15
0, //U+1F16
0, //U+1F17
UNICODE_Lu, //U+1F18
UNICODE_Lu, //U+1F19
UNICODE_Lu, //U+1F1A
UNICODE_Lu, //U+1F1B
UNICODE_Lu, //U+1F1C
UNICODE_Lu, //U+1F1D
0, //U+1F1E
0, //U+1F1F
UNICODE_Ll, //U+1F20
UNICODE_Ll, //U+1F21
UNICODE_Ll, //U+1F22
UNICODE_Ll, //U+1F23
UNICODE_Ll, //U+1F24
UNICODE_Ll, //U+1F25
UNICODE_Ll, //U+1F26
UNICODE_Ll, //U+1F27
UNICODE_Lu, //U+1F28
UNICODE_Lu, //U+1F29
UNICODE_Lu, //U+1F2A
UNICODE_Lu, //U+1F2B
UNICODE_Lu, //U+1F2C
UNICODE_Lu, //U+1F2D
UNICODE_Lu, //U+1F2E
UNICODE_Lu, //U+1F2F
UNICODE_Ll, //U+1F30
UNICODE_Ll, //U+1F31
UNICODE_Ll, //U+1F32
UNICODE_Ll, //U+1F33
UNICODE_Ll, //U+1F34
UNICODE_Ll, //U+1F35
UNICODE_Ll, //U+1F36
UNICODE_Ll, //U+1F37
UNICODE_Lu, //U+1F38
UNICODE_Lu, //U+1F39
UNICODE_Lu, //U+1F3A
UNICODE_Lu, //U+1F3B
UNICODE_Lu, //U+1F3C
UNICODE_Lu, //U+1F3D
UNICODE_Lu, //U+1F3E
UNICODE_Lu, //U+1F3F
UNICODE_Ll, //U+1F40
UNICODE_Ll, //U+1F41
UNICODE_Ll, //U+1F42
UNICODE_Ll, //U+1F43
UNICODE_Ll, //U+1F44
UNICODE_Ll, //U+1F45
0, //U+1F46
0, //U+1F47
UNICODE_Lu, //U+1F48
UNICODE_Lu, //U+1F49
UNICODE_Lu, //U+1F4A
UNICODE_Lu, //U+1F4B
UNICODE_Lu, //U+1F4C
UNICODE_Lu, //U+1F4D
0, //U+1F4E
0, //U+1F4F
UNICODE_Ll, //U+1F50
UNICODE_Ll, //U+1F51
UNICODE_Ll, //U+1F52
UNICODE_Ll, //U+1F53
UNICODE_Ll, //U+1F54
UNICODE_Ll, //U+1F55
UNICODE_Ll, //U+1F56
UNICODE_Ll, //U+1F57
0, //U+1F58
UNICODE_Lu, //U+1F59
0, //U+1F5A
UNICODE_Lu, //U+1F5B
0, //U+1F5C
UNICODE_Lu, //U+1F5D
0, //U+1F5E
UNICODE_Lu, //U+1F5F
UNICODE_Ll, //U+1F60
UNICODE_Ll, //U+1F61
UNICODE_Ll, //U+1F62
UNICODE_Ll, //U+1F63
UNICODE_Ll, //U+1F64
UNICODE_Ll, //U+1F65
UNICODE_Ll, //U+1F66
UNICODE_Ll, //U+1F67
UNICODE_Lu, //U+1F68
UNICODE_Lu, //U+1F69
UNICODE_Lu, //U+1F6A
UNICODE_Lu, //U+1F6B
UNICODE_Lu, //U+1F6C
UNICODE_Lu, //U+1F6D
UNICODE_Lu, //U+1F6E
UNICODE_Lu, //U+1F6F
UNICODE_Ll, //U+1F70
UNICODE_Ll, //U+1F71
UNICODE_Ll, //U+1F72
UNICODE_Ll, //U+1F73
UNICODE_Ll, //U+1F74
UNICODE_Ll, //U+1F75
UNICODE_Ll, //U+1F76
UNICODE_Ll, //U+1F77
UNICODE_Ll, //U+1F78
UNICODE_Ll, //U+1F79
UNICODE_Ll, //U+1F7A
UNICODE_Ll, //U+1F7B
UNICODE_Ll, //U+1F7C
UNICODE_Ll, //U+1F7D
0, //U+1F7E
0, //U+1F7F
UNICODE_Ll, //U+1F80
UNICODE_Ll, //U+1F81
UNICODE_Ll, //U+1F82
UNICODE_Ll, //U+1F83
UNICODE_Ll, //U+1F84
UNICODE_Ll, //U+1F85
UNICODE_Ll, //U+1F86
UNICODE_Ll, //U+1F87
UNICODE_Lt, //U+1F88
UNICODE_Lt, //U+1F89
UNICODE_Lt, //U+1F8A
UNICODE_Lt, //U+1F8B
UNICODE_Lt, //U+1F8C
UNICODE_Lt, //U+1F8D
UNICODE_Lt, //U+1F8E
UNICODE_Lt, //U+1F8F
UNICODE_Ll, //U+1F90
UNICODE_Ll, //U+1F91
UNICODE_Ll, //U+1F92
UNICODE_Ll, //U+1F93
UNICODE_Ll, //U+1F94
UNICODE_Ll, //U+1F95
UNICODE_Ll, //U+1F96
UNICODE_Ll, //U+1F97
UNICODE_Lt, //U+1F98
UNICODE_Lt, //U+1F99
UNICODE_Lt, //U+1F9A
UNICODE_Lt, //U+1F9B
UNICODE_Lt, //U+1F9C
UNICODE_Lt, //U+1F9D
UNICODE_Lt, //U+1F9E
UNICODE_Lt, //U+1F9F
UNICODE_Ll, //U+1FA0
UNICODE_Ll, //U+1FA1
UNICODE_Ll, //U+1FA2
UNICODE_Ll, //U+1FA3
UNICODE_Ll, //U+1FA4
UNICODE_Ll, //U+1FA5
UNICODE_Ll, //U+1FA6
UNICODE_Ll, //U+1FA7
UNICODE_Lt, //U+1FA8
UNICODE_Lt, //U+1FA9
UNICODE_Lt, //U+1FAA
UNICODE_Lt, //U+1FAB
UNICODE_Lt, //U+1FAC
UNICODE_Lt, //U+1FAD
UNICODE_Lt, //U+1FAE
UNICODE_Lt, //U+1FAF
UNICODE_Ll, //U+1FB0
UNICODE_Ll, //U+1FB1
UNICODE_Ll, //U+1FB2
UNICODE_Ll, //U+1FB3
UNICODE_Ll, //U+1FB4
0, //U+1FB5
UNICODE_Ll, //U+1FB6
UNICODE_Ll, //U+1FB7
UNICODE_Lu, //U+1FB8
UNICODE_Lu, //U+1FB9
UNICODE_Lu, //U+1FBA
UNICODE_Lu, //U+1FBB
UNICODE_Lt, //U+1FBC
UNICODE_Sk, //U+1FBD
UNICODE_Ll, //U+1FBE
UNICODE_Sk, //U+1FBF
UNICODE_Sk, //U+1FC0
UNICODE_Sk, //U+1FC1
UNICODE_Ll, //U+1FC2
UNICODE_Ll, //U+1FC3
UNICODE_Ll, //U+1FC4
0, //U+1FC5
UNICODE_Ll, //U+1FC6
UNICODE_Ll, //U+1FC7
UNICODE_Lu, //U+1FC8
UNICODE_Lu, //U+1FC9
UNICODE_Lu, //U+1FCA
UNICODE_Lu, //U+1FCB
UNICODE_Lt, //U+1FCC
UNICODE_Sk, //U+1FCD
UNICODE_Sk, //U+1FCE
UNICODE_Sk, //U+1FCF
UNICODE_Ll, //U+1FD0
UNICODE_Ll, //U+1FD1
UNICODE_Ll, //U+1FD2
UNICODE_Ll, //U+1FD3
0, //U+1FD4
0, //U+1FD5
UNICODE_Ll, //U+1FD6
UNICODE_Ll, //U+1FD7
UNICODE_Lu, //U+1FD8
UNICODE_Lu, //U+1FD9
UNICODE_Lu, //U+1FDA
UNICODE_Lu, //U+1FDB
0, //U+1FDC
UNICODE_Sk, //U+1FDD
UNICODE_Sk, //U+1FDE
UNICODE_Sk, //U+1FDF
UNICODE_Ll, //U+1FE0
UNICODE_Ll, //U+1FE1
UNICODE_Ll, //U+1FE2
UNICODE_Ll, //U+1FE3
UNICODE_Ll, //U+1FE4
UNICODE_Ll, //U+1FE5
UNICODE_Ll, //U+1FE6
UNICODE_Ll, //U+1FE7
UNICODE_Lu, //U+1FE8
UNICODE_Lu, //U+1FE9
UNICODE_Lu, //U+1FEA
UNICODE_Lu, //U+1FEB
UNICODE_Lu, //U+1FEC
UNICODE_Sk, //U+1FED
UNICODE_Sk, //U+1FEE
UNICODE_Sk, //U+1FEF
0, //U+1FF0
0, //U+1FF1
UNICODE_Ll, //U+1FF2
UNICODE_Ll, //U+1FF3
UNICODE_Ll, //U+1FF4
0, //U+1FF5
UNICODE_Ll, //U+1FF6
UNICODE_Ll, //U+1FF7
UNICODE_Lu, //U+1FF8
UNICODE_Lu, //U+1FF9
UNICODE_Lu, //U+1FFA
UNICODE_Lu, //U+1FFB
UNICODE_Lt, //U+1FFC
UNICODE_Sk, //U+1FFD
UNICODE_Sk, //U+1FFE
0, //U+1FFF
UNICODE_Zs, //U+2000
UNICODE_Zs, //U+2001
UNICODE_Zs, //U+2002
UNICODE_Zs, //U+2003
UNICODE_Zs, //U+2004
UNICODE_Zs, //U+2005
UNICODE_Zs, //U+2006
UNICODE_Zs, //U+2007
UNICODE_Zs, //U+2008
UNICODE_Zs, //U+2009
UNICODE_Zs, //U+200A
UNICODE_Cf, //U+200B
UNICODE_Cf, //U+200C
UNICODE_Cf, //U+200D
UNICODE_Cf, //U+200E
UNICODE_Cf, //U+200F
UNICODE_Pd, //U+2010
UNICODE_Pd, //U+2011
UNICODE_Pd, //U+2012
UNICODE_Pd, //U+2013
UNICODE_Pd, //U+2014
UNICODE_Pd, //U+2015
UNICODE_Po, //U+2016
UNICODE_Po, //U+2017
UNICODE_Pi, //U+2018
UNICODE_Pf, //U+2019
UNICODE_Ps, //U+201A
UNICODE_Pi, //U+201B
UNICODE_Pi, //U+201C
UNICODE_Pf, //U+201D
UNICODE_Ps, //U+201E
UNICODE_Pi, //U+201F
UNICODE_Po, //U+2020
UNICODE_Po, //U+2021
UNICODE_Po, //U+2022
UNICODE_Po, //U+2023
UNICODE_Po, //U+2024
UNICODE_Po, //U+2025
UNICODE_Po, //U+2026
UNICODE_Po, //U+2027
UNICODE_Zl, //U+2028
UNICODE_Zp, //U+2029
UNICODE_Cf, //U+202A
UNICODE_Cf, //U+202B
UNICODE_Cf, //U+202C
UNICODE_Cf, //U+202D
UNICODE_Cf, //U+202E
UNICODE_Zs, //U+202F
UNICODE_Po, //U+2030
UNICODE_Po, //U+2031
UNICODE_Po, //U+2032
UNICODE_Po, //U+2033
UNICODE_Po, //U+2034
UNICODE_Po, //U+2035
UNICODE_Po, //U+2036
UNICODE_Po, //U+2037
UNICODE_Po, //U+2038
UNICODE_Pi, //U+2039
UNICODE_Pf, //U+203A
UNICODE_Po, //U+203B
UNICODE_Po, //U+203C
UNICODE_Po, //U+203D
UNICODE_Po, //U+203E
UNICODE_Pc, //U+203F
UNICODE_Pc, //U+2040
UNICODE_Po, //U+2041
UNICODE_Po, //U+2042
UNICODE_Po, //U+2043
UNICODE_Sm, //U+2044
UNICODE_Ps, //U+2045
UNICODE_Pe, //U+2046
UNICODE_Po, //U+2047
UNICODE_Po, //U+2048
UNICODE_Po, //U+2049
UNICODE_Po, //U+204A
UNICODE_Po, //U+204B
UNICODE_Po, //U+204C
UNICODE_Po, //U+204D
UNICODE_Po, //U+204E
UNICODE_Po, //U+204F
UNICODE_Po, //U+2050
UNICODE_Po, //U+2051
UNICODE_Sm, //U+2052
UNICODE_Po, //U+2053
UNICODE_Pc, //U+2054
UNICODE_Po, //U+2055
UNICODE_Po, //U+2056
UNICODE_Po, //U+2057
UNICODE_Po, //U+2058
UNICODE_Po, //U+2059
UNICODE_Po, //U+205A
UNICODE_Po, //U+205B
UNICODE_Po, //U+205C
UNICODE_Po, //U+205D
UNICODE_Po, //U+205E
UNICODE_Zs, //U+205F
UNICODE_Cf, //U+2060
UNICODE_Cf, //U+2061
UNICODE_Cf, //U+2062
UNICODE_Cf, //U+2063
UNICODE_Cf, //U+2064
0, //U+2065
0, //U+2066
0, //U+2067
0, //U+2068
0, //U+2069
UNICODE_Cf, //U+206A
UNICODE_Cf, //U+206B
UNICODE_Cf, //U+206C
UNICODE_Cf, //U+206D
UNICODE_Cf, //U+206E
UNICODE_Cf, //U+206F
UNICODE_No, //U+2070
UNICODE_Lm, //U+2071
0, //U+2072
0, //U+2073
UNICODE_No, //U+2074
UNICODE_No, //U+2075
UNICODE_No, //U+2076
UNICODE_No, //U+2077
UNICODE_No, //U+2078
UNICODE_No, //U+2079
UNICODE_Sm, //U+207A
UNICODE_Sm, //U+207B
UNICODE_Sm, //U+207C
UNICODE_Ps, //U+207D
UNICODE_Pe, //U+207E
UNICODE_Lm, //U+207F
UNICODE_No, //U+2080
UNICODE_No, //U+2081
UNICODE_No, //U+2082
UNICODE_No, //U+2083
UNICODE_No, //U+2084
UNICODE_No, //U+2085
UNICODE_No, //U+2086
UNICODE_No, //U+2087
UNICODE_No, //U+2088
UNICODE_No, //U+2089
UNICODE_Sm, //U+208A
UNICODE_Sm, //U+208B
UNICODE_Sm, //U+208C
UNICODE_Ps, //U+208D
UNICODE_Pe, //U+208E
0, //U+208F
UNICODE_Lm, //U+2090
UNICODE_Lm, //U+2091
UNICODE_Lm, //U+2092
UNICODE_Lm, //U+2093
UNICODE_Lm, //U+2094
UNICODE_Lm, //U+2095
UNICODE_Lm, //U+2096
UNICODE_Lm, //U+2097
UNICODE_Lm, //U+2098
UNICODE_Lm, //U+2099
UNICODE_Lm, //U+209A
UNICODE_Lm, //U+209B
UNICODE_Lm, //U+209C
0, //U+209D
0, //U+209E
0, //U+209F
UNICODE_Sc, //U+20A0
UNICODE_Sc, //U+20A1
UNICODE_Sc, //U+20A2
UNICODE_Sc, //U+20A3
UNICODE_Sc, //U+20A4
UNICODE_Sc, //U+20A5
UNICODE_Sc, //U+20A6
UNICODE_Sc, //U+20A7
UNICODE_Sc, //U+20A8
UNICODE_Sc, //U+20A9
UNICODE_Sc, //U+20AA
UNICODE_Sc, //U+20AB
UNICODE_Sc, //U+20AC
UNICODE_Sc, //U+20AD
UNICODE_Sc, //U+20AE
UNICODE_Sc, //U+20AF
UNICODE_Sc, //U+20B0
UNICODE_Sc, //U+20B1
UNICODE_Sc, //U+20B2
UNICODE_Sc, //U+20B3
UNICODE_Sc, //U+20B4
UNICODE_Sc, //U+20B5
UNICODE_Sc, //U+20B6
UNICODE_Sc, //U+20B7
UNICODE_Sc, //U+20B8
UNICODE_Sc, //U+20B9
0, //U+20BA
0, //U+20BB
0, //U+20BC
0, //U+20BD
0, //U+20BE
0, //U+20BF
0, //U+20C0
0, //U+20C1
0, //U+20C2
0, //U+20C3
0, //U+20C4
0, //U+20C5
0, //U+20C6
0, //U+20C7
0, //U+20C8
0, //U+20C9
0, //U+20CA
0, //U+20CB
0, //U+20CC
0, //U+20CD
0, //U+20CE
0, //U+20CF
UNICODE_Mn, //U+20D0
UNICODE_Mn, //U+20D1
UNICODE_Mn, //U+20D2
UNICODE_Mn, //U+20D3
UNICODE_Mn, //U+20D4
UNICODE_Mn, //U+20D5
UNICODE_Mn, //U+20D6
UNICODE_Mn, //U+20D7
UNICODE_Mn, //U+20D8
UNICODE_Mn, //U+20D9
UNICODE_Mn, //U+20DA
UNICODE_Mn, //U+20DB
UNICODE_Mn, //U+20DC
UNICODE_Me, //U+20DD
UNICODE_Me, //U+20DE
UNICODE_Me, //U+20DF
UNICODE_Me, //U+20E0
UNICODE_Mn, //U+20E1
UNICODE_Me, //U+20E2
UNICODE_Me, //U+20E3
UNICODE_Me, //U+20E4
UNICODE_Mn, //U+20E5
UNICODE_Mn, //U+20E6
UNICODE_Mn, //U+20E7
UNICODE_Mn, //U+20E8
UNICODE_Mn, //U+20E9
UNICODE_Mn, //U+20EA
UNICODE_Mn, //U+20EB
UNICODE_Mn, //U+20EC
UNICODE_Mn, //U+20ED
UNICODE_Mn, //U+20EE
UNICODE_Mn, //U+20EF
UNICODE_Mn, //U+20F0
0, //U+20F1
0, //U+20F2
0, //U+20F3
0, //U+20F4
0, //U+20F5
0, //U+20F6
0, //U+20F7
0, //U+20F8
0, //U+20F9
0, //U+20FA
0, //U+20FB
0, //U+20FC
0, //U+20FD
0, //U+20FE
0, //U+20FF
UNICODE_So, //U+2100
UNICODE_So, //U+2101
UNICODE_Lu, //U+2102
UNICODE_So, //U+2103
UNICODE_So, //U+2104
UNICODE_So, //U+2105
UNICODE_So, //U+2106
UNICODE_Lu, //U+2107
UNICODE_So, //U+2108
UNICODE_So, //U+2109
UNICODE_Ll, //U+210A
UNICODE_Lu, //U+210B
UNICODE_Lu, //U+210C
UNICODE_Lu, //U+210D
UNICODE_Ll, //U+210E
UNICODE_Ll, //U+210F
UNICODE_Lu, //U+2110
UNICODE_Lu, //U+2111
UNICODE_Lu, //U+2112
UNICODE_Ll, //U+2113
UNICODE_So, //U+2114
UNICODE_Lu, //U+2115
UNICODE_So, //U+2116
UNICODE_So, //U+2117
UNICODE_Sm, //U+2118
UNICODE_Lu, //U+2119
UNICODE_Lu, //U+211A
UNICODE_Lu, //U+211B
UNICODE_Lu, //U+211C
UNICODE_Lu, //U+211D
UNICODE_So, //U+211E
UNICODE_So, //U+211F
UNICODE_So, //U+2120
UNICODE_So, //U+2121
UNICODE_So, //U+2122
UNICODE_So, //U+2123
UNICODE_Lu, //U+2124
UNICODE_So, //U+2125
UNICODE_Lu, //U+2126
UNICODE_So, //U+2127
UNICODE_Lu, //U+2128
UNICODE_So, //U+2129
UNICODE_Lu, //U+212A
UNICODE_Lu, //U+212B
UNICODE_Lu, //U+212C
UNICODE_Lu, //U+212D
UNICODE_So, //U+212E
UNICODE_Ll, //U+212F
UNICODE_Lu, //U+2130
UNICODE_Lu, //U+2131
UNICODE_Lu, //U+2132
UNICODE_Lu, //U+2133
UNICODE_Ll, //U+2134
UNICODE_Lo, //U+2135
UNICODE_Lo, //U+2136
UNICODE_Lo, //U+2137
UNICODE_Lo, //U+2138
UNICODE_Ll, //U+2139
UNICODE_So, //U+213A
UNICODE_So, //U+213B
UNICODE_Ll, //U+213C
UNICODE_Ll, //U+213D
UNICODE_Lu, //U+213E
UNICODE_Lu, //U+213F
UNICODE_Sm, //U+2140
UNICODE_Sm, //U+2141
UNICODE_Sm, //U+2142
UNICODE_Sm, //U+2143
UNICODE_Sm, //U+2144
UNICODE_Lu, //U+2145
UNICODE_Ll, //U+2146
UNICODE_Ll, //U+2147
UNICODE_Ll, //U+2148
UNICODE_Ll, //U+2149
UNICODE_So, //U+214A
UNICODE_Sm, //U+214B
UNICODE_So, //U+214C
UNICODE_So, //U+214D
UNICODE_Ll, //U+214E
UNICODE_So, //U+214F
UNICODE_No, //U+2150
UNICODE_No, //U+2151
UNICODE_No, //U+2152
UNICODE_No, //U+2153
UNICODE_No, //U+2154
UNICODE_No, //U+2155
UNICODE_No, //U+2156
UNICODE_No, //U+2157
UNICODE_No, //U+2158
UNICODE_No, //U+2159
UNICODE_No, //U+215A
UNICODE_No, //U+215B
UNICODE_No, //U+215C
UNICODE_No, //U+215D
UNICODE_No, //U+215E
UNICODE_No, //U+215F
UNICODE_Nl, //U+2160
UNICODE_Nl, //U+2161
UNICODE_Nl, //U+2162
UNICODE_Nl, //U+2163
UNICODE_Nl, //U+2164
UNICODE_Nl, //U+2165
UNICODE_Nl, //U+2166
UNICODE_Nl, //U+2167
UNICODE_Nl, //U+2168
UNICODE_Nl, //U+2169
UNICODE_Nl, //U+216A
UNICODE_Nl, //U+216B
UNICODE_Nl, //U+216C
UNICODE_Nl, //U+216D
UNICODE_Nl, //U+216E
UNICODE_Nl, //U+216F
UNICODE_Nl, //U+2170
UNICODE_Nl, //U+2171
UNICODE_Nl, //U+2172
UNICODE_Nl, //U+2173
UNICODE_Nl, //U+2174
UNICODE_Nl, //U+2175
UNICODE_Nl, //U+2176
UNICODE_Nl, //U+2177
UNICODE_Nl, //U+2178
UNICODE_Nl, //U+2179
UNICODE_Nl, //U+217A
UNICODE_Nl, //U+217B
UNICODE_Nl, //U+217C
UNICODE_Nl, //U+217D
UNICODE_Nl, //U+217E
UNICODE_Nl, //U+217F
UNICODE_Nl, //U+2180
UNICODE_Nl, //U+2181
UNICODE_Nl, //U+2182
UNICODE_Lu, //U+2183
UNICODE_Ll, //U+2184
UNICODE_Nl, //U+2185
UNICODE_Nl, //U+2186
UNICODE_Nl, //U+2187
UNICODE_Nl, //U+2188
UNICODE_No, //U+2189
0, //U+218A
0, //U+218B
0, //U+218C
0, //U+218D
0, //U+218E
0, //U+218F
UNICODE_Sm, //U+2190
UNICODE_Sm, //U+2191
UNICODE_Sm, //U+2192
UNICODE_Sm, //U+2193
UNICODE_Sm, //U+2194
UNICODE_So, //U+2195
UNICODE_So, //U+2196
UNICODE_So, //U+2197
UNICODE_So, //U+2198
UNICODE_So, //U+2199
UNICODE_Sm, //U+219A
UNICODE_Sm, //U+219B
UNICODE_So, //U+219C
UNICODE_So, //U+219D
UNICODE_So, //U+219E
UNICODE_So, //U+219F
UNICODE_Sm, //U+21A0
UNICODE_So, //U+21A1
UNICODE_So, //U+21A2
UNICODE_Sm, //U+21A3
UNICODE_So, //U+21A4
UNICODE_So, //U+21A5
UNICODE_Sm, //U+21A6
UNICODE_So, //U+21A7
UNICODE_So, //U+21A8
UNICODE_So, //U+21A9
UNICODE_So, //U+21AA
UNICODE_So, //U+21AB
UNICODE_So, //U+21AC
UNICODE_So, //U+21AD
UNICODE_Sm, //U+21AE
UNICODE_So, //U+21AF
UNICODE_So, //U+21B0
UNICODE_So, //U+21B1
UNICODE_So, //U+21B2
UNICODE_So, //U+21B3
UNICODE_So, //U+21B4
UNICODE_So, //U+21B5
UNICODE_So, //U+21B6
UNICODE_So, //U+21B7
UNICODE_So, //U+21B8
UNICODE_So, //U+21B9
UNICODE_So, //U+21BA
UNICODE_So, //U+21BB
UNICODE_So, //U+21BC
UNICODE_So, //U+21BD
UNICODE_So, //U+21BE
UNICODE_So, //U+21BF
UNICODE_So, //U+21C0
UNICODE_So, //U+21C1
UNICODE_So, //U+21C2
UNICODE_So, //U+21C3
UNICODE_So, //U+21C4
UNICODE_So, //U+21C5
UNICODE_So, //U+21C6
UNICODE_So, //U+21C7
UNICODE_So, //U+21C8
UNICODE_So, //U+21C9
UNICODE_So, //U+21CA
UNICODE_So, //U+21CB
UNICODE_So, //U+21CC
UNICODE_So, //U+21CD
UNICODE_Sm, //U+21CE
UNICODE_Sm, //U+21CF
UNICODE_So, //U+21D0
UNICODE_So, //U+21D1
UNICODE_Sm, //U+21D2
UNICODE_So, //U+21D3
UNICODE_Sm, //U+21D4
UNICODE_So, //U+21D5
UNICODE_So, //U+21D6
UNICODE_So, //U+21D7
UNICODE_So, //U+21D8
UNICODE_So, //U+21D9
UNICODE_So, //U+21DA
UNICODE_So, //U+21DB
UNICODE_So, //U+21DC
UNICODE_So, //U+21DD
UNICODE_So, //U+21DE
UNICODE_So, //U+21DF
UNICODE_So, //U+21E0
UNICODE_So, //U+21E1
UNICODE_So, //U+21E2
UNICODE_So, //U+21E3
UNICODE_So, //U+21E4
UNICODE_So, //U+21E5
UNICODE_So, //U+21E6
UNICODE_So, //U+21E7
UNICODE_So, //U+21E8
UNICODE_So, //U+21E9
UNICODE_So, //U+21EA
UNICODE_So, //U+21EB
UNICODE_So, //U+21EC
UNICODE_So, //U+21ED
UNICODE_So, //U+21EE
UNICODE_So, //U+21EF
UNICODE_So, //U+21F0
UNICODE_So, //U+21F1
UNICODE_So, //U+21F2
UNICODE_So, //U+21F3
UNICODE_Sm, //U+21F4
UNICODE_Sm, //U+21F5
UNICODE_Sm, //U+21F6
UNICODE_Sm, //U+21F7
UNICODE_Sm, //U+21F8
UNICODE_Sm, //U+21F9
UNICODE_Sm, //U+21FA
UNICODE_Sm, //U+21FB
UNICODE_Sm, //U+21FC
UNICODE_Sm, //U+21FD
UNICODE_Sm, //U+21FE
UNICODE_Sm, //U+21FF
UNICODE_Sm, //U+2200
UNICODE_Sm, //U+2201
UNICODE_Sm, //U+2202
UNICODE_Sm, //U+2203
UNICODE_Sm, //U+2204
UNICODE_Sm, //U+2205
UNICODE_Sm, //U+2206
UNICODE_Sm, //U+2207
UNICODE_Sm, //U+2208
UNICODE_Sm, //U+2209
UNICODE_Sm, //U+220A
UNICODE_Sm, //U+220B
UNICODE_Sm, //U+220C
UNICODE_Sm, //U+220D
UNICODE_Sm, //U+220E
UNICODE_Sm, //U+220F
UNICODE_Sm, //U+2210
UNICODE_Sm, //U+2211
UNICODE_Sm, //U+2212
UNICODE_Sm, //U+2213
UNICODE_Sm, //U+2214
UNICODE_Sm, //U+2215
UNICODE_Sm, //U+2216
UNICODE_Sm, //U+2217
UNICODE_Sm, //U+2218
UNICODE_Sm, //U+2219
UNICODE_Sm, //U+221A
UNICODE_Sm, //U+221B
UNICODE_Sm, //U+221C
UNICODE_Sm, //U+221D
UNICODE_Sm, //U+221E
UNICODE_Sm, //U+221F
UNICODE_Sm, //U+2220
UNICODE_Sm, //U+2221
UNICODE_Sm, //U+2222
UNICODE_Sm, //U+2223
UNICODE_Sm, //U+2224
UNICODE_Sm, //U+2225
UNICODE_Sm, //U+2226
UNICODE_Sm, //U+2227
UNICODE_Sm, //U+2228
UNICODE_Sm, //U+2229
UNICODE_Sm, //U+222A
UNICODE_Sm, //U+222B
UNICODE_Sm, //U+222C
UNICODE_Sm, //U+222D
UNICODE_Sm, //U+222E
UNICODE_Sm, //U+222F
UNICODE_Sm, //U+2230
UNICODE_Sm, //U+2231
UNICODE_Sm, //U+2232
UNICODE_Sm, //U+2233
UNICODE_Sm, //U+2234
UNICODE_Sm, //U+2235
UNICODE_Sm, //U+2236
UNICODE_Sm, //U+2237
UNICODE_Sm, //U+2238
UNICODE_Sm, //U+2239
UNICODE_Sm, //U+223A
UNICODE_Sm, //U+223B
UNICODE_Sm, //U+223C
UNICODE_Sm, //U+223D
UNICODE_Sm, //U+223E
UNICODE_Sm, //U+223F
UNICODE_Sm, //U+2240
UNICODE_Sm, //U+2241
UNICODE_Sm, //U+2242
UNICODE_Sm, //U+2243
UNICODE_Sm, //U+2244
UNICODE_Sm, //U+2245
UNICODE_Sm, //U+2246
UNICODE_Sm, //U+2247
UNICODE_Sm, //U+2248
UNICODE_Sm, //U+2249
UNICODE_Sm, //U+224A
UNICODE_Sm, //U+224B
UNICODE_Sm, //U+224C
UNICODE_Sm, //U+224D
UNICODE_Sm, //U+224E
UNICODE_Sm, //U+224F
UNICODE_Sm, //U+2250
UNICODE_Sm, //U+2251
UNICODE_Sm, //U+2252
UNICODE_Sm, //U+2253
UNICODE_Sm, //U+2254
UNICODE_Sm, //U+2255
UNICODE_Sm, //U+2256
UNICODE_Sm, //U+2257
UNICODE_Sm, //U+2258
UNICODE_Sm, //U+2259
UNICODE_Sm, //U+225A
UNICODE_Sm, //U+225B
UNICODE_Sm, //U+225C
UNICODE_Sm, //U+225D
UNICODE_Sm, //U+225E
UNICODE_Sm, //U+225F
UNICODE_Sm, //U+2260
UNICODE_Sm, //U+2261
UNICODE_Sm, //U+2262
UNICODE_Sm, //U+2263
UNICODE_Sm, //U+2264
UNICODE_Sm, //U+2265
UNICODE_Sm, //U+2266
UNICODE_Sm, //U+2267
UNICODE_Sm, //U+2268
UNICODE_Sm, //U+2269
UNICODE_Sm, //U+226A
UNICODE_Sm, //U+226B
UNICODE_Sm, //U+226C
UNICODE_Sm, //U+226D
UNICODE_Sm, //U+226E
UNICODE_Sm, //U+226F
UNICODE_Sm, //U+2270
UNICODE_Sm, //U+2271
UNICODE_Sm, //U+2272
UNICODE_Sm, //U+2273
UNICODE_Sm, //U+2274
UNICODE_Sm, //U+2275
UNICODE_Sm, //U+2276
UNICODE_Sm, //U+2277
UNICODE_Sm, //U+2278
UNICODE_Sm, //U+2279
UNICODE_Sm, //U+227A
UNICODE_Sm, //U+227B
UNICODE_Sm, //U+227C
UNICODE_Sm, //U+227D
UNICODE_Sm, //U+227E
UNICODE_Sm, //U+227F
UNICODE_Sm, //U+2280
UNICODE_Sm, //U+2281
UNICODE_Sm, //U+2282
UNICODE_Sm, //U+2283
UNICODE_Sm, //U+2284
UNICODE_Sm, //U+2285
UNICODE_Sm, //U+2286
UNICODE_Sm, //U+2287
UNICODE_Sm, //U+2288
UNICODE_Sm, //U+2289
UNICODE_Sm, //U+228A
UNICODE_Sm, //U+228B
UNICODE_Sm, //U+228C
UNICODE_Sm, //U+228D
UNICODE_Sm, //U+228E
UNICODE_Sm, //U+228F
UNICODE_Sm, //U+2290
UNICODE_Sm, //U+2291
UNICODE_Sm, //U+2292
UNICODE_Sm, //U+2293
UNICODE_Sm, //U+2294
UNICODE_Sm, //U+2295
UNICODE_Sm, //U+2296
UNICODE_Sm, //U+2297
UNICODE_Sm, //U+2298
UNICODE_Sm, //U+2299
UNICODE_Sm, //U+229A
UNICODE_Sm, //U+229B
UNICODE_Sm, //U+229C
UNICODE_Sm, //U+229D
UNICODE_Sm, //U+229E
UNICODE_Sm, //U+229F
UNICODE_Sm, //U+22A0
UNICODE_Sm, //U+22A1
UNICODE_Sm, //U+22A2
UNICODE_Sm, //U+22A3
UNICODE_Sm, //U+22A4
UNICODE_Sm, //U+22A5
UNICODE_Sm, //U+22A6
UNICODE_Sm, //U+22A7
UNICODE_Sm, //U+22A8
UNICODE_Sm, //U+22A9
UNICODE_Sm, //U+22AA
UNICODE_Sm, //U+22AB
UNICODE_Sm, //U+22AC
UNICODE_Sm, //U+22AD
UNICODE_Sm, //U+22AE
UNICODE_Sm, //U+22AF
UNICODE_Sm, //U+22B0
UNICODE_Sm, //U+22B1
UNICODE_Sm, //U+22B2
UNICODE_Sm, //U+22B3
UNICODE_Sm, //U+22B4
UNICODE_Sm, //U+22B5
UNICODE_Sm, //U+22B6
UNICODE_Sm, //U+22B7
UNICODE_Sm, //U+22B8
UNICODE_Sm, //U+22B9
UNICODE_Sm, //U+22BA
UNICODE_Sm, //U+22BB
UNICODE_Sm, //U+22BC
UNICODE_Sm, //U+22BD
UNICODE_Sm, //U+22BE
UNICODE_Sm, //U+22BF
UNICODE_Sm, //U+22C0
UNICODE_Sm, //U+22C1
UNICODE_Sm, //U+22C2
UNICODE_Sm, //U+22C3
UNICODE_Sm, //U+22C4
UNICODE_Sm, //U+22C5
UNICODE_Sm, //U+22C6
UNICODE_Sm, //U+22C7
UNICODE_Sm, //U+22C8
UNICODE_Sm, //U+22C9
UNICODE_Sm, //U+22CA
UNICODE_Sm, //U+22CB
UNICODE_Sm, //U+22CC
UNICODE_Sm, //U+22CD
UNICODE_Sm, //U+22CE
UNICODE_Sm, //U+22CF
UNICODE_Sm, //U+22D0
UNICODE_Sm, //U+22D1
UNICODE_Sm, //U+22D2
UNICODE_Sm, //U+22D3
UNICODE_Sm, //U+22D4
UNICODE_Sm, //U+22D5
UNICODE_Sm, //U+22D6
UNICODE_Sm, //U+22D7
UNICODE_Sm, //U+22D8
UNICODE_Sm, //U+22D9
UNICODE_Sm, //U+22DA
UNICODE_Sm, //U+22DB
UNICODE_Sm, //U+22DC
UNICODE_Sm, //U+22DD
UNICODE_Sm, //U+22DE
UNICODE_Sm, //U+22DF
UNICODE_Sm, //U+22E0
UNICODE_Sm, //U+22E1
UNICODE_Sm, //U+22E2
UNICODE_Sm, //U+22E3
UNICODE_Sm, //U+22E4
UNICODE_Sm, //U+22E5
UNICODE_Sm, //U+22E6
UNICODE_Sm, //U+22E7
UNICODE_Sm, //U+22E8
UNICODE_Sm, //U+22E9
UNICODE_Sm, //U+22EA
UNICODE_Sm, //U+22EB
UNICODE_Sm, //U+22EC
UNICODE_Sm, //U+22ED
UNICODE_Sm, //U+22EE
UNICODE_Sm, //U+22EF
UNICODE_Sm, //U+22F0
UNICODE_Sm, //U+22F1
UNICODE_Sm, //U+22F2
UNICODE_Sm, //U+22F3
UNICODE_Sm, //U+22F4
UNICODE_Sm, //U+22F5
UNICODE_Sm, //U+22F6
UNICODE_Sm, //U+22F7
UNICODE_Sm, //U+22F8
UNICODE_Sm, //U+22F9
UNICODE_Sm, //U+22FA
UNICODE_Sm, //U+22FB
UNICODE_Sm, //U+22FC
UNICODE_Sm, //U+22FD
UNICODE_Sm, //U+22FE
UNICODE_Sm, //U+22FF
UNICODE_So, //U+2300
UNICODE_So, //U+2301
UNICODE_So, //U+2302
UNICODE_So, //U+2303
UNICODE_So, //U+2304
UNICODE_So, //U+2305
UNICODE_So, //U+2306
UNICODE_So, //U+2307
UNICODE_Sm, //U+2308
UNICODE_Sm, //U+2309
UNICODE_Sm, //U+230A
UNICODE_Sm, //U+230B
UNICODE_So, //U+230C
UNICODE_So, //U+230D
UNICODE_So, //U+230E
UNICODE_So, //U+230F
UNICODE_So, //U+2310
UNICODE_So, //U+2311
UNICODE_So, //U+2312
UNICODE_So, //U+2313
UNICODE_So, //U+2314
UNICODE_So, //U+2315
UNICODE_So, //U+2316
UNICODE_So, //U+2317
UNICODE_So, //U+2318
UNICODE_So, //U+2319
UNICODE_So, //U+231A
UNICODE_So, //U+231B
UNICODE_So, //U+231C
UNICODE_So, //U+231D
UNICODE_So, //U+231E
UNICODE_So, //U+231F
UNICODE_Sm, //U+2320
UNICODE_Sm, //U+2321
UNICODE_So, //U+2322
UNICODE_So, //U+2323
UNICODE_So, //U+2324
UNICODE_So, //U+2325
UNICODE_So, //U+2326
UNICODE_So, //U+2327
UNICODE_So, //U+2328
UNICODE_Ps, //U+2329
UNICODE_Pe, //U+232A
UNICODE_So, //U+232B
UNICODE_So, //U+232C
UNICODE_So, //U+232D
UNICODE_So, //U+232E
UNICODE_So, //U+232F
UNICODE_So, //U+2330
UNICODE_So, //U+2331
UNICODE_So, //U+2332
UNICODE_So, //U+2333
UNICODE_So, //U+2334
UNICODE_So, //U+2335
UNICODE_So, //U+2336
UNICODE_So, //U+2337
UNICODE_So, //U+2338
UNICODE_So, //U+2339
UNICODE_So, //U+233A
UNICODE_So, //U+233B
UNICODE_So, //U+233C
UNICODE_So, //U+233D
UNICODE_So, //U+233E
UNICODE_So, //U+233F
UNICODE_So, //U+2340
UNICODE_So, //U+2341
UNICODE_So, //U+2342
UNICODE_So, //U+2343
UNICODE_So, //U+2344
UNICODE_So, //U+2345
UNICODE_So, //U+2346
UNICODE_So, //U+2347
UNICODE_So, //U+2348
UNICODE_So, //U+2349
UNICODE_So, //U+234A
UNICODE_So, //U+234B
UNICODE_So, //U+234C
UNICODE_So, //U+234D
UNICODE_So, //U+234E
UNICODE_So, //U+234F
UNICODE_So, //U+2350
UNICODE_So, //U+2351
UNICODE_So, //U+2352
UNICODE_So, //U+2353
UNICODE_So, //U+2354
UNICODE_So, //U+2355
UNICODE_So, //U+2356
UNICODE_So, //U+2357
UNICODE_So, //U+2358
UNICODE_So, //U+2359
UNICODE_So, //U+235A
UNICODE_So, //U+235B
UNICODE_So, //U+235C
UNICODE_So, //U+235D
UNICODE_So, //U+235E
UNICODE_So, //U+235F
UNICODE_So, //U+2360
UNICODE_So, //U+2361
UNICODE_So, //U+2362
UNICODE_So, //U+2363
UNICODE_So, //U+2364
UNICODE_So, //U+2365
UNICODE_So, //U+2366
UNICODE_So, //U+2367
UNICODE_So, //U+2368
UNICODE_So, //U+2369
UNICODE_So, //U+236A
UNICODE_So, //U+236B
UNICODE_So, //U+236C
UNICODE_So, //U+236D
UNICODE_So, //U+236E
UNICODE_So, //U+236F
UNICODE_So, //U+2370
UNICODE_So, //U+2371
UNICODE_So, //U+2372
UNICODE_So, //U+2373
UNICODE_So, //U+2374
UNICODE_So, //U+2375
UNICODE_So, //U+2376
UNICODE_So, //U+2377
UNICODE_So, //U+2378
UNICODE_So, //U+2379
UNICODE_So, //U+237A
UNICODE_So, //U+237B
UNICODE_Sm, //U+237C
UNICODE_So, //U+237D
UNICODE_So, //U+237E
UNICODE_So, //U+237F
UNICODE_So, //U+2380
UNICODE_So, //U+2381
UNICODE_So, //U+2382
UNICODE_So, //U+2383
UNICODE_So, //U+2384
UNICODE_So, //U+2385
UNICODE_So, //U+2386
UNICODE_So, //U+2387
UNICODE_So, //U+2388
UNICODE_So, //U+2389
UNICODE_So, //U+238A
UNICODE_So, //U+238B
UNICODE_So, //U+238C
UNICODE_So, //U+238D
UNICODE_So, //U+238E
UNICODE_So, //U+238F
UNICODE_So, //U+2390
UNICODE_So, //U+2391
UNICODE_So, //U+2392
UNICODE_So, //U+2393
UNICODE_So, //U+2394
UNICODE_So, //U+2395
UNICODE_So, //U+2396
UNICODE_So, //U+2397
UNICODE_So, //U+2398
UNICODE_So, //U+2399
UNICODE_So, //U+239A
UNICODE_Sm, //U+239B
UNICODE_Sm, //U+239C
UNICODE_Sm, //U+239D
UNICODE_Sm, //U+239E
UNICODE_Sm, //U+239F
UNICODE_Sm, //U+23A0
UNICODE_Sm, //U+23A1
UNICODE_Sm, //U+23A2
UNICODE_Sm, //U+23A3
UNICODE_Sm, //U+23A4
UNICODE_Sm, //U+23A5
UNICODE_Sm, //U+23A6
UNICODE_Sm, //U+23A7
UNICODE_Sm, //U+23A8
UNICODE_Sm, //U+23A9
UNICODE_Sm, //U+23AA
UNICODE_Sm, //U+23AB
UNICODE_Sm, //U+23AC
UNICODE_Sm, //U+23AD
UNICODE_Sm, //U+23AE
UNICODE_Sm, //U+23AF
UNICODE_Sm, //U+23B0
UNICODE_Sm, //U+23B1
UNICODE_Sm, //U+23B2
UNICODE_Sm, //U+23B3
UNICODE_So, //U+23B4
UNICODE_So, //U+23B5
UNICODE_So, //U+23B6
UNICODE_So, //U+23B7
UNICODE_So, //U+23B8
UNICODE_So, //U+23B9
UNICODE_So, //U+23BA
UNICODE_So, //U+23BB
UNICODE_So, //U+23BC
UNICODE_So, //U+23BD
UNICODE_So, //U+23BE
UNICODE_So, //U+23BF
UNICODE_So, //U+23C0
UNICODE_So, //U+23C1
UNICODE_So, //U+23C2
UNICODE_So, //U+23C3
UNICODE_So, //U+23C4
UNICODE_So, //U+23C5
UNICODE_So, //U+23C6
UNICODE_So, //U+23C7
UNICODE_So, //U+23C8
UNICODE_So, //U+23C9
UNICODE_So, //U+23CA
UNICODE_So, //U+23CB
UNICODE_So, //U+23CC
UNICODE_So, //U+23CD
UNICODE_So, //U+23CE
UNICODE_So, //U+23CF
UNICODE_So, //U+23D0
UNICODE_So, //U+23D1
UNICODE_So, //U+23D2
UNICODE_So, //U+23D3
UNICODE_So, //U+23D4
UNICODE_So, //U+23D5
UNICODE_So, //U+23D6
UNICODE_So, //U+23D7
UNICODE_So, //U+23D8
UNICODE_So, //U+23D9
UNICODE_So, //U+23DA
UNICODE_So, //U+23DB
UNICODE_Sm, //U+23DC
UNICODE_Sm, //U+23DD
UNICODE_Sm, //U+23DE
UNICODE_Sm, //U+23DF
UNICODE_Sm, //U+23E0
UNICODE_Sm, //U+23E1
UNICODE_So, //U+23E2
UNICODE_So, //U+23E3
UNICODE_So, //U+23E4
UNICODE_So, //U+23E5
UNICODE_So, //U+23E6
UNICODE_So, //U+23E7
UNICODE_So, //U+23E8
UNICODE_So, //U+23E9
UNICODE_So, //U+23EA
UNICODE_So, //U+23EB
UNICODE_So, //U+23EC
UNICODE_So, //U+23ED
UNICODE_So, //U+23EE
UNICODE_So, //U+23EF
UNICODE_So, //U+23F0
UNICODE_So, //U+23F1
UNICODE_So, //U+23F2
UNICODE_So, //U+23F3
0, //U+23F4
0, //U+23F5
0, //U+23F6
0, //U+23F7
0, //U+23F8
0, //U+23F9
0, //U+23FA
0, //U+23FB
0, //U+23FC
0, //U+23FD
0, //U+23FE
0, //U+23FF
UNICODE_So, //U+2400
UNICODE_So, //U+2401
UNICODE_So, //U+2402
UNICODE_So, //U+2403
UNICODE_So, //U+2404
UNICODE_So, //U+2405
UNICODE_So, //U+2406
UNICODE_So, //U+2407
UNICODE_So, //U+2408
UNICODE_So, //U+2409
UNICODE_So, //U+240A
UNICODE_So, //U+240B
UNICODE_So, //U+240C
UNICODE_So, //U+240D
UNICODE_So, //U+240E
UNICODE_So, //U+240F
UNICODE_So, //U+2410
UNICODE_So, //U+2411
UNICODE_So, //U+2412
UNICODE_So, //U+2413
UNICODE_So, //U+2414
UNICODE_So, //U+2415
UNICODE_So, //U+2416
UNICODE_So, //U+2417
UNICODE_So, //U+2418
UNICODE_So, //U+2419
UNICODE_So, //U+241A
UNICODE_So, //U+241B
UNICODE_So, //U+241C
UNICODE_So, //U+241D
UNICODE_So, //U+241E
UNICODE_So, //U+241F
UNICODE_So, //U+2420
UNICODE_So, //U+2421
UNICODE_So, //U+2422
UNICODE_So, //U+2423
UNICODE_So, //U+2424
UNICODE_So, //U+2425
UNICODE_So, //U+2426
0, //U+2427
0, //U+2428
0, //U+2429
0, //U+242A
0, //U+242B
0, //U+242C
0, //U+242D
0, //U+242E
0, //U+242F
0, //U+2430
0, //U+2431
0, //U+2432
0, //U+2433
0, //U+2434
0, //U+2435
0, //U+2436
0, //U+2437
0, //U+2438
0, //U+2439
0, //U+243A
0, //U+243B
0, //U+243C
0, //U+243D
0, //U+243E
0, //U+243F
UNICODE_So, //U+2440
UNICODE_So, //U+2441
UNICODE_So, //U+2442
UNICODE_So, //U+2443
UNICODE_So, //U+2444
UNICODE_So, //U+2445
UNICODE_So, //U+2446
UNICODE_So, //U+2447
UNICODE_So, //U+2448
UNICODE_So, //U+2449
UNICODE_So, //U+244A
0, //U+244B
0, //U+244C
0, //U+244D
0, //U+244E
0, //U+244F
0, //U+2450
0, //U+2451
0, //U+2452
0, //U+2453
0, //U+2454
0, //U+2455
0, //U+2456
0, //U+2457
0, //U+2458
0, //U+2459
0, //U+245A
0, //U+245B
0, //U+245C
0, //U+245D
0, //U+245E
0, //U+245F
UNICODE_No, //U+2460
UNICODE_No, //U+2461
UNICODE_No, //U+2462
UNICODE_No, //U+2463
UNICODE_No, //U+2464
UNICODE_No, //U+2465
UNICODE_No, //U+2466
UNICODE_No, //U+2467
UNICODE_No, //U+2468
UNICODE_No, //U+2469
UNICODE_No, //U+246A
UNICODE_No, //U+246B
UNICODE_No, //U+246C
UNICODE_No, //U+246D
UNICODE_No, //U+246E
UNICODE_No, //U+246F
UNICODE_No, //U+2470
UNICODE_No, //U+2471
UNICODE_No, //U+2472
UNICODE_No, //U+2473
UNICODE_No, //U+2474
UNICODE_No, //U+2475
UNICODE_No, //U+2476
UNICODE_No, //U+2477
UNICODE_No, //U+2478
UNICODE_No, //U+2479
UNICODE_No, //U+247A
UNICODE_No, //U+247B
UNICODE_No, //U+247C
UNICODE_No, //U+247D
UNICODE_No, //U+247E
UNICODE_No, //U+247F
UNICODE_No, //U+2480
UNICODE_No, //U+2481
UNICODE_No, //U+2482
UNICODE_No, //U+2483
UNICODE_No, //U+2484
UNICODE_No, //U+2485
UNICODE_No, //U+2486
UNICODE_No, //U+2487
UNICODE_No, //U+2488
UNICODE_No, //U+2489
UNICODE_No, //U+248A
UNICODE_No, //U+248B
UNICODE_No, //U+248C
UNICODE_No, //U+248D
UNICODE_No, //U+248E
UNICODE_No, //U+248F
UNICODE_No, //U+2490
UNICODE_No, //U+2491
UNICODE_No, //U+2492
UNICODE_No, //U+2493
UNICODE_No, //U+2494
UNICODE_No, //U+2495
UNICODE_No, //U+2496
UNICODE_No, //U+2497
UNICODE_No, //U+2498
UNICODE_No, //U+2499
UNICODE_No, //U+249A
UNICODE_No, //U+249B
UNICODE_So, //U+249C
UNICODE_So, //U+249D
UNICODE_So, //U+249E
UNICODE_So, //U+249F
UNICODE_So, //U+24A0
UNICODE_So, //U+24A1
UNICODE_So, //U+24A2
UNICODE_So, //U+24A3
UNICODE_So, //U+24A4
UNICODE_So, //U+24A5
UNICODE_So, //U+24A6
UNICODE_So, //U+24A7
UNICODE_So, //U+24A8
UNICODE_So, //U+24A9
UNICODE_So, //U+24AA
UNICODE_So, //U+24AB
UNICODE_So, //U+24AC
UNICODE_So, //U+24AD
UNICODE_So, //U+24AE
UNICODE_So, //U+24AF
UNICODE_So, //U+24B0
UNICODE_So, //U+24B1
UNICODE_So, //U+24B2
UNICODE_So, //U+24B3
UNICODE_So, //U+24B4
UNICODE_So, //U+24B5
UNICODE_So, //U+24B6
UNICODE_So, //U+24B7
UNICODE_So, //U+24B8
UNICODE_So, //U+24B9
UNICODE_So, //U+24BA
UNICODE_So, //U+24BB
UNICODE_So, //U+24BC
UNICODE_So, //U+24BD
UNICODE_So, //U+24BE
UNICODE_So, //U+24BF
UNICODE_So, //U+24C0
UNICODE_So, //U+24C1
UNICODE_So, //U+24C2
UNICODE_So, //U+24C3
UNICODE_So, //U+24C4
UNICODE_So, //U+24C5
UNICODE_So, //U+24C6
UNICODE_So, //U+24C7
UNICODE_So, //U+24C8
UNICODE_So, //U+24C9
UNICODE_So, //U+24CA
UNICODE_So, //U+24CB
UNICODE_So, //U+24CC
UNICODE_So, //U+24CD
UNICODE_So, //U+24CE
UNICODE_So, //U+24CF
UNICODE_So, //U+24D0
UNICODE_So, //U+24D1
UNICODE_So, //U+24D2
UNICODE_So, //U+24D3
UNICODE_So, //U+24D4
UNICODE_So, //U+24D5
UNICODE_So, //U+24D6
UNICODE_So, //U+24D7
UNICODE_So, //U+24D8
UNICODE_So, //U+24D9
UNICODE_So, //U+24DA
UNICODE_So, //U+24DB
UNICODE_So, //U+24DC
UNICODE_So, //U+24DD
UNICODE_So, //U+24DE
UNICODE_So, //U+24DF
UNICODE_So, //U+24E0
UNICODE_So, //U+24E1
UNICODE_So, //U+24E2
UNICODE_So, //U+24E3
UNICODE_So, //U+24E4
UNICODE_So, //U+24E5
UNICODE_So, //U+24E6
UNICODE_So, //U+24E7
UNICODE_So, //U+24E8
UNICODE_So, //U+24E9
UNICODE_No, //U+24EA
UNICODE_No, //U+24EB
UNICODE_No, //U+24EC
UNICODE_No, //U+24ED
UNICODE_No, //U+24EE
UNICODE_No, //U+24EF
UNICODE_No, //U+24F0
UNICODE_No, //U+24F1
UNICODE_No, //U+24F2
UNICODE_No, //U+24F3
UNICODE_No, //U+24F4
UNICODE_No, //U+24F5
UNICODE_No, //U+24F6
UNICODE_No, //U+24F7
UNICODE_No, //U+24F8
UNICODE_No, //U+24F9
UNICODE_No, //U+24FA
UNICODE_No, //U+24FB
UNICODE_No, //U+24FC
UNICODE_No, //U+24FD
UNICODE_No, //U+24FE
UNICODE_No, //U+24FF
UNICODE_So, //U+2500
UNICODE_So, //U+2501
UNICODE_So, //U+2502
UNICODE_So, //U+2503
UNICODE_So, //U+2504
UNICODE_So, //U+2505
UNICODE_So, //U+2506
UNICODE_So, //U+2507
UNICODE_So, //U+2508
UNICODE_So, //U+2509
UNICODE_So, //U+250A
UNICODE_So, //U+250B
UNICODE_So, //U+250C
UNICODE_So, //U+250D
UNICODE_So, //U+250E
UNICODE_So, //U+250F
UNICODE_So, //U+2510
UNICODE_So, //U+2511
UNICODE_So, //U+2512
UNICODE_So, //U+2513
UNICODE_So, //U+2514
UNICODE_So, //U+2515
UNICODE_So, //U+2516
UNICODE_So, //U+2517
UNICODE_So, //U+2518
UNICODE_So, //U+2519
UNICODE_So, //U+251A
UNICODE_So, //U+251B
UNICODE_So, //U+251C
UNICODE_So, //U+251D
UNICODE_So, //U+251E
UNICODE_So, //U+251F
UNICODE_So, //U+2520
UNICODE_So, //U+2521
UNICODE_So, //U+2522
UNICODE_So, //U+2523
UNICODE_So, //U+2524
UNICODE_So, //U+2525
UNICODE_So, //U+2526
UNICODE_So, //U+2527
UNICODE_So, //U+2528
UNICODE_So, //U+2529
UNICODE_So, //U+252A
UNICODE_So, //U+252B
UNICODE_So, //U+252C
UNICODE_So, //U+252D
UNICODE_So, //U+252E
UNICODE_So, //U+252F
UNICODE_So, //U+2530
UNICODE_So, //U+2531
UNICODE_So, //U+2532
UNICODE_So, //U+2533
UNICODE_So, //U+2534
UNICODE_So, //U+2535
UNICODE_So, //U+2536
UNICODE_So, //U+2537
UNICODE_So, //U+2538
UNICODE_So, //U+2539
UNICODE_So, //U+253A
UNICODE_So, //U+253B
UNICODE_So, //U+253C
UNICODE_So, //U+253D
UNICODE_So, //U+253E
UNICODE_So, //U+253F
UNICODE_So, //U+2540
UNICODE_So, //U+2541
UNICODE_So, //U+2542
UNICODE_So, //U+2543
UNICODE_So, //U+2544
UNICODE_So, //U+2545
UNICODE_So, //U+2546
UNICODE_So, //U+2547
UNICODE_So, //U+2548
UNICODE_So, //U+2549
UNICODE_So, //U+254A
UNICODE_So, //U+254B
UNICODE_So, //U+254C
UNICODE_So, //U+254D
UNICODE_So, //U+254E
UNICODE_So, //U+254F
UNICODE_So, //U+2550
UNICODE_So, //U+2551
UNICODE_So, //U+2552
UNICODE_So, //U+2553
UNICODE_So, //U+2554
UNICODE_So, //U+2555
UNICODE_So, //U+2556
UNICODE_So, //U+2557
UNICODE_So, //U+2558
UNICODE_So, //U+2559
UNICODE_So, //U+255A
UNICODE_So, //U+255B
UNICODE_So, //U+255C
UNICODE_So, //U+255D
UNICODE_So, //U+255E
UNICODE_So, //U+255F
UNICODE_So, //U+2560
UNICODE_So, //U+2561
UNICODE_So, //U+2562
UNICODE_So, //U+2563
UNICODE_So, //U+2564
UNICODE_So, //U+2565
UNICODE_So, //U+2566
UNICODE_So, //U+2567
UNICODE_So, //U+2568
UNICODE_So, //U+2569
UNICODE_So, //U+256A
UNICODE_So, //U+256B
UNICODE_So, //U+256C
UNICODE_So, //U+256D
UNICODE_So, //U+256E
UNICODE_So, //U+256F
UNICODE_So, //U+2570
UNICODE_So, //U+2571
UNICODE_So, //U+2572
UNICODE_So, //U+2573
UNICODE_So, //U+2574
UNICODE_So, //U+2575
UNICODE_So, //U+2576
UNICODE_So, //U+2577
UNICODE_So, //U+2578
UNICODE_So, //U+2579
UNICODE_So, //U+257A
UNICODE_So, //U+257B
UNICODE_So, //U+257C
UNICODE_So, //U+257D
UNICODE_So, //U+257E
UNICODE_So, //U+257F
UNICODE_So, //U+2580
UNICODE_So, //U+2581
UNICODE_So, //U+2582
UNICODE_So, //U+2583
UNICODE_So, //U+2584
UNICODE_So, //U+2585
UNICODE_So, //U+2586
UNICODE_So, //U+2587
UNICODE_So, //U+2588
UNICODE_So, //U+2589
UNICODE_So, //U+258A
UNICODE_So, //U+258B
UNICODE_So, //U+258C
UNICODE_So, //U+258D
UNICODE_So, //U+258E
UNICODE_So, //U+258F
UNICODE_So, //U+2590
UNICODE_So, //U+2591
UNICODE_So, //U+2592
UNICODE_So, //U+2593
UNICODE_So, //U+2594
UNICODE_So, //U+2595
UNICODE_So, //U+2596
UNICODE_So, //U+2597
UNICODE_So, //U+2598
UNICODE_So, //U+2599
UNICODE_So, //U+259A
UNICODE_So, //U+259B
UNICODE_So, //U+259C
UNICODE_So, //U+259D
UNICODE_So, //U+259E
UNICODE_So, //U+259F
UNICODE_So, //U+25A0
UNICODE_So, //U+25A1
UNICODE_So, //U+25A2
UNICODE_So, //U+25A3
UNICODE_So, //U+25A4
UNICODE_So, //U+25A5
UNICODE_So, //U+25A6
UNICODE_So, //U+25A7
UNICODE_So, //U+25A8
UNICODE_So, //U+25A9
UNICODE_So, //U+25AA
UNICODE_So, //U+25AB
UNICODE_So, //U+25AC
UNICODE_So, //U+25AD
UNICODE_So, //U+25AE
UNICODE_So, //U+25AF
UNICODE_So, //U+25B0
UNICODE_So, //U+25B1
UNICODE_So, //U+25B2
UNICODE_So, //U+25B3
UNICODE_So, //U+25B4
UNICODE_So, //U+25B5
UNICODE_So, //U+25B6
UNICODE_Sm, //U+25B7
UNICODE_So, //U+25B8
UNICODE_So, //U+25B9
UNICODE_So, //U+25BA
UNICODE_So, //U+25BB
UNICODE_So, //U+25BC
UNICODE_So, //U+25BD
UNICODE_So, //U+25BE
UNICODE_So, //U+25BF
UNICODE_So, //U+25C0
UNICODE_Sm, //U+25C1
UNICODE_So, //U+25C2
UNICODE_So, //U+25C3
UNICODE_So, //U+25C4
UNICODE_So, //U+25C5
UNICODE_So, //U+25C6
UNICODE_So, //U+25C7
UNICODE_So, //U+25C8
UNICODE_So, //U+25C9
UNICODE_So, //U+25CA
UNICODE_So, //U+25CB
UNICODE_So, //U+25CC
UNICODE_So, //U+25CD
UNICODE_So, //U+25CE
UNICODE_So, //U+25CF
UNICODE_So, //U+25D0
UNICODE_So, //U+25D1
UNICODE_So, //U+25D2
UNICODE_So, //U+25D3
UNICODE_So, //U+25D4
UNICODE_So, //U+25D5
UNICODE_So, //U+25D6
UNICODE_So, //U+25D7
UNICODE_So, //U+25D8
UNICODE_So, //U+25D9
UNICODE_So, //U+25DA
UNICODE_So, //U+25DB
UNICODE_So, //U+25DC
UNICODE_So, //U+25DD
UNICODE_So, //U+25DE
UNICODE_So, //U+25DF
UNICODE_So, //U+25E0
UNICODE_So, //U+25E1
UNICODE_So, //U+25E2
UNICODE_So, //U+25E3
UNICODE_So, //U+25E4
UNICODE_So, //U+25E5
UNICODE_So, //U+25E6
UNICODE_So, //U+25E7
UNICODE_So, //U+25E8
UNICODE_So, //U+25E9
UNICODE_So, //U+25EA
UNICODE_So, //U+25EB
UNICODE_So, //U+25EC
UNICODE_So, //U+25ED
UNICODE_So, //U+25EE
UNICODE_So, //U+25EF
UNICODE_So, //U+25F0
UNICODE_So, //U+25F1
UNICODE_So, //U+25F2
UNICODE_So, //U+25F3
UNICODE_So, //U+25F4
UNICODE_So, //U+25F5
UNICODE_So, //U+25F6
UNICODE_So, //U+25F7
UNICODE_Sm, //U+25F8
UNICODE_Sm, //U+25F9
UNICODE_Sm, //U+25FA
UNICODE_Sm, //U+25FB
UNICODE_Sm, //U+25FC
UNICODE_Sm, //U+25FD
UNICODE_Sm, //U+25FE
UNICODE_Sm, //U+25FF
UNICODE_So, //U+2600
UNICODE_So, //U+2601
UNICODE_So, //U+2602
UNICODE_So, //U+2603
UNICODE_So, //U+2604
UNICODE_So, //U+2605
UNICODE_So, //U+2606
UNICODE_So, //U+2607
UNICODE_So, //U+2608
UNICODE_So, //U+2609
UNICODE_So, //U+260A
UNICODE_So, //U+260B
UNICODE_So, //U+260C
UNICODE_So, //U+260D
UNICODE_So, //U+260E
UNICODE_So, //U+260F
UNICODE_So, //U+2610
UNICODE_So, //U+2611
UNICODE_So, //U+2612
UNICODE_So, //U+2613
UNICODE_So, //U+2614
UNICODE_So, //U+2615
UNICODE_So, //U+2616
UNICODE_So, //U+2617
UNICODE_So, //U+2618
UNICODE_So, //U+2619
UNICODE_So, //U+261A
UNICODE_So, //U+261B
UNICODE_So, //U+261C
UNICODE_So, //U+261D
UNICODE_So, //U+261E
UNICODE_So, //U+261F
UNICODE_So, //U+2620
UNICODE_So, //U+2621
UNICODE_So, //U+2622
UNICODE_So, //U+2623
UNICODE_So, //U+2624
UNICODE_So, //U+2625
UNICODE_So, //U+2626
UNICODE_So, //U+2627
UNICODE_So, //U+2628
UNICODE_So, //U+2629
UNICODE_So, //U+262A
UNICODE_So, //U+262B
UNICODE_So, //U+262C
UNICODE_So, //U+262D
UNICODE_So, //U+262E
UNICODE_So, //U+262F
UNICODE_So, //U+2630
UNICODE_So, //U+2631
UNICODE_So, //U+2632
UNICODE_So, //U+2633
UNICODE_So, //U+2634
UNICODE_So, //U+2635
UNICODE_So, //U+2636
UNICODE_So, //U+2637
UNICODE_So, //U+2638
UNICODE_So, //U+2639
UNICODE_So, //U+263A
UNICODE_So, //U+263B
UNICODE_So, //U+263C
UNICODE_So, //U+263D
UNICODE_So, //U+263E
UNICODE_So, //U+263F
UNICODE_So, //U+2640
UNICODE_So, //U+2641
UNICODE_So, //U+2642
UNICODE_So, //U+2643
UNICODE_So, //U+2644
UNICODE_So, //U+2645
UNICODE_So, //U+2646
UNICODE_So, //U+2647
UNICODE_So, //U+2648
UNICODE_So, //U+2649
UNICODE_So, //U+264A
UNICODE_So, //U+264B
UNICODE_So, //U+264C
UNICODE_So, //U+264D
UNICODE_So, //U+264E
UNICODE_So, //U+264F
UNICODE_So, //U+2650
UNICODE_So, //U+2651
UNICODE_So, //U+2652
UNICODE_So, //U+2653
UNICODE_So, //U+2654
UNICODE_So, //U+2655
UNICODE_So, //U+2656
UNICODE_So, //U+2657
UNICODE_So, //U+2658
UNICODE_So, //U+2659
UNICODE_So, //U+265A
UNICODE_So, //U+265B
UNICODE_So, //U+265C
UNICODE_So, //U+265D
UNICODE_So, //U+265E
UNICODE_So, //U+265F
UNICODE_So, //U+2660
UNICODE_So, //U+2661
UNICODE_So, //U+2662
UNICODE_So, //U+2663
UNICODE_So, //U+2664
UNICODE_So, //U+2665
UNICODE_So, //U+2666
UNICODE_So, //U+2667
UNICODE_So, //U+2668
UNICODE_So, //U+2669
UNICODE_So, //U+266A
UNICODE_So, //U+266B
UNICODE_So, //U+266C
UNICODE_So, //U+266D
UNICODE_So, //U+266E
UNICODE_Sm, //U+266F
UNICODE_So, //U+2670
UNICODE_So, //U+2671
UNICODE_So, //U+2672
UNICODE_So, //U+2673
UNICODE_So, //U+2674
UNICODE_So, //U+2675
UNICODE_So, //U+2676
UNICODE_So, //U+2677
UNICODE_So, //U+2678
UNICODE_So, //U+2679
UNICODE_So, //U+267A
UNICODE_So, //U+267B
UNICODE_So, //U+267C
UNICODE_So, //U+267D
UNICODE_So, //U+267E
UNICODE_So, //U+267F
UNICODE_So, //U+2680
UNICODE_So, //U+2681
UNICODE_So, //U+2682
UNICODE_So, //U+2683
UNICODE_So, //U+2684
UNICODE_So, //U+2685
UNICODE_So, //U+2686
UNICODE_So, //U+2687
UNICODE_So, //U+2688
UNICODE_So, //U+2689
UNICODE_So, //U+268A
UNICODE_So, //U+268B
UNICODE_So, //U+268C
UNICODE_So, //U+268D
UNICODE_So, //U+268E
UNICODE_So, //U+268F
UNICODE_So, //U+2690
UNICODE_So, //U+2691
UNICODE_So, //U+2692
UNICODE_So, //U+2693
UNICODE_So, //U+2694
UNICODE_So, //U+2695
UNICODE_So, //U+2696
UNICODE_So, //U+2697
UNICODE_So, //U+2698
UNICODE_So, //U+2699
UNICODE_So, //U+269A
UNICODE_So, //U+269B
UNICODE_So, //U+269C
UNICODE_So, //U+269D
UNICODE_So, //U+269E
UNICODE_So, //U+269F
UNICODE_So, //U+26A0
UNICODE_So, //U+26A1
UNICODE_So, //U+26A2
UNICODE_So, //U+26A3
UNICODE_So, //U+26A4
UNICODE_So, //U+26A5
UNICODE_So, //U+26A6
UNICODE_So, //U+26A7
UNICODE_So, //U+26A8
UNICODE_So, //U+26A9
UNICODE_So, //U+26AA
UNICODE_So, //U+26AB
UNICODE_So, //U+26AC
UNICODE_So, //U+26AD
UNICODE_So, //U+26AE
UNICODE_So, //U+26AF
UNICODE_So, //U+26B0
UNICODE_So, //U+26B1
UNICODE_So, //U+26B2
UNICODE_So, //U+26B3
UNICODE_So, //U+26B4
UNICODE_So, //U+26B5
UNICODE_So, //U+26B6
UNICODE_So, //U+26B7
UNICODE_So, //U+26B8
UNICODE_So, //U+26B9
UNICODE_So, //U+26BA
UNICODE_So, //U+26BB
UNICODE_So, //U+26BC
UNICODE_So, //U+26BD
UNICODE_So, //U+26BE
UNICODE_So, //U+26BF
UNICODE_So, //U+26C0
UNICODE_So, //U+26C1
UNICODE_So, //U+26C2
UNICODE_So, //U+26C3
UNICODE_So, //U+26C4
UNICODE_So, //U+26C5
UNICODE_So, //U+26C6
UNICODE_So, //U+26C7
UNICODE_So, //U+26C8
UNICODE_So, //U+26C9
UNICODE_So, //U+26CA
UNICODE_So, //U+26CB
UNICODE_So, //U+26CC
UNICODE_So, //U+26CD
UNICODE_So, //U+26CE
UNICODE_So, //U+26CF
UNICODE_So, //U+26D0
UNICODE_So, //U+26D1
UNICODE_So, //U+26D2
UNICODE_So, //U+26D3
UNICODE_So, //U+26D4
UNICODE_So, //U+26D5
UNICODE_So, //U+26D6
UNICODE_So, //U+26D7
UNICODE_So, //U+26D8
UNICODE_So, //U+26D9
UNICODE_So, //U+26DA
UNICODE_So, //U+26DB
UNICODE_So, //U+26DC
UNICODE_So, //U+26DD
UNICODE_So, //U+26DE
UNICODE_So, //U+26DF
UNICODE_So, //U+26E0
UNICODE_So, //U+26E1
UNICODE_So, //U+26E2
UNICODE_So, //U+26E3
UNICODE_So, //U+26E4
UNICODE_So, //U+26E5
UNICODE_So, //U+26E6
UNICODE_So, //U+26E7
UNICODE_So, //U+26E8
UNICODE_So, //U+26E9
UNICODE_So, //U+26EA
UNICODE_So, //U+26EB
UNICODE_So, //U+26EC
UNICODE_So, //U+26ED
UNICODE_So, //U+26EE
UNICODE_So, //U+26EF
UNICODE_So, //U+26F0
UNICODE_So, //U+26F1
UNICODE_So, //U+26F2
UNICODE_So, //U+26F3
UNICODE_So, //U+26F4
UNICODE_So, //U+26F5
UNICODE_So, //U+26F6
UNICODE_So, //U+26F7
UNICODE_So, //U+26F8
UNICODE_So, //U+26F9
UNICODE_So, //U+26FA
UNICODE_So, //U+26FB
UNICODE_So, //U+26FC
UNICODE_So, //U+26FD
UNICODE_So, //U+26FE
UNICODE_So, //U+26FF
0, //U+2700
UNICODE_So, //U+2701
UNICODE_So, //U+2702
UNICODE_So, //U+2703
UNICODE_So, //U+2704
UNICODE_So, //U+2705
UNICODE_So, //U+2706
UNICODE_So, //U+2707
UNICODE_So, //U+2708
UNICODE_So, //U+2709
UNICODE_So, //U+270A
UNICODE_So, //U+270B
UNICODE_So, //U+270C
UNICODE_So, //U+270D
UNICODE_So, //U+270E
UNICODE_So, //U+270F
UNICODE_So, //U+2710
UNICODE_So, //U+2711
UNICODE_So, //U+2712
UNICODE_So, //U+2713
UNICODE_So, //U+2714
UNICODE_So, //U+2715
UNICODE_So, //U+2716
UNICODE_So, //U+2717
UNICODE_So, //U+2718
UNICODE_So, //U+2719
UNICODE_So, //U+271A
UNICODE_So, //U+271B
UNICODE_So, //U+271C
UNICODE_So, //U+271D
UNICODE_So, //U+271E
UNICODE_So, //U+271F
UNICODE_So, //U+2720
UNICODE_So, //U+2721
UNICODE_So, //U+2722
UNICODE_So, //U+2723
UNICODE_So, //U+2724
UNICODE_So, //U+2725
UNICODE_So, //U+2726
UNICODE_So, //U+2727
UNICODE_So, //U+2728
UNICODE_So, //U+2729
UNICODE_So, //U+272A
UNICODE_So, //U+272B
UNICODE_So, //U+272C
UNICODE_So, //U+272D
UNICODE_So, //U+272E
UNICODE_So, //U+272F
UNICODE_So, //U+2730
UNICODE_So, //U+2731
UNICODE_So, //U+2732
UNICODE_So, //U+2733
UNICODE_So, //U+2734
UNICODE_So, //U+2735
UNICODE_So, //U+2736
UNICODE_So, //U+2737
UNICODE_So, //U+2738
UNICODE_So, //U+2739
UNICODE_So, //U+273A
UNICODE_So, //U+273B
UNICODE_So, //U+273C
UNICODE_So, //U+273D
UNICODE_So, //U+273E
UNICODE_So, //U+273F
UNICODE_So, //U+2740
UNICODE_So, //U+2741
UNICODE_So, //U+2742
UNICODE_So, //U+2743
UNICODE_So, //U+2744
UNICODE_So, //U+2745
UNICODE_So, //U+2746
UNICODE_So, //U+2747
UNICODE_So, //U+2748
UNICODE_So, //U+2749
UNICODE_So, //U+274A
UNICODE_So, //U+274B
UNICODE_So, //U+274C
UNICODE_So, //U+274D
UNICODE_So, //U+274E
UNICODE_So, //U+274F
UNICODE_So, //U+2750
UNICODE_So, //U+2751
UNICODE_So, //U+2752
UNICODE_So, //U+2753
UNICODE_So, //U+2754
UNICODE_So, //U+2755
UNICODE_So, //U+2756
UNICODE_So, //U+2757
UNICODE_So, //U+2758
UNICODE_So, //U+2759
UNICODE_So, //U+275A
UNICODE_So, //U+275B
UNICODE_So, //U+275C
UNICODE_So, //U+275D
UNICODE_So, //U+275E
UNICODE_So, //U+275F
UNICODE_So, //U+2760
UNICODE_So, //U+2761
UNICODE_So, //U+2762
UNICODE_So, //U+2763
UNICODE_So, //U+2764
UNICODE_So, //U+2765
UNICODE_So, //U+2766
UNICODE_So, //U+2767
UNICODE_Ps, //U+2768
UNICODE_Pe, //U+2769
UNICODE_Ps, //U+276A
UNICODE_Pe, //U+276B
UNICODE_Ps, //U+276C
UNICODE_Pe, //U+276D
UNICODE_Ps, //U+276E
UNICODE_Pe, //U+276F
UNICODE_Ps, //U+2770
UNICODE_Pe, //U+2771
UNICODE_Ps, //U+2772
UNICODE_Pe, //U+2773
UNICODE_Ps, //U+2774
UNICODE_Pe, //U+2775
UNICODE_No, //U+2776
UNICODE_No, //U+2777
UNICODE_No, //U+2778
UNICODE_No, //U+2779
UNICODE_No, //U+277A
UNICODE_No, //U+277B
UNICODE_No, //U+277C
UNICODE_No, //U+277D
UNICODE_No, //U+277E
UNICODE_No, //U+277F
UNICODE_No, //U+2780
UNICODE_No, //U+2781
UNICODE_No, //U+2782
UNICODE_No, //U+2783
UNICODE_No, //U+2784
UNICODE_No, //U+2785
UNICODE_No, //U+2786
UNICODE_No, //U+2787
UNICODE_No, //U+2788
UNICODE_No, //U+2789
UNICODE_No, //U+278A
UNICODE_No, //U+278B
UNICODE_No, //U+278C
UNICODE_No, //U+278D
UNICODE_No, //U+278E
UNICODE_No, //U+278F
UNICODE_No, //U+2790
UNICODE_No, //U+2791
UNICODE_No, //U+2792
UNICODE_No, //U+2793
UNICODE_So, //U+2794
UNICODE_So, //U+2795
UNICODE_So, //U+2796
UNICODE_So, //U+2797
UNICODE_So, //U+2798
UNICODE_So, //U+2799
UNICODE_So, //U+279A
UNICODE_So, //U+279B
UNICODE_So, //U+279C
UNICODE_So, //U+279D
UNICODE_So, //U+279E
UNICODE_So, //U+279F
UNICODE_So, //U+27A0
UNICODE_So, //U+27A1
UNICODE_So, //U+27A2
UNICODE_So, //U+27A3
UNICODE_So, //U+27A4
UNICODE_So, //U+27A5
UNICODE_So, //U+27A6
UNICODE_So, //U+27A7
UNICODE_So, //U+27A8
UNICODE_So, //U+27A9
UNICODE_So, //U+27AA
UNICODE_So, //U+27AB
UNICODE_So, //U+27AC
UNICODE_So, //U+27AD
UNICODE_So, //U+27AE
UNICODE_So, //U+27AF
UNICODE_So, //U+27B0
UNICODE_So, //U+27B1
UNICODE_So, //U+27B2
UNICODE_So, //U+27B3
UNICODE_So, //U+27B4
UNICODE_So, //U+27B5
UNICODE_So, //U+27B6
UNICODE_So, //U+27B7
UNICODE_So, //U+27B8
UNICODE_So, //U+27B9
UNICODE_So, //U+27BA
UNICODE_So, //U+27BB
UNICODE_So, //U+27BC
UNICODE_So, //U+27BD
UNICODE_So, //U+27BE
UNICODE_So, //U+27BF
UNICODE_Sm, //U+27C0
UNICODE_Sm, //U+27C1
UNICODE_Sm, //U+27C2
UNICODE_Sm, //U+27C3
UNICODE_Sm, //U+27C4
UNICODE_Ps, //U+27C5
UNICODE_Pe, //U+27C6
UNICODE_Sm, //U+27C7
UNICODE_Sm, //U+27C8
UNICODE_Sm, //U+27C9
UNICODE_Sm, //U+27CA
0, //U+27CB
UNICODE_Sm, //U+27CC
0, //U+27CD
UNICODE_Sm, //U+27CE
UNICODE_Sm, //U+27CF
UNICODE_Sm, //U+27D0
UNICODE_Sm, //U+27D1
UNICODE_Sm, //U+27D2
UNICODE_Sm, //U+27D3
UNICODE_Sm, //U+27D4
UNICODE_Sm, //U+27D5
UNICODE_Sm, //U+27D6
UNICODE_Sm, //U+27D7
UNICODE_Sm, //U+27D8
UNICODE_Sm, //U+27D9
UNICODE_Sm, //U+27DA
UNICODE_Sm, //U+27DB
UNICODE_Sm, //U+27DC
UNICODE_Sm, //U+27DD
UNICODE_Sm, //U+27DE
UNICODE_Sm, //U+27DF
UNICODE_Sm, //U+27E0
UNICODE_Sm, //U+27E1
UNICODE_Sm, //U+27E2
UNICODE_Sm, //U+27E3
UNICODE_Sm, //U+27E4
UNICODE_Sm, //U+27E5
UNICODE_Ps, //U+27E6
UNICODE_Pe, //U+27E7
UNICODE_Ps, //U+27E8
UNICODE_Pe, //U+27E9
UNICODE_Ps, //U+27EA
UNICODE_Pe, //U+27EB
UNICODE_Ps, //U+27EC
UNICODE_Pe, //U+27ED
UNICODE_Ps, //U+27EE
UNICODE_Pe, //U+27EF
UNICODE_Sm, //U+27F0
UNICODE_Sm, //U+27F1
UNICODE_Sm, //U+27F2
UNICODE_Sm, //U+27F3
UNICODE_Sm, //U+27F4
UNICODE_Sm, //U+27F5
UNICODE_Sm, //U+27F6
UNICODE_Sm, //U+27F7
UNICODE_Sm, //U+27F8
UNICODE_Sm, //U+27F9
UNICODE_Sm, //U+27FA
UNICODE_Sm, //U+27FB
UNICODE_Sm, //U+27FC
UNICODE_Sm, //U+27FD
UNICODE_Sm, //U+27FE
UNICODE_Sm, //U+27FF
UNICODE_So, //U+2800
UNICODE_So, //U+2801
UNICODE_So, //U+2802
UNICODE_So, //U+2803
UNICODE_So, //U+2804
UNICODE_So, //U+2805
UNICODE_So, //U+2806
UNICODE_So, //U+2807
UNICODE_So, //U+2808
UNICODE_So, //U+2809
UNICODE_So, //U+280A
UNICODE_So, //U+280B
UNICODE_So, //U+280C
UNICODE_So, //U+280D
UNICODE_So, //U+280E
UNICODE_So, //U+280F
UNICODE_So, //U+2810
UNICODE_So, //U+2811
UNICODE_So, //U+2812
UNICODE_So, //U+2813
UNICODE_So, //U+2814
UNICODE_So, //U+2815
UNICODE_So, //U+2816
UNICODE_So, //U+2817
UNICODE_So, //U+2818
UNICODE_So, //U+2819
UNICODE_So, //U+281A
UNICODE_So, //U+281B
UNICODE_So, //U+281C
UNICODE_So, //U+281D
UNICODE_So, //U+281E
UNICODE_So, //U+281F
UNICODE_So, //U+2820
UNICODE_So, //U+2821
UNICODE_So, //U+2822
UNICODE_So, //U+2823
UNICODE_So, //U+2824
UNICODE_So, //U+2825
UNICODE_So, //U+2826
UNICODE_So, //U+2827
UNICODE_So, //U+2828
UNICODE_So, //U+2829
UNICODE_So, //U+282A
UNICODE_So, //U+282B
UNICODE_So, //U+282C
UNICODE_So, //U+282D
UNICODE_So, //U+282E
UNICODE_So, //U+282F
UNICODE_So, //U+2830
UNICODE_So, //U+2831
UNICODE_So, //U+2832
UNICODE_So, //U+2833
UNICODE_So, //U+2834
UNICODE_So, //U+2835
UNICODE_So, //U+2836
UNICODE_So, //U+2837
UNICODE_So, //U+2838
UNICODE_So, //U+2839
UNICODE_So, //U+283A
UNICODE_So, //U+283B
UNICODE_So, //U+283C
UNICODE_So, //U+283D
UNICODE_So, //U+283E
UNICODE_So, //U+283F
UNICODE_So, //U+2840
UNICODE_So, //U+2841
UNICODE_So, //U+2842
UNICODE_So, //U+2843
UNICODE_So, //U+2844
UNICODE_So, //U+2845
UNICODE_So, //U+2846
UNICODE_So, //U+2847
UNICODE_So, //U+2848
UNICODE_So, //U+2849
UNICODE_So, //U+284A
UNICODE_So, //U+284B
UNICODE_So, //U+284C
UNICODE_So, //U+284D
UNICODE_So, //U+284E
UNICODE_So, //U+284F
UNICODE_So, //U+2850
UNICODE_So, //U+2851
UNICODE_So, //U+2852
UNICODE_So, //U+2853
UNICODE_So, //U+2854
UNICODE_So, //U+2855
UNICODE_So, //U+2856
UNICODE_So, //U+2857
UNICODE_So, //U+2858
UNICODE_So, //U+2859
UNICODE_So, //U+285A
UNICODE_So, //U+285B
UNICODE_So, //U+285C
UNICODE_So, //U+285D
UNICODE_So, //U+285E
UNICODE_So, //U+285F
UNICODE_So, //U+2860
UNICODE_So, //U+2861
UNICODE_So, //U+2862
UNICODE_So, //U+2863
UNICODE_So, //U+2864
UNICODE_So, //U+2865
UNICODE_So, //U+2866
UNICODE_So, //U+2867
UNICODE_So, //U+2868
UNICODE_So, //U+2869
UNICODE_So, //U+286A
UNICODE_So, //U+286B
UNICODE_So, //U+286C
UNICODE_So, //U+286D
UNICODE_So, //U+286E
UNICODE_So, //U+286F
UNICODE_So, //U+2870
UNICODE_So, //U+2871
UNICODE_So, //U+2872
UNICODE_So, //U+2873
UNICODE_So, //U+2874
UNICODE_So, //U+2875
UNICODE_So, //U+2876
UNICODE_So, //U+2877
UNICODE_So, //U+2878
UNICODE_So, //U+2879
UNICODE_So, //U+287A
UNICODE_So, //U+287B
UNICODE_So, //U+287C
UNICODE_So, //U+287D
UNICODE_So, //U+287E
UNICODE_So, //U+287F
UNICODE_So, //U+2880
UNICODE_So, //U+2881
UNICODE_So, //U+2882
UNICODE_So, //U+2883
UNICODE_So, //U+2884
UNICODE_So, //U+2885
UNICODE_So, //U+2886
UNICODE_So, //U+2887
UNICODE_So, //U+2888
UNICODE_So, //U+2889
UNICODE_So, //U+288A
UNICODE_So, //U+288B
UNICODE_So, //U+288C
UNICODE_So, //U+288D
UNICODE_So, //U+288E
UNICODE_So, //U+288F
UNICODE_So, //U+2890
UNICODE_So, //U+2891
UNICODE_So, //U+2892
UNICODE_So, //U+2893
UNICODE_So, //U+2894
UNICODE_So, //U+2895
UNICODE_So, //U+2896
UNICODE_So, //U+2897
UNICODE_So, //U+2898
UNICODE_So, //U+2899
UNICODE_So, //U+289A
UNICODE_So, //U+289B
UNICODE_So, //U+289C
UNICODE_So, //U+289D
UNICODE_So, //U+289E
UNICODE_So, //U+289F
UNICODE_So, //U+28A0
UNICODE_So, //U+28A1
UNICODE_So, //U+28A2
UNICODE_So, //U+28A3
UNICODE_So, //U+28A4
UNICODE_So, //U+28A5
UNICODE_So, //U+28A6
UNICODE_So, //U+28A7
UNICODE_So, //U+28A8
UNICODE_So, //U+28A9
UNICODE_So, //U+28AA
UNICODE_So, //U+28AB
UNICODE_So, //U+28AC
UNICODE_So, //U+28AD
UNICODE_So, //U+28AE
UNICODE_So, //U+28AF
UNICODE_So, //U+28B0
UNICODE_So, //U+28B1
UNICODE_So, //U+28B2
UNICODE_So, //U+28B3
UNICODE_So, //U+28B4
UNICODE_So, //U+28B5
UNICODE_So, //U+28B6
UNICODE_So, //U+28B7
UNICODE_So, //U+28B8
UNICODE_So, //U+28B9
UNICODE_So, //U+28BA
UNICODE_So, //U+28BB
UNICODE_So, //U+28BC
UNICODE_So, //U+28BD
UNICODE_So, //U+28BE
UNICODE_So, //U+28BF
UNICODE_So, //U+28C0
UNICODE_So, //U+28C1
UNICODE_So, //U+28C2
UNICODE_So, //U+28C3
UNICODE_So, //U+28C4
UNICODE_So, //U+28C5
UNICODE_So, //U+28C6
UNICODE_So, //U+28C7
UNICODE_So, //U+28C8
UNICODE_So, //U+28C9
UNICODE_So, //U+28CA
UNICODE_So, //U+28CB
UNICODE_So, //U+28CC
UNICODE_So, //U+28CD
UNICODE_So, //U+28CE
UNICODE_So, //U+28CF
UNICODE_So, //U+28D0
UNICODE_So, //U+28D1
UNICODE_So, //U+28D2
UNICODE_So, //U+28D3
UNICODE_So, //U+28D4
UNICODE_So, //U+28D5
UNICODE_So, //U+28D6
UNICODE_So, //U+28D7
UNICODE_So, //U+28D8
UNICODE_So, //U+28D9
UNICODE_So, //U+28DA
UNICODE_So, //U+28DB
UNICODE_So, //U+28DC
UNICODE_So, //U+28DD
UNICODE_So, //U+28DE
UNICODE_So, //U+28DF
UNICODE_So, //U+28E0
UNICODE_So, //U+28E1
UNICODE_So, //U+28E2
UNICODE_So, //U+28E3
UNICODE_So, //U+28E4
UNICODE_So, //U+28E5
UNICODE_So, //U+28E6
UNICODE_So, //U+28E7
UNICODE_So, //U+28E8
UNICODE_So, //U+28E9
UNICODE_So, //U+28EA
UNICODE_So, //U+28EB
UNICODE_So, //U+28EC
UNICODE_So, //U+28ED
UNICODE_So, //U+28EE
UNICODE_So, //U+28EF
UNICODE_So, //U+28F0
UNICODE_So, //U+28F1
UNICODE_So, //U+28F2
UNICODE_So, //U+28F3
UNICODE_So, //U+28F4
UNICODE_So, //U+28F5
UNICODE_So, //U+28F6
UNICODE_So, //U+28F7
UNICODE_So, //U+28F8
UNICODE_So, //U+28F9
UNICODE_So, //U+28FA
UNICODE_So, //U+28FB
UNICODE_So, //U+28FC
UNICODE_So, //U+28FD
UNICODE_So, //U+28FE
UNICODE_So, //U+28FF
UNICODE_Sm, //U+2900
UNICODE_Sm, //U+2901
UNICODE_Sm, //U+2902
UNICODE_Sm, //U+2903
UNICODE_Sm, //U+2904
UNICODE_Sm, //U+2905
UNICODE_Sm, //U+2906
UNICODE_Sm, //U+2907
UNICODE_Sm, //U+2908
UNICODE_Sm, //U+2909
UNICODE_Sm, //U+290A
UNICODE_Sm, //U+290B
UNICODE_Sm, //U+290C
UNICODE_Sm, //U+290D
UNICODE_Sm, //U+290E
UNICODE_Sm, //U+290F
UNICODE_Sm, //U+2910
UNICODE_Sm, //U+2911
UNICODE_Sm, //U+2912
UNICODE_Sm, //U+2913
UNICODE_Sm, //U+2914
UNICODE_Sm, //U+2915
UNICODE_Sm, //U+2916
UNICODE_Sm, //U+2917
UNICODE_Sm, //U+2918
UNICODE_Sm, //U+2919
UNICODE_Sm, //U+291A
UNICODE_Sm, //U+291B
UNICODE_Sm, //U+291C
UNICODE_Sm, //U+291D
UNICODE_Sm, //U+291E
UNICODE_Sm, //U+291F
UNICODE_Sm, //U+2920
UNICODE_Sm, //U+2921
UNICODE_Sm, //U+2922
UNICODE_Sm, //U+2923
UNICODE_Sm, //U+2924
UNICODE_Sm, //U+2925
UNICODE_Sm, //U+2926
UNICODE_Sm, //U+2927
UNICODE_Sm, //U+2928
UNICODE_Sm, //U+2929
UNICODE_Sm, //U+292A
UNICODE_Sm, //U+292B
UNICODE_Sm, //U+292C
UNICODE_Sm, //U+292D
UNICODE_Sm, //U+292E
UNICODE_Sm, //U+292F
UNICODE_Sm, //U+2930
UNICODE_Sm, //U+2931
UNICODE_Sm, //U+2932
UNICODE_Sm, //U+2933
UNICODE_Sm, //U+2934
UNICODE_Sm, //U+2935
UNICODE_Sm, //U+2936
UNICODE_Sm, //U+2937
UNICODE_Sm, //U+2938
UNICODE_Sm, //U+2939
UNICODE_Sm, //U+293A
UNICODE_Sm, //U+293B
UNICODE_Sm, //U+293C
UNICODE_Sm, //U+293D
UNICODE_Sm, //U+293E
UNICODE_Sm, //U+293F
UNICODE_Sm, //U+2940
UNICODE_Sm, //U+2941
UNICODE_Sm, //U+2942
UNICODE_Sm, //U+2943
UNICODE_Sm, //U+2944
UNICODE_Sm, //U+2945
UNICODE_Sm, //U+2946
UNICODE_Sm, //U+2947
UNICODE_Sm, //U+2948
UNICODE_Sm, //U+2949
UNICODE_Sm, //U+294A
UNICODE_Sm, //U+294B
UNICODE_Sm, //U+294C
UNICODE_Sm, //U+294D
UNICODE_Sm, //U+294E
UNICODE_Sm, //U+294F
UNICODE_Sm, //U+2950
UNICODE_Sm, //U+2951
UNICODE_Sm, //U+2952
UNICODE_Sm, //U+2953
UNICODE_Sm, //U+2954
UNICODE_Sm, //U+2955
UNICODE_Sm, //U+2956
UNICODE_Sm, //U+2957
UNICODE_Sm, //U+2958
UNICODE_Sm, //U+2959
UNICODE_Sm, //U+295A
UNICODE_Sm, //U+295B
UNICODE_Sm, //U+295C
UNICODE_Sm, //U+295D
UNICODE_Sm, //U+295E
UNICODE_Sm, //U+295F
UNICODE_Sm, //U+2960
UNICODE_Sm, //U+2961
UNICODE_Sm, //U+2962
UNICODE_Sm, //U+2963
UNICODE_Sm, //U+2964
UNICODE_Sm, //U+2965
UNICODE_Sm, //U+2966
UNICODE_Sm, //U+2967
UNICODE_Sm, //U+2968
UNICODE_Sm, //U+2969
UNICODE_Sm, //U+296A
UNICODE_Sm, //U+296B
UNICODE_Sm, //U+296C
UNICODE_Sm, //U+296D
UNICODE_Sm, //U+296E
UNICODE_Sm, //U+296F
UNICODE_Sm, //U+2970
UNICODE_Sm, //U+2971
UNICODE_Sm, //U+2972
UNICODE_Sm, //U+2973
UNICODE_Sm, //U+2974
UNICODE_Sm, //U+2975
UNICODE_Sm, //U+2976
UNICODE_Sm, //U+2977
UNICODE_Sm, //U+2978
UNICODE_Sm, //U+2979
UNICODE_Sm, //U+297A
UNICODE_Sm, //U+297B
UNICODE_Sm, //U+297C
UNICODE_Sm, //U+297D
UNICODE_Sm, //U+297E
UNICODE_Sm, //U+297F
UNICODE_Sm, //U+2980
UNICODE_Sm, //U+2981
UNICODE_Sm, //U+2982
UNICODE_Ps, //U+2983
UNICODE_Pe, //U+2984
UNICODE_Ps, //U+2985
UNICODE_Pe, //U+2986
UNICODE_Ps, //U+2987
UNICODE_Pe, //U+2988
UNICODE_Ps, //U+2989
UNICODE_Pe, //U+298A
UNICODE_Ps, //U+298B
UNICODE_Pe, //U+298C
UNICODE_Ps, //U+298D
UNICODE_Pe, //U+298E
UNICODE_Ps, //U+298F
UNICODE_Pe, //U+2990
UNICODE_Ps, //U+2991
UNICODE_Pe, //U+2992
UNICODE_Ps, //U+2993
UNICODE_Pe, //U+2994
UNICODE_Ps, //U+2995
UNICODE_Pe, //U+2996
UNICODE_Ps, //U+2997
UNICODE_Pe, //U+2998
UNICODE_Sm, //U+2999
UNICODE_Sm, //U+299A
UNICODE_Sm, //U+299B
UNICODE_Sm, //U+299C
UNICODE_Sm, //U+299D
UNICODE_Sm, //U+299E
UNICODE_Sm, //U+299F
UNICODE_Sm, //U+29A0
UNICODE_Sm, //U+29A1
UNICODE_Sm, //U+29A2
UNICODE_Sm, //U+29A3
UNICODE_Sm, //U+29A4
UNICODE_Sm, //U+29A5
UNICODE_Sm, //U+29A6
UNICODE_Sm, //U+29A7
UNICODE_Sm, //U+29A8
UNICODE_Sm, //U+29A9
UNICODE_Sm, //U+29AA
UNICODE_Sm, //U+29AB
UNICODE_Sm, //U+29AC
UNICODE_Sm, //U+29AD
UNICODE_Sm, //U+29AE
UNICODE_Sm, //U+29AF
UNICODE_Sm, //U+29B0
UNICODE_Sm, //U+29B1
UNICODE_Sm, //U+29B2
UNICODE_Sm, //U+29B3
UNICODE_Sm, //U+29B4
UNICODE_Sm, //U+29B5
UNICODE_Sm, //U+29B6
UNICODE_Sm, //U+29B7
UNICODE_Sm, //U+29B8
UNICODE_Sm, //U+29B9
UNICODE_Sm, //U+29BA
UNICODE_Sm, //U+29BB
UNICODE_Sm, //U+29BC
UNICODE_Sm, //U+29BD
UNICODE_Sm, //U+29BE
UNICODE_Sm, //U+29BF
UNICODE_Sm, //U+29C0
UNICODE_Sm, //U+29C1
UNICODE_Sm, //U+29C2
UNICODE_Sm, //U+29C3
UNICODE_Sm, //U+29C4
UNICODE_Sm, //U+29C5
UNICODE_Sm, //U+29C6
UNICODE_Sm, //U+29C7
UNICODE_Sm, //U+29C8
UNICODE_Sm, //U+29C9
UNICODE_Sm, //U+29CA
UNICODE_Sm, //U+29CB
UNICODE_Sm, //U+29CC
UNICODE_Sm, //U+29CD
UNICODE_Sm, //U+29CE
UNICODE_Sm, //U+29CF
UNICODE_Sm, //U+29D0
UNICODE_Sm, //U+29D1
UNICODE_Sm, //U+29D2
UNICODE_Sm, //U+29D3
UNICODE_Sm, //U+29D4
UNICODE_Sm, //U+29D5
UNICODE_Sm, //U+29D6
UNICODE_Sm, //U+29D7
UNICODE_Ps, //U+29D8
UNICODE_Pe, //U+29D9
UNICODE_Ps, //U+29DA
UNICODE_Pe, //U+29DB
UNICODE_Sm, //U+29DC
UNICODE_Sm, //U+29DD
UNICODE_Sm, //U+29DE
UNICODE_Sm, //U+29DF
UNICODE_Sm, //U+29E0
UNICODE_Sm, //U+29E1
UNICODE_Sm, //U+29E2
UNICODE_Sm, //U+29E3
UNICODE_Sm, //U+29E4
UNICODE_Sm, //U+29E5
UNICODE_Sm, //U+29E6
UNICODE_Sm, //U+29E7
UNICODE_Sm, //U+29E8
UNICODE_Sm, //U+29E9
UNICODE_Sm, //U+29EA
UNICODE_Sm, //U+29EB
UNICODE_Sm, //U+29EC
UNICODE_Sm, //U+29ED
UNICODE_Sm, //U+29EE
UNICODE_Sm, //U+29EF
UNICODE_Sm, //U+29F0
UNICODE_Sm, //U+29F1
UNICODE_Sm, //U+29F2
UNICODE_Sm, //U+29F3
UNICODE_Sm, //U+29F4
UNICODE_Sm, //U+29F5
UNICODE_Sm, //U+29F6
UNICODE_Sm, //U+29F7
UNICODE_Sm, //U+29F8
UNICODE_Sm, //U+29F9
UNICODE_Sm, //U+29FA
UNICODE_Sm, //U+29FB
UNICODE_Ps, //U+29FC
UNICODE_Pe, //U+29FD
UNICODE_Sm, //U+29FE
UNICODE_Sm, //U+29FF
UNICODE_Sm, //U+2A00
UNICODE_Sm, //U+2A01
UNICODE_Sm, //U+2A02
UNICODE_Sm, //U+2A03
UNICODE_Sm, //U+2A04
UNICODE_Sm, //U+2A05
UNICODE_Sm, //U+2A06
UNICODE_Sm, //U+2A07
UNICODE_Sm, //U+2A08
UNICODE_Sm, //U+2A09
UNICODE_Sm, //U+2A0A
UNICODE_Sm, //U+2A0B
UNICODE_Sm, //U+2A0C
UNICODE_Sm, //U+2A0D
UNICODE_Sm, //U+2A0E
UNICODE_Sm, //U+2A0F
UNICODE_Sm, //U+2A10
UNICODE_Sm, //U+2A11
UNICODE_Sm, //U+2A12
UNICODE_Sm, //U+2A13
UNICODE_Sm, //U+2A14
UNICODE_Sm, //U+2A15
UNICODE_Sm, //U+2A16
UNICODE_Sm, //U+2A17
UNICODE_Sm, //U+2A18
UNICODE_Sm, //U+2A19
UNICODE_Sm, //U+2A1A
UNICODE_Sm, //U+2A1B
UNICODE_Sm, //U+2A1C
UNICODE_Sm, //U+2A1D
UNICODE_Sm, //U+2A1E
UNICODE_Sm, //U+2A1F
UNICODE_Sm, //U+2A20
UNICODE_Sm, //U+2A21
UNICODE_Sm, //U+2A22
UNICODE_Sm, //U+2A23
UNICODE_Sm, //U+2A24
UNICODE_Sm, //U+2A25
UNICODE_Sm, //U+2A26
UNICODE_Sm, //U+2A27
UNICODE_Sm, //U+2A28
UNICODE_Sm, //U+2A29
UNICODE_Sm, //U+2A2A
UNICODE_Sm, //U+2A2B
UNICODE_Sm, //U+2A2C
UNICODE_Sm, //U+2A2D
UNICODE_Sm, //U+2A2E
UNICODE_Sm, //U+2A2F
UNICODE_Sm, //U+2A30
UNICODE_Sm, //U+2A31
UNICODE_Sm, //U+2A32
UNICODE_Sm, //U+2A33
UNICODE_Sm, //U+2A34
UNICODE_Sm, //U+2A35
UNICODE_Sm, //U+2A36
UNICODE_Sm, //U+2A37
UNICODE_Sm, //U+2A38
UNICODE_Sm, //U+2A39
UNICODE_Sm, //U+2A3A
UNICODE_Sm, //U+2A3B
UNICODE_Sm, //U+2A3C
UNICODE_Sm, //U+2A3D
UNICODE_Sm, //U+2A3E
UNICODE_Sm, //U+2A3F
UNICODE_Sm, //U+2A40
UNICODE_Sm, //U+2A41
UNICODE_Sm, //U+2A42
UNICODE_Sm, //U+2A43
UNICODE_Sm, //U+2A44
UNICODE_Sm, //U+2A45
UNICODE_Sm, //U+2A46
UNICODE_Sm, //U+2A47
UNICODE_Sm, //U+2A48
UNICODE_Sm, //U+2A49
UNICODE_Sm, //U+2A4A
UNICODE_Sm, //U+2A4B
UNICODE_Sm, //U+2A4C
UNICODE_Sm, //U+2A4D
UNICODE_Sm, //U+2A4E
UNICODE_Sm, //U+2A4F
UNICODE_Sm, //U+2A50
UNICODE_Sm, //U+2A51
UNICODE_Sm, //U+2A52
UNICODE_Sm, //U+2A53
UNICODE_Sm, //U+2A54
UNICODE_Sm, //U+2A55
UNICODE_Sm, //U+2A56
UNICODE_Sm, //U+2A57
UNICODE_Sm, //U+2A58
UNICODE_Sm, //U+2A59
UNICODE_Sm, //U+2A5A
UNICODE_Sm, //U+2A5B
UNICODE_Sm, //U+2A5C
UNICODE_Sm, //U+2A5D
UNICODE_Sm, //U+2A5E
UNICODE_Sm, //U+2A5F
UNICODE_Sm, //U+2A60
UNICODE_Sm, //U+2A61
UNICODE_Sm, //U+2A62
UNICODE_Sm, //U+2A63
UNICODE_Sm, //U+2A64
UNICODE_Sm, //U+2A65
UNICODE_Sm, //U+2A66
UNICODE_Sm, //U+2A67
UNICODE_Sm, //U+2A68
UNICODE_Sm, //U+2A69
UNICODE_Sm, //U+2A6A
UNICODE_Sm, //U+2A6B
UNICODE_Sm, //U+2A6C
UNICODE_Sm, //U+2A6D
UNICODE_Sm, //U+2A6E
UNICODE_Sm, //U+2A6F
UNICODE_Sm, //U+2A70
UNICODE_Sm, //U+2A71
UNICODE_Sm, //U+2A72
UNICODE_Sm, //U+2A73
UNICODE_Sm, //U+2A74
UNICODE_Sm, //U+2A75
UNICODE_Sm, //U+2A76
UNICODE_Sm, //U+2A77
UNICODE_Sm, //U+2A78
UNICODE_Sm, //U+2A79
UNICODE_Sm, //U+2A7A
UNICODE_Sm, //U+2A7B
UNICODE_Sm, //U+2A7C
UNICODE_Sm, //U+2A7D
UNICODE_Sm, //U+2A7E
UNICODE_Sm, //U+2A7F
UNICODE_Sm, //U+2A80
UNICODE_Sm, //U+2A81
UNICODE_Sm, //U+2A82
UNICODE_Sm, //U+2A83
UNICODE_Sm, //U+2A84
UNICODE_Sm, //U+2A85
UNICODE_Sm, //U+2A86
UNICODE_Sm, //U+2A87
UNICODE_Sm, //U+2A88
UNICODE_Sm, //U+2A89
UNICODE_Sm, //U+2A8A
UNICODE_Sm, //U+2A8B
UNICODE_Sm, //U+2A8C
UNICODE_Sm, //U+2A8D
UNICODE_Sm, //U+2A8E
UNICODE_Sm, //U+2A8F
UNICODE_Sm, //U+2A90
UNICODE_Sm, //U+2A91
UNICODE_Sm, //U+2A92
UNICODE_Sm, //U+2A93
UNICODE_Sm, //U+2A94
UNICODE_Sm, //U+2A95
UNICODE_Sm, //U+2A96
UNICODE_Sm, //U+2A97
UNICODE_Sm, //U+2A98
UNICODE_Sm, //U+2A99
UNICODE_Sm, //U+2A9A
UNICODE_Sm, //U+2A9B
UNICODE_Sm, //U+2A9C
UNICODE_Sm, //U+2A9D
UNICODE_Sm, //U+2A9E
UNICODE_Sm, //U+2A9F
UNICODE_Sm, //U+2AA0
UNICODE_Sm, //U+2AA1
UNICODE_Sm, //U+2AA2
UNICODE_Sm, //U+2AA3
UNICODE_Sm, //U+2AA4
UNICODE_Sm, //U+2AA5
UNICODE_Sm, //U+2AA6
UNICODE_Sm, //U+2AA7
UNICODE_Sm, //U+2AA8
UNICODE_Sm, //U+2AA9
UNICODE_Sm, //U+2AAA
UNICODE_Sm, //U+2AAB
UNICODE_Sm, //U+2AAC
UNICODE_Sm, //U+2AAD
UNICODE_Sm, //U+2AAE
UNICODE_Sm, //U+2AAF
UNICODE_Sm, //U+2AB0
UNICODE_Sm, //U+2AB1
UNICODE_Sm, //U+2AB2
UNICODE_Sm, //U+2AB3
UNICODE_Sm, //U+2AB4
UNICODE_Sm, //U+2AB5
UNICODE_Sm, //U+2AB6
UNICODE_Sm, //U+2AB7
UNICODE_Sm, //U+2AB8
UNICODE_Sm, //U+2AB9
UNICODE_Sm, //U+2ABA
UNICODE_Sm, //U+2ABB
UNICODE_Sm, //U+2ABC
UNICODE_Sm, //U+2ABD
UNICODE_Sm, //U+2ABE
UNICODE_Sm, //U+2ABF
UNICODE_Sm, //U+2AC0
UNICODE_Sm, //U+2AC1
UNICODE_Sm, //U+2AC2
UNICODE_Sm, //U+2AC3
UNICODE_Sm, //U+2AC4
UNICODE_Sm, //U+2AC5
UNICODE_Sm, //U+2AC6
UNICODE_Sm, //U+2AC7
UNICODE_Sm, //U+2AC8
UNICODE_Sm, //U+2AC9
UNICODE_Sm, //U+2ACA
UNICODE_Sm, //U+2ACB
UNICODE_Sm, //U+2ACC
UNICODE_Sm, //U+2ACD
UNICODE_Sm, //U+2ACE
UNICODE_Sm, //U+2ACF
UNICODE_Sm, //U+2AD0
UNICODE_Sm, //U+2AD1
UNICODE_Sm, //U+2AD2
UNICODE_Sm, //U+2AD3
UNICODE_Sm, //U+2AD4
UNICODE_Sm, //U+2AD5
UNICODE_Sm, //U+2AD6
UNICODE_Sm, //U+2AD7
UNICODE_Sm, //U+2AD8
UNICODE_Sm, //U+2AD9
UNICODE_Sm, //U+2ADA
UNICODE_Sm, //U+2ADB
UNICODE_Sm, //U+2ADC
UNICODE_Sm, //U+2ADD
UNICODE_Sm, //U+2ADE
UNICODE_Sm, //U+2ADF
UNICODE_Sm, //U+2AE0
UNICODE_Sm, //U+2AE1
UNICODE_Sm, //U+2AE2
UNICODE_Sm, //U+2AE3
UNICODE_Sm, //U+2AE4
UNICODE_Sm, //U+2AE5
UNICODE_Sm, //U+2AE6
UNICODE_Sm, //U+2AE7
UNICODE_Sm, //U+2AE8
UNICODE_Sm, //U+2AE9
UNICODE_Sm, //U+2AEA
UNICODE_Sm, //U+2AEB
UNICODE_Sm, //U+2AEC
UNICODE_Sm, //U+2AED
UNICODE_Sm, //U+2AEE
UNICODE_Sm, //U+2AEF
UNICODE_Sm, //U+2AF0
UNICODE_Sm, //U+2AF1
UNICODE_Sm, //U+2AF2
UNICODE_Sm, //U+2AF3
UNICODE_Sm, //U+2AF4
UNICODE_Sm, //U+2AF5
UNICODE_Sm, //U+2AF6
UNICODE_Sm, //U+2AF7
UNICODE_Sm, //U+2AF8
UNICODE_Sm, //U+2AF9
UNICODE_Sm, //U+2AFA
UNICODE_Sm, //U+2AFB
UNICODE_Sm, //U+2AFC
UNICODE_Sm, //U+2AFD
UNICODE_Sm, //U+2AFE
UNICODE_Sm, //U+2AFF
UNICODE_So, //U+2B00
UNICODE_So, //U+2B01
UNICODE_So, //U+2B02
UNICODE_So, //U+2B03
UNICODE_So, //U+2B04
UNICODE_So, //U+2B05
UNICODE_So, //U+2B06
UNICODE_So, //U+2B07
UNICODE_So, //U+2B08
UNICODE_So, //U+2B09
UNICODE_So, //U+2B0A
UNICODE_So, //U+2B0B
UNICODE_So, //U+2B0C
UNICODE_So, //U+2B0D
UNICODE_So, //U+2B0E
UNICODE_So, //U+2B0F
UNICODE_So, //U+2B10
UNICODE_So, //U+2B11
UNICODE_So, //U+2B12
UNICODE_So, //U+2B13
UNICODE_So, //U+2B14
UNICODE_So, //U+2B15
UNICODE_So, //U+2B16
UNICODE_So, //U+2B17
UNICODE_So, //U+2B18
UNICODE_So, //U+2B19
UNICODE_So, //U+2B1A
UNICODE_So, //U+2B1B
UNICODE_So, //U+2B1C
UNICODE_So, //U+2B1D
UNICODE_So, //U+2B1E
UNICODE_So, //U+2B1F
UNICODE_So, //U+2B20
UNICODE_So, //U+2B21
UNICODE_So, //U+2B22
UNICODE_So, //U+2B23
UNICODE_So, //U+2B24
UNICODE_So, //U+2B25
UNICODE_So, //U+2B26
UNICODE_So, //U+2B27
UNICODE_So, //U+2B28
UNICODE_So, //U+2B29
UNICODE_So, //U+2B2A
UNICODE_So, //U+2B2B
UNICODE_So, //U+2B2C
UNICODE_So, //U+2B2D
UNICODE_So, //U+2B2E
UNICODE_So, //U+2B2F
UNICODE_Sm, //U+2B30
UNICODE_Sm, //U+2B31
UNICODE_Sm, //U+2B32
UNICODE_Sm, //U+2B33
UNICODE_Sm, //U+2B34
UNICODE_Sm, //U+2B35
UNICODE_Sm, //U+2B36
UNICODE_Sm, //U+2B37
UNICODE_Sm, //U+2B38
UNICODE_Sm, //U+2B39
UNICODE_Sm, //U+2B3A
UNICODE_Sm, //U+2B3B
UNICODE_Sm, //U+2B3C
UNICODE_Sm, //U+2B3D
UNICODE_Sm, //U+2B3E
UNICODE_Sm, //U+2B3F
UNICODE_Sm, //U+2B40
UNICODE_Sm, //U+2B41
UNICODE_Sm, //U+2B42
UNICODE_Sm, //U+2B43
UNICODE_Sm, //U+2B44
UNICODE_So, //U+2B45
UNICODE_So, //U+2B46
UNICODE_Sm, //U+2B47
UNICODE_Sm, //U+2B48
UNICODE_Sm, //U+2B49
UNICODE_Sm, //U+2B4A
UNICODE_Sm, //U+2B4B
UNICODE_Sm, //U+2B4C
0, //U+2B4D
0, //U+2B4E
0, //U+2B4F
UNICODE_So, //U+2B50
UNICODE_So, //U+2B51
UNICODE_So, //U+2B52
UNICODE_So, //U+2B53
UNICODE_So, //U+2B54
UNICODE_So, //U+2B55
UNICODE_So, //U+2B56
UNICODE_So, //U+2B57
UNICODE_So, //U+2B58
UNICODE_So, //U+2B59
0, //U+2B5A
0, //U+2B5B
0, //U+2B5C
0, //U+2B5D
0, //U+2B5E
0, //U+2B5F
0, //U+2B60
0, //U+2B61
0, //U+2B62
0, //U+2B63
0, //U+2B64
0, //U+2B65
0, //U+2B66
0, //U+2B67
0, //U+2B68
0, //U+2B69
0, //U+2B6A
0, //U+2B6B
0, //U+2B6C
0, //U+2B6D
0, //U+2B6E
0, //U+2B6F
0, //U+2B70
0, //U+2B71
0, //U+2B72
0, //U+2B73
0, //U+2B74
0, //U+2B75
0, //U+2B76
0, //U+2B77
0, //U+2B78
0, //U+2B79
0, //U+2B7A
0, //U+2B7B
0, //U+2B7C
0, //U+2B7D
0, //U+2B7E
0, //U+2B7F
0, //U+2B80
0, //U+2B81
0, //U+2B82
0, //U+2B83
0, //U+2B84
0, //U+2B85
0, //U+2B86
0, //U+2B87
0, //U+2B88
0, //U+2B89
0, //U+2B8A
0, //U+2B8B
0, //U+2B8C
0, //U+2B8D
0, //U+2B8E
0, //U+2B8F
0, //U+2B90
0, //U+2B91
0, //U+2B92
0, //U+2B93
0, //U+2B94
0, //U+2B95
0, //U+2B96
0, //U+2B97
0, //U+2B98
0, //U+2B99
0, //U+2B9A
0, //U+2B9B
0, //U+2B9C
0, //U+2B9D
0, //U+2B9E
0, //U+2B9F
0, //U+2BA0
0, //U+2BA1
0, //U+2BA2
0, //U+2BA3
0, //U+2BA4
0, //U+2BA5
0, //U+2BA6
0, //U+2BA7
0, //U+2BA8
0, //U+2BA9
0, //U+2BAA
0, //U+2BAB
0, //U+2BAC
0, //U+2BAD
0, //U+2BAE
0, //U+2BAF
0, //U+2BB0
0, //U+2BB1
0, //U+2BB2
0, //U+2BB3
0, //U+2BB4
0, //U+2BB5
0, //U+2BB6
0, //U+2BB7
0, //U+2BB8
0, //U+2BB9
0, //U+2BBA
0, //U+2BBB
0, //U+2BBC
0, //U+2BBD
0, //U+2BBE
0, //U+2BBF
0, //U+2BC0
0, //U+2BC1
0, //U+2BC2
0, //U+2BC3
0, //U+2BC4
0, //U+2BC5
0, //U+2BC6
0, //U+2BC7
0, //U+2BC8
0, //U+2BC9
0, //U+2BCA
0, //U+2BCB
0, //U+2BCC
0, //U+2BCD
0, //U+2BCE
0, //U+2BCF
0, //U+2BD0
0, //U+2BD1
0, //U+2BD2
0, //U+2BD3
0, //U+2BD4
0, //U+2BD5
0, //U+2BD6
0, //U+2BD7
0, //U+2BD8
0, //U+2BD9
0, //U+2BDA
0, //U+2BDB
0, //U+2BDC
0, //U+2BDD
0, //U+2BDE
0, //U+2BDF
0, //U+2BE0
0, //U+2BE1
0, //U+2BE2
0, //U+2BE3
0, //U+2BE4
0, //U+2BE5
0, //U+2BE6
0, //U+2BE7
0, //U+2BE8
0, //U+2BE9
0, //U+2BEA
0, //U+2BEB
0, //U+2BEC
0, //U+2BED
0, //U+2BEE
0, //U+2BEF
0, //U+2BF0
0, //U+2BF1
0, //U+2BF2
0, //U+2BF3
0, //U+2BF4
0, //U+2BF5
0, //U+2BF6
0, //U+2BF7
0, //U+2BF8
0, //U+2BF9
0, //U+2BFA
0, //U+2BFB
0, //U+2BFC
0, //U+2BFD
0, //U+2BFE
0, //U+2BFF
UNICODE_Lu, //U+2C00
UNICODE_Lu, //U+2C01
UNICODE_Lu, //U+2C02
UNICODE_Lu, //U+2C03
UNICODE_Lu, //U+2C04
UNICODE_Lu, //U+2C05
UNICODE_Lu, //U+2C06
UNICODE_Lu, //U+2C07
UNICODE_Lu, //U+2C08
UNICODE_Lu, //U+2C09
UNICODE_Lu, //U+2C0A
UNICODE_Lu, //U+2C0B
UNICODE_Lu, //U+2C0C
UNICODE_Lu, //U+2C0D
UNICODE_Lu, //U+2C0E
UNICODE_Lu, //U+2C0F
UNICODE_Lu, //U+2C10
UNICODE_Lu, //U+2C11
UNICODE_Lu, //U+2C12
UNICODE_Lu, //U+2C13
UNICODE_Lu, //U+2C14
UNICODE_Lu, //U+2C15
UNICODE_Lu, //U+2C16
UNICODE_Lu, //U+2C17
UNICODE_Lu, //U+2C18
UNICODE_Lu, //U+2C19
UNICODE_Lu, //U+2C1A
UNICODE_Lu, //U+2C1B
UNICODE_Lu, //U+2C1C
UNICODE_Lu, //U+2C1D
UNICODE_Lu, //U+2C1E
UNICODE_Lu, //U+2C1F
UNICODE_Lu, //U+2C20
UNICODE_Lu, //U+2C21
UNICODE_Lu, //U+2C22
UNICODE_Lu, //U+2C23
UNICODE_Lu, //U+2C24
UNICODE_Lu, //U+2C25
UNICODE_Lu, //U+2C26
UNICODE_Lu, //U+2C27
UNICODE_Lu, //U+2C28
UNICODE_Lu, //U+2C29
UNICODE_Lu, //U+2C2A
UNICODE_Lu, //U+2C2B
UNICODE_Lu, //U+2C2C
UNICODE_Lu, //U+2C2D
UNICODE_Lu, //U+2C2E
0, //U+2C2F
UNICODE_Ll, //U+2C30
UNICODE_Ll, //U+2C31
UNICODE_Ll, //U+2C32
UNICODE_Ll, //U+2C33
UNICODE_Ll, //U+2C34
UNICODE_Ll, //U+2C35
UNICODE_Ll, //U+2C36
UNICODE_Ll, //U+2C37
UNICODE_Ll, //U+2C38
UNICODE_Ll, //U+2C39
UNICODE_Ll, //U+2C3A
UNICODE_Ll, //U+2C3B
UNICODE_Ll, //U+2C3C
UNICODE_Ll, //U+2C3D
UNICODE_Ll, //U+2C3E
UNICODE_Ll, //U+2C3F
UNICODE_Ll, //U+2C40
UNICODE_Ll, //U+2C41
UNICODE_Ll, //U+2C42
UNICODE_Ll, //U+2C43
UNICODE_Ll, //U+2C44
UNICODE_Ll, //U+2C45
UNICODE_Ll, //U+2C46
UNICODE_Ll, //U+2C47
UNICODE_Ll, //U+2C48
UNICODE_Ll, //U+2C49
UNICODE_Ll, //U+2C4A
UNICODE_Ll, //U+2C4B
UNICODE_Ll, //U+2C4C
UNICODE_Ll, //U+2C4D
UNICODE_Ll, //U+2C4E
UNICODE_Ll, //U+2C4F
UNICODE_Ll, //U+2C50
UNICODE_Ll, //U+2C51
UNICODE_Ll, //U+2C52
UNICODE_Ll, //U+2C53
UNICODE_Ll, //U+2C54
UNICODE_Ll, //U+2C55
UNICODE_Ll, //U+2C56
UNICODE_Ll, //U+2C57
UNICODE_Ll, //U+2C58
UNICODE_Ll, //U+2C59
UNICODE_Ll, //U+2C5A
UNICODE_Ll, //U+2C5B
UNICODE_Ll, //U+2C5C
UNICODE_Ll, //U+2C5D
UNICODE_Ll, //U+2C5E
0, //U+2C5F
UNICODE_Lu, //U+2C60
UNICODE_Ll, //U+2C61
UNICODE_Lu, //U+2C62
UNICODE_Lu, //U+2C63
UNICODE_Lu, //U+2C64
UNICODE_Ll, //U+2C65
UNICODE_Ll, //U+2C66
UNICODE_Lu, //U+2C67
UNICODE_Ll, //U+2C68
UNICODE_Lu, //U+2C69
UNICODE_Ll, //U+2C6A
UNICODE_Lu, //U+2C6B
UNICODE_Ll, //U+2C6C
UNICODE_Lu, //U+2C6D
UNICODE_Lu, //U+2C6E
UNICODE_Lu, //U+2C6F
UNICODE_Lu, //U+2C70
UNICODE_Ll, //U+2C71
UNICODE_Lu, //U+2C72
UNICODE_Ll, //U+2C73
UNICODE_Ll, //U+2C74
UNICODE_Lu, //U+2C75
UNICODE_Ll, //U+2C76
UNICODE_Ll, //U+2C77
UNICODE_Ll, //U+2C78
UNICODE_Ll, //U+2C79
UNICODE_Ll, //U+2C7A
UNICODE_Ll, //U+2C7B
UNICODE_Ll, //U+2C7C
UNICODE_Lm, //U+2C7D
UNICODE_Lu, //U+2C7E
UNICODE_Lu, //U+2C7F
UNICODE_Lu, //U+2C80
UNICODE_Ll, //U+2C81
UNICODE_Lu, //U+2C82
UNICODE_Ll, //U+2C83
UNICODE_Lu, //U+2C84
UNICODE_Ll, //U+2C85
UNICODE_Lu, //U+2C86
UNICODE_Ll, //U+2C87
UNICODE_Lu, //U+2C88
UNICODE_Ll, //U+2C89
UNICODE_Lu, //U+2C8A
UNICODE_Ll, //U+2C8B
UNICODE_Lu, //U+2C8C
UNICODE_Ll, //U+2C8D
UNICODE_Lu, //U+2C8E
UNICODE_Ll, //U+2C8F
UNICODE_Lu, //U+2C90
UNICODE_Ll, //U+2C91
UNICODE_Lu, //U+2C92
UNICODE_Ll, //U+2C93
UNICODE_Lu, //U+2C94
UNICODE_Ll, //U+2C95
UNICODE_Lu, //U+2C96
UNICODE_Ll, //U+2C97
UNICODE_Lu, //U+2C98
UNICODE_Ll, //U+2C99
UNICODE_Lu, //U+2C9A
UNICODE_Ll, //U+2C9B
UNICODE_Lu, //U+2C9C
UNICODE_Ll, //U+2C9D
UNICODE_Lu, //U+2C9E
UNICODE_Ll, //U+2C9F
UNICODE_Lu, //U+2CA0
UNICODE_Ll, //U+2CA1
UNICODE_Lu, //U+2CA2
UNICODE_Ll, //U+2CA3
UNICODE_Lu, //U+2CA4
UNICODE_Ll, //U+2CA5
UNICODE_Lu, //U+2CA6
UNICODE_Ll, //U+2CA7
UNICODE_Lu, //U+2CA8
UNICODE_Ll, //U+2CA9
UNICODE_Lu, //U+2CAA
UNICODE_Ll, //U+2CAB
UNICODE_Lu, //U+2CAC
UNICODE_Ll, //U+2CAD
UNICODE_Lu, //U+2CAE
UNICODE_Ll, //U+2CAF
UNICODE_Lu, //U+2CB0
UNICODE_Ll, //U+2CB1
UNICODE_Lu, //U+2CB2
UNICODE_Ll, //U+2CB3
UNICODE_Lu, //U+2CB4
UNICODE_Ll, //U+2CB5
UNICODE_Lu, //U+2CB6
UNICODE_Ll, //U+2CB7
UNICODE_Lu, //U+2CB8
UNICODE_Ll, //U+2CB9
UNICODE_Lu, //U+2CBA
UNICODE_Ll, //U+2CBB
UNICODE_Lu, //U+2CBC
UNICODE_Ll, //U+2CBD
UNICODE_Lu, //U+2CBE
UNICODE_Ll, //U+2CBF
UNICODE_Lu, //U+2CC0
UNICODE_Ll, //U+2CC1
UNICODE_Lu, //U+2CC2
UNICODE_Ll, //U+2CC3
UNICODE_Lu, //U+2CC4
UNICODE_Ll, //U+2CC5
UNICODE_Lu, //U+2CC6
UNICODE_Ll, //U+2CC7
UNICODE_Lu, //U+2CC8
UNICODE_Ll, //U+2CC9
UNICODE_Lu, //U+2CCA
UNICODE_Ll, //U+2CCB
UNICODE_Lu, //U+2CCC
UNICODE_Ll, //U+2CCD
UNICODE_Lu, //U+2CCE
UNICODE_Ll, //U+2CCF
UNICODE_Lu, //U+2CD0
UNICODE_Ll, //U+2CD1
UNICODE_Lu, //U+2CD2
UNICODE_Ll, //U+2CD3
UNICODE_Lu, //U+2CD4
UNICODE_Ll, //U+2CD5
UNICODE_Lu, //U+2CD6
UNICODE_Ll, //U+2CD7
UNICODE_Lu, //U+2CD8
UNICODE_Ll, //U+2CD9
UNICODE_Lu, //U+2CDA
UNICODE_Ll, //U+2CDB
UNICODE_Lu, //U+2CDC
UNICODE_Ll, //U+2CDD
UNICODE_Lu, //U+2CDE
UNICODE_Ll, //U+2CDF
UNICODE_Lu, //U+2CE0
UNICODE_Ll, //U+2CE1
UNICODE_Lu, //U+2CE2
UNICODE_Ll, //U+2CE3
UNICODE_Ll, //U+2CE4
UNICODE_So, //U+2CE5
UNICODE_So, //U+2CE6
UNICODE_So, //U+2CE7
UNICODE_So, //U+2CE8
UNICODE_So, //U+2CE9
UNICODE_So, //U+2CEA
UNICODE_Lu, //U+2CEB
UNICODE_Ll, //U+2CEC
UNICODE_Lu, //U+2CED
UNICODE_Ll, //U+2CEE
UNICODE_Mn, //U+2CEF
UNICODE_Mn, //U+2CF0
UNICODE_Mn, //U+2CF1
0, //U+2CF2
0, //U+2CF3
0, //U+2CF4
0, //U+2CF5
0, //U+2CF6
0, //U+2CF7
0, //U+2CF8
UNICODE_Po, //U+2CF9
UNICODE_Po, //U+2CFA
UNICODE_Po, //U+2CFB
UNICODE_Po, //U+2CFC
UNICODE_No, //U+2CFD
UNICODE_Po, //U+2CFE
UNICODE_Po, //U+2CFF
UNICODE_Ll, //U+2D00
UNICODE_Ll, //U+2D01
UNICODE_Ll, //U+2D02
UNICODE_Ll, //U+2D03
UNICODE_Ll, //U+2D04
UNICODE_Ll, //U+2D05
UNICODE_Ll, //U+2D06
UNICODE_Ll, //U+2D07
UNICODE_Ll, //U+2D08
UNICODE_Ll, //U+2D09
UNICODE_Ll, //U+2D0A
UNICODE_Ll, //U+2D0B
UNICODE_Ll, //U+2D0C
UNICODE_Ll, //U+2D0D
UNICODE_Ll, //U+2D0E
UNICODE_Ll, //U+2D0F
UNICODE_Ll, //U+2D10
UNICODE_Ll, //U+2D11
UNICODE_Ll, //U+2D12
UNICODE_Ll, //U+2D13
UNICODE_Ll, //U+2D14
UNICODE_Ll, //U+2D15
UNICODE_Ll, //U+2D16
UNICODE_Ll, //U+2D17
UNICODE_Ll, //U+2D18
UNICODE_Ll, //U+2D19
UNICODE_Ll, //U+2D1A
UNICODE_Ll, //U+2D1B
UNICODE_Ll, //U+2D1C
UNICODE_Ll, //U+2D1D
UNICODE_Ll, //U+2D1E
UNICODE_Ll, //U+2D1F
UNICODE_Ll, //U+2D20
UNICODE_Ll, //U+2D21
UNICODE_Ll, //U+2D22
UNICODE_Ll, //U+2D23
UNICODE_Ll, //U+2D24
UNICODE_Ll, //U+2D25
0, //U+2D26
0, //U+2D27
0, //U+2D28
0, //U+2D29
0, //U+2D2A
0, //U+2D2B
0, //U+2D2C
0, //U+2D2D
0, //U+2D2E
0, //U+2D2F
UNICODE_Lo, //U+2D30
UNICODE_Lo, //U+2D31
UNICODE_Lo, //U+2D32
UNICODE_Lo, //U+2D33
UNICODE_Lo, //U+2D34
UNICODE_Lo, //U+2D35
UNICODE_Lo, //U+2D36
UNICODE_Lo, //U+2D37
UNICODE_Lo, //U+2D38
UNICODE_Lo, //U+2D39
UNICODE_Lo, //U+2D3A
UNICODE_Lo, //U+2D3B
UNICODE_Lo, //U+2D3C
UNICODE_Lo, //U+2D3D
UNICODE_Lo, //U+2D3E
UNICODE_Lo, //U+2D3F
UNICODE_Lo, //U+2D40
UNICODE_Lo, //U+2D41
UNICODE_Lo, //U+2D42
UNICODE_Lo, //U+2D43
UNICODE_Lo, //U+2D44
UNICODE_Lo, //U+2D45
UNICODE_Lo, //U+2D46
UNICODE_Lo, //U+2D47
UNICODE_Lo, //U+2D48
UNICODE_Lo, //U+2D49
UNICODE_Lo, //U+2D4A
UNICODE_Lo, //U+2D4B
UNICODE_Lo, //U+2D4C
UNICODE_Lo, //U+2D4D
UNICODE_Lo, //U+2D4E
UNICODE_Lo, //U+2D4F
UNICODE_Lo, //U+2D50
UNICODE_Lo, //U+2D51
UNICODE_Lo, //U+2D52
UNICODE_Lo, //U+2D53
UNICODE_Lo, //U+2D54
UNICODE_Lo, //U+2D55
UNICODE_Lo, //U+2D56
UNICODE_Lo, //U+2D57
UNICODE_Lo, //U+2D58
UNICODE_Lo, //U+2D59
UNICODE_Lo, //U+2D5A
UNICODE_Lo, //U+2D5B
UNICODE_Lo, //U+2D5C
UNICODE_Lo, //U+2D5D
UNICODE_Lo, //U+2D5E
UNICODE_Lo, //U+2D5F
UNICODE_Lo, //U+2D60
UNICODE_Lo, //U+2D61
UNICODE_Lo, //U+2D62
UNICODE_Lo, //U+2D63
UNICODE_Lo, //U+2D64
UNICODE_Lo, //U+2D65
0, //U+2D66
0, //U+2D67
0, //U+2D68
0, //U+2D69
0, //U+2D6A
0, //U+2D6B
0, //U+2D6C
0, //U+2D6D
0, //U+2D6E
UNICODE_Lm, //U+2D6F
UNICODE_Po, //U+2D70
0, //U+2D71
0, //U+2D72
0, //U+2D73
0, //U+2D74
0, //U+2D75
0, //U+2D76
0, //U+2D77
0, //U+2D78
0, //U+2D79
0, //U+2D7A
0, //U+2D7B
0, //U+2D7C
0, //U+2D7D
0, //U+2D7E
UNICODE_Mn, //U+2D7F
UNICODE_Lo, //U+2D80
UNICODE_Lo, //U+2D81
UNICODE_Lo, //U+2D82
UNICODE_Lo, //U+2D83
UNICODE_Lo, //U+2D84
UNICODE_Lo, //U+2D85
UNICODE_Lo, //U+2D86
UNICODE_Lo, //U+2D87
UNICODE_Lo, //U+2D88
UNICODE_Lo, //U+2D89
UNICODE_Lo, //U+2D8A
UNICODE_Lo, //U+2D8B
UNICODE_Lo, //U+2D8C
UNICODE_Lo, //U+2D8D
UNICODE_Lo, //U+2D8E
UNICODE_Lo, //U+2D8F
UNICODE_Lo, //U+2D90
UNICODE_Lo, //U+2D91
UNICODE_Lo, //U+2D92
UNICODE_Lo, //U+2D93
UNICODE_Lo, //U+2D94
UNICODE_Lo, //U+2D95
UNICODE_Lo, //U+2D96
0, //U+2D97
0, //U+2D98
0, //U+2D99
0, //U+2D9A
0, //U+2D9B
0, //U+2D9C
0, //U+2D9D
0, //U+2D9E
0, //U+2D9F
UNICODE_Lo, //U+2DA0
UNICODE_Lo, //U+2DA1
UNICODE_Lo, //U+2DA2
UNICODE_Lo, //U+2DA3
UNICODE_Lo, //U+2DA4
UNICODE_Lo, //U+2DA5
UNICODE_Lo, //U+2DA6
0, //U+2DA7
UNICODE_Lo, //U+2DA8
UNICODE_Lo, //U+2DA9
UNICODE_Lo, //U+2DAA
UNICODE_Lo, //U+2DAB
UNICODE_Lo, //U+2DAC
UNICODE_Lo, //U+2DAD
UNICODE_Lo, //U+2DAE
0, //U+2DAF
UNICODE_Lo, //U+2DB0
UNICODE_Lo, //U+2DB1
UNICODE_Lo, //U+2DB2
UNICODE_Lo, //U+2DB3
UNICODE_Lo, //U+2DB4
UNICODE_Lo, //U+2DB5
UNICODE_Lo, //U+2DB6
0, //U+2DB7
UNICODE_Lo, //U+2DB8
UNICODE_Lo, //U+2DB9
UNICODE_Lo, //U+2DBA
UNICODE_Lo, //U+2DBB
UNICODE_Lo, //U+2DBC
UNICODE_Lo, //U+2DBD
UNICODE_Lo, //U+2DBE
0, //U+2DBF
UNICODE_Lo, //U+2DC0
UNICODE_Lo, //U+2DC1
UNICODE_Lo, //U+2DC2
UNICODE_Lo, //U+2DC3
UNICODE_Lo, //U+2DC4
UNICODE_Lo, //U+2DC5
UNICODE_Lo, //U+2DC6
0, //U+2DC7
UNICODE_Lo, //U+2DC8
UNICODE_Lo, //U+2DC9
UNICODE_Lo, //U+2DCA
UNICODE_Lo, //U+2DCB
UNICODE_Lo, //U+2DCC
UNICODE_Lo, //U+2DCD
UNICODE_Lo, //U+2DCE
0, //U+2DCF
UNICODE_Lo, //U+2DD0
UNICODE_Lo, //U+2DD1
UNICODE_Lo, //U+2DD2
UNICODE_Lo, //U+2DD3
UNICODE_Lo, //U+2DD4
UNICODE_Lo, //U+2DD5
UNICODE_Lo, //U+2DD6
0, //U+2DD7
UNICODE_Lo, //U+2DD8
UNICODE_Lo, //U+2DD9
UNICODE_Lo, //U+2DDA
UNICODE_Lo, //U+2DDB
UNICODE_Lo, //U+2DDC
UNICODE_Lo, //U+2DDD
UNICODE_Lo, //U+2DDE
0, //U+2DDF
UNICODE_Mn, //U+2DE0
UNICODE_Mn, //U+2DE1
UNICODE_Mn, //U+2DE2
UNICODE_Mn, //U+2DE3
UNICODE_Mn, //U+2DE4
UNICODE_Mn, //U+2DE5
UNICODE_Mn, //U+2DE6
UNICODE_Mn, //U+2DE7
UNICODE_Mn, //U+2DE8
UNICODE_Mn, //U+2DE9
UNICODE_Mn, //U+2DEA
UNICODE_Mn, //U+2DEB
UNICODE_Mn, //U+2DEC
UNICODE_Mn, //U+2DED
UNICODE_Mn, //U+2DEE
UNICODE_Mn, //U+2DEF
UNICODE_Mn, //U+2DF0
UNICODE_Mn, //U+2DF1
UNICODE_Mn, //U+2DF2
UNICODE_Mn, //U+2DF3
UNICODE_Mn, //U+2DF4
UNICODE_Mn, //U+2DF5
UNICODE_Mn, //U+2DF6
UNICODE_Mn, //U+2DF7
UNICODE_Mn, //U+2DF8
UNICODE_Mn, //U+2DF9
UNICODE_Mn, //U+2DFA
UNICODE_Mn, //U+2DFB
UNICODE_Mn, //U+2DFC
UNICODE_Mn, //U+2DFD
UNICODE_Mn, //U+2DFE
UNICODE_Mn, //U+2DFF
UNICODE_Po, //U+2E00
UNICODE_Po, //U+2E01
UNICODE_Pi, //U+2E02
UNICODE_Pf, //U+2E03
UNICODE_Pi, //U+2E04
UNICODE_Pf, //U+2E05
UNICODE_Po, //U+2E06
UNICODE_Po, //U+2E07
UNICODE_Po, //U+2E08
UNICODE_Pi, //U+2E09
UNICODE_Pf, //U+2E0A
UNICODE_Po, //U+2E0B
UNICODE_Pi, //U+2E0C
UNICODE_Pf, //U+2E0D
UNICODE_Po, //U+2E0E
UNICODE_Po, //U+2E0F
UNICODE_Po, //U+2E10
UNICODE_Po, //U+2E11
UNICODE_Po, //U+2E12
UNICODE_Po, //U+2E13
UNICODE_Po, //U+2E14
UNICODE_Po, //U+2E15
UNICODE_Po, //U+2E16
UNICODE_Pd, //U+2E17
UNICODE_Po, //U+2E18
UNICODE_Po, //U+2E19
UNICODE_Pd, //U+2E1A
UNICODE_Po, //U+2E1B
UNICODE_Pi, //U+2E1C
UNICODE_Pf, //U+2E1D
UNICODE_Po, //U+2E1E
UNICODE_Po, //U+2E1F
UNICODE_Pi, //U+2E20
UNICODE_Pf, //U+2E21
UNICODE_Ps, //U+2E22
UNICODE_Pe, //U+2E23
UNICODE_Ps, //U+2E24
UNICODE_Pe, //U+2E25
UNICODE_Ps, //U+2E26
UNICODE_Pe, //U+2E27
UNICODE_Ps, //U+2E28
UNICODE_Pe, //U+2E29
UNICODE_Po, //U+2E2A
UNICODE_Po, //U+2E2B
UNICODE_Po, //U+2E2C
UNICODE_Po, //U+2E2D
UNICODE_Po, //U+2E2E
UNICODE_Lm, //U+2E2F
UNICODE_Po, //U+2E30
UNICODE_Po, //U+2E31
0, //U+2E32
0, //U+2E33
0, //U+2E34
0, //U+2E35
0, //U+2E36
0, //U+2E37
0, //U+2E38
0, //U+2E39
0, //U+2E3A
0, //U+2E3B
0, //U+2E3C
0, //U+2E3D
0, //U+2E3E
0, //U+2E3F
0, //U+2E40
0, //U+2E41
0, //U+2E42
0, //U+2E43
0, //U+2E44
0, //U+2E45
0, //U+2E46
0, //U+2E47
0, //U+2E48
0, //U+2E49
0, //U+2E4A
0, //U+2E4B
0, //U+2E4C
0, //U+2E4D
0, //U+2E4E
0, //U+2E4F
0, //U+2E50
0, //U+2E51
0, //U+2E52
0, //U+2E53
0, //U+2E54
0, //U+2E55
0, //U+2E56
0, //U+2E57
0, //U+2E58
0, //U+2E59
0, //U+2E5A
0, //U+2E5B
0, //U+2E5C
0, //U+2E5D
0, //U+2E5E
0, //U+2E5F
0, //U+2E60
0, //U+2E61
0, //U+2E62
0, //U+2E63
0, //U+2E64
0, //U+2E65
0, //U+2E66
0, //U+2E67
0, //U+2E68
0, //U+2E69
0, //U+2E6A
0, //U+2E6B
0, //U+2E6C
0, //U+2E6D
0, //U+2E6E
0, //U+2E6F
0, //U+2E70
0, //U+2E71
0, //U+2E72
0, //U+2E73
0, //U+2E74
0, //U+2E75
0, //U+2E76
0, //U+2E77
0, //U+2E78
0, //U+2E79
0, //U+2E7A
0, //U+2E7B
0, //U+2E7C
0, //U+2E7D
0, //U+2E7E
0, //U+2E7F
UNICODE_So, //U+2E80
UNICODE_So, //U+2E81
UNICODE_So, //U+2E82
UNICODE_So, //U+2E83
UNICODE_So, //U+2E84
UNICODE_So, //U+2E85
UNICODE_So, //U+2E86
UNICODE_So, //U+2E87
UNICODE_So, //U+2E88
UNICODE_So, //U+2E89
UNICODE_So, //U+2E8A
UNICODE_So, //U+2E8B
UNICODE_So, //U+2E8C
UNICODE_So, //U+2E8D
UNICODE_So, //U+2E8E
UNICODE_So, //U+2E8F
UNICODE_So, //U+2E90
UNICODE_So, //U+2E91
UNICODE_So, //U+2E92
UNICODE_So, //U+2E93
UNICODE_So, //U+2E94
UNICODE_So, //U+2E95
UNICODE_So, //U+2E96
UNICODE_So, //U+2E97
UNICODE_So, //U+2E98
UNICODE_So, //U+2E99
0, //U+2E9A
UNICODE_So, //U+2E9B
UNICODE_So, //U+2E9C
UNICODE_So, //U+2E9D
UNICODE_So, //U+2E9E
UNICODE_So, //U+2E9F
UNICODE_So, //U+2EA0
UNICODE_So, //U+2EA1
UNICODE_So, //U+2EA2
UNICODE_So, //U+2EA3
UNICODE_So, //U+2EA4
UNICODE_So, //U+2EA5
UNICODE_So, //U+2EA6
UNICODE_So, //U+2EA7
UNICODE_So, //U+2EA8
UNICODE_So, //U+2EA9
UNICODE_So, //U+2EAA
UNICODE_So, //U+2EAB
UNICODE_So, //U+2EAC
UNICODE_So, //U+2EAD
UNICODE_So, //U+2EAE
UNICODE_So, //U+2EAF
UNICODE_So, //U+2EB0
UNICODE_So, //U+2EB1
UNICODE_So, //U+2EB2
UNICODE_So, //U+2EB3
UNICODE_So, //U+2EB4
UNICODE_So, //U+2EB5
UNICODE_So, //U+2EB6
UNICODE_So, //U+2EB7
UNICODE_So, //U+2EB8
UNICODE_So, //U+2EB9
UNICODE_So, //U+2EBA
UNICODE_So, //U+2EBB
UNICODE_So, //U+2EBC
UNICODE_So, //U+2EBD
UNICODE_So, //U+2EBE
UNICODE_So, //U+2EBF
UNICODE_So, //U+2EC0
UNICODE_So, //U+2EC1
UNICODE_So, //U+2EC2
UNICODE_So, //U+2EC3
UNICODE_So, //U+2EC4
UNICODE_So, //U+2EC5
UNICODE_So, //U+2EC6
UNICODE_So, //U+2EC7
UNICODE_So, //U+2EC8
UNICODE_So, //U+2EC9
UNICODE_So, //U+2ECA
UNICODE_So, //U+2ECB
UNICODE_So, //U+2ECC
UNICODE_So, //U+2ECD
UNICODE_So, //U+2ECE
UNICODE_So, //U+2ECF
UNICODE_So, //U+2ED0
UNICODE_So, //U+2ED1
UNICODE_So, //U+2ED2
UNICODE_So, //U+2ED3
UNICODE_So, //U+2ED4
UNICODE_So, //U+2ED5
UNICODE_So, //U+2ED6
UNICODE_So, //U+2ED7
UNICODE_So, //U+2ED8
UNICODE_So, //U+2ED9
UNICODE_So, //U+2EDA
UNICODE_So, //U+2EDB
UNICODE_So, //U+2EDC
UNICODE_So, //U+2EDD
UNICODE_So, //U+2EDE
UNICODE_So, //U+2EDF
UNICODE_So, //U+2EE0
UNICODE_So, //U+2EE1
UNICODE_So, //U+2EE2
UNICODE_So, //U+2EE3
UNICODE_So, //U+2EE4
UNICODE_So, //U+2EE5
UNICODE_So, //U+2EE6
UNICODE_So, //U+2EE7
UNICODE_So, //U+2EE8
UNICODE_So, //U+2EE9
UNICODE_So, //U+2EEA
UNICODE_So, //U+2EEB
UNICODE_So, //U+2EEC
UNICODE_So, //U+2EED
UNICODE_So, //U+2EEE
UNICODE_So, //U+2EEF
UNICODE_So, //U+2EF0
UNICODE_So, //U+2EF1
UNICODE_So, //U+2EF2
UNICODE_So, //U+2EF3
0, //U+2EF4
0, //U+2EF5
0, //U+2EF6
0, //U+2EF7
0, //U+2EF8
0, //U+2EF9
0, //U+2EFA
0, //U+2EFB
0, //U+2EFC
0, //U+2EFD
0, //U+2EFE
0, //U+2EFF
UNICODE_So, //U+2F00
UNICODE_So, //U+2F01
UNICODE_So, //U+2F02
UNICODE_So, //U+2F03
UNICODE_So, //U+2F04
UNICODE_So, //U+2F05
UNICODE_So, //U+2F06
UNICODE_So, //U+2F07
UNICODE_So, //U+2F08
UNICODE_So, //U+2F09
UNICODE_So, //U+2F0A
UNICODE_So, //U+2F0B
UNICODE_So, //U+2F0C
UNICODE_So, //U+2F0D
UNICODE_So, //U+2F0E
UNICODE_So, //U+2F0F
UNICODE_So, //U+2F10
UNICODE_So, //U+2F11
UNICODE_So, //U+2F12
UNICODE_So, //U+2F13
UNICODE_So, //U+2F14
UNICODE_So, //U+2F15
UNICODE_So, //U+2F16
UNICODE_So, //U+2F17
UNICODE_So, //U+2F18
UNICODE_So, //U+2F19
UNICODE_So, //U+2F1A
UNICODE_So, //U+2F1B
UNICODE_So, //U+2F1C
UNICODE_So, //U+2F1D
UNICODE_So, //U+2F1E
UNICODE_So, //U+2F1F
UNICODE_So, //U+2F20
UNICODE_So, //U+2F21
UNICODE_So, //U+2F22
UNICODE_So, //U+2F23
UNICODE_So, //U+2F24
UNICODE_So, //U+2F25
UNICODE_So, //U+2F26
UNICODE_So, //U+2F27
UNICODE_So, //U+2F28
UNICODE_So, //U+2F29
UNICODE_So, //U+2F2A
UNICODE_So, //U+2F2B
UNICODE_So, //U+2F2C
UNICODE_So, //U+2F2D
UNICODE_So, //U+2F2E
UNICODE_So, //U+2F2F
UNICODE_So, //U+2F30
UNICODE_So, //U+2F31
UNICODE_So, //U+2F32
UNICODE_So, //U+2F33
UNICODE_So, //U+2F34
UNICODE_So, //U+2F35
UNICODE_So, //U+2F36
UNICODE_So, //U+2F37
UNICODE_So, //U+2F38
UNICODE_So, //U+2F39
UNICODE_So, //U+2F3A
UNICODE_So, //U+2F3B
UNICODE_So, //U+2F3C
UNICODE_So, //U+2F3D
UNICODE_So, //U+2F3E
UNICODE_So, //U+2F3F
UNICODE_So, //U+2F40
UNICODE_So, //U+2F41
UNICODE_So, //U+2F42
UNICODE_So, //U+2F43
UNICODE_So, //U+2F44
UNICODE_So, //U+2F45
UNICODE_So, //U+2F46
UNICODE_So, //U+2F47
UNICODE_So, //U+2F48
UNICODE_So, //U+2F49
UNICODE_So, //U+2F4A
UNICODE_So, //U+2F4B
UNICODE_So, //U+2F4C
UNICODE_So, //U+2F4D
UNICODE_So, //U+2F4E
UNICODE_So, //U+2F4F
UNICODE_So, //U+2F50
UNICODE_So, //U+2F51
UNICODE_So, //U+2F52
UNICODE_So, //U+2F53
UNICODE_So, //U+2F54
UNICODE_So, //U+2F55
UNICODE_So, //U+2F56
UNICODE_So, //U+2F57
UNICODE_So, //U+2F58
UNICODE_So, //U+2F59
UNICODE_So, //U+2F5A
UNICODE_So, //U+2F5B
UNICODE_So, //U+2F5C
UNICODE_So, //U+2F5D
UNICODE_So, //U+2F5E
UNICODE_So, //U+2F5F
UNICODE_So, //U+2F60
UNICODE_So, //U+2F61
UNICODE_So, //U+2F62
UNICODE_So, //U+2F63
UNICODE_So, //U+2F64
UNICODE_So, //U+2F65
UNICODE_So, //U+2F66
UNICODE_So, //U+2F67
UNICODE_So, //U+2F68
UNICODE_So, //U+2F69
UNICODE_So, //U+2F6A
UNICODE_So, //U+2F6B
UNICODE_So, //U+2F6C
UNICODE_So, //U+2F6D
UNICODE_So, //U+2F6E
UNICODE_So, //U+2F6F
UNICODE_So, //U+2F70
UNICODE_So, //U+2F71
UNICODE_So, //U+2F72
UNICODE_So, //U+2F73
UNICODE_So, //U+2F74
UNICODE_So, //U+2F75
UNICODE_So, //U+2F76
UNICODE_So, //U+2F77
UNICODE_So, //U+2F78
UNICODE_So, //U+2F79
UNICODE_So, //U+2F7A
UNICODE_So, //U+2F7B
UNICODE_So, //U+2F7C
UNICODE_So, //U+2F7D
UNICODE_So, //U+2F7E
UNICODE_So, //U+2F7F
UNICODE_So, //U+2F80
UNICODE_So, //U+2F81
UNICODE_So, //U+2F82
UNICODE_So, //U+2F83
UNICODE_So, //U+2F84
UNICODE_So, //U+2F85
UNICODE_So, //U+2F86
UNICODE_So, //U+2F87
UNICODE_So, //U+2F88
UNICODE_So, //U+2F89
UNICODE_So, //U+2F8A
UNICODE_So, //U+2F8B
UNICODE_So, //U+2F8C
UNICODE_So, //U+2F8D
UNICODE_So, //U+2F8E
UNICODE_So, //U+2F8F
UNICODE_So, //U+2F90
UNICODE_So, //U+2F91
UNICODE_So, //U+2F92
UNICODE_So, //U+2F93
UNICODE_So, //U+2F94
UNICODE_So, //U+2F95
UNICODE_So, //U+2F96
UNICODE_So, //U+2F97
UNICODE_So, //U+2F98
UNICODE_So, //U+2F99
UNICODE_So, //U+2F9A
UNICODE_So, //U+2F9B
UNICODE_So, //U+2F9C
UNICODE_So, //U+2F9D
UNICODE_So, //U+2F9E
UNICODE_So, //U+2F9F
UNICODE_So, //U+2FA0
UNICODE_So, //U+2FA1
UNICODE_So, //U+2FA2
UNICODE_So, //U+2FA3
UNICODE_So, //U+2FA4
UNICODE_So, //U+2FA5
UNICODE_So, //U+2FA6
UNICODE_So, //U+2FA7
UNICODE_So, //U+2FA8
UNICODE_So, //U+2FA9
UNICODE_So, //U+2FAA
UNICODE_So, //U+2FAB
UNICODE_So, //U+2FAC
UNICODE_So, //U+2FAD
UNICODE_So, //U+2FAE
UNICODE_So, //U+2FAF
UNICODE_So, //U+2FB0
UNICODE_So, //U+2FB1
UNICODE_So, //U+2FB2
UNICODE_So, //U+2FB3
UNICODE_So, //U+2FB4
UNICODE_So, //U+2FB5
UNICODE_So, //U+2FB6
UNICODE_So, //U+2FB7
UNICODE_So, //U+2FB8
UNICODE_So, //U+2FB9
UNICODE_So, //U+2FBA
UNICODE_So, //U+2FBB
UNICODE_So, //U+2FBC
UNICODE_So, //U+2FBD
UNICODE_So, //U+2FBE
UNICODE_So, //U+2FBF
UNICODE_So, //U+2FC0
UNICODE_So, //U+2FC1
UNICODE_So, //U+2FC2
UNICODE_So, //U+2FC3
UNICODE_So, //U+2FC4
UNICODE_So, //U+2FC5
UNICODE_So, //U+2FC6
UNICODE_So, //U+2FC7
UNICODE_So, //U+2FC8
UNICODE_So, //U+2FC9
UNICODE_So, //U+2FCA
UNICODE_So, //U+2FCB
UNICODE_So, //U+2FCC
UNICODE_So, //U+2FCD
UNICODE_So, //U+2FCE
UNICODE_So, //U+2FCF
UNICODE_So, //U+2FD0
UNICODE_So, //U+2FD1
UNICODE_So, //U+2FD2
UNICODE_So, //U+2FD3
UNICODE_So, //U+2FD4
UNICODE_So, //U+2FD5
0, //U+2FD6
0, //U+2FD7
0, //U+2FD8
0, //U+2FD9
0, //U+2FDA
0, //U+2FDB
0, //U+2FDC
0, //U+2FDD
0, //U+2FDE
0, //U+2FDF
0, //U+2FE0
0, //U+2FE1
0, //U+2FE2
0, //U+2FE3
0, //U+2FE4
0, //U+2FE5
0, //U+2FE6
0, //U+2FE7
0, //U+2FE8
0, //U+2FE9
0, //U+2FEA
0, //U+2FEB
0, //U+2FEC
0, //U+2FED
0, //U+2FEE
0, //U+2FEF
UNICODE_So, //U+2FF0
UNICODE_So, //U+2FF1
UNICODE_So, //U+2FF2
UNICODE_So, //U+2FF3
UNICODE_So, //U+2FF4
UNICODE_So, //U+2FF5
UNICODE_So, //U+2FF6
UNICODE_So, //U+2FF7
UNICODE_So, //U+2FF8
UNICODE_So, //U+2FF9
UNICODE_So, //U+2FFA
UNICODE_So, //U+2FFB
0, //U+2FFC
0, //U+2FFD
0, //U+2FFE
0, //U+2FFF
UNICODE_Zs, //U+3000
UNICODE_Po, //U+3001
UNICODE_Po, //U+3002
UNICODE_Po, //U+3003
UNICODE_So, //U+3004
UNICODE_Lm, //U+3005
UNICODE_Lo, //U+3006
UNICODE_Nl, //U+3007
UNICODE_Ps, //U+3008
UNICODE_Pe, //U+3009
UNICODE_Ps, //U+300A
UNICODE_Pe, //U+300B
UNICODE_Ps, //U+300C
UNICODE_Pe, //U+300D
UNICODE_Ps, //U+300E
UNICODE_Pe, //U+300F
UNICODE_Ps, //U+3010
UNICODE_Pe, //U+3011
UNICODE_So, //U+3012
UNICODE_So, //U+3013
UNICODE_Ps, //U+3014
UNICODE_Pe, //U+3015
UNICODE_Ps, //U+3016
UNICODE_Pe, //U+3017
UNICODE_Ps, //U+3018
UNICODE_Pe, //U+3019
UNICODE_Ps, //U+301A
UNICODE_Pe, //U+301B
UNICODE_Pd, //U+301C
UNICODE_Ps, //U+301D
UNICODE_Pe, //U+301E
UNICODE_Pe, //U+301F
UNICODE_So, //U+3020
UNICODE_Nl, //U+3021
UNICODE_Nl, //U+3022
UNICODE_Nl, //U+3023
UNICODE_Nl, //U+3024
UNICODE_Nl, //U+3025
UNICODE_Nl, //U+3026
UNICODE_Nl, //U+3027
UNICODE_Nl, //U+3028
UNICODE_Nl, //U+3029
UNICODE_Mn, //U+302A
UNICODE_Mn, //U+302B
UNICODE_Mn, //U+302C
UNICODE_Mn, //U+302D
UNICODE_Mn, //U+302E
UNICODE_Mn, //U+302F
UNICODE_Pd, //U+3030
UNICODE_Lm, //U+3031
UNICODE_Lm, //U+3032
UNICODE_Lm, //U+3033
UNICODE_Lm, //U+3034
UNICODE_Lm, //U+3035
UNICODE_So, //U+3036
UNICODE_So, //U+3037
UNICODE_Nl, //U+3038
UNICODE_Nl, //U+3039
UNICODE_Nl, //U+303A
UNICODE_Lm, //U+303B
UNICODE_Lo, //U+303C
UNICODE_Po, //U+303D
UNICODE_So, //U+303E
UNICODE_So, //U+303F
0, //U+3040
UNICODE_Lo, //U+3041
UNICODE_Lo, //U+3042
UNICODE_Lo, //U+3043
UNICODE_Lo, //U+3044
UNICODE_Lo, //U+3045
UNICODE_Lo, //U+3046
UNICODE_Lo, //U+3047
UNICODE_Lo, //U+3048
UNICODE_Lo, //U+3049
UNICODE_Lo, //U+304A
UNICODE_Lo, //U+304B
UNICODE_Lo, //U+304C
UNICODE_Lo, //U+304D
UNICODE_Lo, //U+304E
UNICODE_Lo, //U+304F
UNICODE_Lo, //U+3050
UNICODE_Lo, //U+3051
UNICODE_Lo, //U+3052
UNICODE_Lo, //U+3053
UNICODE_Lo, //U+3054
UNICODE_Lo, //U+3055
UNICODE_Lo, //U+3056
UNICODE_Lo, //U+3057
UNICODE_Lo, //U+3058
UNICODE_Lo, //U+3059
UNICODE_Lo, //U+305A
UNICODE_Lo, //U+305B
UNICODE_Lo, //U+305C
UNICODE_Lo, //U+305D
UNICODE_Lo, //U+305E
UNICODE_Lo, //U+305F
UNICODE_Lo, //U+3060
UNICODE_Lo, //U+3061
UNICODE_Lo, //U+3062
UNICODE_Lo, //U+3063
UNICODE_Lo, //U+3064
UNICODE_Lo, //U+3065
UNICODE_Lo, //U+3066
UNICODE_Lo, //U+3067
UNICODE_Lo, //U+3068
UNICODE_Lo, //U+3069
UNICODE_Lo, //U+306A
UNICODE_Lo, //U+306B
UNICODE_Lo, //U+306C
UNICODE_Lo, //U+306D
UNICODE_Lo, //U+306E
UNICODE_Lo, //U+306F
UNICODE_Lo, //U+3070
UNICODE_Lo, //U+3071
UNICODE_Lo, //U+3072
UNICODE_Lo, //U+3073
UNICODE_Lo, //U+3074
UNICODE_Lo, //U+3075
UNICODE_Lo, //U+3076
UNICODE_Lo, //U+3077
UNICODE_Lo, //U+3078
UNICODE_Lo, //U+3079
UNICODE_Lo, //U+307A
UNICODE_Lo, //U+307B
UNICODE_Lo, //U+307C
UNICODE_Lo, //U+307D
UNICODE_Lo, //U+307E
UNICODE_Lo, //U+307F
UNICODE_Lo, //U+3080
UNICODE_Lo, //U+3081
UNICODE_Lo, //U+3082
UNICODE_Lo, //U+3083
UNICODE_Lo, //U+3084
UNICODE_Lo, //U+3085
UNICODE_Lo, //U+3086
UNICODE_Lo, //U+3087
UNICODE_Lo, //U+3088
UNICODE_Lo, //U+3089
UNICODE_Lo, //U+308A
UNICODE_Lo, //U+308B
UNICODE_Lo, //U+308C
UNICODE_Lo, //U+308D
UNICODE_Lo, //U+308E
UNICODE_Lo, //U+308F
UNICODE_Lo, //U+3090
UNICODE_Lo, //U+3091
UNICODE_Lo, //U+3092
UNICODE_Lo, //U+3093
UNICODE_Lo, //U+3094
UNICODE_Lo, //U+3095
UNICODE_Lo, //U+3096
0, //U+3097
0, //U+3098
UNICODE_Mn, //U+3099
UNICODE_Mn, //U+309A
UNICODE_Sk, //U+309B
UNICODE_Sk, //U+309C
UNICODE_Lm, //U+309D
UNICODE_Lm, //U+309E
UNICODE_Lo, //U+309F
UNICODE_Pd, //U+30A0
UNICODE_Lo, //U+30A1
UNICODE_Lo, //U+30A2
UNICODE_Lo, //U+30A3
UNICODE_Lo, //U+30A4
UNICODE_Lo, //U+30A5
UNICODE_Lo, //U+30A6
UNICODE_Lo, //U+30A7
UNICODE_Lo, //U+30A8
UNICODE_Lo, //U+30A9
UNICODE_Lo, //U+30AA
UNICODE_Lo, //U+30AB
UNICODE_Lo, //U+30AC
UNICODE_Lo, //U+30AD
UNICODE_Lo, //U+30AE
UNICODE_Lo, //U+30AF
UNICODE_Lo, //U+30B0
UNICODE_Lo, //U+30B1
UNICODE_Lo, //U+30B2
UNICODE_Lo, //U+30B3
UNICODE_Lo, //U+30B4
UNICODE_Lo, //U+30B5
UNICODE_Lo, //U+30B6
UNICODE_Lo, //U+30B7
UNICODE_Lo, //U+30B8
UNICODE_Lo, //U+30B9
UNICODE_Lo, //U+30BA
UNICODE_Lo, //U+30BB
UNICODE_Lo, //U+30BC
UNICODE_Lo, //U+30BD
UNICODE_Lo, //U+30BE
UNICODE_Lo, //U+30BF
UNICODE_Lo, //U+30C0
UNICODE_Lo, //U+30C1
UNICODE_Lo, //U+30C2
UNICODE_Lo, //U+30C3
UNICODE_Lo, //U+30C4
UNICODE_Lo, //U+30C5
UNICODE_Lo, //U+30C6
UNICODE_Lo, //U+30C7
UNICODE_Lo, //U+30C8
UNICODE_Lo, //U+30C9
UNICODE_Lo, //U+30CA
UNICODE_Lo, //U+30CB
UNICODE_Lo, //U+30CC
UNICODE_Lo, //U+30CD
UNICODE_Lo, //U+30CE
UNICODE_Lo, //U+30CF
UNICODE_Lo, //U+30D0
UNICODE_Lo, //U+30D1
UNICODE_Lo, //U+30D2
UNICODE_Lo, //U+30D3
UNICODE_Lo, //U+30D4
UNICODE_Lo, //U+30D5
UNICODE_Lo, //U+30D6
UNICODE_Lo, //U+30D7
UNICODE_Lo, //U+30D8
UNICODE_Lo, //U+30D9
UNICODE_Lo, //U+30DA
UNICODE_Lo, //U+30DB
UNICODE_Lo, //U+30DC
UNICODE_Lo, //U+30DD
UNICODE_Lo, //U+30DE
UNICODE_Lo, //U+30DF
UNICODE_Lo, //U+30E0
UNICODE_Lo, //U+30E1
UNICODE_Lo, //U+30E2
UNICODE_Lo, //U+30E3
UNICODE_Lo, //U+30E4
UNICODE_Lo, //U+30E5
UNICODE_Lo, //U+30E6
UNICODE_Lo, //U+30E7
UNICODE_Lo, //U+30E8
UNICODE_Lo, //U+30E9
UNICODE_Lo, //U+30EA
UNICODE_Lo, //U+30EB
UNICODE_Lo, //U+30EC
UNICODE_Lo, //U+30ED
UNICODE_Lo, //U+30EE
UNICODE_Lo, //U+30EF
UNICODE_Lo, //U+30F0
UNICODE_Lo, //U+30F1
UNICODE_Lo, //U+30F2
UNICODE_Lo, //U+30F3
UNICODE_Lo, //U+30F4
UNICODE_Lo, //U+30F5
UNICODE_Lo, //U+30F6
UNICODE_Lo, //U+30F7
UNICODE_Lo, //U+30F8
UNICODE_Lo, //U+30F9
UNICODE_Lo, //U+30FA
UNICODE_Po, //U+30FB
UNICODE_Lm, //U+30FC
UNICODE_Lm, //U+30FD
UNICODE_Lm, //U+30FE
UNICODE_Lo, //U+30FF
0, //U+3100
0, //U+3101
0, //U+3102
0, //U+3103
0, //U+3104
UNICODE_Lo, //U+3105
UNICODE_Lo, //U+3106
UNICODE_Lo, //U+3107
UNICODE_Lo, //U+3108
UNICODE_Lo, //U+3109
UNICODE_Lo, //U+310A
UNICODE_Lo, //U+310B
UNICODE_Lo, //U+310C
UNICODE_Lo, //U+310D
UNICODE_Lo, //U+310E
UNICODE_Lo, //U+310F
UNICODE_Lo, //U+3110
UNICODE_Lo, //U+3111
UNICODE_Lo, //U+3112
UNICODE_Lo, //U+3113
UNICODE_Lo, //U+3114
UNICODE_Lo, //U+3115
UNICODE_Lo, //U+3116
UNICODE_Lo, //U+3117
UNICODE_Lo, //U+3118
UNICODE_Lo, //U+3119
UNICODE_Lo, //U+311A
UNICODE_Lo, //U+311B
UNICODE_Lo, //U+311C
UNICODE_Lo, //U+311D
UNICODE_Lo, //U+311E
UNICODE_Lo, //U+311F
UNICODE_Lo, //U+3120
UNICODE_Lo, //U+3121
UNICODE_Lo, //U+3122
UNICODE_Lo, //U+3123
UNICODE_Lo, //U+3124
UNICODE_Lo, //U+3125
UNICODE_Lo, //U+3126
UNICODE_Lo, //U+3127
UNICODE_Lo, //U+3128
UNICODE_Lo, //U+3129
UNICODE_Lo, //U+312A
UNICODE_Lo, //U+312B
UNICODE_Lo, //U+312C
UNICODE_Lo, //U+312D
0, //U+312E
0, //U+312F
0, //U+3130
UNICODE_Lo, //U+3131
UNICODE_Lo, //U+3132
UNICODE_Lo, //U+3133
UNICODE_Lo, //U+3134
UNICODE_Lo, //U+3135
UNICODE_Lo, //U+3136
UNICODE_Lo, //U+3137
UNICODE_Lo, //U+3138
UNICODE_Lo, //U+3139
UNICODE_Lo, //U+313A
UNICODE_Lo, //U+313B
UNICODE_Lo, //U+313C
UNICODE_Lo, //U+313D
UNICODE_Lo, //U+313E
UNICODE_Lo, //U+313F
UNICODE_Lo, //U+3140
UNICODE_Lo, //U+3141
UNICODE_Lo, //U+3142
UNICODE_Lo, //U+3143
UNICODE_Lo, //U+3144
UNICODE_Lo, //U+3145
UNICODE_Lo, //U+3146
UNICODE_Lo, //U+3147
UNICODE_Lo, //U+3148
UNICODE_Lo, //U+3149
UNICODE_Lo, //U+314A
UNICODE_Lo, //U+314B
UNICODE_Lo, //U+314C
UNICODE_Lo, //U+314D
UNICODE_Lo, //U+314E
UNICODE_Lo, //U+314F
UNICODE_Lo, //U+3150
UNICODE_Lo, //U+3151
UNICODE_Lo, //U+3152
UNICODE_Lo, //U+3153
UNICODE_Lo, //U+3154
UNICODE_Lo, //U+3155
UNICODE_Lo, //U+3156
UNICODE_Lo, //U+3157
UNICODE_Lo, //U+3158
UNICODE_Lo, //U+3159
UNICODE_Lo, //U+315A
UNICODE_Lo, //U+315B
UNICODE_Lo, //U+315C
UNICODE_Lo, //U+315D
UNICODE_Lo, //U+315E
UNICODE_Lo, //U+315F
UNICODE_Lo, //U+3160
UNICODE_Lo, //U+3161
UNICODE_Lo, //U+3162
UNICODE_Lo, //U+3163
UNICODE_Lo, //U+3164
UNICODE_Lo, //U+3165
UNICODE_Lo, //U+3166
UNICODE_Lo, //U+3167
UNICODE_Lo, //U+3168
UNICODE_Lo, //U+3169
UNICODE_Lo, //U+316A
UNICODE_Lo, //U+316B
UNICODE_Lo, //U+316C
UNICODE_Lo, //U+316D
UNICODE_Lo, //U+316E
UNICODE_Lo, //U+316F
UNICODE_Lo, //U+3170
UNICODE_Lo, //U+3171
UNICODE_Lo, //U+3172
UNICODE_Lo, //U+3173
UNICODE_Lo, //U+3174
UNICODE_Lo, //U+3175
UNICODE_Lo, //U+3176
UNICODE_Lo, //U+3177
UNICODE_Lo, //U+3178
UNICODE_Lo, //U+3179
UNICODE_Lo, //U+317A
UNICODE_Lo, //U+317B
UNICODE_Lo, //U+317C
UNICODE_Lo, //U+317D
UNICODE_Lo, //U+317E
UNICODE_Lo, //U+317F
UNICODE_Lo, //U+3180
UNICODE_Lo, //U+3181
UNICODE_Lo, //U+3182
UNICODE_Lo, //U+3183
UNICODE_Lo, //U+3184
UNICODE_Lo, //U+3185
UNICODE_Lo, //U+3186
UNICODE_Lo, //U+3187
UNICODE_Lo, //U+3188
UNICODE_Lo, //U+3189
UNICODE_Lo, //U+318A
UNICODE_Lo, //U+318B
UNICODE_Lo, //U+318C
UNICODE_Lo, //U+318D
UNICODE_Lo, //U+318E
0, //U+318F
UNICODE_So, //U+3190
UNICODE_So, //U+3191
UNICODE_No, //U+3192
UNICODE_No, //U+3193
UNICODE_No, //U+3194
UNICODE_No, //U+3195
UNICODE_So, //U+3196
UNICODE_So, //U+3197
UNICODE_So, //U+3198
UNICODE_So, //U+3199
UNICODE_So, //U+319A
UNICODE_So, //U+319B
UNICODE_So, //U+319C
UNICODE_So, //U+319D
UNICODE_So, //U+319E
UNICODE_So, //U+319F
UNICODE_Lo, //U+31A0
UNICODE_Lo, //U+31A1
UNICODE_Lo, //U+31A2
UNICODE_Lo, //U+31A3
UNICODE_Lo, //U+31A4
UNICODE_Lo, //U+31A5
UNICODE_Lo, //U+31A6
UNICODE_Lo, //U+31A7
UNICODE_Lo, //U+31A8
UNICODE_Lo, //U+31A9
UNICODE_Lo, //U+31AA
UNICODE_Lo, //U+31AB
UNICODE_Lo, //U+31AC
UNICODE_Lo, //U+31AD
UNICODE_Lo, //U+31AE
UNICODE_Lo, //U+31AF
UNICODE_Lo, //U+31B0
UNICODE_Lo, //U+31B1
UNICODE_Lo, //U+31B2
UNICODE_Lo, //U+31B3
UNICODE_Lo, //U+31B4
UNICODE_Lo, //U+31B5
UNICODE_Lo, //U+31B6
UNICODE_Lo, //U+31B7
UNICODE_Lo, //U+31B8
UNICODE_Lo, //U+31B9
UNICODE_Lo, //U+31BA
0, //U+31BB
0, //U+31BC
0, //U+31BD
0, //U+31BE
0, //U+31BF
UNICODE_So, //U+31C0
UNICODE_So, //U+31C1
UNICODE_So, //U+31C2
UNICODE_So, //U+31C3
UNICODE_So, //U+31C4
UNICODE_So, //U+31C5
UNICODE_So, //U+31C6
UNICODE_So, //U+31C7
UNICODE_So, //U+31C8
UNICODE_So, //U+31C9
UNICODE_So, //U+31CA
UNICODE_So, //U+31CB
UNICODE_So, //U+31CC
UNICODE_So, //U+31CD
UNICODE_So, //U+31CE
UNICODE_So, //U+31CF
UNICODE_So, //U+31D0
UNICODE_So, //U+31D1
UNICODE_So, //U+31D2
UNICODE_So, //U+31D3
UNICODE_So, //U+31D4
UNICODE_So, //U+31D5
UNICODE_So, //U+31D6
UNICODE_So, //U+31D7
UNICODE_So, //U+31D8
UNICODE_So, //U+31D9
UNICODE_So, //U+31DA
UNICODE_So, //U+31DB
UNICODE_So, //U+31DC
UNICODE_So, //U+31DD
UNICODE_So, //U+31DE
UNICODE_So, //U+31DF
UNICODE_So, //U+31E0
UNICODE_So, //U+31E1
UNICODE_So, //U+31E2
UNICODE_So, //U+31E3
0, //U+31E4
0, //U+31E5
0, //U+31E6
0, //U+31E7
0, //U+31E8
0, //U+31E9
0, //U+31EA
0, //U+31EB
0, //U+31EC
0, //U+31ED
0, //U+31EE
0, //U+31EF
UNICODE_Lo, //U+31F0
UNICODE_Lo, //U+31F1
UNICODE_Lo, //U+31F2
UNICODE_Lo, //U+31F3
UNICODE_Lo, //U+31F4
UNICODE_Lo, //U+31F5
UNICODE_Lo, //U+31F6
UNICODE_Lo, //U+31F7
UNICODE_Lo, //U+31F8
UNICODE_Lo, //U+31F9
UNICODE_Lo, //U+31FA
UNICODE_Lo, //U+31FB
UNICODE_Lo, //U+31FC
UNICODE_Lo, //U+31FD
UNICODE_Lo, //U+31FE
UNICODE_Lo, //U+31FF
UNICODE_So, //U+3200
UNICODE_So, //U+3201
UNICODE_So, //U+3202
UNICODE_So, //U+3203
UNICODE_So, //U+3204
UNICODE_So, //U+3205
UNICODE_So, //U+3206
UNICODE_So, //U+3207
UNICODE_So, //U+3208
UNICODE_So, //U+3209
UNICODE_So, //U+320A
UNICODE_So, //U+320B
UNICODE_So, //U+320C
UNICODE_So, //U+320D
UNICODE_So, //U+320E
UNICODE_So, //U+320F
UNICODE_So, //U+3210
UNICODE_So, //U+3211
UNICODE_So, //U+3212
UNICODE_So, //U+3213
UNICODE_So, //U+3214
UNICODE_So, //U+3215
UNICODE_So, //U+3216
UNICODE_So, //U+3217
UNICODE_So, //U+3218
UNICODE_So, //U+3219
UNICODE_So, //U+321A
UNICODE_So, //U+321B
UNICODE_So, //U+321C
UNICODE_So, //U+321D
UNICODE_So, //U+321E
0, //U+321F
UNICODE_No, //U+3220
UNICODE_No, //U+3221
UNICODE_No, //U+3222
UNICODE_No, //U+3223
UNICODE_No, //U+3224
UNICODE_No, //U+3225
UNICODE_No, //U+3226
UNICODE_No, //U+3227
UNICODE_No, //U+3228
UNICODE_No, //U+3229
UNICODE_So, //U+322A
UNICODE_So, //U+322B
UNICODE_So, //U+322C
UNICODE_So, //U+322D
UNICODE_So, //U+322E
UNICODE_So, //U+322F
UNICODE_So, //U+3230
UNICODE_So, //U+3231
UNICODE_So, //U+3232
UNICODE_So, //U+3233
UNICODE_So, //U+3234
UNICODE_So, //U+3235
UNICODE_So, //U+3236
UNICODE_So, //U+3237
UNICODE_So, //U+3238
UNICODE_So, //U+3239
UNICODE_So, //U+323A
UNICODE_So, //U+323B
UNICODE_So, //U+323C
UNICODE_So, //U+323D
UNICODE_So, //U+323E
UNICODE_So, //U+323F
UNICODE_So, //U+3240
UNICODE_So, //U+3241
UNICODE_So, //U+3242
UNICODE_So, //U+3243
UNICODE_So, //U+3244
UNICODE_So, //U+3245
UNICODE_So, //U+3246
UNICODE_So, //U+3247
UNICODE_So, //U+3248
UNICODE_So, //U+3249
UNICODE_So, //U+324A
UNICODE_So, //U+324B
UNICODE_So, //U+324C
UNICODE_So, //U+324D
UNICODE_So, //U+324E
UNICODE_So, //U+324F
UNICODE_So, //U+3250
UNICODE_No, //U+3251
UNICODE_No, //U+3252
UNICODE_No, //U+3253
UNICODE_No, //U+3254
UNICODE_No, //U+3255
UNICODE_No, //U+3256
UNICODE_No, //U+3257
UNICODE_No, //U+3258
UNICODE_No, //U+3259
UNICODE_No, //U+325A
UNICODE_No, //U+325B
UNICODE_No, //U+325C
UNICODE_No, //U+325D
UNICODE_No, //U+325E
UNICODE_No, //U+325F
UNICODE_So, //U+3260
UNICODE_So, //U+3261
UNICODE_So, //U+3262
UNICODE_So, //U+3263
UNICODE_So, //U+3264
UNICODE_So, //U+3265
UNICODE_So, //U+3266
UNICODE_So, //U+3267
UNICODE_So, //U+3268
UNICODE_So, //U+3269
UNICODE_So, //U+326A
UNICODE_So, //U+326B
UNICODE_So, //U+326C
UNICODE_So, //U+326D
UNICODE_So, //U+326E
UNICODE_So, //U+326F
UNICODE_So, //U+3270
UNICODE_So, //U+3271
UNICODE_So, //U+3272
UNICODE_So, //U+3273
UNICODE_So, //U+3274
UNICODE_So, //U+3275
UNICODE_So, //U+3276
UNICODE_So, //U+3277
UNICODE_So, //U+3278
UNICODE_So, //U+3279
UNICODE_So, //U+327A
UNICODE_So, //U+327B
UNICODE_So, //U+327C
UNICODE_So, //U+327D
UNICODE_So, //U+327E
UNICODE_So, //U+327F
UNICODE_No, //U+3280
UNICODE_No, //U+3281
UNICODE_No, //U+3282
UNICODE_No, //U+3283
UNICODE_No, //U+3284
UNICODE_No, //U+3285
UNICODE_No, //U+3286
UNICODE_No, //U+3287
UNICODE_No, //U+3288
UNICODE_No, //U+3289
UNICODE_So, //U+328A
UNICODE_So, //U+328B
UNICODE_So, //U+328C
UNICODE_So, //U+328D
UNICODE_So, //U+328E
UNICODE_So, //U+328F
UNICODE_So, //U+3290
UNICODE_So, //U+3291
UNICODE_So, //U+3292
UNICODE_So, //U+3293
UNICODE_So, //U+3294
UNICODE_So, //U+3295
UNICODE_So, //U+3296
UNICODE_So, //U+3297
UNICODE_So, //U+3298
UNICODE_So, //U+3299
UNICODE_So, //U+329A
UNICODE_So, //U+329B
UNICODE_So, //U+329C
UNICODE_So, //U+329D
UNICODE_So, //U+329E
UNICODE_So, //U+329F
UNICODE_So, //U+32A0
UNICODE_So, //U+32A1
UNICODE_So, //U+32A2
UNICODE_So, //U+32A3
UNICODE_So, //U+32A4
UNICODE_So, //U+32A5
UNICODE_So, //U+32A6
UNICODE_So, //U+32A7
UNICODE_So, //U+32A8
UNICODE_So, //U+32A9
UNICODE_So, //U+32AA
UNICODE_So, //U+32AB
UNICODE_So, //U+32AC
UNICODE_So, //U+32AD
UNICODE_So, //U+32AE
UNICODE_So, //U+32AF
UNICODE_So, //U+32B0
UNICODE_No, //U+32B1
UNICODE_No, //U+32B2
UNICODE_No, //U+32B3
UNICODE_No, //U+32B4
UNICODE_No, //U+32B5
UNICODE_No, //U+32B6
UNICODE_No, //U+32B7
UNICODE_No, //U+32B8
UNICODE_No, //U+32B9
UNICODE_No, //U+32BA
UNICODE_No, //U+32BB
UNICODE_No, //U+32BC
UNICODE_No, //U+32BD
UNICODE_No, //U+32BE
UNICODE_No, //U+32BF
UNICODE_So, //U+32C0
UNICODE_So, //U+32C1
UNICODE_So, //U+32C2
UNICODE_So, //U+32C3
UNICODE_So, //U+32C4
UNICODE_So, //U+32C5
UNICODE_So, //U+32C6
UNICODE_So, //U+32C7
UNICODE_So, //U+32C8
UNICODE_So, //U+32C9
UNICODE_So, //U+32CA
UNICODE_So, //U+32CB
UNICODE_So, //U+32CC
UNICODE_So, //U+32CD
UNICODE_So, //U+32CE
UNICODE_So, //U+32CF
UNICODE_So, //U+32D0
UNICODE_So, //U+32D1
UNICODE_So, //U+32D2
UNICODE_So, //U+32D3
UNICODE_So, //U+32D4
UNICODE_So, //U+32D5
UNICODE_So, //U+32D6
UNICODE_So, //U+32D7
UNICODE_So, //U+32D8
UNICODE_So, //U+32D9
UNICODE_So, //U+32DA
UNICODE_So, //U+32DB
UNICODE_So, //U+32DC
UNICODE_So, //U+32DD
UNICODE_So, //U+32DE
UNICODE_So, //U+32DF
UNICODE_So, //U+32E0
UNICODE_So, //U+32E1
UNICODE_So, //U+32E2
UNICODE_So, //U+32E3
UNICODE_So, //U+32E4
UNICODE_So, //U+32E5
UNICODE_So, //U+32E6
UNICODE_So, //U+32E7
UNICODE_So, //U+32E8
UNICODE_So, //U+32E9
UNICODE_So, //U+32EA
UNICODE_So, //U+32EB
UNICODE_So, //U+32EC
UNICODE_So, //U+32ED
UNICODE_So, //U+32EE
UNICODE_So, //U+32EF
UNICODE_So, //U+32F0
UNICODE_So, //U+32F1
UNICODE_So, //U+32F2
UNICODE_So, //U+32F3
UNICODE_So, //U+32F4
UNICODE_So, //U+32F5
UNICODE_So, //U+32F6
UNICODE_So, //U+32F7
UNICODE_So, //U+32F8
UNICODE_So, //U+32F9
UNICODE_So, //U+32FA
UNICODE_So, //U+32FB
UNICODE_So, //U+32FC
UNICODE_So, //U+32FD
UNICODE_So, //U+32FE
0, //U+32FF
UNICODE_So, //U+3300
UNICODE_So, //U+3301
UNICODE_So, //U+3302
UNICODE_So, //U+3303
UNICODE_So, //U+3304
UNICODE_So, //U+3305
UNICODE_So, //U+3306
UNICODE_So, //U+3307
UNICODE_So, //U+3308
UNICODE_So, //U+3309
UNICODE_So, //U+330A
UNICODE_So, //U+330B
UNICODE_So, //U+330C
UNICODE_So, //U+330D
UNICODE_So, //U+330E
UNICODE_So, //U+330F
UNICODE_So, //U+3310
UNICODE_So, //U+3311
UNICODE_So, //U+3312
UNICODE_So, //U+3313
UNICODE_So, //U+3314
UNICODE_So, //U+3315
UNICODE_So, //U+3316
UNICODE_So, //U+3317
UNICODE_So, //U+3318
UNICODE_So, //U+3319
UNICODE_So, //U+331A
UNICODE_So, //U+331B
UNICODE_So, //U+331C
UNICODE_So, //U+331D
UNICODE_So, //U+331E
UNICODE_So, //U+331F
UNICODE_So, //U+3320
UNICODE_So, //U+3321
UNICODE_So, //U+3322
UNICODE_So, //U+3323
UNICODE_So, //U+3324
UNICODE_So, //U+3325
UNICODE_So, //U+3326
UNICODE_So, //U+3327
UNICODE_So, //U+3328
UNICODE_So, //U+3329
UNICODE_So, //U+332A
UNICODE_So, //U+332B
UNICODE_So, //U+332C
UNICODE_So, //U+332D
UNICODE_So, //U+332E
UNICODE_So, //U+332F
UNICODE_So, //U+3330
UNICODE_So, //U+3331
UNICODE_So, //U+3332
UNICODE_So, //U+3333
UNICODE_So, //U+3334
UNICODE_So, //U+3335
UNICODE_So, //U+3336
UNICODE_So, //U+3337
UNICODE_So, //U+3338
UNICODE_So, //U+3339
UNICODE_So, //U+333A
UNICODE_So, //U+333B
UNICODE_So, //U+333C
UNICODE_So, //U+333D
UNICODE_So, //U+333E
UNICODE_So, //U+333F
UNICODE_So, //U+3340
UNICODE_So, //U+3341
UNICODE_So, //U+3342
UNICODE_So, //U+3343
UNICODE_So, //U+3344
UNICODE_So, //U+3345
UNICODE_So, //U+3346
UNICODE_So, //U+3347
UNICODE_So, //U+3348
UNICODE_So, //U+3349
UNICODE_So, //U+334A
UNICODE_So, //U+334B
UNICODE_So, //U+334C
UNICODE_So, //U+334D
UNICODE_So, //U+334E
UNICODE_So, //U+334F
UNICODE_So, //U+3350
UNICODE_So, //U+3351
UNICODE_So, //U+3352
UNICODE_So, //U+3353
UNICODE_So, //U+3354
UNICODE_So, //U+3355
UNICODE_So, //U+3356
UNICODE_So, //U+3357
UNICODE_So, //U+3358
UNICODE_So, //U+3359
UNICODE_So, //U+335A
UNICODE_So, //U+335B
UNICODE_So, //U+335C
UNICODE_So, //U+335D
UNICODE_So, //U+335E
UNICODE_So, //U+335F
UNICODE_So, //U+3360
UNICODE_So, //U+3361
UNICODE_So, //U+3362
UNICODE_So, //U+3363
UNICODE_So, //U+3364
UNICODE_So, //U+3365
UNICODE_So, //U+3366
UNICODE_So, //U+3367
UNICODE_So, //U+3368
UNICODE_So, //U+3369
UNICODE_So, //U+336A
UNICODE_So, //U+336B
UNICODE_So, //U+336C
UNICODE_So, //U+336D
UNICODE_So, //U+336E
UNICODE_So, //U+336F
UNICODE_So, //U+3370
UNICODE_So, //U+3371
UNICODE_So, //U+3372
UNICODE_So, //U+3373
UNICODE_So, //U+3374
UNICODE_So, //U+3375
UNICODE_So, //U+3376
UNICODE_So, //U+3377
UNICODE_So, //U+3378
UNICODE_So, //U+3379
UNICODE_So, //U+337A
UNICODE_So, //U+337B
UNICODE_So, //U+337C
UNICODE_So, //U+337D
UNICODE_So, //U+337E
UNICODE_So, //U+337F
UNICODE_So, //U+3380
UNICODE_So, //U+3381
UNICODE_So, //U+3382
UNICODE_So, //U+3383
UNICODE_So, //U+3384
UNICODE_So, //U+3385
UNICODE_So, //U+3386
UNICODE_So, //U+3387
UNICODE_So, //U+3388
UNICODE_So, //U+3389
UNICODE_So, //U+338A
UNICODE_So, //U+338B
UNICODE_So, //U+338C
UNICODE_So, //U+338D
UNICODE_So, //U+338E
UNICODE_So, //U+338F
UNICODE_So, //U+3390
UNICODE_So, //U+3391
UNICODE_So, //U+3392
UNICODE_So, //U+3393
UNICODE_So, //U+3394
UNICODE_So, //U+3395
UNICODE_So, //U+3396
UNICODE_So, //U+3397
UNICODE_So, //U+3398
UNICODE_So, //U+3399
UNICODE_So, //U+339A
UNICODE_So, //U+339B
UNICODE_So, //U+339C
UNICODE_So, //U+339D
UNICODE_So, //U+339E
UNICODE_So, //U+339F
UNICODE_So, //U+33A0
UNICODE_So, //U+33A1
UNICODE_So, //U+33A2
UNICODE_So, //U+33A3
UNICODE_So, //U+33A4
UNICODE_So, //U+33A5
UNICODE_So, //U+33A6
UNICODE_So, //U+33A7
UNICODE_So, //U+33A8
UNICODE_So, //U+33A9
UNICODE_So, //U+33AA
UNICODE_So, //U+33AB
UNICODE_So, //U+33AC
UNICODE_So, //U+33AD
UNICODE_So, //U+33AE
UNICODE_So, //U+33AF
UNICODE_So, //U+33B0
UNICODE_So, //U+33B1
UNICODE_So, //U+33B2
UNICODE_So, //U+33B3
UNICODE_So, //U+33B4
UNICODE_So, //U+33B5
UNICODE_So, //U+33B6
UNICODE_So, //U+33B7
UNICODE_So, //U+33B8
UNICODE_So, //U+33B9
UNICODE_So, //U+33BA
UNICODE_So, //U+33BB
UNICODE_So, //U+33BC
UNICODE_So, //U+33BD
UNICODE_So, //U+33BE
UNICODE_So, //U+33BF
UNICODE_So, //U+33C0
UNICODE_So, //U+33C1
UNICODE_So, //U+33C2
UNICODE_So, //U+33C3
UNICODE_So, //U+33C4
UNICODE_So, //U+33C5
UNICODE_So, //U+33C6
UNICODE_So, //U+33C7
UNICODE_So, //U+33C8
UNICODE_So, //U+33C9
UNICODE_So, //U+33CA
UNICODE_So, //U+33CB
UNICODE_So, //U+33CC
UNICODE_So, //U+33CD
UNICODE_So, //U+33CE
UNICODE_So, //U+33CF
UNICODE_So, //U+33D0
UNICODE_So, //U+33D1
UNICODE_So, //U+33D2
UNICODE_So, //U+33D3
UNICODE_So, //U+33D4
UNICODE_So, //U+33D5
UNICODE_So, //U+33D6
UNICODE_So, //U+33D7
UNICODE_So, //U+33D8
UNICODE_So, //U+33D9
UNICODE_So, //U+33DA
UNICODE_So, //U+33DB
UNICODE_So, //U+33DC
UNICODE_So, //U+33DD
UNICODE_So, //U+33DE
UNICODE_So, //U+33DF
UNICODE_So, //U+33E0
UNICODE_So, //U+33E1
UNICODE_So, //U+33E2
UNICODE_So, //U+33E3
UNICODE_So, //U+33E4
UNICODE_So, //U+33E5
UNICODE_So, //U+33E6
UNICODE_So, //U+33E7
UNICODE_So, //U+33E8
UNICODE_So, //U+33E9
UNICODE_So, //U+33EA
UNICODE_So, //U+33EB
UNICODE_So, //U+33EC
UNICODE_So, //U+33ED
UNICODE_So, //U+33EE
UNICODE_So, //U+33EF
UNICODE_So, //U+33F0
UNICODE_So, //U+33F1
UNICODE_So, //U+33F2
UNICODE_So, //U+33F3
UNICODE_So, //U+33F4
UNICODE_So, //U+33F5
UNICODE_So, //U+33F6
UNICODE_So, //U+33F7
UNICODE_So, //U+33F8
UNICODE_So, //U+33F9
UNICODE_So, //U+33FA
UNICODE_So, //U+33FB
UNICODE_So, //U+33FC
UNICODE_So, //U+33FD
UNICODE_So, //U+33FE
UNICODE_So, //U+33FF
UNICODE_Lo, //U+3400
0, //U+3401
0, //U+3402
0, //U+3403
0, //U+3404
0, //U+3405
0, //U+3406
0, //U+3407
0, //U+3408
0, //U+3409
0, //U+340A
0, //U+340B
0, //U+340C
0, //U+340D
0, //U+340E
0, //U+340F
0, //U+3410
0, //U+3411
0, //U+3412
0, //U+3413
0, //U+3414
0, //U+3415
0, //U+3416
0, //U+3417
0, //U+3418
0, //U+3419
0, //U+341A
0, //U+341B
0, //U+341C
0, //U+341D
0, //U+341E
0, //U+341F
0, //U+3420
0, //U+3421
0, //U+3422
0, //U+3423
0, //U+3424
0, //U+3425
0, //U+3426
0, //U+3427
0, //U+3428
0, //U+3429
0, //U+342A
0, //U+342B
0, //U+342C
0, //U+342D
0, //U+342E
0, //U+342F
0, //U+3430
0, //U+3431
0, //U+3432
0, //U+3433
0, //U+3434
0, //U+3435
0, //U+3436
0, //U+3437
0, //U+3438
0, //U+3439
0, //U+343A
0, //U+343B
0, //U+343C
0, //U+343D
0, //U+343E
0, //U+343F
0, //U+3440
0, //U+3441
0, //U+3442
0, //U+3443
0, //U+3444
0, //U+3445
0, //U+3446
0, //U+3447
0, //U+3448
0, //U+3449
0, //U+344A
0, //U+344B
0, //U+344C
0, //U+344D
0, //U+344E
0, //U+344F
0, //U+3450
0, //U+3451
0, //U+3452
0, //U+3453
0, //U+3454
0, //U+3455
0, //U+3456
0, //U+3457
0, //U+3458
0, //U+3459
0, //U+345A
0, //U+345B
0, //U+345C
0, //U+345D
0, //U+345E
0, //U+345F
0, //U+3460
0, //U+3461
0, //U+3462
0, //U+3463
0, //U+3464
0, //U+3465
0, //U+3466
0, //U+3467
0, //U+3468
0, //U+3469
0, //U+346A
0, //U+346B
0, //U+346C
0, //U+346D
0, //U+346E
0, //U+346F
0, //U+3470
0, //U+3471
0, //U+3472
0, //U+3473
0, //U+3474
0, //U+3475
0, //U+3476
0, //U+3477
0, //U+3478
0, //U+3479
0, //U+347A
0, //U+347B
0, //U+347C
0, //U+347D
0, //U+347E
0, //U+347F
0, //U+3480
0, //U+3481
0, //U+3482
0, //U+3483
0, //U+3484
0, //U+3485
0, //U+3486
0, //U+3487
0, //U+3488
0, //U+3489
0, //U+348A
0, //U+348B
0, //U+348C
0, //U+348D
0, //U+348E
0, //U+348F
0, //U+3490
0, //U+3491
0, //U+3492
0, //U+3493
0, //U+3494
0, //U+3495
0, //U+3496
0, //U+3497
0, //U+3498
0, //U+3499
0, //U+349A
0, //U+349B
0, //U+349C
0, //U+349D
0, //U+349E
0, //U+349F
0, //U+34A0
0, //U+34A1
0, //U+34A2
0, //U+34A3
0, //U+34A4
0, //U+34A5
0, //U+34A6
0, //U+34A7
0, //U+34A8
0, //U+34A9
0, //U+34AA
0, //U+34AB
0, //U+34AC
0, //U+34AD
0, //U+34AE
0, //U+34AF
0, //U+34B0
0, //U+34B1
0, //U+34B2
0, //U+34B3
0, //U+34B4
0, //U+34B5
0, //U+34B6
0, //U+34B7
0, //U+34B8
0, //U+34B9
0, //U+34BA
0, //U+34BB
0, //U+34BC
0, //U+34BD
0, //U+34BE
0, //U+34BF
0, //U+34C0
0, //U+34C1
0, //U+34C2
0, //U+34C3
0, //U+34C4
0, //U+34C5
0, //U+34C6
0, //U+34C7
0, //U+34C8
0, //U+34C9
0, //U+34CA
0, //U+34CB
0, //U+34CC
0, //U+34CD
0, //U+34CE
0, //U+34CF
0, //U+34D0
0, //U+34D1
0, //U+34D2
0, //U+34D3
0, //U+34D4
0, //U+34D5
0, //U+34D6
0, //U+34D7
0, //U+34D8
0, //U+34D9
0, //U+34DA
0, //U+34DB
0, //U+34DC
0, //U+34DD
0, //U+34DE
0, //U+34DF
0, //U+34E0
0, //U+34E1
0, //U+34E2
0, //U+34E3
0, //U+34E4
0, //U+34E5
0, //U+34E6
0, //U+34E7
0, //U+34E8
0, //U+34E9
0, //U+34EA
0, //U+34EB
0, //U+34EC
0, //U+34ED
0, //U+34EE
0, //U+34EF
0, //U+34F0
0, //U+34F1
0, //U+34F2
0, //U+34F3
0, //U+34F4
0, //U+34F5
0, //U+34F6
0, //U+34F7
0, //U+34F8
0, //U+34F9
0, //U+34FA
0, //U+34FB
0, //U+34FC
0, //U+34FD
0, //U+34FE
0, //U+34FF
0, //U+3500
0, //U+3501
0, //U+3502
0, //U+3503
0, //U+3504
0, //U+3505
0, //U+3506
0, //U+3507
0, //U+3508
0, //U+3509
0, //U+350A
0, //U+350B
0, //U+350C
0, //U+350D
0, //U+350E
0, //U+350F
0, //U+3510
0, //U+3511
0, //U+3512
0, //U+3513
0, //U+3514
0, //U+3515
0, //U+3516
0, //U+3517
0, //U+3518
0, //U+3519
0, //U+351A
0, //U+351B
0, //U+351C
0, //U+351D
0, //U+351E
0, //U+351F
0, //U+3520
0, //U+3521
0, //U+3522
0, //U+3523
0, //U+3524
0, //U+3525
0, //U+3526
0, //U+3527
0, //U+3528
0, //U+3529
0, //U+352A
0, //U+352B
0, //U+352C
0, //U+352D
0, //U+352E
0, //U+352F
0, //U+3530
0, //U+3531
0, //U+3532
0, //U+3533
0, //U+3534
0, //U+3535
0, //U+3536
0, //U+3537
0, //U+3538
0, //U+3539
0, //U+353A
0, //U+353B
0, //U+353C
0, //U+353D
0, //U+353E
0, //U+353F
0, //U+3540
0, //U+3541
0, //U+3542
0, //U+3543
0, //U+3544
0, //U+3545
0, //U+3546
0, //U+3547
0, //U+3548
0, //U+3549
0, //U+354A
0, //U+354B
0, //U+354C
0, //U+354D
0, //U+354E
0, //U+354F
0, //U+3550
0, //U+3551
0, //U+3552
0, //U+3553
0, //U+3554
0, //U+3555
0, //U+3556
0, //U+3557
0, //U+3558
0, //U+3559
0, //U+355A
0, //U+355B
0, //U+355C
0, //U+355D
0, //U+355E
0, //U+355F
0, //U+3560
0, //U+3561
0, //U+3562
0, //U+3563
0, //U+3564
0, //U+3565
0, //U+3566
0, //U+3567
0, //U+3568
0, //U+3569
0, //U+356A
0, //U+356B
0, //U+356C
0, //U+356D
0, //U+356E
0, //U+356F
0, //U+3570
0, //U+3571
0, //U+3572
0, //U+3573
0, //U+3574
0, //U+3575
0, //U+3576
0, //U+3577
0, //U+3578
0, //U+3579
0, //U+357A
0, //U+357B
0, //U+357C
0, //U+357D
0, //U+357E
0, //U+357F
0, //U+3580
0, //U+3581
0, //U+3582
0, //U+3583
0, //U+3584
0, //U+3585
0, //U+3586
0, //U+3587
0, //U+3588
0, //U+3589
0, //U+358A
0, //U+358B
0, //U+358C
0, //U+358D
0, //U+358E
0, //U+358F
0, //U+3590
0, //U+3591
0, //U+3592
0, //U+3593
0, //U+3594
0, //U+3595
0, //U+3596
0, //U+3597
0, //U+3598
0, //U+3599
0, //U+359A
0, //U+359B
0, //U+359C
0, //U+359D
0, //U+359E
0, //U+359F
0, //U+35A0
0, //U+35A1
0, //U+35A2
0, //U+35A3
0, //U+35A4
0, //U+35A5
0, //U+35A6
0, //U+35A7
0, //U+35A8
0, //U+35A9
0, //U+35AA
0, //U+35AB
0, //U+35AC
0, //U+35AD
0, //U+35AE
0, //U+35AF
0, //U+35B0
0, //U+35B1
0, //U+35B2
0, //U+35B3
0, //U+35B4
0, //U+35B5
0, //U+35B6
0, //U+35B7
0, //U+35B8
0, //U+35B9
0, //U+35BA
0, //U+35BB
0, //U+35BC
0, //U+35BD
0, //U+35BE
0, //U+35BF
0, //U+35C0
0, //U+35C1
0, //U+35C2
0, //U+35C3
0, //U+35C4
0, //U+35C5
0, //U+35C6
0, //U+35C7
0, //U+35C8
0, //U+35C9
0, //U+35CA
0, //U+35CB
0, //U+35CC
0, //U+35CD
0, //U+35CE
0, //U+35CF
0, //U+35D0
0, //U+35D1
0, //U+35D2
0, //U+35D3
0, //U+35D4
0, //U+35D5
0, //U+35D6
0, //U+35D7
0, //U+35D8
0, //U+35D9
0, //U+35DA
0, //U+35DB
0, //U+35DC
0, //U+35DD
0, //U+35DE
0, //U+35DF
0, //U+35E0
0, //U+35E1
0, //U+35E2
0, //U+35E3
0, //U+35E4
0, //U+35E5
0, //U+35E6
0, //U+35E7
0, //U+35E8
0, //U+35E9
0, //U+35EA
0, //U+35EB
0, //U+35EC
0, //U+35ED
0, //U+35EE
0, //U+35EF
0, //U+35F0
0, //U+35F1
0, //U+35F2
0, //U+35F3
0, //U+35F4
0, //U+35F5
0, //U+35F6
0, //U+35F7
0, //U+35F8
0, //U+35F9
0, //U+35FA
0, //U+35FB
0, //U+35FC
0, //U+35FD
0, //U+35FE
0, //U+35FF
0, //U+3600
0, //U+3601
0, //U+3602
0, //U+3603
0, //U+3604
0, //U+3605
0, //U+3606
0, //U+3607
0, //U+3608
0, //U+3609
0, //U+360A
0, //U+360B
0, //U+360C
0, //U+360D
0, //U+360E
0, //U+360F
0, //U+3610
0, //U+3611
0, //U+3612
0, //U+3613
0, //U+3614
0, //U+3615
0, //U+3616
0, //U+3617
0, //U+3618
0, //U+3619
0, //U+361A
0, //U+361B
0, //U+361C
0, //U+361D
0, //U+361E
0, //U+361F
0, //U+3620
0, //U+3621
0, //U+3622
0, //U+3623
0, //U+3624
0, //U+3625
0, //U+3626
0, //U+3627
0, //U+3628
0, //U+3629
0, //U+362A
0, //U+362B
0, //U+362C
0, //U+362D
0, //U+362E
0, //U+362F
0, //U+3630
0, //U+3631
0, //U+3632
0, //U+3633
0, //U+3634
0, //U+3635
0, //U+3636
0, //U+3637
0, //U+3638
0, //U+3639
0, //U+363A
0, //U+363B
0, //U+363C
0, //U+363D
0, //U+363E
0, //U+363F
0, //U+3640
0, //U+3641
0, //U+3642
0, //U+3643
0, //U+3644
0, //U+3645
0, //U+3646
0, //U+3647
0, //U+3648
0, //U+3649
0, //U+364A
0, //U+364B
0, //U+364C
0, //U+364D
0, //U+364E
0, //U+364F
0, //U+3650
0, //U+3651
0, //U+3652
0, //U+3653
0, //U+3654
0, //U+3655
0, //U+3656
0, //U+3657
0, //U+3658
0, //U+3659
0, //U+365A
0, //U+365B
0, //U+365C
0, //U+365D
0, //U+365E
0, //U+365F
0, //U+3660
0, //U+3661
0, //U+3662
0, //U+3663
0, //U+3664
0, //U+3665
0, //U+3666
0, //U+3667
0, //U+3668
0, //U+3669
0, //U+366A
0, //U+366B
0, //U+366C
0, //U+366D
0, //U+366E
0, //U+366F
0, //U+3670
0, //U+3671
0, //U+3672
0, //U+3673
0, //U+3674
0, //U+3675
0, //U+3676
0, //U+3677
0, //U+3678
0, //U+3679
0, //U+367A
0, //U+367B
0, //U+367C
0, //U+367D
0, //U+367E
0, //U+367F
0, //U+3680
0, //U+3681
0, //U+3682
0, //U+3683
0, //U+3684
0, //U+3685
0, //U+3686
0, //U+3687
0, //U+3688
0, //U+3689
0, //U+368A
0, //U+368B
0, //U+368C
0, //U+368D
0, //U+368E
0, //U+368F
0, //U+3690
0, //U+3691
0, //U+3692
0, //U+3693
0, //U+3694
0, //U+3695
0, //U+3696
0, //U+3697
0, //U+3698
0, //U+3699
0, //U+369A
0, //U+369B
0, //U+369C
0, //U+369D
0, //U+369E
0, //U+369F
0, //U+36A0
0, //U+36A1
0, //U+36A2
0, //U+36A3
0, //U+36A4
0, //U+36A5
0, //U+36A6
0, //U+36A7
0, //U+36A8
0, //U+36A9
0, //U+36AA
0, //U+36AB
0, //U+36AC
0, //U+36AD
0, //U+36AE
0, //U+36AF
0, //U+36B0
0, //U+36B1
0, //U+36B2
0, //U+36B3
0, //U+36B4
0, //U+36B5
0, //U+36B6
0, //U+36B7
0, //U+36B8
0, //U+36B9
0, //U+36BA
0, //U+36BB
0, //U+36BC
0, //U+36BD
0, //U+36BE
0, //U+36BF
0, //U+36C0
0, //U+36C1
0, //U+36C2
0, //U+36C3
0, //U+36C4
0, //U+36C5
0, //U+36C6
0, //U+36C7
0, //U+36C8
0, //U+36C9
0, //U+36CA
0, //U+36CB
0, //U+36CC
0, //U+36CD
0, //U+36CE
0, //U+36CF
0, //U+36D0
0, //U+36D1
0, //U+36D2
0, //U+36D3
0, //U+36D4
0, //U+36D5
0, //U+36D6
0, //U+36D7
0, //U+36D8
0, //U+36D9
0, //U+36DA
0, //U+36DB
0, //U+36DC
0, //U+36DD
0, //U+36DE
0, //U+36DF
0, //U+36E0
0, //U+36E1
0, //U+36E2
0, //U+36E3
0, //U+36E4
0, //U+36E5
0, //U+36E6
0, //U+36E7
0, //U+36E8
0, //U+36E9
0, //U+36EA
0, //U+36EB
0, //U+36EC
0, //U+36ED
0, //U+36EE
0, //U+36EF
0, //U+36F0
0, //U+36F1
0, //U+36F2
0, //U+36F3
0, //U+36F4
0, //U+36F5
0, //U+36F6
0, //U+36F7
0, //U+36F8
0, //U+36F9
0, //U+36FA
0, //U+36FB
0, //U+36FC
0, //U+36FD
0, //U+36FE
0, //U+36FF
0, //U+3700
0, //U+3701
0, //U+3702
0, //U+3703
0, //U+3704
0, //U+3705
0, //U+3706
0, //U+3707
0, //U+3708
0, //U+3709
0, //U+370A
0, //U+370B
0, //U+370C
0, //U+370D
0, //U+370E
0, //U+370F
0, //U+3710
0, //U+3711
0, //U+3712
0, //U+3713
0, //U+3714
0, //U+3715
0, //U+3716
0, //U+3717
0, //U+3718
0, //U+3719
0, //U+371A
0, //U+371B
0, //U+371C
0, //U+371D
0, //U+371E
0, //U+371F
0, //U+3720
0, //U+3721
0, //U+3722
0, //U+3723
0, //U+3724
0, //U+3725
0, //U+3726
0, //U+3727
0, //U+3728
0, //U+3729
0, //U+372A
0, //U+372B
0, //U+372C
0, //U+372D
0, //U+372E
0, //U+372F
0, //U+3730
0, //U+3731
0, //U+3732
0, //U+3733
0, //U+3734
0, //U+3735
0, //U+3736
0, //U+3737
0, //U+3738
0, //U+3739
0, //U+373A
0, //U+373B
0, //U+373C
0, //U+373D
0, //U+373E
0, //U+373F
0, //U+3740
0, //U+3741
0, //U+3742
0, //U+3743
0, //U+3744
0, //U+3745
0, //U+3746
0, //U+3747
0, //U+3748
0, //U+3749
0, //U+374A
0, //U+374B
0, //U+374C
0, //U+374D
0, //U+374E
0, //U+374F
0, //U+3750
0, //U+3751
0, //U+3752
0, //U+3753
0, //U+3754
0, //U+3755
0, //U+3756
0, //U+3757
0, //U+3758
0, //U+3759
0, //U+375A
0, //U+375B
0, //U+375C
0, //U+375D
0, //U+375E
0, //U+375F
0, //U+3760
0, //U+3761
0, //U+3762
0, //U+3763
0, //U+3764
0, //U+3765
0, //U+3766
0, //U+3767
0, //U+3768
0, //U+3769
0, //U+376A
0, //U+376B
0, //U+376C
0, //U+376D
0, //U+376E
0, //U+376F
0, //U+3770
0, //U+3771
0, //U+3772
0, //U+3773
0, //U+3774
0, //U+3775
0, //U+3776
0, //U+3777
0, //U+3778
0, //U+3779
0, //U+377A
0, //U+377B
0, //U+377C
0, //U+377D
0, //U+377E
0, //U+377F
0, //U+3780
0, //U+3781
0, //U+3782
0, //U+3783
0, //U+3784
0, //U+3785
0, //U+3786
0, //U+3787
0, //U+3788
0, //U+3789
0, //U+378A
0, //U+378B
0, //U+378C
0, //U+378D
0, //U+378E
0, //U+378F
0, //U+3790
0, //U+3791
0, //U+3792
0, //U+3793
0, //U+3794
0, //U+3795
0, //U+3796
0, //U+3797
0, //U+3798
0, //U+3799
0, //U+379A
0, //U+379B
0, //U+379C
0, //U+379D
0, //U+379E
0, //U+379F
0, //U+37A0
0, //U+37A1
0, //U+37A2
0, //U+37A3
0, //U+37A4
0, //U+37A5
0, //U+37A6
0, //U+37A7
0, //U+37A8
0, //U+37A9
0, //U+37AA
0, //U+37AB
0, //U+37AC
0, //U+37AD
0, //U+37AE
0, //U+37AF
0, //U+37B0
0, //U+37B1
0, //U+37B2
0, //U+37B3
0, //U+37B4
0, //U+37B5
0, //U+37B6
0, //U+37B7
0, //U+37B8
0, //U+37B9
0, //U+37BA
0, //U+37BB
0, //U+37BC
0, //U+37BD
0, //U+37BE
0, //U+37BF
0, //U+37C0
0, //U+37C1
0, //U+37C2
0, //U+37C3
0, //U+37C4
0, //U+37C5
0, //U+37C6
0, //U+37C7
0, //U+37C8
0, //U+37C9
0, //U+37CA
0, //U+37CB
0, //U+37CC
0, //U+37CD
0, //U+37CE
0, //U+37CF
0, //U+37D0
0, //U+37D1
0, //U+37D2
0, //U+37D3
0, //U+37D4
0, //U+37D5
0, //U+37D6
0, //U+37D7
0, //U+37D8
0, //U+37D9
0, //U+37DA
0, //U+37DB
0, //U+37DC
0, //U+37DD
0, //U+37DE
0, //U+37DF
0, //U+37E0
0, //U+37E1
0, //U+37E2
0, //U+37E3
0, //U+37E4
0, //U+37E5
0, //U+37E6
0, //U+37E7
0, //U+37E8
0, //U+37E9
0, //U+37EA
0, //U+37EB
0, //U+37EC
0, //U+37ED
0, //U+37EE
0, //U+37EF
0, //U+37F0
0, //U+37F1
0, //U+37F2
0, //U+37F3
0, //U+37F4
0, //U+37F5
0, //U+37F6
0, //U+37F7
0, //U+37F8
0, //U+37F9
0, //U+37FA
0, //U+37FB
0, //U+37FC
0, //U+37FD
0, //U+37FE
0, //U+37FF
0, //U+3800
0, //U+3801
0, //U+3802
0, //U+3803
0, //U+3804
0, //U+3805
0, //U+3806
0, //U+3807
0, //U+3808
0, //U+3809
0, //U+380A
0, //U+380B
0, //U+380C
0, //U+380D
0, //U+380E
0, //U+380F
0, //U+3810
0, //U+3811
0, //U+3812
0, //U+3813
0, //U+3814
0, //U+3815
0, //U+3816
0, //U+3817
0, //U+3818
0, //U+3819
0, //U+381A
0, //U+381B
0, //U+381C
0, //U+381D
0, //U+381E
0, //U+381F
0, //U+3820
0, //U+3821
0, //U+3822
0, //U+3823
0, //U+3824
0, //U+3825
0, //U+3826
0, //U+3827
0, //U+3828
0, //U+3829
0, //U+382A
0, //U+382B
0, //U+382C
0, //U+382D
0, //U+382E
0, //U+382F
0, //U+3830
0, //U+3831
0, //U+3832
0, //U+3833
0, //U+3834
0, //U+3835
0, //U+3836
0, //U+3837
0, //U+3838
0, //U+3839
0, //U+383A
0, //U+383B
0, //U+383C
0, //U+383D
0, //U+383E
0, //U+383F
0, //U+3840
0, //U+3841
0, //U+3842
0, //U+3843
0, //U+3844
0, //U+3845
0, //U+3846
0, //U+3847
0, //U+3848
0, //U+3849
0, //U+384A
0, //U+384B
0, //U+384C
0, //U+384D
0, //U+384E
0, //U+384F
0, //U+3850
0, //U+3851
0, //U+3852
0, //U+3853
0, //U+3854
0, //U+3855
0, //U+3856
0, //U+3857
0, //U+3858
0, //U+3859
0, //U+385A
0, //U+385B
0, //U+385C
0, //U+385D
0, //U+385E
0, //U+385F
0, //U+3860
0, //U+3861
0, //U+3862
0, //U+3863
0, //U+3864
0, //U+3865
0, //U+3866
0, //U+3867
0, //U+3868
0, //U+3869
0, //U+386A
0, //U+386B
0, //U+386C
0, //U+386D
0, //U+386E
0, //U+386F
0, //U+3870
0, //U+3871
0, //U+3872
0, //U+3873
0, //U+3874
0, //U+3875
0, //U+3876
0, //U+3877
0, //U+3878
0, //U+3879
0, //U+387A
0, //U+387B
0, //U+387C
0, //U+387D
0, //U+387E
0, //U+387F
0, //U+3880
0, //U+3881
0, //U+3882
0, //U+3883
0, //U+3884
0, //U+3885
0, //U+3886
0, //U+3887
0, //U+3888
0, //U+3889
0, //U+388A
0, //U+388B
0, //U+388C
0, //U+388D
0, //U+388E
0, //U+388F
0, //U+3890
0, //U+3891
0, //U+3892
0, //U+3893
0, //U+3894
0, //U+3895
0, //U+3896
0, //U+3897
0, //U+3898
0, //U+3899
0, //U+389A
0, //U+389B
0, //U+389C
0, //U+389D
0, //U+389E
0, //U+389F
0, //U+38A0
0, //U+38A1
0, //U+38A2
0, //U+38A3
0, //U+38A4
0, //U+38A5
0, //U+38A6
0, //U+38A7
0, //U+38A8
0, //U+38A9
0, //U+38AA
0, //U+38AB
0, //U+38AC
0, //U+38AD
0, //U+38AE
0, //U+38AF
0, //U+38B0
0, //U+38B1
0, //U+38B2
0, //U+38B3
0, //U+38B4
0, //U+38B5
0, //U+38B6
0, //U+38B7
0, //U+38B8
0, //U+38B9
0, //U+38BA
0, //U+38BB
0, //U+38BC
0, //U+38BD
0, //U+38BE
0, //U+38BF
0, //U+38C0
0, //U+38C1
0, //U+38C2
0, //U+38C3
0, //U+38C4
0, //U+38C5
0, //U+38C6
0, //U+38C7
0, //U+38C8
0, //U+38C9
0, //U+38CA
0, //U+38CB
0, //U+38CC
0, //U+38CD
0, //U+38CE
0, //U+38CF
0, //U+38D0
0, //U+38D1
0, //U+38D2
0, //U+38D3
0, //U+38D4
0, //U+38D5
0, //U+38D6
0, //U+38D7
0, //U+38D8
0, //U+38D9
0, //U+38DA
0, //U+38DB
0, //U+38DC
0, //U+38DD
0, //U+38DE
0, //U+38DF
0, //U+38E0
0, //U+38E1
0, //U+38E2
0, //U+38E3
0, //U+38E4
0, //U+38E5
0, //U+38E6
0, //U+38E7
0, //U+38E8
0, //U+38E9
0, //U+38EA
0, //U+38EB
0, //U+38EC
0, //U+38ED
0, //U+38EE
0, //U+38EF
0, //U+38F0
0, //U+38F1
0, //U+38F2
0, //U+38F3
0, //U+38F4
0, //U+38F5
0, //U+38F6
0, //U+38F7
0, //U+38F8
0, //U+38F9
0, //U+38FA
0, //U+38FB
0, //U+38FC
0, //U+38FD
0, //U+38FE
0, //U+38FF
0, //U+3900
0, //U+3901
0, //U+3902
0, //U+3903
0, //U+3904
0, //U+3905
0, //U+3906
0, //U+3907
0, //U+3908
0, //U+3909
0, //U+390A
0, //U+390B
0, //U+390C
0, //U+390D
0, //U+390E
0, //U+390F
0, //U+3910
0, //U+3911
0, //U+3912
0, //U+3913
0, //U+3914
0, //U+3915
0, //U+3916
0, //U+3917
0, //U+3918
0, //U+3919
0, //U+391A
0, //U+391B
0, //U+391C
0, //U+391D
0, //U+391E
0, //U+391F
0, //U+3920
0, //U+3921
0, //U+3922
0, //U+3923
0, //U+3924
0, //U+3925
0, //U+3926
0, //U+3927
0, //U+3928
0, //U+3929
0, //U+392A
0, //U+392B
0, //U+392C
0, //U+392D
0, //U+392E
0, //U+392F
0, //U+3930
0, //U+3931
0, //U+3932
0, //U+3933
0, //U+3934
0, //U+3935
0, //U+3936
0, //U+3937
0, //U+3938
0, //U+3939
0, //U+393A
0, //U+393B
0, //U+393C
0, //U+393D
0, //U+393E
0, //U+393F
0, //U+3940
0, //U+3941
0, //U+3942
0, //U+3943
0, //U+3944
0, //U+3945
0, //U+3946
0, //U+3947
0, //U+3948
0, //U+3949
0, //U+394A
0, //U+394B
0, //U+394C
0, //U+394D
0, //U+394E
0, //U+394F
0, //U+3950
0, //U+3951
0, //U+3952
0, //U+3953
0, //U+3954
0, //U+3955
0, //U+3956
0, //U+3957
0, //U+3958
0, //U+3959
0, //U+395A
0, //U+395B
0, //U+395C
0, //U+395D
0, //U+395E
0, //U+395F
0, //U+3960
0, //U+3961
0, //U+3962
0, //U+3963
0, //U+3964
0, //U+3965
0, //U+3966
0, //U+3967
0, //U+3968
0, //U+3969
0, //U+396A
0, //U+396B
0, //U+396C
0, //U+396D
0, //U+396E
0, //U+396F
0, //U+3970
0, //U+3971
0, //U+3972
0, //U+3973
0, //U+3974
0, //U+3975
0, //U+3976
0, //U+3977
0, //U+3978
0, //U+3979
0, //U+397A
0, //U+397B
0, //U+397C
0, //U+397D
0, //U+397E
0, //U+397F
0, //U+3980
0, //U+3981
0, //U+3982
0, //U+3983
0, //U+3984
0, //U+3985
0, //U+3986
0, //U+3987
0, //U+3988
0, //U+3989
0, //U+398A
0, //U+398B
0, //U+398C
0, //U+398D
0, //U+398E
0, //U+398F
0, //U+3990
0, //U+3991
0, //U+3992
0, //U+3993
0, //U+3994
0, //U+3995
0, //U+3996
0, //U+3997
0, //U+3998
0, //U+3999
0, //U+399A
0, //U+399B
0, //U+399C
0, //U+399D
0, //U+399E
0, //U+399F
0, //U+39A0
0, //U+39A1
0, //U+39A2
0, //U+39A3
0, //U+39A4
0, //U+39A5
0, //U+39A6
0, //U+39A7
0, //U+39A8
0, //U+39A9
0, //U+39AA
0, //U+39AB
0, //U+39AC
0, //U+39AD
0, //U+39AE
0, //U+39AF
0, //U+39B0
0, //U+39B1
0, //U+39B2
0, //U+39B3
0, //U+39B4
0, //U+39B5
0, //U+39B6
0, //U+39B7
0, //U+39B8
0, //U+39B9
0, //U+39BA
0, //U+39BB
0, //U+39BC
0, //U+39BD
0, //U+39BE
0, //U+39BF
0, //U+39C0
0, //U+39C1
0, //U+39C2
0, //U+39C3
0, //U+39C4
0, //U+39C5
0, //U+39C6
0, //U+39C7
0, //U+39C8
0, //U+39C9
0, //U+39CA
0, //U+39CB
0, //U+39CC
0, //U+39CD
0, //U+39CE
0, //U+39CF
0, //U+39D0
0, //U+39D1
0, //U+39D2
0, //U+39D3
0, //U+39D4
0, //U+39D5
0, //U+39D6
0, //U+39D7
0, //U+39D8
0, //U+39D9
0, //U+39DA
0, //U+39DB
0, //U+39DC
0, //U+39DD
0, //U+39DE
0, //U+39DF
0, //U+39E0
0, //U+39E1
0, //U+39E2
0, //U+39E3
0, //U+39E4
0, //U+39E5
0, //U+39E6
0, //U+39E7
0, //U+39E8
0, //U+39E9
0, //U+39EA
0, //U+39EB
0, //U+39EC
0, //U+39ED
0, //U+39EE
0, //U+39EF
0, //U+39F0
0, //U+39F1
0, //U+39F2
0, //U+39F3
0, //U+39F4
0, //U+39F5
0, //U+39F6
0, //U+39F7
0, //U+39F8
0, //U+39F9
0, //U+39FA
0, //U+39FB
0, //U+39FC
0, //U+39FD
0, //U+39FE
0, //U+39FF
0, //U+3A00
0, //U+3A01
0, //U+3A02
0, //U+3A03
0, //U+3A04
0, //U+3A05
0, //U+3A06
0, //U+3A07
0, //U+3A08
0, //U+3A09
0, //U+3A0A
0, //U+3A0B
0, //U+3A0C
0, //U+3A0D
0, //U+3A0E
0, //U+3A0F
0, //U+3A10
0, //U+3A11
0, //U+3A12
0, //U+3A13
0, //U+3A14
0, //U+3A15
0, //U+3A16
0, //U+3A17
0, //U+3A18
0, //U+3A19
0, //U+3A1A
0, //U+3A1B
0, //U+3A1C
0, //U+3A1D
0, //U+3A1E
0, //U+3A1F
0, //U+3A20
0, //U+3A21
0, //U+3A22
0, //U+3A23
0, //U+3A24
0, //U+3A25
0, //U+3A26
0, //U+3A27
0, //U+3A28
0, //U+3A29
0, //U+3A2A
0, //U+3A2B
0, //U+3A2C
0, //U+3A2D
0, //U+3A2E
0, //U+3A2F
0, //U+3A30
0, //U+3A31
0, //U+3A32
0, //U+3A33
0, //U+3A34
0, //U+3A35
0, //U+3A36
0, //U+3A37
0, //U+3A38
0, //U+3A39
0, //U+3A3A
0, //U+3A3B
0, //U+3A3C
0, //U+3A3D
0, //U+3A3E
0, //U+3A3F
0, //U+3A40
0, //U+3A41
0, //U+3A42
0, //U+3A43
0, //U+3A44
0, //U+3A45
0, //U+3A46
0, //U+3A47
0, //U+3A48
0, //U+3A49
0, //U+3A4A
0, //U+3A4B
0, //U+3A4C
0, //U+3A4D
0, //U+3A4E
0, //U+3A4F
0, //U+3A50
0, //U+3A51
0, //U+3A52
0, //U+3A53
0, //U+3A54
0, //U+3A55
0, //U+3A56
0, //U+3A57
0, //U+3A58
0, //U+3A59
0, //U+3A5A
0, //U+3A5B
0, //U+3A5C
0, //U+3A5D
0, //U+3A5E
0, //U+3A5F
0, //U+3A60
0, //U+3A61
0, //U+3A62
0, //U+3A63
0, //U+3A64
0, //U+3A65
0, //U+3A66
0, //U+3A67
0, //U+3A68
0, //U+3A69
0, //U+3A6A
0, //U+3A6B
0, //U+3A6C
0, //U+3A6D
0, //U+3A6E
0, //U+3A6F
0, //U+3A70
0, //U+3A71
0, //U+3A72
0, //U+3A73
0, //U+3A74
0, //U+3A75
0, //U+3A76
0, //U+3A77
0, //U+3A78
0, //U+3A79
0, //U+3A7A
0, //U+3A7B
0, //U+3A7C
0, //U+3A7D
0, //U+3A7E
0, //U+3A7F
0, //U+3A80
0, //U+3A81
0, //U+3A82
0, //U+3A83
0, //U+3A84
0, //U+3A85
0, //U+3A86
0, //U+3A87
0, //U+3A88
0, //U+3A89
0, //U+3A8A
0, //U+3A8B
0, //U+3A8C
0, //U+3A8D
0, //U+3A8E
0, //U+3A8F
0, //U+3A90
0, //U+3A91
0, //U+3A92
0, //U+3A93
0, //U+3A94
0, //U+3A95
0, //U+3A96
0, //U+3A97
0, //U+3A98
0, //U+3A99
0, //U+3A9A
0, //U+3A9B
0, //U+3A9C
0, //U+3A9D
0, //U+3A9E
0, //U+3A9F
0, //U+3AA0
0, //U+3AA1
0, //U+3AA2
0, //U+3AA3
0, //U+3AA4
0, //U+3AA5
0, //U+3AA6
0, //U+3AA7
0, //U+3AA8
0, //U+3AA9
0, //U+3AAA
0, //U+3AAB
0, //U+3AAC
0, //U+3AAD
0, //U+3AAE
0, //U+3AAF
0, //U+3AB0
0, //U+3AB1
0, //U+3AB2
0, //U+3AB3
0, //U+3AB4
0, //U+3AB5
0, //U+3AB6
0, //U+3AB7
0, //U+3AB8
0, //U+3AB9
0, //U+3ABA
0, //U+3ABB
0, //U+3ABC
0, //U+3ABD
0, //U+3ABE
0, //U+3ABF
0, //U+3AC0
0, //U+3AC1
0, //U+3AC2
0, //U+3AC3
0, //U+3AC4
0, //U+3AC5
0, //U+3AC6
0, //U+3AC7
0, //U+3AC8
0, //U+3AC9
0, //U+3ACA
0, //U+3ACB
0, //U+3ACC
0, //U+3ACD
0, //U+3ACE
0, //U+3ACF
0, //U+3AD0
0, //U+3AD1
0, //U+3AD2
0, //U+3AD3
0, //U+3AD4
0, //U+3AD5
0, //U+3AD6
0, //U+3AD7
0, //U+3AD8
0, //U+3AD9
0, //U+3ADA
0, //U+3ADB
0, //U+3ADC
0, //U+3ADD
0, //U+3ADE
0, //U+3ADF
0, //U+3AE0
0, //U+3AE1
0, //U+3AE2
0, //U+3AE3
0, //U+3AE4
0, //U+3AE5
0, //U+3AE6
0, //U+3AE7
0, //U+3AE8
0, //U+3AE9
0, //U+3AEA
0, //U+3AEB
0, //U+3AEC
0, //U+3AED
0, //U+3AEE
0, //U+3AEF
0, //U+3AF0
0, //U+3AF1
0, //U+3AF2
0, //U+3AF3
0, //U+3AF4
0, //U+3AF5
0, //U+3AF6
0, //U+3AF7
0, //U+3AF8
0, //U+3AF9
0, //U+3AFA
0, //U+3AFB
0, //U+3AFC
0, //U+3AFD
0, //U+3AFE
0, //U+3AFF
0, //U+3B00
0, //U+3B01
0, //U+3B02
0, //U+3B03
0, //U+3B04
0, //U+3B05
0, //U+3B06
0, //U+3B07
0, //U+3B08
0, //U+3B09
0, //U+3B0A
0, //U+3B0B
0, //U+3B0C
0, //U+3B0D
0, //U+3B0E
0, //U+3B0F
0, //U+3B10
0, //U+3B11
0, //U+3B12
0, //U+3B13
0, //U+3B14
0, //U+3B15
0, //U+3B16
0, //U+3B17
0, //U+3B18
0, //U+3B19
0, //U+3B1A
0, //U+3B1B
0, //U+3B1C
0, //U+3B1D
0, //U+3B1E
0, //U+3B1F
0, //U+3B20
0, //U+3B21
0, //U+3B22
0, //U+3B23
0, //U+3B24
0, //U+3B25
0, //U+3B26
0, //U+3B27
0, //U+3B28
0, //U+3B29
0, //U+3B2A
0, //U+3B2B
0, //U+3B2C
0, //U+3B2D
0, //U+3B2E
0, //U+3B2F
0, //U+3B30
0, //U+3B31
0, //U+3B32
0, //U+3B33
0, //U+3B34
0, //U+3B35
0, //U+3B36
0, //U+3B37
0, //U+3B38
0, //U+3B39
0, //U+3B3A
0, //U+3B3B
0, //U+3B3C
0, //U+3B3D
0, //U+3B3E
0, //U+3B3F
0, //U+3B40
0, //U+3B41
0, //U+3B42
0, //U+3B43
0, //U+3B44
0, //U+3B45
0, //U+3B46
0, //U+3B47
0, //U+3B48
0, //U+3B49
0, //U+3B4A
0, //U+3B4B
0, //U+3B4C
0, //U+3B4D
0, //U+3B4E
0, //U+3B4F
0, //U+3B50
0, //U+3B51
0, //U+3B52
0, //U+3B53
0, //U+3B54
0, //U+3B55
0, //U+3B56
0, //U+3B57
0, //U+3B58
0, //U+3B59
0, //U+3B5A
0, //U+3B5B
0, //U+3B5C
0, //U+3B5D
0, //U+3B5E
0, //U+3B5F
0, //U+3B60
0, //U+3B61
0, //U+3B62
0, //U+3B63
0, //U+3B64
0, //U+3B65
0, //U+3B66
0, //U+3B67
0, //U+3B68
0, //U+3B69
0, //U+3B6A
0, //U+3B6B
0, //U+3B6C
0, //U+3B6D
0, //U+3B6E
0, //U+3B6F
0, //U+3B70
0, //U+3B71
0, //U+3B72
0, //U+3B73
0, //U+3B74
0, //U+3B75
0, //U+3B76
0, //U+3B77
0, //U+3B78
0, //U+3B79
0, //U+3B7A
0, //U+3B7B
0, //U+3B7C
0, //U+3B7D
0, //U+3B7E
0, //U+3B7F
0, //U+3B80
0, //U+3B81
0, //U+3B82
0, //U+3B83
0, //U+3B84
0, //U+3B85
0, //U+3B86
0, //U+3B87
0, //U+3B88
0, //U+3B89
0, //U+3B8A
0, //U+3B8B
0, //U+3B8C
0, //U+3B8D
0, //U+3B8E
0, //U+3B8F
0, //U+3B90
0, //U+3B91
0, //U+3B92
0, //U+3B93
0, //U+3B94
0, //U+3B95
0, //U+3B96
0, //U+3B97
0, //U+3B98
0, //U+3B99
0, //U+3B9A
0, //U+3B9B
0, //U+3B9C
0, //U+3B9D
0, //U+3B9E
0, //U+3B9F
0, //U+3BA0
0, //U+3BA1
0, //U+3BA2
0, //U+3BA3
0, //U+3BA4
0, //U+3BA5
0, //U+3BA6
0, //U+3BA7
0, //U+3BA8
0, //U+3BA9
0, //U+3BAA
0, //U+3BAB
0, //U+3BAC
0, //U+3BAD
0, //U+3BAE
0, //U+3BAF
0, //U+3BB0
0, //U+3BB1
0, //U+3BB2
0, //U+3BB3
0, //U+3BB4
0, //U+3BB5
0, //U+3BB6
0, //U+3BB7
0, //U+3BB8
0, //U+3BB9
0, //U+3BBA
0, //U+3BBB
0, //U+3BBC
0, //U+3BBD
0, //U+3BBE
0, //U+3BBF
0, //U+3BC0
0, //U+3BC1
0, //U+3BC2
0, //U+3BC3
0, //U+3BC4
0, //U+3BC5
0, //U+3BC6
0, //U+3BC7
0, //U+3BC8
0, //U+3BC9
0, //U+3BCA
0, //U+3BCB
0, //U+3BCC
0, //U+3BCD
0, //U+3BCE
0, //U+3BCF
0, //U+3BD0
0, //U+3BD1
0, //U+3BD2
0, //U+3BD3
0, //U+3BD4
0, //U+3BD5
0, //U+3BD6
0, //U+3BD7
0, //U+3BD8
0, //U+3BD9
0, //U+3BDA
0, //U+3BDB
0, //U+3BDC
0, //U+3BDD
0, //U+3BDE
0, //U+3BDF
0, //U+3BE0
0, //U+3BE1
0, //U+3BE2
0, //U+3BE3
0, //U+3BE4
0, //U+3BE5
0, //U+3BE6
0, //U+3BE7
0, //U+3BE8
0, //U+3BE9
0, //U+3BEA
0, //U+3BEB
0, //U+3BEC
0, //U+3BED
0, //U+3BEE
0, //U+3BEF
0, //U+3BF0
0, //U+3BF1
0, //U+3BF2
0, //U+3BF3
0, //U+3BF4
0, //U+3BF5
0, //U+3BF6
0, //U+3BF7
0, //U+3BF8
0, //U+3BF9
0, //U+3BFA
0, //U+3BFB
0, //U+3BFC
0, //U+3BFD
0, //U+3BFE
0, //U+3BFF
0, //U+3C00
0, //U+3C01
0, //U+3C02
0, //U+3C03
0, //U+3C04
0, //U+3C05
0, //U+3C06
0, //U+3C07
0, //U+3C08
0, //U+3C09
0, //U+3C0A
0, //U+3C0B
0, //U+3C0C
0, //U+3C0D
0, //U+3C0E
0, //U+3C0F
0, //U+3C10
0, //U+3C11
0, //U+3C12
0, //U+3C13
0, //U+3C14
0, //U+3C15
0, //U+3C16
0, //U+3C17
0, //U+3C18
0, //U+3C19
0, //U+3C1A
0, //U+3C1B
0, //U+3C1C
0, //U+3C1D
0, //U+3C1E
0, //U+3C1F
0, //U+3C20
0, //U+3C21
0, //U+3C22
0, //U+3C23
0, //U+3C24
0, //U+3C25
0, //U+3C26
0, //U+3C27
0, //U+3C28
0, //U+3C29
0, //U+3C2A
0, //U+3C2B
0, //U+3C2C
0, //U+3C2D
0, //U+3C2E
0, //U+3C2F
0, //U+3C30
0, //U+3C31
0, //U+3C32
0, //U+3C33
0, //U+3C34
0, //U+3C35
0, //U+3C36
0, //U+3C37
0, //U+3C38
0, //U+3C39
0, //U+3C3A
0, //U+3C3B
0, //U+3C3C
0, //U+3C3D
0, //U+3C3E
0, //U+3C3F
0, //U+3C40
0, //U+3C41
0, //U+3C42
0, //U+3C43
0, //U+3C44
0, //U+3C45
0, //U+3C46
0, //U+3C47
0, //U+3C48
0, //U+3C49
0, //U+3C4A
0, //U+3C4B
0, //U+3C4C
0, //U+3C4D
0, //U+3C4E
0, //U+3C4F
0, //U+3C50
0, //U+3C51
0, //U+3C52
0, //U+3C53
0, //U+3C54
0, //U+3C55
0, //U+3C56
0, //U+3C57
0, //U+3C58
0, //U+3C59
0, //U+3C5A
0, //U+3C5B
0, //U+3C5C
0, //U+3C5D
0, //U+3C5E
0, //U+3C5F
0, //U+3C60
0, //U+3C61
0, //U+3C62
0, //U+3C63
0, //U+3C64
0, //U+3C65
0, //U+3C66
0, //U+3C67
0, //U+3C68
0, //U+3C69
0, //U+3C6A
0, //U+3C6B
0, //U+3C6C
0, //U+3C6D
0, //U+3C6E
0, //U+3C6F
0, //U+3C70
0, //U+3C71
0, //U+3C72
0, //U+3C73
0, //U+3C74
0, //U+3C75
0, //U+3C76
0, //U+3C77
0, //U+3C78
0, //U+3C79
0, //U+3C7A
0, //U+3C7B
0, //U+3C7C
0, //U+3C7D
0, //U+3C7E
0, //U+3C7F
0, //U+3C80
0, //U+3C81
0, //U+3C82
0, //U+3C83
0, //U+3C84
0, //U+3C85
0, //U+3C86
0, //U+3C87
0, //U+3C88
0, //U+3C89
0, //U+3C8A
0, //U+3C8B
0, //U+3C8C
0, //U+3C8D
0, //U+3C8E
0, //U+3C8F
0, //U+3C90
0, //U+3C91
0, //U+3C92
0, //U+3C93
0, //U+3C94
0, //U+3C95
0, //U+3C96
0, //U+3C97
0, //U+3C98
0, //U+3C99
0, //U+3C9A
0, //U+3C9B
0, //U+3C9C
0, //U+3C9D
0, //U+3C9E
0, //U+3C9F
0, //U+3CA0
0, //U+3CA1
0, //U+3CA2
0, //U+3CA3
0, //U+3CA4
0, //U+3CA5
0, //U+3CA6
0, //U+3CA7
0, //U+3CA8
0, //U+3CA9
0, //U+3CAA
0, //U+3CAB
0, //U+3CAC
0, //U+3CAD
0, //U+3CAE
0, //U+3CAF
0, //U+3CB0
0, //U+3CB1
0, //U+3CB2
0, //U+3CB3
0, //U+3CB4
0, //U+3CB5
0, //U+3CB6
0, //U+3CB7
0, //U+3CB8
0, //U+3CB9
0, //U+3CBA
0, //U+3CBB
0, //U+3CBC
0, //U+3CBD
0, //U+3CBE
0, //U+3CBF
0, //U+3CC0
0, //U+3CC1
0, //U+3CC2
0, //U+3CC3
0, //U+3CC4
0, //U+3CC5
0, //U+3CC6
0, //U+3CC7
0, //U+3CC8
0, //U+3CC9
0, //U+3CCA
0, //U+3CCB
0, //U+3CCC
0, //U+3CCD
0, //U+3CCE
0, //U+3CCF
0, //U+3CD0
0, //U+3CD1
0, //U+3CD2
0, //U+3CD3
0, //U+3CD4
0, //U+3CD5
0, //U+3CD6
0, //U+3CD7
0, //U+3CD8
0, //U+3CD9
0, //U+3CDA
0, //U+3CDB
0, //U+3CDC
0, //U+3CDD
0, //U+3CDE
0, //U+3CDF
0, //U+3CE0
0, //U+3CE1
0, //U+3CE2
0, //U+3CE3
0, //U+3CE4
0, //U+3CE5
0, //U+3CE6
0, //U+3CE7
0, //U+3CE8
0, //U+3CE9
0, //U+3CEA
0, //U+3CEB
0, //U+3CEC
0, //U+3CED
0, //U+3CEE
0, //U+3CEF
0, //U+3CF0
0, //U+3CF1
0, //U+3CF2
0, //U+3CF3
0, //U+3CF4
0, //U+3CF5
0, //U+3CF6
0, //U+3CF7
0, //U+3CF8
0, //U+3CF9
0, //U+3CFA
0, //U+3CFB
0, //U+3CFC
0, //U+3CFD
0, //U+3CFE
0, //U+3CFF
0, //U+3D00
0, //U+3D01
0, //U+3D02
0, //U+3D03
0, //U+3D04
0, //U+3D05
0, //U+3D06
0, //U+3D07
0, //U+3D08
0, //U+3D09
0, //U+3D0A
0, //U+3D0B
0, //U+3D0C
0, //U+3D0D
0, //U+3D0E
0, //U+3D0F
0, //U+3D10
0, //U+3D11
0, //U+3D12
0, //U+3D13
0, //U+3D14
0, //U+3D15
0, //U+3D16
0, //U+3D17
0, //U+3D18
0, //U+3D19
0, //U+3D1A
0, //U+3D1B
0, //U+3D1C
0, //U+3D1D
0, //U+3D1E
0, //U+3D1F
0, //U+3D20
0, //U+3D21
0, //U+3D22
0, //U+3D23
0, //U+3D24
0, //U+3D25
0, //U+3D26
0, //U+3D27
0, //U+3D28
0, //U+3D29
0, //U+3D2A
0, //U+3D2B
0, //U+3D2C
0, //U+3D2D
0, //U+3D2E
0, //U+3D2F
0, //U+3D30
0, //U+3D31
0, //U+3D32
0, //U+3D33
0, //U+3D34
0, //U+3D35
0, //U+3D36
0, //U+3D37
0, //U+3D38
0, //U+3D39
0, //U+3D3A
0, //U+3D3B
0, //U+3D3C
0, //U+3D3D
0, //U+3D3E
0, //U+3D3F
0, //U+3D40
0, //U+3D41
0, //U+3D42
0, //U+3D43
0, //U+3D44
0, //U+3D45
0, //U+3D46
0, //U+3D47
0, //U+3D48
0, //U+3D49
0, //U+3D4A
0, //U+3D4B
0, //U+3D4C
0, //U+3D4D
0, //U+3D4E
0, //U+3D4F
0, //U+3D50
0, //U+3D51
0, //U+3D52
0, //U+3D53
0, //U+3D54
0, //U+3D55
0, //U+3D56
0, //U+3D57
0, //U+3D58
0, //U+3D59
0, //U+3D5A
0, //U+3D5B
0, //U+3D5C
0, //U+3D5D
0, //U+3D5E
0, //U+3D5F
0, //U+3D60
0, //U+3D61
0, //U+3D62
0, //U+3D63
0, //U+3D64
0, //U+3D65
0, //U+3D66
0, //U+3D67
0, //U+3D68
0, //U+3D69
0, //U+3D6A
0, //U+3D6B
0, //U+3D6C
0, //U+3D6D
0, //U+3D6E
0, //U+3D6F
0, //U+3D70
0, //U+3D71
0, //U+3D72
0, //U+3D73
0, //U+3D74
0, //U+3D75
0, //U+3D76
0, //U+3D77
0, //U+3D78
0, //U+3D79
0, //U+3D7A
0, //U+3D7B
0, //U+3D7C
0, //U+3D7D
0, //U+3D7E
0, //U+3D7F
0, //U+3D80
0, //U+3D81
0, //U+3D82
0, //U+3D83
0, //U+3D84
0, //U+3D85
0, //U+3D86
0, //U+3D87
0, //U+3D88
0, //U+3D89
0, //U+3D8A
0, //U+3D8B
0, //U+3D8C
0, //U+3D8D
0, //U+3D8E
0, //U+3D8F
0, //U+3D90
0, //U+3D91
0, //U+3D92
0, //U+3D93
0, //U+3D94
0, //U+3D95
0, //U+3D96
0, //U+3D97
0, //U+3D98
0, //U+3D99
0, //U+3D9A
0, //U+3D9B
0, //U+3D9C
0, //U+3D9D
0, //U+3D9E
0, //U+3D9F
0, //U+3DA0
0, //U+3DA1
0, //U+3DA2
0, //U+3DA3
0, //U+3DA4
0, //U+3DA5
0, //U+3DA6
0, //U+3DA7
0, //U+3DA8
0, //U+3DA9
0, //U+3DAA
0, //U+3DAB
0, //U+3DAC
0, //U+3DAD
0, //U+3DAE
0, //U+3DAF
0, //U+3DB0
0, //U+3DB1
0, //U+3DB2
0, //U+3DB3
0, //U+3DB4
0, //U+3DB5
0, //U+3DB6
0, //U+3DB7
0, //U+3DB8
0, //U+3DB9
0, //U+3DBA
0, //U+3DBB
0, //U+3DBC
0, //U+3DBD
0, //U+3DBE
0, //U+3DBF
0, //U+3DC0
0, //U+3DC1
0, //U+3DC2
0, //U+3DC3
0, //U+3DC4
0, //U+3DC5
0, //U+3DC6
0, //U+3DC7
0, //U+3DC8
0, //U+3DC9
0, //U+3DCA
0, //U+3DCB
0, //U+3DCC
0, //U+3DCD
0, //U+3DCE
0, //U+3DCF
0, //U+3DD0
0, //U+3DD1
0, //U+3DD2
0, //U+3DD3
0, //U+3DD4
0, //U+3DD5
0, //U+3DD6
0, //U+3DD7
0, //U+3DD8
0, //U+3DD9
0, //U+3DDA
0, //U+3DDB
0, //U+3DDC
0, //U+3DDD
0, //U+3DDE
0, //U+3DDF
0, //U+3DE0
0, //U+3DE1
0, //U+3DE2
0, //U+3DE3
0, //U+3DE4
0, //U+3DE5
0, //U+3DE6
0, //U+3DE7
0, //U+3DE8
0, //U+3DE9
0, //U+3DEA
0, //U+3DEB
0, //U+3DEC
0, //U+3DED
0, //U+3DEE
0, //U+3DEF
0, //U+3DF0
0, //U+3DF1
0, //U+3DF2
0, //U+3DF3
0, //U+3DF4
0, //U+3DF5
0, //U+3DF6
0, //U+3DF7
0, //U+3DF8
0, //U+3DF9
0, //U+3DFA
0, //U+3DFB
0, //U+3DFC
0, //U+3DFD
0, //U+3DFE
0, //U+3DFF
0, //U+3E00
0, //U+3E01
0, //U+3E02
0, //U+3E03
0, //U+3E04
0, //U+3E05
0, //U+3E06
0, //U+3E07
0, //U+3E08
0, //U+3E09
0, //U+3E0A
0, //U+3E0B
0, //U+3E0C
0, //U+3E0D
0, //U+3E0E
0, //U+3E0F
0, //U+3E10
0, //U+3E11
0, //U+3E12
0, //U+3E13
0, //U+3E14
0, //U+3E15
0, //U+3E16
0, //U+3E17
0, //U+3E18
0, //U+3E19
0, //U+3E1A
0, //U+3E1B
0, //U+3E1C
0, //U+3E1D
0, //U+3E1E
0, //U+3E1F
0, //U+3E20
0, //U+3E21
0, //U+3E22
0, //U+3E23
0, //U+3E24
0, //U+3E25
0, //U+3E26
0, //U+3E27
0, //U+3E28
0, //U+3E29
0, //U+3E2A
0, //U+3E2B
0, //U+3E2C
0, //U+3E2D
0, //U+3E2E
0, //U+3E2F
0, //U+3E30
0, //U+3E31
0, //U+3E32
0, //U+3E33
0, //U+3E34
0, //U+3E35
0, //U+3E36
0, //U+3E37
0, //U+3E38
0, //U+3E39
0, //U+3E3A
0, //U+3E3B
0, //U+3E3C
0, //U+3E3D
0, //U+3E3E
0, //U+3E3F
0, //U+3E40
0, //U+3E41
0, //U+3E42
0, //U+3E43
0, //U+3E44
0, //U+3E45
0, //U+3E46
0, //U+3E47
0, //U+3E48
0, //U+3E49
0, //U+3E4A
0, //U+3E4B
0, //U+3E4C
0, //U+3E4D
0, //U+3E4E
0, //U+3E4F
0, //U+3E50
0, //U+3E51
0, //U+3E52
0, //U+3E53
0, //U+3E54
0, //U+3E55
0, //U+3E56
0, //U+3E57
0, //U+3E58
0, //U+3E59
0, //U+3E5A
0, //U+3E5B
0, //U+3E5C
0, //U+3E5D
0, //U+3E5E
0, //U+3E5F
0, //U+3E60
0, //U+3E61
0, //U+3E62
0, //U+3E63
0, //U+3E64
0, //U+3E65
0, //U+3E66
0, //U+3E67
0, //U+3E68
0, //U+3E69
0, //U+3E6A
0, //U+3E6B
0, //U+3E6C
0, //U+3E6D
0, //U+3E6E
0, //U+3E6F
0, //U+3E70
0, //U+3E71
0, //U+3E72
0, //U+3E73
0, //U+3E74
0, //U+3E75
0, //U+3E76
0, //U+3E77
0, //U+3E78
0, //U+3E79
0, //U+3E7A
0, //U+3E7B
0, //U+3E7C
0, //U+3E7D
0, //U+3E7E
0, //U+3E7F
0, //U+3E80
0, //U+3E81
0, //U+3E82
0, //U+3E83
0, //U+3E84
0, //U+3E85
0, //U+3E86
0, //U+3E87
0, //U+3E88
0, //U+3E89
0, //U+3E8A
0, //U+3E8B
0, //U+3E8C
0, //U+3E8D
0, //U+3E8E
0, //U+3E8F
0, //U+3E90
0, //U+3E91
0, //U+3E92
0, //U+3E93
0, //U+3E94
0, //U+3E95
0, //U+3E96
0, //U+3E97
0, //U+3E98
0, //U+3E99
0, //U+3E9A
0, //U+3E9B
0, //U+3E9C
0, //U+3E9D
0, //U+3E9E
0, //U+3E9F
0, //U+3EA0
0, //U+3EA1
0, //U+3EA2
0, //U+3EA3
0, //U+3EA4
0, //U+3EA5
0, //U+3EA6
0, //U+3EA7
0, //U+3EA8
0, //U+3EA9
0, //U+3EAA
0, //U+3EAB
0, //U+3EAC
0, //U+3EAD
0, //U+3EAE
0, //U+3EAF
0, //U+3EB0
0, //U+3EB1
0, //U+3EB2
0, //U+3EB3
0, //U+3EB4
0, //U+3EB5
0, //U+3EB6
0, //U+3EB7
0, //U+3EB8
0, //U+3EB9
0, //U+3EBA
0, //U+3EBB
0, //U+3EBC
0, //U+3EBD
0, //U+3EBE
0, //U+3EBF
0, //U+3EC0
0, //U+3EC1
0, //U+3EC2
0, //U+3EC3
0, //U+3EC4
0, //U+3EC5
0, //U+3EC6
0, //U+3EC7
0, //U+3EC8
0, //U+3EC9
0, //U+3ECA
0, //U+3ECB
0, //U+3ECC
0, //U+3ECD
0, //U+3ECE
0, //U+3ECF
0, //U+3ED0
0, //U+3ED1
0, //U+3ED2
0, //U+3ED3
0, //U+3ED4
0, //U+3ED5
0, //U+3ED6
0, //U+3ED7
0, //U+3ED8
0, //U+3ED9
0, //U+3EDA
0, //U+3EDB
0, //U+3EDC
0, //U+3EDD
0, //U+3EDE
0, //U+3EDF
0, //U+3EE0
0, //U+3EE1
0, //U+3EE2
0, //U+3EE3
0, //U+3EE4
0, //U+3EE5
0, //U+3EE6
0, //U+3EE7
0, //U+3EE8
0, //U+3EE9
0, //U+3EEA
0, //U+3EEB
0, //U+3EEC
0, //U+3EED
0, //U+3EEE
0, //U+3EEF
0, //U+3EF0
0, //U+3EF1
0, //U+3EF2
0, //U+3EF3
0, //U+3EF4
0, //U+3EF5
0, //U+3EF6
0, //U+3EF7
0, //U+3EF8
0, //U+3EF9
0, //U+3EFA
0, //U+3EFB
0, //U+3EFC
0, //U+3EFD
0, //U+3EFE
0, //U+3EFF
0, //U+3F00
0, //U+3F01
0, //U+3F02
0, //U+3F03
0, //U+3F04
0, //U+3F05
0, //U+3F06
0, //U+3F07
0, //U+3F08
0, //U+3F09
0, //U+3F0A
0, //U+3F0B
0, //U+3F0C
0, //U+3F0D
0, //U+3F0E
0, //U+3F0F
0, //U+3F10
0, //U+3F11
0, //U+3F12
0, //U+3F13
0, //U+3F14
0, //U+3F15
0, //U+3F16
0, //U+3F17
0, //U+3F18
0, //U+3F19
0, //U+3F1A
0, //U+3F1B
0, //U+3F1C
0, //U+3F1D
0, //U+3F1E
0, //U+3F1F
0, //U+3F20
0, //U+3F21
0, //U+3F22
0, //U+3F23
0, //U+3F24
0, //U+3F25
0, //U+3F26
0, //U+3F27
0, //U+3F28
0, //U+3F29
0, //U+3F2A
0, //U+3F2B
0, //U+3F2C
0, //U+3F2D
0, //U+3F2E
0, //U+3F2F
0, //U+3F30
0, //U+3F31
0, //U+3F32
0, //U+3F33
0, //U+3F34
0, //U+3F35
0, //U+3F36
0, //U+3F37
0, //U+3F38
0, //U+3F39
0, //U+3F3A
0, //U+3F3B
0, //U+3F3C
0, //U+3F3D
0, //U+3F3E
0, //U+3F3F
0, //U+3F40
0, //U+3F41
0, //U+3F42
0, //U+3F43
0, //U+3F44
0, //U+3F45
0, //U+3F46
0, //U+3F47
0, //U+3F48
0, //U+3F49
0, //U+3F4A
0, //U+3F4B
0, //U+3F4C
0, //U+3F4D
0, //U+3F4E
0, //U+3F4F
0, //U+3F50
0, //U+3F51
0, //U+3F52
0, //U+3F53
0, //U+3F54
0, //U+3F55
0, //U+3F56
0, //U+3F57
0, //U+3F58
0, //U+3F59
0, //U+3F5A
0, //U+3F5B
0, //U+3F5C
0, //U+3F5D
0, //U+3F5E
0, //U+3F5F
0, //U+3F60
0, //U+3F61
0, //U+3F62
0, //U+3F63
0, //U+3F64
0, //U+3F65
0, //U+3F66
0, //U+3F67
0, //U+3F68
0, //U+3F69
0, //U+3F6A
0, //U+3F6B
0, //U+3F6C
0, //U+3F6D
0, //U+3F6E
0, //U+3F6F
0, //U+3F70
0, //U+3F71
0, //U+3F72
0, //U+3F73
0, //U+3F74
0, //U+3F75
0, //U+3F76
0, //U+3F77
0, //U+3F78
0, //U+3F79
0, //U+3F7A
0, //U+3F7B
0, //U+3F7C
0, //U+3F7D
0, //U+3F7E
0, //U+3F7F
0, //U+3F80
0, //U+3F81
0, //U+3F82
0, //U+3F83
0, //U+3F84
0, //U+3F85
0, //U+3F86
0, //U+3F87
0, //U+3F88
0, //U+3F89
0, //U+3F8A
0, //U+3F8B
0, //U+3F8C
0, //U+3F8D
0, //U+3F8E
0, //U+3F8F
0, //U+3F90
0, //U+3F91
0, //U+3F92
0, //U+3F93
0, //U+3F94
0, //U+3F95
0, //U+3F96
0, //U+3F97
0, //U+3F98
0, //U+3F99
0, //U+3F9A
0, //U+3F9B
0, //U+3F9C
0, //U+3F9D
0, //U+3F9E
0, //U+3F9F
0, //U+3FA0
0, //U+3FA1
0, //U+3FA2
0, //U+3FA3
0, //U+3FA4
0, //U+3FA5
0, //U+3FA6
0, //U+3FA7
0, //U+3FA8
0, //U+3FA9
0, //U+3FAA
0, //U+3FAB
0, //U+3FAC
0, //U+3FAD
0, //U+3FAE
0, //U+3FAF
0, //U+3FB0
0, //U+3FB1
0, //U+3FB2
0, //U+3FB3
0, //U+3FB4
0, //U+3FB5
0, //U+3FB6
0, //U+3FB7
0, //U+3FB8
0, //U+3FB9
0, //U+3FBA
0, //U+3FBB
0, //U+3FBC
0, //U+3FBD
0, //U+3FBE
0, //U+3FBF
0, //U+3FC0
0, //U+3FC1
0, //U+3FC2
0, //U+3FC3
0, //U+3FC4
0, //U+3FC5
0, //U+3FC6
0, //U+3FC7
0, //U+3FC8
0, //U+3FC9
0, //U+3FCA
0, //U+3FCB
0, //U+3FCC
0, //U+3FCD
0, //U+3FCE
0, //U+3FCF
0, //U+3FD0
0, //U+3FD1
0, //U+3FD2
0, //U+3FD3
0, //U+3FD4
0, //U+3FD5
0, //U+3FD6
0, //U+3FD7
0, //U+3FD8
0, //U+3FD9
0, //U+3FDA
0, //U+3FDB
0, //U+3FDC
0, //U+3FDD
0, //U+3FDE
0, //U+3FDF
0, //U+3FE0
0, //U+3FE1
0, //U+3FE2
0, //U+3FE3
0, //U+3FE4
0, //U+3FE5
0, //U+3FE6
0, //U+3FE7
0, //U+3FE8
0, //U+3FE9
0, //U+3FEA
0, //U+3FEB
0, //U+3FEC
0, //U+3FED
0, //U+3FEE
0, //U+3FEF
0, //U+3FF0
0, //U+3FF1
0, //U+3FF2
0, //U+3FF3
0, //U+3FF4
0, //U+3FF5
0, //U+3FF6
0, //U+3FF7
0, //U+3FF8
0, //U+3FF9
0, //U+3FFA
0, //U+3FFB
0, //U+3FFC
0, //U+3FFD
0, //U+3FFE
0, //U+3FFF
0, //U+4000
0, //U+4001
0, //U+4002
0, //U+4003
0, //U+4004
0, //U+4005
0, //U+4006
0, //U+4007
0, //U+4008
0, //U+4009
0, //U+400A
0, //U+400B
0, //U+400C
0, //U+400D
0, //U+400E
0, //U+400F
0, //U+4010
0, //U+4011
0, //U+4012
0, //U+4013
0, //U+4014
0, //U+4015
0, //U+4016
0, //U+4017
0, //U+4018
0, //U+4019
0, //U+401A
0, //U+401B
0, //U+401C
0, //U+401D
0, //U+401E
0, //U+401F
0, //U+4020
0, //U+4021
0, //U+4022
0, //U+4023
0, //U+4024
0, //U+4025
0, //U+4026
0, //U+4027
0, //U+4028
0, //U+4029
0, //U+402A
0, //U+402B
0, //U+402C
0, //U+402D
0, //U+402E
0, //U+402F
0, //U+4030
0, //U+4031
0, //U+4032
0, //U+4033
0, //U+4034
0, //U+4035
0, //U+4036
0, //U+4037
0, //U+4038
0, //U+4039
0, //U+403A
0, //U+403B
0, //U+403C
0, //U+403D
0, //U+403E
0, //U+403F
0, //U+4040
0, //U+4041
0, //U+4042
0, //U+4043
0, //U+4044
0, //U+4045
0, //U+4046
0, //U+4047
0, //U+4048
0, //U+4049
0, //U+404A
0, //U+404B
0, //U+404C
0, //U+404D
0, //U+404E
0, //U+404F
0, //U+4050
0, //U+4051
0, //U+4052
0, //U+4053
0, //U+4054
0, //U+4055
0, //U+4056
0, //U+4057
0, //U+4058
0, //U+4059
0, //U+405A
0, //U+405B
0, //U+405C
0, //U+405D
0, //U+405E
0, //U+405F
0, //U+4060
0, //U+4061
0, //U+4062
0, //U+4063
0, //U+4064
0, //U+4065
0, //U+4066
0, //U+4067
0, //U+4068
0, //U+4069
0, //U+406A
0, //U+406B
0, //U+406C
0, //U+406D
0, //U+406E
0, //U+406F
0, //U+4070
0, //U+4071
0, //U+4072
0, //U+4073
0, //U+4074
0, //U+4075
0, //U+4076
0, //U+4077
0, //U+4078
0, //U+4079
0, //U+407A
0, //U+407B
0, //U+407C
0, //U+407D
0, //U+407E
0, //U+407F
0, //U+4080
0, //U+4081
0, //U+4082
0, //U+4083
0, //U+4084
0, //U+4085
0, //U+4086
0, //U+4087
0, //U+4088
0, //U+4089
0, //U+408A
0, //U+408B
0, //U+408C
0, //U+408D
0, //U+408E
0, //U+408F
0, //U+4090
0, //U+4091
0, //U+4092
0, //U+4093
0, //U+4094
0, //U+4095
0, //U+4096
0, //U+4097
0, //U+4098
0, //U+4099
0, //U+409A
0, //U+409B
0, //U+409C
0, //U+409D
0, //U+409E
0, //U+409F
0, //U+40A0
0, //U+40A1
0, //U+40A2
0, //U+40A3
0, //U+40A4
0, //U+40A5
0, //U+40A6
0, //U+40A7
0, //U+40A8
0, //U+40A9
0, //U+40AA
0, //U+40AB
0, //U+40AC
0, //U+40AD
0, //U+40AE
0, //U+40AF
0, //U+40B0
0, //U+40B1
0, //U+40B2
0, //U+40B3
0, //U+40B4
0, //U+40B5
0, //U+40B6
0, //U+40B7
0, //U+40B8
0, //U+40B9
0, //U+40BA
0, //U+40BB
0, //U+40BC
0, //U+40BD
0, //U+40BE
0, //U+40BF
0, //U+40C0
0, //U+40C1
0, //U+40C2
0, //U+40C3
0, //U+40C4
0, //U+40C5
0, //U+40C6
0, //U+40C7
0, //U+40C8
0, //U+40C9
0, //U+40CA
0, //U+40CB
0, //U+40CC
0, //U+40CD
0, //U+40CE
0, //U+40CF
0, //U+40D0
0, //U+40D1
0, //U+40D2
0, //U+40D3
0, //U+40D4
0, //U+40D5
0, //U+40D6
0, //U+40D7
0, //U+40D8
0, //U+40D9
0, //U+40DA
0, //U+40DB
0, //U+40DC
0, //U+40DD
0, //U+40DE
0, //U+40DF
0, //U+40E0
0, //U+40E1
0, //U+40E2
0, //U+40E3
0, //U+40E4
0, //U+40E5
0, //U+40E6
0, //U+40E7
0, //U+40E8
0, //U+40E9
0, //U+40EA
0, //U+40EB
0, //U+40EC
0, //U+40ED
0, //U+40EE
0, //U+40EF
0, //U+40F0
0, //U+40F1
0, //U+40F2
0, //U+40F3
0, //U+40F4
0, //U+40F5
0, //U+40F6
0, //U+40F7
0, //U+40F8
0, //U+40F9
0, //U+40FA
0, //U+40FB
0, //U+40FC
0, //U+40FD
0, //U+40FE
0, //U+40FF
0, //U+4100
0, //U+4101
0, //U+4102
0, //U+4103
0, //U+4104
0, //U+4105
0, //U+4106
0, //U+4107
0, //U+4108
0, //U+4109
0, //U+410A
0, //U+410B
0, //U+410C
0, //U+410D
0, //U+410E
0, //U+410F
0, //U+4110
0, //U+4111
0, //U+4112
0, //U+4113
0, //U+4114
0, //U+4115
0, //U+4116
0, //U+4117
0, //U+4118
0, //U+4119
0, //U+411A
0, //U+411B
0, //U+411C
0, //U+411D
0, //U+411E
0, //U+411F
0, //U+4120
0, //U+4121
0, //U+4122
0, //U+4123
0, //U+4124
0, //U+4125
0, //U+4126
0, //U+4127
0, //U+4128
0, //U+4129
0, //U+412A
0, //U+412B
0, //U+412C
0, //U+412D
0, //U+412E
0, //U+412F
0, //U+4130
0, //U+4131
0, //U+4132
0, //U+4133
0, //U+4134
0, //U+4135
0, //U+4136
0, //U+4137
0, //U+4138
0, //U+4139
0, //U+413A
0, //U+413B
0, //U+413C
0, //U+413D
0, //U+413E
0, //U+413F
0, //U+4140
0, //U+4141
0, //U+4142
0, //U+4143
0, //U+4144
0, //U+4145
0, //U+4146
0, //U+4147
0, //U+4148
0, //U+4149
0, //U+414A
0, //U+414B
0, //U+414C
0, //U+414D
0, //U+414E
0, //U+414F
0, //U+4150
0, //U+4151
0, //U+4152
0, //U+4153
0, //U+4154
0, //U+4155
0, //U+4156
0, //U+4157
0, //U+4158
0, //U+4159
0, //U+415A
0, //U+415B
0, //U+415C
0, //U+415D
0, //U+415E
0, //U+415F
0, //U+4160
0, //U+4161
0, //U+4162
0, //U+4163
0, //U+4164
0, //U+4165
0, //U+4166
0, //U+4167
0, //U+4168
0, //U+4169
0, //U+416A
0, //U+416B
0, //U+416C
0, //U+416D
0, //U+416E
0, //U+416F
0, //U+4170
0, //U+4171
0, //U+4172
0, //U+4173
0, //U+4174
0, //U+4175
0, //U+4176
0, //U+4177
0, //U+4178
0, //U+4179
0, //U+417A
0, //U+417B
0, //U+417C
0, //U+417D
0, //U+417E
0, //U+417F
0, //U+4180
0, //U+4181
0, //U+4182
0, //U+4183
0, //U+4184
0, //U+4185
0, //U+4186
0, //U+4187
0, //U+4188
0, //U+4189
0, //U+418A
0, //U+418B
0, //U+418C
0, //U+418D
0, //U+418E
0, //U+418F
0, //U+4190
0, //U+4191
0, //U+4192
0, //U+4193
0, //U+4194
0, //U+4195
0, //U+4196
0, //U+4197
0, //U+4198
0, //U+4199
0, //U+419A
0, //U+419B
0, //U+419C
0, //U+419D
0, //U+419E
0, //U+419F
0, //U+41A0
0, //U+41A1
0, //U+41A2
0, //U+41A3
0, //U+41A4
0, //U+41A5
0, //U+41A6
0, //U+41A7
0, //U+41A8
0, //U+41A9
0, //U+41AA
0, //U+41AB
0, //U+41AC
0, //U+41AD
0, //U+41AE
0, //U+41AF
0, //U+41B0
0, //U+41B1
0, //U+41B2
0, //U+41B3
0, //U+41B4
0, //U+41B5
0, //U+41B6
0, //U+41B7
0, //U+41B8
0, //U+41B9
0, //U+41BA
0, //U+41BB
0, //U+41BC
0, //U+41BD
0, //U+41BE
0, //U+41BF
0, //U+41C0
0, //U+41C1
0, //U+41C2
0, //U+41C3
0, //U+41C4
0, //U+41C5
0, //U+41C6
0, //U+41C7
0, //U+41C8
0, //U+41C9
0, //U+41CA
0, //U+41CB
0, //U+41CC
0, //U+41CD
0, //U+41CE
0, //U+41CF
0, //U+41D0
0, //U+41D1
0, //U+41D2
0, //U+41D3
0, //U+41D4
0, //U+41D5
0, //U+41D6
0, //U+41D7
0, //U+41D8
0, //U+41D9
0, //U+41DA
0, //U+41DB
0, //U+41DC
0, //U+41DD
0, //U+41DE
0, //U+41DF
0, //U+41E0
0, //U+41E1
0, //U+41E2
0, //U+41E3
0, //U+41E4
0, //U+41E5
0, //U+41E6
0, //U+41E7
0, //U+41E8
0, //U+41E9
0, //U+41EA
0, //U+41EB
0, //U+41EC
0, //U+41ED
0, //U+41EE
0, //U+41EF
0, //U+41F0
0, //U+41F1
0, //U+41F2
0, //U+41F3
0, //U+41F4
0, //U+41F5
0, //U+41F6
0, //U+41F7
0, //U+41F8
0, //U+41F9
0, //U+41FA
0, //U+41FB
0, //U+41FC
0, //U+41FD
0, //U+41FE
0, //U+41FF
0, //U+4200
0, //U+4201
0, //U+4202
0, //U+4203
0, //U+4204
0, //U+4205
0, //U+4206
0, //U+4207
0, //U+4208
0, //U+4209
0, //U+420A
0, //U+420B
0, //U+420C
0, //U+420D
0, //U+420E
0, //U+420F
0, //U+4210
0, //U+4211
0, //U+4212
0, //U+4213
0, //U+4214
0, //U+4215
0, //U+4216
0, //U+4217
0, //U+4218
0, //U+4219
0, //U+421A
0, //U+421B
0, //U+421C
0, //U+421D
0, //U+421E
0, //U+421F
0, //U+4220
0, //U+4221
0, //U+4222
0, //U+4223
0, //U+4224
0, //U+4225
0, //U+4226
0, //U+4227
0, //U+4228
0, //U+4229
0, //U+422A
0, //U+422B
0, //U+422C
0, //U+422D
0, //U+422E
0, //U+422F
0, //U+4230
0, //U+4231
0, //U+4232
0, //U+4233
0, //U+4234
0, //U+4235
0, //U+4236
0, //U+4237
0, //U+4238
0, //U+4239
0, //U+423A
0, //U+423B
0, //U+423C
0, //U+423D
0, //U+423E
0, //U+423F
0, //U+4240
0, //U+4241
0, //U+4242
0, //U+4243
0, //U+4244
0, //U+4245
0, //U+4246
0, //U+4247
0, //U+4248
0, //U+4249
0, //U+424A
0, //U+424B
0, //U+424C
0, //U+424D
0, //U+424E
0, //U+424F
0, //U+4250
0, //U+4251
0, //U+4252
0, //U+4253
0, //U+4254
0, //U+4255
0, //U+4256
0, //U+4257
0, //U+4258
0, //U+4259
0, //U+425A
0, //U+425B
0, //U+425C
0, //U+425D
0, //U+425E
0, //U+425F
0, //U+4260
0, //U+4261
0, //U+4262
0, //U+4263
0, //U+4264
0, //U+4265
0, //U+4266
0, //U+4267
0, //U+4268
0, //U+4269
0, //U+426A
0, //U+426B
0, //U+426C
0, //U+426D
0, //U+426E
0, //U+426F
0, //U+4270
0, //U+4271
0, //U+4272
0, //U+4273
0, //U+4274
0, //U+4275
0, //U+4276
0, //U+4277
0, //U+4278
0, //U+4279
0, //U+427A
0, //U+427B
0, //U+427C
0, //U+427D
0, //U+427E
0, //U+427F
0, //U+4280
0, //U+4281
0, //U+4282
0, //U+4283
0, //U+4284
0, //U+4285
0, //U+4286
0, //U+4287
0, //U+4288
0, //U+4289
0, //U+428A
0, //U+428B
0, //U+428C
0, //U+428D
0, //U+428E
0, //U+428F
0, //U+4290
0, //U+4291
0, //U+4292
0, //U+4293
0, //U+4294
0, //U+4295
0, //U+4296
0, //U+4297
0, //U+4298
0, //U+4299
0, //U+429A
0, //U+429B
0, //U+429C
0, //U+429D
0, //U+429E
0, //U+429F
0, //U+42A0
0, //U+42A1
0, //U+42A2
0, //U+42A3
0, //U+42A4
0, //U+42A5
0, //U+42A6
0, //U+42A7
0, //U+42A8
0, //U+42A9
0, //U+42AA
0, //U+42AB
0, //U+42AC
0, //U+42AD
0, //U+42AE
0, //U+42AF
0, //U+42B0
0, //U+42B1
0, //U+42B2
0, //U+42B3
0, //U+42B4
0, //U+42B5
0, //U+42B6
0, //U+42B7
0, //U+42B8
0, //U+42B9
0, //U+42BA
0, //U+42BB
0, //U+42BC
0, //U+42BD
0, //U+42BE
0, //U+42BF
0, //U+42C0
0, //U+42C1
0, //U+42C2
0, //U+42C3
0, //U+42C4
0, //U+42C5
0, //U+42C6
0, //U+42C7
0, //U+42C8
0, //U+42C9
0, //U+42CA
0, //U+42CB
0, //U+42CC
0, //U+42CD
0, //U+42CE
0, //U+42CF
0, //U+42D0
0, //U+42D1
0, //U+42D2
0, //U+42D3
0, //U+42D4
0, //U+42D5
0, //U+42D6
0, //U+42D7
0, //U+42D8
0, //U+42D9
0, //U+42DA
0, //U+42DB
0, //U+42DC
0, //U+42DD
0, //U+42DE
0, //U+42DF
0, //U+42E0
0, //U+42E1
0, //U+42E2
0, //U+42E3
0, //U+42E4
0, //U+42E5
0, //U+42E6
0, //U+42E7
0, //U+42E8
0, //U+42E9
0, //U+42EA
0, //U+42EB
0, //U+42EC
0, //U+42ED
0, //U+42EE
0, //U+42EF
0, //U+42F0
0, //U+42F1
0, //U+42F2
0, //U+42F3
0, //U+42F4
0, //U+42F5
0, //U+42F6
0, //U+42F7
0, //U+42F8
0, //U+42F9
0, //U+42FA
0, //U+42FB
0, //U+42FC
0, //U+42FD
0, //U+42FE
0, //U+42FF
0, //U+4300
0, //U+4301
0, //U+4302
0, //U+4303
0, //U+4304
0, //U+4305
0, //U+4306
0, //U+4307
0, //U+4308
0, //U+4309
0, //U+430A
0, //U+430B
0, //U+430C
0, //U+430D
0, //U+430E
0, //U+430F
0, //U+4310
0, //U+4311
0, //U+4312
0, //U+4313
0, //U+4314
0, //U+4315
0, //U+4316
0, //U+4317
0, //U+4318
0, //U+4319
0, //U+431A
0, //U+431B
0, //U+431C
0, //U+431D
0, //U+431E
0, //U+431F
0, //U+4320
0, //U+4321
0, //U+4322
0, //U+4323
0, //U+4324
0, //U+4325
0, //U+4326
0, //U+4327
0, //U+4328
0, //U+4329
0, //U+432A
0, //U+432B
0, //U+432C
0, //U+432D
0, //U+432E
0, //U+432F
0, //U+4330
0, //U+4331
0, //U+4332
0, //U+4333
0, //U+4334
0, //U+4335
0, //U+4336
0, //U+4337
0, //U+4338
0, //U+4339
0, //U+433A
0, //U+433B
0, //U+433C
0, //U+433D
0, //U+433E
0, //U+433F
0, //U+4340
0, //U+4341
0, //U+4342
0, //U+4343
0, //U+4344
0, //U+4345
0, //U+4346
0, //U+4347
0, //U+4348
0, //U+4349
0, //U+434A
0, //U+434B
0, //U+434C
0, //U+434D
0, //U+434E
0, //U+434F
0, //U+4350
0, //U+4351
0, //U+4352
0, //U+4353
0, //U+4354
0, //U+4355
0, //U+4356
0, //U+4357
0, //U+4358
0, //U+4359
0, //U+435A
0, //U+435B
0, //U+435C
0, //U+435D
0, //U+435E
0, //U+435F
0, //U+4360
0, //U+4361
0, //U+4362
0, //U+4363
0, //U+4364
0, //U+4365
0, //U+4366
0, //U+4367
0, //U+4368
0, //U+4369
0, //U+436A
0, //U+436B
0, //U+436C
0, //U+436D
0, //U+436E
0, //U+436F
0, //U+4370
0, //U+4371
0, //U+4372
0, //U+4373
0, //U+4374
0, //U+4375
0, //U+4376
0, //U+4377
0, //U+4378
0, //U+4379
0, //U+437A
0, //U+437B
0, //U+437C
0, //U+437D
0, //U+437E
0, //U+437F
0, //U+4380
0, //U+4381
0, //U+4382
0, //U+4383
0, //U+4384
0, //U+4385
0, //U+4386
0, //U+4387
0, //U+4388
0, //U+4389
0, //U+438A
0, //U+438B
0, //U+438C
0, //U+438D
0, //U+438E
0, //U+438F
0, //U+4390
0, //U+4391
0, //U+4392
0, //U+4393
0, //U+4394
0, //U+4395
0, //U+4396
0, //U+4397
0, //U+4398
0, //U+4399
0, //U+439A
0, //U+439B
0, //U+439C
0, //U+439D
0, //U+439E
0, //U+439F
0, //U+43A0
0, //U+43A1
0, //U+43A2
0, //U+43A3
0, //U+43A4
0, //U+43A5
0, //U+43A6
0, //U+43A7
0, //U+43A8
0, //U+43A9
0, //U+43AA
0, //U+43AB
0, //U+43AC
0, //U+43AD
0, //U+43AE
0, //U+43AF
0, //U+43B0
0, //U+43B1
0, //U+43B2
0, //U+43B3
0, //U+43B4
0, //U+43B5
0, //U+43B6
0, //U+43B7
0, //U+43B8
0, //U+43B9
0, //U+43BA
0, //U+43BB
0, //U+43BC
0, //U+43BD
0, //U+43BE
0, //U+43BF
0, //U+43C0
0, //U+43C1
0, //U+43C2
0, //U+43C3
0, //U+43C4
0, //U+43C5
0, //U+43C6
0, //U+43C7
0, //U+43C8
0, //U+43C9
0, //U+43CA
0, //U+43CB
0, //U+43CC
0, //U+43CD
0, //U+43CE
0, //U+43CF
0, //U+43D0
0, //U+43D1
0, //U+43D2
0, //U+43D3
0, //U+43D4
0, //U+43D5
0, //U+43D6
0, //U+43D7
0, //U+43D8
0, //U+43D9
0, //U+43DA
0, //U+43DB
0, //U+43DC
0, //U+43DD
0, //U+43DE
0, //U+43DF
0, //U+43E0
0, //U+43E1
0, //U+43E2
0, //U+43E3
0, //U+43E4
0, //U+43E5
0, //U+43E6
0, //U+43E7
0, //U+43E8
0, //U+43E9
0, //U+43EA
0, //U+43EB
0, //U+43EC
0, //U+43ED
0, //U+43EE
0, //U+43EF
0, //U+43F0
0, //U+43F1
0, //U+43F2
0, //U+43F3
0, //U+43F4
0, //U+43F5
0, //U+43F6
0, //U+43F7
0, //U+43F8
0, //U+43F9
0, //U+43FA
0, //U+43FB
0, //U+43FC
0, //U+43FD
0, //U+43FE
0, //U+43FF
0, //U+4400
0, //U+4401
0, //U+4402
0, //U+4403
0, //U+4404
0, //U+4405
0, //U+4406
0, //U+4407
0, //U+4408
0, //U+4409
0, //U+440A
0, //U+440B
0, //U+440C
0, //U+440D
0, //U+440E
0, //U+440F
0, //U+4410
0, //U+4411
0, //U+4412
0, //U+4413
0, //U+4414
0, //U+4415
0, //U+4416
0, //U+4417
0, //U+4418
0, //U+4419
0, //U+441A
0, //U+441B
0, //U+441C
0, //U+441D
0, //U+441E
0, //U+441F
0, //U+4420
0, //U+4421
0, //U+4422
0, //U+4423
0, //U+4424
0, //U+4425
0, //U+4426
0, //U+4427
0, //U+4428
0, //U+4429
0, //U+442A
0, //U+442B
0, //U+442C
0, //U+442D
0, //U+442E
0, //U+442F
0, //U+4430
0, //U+4431
0, //U+4432
0, //U+4433
0, //U+4434
0, //U+4435
0, //U+4436
0, //U+4437
0, //U+4438
0, //U+4439
0, //U+443A
0, //U+443B
0, //U+443C
0, //U+443D
0, //U+443E
0, //U+443F
0, //U+4440
0, //U+4441
0, //U+4442
0, //U+4443
0, //U+4444
0, //U+4445
0, //U+4446
0, //U+4447
0, //U+4448
0, //U+4449
0, //U+444A
0, //U+444B
0, //U+444C
0, //U+444D
0, //U+444E
0, //U+444F
0, //U+4450
0, //U+4451
0, //U+4452
0, //U+4453
0, //U+4454
0, //U+4455
0, //U+4456
0, //U+4457
0, //U+4458
0, //U+4459
0, //U+445A
0, //U+445B
0, //U+445C
0, //U+445D
0, //U+445E
0, //U+445F
0, //U+4460
0, //U+4461
0, //U+4462
0, //U+4463
0, //U+4464
0, //U+4465
0, //U+4466
0, //U+4467
0, //U+4468
0, //U+4469
0, //U+446A
0, //U+446B
0, //U+446C
0, //U+446D
0, //U+446E
0, //U+446F
0, //U+4470
0, //U+4471
0, //U+4472
0, //U+4473
0, //U+4474
0, //U+4475
0, //U+4476
0, //U+4477
0, //U+4478
0, //U+4479
0, //U+447A
0, //U+447B
0, //U+447C
0, //U+447D
0, //U+447E
0, //U+447F
0, //U+4480
0, //U+4481
0, //U+4482
0, //U+4483
0, //U+4484
0, //U+4485
0, //U+4486
0, //U+4487
0, //U+4488
0, //U+4489
0, //U+448A
0, //U+448B
0, //U+448C
0, //U+448D
0, //U+448E
0, //U+448F
0, //U+4490
0, //U+4491
0, //U+4492
0, //U+4493
0, //U+4494
0, //U+4495
0, //U+4496
0, //U+4497
0, //U+4498
0, //U+4499
0, //U+449A
0, //U+449B
0, //U+449C
0, //U+449D
0, //U+449E
0, //U+449F
0, //U+44A0
0, //U+44A1
0, //U+44A2
0, //U+44A3
0, //U+44A4
0, //U+44A5
0, //U+44A6
0, //U+44A7
0, //U+44A8
0, //U+44A9
0, //U+44AA
0, //U+44AB
0, //U+44AC
0, //U+44AD
0, //U+44AE
0, //U+44AF
0, //U+44B0
0, //U+44B1
0, //U+44B2
0, //U+44B3
0, //U+44B4
0, //U+44B5
0, //U+44B6
0, //U+44B7
0, //U+44B8
0, //U+44B9
0, //U+44BA
0, //U+44BB
0, //U+44BC
0, //U+44BD
0, //U+44BE
0, //U+44BF
0, //U+44C0
0, //U+44C1
0, //U+44C2
0, //U+44C3
0, //U+44C4
0, //U+44C5
0, //U+44C6
0, //U+44C7
0, //U+44C8
0, //U+44C9
0, //U+44CA
0, //U+44CB
0, //U+44CC
0, //U+44CD
0, //U+44CE
0, //U+44CF
0, //U+44D0
0, //U+44D1
0, //U+44D2
0, //U+44D3
0, //U+44D4
0, //U+44D5
0, //U+44D6
0, //U+44D7
0, //U+44D8
0, //U+44D9
0, //U+44DA
0, //U+44DB
0, //U+44DC
0, //U+44DD
0, //U+44DE
0, //U+44DF
0, //U+44E0
0, //U+44E1
0, //U+44E2
0, //U+44E3
0, //U+44E4
0, //U+44E5
0, //U+44E6
0, //U+44E7
0, //U+44E8
0, //U+44E9
0, //U+44EA
0, //U+44EB
0, //U+44EC
0, //U+44ED
0, //U+44EE
0, //U+44EF
0, //U+44F0
0, //U+44F1
0, //U+44F2
0, //U+44F3
0, //U+44F4
0, //U+44F5
0, //U+44F6
0, //U+44F7
0, //U+44F8
0, //U+44F9
0, //U+44FA
0, //U+44FB
0, //U+44FC
0, //U+44FD
0, //U+44FE
0, //U+44FF
0, //U+4500
0, //U+4501
0, //U+4502
0, //U+4503
0, //U+4504
0, //U+4505
0, //U+4506
0, //U+4507
0, //U+4508
0, //U+4509
0, //U+450A
0, //U+450B
0, //U+450C
0, //U+450D
0, //U+450E
0, //U+450F
0, //U+4510
0, //U+4511
0, //U+4512
0, //U+4513
0, //U+4514
0, //U+4515
0, //U+4516
0, //U+4517
0, //U+4518
0, //U+4519
0, //U+451A
0, //U+451B
0, //U+451C
0, //U+451D
0, //U+451E
0, //U+451F
0, //U+4520
0, //U+4521
0, //U+4522
0, //U+4523
0, //U+4524
0, //U+4525
0, //U+4526
0, //U+4527
0, //U+4528
0, //U+4529
0, //U+452A
0, //U+452B
0, //U+452C
0, //U+452D
0, //U+452E
0, //U+452F
0, //U+4530
0, //U+4531
0, //U+4532
0, //U+4533
0, //U+4534
0, //U+4535
0, //U+4536
0, //U+4537
0, //U+4538
0, //U+4539
0, //U+453A
0, //U+453B
0, //U+453C
0, //U+453D
0, //U+453E
0, //U+453F
0, //U+4540
0, //U+4541
0, //U+4542
0, //U+4543
0, //U+4544
0, //U+4545
0, //U+4546
0, //U+4547
0, //U+4548
0, //U+4549
0, //U+454A
0, //U+454B
0, //U+454C
0, //U+454D
0, //U+454E
0, //U+454F
0, //U+4550
0, //U+4551
0, //U+4552
0, //U+4553
0, //U+4554
0, //U+4555
0, //U+4556
0, //U+4557
0, //U+4558
0, //U+4559
0, //U+455A
0, //U+455B
0, //U+455C
0, //U+455D
0, //U+455E
0, //U+455F
0, //U+4560
0, //U+4561
0, //U+4562
0, //U+4563
0, //U+4564
0, //U+4565
0, //U+4566
0, //U+4567
0, //U+4568
0, //U+4569
0, //U+456A
0, //U+456B
0, //U+456C
0, //U+456D
0, //U+456E
0, //U+456F
0, //U+4570
0, //U+4571
0, //U+4572
0, //U+4573
0, //U+4574
0, //U+4575
0, //U+4576
0, //U+4577
0, //U+4578
0, //U+4579
0, //U+457A
0, //U+457B
0, //U+457C
0, //U+457D
0, //U+457E
0, //U+457F
0, //U+4580
0, //U+4581
0, //U+4582
0, //U+4583
0, //U+4584
0, //U+4585
0, //U+4586
0, //U+4587
0, //U+4588
0, //U+4589
0, //U+458A
0, //U+458B
0, //U+458C
0, //U+458D
0, //U+458E
0, //U+458F
0, //U+4590
0, //U+4591
0, //U+4592
0, //U+4593
0, //U+4594
0, //U+4595
0, //U+4596
0, //U+4597
0, //U+4598
0, //U+4599
0, //U+459A
0, //U+459B
0, //U+459C
0, //U+459D
0, //U+459E
0, //U+459F
0, //U+45A0
0, //U+45A1
0, //U+45A2
0, //U+45A3
0, //U+45A4
0, //U+45A5
0, //U+45A6
0, //U+45A7
0, //U+45A8
0, //U+45A9
0, //U+45AA
0, //U+45AB
0, //U+45AC
0, //U+45AD
0, //U+45AE
0, //U+45AF
0, //U+45B0
0, //U+45B1
0, //U+45B2
0, //U+45B3
0, //U+45B4
0, //U+45B5
0, //U+45B6
0, //U+45B7
0, //U+45B8
0, //U+45B9
0, //U+45BA
0, //U+45BB
0, //U+45BC
0, //U+45BD
0, //U+45BE
0, //U+45BF
0, //U+45C0
0, //U+45C1
0, //U+45C2
0, //U+45C3
0, //U+45C4
0, //U+45C5
0, //U+45C6
0, //U+45C7
0, //U+45C8
0, //U+45C9
0, //U+45CA
0, //U+45CB
0, //U+45CC
0, //U+45CD
0, //U+45CE
0, //U+45CF
0, //U+45D0
0, //U+45D1
0, //U+45D2
0, //U+45D3
0, //U+45D4
0, //U+45D5
0, //U+45D6
0, //U+45D7
0, //U+45D8
0, //U+45D9
0, //U+45DA
0, //U+45DB
0, //U+45DC
0, //U+45DD
0, //U+45DE
0, //U+45DF
0, //U+45E0
0, //U+45E1
0, //U+45E2
0, //U+45E3
0, //U+45E4
0, //U+45E5
0, //U+45E6
0, //U+45E7
0, //U+45E8
0, //U+45E9
0, //U+45EA
0, //U+45EB
0, //U+45EC
0, //U+45ED
0, //U+45EE
0, //U+45EF
0, //U+45F0
0, //U+45F1
0, //U+45F2
0, //U+45F3
0, //U+45F4
0, //U+45F5
0, //U+45F6
0, //U+45F7
0, //U+45F8
0, //U+45F9
0, //U+45FA
0, //U+45FB
0, //U+45FC
0, //U+45FD
0, //U+45FE
0, //U+45FF
0, //U+4600
0, //U+4601
0, //U+4602
0, //U+4603
0, //U+4604
0, //U+4605
0, //U+4606
0, //U+4607
0, //U+4608
0, //U+4609
0, //U+460A
0, //U+460B
0, //U+460C
0, //U+460D
0, //U+460E
0, //U+460F
0, //U+4610
0, //U+4611
0, //U+4612
0, //U+4613
0, //U+4614
0, //U+4615
0, //U+4616
0, //U+4617
0, //U+4618
0, //U+4619
0, //U+461A
0, //U+461B
0, //U+461C
0, //U+461D
0, //U+461E
0, //U+461F
0, //U+4620
0, //U+4621
0, //U+4622
0, //U+4623
0, //U+4624
0, //U+4625
0, //U+4626
0, //U+4627
0, //U+4628
0, //U+4629
0, //U+462A
0, //U+462B
0, //U+462C
0, //U+462D
0, //U+462E
0, //U+462F
0, //U+4630
0, //U+4631
0, //U+4632
0, //U+4633
0, //U+4634
0, //U+4635
0, //U+4636
0, //U+4637
0, //U+4638
0, //U+4639
0, //U+463A
0, //U+463B
0, //U+463C
0, //U+463D
0, //U+463E
0, //U+463F
0, //U+4640
0, //U+4641
0, //U+4642
0, //U+4643
0, //U+4644
0, //U+4645
0, //U+4646
0, //U+4647
0, //U+4648
0, //U+4649
0, //U+464A
0, //U+464B
0, //U+464C
0, //U+464D
0, //U+464E
0, //U+464F
0, //U+4650
0, //U+4651
0, //U+4652
0, //U+4653
0, //U+4654
0, //U+4655
0, //U+4656
0, //U+4657
0, //U+4658
0, //U+4659
0, //U+465A
0, //U+465B
0, //U+465C
0, //U+465D
0, //U+465E
0, //U+465F
0, //U+4660
0, //U+4661
0, //U+4662
0, //U+4663
0, //U+4664
0, //U+4665
0, //U+4666
0, //U+4667
0, //U+4668
0, //U+4669
0, //U+466A
0, //U+466B
0, //U+466C
0, //U+466D
0, //U+466E
0, //U+466F
0, //U+4670
0, //U+4671
0, //U+4672
0, //U+4673
0, //U+4674
0, //U+4675
0, //U+4676
0, //U+4677
0, //U+4678
0, //U+4679
0, //U+467A
0, //U+467B
0, //U+467C
0, //U+467D
0, //U+467E
0, //U+467F
0, //U+4680
0, //U+4681
0, //U+4682
0, //U+4683
0, //U+4684
0, //U+4685
0, //U+4686
0, //U+4687
0, //U+4688
0, //U+4689
0, //U+468A
0, //U+468B
0, //U+468C
0, //U+468D
0, //U+468E
0, //U+468F
0, //U+4690
0, //U+4691
0, //U+4692
0, //U+4693
0, //U+4694
0, //U+4695
0, //U+4696
0, //U+4697
0, //U+4698
0, //U+4699
0, //U+469A
0, //U+469B
0, //U+469C
0, //U+469D
0, //U+469E
0, //U+469F
0, //U+46A0
0, //U+46A1
0, //U+46A2
0, //U+46A3
0, //U+46A4
0, //U+46A5
0, //U+46A6
0, //U+46A7
0, //U+46A8
0, //U+46A9
0, //U+46AA
0, //U+46AB
0, //U+46AC
0, //U+46AD
0, //U+46AE
0, //U+46AF
0, //U+46B0
0, //U+46B1
0, //U+46B2
0, //U+46B3
0, //U+46B4
0, //U+46B5
0, //U+46B6
0, //U+46B7
0, //U+46B8
0, //U+46B9
0, //U+46BA
0, //U+46BB
0, //U+46BC
0, //U+46BD
0, //U+46BE
0, //U+46BF
0, //U+46C0
0, //U+46C1
0, //U+46C2
0, //U+46C3
0, //U+46C4
0, //U+46C5
0, //U+46C6
0, //U+46C7
0, //U+46C8
0, //U+46C9
0, //U+46CA
0, //U+46CB
0, //U+46CC
0, //U+46CD
0, //U+46CE
0, //U+46CF
0, //U+46D0
0, //U+46D1
0, //U+46D2
0, //U+46D3
0, //U+46D4
0, //U+46D5
0, //U+46D6
0, //U+46D7
0, //U+46D8
0, //U+46D9
0, //U+46DA
0, //U+46DB
0, //U+46DC
0, //U+46DD
0, //U+46DE
0, //U+46DF
0, //U+46E0
0, //U+46E1
0, //U+46E2
0, //U+46E3
0, //U+46E4
0, //U+46E5
0, //U+46E6
0, //U+46E7
0, //U+46E8
0, //U+46E9
0, //U+46EA
0, //U+46EB
0, //U+46EC
0, //U+46ED
0, //U+46EE
0, //U+46EF
0, //U+46F0
0, //U+46F1
0, //U+46F2
0, //U+46F3
0, //U+46F4
0, //U+46F5
0, //U+46F6
0, //U+46F7
0, //U+46F8
0, //U+46F9
0, //U+46FA
0, //U+46FB
0, //U+46FC
0, //U+46FD
0, //U+46FE
0, //U+46FF
0, //U+4700
0, //U+4701
0, //U+4702
0, //U+4703
0, //U+4704
0, //U+4705
0, //U+4706
0, //U+4707
0, //U+4708
0, //U+4709
0, //U+470A
0, //U+470B
0, //U+470C
0, //U+470D
0, //U+470E
0, //U+470F
0, //U+4710
0, //U+4711
0, //U+4712
0, //U+4713
0, //U+4714
0, //U+4715
0, //U+4716
0, //U+4717
0, //U+4718
0, //U+4719
0, //U+471A
0, //U+471B
0, //U+471C
0, //U+471D
0, //U+471E
0, //U+471F
0, //U+4720
0, //U+4721
0, //U+4722
0, //U+4723
0, //U+4724
0, //U+4725
0, //U+4726
0, //U+4727
0, //U+4728
0, //U+4729
0, //U+472A
0, //U+472B
0, //U+472C
0, //U+472D
0, //U+472E
0, //U+472F
0, //U+4730
0, //U+4731
0, //U+4732
0, //U+4733
0, //U+4734
0, //U+4735
0, //U+4736
0, //U+4737
0, //U+4738
0, //U+4739
0, //U+473A
0, //U+473B
0, //U+473C
0, //U+473D
0, //U+473E
0, //U+473F
0, //U+4740
0, //U+4741
0, //U+4742
0, //U+4743
0, //U+4744
0, //U+4745
0, //U+4746
0, //U+4747
0, //U+4748
0, //U+4749
0, //U+474A
0, //U+474B
0, //U+474C
0, //U+474D
0, //U+474E
0, //U+474F
0, //U+4750
0, //U+4751
0, //U+4752
0, //U+4753
0, //U+4754
0, //U+4755
0, //U+4756
0, //U+4757
0, //U+4758
0, //U+4759
0, //U+475A
0, //U+475B
0, //U+475C
0, //U+475D
0, //U+475E
0, //U+475F
0, //U+4760
0, //U+4761
0, //U+4762
0, //U+4763
0, //U+4764
0, //U+4765
0, //U+4766
0, //U+4767
0, //U+4768
0, //U+4769
0, //U+476A
0, //U+476B
0, //U+476C
0, //U+476D
0, //U+476E
0, //U+476F
0, //U+4770
0, //U+4771
0, //U+4772
0, //U+4773
0, //U+4774
0, //U+4775
0, //U+4776
0, //U+4777
0, //U+4778
0, //U+4779
0, //U+477A
0, //U+477B
0, //U+477C
0, //U+477D
0, //U+477E
0, //U+477F
0, //U+4780
0, //U+4781
0, //U+4782
0, //U+4783
0, //U+4784
0, //U+4785
0, //U+4786
0, //U+4787
0, //U+4788
0, //U+4789
0, //U+478A
0, //U+478B
0, //U+478C
0, //U+478D
0, //U+478E
0, //U+478F
0, //U+4790
0, //U+4791
0, //U+4792
0, //U+4793
0, //U+4794
0, //U+4795
0, //U+4796
0, //U+4797
0, //U+4798
0, //U+4799
0, //U+479A
0, //U+479B
0, //U+479C
0, //U+479D
0, //U+479E
0, //U+479F
0, //U+47A0
0, //U+47A1
0, //U+47A2
0, //U+47A3
0, //U+47A4
0, //U+47A5
0, //U+47A6
0, //U+47A7
0, //U+47A8
0, //U+47A9
0, //U+47AA
0, //U+47AB
0, //U+47AC
0, //U+47AD
0, //U+47AE
0, //U+47AF
0, //U+47B0
0, //U+47B1
0, //U+47B2
0, //U+47B3
0, //U+47B4
0, //U+47B5
0, //U+47B6
0, //U+47B7
0, //U+47B8
0, //U+47B9
0, //U+47BA
0, //U+47BB
0, //U+47BC
0, //U+47BD
0, //U+47BE
0, //U+47BF
0, //U+47C0
0, //U+47C1
0, //U+47C2
0, //U+47C3
0, //U+47C4
0, //U+47C5
0, //U+47C6
0, //U+47C7
0, //U+47C8
0, //U+47C9
0, //U+47CA
0, //U+47CB
0, //U+47CC
0, //U+47CD
0, //U+47CE
0, //U+47CF
0, //U+47D0
0, //U+47D1
0, //U+47D2
0, //U+47D3
0, //U+47D4
0, //U+47D5
0, //U+47D6
0, //U+47D7
0, //U+47D8
0, //U+47D9
0, //U+47DA
0, //U+47DB
0, //U+47DC
0, //U+47DD
0, //U+47DE
0, //U+47DF
0, //U+47E0
0, //U+47E1
0, //U+47E2
0, //U+47E3
0, //U+47E4
0, //U+47E5
0, //U+47E6
0, //U+47E7
0, //U+47E8
0, //U+47E9
0, //U+47EA
0, //U+47EB
0, //U+47EC
0, //U+47ED
0, //U+47EE
0, //U+47EF
0, //U+47F0
0, //U+47F1
0, //U+47F2
0, //U+47F3
0, //U+47F4
0, //U+47F5
0, //U+47F6
0, //U+47F7
0, //U+47F8
0, //U+47F9
0, //U+47FA
0, //U+47FB
0, //U+47FC
0, //U+47FD
0, //U+47FE
0, //U+47FF
0, //U+4800
0, //U+4801
0, //U+4802
0, //U+4803
0, //U+4804
0, //U+4805
0, //U+4806
0, //U+4807
0, //U+4808
0, //U+4809
0, //U+480A
0, //U+480B
0, //U+480C
0, //U+480D
0, //U+480E
0, //U+480F
0, //U+4810
0, //U+4811
0, //U+4812
0, //U+4813
0, //U+4814
0, //U+4815
0, //U+4816
0, //U+4817
0, //U+4818
0, //U+4819
0, //U+481A
0, //U+481B
0, //U+481C
0, //U+481D
0, //U+481E
0, //U+481F
0, //U+4820
0, //U+4821
0, //U+4822
0, //U+4823
0, //U+4824
0, //U+4825
0, //U+4826
0, //U+4827
0, //U+4828
0, //U+4829
0, //U+482A
0, //U+482B
0, //U+482C
0, //U+482D
0, //U+482E
0, //U+482F
0, //U+4830
0, //U+4831
0, //U+4832
0, //U+4833
0, //U+4834
0, //U+4835
0, //U+4836
0, //U+4837
0, //U+4838
0, //U+4839
0, //U+483A
0, //U+483B
0, //U+483C
0, //U+483D
0, //U+483E
0, //U+483F
0, //U+4840
0, //U+4841
0, //U+4842
0, //U+4843
0, //U+4844
0, //U+4845
0, //U+4846
0, //U+4847
0, //U+4848
0, //U+4849
0, //U+484A
0, //U+484B
0, //U+484C
0, //U+484D
0, //U+484E
0, //U+484F
0, //U+4850
0, //U+4851
0, //U+4852
0, //U+4853
0, //U+4854
0, //U+4855
0, //U+4856
0, //U+4857
0, //U+4858
0, //U+4859
0, //U+485A
0, //U+485B
0, //U+485C
0, //U+485D
0, //U+485E
0, //U+485F
0, //U+4860
0, //U+4861
0, //U+4862
0, //U+4863
0, //U+4864
0, //U+4865
0, //U+4866
0, //U+4867
0, //U+4868
0, //U+4869
0, //U+486A
0, //U+486B
0, //U+486C
0, //U+486D
0, //U+486E
0, //U+486F
0, //U+4870
0, //U+4871
0, //U+4872
0, //U+4873
0, //U+4874
0, //U+4875
0, //U+4876
0, //U+4877
0, //U+4878
0, //U+4879
0, //U+487A
0, //U+487B
0, //U+487C
0, //U+487D
0, //U+487E
0, //U+487F
0, //U+4880
0, //U+4881
0, //U+4882
0, //U+4883
0, //U+4884
0, //U+4885
0, //U+4886
0, //U+4887
0, //U+4888
0, //U+4889
0, //U+488A
0, //U+488B
0, //U+488C
0, //U+488D
0, //U+488E
0, //U+488F
0, //U+4890
0, //U+4891
0, //U+4892
0, //U+4893
0, //U+4894
0, //U+4895
0, //U+4896
0, //U+4897
0, //U+4898
0, //U+4899
0, //U+489A
0, //U+489B
0, //U+489C
0, //U+489D
0, //U+489E
0, //U+489F
0, //U+48A0
0, //U+48A1
0, //U+48A2
0, //U+48A3
0, //U+48A4
0, //U+48A5
0, //U+48A6
0, //U+48A7
0, //U+48A8
0, //U+48A9
0, //U+48AA
0, //U+48AB
0, //U+48AC
0, //U+48AD
0, //U+48AE
0, //U+48AF
0, //U+48B0
0, //U+48B1
0, //U+48B2
0, //U+48B3
0, //U+48B4
0, //U+48B5
0, //U+48B6
0, //U+48B7
0, //U+48B8
0, //U+48B9
0, //U+48BA
0, //U+48BB
0, //U+48BC
0, //U+48BD
0, //U+48BE
0, //U+48BF
0, //U+48C0
0, //U+48C1
0, //U+48C2
0, //U+48C3
0, //U+48C4
0, //U+48C5
0, //U+48C6
0, //U+48C7
0, //U+48C8
0, //U+48C9
0, //U+48CA
0, //U+48CB
0, //U+48CC
0, //U+48CD
0, //U+48CE
0, //U+48CF
0, //U+48D0
0, //U+48D1
0, //U+48D2
0, //U+48D3
0, //U+48D4
0, //U+48D5
0, //U+48D6
0, //U+48D7
0, //U+48D8
0, //U+48D9
0, //U+48DA
0, //U+48DB
0, //U+48DC
0, //U+48DD
0, //U+48DE
0, //U+48DF
0, //U+48E0
0, //U+48E1
0, //U+48E2
0, //U+48E3
0, //U+48E4
0, //U+48E5
0, //U+48E6
0, //U+48E7
0, //U+48E8
0, //U+48E9
0, //U+48EA
0, //U+48EB
0, //U+48EC
0, //U+48ED
0, //U+48EE
0, //U+48EF
0, //U+48F0
0, //U+48F1
0, //U+48F2
0, //U+48F3
0, //U+48F4
0, //U+48F5
0, //U+48F6
0, //U+48F7
0, //U+48F8
0, //U+48F9
0, //U+48FA
0, //U+48FB
0, //U+48FC
0, //U+48FD
0, //U+48FE
0, //U+48FF
0, //U+4900
0, //U+4901
0, //U+4902
0, //U+4903
0, //U+4904
0, //U+4905
0, //U+4906
0, //U+4907
0, //U+4908
0, //U+4909
0, //U+490A
0, //U+490B
0, //U+490C
0, //U+490D
0, //U+490E
0, //U+490F
0, //U+4910
0, //U+4911
0, //U+4912
0, //U+4913
0, //U+4914
0, //U+4915
0, //U+4916
0, //U+4917
0, //U+4918
0, //U+4919
0, //U+491A
0, //U+491B
0, //U+491C
0, //U+491D
0, //U+491E
0, //U+491F
0, //U+4920
0, //U+4921
0, //U+4922
0, //U+4923
0, //U+4924
0, //U+4925
0, //U+4926
0, //U+4927
0, //U+4928
0, //U+4929
0, //U+492A
0, //U+492B
0, //U+492C
0, //U+492D
0, //U+492E
0, //U+492F
0, //U+4930
0, //U+4931
0, //U+4932
0, //U+4933
0, //U+4934
0, //U+4935
0, //U+4936
0, //U+4937
0, //U+4938
0, //U+4939
0, //U+493A
0, //U+493B
0, //U+493C
0, //U+493D
0, //U+493E
0, //U+493F
0, //U+4940
0, //U+4941
0, //U+4942
0, //U+4943
0, //U+4944
0, //U+4945
0, //U+4946
0, //U+4947
0, //U+4948
0, //U+4949
0, //U+494A
0, //U+494B
0, //U+494C
0, //U+494D
0, //U+494E
0, //U+494F
0, //U+4950
0, //U+4951
0, //U+4952
0, //U+4953
0, //U+4954
0, //U+4955
0, //U+4956
0, //U+4957
0, //U+4958
0, //U+4959
0, //U+495A
0, //U+495B
0, //U+495C
0, //U+495D
0, //U+495E
0, //U+495F
0, //U+4960
0, //U+4961
0, //U+4962
0, //U+4963
0, //U+4964
0, //U+4965
0, //U+4966
0, //U+4967
0, //U+4968
0, //U+4969
0, //U+496A
0, //U+496B
0, //U+496C
0, //U+496D
0, //U+496E
0, //U+496F
0, //U+4970
0, //U+4971
0, //U+4972
0, //U+4973
0, //U+4974
0, //U+4975
0, //U+4976
0, //U+4977
0, //U+4978
0, //U+4979
0, //U+497A
0, //U+497B
0, //U+497C
0, //U+497D
0, //U+497E
0, //U+497F
0, //U+4980
0, //U+4981
0, //U+4982
0, //U+4983
0, //U+4984
0, //U+4985
0, //U+4986
0, //U+4987
0, //U+4988
0, //U+4989
0, //U+498A
0, //U+498B
0, //U+498C
0, //U+498D
0, //U+498E
0, //U+498F
0, //U+4990
0, //U+4991
0, //U+4992
0, //U+4993
0, //U+4994
0, //U+4995
0, //U+4996
0, //U+4997
0, //U+4998
0, //U+4999
0, //U+499A
0, //U+499B
0, //U+499C
0, //U+499D
0, //U+499E
0, //U+499F
0, //U+49A0
0, //U+49A1
0, //U+49A2
0, //U+49A3
0, //U+49A4
0, //U+49A5
0, //U+49A6
0, //U+49A7
0, //U+49A8
0, //U+49A9
0, //U+49AA
0, //U+49AB
0, //U+49AC
0, //U+49AD
0, //U+49AE
0, //U+49AF
0, //U+49B0
0, //U+49B1
0, //U+49B2
0, //U+49B3
0, //U+49B4
0, //U+49B5
0, //U+49B6
0, //U+49B7
0, //U+49B8
0, //U+49B9
0, //U+49BA
0, //U+49BB
0, //U+49BC
0, //U+49BD
0, //U+49BE
0, //U+49BF
0, //U+49C0
0, //U+49C1
0, //U+49C2
0, //U+49C3
0, //U+49C4
0, //U+49C5
0, //U+49C6
0, //U+49C7
0, //U+49C8
0, //U+49C9
0, //U+49CA
0, //U+49CB
0, //U+49CC
0, //U+49CD
0, //U+49CE
0, //U+49CF
0, //U+49D0
0, //U+49D1
0, //U+49D2
0, //U+49D3
0, //U+49D4
0, //U+49D5
0, //U+49D6
0, //U+49D7
0, //U+49D8
0, //U+49D9
0, //U+49DA
0, //U+49DB
0, //U+49DC
0, //U+49DD
0, //U+49DE
0, //U+49DF
0, //U+49E0
0, //U+49E1
0, //U+49E2
0, //U+49E3
0, //U+49E4
0, //U+49E5
0, //U+49E6
0, //U+49E7
0, //U+49E8
0, //U+49E9
0, //U+49EA
0, //U+49EB
0, //U+49EC
0, //U+49ED
0, //U+49EE
0, //U+49EF
0, //U+49F0
0, //U+49F1
0, //U+49F2
0, //U+49F3
0, //U+49F4
0, //U+49F5
0, //U+49F6
0, //U+49F7
0, //U+49F8
0, //U+49F9
0, //U+49FA
0, //U+49FB
0, //U+49FC
0, //U+49FD
0, //U+49FE
0, //U+49FF
0, //U+4A00
0, //U+4A01
0, //U+4A02
0, //U+4A03
0, //U+4A04
0, //U+4A05
0, //U+4A06
0, //U+4A07
0, //U+4A08
0, //U+4A09
0, //U+4A0A
0, //U+4A0B
0, //U+4A0C
0, //U+4A0D
0, //U+4A0E
0, //U+4A0F
0, //U+4A10
0, //U+4A11
0, //U+4A12
0, //U+4A13
0, //U+4A14
0, //U+4A15
0, //U+4A16
0, //U+4A17
0, //U+4A18
0, //U+4A19
0, //U+4A1A
0, //U+4A1B
0, //U+4A1C
0, //U+4A1D
0, //U+4A1E
0, //U+4A1F
0, //U+4A20
0, //U+4A21
0, //U+4A22
0, //U+4A23
0, //U+4A24
0, //U+4A25
0, //U+4A26
0, //U+4A27
0, //U+4A28
0, //U+4A29
0, //U+4A2A
0, //U+4A2B
0, //U+4A2C
0, //U+4A2D
0, //U+4A2E
0, //U+4A2F
0, //U+4A30
0, //U+4A31
0, //U+4A32
0, //U+4A33
0, //U+4A34
0, //U+4A35
0, //U+4A36
0, //U+4A37
0, //U+4A38
0, //U+4A39
0, //U+4A3A
0, //U+4A3B
0, //U+4A3C
0, //U+4A3D
0, //U+4A3E
0, //U+4A3F
0, //U+4A40
0, //U+4A41
0, //U+4A42
0, //U+4A43
0, //U+4A44
0, //U+4A45
0, //U+4A46
0, //U+4A47
0, //U+4A48
0, //U+4A49
0, //U+4A4A
0, //U+4A4B
0, //U+4A4C
0, //U+4A4D
0, //U+4A4E
0, //U+4A4F
0, //U+4A50
0, //U+4A51
0, //U+4A52
0, //U+4A53
0, //U+4A54
0, //U+4A55
0, //U+4A56
0, //U+4A57
0, //U+4A58
0, //U+4A59
0, //U+4A5A
0, //U+4A5B
0, //U+4A5C
0, //U+4A5D
0, //U+4A5E
0, //U+4A5F
0, //U+4A60
0, //U+4A61
0, //U+4A62
0, //U+4A63
0, //U+4A64
0, //U+4A65
0, //U+4A66
0, //U+4A67
0, //U+4A68
0, //U+4A69
0, //U+4A6A
0, //U+4A6B
0, //U+4A6C
0, //U+4A6D
0, //U+4A6E
0, //U+4A6F
0, //U+4A70
0, //U+4A71
0, //U+4A72
0, //U+4A73
0, //U+4A74
0, //U+4A75
0, //U+4A76
0, //U+4A77
0, //U+4A78
0, //U+4A79
0, //U+4A7A
0, //U+4A7B
0, //U+4A7C
0, //U+4A7D
0, //U+4A7E
0, //U+4A7F
0, //U+4A80
0, //U+4A81
0, //U+4A82
0, //U+4A83
0, //U+4A84
0, //U+4A85
0, //U+4A86
0, //U+4A87
0, //U+4A88
0, //U+4A89
0, //U+4A8A
0, //U+4A8B
0, //U+4A8C
0, //U+4A8D
0, //U+4A8E
0, //U+4A8F
0, //U+4A90
0, //U+4A91
0, //U+4A92
0, //U+4A93
0, //U+4A94
0, //U+4A95
0, //U+4A96
0, //U+4A97
0, //U+4A98
0, //U+4A99
0, //U+4A9A
0, //U+4A9B
0, //U+4A9C
0, //U+4A9D
0, //U+4A9E
0, //U+4A9F
0, //U+4AA0
0, //U+4AA1
0, //U+4AA2
0, //U+4AA3
0, //U+4AA4
0, //U+4AA5
0, //U+4AA6
0, //U+4AA7
0, //U+4AA8
0, //U+4AA9
0, //U+4AAA
0, //U+4AAB
0, //U+4AAC
0, //U+4AAD
0, //U+4AAE
0, //U+4AAF
0, //U+4AB0
0, //U+4AB1
0, //U+4AB2
0, //U+4AB3
0, //U+4AB4
0, //U+4AB5
0, //U+4AB6
0, //U+4AB7
0, //U+4AB8
0, //U+4AB9
0, //U+4ABA
0, //U+4ABB
0, //U+4ABC
0, //U+4ABD
0, //U+4ABE
0, //U+4ABF
0, //U+4AC0
0, //U+4AC1
0, //U+4AC2
0, //U+4AC3
0, //U+4AC4
0, //U+4AC5
0, //U+4AC6
0, //U+4AC7
0, //U+4AC8
0, //U+4AC9
0, //U+4ACA
0, //U+4ACB
0, //U+4ACC
0, //U+4ACD
0, //U+4ACE
0, //U+4ACF
0, //U+4AD0
0, //U+4AD1
0, //U+4AD2
0, //U+4AD3
0, //U+4AD4
0, //U+4AD5
0, //U+4AD6
0, //U+4AD7
0, //U+4AD8
0, //U+4AD9
0, //U+4ADA
0, //U+4ADB
0, //U+4ADC
0, //U+4ADD
0, //U+4ADE
0, //U+4ADF
0, //U+4AE0
0, //U+4AE1
0, //U+4AE2
0, //U+4AE3
0, //U+4AE4
0, //U+4AE5
0, //U+4AE6
0, //U+4AE7
0, //U+4AE8
0, //U+4AE9
0, //U+4AEA
0, //U+4AEB
0, //U+4AEC
0, //U+4AED
0, //U+4AEE
0, //U+4AEF
0, //U+4AF0
0, //U+4AF1
0, //U+4AF2
0, //U+4AF3
0, //U+4AF4
0, //U+4AF5
0, //U+4AF6
0, //U+4AF7
0, //U+4AF8
0, //U+4AF9
0, //U+4AFA
0, //U+4AFB
0, //U+4AFC
0, //U+4AFD
0, //U+4AFE
0, //U+4AFF
0, //U+4B00
0, //U+4B01
0, //U+4B02
0, //U+4B03
0, //U+4B04
0, //U+4B05
0, //U+4B06
0, //U+4B07
0, //U+4B08
0, //U+4B09
0, //U+4B0A
0, //U+4B0B
0, //U+4B0C
0, //U+4B0D
0, //U+4B0E
0, //U+4B0F
0, //U+4B10
0, //U+4B11
0, //U+4B12
0, //U+4B13
0, //U+4B14
0, //U+4B15
0, //U+4B16
0, //U+4B17
0, //U+4B18
0, //U+4B19
0, //U+4B1A
0, //U+4B1B
0, //U+4B1C
0, //U+4B1D
0, //U+4B1E
0, //U+4B1F
0, //U+4B20
0, //U+4B21
0, //U+4B22
0, //U+4B23
0, //U+4B24
0, //U+4B25
0, //U+4B26
0, //U+4B27
0, //U+4B28
0, //U+4B29
0, //U+4B2A
0, //U+4B2B
0, //U+4B2C
0, //U+4B2D
0, //U+4B2E
0, //U+4B2F
0, //U+4B30
0, //U+4B31
0, //U+4B32
0, //U+4B33
0, //U+4B34
0, //U+4B35
0, //U+4B36
0, //U+4B37
0, //U+4B38
0, //U+4B39
0, //U+4B3A
0, //U+4B3B
0, //U+4B3C
0, //U+4B3D
0, //U+4B3E
0, //U+4B3F
0, //U+4B40
0, //U+4B41
0, //U+4B42
0, //U+4B43
0, //U+4B44
0, //U+4B45
0, //U+4B46
0, //U+4B47
0, //U+4B48
0, //U+4B49
0, //U+4B4A
0, //U+4B4B
0, //U+4B4C
0, //U+4B4D
0, //U+4B4E
0, //U+4B4F
0, //U+4B50
0, //U+4B51
0, //U+4B52
0, //U+4B53
0, //U+4B54
0, //U+4B55
0, //U+4B56
0, //U+4B57
0, //U+4B58
0, //U+4B59
0, //U+4B5A
0, //U+4B5B
0, //U+4B5C
0, //U+4B5D
0, //U+4B5E
0, //U+4B5F
0, //U+4B60
0, //U+4B61
0, //U+4B62
0, //U+4B63
0, //U+4B64
0, //U+4B65
0, //U+4B66
0, //U+4B67
0, //U+4B68
0, //U+4B69
0, //U+4B6A
0, //U+4B6B
0, //U+4B6C
0, //U+4B6D
0, //U+4B6E
0, //U+4B6F
0, //U+4B70
0, //U+4B71
0, //U+4B72
0, //U+4B73
0, //U+4B74
0, //U+4B75
0, //U+4B76
0, //U+4B77
0, //U+4B78
0, //U+4B79
0, //U+4B7A
0, //U+4B7B
0, //U+4B7C
0, //U+4B7D
0, //U+4B7E
0, //U+4B7F
0, //U+4B80
0, //U+4B81
0, //U+4B82
0, //U+4B83
0, //U+4B84
0, //U+4B85
0, //U+4B86
0, //U+4B87
0, //U+4B88
0, //U+4B89
0, //U+4B8A
0, //U+4B8B
0, //U+4B8C
0, //U+4B8D
0, //U+4B8E
0, //U+4B8F
0, //U+4B90
0, //U+4B91
0, //U+4B92
0, //U+4B93
0, //U+4B94
0, //U+4B95
0, //U+4B96
0, //U+4B97
0, //U+4B98
0, //U+4B99
0, //U+4B9A
0, //U+4B9B
0, //U+4B9C
0, //U+4B9D
0, //U+4B9E
0, //U+4B9F
0, //U+4BA0
0, //U+4BA1
0, //U+4BA2
0, //U+4BA3
0, //U+4BA4
0, //U+4BA5
0, //U+4BA6
0, //U+4BA7
0, //U+4BA8
0, //U+4BA9
0, //U+4BAA
0, //U+4BAB
0, //U+4BAC
0, //U+4BAD
0, //U+4BAE
0, //U+4BAF
0, //U+4BB0
0, //U+4BB1
0, //U+4BB2
0, //U+4BB3
0, //U+4BB4
0, //U+4BB5
0, //U+4BB6
0, //U+4BB7
0, //U+4BB8
0, //U+4BB9
0, //U+4BBA
0, //U+4BBB
0, //U+4BBC
0, //U+4BBD
0, //U+4BBE
0, //U+4BBF
0, //U+4BC0
0, //U+4BC1
0, //U+4BC2
0, //U+4BC3
0, //U+4BC4
0, //U+4BC5
0, //U+4BC6
0, //U+4BC7
0, //U+4BC8
0, //U+4BC9
0, //U+4BCA
0, //U+4BCB
0, //U+4BCC
0, //U+4BCD
0, //U+4BCE
0, //U+4BCF
0, //U+4BD0
0, //U+4BD1
0, //U+4BD2
0, //U+4BD3
0, //U+4BD4
0, //U+4BD5
0, //U+4BD6
0, //U+4BD7
0, //U+4BD8
0, //U+4BD9
0, //U+4BDA
0, //U+4BDB
0, //U+4BDC
0, //U+4BDD
0, //U+4BDE
0, //U+4BDF
0, //U+4BE0
0, //U+4BE1
0, //U+4BE2
0, //U+4BE3
0, //U+4BE4
0, //U+4BE5
0, //U+4BE6
0, //U+4BE7
0, //U+4BE8
0, //U+4BE9
0, //U+4BEA
0, //U+4BEB
0, //U+4BEC
0, //U+4BED
0, //U+4BEE
0, //U+4BEF
0, //U+4BF0
0, //U+4BF1
0, //U+4BF2
0, //U+4BF3
0, //U+4BF4
0, //U+4BF5
0, //U+4BF6
0, //U+4BF7
0, //U+4BF8
0, //U+4BF9
0, //U+4BFA
0, //U+4BFB
0, //U+4BFC
0, //U+4BFD
0, //U+4BFE
0, //U+4BFF
0, //U+4C00
0, //U+4C01
0, //U+4C02
0, //U+4C03
0, //U+4C04
0, //U+4C05
0, //U+4C06
0, //U+4C07
0, //U+4C08
0, //U+4C09
0, //U+4C0A
0, //U+4C0B
0, //U+4C0C
0, //U+4C0D
0, //U+4C0E
0, //U+4C0F
0, //U+4C10
0, //U+4C11
0, //U+4C12
0, //U+4C13
0, //U+4C14
0, //U+4C15
0, //U+4C16
0, //U+4C17
0, //U+4C18
0, //U+4C19
0, //U+4C1A
0, //U+4C1B
0, //U+4C1C
0, //U+4C1D
0, //U+4C1E
0, //U+4C1F
0, //U+4C20
0, //U+4C21
0, //U+4C22
0, //U+4C23
0, //U+4C24
0, //U+4C25
0, //U+4C26
0, //U+4C27
0, //U+4C28
0, //U+4C29
0, //U+4C2A
0, //U+4C2B
0, //U+4C2C
0, //U+4C2D
0, //U+4C2E
0, //U+4C2F
0, //U+4C30
0, //U+4C31
0, //U+4C32
0, //U+4C33
0, //U+4C34
0, //U+4C35
0, //U+4C36
0, //U+4C37
0, //U+4C38
0, //U+4C39
0, //U+4C3A
0, //U+4C3B
0, //U+4C3C
0, //U+4C3D
0, //U+4C3E
0, //U+4C3F
0, //U+4C40
0, //U+4C41
0, //U+4C42
0, //U+4C43
0, //U+4C44
0, //U+4C45
0, //U+4C46
0, //U+4C47
0, //U+4C48
0, //U+4C49
0, //U+4C4A
0, //U+4C4B
0, //U+4C4C
0, //U+4C4D
0, //U+4C4E
0, //U+4C4F
0, //U+4C50
0, //U+4C51
0, //U+4C52
0, //U+4C53
0, //U+4C54
0, //U+4C55
0, //U+4C56
0, //U+4C57
0, //U+4C58
0, //U+4C59
0, //U+4C5A
0, //U+4C5B
0, //U+4C5C
0, //U+4C5D
0, //U+4C5E
0, //U+4C5F
0, //U+4C60
0, //U+4C61
0, //U+4C62
0, //U+4C63
0, //U+4C64
0, //U+4C65
0, //U+4C66
0, //U+4C67
0, //U+4C68
0, //U+4C69
0, //U+4C6A
0, //U+4C6B
0, //U+4C6C
0, //U+4C6D
0, //U+4C6E
0, //U+4C6F
0, //U+4C70
0, //U+4C71
0, //U+4C72
0, //U+4C73
0, //U+4C74
0, //U+4C75
0, //U+4C76
0, //U+4C77
0, //U+4C78
0, //U+4C79
0, //U+4C7A
0, //U+4C7B
0, //U+4C7C
0, //U+4C7D
0, //U+4C7E
0, //U+4C7F
0, //U+4C80
0, //U+4C81
0, //U+4C82
0, //U+4C83
0, //U+4C84
0, //U+4C85
0, //U+4C86
0, //U+4C87
0, //U+4C88
0, //U+4C89
0, //U+4C8A
0, //U+4C8B
0, //U+4C8C
0, //U+4C8D
0, //U+4C8E
0, //U+4C8F
0, //U+4C90
0, //U+4C91
0, //U+4C92
0, //U+4C93
0, //U+4C94
0, //U+4C95
0, //U+4C96
0, //U+4C97
0, //U+4C98
0, //U+4C99
0, //U+4C9A
0, //U+4C9B
0, //U+4C9C
0, //U+4C9D
0, //U+4C9E
0, //U+4C9F
0, //U+4CA0
0, //U+4CA1
0, //U+4CA2
0, //U+4CA3
0, //U+4CA4
0, //U+4CA5
0, //U+4CA6
0, //U+4CA7
0, //U+4CA8
0, //U+4CA9
0, //U+4CAA
0, //U+4CAB
0, //U+4CAC
0, //U+4CAD
0, //U+4CAE
0, //U+4CAF
0, //U+4CB0
0, //U+4CB1
0, //U+4CB2
0, //U+4CB3
0, //U+4CB4
0, //U+4CB5
0, //U+4CB6
0, //U+4CB7
0, //U+4CB8
0, //U+4CB9
0, //U+4CBA
0, //U+4CBB
0, //U+4CBC
0, //U+4CBD
0, //U+4CBE
0, //U+4CBF
0, //U+4CC0
0, //U+4CC1
0, //U+4CC2
0, //U+4CC3
0, //U+4CC4
0, //U+4CC5
0, //U+4CC6
0, //U+4CC7
0, //U+4CC8
0, //U+4CC9
0, //U+4CCA
0, //U+4CCB
0, //U+4CCC
0, //U+4CCD
0, //U+4CCE
0, //U+4CCF
0, //U+4CD0
0, //U+4CD1
0, //U+4CD2
0, //U+4CD3
0, //U+4CD4
0, //U+4CD5
0, //U+4CD6
0, //U+4CD7
0, //U+4CD8
0, //U+4CD9
0, //U+4CDA
0, //U+4CDB
0, //U+4CDC
0, //U+4CDD
0, //U+4CDE
0, //U+4CDF
0, //U+4CE0
0, //U+4CE1
0, //U+4CE2
0, //U+4CE3
0, //U+4CE4
0, //U+4CE5
0, //U+4CE6
0, //U+4CE7
0, //U+4CE8
0, //U+4CE9
0, //U+4CEA
0, //U+4CEB
0, //U+4CEC
0, //U+4CED
0, //U+4CEE
0, //U+4CEF
0, //U+4CF0
0, //U+4CF1
0, //U+4CF2
0, //U+4CF3
0, //U+4CF4
0, //U+4CF5
0, //U+4CF6
0, //U+4CF7
0, //U+4CF8
0, //U+4CF9
0, //U+4CFA
0, //U+4CFB
0, //U+4CFC
0, //U+4CFD
0, //U+4CFE
0, //U+4CFF
0, //U+4D00
0, //U+4D01
0, //U+4D02
0, //U+4D03
0, //U+4D04
0, //U+4D05
0, //U+4D06
0, //U+4D07
0, //U+4D08
0, //U+4D09
0, //U+4D0A
0, //U+4D0B
0, //U+4D0C
0, //U+4D0D
0, //U+4D0E
0, //U+4D0F
0, //U+4D10
0, //U+4D11
0, //U+4D12
0, //U+4D13
0, //U+4D14
0, //U+4D15
0, //U+4D16
0, //U+4D17
0, //U+4D18
0, //U+4D19
0, //U+4D1A
0, //U+4D1B
0, //U+4D1C
0, //U+4D1D
0, //U+4D1E
0, //U+4D1F
0, //U+4D20
0, //U+4D21
0, //U+4D22
0, //U+4D23
0, //U+4D24
0, //U+4D25
0, //U+4D26
0, //U+4D27
0, //U+4D28
0, //U+4D29
0, //U+4D2A
0, //U+4D2B
0, //U+4D2C
0, //U+4D2D
0, //U+4D2E
0, //U+4D2F
0, //U+4D30
0, //U+4D31
0, //U+4D32
0, //U+4D33
0, //U+4D34
0, //U+4D35
0, //U+4D36
0, //U+4D37
0, //U+4D38
0, //U+4D39
0, //U+4D3A
0, //U+4D3B
0, //U+4D3C
0, //U+4D3D
0, //U+4D3E
0, //U+4D3F
0, //U+4D40
0, //U+4D41
0, //U+4D42
0, //U+4D43
0, //U+4D44
0, //U+4D45
0, //U+4D46
0, //U+4D47
0, //U+4D48
0, //U+4D49
0, //U+4D4A
0, //U+4D4B
0, //U+4D4C
0, //U+4D4D
0, //U+4D4E
0, //U+4D4F
0, //U+4D50
0, //U+4D51
0, //U+4D52
0, //U+4D53
0, //U+4D54
0, //U+4D55
0, //U+4D56
0, //U+4D57
0, //U+4D58
0, //U+4D59
0, //U+4D5A
0, //U+4D5B
0, //U+4D5C
0, //U+4D5D
0, //U+4D5E
0, //U+4D5F
0, //U+4D60
0, //U+4D61
0, //U+4D62
0, //U+4D63
0, //U+4D64
0, //U+4D65
0, //U+4D66
0, //U+4D67
0, //U+4D68
0, //U+4D69
0, //U+4D6A
0, //U+4D6B
0, //U+4D6C
0, //U+4D6D
0, //U+4D6E
0, //U+4D6F
0, //U+4D70
0, //U+4D71
0, //U+4D72
0, //U+4D73
0, //U+4D74
0, //U+4D75
0, //U+4D76
0, //U+4D77
0, //U+4D78
0, //U+4D79
0, //U+4D7A
0, //U+4D7B
0, //U+4D7C
0, //U+4D7D
0, //U+4D7E
0, //U+4D7F
0, //U+4D80
0, //U+4D81
0, //U+4D82
0, //U+4D83
0, //U+4D84
0, //U+4D85
0, //U+4D86
0, //U+4D87
0, //U+4D88
0, //U+4D89
0, //U+4D8A
0, //U+4D8B
0, //U+4D8C
0, //U+4D8D
0, //U+4D8E
0, //U+4D8F
0, //U+4D90
0, //U+4D91
0, //U+4D92
0, //U+4D93
0, //U+4D94
0, //U+4D95
0, //U+4D96
0, //U+4D97
0, //U+4D98
0, //U+4D99
0, //U+4D9A
0, //U+4D9B
0, //U+4D9C
0, //U+4D9D
0, //U+4D9E
0, //U+4D9F
0, //U+4DA0
0, //U+4DA1
0, //U+4DA2
0, //U+4DA3
0, //U+4DA4
0, //U+4DA5
0, //U+4DA6
0, //U+4DA7
0, //U+4DA8
0, //U+4DA9
0, //U+4DAA
0, //U+4DAB
0, //U+4DAC
0, //U+4DAD
0, //U+4DAE
0, //U+4DAF
0, //U+4DB0
0, //U+4DB1
0, //U+4DB2
0, //U+4DB3
0, //U+4DB4
UNICODE_Lo, //U+4DB5
0, //U+4DB6
0, //U+4DB7
0, //U+4DB8
0, //U+4DB9
0, //U+4DBA
0, //U+4DBB
0, //U+4DBC
0, //U+4DBD
0, //U+4DBE
0, //U+4DBF
UNICODE_So, //U+4DC0
UNICODE_So, //U+4DC1
UNICODE_So, //U+4DC2
UNICODE_So, //U+4DC3
UNICODE_So, //U+4DC4
UNICODE_So, //U+4DC5
UNICODE_So, //U+4DC6
UNICODE_So, //U+4DC7
UNICODE_So, //U+4DC8
UNICODE_So, //U+4DC9
UNICODE_So, //U+4DCA
UNICODE_So, //U+4DCB
UNICODE_So, //U+4DCC
UNICODE_So, //U+4DCD
UNICODE_So, //U+4DCE
UNICODE_So, //U+4DCF
UNICODE_So, //U+4DD0
UNICODE_So, //U+4DD1
UNICODE_So, //U+4DD2
UNICODE_So, //U+4DD3
UNICODE_So, //U+4DD4
UNICODE_So, //U+4DD5
UNICODE_So, //U+4DD6
UNICODE_So, //U+4DD7
UNICODE_So, //U+4DD8
UNICODE_So, //U+4DD9
UNICODE_So, //U+4DDA
UNICODE_So, //U+4DDB
UNICODE_So, //U+4DDC
UNICODE_So, //U+4DDD
UNICODE_So, //U+4DDE
UNICODE_So, //U+4DDF
UNICODE_So, //U+4DE0
UNICODE_So, //U+4DE1
UNICODE_So, //U+4DE2
UNICODE_So, //U+4DE3
UNICODE_So, //U+4DE4
UNICODE_So, //U+4DE5
UNICODE_So, //U+4DE6
UNICODE_So, //U+4DE7
UNICODE_So, //U+4DE8
UNICODE_So, //U+4DE9
UNICODE_So, //U+4DEA
UNICODE_So, //U+4DEB
UNICODE_So, //U+4DEC
UNICODE_So, //U+4DED
UNICODE_So, //U+4DEE
UNICODE_So, //U+4DEF
UNICODE_So, //U+4DF0
UNICODE_So, //U+4DF1
UNICODE_So, //U+4DF2
UNICODE_So, //U+4DF3
UNICODE_So, //U+4DF4
UNICODE_So, //U+4DF5
UNICODE_So, //U+4DF6
UNICODE_So, //U+4DF7
UNICODE_So, //U+4DF8
UNICODE_So, //U+4DF9
UNICODE_So, //U+4DFA
UNICODE_So, //U+4DFB
UNICODE_So, //U+4DFC
UNICODE_So, //U+4DFD
UNICODE_So, //U+4DFE
UNICODE_So, //U+4DFF
UNICODE_Lo, //U+4E00
0, //U+4E01
0, //U+4E02
0, //U+4E03
0, //U+4E04
0, //U+4E05
0, //U+4E06
0, //U+4E07
0, //U+4E08
0, //U+4E09
0, //U+4E0A
0, //U+4E0B
0, //U+4E0C
0, //U+4E0D
0, //U+4E0E
0, //U+4E0F
0, //U+4E10
0, //U+4E11
0, //U+4E12
0, //U+4E13
0, //U+4E14
0, //U+4E15
0, //U+4E16
0, //U+4E17
0, //U+4E18
0, //U+4E19
0, //U+4E1A
0, //U+4E1B
0, //U+4E1C
0, //U+4E1D
0, //U+4E1E
0, //U+4E1F
0, //U+4E20
0, //U+4E21
0, //U+4E22
0, //U+4E23
0, //U+4E24
0, //U+4E25
0, //U+4E26
0, //U+4E27
0, //U+4E28
0, //U+4E29
0, //U+4E2A
0, //U+4E2B
0, //U+4E2C
0, //U+4E2D
0, //U+4E2E
0, //U+4E2F
0, //U+4E30
0, //U+4E31
0, //U+4E32
0, //U+4E33
0, //U+4E34
0, //U+4E35
0, //U+4E36
0, //U+4E37
0, //U+4E38
0, //U+4E39
0, //U+4E3A
0, //U+4E3B
0, //U+4E3C
0, //U+4E3D
0, //U+4E3E
0, //U+4E3F
0, //U+4E40
0, //U+4E41
0, //U+4E42
0, //U+4E43
0, //U+4E44
0, //U+4E45
0, //U+4E46
0, //U+4E47
0, //U+4E48
0, //U+4E49
0, //U+4E4A
0, //U+4E4B
0, //U+4E4C
0, //U+4E4D
0, //U+4E4E
0, //U+4E4F
0, //U+4E50
0, //U+4E51
0, //U+4E52
0, //U+4E53
0, //U+4E54
0, //U+4E55
0, //U+4E56
0, //U+4E57
0, //U+4E58
0, //U+4E59
0, //U+4E5A
0, //U+4E5B
0, //U+4E5C
0, //U+4E5D
0, //U+4E5E
0, //U+4E5F
0, //U+4E60
0, //U+4E61
0, //U+4E62
0, //U+4E63
0, //U+4E64
0, //U+4E65
0, //U+4E66
0, //U+4E67
0, //U+4E68
0, //U+4E69
0, //U+4E6A
0, //U+4E6B
0, //U+4E6C
0, //U+4E6D
0, //U+4E6E
0, //U+4E6F
0, //U+4E70
0, //U+4E71
0, //U+4E72
0, //U+4E73
0, //U+4E74
0, //U+4E75
0, //U+4E76
0, //U+4E77
0, //U+4E78
0, //U+4E79
0, //U+4E7A
0, //U+4E7B
0, //U+4E7C
0, //U+4E7D
0, //U+4E7E
0, //U+4E7F
0, //U+4E80
0, //U+4E81
0, //U+4E82
0, //U+4E83
0, //U+4E84
0, //U+4E85
0, //U+4E86
0, //U+4E87
0, //U+4E88
0, //U+4E89
0, //U+4E8A
0, //U+4E8B
0, //U+4E8C
0, //U+4E8D
0, //U+4E8E
0, //U+4E8F
0, //U+4E90
0, //U+4E91
0, //U+4E92
0, //U+4E93
0, //U+4E94
0, //U+4E95
0, //U+4E96
0, //U+4E97
0, //U+4E98
0, //U+4E99
0, //U+4E9A
0, //U+4E9B
0, //U+4E9C
0, //U+4E9D
0, //U+4E9E
0, //U+4E9F
0, //U+4EA0
0, //U+4EA1
0, //U+4EA2
0, //U+4EA3
0, //U+4EA4
0, //U+4EA5
0, //U+4EA6
0, //U+4EA7
0, //U+4EA8
0, //U+4EA9
0, //U+4EAA
0, //U+4EAB
0, //U+4EAC
0, //U+4EAD
0, //U+4EAE
0, //U+4EAF
0, //U+4EB0
0, //U+4EB1
0, //U+4EB2
0, //U+4EB3
0, //U+4EB4
0, //U+4EB5
0, //U+4EB6
0, //U+4EB7
0, //U+4EB8
0, //U+4EB9
0, //U+4EBA
0, //U+4EBB
0, //U+4EBC
0, //U+4EBD
0, //U+4EBE
0, //U+4EBF
0, //U+4EC0
0, //U+4EC1
0, //U+4EC2
0, //U+4EC3
0, //U+4EC4
0, //U+4EC5
0, //U+4EC6
0, //U+4EC7
0, //U+4EC8
0, //U+4EC9
0, //U+4ECA
0, //U+4ECB
0, //U+4ECC
0, //U+4ECD
0, //U+4ECE
0, //U+4ECF
0, //U+4ED0
0, //U+4ED1
0, //U+4ED2
0, //U+4ED3
0, //U+4ED4
0, //U+4ED5
0, //U+4ED6
0, //U+4ED7
0, //U+4ED8
0, //U+4ED9
0, //U+4EDA
0, //U+4EDB
0, //U+4EDC
0, //U+4EDD
0, //U+4EDE
0, //U+4EDF
0, //U+4EE0
0, //U+4EE1
0, //U+4EE2
0, //U+4EE3
0, //U+4EE4
0, //U+4EE5
0, //U+4EE6
0, //U+4EE7
0, //U+4EE8
0, //U+4EE9
0, //U+4EEA
0, //U+4EEB
0, //U+4EEC
0, //U+4EED
0, //U+4EEE
0, //U+4EEF
0, //U+4EF0
0, //U+4EF1
0, //U+4EF2
0, //U+4EF3
0, //U+4EF4
0, //U+4EF5
0, //U+4EF6
0, //U+4EF7
0, //U+4EF8
0, //U+4EF9
0, //U+4EFA
0, //U+4EFB
0, //U+4EFC
0, //U+4EFD
0, //U+4EFE
0, //U+4EFF
0, //U+4F00
0, //U+4F01
0, //U+4F02
0, //U+4F03
0, //U+4F04
0, //U+4F05
0, //U+4F06
0, //U+4F07
0, //U+4F08
0, //U+4F09
0, //U+4F0A
0, //U+4F0B
0, //U+4F0C
0, //U+4F0D
0, //U+4F0E
0, //U+4F0F
0, //U+4F10
0, //U+4F11
0, //U+4F12
0, //U+4F13
0, //U+4F14
0, //U+4F15
0, //U+4F16
0, //U+4F17
0, //U+4F18
0, //U+4F19
0, //U+4F1A
0, //U+4F1B
0, //U+4F1C
0, //U+4F1D
0, //U+4F1E
0, //U+4F1F
0, //U+4F20
0, //U+4F21
0, //U+4F22
0, //U+4F23
0, //U+4F24
0, //U+4F25
0, //U+4F26
0, //U+4F27
0, //U+4F28
0, //U+4F29
0, //U+4F2A
0, //U+4F2B
0, //U+4F2C
0, //U+4F2D
0, //U+4F2E
0, //U+4F2F
0, //U+4F30
0, //U+4F31
0, //U+4F32
0, //U+4F33
0, //U+4F34
0, //U+4F35
0, //U+4F36
0, //U+4F37
0, //U+4F38
0, //U+4F39
0, //U+4F3A
0, //U+4F3B
0, //U+4F3C
0, //U+4F3D
0, //U+4F3E
0, //U+4F3F
0, //U+4F40
0, //U+4F41
0, //U+4F42
0, //U+4F43
0, //U+4F44
0, //U+4F45
0, //U+4F46
0, //U+4F47
0, //U+4F48
0, //U+4F49
0, //U+4F4A
0, //U+4F4B
0, //U+4F4C
0, //U+4F4D
0, //U+4F4E
0, //U+4F4F
0, //U+4F50
0, //U+4F51
0, //U+4F52
0, //U+4F53
0, //U+4F54
0, //U+4F55
0, //U+4F56
0, //U+4F57
0, //U+4F58
0, //U+4F59
0, //U+4F5A
0, //U+4F5B
0, //U+4F5C
0, //U+4F5D
0, //U+4F5E
0, //U+4F5F
0, //U+4F60
0, //U+4F61
0, //U+4F62
0, //U+4F63
0, //U+4F64
0, //U+4F65
0, //U+4F66
0, //U+4F67
0, //U+4F68
0, //U+4F69
0, //U+4F6A
0, //U+4F6B
0, //U+4F6C
0, //U+4F6D
0, //U+4F6E
0, //U+4F6F
0, //U+4F70
0, //U+4F71
0, //U+4F72
0, //U+4F73
0, //U+4F74
0, //U+4F75
0, //U+4F76
0, //U+4F77
0, //U+4F78
0, //U+4F79
0, //U+4F7A
0, //U+4F7B
0, //U+4F7C
0, //U+4F7D
0, //U+4F7E
0, //U+4F7F
0, //U+4F80
0, //U+4F81
0, //U+4F82
0, //U+4F83
0, //U+4F84
0, //U+4F85
0, //U+4F86
0, //U+4F87
0, //U+4F88
0, //U+4F89
0, //U+4F8A
0, //U+4F8B
0, //U+4F8C
0, //U+4F8D
0, //U+4F8E
0, //U+4F8F
0, //U+4F90
0, //U+4F91
0, //U+4F92
0, //U+4F93
0, //U+4F94
0, //U+4F95
0, //U+4F96
0, //U+4F97
0, //U+4F98
0, //U+4F99
0, //U+4F9A
0, //U+4F9B
0, //U+4F9C
0, //U+4F9D
0, //U+4F9E
0, //U+4F9F
0, //U+4FA0
0, //U+4FA1
0, //U+4FA2
0, //U+4FA3
0, //U+4FA4
0, //U+4FA5
0, //U+4FA6
0, //U+4FA7
0, //U+4FA8
0, //U+4FA9
0, //U+4FAA
0, //U+4FAB
0, //U+4FAC
0, //U+4FAD
0, //U+4FAE
0, //U+4FAF
0, //U+4FB0
0, //U+4FB1
0, //U+4FB2
0, //U+4FB3
0, //U+4FB4
0, //U+4FB5
0, //U+4FB6
0, //U+4FB7
0, //U+4FB8
0, //U+4FB9
0, //U+4FBA
0, //U+4FBB
0, //U+4FBC
0, //U+4FBD
0, //U+4FBE
0, //U+4FBF
0, //U+4FC0
0, //U+4FC1
0, //U+4FC2
0, //U+4FC3
0, //U+4FC4
0, //U+4FC5
0, //U+4FC6
0, //U+4FC7
0, //U+4FC8
0, //U+4FC9
0, //U+4FCA
0, //U+4FCB
0, //U+4FCC
0, //U+4FCD
0, //U+4FCE
0, //U+4FCF
0, //U+4FD0
0, //U+4FD1
0, //U+4FD2
0, //U+4FD3
0, //U+4FD4
0, //U+4FD5
0, //U+4FD6
0, //U+4FD7
0, //U+4FD8
0, //U+4FD9
0, //U+4FDA
0, //U+4FDB
0, //U+4FDC
0, //U+4FDD
0, //U+4FDE
0, //U+4FDF
0, //U+4FE0
0, //U+4FE1
0, //U+4FE2
0, //U+4FE3
0, //U+4FE4
0, //U+4FE5
0, //U+4FE6
0, //U+4FE7
0, //U+4FE8
0, //U+4FE9
0, //U+4FEA
0, //U+4FEB
0, //U+4FEC
0, //U+4FED
0, //U+4FEE
0, //U+4FEF
0, //U+4FF0
0, //U+4FF1
0, //U+4FF2
0, //U+4FF3
0, //U+4FF4
0, //U+4FF5
0, //U+4FF6
0, //U+4FF7
0, //U+4FF8
0, //U+4FF9
0, //U+4FFA
0, //U+4FFB
0, //U+4FFC
0, //U+4FFD
0, //U+4FFE
0, //U+4FFF
0, //U+5000
0, //U+5001
0, //U+5002
0, //U+5003
0, //U+5004
0, //U+5005
0, //U+5006
0, //U+5007
0, //U+5008
0, //U+5009
0, //U+500A
0, //U+500B
0, //U+500C
0, //U+500D
0, //U+500E
0, //U+500F
0, //U+5010
0, //U+5011
0, //U+5012
0, //U+5013
0, //U+5014
0, //U+5015
0, //U+5016
0, //U+5017
0, //U+5018
0, //U+5019
0, //U+501A
0, //U+501B
0, //U+501C
0, //U+501D
0, //U+501E
0, //U+501F
0, //U+5020
0, //U+5021
0, //U+5022
0, //U+5023
0, //U+5024
0, //U+5025
0, //U+5026
0, //U+5027
0, //U+5028
0, //U+5029
0, //U+502A
0, //U+502B
0, //U+502C
0, //U+502D
0, //U+502E
0, //U+502F
0, //U+5030
0, //U+5031
0, //U+5032
0, //U+5033
0, //U+5034
0, //U+5035
0, //U+5036
0, //U+5037
0, //U+5038
0, //U+5039
0, //U+503A
0, //U+503B
0, //U+503C
0, //U+503D
0, //U+503E
0, //U+503F
0, //U+5040
0, //U+5041
0, //U+5042
0, //U+5043
0, //U+5044
0, //U+5045
0, //U+5046
0, //U+5047
0, //U+5048
0, //U+5049
0, //U+504A
0, //U+504B
0, //U+504C
0, //U+504D
0, //U+504E
0, //U+504F
0, //U+5050
0, //U+5051
0, //U+5052
0, //U+5053
0, //U+5054
0, //U+5055
0, //U+5056
0, //U+5057
0, //U+5058
0, //U+5059
0, //U+505A
0, //U+505B
0, //U+505C
0, //U+505D
0, //U+505E
0, //U+505F
0, //U+5060
0, //U+5061
0, //U+5062
0, //U+5063
0, //U+5064
0, //U+5065
0, //U+5066
0, //U+5067
0, //U+5068
0, //U+5069
0, //U+506A
0, //U+506B
0, //U+506C
0, //U+506D
0, //U+506E
0, //U+506F
0, //U+5070
0, //U+5071
0, //U+5072
0, //U+5073
0, //U+5074
0, //U+5075
0, //U+5076
0, //U+5077
0, //U+5078
0, //U+5079
0, //U+507A
0, //U+507B
0, //U+507C
0, //U+507D
0, //U+507E
0, //U+507F
0, //U+5080
0, //U+5081
0, //U+5082
0, //U+5083
0, //U+5084
0, //U+5085
0, //U+5086
0, //U+5087
0, //U+5088
0, //U+5089
0, //U+508A
0, //U+508B
0, //U+508C
0, //U+508D
0, //U+508E
0, //U+508F
0, //U+5090
0, //U+5091
0, //U+5092
0, //U+5093
0, //U+5094
0, //U+5095
0, //U+5096
0, //U+5097
0, //U+5098
0, //U+5099
0, //U+509A
0, //U+509B
0, //U+509C
0, //U+509D
0, //U+509E
0, //U+509F
0, //U+50A0
0, //U+50A1
0, //U+50A2
0, //U+50A3
0, //U+50A4
0, //U+50A5
0, //U+50A6
0, //U+50A7
0, //U+50A8
0, //U+50A9
0, //U+50AA
0, //U+50AB
0, //U+50AC
0, //U+50AD
0, //U+50AE
0, //U+50AF
0, //U+50B0
0, //U+50B1
0, //U+50B2
0, //U+50B3
0, //U+50B4
0, //U+50B5
0, //U+50B6
0, //U+50B7
0, //U+50B8
0, //U+50B9
0, //U+50BA
0, //U+50BB
0, //U+50BC
0, //U+50BD
0, //U+50BE
0, //U+50BF
0, //U+50C0
0, //U+50C1
0, //U+50C2
0, //U+50C3
0, //U+50C4
0, //U+50C5
0, //U+50C6
0, //U+50C7
0, //U+50C8
0, //U+50C9
0, //U+50CA
0, //U+50CB
0, //U+50CC
0, //U+50CD
0, //U+50CE
0, //U+50CF
0, //U+50D0
0, //U+50D1
0, //U+50D2
0, //U+50D3
0, //U+50D4
0, //U+50D5
0, //U+50D6
0, //U+50D7
0, //U+50D8
0, //U+50D9
0, //U+50DA
0, //U+50DB
0, //U+50DC
0, //U+50DD
0, //U+50DE
0, //U+50DF
0, //U+50E0
0, //U+50E1
0, //U+50E2
0, //U+50E3
0, //U+50E4
0, //U+50E5
0, //U+50E6
0, //U+50E7
0, //U+50E8
0, //U+50E9
0, //U+50EA
0, //U+50EB
0, //U+50EC
0, //U+50ED
0, //U+50EE
0, //U+50EF
0, //U+50F0
0, //U+50F1
0, //U+50F2
0, //U+50F3
0, //U+50F4
0, //U+50F5
0, //U+50F6
0, //U+50F7
0, //U+50F8
0, //U+50F9
0, //U+50FA
0, //U+50FB
0, //U+50FC
0, //U+50FD
0, //U+50FE
0, //U+50FF
0, //U+5100
0, //U+5101
0, //U+5102
0, //U+5103
0, //U+5104
0, //U+5105
0, //U+5106
0, //U+5107
0, //U+5108
0, //U+5109
0, //U+510A
0, //U+510B
0, //U+510C
0, //U+510D
0, //U+510E
0, //U+510F
0, //U+5110
0, //U+5111
0, //U+5112
0, //U+5113
0, //U+5114
0, //U+5115
0, //U+5116
0, //U+5117
0, //U+5118
0, //U+5119
0, //U+511A
0, //U+511B
0, //U+511C
0, //U+511D
0, //U+511E
0, //U+511F
0, //U+5120
0, //U+5121
0, //U+5122
0, //U+5123
0, //U+5124
0, //U+5125
0, //U+5126
0, //U+5127
0, //U+5128
0, //U+5129
0, //U+512A
0, //U+512B
0, //U+512C
0, //U+512D
0, //U+512E
0, //U+512F
0, //U+5130
0, //U+5131
0, //U+5132
0, //U+5133
0, //U+5134
0, //U+5135
0, //U+5136
0, //U+5137
0, //U+5138
0, //U+5139
0, //U+513A
0, //U+513B
0, //U+513C
0, //U+513D
0, //U+513E
0, //U+513F
0, //U+5140
0, //U+5141
0, //U+5142
0, //U+5143
0, //U+5144
0, //U+5145
0, //U+5146
0, //U+5147
0, //U+5148
0, //U+5149
0, //U+514A
0, //U+514B
0, //U+514C
0, //U+514D
0, //U+514E
0, //U+514F
0, //U+5150
0, //U+5151
0, //U+5152
0, //U+5153
0, //U+5154
0, //U+5155
0, //U+5156
0, //U+5157
0, //U+5158
0, //U+5159
0, //U+515A
0, //U+515B
0, //U+515C
0, //U+515D
0, //U+515E
0, //U+515F
0, //U+5160
0, //U+5161
0, //U+5162
0, //U+5163
0, //U+5164
0, //U+5165
0, //U+5166
0, //U+5167
0, //U+5168
0, //U+5169
0, //U+516A
0, //U+516B
0, //U+516C
0, //U+516D
0, //U+516E
0, //U+516F
0, //U+5170
0, //U+5171
0, //U+5172
0, //U+5173
0, //U+5174
0, //U+5175
0, //U+5176
0, //U+5177
0, //U+5178
0, //U+5179
0, //U+517A
0, //U+517B
0, //U+517C
0, //U+517D
0, //U+517E
0, //U+517F
0, //U+5180
0, //U+5181
0, //U+5182
0, //U+5183
0, //U+5184
0, //U+5185
0, //U+5186
0, //U+5187
0, //U+5188
0, //U+5189
0, //U+518A
0, //U+518B
0, //U+518C
0, //U+518D
0, //U+518E
0, //U+518F
0, //U+5190
0, //U+5191
0, //U+5192
0, //U+5193
0, //U+5194
0, //U+5195
0, //U+5196
0, //U+5197
0, //U+5198
0, //U+5199
0, //U+519A
0, //U+519B
0, //U+519C
0, //U+519D
0, //U+519E
0, //U+519F
0, //U+51A0
0, //U+51A1
0, //U+51A2
0, //U+51A3
0, //U+51A4
0, //U+51A5
0, //U+51A6
0, //U+51A7
0, //U+51A8
0, //U+51A9
0, //U+51AA
0, //U+51AB
0, //U+51AC
0, //U+51AD
0, //U+51AE
0, //U+51AF
0, //U+51B0
0, //U+51B1
0, //U+51B2
0, //U+51B3
0, //U+51B4
0, //U+51B5
0, //U+51B6
0, //U+51B7
0, //U+51B8
0, //U+51B9
0, //U+51BA
0, //U+51BB
0, //U+51BC
0, //U+51BD
0, //U+51BE
0, //U+51BF
0, //U+51C0
0, //U+51C1
0, //U+51C2
0, //U+51C3
0, //U+51C4
0, //U+51C5
0, //U+51C6
0, //U+51C7
0, //U+51C8
0, //U+51C9
0, //U+51CA
0, //U+51CB
0, //U+51CC
0, //U+51CD
0, //U+51CE
0, //U+51CF
0, //U+51D0
0, //U+51D1
0, //U+51D2
0, //U+51D3
0, //U+51D4
0, //U+51D5
0, //U+51D6
0, //U+51D7
0, //U+51D8
0, //U+51D9
0, //U+51DA
0, //U+51DB
0, //U+51DC
0, //U+51DD
0, //U+51DE
0, //U+51DF
0, //U+51E0
0, //U+51E1
0, //U+51E2
0, //U+51E3
0, //U+51E4
0, //U+51E5
0, //U+51E6
0, //U+51E7
0, //U+51E8
0, //U+51E9
0, //U+51EA
0, //U+51EB
0, //U+51EC
0, //U+51ED
0, //U+51EE
0, //U+51EF
0, //U+51F0
0, //U+51F1
0, //U+51F2
0, //U+51F3
0, //U+51F4
0, //U+51F5
0, //U+51F6
0, //U+51F7
0, //U+51F8
0, //U+51F9
0, //U+51FA
0, //U+51FB
0, //U+51FC
0, //U+51FD
0, //U+51FE
0, //U+51FF
0, //U+5200
0, //U+5201
0, //U+5202
0, //U+5203
0, //U+5204
0, //U+5205
0, //U+5206
0, //U+5207
0, //U+5208
0, //U+5209
0, //U+520A
0, //U+520B
0, //U+520C
0, //U+520D
0, //U+520E
0, //U+520F
0, //U+5210
0, //U+5211
0, //U+5212
0, //U+5213
0, //U+5214
0, //U+5215
0, //U+5216
0, //U+5217
0, //U+5218
0, //U+5219
0, //U+521A
0, //U+521B
0, //U+521C
0, //U+521D
0, //U+521E
0, //U+521F
0, //U+5220
0, //U+5221
0, //U+5222
0, //U+5223
0, //U+5224
0, //U+5225
0, //U+5226
0, //U+5227
0, //U+5228
0, //U+5229
0, //U+522A
0, //U+522B
0, //U+522C
0, //U+522D
0, //U+522E
0, //U+522F
0, //U+5230
0, //U+5231
0, //U+5232
0, //U+5233
0, //U+5234
0, //U+5235
0, //U+5236
0, //U+5237
0, //U+5238
0, //U+5239
0, //U+523A
0, //U+523B
0, //U+523C
0, //U+523D
0, //U+523E
0, //U+523F
0, //U+5240
0, //U+5241
0, //U+5242
0, //U+5243
0, //U+5244
0, //U+5245
0, //U+5246
0, //U+5247
0, //U+5248
0, //U+5249
0, //U+524A
0, //U+524B
0, //U+524C
0, //U+524D
0, //U+524E
0, //U+524F
0, //U+5250
0, //U+5251
0, //U+5252
0, //U+5253
0, //U+5254
0, //U+5255
0, //U+5256
0, //U+5257
0, //U+5258
0, //U+5259
0, //U+525A
0, //U+525B
0, //U+525C
0, //U+525D
0, //U+525E
0, //U+525F
0, //U+5260
0, //U+5261
0, //U+5262
0, //U+5263
0, //U+5264
0, //U+5265
0, //U+5266
0, //U+5267
0, //U+5268
0, //U+5269
0, //U+526A
0, //U+526B
0, //U+526C
0, //U+526D
0, //U+526E
0, //U+526F
0, //U+5270
0, //U+5271
0, //U+5272
0, //U+5273
0, //U+5274
0, //U+5275
0, //U+5276
0, //U+5277
0, //U+5278
0, //U+5279
0, //U+527A
0, //U+527B
0, //U+527C
0, //U+527D
0, //U+527E
0, //U+527F
0, //U+5280
0, //U+5281
0, //U+5282
0, //U+5283
0, //U+5284
0, //U+5285
0, //U+5286
0, //U+5287
0, //U+5288
0, //U+5289
0, //U+528A
0, //U+528B
0, //U+528C
0, //U+528D
0, //U+528E
0, //U+528F
0, //U+5290
0, //U+5291
0, //U+5292
0, //U+5293
0, //U+5294
0, //U+5295
0, //U+5296
0, //U+5297
0, //U+5298
0, //U+5299
0, //U+529A
0, //U+529B
0, //U+529C
0, //U+529D
0, //U+529E
0, //U+529F
0, //U+52A0
0, //U+52A1
0, //U+52A2
0, //U+52A3
0, //U+52A4
0, //U+52A5
0, //U+52A6
0, //U+52A7
0, //U+52A8
0, //U+52A9
0, //U+52AA
0, //U+52AB
0, //U+52AC
0, //U+52AD
0, //U+52AE
0, //U+52AF
0, //U+52B0
0, //U+52B1
0, //U+52B2
0, //U+52B3
0, //U+52B4
0, //U+52B5
0, //U+52B6
0, //U+52B7
0, //U+52B8
0, //U+52B9
0, //U+52BA
0, //U+52BB
0, //U+52BC
0, //U+52BD
0, //U+52BE
0, //U+52BF
0, //U+52C0
0, //U+52C1
0, //U+52C2
0, //U+52C3
0, //U+52C4
0, //U+52C5
0, //U+52C6
0, //U+52C7
0, //U+52C8
0, //U+52C9
0, //U+52CA
0, //U+52CB
0, //U+52CC
0, //U+52CD
0, //U+52CE
0, //U+52CF
0, //U+52D0
0, //U+52D1
0, //U+52D2
0, //U+52D3
0, //U+52D4
0, //U+52D5
0, //U+52D6
0, //U+52D7
0, //U+52D8
0, //U+52D9
0, //U+52DA
0, //U+52DB
0, //U+52DC
0, //U+52DD
0, //U+52DE
0, //U+52DF
0, //U+52E0
0, //U+52E1
0, //U+52E2
0, //U+52E3
0, //U+52E4
0, //U+52E5
0, //U+52E6
0, //U+52E7
0, //U+52E8
0, //U+52E9
0, //U+52EA
0, //U+52EB
0, //U+52EC
0, //U+52ED
0, //U+52EE
0, //U+52EF
0, //U+52F0
0, //U+52F1
0, //U+52F2
0, //U+52F3
0, //U+52F4
0, //U+52F5
0, //U+52F6
0, //U+52F7
0, //U+52F8
0, //U+52F9
0, //U+52FA
0, //U+52FB
0, //U+52FC
0, //U+52FD
0, //U+52FE
0, //U+52FF
0, //U+5300
0, //U+5301
0, //U+5302
0, //U+5303
0, //U+5304
0, //U+5305
0, //U+5306
0, //U+5307
0, //U+5308
0, //U+5309
0, //U+530A
0, //U+530B
0, //U+530C
0, //U+530D
0, //U+530E
0, //U+530F
0, //U+5310
0, //U+5311
0, //U+5312
0, //U+5313
0, //U+5314
0, //U+5315
0, //U+5316
0, //U+5317
0, //U+5318
0, //U+5319
0, //U+531A
0, //U+531B
0, //U+531C
0, //U+531D
0, //U+531E
0, //U+531F
0, //U+5320
0, //U+5321
0, //U+5322
0, //U+5323
0, //U+5324
0, //U+5325
0, //U+5326
0, //U+5327
0, //U+5328
0, //U+5329
0, //U+532A
0, //U+532B
0, //U+532C
0, //U+532D
0, //U+532E
0, //U+532F
0, //U+5330
0, //U+5331
0, //U+5332
0, //U+5333
0, //U+5334
0, //U+5335
0, //U+5336
0, //U+5337
0, //U+5338
0, //U+5339
0, //U+533A
0, //U+533B
0, //U+533C
0, //U+533D
0, //U+533E
0, //U+533F
0, //U+5340
0, //U+5341
0, //U+5342
0, //U+5343
0, //U+5344
0, //U+5345
0, //U+5346
0, //U+5347
0, //U+5348
0, //U+5349
0, //U+534A
0, //U+534B
0, //U+534C
0, //U+534D
0, //U+534E
0, //U+534F
0, //U+5350
0, //U+5351
0, //U+5352
0, //U+5353
0, //U+5354
0, //U+5355
0, //U+5356
0, //U+5357
0, //U+5358
0, //U+5359
0, //U+535A
0, //U+535B
0, //U+535C
0, //U+535D
0, //U+535E
0, //U+535F
0, //U+5360
0, //U+5361
0, //U+5362
0, //U+5363
0, //U+5364
0, //U+5365
0, //U+5366
0, //U+5367
0, //U+5368
0, //U+5369
0, //U+536A
0, //U+536B
0, //U+536C
0, //U+536D
0, //U+536E
0, //U+536F
0, //U+5370
0, //U+5371
0, //U+5372
0, //U+5373
0, //U+5374
0, //U+5375
0, //U+5376
0, //U+5377
0, //U+5378
0, //U+5379
0, //U+537A
0, //U+537B
0, //U+537C
0, //U+537D
0, //U+537E
0, //U+537F
0, //U+5380
0, //U+5381
0, //U+5382
0, //U+5383
0, //U+5384
0, //U+5385
0, //U+5386
0, //U+5387
0, //U+5388
0, //U+5389
0, //U+538A
0, //U+538B
0, //U+538C
0, //U+538D
0, //U+538E
0, //U+538F
0, //U+5390
0, //U+5391
0, //U+5392
0, //U+5393
0, //U+5394
0, //U+5395
0, //U+5396
0, //U+5397
0, //U+5398
0, //U+5399
0, //U+539A
0, //U+539B
0, //U+539C
0, //U+539D
0, //U+539E
0, //U+539F
0, //U+53A0
0, //U+53A1
0, //U+53A2
0, //U+53A3
0, //U+53A4
0, //U+53A5
0, //U+53A6
0, //U+53A7
0, //U+53A8
0, //U+53A9
0, //U+53AA
0, //U+53AB
0, //U+53AC
0, //U+53AD
0, //U+53AE
0, //U+53AF
0, //U+53B0
0, //U+53B1
0, //U+53B2
0, //U+53B3
0, //U+53B4
0, //U+53B5
0, //U+53B6
0, //U+53B7
0, //U+53B8
0, //U+53B9
0, //U+53BA
0, //U+53BB
0, //U+53BC
0, //U+53BD
0, //U+53BE
0, //U+53BF
0, //U+53C0
0, //U+53C1
0, //U+53C2
0, //U+53C3
0, //U+53C4
0, //U+53C5
0, //U+53C6
0, //U+53C7
0, //U+53C8
0, //U+53C9
0, //U+53CA
0, //U+53CB
0, //U+53CC
0, //U+53CD
0, //U+53CE
0, //U+53CF
0, //U+53D0
0, //U+53D1
0, //U+53D2
0, //U+53D3
0, //U+53D4
0, //U+53D5
0, //U+53D6
0, //U+53D7
0, //U+53D8
0, //U+53D9
0, //U+53DA
0, //U+53DB
0, //U+53DC
0, //U+53DD
0, //U+53DE
0, //U+53DF
0, //U+53E0
0, //U+53E1
0, //U+53E2
0, //U+53E3
0, //U+53E4
0, //U+53E5
0, //U+53E6
0, //U+53E7
0, //U+53E8
0, //U+53E9
0, //U+53EA
0, //U+53EB
0, //U+53EC
0, //U+53ED
0, //U+53EE
0, //U+53EF
0, //U+53F0
0, //U+53F1
0, //U+53F2
0, //U+53F3
0, //U+53F4
0, //U+53F5
0, //U+53F6
0, //U+53F7
0, //U+53F8
0, //U+53F9
0, //U+53FA
0, //U+53FB
0, //U+53FC
0, //U+53FD
0, //U+53FE
0, //U+53FF
0, //U+5400
0, //U+5401
0, //U+5402
0, //U+5403
0, //U+5404
0, //U+5405
0, //U+5406
0, //U+5407
0, //U+5408
0, //U+5409
0, //U+540A
0, //U+540B
0, //U+540C
0, //U+540D
0, //U+540E
0, //U+540F
0, //U+5410
0, //U+5411
0, //U+5412
0, //U+5413
0, //U+5414
0, //U+5415
0, //U+5416
0, //U+5417
0, //U+5418
0, //U+5419
0, //U+541A
0, //U+541B
0, //U+541C
0, //U+541D
0, //U+541E
0, //U+541F
0, //U+5420
0, //U+5421
0, //U+5422
0, //U+5423
0, //U+5424
0, //U+5425
0, //U+5426
0, //U+5427
0, //U+5428
0, //U+5429
0, //U+542A
0, //U+542B
0, //U+542C
0, //U+542D
0, //U+542E
0, //U+542F
0, //U+5430
0, //U+5431
0, //U+5432
0, //U+5433
0, //U+5434
0, //U+5435
0, //U+5436
0, //U+5437
0, //U+5438
0, //U+5439
0, //U+543A
0, //U+543B
0, //U+543C
0, //U+543D
0, //U+543E
0, //U+543F
0, //U+5440
0, //U+5441
0, //U+5442
0, //U+5443
0, //U+5444
0, //U+5445
0, //U+5446
0, //U+5447
0, //U+5448
0, //U+5449
0, //U+544A
0, //U+544B
0, //U+544C
0, //U+544D
0, //U+544E
0, //U+544F
0, //U+5450
0, //U+5451
0, //U+5452
0, //U+5453
0, //U+5454
0, //U+5455
0, //U+5456
0, //U+5457
0, //U+5458
0, //U+5459
0, //U+545A
0, //U+545B
0, //U+545C
0, //U+545D
0, //U+545E
0, //U+545F
0, //U+5460
0, //U+5461
0, //U+5462
0, //U+5463
0, //U+5464
0, //U+5465
0, //U+5466
0, //U+5467
0, //U+5468
0, //U+5469
0, //U+546A
0, //U+546B
0, //U+546C
0, //U+546D
0, //U+546E
0, //U+546F
0, //U+5470
0, //U+5471
0, //U+5472
0, //U+5473
0, //U+5474
0, //U+5475
0, //U+5476
0, //U+5477
0, //U+5478
0, //U+5479
0, //U+547A
0, //U+547B
0, //U+547C
0, //U+547D
0, //U+547E
0, //U+547F
0, //U+5480
0, //U+5481
0, //U+5482
0, //U+5483
0, //U+5484
0, //U+5485
0, //U+5486
0, //U+5487
0, //U+5488
0, //U+5489
0, //U+548A
0, //U+548B
0, //U+548C
0, //U+548D
0, //U+548E
0, //U+548F
0, //U+5490
0, //U+5491
0, //U+5492
0, //U+5493
0, //U+5494
0, //U+5495
0, //U+5496
0, //U+5497
0, //U+5498
0, //U+5499
0, //U+549A
0, //U+549B
0, //U+549C
0, //U+549D
0, //U+549E
0, //U+549F
0, //U+54A0
0, //U+54A1
0, //U+54A2
0, //U+54A3
0, //U+54A4
0, //U+54A5
0, //U+54A6
0, //U+54A7
0, //U+54A8
0, //U+54A9
0, //U+54AA
0, //U+54AB
0, //U+54AC
0, //U+54AD
0, //U+54AE
0, //U+54AF
0, //U+54B0
0, //U+54B1
0, //U+54B2
0, //U+54B3
0, //U+54B4
0, //U+54B5
0, //U+54B6
0, //U+54B7
0, //U+54B8
0, //U+54B9
0, //U+54BA
0, //U+54BB
0, //U+54BC
0, //U+54BD
0, //U+54BE
0, //U+54BF
0, //U+54C0
0, //U+54C1
0, //U+54C2
0, //U+54C3
0, //U+54C4
0, //U+54C5
0, //U+54C6
0, //U+54C7
0, //U+54C8
0, //U+54C9
0, //U+54CA
0, //U+54CB
0, //U+54CC
0, //U+54CD
0, //U+54CE
0, //U+54CF
0, //U+54D0
0, //U+54D1
0, //U+54D2
0, //U+54D3
0, //U+54D4
0, //U+54D5
0, //U+54D6
0, //U+54D7
0, //U+54D8
0, //U+54D9
0, //U+54DA
0, //U+54DB
0, //U+54DC
0, //U+54DD
0, //U+54DE
0, //U+54DF
0, //U+54E0
0, //U+54E1
0, //U+54E2
0, //U+54E3
0, //U+54E4
0, //U+54E5
0, //U+54E6
0, //U+54E7
0, //U+54E8
0, //U+54E9
0, //U+54EA
0, //U+54EB
0, //U+54EC
0, //U+54ED
0, //U+54EE
0, //U+54EF
0, //U+54F0
0, //U+54F1
0, //U+54F2
0, //U+54F3
0, //U+54F4
0, //U+54F5
0, //U+54F6
0, //U+54F7
0, //U+54F8
0, //U+54F9
0, //U+54FA
0, //U+54FB
0, //U+54FC
0, //U+54FD
0, //U+54FE
0, //U+54FF
0, //U+5500
0, //U+5501
0, //U+5502
0, //U+5503
0, //U+5504
0, //U+5505
0, //U+5506
0, //U+5507
0, //U+5508
0, //U+5509
0, //U+550A
0, //U+550B
0, //U+550C
0, //U+550D
0, //U+550E
0, //U+550F
0, //U+5510
0, //U+5511
0, //U+5512
0, //U+5513
0, //U+5514
0, //U+5515
0, //U+5516
0, //U+5517
0, //U+5518
0, //U+5519
0, //U+551A
0, //U+551B
0, //U+551C
0, //U+551D
0, //U+551E
0, //U+551F
0, //U+5520
0, //U+5521
0, //U+5522
0, //U+5523
0, //U+5524
0, //U+5525
0, //U+5526
0, //U+5527
0, //U+5528
0, //U+5529
0, //U+552A
0, //U+552B
0, //U+552C
0, //U+552D
0, //U+552E
0, //U+552F
0, //U+5530
0, //U+5531
0, //U+5532
0, //U+5533
0, //U+5534
0, //U+5535
0, //U+5536
0, //U+5537
0, //U+5538
0, //U+5539
0, //U+553A
0, //U+553B
0, //U+553C
0, //U+553D
0, //U+553E
0, //U+553F
0, //U+5540
0, //U+5541
0, //U+5542
0, //U+5543
0, //U+5544
0, //U+5545
0, //U+5546
0, //U+5547
0, //U+5548
0, //U+5549
0, //U+554A
0, //U+554B
0, //U+554C
0, //U+554D
0, //U+554E
0, //U+554F
0, //U+5550
0, //U+5551
0, //U+5552
0, //U+5553
0, //U+5554
0, //U+5555
0, //U+5556
0, //U+5557
0, //U+5558
0, //U+5559
0, //U+555A
0, //U+555B
0, //U+555C
0, //U+555D
0, //U+555E
0, //U+555F
0, //U+5560
0, //U+5561
0, //U+5562
0, //U+5563
0, //U+5564
0, //U+5565
0, //U+5566
0, //U+5567
0, //U+5568
0, //U+5569
0, //U+556A
0, //U+556B
0, //U+556C
0, //U+556D
0, //U+556E
0, //U+556F
0, //U+5570
0, //U+5571
0, //U+5572
0, //U+5573
0, //U+5574
0, //U+5575
0, //U+5576
0, //U+5577
0, //U+5578
0, //U+5579
0, //U+557A
0, //U+557B
0, //U+557C
0, //U+557D
0, //U+557E
0, //U+557F
0, //U+5580
0, //U+5581
0, //U+5582
0, //U+5583
0, //U+5584
0, //U+5585
0, //U+5586
0, //U+5587
0, //U+5588
0, //U+5589
0, //U+558A
0, //U+558B
0, //U+558C
0, //U+558D
0, //U+558E
0, //U+558F
0, //U+5590
0, //U+5591
0, //U+5592
0, //U+5593
0, //U+5594
0, //U+5595
0, //U+5596
0, //U+5597
0, //U+5598
0, //U+5599
0, //U+559A
0, //U+559B
0, //U+559C
0, //U+559D
0, //U+559E
0, //U+559F
0, //U+55A0
0, //U+55A1
0, //U+55A2
0, //U+55A3
0, //U+55A4
0, //U+55A5
0, //U+55A6
0, //U+55A7
0, //U+55A8
0, //U+55A9
0, //U+55AA
0, //U+55AB
0, //U+55AC
0, //U+55AD
0, //U+55AE
0, //U+55AF
0, //U+55B0
0, //U+55B1
0, //U+55B2
0, //U+55B3
0, //U+55B4
0, //U+55B5
0, //U+55B6
0, //U+55B7
0, //U+55B8
0, //U+55B9
0, //U+55BA
0, //U+55BB
0, //U+55BC
0, //U+55BD
0, //U+55BE
0, //U+55BF
0, //U+55C0
0, //U+55C1
0, //U+55C2
0, //U+55C3
0, //U+55C4
0, //U+55C5
0, //U+55C6
0, //U+55C7
0, //U+55C8
0, //U+55C9
0, //U+55CA
0, //U+55CB
0, //U+55CC
0, //U+55CD
0, //U+55CE
0, //U+55CF
0, //U+55D0
0, //U+55D1
0, //U+55D2
0, //U+55D3
0, //U+55D4
0, //U+55D5
0, //U+55D6
0, //U+55D7
0, //U+55D8
0, //U+55D9
0, //U+55DA
0, //U+55DB
0, //U+55DC
0, //U+55DD
0, //U+55DE
0, //U+55DF
0, //U+55E0
0, //U+55E1
0, //U+55E2
0, //U+55E3
0, //U+55E4
0, //U+55E5
0, //U+55E6
0, //U+55E7
0, //U+55E8
0, //U+55E9
0, //U+55EA
0, //U+55EB
0, //U+55EC
0, //U+55ED
0, //U+55EE
0, //U+55EF
0, //U+55F0
0, //U+55F1
0, //U+55F2
0, //U+55F3
0, //U+55F4
0, //U+55F5
0, //U+55F6
0, //U+55F7
0, //U+55F8
0, //U+55F9
0, //U+55FA
0, //U+55FB
0, //U+55FC
0, //U+55FD
0, //U+55FE
0, //U+55FF
0, //U+5600
0, //U+5601
0, //U+5602
0, //U+5603
0, //U+5604
0, //U+5605
0, //U+5606
0, //U+5607
0, //U+5608
0, //U+5609
0, //U+560A
0, //U+560B
0, //U+560C
0, //U+560D
0, //U+560E
0, //U+560F
0, //U+5610
0, //U+5611
0, //U+5612
0, //U+5613
0, //U+5614
0, //U+5615
0, //U+5616
0, //U+5617
0, //U+5618
0, //U+5619
0, //U+561A
0, //U+561B
0, //U+561C
0, //U+561D
0, //U+561E
0, //U+561F
0, //U+5620
0, //U+5621
0, //U+5622
0, //U+5623
0, //U+5624
0, //U+5625
0, //U+5626
0, //U+5627
0, //U+5628
0, //U+5629
0, //U+562A
0, //U+562B
0, //U+562C
0, //U+562D
0, //U+562E
0, //U+562F
0, //U+5630
0, //U+5631
0, //U+5632
0, //U+5633
0, //U+5634
0, //U+5635
0, //U+5636
0, //U+5637
0, //U+5638
0, //U+5639
0, //U+563A
0, //U+563B
0, //U+563C
0, //U+563D
0, //U+563E
0, //U+563F
0, //U+5640
0, //U+5641
0, //U+5642
0, //U+5643
0, //U+5644
0, //U+5645
0, //U+5646
0, //U+5647
0, //U+5648
0, //U+5649
0, //U+564A
0, //U+564B
0, //U+564C
0, //U+564D
0, //U+564E
0, //U+564F
0, //U+5650
0, //U+5651
0, //U+5652
0, //U+5653
0, //U+5654
0, //U+5655
0, //U+5656
0, //U+5657
0, //U+5658
0, //U+5659
0, //U+565A
0, //U+565B
0, //U+565C
0, //U+565D
0, //U+565E
0, //U+565F
0, //U+5660
0, //U+5661
0, //U+5662
0, //U+5663
0, //U+5664
0, //U+5665
0, //U+5666
0, //U+5667
0, //U+5668
0, //U+5669
0, //U+566A
0, //U+566B
0, //U+566C
0, //U+566D
0, //U+566E
0, //U+566F
0, //U+5670
0, //U+5671
0, //U+5672
0, //U+5673
0, //U+5674
0, //U+5675
0, //U+5676
0, //U+5677
0, //U+5678
0, //U+5679
0, //U+567A
0, //U+567B
0, //U+567C
0, //U+567D
0, //U+567E
0, //U+567F
0, //U+5680
0, //U+5681
0, //U+5682
0, //U+5683
0, //U+5684
0, //U+5685
0, //U+5686
0, //U+5687
0, //U+5688
0, //U+5689
0, //U+568A
0, //U+568B
0, //U+568C
0, //U+568D
0, //U+568E
0, //U+568F
0, //U+5690
0, //U+5691
0, //U+5692
0, //U+5693
0, //U+5694
0, //U+5695
0, //U+5696
0, //U+5697
0, //U+5698
0, //U+5699
0, //U+569A
0, //U+569B
0, //U+569C
0, //U+569D
0, //U+569E
0, //U+569F
0, //U+56A0
0, //U+56A1
0, //U+56A2
0, //U+56A3
0, //U+56A4
0, //U+56A5
0, //U+56A6
0, //U+56A7
0, //U+56A8
0, //U+56A9
0, //U+56AA
0, //U+56AB
0, //U+56AC
0, //U+56AD
0, //U+56AE
0, //U+56AF
0, //U+56B0
0, //U+56B1
0, //U+56B2
0, //U+56B3
0, //U+56B4
0, //U+56B5
0, //U+56B6
0, //U+56B7
0, //U+56B8
0, //U+56B9
0, //U+56BA
0, //U+56BB
0, //U+56BC
0, //U+56BD
0, //U+56BE
0, //U+56BF
0, //U+56C0
0, //U+56C1
0, //U+56C2
0, //U+56C3
0, //U+56C4
0, //U+56C5
0, //U+56C6
0, //U+56C7
0, //U+56C8
0, //U+56C9
0, //U+56CA
0, //U+56CB
0, //U+56CC
0, //U+56CD
0, //U+56CE
0, //U+56CF
0, //U+56D0
0, //U+56D1
0, //U+56D2
0, //U+56D3
0, //U+56D4
0, //U+56D5
0, //U+56D6
0, //U+56D7
0, //U+56D8
0, //U+56D9
0, //U+56DA
0, //U+56DB
0, //U+56DC
0, //U+56DD
0, //U+56DE
0, //U+56DF
0, //U+56E0
0, //U+56E1
0, //U+56E2
0, //U+56E3
0, //U+56E4
0, //U+56E5
0, //U+56E6
0, //U+56E7
0, //U+56E8
0, //U+56E9
0, //U+56EA
0, //U+56EB
0, //U+56EC
0, //U+56ED
0, //U+56EE
0, //U+56EF
0, //U+56F0
0, //U+56F1
0, //U+56F2
0, //U+56F3
0, //U+56F4
0, //U+56F5
0, //U+56F6
0, //U+56F7
0, //U+56F8
0, //U+56F9
0, //U+56FA
0, //U+56FB
0, //U+56FC
0, //U+56FD
0, //U+56FE
0, //U+56FF
0, //U+5700
0, //U+5701
0, //U+5702
0, //U+5703
0, //U+5704
0, //U+5705
0, //U+5706
0, //U+5707
0, //U+5708
0, //U+5709
0, //U+570A
0, //U+570B
0, //U+570C
0, //U+570D
0, //U+570E
0, //U+570F
0, //U+5710
0, //U+5711
0, //U+5712
0, //U+5713
0, //U+5714
0, //U+5715
0, //U+5716
0, //U+5717
0, //U+5718
0, //U+5719
0, //U+571A
0, //U+571B
0, //U+571C
0, //U+571D
0, //U+571E
0, //U+571F
0, //U+5720
0, //U+5721
0, //U+5722
0, //U+5723
0, //U+5724
0, //U+5725
0, //U+5726
0, //U+5727
0, //U+5728
0, //U+5729
0, //U+572A
0, //U+572B
0, //U+572C
0, //U+572D
0, //U+572E
0, //U+572F
0, //U+5730
0, //U+5731
0, //U+5732
0, //U+5733
0, //U+5734
0, //U+5735
0, //U+5736
0, //U+5737
0, //U+5738
0, //U+5739
0, //U+573A
0, //U+573B
0, //U+573C
0, //U+573D
0, //U+573E
0, //U+573F
0, //U+5740
0, //U+5741
0, //U+5742
0, //U+5743
0, //U+5744
0, //U+5745
0, //U+5746
0, //U+5747
0, //U+5748
0, //U+5749
0, //U+574A
0, //U+574B
0, //U+574C
0, //U+574D
0, //U+574E
0, //U+574F
0, //U+5750
0, //U+5751
0, //U+5752
0, //U+5753
0, //U+5754
0, //U+5755
0, //U+5756
0, //U+5757
0, //U+5758
0, //U+5759
0, //U+575A
0, //U+575B
0, //U+575C
0, //U+575D
0, //U+575E
0, //U+575F
0, //U+5760
0, //U+5761
0, //U+5762
0, //U+5763
0, //U+5764
0, //U+5765
0, //U+5766
0, //U+5767
0, //U+5768
0, //U+5769
0, //U+576A
0, //U+576B
0, //U+576C
0, //U+576D
0, //U+576E
0, //U+576F
0, //U+5770
0, //U+5771
0, //U+5772
0, //U+5773
0, //U+5774
0, //U+5775
0, //U+5776
0, //U+5777
0, //U+5778
0, //U+5779
0, //U+577A
0, //U+577B
0, //U+577C
0, //U+577D
0, //U+577E
0, //U+577F
0, //U+5780
0, //U+5781
0, //U+5782
0, //U+5783
0, //U+5784
0, //U+5785
0, //U+5786
0, //U+5787
0, //U+5788
0, //U+5789
0, //U+578A
0, //U+578B
0, //U+578C
0, //U+578D
0, //U+578E
0, //U+578F
0, //U+5790
0, //U+5791
0, //U+5792
0, //U+5793
0, //U+5794
0, //U+5795
0, //U+5796
0, //U+5797
0, //U+5798
0, //U+5799
0, //U+579A
0, //U+579B
0, //U+579C
0, //U+579D
0, //U+579E
0, //U+579F
0, //U+57A0
0, //U+57A1
0, //U+57A2
0, //U+57A3
0, //U+57A4
0, //U+57A5
0, //U+57A6
0, //U+57A7
0, //U+57A8
0, //U+57A9
0, //U+57AA
0, //U+57AB
0, //U+57AC
0, //U+57AD
0, //U+57AE
0, //U+57AF
0, //U+57B0
0, //U+57B1
0, //U+57B2
0, //U+57B3
0, //U+57B4
0, //U+57B5
0, //U+57B6
0, //U+57B7
0, //U+57B8
0, //U+57B9
0, //U+57BA
0, //U+57BB
0, //U+57BC
0, //U+57BD
0, //U+57BE
0, //U+57BF
0, //U+57C0
0, //U+57C1
0, //U+57C2
0, //U+57C3
0, //U+57C4
0, //U+57C5
0, //U+57C6
0, //U+57C7
0, //U+57C8
0, //U+57C9
0, //U+57CA
0, //U+57CB
0, //U+57CC
0, //U+57CD
0, //U+57CE
0, //U+57CF
0, //U+57D0
0, //U+57D1
0, //U+57D2
0, //U+57D3
0, //U+57D4
0, //U+57D5
0, //U+57D6
0, //U+57D7
0, //U+57D8
0, //U+57D9
0, //U+57DA
0, //U+57DB
0, //U+57DC
0, //U+57DD
0, //U+57DE
0, //U+57DF
0, //U+57E0
0, //U+57E1
0, //U+57E2
0, //U+57E3
0, //U+57E4
0, //U+57E5
0, //U+57E6
0, //U+57E7
0, //U+57E8
0, //U+57E9
0, //U+57EA
0, //U+57EB
0, //U+57EC
0, //U+57ED
0, //U+57EE
0, //U+57EF
0, //U+57F0
0, //U+57F1
0, //U+57F2
0, //U+57F3
0, //U+57F4
0, //U+57F5
0, //U+57F6
0, //U+57F7
0, //U+57F8
0, //U+57F9
0, //U+57FA
0, //U+57FB
0, //U+57FC
0, //U+57FD
0, //U+57FE
0, //U+57FF
0, //U+5800
0, //U+5801
0, //U+5802
0, //U+5803
0, //U+5804
0, //U+5805
0, //U+5806
0, //U+5807
0, //U+5808
0, //U+5809
0, //U+580A
0, //U+580B
0, //U+580C
0, //U+580D
0, //U+580E
0, //U+580F
0, //U+5810
0, //U+5811
0, //U+5812
0, //U+5813
0, //U+5814
0, //U+5815
0, //U+5816
0, //U+5817
0, //U+5818
0, //U+5819
0, //U+581A
0, //U+581B
0, //U+581C
0, //U+581D
0, //U+581E
0, //U+581F
0, //U+5820
0, //U+5821
0, //U+5822
0, //U+5823
0, //U+5824
0, //U+5825
0, //U+5826
0, //U+5827
0, //U+5828
0, //U+5829
0, //U+582A
0, //U+582B
0, //U+582C
0, //U+582D
0, //U+582E
0, //U+582F
0, //U+5830
0, //U+5831
0, //U+5832
0, //U+5833
0, //U+5834
0, //U+5835
0, //U+5836
0, //U+5837
0, //U+5838
0, //U+5839
0, //U+583A
0, //U+583B
0, //U+583C
0, //U+583D
0, //U+583E
0, //U+583F
0, //U+5840
0, //U+5841
0, //U+5842
0, //U+5843
0, //U+5844
0, //U+5845
0, //U+5846
0, //U+5847
0, //U+5848
0, //U+5849
0, //U+584A
0, //U+584B
0, //U+584C
0, //U+584D
0, //U+584E
0, //U+584F
0, //U+5850
0, //U+5851
0, //U+5852
0, //U+5853
0, //U+5854
0, //U+5855
0, //U+5856
0, //U+5857
0, //U+5858
0, //U+5859
0, //U+585A
0, //U+585B
0, //U+585C
0, //U+585D
0, //U+585E
0, //U+585F
0, //U+5860
0, //U+5861
0, //U+5862
0, //U+5863
0, //U+5864
0, //U+5865
0, //U+5866
0, //U+5867
0, //U+5868
0, //U+5869
0, //U+586A
0, //U+586B
0, //U+586C
0, //U+586D
0, //U+586E
0, //U+586F
0, //U+5870
0, //U+5871
0, //U+5872
0, //U+5873
0, //U+5874
0, //U+5875
0, //U+5876
0, //U+5877
0, //U+5878
0, //U+5879
0, //U+587A
0, //U+587B
0, //U+587C
0, //U+587D
0, //U+587E
0, //U+587F
0, //U+5880
0, //U+5881
0, //U+5882
0, //U+5883
0, //U+5884
0, //U+5885
0, //U+5886
0, //U+5887
0, //U+5888
0, //U+5889
0, //U+588A
0, //U+588B
0, //U+588C
0, //U+588D
0, //U+588E
0, //U+588F
0, //U+5890
0, //U+5891
0, //U+5892
0, //U+5893
0, //U+5894
0, //U+5895
0, //U+5896
0, //U+5897
0, //U+5898
0, //U+5899
0, //U+589A
0, //U+589B
0, //U+589C
0, //U+589D
0, //U+589E
0, //U+589F
0, //U+58A0
0, //U+58A1
0, //U+58A2
0, //U+58A3
0, //U+58A4
0, //U+58A5
0, //U+58A6
0, //U+58A7
0, //U+58A8
0, //U+58A9
0, //U+58AA
0, //U+58AB
0, //U+58AC
0, //U+58AD
0, //U+58AE
0, //U+58AF
0, //U+58B0
0, //U+58B1
0, //U+58B2
0, //U+58B3
0, //U+58B4
0, //U+58B5
0, //U+58B6
0, //U+58B7
0, //U+58B8
0, //U+58B9
0, //U+58BA
0, //U+58BB
0, //U+58BC
0, //U+58BD
0, //U+58BE
0, //U+58BF
0, //U+58C0
0, //U+58C1
0, //U+58C2
0, //U+58C3
0, //U+58C4
0, //U+58C5
0, //U+58C6
0, //U+58C7
0, //U+58C8
0, //U+58C9
0, //U+58CA
0, //U+58CB
0, //U+58CC
0, //U+58CD
0, //U+58CE
0, //U+58CF
0, //U+58D0
0, //U+58D1
0, //U+58D2
0, //U+58D3
0, //U+58D4
0, //U+58D5
0, //U+58D6
0, //U+58D7
0, //U+58D8
0, //U+58D9
0, //U+58DA
0, //U+58DB
0, //U+58DC
0, //U+58DD
0, //U+58DE
0, //U+58DF
0, //U+58E0
0, //U+58E1
0, //U+58E2
0, //U+58E3
0, //U+58E4
0, //U+58E5
0, //U+58E6
0, //U+58E7
0, //U+58E8
0, //U+58E9
0, //U+58EA
0, //U+58EB
0, //U+58EC
0, //U+58ED
0, //U+58EE
0, //U+58EF
0, //U+58F0
0, //U+58F1
0, //U+58F2
0, //U+58F3
0, //U+58F4
0, //U+58F5
0, //U+58F6
0, //U+58F7
0, //U+58F8
0, //U+58F9
0, //U+58FA
0, //U+58FB
0, //U+58FC
0, //U+58FD
0, //U+58FE
0, //U+58FF
0, //U+5900
0, //U+5901
0, //U+5902
0, //U+5903
0, //U+5904
0, //U+5905
0, //U+5906
0, //U+5907
0, //U+5908
0, //U+5909
0, //U+590A
0, //U+590B
0, //U+590C
0, //U+590D
0, //U+590E
0, //U+590F
0, //U+5910
0, //U+5911
0, //U+5912
0, //U+5913
0, //U+5914
0, //U+5915
0, //U+5916
0, //U+5917
0, //U+5918
0, //U+5919
0, //U+591A
0, //U+591B
0, //U+591C
0, //U+591D
0, //U+591E
0, //U+591F
0, //U+5920
0, //U+5921
0, //U+5922
0, //U+5923
0, //U+5924
0, //U+5925
0, //U+5926
0, //U+5927
0, //U+5928
0, //U+5929
0, //U+592A
0, //U+592B
0, //U+592C
0, //U+592D
0, //U+592E
0, //U+592F
0, //U+5930
0, //U+5931
0, //U+5932
0, //U+5933
0, //U+5934
0, //U+5935
0, //U+5936
0, //U+5937
0, //U+5938
0, //U+5939
0, //U+593A
0, //U+593B
0, //U+593C
0, //U+593D
0, //U+593E
0, //U+593F
0, //U+5940
0, //U+5941
0, //U+5942
0, //U+5943
0, //U+5944
0, //U+5945
0, //U+5946
0, //U+5947
0, //U+5948
0, //U+5949
0, //U+594A
0, //U+594B
0, //U+594C
0, //U+594D
0, //U+594E
0, //U+594F
0, //U+5950
0, //U+5951
0, //U+5952
0, //U+5953
0, //U+5954
0, //U+5955
0, //U+5956
0, //U+5957
0, //U+5958
0, //U+5959
0, //U+595A
0, //U+595B
0, //U+595C
0, //U+595D
0, //U+595E
0, //U+595F
0, //U+5960
0, //U+5961
0, //U+5962
0, //U+5963
0, //U+5964
0, //U+5965
0, //U+5966
0, //U+5967
0, //U+5968
0, //U+5969
0, //U+596A
0, //U+596B
0, //U+596C
0, //U+596D
0, //U+596E
0, //U+596F
0, //U+5970
0, //U+5971
0, //U+5972
0, //U+5973
0, //U+5974
0, //U+5975
0, //U+5976
0, //U+5977
0, //U+5978
0, //U+5979
0, //U+597A
0, //U+597B
0, //U+597C
0, //U+597D
0, //U+597E
0, //U+597F
0, //U+5980
0, //U+5981
0, //U+5982
0, //U+5983
0, //U+5984
0, //U+5985
0, //U+5986
0, //U+5987
0, //U+5988
0, //U+5989
0, //U+598A
0, //U+598B
0, //U+598C
0, //U+598D
0, //U+598E
0, //U+598F
0, //U+5990
0, //U+5991
0, //U+5992
0, //U+5993
0, //U+5994
0, //U+5995
0, //U+5996
0, //U+5997
0, //U+5998
0, //U+5999
0, //U+599A
0, //U+599B
0, //U+599C
0, //U+599D
0, //U+599E
0, //U+599F
0, //U+59A0
0, //U+59A1
0, //U+59A2
0, //U+59A3
0, //U+59A4
0, //U+59A5
0, //U+59A6
0, //U+59A7
0, //U+59A8
0, //U+59A9
0, //U+59AA
0, //U+59AB
0, //U+59AC
0, //U+59AD
0, //U+59AE
0, //U+59AF
0, //U+59B0
0, //U+59B1
0, //U+59B2
0, //U+59B3
0, //U+59B4
0, //U+59B5
0, //U+59B6
0, //U+59B7
0, //U+59B8
0, //U+59B9
0, //U+59BA
0, //U+59BB
0, //U+59BC
0, //U+59BD
0, //U+59BE
0, //U+59BF
0, //U+59C0
0, //U+59C1
0, //U+59C2
0, //U+59C3
0, //U+59C4
0, //U+59C5
0, //U+59C6
0, //U+59C7
0, //U+59C8
0, //U+59C9
0, //U+59CA
0, //U+59CB
0, //U+59CC
0, //U+59CD
0, //U+59CE
0, //U+59CF
0, //U+59D0
0, //U+59D1
0, //U+59D2
0, //U+59D3
0, //U+59D4
0, //U+59D5
0, //U+59D6
0, //U+59D7
0, //U+59D8
0, //U+59D9
0, //U+59DA
0, //U+59DB
0, //U+59DC
0, //U+59DD
0, //U+59DE
0, //U+59DF
0, //U+59E0
0, //U+59E1
0, //U+59E2
0, //U+59E3
0, //U+59E4
0, //U+59E5
0, //U+59E6
0, //U+59E7
0, //U+59E8
0, //U+59E9
0, //U+59EA
0, //U+59EB
0, //U+59EC
0, //U+59ED
0, //U+59EE
0, //U+59EF
0, //U+59F0
0, //U+59F1
0, //U+59F2
0, //U+59F3
0, //U+59F4
0, //U+59F5
0, //U+59F6
0, //U+59F7
0, //U+59F8
0, //U+59F9
0, //U+59FA
0, //U+59FB
0, //U+59FC
0, //U+59FD
0, //U+59FE
0, //U+59FF
0, //U+5A00
0, //U+5A01
0, //U+5A02
0, //U+5A03
0, //U+5A04
0, //U+5A05
0, //U+5A06
0, //U+5A07
0, //U+5A08
0, //U+5A09
0, //U+5A0A
0, //U+5A0B
0, //U+5A0C
0, //U+5A0D
0, //U+5A0E
0, //U+5A0F
0, //U+5A10
0, //U+5A11
0, //U+5A12
0, //U+5A13
0, //U+5A14
0, //U+5A15
0, //U+5A16
0, //U+5A17
0, //U+5A18
0, //U+5A19
0, //U+5A1A
0, //U+5A1B
0, //U+5A1C
0, //U+5A1D
0, //U+5A1E
0, //U+5A1F
0, //U+5A20
0, //U+5A21
0, //U+5A22
0, //U+5A23
0, //U+5A24
0, //U+5A25
0, //U+5A26
0, //U+5A27
0, //U+5A28
0, //U+5A29
0, //U+5A2A
0, //U+5A2B
0, //U+5A2C
0, //U+5A2D
0, //U+5A2E
0, //U+5A2F
0, //U+5A30
0, //U+5A31
0, //U+5A32
0, //U+5A33
0, //U+5A34
0, //U+5A35
0, //U+5A36
0, //U+5A37
0, //U+5A38
0, //U+5A39
0, //U+5A3A
0, //U+5A3B
0, //U+5A3C
0, //U+5A3D
0, //U+5A3E
0, //U+5A3F
0, //U+5A40
0, //U+5A41
0, //U+5A42
0, //U+5A43
0, //U+5A44
0, //U+5A45
0, //U+5A46
0, //U+5A47
0, //U+5A48
0, //U+5A49
0, //U+5A4A
0, //U+5A4B
0, //U+5A4C
0, //U+5A4D
0, //U+5A4E
0, //U+5A4F
0, //U+5A50
0, //U+5A51
0, //U+5A52
0, //U+5A53
0, //U+5A54
0, //U+5A55
0, //U+5A56
0, //U+5A57
0, //U+5A58
0, //U+5A59
0, //U+5A5A
0, //U+5A5B
0, //U+5A5C
0, //U+5A5D
0, //U+5A5E
0, //U+5A5F
0, //U+5A60
0, //U+5A61
0, //U+5A62
0, //U+5A63
0, //U+5A64
0, //U+5A65
0, //U+5A66
0, //U+5A67
0, //U+5A68
0, //U+5A69
0, //U+5A6A
0, //U+5A6B
0, //U+5A6C
0, //U+5A6D
0, //U+5A6E
0, //U+5A6F
0, //U+5A70
0, //U+5A71
0, //U+5A72
0, //U+5A73
0, //U+5A74
0, //U+5A75
0, //U+5A76
0, //U+5A77
0, //U+5A78
0, //U+5A79
0, //U+5A7A
0, //U+5A7B
0, //U+5A7C
0, //U+5A7D
0, //U+5A7E
0, //U+5A7F
0, //U+5A80
0, //U+5A81
0, //U+5A82
0, //U+5A83
0, //U+5A84
0, //U+5A85
0, //U+5A86
0, //U+5A87
0, //U+5A88
0, //U+5A89
0, //U+5A8A
0, //U+5A8B
0, //U+5A8C
0, //U+5A8D
0, //U+5A8E
0, //U+5A8F
0, //U+5A90
0, //U+5A91
0, //U+5A92
0, //U+5A93
0, //U+5A94
0, //U+5A95
0, //U+5A96
0, //U+5A97
0, //U+5A98
0, //U+5A99
0, //U+5A9A
0, //U+5A9B
0, //U+5A9C
0, //U+5A9D
0, //U+5A9E
0, //U+5A9F
0, //U+5AA0
0, //U+5AA1
0, //U+5AA2
0, //U+5AA3
0, //U+5AA4
0, //U+5AA5
0, //U+5AA6
0, //U+5AA7
0, //U+5AA8
0, //U+5AA9
0, //U+5AAA
0, //U+5AAB
0, //U+5AAC
0, //U+5AAD
0, //U+5AAE
0, //U+5AAF
0, //U+5AB0
0, //U+5AB1
0, //U+5AB2
0, //U+5AB3
0, //U+5AB4
0, //U+5AB5
0, //U+5AB6
0, //U+5AB7
0, //U+5AB8
0, //U+5AB9
0, //U+5ABA
0, //U+5ABB
0, //U+5ABC
0, //U+5ABD
0, //U+5ABE
0, //U+5ABF
0, //U+5AC0
0, //U+5AC1
0, //U+5AC2
0, //U+5AC3
0, //U+5AC4
0, //U+5AC5
0, //U+5AC6
0, //U+5AC7
0, //U+5AC8
0, //U+5AC9
0, //U+5ACA
0, //U+5ACB
0, //U+5ACC
0, //U+5ACD
0, //U+5ACE
0, //U+5ACF
0, //U+5AD0
0, //U+5AD1
0, //U+5AD2
0, //U+5AD3
0, //U+5AD4
0, //U+5AD5
0, //U+5AD6
0, //U+5AD7
0, //U+5AD8
0, //U+5AD9
0, //U+5ADA
0, //U+5ADB
0, //U+5ADC
0, //U+5ADD
0, //U+5ADE
0, //U+5ADF
0, //U+5AE0
0, //U+5AE1
0, //U+5AE2
0, //U+5AE3
0, //U+5AE4
0, //U+5AE5
0, //U+5AE6
0, //U+5AE7
0, //U+5AE8
0, //U+5AE9
0, //U+5AEA
0, //U+5AEB
0, //U+5AEC
0, //U+5AED
0, //U+5AEE
0, //U+5AEF
0, //U+5AF0
0, //U+5AF1
0, //U+5AF2
0, //U+5AF3
0, //U+5AF4
0, //U+5AF5
0, //U+5AF6
0, //U+5AF7
0, //U+5AF8
0, //U+5AF9
0, //U+5AFA
0, //U+5AFB
0, //U+5AFC
0, //U+5AFD
0, //U+5AFE
0, //U+5AFF
0, //U+5B00
0, //U+5B01
0, //U+5B02
0, //U+5B03
0, //U+5B04
0, //U+5B05
0, //U+5B06
0, //U+5B07
0, //U+5B08
0, //U+5B09
0, //U+5B0A
0, //U+5B0B
0, //U+5B0C
0, //U+5B0D
0, //U+5B0E
0, //U+5B0F
0, //U+5B10
0, //U+5B11
0, //U+5B12
0, //U+5B13
0, //U+5B14
0, //U+5B15
0, //U+5B16
0, //U+5B17
0, //U+5B18
0, //U+5B19
0, //U+5B1A
0, //U+5B1B
0, //U+5B1C
0, //U+5B1D
0, //U+5B1E
0, //U+5B1F
0, //U+5B20
0, //U+5B21
0, //U+5B22
0, //U+5B23
0, //U+5B24
0, //U+5B25
0, //U+5B26
0, //U+5B27
0, //U+5B28
0, //U+5B29
0, //U+5B2A
0, //U+5B2B
0, //U+5B2C
0, //U+5B2D
0, //U+5B2E
0, //U+5B2F
0, //U+5B30
0, //U+5B31
0, //U+5B32
0, //U+5B33
0, //U+5B34
0, //U+5B35
0, //U+5B36
0, //U+5B37
0, //U+5B38
0, //U+5B39
0, //U+5B3A
0, //U+5B3B
0, //U+5B3C
0, //U+5B3D
0, //U+5B3E
0, //U+5B3F
0, //U+5B40
0, //U+5B41
0, //U+5B42
0, //U+5B43
0, //U+5B44
0, //U+5B45
0, //U+5B46
0, //U+5B47
0, //U+5B48
0, //U+5B49
0, //U+5B4A
0, //U+5B4B
0, //U+5B4C
0, //U+5B4D
0, //U+5B4E
0, //U+5B4F
0, //U+5B50
0, //U+5B51
0, //U+5B52
0, //U+5B53
0, //U+5B54
0, //U+5B55
0, //U+5B56
0, //U+5B57
0, //U+5B58
0, //U+5B59
0, //U+5B5A
0, //U+5B5B
0, //U+5B5C
0, //U+5B5D
0, //U+5B5E
0, //U+5B5F
0, //U+5B60
0, //U+5B61
0, //U+5B62
0, //U+5B63
0, //U+5B64
0, //U+5B65
0, //U+5B66
0, //U+5B67
0, //U+5B68
0, //U+5B69
0, //U+5B6A
0, //U+5B6B
0, //U+5B6C
0, //U+5B6D
0, //U+5B6E
0, //U+5B6F
0, //U+5B70
0, //U+5B71
0, //U+5B72
0, //U+5B73
0, //U+5B74
0, //U+5B75
0, //U+5B76
0, //U+5B77
0, //U+5B78
0, //U+5B79
0, //U+5B7A
0, //U+5B7B
0, //U+5B7C
0, //U+5B7D
0, //U+5B7E
0, //U+5B7F
0, //U+5B80
0, //U+5B81
0, //U+5B82
0, //U+5B83
0, //U+5B84
0, //U+5B85
0, //U+5B86
0, //U+5B87
0, //U+5B88
0, //U+5B89
0, //U+5B8A
0, //U+5B8B
0, //U+5B8C
0, //U+5B8D
0, //U+5B8E
0, //U+5B8F
0, //U+5B90
0, //U+5B91
0, //U+5B92
0, //U+5B93
0, //U+5B94
0, //U+5B95
0, //U+5B96
0, //U+5B97
0, //U+5B98
0, //U+5B99
0, //U+5B9A
0, //U+5B9B
0, //U+5B9C
0, //U+5B9D
0, //U+5B9E
0, //U+5B9F
0, //U+5BA0
0, //U+5BA1
0, //U+5BA2
0, //U+5BA3
0, //U+5BA4
0, //U+5BA5
0, //U+5BA6
0, //U+5BA7
0, //U+5BA8
0, //U+5BA9
0, //U+5BAA
0, //U+5BAB
0, //U+5BAC
0, //U+5BAD
0, //U+5BAE
0, //U+5BAF
0, //U+5BB0
0, //U+5BB1
0, //U+5BB2
0, //U+5BB3
0, //U+5BB4
0, //U+5BB5
0, //U+5BB6
0, //U+5BB7
0, //U+5BB8
0, //U+5BB9
0, //U+5BBA
0, //U+5BBB
0, //U+5BBC
0, //U+5BBD
0, //U+5BBE
0, //U+5BBF
0, //U+5BC0
0, //U+5BC1
0, //U+5BC2
0, //U+5BC3
0, //U+5BC4
0, //U+5BC5
0, //U+5BC6
0, //U+5BC7
0, //U+5BC8
0, //U+5BC9
0, //U+5BCA
0, //U+5BCB
0, //U+5BCC
0, //U+5BCD
0, //U+5BCE
0, //U+5BCF
0, //U+5BD0
0, //U+5BD1
0, //U+5BD2
0, //U+5BD3
0, //U+5BD4
0, //U+5BD5
0, //U+5BD6
0, //U+5BD7
0, //U+5BD8
0, //U+5BD9
0, //U+5BDA
0, //U+5BDB
0, //U+5BDC
0, //U+5BDD
0, //U+5BDE
0, //U+5BDF
0, //U+5BE0
0, //U+5BE1
0, //U+5BE2
0, //U+5BE3
0, //U+5BE4
0, //U+5BE5
0, //U+5BE6
0, //U+5BE7
0, //U+5BE8
0, //U+5BE9
0, //U+5BEA
0, //U+5BEB
0, //U+5BEC
0, //U+5BED
0, //U+5BEE
0, //U+5BEF
0, //U+5BF0
0, //U+5BF1
0, //U+5BF2
0, //U+5BF3
0, //U+5BF4
0, //U+5BF5
0, //U+5BF6
0, //U+5BF7
0, //U+5BF8
0, //U+5BF9
0, //U+5BFA
0, //U+5BFB
0, //U+5BFC
0, //U+5BFD
0, //U+5BFE
0, //U+5BFF
0, //U+5C00
0, //U+5C01
0, //U+5C02
0, //U+5C03
0, //U+5C04
0, //U+5C05
0, //U+5C06
0, //U+5C07
0, //U+5C08
0, //U+5C09
0, //U+5C0A
0, //U+5C0B
0, //U+5C0C
0, //U+5C0D
0, //U+5C0E
0, //U+5C0F
0, //U+5C10
0, //U+5C11
0, //U+5C12
0, //U+5C13
0, //U+5C14
0, //U+5C15
0, //U+5C16
0, //U+5C17
0, //U+5C18
0, //U+5C19
0, //U+5C1A
0, //U+5C1B
0, //U+5C1C
0, //U+5C1D
0, //U+5C1E
0, //U+5C1F
0, //U+5C20
0, //U+5C21
0, //U+5C22
0, //U+5C23
0, //U+5C24
0, //U+5C25
0, //U+5C26
0, //U+5C27
0, //U+5C28
0, //U+5C29
0, //U+5C2A
0, //U+5C2B
0, //U+5C2C
0, //U+5C2D
0, //U+5C2E
0, //U+5C2F
0, //U+5C30
0, //U+5C31
0, //U+5C32
0, //U+5C33
0, //U+5C34
0, //U+5C35
0, //U+5C36
0, //U+5C37
0, //U+5C38
0, //U+5C39
0, //U+5C3A
0, //U+5C3B
0, //U+5C3C
0, //U+5C3D
0, //U+5C3E
0, //U+5C3F
0, //U+5C40
0, //U+5C41
0, //U+5C42
0, //U+5C43
0, //U+5C44
0, //U+5C45
0, //U+5C46
0, //U+5C47
0, //U+5C48
0, //U+5C49
0, //U+5C4A
0, //U+5C4B
0, //U+5C4C
0, //U+5C4D
0, //U+5C4E
0, //U+5C4F
0, //U+5C50
0, //U+5C51
0, //U+5C52
0, //U+5C53
0, //U+5C54
0, //U+5C55
0, //U+5C56
0, //U+5C57
0, //U+5C58
0, //U+5C59
0, //U+5C5A
0, //U+5C5B
0, //U+5C5C
0, //U+5C5D
0, //U+5C5E
0, //U+5C5F
0, //U+5C60
0, //U+5C61
0, //U+5C62
0, //U+5C63
0, //U+5C64
0, //U+5C65
0, //U+5C66
0, //U+5C67
0, //U+5C68
0, //U+5C69
0, //U+5C6A
0, //U+5C6B
0, //U+5C6C
0, //U+5C6D
0, //U+5C6E
0, //U+5C6F
0, //U+5C70
0, //U+5C71
0, //U+5C72
0, //U+5C73
0, //U+5C74
0, //U+5C75
0, //U+5C76
0, //U+5C77
0, //U+5C78
0, //U+5C79
0, //U+5C7A
0, //U+5C7B
0, //U+5C7C
0, //U+5C7D
0, //U+5C7E
0, //U+5C7F
0, //U+5C80
0, //U+5C81
0, //U+5C82
0, //U+5C83
0, //U+5C84
0, //U+5C85
0, //U+5C86
0, //U+5C87
0, //U+5C88
0, //U+5C89
0, //U+5C8A
0, //U+5C8B
0, //U+5C8C
0, //U+5C8D
0, //U+5C8E
0, //U+5C8F
0, //U+5C90
0, //U+5C91
0, //U+5C92
0, //U+5C93
0, //U+5C94
0, //U+5C95
0, //U+5C96
0, //U+5C97
0, //U+5C98
0, //U+5C99
0, //U+5C9A
0, //U+5C9B
0, //U+5C9C
0, //U+5C9D
0, //U+5C9E
0, //U+5C9F
0, //U+5CA0
0, //U+5CA1
0, //U+5CA2
0, //U+5CA3
0, //U+5CA4
0, //U+5CA5
0, //U+5CA6
0, //U+5CA7
0, //U+5CA8
0, //U+5CA9
0, //U+5CAA
0, //U+5CAB
0, //U+5CAC
0, //U+5CAD
0, //U+5CAE
0, //U+5CAF
0, //U+5CB0
0, //U+5CB1
0, //U+5CB2
0, //U+5CB3
0, //U+5CB4
0, //U+5CB5
0, //U+5CB6
0, //U+5CB7
0, //U+5CB8
0, //U+5CB9
0, //U+5CBA
0, //U+5CBB
0, //U+5CBC
0, //U+5CBD
0, //U+5CBE
0, //U+5CBF
0, //U+5CC0
0, //U+5CC1
0, //U+5CC2
0, //U+5CC3
0, //U+5CC4
0, //U+5CC5
0, //U+5CC6
0, //U+5CC7
0, //U+5CC8
0, //U+5CC9
0, //U+5CCA
0, //U+5CCB
0, //U+5CCC
0, //U+5CCD
0, //U+5CCE
0, //U+5CCF
0, //U+5CD0
0, //U+5CD1
0, //U+5CD2
0, //U+5CD3
0, //U+5CD4
0, //U+5CD5
0, //U+5CD6
0, //U+5CD7
0, //U+5CD8
0, //U+5CD9
0, //U+5CDA
0, //U+5CDB
0, //U+5CDC
0, //U+5CDD
0, //U+5CDE
0, //U+5CDF
0, //U+5CE0
0, //U+5CE1
0, //U+5CE2
0, //U+5CE3
0, //U+5CE4
0, //U+5CE5
0, //U+5CE6
0, //U+5CE7
0, //U+5CE8
0, //U+5CE9
0, //U+5CEA
0, //U+5CEB
0, //U+5CEC
0, //U+5CED
0, //U+5CEE
0, //U+5CEF
0, //U+5CF0
0, //U+5CF1
0, //U+5CF2
0, //U+5CF3
0, //U+5CF4
0, //U+5CF5
0, //U+5CF6
0, //U+5CF7
0, //U+5CF8
0, //U+5CF9
0, //U+5CFA
0, //U+5CFB
0, //U+5CFC
0, //U+5CFD
0, //U+5CFE
0, //U+5CFF
0, //U+5D00
0, //U+5D01
0, //U+5D02
0, //U+5D03
0, //U+5D04
0, //U+5D05
0, //U+5D06
0, //U+5D07
0, //U+5D08
0, //U+5D09
0, //U+5D0A
0, //U+5D0B
0, //U+5D0C
0, //U+5D0D
0, //U+5D0E
0, //U+5D0F
0, //U+5D10
0, //U+5D11
0, //U+5D12
0, //U+5D13
0, //U+5D14
0, //U+5D15
0, //U+5D16
0, //U+5D17
0, //U+5D18
0, //U+5D19
0, //U+5D1A
0, //U+5D1B
0, //U+5D1C
0, //U+5D1D
0, //U+5D1E
0, //U+5D1F
0, //U+5D20
0, //U+5D21
0, //U+5D22
0, //U+5D23
0, //U+5D24
0, //U+5D25
0, //U+5D26
0, //U+5D27
0, //U+5D28
0, //U+5D29
0, //U+5D2A
0, //U+5D2B
0, //U+5D2C
0, //U+5D2D
0, //U+5D2E
0, //U+5D2F
0, //U+5D30
0, //U+5D31
0, //U+5D32
0, //U+5D33
0, //U+5D34
0, //U+5D35
0, //U+5D36
0, //U+5D37
0, //U+5D38
0, //U+5D39
0, //U+5D3A
0, //U+5D3B
0, //U+5D3C
0, //U+5D3D
0, //U+5D3E
0, //U+5D3F
0, //U+5D40
0, //U+5D41
0, //U+5D42
0, //U+5D43
0, //U+5D44
0, //U+5D45
0, //U+5D46
0, //U+5D47
0, //U+5D48
0, //U+5D49
0, //U+5D4A
0, //U+5D4B
0, //U+5D4C
0, //U+5D4D
0, //U+5D4E
0, //U+5D4F
0, //U+5D50
0, //U+5D51
0, //U+5D52
0, //U+5D53
0, //U+5D54
0, //U+5D55
0, //U+5D56
0, //U+5D57
0, //U+5D58
0, //U+5D59
0, //U+5D5A
0, //U+5D5B
0, //U+5D5C
0, //U+5D5D
0, //U+5D5E
0, //U+5D5F
0, //U+5D60
0, //U+5D61
0, //U+5D62
0, //U+5D63
0, //U+5D64
0, //U+5D65
0, //U+5D66
0, //U+5D67
0, //U+5D68
0, //U+5D69
0, //U+5D6A
0, //U+5D6B
0, //U+5D6C
0, //U+5D6D
0, //U+5D6E
0, //U+5D6F
0, //U+5D70
0, //U+5D71
0, //U+5D72
0, //U+5D73
0, //U+5D74
0, //U+5D75
0, //U+5D76
0, //U+5D77
0, //U+5D78
0, //U+5D79
0, //U+5D7A
0, //U+5D7B
0, //U+5D7C
0, //U+5D7D
0, //U+5D7E
0, //U+5D7F
0, //U+5D80
0, //U+5D81
0, //U+5D82
0, //U+5D83
0, //U+5D84
0, //U+5D85
0, //U+5D86
0, //U+5D87
0, //U+5D88
0, //U+5D89
0, //U+5D8A
0, //U+5D8B
0, //U+5D8C
0, //U+5D8D
0, //U+5D8E
0, //U+5D8F
0, //U+5D90
0, //U+5D91
0, //U+5D92
0, //U+5D93
0, //U+5D94
0, //U+5D95
0, //U+5D96
0, //U+5D97
0, //U+5D98
0, //U+5D99
0, //U+5D9A
0, //U+5D9B
0, //U+5D9C
0, //U+5D9D
0, //U+5D9E
0, //U+5D9F
0, //U+5DA0
0, //U+5DA1
0, //U+5DA2
0, //U+5DA3
0, //U+5DA4
0, //U+5DA5
0, //U+5DA6
0, //U+5DA7
0, //U+5DA8
0, //U+5DA9
0, //U+5DAA
0, //U+5DAB
0, //U+5DAC
0, //U+5DAD
0, //U+5DAE
0, //U+5DAF
0, //U+5DB0
0, //U+5DB1
0, //U+5DB2
0, //U+5DB3
0, //U+5DB4
0, //U+5DB5
0, //U+5DB6
0, //U+5DB7
0, //U+5DB8
0, //U+5DB9
0, //U+5DBA
0, //U+5DBB
0, //U+5DBC
0, //U+5DBD
0, //U+5DBE
0, //U+5DBF
0, //U+5DC0
0, //U+5DC1
0, //U+5DC2
0, //U+5DC3
0, //U+5DC4
0, //U+5DC5
0, //U+5DC6
0, //U+5DC7
0, //U+5DC8
0, //U+5DC9
0, //U+5DCA
0, //U+5DCB
0, //U+5DCC
0, //U+5DCD
0, //U+5DCE
0, //U+5DCF
0, //U+5DD0
0, //U+5DD1
0, //U+5DD2
0, //U+5DD3
0, //U+5DD4
0, //U+5DD5
0, //U+5DD6
0, //U+5DD7
0, //U+5DD8
0, //U+5DD9
0, //U+5DDA
0, //U+5DDB
0, //U+5DDC
0, //U+5DDD
0, //U+5DDE
0, //U+5DDF
0, //U+5DE0
0, //U+5DE1
0, //U+5DE2
0, //U+5DE3
0, //U+5DE4
0, //U+5DE5
0, //U+5DE6
0, //U+5DE7
0, //U+5DE8
0, //U+5DE9
0, //U+5DEA
0, //U+5DEB
0, //U+5DEC
0, //U+5DED
0, //U+5DEE
0, //U+5DEF
0, //U+5DF0
0, //U+5DF1
0, //U+5DF2
0, //U+5DF3
0, //U+5DF4
0, //U+5DF5
0, //U+5DF6
0, //U+5DF7
0, //U+5DF8
0, //U+5DF9
0, //U+5DFA
0, //U+5DFB
0, //U+5DFC
0, //U+5DFD
0, //U+5DFE
0, //U+5DFF
0, //U+5E00
0, //U+5E01
0, //U+5E02
0, //U+5E03
0, //U+5E04
0, //U+5E05
0, //U+5E06
0, //U+5E07
0, //U+5E08
0, //U+5E09
0, //U+5E0A
0, //U+5E0B
0, //U+5E0C
0, //U+5E0D
0, //U+5E0E
0, //U+5E0F
0, //U+5E10
0, //U+5E11
0, //U+5E12
0, //U+5E13
0, //U+5E14
0, //U+5E15
0, //U+5E16
0, //U+5E17
0, //U+5E18
0, //U+5E19
0, //U+5E1A
0, //U+5E1B
0, //U+5E1C
0, //U+5E1D
0, //U+5E1E
0, //U+5E1F
0, //U+5E20
0, //U+5E21
0, //U+5E22
0, //U+5E23
0, //U+5E24
0, //U+5E25
0, //U+5E26
0, //U+5E27
0, //U+5E28
0, //U+5E29
0, //U+5E2A
0, //U+5E2B
0, //U+5E2C
0, //U+5E2D
0, //U+5E2E
0, //U+5E2F
0, //U+5E30
0, //U+5E31
0, //U+5E32
0, //U+5E33
0, //U+5E34
0, //U+5E35
0, //U+5E36
0, //U+5E37
0, //U+5E38
0, //U+5E39
0, //U+5E3A
0, //U+5E3B
0, //U+5E3C
0, //U+5E3D
0, //U+5E3E
0, //U+5E3F
0, //U+5E40
0, //U+5E41
0, //U+5E42
0, //U+5E43
0, //U+5E44
0, //U+5E45
0, //U+5E46
0, //U+5E47
0, //U+5E48
0, //U+5E49
0, //U+5E4A
0, //U+5E4B
0, //U+5E4C
0, //U+5E4D
0, //U+5E4E
0, //U+5E4F
0, //U+5E50
0, //U+5E51
0, //U+5E52
0, //U+5E53
0, //U+5E54
0, //U+5E55
0, //U+5E56
0, //U+5E57
0, //U+5E58
0, //U+5E59
0, //U+5E5A
0, //U+5E5B
0, //U+5E5C
0, //U+5E5D
0, //U+5E5E
0, //U+5E5F
0, //U+5E60
0, //U+5E61
0, //U+5E62
0, //U+5E63
0, //U+5E64
0, //U+5E65
0, //U+5E66
0, //U+5E67
0, //U+5E68
0, //U+5E69
0, //U+5E6A
0, //U+5E6B
0, //U+5E6C
0, //U+5E6D
0, //U+5E6E
0, //U+5E6F
0, //U+5E70
0, //U+5E71
0, //U+5E72
0, //U+5E73
0, //U+5E74
0, //U+5E75
0, //U+5E76
0, //U+5E77
0, //U+5E78
0, //U+5E79
0, //U+5E7A
0, //U+5E7B
0, //U+5E7C
0, //U+5E7D
0, //U+5E7E
0, //U+5E7F
0, //U+5E80
0, //U+5E81
0, //U+5E82
0, //U+5E83
0, //U+5E84
0, //U+5E85
0, //U+5E86
0, //U+5E87
0, //U+5E88
0, //U+5E89
0, //U+5E8A
0, //U+5E8B
0, //U+5E8C
0, //U+5E8D
0, //U+5E8E
0, //U+5E8F
0, //U+5E90
0, //U+5E91
0, //U+5E92
0, //U+5E93
0, //U+5E94
0, //U+5E95
0, //U+5E96
0, //U+5E97
0, //U+5E98
0, //U+5E99
0, //U+5E9A
0, //U+5E9B
0, //U+5E9C
0, //U+5E9D
0, //U+5E9E
0, //U+5E9F
0, //U+5EA0
0, //U+5EA1
0, //U+5EA2
0, //U+5EA3
0, //U+5EA4
0, //U+5EA5
0, //U+5EA6
0, //U+5EA7
0, //U+5EA8
0, //U+5EA9
0, //U+5EAA
0, //U+5EAB
0, //U+5EAC
0, //U+5EAD
0, //U+5EAE
0, //U+5EAF
0, //U+5EB0
0, //U+5EB1
0, //U+5EB2
0, //U+5EB3
0, //U+5EB4
0, //U+5EB5
0, //U+5EB6
0, //U+5EB7
0, //U+5EB8
0, //U+5EB9
0, //U+5EBA
0, //U+5EBB
0, //U+5EBC
0, //U+5EBD
0, //U+5EBE
0, //U+5EBF
0, //U+5EC0
0, //U+5EC1
0, //U+5EC2
0, //U+5EC3
0, //U+5EC4
0, //U+5EC5
0, //U+5EC6
0, //U+5EC7
0, //U+5EC8
0, //U+5EC9
0, //U+5ECA
0, //U+5ECB
0, //U+5ECC
0, //U+5ECD
0, //U+5ECE
0, //U+5ECF
0, //U+5ED0
0, //U+5ED1
0, //U+5ED2
0, //U+5ED3
0, //U+5ED4
0, //U+5ED5
0, //U+5ED6
0, //U+5ED7
0, //U+5ED8
0, //U+5ED9
0, //U+5EDA
0, //U+5EDB
0, //U+5EDC
0, //U+5EDD
0, //U+5EDE
0, //U+5EDF
0, //U+5EE0
0, //U+5EE1
0, //U+5EE2
0, //U+5EE3
0, //U+5EE4
0, //U+5EE5
0, //U+5EE6
0, //U+5EE7
0, //U+5EE8
0, //U+5EE9
0, //U+5EEA
0, //U+5EEB
0, //U+5EEC
0, //U+5EED
0, //U+5EEE
0, //U+5EEF
0, //U+5EF0
0, //U+5EF1
0, //U+5EF2
0, //U+5EF3
0, //U+5EF4
0, //U+5EF5
0, //U+5EF6
0, //U+5EF7
0, //U+5EF8
0, //U+5EF9
0, //U+5EFA
0, //U+5EFB
0, //U+5EFC
0, //U+5EFD
0, //U+5EFE
0, //U+5EFF
0, //U+5F00
0, //U+5F01
0, //U+5F02
0, //U+5F03
0, //U+5F04
0, //U+5F05
0, //U+5F06
0, //U+5F07
0, //U+5F08
0, //U+5F09
0, //U+5F0A
0, //U+5F0B
0, //U+5F0C
0, //U+5F0D
0, //U+5F0E
0, //U+5F0F
0, //U+5F10
0, //U+5F11
0, //U+5F12
0, //U+5F13
0, //U+5F14
0, //U+5F15
0, //U+5F16
0, //U+5F17
0, //U+5F18
0, //U+5F19
0, //U+5F1A
0, //U+5F1B
0, //U+5F1C
0, //U+5F1D
0, //U+5F1E
0, //U+5F1F
0, //U+5F20
0, //U+5F21
0, //U+5F22
0, //U+5F23
0, //U+5F24
0, //U+5F25
0, //U+5F26
0, //U+5F27
0, //U+5F28
0, //U+5F29
0, //U+5F2A
0, //U+5F2B
0, //U+5F2C
0, //U+5F2D
0, //U+5F2E
0, //U+5F2F
0, //U+5F30
0, //U+5F31
0, //U+5F32
0, //U+5F33
0, //U+5F34
0, //U+5F35
0, //U+5F36
0, //U+5F37
0, //U+5F38
0, //U+5F39
0, //U+5F3A
0, //U+5F3B
0, //U+5F3C
0, //U+5F3D
0, //U+5F3E
0, //U+5F3F
0, //U+5F40
0, //U+5F41
0, //U+5F42
0, //U+5F43
0, //U+5F44
0, //U+5F45
0, //U+5F46
0, //U+5F47
0, //U+5F48
0, //U+5F49
0, //U+5F4A
0, //U+5F4B
0, //U+5F4C
0, //U+5F4D
0, //U+5F4E
0, //U+5F4F
0, //U+5F50
0, //U+5F51
0, //U+5F52
0, //U+5F53
0, //U+5F54
0, //U+5F55
0, //U+5F56
0, //U+5F57
0, //U+5F58
0, //U+5F59
0, //U+5F5A
0, //U+5F5B
0, //U+5F5C
0, //U+5F5D
0, //U+5F5E
0, //U+5F5F
0, //U+5F60
0, //U+5F61
0, //U+5F62
0, //U+5F63
0, //U+5F64
0, //U+5F65
0, //U+5F66
0, //U+5F67
0, //U+5F68
0, //U+5F69
0, //U+5F6A
0, //U+5F6B
0, //U+5F6C
0, //U+5F6D
0, //U+5F6E
0, //U+5F6F
0, //U+5F70
0, //U+5F71
0, //U+5F72
0, //U+5F73
0, //U+5F74
0, //U+5F75
0, //U+5F76
0, //U+5F77
0, //U+5F78
0, //U+5F79
0, //U+5F7A
0, //U+5F7B
0, //U+5F7C
0, //U+5F7D
0, //U+5F7E
0, //U+5F7F
0, //U+5F80
0, //U+5F81
0, //U+5F82
0, //U+5F83
0, //U+5F84
0, //U+5F85
0, //U+5F86
0, //U+5F87
0, //U+5F88
0, //U+5F89
0, //U+5F8A
0, //U+5F8B
0, //U+5F8C
0, //U+5F8D
0, //U+5F8E
0, //U+5F8F
0, //U+5F90
0, //U+5F91
0, //U+5F92
0, //U+5F93
0, //U+5F94
0, //U+5F95
0, //U+5F96
0, //U+5F97
0, //U+5F98
0, //U+5F99
0, //U+5F9A
0, //U+5F9B
0, //U+5F9C
0, //U+5F9D
0, //U+5F9E
0, //U+5F9F
0, //U+5FA0
0, //U+5FA1
0, //U+5FA2
0, //U+5FA3
0, //U+5FA4
0, //U+5FA5
0, //U+5FA6
0, //U+5FA7
0, //U+5FA8
0, //U+5FA9
0, //U+5FAA
0, //U+5FAB
0, //U+5FAC
0, //U+5FAD
0, //U+5FAE
0, //U+5FAF
0, //U+5FB0
0, //U+5FB1
0, //U+5FB2
0, //U+5FB3
0, //U+5FB4
0, //U+5FB5
0, //U+5FB6
0, //U+5FB7
0, //U+5FB8
0, //U+5FB9
0, //U+5FBA
0, //U+5FBB
0, //U+5FBC
0, //U+5FBD
0, //U+5FBE
0, //U+5FBF
0, //U+5FC0
0, //U+5FC1
0, //U+5FC2
0, //U+5FC3
0, //U+5FC4
0, //U+5FC5
0, //U+5FC6
0, //U+5FC7
0, //U+5FC8
0, //U+5FC9
0, //U+5FCA
0, //U+5FCB
0, //U+5FCC
0, //U+5FCD
0, //U+5FCE
0, //U+5FCF
0, //U+5FD0
0, //U+5FD1
0, //U+5FD2
0, //U+5FD3
0, //U+5FD4
0, //U+5FD5
0, //U+5FD6
0, //U+5FD7
0, //U+5FD8
0, //U+5FD9
0, //U+5FDA
0, //U+5FDB
0, //U+5FDC
0, //U+5FDD
0, //U+5FDE
0, //U+5FDF
0, //U+5FE0
0, //U+5FE1
0, //U+5FE2
0, //U+5FE3
0, //U+5FE4
0, //U+5FE5
0, //U+5FE6
0, //U+5FE7
0, //U+5FE8
0, //U+5FE9
0, //U+5FEA
0, //U+5FEB
0, //U+5FEC
0, //U+5FED
0, //U+5FEE
0, //U+5FEF
0, //U+5FF0
0, //U+5FF1
0, //U+5FF2
0, //U+5FF3
0, //U+5FF4
0, //U+5FF5
0, //U+5FF6
0, //U+5FF7
0, //U+5FF8
0, //U+5FF9
0, //U+5FFA
0, //U+5FFB
0, //U+5FFC
0, //U+5FFD
0, //U+5FFE
0, //U+5FFF
0, //U+6000
0, //U+6001
0, //U+6002
0, //U+6003
0, //U+6004
0, //U+6005
0, //U+6006
0, //U+6007
0, //U+6008
0, //U+6009
0, //U+600A
0, //U+600B
0, //U+600C
0, //U+600D
0, //U+600E
0, //U+600F
0, //U+6010
0, //U+6011
0, //U+6012
0, //U+6013
0, //U+6014
0, //U+6015
0, //U+6016
0, //U+6017
0, //U+6018
0, //U+6019
0, //U+601A
0, //U+601B
0, //U+601C
0, //U+601D
0, //U+601E
0, //U+601F
0, //U+6020
0, //U+6021
0, //U+6022
0, //U+6023
0, //U+6024
0, //U+6025
0, //U+6026
0, //U+6027
0, //U+6028
0, //U+6029
0, //U+602A
0, //U+602B
0, //U+602C
0, //U+602D
0, //U+602E
0, //U+602F
0, //U+6030
0, //U+6031
0, //U+6032
0, //U+6033
0, //U+6034
0, //U+6035
0, //U+6036
0, //U+6037
0, //U+6038
0, //U+6039
0, //U+603A
0, //U+603B
0, //U+603C
0, //U+603D
0, //U+603E
0, //U+603F
0, //U+6040
0, //U+6041
0, //U+6042
0, //U+6043
0, //U+6044
0, //U+6045
0, //U+6046
0, //U+6047
0, //U+6048
0, //U+6049
0, //U+604A
0, //U+604B
0, //U+604C
0, //U+604D
0, //U+604E
0, //U+604F
0, //U+6050
0, //U+6051
0, //U+6052
0, //U+6053
0, //U+6054
0, //U+6055
0, //U+6056
0, //U+6057
0, //U+6058
0, //U+6059
0, //U+605A
0, //U+605B
0, //U+605C
0, //U+605D
0, //U+605E
0, //U+605F
0, //U+6060
0, //U+6061
0, //U+6062
0, //U+6063
0, //U+6064
0, //U+6065
0, //U+6066
0, //U+6067
0, //U+6068
0, //U+6069
0, //U+606A
0, //U+606B
0, //U+606C
0, //U+606D
0, //U+606E
0, //U+606F
0, //U+6070
0, //U+6071
0, //U+6072
0, //U+6073
0, //U+6074
0, //U+6075
0, //U+6076
0, //U+6077
0, //U+6078
0, //U+6079
0, //U+607A
0, //U+607B
0, //U+607C
0, //U+607D
0, //U+607E
0, //U+607F
0, //U+6080
0, //U+6081
0, //U+6082
0, //U+6083
0, //U+6084
0, //U+6085
0, //U+6086
0, //U+6087
0, //U+6088
0, //U+6089
0, //U+608A
0, //U+608B
0, //U+608C
0, //U+608D
0, //U+608E
0, //U+608F
0, //U+6090
0, //U+6091
0, //U+6092
0, //U+6093
0, //U+6094
0, //U+6095
0, //U+6096
0, //U+6097
0, //U+6098
0, //U+6099
0, //U+609A
0, //U+609B
0, //U+609C
0, //U+609D
0, //U+609E
0, //U+609F
0, //U+60A0
0, //U+60A1
0, //U+60A2
0, //U+60A3
0, //U+60A4
0, //U+60A5
0, //U+60A6
0, //U+60A7
0, //U+60A8
0, //U+60A9
0, //U+60AA
0, //U+60AB
0, //U+60AC
0, //U+60AD
0, //U+60AE
0, //U+60AF
0, //U+60B0
0, //U+60B1
0, //U+60B2
0, //U+60B3
0, //U+60B4
0, //U+60B5
0, //U+60B6
0, //U+60B7
0, //U+60B8
0, //U+60B9
0, //U+60BA
0, //U+60BB
0, //U+60BC
0, //U+60BD
0, //U+60BE
0, //U+60BF
0, //U+60C0
0, //U+60C1
0, //U+60C2
0, //U+60C3
0, //U+60C4
0, //U+60C5
0, //U+60C6
0, //U+60C7
0, //U+60C8
0, //U+60C9
0, //U+60CA
0, //U+60CB
0, //U+60CC
0, //U+60CD
0, //U+60CE
0, //U+60CF
0, //U+60D0
0, //U+60D1
0, //U+60D2
0, //U+60D3
0, //U+60D4
0, //U+60D5
0, //U+60D6
0, //U+60D7
0, //U+60D8
0, //U+60D9
0, //U+60DA
0, //U+60DB
0, //U+60DC
0, //U+60DD
0, //U+60DE
0, //U+60DF
0, //U+60E0
0, //U+60E1
0, //U+60E2
0, //U+60E3
0, //U+60E4
0, //U+60E5
0, //U+60E6
0, //U+60E7
0, //U+60E8
0, //U+60E9
0, //U+60EA
0, //U+60EB
0, //U+60EC
0, //U+60ED
0, //U+60EE
0, //U+60EF
0, //U+60F0
0, //U+60F1
0, //U+60F2
0, //U+60F3
0, //U+60F4
0, //U+60F5
0, //U+60F6
0, //U+60F7
0, //U+60F8
0, //U+60F9
0, //U+60FA
0, //U+60FB
0, //U+60FC
0, //U+60FD
0, //U+60FE
0, //U+60FF
0, //U+6100
0, //U+6101
0, //U+6102
0, //U+6103
0, //U+6104
0, //U+6105
0, //U+6106
0, //U+6107
0, //U+6108
0, //U+6109
0, //U+610A
0, //U+610B
0, //U+610C
0, //U+610D
0, //U+610E
0, //U+610F
0, //U+6110
0, //U+6111
0, //U+6112
0, //U+6113
0, //U+6114
0, //U+6115
0, //U+6116
0, //U+6117
0, //U+6118
0, //U+6119
0, //U+611A
0, //U+611B
0, //U+611C
0, //U+611D
0, //U+611E
0, //U+611F
0, //U+6120
0, //U+6121
0, //U+6122
0, //U+6123
0, //U+6124
0, //U+6125
0, //U+6126
0, //U+6127
0, //U+6128
0, //U+6129
0, //U+612A
0, //U+612B
0, //U+612C
0, //U+612D
0, //U+612E
0, //U+612F
0, //U+6130
0, //U+6131
0, //U+6132
0, //U+6133
0, //U+6134
0, //U+6135
0, //U+6136
0, //U+6137
0, //U+6138
0, //U+6139
0, //U+613A
0, //U+613B
0, //U+613C
0, //U+613D
0, //U+613E
0, //U+613F
0, //U+6140
0, //U+6141
0, //U+6142
0, //U+6143
0, //U+6144
0, //U+6145
0, //U+6146
0, //U+6147
0, //U+6148
0, //U+6149
0, //U+614A
0, //U+614B
0, //U+614C
0, //U+614D
0, //U+614E
0, //U+614F
0, //U+6150
0, //U+6151
0, //U+6152
0, //U+6153
0, //U+6154
0, //U+6155
0, //U+6156
0, //U+6157
0, //U+6158
0, //U+6159
0, //U+615A
0, //U+615B
0, //U+615C
0, //U+615D
0, //U+615E
0, //U+615F
0, //U+6160
0, //U+6161
0, //U+6162
0, //U+6163
0, //U+6164
0, //U+6165
0, //U+6166
0, //U+6167
0, //U+6168
0, //U+6169
0, //U+616A
0, //U+616B
0, //U+616C
0, //U+616D
0, //U+616E
0, //U+616F
0, //U+6170
0, //U+6171
0, //U+6172
0, //U+6173
0, //U+6174
0, //U+6175
0, //U+6176
0, //U+6177
0, //U+6178
0, //U+6179
0, //U+617A
0, //U+617B
0, //U+617C
0, //U+617D
0, //U+617E
0, //U+617F
0, //U+6180
0, //U+6181
0, //U+6182
0, //U+6183
0, //U+6184
0, //U+6185
0, //U+6186
0, //U+6187
0, //U+6188
0, //U+6189
0, //U+618A
0, //U+618B
0, //U+618C
0, //U+618D
0, //U+618E
0, //U+618F
0, //U+6190
0, //U+6191
0, //U+6192
0, //U+6193
0, //U+6194
0, //U+6195
0, //U+6196
0, //U+6197
0, //U+6198
0, //U+6199
0, //U+619A
0, //U+619B
0, //U+619C
0, //U+619D
0, //U+619E
0, //U+619F
0, //U+61A0
0, //U+61A1
0, //U+61A2
0, //U+61A3
0, //U+61A4
0, //U+61A5
0, //U+61A6
0, //U+61A7
0, //U+61A8
0, //U+61A9
0, //U+61AA
0, //U+61AB
0, //U+61AC
0, //U+61AD
0, //U+61AE
0, //U+61AF
0, //U+61B0
0, //U+61B1
0, //U+61B2
0, //U+61B3
0, //U+61B4
0, //U+61B5
0, //U+61B6
0, //U+61B7
0, //U+61B8
0, //U+61B9
0, //U+61BA
0, //U+61BB
0, //U+61BC
0, //U+61BD
0, //U+61BE
0, //U+61BF
0, //U+61C0
0, //U+61C1
0, //U+61C2
0, //U+61C3
0, //U+61C4
0, //U+61C5
0, //U+61C6
0, //U+61C7
0, //U+61C8
0, //U+61C9
0, //U+61CA
0, //U+61CB
0, //U+61CC
0, //U+61CD
0, //U+61CE
0, //U+61CF
0, //U+61D0
0, //U+61D1
0, //U+61D2
0, //U+61D3
0, //U+61D4
0, //U+61D5
0, //U+61D6
0, //U+61D7
0, //U+61D8
0, //U+61D9
0, //U+61DA
0, //U+61DB
0, //U+61DC
0, //U+61DD
0, //U+61DE
0, //U+61DF
0, //U+61E0
0, //U+61E1
0, //U+61E2
0, //U+61E3
0, //U+61E4
0, //U+61E5
0, //U+61E6
0, //U+61E7
0, //U+61E8
0, //U+61E9
0, //U+61EA
0, //U+61EB
0, //U+61EC
0, //U+61ED
0, //U+61EE
0, //U+61EF
0, //U+61F0
0, //U+61F1
0, //U+61F2
0, //U+61F3
0, //U+61F4
0, //U+61F5
0, //U+61F6
0, //U+61F7
0, //U+61F8
0, //U+61F9
0, //U+61FA
0, //U+61FB
0, //U+61FC
0, //U+61FD
0, //U+61FE
0, //U+61FF
0, //U+6200
0, //U+6201
0, //U+6202
0, //U+6203
0, //U+6204
0, //U+6205
0, //U+6206
0, //U+6207
0, //U+6208
0, //U+6209
0, //U+620A
0, //U+620B
0, //U+620C
0, //U+620D
0, //U+620E
0, //U+620F
0, //U+6210
0, //U+6211
0, //U+6212
0, //U+6213
0, //U+6214
0, //U+6215
0, //U+6216
0, //U+6217
0, //U+6218
0, //U+6219
0, //U+621A
0, //U+621B
0, //U+621C
0, //U+621D
0, //U+621E
0, //U+621F
0, //U+6220
0, //U+6221
0, //U+6222
0, //U+6223
0, //U+6224
0, //U+6225
0, //U+6226
0, //U+6227
0, //U+6228
0, //U+6229
0, //U+622A
0, //U+622B
0, //U+622C
0, //U+622D
0, //U+622E
0, //U+622F
0, //U+6230
0, //U+6231
0, //U+6232
0, //U+6233
0, //U+6234
0, //U+6235
0, //U+6236
0, //U+6237
0, //U+6238
0, //U+6239
0, //U+623A
0, //U+623B
0, //U+623C
0, //U+623D
0, //U+623E
0, //U+623F
0, //U+6240
0, //U+6241
0, //U+6242
0, //U+6243
0, //U+6244
0, //U+6245
0, //U+6246
0, //U+6247
0, //U+6248
0, //U+6249
0, //U+624A
0, //U+624B
0, //U+624C
0, //U+624D
0, //U+624E
0, //U+624F
0, //U+6250
0, //U+6251
0, //U+6252
0, //U+6253
0, //U+6254
0, //U+6255
0, //U+6256
0, //U+6257
0, //U+6258
0, //U+6259
0, //U+625A
0, //U+625B
0, //U+625C
0, //U+625D
0, //U+625E
0, //U+625F
0, //U+6260
0, //U+6261
0, //U+6262
0, //U+6263
0, //U+6264
0, //U+6265
0, //U+6266
0, //U+6267
0, //U+6268
0, //U+6269
0, //U+626A
0, //U+626B
0, //U+626C
0, //U+626D
0, //U+626E
0, //U+626F
0, //U+6270
0, //U+6271
0, //U+6272
0, //U+6273
0, //U+6274
0, //U+6275
0, //U+6276
0, //U+6277
0, //U+6278
0, //U+6279
0, //U+627A
0, //U+627B
0, //U+627C
0, //U+627D
0, //U+627E
0, //U+627F
0, //U+6280
0, //U+6281
0, //U+6282
0, //U+6283
0, //U+6284
0, //U+6285
0, //U+6286
0, //U+6287
0, //U+6288
0, //U+6289
0, //U+628A
0, //U+628B
0, //U+628C
0, //U+628D
0, //U+628E
0, //U+628F
0, //U+6290
0, //U+6291
0, //U+6292
0, //U+6293
0, //U+6294
0, //U+6295
0, //U+6296
0, //U+6297
0, //U+6298
0, //U+6299
0, //U+629A
0, //U+629B
0, //U+629C
0, //U+629D
0, //U+629E
0, //U+629F
0, //U+62A0
0, //U+62A1
0, //U+62A2
0, //U+62A3
0, //U+62A4
0, //U+62A5
0, //U+62A6
0, //U+62A7
0, //U+62A8
0, //U+62A9
0, //U+62AA
0, //U+62AB
0, //U+62AC
0, //U+62AD
0, //U+62AE
0, //U+62AF
0, //U+62B0
0, //U+62B1
0, //U+62B2
0, //U+62B3
0, //U+62B4
0, //U+62B5
0, //U+62B6
0, //U+62B7
0, //U+62B8
0, //U+62B9
0, //U+62BA
0, //U+62BB
0, //U+62BC
0, //U+62BD
0, //U+62BE
0, //U+62BF
0, //U+62C0
0, //U+62C1
0, //U+62C2
0, //U+62C3
0, //U+62C4
0, //U+62C5
0, //U+62C6
0, //U+62C7
0, //U+62C8
0, //U+62C9
0, //U+62CA
0, //U+62CB
0, //U+62CC
0, //U+62CD
0, //U+62CE
0, //U+62CF
0, //U+62D0
0, //U+62D1
0, //U+62D2
0, //U+62D3
0, //U+62D4
0, //U+62D5
0, //U+62D6
0, //U+62D7
0, //U+62D8
0, //U+62D9
0, //U+62DA
0, //U+62DB
0, //U+62DC
0, //U+62DD
0, //U+62DE
0, //U+62DF
0, //U+62E0
0, //U+62E1
0, //U+62E2
0, //U+62E3
0, //U+62E4
0, //U+62E5
0, //U+62E6
0, //U+62E7
0, //U+62E8
0, //U+62E9
0, //U+62EA
0, //U+62EB
0, //U+62EC
0, //U+62ED
0, //U+62EE
0, //U+62EF
0, //U+62F0
0, //U+62F1
0, //U+62F2
0, //U+62F3
0, //U+62F4
0, //U+62F5
0, //U+62F6
0, //U+62F7
0, //U+62F8
0, //U+62F9
0, //U+62FA
0, //U+62FB
0, //U+62FC
0, //U+62FD
0, //U+62FE
0, //U+62FF
0, //U+6300
0, //U+6301
0, //U+6302
0, //U+6303
0, //U+6304
0, //U+6305
0, //U+6306
0, //U+6307
0, //U+6308
0, //U+6309
0, //U+630A
0, //U+630B
0, //U+630C
0, //U+630D
0, //U+630E
0, //U+630F
0, //U+6310
0, //U+6311
0, //U+6312
0, //U+6313
0, //U+6314
0, //U+6315
0, //U+6316
0, //U+6317
0, //U+6318
0, //U+6319
0, //U+631A
0, //U+631B
0, //U+631C
0, //U+631D
0, //U+631E
0, //U+631F
0, //U+6320
0, //U+6321
0, //U+6322
0, //U+6323
0, //U+6324
0, //U+6325
0, //U+6326
0, //U+6327
0, //U+6328
0, //U+6329
0, //U+632A
0, //U+632B
0, //U+632C
0, //U+632D
0, //U+632E
0, //U+632F
0, //U+6330
0, //U+6331
0, //U+6332
0, //U+6333
0, //U+6334
0, //U+6335
0, //U+6336
0, //U+6337
0, //U+6338
0, //U+6339
0, //U+633A
0, //U+633B
0, //U+633C
0, //U+633D
0, //U+633E
0, //U+633F
0, //U+6340
0, //U+6341
0, //U+6342
0, //U+6343
0, //U+6344
0, //U+6345
0, //U+6346
0, //U+6347
0, //U+6348
0, //U+6349
0, //U+634A
0, //U+634B
0, //U+634C
0, //U+634D
0, //U+634E
0, //U+634F
0, //U+6350
0, //U+6351
0, //U+6352
0, //U+6353
0, //U+6354
0, //U+6355
0, //U+6356
0, //U+6357
0, //U+6358
0, //U+6359
0, //U+635A
0, //U+635B
0, //U+635C
0, //U+635D
0, //U+635E
0, //U+635F
0, //U+6360
0, //U+6361
0, //U+6362
0, //U+6363
0, //U+6364
0, //U+6365
0, //U+6366
0, //U+6367
0, //U+6368
0, //U+6369
0, //U+636A
0, //U+636B
0, //U+636C
0, //U+636D
0, //U+636E
0, //U+636F
0, //U+6370
0, //U+6371
0, //U+6372
0, //U+6373
0, //U+6374
0, //U+6375
0, //U+6376
0, //U+6377
0, //U+6378
0, //U+6379
0, //U+637A
0, //U+637B
0, //U+637C
0, //U+637D
0, //U+637E
0, //U+637F
0, //U+6380
0, //U+6381
0, //U+6382
0, //U+6383
0, //U+6384
0, //U+6385
0, //U+6386
0, //U+6387
0, //U+6388
0, //U+6389
0, //U+638A
0, //U+638B
0, //U+638C
0, //U+638D
0, //U+638E
0, //U+638F
0, //U+6390
0, //U+6391
0, //U+6392
0, //U+6393
0, //U+6394
0, //U+6395
0, //U+6396
0, //U+6397
0, //U+6398
0, //U+6399
0, //U+639A
0, //U+639B
0, //U+639C
0, //U+639D
0, //U+639E
0, //U+639F
0, //U+63A0
0, //U+63A1
0, //U+63A2
0, //U+63A3
0, //U+63A4
0, //U+63A5
0, //U+63A6
0, //U+63A7
0, //U+63A8
0, //U+63A9
0, //U+63AA
0, //U+63AB
0, //U+63AC
0, //U+63AD
0, //U+63AE
0, //U+63AF
0, //U+63B0
0, //U+63B1
0, //U+63B2
0, //U+63B3
0, //U+63B4
0, //U+63B5
0, //U+63B6
0, //U+63B7
0, //U+63B8
0, //U+63B9
0, //U+63BA
0, //U+63BB
0, //U+63BC
0, //U+63BD
0, //U+63BE
0, //U+63BF
0, //U+63C0
0, //U+63C1
0, //U+63C2
0, //U+63C3
0, //U+63C4
0, //U+63C5
0, //U+63C6
0, //U+63C7
0, //U+63C8
0, //U+63C9
0, //U+63CA
0, //U+63CB
0, //U+63CC
0, //U+63CD
0, //U+63CE
0, //U+63CF
0, //U+63D0
0, //U+63D1
0, //U+63D2
0, //U+63D3
0, //U+63D4
0, //U+63D5
0, //U+63D6
0, //U+63D7
0, //U+63D8
0, //U+63D9
0, //U+63DA
0, //U+63DB
0, //U+63DC
0, //U+63DD
0, //U+63DE
0, //U+63DF
0, //U+63E0
0, //U+63E1
0, //U+63E2
0, //U+63E3
0, //U+63E4
0, //U+63E5
0, //U+63E6
0, //U+63E7
0, //U+63E8
0, //U+63E9
0, //U+63EA
0, //U+63EB
0, //U+63EC
0, //U+63ED
0, //U+63EE
0, //U+63EF
0, //U+63F0
0, //U+63F1
0, //U+63F2
0, //U+63F3
0, //U+63F4
0, //U+63F5
0, //U+63F6
0, //U+63F7
0, //U+63F8
0, //U+63F9
0, //U+63FA
0, //U+63FB
0, //U+63FC
0, //U+63FD
0, //U+63FE
0, //U+63FF
0, //U+6400
0, //U+6401
0, //U+6402
0, //U+6403
0, //U+6404
0, //U+6405
0, //U+6406
0, //U+6407
0, //U+6408
0, //U+6409
0, //U+640A
0, //U+640B
0, //U+640C
0, //U+640D
0, //U+640E
0, //U+640F
0, //U+6410
0, //U+6411
0, //U+6412
0, //U+6413
0, //U+6414
0, //U+6415
0, //U+6416
0, //U+6417
0, //U+6418
0, //U+6419
0, //U+641A
0, //U+641B
0, //U+641C
0, //U+641D
0, //U+641E
0, //U+641F
0, //U+6420
0, //U+6421
0, //U+6422
0, //U+6423
0, //U+6424
0, //U+6425
0, //U+6426
0, //U+6427
0, //U+6428
0, //U+6429
0, //U+642A
0, //U+642B
0, //U+642C
0, //U+642D
0, //U+642E
0, //U+642F
0, //U+6430
0, //U+6431
0, //U+6432
0, //U+6433
0, //U+6434
0, //U+6435
0, //U+6436
0, //U+6437
0, //U+6438
0, //U+6439
0, //U+643A
0, //U+643B
0, //U+643C
0, //U+643D
0, //U+643E
0, //U+643F
0, //U+6440
0, //U+6441
0, //U+6442
0, //U+6443
0, //U+6444
0, //U+6445
0, //U+6446
0, //U+6447
0, //U+6448
0, //U+6449
0, //U+644A
0, //U+644B
0, //U+644C
0, //U+644D
0, //U+644E
0, //U+644F
0, //U+6450
0, //U+6451
0, //U+6452
0, //U+6453
0, //U+6454
0, //U+6455
0, //U+6456
0, //U+6457
0, //U+6458
0, //U+6459
0, //U+645A
0, //U+645B
0, //U+645C
0, //U+645D
0, //U+645E
0, //U+645F
0, //U+6460
0, //U+6461
0, //U+6462
0, //U+6463
0, //U+6464
0, //U+6465
0, //U+6466
0, //U+6467
0, //U+6468
0, //U+6469
0, //U+646A
0, //U+646B
0, //U+646C
0, //U+646D
0, //U+646E
0, //U+646F
0, //U+6470
0, //U+6471
0, //U+6472
0, //U+6473
0, //U+6474
0, //U+6475
0, //U+6476
0, //U+6477
0, //U+6478
0, //U+6479
0, //U+647A
0, //U+647B
0, //U+647C
0, //U+647D
0, //U+647E
0, //U+647F
0, //U+6480
0, //U+6481
0, //U+6482
0, //U+6483
0, //U+6484
0, //U+6485
0, //U+6486
0, //U+6487
0, //U+6488
0, //U+6489
0, //U+648A
0, //U+648B
0, //U+648C
0, //U+648D
0, //U+648E
0, //U+648F
0, //U+6490
0, //U+6491
0, //U+6492
0, //U+6493
0, //U+6494
0, //U+6495
0, //U+6496
0, //U+6497
0, //U+6498
0, //U+6499
0, //U+649A
0, //U+649B
0, //U+649C
0, //U+649D
0, //U+649E
0, //U+649F
0, //U+64A0
0, //U+64A1
0, //U+64A2
0, //U+64A3
0, //U+64A4
0, //U+64A5
0, //U+64A6
0, //U+64A7
0, //U+64A8
0, //U+64A9
0, //U+64AA
0, //U+64AB
0, //U+64AC
0, //U+64AD
0, //U+64AE
0, //U+64AF
0, //U+64B0
0, //U+64B1
0, //U+64B2
0, //U+64B3
0, //U+64B4
0, //U+64B5
0, //U+64B6
0, //U+64B7
0, //U+64B8
0, //U+64B9
0, //U+64BA
0, //U+64BB
0, //U+64BC
0, //U+64BD
0, //U+64BE
0, //U+64BF
0, //U+64C0
0, //U+64C1
0, //U+64C2
0, //U+64C3
0, //U+64C4
0, //U+64C5
0, //U+64C6
0, //U+64C7
0, //U+64C8
0, //U+64C9
0, //U+64CA
0, //U+64CB
0, //U+64CC
0, //U+64CD
0, //U+64CE
0, //U+64CF
0, //U+64D0
0, //U+64D1
0, //U+64D2
0, //U+64D3
0, //U+64D4
0, //U+64D5
0, //U+64D6
0, //U+64D7
0, //U+64D8
0, //U+64D9
0, //U+64DA
0, //U+64DB
0, //U+64DC
0, //U+64DD
0, //U+64DE
0, //U+64DF
0, //U+64E0
0, //U+64E1
0, //U+64E2
0, //U+64E3
0, //U+64E4
0, //U+64E5
0, //U+64E6
0, //U+64E7
0, //U+64E8
0, //U+64E9
0, //U+64EA
0, //U+64EB
0, //U+64EC
0, //U+64ED
0, //U+64EE
0, //U+64EF
0, //U+64F0
0, //U+64F1
0, //U+64F2
0, //U+64F3
0, //U+64F4
0, //U+64F5
0, //U+64F6
0, //U+64F7
0, //U+64F8
0, //U+64F9
0, //U+64FA
0, //U+64FB
0, //U+64FC
0, //U+64FD
0, //U+64FE
0, //U+64FF
0, //U+6500
0, //U+6501
0, //U+6502
0, //U+6503
0, //U+6504
0, //U+6505
0, //U+6506
0, //U+6507
0, //U+6508
0, //U+6509
0, //U+650A
0, //U+650B
0, //U+650C
0, //U+650D
0, //U+650E
0, //U+650F
0, //U+6510
0, //U+6511
0, //U+6512
0, //U+6513
0, //U+6514
0, //U+6515
0, //U+6516
0, //U+6517
0, //U+6518
0, //U+6519
0, //U+651A
0, //U+651B
0, //U+651C
0, //U+651D
0, //U+651E
0, //U+651F
0, //U+6520
0, //U+6521
0, //U+6522
0, //U+6523
0, //U+6524
0, //U+6525
0, //U+6526
0, //U+6527
0, //U+6528
0, //U+6529
0, //U+652A
0, //U+652B
0, //U+652C
0, //U+652D
0, //U+652E
0, //U+652F
0, //U+6530
0, //U+6531
0, //U+6532
0, //U+6533
0, //U+6534
0, //U+6535
0, //U+6536
0, //U+6537
0, //U+6538
0, //U+6539
0, //U+653A
0, //U+653B
0, //U+653C
0, //U+653D
0, //U+653E
0, //U+653F
0, //U+6540
0, //U+6541
0, //U+6542
0, //U+6543
0, //U+6544
0, //U+6545
0, //U+6546
0, //U+6547
0, //U+6548
0, //U+6549
0, //U+654A
0, //U+654B
0, //U+654C
0, //U+654D
0, //U+654E
0, //U+654F
0, //U+6550
0, //U+6551
0, //U+6552
0, //U+6553
0, //U+6554
0, //U+6555
0, //U+6556
0, //U+6557
0, //U+6558
0, //U+6559
0, //U+655A
0, //U+655B
0, //U+655C
0, //U+655D
0, //U+655E
0, //U+655F
0, //U+6560
0, //U+6561
0, //U+6562
0, //U+6563
0, //U+6564
0, //U+6565
0, //U+6566
0, //U+6567
0, //U+6568
0, //U+6569
0, //U+656A
0, //U+656B
0, //U+656C
0, //U+656D
0, //U+656E
0, //U+656F
0, //U+6570
0, //U+6571
0, //U+6572
0, //U+6573
0, //U+6574
0, //U+6575
0, //U+6576
0, //U+6577
0, //U+6578
0, //U+6579
0, //U+657A
0, //U+657B
0, //U+657C
0, //U+657D
0, //U+657E
0, //U+657F
0, //U+6580
0, //U+6581
0, //U+6582
0, //U+6583
0, //U+6584
0, //U+6585
0, //U+6586
0, //U+6587
0, //U+6588
0, //U+6589
0, //U+658A
0, //U+658B
0, //U+658C
0, //U+658D
0, //U+658E
0, //U+658F
0, //U+6590
0, //U+6591
0, //U+6592
0, //U+6593
0, //U+6594
0, //U+6595
0, //U+6596
0, //U+6597
0, //U+6598
0, //U+6599
0, //U+659A
0, //U+659B
0, //U+659C
0, //U+659D
0, //U+659E
0, //U+659F
0, //U+65A0
0, //U+65A1
0, //U+65A2
0, //U+65A3
0, //U+65A4
0, //U+65A5
0, //U+65A6
0, //U+65A7
0, //U+65A8
0, //U+65A9
0, //U+65AA
0, //U+65AB
0, //U+65AC
0, //U+65AD
0, //U+65AE
0, //U+65AF
0, //U+65B0
0, //U+65B1
0, //U+65B2
0, //U+65B3
0, //U+65B4
0, //U+65B5
0, //U+65B6
0, //U+65B7
0, //U+65B8
0, //U+65B9
0, //U+65BA
0, //U+65BB
0, //U+65BC
0, //U+65BD
0, //U+65BE
0, //U+65BF
0, //U+65C0
0, //U+65C1
0, //U+65C2
0, //U+65C3
0, //U+65C4
0, //U+65C5
0, //U+65C6
0, //U+65C7
0, //U+65C8
0, //U+65C9
0, //U+65CA
0, //U+65CB
0, //U+65CC
0, //U+65CD
0, //U+65CE
0, //U+65CF
0, //U+65D0
0, //U+65D1
0, //U+65D2
0, //U+65D3
0, //U+65D4
0, //U+65D5
0, //U+65D6
0, //U+65D7
0, //U+65D8
0, //U+65D9
0, //U+65DA
0, //U+65DB
0, //U+65DC
0, //U+65DD
0, //U+65DE
0, //U+65DF
0, //U+65E0
0, //U+65E1
0, //U+65E2
0, //U+65E3
0, //U+65E4
0, //U+65E5
0, //U+65E6
0, //U+65E7
0, //U+65E8
0, //U+65E9
0, //U+65EA
0, //U+65EB
0, //U+65EC
0, //U+65ED
0, //U+65EE
0, //U+65EF
0, //U+65F0
0, //U+65F1
0, //U+65F2
0, //U+65F3
0, //U+65F4
0, //U+65F5
0, //U+65F6
0, //U+65F7
0, //U+65F8
0, //U+65F9
0, //U+65FA
0, //U+65FB
0, //U+65FC
0, //U+65FD
0, //U+65FE
0, //U+65FF
0, //U+6600
0, //U+6601
0, //U+6602
0, //U+6603
0, //U+6604
0, //U+6605
0, //U+6606
0, //U+6607
0, //U+6608
0, //U+6609
0, //U+660A
0, //U+660B
0, //U+660C
0, //U+660D
0, //U+660E
0, //U+660F
0, //U+6610
0, //U+6611
0, //U+6612
0, //U+6613
0, //U+6614
0, //U+6615
0, //U+6616
0, //U+6617
0, //U+6618
0, //U+6619
0, //U+661A
0, //U+661B
0, //U+661C
0, //U+661D
0, //U+661E
0, //U+661F
0, //U+6620
0, //U+6621
0, //U+6622
0, //U+6623
0, //U+6624
0, //U+6625
0, //U+6626
0, //U+6627
0, //U+6628
0, //U+6629
0, //U+662A
0, //U+662B
0, //U+662C
0, //U+662D
0, //U+662E
0, //U+662F
0, //U+6630
0, //U+6631
0, //U+6632
0, //U+6633
0, //U+6634
0, //U+6635
0, //U+6636
0, //U+6637
0, //U+6638
0, //U+6639
0, //U+663A
0, //U+663B
0, //U+663C
0, //U+663D
0, //U+663E
0, //U+663F
0, //U+6640
0, //U+6641
0, //U+6642
0, //U+6643
0, //U+6644
0, //U+6645
0, //U+6646
0, //U+6647
0, //U+6648
0, //U+6649
0, //U+664A
0, //U+664B
0, //U+664C
0, //U+664D
0, //U+664E
0, //U+664F
0, //U+6650
0, //U+6651
0, //U+6652
0, //U+6653
0, //U+6654
0, //U+6655
0, //U+6656
0, //U+6657
0, //U+6658
0, //U+6659
0, //U+665A
0, //U+665B
0, //U+665C
0, //U+665D
0, //U+665E
0, //U+665F
0, //U+6660
0, //U+6661
0, //U+6662
0, //U+6663
0, //U+6664
0, //U+6665
0, //U+6666
0, //U+6667
0, //U+6668
0, //U+6669
0, //U+666A
0, //U+666B
0, //U+666C
0, //U+666D
0, //U+666E
0, //U+666F
0, //U+6670
0, //U+6671
0, //U+6672
0, //U+6673
0, //U+6674
0, //U+6675
0, //U+6676
0, //U+6677
0, //U+6678
0, //U+6679
0, //U+667A
0, //U+667B
0, //U+667C
0, //U+667D
0, //U+667E
0, //U+667F
0, //U+6680
0, //U+6681
0, //U+6682
0, //U+6683
0, //U+6684
0, //U+6685
0, //U+6686
0, //U+6687
0, //U+6688
0, //U+6689
0, //U+668A
0, //U+668B
0, //U+668C
0, //U+668D
0, //U+668E
0, //U+668F
0, //U+6690
0, //U+6691
0, //U+6692
0, //U+6693
0, //U+6694
0, //U+6695
0, //U+6696
0, //U+6697
0, //U+6698
0, //U+6699
0, //U+669A
0, //U+669B
0, //U+669C
0, //U+669D
0, //U+669E
0, //U+669F
0, //U+66A0
0, //U+66A1
0, //U+66A2
0, //U+66A3
0, //U+66A4
0, //U+66A5
0, //U+66A6
0, //U+66A7
0, //U+66A8
0, //U+66A9
0, //U+66AA
0, //U+66AB
0, //U+66AC
0, //U+66AD
0, //U+66AE
0, //U+66AF
0, //U+66B0
0, //U+66B1
0, //U+66B2
0, //U+66B3
0, //U+66B4
0, //U+66B5
0, //U+66B6
0, //U+66B7
0, //U+66B8
0, //U+66B9
0, //U+66BA
0, //U+66BB
0, //U+66BC
0, //U+66BD
0, //U+66BE
0, //U+66BF
0, //U+66C0
0, //U+66C1
0, //U+66C2
0, //U+66C3
0, //U+66C4
0, //U+66C5
0, //U+66C6
0, //U+66C7
0, //U+66C8
0, //U+66C9
0, //U+66CA
0, //U+66CB
0, //U+66CC
0, //U+66CD
0, //U+66CE
0, //U+66CF
0, //U+66D0
0, //U+66D1
0, //U+66D2
0, //U+66D3
0, //U+66D4
0, //U+66D5
0, //U+66D6
0, //U+66D7
0, //U+66D8
0, //U+66D9
0, //U+66DA
0, //U+66DB
0, //U+66DC
0, //U+66DD
0, //U+66DE
0, //U+66DF
0, //U+66E0
0, //U+66E1
0, //U+66E2
0, //U+66E3
0, //U+66E4
0, //U+66E5
0, //U+66E6
0, //U+66E7
0, //U+66E8
0, //U+66E9
0, //U+66EA
0, //U+66EB
0, //U+66EC
0, //U+66ED
0, //U+66EE
0, //U+66EF
0, //U+66F0
0, //U+66F1
0, //U+66F2
0, //U+66F3
0, //U+66F4
0, //U+66F5
0, //U+66F6
0, //U+66F7
0, //U+66F8
0, //U+66F9
0, //U+66FA
0, //U+66FB
0, //U+66FC
0, //U+66FD
0, //U+66FE
0, //U+66FF
0, //U+6700
0, //U+6701
0, //U+6702
0, //U+6703
0, //U+6704
0, //U+6705
0, //U+6706
0, //U+6707
0, //U+6708
0, //U+6709
0, //U+670A
0, //U+670B
0, //U+670C
0, //U+670D
0, //U+670E
0, //U+670F
0, //U+6710
0, //U+6711
0, //U+6712
0, //U+6713
0, //U+6714
0, //U+6715
0, //U+6716
0, //U+6717
0, //U+6718
0, //U+6719
0, //U+671A
0, //U+671B
0, //U+671C
0, //U+671D
0, //U+671E
0, //U+671F
0, //U+6720
0, //U+6721
0, //U+6722
0, //U+6723
0, //U+6724
0, //U+6725
0, //U+6726
0, //U+6727
0, //U+6728
0, //U+6729
0, //U+672A
0, //U+672B
0, //U+672C
0, //U+672D
0, //U+672E
0, //U+672F
0, //U+6730
0, //U+6731
0, //U+6732
0, //U+6733
0, //U+6734
0, //U+6735
0, //U+6736
0, //U+6737
0, //U+6738
0, //U+6739
0, //U+673A
0, //U+673B
0, //U+673C
0, //U+673D
0, //U+673E
0, //U+673F
0, //U+6740
0, //U+6741
0, //U+6742
0, //U+6743
0, //U+6744
0, //U+6745
0, //U+6746
0, //U+6747
0, //U+6748
0, //U+6749
0, //U+674A
0, //U+674B
0, //U+674C
0, //U+674D
0, //U+674E
0, //U+674F
0, //U+6750
0, //U+6751
0, //U+6752
0, //U+6753
0, //U+6754
0, //U+6755
0, //U+6756
0, //U+6757
0, //U+6758
0, //U+6759
0, //U+675A
0, //U+675B
0, //U+675C
0, //U+675D
0, //U+675E
0, //U+675F
0, //U+6760
0, //U+6761
0, //U+6762
0, //U+6763
0, //U+6764
0, //U+6765
0, //U+6766
0, //U+6767
0, //U+6768
0, //U+6769
0, //U+676A
0, //U+676B
0, //U+676C
0, //U+676D
0, //U+676E
0, //U+676F
0, //U+6770
0, //U+6771
0, //U+6772
0, //U+6773
0, //U+6774
0, //U+6775
0, //U+6776
0, //U+6777
0, //U+6778
0, //U+6779
0, //U+677A
0, //U+677B
0, //U+677C
0, //U+677D
0, //U+677E
0, //U+677F
0, //U+6780
0, //U+6781
0, //U+6782
0, //U+6783
0, //U+6784
0, //U+6785
0, //U+6786
0, //U+6787
0, //U+6788
0, //U+6789
0, //U+678A
0, //U+678B
0, //U+678C
0, //U+678D
0, //U+678E
0, //U+678F
0, //U+6790
0, //U+6791
0, //U+6792
0, //U+6793
0, //U+6794
0, //U+6795
0, //U+6796
0, //U+6797
0, //U+6798
0, //U+6799
0, //U+679A
0, //U+679B
0, //U+679C
0, //U+679D
0, //U+679E
0, //U+679F
0, //U+67A0
0, //U+67A1
0, //U+67A2
0, //U+67A3
0, //U+67A4
0, //U+67A5
0, //U+67A6
0, //U+67A7
0, //U+67A8
0, //U+67A9
0, //U+67AA
0, //U+67AB
0, //U+67AC
0, //U+67AD
0, //U+67AE
0, //U+67AF
0, //U+67B0
0, //U+67B1
0, //U+67B2
0, //U+67B3
0, //U+67B4
0, //U+67B5
0, //U+67B6
0, //U+67B7
0, //U+67B8
0, //U+67B9
0, //U+67BA
0, //U+67BB
0, //U+67BC
0, //U+67BD
0, //U+67BE
0, //U+67BF
0, //U+67C0
0, //U+67C1
0, //U+67C2
0, //U+67C3
0, //U+67C4
0, //U+67C5
0, //U+67C6
0, //U+67C7
0, //U+67C8
0, //U+67C9
0, //U+67CA
0, //U+67CB
0, //U+67CC
0, //U+67CD
0, //U+67CE
0, //U+67CF
0, //U+67D0
0, //U+67D1
0, //U+67D2
0, //U+67D3
0, //U+67D4
0, //U+67D5
0, //U+67D6
0, //U+67D7
0, //U+67D8
0, //U+67D9
0, //U+67DA
0, //U+67DB
0, //U+67DC
0, //U+67DD
0, //U+67DE
0, //U+67DF
0, //U+67E0
0, //U+67E1
0, //U+67E2
0, //U+67E3
0, //U+67E4
0, //U+67E5
0, //U+67E6
0, //U+67E7
0, //U+67E8
0, //U+67E9
0, //U+67EA
0, //U+67EB
0, //U+67EC
0, //U+67ED
0, //U+67EE
0, //U+67EF
0, //U+67F0
0, //U+67F1
0, //U+67F2
0, //U+67F3
0, //U+67F4
0, //U+67F5
0, //U+67F6
0, //U+67F7
0, //U+67F8
0, //U+67F9
0, //U+67FA
0, //U+67FB
0, //U+67FC
0, //U+67FD
0, //U+67FE
0, //U+67FF
0, //U+6800
0, //U+6801
0, //U+6802
0, //U+6803
0, //U+6804
0, //U+6805
0, //U+6806
0, //U+6807
0, //U+6808
0, //U+6809
0, //U+680A
0, //U+680B
0, //U+680C
0, //U+680D
0, //U+680E
0, //U+680F
0, //U+6810
0, //U+6811
0, //U+6812
0, //U+6813
0, //U+6814
0, //U+6815
0, //U+6816
0, //U+6817
0, //U+6818
0, //U+6819
0, //U+681A
0, //U+681B
0, //U+681C
0, //U+681D
0, //U+681E
0, //U+681F
0, //U+6820
0, //U+6821
0, //U+6822
0, //U+6823
0, //U+6824
0, //U+6825
0, //U+6826
0, //U+6827
0, //U+6828
0, //U+6829
0, //U+682A
0, //U+682B
0, //U+682C
0, //U+682D
0, //U+682E
0, //U+682F
0, //U+6830
0, //U+6831
0, //U+6832
0, //U+6833
0, //U+6834
0, //U+6835
0, //U+6836
0, //U+6837
0, //U+6838
0, //U+6839
0, //U+683A
0, //U+683B
0, //U+683C
0, //U+683D
0, //U+683E
0, //U+683F
0, //U+6840
0, //U+6841
0, //U+6842
0, //U+6843
0, //U+6844
0, //U+6845
0, //U+6846
0, //U+6847
0, //U+6848
0, //U+6849
0, //U+684A
0, //U+684B
0, //U+684C
0, //U+684D
0, //U+684E
0, //U+684F
0, //U+6850
0, //U+6851
0, //U+6852
0, //U+6853
0, //U+6854
0, //U+6855
0, //U+6856
0, //U+6857
0, //U+6858
0, //U+6859
0, //U+685A
0, //U+685B
0, //U+685C
0, //U+685D
0, //U+685E
0, //U+685F
0, //U+6860
0, //U+6861
0, //U+6862
0, //U+6863
0, //U+6864
0, //U+6865
0, //U+6866
0, //U+6867
0, //U+6868
0, //U+6869
0, //U+686A
0, //U+686B
0, //U+686C
0, //U+686D
0, //U+686E
0, //U+686F
0, //U+6870
0, //U+6871
0, //U+6872
0, //U+6873
0, //U+6874
0, //U+6875
0, //U+6876
0, //U+6877
0, //U+6878
0, //U+6879
0, //U+687A
0, //U+687B
0, //U+687C
0, //U+687D
0, //U+687E
0, //U+687F
0, //U+6880
0, //U+6881
0, //U+6882
0, //U+6883
0, //U+6884
0, //U+6885
0, //U+6886
0, //U+6887
0, //U+6888
0, //U+6889
0, //U+688A
0, //U+688B
0, //U+688C
0, //U+688D
0, //U+688E
0, //U+688F
0, //U+6890
0, //U+6891
0, //U+6892
0, //U+6893
0, //U+6894
0, //U+6895
0, //U+6896
0, //U+6897
0, //U+6898
0, //U+6899
0, //U+689A
0, //U+689B
0, //U+689C
0, //U+689D
0, //U+689E
0, //U+689F
0, //U+68A0
0, //U+68A1
0, //U+68A2
0, //U+68A3
0, //U+68A4
0, //U+68A5
0, //U+68A6
0, //U+68A7
0, //U+68A8
0, //U+68A9
0, //U+68AA
0, //U+68AB
0, //U+68AC
0, //U+68AD
0, //U+68AE
0, //U+68AF
0, //U+68B0
0, //U+68B1
0, //U+68B2
0, //U+68B3
0, //U+68B4
0, //U+68B5
0, //U+68B6
0, //U+68B7
0, //U+68B8
0, //U+68B9
0, //U+68BA
0, //U+68BB
0, //U+68BC
0, //U+68BD
0, //U+68BE
0, //U+68BF
0, //U+68C0
0, //U+68C1
0, //U+68C2
0, //U+68C3
0, //U+68C4
0, //U+68C5
0, //U+68C6
0, //U+68C7
0, //U+68C8
0, //U+68C9
0, //U+68CA
0, //U+68CB
0, //U+68CC
0, //U+68CD
0, //U+68CE
0, //U+68CF
0, //U+68D0
0, //U+68D1
0, //U+68D2
0, //U+68D3
0, //U+68D4
0, //U+68D5
0, //U+68D6
0, //U+68D7
0, //U+68D8
0, //U+68D9
0, //U+68DA
0, //U+68DB
0, //U+68DC
0, //U+68DD
0, //U+68DE
0, //U+68DF
0, //U+68E0
0, //U+68E1
0, //U+68E2
0, //U+68E3
0, //U+68E4
0, //U+68E5
0, //U+68E6
0, //U+68E7
0, //U+68E8
0, //U+68E9
0, //U+68EA
0, //U+68EB
0, //U+68EC
0, //U+68ED
0, //U+68EE
0, //U+68EF
0, //U+68F0
0, //U+68F1
0, //U+68F2
0, //U+68F3
0, //U+68F4
0, //U+68F5
0, //U+68F6
0, //U+68F7
0, //U+68F8
0, //U+68F9
0, //U+68FA
0, //U+68FB
0, //U+68FC
0, //U+68FD
0, //U+68FE
0, //U+68FF
0, //U+6900
0, //U+6901
0, //U+6902
0, //U+6903
0, //U+6904
0, //U+6905
0, //U+6906
0, //U+6907
0, //U+6908
0, //U+6909
0, //U+690A
0, //U+690B
0, //U+690C
0, //U+690D
0, //U+690E
0, //U+690F
0, //U+6910
0, //U+6911
0, //U+6912
0, //U+6913
0, //U+6914
0, //U+6915
0, //U+6916
0, //U+6917
0, //U+6918
0, //U+6919
0, //U+691A
0, //U+691B
0, //U+691C
0, //U+691D
0, //U+691E
0, //U+691F
0, //U+6920
0, //U+6921
0, //U+6922
0, //U+6923
0, //U+6924
0, //U+6925
0, //U+6926
0, //U+6927
0, //U+6928
0, //U+6929
0, //U+692A
0, //U+692B
0, //U+692C
0, //U+692D
0, //U+692E
0, //U+692F
0, //U+6930
0, //U+6931
0, //U+6932
0, //U+6933
0, //U+6934
0, //U+6935
0, //U+6936
0, //U+6937
0, //U+6938
0, //U+6939
0, //U+693A
0, //U+693B
0, //U+693C
0, //U+693D
0, //U+693E
0, //U+693F
0, //U+6940
0, //U+6941
0, //U+6942
0, //U+6943
0, //U+6944
0, //U+6945
0, //U+6946
0, //U+6947
0, //U+6948
0, //U+6949
0, //U+694A
0, //U+694B
0, //U+694C
0, //U+694D
0, //U+694E
0, //U+694F
0, //U+6950
0, //U+6951
0, //U+6952
0, //U+6953
0, //U+6954
0, //U+6955
0, //U+6956
0, //U+6957
0, //U+6958
0, //U+6959
0, //U+695A
0, //U+695B
0, //U+695C
0, //U+695D
0, //U+695E
0, //U+695F
0, //U+6960
0, //U+6961
0, //U+6962
0, //U+6963
0, //U+6964
0, //U+6965
0, //U+6966
0, //U+6967
0, //U+6968
0, //U+6969
0, //U+696A
0, //U+696B
0, //U+696C
0, //U+696D
0, //U+696E
0, //U+696F
0, //U+6970
0, //U+6971
0, //U+6972
0, //U+6973
0, //U+6974
0, //U+6975
0, //U+6976
0, //U+6977
0, //U+6978
0, //U+6979
0, //U+697A
0, //U+697B
0, //U+697C
0, //U+697D
0, //U+697E
0, //U+697F
0, //U+6980
0, //U+6981
0, //U+6982
0, //U+6983
0, //U+6984
0, //U+6985
0, //U+6986
0, //U+6987
0, //U+6988
0, //U+6989
0, //U+698A
0, //U+698B
0, //U+698C
0, //U+698D
0, //U+698E
0, //U+698F
0, //U+6990
0, //U+6991
0, //U+6992
0, //U+6993
0, //U+6994
0, //U+6995
0, //U+6996
0, //U+6997
0, //U+6998
0, //U+6999
0, //U+699A
0, //U+699B
0, //U+699C
0, //U+699D
0, //U+699E
0, //U+699F
0, //U+69A0
0, //U+69A1
0, //U+69A2
0, //U+69A3
0, //U+69A4
0, //U+69A5
0, //U+69A6
0, //U+69A7
0, //U+69A8
0, //U+69A9
0, //U+69AA
0, //U+69AB
0, //U+69AC
0, //U+69AD
0, //U+69AE
0, //U+69AF
0, //U+69B0
0, //U+69B1
0, //U+69B2
0, //U+69B3
0, //U+69B4
0, //U+69B5
0, //U+69B6
0, //U+69B7
0, //U+69B8
0, //U+69B9
0, //U+69BA
0, //U+69BB
0, //U+69BC
0, //U+69BD
0, //U+69BE
0, //U+69BF
0, //U+69C0
0, //U+69C1
0, //U+69C2
0, //U+69C3
0, //U+69C4
0, //U+69C5
0, //U+69C6
0, //U+69C7
0, //U+69C8
0, //U+69C9
0, //U+69CA
0, //U+69CB
0, //U+69CC
0, //U+69CD
0, //U+69CE
0, //U+69CF
0, //U+69D0
0, //U+69D1
0, //U+69D2
0, //U+69D3
0, //U+69D4
0, //U+69D5
0, //U+69D6
0, //U+69D7
0, //U+69D8
0, //U+69D9
0, //U+69DA
0, //U+69DB
0, //U+69DC
0, //U+69DD
0, //U+69DE
0, //U+69DF
0, //U+69E0
0, //U+69E1
0, //U+69E2
0, //U+69E3
0, //U+69E4
0, //U+69E5
0, //U+69E6
0, //U+69E7
0, //U+69E8
0, //U+69E9
0, //U+69EA
0, //U+69EB
0, //U+69EC
0, //U+69ED
0, //U+69EE
0, //U+69EF
0, //U+69F0
0, //U+69F1
0, //U+69F2
0, //U+69F3
0, //U+69F4
0, //U+69F5
0, //U+69F6
0, //U+69F7
0, //U+69F8
0, //U+69F9
0, //U+69FA
0, //U+69FB
0, //U+69FC
0, //U+69FD
0, //U+69FE
0, //U+69FF
0, //U+6A00
0, //U+6A01
0, //U+6A02
0, //U+6A03
0, //U+6A04
0, //U+6A05
0, //U+6A06
0, //U+6A07
0, //U+6A08
0, //U+6A09
0, //U+6A0A
0, //U+6A0B
0, //U+6A0C
0, //U+6A0D
0, //U+6A0E
0, //U+6A0F
0, //U+6A10
0, //U+6A11
0, //U+6A12
0, //U+6A13
0, //U+6A14
0, //U+6A15
0, //U+6A16
0, //U+6A17
0, //U+6A18
0, //U+6A19
0, //U+6A1A
0, //U+6A1B
0, //U+6A1C
0, //U+6A1D
0, //U+6A1E
0, //U+6A1F
0, //U+6A20
0, //U+6A21
0, //U+6A22
0, //U+6A23
0, //U+6A24
0, //U+6A25
0, //U+6A26
0, //U+6A27
0, //U+6A28
0, //U+6A29
0, //U+6A2A
0, //U+6A2B
0, //U+6A2C
0, //U+6A2D
0, //U+6A2E
0, //U+6A2F
0, //U+6A30
0, //U+6A31
0, //U+6A32
0, //U+6A33
0, //U+6A34
0, //U+6A35
0, //U+6A36
0, //U+6A37
0, //U+6A38
0, //U+6A39
0, //U+6A3A
0, //U+6A3B
0, //U+6A3C
0, //U+6A3D
0, //U+6A3E
0, //U+6A3F
0, //U+6A40
0, //U+6A41
0, //U+6A42
0, //U+6A43
0, //U+6A44
0, //U+6A45
0, //U+6A46
0, //U+6A47
0, //U+6A48
0, //U+6A49
0, //U+6A4A
0, //U+6A4B
0, //U+6A4C
0, //U+6A4D
0, //U+6A4E
0, //U+6A4F
0, //U+6A50
0, //U+6A51
0, //U+6A52
0, //U+6A53
0, //U+6A54
0, //U+6A55
0, //U+6A56
0, //U+6A57
0, //U+6A58
0, //U+6A59
0, //U+6A5A
0, //U+6A5B
0, //U+6A5C
0, //U+6A5D
0, //U+6A5E
0, //U+6A5F
0, //U+6A60
0, //U+6A61
0, //U+6A62
0, //U+6A63
0, //U+6A64
0, //U+6A65
0, //U+6A66
0, //U+6A67
0, //U+6A68
0, //U+6A69
0, //U+6A6A
0, //U+6A6B
0, //U+6A6C
0, //U+6A6D
0, //U+6A6E
0, //U+6A6F
0, //U+6A70
0, //U+6A71
0, //U+6A72
0, //U+6A73
0, //U+6A74
0, //U+6A75
0, //U+6A76
0, //U+6A77
0, //U+6A78
0, //U+6A79
0, //U+6A7A
0, //U+6A7B
0, //U+6A7C
0, //U+6A7D
0, //U+6A7E
0, //U+6A7F
0, //U+6A80
0, //U+6A81
0, //U+6A82
0, //U+6A83
0, //U+6A84
0, //U+6A85
0, //U+6A86
0, //U+6A87
0, //U+6A88
0, //U+6A89
0, //U+6A8A
0, //U+6A8B
0, //U+6A8C
0, //U+6A8D
0, //U+6A8E
0, //U+6A8F
0, //U+6A90
0, //U+6A91
0, //U+6A92
0, //U+6A93
0, //U+6A94
0, //U+6A95
0, //U+6A96
0, //U+6A97
0, //U+6A98
0, //U+6A99
0, //U+6A9A
0, //U+6A9B
0, //U+6A9C
0, //U+6A9D
0, //U+6A9E
0, //U+6A9F
0, //U+6AA0
0, //U+6AA1
0, //U+6AA2
0, //U+6AA3
0, //U+6AA4
0, //U+6AA5
0, //U+6AA6
0, //U+6AA7
0, //U+6AA8
0, //U+6AA9
0, //U+6AAA
0, //U+6AAB
0, //U+6AAC
0, //U+6AAD
0, //U+6AAE
0, //U+6AAF
0, //U+6AB0
0, //U+6AB1
0, //U+6AB2
0, //U+6AB3
0, //U+6AB4
0, //U+6AB5
0, //U+6AB6
0, //U+6AB7
0, //U+6AB8
0, //U+6AB9
0, //U+6ABA
0, //U+6ABB
0, //U+6ABC
0, //U+6ABD
0, //U+6ABE
0, //U+6ABF
0, //U+6AC0
0, //U+6AC1
0, //U+6AC2
0, //U+6AC3
0, //U+6AC4
0, //U+6AC5
0, //U+6AC6
0, //U+6AC7
0, //U+6AC8
0, //U+6AC9
0, //U+6ACA
0, //U+6ACB
0, //U+6ACC
0, //U+6ACD
0, //U+6ACE
0, //U+6ACF
0, //U+6AD0
0, //U+6AD1
0, //U+6AD2
0, //U+6AD3
0, //U+6AD4
0, //U+6AD5
0, //U+6AD6
0, //U+6AD7
0, //U+6AD8
0, //U+6AD9
0, //U+6ADA
0, //U+6ADB
0, //U+6ADC
0, //U+6ADD
0, //U+6ADE
0, //U+6ADF
0, //U+6AE0
0, //U+6AE1
0, //U+6AE2
0, //U+6AE3
0, //U+6AE4
0, //U+6AE5
0, //U+6AE6
0, //U+6AE7
0, //U+6AE8
0, //U+6AE9
0, //U+6AEA
0, //U+6AEB
0, //U+6AEC
0, //U+6AED
0, //U+6AEE
0, //U+6AEF
0, //U+6AF0
0, //U+6AF1
0, //U+6AF2
0, //U+6AF3
0, //U+6AF4
0, //U+6AF5
0, //U+6AF6
0, //U+6AF7
0, //U+6AF8
0, //U+6AF9
0, //U+6AFA
0, //U+6AFB
0, //U+6AFC
0, //U+6AFD
0, //U+6AFE
0, //U+6AFF
0, //U+6B00
0, //U+6B01
0, //U+6B02
0, //U+6B03
0, //U+6B04
0, //U+6B05
0, //U+6B06
0, //U+6B07
0, //U+6B08
0, //U+6B09
0, //U+6B0A
0, //U+6B0B
0, //U+6B0C
0, //U+6B0D
0, //U+6B0E
0, //U+6B0F
0, //U+6B10
0, //U+6B11
0, //U+6B12
0, //U+6B13
0, //U+6B14
0, //U+6B15
0, //U+6B16
0, //U+6B17
0, //U+6B18
0, //U+6B19
0, //U+6B1A
0, //U+6B1B
0, //U+6B1C
0, //U+6B1D
0, //U+6B1E
0, //U+6B1F
0, //U+6B20
0, //U+6B21
0, //U+6B22
0, //U+6B23
0, //U+6B24
0, //U+6B25
0, //U+6B26
0, //U+6B27
0, //U+6B28
0, //U+6B29
0, //U+6B2A
0, //U+6B2B
0, //U+6B2C
0, //U+6B2D
0, //U+6B2E
0, //U+6B2F
0, //U+6B30
0, //U+6B31
0, //U+6B32
0, //U+6B33
0, //U+6B34
0, //U+6B35
0, //U+6B36
0, //U+6B37
0, //U+6B38
0, //U+6B39
0, //U+6B3A
0, //U+6B3B
0, //U+6B3C
0, //U+6B3D
0, //U+6B3E
0, //U+6B3F
0, //U+6B40
0, //U+6B41
0, //U+6B42
0, //U+6B43
0, //U+6B44
0, //U+6B45
0, //U+6B46
0, //U+6B47
0, //U+6B48
0, //U+6B49
0, //U+6B4A
0, //U+6B4B
0, //U+6B4C
0, //U+6B4D
0, //U+6B4E
0, //U+6B4F
0, //U+6B50
0, //U+6B51
0, //U+6B52
0, //U+6B53
0, //U+6B54
0, //U+6B55
0, //U+6B56
0, //U+6B57
0, //U+6B58
0, //U+6B59
0, //U+6B5A
0, //U+6B5B
0, //U+6B5C
0, //U+6B5D
0, //U+6B5E
0, //U+6B5F
0, //U+6B60
0, //U+6B61
0, //U+6B62
0, //U+6B63
0, //U+6B64
0, //U+6B65
0, //U+6B66
0, //U+6B67
0, //U+6B68
0, //U+6B69
0, //U+6B6A
0, //U+6B6B
0, //U+6B6C
0, //U+6B6D
0, //U+6B6E
0, //U+6B6F
0, //U+6B70
0, //U+6B71
0, //U+6B72
0, //U+6B73
0, //U+6B74
0, //U+6B75
0, //U+6B76
0, //U+6B77
0, //U+6B78
0, //U+6B79
0, //U+6B7A
0, //U+6B7B
0, //U+6B7C
0, //U+6B7D
0, //U+6B7E
0, //U+6B7F
0, //U+6B80
0, //U+6B81
0, //U+6B82
0, //U+6B83
0, //U+6B84
0, //U+6B85
0, //U+6B86
0, //U+6B87
0, //U+6B88
0, //U+6B89
0, //U+6B8A
0, //U+6B8B
0, //U+6B8C
0, //U+6B8D
0, //U+6B8E
0, //U+6B8F
0, //U+6B90
0, //U+6B91
0, //U+6B92
0, //U+6B93
0, //U+6B94
0, //U+6B95
0, //U+6B96
0, //U+6B97
0, //U+6B98
0, //U+6B99
0, //U+6B9A
0, //U+6B9B
0, //U+6B9C
0, //U+6B9D
0, //U+6B9E
0, //U+6B9F
0, //U+6BA0
0, //U+6BA1
0, //U+6BA2
0, //U+6BA3
0, //U+6BA4
0, //U+6BA5
0, //U+6BA6
0, //U+6BA7
0, //U+6BA8
0, //U+6BA9
0, //U+6BAA
0, //U+6BAB
0, //U+6BAC
0, //U+6BAD
0, //U+6BAE
0, //U+6BAF
0, //U+6BB0
0, //U+6BB1
0, //U+6BB2
0, //U+6BB3
0, //U+6BB4
0, //U+6BB5
0, //U+6BB6
0, //U+6BB7
0, //U+6BB8
0, //U+6BB9
0, //U+6BBA
0, //U+6BBB
0, //U+6BBC
0, //U+6BBD
0, //U+6BBE
0, //U+6BBF
0, //U+6BC0
0, //U+6BC1
0, //U+6BC2
0, //U+6BC3
0, //U+6BC4
0, //U+6BC5
0, //U+6BC6
0, //U+6BC7
0, //U+6BC8
0, //U+6BC9
0, //U+6BCA
0, //U+6BCB
0, //U+6BCC
0, //U+6BCD
0, //U+6BCE
0, //U+6BCF
0, //U+6BD0
0, //U+6BD1
0, //U+6BD2
0, //U+6BD3
0, //U+6BD4
0, //U+6BD5
0, //U+6BD6
0, //U+6BD7
0, //U+6BD8
0, //U+6BD9
0, //U+6BDA
0, //U+6BDB
0, //U+6BDC
0, //U+6BDD
0, //U+6BDE
0, //U+6BDF
0, //U+6BE0
0, //U+6BE1
0, //U+6BE2
0, //U+6BE3
0, //U+6BE4
0, //U+6BE5
0, //U+6BE6
0, //U+6BE7
0, //U+6BE8
0, //U+6BE9
0, //U+6BEA
0, //U+6BEB
0, //U+6BEC
0, //U+6BED
0, //U+6BEE
0, //U+6BEF
0, //U+6BF0
0, //U+6BF1
0, //U+6BF2
0, //U+6BF3
0, //U+6BF4
0, //U+6BF5
0, //U+6BF6
0, //U+6BF7
0, //U+6BF8
0, //U+6BF9
0, //U+6BFA
0, //U+6BFB
0, //U+6BFC
0, //U+6BFD
0, //U+6BFE
0, //U+6BFF
0, //U+6C00
0, //U+6C01
0, //U+6C02
0, //U+6C03
0, //U+6C04
0, //U+6C05
0, //U+6C06
0, //U+6C07
0, //U+6C08
0, //U+6C09
0, //U+6C0A
0, //U+6C0B
0, //U+6C0C
0, //U+6C0D
0, //U+6C0E
0, //U+6C0F
0, //U+6C10
0, //U+6C11
0, //U+6C12
0, //U+6C13
0, //U+6C14
0, //U+6C15
0, //U+6C16
0, //U+6C17
0, //U+6C18
0, //U+6C19
0, //U+6C1A
0, //U+6C1B
0, //U+6C1C
0, //U+6C1D
0, //U+6C1E
0, //U+6C1F
0, //U+6C20
0, //U+6C21
0, //U+6C22
0, //U+6C23
0, //U+6C24
0, //U+6C25
0, //U+6C26
0, //U+6C27
0, //U+6C28
0, //U+6C29
0, //U+6C2A
0, //U+6C2B
0, //U+6C2C
0, //U+6C2D
0, //U+6C2E
0, //U+6C2F
0, //U+6C30
0, //U+6C31
0, //U+6C32
0, //U+6C33
0, //U+6C34
0, //U+6C35
0, //U+6C36
0, //U+6C37
0, //U+6C38
0, //U+6C39
0, //U+6C3A
0, //U+6C3B
0, //U+6C3C
0, //U+6C3D
0, //U+6C3E
0, //U+6C3F
0, //U+6C40
0, //U+6C41
0, //U+6C42
0, //U+6C43
0, //U+6C44
0, //U+6C45
0, //U+6C46
0, //U+6C47
0, //U+6C48
0, //U+6C49
0, //U+6C4A
0, //U+6C4B
0, //U+6C4C
0, //U+6C4D
0, //U+6C4E
0, //U+6C4F
0, //U+6C50
0, //U+6C51
0, //U+6C52
0, //U+6C53
0, //U+6C54
0, //U+6C55
0, //U+6C56
0, //U+6C57
0, //U+6C58
0, //U+6C59
0, //U+6C5A
0, //U+6C5B
0, //U+6C5C
0, //U+6C5D
0, //U+6C5E
0, //U+6C5F
0, //U+6C60
0, //U+6C61
0, //U+6C62
0, //U+6C63
0, //U+6C64
0, //U+6C65
0, //U+6C66
0, //U+6C67
0, //U+6C68
0, //U+6C69
0, //U+6C6A
0, //U+6C6B
0, //U+6C6C
0, //U+6C6D
0, //U+6C6E
0, //U+6C6F
0, //U+6C70
0, //U+6C71
0, //U+6C72
0, //U+6C73
0, //U+6C74
0, //U+6C75
0, //U+6C76
0, //U+6C77
0, //U+6C78
0, //U+6C79
0, //U+6C7A
0, //U+6C7B
0, //U+6C7C
0, //U+6C7D
0, //U+6C7E
0, //U+6C7F
0, //U+6C80
0, //U+6C81
0, //U+6C82
0, //U+6C83
0, //U+6C84
0, //U+6C85
0, //U+6C86
0, //U+6C87
0, //U+6C88
0, //U+6C89
0, //U+6C8A
0, //U+6C8B
0, //U+6C8C
0, //U+6C8D
0, //U+6C8E
0, //U+6C8F
0, //U+6C90
0, //U+6C91
0, //U+6C92
0, //U+6C93
0, //U+6C94
0, //U+6C95
0, //U+6C96
0, //U+6C97
0, //U+6C98
0, //U+6C99
0, //U+6C9A
0, //U+6C9B
0, //U+6C9C
0, //U+6C9D
0, //U+6C9E
0, //U+6C9F
0, //U+6CA0
0, //U+6CA1
0, //U+6CA2
0, //U+6CA3
0, //U+6CA4
0, //U+6CA5
0, //U+6CA6
0, //U+6CA7
0, //U+6CA8
0, //U+6CA9
0, //U+6CAA
0, //U+6CAB
0, //U+6CAC
0, //U+6CAD
0, //U+6CAE
0, //U+6CAF
0, //U+6CB0
0, //U+6CB1
0, //U+6CB2
0, //U+6CB3
0, //U+6CB4
0, //U+6CB5
0, //U+6CB6
0, //U+6CB7
0, //U+6CB8
0, //U+6CB9
0, //U+6CBA
0, //U+6CBB
0, //U+6CBC
0, //U+6CBD
0, //U+6CBE
0, //U+6CBF
0, //U+6CC0
0, //U+6CC1
0, //U+6CC2
0, //U+6CC3
0, //U+6CC4
0, //U+6CC5
0, //U+6CC6
0, //U+6CC7
0, //U+6CC8
0, //U+6CC9
0, //U+6CCA
0, //U+6CCB
0, //U+6CCC
0, //U+6CCD
0, //U+6CCE
0, //U+6CCF
0, //U+6CD0
0, //U+6CD1
0, //U+6CD2
0, //U+6CD3
0, //U+6CD4
0, //U+6CD5
0, //U+6CD6
0, //U+6CD7
0, //U+6CD8
0, //U+6CD9
0, //U+6CDA
0, //U+6CDB
0, //U+6CDC
0, //U+6CDD
0, //U+6CDE
0, //U+6CDF
0, //U+6CE0
0, //U+6CE1
0, //U+6CE2
0, //U+6CE3
0, //U+6CE4
0, //U+6CE5
0, //U+6CE6
0, //U+6CE7
0, //U+6CE8
0, //U+6CE9
0, //U+6CEA
0, //U+6CEB
0, //U+6CEC
0, //U+6CED
0, //U+6CEE
0, //U+6CEF
0, //U+6CF0
0, //U+6CF1
0, //U+6CF2
0, //U+6CF3
0, //U+6CF4
0, //U+6CF5
0, //U+6CF6
0, //U+6CF7
0, //U+6CF8
0, //U+6CF9
0, //U+6CFA
0, //U+6CFB
0, //U+6CFC
0, //U+6CFD
0, //U+6CFE
0, //U+6CFF
0, //U+6D00
0, //U+6D01
0, //U+6D02
0, //U+6D03
0, //U+6D04
0, //U+6D05
0, //U+6D06
0, //U+6D07
0, //U+6D08
0, //U+6D09
0, //U+6D0A
0, //U+6D0B
0, //U+6D0C
0, //U+6D0D
0, //U+6D0E
0, //U+6D0F
0, //U+6D10
0, //U+6D11
0, //U+6D12
0, //U+6D13
0, //U+6D14
0, //U+6D15
0, //U+6D16
0, //U+6D17
0, //U+6D18
0, //U+6D19
0, //U+6D1A
0, //U+6D1B
0, //U+6D1C
0, //U+6D1D
0, //U+6D1E
0, //U+6D1F
0, //U+6D20
0, //U+6D21
0, //U+6D22
0, //U+6D23
0, //U+6D24
0, //U+6D25
0, //U+6D26
0, //U+6D27
0, //U+6D28
0, //U+6D29
0, //U+6D2A
0, //U+6D2B
0, //U+6D2C
0, //U+6D2D
0, //U+6D2E
0, //U+6D2F
0, //U+6D30
0, //U+6D31
0, //U+6D32
0, //U+6D33
0, //U+6D34
0, //U+6D35
0, //U+6D36
0, //U+6D37
0, //U+6D38
0, //U+6D39
0, //U+6D3A
0, //U+6D3B
0, //U+6D3C
0, //U+6D3D
0, //U+6D3E
0, //U+6D3F
0, //U+6D40
0, //U+6D41
0, //U+6D42
0, //U+6D43
0, //U+6D44
0, //U+6D45
0, //U+6D46
0, //U+6D47
0, //U+6D48
0, //U+6D49
0, //U+6D4A
0, //U+6D4B
0, //U+6D4C
0, //U+6D4D
0, //U+6D4E
0, //U+6D4F
0, //U+6D50
0, //U+6D51
0, //U+6D52
0, //U+6D53
0, //U+6D54
0, //U+6D55
0, //U+6D56
0, //U+6D57
0, //U+6D58
0, //U+6D59
0, //U+6D5A
0, //U+6D5B
0, //U+6D5C
0, //U+6D5D
0, //U+6D5E
0, //U+6D5F
0, //U+6D60
0, //U+6D61
0, //U+6D62
0, //U+6D63
0, //U+6D64
0, //U+6D65
0, //U+6D66
0, //U+6D67
0, //U+6D68
0, //U+6D69
0, //U+6D6A
0, //U+6D6B
0, //U+6D6C
0, //U+6D6D
0, //U+6D6E
0, //U+6D6F
0, //U+6D70
0, //U+6D71
0, //U+6D72
0, //U+6D73
0, //U+6D74
0, //U+6D75
0, //U+6D76
0, //U+6D77
0, //U+6D78
0, //U+6D79
0, //U+6D7A
0, //U+6D7B
0, //U+6D7C
0, //U+6D7D
0, //U+6D7E
0, //U+6D7F
0, //U+6D80
0, //U+6D81
0, //U+6D82
0, //U+6D83
0, //U+6D84
0, //U+6D85
0, //U+6D86
0, //U+6D87
0, //U+6D88
0, //U+6D89
0, //U+6D8A
0, //U+6D8B
0, //U+6D8C
0, //U+6D8D
0, //U+6D8E
0, //U+6D8F
0, //U+6D90
0, //U+6D91
0, //U+6D92
0, //U+6D93
0, //U+6D94
0, //U+6D95
0, //U+6D96
0, //U+6D97
0, //U+6D98
0, //U+6D99
0, //U+6D9A
0, //U+6D9B
0, //U+6D9C
0, //U+6D9D
0, //U+6D9E
0, //U+6D9F
0, //U+6DA0
0, //U+6DA1
0, //U+6DA2
0, //U+6DA3
0, //U+6DA4
0, //U+6DA5
0, //U+6DA6
0, //U+6DA7
0, //U+6DA8
0, //U+6DA9
0, //U+6DAA
0, //U+6DAB
0, //U+6DAC
0, //U+6DAD
0, //U+6DAE
0, //U+6DAF
0, //U+6DB0
0, //U+6DB1
0, //U+6DB2
0, //U+6DB3
0, //U+6DB4
0, //U+6DB5
0, //U+6DB6
0, //U+6DB7
0, //U+6DB8
0, //U+6DB9
0, //U+6DBA
0, //U+6DBB
0, //U+6DBC
0, //U+6DBD
0, //U+6DBE
0, //U+6DBF
0, //U+6DC0
0, //U+6DC1
0, //U+6DC2
0, //U+6DC3
0, //U+6DC4
0, //U+6DC5
0, //U+6DC6
0, //U+6DC7
0, //U+6DC8
0, //U+6DC9
0, //U+6DCA
0, //U+6DCB
0, //U+6DCC
0, //U+6DCD
0, //U+6DCE
0, //U+6DCF
0, //U+6DD0
0, //U+6DD1
0, //U+6DD2
0, //U+6DD3
0, //U+6DD4
0, //U+6DD5
0, //U+6DD6
0, //U+6DD7
0, //U+6DD8
0, //U+6DD9
0, //U+6DDA
0, //U+6DDB
0, //U+6DDC
0, //U+6DDD
0, //U+6DDE
0, //U+6DDF
0, //U+6DE0
0, //U+6DE1
0, //U+6DE2
0, //U+6DE3
0, //U+6DE4
0, //U+6DE5
0, //U+6DE6
0, //U+6DE7
0, //U+6DE8
0, //U+6DE9
0, //U+6DEA
0, //U+6DEB
0, //U+6DEC
0, //U+6DED
0, //U+6DEE
0, //U+6DEF
0, //U+6DF0
0, //U+6DF1
0, //U+6DF2
0, //U+6DF3
0, //U+6DF4
0, //U+6DF5
0, //U+6DF6
0, //U+6DF7
0, //U+6DF8
0, //U+6DF9
0, //U+6DFA
0, //U+6DFB
0, //U+6DFC
0, //U+6DFD
0, //U+6DFE
0, //U+6DFF
0, //U+6E00
0, //U+6E01
0, //U+6E02
0, //U+6E03
0, //U+6E04
0, //U+6E05
0, //U+6E06
0, //U+6E07
0, //U+6E08
0, //U+6E09
0, //U+6E0A
0, //U+6E0B
0, //U+6E0C
0, //U+6E0D
0, //U+6E0E
0, //U+6E0F
0, //U+6E10
0, //U+6E11
0, //U+6E12
0, //U+6E13
0, //U+6E14
0, //U+6E15
0, //U+6E16
0, //U+6E17
0, //U+6E18
0, //U+6E19
0, //U+6E1A
0, //U+6E1B
0, //U+6E1C
0, //U+6E1D
0, //U+6E1E
0, //U+6E1F
0, //U+6E20
0, //U+6E21
0, //U+6E22
0, //U+6E23
0, //U+6E24
0, //U+6E25
0, //U+6E26
0, //U+6E27
0, //U+6E28
0, //U+6E29
0, //U+6E2A
0, //U+6E2B
0, //U+6E2C
0, //U+6E2D
0, //U+6E2E
0, //U+6E2F
0, //U+6E30
0, //U+6E31
0, //U+6E32
0, //U+6E33
0, //U+6E34
0, //U+6E35
0, //U+6E36
0, //U+6E37
0, //U+6E38
0, //U+6E39
0, //U+6E3A
0, //U+6E3B
0, //U+6E3C
0, //U+6E3D
0, //U+6E3E
0, //U+6E3F
0, //U+6E40
0, //U+6E41
0, //U+6E42
0, //U+6E43
0, //U+6E44
0, //U+6E45
0, //U+6E46
0, //U+6E47
0, //U+6E48
0, //U+6E49
0, //U+6E4A
0, //U+6E4B
0, //U+6E4C
0, //U+6E4D
0, //U+6E4E
0, //U+6E4F
0, //U+6E50
0, //U+6E51
0, //U+6E52
0, //U+6E53
0, //U+6E54
0, //U+6E55
0, //U+6E56
0, //U+6E57
0, //U+6E58
0, //U+6E59
0, //U+6E5A
0, //U+6E5B
0, //U+6E5C
0, //U+6E5D
0, //U+6E5E
0, //U+6E5F
0, //U+6E60
0, //U+6E61
0, //U+6E62
0, //U+6E63
0, //U+6E64
0, //U+6E65
0, //U+6E66
0, //U+6E67
0, //U+6E68
0, //U+6E69
0, //U+6E6A
0, //U+6E6B
0, //U+6E6C
0, //U+6E6D
0, //U+6E6E
0, //U+6E6F
0, //U+6E70
0, //U+6E71
0, //U+6E72
0, //U+6E73
0, //U+6E74
0, //U+6E75
0, //U+6E76
0, //U+6E77
0, //U+6E78
0, //U+6E79
0, //U+6E7A
0, //U+6E7B
0, //U+6E7C
0, //U+6E7D
0, //U+6E7E
0, //U+6E7F
0, //U+6E80
0, //U+6E81
0, //U+6E82
0, //U+6E83
0, //U+6E84
0, //U+6E85
0, //U+6E86
0, //U+6E87
0, //U+6E88
0, //U+6E89
0, //U+6E8A
0, //U+6E8B
0, //U+6E8C
0, //U+6E8D
0, //U+6E8E
0, //U+6E8F
0, //U+6E90
0, //U+6E91
0, //U+6E92
0, //U+6E93
0, //U+6E94
0, //U+6E95
0, //U+6E96
0, //U+6E97
0, //U+6E98
0, //U+6E99
0, //U+6E9A
0, //U+6E9B
0, //U+6E9C
0, //U+6E9D
0, //U+6E9E
0, //U+6E9F
0, //U+6EA0
0, //U+6EA1
0, //U+6EA2
0, //U+6EA3
0, //U+6EA4
0, //U+6EA5
0, //U+6EA6
0, //U+6EA7
0, //U+6EA8
0, //U+6EA9
0, //U+6EAA
0, //U+6EAB
0, //U+6EAC
0, //U+6EAD
0, //U+6EAE
0, //U+6EAF
0, //U+6EB0
0, //U+6EB1
0, //U+6EB2
0, //U+6EB3
0, //U+6EB4
0, //U+6EB5
0, //U+6EB6
0, //U+6EB7
0, //U+6EB8
0, //U+6EB9
0, //U+6EBA
0, //U+6EBB
0, //U+6EBC
0, //U+6EBD
0, //U+6EBE
0, //U+6EBF
0, //U+6EC0
0, //U+6EC1
0, //U+6EC2
0, //U+6EC3
0, //U+6EC4
0, //U+6EC5
0, //U+6EC6
0, //U+6EC7
0, //U+6EC8
0, //U+6EC9
0, //U+6ECA
0, //U+6ECB
0, //U+6ECC
0, //U+6ECD
0, //U+6ECE
0, //U+6ECF
0, //U+6ED0
0, //U+6ED1
0, //U+6ED2
0, //U+6ED3
0, //U+6ED4
0, //U+6ED5
0, //U+6ED6
0, //U+6ED7
0, //U+6ED8
0, //U+6ED9
0, //U+6EDA
0, //U+6EDB
0, //U+6EDC
0, //U+6EDD
0, //U+6EDE
0, //U+6EDF
0, //U+6EE0
0, //U+6EE1
0, //U+6EE2
0, //U+6EE3
0, //U+6EE4
0, //U+6EE5
0, //U+6EE6
0, //U+6EE7
0, //U+6EE8
0, //U+6EE9
0, //U+6EEA
0, //U+6EEB
0, //U+6EEC
0, //U+6EED
0, //U+6EEE
0, //U+6EEF
0, //U+6EF0
0, //U+6EF1
0, //U+6EF2
0, //U+6EF3
0, //U+6EF4
0, //U+6EF5
0, //U+6EF6
0, //U+6EF7
0, //U+6EF8
0, //U+6EF9
0, //U+6EFA
0, //U+6EFB
0, //U+6EFC
0, //U+6EFD
0, //U+6EFE
0, //U+6EFF
0, //U+6F00
0, //U+6F01
0, //U+6F02
0, //U+6F03
0, //U+6F04
0, //U+6F05
0, //U+6F06
0, //U+6F07
0, //U+6F08
0, //U+6F09
0, //U+6F0A
0, //U+6F0B
0, //U+6F0C
0, //U+6F0D
0, //U+6F0E
0, //U+6F0F
0, //U+6F10
0, //U+6F11
0, //U+6F12
0, //U+6F13
0, //U+6F14
0, //U+6F15
0, //U+6F16
0, //U+6F17
0, //U+6F18
0, //U+6F19
0, //U+6F1A
0, //U+6F1B
0, //U+6F1C
0, //U+6F1D
0, //U+6F1E
0, //U+6F1F
0, //U+6F20
0, //U+6F21
0, //U+6F22
0, //U+6F23
0, //U+6F24
0, //U+6F25
0, //U+6F26
0, //U+6F27
0, //U+6F28
0, //U+6F29
0, //U+6F2A
0, //U+6F2B
0, //U+6F2C
0, //U+6F2D
0, //U+6F2E
0, //U+6F2F
0, //U+6F30
0, //U+6F31
0, //U+6F32
0, //U+6F33
0, //U+6F34
0, //U+6F35
0, //U+6F36
0, //U+6F37
0, //U+6F38
0, //U+6F39
0, //U+6F3A
0, //U+6F3B
0, //U+6F3C
0, //U+6F3D
0, //U+6F3E
0, //U+6F3F
0, //U+6F40
0, //U+6F41
0, //U+6F42
0, //U+6F43
0, //U+6F44
0, //U+6F45
0, //U+6F46
0, //U+6F47
0, //U+6F48
0, //U+6F49
0, //U+6F4A
0, //U+6F4B
0, //U+6F4C
0, //U+6F4D
0, //U+6F4E
0, //U+6F4F
0, //U+6F50
0, //U+6F51
0, //U+6F52
0, //U+6F53
0, //U+6F54
0, //U+6F55
0, //U+6F56
0, //U+6F57
0, //U+6F58
0, //U+6F59
0, //U+6F5A
0, //U+6F5B
0, //U+6F5C
0, //U+6F5D
0, //U+6F5E
0, //U+6F5F
0, //U+6F60
0, //U+6F61
0, //U+6F62
0, //U+6F63
0, //U+6F64
0, //U+6F65
0, //U+6F66
0, //U+6F67
0, //U+6F68
0, //U+6F69
0, //U+6F6A
0, //U+6F6B
0, //U+6F6C
0, //U+6F6D
0, //U+6F6E
0, //U+6F6F
0, //U+6F70
0, //U+6F71
0, //U+6F72
0, //U+6F73
0, //U+6F74
0, //U+6F75
0, //U+6F76
0, //U+6F77
0, //U+6F78
0, //U+6F79
0, //U+6F7A
0, //U+6F7B
0, //U+6F7C
0, //U+6F7D
0, //U+6F7E
0, //U+6F7F
0, //U+6F80
0, //U+6F81
0, //U+6F82
0, //U+6F83
0, //U+6F84
0, //U+6F85
0, //U+6F86
0, //U+6F87
0, //U+6F88
0, //U+6F89
0, //U+6F8A
0, //U+6F8B
0, //U+6F8C
0, //U+6F8D
0, //U+6F8E
0, //U+6F8F
0, //U+6F90
0, //U+6F91
0, //U+6F92
0, //U+6F93
0, //U+6F94
0, //U+6F95
0, //U+6F96
0, //U+6F97
0, //U+6F98
0, //U+6F99
0, //U+6F9A
0, //U+6F9B
0, //U+6F9C
0, //U+6F9D
0, //U+6F9E
0, //U+6F9F
0, //U+6FA0
0, //U+6FA1
0, //U+6FA2
0, //U+6FA3
0, //U+6FA4
0, //U+6FA5
0, //U+6FA6
0, //U+6FA7
0, //U+6FA8
0, //U+6FA9
0, //U+6FAA
0, //U+6FAB
0, //U+6FAC
0, //U+6FAD
0, //U+6FAE
0, //U+6FAF
0, //U+6FB0
0, //U+6FB1
0, //U+6FB2
0, //U+6FB3
0, //U+6FB4
0, //U+6FB5
0, //U+6FB6
0, //U+6FB7
0, //U+6FB8
0, //U+6FB9
0, //U+6FBA
0, //U+6FBB
0, //U+6FBC
0, //U+6FBD
0, //U+6FBE
0, //U+6FBF
0, //U+6FC0
0, //U+6FC1
0, //U+6FC2
0, //U+6FC3
0, //U+6FC4
0, //U+6FC5
0, //U+6FC6
0, //U+6FC7
0, //U+6FC8
0, //U+6FC9
0, //U+6FCA
0, //U+6FCB
0, //U+6FCC
0, //U+6FCD
0, //U+6FCE
0, //U+6FCF
0, //U+6FD0
0, //U+6FD1
0, //U+6FD2
0, //U+6FD3
0, //U+6FD4
0, //U+6FD5
0, //U+6FD6
0, //U+6FD7
0, //U+6FD8
0, //U+6FD9
0, //U+6FDA
0, //U+6FDB
0, //U+6FDC
0, //U+6FDD
0, //U+6FDE
0, //U+6FDF
0, //U+6FE0
0, //U+6FE1
0, //U+6FE2
0, //U+6FE3
0, //U+6FE4
0, //U+6FE5
0, //U+6FE6
0, //U+6FE7
0, //U+6FE8
0, //U+6FE9
0, //U+6FEA
0, //U+6FEB
0, //U+6FEC
0, //U+6FED
0, //U+6FEE
0, //U+6FEF
0, //U+6FF0
0, //U+6FF1
0, //U+6FF2
0, //U+6FF3
0, //U+6FF4
0, //U+6FF5
0, //U+6FF6
0, //U+6FF7
0, //U+6FF8
0, //U+6FF9
0, //U+6FFA
0, //U+6FFB
0, //U+6FFC
0, //U+6FFD
0, //U+6FFE
0, //U+6FFF
0, //U+7000
0, //U+7001
0, //U+7002
0, //U+7003
0, //U+7004
0, //U+7005
0, //U+7006
0, //U+7007
0, //U+7008
0, //U+7009
0, //U+700A
0, //U+700B
0, //U+700C
0, //U+700D
0, //U+700E
0, //U+700F
0, //U+7010
0, //U+7011
0, //U+7012
0, //U+7013
0, //U+7014
0, //U+7015
0, //U+7016
0, //U+7017
0, //U+7018
0, //U+7019
0, //U+701A
0, //U+701B
0, //U+701C
0, //U+701D
0, //U+701E
0, //U+701F
0, //U+7020
0, //U+7021
0, //U+7022
0, //U+7023
0, //U+7024
0, //U+7025
0, //U+7026
0, //U+7027
0, //U+7028
0, //U+7029
0, //U+702A
0, //U+702B
0, //U+702C
0, //U+702D
0, //U+702E
0, //U+702F
0, //U+7030
0, //U+7031
0, //U+7032
0, //U+7033
0, //U+7034
0, //U+7035
0, //U+7036
0, //U+7037
0, //U+7038
0, //U+7039
0, //U+703A
0, //U+703B
0, //U+703C
0, //U+703D
0, //U+703E
0, //U+703F
0, //U+7040
0, //U+7041
0, //U+7042
0, //U+7043
0, //U+7044
0, //U+7045
0, //U+7046
0, //U+7047
0, //U+7048
0, //U+7049
0, //U+704A
0, //U+704B
0, //U+704C
0, //U+704D
0, //U+704E
0, //U+704F
0, //U+7050
0, //U+7051
0, //U+7052
0, //U+7053
0, //U+7054
0, //U+7055
0, //U+7056
0, //U+7057
0, //U+7058
0, //U+7059
0, //U+705A
0, //U+705B
0, //U+705C
0, //U+705D
0, //U+705E
0, //U+705F
0, //U+7060
0, //U+7061
0, //U+7062
0, //U+7063
0, //U+7064
0, //U+7065
0, //U+7066
0, //U+7067
0, //U+7068
0, //U+7069
0, //U+706A
0, //U+706B
0, //U+706C
0, //U+706D
0, //U+706E
0, //U+706F
0, //U+7070
0, //U+7071
0, //U+7072
0, //U+7073
0, //U+7074
0, //U+7075
0, //U+7076
0, //U+7077
0, //U+7078
0, //U+7079
0, //U+707A
0, //U+707B
0, //U+707C
0, //U+707D
0, //U+707E
0, //U+707F
0, //U+7080
0, //U+7081
0, //U+7082
0, //U+7083
0, //U+7084
0, //U+7085
0, //U+7086
0, //U+7087
0, //U+7088
0, //U+7089
0, //U+708A
0, //U+708B
0, //U+708C
0, //U+708D
0, //U+708E
0, //U+708F
0, //U+7090
0, //U+7091
0, //U+7092
0, //U+7093
0, //U+7094
0, //U+7095
0, //U+7096
0, //U+7097
0, //U+7098
0, //U+7099
0, //U+709A
0, //U+709B
0, //U+709C
0, //U+709D
0, //U+709E
0, //U+709F
0, //U+70A0
0, //U+70A1
0, //U+70A2
0, //U+70A3
0, //U+70A4
0, //U+70A5
0, //U+70A6
0, //U+70A7
0, //U+70A8
0, //U+70A9
0, //U+70AA
0, //U+70AB
0, //U+70AC
0, //U+70AD
0, //U+70AE
0, //U+70AF
0, //U+70B0
0, //U+70B1
0, //U+70B2
0, //U+70B3
0, //U+70B4
0, //U+70B5
0, //U+70B6
0, //U+70B7
0, //U+70B8
0, //U+70B9
0, //U+70BA
0, //U+70BB
0, //U+70BC
0, //U+70BD
0, //U+70BE
0, //U+70BF
0, //U+70C0
0, //U+70C1
0, //U+70C2
0, //U+70C3
0, //U+70C4
0, //U+70C5
0, //U+70C6
0, //U+70C7
0, //U+70C8
0, //U+70C9
0, //U+70CA
0, //U+70CB
0, //U+70CC
0, //U+70CD
0, //U+70CE
0, //U+70CF
0, //U+70D0
0, //U+70D1
0, //U+70D2
0, //U+70D3
0, //U+70D4
0, //U+70D5
0, //U+70D6
0, //U+70D7
0, //U+70D8
0, //U+70D9
0, //U+70DA
0, //U+70DB
0, //U+70DC
0, //U+70DD
0, //U+70DE
0, //U+70DF
0, //U+70E0
0, //U+70E1
0, //U+70E2
0, //U+70E3
0, //U+70E4
0, //U+70E5
0, //U+70E6
0, //U+70E7
0, //U+70E8
0, //U+70E9
0, //U+70EA
0, //U+70EB
0, //U+70EC
0, //U+70ED
0, //U+70EE
0, //U+70EF
0, //U+70F0
0, //U+70F1
0, //U+70F2
0, //U+70F3
0, //U+70F4
0, //U+70F5
0, //U+70F6
0, //U+70F7
0, //U+70F8
0, //U+70F9
0, //U+70FA
0, //U+70FB
0, //U+70FC
0, //U+70FD
0, //U+70FE
0, //U+70FF
0, //U+7100
0, //U+7101
0, //U+7102
0, //U+7103
0, //U+7104
0, //U+7105
0, //U+7106
0, //U+7107
0, //U+7108
0, //U+7109
0, //U+710A
0, //U+710B
0, //U+710C
0, //U+710D
0, //U+710E
0, //U+710F
0, //U+7110
0, //U+7111
0, //U+7112
0, //U+7113
0, //U+7114
0, //U+7115
0, //U+7116
0, //U+7117
0, //U+7118
0, //U+7119
0, //U+711A
0, //U+711B
0, //U+711C
0, //U+711D
0, //U+711E
0, //U+711F
0, //U+7120
0, //U+7121
0, //U+7122
0, //U+7123
0, //U+7124
0, //U+7125
0, //U+7126
0, //U+7127
0, //U+7128
0, //U+7129
0, //U+712A
0, //U+712B
0, //U+712C
0, //U+712D
0, //U+712E
0, //U+712F
0, //U+7130
0, //U+7131
0, //U+7132
0, //U+7133
0, //U+7134
0, //U+7135
0, //U+7136
0, //U+7137
0, //U+7138
0, //U+7139
0, //U+713A
0, //U+713B
0, //U+713C
0, //U+713D
0, //U+713E
0, //U+713F
0, //U+7140
0, //U+7141
0, //U+7142
0, //U+7143
0, //U+7144
0, //U+7145
0, //U+7146
0, //U+7147
0, //U+7148
0, //U+7149
0, //U+714A
0, //U+714B
0, //U+714C
0, //U+714D
0, //U+714E
0, //U+714F
0, //U+7150
0, //U+7151
0, //U+7152
0, //U+7153
0, //U+7154
0, //U+7155
0, //U+7156
0, //U+7157
0, //U+7158
0, //U+7159
0, //U+715A
0, //U+715B
0, //U+715C
0, //U+715D
0, //U+715E
0, //U+715F
0, //U+7160
0, //U+7161
0, //U+7162
0, //U+7163
0, //U+7164
0, //U+7165
0, //U+7166
0, //U+7167
0, //U+7168
0, //U+7169
0, //U+716A
0, //U+716B
0, //U+716C
0, //U+716D
0, //U+716E
0, //U+716F
0, //U+7170
0, //U+7171
0, //U+7172
0, //U+7173
0, //U+7174
0, //U+7175
0, //U+7176
0, //U+7177
0, //U+7178
0, //U+7179
0, //U+717A
0, //U+717B
0, //U+717C
0, //U+717D
0, //U+717E
0, //U+717F
0, //U+7180
0, //U+7181
0, //U+7182
0, //U+7183
0, //U+7184
0, //U+7185
0, //U+7186
0, //U+7187
0, //U+7188
0, //U+7189
0, //U+718A
0, //U+718B
0, //U+718C
0, //U+718D
0, //U+718E
0, //U+718F
0, //U+7190
0, //U+7191
0, //U+7192
0, //U+7193
0, //U+7194
0, //U+7195
0, //U+7196
0, //U+7197
0, //U+7198
0, //U+7199
0, //U+719A
0, //U+719B
0, //U+719C
0, //U+719D
0, //U+719E
0, //U+719F
0, //U+71A0
0, //U+71A1
0, //U+71A2
0, //U+71A3
0, //U+71A4
0, //U+71A5
0, //U+71A6
0, //U+71A7
0, //U+71A8
0, //U+71A9
0, //U+71AA
0, //U+71AB
0, //U+71AC
0, //U+71AD
0, //U+71AE
0, //U+71AF
0, //U+71B0
0, //U+71B1
0, //U+71B2
0, //U+71B3
0, //U+71B4
0, //U+71B5
0, //U+71B6
0, //U+71B7
0, //U+71B8
0, //U+71B9
0, //U+71BA
0, //U+71BB
0, //U+71BC
0, //U+71BD
0, //U+71BE
0, //U+71BF
0, //U+71C0
0, //U+71C1
0, //U+71C2
0, //U+71C3
0, //U+71C4
0, //U+71C5
0, //U+71C6
0, //U+71C7
0, //U+71C8
0, //U+71C9
0, //U+71CA
0, //U+71CB
0, //U+71CC
0, //U+71CD
0, //U+71CE
0, //U+71CF
0, //U+71D0
0, //U+71D1
0, //U+71D2
0, //U+71D3
0, //U+71D4
0, //U+71D5
0, //U+71D6
0, //U+71D7
0, //U+71D8
0, //U+71D9
0, //U+71DA
0, //U+71DB
0, //U+71DC
0, //U+71DD
0, //U+71DE
0, //U+71DF
0, //U+71E0
0, //U+71E1
0, //U+71E2
0, //U+71E3
0, //U+71E4
0, //U+71E5
0, //U+71E6
0, //U+71E7
0, //U+71E8
0, //U+71E9
0, //U+71EA
0, //U+71EB
0, //U+71EC
0, //U+71ED
0, //U+71EE
0, //U+71EF
0, //U+71F0
0, //U+71F1
0, //U+71F2
0, //U+71F3
0, //U+71F4
0, //U+71F5
0, //U+71F6
0, //U+71F7
0, //U+71F8
0, //U+71F9
0, //U+71FA
0, //U+71FB
0, //U+71FC
0, //U+71FD
0, //U+71FE
0, //U+71FF
0, //U+7200
0, //U+7201
0, //U+7202
0, //U+7203
0, //U+7204
0, //U+7205
0, //U+7206
0, //U+7207
0, //U+7208
0, //U+7209
0, //U+720A
0, //U+720B
0, //U+720C
0, //U+720D
0, //U+720E
0, //U+720F
0, //U+7210
0, //U+7211
0, //U+7212
0, //U+7213
0, //U+7214
0, //U+7215
0, //U+7216
0, //U+7217
0, //U+7218
0, //U+7219
0, //U+721A
0, //U+721B
0, //U+721C
0, //U+721D
0, //U+721E
0, //U+721F
0, //U+7220
0, //U+7221
0, //U+7222
0, //U+7223
0, //U+7224
0, //U+7225
0, //U+7226
0, //U+7227
0, //U+7228
0, //U+7229
0, //U+722A
0, //U+722B
0, //U+722C
0, //U+722D
0, //U+722E
0, //U+722F
0, //U+7230
0, //U+7231
0, //U+7232
0, //U+7233
0, //U+7234
0, //U+7235
0, //U+7236
0, //U+7237
0, //U+7238
0, //U+7239
0, //U+723A
0, //U+723B
0, //U+723C
0, //U+723D
0, //U+723E
0, //U+723F
0, //U+7240
0, //U+7241
0, //U+7242
0, //U+7243
0, //U+7244
0, //U+7245
0, //U+7246
0, //U+7247
0, //U+7248
0, //U+7249
0, //U+724A
0, //U+724B
0, //U+724C
0, //U+724D
0, //U+724E
0, //U+724F
0, //U+7250
0, //U+7251
0, //U+7252
0, //U+7253
0, //U+7254
0, //U+7255
0, //U+7256
0, //U+7257
0, //U+7258
0, //U+7259
0, //U+725A
0, //U+725B
0, //U+725C
0, //U+725D
0, //U+725E
0, //U+725F
0, //U+7260
0, //U+7261
0, //U+7262
0, //U+7263
0, //U+7264
0, //U+7265
0, //U+7266
0, //U+7267
0, //U+7268
0, //U+7269
0, //U+726A
0, //U+726B
0, //U+726C
0, //U+726D
0, //U+726E
0, //U+726F
0, //U+7270
0, //U+7271
0, //U+7272
0, //U+7273
0, //U+7274
0, //U+7275
0, //U+7276
0, //U+7277
0, //U+7278
0, //U+7279
0, //U+727A
0, //U+727B
0, //U+727C
0, //U+727D
0, //U+727E
0, //U+727F
0, //U+7280
0, //U+7281
0, //U+7282
0, //U+7283
0, //U+7284
0, //U+7285
0, //U+7286
0, //U+7287
0, //U+7288
0, //U+7289
0, //U+728A
0, //U+728B
0, //U+728C
0, //U+728D
0, //U+728E
0, //U+728F
0, //U+7290
0, //U+7291
0, //U+7292
0, //U+7293
0, //U+7294
0, //U+7295
0, //U+7296
0, //U+7297
0, //U+7298
0, //U+7299
0, //U+729A
0, //U+729B
0, //U+729C
0, //U+729D
0, //U+729E
0, //U+729F
0, //U+72A0
0, //U+72A1
0, //U+72A2
0, //U+72A3
0, //U+72A4
0, //U+72A5
0, //U+72A6
0, //U+72A7
0, //U+72A8
0, //U+72A9
0, //U+72AA
0, //U+72AB
0, //U+72AC
0, //U+72AD
0, //U+72AE
0, //U+72AF
0, //U+72B0
0, //U+72B1
0, //U+72B2
0, //U+72B3
0, //U+72B4
0, //U+72B5
0, //U+72B6
0, //U+72B7
0, //U+72B8
0, //U+72B9
0, //U+72BA
0, //U+72BB
0, //U+72BC
0, //U+72BD
0, //U+72BE
0, //U+72BF
0, //U+72C0
0, //U+72C1
0, //U+72C2
0, //U+72C3
0, //U+72C4
0, //U+72C5
0, //U+72C6
0, //U+72C7
0, //U+72C8
0, //U+72C9
0, //U+72CA
0, //U+72CB
0, //U+72CC
0, //U+72CD
0, //U+72CE
0, //U+72CF
0, //U+72D0
0, //U+72D1
0, //U+72D2
0, //U+72D3
0, //U+72D4
0, //U+72D5
0, //U+72D6
0, //U+72D7
0, //U+72D8
0, //U+72D9
0, //U+72DA
0, //U+72DB
0, //U+72DC
0, //U+72DD
0, //U+72DE
0, //U+72DF
0, //U+72E0
0, //U+72E1
0, //U+72E2
0, //U+72E3
0, //U+72E4
0, //U+72E5
0, //U+72E6
0, //U+72E7
0, //U+72E8
0, //U+72E9
0, //U+72EA
0, //U+72EB
0, //U+72EC
0, //U+72ED
0, //U+72EE
0, //U+72EF
0, //U+72F0
0, //U+72F1
0, //U+72F2
0, //U+72F3
0, //U+72F4
0, //U+72F5
0, //U+72F6
0, //U+72F7
0, //U+72F8
0, //U+72F9
0, //U+72FA
0, //U+72FB
0, //U+72FC
0, //U+72FD
0, //U+72FE
0, //U+72FF
0, //U+7300
0, //U+7301
0, //U+7302
0, //U+7303
0, //U+7304
0, //U+7305
0, //U+7306
0, //U+7307
0, //U+7308
0, //U+7309
0, //U+730A
0, //U+730B
0, //U+730C
0, //U+730D
0, //U+730E
0, //U+730F
0, //U+7310
0, //U+7311
0, //U+7312
0, //U+7313
0, //U+7314
0, //U+7315
0, //U+7316
0, //U+7317
0, //U+7318
0, //U+7319
0, //U+731A
0, //U+731B
0, //U+731C
0, //U+731D
0, //U+731E
0, //U+731F
0, //U+7320
0, //U+7321
0, //U+7322
0, //U+7323
0, //U+7324
0, //U+7325
0, //U+7326
0, //U+7327
0, //U+7328
0, //U+7329
0, //U+732A
0, //U+732B
0, //U+732C
0, //U+732D
0, //U+732E
0, //U+732F
0, //U+7330
0, //U+7331
0, //U+7332
0, //U+7333
0, //U+7334
0, //U+7335
0, //U+7336
0, //U+7337
0, //U+7338
0, //U+7339
0, //U+733A
0, //U+733B
0, //U+733C
0, //U+733D
0, //U+733E
0, //U+733F
0, //U+7340
0, //U+7341
0, //U+7342
0, //U+7343
0, //U+7344
0, //U+7345
0, //U+7346
0, //U+7347
0, //U+7348
0, //U+7349
0, //U+734A
0, //U+734B
0, //U+734C
0, //U+734D
0, //U+734E
0, //U+734F
0, //U+7350
0, //U+7351
0, //U+7352
0, //U+7353
0, //U+7354
0, //U+7355
0, //U+7356
0, //U+7357
0, //U+7358
0, //U+7359
0, //U+735A
0, //U+735B
0, //U+735C
0, //U+735D
0, //U+735E
0, //U+735F
0, //U+7360
0, //U+7361
0, //U+7362
0, //U+7363
0, //U+7364
0, //U+7365
0, //U+7366
0, //U+7367
0, //U+7368
0, //U+7369
0, //U+736A
0, //U+736B
0, //U+736C
0, //U+736D
0, //U+736E
0, //U+736F
0, //U+7370
0, //U+7371
0, //U+7372
0, //U+7373
0, //U+7374
0, //U+7375
0, //U+7376
0, //U+7377
0, //U+7378
0, //U+7379
0, //U+737A
0, //U+737B
0, //U+737C
0, //U+737D
0, //U+737E
0, //U+737F
0, //U+7380
0, //U+7381
0, //U+7382
0, //U+7383
0, //U+7384
0, //U+7385
0, //U+7386
0, //U+7387
0, //U+7388
0, //U+7389
0, //U+738A
0, //U+738B
0, //U+738C
0, //U+738D
0, //U+738E
0, //U+738F
0, //U+7390
0, //U+7391
0, //U+7392
0, //U+7393
0, //U+7394
0, //U+7395
0, //U+7396
0, //U+7397
0, //U+7398
0, //U+7399
0, //U+739A
0, //U+739B
0, //U+739C
0, //U+739D
0, //U+739E
0, //U+739F
0, //U+73A0
0, //U+73A1
0, //U+73A2
0, //U+73A3
0, //U+73A4
0, //U+73A5
0, //U+73A6
0, //U+73A7
0, //U+73A8
0, //U+73A9
0, //U+73AA
0, //U+73AB
0, //U+73AC
0, //U+73AD
0, //U+73AE
0, //U+73AF
0, //U+73B0
0, //U+73B1
0, //U+73B2
0, //U+73B3
0, //U+73B4
0, //U+73B5
0, //U+73B6
0, //U+73B7
0, //U+73B8
0, //U+73B9
0, //U+73BA
0, //U+73BB
0, //U+73BC
0, //U+73BD
0, //U+73BE
0, //U+73BF
0, //U+73C0
0, //U+73C1
0, //U+73C2
0, //U+73C3
0, //U+73C4
0, //U+73C5
0, //U+73C6
0, //U+73C7
0, //U+73C8
0, //U+73C9
0, //U+73CA
0, //U+73CB
0, //U+73CC
0, //U+73CD
0, //U+73CE
0, //U+73CF
0, //U+73D0
0, //U+73D1
0, //U+73D2
0, //U+73D3
0, //U+73D4
0, //U+73D5
0, //U+73D6
0, //U+73D7
0, //U+73D8
0, //U+73D9
0, //U+73DA
0, //U+73DB
0, //U+73DC
0, //U+73DD
0, //U+73DE
0, //U+73DF
0, //U+73E0
0, //U+73E1
0, //U+73E2
0, //U+73E3
0, //U+73E4
0, //U+73E5
0, //U+73E6
0, //U+73E7
0, //U+73E8
0, //U+73E9
0, //U+73EA
0, //U+73EB
0, //U+73EC
0, //U+73ED
0, //U+73EE
0, //U+73EF
0, //U+73F0
0, //U+73F1
0, //U+73F2
0, //U+73F3
0, //U+73F4
0, //U+73F5
0, //U+73F6
0, //U+73F7
0, //U+73F8
0, //U+73F9
0, //U+73FA
0, //U+73FB
0, //U+73FC
0, //U+73FD
0, //U+73FE
0, //U+73FF
0, //U+7400
0, //U+7401
0, //U+7402
0, //U+7403
0, //U+7404
0, //U+7405
0, //U+7406
0, //U+7407
0, //U+7408
0, //U+7409
0, //U+740A
0, //U+740B
0, //U+740C
0, //U+740D
0, //U+740E
0, //U+740F
0, //U+7410
0, //U+7411
0, //U+7412
0, //U+7413
0, //U+7414
0, //U+7415
0, //U+7416
0, //U+7417
0, //U+7418
0, //U+7419
0, //U+741A
0, //U+741B
0, //U+741C
0, //U+741D
0, //U+741E
0, //U+741F
0, //U+7420
0, //U+7421
0, //U+7422
0, //U+7423
0, //U+7424
0, //U+7425
0, //U+7426
0, //U+7427
0, //U+7428
0, //U+7429
0, //U+742A
0, //U+742B
0, //U+742C
0, //U+742D
0, //U+742E
0, //U+742F
0, //U+7430
0, //U+7431
0, //U+7432
0, //U+7433
0, //U+7434
0, //U+7435
0, //U+7436
0, //U+7437
0, //U+7438
0, //U+7439
0, //U+743A
0, //U+743B
0, //U+743C
0, //U+743D
0, //U+743E
0, //U+743F
0, //U+7440
0, //U+7441
0, //U+7442
0, //U+7443
0, //U+7444
0, //U+7445
0, //U+7446
0, //U+7447
0, //U+7448
0, //U+7449
0, //U+744A
0, //U+744B
0, //U+744C
0, //U+744D
0, //U+744E
0, //U+744F
0, //U+7450
0, //U+7451
0, //U+7452
0, //U+7453
0, //U+7454
0, //U+7455
0, //U+7456
0, //U+7457
0, //U+7458
0, //U+7459
0, //U+745A
0, //U+745B
0, //U+745C
0, //U+745D
0, //U+745E
0, //U+745F
0, //U+7460
0, //U+7461
0, //U+7462
0, //U+7463
0, //U+7464
0, //U+7465
0, //U+7466
0, //U+7467
0, //U+7468
0, //U+7469
0, //U+746A
0, //U+746B
0, //U+746C
0, //U+746D
0, //U+746E
0, //U+746F
0, //U+7470
0, //U+7471
0, //U+7472
0, //U+7473
0, //U+7474
0, //U+7475
0, //U+7476
0, //U+7477
0, //U+7478
0, //U+7479
0, //U+747A
0, //U+747B
0, //U+747C
0, //U+747D
0, //U+747E
0, //U+747F
0, //U+7480
0, //U+7481
0, //U+7482
0, //U+7483
0, //U+7484
0, //U+7485
0, //U+7486
0, //U+7487
0, //U+7488
0, //U+7489
0, //U+748A
0, //U+748B
0, //U+748C
0, //U+748D
0, //U+748E
0, //U+748F
0, //U+7490
0, //U+7491
0, //U+7492
0, //U+7493
0, //U+7494
0, //U+7495
0, //U+7496
0, //U+7497
0, //U+7498
0, //U+7499
0, //U+749A
0, //U+749B
0, //U+749C
0, //U+749D
0, //U+749E
0, //U+749F
0, //U+74A0
0, //U+74A1
0, //U+74A2
0, //U+74A3
0, //U+74A4
0, //U+74A5
0, //U+74A6
0, //U+74A7
0, //U+74A8
0, //U+74A9
0, //U+74AA
0, //U+74AB
0, //U+74AC
0, //U+74AD
0, //U+74AE
0, //U+74AF
0, //U+74B0
0, //U+74B1
0, //U+74B2
0, //U+74B3
0, //U+74B4
0, //U+74B5
0, //U+74B6
0, //U+74B7
0, //U+74B8
0, //U+74B9
0, //U+74BA
0, //U+74BB
0, //U+74BC
0, //U+74BD
0, //U+74BE
0, //U+74BF
0, //U+74C0
0, //U+74C1
0, //U+74C2
0, //U+74C3
0, //U+74C4
0, //U+74C5
0, //U+74C6
0, //U+74C7
0, //U+74C8
0, //U+74C9
0, //U+74CA
0, //U+74CB
0, //U+74CC
0, //U+74CD
0, //U+74CE
0, //U+74CF
0, //U+74D0
0, //U+74D1
0, //U+74D2
0, //U+74D3
0, //U+74D4
0, //U+74D5
0, //U+74D6
0, //U+74D7
0, //U+74D8
0, //U+74D9
0, //U+74DA
0, //U+74DB
0, //U+74DC
0, //U+74DD
0, //U+74DE
0, //U+74DF
0, //U+74E0
0, //U+74E1
0, //U+74E2
0, //U+74E3
0, //U+74E4
0, //U+74E5
0, //U+74E6
0, //U+74E7
0, //U+74E8
0, //U+74E9
0, //U+74EA
0, //U+74EB
0, //U+74EC
0, //U+74ED
0, //U+74EE
0, //U+74EF
0, //U+74F0
0, //U+74F1
0, //U+74F2
0, //U+74F3
0, //U+74F4
0, //U+74F5
0, //U+74F6
0, //U+74F7
0, //U+74F8
0, //U+74F9
0, //U+74FA
0, //U+74FB
0, //U+74FC
0, //U+74FD
0, //U+74FE
0, //U+74FF
0, //U+7500
0, //U+7501
0, //U+7502
0, //U+7503
0, //U+7504
0, //U+7505
0, //U+7506
0, //U+7507
0, //U+7508
0, //U+7509
0, //U+750A
0, //U+750B
0, //U+750C
0, //U+750D
0, //U+750E
0, //U+750F
0, //U+7510
0, //U+7511
0, //U+7512
0, //U+7513
0, //U+7514
0, //U+7515
0, //U+7516
0, //U+7517
0, //U+7518
0, //U+7519
0, //U+751A
0, //U+751B
0, //U+751C
0, //U+751D
0, //U+751E
0, //U+751F
0, //U+7520
0, //U+7521
0, //U+7522
0, //U+7523
0, //U+7524
0, //U+7525
0, //U+7526
0, //U+7527
0, //U+7528
0, //U+7529
0, //U+752A
0, //U+752B
0, //U+752C
0, //U+752D
0, //U+752E
0, //U+752F
0, //U+7530
0, //U+7531
0, //U+7532
0, //U+7533
0, //U+7534
0, //U+7535
0, //U+7536
0, //U+7537
0, //U+7538
0, //U+7539
0, //U+753A
0, //U+753B
0, //U+753C
0, //U+753D
0, //U+753E
0, //U+753F
0, //U+7540
0, //U+7541
0, //U+7542
0, //U+7543
0, //U+7544
0, //U+7545
0, //U+7546
0, //U+7547
0, //U+7548
0, //U+7549
0, //U+754A
0, //U+754B
0, //U+754C
0, //U+754D
0, //U+754E
0, //U+754F
0, //U+7550
0, //U+7551
0, //U+7552
0, //U+7553
0, //U+7554
0, //U+7555
0, //U+7556
0, //U+7557
0, //U+7558
0, //U+7559
0, //U+755A
0, //U+755B
0, //U+755C
0, //U+755D
0, //U+755E
0, //U+755F
0, //U+7560
0, //U+7561
0, //U+7562
0, //U+7563
0, //U+7564
0, //U+7565
0, //U+7566
0, //U+7567
0, //U+7568
0, //U+7569
0, //U+756A
0, //U+756B
0, //U+756C
0, //U+756D
0, //U+756E
0, //U+756F
0, //U+7570
0, //U+7571
0, //U+7572
0, //U+7573
0, //U+7574
0, //U+7575
0, //U+7576
0, //U+7577
0, //U+7578
0, //U+7579
0, //U+757A
0, //U+757B
0, //U+757C
0, //U+757D
0, //U+757E
0, //U+757F
0, //U+7580
0, //U+7581
0, //U+7582
0, //U+7583
0, //U+7584
0, //U+7585
0, //U+7586
0, //U+7587
0, //U+7588
0, //U+7589
0, //U+758A
0, //U+758B
0, //U+758C
0, //U+758D
0, //U+758E
0, //U+758F
0, //U+7590
0, //U+7591
0, //U+7592
0, //U+7593
0, //U+7594
0, //U+7595
0, //U+7596
0, //U+7597
0, //U+7598
0, //U+7599
0, //U+759A
0, //U+759B
0, //U+759C
0, //U+759D
0, //U+759E
0, //U+759F
0, //U+75A0
0, //U+75A1
0, //U+75A2
0, //U+75A3
0, //U+75A4
0, //U+75A5
0, //U+75A6
0, //U+75A7
0, //U+75A8
0, //U+75A9
0, //U+75AA
0, //U+75AB
0, //U+75AC
0, //U+75AD
0, //U+75AE
0, //U+75AF
0, //U+75B0
0, //U+75B1
0, //U+75B2
0, //U+75B3
0, //U+75B4
0, //U+75B5
0, //U+75B6
0, //U+75B7
0, //U+75B8
0, //U+75B9
0, //U+75BA
0, //U+75BB
0, //U+75BC
0, //U+75BD
0, //U+75BE
0, //U+75BF
0, //U+75C0
0, //U+75C1
0, //U+75C2
0, //U+75C3
0, //U+75C4
0, //U+75C5
0, //U+75C6
0, //U+75C7
0, //U+75C8
0, //U+75C9
0, //U+75CA
0, //U+75CB
0, //U+75CC
0, //U+75CD
0, //U+75CE
0, //U+75CF
0, //U+75D0
0, //U+75D1
0, //U+75D2
0, //U+75D3
0, //U+75D4
0, //U+75D5
0, //U+75D6
0, //U+75D7
0, //U+75D8
0, //U+75D9
0, //U+75DA
0, //U+75DB
0, //U+75DC
0, //U+75DD
0, //U+75DE
0, //U+75DF
0, //U+75E0
0, //U+75E1
0, //U+75E2
0, //U+75E3
0, //U+75E4
0, //U+75E5
0, //U+75E6
0, //U+75E7
0, //U+75E8
0, //U+75E9
0, //U+75EA
0, //U+75EB
0, //U+75EC
0, //U+75ED
0, //U+75EE
0, //U+75EF
0, //U+75F0
0, //U+75F1
0, //U+75F2
0, //U+75F3
0, //U+75F4
0, //U+75F5
0, //U+75F6
0, //U+75F7
0, //U+75F8
0, //U+75F9
0, //U+75FA
0, //U+75FB
0, //U+75FC
0, //U+75FD
0, //U+75FE
0, //U+75FF
0, //U+7600
0, //U+7601
0, //U+7602
0, //U+7603
0, //U+7604
0, //U+7605
0, //U+7606
0, //U+7607
0, //U+7608
0, //U+7609
0, //U+760A
0, //U+760B
0, //U+760C
0, //U+760D
0, //U+760E
0, //U+760F
0, //U+7610
0, //U+7611
0, //U+7612
0, //U+7613
0, //U+7614
0, //U+7615
0, //U+7616
0, //U+7617
0, //U+7618
0, //U+7619
0, //U+761A
0, //U+761B
0, //U+761C
0, //U+761D
0, //U+761E
0, //U+761F
0, //U+7620
0, //U+7621
0, //U+7622
0, //U+7623
0, //U+7624
0, //U+7625
0, //U+7626
0, //U+7627
0, //U+7628
0, //U+7629
0, //U+762A
0, //U+762B
0, //U+762C
0, //U+762D
0, //U+762E
0, //U+762F
0, //U+7630
0, //U+7631
0, //U+7632
0, //U+7633
0, //U+7634
0, //U+7635
0, //U+7636
0, //U+7637
0, //U+7638
0, //U+7639
0, //U+763A
0, //U+763B
0, //U+763C
0, //U+763D
0, //U+763E
0, //U+763F
0, //U+7640
0, //U+7641
0, //U+7642
0, //U+7643
0, //U+7644
0, //U+7645
0, //U+7646
0, //U+7647
0, //U+7648
0, //U+7649
0, //U+764A
0, //U+764B
0, //U+764C
0, //U+764D
0, //U+764E
0, //U+764F
0, //U+7650
0, //U+7651
0, //U+7652
0, //U+7653
0, //U+7654
0, //U+7655
0, //U+7656
0, //U+7657
0, //U+7658
0, //U+7659
0, //U+765A
0, //U+765B
0, //U+765C
0, //U+765D
0, //U+765E
0, //U+765F
0, //U+7660
0, //U+7661
0, //U+7662
0, //U+7663
0, //U+7664
0, //U+7665
0, //U+7666
0, //U+7667
0, //U+7668
0, //U+7669
0, //U+766A
0, //U+766B
0, //U+766C
0, //U+766D
0, //U+766E
0, //U+766F
0, //U+7670
0, //U+7671
0, //U+7672
0, //U+7673
0, //U+7674
0, //U+7675
0, //U+7676
0, //U+7677
0, //U+7678
0, //U+7679
0, //U+767A
0, //U+767B
0, //U+767C
0, //U+767D
0, //U+767E
0, //U+767F
0, //U+7680
0, //U+7681
0, //U+7682
0, //U+7683
0, //U+7684
0, //U+7685
0, //U+7686
0, //U+7687
0, //U+7688
0, //U+7689
0, //U+768A
0, //U+768B
0, //U+768C
0, //U+768D
0, //U+768E
0, //U+768F
0, //U+7690
0, //U+7691
0, //U+7692
0, //U+7693
0, //U+7694
0, //U+7695
0, //U+7696
0, //U+7697
0, //U+7698
0, //U+7699
0, //U+769A
0, //U+769B
0, //U+769C
0, //U+769D
0, //U+769E
0, //U+769F
0, //U+76A0
0, //U+76A1
0, //U+76A2
0, //U+76A3
0, //U+76A4
0, //U+76A5
0, //U+76A6
0, //U+76A7
0, //U+76A8
0, //U+76A9
0, //U+76AA
0, //U+76AB
0, //U+76AC
0, //U+76AD
0, //U+76AE
0, //U+76AF
0, //U+76B0
0, //U+76B1
0, //U+76B2
0, //U+76B3
0, //U+76B4
0, //U+76B5
0, //U+76B6
0, //U+76B7
0, //U+76B8
0, //U+76B9
0, //U+76BA
0, //U+76BB
0, //U+76BC
0, //U+76BD
0, //U+76BE
0, //U+76BF
0, //U+76C0
0, //U+76C1
0, //U+76C2
0, //U+76C3
0, //U+76C4
0, //U+76C5
0, //U+76C6
0, //U+76C7
0, //U+76C8
0, //U+76C9
0, //U+76CA
0, //U+76CB
0, //U+76CC
0, //U+76CD
0, //U+76CE
0, //U+76CF
0, //U+76D0
0, //U+76D1
0, //U+76D2
0, //U+76D3
0, //U+76D4
0, //U+76D5
0, //U+76D6
0, //U+76D7
0, //U+76D8
0, //U+76D9
0, //U+76DA
0, //U+76DB
0, //U+76DC
0, //U+76DD
0, //U+76DE
0, //U+76DF
0, //U+76E0
0, //U+76E1
0, //U+76E2
0, //U+76E3
0, //U+76E4
0, //U+76E5
0, //U+76E6
0, //U+76E7
0, //U+76E8
0, //U+76E9
0, //U+76EA
0, //U+76EB
0, //U+76EC
0, //U+76ED
0, //U+76EE
0, //U+76EF
0, //U+76F0
0, //U+76F1
0, //U+76F2
0, //U+76F3
0, //U+76F4
0, //U+76F5
0, //U+76F6
0, //U+76F7
0, //U+76F8
0, //U+76F9
0, //U+76FA
0, //U+76FB
0, //U+76FC
0, //U+76FD
0, //U+76FE
0, //U+76FF
0, //U+7700
0, //U+7701
0, //U+7702
0, //U+7703
0, //U+7704
0, //U+7705
0, //U+7706
0, //U+7707
0, //U+7708
0, //U+7709
0, //U+770A
0, //U+770B
0, //U+770C
0, //U+770D
0, //U+770E
0, //U+770F
0, //U+7710
0, //U+7711
0, //U+7712
0, //U+7713
0, //U+7714
0, //U+7715
0, //U+7716
0, //U+7717
0, //U+7718
0, //U+7719
0, //U+771A
0, //U+771B
0, //U+771C
0, //U+771D
0, //U+771E
0, //U+771F
0, //U+7720
0, //U+7721
0, //U+7722
0, //U+7723
0, //U+7724
0, //U+7725
0, //U+7726
0, //U+7727
0, //U+7728
0, //U+7729
0, //U+772A
0, //U+772B
0, //U+772C
0, //U+772D
0, //U+772E
0, //U+772F
0, //U+7730
0, //U+7731
0, //U+7732
0, //U+7733
0, //U+7734
0, //U+7735
0, //U+7736
0, //U+7737
0, //U+7738
0, //U+7739
0, //U+773A
0, //U+773B
0, //U+773C
0, //U+773D
0, //U+773E
0, //U+773F
0, //U+7740
0, //U+7741
0, //U+7742
0, //U+7743
0, //U+7744
0, //U+7745
0, //U+7746
0, //U+7747
0, //U+7748
0, //U+7749
0, //U+774A
0, //U+774B
0, //U+774C
0, //U+774D
0, //U+774E
0, //U+774F
0, //U+7750
0, //U+7751
0, //U+7752
0, //U+7753
0, //U+7754
0, //U+7755
0, //U+7756
0, //U+7757
0, //U+7758
0, //U+7759
0, //U+775A
0, //U+775B
0, //U+775C
0, //U+775D
0, //U+775E
0, //U+775F
0, //U+7760
0, //U+7761
0, //U+7762
0, //U+7763
0, //U+7764
0, //U+7765
0, //U+7766
0, //U+7767
0, //U+7768
0, //U+7769
0, //U+776A
0, //U+776B
0, //U+776C
0, //U+776D
0, //U+776E
0, //U+776F
0, //U+7770
0, //U+7771
0, //U+7772
0, //U+7773
0, //U+7774
0, //U+7775
0, //U+7776
0, //U+7777
0, //U+7778
0, //U+7779
0, //U+777A
0, //U+777B
0, //U+777C
0, //U+777D
0, //U+777E
0, //U+777F
0, //U+7780
0, //U+7781
0, //U+7782
0, //U+7783
0, //U+7784
0, //U+7785
0, //U+7786
0, //U+7787
0, //U+7788
0, //U+7789
0, //U+778A
0, //U+778B
0, //U+778C
0, //U+778D
0, //U+778E
0, //U+778F
0, //U+7790
0, //U+7791
0, //U+7792
0, //U+7793
0, //U+7794
0, //U+7795
0, //U+7796
0, //U+7797
0, //U+7798
0, //U+7799
0, //U+779A
0, //U+779B
0, //U+779C
0, //U+779D
0, //U+779E
0, //U+779F
0, //U+77A0
0, //U+77A1
0, //U+77A2
0, //U+77A3
0, //U+77A4
0, //U+77A5
0, //U+77A6
0, //U+77A7
0, //U+77A8
0, //U+77A9
0, //U+77AA
0, //U+77AB
0, //U+77AC
0, //U+77AD
0, //U+77AE
0, //U+77AF
0, //U+77B0
0, //U+77B1
0, //U+77B2
0, //U+77B3
0, //U+77B4
0, //U+77B5
0, //U+77B6
0, //U+77B7
0, //U+77B8
0, //U+77B9
0, //U+77BA
0, //U+77BB
0, //U+77BC
0, //U+77BD
0, //U+77BE
0, //U+77BF
0, //U+77C0
0, //U+77C1
0, //U+77C2
0, //U+77C3
0, //U+77C4
0, //U+77C5
0, //U+77C6
0, //U+77C7
0, //U+77C8
0, //U+77C9
0, //U+77CA
0, //U+77CB
0, //U+77CC
0, //U+77CD
0, //U+77CE
0, //U+77CF
0, //U+77D0
0, //U+77D1
0, //U+77D2
0, //U+77D3
0, //U+77D4
0, //U+77D5
0, //U+77D6
0, //U+77D7
0, //U+77D8
0, //U+77D9
0, //U+77DA
0, //U+77DB
0, //U+77DC
0, //U+77DD
0, //U+77DE
0, //U+77DF
0, //U+77E0
0, //U+77E1
0, //U+77E2
0, //U+77E3
0, //U+77E4
0, //U+77E5
0, //U+77E6
0, //U+77E7
0, //U+77E8
0, //U+77E9
0, //U+77EA
0, //U+77EB
0, //U+77EC
0, //U+77ED
0, //U+77EE
0, //U+77EF
0, //U+77F0
0, //U+77F1
0, //U+77F2
0, //U+77F3
0, //U+77F4
0, //U+77F5
0, //U+77F6
0, //U+77F7
0, //U+77F8
0, //U+77F9
0, //U+77FA
0, //U+77FB
0, //U+77FC
0, //U+77FD
0, //U+77FE
0, //U+77FF
0, //U+7800
0, //U+7801
0, //U+7802
0, //U+7803
0, //U+7804
0, //U+7805
0, //U+7806
0, //U+7807
0, //U+7808
0, //U+7809
0, //U+780A
0, //U+780B
0, //U+780C
0, //U+780D
0, //U+780E
0, //U+780F
0, //U+7810
0, //U+7811
0, //U+7812
0, //U+7813
0, //U+7814
0, //U+7815
0, //U+7816
0, //U+7817
0, //U+7818
0, //U+7819
0, //U+781A
0, //U+781B
0, //U+781C
0, //U+781D
0, //U+781E
0, //U+781F
0, //U+7820
0, //U+7821
0, //U+7822
0, //U+7823
0, //U+7824
0, //U+7825
0, //U+7826
0, //U+7827
0, //U+7828
0, //U+7829
0, //U+782A
0, //U+782B
0, //U+782C
0, //U+782D
0, //U+782E
0, //U+782F
0, //U+7830
0, //U+7831
0, //U+7832
0, //U+7833
0, //U+7834
0, //U+7835
0, //U+7836
0, //U+7837
0, //U+7838
0, //U+7839
0, //U+783A
0, //U+783B
0, //U+783C
0, //U+783D
0, //U+783E
0, //U+783F
0, //U+7840
0, //U+7841
0, //U+7842
0, //U+7843
0, //U+7844
0, //U+7845
0, //U+7846
0, //U+7847
0, //U+7848
0, //U+7849
0, //U+784A
0, //U+784B
0, //U+784C
0, //U+784D
0, //U+784E
0, //U+784F
0, //U+7850
0, //U+7851
0, //U+7852
0, //U+7853
0, //U+7854
0, //U+7855
0, //U+7856
0, //U+7857
0, //U+7858
0, //U+7859
0, //U+785A
0, //U+785B
0, //U+785C
0, //U+785D
0, //U+785E
0, //U+785F
0, //U+7860
0, //U+7861
0, //U+7862
0, //U+7863
0, //U+7864
0, //U+7865
0, //U+7866
0, //U+7867
0, //U+7868
0, //U+7869
0, //U+786A
0, //U+786B
0, //U+786C
0, //U+786D
0, //U+786E
0, //U+786F
0, //U+7870
0, //U+7871
0, //U+7872
0, //U+7873
0, //U+7874
0, //U+7875
0, //U+7876
0, //U+7877
0, //U+7878
0, //U+7879
0, //U+787A
0, //U+787B
0, //U+787C
0, //U+787D
0, //U+787E
0, //U+787F
0, //U+7880
0, //U+7881
0, //U+7882
0, //U+7883
0, //U+7884
0, //U+7885
0, //U+7886
0, //U+7887
0, //U+7888
0, //U+7889
0, //U+788A
0, //U+788B
0, //U+788C
0, //U+788D
0, //U+788E
0, //U+788F
0, //U+7890
0, //U+7891
0, //U+7892
0, //U+7893
0, //U+7894
0, //U+7895
0, //U+7896
0, //U+7897
0, //U+7898
0, //U+7899
0, //U+789A
0, //U+789B
0, //U+789C
0, //U+789D
0, //U+789E
0, //U+789F
0, //U+78A0
0, //U+78A1
0, //U+78A2
0, //U+78A3
0, //U+78A4
0, //U+78A5
0, //U+78A6
0, //U+78A7
0, //U+78A8
0, //U+78A9
0, //U+78AA
0, //U+78AB
0, //U+78AC
0, //U+78AD
0, //U+78AE
0, //U+78AF
0, //U+78B0
0, //U+78B1
0, //U+78B2
0, //U+78B3
0, //U+78B4
0, //U+78B5
0, //U+78B6
0, //U+78B7
0, //U+78B8
0, //U+78B9
0, //U+78BA
0, //U+78BB
0, //U+78BC
0, //U+78BD
0, //U+78BE
0, //U+78BF
0, //U+78C0
0, //U+78C1
0, //U+78C2
0, //U+78C3
0, //U+78C4
0, //U+78C5
0, //U+78C6
0, //U+78C7
0, //U+78C8
0, //U+78C9
0, //U+78CA
0, //U+78CB
0, //U+78CC
0, //U+78CD
0, //U+78CE
0, //U+78CF
0, //U+78D0
0, //U+78D1
0, //U+78D2
0, //U+78D3
0, //U+78D4
0, //U+78D5
0, //U+78D6
0, //U+78D7
0, //U+78D8
0, //U+78D9
0, //U+78DA
0, //U+78DB
0, //U+78DC
0, //U+78DD
0, //U+78DE
0, //U+78DF
0, //U+78E0
0, //U+78E1
0, //U+78E2
0, //U+78E3
0, //U+78E4
0, //U+78E5
0, //U+78E6
0, //U+78E7
0, //U+78E8
0, //U+78E9
0, //U+78EA
0, //U+78EB
0, //U+78EC
0, //U+78ED
0, //U+78EE
0, //U+78EF
0, //U+78F0
0, //U+78F1
0, //U+78F2
0, //U+78F3
0, //U+78F4
0, //U+78F5
0, //U+78F6
0, //U+78F7
0, //U+78F8
0, //U+78F9
0, //U+78FA
0, //U+78FB
0, //U+78FC
0, //U+78FD
0, //U+78FE
0, //U+78FF
0, //U+7900
0, //U+7901
0, //U+7902
0, //U+7903
0, //U+7904
0, //U+7905
0, //U+7906
0, //U+7907
0, //U+7908
0, //U+7909
0, //U+790A
0, //U+790B
0, //U+790C
0, //U+790D
0, //U+790E
0, //U+790F
0, //U+7910
0, //U+7911
0, //U+7912
0, //U+7913
0, //U+7914
0, //U+7915
0, //U+7916
0, //U+7917
0, //U+7918
0, //U+7919
0, //U+791A
0, //U+791B
0, //U+791C
0, //U+791D
0, //U+791E
0, //U+791F
0, //U+7920
0, //U+7921
0, //U+7922
0, //U+7923
0, //U+7924
0, //U+7925
0, //U+7926
0, //U+7927
0, //U+7928
0, //U+7929
0, //U+792A
0, //U+792B
0, //U+792C
0, //U+792D
0, //U+792E
0, //U+792F
0, //U+7930
0, //U+7931
0, //U+7932
0, //U+7933
0, //U+7934
0, //U+7935
0, //U+7936
0, //U+7937
0, //U+7938
0, //U+7939
0, //U+793A
0, //U+793B
0, //U+793C
0, //U+793D
0, //U+793E
0, //U+793F
0, //U+7940
0, //U+7941
0, //U+7942
0, //U+7943
0, //U+7944
0, //U+7945
0, //U+7946
0, //U+7947
0, //U+7948
0, //U+7949
0, //U+794A
0, //U+794B
0, //U+794C
0, //U+794D
0, //U+794E
0, //U+794F
0, //U+7950
0, //U+7951
0, //U+7952
0, //U+7953
0, //U+7954
0, //U+7955
0, //U+7956
0, //U+7957
0, //U+7958
0, //U+7959
0, //U+795A
0, //U+795B
0, //U+795C
0, //U+795D
0, //U+795E
0, //U+795F
0, //U+7960
0, //U+7961
0, //U+7962
0, //U+7963
0, //U+7964
0, //U+7965
0, //U+7966
0, //U+7967
0, //U+7968
0, //U+7969
0, //U+796A
0, //U+796B
0, //U+796C
0, //U+796D
0, //U+796E
0, //U+796F
0, //U+7970
0, //U+7971
0, //U+7972
0, //U+7973
0, //U+7974
0, //U+7975
0, //U+7976
0, //U+7977
0, //U+7978
0, //U+7979
0, //U+797A
0, //U+797B
0, //U+797C
0, //U+797D
0, //U+797E
0, //U+797F
0, //U+7980
0, //U+7981
0, //U+7982
0, //U+7983
0, //U+7984
0, //U+7985
0, //U+7986
0, //U+7987
0, //U+7988
0, //U+7989
0, //U+798A
0, //U+798B
0, //U+798C
0, //U+798D
0, //U+798E
0, //U+798F
0, //U+7990
0, //U+7991
0, //U+7992
0, //U+7993
0, //U+7994
0, //U+7995
0, //U+7996
0, //U+7997
0, //U+7998
0, //U+7999
0, //U+799A
0, //U+799B
0, //U+799C
0, //U+799D
0, //U+799E
0, //U+799F
0, //U+79A0
0, //U+79A1
0, //U+79A2
0, //U+79A3
0, //U+79A4
0, //U+79A5
0, //U+79A6
0, //U+79A7
0, //U+79A8
0, //U+79A9
0, //U+79AA
0, //U+79AB
0, //U+79AC
0, //U+79AD
0, //U+79AE
0, //U+79AF
0, //U+79B0
0, //U+79B1
0, //U+79B2
0, //U+79B3
0, //U+79B4
0, //U+79B5
0, //U+79B6
0, //U+79B7
0, //U+79B8
0, //U+79B9
0, //U+79BA
0, //U+79BB
0, //U+79BC
0, //U+79BD
0, //U+79BE
0, //U+79BF
0, //U+79C0
0, //U+79C1
0, //U+79C2
0, //U+79C3
0, //U+79C4
0, //U+79C5
0, //U+79C6
0, //U+79C7
0, //U+79C8
0, //U+79C9
0, //U+79CA
0, //U+79CB
0, //U+79CC
0, //U+79CD
0, //U+79CE
0, //U+79CF
0, //U+79D0
0, //U+79D1
0, //U+79D2
0, //U+79D3
0, //U+79D4
0, //U+79D5
0, //U+79D6
0, //U+79D7
0, //U+79D8
0, //U+79D9
0, //U+79DA
0, //U+79DB
0, //U+79DC
0, //U+79DD
0, //U+79DE
0, //U+79DF
0, //U+79E0
0, //U+79E1
0, //U+79E2
0, //U+79E3
0, //U+79E4
0, //U+79E5
0, //U+79E6
0, //U+79E7
0, //U+79E8
0, //U+79E9
0, //U+79EA
0, //U+79EB
0, //U+79EC
0, //U+79ED
0, //U+79EE
0, //U+79EF
0, //U+79F0
0, //U+79F1
0, //U+79F2
0, //U+79F3
0, //U+79F4
0, //U+79F5
0, //U+79F6
0, //U+79F7
0, //U+79F8
0, //U+79F9
0, //U+79FA
0, //U+79FB
0, //U+79FC
0, //U+79FD
0, //U+79FE
0, //U+79FF
0, //U+7A00
0, //U+7A01
0, //U+7A02
0, //U+7A03
0, //U+7A04
0, //U+7A05
0, //U+7A06
0, //U+7A07
0, //U+7A08
0, //U+7A09
0, //U+7A0A
0, //U+7A0B
0, //U+7A0C
0, //U+7A0D
0, //U+7A0E
0, //U+7A0F
0, //U+7A10
0, //U+7A11
0, //U+7A12
0, //U+7A13
0, //U+7A14
0, //U+7A15
0, //U+7A16
0, //U+7A17
0, //U+7A18
0, //U+7A19
0, //U+7A1A
0, //U+7A1B
0, //U+7A1C
0, //U+7A1D
0, //U+7A1E
0, //U+7A1F
0, //U+7A20
0, //U+7A21
0, //U+7A22
0, //U+7A23
0, //U+7A24
0, //U+7A25
0, //U+7A26
0, //U+7A27
0, //U+7A28
0, //U+7A29
0, //U+7A2A
0, //U+7A2B
0, //U+7A2C
0, //U+7A2D
0, //U+7A2E
0, //U+7A2F
0, //U+7A30
0, //U+7A31
0, //U+7A32
0, //U+7A33
0, //U+7A34
0, //U+7A35
0, //U+7A36
0, //U+7A37
0, //U+7A38
0, //U+7A39
0, //U+7A3A
0, //U+7A3B
0, //U+7A3C
0, //U+7A3D
0, //U+7A3E
0, //U+7A3F
0, //U+7A40
0, //U+7A41
0, //U+7A42
0, //U+7A43
0, //U+7A44
0, //U+7A45
0, //U+7A46
0, //U+7A47
0, //U+7A48
0, //U+7A49
0, //U+7A4A
0, //U+7A4B
0, //U+7A4C
0, //U+7A4D
0, //U+7A4E
0, //U+7A4F
0, //U+7A50
0, //U+7A51
0, //U+7A52
0, //U+7A53
0, //U+7A54
0, //U+7A55
0, //U+7A56
0, //U+7A57
0, //U+7A58
0, //U+7A59
0, //U+7A5A
0, //U+7A5B
0, //U+7A5C
0, //U+7A5D
0, //U+7A5E
0, //U+7A5F
0, //U+7A60
0, //U+7A61
0, //U+7A62
0, //U+7A63
0, //U+7A64
0, //U+7A65
0, //U+7A66
0, //U+7A67
0, //U+7A68
0, //U+7A69
0, //U+7A6A
0, //U+7A6B
0, //U+7A6C
0, //U+7A6D
0, //U+7A6E
0, //U+7A6F
0, //U+7A70
0, //U+7A71
0, //U+7A72
0, //U+7A73
0, //U+7A74
0, //U+7A75
0, //U+7A76
0, //U+7A77
0, //U+7A78
0, //U+7A79
0, //U+7A7A
0, //U+7A7B
0, //U+7A7C
0, //U+7A7D
0, //U+7A7E
0, //U+7A7F
0, //U+7A80
0, //U+7A81
0, //U+7A82
0, //U+7A83
0, //U+7A84
0, //U+7A85
0, //U+7A86
0, //U+7A87
0, //U+7A88
0, //U+7A89
0, //U+7A8A
0, //U+7A8B
0, //U+7A8C
0, //U+7A8D
0, //U+7A8E
0, //U+7A8F
0, //U+7A90
0, //U+7A91
0, //U+7A92
0, //U+7A93
0, //U+7A94
0, //U+7A95
0, //U+7A96
0, //U+7A97
0, //U+7A98
0, //U+7A99
0, //U+7A9A
0, //U+7A9B
0, //U+7A9C
0, //U+7A9D
0, //U+7A9E
0, //U+7A9F
0, //U+7AA0
0, //U+7AA1
0, //U+7AA2
0, //U+7AA3
0, //U+7AA4
0, //U+7AA5
0, //U+7AA6
0, //U+7AA7
0, //U+7AA8
0, //U+7AA9
0, //U+7AAA
0, //U+7AAB
0, //U+7AAC
0, //U+7AAD
0, //U+7AAE
0, //U+7AAF
0, //U+7AB0
0, //U+7AB1
0, //U+7AB2
0, //U+7AB3
0, //U+7AB4
0, //U+7AB5
0, //U+7AB6
0, //U+7AB7
0, //U+7AB8
0, //U+7AB9
0, //U+7ABA
0, //U+7ABB
0, //U+7ABC
0, //U+7ABD
0, //U+7ABE
0, //U+7ABF
0, //U+7AC0
0, //U+7AC1
0, //U+7AC2
0, //U+7AC3
0, //U+7AC4
0, //U+7AC5
0, //U+7AC6
0, //U+7AC7
0, //U+7AC8
0, //U+7AC9
0, //U+7ACA
0, //U+7ACB
0, //U+7ACC
0, //U+7ACD
0, //U+7ACE
0, //U+7ACF
0, //U+7AD0
0, //U+7AD1
0, //U+7AD2
0, //U+7AD3
0, //U+7AD4
0, //U+7AD5
0, //U+7AD6
0, //U+7AD7
0, //U+7AD8
0, //U+7AD9
0, //U+7ADA
0, //U+7ADB
0, //U+7ADC
0, //U+7ADD
0, //U+7ADE
0, //U+7ADF
0, //U+7AE0
0, //U+7AE1
0, //U+7AE2
0, //U+7AE3
0, //U+7AE4
0, //U+7AE5
0, //U+7AE6
0, //U+7AE7
0, //U+7AE8
0, //U+7AE9
0, //U+7AEA
0, //U+7AEB
0, //U+7AEC
0, //U+7AED
0, //U+7AEE
0, //U+7AEF
0, //U+7AF0
0, //U+7AF1
0, //U+7AF2
0, //U+7AF3
0, //U+7AF4
0, //U+7AF5
0, //U+7AF6
0, //U+7AF7
0, //U+7AF8
0, //U+7AF9
0, //U+7AFA
0, //U+7AFB
0, //U+7AFC
0, //U+7AFD
0, //U+7AFE
0, //U+7AFF
0, //U+7B00
0, //U+7B01
0, //U+7B02
0, //U+7B03
0, //U+7B04
0, //U+7B05
0, //U+7B06
0, //U+7B07
0, //U+7B08
0, //U+7B09
0, //U+7B0A
0, //U+7B0B
0, //U+7B0C
0, //U+7B0D
0, //U+7B0E
0, //U+7B0F
0, //U+7B10
0, //U+7B11
0, //U+7B12
0, //U+7B13
0, //U+7B14
0, //U+7B15
0, //U+7B16
0, //U+7B17
0, //U+7B18
0, //U+7B19
0, //U+7B1A
0, //U+7B1B
0, //U+7B1C
0, //U+7B1D
0, //U+7B1E
0, //U+7B1F
0, //U+7B20
0, //U+7B21
0, //U+7B22
0, //U+7B23
0, //U+7B24
0, //U+7B25
0, //U+7B26
0, //U+7B27
0, //U+7B28
0, //U+7B29
0, //U+7B2A
0, //U+7B2B
0, //U+7B2C
0, //U+7B2D
0, //U+7B2E
0, //U+7B2F
0, //U+7B30
0, //U+7B31
0, //U+7B32
0, //U+7B33
0, //U+7B34
0, //U+7B35
0, //U+7B36
0, //U+7B37
0, //U+7B38
0, //U+7B39
0, //U+7B3A
0, //U+7B3B
0, //U+7B3C
0, //U+7B3D
0, //U+7B3E
0, //U+7B3F
0, //U+7B40
0, //U+7B41
0, //U+7B42
0, //U+7B43
0, //U+7B44
0, //U+7B45
0, //U+7B46
0, //U+7B47
0, //U+7B48
0, //U+7B49
0, //U+7B4A
0, //U+7B4B
0, //U+7B4C
0, //U+7B4D
0, //U+7B4E
0, //U+7B4F
0, //U+7B50
0, //U+7B51
0, //U+7B52
0, //U+7B53
0, //U+7B54
0, //U+7B55
0, //U+7B56
0, //U+7B57
0, //U+7B58
0, //U+7B59
0, //U+7B5A
0, //U+7B5B
0, //U+7B5C
0, //U+7B5D
0, //U+7B5E
0, //U+7B5F
0, //U+7B60
0, //U+7B61
0, //U+7B62
0, //U+7B63
0, //U+7B64
0, //U+7B65
0, //U+7B66
0, //U+7B67
0, //U+7B68
0, //U+7B69
0, //U+7B6A
0, //U+7B6B
0, //U+7B6C
0, //U+7B6D
0, //U+7B6E
0, //U+7B6F
0, //U+7B70
0, //U+7B71
0, //U+7B72
0, //U+7B73
0, //U+7B74
0, //U+7B75
0, //U+7B76
0, //U+7B77
0, //U+7B78
0, //U+7B79
0, //U+7B7A
0, //U+7B7B
0, //U+7B7C
0, //U+7B7D
0, //U+7B7E
0, //U+7B7F
0, //U+7B80
0, //U+7B81
0, //U+7B82
0, //U+7B83
0, //U+7B84
0, //U+7B85
0, //U+7B86
0, //U+7B87
0, //U+7B88
0, //U+7B89
0, //U+7B8A
0, //U+7B8B
0, //U+7B8C
0, //U+7B8D
0, //U+7B8E
0, //U+7B8F
0, //U+7B90
0, //U+7B91
0, //U+7B92
0, //U+7B93
0, //U+7B94
0, //U+7B95
0, //U+7B96
0, //U+7B97
0, //U+7B98
0, //U+7B99
0, //U+7B9A
0, //U+7B9B
0, //U+7B9C
0, //U+7B9D
0, //U+7B9E
0, //U+7B9F
0, //U+7BA0
0, //U+7BA1
0, //U+7BA2
0, //U+7BA3
0, //U+7BA4
0, //U+7BA5
0, //U+7BA6
0, //U+7BA7
0, //U+7BA8
0, //U+7BA9
0, //U+7BAA
0, //U+7BAB
0, //U+7BAC
0, //U+7BAD
0, //U+7BAE
0, //U+7BAF
0, //U+7BB0
0, //U+7BB1
0, //U+7BB2
0, //U+7BB3
0, //U+7BB4
0, //U+7BB5
0, //U+7BB6
0, //U+7BB7
0, //U+7BB8
0, //U+7BB9
0, //U+7BBA
0, //U+7BBB
0, //U+7BBC
0, //U+7BBD
0, //U+7BBE
0, //U+7BBF
0, //U+7BC0
0, //U+7BC1
0, //U+7BC2
0, //U+7BC3
0, //U+7BC4
0, //U+7BC5
0, //U+7BC6
0, //U+7BC7
0, //U+7BC8
0, //U+7BC9
0, //U+7BCA
0, //U+7BCB
0, //U+7BCC
0, //U+7BCD
0, //U+7BCE
0, //U+7BCF
0, //U+7BD0
0, //U+7BD1
0, //U+7BD2
0, //U+7BD3
0, //U+7BD4
0, //U+7BD5
0, //U+7BD6
0, //U+7BD7
0, //U+7BD8
0, //U+7BD9
0, //U+7BDA
0, //U+7BDB
0, //U+7BDC
0, //U+7BDD
0, //U+7BDE
0, //U+7BDF
0, //U+7BE0
0, //U+7BE1
0, //U+7BE2
0, //U+7BE3
0, //U+7BE4
0, //U+7BE5
0, //U+7BE6
0, //U+7BE7
0, //U+7BE8
0, //U+7BE9
0, //U+7BEA
0, //U+7BEB
0, //U+7BEC
0, //U+7BED
0, //U+7BEE
0, //U+7BEF
0, //U+7BF0
0, //U+7BF1
0, //U+7BF2
0, //U+7BF3
0, //U+7BF4
0, //U+7BF5
0, //U+7BF6
0, //U+7BF7
0, //U+7BF8
0, //U+7BF9
0, //U+7BFA
0, //U+7BFB
0, //U+7BFC
0, //U+7BFD
0, //U+7BFE
0, //U+7BFF
0, //U+7C00
0, //U+7C01
0, //U+7C02
0, //U+7C03
0, //U+7C04
0, //U+7C05
0, //U+7C06
0, //U+7C07
0, //U+7C08
0, //U+7C09
0, //U+7C0A
0, //U+7C0B
0, //U+7C0C
0, //U+7C0D
0, //U+7C0E
0, //U+7C0F
0, //U+7C10
0, //U+7C11
0, //U+7C12
0, //U+7C13
0, //U+7C14
0, //U+7C15
0, //U+7C16
0, //U+7C17
0, //U+7C18
0, //U+7C19
0, //U+7C1A
0, //U+7C1B
0, //U+7C1C
0, //U+7C1D
0, //U+7C1E
0, //U+7C1F
0, //U+7C20
0, //U+7C21
0, //U+7C22
0, //U+7C23
0, //U+7C24
0, //U+7C25
0, //U+7C26
0, //U+7C27
0, //U+7C28
0, //U+7C29
0, //U+7C2A
0, //U+7C2B
0, //U+7C2C
0, //U+7C2D
0, //U+7C2E
0, //U+7C2F
0, //U+7C30
0, //U+7C31
0, //U+7C32
0, //U+7C33
0, //U+7C34
0, //U+7C35
0, //U+7C36
0, //U+7C37
0, //U+7C38
0, //U+7C39
0, //U+7C3A
0, //U+7C3B
0, //U+7C3C
0, //U+7C3D
0, //U+7C3E
0, //U+7C3F
0, //U+7C40
0, //U+7C41
0, //U+7C42
0, //U+7C43
0, //U+7C44
0, //U+7C45
0, //U+7C46
0, //U+7C47
0, //U+7C48
0, //U+7C49
0, //U+7C4A
0, //U+7C4B
0, //U+7C4C
0, //U+7C4D
0, //U+7C4E
0, //U+7C4F
0, //U+7C50
0, //U+7C51
0, //U+7C52
0, //U+7C53
0, //U+7C54
0, //U+7C55
0, //U+7C56
0, //U+7C57
0, //U+7C58
0, //U+7C59
0, //U+7C5A
0, //U+7C5B
0, //U+7C5C
0, //U+7C5D
0, //U+7C5E
0, //U+7C5F
0, //U+7C60
0, //U+7C61
0, //U+7C62
0, //U+7C63
0, //U+7C64
0, //U+7C65
0, //U+7C66
0, //U+7C67
0, //U+7C68
0, //U+7C69
0, //U+7C6A
0, //U+7C6B
0, //U+7C6C
0, //U+7C6D
0, //U+7C6E
0, //U+7C6F
0, //U+7C70
0, //U+7C71
0, //U+7C72
0, //U+7C73
0, //U+7C74
0, //U+7C75
0, //U+7C76
0, //U+7C77
0, //U+7C78
0, //U+7C79
0, //U+7C7A
0, //U+7C7B
0, //U+7C7C
0, //U+7C7D
0, //U+7C7E
0, //U+7C7F
0, //U+7C80
0, //U+7C81
0, //U+7C82
0, //U+7C83
0, //U+7C84
0, //U+7C85
0, //U+7C86
0, //U+7C87
0, //U+7C88
0, //U+7C89
0, //U+7C8A
0, //U+7C8B
0, //U+7C8C
0, //U+7C8D
0, //U+7C8E
0, //U+7C8F
0, //U+7C90
0, //U+7C91
0, //U+7C92
0, //U+7C93
0, //U+7C94
0, //U+7C95
0, //U+7C96
0, //U+7C97
0, //U+7C98
0, //U+7C99
0, //U+7C9A
0, //U+7C9B
0, //U+7C9C
0, //U+7C9D
0, //U+7C9E
0, //U+7C9F
0, //U+7CA0
0, //U+7CA1
0, //U+7CA2
0, //U+7CA3
0, //U+7CA4
0, //U+7CA5
0, //U+7CA6
0, //U+7CA7
0, //U+7CA8
0, //U+7CA9
0, //U+7CAA
0, //U+7CAB
0, //U+7CAC
0, //U+7CAD
0, //U+7CAE
0, //U+7CAF
0, //U+7CB0
0, //U+7CB1
0, //U+7CB2
0, //U+7CB3
0, //U+7CB4
0, //U+7CB5
0, //U+7CB6
0, //U+7CB7
0, //U+7CB8
0, //U+7CB9
0, //U+7CBA
0, //U+7CBB
0, //U+7CBC
0, //U+7CBD
0, //U+7CBE
0, //U+7CBF
0, //U+7CC0
0, //U+7CC1
0, //U+7CC2
0, //U+7CC3
0, //U+7CC4
0, //U+7CC5
0, //U+7CC6
0, //U+7CC7
0, //U+7CC8
0, //U+7CC9
0, //U+7CCA
0, //U+7CCB
0, //U+7CCC
0, //U+7CCD
0, //U+7CCE
0, //U+7CCF
0, //U+7CD0
0, //U+7CD1
0, //U+7CD2
0, //U+7CD3
0, //U+7CD4
0, //U+7CD5
0, //U+7CD6
0, //U+7CD7
0, //U+7CD8
0, //U+7CD9
0, //U+7CDA
0, //U+7CDB
0, //U+7CDC
0, //U+7CDD
0, //U+7CDE
0, //U+7CDF
0, //U+7CE0
0, //U+7CE1
0, //U+7CE2
0, //U+7CE3
0, //U+7CE4
0, //U+7CE5
0, //U+7CE6
0, //U+7CE7
0, //U+7CE8
0, //U+7CE9
0, //U+7CEA
0, //U+7CEB
0, //U+7CEC
0, //U+7CED
0, //U+7CEE
0, //U+7CEF
0, //U+7CF0
0, //U+7CF1
0, //U+7CF2
0, //U+7CF3
0, //U+7CF4
0, //U+7CF5
0, //U+7CF6
0, //U+7CF7
0, //U+7CF8
0, //U+7CF9
0, //U+7CFA
0, //U+7CFB
0, //U+7CFC
0, //U+7CFD
0, //U+7CFE
0, //U+7CFF
0, //U+7D00
0, //U+7D01
0, //U+7D02
0, //U+7D03
0, //U+7D04
0, //U+7D05
0, //U+7D06
0, //U+7D07
0, //U+7D08
0, //U+7D09
0, //U+7D0A
0, //U+7D0B
0, //U+7D0C
0, //U+7D0D
0, //U+7D0E
0, //U+7D0F
0, //U+7D10
0, //U+7D11
0, //U+7D12
0, //U+7D13
0, //U+7D14
0, //U+7D15
0, //U+7D16
0, //U+7D17
0, //U+7D18
0, //U+7D19
0, //U+7D1A
0, //U+7D1B
0, //U+7D1C
0, //U+7D1D
0, //U+7D1E
0, //U+7D1F
0, //U+7D20
0, //U+7D21
0, //U+7D22
0, //U+7D23
0, //U+7D24
0, //U+7D25
0, //U+7D26
0, //U+7D27
0, //U+7D28
0, //U+7D29
0, //U+7D2A
0, //U+7D2B
0, //U+7D2C
0, //U+7D2D
0, //U+7D2E
0, //U+7D2F
0, //U+7D30
0, //U+7D31
0, //U+7D32
0, //U+7D33
0, //U+7D34
0, //U+7D35
0, //U+7D36
0, //U+7D37
0, //U+7D38
0, //U+7D39
0, //U+7D3A
0, //U+7D3B
0, //U+7D3C
0, //U+7D3D
0, //U+7D3E
0, //U+7D3F
0, //U+7D40
0, //U+7D41
0, //U+7D42
0, //U+7D43
0, //U+7D44
0, //U+7D45
0, //U+7D46
0, //U+7D47
0, //U+7D48
0, //U+7D49
0, //U+7D4A
0, //U+7D4B
0, //U+7D4C
0, //U+7D4D
0, //U+7D4E
0, //U+7D4F
0, //U+7D50
0, //U+7D51
0, //U+7D52
0, //U+7D53
0, //U+7D54
0, //U+7D55
0, //U+7D56
0, //U+7D57
0, //U+7D58
0, //U+7D59
0, //U+7D5A
0, //U+7D5B
0, //U+7D5C
0, //U+7D5D
0, //U+7D5E
0, //U+7D5F
0, //U+7D60
0, //U+7D61
0, //U+7D62
0, //U+7D63
0, //U+7D64
0, //U+7D65
0, //U+7D66
0, //U+7D67
0, //U+7D68
0, //U+7D69
0, //U+7D6A
0, //U+7D6B
0, //U+7D6C
0, //U+7D6D
0, //U+7D6E
0, //U+7D6F
0, //U+7D70
0, //U+7D71
0, //U+7D72
0, //U+7D73
0, //U+7D74
0, //U+7D75
0, //U+7D76
0, //U+7D77
0, //U+7D78
0, //U+7D79
0, //U+7D7A
0, //U+7D7B
0, //U+7D7C
0, //U+7D7D
0, //U+7D7E
0, //U+7D7F
0, //U+7D80
0, //U+7D81
0, //U+7D82
0, //U+7D83
0, //U+7D84
0, //U+7D85
0, //U+7D86
0, //U+7D87
0, //U+7D88
0, //U+7D89
0, //U+7D8A
0, //U+7D8B
0, //U+7D8C
0, //U+7D8D
0, //U+7D8E
0, //U+7D8F
0, //U+7D90
0, //U+7D91
0, //U+7D92
0, //U+7D93
0, //U+7D94
0, //U+7D95
0, //U+7D96
0, //U+7D97
0, //U+7D98
0, //U+7D99
0, //U+7D9A
0, //U+7D9B
0, //U+7D9C
0, //U+7D9D
0, //U+7D9E
0, //U+7D9F
0, //U+7DA0
0, //U+7DA1
0, //U+7DA2
0, //U+7DA3
0, //U+7DA4
0, //U+7DA5
0, //U+7DA6
0, //U+7DA7
0, //U+7DA8
0, //U+7DA9
0, //U+7DAA
0, //U+7DAB
0, //U+7DAC
0, //U+7DAD
0, //U+7DAE
0, //U+7DAF
0, //U+7DB0
0, //U+7DB1
0, //U+7DB2
0, //U+7DB3
0, //U+7DB4
0, //U+7DB5
0, //U+7DB6
0, //U+7DB7
0, //U+7DB8
0, //U+7DB9
0, //U+7DBA
0, //U+7DBB
0, //U+7DBC
0, //U+7DBD
0, //U+7DBE
0, //U+7DBF
0, //U+7DC0
0, //U+7DC1
0, //U+7DC2
0, //U+7DC3
0, //U+7DC4
0, //U+7DC5
0, //U+7DC6
0, //U+7DC7
0, //U+7DC8
0, //U+7DC9
0, //U+7DCA
0, //U+7DCB
0, //U+7DCC
0, //U+7DCD
0, //U+7DCE
0, //U+7DCF
0, //U+7DD0
0, //U+7DD1
0, //U+7DD2
0, //U+7DD3
0, //U+7DD4
0, //U+7DD5
0, //U+7DD6
0, //U+7DD7
0, //U+7DD8
0, //U+7DD9
0, //U+7DDA
0, //U+7DDB
0, //U+7DDC
0, //U+7DDD
0, //U+7DDE
0, //U+7DDF
0, //U+7DE0
0, //U+7DE1
0, //U+7DE2
0, //U+7DE3
0, //U+7DE4
0, //U+7DE5
0, //U+7DE6
0, //U+7DE7
0, //U+7DE8
0, //U+7DE9
0, //U+7DEA
0, //U+7DEB
0, //U+7DEC
0, //U+7DED
0, //U+7DEE
0, //U+7DEF
0, //U+7DF0
0, //U+7DF1
0, //U+7DF2
0, //U+7DF3
0, //U+7DF4
0, //U+7DF5
0, //U+7DF6
0, //U+7DF7
0, //U+7DF8
0, //U+7DF9
0, //U+7DFA
0, //U+7DFB
0, //U+7DFC
0, //U+7DFD
0, //U+7DFE
0, //U+7DFF
0, //U+7E00
0, //U+7E01
0, //U+7E02
0, //U+7E03
0, //U+7E04
0, //U+7E05
0, //U+7E06
0, //U+7E07
0, //U+7E08
0, //U+7E09
0, //U+7E0A
0, //U+7E0B
0, //U+7E0C
0, //U+7E0D
0, //U+7E0E
0, //U+7E0F
0, //U+7E10
0, //U+7E11
0, //U+7E12
0, //U+7E13
0, //U+7E14
0, //U+7E15
0, //U+7E16
0, //U+7E17
0, //U+7E18
0, //U+7E19
0, //U+7E1A
0, //U+7E1B
0, //U+7E1C
0, //U+7E1D
0, //U+7E1E
0, //U+7E1F
0, //U+7E20
0, //U+7E21
0, //U+7E22
0, //U+7E23
0, //U+7E24
0, //U+7E25
0, //U+7E26
0, //U+7E27
0, //U+7E28
0, //U+7E29
0, //U+7E2A
0, //U+7E2B
0, //U+7E2C
0, //U+7E2D
0, //U+7E2E
0, //U+7E2F
0, //U+7E30
0, //U+7E31
0, //U+7E32
0, //U+7E33
0, //U+7E34
0, //U+7E35
0, //U+7E36
0, //U+7E37
0, //U+7E38
0, //U+7E39
0, //U+7E3A
0, //U+7E3B
0, //U+7E3C
0, //U+7E3D
0, //U+7E3E
0, //U+7E3F
0, //U+7E40
0, //U+7E41
0, //U+7E42
0, //U+7E43
0, //U+7E44
0, //U+7E45
0, //U+7E46
0, //U+7E47
0, //U+7E48
0, //U+7E49
0, //U+7E4A
0, //U+7E4B
0, //U+7E4C
0, //U+7E4D
0, //U+7E4E
0, //U+7E4F
0, //U+7E50
0, //U+7E51
0, //U+7E52
0, //U+7E53
0, //U+7E54
0, //U+7E55
0, //U+7E56
0, //U+7E57
0, //U+7E58
0, //U+7E59
0, //U+7E5A
0, //U+7E5B
0, //U+7E5C
0, //U+7E5D
0, //U+7E5E
0, //U+7E5F
0, //U+7E60
0, //U+7E61
0, //U+7E62
0, //U+7E63
0, //U+7E64
0, //U+7E65
0, //U+7E66
0, //U+7E67
0, //U+7E68
0, //U+7E69
0, //U+7E6A
0, //U+7E6B
0, //U+7E6C
0, //U+7E6D
0, //U+7E6E
0, //U+7E6F
0, //U+7E70
0, //U+7E71
0, //U+7E72
0, //U+7E73
0, //U+7E74
0, //U+7E75
0, //U+7E76
0, //U+7E77
0, //U+7E78
0, //U+7E79
0, //U+7E7A
0, //U+7E7B
0, //U+7E7C
0, //U+7E7D
0, //U+7E7E
0, //U+7E7F
0, //U+7E80
0, //U+7E81
0, //U+7E82
0, //U+7E83
0, //U+7E84
0, //U+7E85
0, //U+7E86
0, //U+7E87
0, //U+7E88
0, //U+7E89
0, //U+7E8A
0, //U+7E8B
0, //U+7E8C
0, //U+7E8D
0, //U+7E8E
0, //U+7E8F
0, //U+7E90
0, //U+7E91
0, //U+7E92
0, //U+7E93
0, //U+7E94
0, //U+7E95
0, //U+7E96
0, //U+7E97
0, //U+7E98
0, //U+7E99
0, //U+7E9A
0, //U+7E9B
0, //U+7E9C
0, //U+7E9D
0, //U+7E9E
0, //U+7E9F
0, //U+7EA0
0, //U+7EA1
0, //U+7EA2
0, //U+7EA3
0, //U+7EA4
0, //U+7EA5
0, //U+7EA6
0, //U+7EA7
0, //U+7EA8
0, //U+7EA9
0, //U+7EAA
0, //U+7EAB
0, //U+7EAC
0, //U+7EAD
0, //U+7EAE
0, //U+7EAF
0, //U+7EB0
0, //U+7EB1
0, //U+7EB2
0, //U+7EB3
0, //U+7EB4
0, //U+7EB5
0, //U+7EB6
0, //U+7EB7
0, //U+7EB8
0, //U+7EB9
0, //U+7EBA
0, //U+7EBB
0, //U+7EBC
0, //U+7EBD
0, //U+7EBE
0, //U+7EBF
0, //U+7EC0
0, //U+7EC1
0, //U+7EC2
0, //U+7EC3
0, //U+7EC4
0, //U+7EC5
0, //U+7EC6
0, //U+7EC7
0, //U+7EC8
0, //U+7EC9
0, //U+7ECA
0, //U+7ECB
0, //U+7ECC
0, //U+7ECD
0, //U+7ECE
0, //U+7ECF
0, //U+7ED0
0, //U+7ED1
0, //U+7ED2
0, //U+7ED3
0, //U+7ED4
0, //U+7ED5
0, //U+7ED6
0, //U+7ED7
0, //U+7ED8
0, //U+7ED9
0, //U+7EDA
0, //U+7EDB
0, //U+7EDC
0, //U+7EDD
0, //U+7EDE
0, //U+7EDF
0, //U+7EE0
0, //U+7EE1
0, //U+7EE2
0, //U+7EE3
0, //U+7EE4
0, //U+7EE5
0, //U+7EE6
0, //U+7EE7
0, //U+7EE8
0, //U+7EE9
0, //U+7EEA
0, //U+7EEB
0, //U+7EEC
0, //U+7EED
0, //U+7EEE
0, //U+7EEF
0, //U+7EF0
0, //U+7EF1
0, //U+7EF2
0, //U+7EF3
0, //U+7EF4
0, //U+7EF5
0, //U+7EF6
0, //U+7EF7
0, //U+7EF8
0, //U+7EF9
0, //U+7EFA
0, //U+7EFB
0, //U+7EFC
0, //U+7EFD
0, //U+7EFE
0, //U+7EFF
0, //U+7F00
0, //U+7F01
0, //U+7F02
0, //U+7F03
0, //U+7F04
0, //U+7F05
0, //U+7F06
0, //U+7F07
0, //U+7F08
0, //U+7F09
0, //U+7F0A
0, //U+7F0B
0, //U+7F0C
0, //U+7F0D
0, //U+7F0E
0, //U+7F0F
0, //U+7F10
0, //U+7F11
0, //U+7F12
0, //U+7F13
0, //U+7F14
0, //U+7F15
0, //U+7F16
0, //U+7F17
0, //U+7F18
0, //U+7F19
0, //U+7F1A
0, //U+7F1B
0, //U+7F1C
0, //U+7F1D
0, //U+7F1E
0, //U+7F1F
0, //U+7F20
0, //U+7F21
0, //U+7F22
0, //U+7F23
0, //U+7F24
0, //U+7F25
0, //U+7F26
0, //U+7F27
0, //U+7F28
0, //U+7F29
0, //U+7F2A
0, //U+7F2B
0, //U+7F2C
0, //U+7F2D
0, //U+7F2E
0, //U+7F2F
0, //U+7F30
0, //U+7F31
0, //U+7F32
0, //U+7F33
0, //U+7F34
0, //U+7F35
0, //U+7F36
0, //U+7F37
0, //U+7F38
0, //U+7F39
0, //U+7F3A
0, //U+7F3B
0, //U+7F3C
0, //U+7F3D
0, //U+7F3E
0, //U+7F3F
0, //U+7F40
0, //U+7F41
0, //U+7F42
0, //U+7F43
0, //U+7F44
0, //U+7F45
0, //U+7F46
0, //U+7F47
0, //U+7F48
0, //U+7F49
0, //U+7F4A
0, //U+7F4B
0, //U+7F4C
0, //U+7F4D
0, //U+7F4E
0, //U+7F4F
0, //U+7F50
0, //U+7F51
0, //U+7F52
0, //U+7F53
0, //U+7F54
0, //U+7F55
0, //U+7F56
0, //U+7F57
0, //U+7F58
0, //U+7F59
0, //U+7F5A
0, //U+7F5B
0, //U+7F5C
0, //U+7F5D
0, //U+7F5E
0, //U+7F5F
0, //U+7F60
0, //U+7F61
0, //U+7F62
0, //U+7F63
0, //U+7F64
0, //U+7F65
0, //U+7F66
0, //U+7F67
0, //U+7F68
0, //U+7F69
0, //U+7F6A
0, //U+7F6B
0, //U+7F6C
0, //U+7F6D
0, //U+7F6E
0, //U+7F6F
0, //U+7F70
0, //U+7F71
0, //U+7F72
0, //U+7F73
0, //U+7F74
0, //U+7F75
0, //U+7F76
0, //U+7F77
0, //U+7F78
0, //U+7F79
0, //U+7F7A
0, //U+7F7B
0, //U+7F7C
0, //U+7F7D
0, //U+7F7E
0, //U+7F7F
0, //U+7F80
0, //U+7F81
0, //U+7F82
0, //U+7F83
0, //U+7F84
0, //U+7F85
0, //U+7F86
0, //U+7F87
0, //U+7F88
0, //U+7F89
0, //U+7F8A
0, //U+7F8B
0, //U+7F8C
0, //U+7F8D
0, //U+7F8E
0, //U+7F8F
0, //U+7F90
0, //U+7F91
0, //U+7F92
0, //U+7F93
0, //U+7F94
0, //U+7F95
0, //U+7F96
0, //U+7F97
0, //U+7F98
0, //U+7F99
0, //U+7F9A
0, //U+7F9B
0, //U+7F9C
0, //U+7F9D
0, //U+7F9E
0, //U+7F9F
0, //U+7FA0
0, //U+7FA1
0, //U+7FA2
0, //U+7FA3
0, //U+7FA4
0, //U+7FA5
0, //U+7FA6
0, //U+7FA7
0, //U+7FA8
0, //U+7FA9
0, //U+7FAA
0, //U+7FAB
0, //U+7FAC
0, //U+7FAD
0, //U+7FAE
0, //U+7FAF
0, //U+7FB0
0, //U+7FB1
0, //U+7FB2
0, //U+7FB3
0, //U+7FB4
0, //U+7FB5
0, //U+7FB6
0, //U+7FB7
0, //U+7FB8
0, //U+7FB9
0, //U+7FBA
0, //U+7FBB
0, //U+7FBC
0, //U+7FBD
0, //U+7FBE
0, //U+7FBF
0, //U+7FC0
0, //U+7FC1
0, //U+7FC2
0, //U+7FC3
0, //U+7FC4
0, //U+7FC5
0, //U+7FC6
0, //U+7FC7
0, //U+7FC8
0, //U+7FC9
0, //U+7FCA
0, //U+7FCB
0, //U+7FCC
0, //U+7FCD
0, //U+7FCE
0, //U+7FCF
0, //U+7FD0
0, //U+7FD1
0, //U+7FD2
0, //U+7FD3
0, //U+7FD4
0, //U+7FD5
0, //U+7FD6
0, //U+7FD7
0, //U+7FD8
0, //U+7FD9
0, //U+7FDA
0, //U+7FDB
0, //U+7FDC
0, //U+7FDD
0, //U+7FDE
0, //U+7FDF
0, //U+7FE0
0, //U+7FE1
0, //U+7FE2
0, //U+7FE3
0, //U+7FE4
0, //U+7FE5
0, //U+7FE6
0, //U+7FE7
0, //U+7FE8
0, //U+7FE9
0, //U+7FEA
0, //U+7FEB
0, //U+7FEC
0, //U+7FED
0, //U+7FEE
0, //U+7FEF
0, //U+7FF0
0, //U+7FF1
0, //U+7FF2
0, //U+7FF3
0, //U+7FF4
0, //U+7FF5
0, //U+7FF6
0, //U+7FF7
0, //U+7FF8
0, //U+7FF9
0, //U+7FFA
0, //U+7FFB
0, //U+7FFC
0, //U+7FFD
0, //U+7FFE
0, //U+7FFF
0, //U+8000
0, //U+8001
0, //U+8002
0, //U+8003
0, //U+8004
0, //U+8005
0, //U+8006
0, //U+8007
0, //U+8008
0, //U+8009
0, //U+800A
0, //U+800B
0, //U+800C
0, //U+800D
0, //U+800E
0, //U+800F
0, //U+8010
0, //U+8011
0, //U+8012
0, //U+8013
0, //U+8014
0, //U+8015
0, //U+8016
0, //U+8017
0, //U+8018
0, //U+8019
0, //U+801A
0, //U+801B
0, //U+801C
0, //U+801D
0, //U+801E
0, //U+801F
0, //U+8020
0, //U+8021
0, //U+8022
0, //U+8023
0, //U+8024
0, //U+8025
0, //U+8026
0, //U+8027
0, //U+8028
0, //U+8029
0, //U+802A
0, //U+802B
0, //U+802C
0, //U+802D
0, //U+802E
0, //U+802F
0, //U+8030
0, //U+8031
0, //U+8032
0, //U+8033
0, //U+8034
0, //U+8035
0, //U+8036
0, //U+8037
0, //U+8038
0, //U+8039
0, //U+803A
0, //U+803B
0, //U+803C
0, //U+803D
0, //U+803E
0, //U+803F
0, //U+8040
0, //U+8041
0, //U+8042
0, //U+8043
0, //U+8044
0, //U+8045
0, //U+8046
0, //U+8047
0, //U+8048
0, //U+8049
0, //U+804A
0, //U+804B
0, //U+804C
0, //U+804D
0, //U+804E
0, //U+804F
0, //U+8050
0, //U+8051
0, //U+8052
0, //U+8053
0, //U+8054
0, //U+8055
0, //U+8056
0, //U+8057
0, //U+8058
0, //U+8059
0, //U+805A
0, //U+805B
0, //U+805C
0, //U+805D
0, //U+805E
0, //U+805F
0, //U+8060
0, //U+8061
0, //U+8062
0, //U+8063
0, //U+8064
0, //U+8065
0, //U+8066
0, //U+8067
0, //U+8068
0, //U+8069
0, //U+806A
0, //U+806B
0, //U+806C
0, //U+806D
0, //U+806E
0, //U+806F
0, //U+8070
0, //U+8071
0, //U+8072
0, //U+8073
0, //U+8074
0, //U+8075
0, //U+8076
0, //U+8077
0, //U+8078
0, //U+8079
0, //U+807A
0, //U+807B
0, //U+807C
0, //U+807D
0, //U+807E
0, //U+807F
0, //U+8080
0, //U+8081
0, //U+8082
0, //U+8083
0, //U+8084
0, //U+8085
0, //U+8086
0, //U+8087
0, //U+8088
0, //U+8089
0, //U+808A
0, //U+808B
0, //U+808C
0, //U+808D
0, //U+808E
0, //U+808F
0, //U+8090
0, //U+8091
0, //U+8092
0, //U+8093
0, //U+8094
0, //U+8095
0, //U+8096
0, //U+8097
0, //U+8098
0, //U+8099
0, //U+809A
0, //U+809B
0, //U+809C
0, //U+809D
0, //U+809E
0, //U+809F
0, //U+80A0
0, //U+80A1
0, //U+80A2
0, //U+80A3
0, //U+80A4
0, //U+80A5
0, //U+80A6
0, //U+80A7
0, //U+80A8
0, //U+80A9
0, //U+80AA
0, //U+80AB
0, //U+80AC
0, //U+80AD
0, //U+80AE
0, //U+80AF
0, //U+80B0
0, //U+80B1
0, //U+80B2
0, //U+80B3
0, //U+80B4
0, //U+80B5
0, //U+80B6
0, //U+80B7
0, //U+80B8
0, //U+80B9
0, //U+80BA
0, //U+80BB
0, //U+80BC
0, //U+80BD
0, //U+80BE
0, //U+80BF
0, //U+80C0
0, //U+80C1
0, //U+80C2
0, //U+80C3
0, //U+80C4
0, //U+80C5
0, //U+80C6
0, //U+80C7
0, //U+80C8
0, //U+80C9
0, //U+80CA
0, //U+80CB
0, //U+80CC
0, //U+80CD
0, //U+80CE
0, //U+80CF
0, //U+80D0
0, //U+80D1
0, //U+80D2
0, //U+80D3
0, //U+80D4
0, //U+80D5
0, //U+80D6
0, //U+80D7
0, //U+80D8
0, //U+80D9
0, //U+80DA
0, //U+80DB
0, //U+80DC
0, //U+80DD
0, //U+80DE
0, //U+80DF
0, //U+80E0
0, //U+80E1
0, //U+80E2
0, //U+80E3
0, //U+80E4
0, //U+80E5
0, //U+80E6
0, //U+80E7
0, //U+80E8
0, //U+80E9
0, //U+80EA
0, //U+80EB
0, //U+80EC
0, //U+80ED
0, //U+80EE
0, //U+80EF
0, //U+80F0
0, //U+80F1
0, //U+80F2
0, //U+80F3
0, //U+80F4
0, //U+80F5
0, //U+80F6
0, //U+80F7
0, //U+80F8
0, //U+80F9
0, //U+80FA
0, //U+80FB
0, //U+80FC
0, //U+80FD
0, //U+80FE
0, //U+80FF
0, //U+8100
0, //U+8101
0, //U+8102
0, //U+8103
0, //U+8104
0, //U+8105
0, //U+8106
0, //U+8107
0, //U+8108
0, //U+8109
0, //U+810A
0, //U+810B
0, //U+810C
0, //U+810D
0, //U+810E
0, //U+810F
0, //U+8110
0, //U+8111
0, //U+8112
0, //U+8113
0, //U+8114
0, //U+8115
0, //U+8116
0, //U+8117
0, //U+8118
0, //U+8119
0, //U+811A
0, //U+811B
0, //U+811C
0, //U+811D
0, //U+811E
0, //U+811F
0, //U+8120
0, //U+8121
0, //U+8122
0, //U+8123
0, //U+8124
0, //U+8125
0, //U+8126
0, //U+8127
0, //U+8128
0, //U+8129
0, //U+812A
0, //U+812B
0, //U+812C
0, //U+812D
0, //U+812E
0, //U+812F
0, //U+8130
0, //U+8131
0, //U+8132
0, //U+8133
0, //U+8134
0, //U+8135
0, //U+8136
0, //U+8137
0, //U+8138
0, //U+8139
0, //U+813A
0, //U+813B
0, //U+813C
0, //U+813D
0, //U+813E
0, //U+813F
0, //U+8140
0, //U+8141
0, //U+8142
0, //U+8143
0, //U+8144
0, //U+8145
0, //U+8146
0, //U+8147
0, //U+8148
0, //U+8149
0, //U+814A
0, //U+814B
0, //U+814C
0, //U+814D
0, //U+814E
0, //U+814F
0, //U+8150
0, //U+8151
0, //U+8152
0, //U+8153
0, //U+8154
0, //U+8155
0, //U+8156
0, //U+8157
0, //U+8158
0, //U+8159
0, //U+815A
0, //U+815B
0, //U+815C
0, //U+815D
0, //U+815E
0, //U+815F
0, //U+8160
0, //U+8161
0, //U+8162
0, //U+8163
0, //U+8164
0, //U+8165
0, //U+8166
0, //U+8167
0, //U+8168
0, //U+8169
0, //U+816A
0, //U+816B
0, //U+816C
0, //U+816D
0, //U+816E
0, //U+816F
0, //U+8170
0, //U+8171
0, //U+8172
0, //U+8173
0, //U+8174
0, //U+8175
0, //U+8176
0, //U+8177
0, //U+8178
0, //U+8179
0, //U+817A
0, //U+817B
0, //U+817C
0, //U+817D
0, //U+817E
0, //U+817F
0, //U+8180
0, //U+8181
0, //U+8182
0, //U+8183
0, //U+8184
0, //U+8185
0, //U+8186
0, //U+8187
0, //U+8188
0, //U+8189
0, //U+818A
0, //U+818B
0, //U+818C
0, //U+818D
0, //U+818E
0, //U+818F
0, //U+8190
0, //U+8191
0, //U+8192
0, //U+8193
0, //U+8194
0, //U+8195
0, //U+8196
0, //U+8197
0, //U+8198
0, //U+8199
0, //U+819A
0, //U+819B
0, //U+819C
0, //U+819D
0, //U+819E
0, //U+819F
0, //U+81A0
0, //U+81A1
0, //U+81A2
0, //U+81A3
0, //U+81A4
0, //U+81A5
0, //U+81A6
0, //U+81A7
0, //U+81A8
0, //U+81A9
0, //U+81AA
0, //U+81AB
0, //U+81AC
0, //U+81AD
0, //U+81AE
0, //U+81AF
0, //U+81B0
0, //U+81B1
0, //U+81B2
0, //U+81B3
0, //U+81B4
0, //U+81B5
0, //U+81B6
0, //U+81B7
0, //U+81B8
0, //U+81B9
0, //U+81BA
0, //U+81BB
0, //U+81BC
0, //U+81BD
0, //U+81BE
0, //U+81BF
0, //U+81C0
0, //U+81C1
0, //U+81C2
0, //U+81C3
0, //U+81C4
0, //U+81C5
0, //U+81C6
0, //U+81C7
0, //U+81C8
0, //U+81C9
0, //U+81CA
0, //U+81CB
0, //U+81CC
0, //U+81CD
0, //U+81CE
0, //U+81CF
0, //U+81D0
0, //U+81D1
0, //U+81D2
0, //U+81D3
0, //U+81D4
0, //U+81D5
0, //U+81D6
0, //U+81D7
0, //U+81D8
0, //U+81D9
0, //U+81DA
0, //U+81DB
0, //U+81DC
0, //U+81DD
0, //U+81DE
0, //U+81DF
0, //U+81E0
0, //U+81E1
0, //U+81E2
0, //U+81E3
0, //U+81E4
0, //U+81E5
0, //U+81E6
0, //U+81E7
0, //U+81E8
0, //U+81E9
0, //U+81EA
0, //U+81EB
0, //U+81EC
0, //U+81ED
0, //U+81EE
0, //U+81EF
0, //U+81F0
0, //U+81F1
0, //U+81F2
0, //U+81F3
0, //U+81F4
0, //U+81F5
0, //U+81F6
0, //U+81F7
0, //U+81F8
0, //U+81F9
0, //U+81FA
0, //U+81FB
0, //U+81FC
0, //U+81FD
0, //U+81FE
0, //U+81FF
0, //U+8200
0, //U+8201
0, //U+8202
0, //U+8203
0, //U+8204
0, //U+8205
0, //U+8206
0, //U+8207
0, //U+8208
0, //U+8209
0, //U+820A
0, //U+820B
0, //U+820C
0, //U+820D
0, //U+820E
0, //U+820F
0, //U+8210
0, //U+8211
0, //U+8212
0, //U+8213
0, //U+8214
0, //U+8215
0, //U+8216
0, //U+8217
0, //U+8218
0, //U+8219
0, //U+821A
0, //U+821B
0, //U+821C
0, //U+821D
0, //U+821E
0, //U+821F
0, //U+8220
0, //U+8221
0, //U+8222
0, //U+8223
0, //U+8224
0, //U+8225
0, //U+8226
0, //U+8227
0, //U+8228
0, //U+8229
0, //U+822A
0, //U+822B
0, //U+822C
0, //U+822D
0, //U+822E
0, //U+822F
0, //U+8230
0, //U+8231
0, //U+8232
0, //U+8233
0, //U+8234
0, //U+8235
0, //U+8236
0, //U+8237
0, //U+8238
0, //U+8239
0, //U+823A
0, //U+823B
0, //U+823C
0, //U+823D
0, //U+823E
0, //U+823F
0, //U+8240
0, //U+8241
0, //U+8242
0, //U+8243
0, //U+8244
0, //U+8245
0, //U+8246
0, //U+8247
0, //U+8248
0, //U+8249
0, //U+824A
0, //U+824B
0, //U+824C
0, //U+824D
0, //U+824E
0, //U+824F
0, //U+8250
0, //U+8251
0, //U+8252
0, //U+8253
0, //U+8254
0, //U+8255
0, //U+8256
0, //U+8257
0, //U+8258
0, //U+8259
0, //U+825A
0, //U+825B
0, //U+825C
0, //U+825D
0, //U+825E
0, //U+825F
0, //U+8260
0, //U+8261
0, //U+8262
0, //U+8263
0, //U+8264
0, //U+8265
0, //U+8266
0, //U+8267
0, //U+8268
0, //U+8269
0, //U+826A
0, //U+826B
0, //U+826C
0, //U+826D
0, //U+826E
0, //U+826F
0, //U+8270
0, //U+8271
0, //U+8272
0, //U+8273
0, //U+8274
0, //U+8275
0, //U+8276
0, //U+8277
0, //U+8278
0, //U+8279
0, //U+827A
0, //U+827B
0, //U+827C
0, //U+827D
0, //U+827E
0, //U+827F
0, //U+8280
0, //U+8281
0, //U+8282
0, //U+8283
0, //U+8284
0, //U+8285
0, //U+8286
0, //U+8287
0, //U+8288
0, //U+8289
0, //U+828A
0, //U+828B
0, //U+828C
0, //U+828D
0, //U+828E
0, //U+828F
0, //U+8290
0, //U+8291
0, //U+8292
0, //U+8293
0, //U+8294
0, //U+8295
0, //U+8296
0, //U+8297
0, //U+8298
0, //U+8299
0, //U+829A
0, //U+829B
0, //U+829C
0, //U+829D
0, //U+829E
0, //U+829F
0, //U+82A0
0, //U+82A1
0, //U+82A2
0, //U+82A3
0, //U+82A4
0, //U+82A5
0, //U+82A6
0, //U+82A7
0, //U+82A8
0, //U+82A9
0, //U+82AA
0, //U+82AB
0, //U+82AC
0, //U+82AD
0, //U+82AE
0, //U+82AF
0, //U+82B0
0, //U+82B1
0, //U+82B2
0, //U+82B3
0, //U+82B4
0, //U+82B5
0, //U+82B6
0, //U+82B7
0, //U+82B8
0, //U+82B9
0, //U+82BA
0, //U+82BB
0, //U+82BC
0, //U+82BD
0, //U+82BE
0, //U+82BF
0, //U+82C0
0, //U+82C1
0, //U+82C2
0, //U+82C3
0, //U+82C4
0, //U+82C5
0, //U+82C6
0, //U+82C7
0, //U+82C8
0, //U+82C9
0, //U+82CA
0, //U+82CB
0, //U+82CC
0, //U+82CD
0, //U+82CE
0, //U+82CF
0, //U+82D0
0, //U+82D1
0, //U+82D2
0, //U+82D3
0, //U+82D4
0, //U+82D5
0, //U+82D6
0, //U+82D7
0, //U+82D8
0, //U+82D9
0, //U+82DA
0, //U+82DB
0, //U+82DC
0, //U+82DD
0, //U+82DE
0, //U+82DF
0, //U+82E0
0, //U+82E1
0, //U+82E2
0, //U+82E3
0, //U+82E4
0, //U+82E5
0, //U+82E6
0, //U+82E7
0, //U+82E8
0, //U+82E9
0, //U+82EA
0, //U+82EB
0, //U+82EC
0, //U+82ED
0, //U+82EE
0, //U+82EF
0, //U+82F0
0, //U+82F1
0, //U+82F2
0, //U+82F3
0, //U+82F4
0, //U+82F5
0, //U+82F6
0, //U+82F7
0, //U+82F8
0, //U+82F9
0, //U+82FA
0, //U+82FB
0, //U+82FC
0, //U+82FD
0, //U+82FE
0, //U+82FF
0, //U+8300
0, //U+8301
0, //U+8302
0, //U+8303
0, //U+8304
0, //U+8305
0, //U+8306
0, //U+8307
0, //U+8308
0, //U+8309
0, //U+830A
0, //U+830B
0, //U+830C
0, //U+830D
0, //U+830E
0, //U+830F
0, //U+8310
0, //U+8311
0, //U+8312
0, //U+8313
0, //U+8314
0, //U+8315
0, //U+8316
0, //U+8317
0, //U+8318
0, //U+8319
0, //U+831A
0, //U+831B
0, //U+831C
0, //U+831D
0, //U+831E
0, //U+831F
0, //U+8320
0, //U+8321
0, //U+8322
0, //U+8323
0, //U+8324
0, //U+8325
0, //U+8326
0, //U+8327
0, //U+8328
0, //U+8329
0, //U+832A
0, //U+832B
0, //U+832C
0, //U+832D
0, //U+832E
0, //U+832F
0, //U+8330
0, //U+8331
0, //U+8332
0, //U+8333
0, //U+8334
0, //U+8335
0, //U+8336
0, //U+8337
0, //U+8338
0, //U+8339
0, //U+833A
0, //U+833B
0, //U+833C
0, //U+833D
0, //U+833E
0, //U+833F
0, //U+8340
0, //U+8341
0, //U+8342
0, //U+8343
0, //U+8344
0, //U+8345
0, //U+8346
0, //U+8347
0, //U+8348
0, //U+8349
0, //U+834A
0, //U+834B
0, //U+834C
0, //U+834D
0, //U+834E
0, //U+834F
0, //U+8350
0, //U+8351
0, //U+8352
0, //U+8353
0, //U+8354
0, //U+8355
0, //U+8356
0, //U+8357
0, //U+8358
0, //U+8359
0, //U+835A
0, //U+835B
0, //U+835C
0, //U+835D
0, //U+835E
0, //U+835F
0, //U+8360
0, //U+8361
0, //U+8362
0, //U+8363
0, //U+8364
0, //U+8365
0, //U+8366
0, //U+8367
0, //U+8368
0, //U+8369
0, //U+836A
0, //U+836B
0, //U+836C
0, //U+836D
0, //U+836E
0, //U+836F
0, //U+8370
0, //U+8371
0, //U+8372
0, //U+8373
0, //U+8374
0, //U+8375
0, //U+8376
0, //U+8377
0, //U+8378
0, //U+8379
0, //U+837A
0, //U+837B
0, //U+837C
0, //U+837D
0, //U+837E
0, //U+837F
0, //U+8380
0, //U+8381
0, //U+8382
0, //U+8383
0, //U+8384
0, //U+8385
0, //U+8386
0, //U+8387
0, //U+8388
0, //U+8389
0, //U+838A
0, //U+838B
0, //U+838C
0, //U+838D
0, //U+838E
0, //U+838F
0, //U+8390
0, //U+8391
0, //U+8392
0, //U+8393
0, //U+8394
0, //U+8395
0, //U+8396
0, //U+8397
0, //U+8398
0, //U+8399
0, //U+839A
0, //U+839B
0, //U+839C
0, //U+839D
0, //U+839E
0, //U+839F
0, //U+83A0
0, //U+83A1
0, //U+83A2
0, //U+83A3
0, //U+83A4
0, //U+83A5
0, //U+83A6
0, //U+83A7
0, //U+83A8
0, //U+83A9
0, //U+83AA
0, //U+83AB
0, //U+83AC
0, //U+83AD
0, //U+83AE
0, //U+83AF
0, //U+83B0
0, //U+83B1
0, //U+83B2
0, //U+83B3
0, //U+83B4
0, //U+83B5
0, //U+83B6
0, //U+83B7
0, //U+83B8
0, //U+83B9
0, //U+83BA
0, //U+83BB
0, //U+83BC
0, //U+83BD
0, //U+83BE
0, //U+83BF
0, //U+83C0
0, //U+83C1
0, //U+83C2
0, //U+83C3
0, //U+83C4
0, //U+83C5
0, //U+83C6
0, //U+83C7
0, //U+83C8
0, //U+83C9
0, //U+83CA
0, //U+83CB
0, //U+83CC
0, //U+83CD
0, //U+83CE
0, //U+83CF
0, //U+83D0
0, //U+83D1
0, //U+83D2
0, //U+83D3
0, //U+83D4
0, //U+83D5
0, //U+83D6
0, //U+83D7
0, //U+83D8
0, //U+83D9
0, //U+83DA
0, //U+83DB
0, //U+83DC
0, //U+83DD
0, //U+83DE
0, //U+83DF
0, //U+83E0
0, //U+83E1
0, //U+83E2
0, //U+83E3
0, //U+83E4
0, //U+83E5
0, //U+83E6
0, //U+83E7
0, //U+83E8
0, //U+83E9
0, //U+83EA
0, //U+83EB
0, //U+83EC
0, //U+83ED
0, //U+83EE
0, //U+83EF
0, //U+83F0
0, //U+83F1
0, //U+83F2
0, //U+83F3
0, //U+83F4
0, //U+83F5
0, //U+83F6
0, //U+83F7
0, //U+83F8
0, //U+83F9
0, //U+83FA
0, //U+83FB
0, //U+83FC
0, //U+83FD
0, //U+83FE
0, //U+83FF
0, //U+8400
0, //U+8401
0, //U+8402
0, //U+8403
0, //U+8404
0, //U+8405
0, //U+8406
0, //U+8407
0, //U+8408
0, //U+8409
0, //U+840A
0, //U+840B
0, //U+840C
0, //U+840D
0, //U+840E
0, //U+840F
0, //U+8410
0, //U+8411
0, //U+8412
0, //U+8413
0, //U+8414
0, //U+8415
0, //U+8416
0, //U+8417
0, //U+8418
0, //U+8419
0, //U+841A
0, //U+841B
0, //U+841C
0, //U+841D
0, //U+841E
0, //U+841F
0, //U+8420
0, //U+8421
0, //U+8422
0, //U+8423
0, //U+8424
0, //U+8425
0, //U+8426
0, //U+8427
0, //U+8428
0, //U+8429
0, //U+842A
0, //U+842B
0, //U+842C
0, //U+842D
0, //U+842E
0, //U+842F
0, //U+8430
0, //U+8431
0, //U+8432
0, //U+8433
0, //U+8434
0, //U+8435
0, //U+8436
0, //U+8437
0, //U+8438
0, //U+8439
0, //U+843A
0, //U+843B
0, //U+843C
0, //U+843D
0, //U+843E
0, //U+843F
0, //U+8440
0, //U+8441
0, //U+8442
0, //U+8443
0, //U+8444
0, //U+8445
0, //U+8446
0, //U+8447
0, //U+8448
0, //U+8449
0, //U+844A
0, //U+844B
0, //U+844C
0, //U+844D
0, //U+844E
0, //U+844F
0, //U+8450
0, //U+8451
0, //U+8452
0, //U+8453
0, //U+8454
0, //U+8455
0, //U+8456
0, //U+8457
0, //U+8458
0, //U+8459
0, //U+845A
0, //U+845B
0, //U+845C
0, //U+845D
0, //U+845E
0, //U+845F
0, //U+8460
0, //U+8461
0, //U+8462
0, //U+8463
0, //U+8464
0, //U+8465
0, //U+8466
0, //U+8467
0, //U+8468
0, //U+8469
0, //U+846A
0, //U+846B
0, //U+846C
0, //U+846D
0, //U+846E
0, //U+846F
0, //U+8470
0, //U+8471
0, //U+8472
0, //U+8473
0, //U+8474
0, //U+8475
0, //U+8476
0, //U+8477
0, //U+8478
0, //U+8479
0, //U+847A
0, //U+847B
0, //U+847C
0, //U+847D
0, //U+847E
0, //U+847F
0, //U+8480
0, //U+8481
0, //U+8482
0, //U+8483
0, //U+8484
0, //U+8485
0, //U+8486
0, //U+8487
0, //U+8488
0, //U+8489
0, //U+848A
0, //U+848B
0, //U+848C
0, //U+848D
0, //U+848E
0, //U+848F
0, //U+8490
0, //U+8491
0, //U+8492
0, //U+8493
0, //U+8494
0, //U+8495
0, //U+8496
0, //U+8497
0, //U+8498
0, //U+8499
0, //U+849A
0, //U+849B
0, //U+849C
0, //U+849D
0, //U+849E
0, //U+849F
0, //U+84A0
0, //U+84A1
0, //U+84A2
0, //U+84A3
0, //U+84A4
0, //U+84A5
0, //U+84A6
0, //U+84A7
0, //U+84A8
0, //U+84A9
0, //U+84AA
0, //U+84AB
0, //U+84AC
0, //U+84AD
0, //U+84AE
0, //U+84AF
0, //U+84B0
0, //U+84B1
0, //U+84B2
0, //U+84B3
0, //U+84B4
0, //U+84B5
0, //U+84B6
0, //U+84B7
0, //U+84B8
0, //U+84B9
0, //U+84BA
0, //U+84BB
0, //U+84BC
0, //U+84BD
0, //U+84BE
0, //U+84BF
0, //U+84C0
0, //U+84C1
0, //U+84C2
0, //U+84C3
0, //U+84C4
0, //U+84C5
0, //U+84C6
0, //U+84C7
0, //U+84C8
0, //U+84C9
0, //U+84CA
0, //U+84CB
0, //U+84CC
0, //U+84CD
0, //U+84CE
0, //U+84CF
0, //U+84D0
0, //U+84D1
0, //U+84D2
0, //U+84D3
0, //U+84D4
0, //U+84D5
0, //U+84D6
0, //U+84D7
0, //U+84D8
0, //U+84D9
0, //U+84DA
0, //U+84DB
0, //U+84DC
0, //U+84DD
0, //U+84DE
0, //U+84DF
0, //U+84E0
0, //U+84E1
0, //U+84E2
0, //U+84E3
0, //U+84E4
0, //U+84E5
0, //U+84E6
0, //U+84E7
0, //U+84E8
0, //U+84E9
0, //U+84EA
0, //U+84EB
0, //U+84EC
0, //U+84ED
0, //U+84EE
0, //U+84EF
0, //U+84F0
0, //U+84F1
0, //U+84F2
0, //U+84F3
0, //U+84F4
0, //U+84F5
0, //U+84F6
0, //U+84F7
0, //U+84F8
0, //U+84F9
0, //U+84FA
0, //U+84FB
0, //U+84FC
0, //U+84FD
0, //U+84FE
0, //U+84FF
0, //U+8500
0, //U+8501
0, //U+8502
0, //U+8503
0, //U+8504
0, //U+8505
0, //U+8506
0, //U+8507
0, //U+8508
0, //U+8509
0, //U+850A
0, //U+850B
0, //U+850C
0, //U+850D
0, //U+850E
0, //U+850F
0, //U+8510
0, //U+8511
0, //U+8512
0, //U+8513
0, //U+8514
0, //U+8515
0, //U+8516
0, //U+8517
0, //U+8518
0, //U+8519
0, //U+851A
0, //U+851B
0, //U+851C
0, //U+851D
0, //U+851E
0, //U+851F
0, //U+8520
0, //U+8521
0, //U+8522
0, //U+8523
0, //U+8524
0, //U+8525
0, //U+8526
0, //U+8527
0, //U+8528
0, //U+8529
0, //U+852A
0, //U+852B
0, //U+852C
0, //U+852D
0, //U+852E
0, //U+852F
0, //U+8530
0, //U+8531
0, //U+8532
0, //U+8533
0, //U+8534
0, //U+8535
0, //U+8536
0, //U+8537
0, //U+8538
0, //U+8539
0, //U+853A
0, //U+853B
0, //U+853C
0, //U+853D
0, //U+853E
0, //U+853F
0, //U+8540
0, //U+8541
0, //U+8542
0, //U+8543
0, //U+8544
0, //U+8545
0, //U+8546
0, //U+8547
0, //U+8548
0, //U+8549
0, //U+854A
0, //U+854B
0, //U+854C
0, //U+854D
0, //U+854E
0, //U+854F
0, //U+8550
0, //U+8551
0, //U+8552
0, //U+8553
0, //U+8554
0, //U+8555
0, //U+8556
0, //U+8557
0, //U+8558
0, //U+8559
0, //U+855A
0, //U+855B
0, //U+855C
0, //U+855D
0, //U+855E
0, //U+855F
0, //U+8560
0, //U+8561
0, //U+8562
0, //U+8563
0, //U+8564
0, //U+8565
0, //U+8566
0, //U+8567
0, //U+8568
0, //U+8569
0, //U+856A
0, //U+856B
0, //U+856C
0, //U+856D
0, //U+856E
0, //U+856F
0, //U+8570
0, //U+8571
0, //U+8572
0, //U+8573
0, //U+8574
0, //U+8575
0, //U+8576
0, //U+8577
0, //U+8578
0, //U+8579
0, //U+857A
0, //U+857B
0, //U+857C
0, //U+857D
0, //U+857E
0, //U+857F
0, //U+8580
0, //U+8581
0, //U+8582
0, //U+8583
0, //U+8584
0, //U+8585
0, //U+8586
0, //U+8587
0, //U+8588
0, //U+8589
0, //U+858A
0, //U+858B
0, //U+858C
0, //U+858D
0, //U+858E
0, //U+858F
0, //U+8590
0, //U+8591
0, //U+8592
0, //U+8593
0, //U+8594
0, //U+8595
0, //U+8596
0, //U+8597
0, //U+8598
0, //U+8599
0, //U+859A
0, //U+859B
0, //U+859C
0, //U+859D
0, //U+859E
0, //U+859F
0, //U+85A0
0, //U+85A1
0, //U+85A2
0, //U+85A3
0, //U+85A4
0, //U+85A5
0, //U+85A6
0, //U+85A7
0, //U+85A8
0, //U+85A9
0, //U+85AA
0, //U+85AB
0, //U+85AC
0, //U+85AD
0, //U+85AE
0, //U+85AF
0, //U+85B0
0, //U+85B1
0, //U+85B2
0, //U+85B3
0, //U+85B4
0, //U+85B5
0, //U+85B6
0, //U+85B7
0, //U+85B8
0, //U+85B9
0, //U+85BA
0, //U+85BB
0, //U+85BC
0, //U+85BD
0, //U+85BE
0, //U+85BF
0, //U+85C0
0, //U+85C1
0, //U+85C2
0, //U+85C3
0, //U+85C4
0, //U+85C5
0, //U+85C6
0, //U+85C7
0, //U+85C8
0, //U+85C9
0, //U+85CA
0, //U+85CB
0, //U+85CC
0, //U+85CD
0, //U+85CE
0, //U+85CF
0, //U+85D0
0, //U+85D1
0, //U+85D2
0, //U+85D3
0, //U+85D4
0, //U+85D5
0, //U+85D6
0, //U+85D7
0, //U+85D8
0, //U+85D9
0, //U+85DA
0, //U+85DB
0, //U+85DC
0, //U+85DD
0, //U+85DE
0, //U+85DF
0, //U+85E0
0, //U+85E1
0, //U+85E2
0, //U+85E3
0, //U+85E4
0, //U+85E5
0, //U+85E6
0, //U+85E7
0, //U+85E8
0, //U+85E9
0, //U+85EA
0, //U+85EB
0, //U+85EC
0, //U+85ED
0, //U+85EE
0, //U+85EF
0, //U+85F0
0, //U+85F1
0, //U+85F2
0, //U+85F3
0, //U+85F4
0, //U+85F5
0, //U+85F6
0, //U+85F7
0, //U+85F8
0, //U+85F9
0, //U+85FA
0, //U+85FB
0, //U+85FC
0, //U+85FD
0, //U+85FE
0, //U+85FF
0, //U+8600
0, //U+8601
0, //U+8602
0, //U+8603
0, //U+8604
0, //U+8605
0, //U+8606
0, //U+8607
0, //U+8608
0, //U+8609
0, //U+860A
0, //U+860B
0, //U+860C
0, //U+860D
0, //U+860E
0, //U+860F
0, //U+8610
0, //U+8611
0, //U+8612
0, //U+8613
0, //U+8614
0, //U+8615
0, //U+8616
0, //U+8617
0, //U+8618
0, //U+8619
0, //U+861A
0, //U+861B
0, //U+861C
0, //U+861D
0, //U+861E
0, //U+861F
0, //U+8620
0, //U+8621
0, //U+8622
0, //U+8623
0, //U+8624
0, //U+8625
0, //U+8626
0, //U+8627
0, //U+8628
0, //U+8629
0, //U+862A
0, //U+862B
0, //U+862C
0, //U+862D
0, //U+862E
0, //U+862F
0, //U+8630
0, //U+8631
0, //U+8632
0, //U+8633
0, //U+8634
0, //U+8635
0, //U+8636
0, //U+8637
0, //U+8638
0, //U+8639
0, //U+863A
0, //U+863B
0, //U+863C
0, //U+863D
0, //U+863E
0, //U+863F
0, //U+8640
0, //U+8641
0, //U+8642
0, //U+8643
0, //U+8644
0, //U+8645
0, //U+8646
0, //U+8647
0, //U+8648
0, //U+8649
0, //U+864A
0, //U+864B
0, //U+864C
0, //U+864D
0, //U+864E
0, //U+864F
0, //U+8650
0, //U+8651
0, //U+8652
0, //U+8653
0, //U+8654
0, //U+8655
0, //U+8656
0, //U+8657
0, //U+8658
0, //U+8659
0, //U+865A
0, //U+865B
0, //U+865C
0, //U+865D
0, //U+865E
0, //U+865F
0, //U+8660
0, //U+8661
0, //U+8662
0, //U+8663
0, //U+8664
0, //U+8665
0, //U+8666
0, //U+8667
0, //U+8668
0, //U+8669
0, //U+866A
0, //U+866B
0, //U+866C
0, //U+866D
0, //U+866E
0, //U+866F
0, //U+8670
0, //U+8671
0, //U+8672
0, //U+8673
0, //U+8674
0, //U+8675
0, //U+8676
0, //U+8677
0, //U+8678
0, //U+8679
0, //U+867A
0, //U+867B
0, //U+867C
0, //U+867D
0, //U+867E
0, //U+867F
0, //U+8680
0, //U+8681
0, //U+8682
0, //U+8683
0, //U+8684
0, //U+8685
0, //U+8686
0, //U+8687
0, //U+8688
0, //U+8689
0, //U+868A
0, //U+868B
0, //U+868C
0, //U+868D
0, //U+868E
0, //U+868F
0, //U+8690
0, //U+8691
0, //U+8692
0, //U+8693
0, //U+8694
0, //U+8695
0, //U+8696
0, //U+8697
0, //U+8698
0, //U+8699
0, //U+869A
0, //U+869B
0, //U+869C
0, //U+869D
0, //U+869E
0, //U+869F
0, //U+86A0
0, //U+86A1
0, //U+86A2
0, //U+86A3
0, //U+86A4
0, //U+86A5
0, //U+86A6
0, //U+86A7
0, //U+86A8
0, //U+86A9
0, //U+86AA
0, //U+86AB
0, //U+86AC
0, //U+86AD
0, //U+86AE
0, //U+86AF
0, //U+86B0
0, //U+86B1
0, //U+86B2
0, //U+86B3
0, //U+86B4
0, //U+86B5
0, //U+86B6
0, //U+86B7
0, //U+86B8
0, //U+86B9
0, //U+86BA
0, //U+86BB
0, //U+86BC
0, //U+86BD
0, //U+86BE
0, //U+86BF
0, //U+86C0
0, //U+86C1
0, //U+86C2
0, //U+86C3
0, //U+86C4
0, //U+86C5
0, //U+86C6
0, //U+86C7
0, //U+86C8
0, //U+86C9
0, //U+86CA
0, //U+86CB
0, //U+86CC
0, //U+86CD
0, //U+86CE
0, //U+86CF
0, //U+86D0
0, //U+86D1
0, //U+86D2
0, //U+86D3
0, //U+86D4
0, //U+86D5
0, //U+86D6
0, //U+86D7
0, //U+86D8
0, //U+86D9
0, //U+86DA
0, //U+86DB
0, //U+86DC
0, //U+86DD
0, //U+86DE
0, //U+86DF
0, //U+86E0
0, //U+86E1
0, //U+86E2
0, //U+86E3
0, //U+86E4
0, //U+86E5
0, //U+86E6
0, //U+86E7
0, //U+86E8
0, //U+86E9
0, //U+86EA
0, //U+86EB
0, //U+86EC
0, //U+86ED
0, //U+86EE
0, //U+86EF
0, //U+86F0
0, //U+86F1
0, //U+86F2
0, //U+86F3
0, //U+86F4
0, //U+86F5
0, //U+86F6
0, //U+86F7
0, //U+86F8
0, //U+86F9
0, //U+86FA
0, //U+86FB
0, //U+86FC
0, //U+86FD
0, //U+86FE
0, //U+86FF
0, //U+8700
0, //U+8701
0, //U+8702
0, //U+8703
0, //U+8704
0, //U+8705
0, //U+8706
0, //U+8707
0, //U+8708
0, //U+8709
0, //U+870A
0, //U+870B
0, //U+870C
0, //U+870D
0, //U+870E
0, //U+870F
0, //U+8710
0, //U+8711
0, //U+8712
0, //U+8713
0, //U+8714
0, //U+8715
0, //U+8716
0, //U+8717
0, //U+8718
0, //U+8719
0, //U+871A
0, //U+871B
0, //U+871C
0, //U+871D
0, //U+871E
0, //U+871F
0, //U+8720
0, //U+8721
0, //U+8722
0, //U+8723
0, //U+8724
0, //U+8725
0, //U+8726
0, //U+8727
0, //U+8728
0, //U+8729
0, //U+872A
0, //U+872B
0, //U+872C
0, //U+872D
0, //U+872E
0, //U+872F
0, //U+8730
0, //U+8731
0, //U+8732
0, //U+8733
0, //U+8734
0, //U+8735
0, //U+8736
0, //U+8737
0, //U+8738
0, //U+8739
0, //U+873A
0, //U+873B
0, //U+873C
0, //U+873D
0, //U+873E
0, //U+873F
0, //U+8740
0, //U+8741
0, //U+8742
0, //U+8743
0, //U+8744
0, //U+8745
0, //U+8746
0, //U+8747
0, //U+8748
0, //U+8749
0, //U+874A
0, //U+874B
0, //U+874C
0, //U+874D
0, //U+874E
0, //U+874F
0, //U+8750
0, //U+8751
0, //U+8752
0, //U+8753
0, //U+8754
0, //U+8755
0, //U+8756
0, //U+8757
0, //U+8758
0, //U+8759
0, //U+875A
0, //U+875B
0, //U+875C
0, //U+875D
0, //U+875E
0, //U+875F
0, //U+8760
0, //U+8761
0, //U+8762
0, //U+8763
0, //U+8764
0, //U+8765
0, //U+8766
0, //U+8767
0, //U+8768
0, //U+8769
0, //U+876A
0, //U+876B
0, //U+876C
0, //U+876D
0, //U+876E
0, //U+876F
0, //U+8770
0, //U+8771
0, //U+8772
0, //U+8773
0, //U+8774
0, //U+8775
0, //U+8776
0, //U+8777
0, //U+8778
0, //U+8779
0, //U+877A
0, //U+877B
0, //U+877C
0, //U+877D
0, //U+877E
0, //U+877F
0, //U+8780
0, //U+8781
0, //U+8782
0, //U+8783
0, //U+8784
0, //U+8785
0, //U+8786
0, //U+8787
0, //U+8788
0, //U+8789
0, //U+878A
0, //U+878B
0, //U+878C
0, //U+878D
0, //U+878E
0, //U+878F
0, //U+8790
0, //U+8791
0, //U+8792
0, //U+8793
0, //U+8794
0, //U+8795
0, //U+8796
0, //U+8797
0, //U+8798
0, //U+8799
0, //U+879A
0, //U+879B
0, //U+879C
0, //U+879D
0, //U+879E
0, //U+879F
0, //U+87A0
0, //U+87A1
0, //U+87A2
0, //U+87A3
0, //U+87A4
0, //U+87A5
0, //U+87A6
0, //U+87A7
0, //U+87A8
0, //U+87A9
0, //U+87AA
0, //U+87AB
0, //U+87AC
0, //U+87AD
0, //U+87AE
0, //U+87AF
0, //U+87B0
0, //U+87B1
0, //U+87B2
0, //U+87B3
0, //U+87B4
0, //U+87B5
0, //U+87B6
0, //U+87B7
0, //U+87B8
0, //U+87B9
0, //U+87BA
0, //U+87BB
0, //U+87BC
0, //U+87BD
0, //U+87BE
0, //U+87BF
0, //U+87C0
0, //U+87C1
0, //U+87C2
0, //U+87C3
0, //U+87C4
0, //U+87C5
0, //U+87C6
0, //U+87C7
0, //U+87C8
0, //U+87C9
0, //U+87CA
0, //U+87CB
0, //U+87CC
0, //U+87CD
0, //U+87CE
0, //U+87CF
0, //U+87D0
0, //U+87D1
0, //U+87D2
0, //U+87D3
0, //U+87D4
0, //U+87D5
0, //U+87D6
0, //U+87D7
0, //U+87D8
0, //U+87D9
0, //U+87DA
0, //U+87DB
0, //U+87DC
0, //U+87DD
0, //U+87DE
0, //U+87DF
0, //U+87E0
0, //U+87E1
0, //U+87E2
0, //U+87E3
0, //U+87E4
0, //U+87E5
0, //U+87E6
0, //U+87E7
0, //U+87E8
0, //U+87E9
0, //U+87EA
0, //U+87EB
0, //U+87EC
0, //U+87ED
0, //U+87EE
0, //U+87EF
0, //U+87F0
0, //U+87F1
0, //U+87F2
0, //U+87F3
0, //U+87F4
0, //U+87F5
0, //U+87F6
0, //U+87F7
0, //U+87F8
0, //U+87F9
0, //U+87FA
0, //U+87FB
0, //U+87FC
0, //U+87FD
0, //U+87FE
0, //U+87FF
0, //U+8800
0, //U+8801
0, //U+8802
0, //U+8803
0, //U+8804
0, //U+8805
0, //U+8806
0, //U+8807
0, //U+8808
0, //U+8809
0, //U+880A
0, //U+880B
0, //U+880C
0, //U+880D
0, //U+880E
0, //U+880F
0, //U+8810
0, //U+8811
0, //U+8812
0, //U+8813
0, //U+8814
0, //U+8815
0, //U+8816
0, //U+8817
0, //U+8818
0, //U+8819
0, //U+881A
0, //U+881B
0, //U+881C
0, //U+881D
0, //U+881E
0, //U+881F
0, //U+8820
0, //U+8821
0, //U+8822
0, //U+8823
0, //U+8824
0, //U+8825
0, //U+8826
0, //U+8827
0, //U+8828
0, //U+8829
0, //U+882A
0, //U+882B
0, //U+882C
0, //U+882D
0, //U+882E
0, //U+882F
0, //U+8830
0, //U+8831
0, //U+8832
0, //U+8833
0, //U+8834
0, //U+8835
0, //U+8836
0, //U+8837
0, //U+8838
0, //U+8839
0, //U+883A
0, //U+883B
0, //U+883C
0, //U+883D
0, //U+883E
0, //U+883F
0, //U+8840
0, //U+8841
0, //U+8842
0, //U+8843
0, //U+8844
0, //U+8845
0, //U+8846
0, //U+8847
0, //U+8848
0, //U+8849
0, //U+884A
0, //U+884B
0, //U+884C
0, //U+884D
0, //U+884E
0, //U+884F
0, //U+8850
0, //U+8851
0, //U+8852
0, //U+8853
0, //U+8854
0, //U+8855
0, //U+8856
0, //U+8857
0, //U+8858
0, //U+8859
0, //U+885A
0, //U+885B
0, //U+885C
0, //U+885D
0, //U+885E
0, //U+885F
0, //U+8860
0, //U+8861
0, //U+8862
0, //U+8863
0, //U+8864
0, //U+8865
0, //U+8866
0, //U+8867
0, //U+8868
0, //U+8869
0, //U+886A
0, //U+886B
0, //U+886C
0, //U+886D
0, //U+886E
0, //U+886F
0, //U+8870
0, //U+8871
0, //U+8872
0, //U+8873
0, //U+8874
0, //U+8875
0, //U+8876
0, //U+8877
0, //U+8878
0, //U+8879
0, //U+887A
0, //U+887B
0, //U+887C
0, //U+887D
0, //U+887E
0, //U+887F
0, //U+8880
0, //U+8881
0, //U+8882
0, //U+8883
0, //U+8884
0, //U+8885
0, //U+8886
0, //U+8887
0, //U+8888
0, //U+8889
0, //U+888A
0, //U+888B
0, //U+888C
0, //U+888D
0, //U+888E
0, //U+888F
0, //U+8890
0, //U+8891
0, //U+8892
0, //U+8893
0, //U+8894
0, //U+8895
0, //U+8896
0, //U+8897
0, //U+8898
0, //U+8899
0, //U+889A
0, //U+889B
0, //U+889C
0, //U+889D
0, //U+889E
0, //U+889F
0, //U+88A0
0, //U+88A1
0, //U+88A2
0, //U+88A3
0, //U+88A4
0, //U+88A5
0, //U+88A6
0, //U+88A7
0, //U+88A8
0, //U+88A9
0, //U+88AA
0, //U+88AB
0, //U+88AC
0, //U+88AD
0, //U+88AE
0, //U+88AF
0, //U+88B0
0, //U+88B1
0, //U+88B2
0, //U+88B3
0, //U+88B4
0, //U+88B5
0, //U+88B6
0, //U+88B7
0, //U+88B8
0, //U+88B9
0, //U+88BA
0, //U+88BB
0, //U+88BC
0, //U+88BD
0, //U+88BE
0, //U+88BF
0, //U+88C0
0, //U+88C1
0, //U+88C2
0, //U+88C3
0, //U+88C4
0, //U+88C5
0, //U+88C6
0, //U+88C7
0, //U+88C8
0, //U+88C9
0, //U+88CA
0, //U+88CB
0, //U+88CC
0, //U+88CD
0, //U+88CE
0, //U+88CF
0, //U+88D0
0, //U+88D1
0, //U+88D2
0, //U+88D3
0, //U+88D4
0, //U+88D5
0, //U+88D6
0, //U+88D7
0, //U+88D8
0, //U+88D9
0, //U+88DA
0, //U+88DB
0, //U+88DC
0, //U+88DD
0, //U+88DE
0, //U+88DF
0, //U+88E0
0, //U+88E1
0, //U+88E2
0, //U+88E3
0, //U+88E4
0, //U+88E5
0, //U+88E6
0, //U+88E7
0, //U+88E8
0, //U+88E9
0, //U+88EA
0, //U+88EB
0, //U+88EC
0, //U+88ED
0, //U+88EE
0, //U+88EF
0, //U+88F0
0, //U+88F1
0, //U+88F2
0, //U+88F3
0, //U+88F4
0, //U+88F5
0, //U+88F6
0, //U+88F7
0, //U+88F8
0, //U+88F9
0, //U+88FA
0, //U+88FB
0, //U+88FC
0, //U+88FD
0, //U+88FE
0, //U+88FF
0, //U+8900
0, //U+8901
0, //U+8902
0, //U+8903
0, //U+8904
0, //U+8905
0, //U+8906
0, //U+8907
0, //U+8908
0, //U+8909
0, //U+890A
0, //U+890B
0, //U+890C
0, //U+890D
0, //U+890E
0, //U+890F
0, //U+8910
0, //U+8911
0, //U+8912
0, //U+8913
0, //U+8914
0, //U+8915
0, //U+8916
0, //U+8917
0, //U+8918
0, //U+8919
0, //U+891A
0, //U+891B
0, //U+891C
0, //U+891D
0, //U+891E
0, //U+891F
0, //U+8920
0, //U+8921
0, //U+8922
0, //U+8923
0, //U+8924
0, //U+8925
0, //U+8926
0, //U+8927
0, //U+8928
0, //U+8929
0, //U+892A
0, //U+892B
0, //U+892C
0, //U+892D
0, //U+892E
0, //U+892F
0, //U+8930
0, //U+8931
0, //U+8932
0, //U+8933
0, //U+8934
0, //U+8935
0, //U+8936
0, //U+8937
0, //U+8938
0, //U+8939
0, //U+893A
0, //U+893B
0, //U+893C
0, //U+893D
0, //U+893E
0, //U+893F
0, //U+8940
0, //U+8941
0, //U+8942
0, //U+8943
0, //U+8944
0, //U+8945
0, //U+8946
0, //U+8947
0, //U+8948
0, //U+8949
0, //U+894A
0, //U+894B
0, //U+894C
0, //U+894D
0, //U+894E
0, //U+894F
0, //U+8950
0, //U+8951
0, //U+8952
0, //U+8953
0, //U+8954
0, //U+8955
0, //U+8956
0, //U+8957
0, //U+8958
0, //U+8959
0, //U+895A
0, //U+895B
0, //U+895C
0, //U+895D
0, //U+895E
0, //U+895F
0, //U+8960
0, //U+8961
0, //U+8962
0, //U+8963
0, //U+8964
0, //U+8965
0, //U+8966
0, //U+8967
0, //U+8968
0, //U+8969
0, //U+896A
0, //U+896B
0, //U+896C
0, //U+896D
0, //U+896E
0, //U+896F
0, //U+8970
0, //U+8971
0, //U+8972
0, //U+8973
0, //U+8974
0, //U+8975
0, //U+8976
0, //U+8977
0, //U+8978
0, //U+8979
0, //U+897A
0, //U+897B
0, //U+897C
0, //U+897D
0, //U+897E
0, //U+897F
0, //U+8980
0, //U+8981
0, //U+8982
0, //U+8983
0, //U+8984
0, //U+8985
0, //U+8986
0, //U+8987
0, //U+8988
0, //U+8989
0, //U+898A
0, //U+898B
0, //U+898C
0, //U+898D
0, //U+898E
0, //U+898F
0, //U+8990
0, //U+8991
0, //U+8992
0, //U+8993
0, //U+8994
0, //U+8995
0, //U+8996
0, //U+8997
0, //U+8998
0, //U+8999
0, //U+899A
0, //U+899B
0, //U+899C
0, //U+899D
0, //U+899E
0, //U+899F
0, //U+89A0
0, //U+89A1
0, //U+89A2
0, //U+89A3
0, //U+89A4
0, //U+89A5
0, //U+89A6
0, //U+89A7
0, //U+89A8
0, //U+89A9
0, //U+89AA
0, //U+89AB
0, //U+89AC
0, //U+89AD
0, //U+89AE
0, //U+89AF
0, //U+89B0
0, //U+89B1
0, //U+89B2
0, //U+89B3
0, //U+89B4
0, //U+89B5
0, //U+89B6
0, //U+89B7
0, //U+89B8
0, //U+89B9
0, //U+89BA
0, //U+89BB
0, //U+89BC
0, //U+89BD
0, //U+89BE
0, //U+89BF
0, //U+89C0
0, //U+89C1
0, //U+89C2
0, //U+89C3
0, //U+89C4
0, //U+89C5
0, //U+89C6
0, //U+89C7
0, //U+89C8
0, //U+89C9
0, //U+89CA
0, //U+89CB
0, //U+89CC
0, //U+89CD
0, //U+89CE
0, //U+89CF
0, //U+89D0
0, //U+89D1
0, //U+89D2
0, //U+89D3
0, //U+89D4
0, //U+89D5
0, //U+89D6
0, //U+89D7
0, //U+89D8
0, //U+89D9
0, //U+89DA
0, //U+89DB
0, //U+89DC
0, //U+89DD
0, //U+89DE
0, //U+89DF
0, //U+89E0
0, //U+89E1
0, //U+89E2
0, //U+89E3
0, //U+89E4
0, //U+89E5
0, //U+89E6
0, //U+89E7
0, //U+89E8
0, //U+89E9
0, //U+89EA
0, //U+89EB
0, //U+89EC
0, //U+89ED
0, //U+89EE
0, //U+89EF
0, //U+89F0
0, //U+89F1
0, //U+89F2
0, //U+89F3
0, //U+89F4
0, //U+89F5
0, //U+89F6
0, //U+89F7
0, //U+89F8
0, //U+89F9
0, //U+89FA
0, //U+89FB
0, //U+89FC
0, //U+89FD
0, //U+89FE
0, //U+89FF
0, //U+8A00
0, //U+8A01
0, //U+8A02
0, //U+8A03
0, //U+8A04
0, //U+8A05
0, //U+8A06
0, //U+8A07
0, //U+8A08
0, //U+8A09
0, //U+8A0A
0, //U+8A0B
0, //U+8A0C
0, //U+8A0D
0, //U+8A0E
0, //U+8A0F
0, //U+8A10
0, //U+8A11
0, //U+8A12
0, //U+8A13
0, //U+8A14
0, //U+8A15
0, //U+8A16
0, //U+8A17
0, //U+8A18
0, //U+8A19
0, //U+8A1A
0, //U+8A1B
0, //U+8A1C
0, //U+8A1D
0, //U+8A1E
0, //U+8A1F
0, //U+8A20
0, //U+8A21
0, //U+8A22
0, //U+8A23
0, //U+8A24
0, //U+8A25
0, //U+8A26
0, //U+8A27
0, //U+8A28
0, //U+8A29
0, //U+8A2A
0, //U+8A2B
0, //U+8A2C
0, //U+8A2D
0, //U+8A2E
0, //U+8A2F
0, //U+8A30
0, //U+8A31
0, //U+8A32
0, //U+8A33
0, //U+8A34
0, //U+8A35
0, //U+8A36
0, //U+8A37
0, //U+8A38
0, //U+8A39
0, //U+8A3A
0, //U+8A3B
0, //U+8A3C
0, //U+8A3D
0, //U+8A3E
0, //U+8A3F
0, //U+8A40
0, //U+8A41
0, //U+8A42
0, //U+8A43
0, //U+8A44
0, //U+8A45
0, //U+8A46
0, //U+8A47
0, //U+8A48
0, //U+8A49
0, //U+8A4A
0, //U+8A4B
0, //U+8A4C
0, //U+8A4D
0, //U+8A4E
0, //U+8A4F
0, //U+8A50
0, //U+8A51
0, //U+8A52
0, //U+8A53
0, //U+8A54
0, //U+8A55
0, //U+8A56
0, //U+8A57
0, //U+8A58
0, //U+8A59
0, //U+8A5A
0, //U+8A5B
0, //U+8A5C
0, //U+8A5D
0, //U+8A5E
0, //U+8A5F
0, //U+8A60
0, //U+8A61
0, //U+8A62
0, //U+8A63
0, //U+8A64
0, //U+8A65
0, //U+8A66
0, //U+8A67
0, //U+8A68
0, //U+8A69
0, //U+8A6A
0, //U+8A6B
0, //U+8A6C
0, //U+8A6D
0, //U+8A6E
0, //U+8A6F
0, //U+8A70
0, //U+8A71
0, //U+8A72
0, //U+8A73
0, //U+8A74
0, //U+8A75
0, //U+8A76
0, //U+8A77
0, //U+8A78
0, //U+8A79
0, //U+8A7A
0, //U+8A7B
0, //U+8A7C
0, //U+8A7D
0, //U+8A7E
0, //U+8A7F
0, //U+8A80
0, //U+8A81
0, //U+8A82
0, //U+8A83
0, //U+8A84
0, //U+8A85
0, //U+8A86
0, //U+8A87
0, //U+8A88
0, //U+8A89
0, //U+8A8A
0, //U+8A8B
0, //U+8A8C
0, //U+8A8D
0, //U+8A8E
0, //U+8A8F
0, //U+8A90
0, //U+8A91
0, //U+8A92
0, //U+8A93
0, //U+8A94
0, //U+8A95
0, //U+8A96
0, //U+8A97
0, //U+8A98
0, //U+8A99
0, //U+8A9A
0, //U+8A9B
0, //U+8A9C
0, //U+8A9D
0, //U+8A9E
0, //U+8A9F
0, //U+8AA0
0, //U+8AA1
0, //U+8AA2
0, //U+8AA3
0, //U+8AA4
0, //U+8AA5
0, //U+8AA6
0, //U+8AA7
0, //U+8AA8
0, //U+8AA9
0, //U+8AAA
0, //U+8AAB
0, //U+8AAC
0, //U+8AAD
0, //U+8AAE
0, //U+8AAF
0, //U+8AB0
0, //U+8AB1
0, //U+8AB2
0, //U+8AB3
0, //U+8AB4
0, //U+8AB5
0, //U+8AB6
0, //U+8AB7
0, //U+8AB8
0, //U+8AB9
0, //U+8ABA
0, //U+8ABB
0, //U+8ABC
0, //U+8ABD
0, //U+8ABE
0, //U+8ABF
0, //U+8AC0
0, //U+8AC1
0, //U+8AC2
0, //U+8AC3
0, //U+8AC4
0, //U+8AC5
0, //U+8AC6
0, //U+8AC7
0, //U+8AC8
0, //U+8AC9
0, //U+8ACA
0, //U+8ACB
0, //U+8ACC
0, //U+8ACD
0, //U+8ACE
0, //U+8ACF
0, //U+8AD0
0, //U+8AD1
0, //U+8AD2
0, //U+8AD3
0, //U+8AD4
0, //U+8AD5
0, //U+8AD6
0, //U+8AD7
0, //U+8AD8
0, //U+8AD9
0, //U+8ADA
0, //U+8ADB
0, //U+8ADC
0, //U+8ADD
0, //U+8ADE
0, //U+8ADF
0, //U+8AE0
0, //U+8AE1
0, //U+8AE2
0, //U+8AE3
0, //U+8AE4
0, //U+8AE5
0, //U+8AE6
0, //U+8AE7
0, //U+8AE8
0, //U+8AE9
0, //U+8AEA
0, //U+8AEB
0, //U+8AEC
0, //U+8AED
0, //U+8AEE
0, //U+8AEF
0, //U+8AF0
0, //U+8AF1
0, //U+8AF2
0, //U+8AF3
0, //U+8AF4
0, //U+8AF5
0, //U+8AF6
0, //U+8AF7
0, //U+8AF8
0, //U+8AF9
0, //U+8AFA
0, //U+8AFB
0, //U+8AFC
0, //U+8AFD
0, //U+8AFE
0, //U+8AFF
0, //U+8B00
0, //U+8B01
0, //U+8B02
0, //U+8B03
0, //U+8B04
0, //U+8B05
0, //U+8B06
0, //U+8B07
0, //U+8B08
0, //U+8B09
0, //U+8B0A
0, //U+8B0B
0, //U+8B0C
0, //U+8B0D
0, //U+8B0E
0, //U+8B0F
0, //U+8B10
0, //U+8B11
0, //U+8B12
0, //U+8B13
0, //U+8B14
0, //U+8B15
0, //U+8B16
0, //U+8B17
0, //U+8B18
0, //U+8B19
0, //U+8B1A
0, //U+8B1B
0, //U+8B1C
0, //U+8B1D
0, //U+8B1E
0, //U+8B1F
0, //U+8B20
0, //U+8B21
0, //U+8B22
0, //U+8B23
0, //U+8B24
0, //U+8B25
0, //U+8B26
0, //U+8B27
0, //U+8B28
0, //U+8B29
0, //U+8B2A
0, //U+8B2B
0, //U+8B2C
0, //U+8B2D
0, //U+8B2E
0, //U+8B2F
0, //U+8B30
0, //U+8B31
0, //U+8B32
0, //U+8B33
0, //U+8B34
0, //U+8B35
0, //U+8B36
0, //U+8B37
0, //U+8B38
0, //U+8B39
0, //U+8B3A
0, //U+8B3B
0, //U+8B3C
0, //U+8B3D
0, //U+8B3E
0, //U+8B3F
0, //U+8B40
0, //U+8B41
0, //U+8B42
0, //U+8B43
0, //U+8B44
0, //U+8B45
0, //U+8B46
0, //U+8B47
0, //U+8B48
0, //U+8B49
0, //U+8B4A
0, //U+8B4B
0, //U+8B4C
0, //U+8B4D
0, //U+8B4E
0, //U+8B4F
0, //U+8B50
0, //U+8B51
0, //U+8B52
0, //U+8B53
0, //U+8B54
0, //U+8B55
0, //U+8B56
0, //U+8B57
0, //U+8B58
0, //U+8B59
0, //U+8B5A
0, //U+8B5B
0, //U+8B5C
0, //U+8B5D
0, //U+8B5E
0, //U+8B5F
0, //U+8B60
0, //U+8B61
0, //U+8B62
0, //U+8B63
0, //U+8B64
0, //U+8B65
0, //U+8B66
0, //U+8B67
0, //U+8B68
0, //U+8B69
0, //U+8B6A
0, //U+8B6B
0, //U+8B6C
0, //U+8B6D
0, //U+8B6E
0, //U+8B6F
0, //U+8B70
0, //U+8B71
0, //U+8B72
0, //U+8B73
0, //U+8B74
0, //U+8B75
0, //U+8B76
0, //U+8B77
0, //U+8B78
0, //U+8B79
0, //U+8B7A
0, //U+8B7B
0, //U+8B7C
0, //U+8B7D
0, //U+8B7E
0, //U+8B7F
0, //U+8B80
0, //U+8B81
0, //U+8B82
0, //U+8B83
0, //U+8B84
0, //U+8B85
0, //U+8B86
0, //U+8B87
0, //U+8B88
0, //U+8B89
0, //U+8B8A
0, //U+8B8B
0, //U+8B8C
0, //U+8B8D
0, //U+8B8E
0, //U+8B8F
0, //U+8B90
0, //U+8B91
0, //U+8B92
0, //U+8B93
0, //U+8B94
0, //U+8B95
0, //U+8B96
0, //U+8B97
0, //U+8B98
0, //U+8B99
0, //U+8B9A
0, //U+8B9B
0, //U+8B9C
0, //U+8B9D
0, //U+8B9E
0, //U+8B9F
0, //U+8BA0
0, //U+8BA1
0, //U+8BA2
0, //U+8BA3
0, //U+8BA4
0, //U+8BA5
0, //U+8BA6
0, //U+8BA7
0, //U+8BA8
0, //U+8BA9
0, //U+8BAA
0, //U+8BAB
0, //U+8BAC
0, //U+8BAD
0, //U+8BAE
0, //U+8BAF
0, //U+8BB0
0, //U+8BB1
0, //U+8BB2
0, //U+8BB3
0, //U+8BB4
0, //U+8BB5
0, //U+8BB6
0, //U+8BB7
0, //U+8BB8
0, //U+8BB9
0, //U+8BBA
0, //U+8BBB
0, //U+8BBC
0, //U+8BBD
0, //U+8BBE
0, //U+8BBF
0, //U+8BC0
0, //U+8BC1
0, //U+8BC2
0, //U+8BC3
0, //U+8BC4
0, //U+8BC5
0, //U+8BC6
0, //U+8BC7
0, //U+8BC8
0, //U+8BC9
0, //U+8BCA
0, //U+8BCB
0, //U+8BCC
0, //U+8BCD
0, //U+8BCE
0, //U+8BCF
0, //U+8BD0
0, //U+8BD1
0, //U+8BD2
0, //U+8BD3
0, //U+8BD4
0, //U+8BD5
0, //U+8BD6
0, //U+8BD7
0, //U+8BD8
0, //U+8BD9
0, //U+8BDA
0, //U+8BDB
0, //U+8BDC
0, //U+8BDD
0, //U+8BDE
0, //U+8BDF
0, //U+8BE0
0, //U+8BE1
0, //U+8BE2
0, //U+8BE3
0, //U+8BE4
0, //U+8BE5
0, //U+8BE6
0, //U+8BE7
0, //U+8BE8
0, //U+8BE9
0, //U+8BEA
0, //U+8BEB
0, //U+8BEC
0, //U+8BED
0, //U+8BEE
0, //U+8BEF
0, //U+8BF0
0, //U+8BF1
0, //U+8BF2
0, //U+8BF3
0, //U+8BF4
0, //U+8BF5
0, //U+8BF6
0, //U+8BF7
0, //U+8BF8
0, //U+8BF9
0, //U+8BFA
0, //U+8BFB
0, //U+8BFC
0, //U+8BFD
0, //U+8BFE
0, //U+8BFF
0, //U+8C00
0, //U+8C01
0, //U+8C02
0, //U+8C03
0, //U+8C04
0, //U+8C05
0, //U+8C06
0, //U+8C07
0, //U+8C08
0, //U+8C09
0, //U+8C0A
0, //U+8C0B
0, //U+8C0C
0, //U+8C0D
0, //U+8C0E
0, //U+8C0F
0, //U+8C10
0, //U+8C11
0, //U+8C12
0, //U+8C13
0, //U+8C14
0, //U+8C15
0, //U+8C16
0, //U+8C17
0, //U+8C18
0, //U+8C19
0, //U+8C1A
0, //U+8C1B
0, //U+8C1C
0, //U+8C1D
0, //U+8C1E
0, //U+8C1F
0, //U+8C20
0, //U+8C21
0, //U+8C22
0, //U+8C23
0, //U+8C24
0, //U+8C25
0, //U+8C26
0, //U+8C27
0, //U+8C28
0, //U+8C29
0, //U+8C2A
0, //U+8C2B
0, //U+8C2C
0, //U+8C2D
0, //U+8C2E
0, //U+8C2F
0, //U+8C30
0, //U+8C31
0, //U+8C32
0, //U+8C33
0, //U+8C34
0, //U+8C35
0, //U+8C36
0, //U+8C37
0, //U+8C38
0, //U+8C39
0, //U+8C3A
0, //U+8C3B
0, //U+8C3C
0, //U+8C3D
0, //U+8C3E
0, //U+8C3F
0, //U+8C40
0, //U+8C41
0, //U+8C42
0, //U+8C43
0, //U+8C44
0, //U+8C45
0, //U+8C46
0, //U+8C47
0, //U+8C48
0, //U+8C49
0, //U+8C4A
0, //U+8C4B
0, //U+8C4C
0, //U+8C4D
0, //U+8C4E
0, //U+8C4F
0, //U+8C50
0, //U+8C51
0, //U+8C52
0, //U+8C53
0, //U+8C54
0, //U+8C55
0, //U+8C56
0, //U+8C57
0, //U+8C58
0, //U+8C59
0, //U+8C5A
0, //U+8C5B
0, //U+8C5C
0, //U+8C5D
0, //U+8C5E
0, //U+8C5F
0, //U+8C60
0, //U+8C61
0, //U+8C62
0, //U+8C63
0, //U+8C64
0, //U+8C65
0, //U+8C66
0, //U+8C67
0, //U+8C68
0, //U+8C69
0, //U+8C6A
0, //U+8C6B
0, //U+8C6C
0, //U+8C6D
0, //U+8C6E
0, //U+8C6F
0, //U+8C70
0, //U+8C71
0, //U+8C72
0, //U+8C73
0, //U+8C74
0, //U+8C75
0, //U+8C76
0, //U+8C77
0, //U+8C78
0, //U+8C79
0, //U+8C7A
0, //U+8C7B
0, //U+8C7C
0, //U+8C7D
0, //U+8C7E
0, //U+8C7F
0, //U+8C80
0, //U+8C81
0, //U+8C82
0, //U+8C83
0, //U+8C84
0, //U+8C85
0, //U+8C86
0, //U+8C87
0, //U+8C88
0, //U+8C89
0, //U+8C8A
0, //U+8C8B
0, //U+8C8C
0, //U+8C8D
0, //U+8C8E
0, //U+8C8F
0, //U+8C90
0, //U+8C91
0, //U+8C92
0, //U+8C93
0, //U+8C94
0, //U+8C95
0, //U+8C96
0, //U+8C97
0, //U+8C98
0, //U+8C99
0, //U+8C9A
0, //U+8C9B
0, //U+8C9C
0, //U+8C9D
0, //U+8C9E
0, //U+8C9F
0, //U+8CA0
0, //U+8CA1
0, //U+8CA2
0, //U+8CA3
0, //U+8CA4
0, //U+8CA5
0, //U+8CA6
0, //U+8CA7
0, //U+8CA8
0, //U+8CA9
0, //U+8CAA
0, //U+8CAB
0, //U+8CAC
0, //U+8CAD
0, //U+8CAE
0, //U+8CAF
0, //U+8CB0
0, //U+8CB1
0, //U+8CB2
0, //U+8CB3
0, //U+8CB4
0, //U+8CB5
0, //U+8CB6
0, //U+8CB7
0, //U+8CB8
0, //U+8CB9
0, //U+8CBA
0, //U+8CBB
0, //U+8CBC
0, //U+8CBD
0, //U+8CBE
0, //U+8CBF
0, //U+8CC0
0, //U+8CC1
0, //U+8CC2
0, //U+8CC3
0, //U+8CC4
0, //U+8CC5
0, //U+8CC6
0, //U+8CC7
0, //U+8CC8
0, //U+8CC9
0, //U+8CCA
0, //U+8CCB
0, //U+8CCC
0, //U+8CCD
0, //U+8CCE
0, //U+8CCF
0, //U+8CD0
0, //U+8CD1
0, //U+8CD2
0, //U+8CD3
0, //U+8CD4
0, //U+8CD5
0, //U+8CD6
0, //U+8CD7
0, //U+8CD8
0, //U+8CD9
0, //U+8CDA
0, //U+8CDB
0, //U+8CDC
0, //U+8CDD
0, //U+8CDE
0, //U+8CDF
0, //U+8CE0
0, //U+8CE1
0, //U+8CE2
0, //U+8CE3
0, //U+8CE4
0, //U+8CE5
0, //U+8CE6
0, //U+8CE7
0, //U+8CE8
0, //U+8CE9
0, //U+8CEA
0, //U+8CEB
0, //U+8CEC
0, //U+8CED
0, //U+8CEE
0, //U+8CEF
0, //U+8CF0
0, //U+8CF1
0, //U+8CF2
0, //U+8CF3
0, //U+8CF4
0, //U+8CF5
0, //U+8CF6
0, //U+8CF7
0, //U+8CF8
0, //U+8CF9
0, //U+8CFA
0, //U+8CFB
0, //U+8CFC
0, //U+8CFD
0, //U+8CFE
0, //U+8CFF
0, //U+8D00
0, //U+8D01
0, //U+8D02
0, //U+8D03
0, //U+8D04
0, //U+8D05
0, //U+8D06
0, //U+8D07
0, //U+8D08
0, //U+8D09
0, //U+8D0A
0, //U+8D0B
0, //U+8D0C
0, //U+8D0D
0, //U+8D0E
0, //U+8D0F
0, //U+8D10
0, //U+8D11
0, //U+8D12
0, //U+8D13
0, //U+8D14
0, //U+8D15
0, //U+8D16
0, //U+8D17
0, //U+8D18
0, //U+8D19
0, //U+8D1A
0, //U+8D1B
0, //U+8D1C
0, //U+8D1D
0, //U+8D1E
0, //U+8D1F
0, //U+8D20
0, //U+8D21
0, //U+8D22
0, //U+8D23
0, //U+8D24
0, //U+8D25
0, //U+8D26
0, //U+8D27
0, //U+8D28
0, //U+8D29
0, //U+8D2A
0, //U+8D2B
0, //U+8D2C
0, //U+8D2D
0, //U+8D2E
0, //U+8D2F
0, //U+8D30
0, //U+8D31
0, //U+8D32
0, //U+8D33
0, //U+8D34
0, //U+8D35
0, //U+8D36
0, //U+8D37
0, //U+8D38
0, //U+8D39
0, //U+8D3A
0, //U+8D3B
0, //U+8D3C
0, //U+8D3D
0, //U+8D3E
0, //U+8D3F
0, //U+8D40
0, //U+8D41
0, //U+8D42
0, //U+8D43
0, //U+8D44
0, //U+8D45
0, //U+8D46
0, //U+8D47
0, //U+8D48
0, //U+8D49
0, //U+8D4A
0, //U+8D4B
0, //U+8D4C
0, //U+8D4D
0, //U+8D4E
0, //U+8D4F
0, //U+8D50
0, //U+8D51
0, //U+8D52
0, //U+8D53
0, //U+8D54
0, //U+8D55
0, //U+8D56
0, //U+8D57
0, //U+8D58
0, //U+8D59
0, //U+8D5A
0, //U+8D5B
0, //U+8D5C
0, //U+8D5D
0, //U+8D5E
0, //U+8D5F
0, //U+8D60
0, //U+8D61
0, //U+8D62
0, //U+8D63
0, //U+8D64
0, //U+8D65
0, //U+8D66
0, //U+8D67
0, //U+8D68
0, //U+8D69
0, //U+8D6A
0, //U+8D6B
0, //U+8D6C
0, //U+8D6D
0, //U+8D6E
0, //U+8D6F
0, //U+8D70
0, //U+8D71
0, //U+8D72
0, //U+8D73
0, //U+8D74
0, //U+8D75
0, //U+8D76
0, //U+8D77
0, //U+8D78
0, //U+8D79
0, //U+8D7A
0, //U+8D7B
0, //U+8D7C
0, //U+8D7D
0, //U+8D7E
0, //U+8D7F
0, //U+8D80
0, //U+8D81
0, //U+8D82
0, //U+8D83
0, //U+8D84
0, //U+8D85
0, //U+8D86
0, //U+8D87
0, //U+8D88
0, //U+8D89
0, //U+8D8A
0, //U+8D8B
0, //U+8D8C
0, //U+8D8D
0, //U+8D8E
0, //U+8D8F
0, //U+8D90
0, //U+8D91
0, //U+8D92
0, //U+8D93
0, //U+8D94
0, //U+8D95
0, //U+8D96
0, //U+8D97
0, //U+8D98
0, //U+8D99
0, //U+8D9A
0, //U+8D9B
0, //U+8D9C
0, //U+8D9D
0, //U+8D9E
0, //U+8D9F
0, //U+8DA0
0, //U+8DA1
0, //U+8DA2
0, //U+8DA3
0, //U+8DA4
0, //U+8DA5
0, //U+8DA6
0, //U+8DA7
0, //U+8DA8
0, //U+8DA9
0, //U+8DAA
0, //U+8DAB
0, //U+8DAC
0, //U+8DAD
0, //U+8DAE
0, //U+8DAF
0, //U+8DB0
0, //U+8DB1
0, //U+8DB2
0, //U+8DB3
0, //U+8DB4
0, //U+8DB5
0, //U+8DB6
0, //U+8DB7
0, //U+8DB8
0, //U+8DB9
0, //U+8DBA
0, //U+8DBB
0, //U+8DBC
0, //U+8DBD
0, //U+8DBE
0, //U+8DBF
0, //U+8DC0
0, //U+8DC1
0, //U+8DC2
0, //U+8DC3
0, //U+8DC4
0, //U+8DC5
0, //U+8DC6
0, //U+8DC7
0, //U+8DC8
0, //U+8DC9
0, //U+8DCA
0, //U+8DCB
0, //U+8DCC
0, //U+8DCD
0, //U+8DCE
0, //U+8DCF
0, //U+8DD0
0, //U+8DD1
0, //U+8DD2
0, //U+8DD3
0, //U+8DD4
0, //U+8DD5
0, //U+8DD6
0, //U+8DD7
0, //U+8DD8
0, //U+8DD9
0, //U+8DDA
0, //U+8DDB
0, //U+8DDC
0, //U+8DDD
0, //U+8DDE
0, //U+8DDF
0, //U+8DE0
0, //U+8DE1
0, //U+8DE2
0, //U+8DE3
0, //U+8DE4
0, //U+8DE5
0, //U+8DE6
0, //U+8DE7
0, //U+8DE8
0, //U+8DE9
0, //U+8DEA
0, //U+8DEB
0, //U+8DEC
0, //U+8DED
0, //U+8DEE
0, //U+8DEF
0, //U+8DF0
0, //U+8DF1
0, //U+8DF2
0, //U+8DF3
0, //U+8DF4
0, //U+8DF5
0, //U+8DF6
0, //U+8DF7
0, //U+8DF8
0, //U+8DF9
0, //U+8DFA
0, //U+8DFB
0, //U+8DFC
0, //U+8DFD
0, //U+8DFE
0, //U+8DFF
0, //U+8E00
0, //U+8E01
0, //U+8E02
0, //U+8E03
0, //U+8E04
0, //U+8E05
0, //U+8E06
0, //U+8E07
0, //U+8E08
0, //U+8E09
0, //U+8E0A
0, //U+8E0B
0, //U+8E0C
0, //U+8E0D
0, //U+8E0E
0, //U+8E0F
0, //U+8E10
0, //U+8E11
0, //U+8E12
0, //U+8E13
0, //U+8E14
0, //U+8E15
0, //U+8E16
0, //U+8E17
0, //U+8E18
0, //U+8E19
0, //U+8E1A
0, //U+8E1B
0, //U+8E1C
0, //U+8E1D
0, //U+8E1E
0, //U+8E1F
0, //U+8E20
0, //U+8E21
0, //U+8E22
0, //U+8E23
0, //U+8E24
0, //U+8E25
0, //U+8E26
0, //U+8E27
0, //U+8E28
0, //U+8E29
0, //U+8E2A
0, //U+8E2B
0, //U+8E2C
0, //U+8E2D
0, //U+8E2E
0, //U+8E2F
0, //U+8E30
0, //U+8E31
0, //U+8E32
0, //U+8E33
0, //U+8E34
0, //U+8E35
0, //U+8E36
0, //U+8E37
0, //U+8E38
0, //U+8E39
0, //U+8E3A
0, //U+8E3B
0, //U+8E3C
0, //U+8E3D
0, //U+8E3E
0, //U+8E3F
0, //U+8E40
0, //U+8E41
0, //U+8E42
0, //U+8E43
0, //U+8E44
0, //U+8E45
0, //U+8E46
0, //U+8E47
0, //U+8E48
0, //U+8E49
0, //U+8E4A
0, //U+8E4B
0, //U+8E4C
0, //U+8E4D
0, //U+8E4E
0, //U+8E4F
0, //U+8E50
0, //U+8E51
0, //U+8E52
0, //U+8E53
0, //U+8E54
0, //U+8E55
0, //U+8E56
0, //U+8E57
0, //U+8E58
0, //U+8E59
0, //U+8E5A
0, //U+8E5B
0, //U+8E5C
0, //U+8E5D
0, //U+8E5E
0, //U+8E5F
0, //U+8E60
0, //U+8E61
0, //U+8E62
0, //U+8E63
0, //U+8E64
0, //U+8E65
0, //U+8E66
0, //U+8E67
0, //U+8E68
0, //U+8E69
0, //U+8E6A
0, //U+8E6B
0, //U+8E6C
0, //U+8E6D
0, //U+8E6E
0, //U+8E6F
0, //U+8E70
0, //U+8E71
0, //U+8E72
0, //U+8E73
0, //U+8E74
0, //U+8E75
0, //U+8E76
0, //U+8E77
0, //U+8E78
0, //U+8E79
0, //U+8E7A
0, //U+8E7B
0, //U+8E7C
0, //U+8E7D
0, //U+8E7E
0, //U+8E7F
0, //U+8E80
0, //U+8E81
0, //U+8E82
0, //U+8E83
0, //U+8E84
0, //U+8E85
0, //U+8E86
0, //U+8E87
0, //U+8E88
0, //U+8E89
0, //U+8E8A
0, //U+8E8B
0, //U+8E8C
0, //U+8E8D
0, //U+8E8E
0, //U+8E8F
0, //U+8E90
0, //U+8E91
0, //U+8E92
0, //U+8E93
0, //U+8E94
0, //U+8E95
0, //U+8E96
0, //U+8E97
0, //U+8E98
0, //U+8E99
0, //U+8E9A
0, //U+8E9B
0, //U+8E9C
0, //U+8E9D
0, //U+8E9E
0, //U+8E9F
0, //U+8EA0
0, //U+8EA1
0, //U+8EA2
0, //U+8EA3
0, //U+8EA4
0, //U+8EA5
0, //U+8EA6
0, //U+8EA7
0, //U+8EA8
0, //U+8EA9
0, //U+8EAA
0, //U+8EAB
0, //U+8EAC
0, //U+8EAD
0, //U+8EAE
0, //U+8EAF
0, //U+8EB0
0, //U+8EB1
0, //U+8EB2
0, //U+8EB3
0, //U+8EB4
0, //U+8EB5
0, //U+8EB6
0, //U+8EB7
0, //U+8EB8
0, //U+8EB9
0, //U+8EBA
0, //U+8EBB
0, //U+8EBC
0, //U+8EBD
0, //U+8EBE
0, //U+8EBF
0, //U+8EC0
0, //U+8EC1
0, //U+8EC2
0, //U+8EC3
0, //U+8EC4
0, //U+8EC5
0, //U+8EC6
0, //U+8EC7
0, //U+8EC8
0, //U+8EC9
0, //U+8ECA
0, //U+8ECB
0, //U+8ECC
0, //U+8ECD
0, //U+8ECE
0, //U+8ECF
0, //U+8ED0
0, //U+8ED1
0, //U+8ED2
0, //U+8ED3
0, //U+8ED4
0, //U+8ED5
0, //U+8ED6
0, //U+8ED7
0, //U+8ED8
0, //U+8ED9
0, //U+8EDA
0, //U+8EDB
0, //U+8EDC
0, //U+8EDD
0, //U+8EDE
0, //U+8EDF
0, //U+8EE0
0, //U+8EE1
0, //U+8EE2
0, //U+8EE3
0, //U+8EE4
0, //U+8EE5
0, //U+8EE6
0, //U+8EE7
0, //U+8EE8
0, //U+8EE9
0, //U+8EEA
0, //U+8EEB
0, //U+8EEC
0, //U+8EED
0, //U+8EEE
0, //U+8EEF
0, //U+8EF0
0, //U+8EF1
0, //U+8EF2
0, //U+8EF3
0, //U+8EF4
0, //U+8EF5
0, //U+8EF6
0, //U+8EF7
0, //U+8EF8
0, //U+8EF9
0, //U+8EFA
0, //U+8EFB
0, //U+8EFC
0, //U+8EFD
0, //U+8EFE
0, //U+8EFF
0, //U+8F00
0, //U+8F01
0, //U+8F02
0, //U+8F03
0, //U+8F04
0, //U+8F05
0, //U+8F06
0, //U+8F07
0, //U+8F08
0, //U+8F09
0, //U+8F0A
0, //U+8F0B
0, //U+8F0C
0, //U+8F0D
0, //U+8F0E
0, //U+8F0F
0, //U+8F10
0, //U+8F11
0, //U+8F12
0, //U+8F13
0, //U+8F14
0, //U+8F15
0, //U+8F16
0, //U+8F17
0, //U+8F18
0, //U+8F19
0, //U+8F1A
0, //U+8F1B
0, //U+8F1C
0, //U+8F1D
0, //U+8F1E
0, //U+8F1F
0, //U+8F20
0, //U+8F21
0, //U+8F22
0, //U+8F23
0, //U+8F24
0, //U+8F25
0, //U+8F26
0, //U+8F27
0, //U+8F28
0, //U+8F29
0, //U+8F2A
0, //U+8F2B
0, //U+8F2C
0, //U+8F2D
0, //U+8F2E
0, //U+8F2F
0, //U+8F30
0, //U+8F31
0, //U+8F32
0, //U+8F33
0, //U+8F34
0, //U+8F35
0, //U+8F36
0, //U+8F37
0, //U+8F38
0, //U+8F39
0, //U+8F3A
0, //U+8F3B
0, //U+8F3C
0, //U+8F3D
0, //U+8F3E
0, //U+8F3F
0, //U+8F40
0, //U+8F41
0, //U+8F42
0, //U+8F43
0, //U+8F44
0, //U+8F45
0, //U+8F46
0, //U+8F47
0, //U+8F48
0, //U+8F49
0, //U+8F4A
0, //U+8F4B
0, //U+8F4C
0, //U+8F4D
0, //U+8F4E
0, //U+8F4F
0, //U+8F50
0, //U+8F51
0, //U+8F52
0, //U+8F53
0, //U+8F54
0, //U+8F55
0, //U+8F56
0, //U+8F57
0, //U+8F58
0, //U+8F59
0, //U+8F5A
0, //U+8F5B
0, //U+8F5C
0, //U+8F5D
0, //U+8F5E
0, //U+8F5F
0, //U+8F60
0, //U+8F61
0, //U+8F62
0, //U+8F63
0, //U+8F64
0, //U+8F65
0, //U+8F66
0, //U+8F67
0, //U+8F68
0, //U+8F69
0, //U+8F6A
0, //U+8F6B
0, //U+8F6C
0, //U+8F6D
0, //U+8F6E
0, //U+8F6F
0, //U+8F70
0, //U+8F71
0, //U+8F72
0, //U+8F73
0, //U+8F74
0, //U+8F75
0, //U+8F76
0, //U+8F77
0, //U+8F78
0, //U+8F79
0, //U+8F7A
0, //U+8F7B
0, //U+8F7C
0, //U+8F7D
0, //U+8F7E
0, //U+8F7F
0, //U+8F80
0, //U+8F81
0, //U+8F82
0, //U+8F83
0, //U+8F84
0, //U+8F85
0, //U+8F86
0, //U+8F87
0, //U+8F88
0, //U+8F89
0, //U+8F8A
0, //U+8F8B
0, //U+8F8C
0, //U+8F8D
0, //U+8F8E
0, //U+8F8F
0, //U+8F90
0, //U+8F91
0, //U+8F92
0, //U+8F93
0, //U+8F94
0, //U+8F95
0, //U+8F96
0, //U+8F97
0, //U+8F98
0, //U+8F99
0, //U+8F9A
0, //U+8F9B
0, //U+8F9C
0, //U+8F9D
0, //U+8F9E
0, //U+8F9F
0, //U+8FA0
0, //U+8FA1
0, //U+8FA2
0, //U+8FA3
0, //U+8FA4
0, //U+8FA5
0, //U+8FA6
0, //U+8FA7
0, //U+8FA8
0, //U+8FA9
0, //U+8FAA
0, //U+8FAB
0, //U+8FAC
0, //U+8FAD
0, //U+8FAE
0, //U+8FAF
0, //U+8FB0
0, //U+8FB1
0, //U+8FB2
0, //U+8FB3
0, //U+8FB4
0, //U+8FB5
0, //U+8FB6
0, //U+8FB7
0, //U+8FB8
0, //U+8FB9
0, //U+8FBA
0, //U+8FBB
0, //U+8FBC
0, //U+8FBD
0, //U+8FBE
0, //U+8FBF
0, //U+8FC0
0, //U+8FC1
0, //U+8FC2
0, //U+8FC3
0, //U+8FC4
0, //U+8FC5
0, //U+8FC6
0, //U+8FC7
0, //U+8FC8
0, //U+8FC9
0, //U+8FCA
0, //U+8FCB
0, //U+8FCC
0, //U+8FCD
0, //U+8FCE
0, //U+8FCF
0, //U+8FD0
0, //U+8FD1
0, //U+8FD2
0, //U+8FD3
0, //U+8FD4
0, //U+8FD5
0, //U+8FD6
0, //U+8FD7
0, //U+8FD8
0, //U+8FD9
0, //U+8FDA
0, //U+8FDB
0, //U+8FDC
0, //U+8FDD
0, //U+8FDE
0, //U+8FDF
0, //U+8FE0
0, //U+8FE1
0, //U+8FE2
0, //U+8FE3
0, //U+8FE4
0, //U+8FE5
0, //U+8FE6
0, //U+8FE7
0, //U+8FE8
0, //U+8FE9
0, //U+8FEA
0, //U+8FEB
0, //U+8FEC
0, //U+8FED
0, //U+8FEE
0, //U+8FEF
0, //U+8FF0
0, //U+8FF1
0, //U+8FF2
0, //U+8FF3
0, //U+8FF4
0, //U+8FF5
0, //U+8FF6
0, //U+8FF7
0, //U+8FF8
0, //U+8FF9
0, //U+8FFA
0, //U+8FFB
0, //U+8FFC
0, //U+8FFD
0, //U+8FFE
0, //U+8FFF
0, //U+9000
0, //U+9001
0, //U+9002
0, //U+9003
0, //U+9004
0, //U+9005
0, //U+9006
0, //U+9007
0, //U+9008
0, //U+9009
0, //U+900A
0, //U+900B
0, //U+900C
0, //U+900D
0, //U+900E
0, //U+900F
0, //U+9010
0, //U+9011
0, //U+9012
0, //U+9013
0, //U+9014
0, //U+9015
0, //U+9016
0, //U+9017
0, //U+9018
0, //U+9019
0, //U+901A
0, //U+901B
0, //U+901C
0, //U+901D
0, //U+901E
0, //U+901F
0, //U+9020
0, //U+9021
0, //U+9022
0, //U+9023
0, //U+9024
0, //U+9025
0, //U+9026
0, //U+9027
0, //U+9028
0, //U+9029
0, //U+902A
0, //U+902B
0, //U+902C
0, //U+902D
0, //U+902E
0, //U+902F
0, //U+9030
0, //U+9031
0, //U+9032
0, //U+9033
0, //U+9034
0, //U+9035
0, //U+9036
0, //U+9037
0, //U+9038
0, //U+9039
0, //U+903A
0, //U+903B
0, //U+903C
0, //U+903D
0, //U+903E
0, //U+903F
0, //U+9040
0, //U+9041
0, //U+9042
0, //U+9043
0, //U+9044
0, //U+9045
0, //U+9046
0, //U+9047
0, //U+9048
0, //U+9049
0, //U+904A
0, //U+904B
0, //U+904C
0, //U+904D
0, //U+904E
0, //U+904F
0, //U+9050
0, //U+9051
0, //U+9052
0, //U+9053
0, //U+9054
0, //U+9055
0, //U+9056
0, //U+9057
0, //U+9058
0, //U+9059
0, //U+905A
0, //U+905B
0, //U+905C
0, //U+905D
0, //U+905E
0, //U+905F
0, //U+9060
0, //U+9061
0, //U+9062
0, //U+9063
0, //U+9064
0, //U+9065
0, //U+9066
0, //U+9067
0, //U+9068
0, //U+9069
0, //U+906A
0, //U+906B
0, //U+906C
0, //U+906D
0, //U+906E
0, //U+906F
0, //U+9070
0, //U+9071
0, //U+9072
0, //U+9073
0, //U+9074
0, //U+9075
0, //U+9076
0, //U+9077
0, //U+9078
0, //U+9079
0, //U+907A
0, //U+907B
0, //U+907C
0, //U+907D
0, //U+907E
0, //U+907F
0, //U+9080
0, //U+9081
0, //U+9082
0, //U+9083
0, //U+9084
0, //U+9085
0, //U+9086
0, //U+9087
0, //U+9088
0, //U+9089
0, //U+908A
0, //U+908B
0, //U+908C
0, //U+908D
0, //U+908E
0, //U+908F
0, //U+9090
0, //U+9091
0, //U+9092
0, //U+9093
0, //U+9094
0, //U+9095
0, //U+9096
0, //U+9097
0, //U+9098
0, //U+9099
0, //U+909A
0, //U+909B
0, //U+909C
0, //U+909D
0, //U+909E
0, //U+909F
0, //U+90A0
0, //U+90A1
0, //U+90A2
0, //U+90A3
0, //U+90A4
0, //U+90A5
0, //U+90A6
0, //U+90A7
0, //U+90A8
0, //U+90A9
0, //U+90AA
0, //U+90AB
0, //U+90AC
0, //U+90AD
0, //U+90AE
0, //U+90AF
0, //U+90B0
0, //U+90B1
0, //U+90B2
0, //U+90B3
0, //U+90B4
0, //U+90B5
0, //U+90B6
0, //U+90B7
0, //U+90B8
0, //U+90B9
0, //U+90BA
0, //U+90BB
0, //U+90BC
0, //U+90BD
0, //U+90BE
0, //U+90BF
0, //U+90C0
0, //U+90C1
0, //U+90C2
0, //U+90C3
0, //U+90C4
0, //U+90C5
0, //U+90C6
0, //U+90C7
0, //U+90C8
0, //U+90C9
0, //U+90CA
0, //U+90CB
0, //U+90CC
0, //U+90CD
0, //U+90CE
0, //U+90CF
0, //U+90D0
0, //U+90D1
0, //U+90D2
0, //U+90D3
0, //U+90D4
0, //U+90D5
0, //U+90D6
0, //U+90D7
0, //U+90D8
0, //U+90D9
0, //U+90DA
0, //U+90DB
0, //U+90DC
0, //U+90DD
0, //U+90DE
0, //U+90DF
0, //U+90E0
0, //U+90E1
0, //U+90E2
0, //U+90E3
0, //U+90E4
0, //U+90E5
0, //U+90E6
0, //U+90E7
0, //U+90E8
0, //U+90E9
0, //U+90EA
0, //U+90EB
0, //U+90EC
0, //U+90ED
0, //U+90EE
0, //U+90EF
0, //U+90F0
0, //U+90F1
0, //U+90F2
0, //U+90F3
0, //U+90F4
0, //U+90F5
0, //U+90F6
0, //U+90F7
0, //U+90F8
0, //U+90F9
0, //U+90FA
0, //U+90FB
0, //U+90FC
0, //U+90FD
0, //U+90FE
0, //U+90FF
0, //U+9100
0, //U+9101
0, //U+9102
0, //U+9103
0, //U+9104
0, //U+9105
0, //U+9106
0, //U+9107
0, //U+9108
0, //U+9109
0, //U+910A
0, //U+910B
0, //U+910C
0, //U+910D
0, //U+910E
0, //U+910F
0, //U+9110
0, //U+9111
0, //U+9112
0, //U+9113
0, //U+9114
0, //U+9115
0, //U+9116
0, //U+9117
0, //U+9118
0, //U+9119
0, //U+911A
0, //U+911B
0, //U+911C
0, //U+911D
0, //U+911E
0, //U+911F
0, //U+9120
0, //U+9121
0, //U+9122
0, //U+9123
0, //U+9124
0, //U+9125
0, //U+9126
0, //U+9127
0, //U+9128
0, //U+9129
0, //U+912A
0, //U+912B
0, //U+912C
0, //U+912D
0, //U+912E
0, //U+912F
0, //U+9130
0, //U+9131
0, //U+9132
0, //U+9133
0, //U+9134
0, //U+9135
0, //U+9136
0, //U+9137
0, //U+9138
0, //U+9139
0, //U+913A
0, //U+913B
0, //U+913C
0, //U+913D
0, //U+913E
0, //U+913F
0, //U+9140
0, //U+9141
0, //U+9142
0, //U+9143
0, //U+9144
0, //U+9145
0, //U+9146
0, //U+9147
0, //U+9148
0, //U+9149
0, //U+914A
0, //U+914B
0, //U+914C
0, //U+914D
0, //U+914E
0, //U+914F
0, //U+9150
0, //U+9151
0, //U+9152
0, //U+9153
0, //U+9154
0, //U+9155
0, //U+9156
0, //U+9157
0, //U+9158
0, //U+9159
0, //U+915A
0, //U+915B
0, //U+915C
0, //U+915D
0, //U+915E
0, //U+915F
0, //U+9160
0, //U+9161
0, //U+9162
0, //U+9163
0, //U+9164
0, //U+9165
0, //U+9166
0, //U+9167
0, //U+9168
0, //U+9169
0, //U+916A
0, //U+916B
0, //U+916C
0, //U+916D
0, //U+916E
0, //U+916F
0, //U+9170
0, //U+9171
0, //U+9172
0, //U+9173
0, //U+9174
0, //U+9175
0, //U+9176
0, //U+9177
0, //U+9178
0, //U+9179
0, //U+917A
0, //U+917B
0, //U+917C
0, //U+917D
0, //U+917E
0, //U+917F
0, //U+9180
0, //U+9181
0, //U+9182
0, //U+9183
0, //U+9184
0, //U+9185
0, //U+9186
0, //U+9187
0, //U+9188
0, //U+9189
0, //U+918A
0, //U+918B
0, //U+918C
0, //U+918D
0, //U+918E
0, //U+918F
0, //U+9190
0, //U+9191
0, //U+9192
0, //U+9193
0, //U+9194
0, //U+9195
0, //U+9196
0, //U+9197
0, //U+9198
0, //U+9199
0, //U+919A
0, //U+919B
0, //U+919C
0, //U+919D
0, //U+919E
0, //U+919F
0, //U+91A0
0, //U+91A1
0, //U+91A2
0, //U+91A3
0, //U+91A4
0, //U+91A5
0, //U+91A6
0, //U+91A7
0, //U+91A8
0, //U+91A9
0, //U+91AA
0, //U+91AB
0, //U+91AC
0, //U+91AD
0, //U+91AE
0, //U+91AF
0, //U+91B0
0, //U+91B1
0, //U+91B2
0, //U+91B3
0, //U+91B4
0, //U+91B5
0, //U+91B6
0, //U+91B7
0, //U+91B8
0, //U+91B9
0, //U+91BA
0, //U+91BB
0, //U+91BC
0, //U+91BD
0, //U+91BE
0, //U+91BF
0, //U+91C0
0, //U+91C1
0, //U+91C2
0, //U+91C3
0, //U+91C4
0, //U+91C5
0, //U+91C6
0, //U+91C7
0, //U+91C8
0, //U+91C9
0, //U+91CA
0, //U+91CB
0, //U+91CC
0, //U+91CD
0, //U+91CE
0, //U+91CF
0, //U+91D0
0, //U+91D1
0, //U+91D2
0, //U+91D3
0, //U+91D4
0, //U+91D5
0, //U+91D6
0, //U+91D7
0, //U+91D8
0, //U+91D9
0, //U+91DA
0, //U+91DB
0, //U+91DC
0, //U+91DD
0, //U+91DE
0, //U+91DF
0, //U+91E0
0, //U+91E1
0, //U+91E2
0, //U+91E3
0, //U+91E4
0, //U+91E5
0, //U+91E6
0, //U+91E7
0, //U+91E8
0, //U+91E9
0, //U+91EA
0, //U+91EB
0, //U+91EC
0, //U+91ED
0, //U+91EE
0, //U+91EF
0, //U+91F0
0, //U+91F1
0, //U+91F2
0, //U+91F3
0, //U+91F4
0, //U+91F5
0, //U+91F6
0, //U+91F7
0, //U+91F8
0, //U+91F9
0, //U+91FA
0, //U+91FB
0, //U+91FC
0, //U+91FD
0, //U+91FE
0, //U+91FF
0, //U+9200
0, //U+9201
0, //U+9202
0, //U+9203
0, //U+9204
0, //U+9205
0, //U+9206
0, //U+9207
0, //U+9208
0, //U+9209
0, //U+920A
0, //U+920B
0, //U+920C
0, //U+920D
0, //U+920E
0, //U+920F
0, //U+9210
0, //U+9211
0, //U+9212
0, //U+9213
0, //U+9214
0, //U+9215
0, //U+9216
0, //U+9217
0, //U+9218
0, //U+9219
0, //U+921A
0, //U+921B
0, //U+921C
0, //U+921D
0, //U+921E
0, //U+921F
0, //U+9220
0, //U+9221
0, //U+9222
0, //U+9223
0, //U+9224
0, //U+9225
0, //U+9226
0, //U+9227
0, //U+9228
0, //U+9229
0, //U+922A
0, //U+922B
0, //U+922C
0, //U+922D
0, //U+922E
0, //U+922F
0, //U+9230
0, //U+9231
0, //U+9232
0, //U+9233
0, //U+9234
0, //U+9235
0, //U+9236
0, //U+9237
0, //U+9238
0, //U+9239
0, //U+923A
0, //U+923B
0, //U+923C
0, //U+923D
0, //U+923E
0, //U+923F
0, //U+9240
0, //U+9241
0, //U+9242
0, //U+9243
0, //U+9244
0, //U+9245
0, //U+9246
0, //U+9247
0, //U+9248
0, //U+9249
0, //U+924A
0, //U+924B
0, //U+924C
0, //U+924D
0, //U+924E
0, //U+924F
0, //U+9250
0, //U+9251
0, //U+9252
0, //U+9253
0, //U+9254
0, //U+9255
0, //U+9256
0, //U+9257
0, //U+9258
0, //U+9259
0, //U+925A
0, //U+925B
0, //U+925C
0, //U+925D
0, //U+925E
0, //U+925F
0, //U+9260
0, //U+9261
0, //U+9262
0, //U+9263
0, //U+9264
0, //U+9265
0, //U+9266
0, //U+9267
0, //U+9268
0, //U+9269
0, //U+926A
0, //U+926B
0, //U+926C
0, //U+926D
0, //U+926E
0, //U+926F
0, //U+9270
0, //U+9271
0, //U+9272
0, //U+9273
0, //U+9274
0, //U+9275
0, //U+9276
0, //U+9277
0, //U+9278
0, //U+9279
0, //U+927A
0, //U+927B
0, //U+927C
0, //U+927D
0, //U+927E
0, //U+927F
0, //U+9280
0, //U+9281
0, //U+9282
0, //U+9283
0, //U+9284
0, //U+9285
0, //U+9286
0, //U+9287
0, //U+9288
0, //U+9289
0, //U+928A
0, //U+928B
0, //U+928C
0, //U+928D
0, //U+928E
0, //U+928F
0, //U+9290
0, //U+9291
0, //U+9292
0, //U+9293
0, //U+9294
0, //U+9295
0, //U+9296
0, //U+9297
0, //U+9298
0, //U+9299
0, //U+929A
0, //U+929B
0, //U+929C
0, //U+929D
0, //U+929E
0, //U+929F
0, //U+92A0
0, //U+92A1
0, //U+92A2
0, //U+92A3
0, //U+92A4
0, //U+92A5
0, //U+92A6
0, //U+92A7
0, //U+92A8
0, //U+92A9
0, //U+92AA
0, //U+92AB
0, //U+92AC
0, //U+92AD
0, //U+92AE
0, //U+92AF
0, //U+92B0
0, //U+92B1
0, //U+92B2
0, //U+92B3
0, //U+92B4
0, //U+92B5
0, //U+92B6
0, //U+92B7
0, //U+92B8
0, //U+92B9
0, //U+92BA
0, //U+92BB
0, //U+92BC
0, //U+92BD
0, //U+92BE
0, //U+92BF
0, //U+92C0
0, //U+92C1
0, //U+92C2
0, //U+92C3
0, //U+92C4
0, //U+92C5
0, //U+92C6
0, //U+92C7
0, //U+92C8
0, //U+92C9
0, //U+92CA
0, //U+92CB
0, //U+92CC
0, //U+92CD
0, //U+92CE
0, //U+92CF
0, //U+92D0
0, //U+92D1
0, //U+92D2
0, //U+92D3
0, //U+92D4
0, //U+92D5
0, //U+92D6
0, //U+92D7
0, //U+92D8
0, //U+92D9
0, //U+92DA
0, //U+92DB
0, //U+92DC
0, //U+92DD
0, //U+92DE
0, //U+92DF
0, //U+92E0
0, //U+92E1
0, //U+92E2
0, //U+92E3
0, //U+92E4
0, //U+92E5
0, //U+92E6
0, //U+92E7
0, //U+92E8
0, //U+92E9
0, //U+92EA
0, //U+92EB
0, //U+92EC
0, //U+92ED
0, //U+92EE
0, //U+92EF
0, //U+92F0
0, //U+92F1
0, //U+92F2
0, //U+92F3
0, //U+92F4
0, //U+92F5
0, //U+92F6
0, //U+92F7
0, //U+92F8
0, //U+92F9
0, //U+92FA
0, //U+92FB
0, //U+92FC
0, //U+92FD
0, //U+92FE
0, //U+92FF
0, //U+9300
0, //U+9301
0, //U+9302
0, //U+9303
0, //U+9304
0, //U+9305
0, //U+9306
0, //U+9307
0, //U+9308
0, //U+9309
0, //U+930A
0, //U+930B
0, //U+930C
0, //U+930D
0, //U+930E
0, //U+930F
0, //U+9310
0, //U+9311
0, //U+9312
0, //U+9313
0, //U+9314
0, //U+9315
0, //U+9316
0, //U+9317
0, //U+9318
0, //U+9319
0, //U+931A
0, //U+931B
0, //U+931C
0, //U+931D
0, //U+931E
0, //U+931F
0, //U+9320
0, //U+9321
0, //U+9322
0, //U+9323
0, //U+9324
0, //U+9325
0, //U+9326
0, //U+9327
0, //U+9328
0, //U+9329
0, //U+932A
0, //U+932B
0, //U+932C
0, //U+932D
0, //U+932E
0, //U+932F
0, //U+9330
0, //U+9331
0, //U+9332
0, //U+9333
0, //U+9334
0, //U+9335
0, //U+9336
0, //U+9337
0, //U+9338
0, //U+9339
0, //U+933A
0, //U+933B
0, //U+933C
0, //U+933D
0, //U+933E
0, //U+933F
0, //U+9340
0, //U+9341
0, //U+9342
0, //U+9343
0, //U+9344
0, //U+9345
0, //U+9346
0, //U+9347
0, //U+9348
0, //U+9349
0, //U+934A
0, //U+934B
0, //U+934C
0, //U+934D
0, //U+934E
0, //U+934F
0, //U+9350
0, //U+9351
0, //U+9352
0, //U+9353
0, //U+9354
0, //U+9355
0, //U+9356
0, //U+9357
0, //U+9358
0, //U+9359
0, //U+935A
0, //U+935B
0, //U+935C
0, //U+935D
0, //U+935E
0, //U+935F
0, //U+9360
0, //U+9361
0, //U+9362
0, //U+9363
0, //U+9364
0, //U+9365
0, //U+9366
0, //U+9367
0, //U+9368
0, //U+9369
0, //U+936A
0, //U+936B
0, //U+936C
0, //U+936D
0, //U+936E
0, //U+936F
0, //U+9370
0, //U+9371
0, //U+9372
0, //U+9373
0, //U+9374
0, //U+9375
0, //U+9376
0, //U+9377
0, //U+9378
0, //U+9379
0, //U+937A
0, //U+937B
0, //U+937C
0, //U+937D
0, //U+937E
0, //U+937F
0, //U+9380
0, //U+9381
0, //U+9382
0, //U+9383
0, //U+9384
0, //U+9385
0, //U+9386
0, //U+9387
0, //U+9388
0, //U+9389
0, //U+938A
0, //U+938B
0, //U+938C
0, //U+938D
0, //U+938E
0, //U+938F
0, //U+9390
0, //U+9391
0, //U+9392
0, //U+9393
0, //U+9394
0, //U+9395
0, //U+9396
0, //U+9397
0, //U+9398
0, //U+9399
0, //U+939A
0, //U+939B
0, //U+939C
0, //U+939D
0, //U+939E
0, //U+939F
0, //U+93A0
0, //U+93A1
0, //U+93A2
0, //U+93A3
0, //U+93A4
0, //U+93A5
0, //U+93A6
0, //U+93A7
0, //U+93A8
0, //U+93A9
0, //U+93AA
0, //U+93AB
0, //U+93AC
0, //U+93AD
0, //U+93AE
0, //U+93AF
0, //U+93B0
0, //U+93B1
0, //U+93B2
0, //U+93B3
0, //U+93B4
0, //U+93B5
0, //U+93B6
0, //U+93B7
0, //U+93B8
0, //U+93B9
0, //U+93BA
0, //U+93BB
0, //U+93BC
0, //U+93BD
0, //U+93BE
0, //U+93BF
0, //U+93C0
0, //U+93C1
0, //U+93C2
0, //U+93C3
0, //U+93C4
0, //U+93C5
0, //U+93C6
0, //U+93C7
0, //U+93C8
0, //U+93C9
0, //U+93CA
0, //U+93CB
0, //U+93CC
0, //U+93CD
0, //U+93CE
0, //U+93CF
0, //U+93D0
0, //U+93D1
0, //U+93D2
0, //U+93D3
0, //U+93D4
0, //U+93D5
0, //U+93D6
0, //U+93D7
0, //U+93D8
0, //U+93D9
0, //U+93DA
0, //U+93DB
0, //U+93DC
0, //U+93DD
0, //U+93DE
0, //U+93DF
0, //U+93E0
0, //U+93E1
0, //U+93E2
0, //U+93E3
0, //U+93E4
0, //U+93E5
0, //U+93E6
0, //U+93E7
0, //U+93E8
0, //U+93E9
0, //U+93EA
0, //U+93EB
0, //U+93EC
0, //U+93ED
0, //U+93EE
0, //U+93EF
0, //U+93F0
0, //U+93F1
0, //U+93F2
0, //U+93F3
0, //U+93F4
0, //U+93F5
0, //U+93F6
0, //U+93F7
0, //U+93F8
0, //U+93F9
0, //U+93FA
0, //U+93FB
0, //U+93FC
0, //U+93FD
0, //U+93FE
0, //U+93FF
0, //U+9400
0, //U+9401
0, //U+9402
0, //U+9403
0, //U+9404
0, //U+9405
0, //U+9406
0, //U+9407
0, //U+9408
0, //U+9409
0, //U+940A
0, //U+940B
0, //U+940C
0, //U+940D
0, //U+940E
0, //U+940F
0, //U+9410
0, //U+9411
0, //U+9412
0, //U+9413
0, //U+9414
0, //U+9415
0, //U+9416
0, //U+9417
0, //U+9418
0, //U+9419
0, //U+941A
0, //U+941B
0, //U+941C
0, //U+941D
0, //U+941E
0, //U+941F
0, //U+9420
0, //U+9421
0, //U+9422
0, //U+9423
0, //U+9424
0, //U+9425
0, //U+9426
0, //U+9427
0, //U+9428
0, //U+9429
0, //U+942A
0, //U+942B
0, //U+942C
0, //U+942D
0, //U+942E
0, //U+942F
0, //U+9430
0, //U+9431
0, //U+9432
0, //U+9433
0, //U+9434
0, //U+9435
0, //U+9436
0, //U+9437
0, //U+9438
0, //U+9439
0, //U+943A
0, //U+943B
0, //U+943C
0, //U+943D
0, //U+943E
0, //U+943F
0, //U+9440
0, //U+9441
0, //U+9442
0, //U+9443
0, //U+9444
0, //U+9445
0, //U+9446
0, //U+9447
0, //U+9448
0, //U+9449
0, //U+944A
0, //U+944B
0, //U+944C
0, //U+944D
0, //U+944E
0, //U+944F
0, //U+9450
0, //U+9451
0, //U+9452
0, //U+9453
0, //U+9454
0, //U+9455
0, //U+9456
0, //U+9457
0, //U+9458
0, //U+9459
0, //U+945A
0, //U+945B
0, //U+945C
0, //U+945D
0, //U+945E
0, //U+945F
0, //U+9460
0, //U+9461
0, //U+9462
0, //U+9463
0, //U+9464
0, //U+9465
0, //U+9466
0, //U+9467
0, //U+9468
0, //U+9469
0, //U+946A
0, //U+946B
0, //U+946C
0, //U+946D
0, //U+946E
0, //U+946F
0, //U+9470
0, //U+9471
0, //U+9472
0, //U+9473
0, //U+9474
0, //U+9475
0, //U+9476
0, //U+9477
0, //U+9478
0, //U+9479
0, //U+947A
0, //U+947B
0, //U+947C
0, //U+947D
0, //U+947E
0, //U+947F
0, //U+9480
0, //U+9481
0, //U+9482
0, //U+9483
0, //U+9484
0, //U+9485
0, //U+9486
0, //U+9487
0, //U+9488
0, //U+9489
0, //U+948A
0, //U+948B
0, //U+948C
0, //U+948D
0, //U+948E
0, //U+948F
0, //U+9490
0, //U+9491
0, //U+9492
0, //U+9493
0, //U+9494
0, //U+9495
0, //U+9496
0, //U+9497
0, //U+9498
0, //U+9499
0, //U+949A
0, //U+949B
0, //U+949C
0, //U+949D
0, //U+949E
0, //U+949F
0, //U+94A0
0, //U+94A1
0, //U+94A2
0, //U+94A3
0, //U+94A4
0, //U+94A5
0, //U+94A6
0, //U+94A7
0, //U+94A8
0, //U+94A9
0, //U+94AA
0, //U+94AB
0, //U+94AC
0, //U+94AD
0, //U+94AE
0, //U+94AF
0, //U+94B0
0, //U+94B1
0, //U+94B2
0, //U+94B3
0, //U+94B4
0, //U+94B5
0, //U+94B6
0, //U+94B7
0, //U+94B8
0, //U+94B9
0, //U+94BA
0, //U+94BB
0, //U+94BC
0, //U+94BD
0, //U+94BE
0, //U+94BF
0, //U+94C0
0, //U+94C1
0, //U+94C2
0, //U+94C3
0, //U+94C4
0, //U+94C5
0, //U+94C6
0, //U+94C7
0, //U+94C8
0, //U+94C9
0, //U+94CA
0, //U+94CB
0, //U+94CC
0, //U+94CD
0, //U+94CE
0, //U+94CF
0, //U+94D0
0, //U+94D1
0, //U+94D2
0, //U+94D3
0, //U+94D4
0, //U+94D5
0, //U+94D6
0, //U+94D7
0, //U+94D8
0, //U+94D9
0, //U+94DA
0, //U+94DB
0, //U+94DC
0, //U+94DD
0, //U+94DE
0, //U+94DF
0, //U+94E0
0, //U+94E1
0, //U+94E2
0, //U+94E3
0, //U+94E4
0, //U+94E5
0, //U+94E6
0, //U+94E7
0, //U+94E8
0, //U+94E9
0, //U+94EA
0, //U+94EB
0, //U+94EC
0, //U+94ED
0, //U+94EE
0, //U+94EF
0, //U+94F0
0, //U+94F1
0, //U+94F2
0, //U+94F3
0, //U+94F4
0, //U+94F5
0, //U+94F6
0, //U+94F7
0, //U+94F8
0, //U+94F9
0, //U+94FA
0, //U+94FB
0, //U+94FC
0, //U+94FD
0, //U+94FE
0, //U+94FF
0, //U+9500
0, //U+9501
0, //U+9502
0, //U+9503
0, //U+9504
0, //U+9505
0, //U+9506
0, //U+9507
0, //U+9508
0, //U+9509
0, //U+950A
0, //U+950B
0, //U+950C
0, //U+950D
0, //U+950E
0, //U+950F
0, //U+9510
0, //U+9511
0, //U+9512
0, //U+9513
0, //U+9514
0, //U+9515
0, //U+9516
0, //U+9517
0, //U+9518
0, //U+9519
0, //U+951A
0, //U+951B
0, //U+951C
0, //U+951D
0, //U+951E
0, //U+951F
0, //U+9520
0, //U+9521
0, //U+9522
0, //U+9523
0, //U+9524
0, //U+9525
0, //U+9526
0, //U+9527
0, //U+9528
0, //U+9529
0, //U+952A
0, //U+952B
0, //U+952C
0, //U+952D
0, //U+952E
0, //U+952F
0, //U+9530
0, //U+9531
0, //U+9532
0, //U+9533
0, //U+9534
0, //U+9535
0, //U+9536
0, //U+9537
0, //U+9538
0, //U+9539
0, //U+953A
0, //U+953B
0, //U+953C
0, //U+953D
0, //U+953E
0, //U+953F
0, //U+9540
0, //U+9541
0, //U+9542
0, //U+9543
0, //U+9544
0, //U+9545
0, //U+9546
0, //U+9547
0, //U+9548
0, //U+9549
0, //U+954A
0, //U+954B
0, //U+954C
0, //U+954D
0, //U+954E
0, //U+954F
0, //U+9550
0, //U+9551
0, //U+9552
0, //U+9553
0, //U+9554
0, //U+9555
0, //U+9556
0, //U+9557
0, //U+9558
0, //U+9559
0, //U+955A
0, //U+955B
0, //U+955C
0, //U+955D
0, //U+955E
0, //U+955F
0, //U+9560
0, //U+9561
0, //U+9562
0, //U+9563
0, //U+9564
0, //U+9565
0, //U+9566
0, //U+9567
0, //U+9568
0, //U+9569
0, //U+956A
0, //U+956B
0, //U+956C
0, //U+956D
0, //U+956E
0, //U+956F
0, //U+9570
0, //U+9571
0, //U+9572
0, //U+9573
0, //U+9574
0, //U+9575
0, //U+9576
0, //U+9577
0, //U+9578
0, //U+9579
0, //U+957A
0, //U+957B
0, //U+957C
0, //U+957D
0, //U+957E
0, //U+957F
0, //U+9580
0, //U+9581
0, //U+9582
0, //U+9583
0, //U+9584
0, //U+9585
0, //U+9586
0, //U+9587
0, //U+9588
0, //U+9589
0, //U+958A
0, //U+958B
0, //U+958C
0, //U+958D
0, //U+958E
0, //U+958F
0, //U+9590
0, //U+9591
0, //U+9592
0, //U+9593
0, //U+9594
0, //U+9595
0, //U+9596
0, //U+9597
0, //U+9598
0, //U+9599
0, //U+959A
0, //U+959B
0, //U+959C
0, //U+959D
0, //U+959E
0, //U+959F
0, //U+95A0
0, //U+95A1
0, //U+95A2
0, //U+95A3
0, //U+95A4
0, //U+95A5
0, //U+95A6
0, //U+95A7
0, //U+95A8
0, //U+95A9
0, //U+95AA
0, //U+95AB
0, //U+95AC
0, //U+95AD
0, //U+95AE
0, //U+95AF
0, //U+95B0
0, //U+95B1
0, //U+95B2
0, //U+95B3
0, //U+95B4
0, //U+95B5
0, //U+95B6
0, //U+95B7
0, //U+95B8
0, //U+95B9
0, //U+95BA
0, //U+95BB
0, //U+95BC
0, //U+95BD
0, //U+95BE
0, //U+95BF
0, //U+95C0
0, //U+95C1
0, //U+95C2
0, //U+95C3
0, //U+95C4
0, //U+95C5
0, //U+95C6
0, //U+95C7
0, //U+95C8
0, //U+95C9
0, //U+95CA
0, //U+95CB
0, //U+95CC
0, //U+95CD
0, //U+95CE
0, //U+95CF
0, //U+95D0
0, //U+95D1
0, //U+95D2
0, //U+95D3
0, //U+95D4
0, //U+95D5
0, //U+95D6
0, //U+95D7
0, //U+95D8
0, //U+95D9
0, //U+95DA
0, //U+95DB
0, //U+95DC
0, //U+95DD
0, //U+95DE
0, //U+95DF
0, //U+95E0
0, //U+95E1
0, //U+95E2
0, //U+95E3
0, //U+95E4
0, //U+95E5
0, //U+95E6
0, //U+95E7
0, //U+95E8
0, //U+95E9
0, //U+95EA
0, //U+95EB
0, //U+95EC
0, //U+95ED
0, //U+95EE
0, //U+95EF
0, //U+95F0
0, //U+95F1
0, //U+95F2
0, //U+95F3
0, //U+95F4
0, //U+95F5
0, //U+95F6
0, //U+95F7
0, //U+95F8
0, //U+95F9
0, //U+95FA
0, //U+95FB
0, //U+95FC
0, //U+95FD
0, //U+95FE
0, //U+95FF
0, //U+9600
0, //U+9601
0, //U+9602
0, //U+9603
0, //U+9604
0, //U+9605
0, //U+9606
0, //U+9607
0, //U+9608
0, //U+9609
0, //U+960A
0, //U+960B
0, //U+960C
0, //U+960D
0, //U+960E
0, //U+960F
0, //U+9610
0, //U+9611
0, //U+9612
0, //U+9613
0, //U+9614
0, //U+9615
0, //U+9616
0, //U+9617
0, //U+9618
0, //U+9619
0, //U+961A
0, //U+961B
0, //U+961C
0, //U+961D
0, //U+961E
0, //U+961F
0, //U+9620
0, //U+9621
0, //U+9622
0, //U+9623
0, //U+9624
0, //U+9625
0, //U+9626
0, //U+9627
0, //U+9628
0, //U+9629
0, //U+962A
0, //U+962B
0, //U+962C
0, //U+962D
0, //U+962E
0, //U+962F
0, //U+9630
0, //U+9631
0, //U+9632
0, //U+9633
0, //U+9634
0, //U+9635
0, //U+9636
0, //U+9637
0, //U+9638
0, //U+9639
0, //U+963A
0, //U+963B
0, //U+963C
0, //U+963D
0, //U+963E
0, //U+963F
0, //U+9640
0, //U+9641
0, //U+9642
0, //U+9643
0, //U+9644
0, //U+9645
0, //U+9646
0, //U+9647
0, //U+9648
0, //U+9649
0, //U+964A
0, //U+964B
0, //U+964C
0, //U+964D
0, //U+964E
0, //U+964F
0, //U+9650
0, //U+9651
0, //U+9652
0, //U+9653
0, //U+9654
0, //U+9655
0, //U+9656
0, //U+9657
0, //U+9658
0, //U+9659
0, //U+965A
0, //U+965B
0, //U+965C
0, //U+965D
0, //U+965E
0, //U+965F
0, //U+9660
0, //U+9661
0, //U+9662
0, //U+9663
0, //U+9664
0, //U+9665
0, //U+9666
0, //U+9667
0, //U+9668
0, //U+9669
0, //U+966A
0, //U+966B
0, //U+966C
0, //U+966D
0, //U+966E
0, //U+966F
0, //U+9670
0, //U+9671
0, //U+9672
0, //U+9673
0, //U+9674
0, //U+9675
0, //U+9676
0, //U+9677
0, //U+9678
0, //U+9679
0, //U+967A
0, //U+967B
0, //U+967C
0, //U+967D
0, //U+967E
0, //U+967F
0, //U+9680
0, //U+9681
0, //U+9682
0, //U+9683
0, //U+9684
0, //U+9685
0, //U+9686
0, //U+9687
0, //U+9688
0, //U+9689
0, //U+968A
0, //U+968B
0, //U+968C
0, //U+968D
0, //U+968E
0, //U+968F
0, //U+9690
0, //U+9691
0, //U+9692
0, //U+9693
0, //U+9694
0, //U+9695
0, //U+9696
0, //U+9697
0, //U+9698
0, //U+9699
0, //U+969A
0, //U+969B
0, //U+969C
0, //U+969D
0, //U+969E
0, //U+969F
0, //U+96A0
0, //U+96A1
0, //U+96A2
0, //U+96A3
0, //U+96A4
0, //U+96A5
0, //U+96A6
0, //U+96A7
0, //U+96A8
0, //U+96A9
0, //U+96AA
0, //U+96AB
0, //U+96AC
0, //U+96AD
0, //U+96AE
0, //U+96AF
0, //U+96B0
0, //U+96B1
0, //U+96B2
0, //U+96B3
0, //U+96B4
0, //U+96B5
0, //U+96B6
0, //U+96B7
0, //U+96B8
0, //U+96B9
0, //U+96BA
0, //U+96BB
0, //U+96BC
0, //U+96BD
0, //U+96BE
0, //U+96BF
0, //U+96C0
0, //U+96C1
0, //U+96C2
0, //U+96C3
0, //U+96C4
0, //U+96C5
0, //U+96C6
0, //U+96C7
0, //U+96C8
0, //U+96C9
0, //U+96CA
0, //U+96CB
0, //U+96CC
0, //U+96CD
0, //U+96CE
0, //U+96CF
0, //U+96D0
0, //U+96D1
0, //U+96D2
0, //U+96D3
0, //U+96D4
0, //U+96D5
0, //U+96D6
0, //U+96D7
0, //U+96D8
0, //U+96D9
0, //U+96DA
0, //U+96DB
0, //U+96DC
0, //U+96DD
0, //U+96DE
0, //U+96DF
0, //U+96E0
0, //U+96E1
0, //U+96E2
0, //U+96E3
0, //U+96E4
0, //U+96E5
0, //U+96E6
0, //U+96E7
0, //U+96E8
0, //U+96E9
0, //U+96EA
0, //U+96EB
0, //U+96EC
0, //U+96ED
0, //U+96EE
0, //U+96EF
0, //U+96F0
0, //U+96F1
0, //U+96F2
0, //U+96F3
0, //U+96F4
0, //U+96F5
0, //U+96F6
0, //U+96F7
0, //U+96F8
0, //U+96F9
0, //U+96FA
0, //U+96FB
0, //U+96FC
0, //U+96FD
0, //U+96FE
0, //U+96FF
0, //U+9700
0, //U+9701
0, //U+9702
0, //U+9703
0, //U+9704
0, //U+9705
0, //U+9706
0, //U+9707
0, //U+9708
0, //U+9709
0, //U+970A
0, //U+970B
0, //U+970C
0, //U+970D
0, //U+970E
0, //U+970F
0, //U+9710
0, //U+9711
0, //U+9712
0, //U+9713
0, //U+9714
0, //U+9715
0, //U+9716
0, //U+9717
0, //U+9718
0, //U+9719
0, //U+971A
0, //U+971B
0, //U+971C
0, //U+971D
0, //U+971E
0, //U+971F
0, //U+9720
0, //U+9721
0, //U+9722
0, //U+9723
0, //U+9724
0, //U+9725
0, //U+9726
0, //U+9727
0, //U+9728
0, //U+9729
0, //U+972A
0, //U+972B
0, //U+972C
0, //U+972D
0, //U+972E
0, //U+972F
0, //U+9730
0, //U+9731
0, //U+9732
0, //U+9733
0, //U+9734
0, //U+9735
0, //U+9736
0, //U+9737
0, //U+9738
0, //U+9739
0, //U+973A
0, //U+973B
0, //U+973C
0, //U+973D
0, //U+973E
0, //U+973F
0, //U+9740
0, //U+9741
0, //U+9742
0, //U+9743
0, //U+9744
0, //U+9745
0, //U+9746
0, //U+9747
0, //U+9748
0, //U+9749
0, //U+974A
0, //U+974B
0, //U+974C
0, //U+974D
0, //U+974E
0, //U+974F
0, //U+9750
0, //U+9751
0, //U+9752
0, //U+9753
0, //U+9754
0, //U+9755
0, //U+9756
0, //U+9757
0, //U+9758
0, //U+9759
0, //U+975A
0, //U+975B
0, //U+975C
0, //U+975D
0, //U+975E
0, //U+975F
0, //U+9760
0, //U+9761
0, //U+9762
0, //U+9763
0, //U+9764
0, //U+9765
0, //U+9766
0, //U+9767
0, //U+9768
0, //U+9769
0, //U+976A
0, //U+976B
0, //U+976C
0, //U+976D
0, //U+976E
0, //U+976F
0, //U+9770
0, //U+9771
0, //U+9772
0, //U+9773
0, //U+9774
0, //U+9775
0, //U+9776
0, //U+9777
0, //U+9778
0, //U+9779
0, //U+977A
0, //U+977B
0, //U+977C
0, //U+977D
0, //U+977E
0, //U+977F
0, //U+9780
0, //U+9781
0, //U+9782
0, //U+9783
0, //U+9784
0, //U+9785
0, //U+9786
0, //U+9787
0, //U+9788
0, //U+9789
0, //U+978A
0, //U+978B
0, //U+978C
0, //U+978D
0, //U+978E
0, //U+978F
0, //U+9790
0, //U+9791
0, //U+9792
0, //U+9793
0, //U+9794
0, //U+9795
0, //U+9796
0, //U+9797
0, //U+9798
0, //U+9799
0, //U+979A
0, //U+979B
0, //U+979C
0, //U+979D
0, //U+979E
0, //U+979F
0, //U+97A0
0, //U+97A1
0, //U+97A2
0, //U+97A3
0, //U+97A4
0, //U+97A5
0, //U+97A6
0, //U+97A7
0, //U+97A8
0, //U+97A9
0, //U+97AA
0, //U+97AB
0, //U+97AC
0, //U+97AD
0, //U+97AE
0, //U+97AF
0, //U+97B0
0, //U+97B1
0, //U+97B2
0, //U+97B3
0, //U+97B4
0, //U+97B5
0, //U+97B6
0, //U+97B7
0, //U+97B8
0, //U+97B9
0, //U+97BA
0, //U+97BB
0, //U+97BC
0, //U+97BD
0, //U+97BE
0, //U+97BF
0, //U+97C0
0, //U+97C1
0, //U+97C2
0, //U+97C3
0, //U+97C4
0, //U+97C5
0, //U+97C6
0, //U+97C7
0, //U+97C8
0, //U+97C9
0, //U+97CA
0, //U+97CB
0, //U+97CC
0, //U+97CD
0, //U+97CE
0, //U+97CF
0, //U+97D0
0, //U+97D1
0, //U+97D2
0, //U+97D3
0, //U+97D4
0, //U+97D5
0, //U+97D6
0, //U+97D7
0, //U+97D8
0, //U+97D9
0, //U+97DA
0, //U+97DB
0, //U+97DC
0, //U+97DD
0, //U+97DE
0, //U+97DF
0, //U+97E0
0, //U+97E1
0, //U+97E2
0, //U+97E3
0, //U+97E4
0, //U+97E5
0, //U+97E6
0, //U+97E7
0, //U+97E8
0, //U+97E9
0, //U+97EA
0, //U+97EB
0, //U+97EC
0, //U+97ED
0, //U+97EE
0, //U+97EF
0, //U+97F0
0, //U+97F1
0, //U+97F2
0, //U+97F3
0, //U+97F4
0, //U+97F5
0, //U+97F6
0, //U+97F7
0, //U+97F8
0, //U+97F9
0, //U+97FA
0, //U+97FB
0, //U+97FC
0, //U+97FD
0, //U+97FE
0, //U+97FF
0, //U+9800
0, //U+9801
0, //U+9802
0, //U+9803
0, //U+9804
0, //U+9805
0, //U+9806
0, //U+9807
0, //U+9808
0, //U+9809
0, //U+980A
0, //U+980B
0, //U+980C
0, //U+980D
0, //U+980E
0, //U+980F
0, //U+9810
0, //U+9811
0, //U+9812
0, //U+9813
0, //U+9814
0, //U+9815
0, //U+9816
0, //U+9817
0, //U+9818
0, //U+9819
0, //U+981A
0, //U+981B
0, //U+981C
0, //U+981D
0, //U+981E
0, //U+981F
0, //U+9820
0, //U+9821
0, //U+9822
0, //U+9823
0, //U+9824
0, //U+9825
0, //U+9826
0, //U+9827
0, //U+9828
0, //U+9829
0, //U+982A
0, //U+982B
0, //U+982C
0, //U+982D
0, //U+982E
0, //U+982F
0, //U+9830
0, //U+9831
0, //U+9832
0, //U+9833
0, //U+9834
0, //U+9835
0, //U+9836
0, //U+9837
0, //U+9838
0, //U+9839
0, //U+983A
0, //U+983B
0, //U+983C
0, //U+983D
0, //U+983E
0, //U+983F
0, //U+9840
0, //U+9841
0, //U+9842
0, //U+9843
0, //U+9844
0, //U+9845
0, //U+9846
0, //U+9847
0, //U+9848
0, //U+9849
0, //U+984A
0, //U+984B
0, //U+984C
0, //U+984D
0, //U+984E
0, //U+984F
0, //U+9850
0, //U+9851
0, //U+9852
0, //U+9853
0, //U+9854
0, //U+9855
0, //U+9856
0, //U+9857
0, //U+9858
0, //U+9859
0, //U+985A
0, //U+985B
0, //U+985C
0, //U+985D
0, //U+985E
0, //U+985F
0, //U+9860
0, //U+9861
0, //U+9862
0, //U+9863
0, //U+9864
0, //U+9865
0, //U+9866
0, //U+9867
0, //U+9868
0, //U+9869
0, //U+986A
0, //U+986B
0, //U+986C
0, //U+986D
0, //U+986E
0, //U+986F
0, //U+9870
0, //U+9871
0, //U+9872
0, //U+9873
0, //U+9874
0, //U+9875
0, //U+9876
0, //U+9877
0, //U+9878
0, //U+9879
0, //U+987A
0, //U+987B
0, //U+987C
0, //U+987D
0, //U+987E
0, //U+987F
0, //U+9880
0, //U+9881
0, //U+9882
0, //U+9883
0, //U+9884
0, //U+9885
0, //U+9886
0, //U+9887
0, //U+9888
0, //U+9889
0, //U+988A
0, //U+988B
0, //U+988C
0, //U+988D
0, //U+988E
0, //U+988F
0, //U+9890
0, //U+9891
0, //U+9892
0, //U+9893
0, //U+9894
0, //U+9895
0, //U+9896
0, //U+9897
0, //U+9898
0, //U+9899
0, //U+989A
0, //U+989B
0, //U+989C
0, //U+989D
0, //U+989E
0, //U+989F
0, //U+98A0
0, //U+98A1
0, //U+98A2
0, //U+98A3
0, //U+98A4
0, //U+98A5
0, //U+98A6
0, //U+98A7
0, //U+98A8
0, //U+98A9
0, //U+98AA
0, //U+98AB
0, //U+98AC
0, //U+98AD
0, //U+98AE
0, //U+98AF
0, //U+98B0
0, //U+98B1
0, //U+98B2
0, //U+98B3
0, //U+98B4
0, //U+98B5
0, //U+98B6
0, //U+98B7
0, //U+98B8
0, //U+98B9
0, //U+98BA
0, //U+98BB
0, //U+98BC
0, //U+98BD
0, //U+98BE
0, //U+98BF
0, //U+98C0
0, //U+98C1
0, //U+98C2
0, //U+98C3
0, //U+98C4
0, //U+98C5
0, //U+98C6
0, //U+98C7
0, //U+98C8
0, //U+98C9
0, //U+98CA
0, //U+98CB
0, //U+98CC
0, //U+98CD
0, //U+98CE
0, //U+98CF
0, //U+98D0
0, //U+98D1
0, //U+98D2
0, //U+98D3
0, //U+98D4
0, //U+98D5
0, //U+98D6
0, //U+98D7
0, //U+98D8
0, //U+98D9
0, //U+98DA
0, //U+98DB
0, //U+98DC
0, //U+98DD
0, //U+98DE
0, //U+98DF
0, //U+98E0
0, //U+98E1
0, //U+98E2
0, //U+98E3
0, //U+98E4
0, //U+98E5
0, //U+98E6
0, //U+98E7
0, //U+98E8
0, //U+98E9
0, //U+98EA
0, //U+98EB
0, //U+98EC
0, //U+98ED
0, //U+98EE
0, //U+98EF
0, //U+98F0
0, //U+98F1
0, //U+98F2
0, //U+98F3
0, //U+98F4
0, //U+98F5
0, //U+98F6
0, //U+98F7
0, //U+98F8
0, //U+98F9
0, //U+98FA
0, //U+98FB
0, //U+98FC
0, //U+98FD
0, //U+98FE
0, //U+98FF
0, //U+9900
0, //U+9901
0, //U+9902
0, //U+9903
0, //U+9904
0, //U+9905
0, //U+9906
0, //U+9907
0, //U+9908
0, //U+9909
0, //U+990A
0, //U+990B
0, //U+990C
0, //U+990D
0, //U+990E
0, //U+990F
0, //U+9910
0, //U+9911
0, //U+9912
0, //U+9913
0, //U+9914
0, //U+9915
0, //U+9916
0, //U+9917
0, //U+9918
0, //U+9919
0, //U+991A
0, //U+991B
0, //U+991C
0, //U+991D
0, //U+991E
0, //U+991F
0, //U+9920
0, //U+9921
0, //U+9922
0, //U+9923
0, //U+9924
0, //U+9925
0, //U+9926
0, //U+9927
0, //U+9928
0, //U+9929
0, //U+992A
0, //U+992B
0, //U+992C
0, //U+992D
0, //U+992E
0, //U+992F
0, //U+9930
0, //U+9931
0, //U+9932
0, //U+9933
0, //U+9934
0, //U+9935
0, //U+9936
0, //U+9937
0, //U+9938
0, //U+9939
0, //U+993A
0, //U+993B
0, //U+993C
0, //U+993D
0, //U+993E
0, //U+993F
0, //U+9940
0, //U+9941
0, //U+9942
0, //U+9943
0, //U+9944
0, //U+9945
0, //U+9946
0, //U+9947
0, //U+9948
0, //U+9949
0, //U+994A
0, //U+994B
0, //U+994C
0, //U+994D
0, //U+994E
0, //U+994F
0, //U+9950
0, //U+9951
0, //U+9952
0, //U+9953
0, //U+9954
0, //U+9955
0, //U+9956
0, //U+9957
0, //U+9958
0, //U+9959
0, //U+995A
0, //U+995B
0, //U+995C
0, //U+995D
0, //U+995E
0, //U+995F
0, //U+9960
0, //U+9961
0, //U+9962
0, //U+9963
0, //U+9964
0, //U+9965
0, //U+9966
0, //U+9967
0, //U+9968
0, //U+9969
0, //U+996A
0, //U+996B
0, //U+996C
0, //U+996D
0, //U+996E
0, //U+996F
0, //U+9970
0, //U+9971
0, //U+9972
0, //U+9973
0, //U+9974
0, //U+9975
0, //U+9976
0, //U+9977
0, //U+9978
0, //U+9979
0, //U+997A
0, //U+997B
0, //U+997C
0, //U+997D
0, //U+997E
0, //U+997F
0, //U+9980
0, //U+9981
0, //U+9982
0, //U+9983
0, //U+9984
0, //U+9985
0, //U+9986
0, //U+9987
0, //U+9988
0, //U+9989
0, //U+998A
0, //U+998B
0, //U+998C
0, //U+998D
0, //U+998E
0, //U+998F
0, //U+9990
0, //U+9991
0, //U+9992
0, //U+9993
0, //U+9994
0, //U+9995
0, //U+9996
0, //U+9997
0, //U+9998
0, //U+9999
0, //U+999A
0, //U+999B
0, //U+999C
0, //U+999D
0, //U+999E
0, //U+999F
0, //U+99A0
0, //U+99A1
0, //U+99A2
0, //U+99A3
0, //U+99A4
0, //U+99A5
0, //U+99A6
0, //U+99A7
0, //U+99A8
0, //U+99A9
0, //U+99AA
0, //U+99AB
0, //U+99AC
0, //U+99AD
0, //U+99AE
0, //U+99AF
0, //U+99B0
0, //U+99B1
0, //U+99B2
0, //U+99B3
0, //U+99B4
0, //U+99B5
0, //U+99B6
0, //U+99B7
0, //U+99B8
0, //U+99B9
0, //U+99BA
0, //U+99BB
0, //U+99BC
0, //U+99BD
0, //U+99BE
0, //U+99BF
0, //U+99C0
0, //U+99C1
0, //U+99C2
0, //U+99C3
0, //U+99C4
0, //U+99C5
0, //U+99C6
0, //U+99C7
0, //U+99C8
0, //U+99C9
0, //U+99CA
0, //U+99CB
0, //U+99CC
0, //U+99CD
0, //U+99CE
0, //U+99CF
0, //U+99D0
0, //U+99D1
0, //U+99D2
0, //U+99D3
0, //U+99D4
0, //U+99D5
0, //U+99D6
0, //U+99D7
0, //U+99D8
0, //U+99D9
0, //U+99DA
0, //U+99DB
0, //U+99DC
0, //U+99DD
0, //U+99DE
0, //U+99DF
0, //U+99E0
0, //U+99E1
0, //U+99E2
0, //U+99E3
0, //U+99E4
0, //U+99E5
0, //U+99E6
0, //U+99E7
0, //U+99E8
0, //U+99E9
0, //U+99EA
0, //U+99EB
0, //U+99EC
0, //U+99ED
0, //U+99EE
0, //U+99EF
0, //U+99F0
0, //U+99F1
0, //U+99F2
0, //U+99F3
0, //U+99F4
0, //U+99F5
0, //U+99F6
0, //U+99F7
0, //U+99F8
0, //U+99F9
0, //U+99FA
0, //U+99FB
0, //U+99FC
0, //U+99FD
0, //U+99FE
0, //U+99FF
0, //U+9A00
0, //U+9A01
0, //U+9A02
0, //U+9A03
0, //U+9A04
0, //U+9A05
0, //U+9A06
0, //U+9A07
0, //U+9A08
0, //U+9A09
0, //U+9A0A
0, //U+9A0B
0, //U+9A0C
0, //U+9A0D
0, //U+9A0E
0, //U+9A0F
0, //U+9A10
0, //U+9A11
0, //U+9A12
0, //U+9A13
0, //U+9A14
0, //U+9A15
0, //U+9A16
0, //U+9A17
0, //U+9A18
0, //U+9A19
0, //U+9A1A
0, //U+9A1B
0, //U+9A1C
0, //U+9A1D
0, //U+9A1E
0, //U+9A1F
0, //U+9A20
0, //U+9A21
0, //U+9A22
0, //U+9A23
0, //U+9A24
0, //U+9A25
0, //U+9A26
0, //U+9A27
0, //U+9A28
0, //U+9A29
0, //U+9A2A
0, //U+9A2B
0, //U+9A2C
0, //U+9A2D
0, //U+9A2E
0, //U+9A2F
0, //U+9A30
0, //U+9A31
0, //U+9A32
0, //U+9A33
0, //U+9A34
0, //U+9A35
0, //U+9A36
0, //U+9A37
0, //U+9A38
0, //U+9A39
0, //U+9A3A
0, //U+9A3B
0, //U+9A3C
0, //U+9A3D
0, //U+9A3E
0, //U+9A3F
0, //U+9A40
0, //U+9A41
0, //U+9A42
0, //U+9A43
0, //U+9A44
0, //U+9A45
0, //U+9A46
0, //U+9A47
0, //U+9A48
0, //U+9A49
0, //U+9A4A
0, //U+9A4B
0, //U+9A4C
0, //U+9A4D
0, //U+9A4E
0, //U+9A4F
0, //U+9A50
0, //U+9A51
0, //U+9A52
0, //U+9A53
0, //U+9A54
0, //U+9A55
0, //U+9A56
0, //U+9A57
0, //U+9A58
0, //U+9A59
0, //U+9A5A
0, //U+9A5B
0, //U+9A5C
0, //U+9A5D
0, //U+9A5E
0, //U+9A5F
0, //U+9A60
0, //U+9A61
0, //U+9A62
0, //U+9A63
0, //U+9A64
0, //U+9A65
0, //U+9A66
0, //U+9A67
0, //U+9A68
0, //U+9A69
0, //U+9A6A
0, //U+9A6B
0, //U+9A6C
0, //U+9A6D
0, //U+9A6E
0, //U+9A6F
0, //U+9A70
0, //U+9A71
0, //U+9A72
0, //U+9A73
0, //U+9A74
0, //U+9A75
0, //U+9A76
0, //U+9A77
0, //U+9A78
0, //U+9A79
0, //U+9A7A
0, //U+9A7B
0, //U+9A7C
0, //U+9A7D
0, //U+9A7E
0, //U+9A7F
0, //U+9A80
0, //U+9A81
0, //U+9A82
0, //U+9A83
0, //U+9A84
0, //U+9A85
0, //U+9A86
0, //U+9A87
0, //U+9A88
0, //U+9A89
0, //U+9A8A
0, //U+9A8B
0, //U+9A8C
0, //U+9A8D
0, //U+9A8E
0, //U+9A8F
0, //U+9A90
0, //U+9A91
0, //U+9A92
0, //U+9A93
0, //U+9A94
0, //U+9A95
0, //U+9A96
0, //U+9A97
0, //U+9A98
0, //U+9A99
0, //U+9A9A
0, //U+9A9B
0, //U+9A9C
0, //U+9A9D
0, //U+9A9E
0, //U+9A9F
0, //U+9AA0
0, //U+9AA1
0, //U+9AA2
0, //U+9AA3
0, //U+9AA4
0, //U+9AA5
0, //U+9AA6
0, //U+9AA7
0, //U+9AA8
0, //U+9AA9
0, //U+9AAA
0, //U+9AAB
0, //U+9AAC
0, //U+9AAD
0, //U+9AAE
0, //U+9AAF
0, //U+9AB0
0, //U+9AB1
0, //U+9AB2
0, //U+9AB3
0, //U+9AB4
0, //U+9AB5
0, //U+9AB6
0, //U+9AB7
0, //U+9AB8
0, //U+9AB9
0, //U+9ABA
0, //U+9ABB
0, //U+9ABC
0, //U+9ABD
0, //U+9ABE
0, //U+9ABF
0, //U+9AC0
0, //U+9AC1
0, //U+9AC2
0, //U+9AC3
0, //U+9AC4
0, //U+9AC5
0, //U+9AC6
0, //U+9AC7
0, //U+9AC8
0, //U+9AC9
0, //U+9ACA
0, //U+9ACB
0, //U+9ACC
0, //U+9ACD
0, //U+9ACE
0, //U+9ACF
0, //U+9AD0
0, //U+9AD1
0, //U+9AD2
0, //U+9AD3
0, //U+9AD4
0, //U+9AD5
0, //U+9AD6
0, //U+9AD7
0, //U+9AD8
0, //U+9AD9
0, //U+9ADA
0, //U+9ADB
0, //U+9ADC
0, //U+9ADD
0, //U+9ADE
0, //U+9ADF
0, //U+9AE0
0, //U+9AE1
0, //U+9AE2
0, //U+9AE3
0, //U+9AE4
0, //U+9AE5
0, //U+9AE6
0, //U+9AE7
0, //U+9AE8
0, //U+9AE9
0, //U+9AEA
0, //U+9AEB
0, //U+9AEC
0, //U+9AED
0, //U+9AEE
0, //U+9AEF
0, //U+9AF0
0, //U+9AF1
0, //U+9AF2
0, //U+9AF3
0, //U+9AF4
0, //U+9AF5
0, //U+9AF6
0, //U+9AF7
0, //U+9AF8
0, //U+9AF9
0, //U+9AFA
0, //U+9AFB
0, //U+9AFC
0, //U+9AFD
0, //U+9AFE
0, //U+9AFF
0, //U+9B00
0, //U+9B01
0, //U+9B02
0, //U+9B03
0, //U+9B04
0, //U+9B05
0, //U+9B06
0, //U+9B07
0, //U+9B08
0, //U+9B09
0, //U+9B0A
0, //U+9B0B
0, //U+9B0C
0, //U+9B0D
0, //U+9B0E
0, //U+9B0F
0, //U+9B10
0, //U+9B11
0, //U+9B12
0, //U+9B13
0, //U+9B14
0, //U+9B15
0, //U+9B16
0, //U+9B17
0, //U+9B18
0, //U+9B19
0, //U+9B1A
0, //U+9B1B
0, //U+9B1C
0, //U+9B1D
0, //U+9B1E
0, //U+9B1F
0, //U+9B20
0, //U+9B21
0, //U+9B22
0, //U+9B23
0, //U+9B24
0, //U+9B25
0, //U+9B26
0, //U+9B27
0, //U+9B28
0, //U+9B29
0, //U+9B2A
0, //U+9B2B
0, //U+9B2C
0, //U+9B2D
0, //U+9B2E
0, //U+9B2F
0, //U+9B30
0, //U+9B31
0, //U+9B32
0, //U+9B33
0, //U+9B34
0, //U+9B35
0, //U+9B36
0, //U+9B37
0, //U+9B38
0, //U+9B39
0, //U+9B3A
0, //U+9B3B
0, //U+9B3C
0, //U+9B3D
0, //U+9B3E
0, //U+9B3F
0, //U+9B40
0, //U+9B41
0, //U+9B42
0, //U+9B43
0, //U+9B44
0, //U+9B45
0, //U+9B46
0, //U+9B47
0, //U+9B48
0, //U+9B49
0, //U+9B4A
0, //U+9B4B
0, //U+9B4C
0, //U+9B4D
0, //U+9B4E
0, //U+9B4F
0, //U+9B50
0, //U+9B51
0, //U+9B52
0, //U+9B53
0, //U+9B54
0, //U+9B55
0, //U+9B56
0, //U+9B57
0, //U+9B58
0, //U+9B59
0, //U+9B5A
0, //U+9B5B
0, //U+9B5C
0, //U+9B5D
0, //U+9B5E
0, //U+9B5F
0, //U+9B60
0, //U+9B61
0, //U+9B62
0, //U+9B63
0, //U+9B64
0, //U+9B65
0, //U+9B66
0, //U+9B67
0, //U+9B68
0, //U+9B69
0, //U+9B6A
0, //U+9B6B
0, //U+9B6C
0, //U+9B6D
0, //U+9B6E
0, //U+9B6F
0, //U+9B70
0, //U+9B71
0, //U+9B72
0, //U+9B73
0, //U+9B74
0, //U+9B75
0, //U+9B76
0, //U+9B77
0, //U+9B78
0, //U+9B79
0, //U+9B7A
0, //U+9B7B
0, //U+9B7C
0, //U+9B7D
0, //U+9B7E
0, //U+9B7F
0, //U+9B80
0, //U+9B81
0, //U+9B82
0, //U+9B83
0, //U+9B84
0, //U+9B85
0, //U+9B86
0, //U+9B87
0, //U+9B88
0, //U+9B89
0, //U+9B8A
0, //U+9B8B
0, //U+9B8C
0, //U+9B8D
0, //U+9B8E
0, //U+9B8F
0, //U+9B90
0, //U+9B91
0, //U+9B92
0, //U+9B93
0, //U+9B94
0, //U+9B95
0, //U+9B96
0, //U+9B97
0, //U+9B98
0, //U+9B99
0, //U+9B9A
0, //U+9B9B
0, //U+9B9C
0, //U+9B9D
0, //U+9B9E
0, //U+9B9F
0, //U+9BA0
0, //U+9BA1
0, //U+9BA2
0, //U+9BA3
0, //U+9BA4
0, //U+9BA5
0, //U+9BA6
0, //U+9BA7
0, //U+9BA8
0, //U+9BA9
0, //U+9BAA
0, //U+9BAB
0, //U+9BAC
0, //U+9BAD
0, //U+9BAE
0, //U+9BAF
0, //U+9BB0
0, //U+9BB1
0, //U+9BB2
0, //U+9BB3
0, //U+9BB4
0, //U+9BB5
0, //U+9BB6
0, //U+9BB7
0, //U+9BB8
0, //U+9BB9
0, //U+9BBA
0, //U+9BBB
0, //U+9BBC
0, //U+9BBD
0, //U+9BBE
0, //U+9BBF
0, //U+9BC0
0, //U+9BC1
0, //U+9BC2
0, //U+9BC3
0, //U+9BC4
0, //U+9BC5
0, //U+9BC6
0, //U+9BC7
0, //U+9BC8
0, //U+9BC9
0, //U+9BCA
0, //U+9BCB
0, //U+9BCC
0, //U+9BCD
0, //U+9BCE
0, //U+9BCF
0, //U+9BD0
0, //U+9BD1
0, //U+9BD2
0, //U+9BD3
0, //U+9BD4
0, //U+9BD5
0, //U+9BD6
0, //U+9BD7
0, //U+9BD8
0, //U+9BD9
0, //U+9BDA
0, //U+9BDB
0, //U+9BDC
0, //U+9BDD
0, //U+9BDE
0, //U+9BDF
0, //U+9BE0
0, //U+9BE1
0, //U+9BE2
0, //U+9BE3
0, //U+9BE4
0, //U+9BE5
0, //U+9BE6
0, //U+9BE7
0, //U+9BE8
0, //U+9BE9
0, //U+9BEA
0, //U+9BEB
0, //U+9BEC
0, //U+9BED
0, //U+9BEE
0, //U+9BEF
0, //U+9BF0
0, //U+9BF1
0, //U+9BF2
0, //U+9BF3
0, //U+9BF4
0, //U+9BF5
0, //U+9BF6
0, //U+9BF7
0, //U+9BF8
0, //U+9BF9
0, //U+9BFA
0, //U+9BFB
0, //U+9BFC
0, //U+9BFD
0, //U+9BFE
0, //U+9BFF
0, //U+9C00
0, //U+9C01
0, //U+9C02
0, //U+9C03
0, //U+9C04
0, //U+9C05
0, //U+9C06
0, //U+9C07
0, //U+9C08
0, //U+9C09
0, //U+9C0A
0, //U+9C0B
0, //U+9C0C
0, //U+9C0D
0, //U+9C0E
0, //U+9C0F
0, //U+9C10
0, //U+9C11
0, //U+9C12
0, //U+9C13
0, //U+9C14
0, //U+9C15
0, //U+9C16
0, //U+9C17
0, //U+9C18
0, //U+9C19
0, //U+9C1A
0, //U+9C1B
0, //U+9C1C
0, //U+9C1D
0, //U+9C1E
0, //U+9C1F
0, //U+9C20
0, //U+9C21
0, //U+9C22
0, //U+9C23
0, //U+9C24
0, //U+9C25
0, //U+9C26
0, //U+9C27
0, //U+9C28
0, //U+9C29
0, //U+9C2A
0, //U+9C2B
0, //U+9C2C
0, //U+9C2D
0, //U+9C2E
0, //U+9C2F
0, //U+9C30
0, //U+9C31
0, //U+9C32
0, //U+9C33
0, //U+9C34
0, //U+9C35
0, //U+9C36
0, //U+9C37
0, //U+9C38
0, //U+9C39
0, //U+9C3A
0, //U+9C3B
0, //U+9C3C
0, //U+9C3D
0, //U+9C3E
0, //U+9C3F
0, //U+9C40
0, //U+9C41
0, //U+9C42
0, //U+9C43
0, //U+9C44
0, //U+9C45
0, //U+9C46
0, //U+9C47
0, //U+9C48
0, //U+9C49
0, //U+9C4A
0, //U+9C4B
0, //U+9C4C
0, //U+9C4D
0, //U+9C4E
0, //U+9C4F
0, //U+9C50
0, //U+9C51
0, //U+9C52
0, //U+9C53
0, //U+9C54
0, //U+9C55
0, //U+9C56
0, //U+9C57
0, //U+9C58
0, //U+9C59
0, //U+9C5A
0, //U+9C5B
0, //U+9C5C
0, //U+9C5D
0, //U+9C5E
0, //U+9C5F
0, //U+9C60
0, //U+9C61
0, //U+9C62
0, //U+9C63
0, //U+9C64
0, //U+9C65
0, //U+9C66
0, //U+9C67
0, //U+9C68
0, //U+9C69
0, //U+9C6A
0, //U+9C6B
0, //U+9C6C
0, //U+9C6D
0, //U+9C6E
0, //U+9C6F
0, //U+9C70
0, //U+9C71
0, //U+9C72
0, //U+9C73
0, //U+9C74
0, //U+9C75
0, //U+9C76
0, //U+9C77
0, //U+9C78
0, //U+9C79
0, //U+9C7A
0, //U+9C7B
0, //U+9C7C
0, //U+9C7D
0, //U+9C7E
0, //U+9C7F
0, //U+9C80
0, //U+9C81
0, //U+9C82
0, //U+9C83
0, //U+9C84
0, //U+9C85
0, //U+9C86
0, //U+9C87
0, //U+9C88
0, //U+9C89
0, //U+9C8A
0, //U+9C8B
0, //U+9C8C
0, //U+9C8D
0, //U+9C8E
0, //U+9C8F
0, //U+9C90
0, //U+9C91
0, //U+9C92
0, //U+9C93
0, //U+9C94
0, //U+9C95
0, //U+9C96
0, //U+9C97
0, //U+9C98
0, //U+9C99
0, //U+9C9A
0, //U+9C9B
0, //U+9C9C
0, //U+9C9D
0, //U+9C9E
0, //U+9C9F
0, //U+9CA0
0, //U+9CA1
0, //U+9CA2
0, //U+9CA3
0, //U+9CA4
0, //U+9CA5
0, //U+9CA6
0, //U+9CA7
0, //U+9CA8
0, //U+9CA9
0, //U+9CAA
0, //U+9CAB
0, //U+9CAC
0, //U+9CAD
0, //U+9CAE
0, //U+9CAF
0, //U+9CB0
0, //U+9CB1
0, //U+9CB2
0, //U+9CB3
0, //U+9CB4
0, //U+9CB5
0, //U+9CB6
0, //U+9CB7
0, //U+9CB8
0, //U+9CB9
0, //U+9CBA
0, //U+9CBB
0, //U+9CBC
0, //U+9CBD
0, //U+9CBE
0, //U+9CBF
0, //U+9CC0
0, //U+9CC1
0, //U+9CC2
0, //U+9CC3
0, //U+9CC4
0, //U+9CC5
0, //U+9CC6
0, //U+9CC7
0, //U+9CC8
0, //U+9CC9
0, //U+9CCA
0, //U+9CCB
0, //U+9CCC
0, //U+9CCD
0, //U+9CCE
0, //U+9CCF
0, //U+9CD0
0, //U+9CD1
0, //U+9CD2
0, //U+9CD3
0, //U+9CD4
0, //U+9CD5
0, //U+9CD6
0, //U+9CD7
0, //U+9CD8
0, //U+9CD9
0, //U+9CDA
0, //U+9CDB
0, //U+9CDC
0, //U+9CDD
0, //U+9CDE
0, //U+9CDF
0, //U+9CE0
0, //U+9CE1
0, //U+9CE2
0, //U+9CE3
0, //U+9CE4
0, //U+9CE5
0, //U+9CE6
0, //U+9CE7
0, //U+9CE8
0, //U+9CE9
0, //U+9CEA
0, //U+9CEB
0, //U+9CEC
0, //U+9CED
0, //U+9CEE
0, //U+9CEF
0, //U+9CF0
0, //U+9CF1
0, //U+9CF2
0, //U+9CF3
0, //U+9CF4
0, //U+9CF5
0, //U+9CF6
0, //U+9CF7
0, //U+9CF8
0, //U+9CF9
0, //U+9CFA
0, //U+9CFB
0, //U+9CFC
0, //U+9CFD
0, //U+9CFE
0, //U+9CFF
0, //U+9D00
0, //U+9D01
0, //U+9D02
0, //U+9D03
0, //U+9D04
0, //U+9D05
0, //U+9D06
0, //U+9D07
0, //U+9D08
0, //U+9D09
0, //U+9D0A
0, //U+9D0B
0, //U+9D0C
0, //U+9D0D
0, //U+9D0E
0, //U+9D0F
0, //U+9D10
0, //U+9D11
0, //U+9D12
0, //U+9D13
0, //U+9D14
0, //U+9D15
0, //U+9D16
0, //U+9D17
0, //U+9D18
0, //U+9D19
0, //U+9D1A
0, //U+9D1B
0, //U+9D1C
0, //U+9D1D
0, //U+9D1E
0, //U+9D1F
0, //U+9D20
0, //U+9D21
0, //U+9D22
0, //U+9D23
0, //U+9D24
0, //U+9D25
0, //U+9D26
0, //U+9D27
0, //U+9D28
0, //U+9D29
0, //U+9D2A
0, //U+9D2B
0, //U+9D2C
0, //U+9D2D
0, //U+9D2E
0, //U+9D2F
0, //U+9D30
0, //U+9D31
0, //U+9D32
0, //U+9D33
0, //U+9D34
0, //U+9D35
0, //U+9D36
0, //U+9D37
0, //U+9D38
0, //U+9D39
0, //U+9D3A
0, //U+9D3B
0, //U+9D3C
0, //U+9D3D
0, //U+9D3E
0, //U+9D3F
0, //U+9D40
0, //U+9D41
0, //U+9D42
0, //U+9D43
0, //U+9D44
0, //U+9D45
0, //U+9D46
0, //U+9D47
0, //U+9D48
0, //U+9D49
0, //U+9D4A
0, //U+9D4B
0, //U+9D4C
0, //U+9D4D
0, //U+9D4E
0, //U+9D4F
0, //U+9D50
0, //U+9D51
0, //U+9D52
0, //U+9D53
0, //U+9D54
0, //U+9D55
0, //U+9D56
0, //U+9D57
0, //U+9D58
0, //U+9D59
0, //U+9D5A
0, //U+9D5B
0, //U+9D5C
0, //U+9D5D
0, //U+9D5E
0, //U+9D5F
0, //U+9D60
0, //U+9D61
0, //U+9D62
0, //U+9D63
0, //U+9D64
0, //U+9D65
0, //U+9D66
0, //U+9D67
0, //U+9D68
0, //U+9D69
0, //U+9D6A
0, //U+9D6B
0, //U+9D6C
0, //U+9D6D
0, //U+9D6E
0, //U+9D6F
0, //U+9D70
0, //U+9D71
0, //U+9D72
0, //U+9D73
0, //U+9D74
0, //U+9D75
0, //U+9D76
0, //U+9D77
0, //U+9D78
0, //U+9D79
0, //U+9D7A
0, //U+9D7B
0, //U+9D7C
0, //U+9D7D
0, //U+9D7E
0, //U+9D7F
0, //U+9D80
0, //U+9D81
0, //U+9D82
0, //U+9D83
0, //U+9D84
0, //U+9D85
0, //U+9D86
0, //U+9D87
0, //U+9D88
0, //U+9D89
0, //U+9D8A
0, //U+9D8B
0, //U+9D8C
0, //U+9D8D
0, //U+9D8E
0, //U+9D8F
0, //U+9D90
0, //U+9D91
0, //U+9D92
0, //U+9D93
0, //U+9D94
0, //U+9D95
0, //U+9D96
0, //U+9D97
0, //U+9D98
0, //U+9D99
0, //U+9D9A
0, //U+9D9B
0, //U+9D9C
0, //U+9D9D
0, //U+9D9E
0, //U+9D9F
0, //U+9DA0
0, //U+9DA1
0, //U+9DA2
0, //U+9DA3
0, //U+9DA4
0, //U+9DA5
0, //U+9DA6
0, //U+9DA7
0, //U+9DA8
0, //U+9DA9
0, //U+9DAA
0, //U+9DAB
0, //U+9DAC
0, //U+9DAD
0, //U+9DAE
0, //U+9DAF
0, //U+9DB0
0, //U+9DB1
0, //U+9DB2
0, //U+9DB3
0, //U+9DB4
0, //U+9DB5
0, //U+9DB6
0, //U+9DB7
0, //U+9DB8
0, //U+9DB9
0, //U+9DBA
0, //U+9DBB
0, //U+9DBC
0, //U+9DBD
0, //U+9DBE
0, //U+9DBF
0, //U+9DC0
0, //U+9DC1
0, //U+9DC2
0, //U+9DC3
0, //U+9DC4
0, //U+9DC5
0, //U+9DC6
0, //U+9DC7
0, //U+9DC8
0, //U+9DC9
0, //U+9DCA
0, //U+9DCB
0, //U+9DCC
0, //U+9DCD
0, //U+9DCE
0, //U+9DCF
0, //U+9DD0
0, //U+9DD1
0, //U+9DD2
0, //U+9DD3
0, //U+9DD4
0, //U+9DD5
0, //U+9DD6
0, //U+9DD7
0, //U+9DD8
0, //U+9DD9
0, //U+9DDA
0, //U+9DDB
0, //U+9DDC
0, //U+9DDD
0, //U+9DDE
0, //U+9DDF
0, //U+9DE0
0, //U+9DE1
0, //U+9DE2
0, //U+9DE3
0, //U+9DE4
0, //U+9DE5
0, //U+9DE6
0, //U+9DE7
0, //U+9DE8
0, //U+9DE9
0, //U+9DEA
0, //U+9DEB
0, //U+9DEC
0, //U+9DED
0, //U+9DEE
0, //U+9DEF
0, //U+9DF0
0, //U+9DF1
0, //U+9DF2
0, //U+9DF3
0, //U+9DF4
0, //U+9DF5
0, //U+9DF6
0, //U+9DF7
0, //U+9DF8
0, //U+9DF9
0, //U+9DFA
0, //U+9DFB
0, //U+9DFC
0, //U+9DFD
0, //U+9DFE
0, //U+9DFF
0, //U+9E00
0, //U+9E01
0, //U+9E02
0, //U+9E03
0, //U+9E04
0, //U+9E05
0, //U+9E06
0, //U+9E07
0, //U+9E08
0, //U+9E09
0, //U+9E0A
0, //U+9E0B
0, //U+9E0C
0, //U+9E0D
0, //U+9E0E
0, //U+9E0F
0, //U+9E10
0, //U+9E11
0, //U+9E12
0, //U+9E13
0, //U+9E14
0, //U+9E15
0, //U+9E16
0, //U+9E17
0, //U+9E18
0, //U+9E19
0, //U+9E1A
0, //U+9E1B
0, //U+9E1C
0, //U+9E1D
0, //U+9E1E
0, //U+9E1F
0, //U+9E20
0, //U+9E21
0, //U+9E22
0, //U+9E23
0, //U+9E24
0, //U+9E25
0, //U+9E26
0, //U+9E27
0, //U+9E28
0, //U+9E29
0, //U+9E2A
0, //U+9E2B
0, //U+9E2C
0, //U+9E2D
0, //U+9E2E
0, //U+9E2F
0, //U+9E30
0, //U+9E31
0, //U+9E32
0, //U+9E33
0, //U+9E34
0, //U+9E35
0, //U+9E36
0, //U+9E37
0, //U+9E38
0, //U+9E39
0, //U+9E3A
0, //U+9E3B
0, //U+9E3C
0, //U+9E3D
0, //U+9E3E
0, //U+9E3F
0, //U+9E40
0, //U+9E41
0, //U+9E42
0, //U+9E43
0, //U+9E44
0, //U+9E45
0, //U+9E46
0, //U+9E47
0, //U+9E48
0, //U+9E49
0, //U+9E4A
0, //U+9E4B
0, //U+9E4C
0, //U+9E4D
0, //U+9E4E
0, //U+9E4F
0, //U+9E50
0, //U+9E51
0, //U+9E52
0, //U+9E53
0, //U+9E54
0, //U+9E55
0, //U+9E56
0, //U+9E57
0, //U+9E58
0, //U+9E59
0, //U+9E5A
0, //U+9E5B
0, //U+9E5C
0, //U+9E5D
0, //U+9E5E
0, //U+9E5F
0, //U+9E60
0, //U+9E61
0, //U+9E62
0, //U+9E63
0, //U+9E64
0, //U+9E65
0, //U+9E66
0, //U+9E67
0, //U+9E68
0, //U+9E69
0, //U+9E6A
0, //U+9E6B
0, //U+9E6C
0, //U+9E6D
0, //U+9E6E
0, //U+9E6F
0, //U+9E70
0, //U+9E71
0, //U+9E72
0, //U+9E73
0, //U+9E74
0, //U+9E75
0, //U+9E76
0, //U+9E77
0, //U+9E78
0, //U+9E79
0, //U+9E7A
0, //U+9E7B
0, //U+9E7C
0, //U+9E7D
0, //U+9E7E
0, //U+9E7F
0, //U+9E80
0, //U+9E81
0, //U+9E82
0, //U+9E83
0, //U+9E84
0, //U+9E85
0, //U+9E86
0, //U+9E87
0, //U+9E88
0, //U+9E89
0, //U+9E8A
0, //U+9E8B
0, //U+9E8C
0, //U+9E8D
0, //U+9E8E
0, //U+9E8F
0, //U+9E90
0, //U+9E91
0, //U+9E92
0, //U+9E93
0, //U+9E94
0, //U+9E95
0, //U+9E96
0, //U+9E97
0, //U+9E98
0, //U+9E99
0, //U+9E9A
0, //U+9E9B
0, //U+9E9C
0, //U+9E9D
0, //U+9E9E
0, //U+9E9F
0, //U+9EA0
0, //U+9EA1
0, //U+9EA2
0, //U+9EA3
0, //U+9EA4
0, //U+9EA5
0, //U+9EA6
0, //U+9EA7
0, //U+9EA8
0, //U+9EA9
0, //U+9EAA
0, //U+9EAB
0, //U+9EAC
0, //U+9EAD
0, //U+9EAE
0, //U+9EAF
0, //U+9EB0
0, //U+9EB1
0, //U+9EB2
0, //U+9EB3
0, //U+9EB4
0, //U+9EB5
0, //U+9EB6
0, //U+9EB7
0, //U+9EB8
0, //U+9EB9
0, //U+9EBA
0, //U+9EBB
0, //U+9EBC
0, //U+9EBD
0, //U+9EBE
0, //U+9EBF
0, //U+9EC0
0, //U+9EC1
0, //U+9EC2
0, //U+9EC3
0, //U+9EC4
0, //U+9EC5
0, //U+9EC6
0, //U+9EC7
0, //U+9EC8
0, //U+9EC9
0, //U+9ECA
0, //U+9ECB
0, //U+9ECC
0, //U+9ECD
0, //U+9ECE
0, //U+9ECF
0, //U+9ED0
0, //U+9ED1
0, //U+9ED2
0, //U+9ED3
0, //U+9ED4
0, //U+9ED5
0, //U+9ED6
0, //U+9ED7
0, //U+9ED8
0, //U+9ED9
0, //U+9EDA
0, //U+9EDB
0, //U+9EDC
0, //U+9EDD
0, //U+9EDE
0, //U+9EDF
0, //U+9EE0
0, //U+9EE1
0, //U+9EE2
0, //U+9EE3
0, //U+9EE4
0, //U+9EE5
0, //U+9EE6
0, //U+9EE7
0, //U+9EE8
0, //U+9EE9
0, //U+9EEA
0, //U+9EEB
0, //U+9EEC
0, //U+9EED
0, //U+9EEE
0, //U+9EEF
0, //U+9EF0
0, //U+9EF1
0, //U+9EF2
0, //U+9EF3
0, //U+9EF4
0, //U+9EF5
0, //U+9EF6
0, //U+9EF7
0, //U+9EF8
0, //U+9EF9
0, //U+9EFA
0, //U+9EFB
0, //U+9EFC
0, //U+9EFD
0, //U+9EFE
0, //U+9EFF
0, //U+9F00
0, //U+9F01
0, //U+9F02
0, //U+9F03
0, //U+9F04
0, //U+9F05
0, //U+9F06
0, //U+9F07
0, //U+9F08
0, //U+9F09
0, //U+9F0A
0, //U+9F0B
0, //U+9F0C
0, //U+9F0D
0, //U+9F0E
0, //U+9F0F
0, //U+9F10
0, //U+9F11
0, //U+9F12
0, //U+9F13
0, //U+9F14
0, //U+9F15
0, //U+9F16
0, //U+9F17
0, //U+9F18
0, //U+9F19
0, //U+9F1A
0, //U+9F1B
0, //U+9F1C
0, //U+9F1D
0, //U+9F1E
0, //U+9F1F
0, //U+9F20
0, //U+9F21
0, //U+9F22
0, //U+9F23
0, //U+9F24
0, //U+9F25
0, //U+9F26
0, //U+9F27
0, //U+9F28
0, //U+9F29
0, //U+9F2A
0, //U+9F2B
0, //U+9F2C
0, //U+9F2D
0, //U+9F2E
0, //U+9F2F
0, //U+9F30
0, //U+9F31
0, //U+9F32
0, //U+9F33
0, //U+9F34
0, //U+9F35
0, //U+9F36
0, //U+9F37
0, //U+9F38
0, //U+9F39
0, //U+9F3A
0, //U+9F3B
0, //U+9F3C
0, //U+9F3D
0, //U+9F3E
0, //U+9F3F
0, //U+9F40
0, //U+9F41
0, //U+9F42
0, //U+9F43
0, //U+9F44
0, //U+9F45
0, //U+9F46
0, //U+9F47
0, //U+9F48
0, //U+9F49
0, //U+9F4A
0, //U+9F4B
0, //U+9F4C
0, //U+9F4D
0, //U+9F4E
0, //U+9F4F
0, //U+9F50
0, //U+9F51
0, //U+9F52
0, //U+9F53
0, //U+9F54
0, //U+9F55
0, //U+9F56
0, //U+9F57
0, //U+9F58
0, //U+9F59
0, //U+9F5A
0, //U+9F5B
0, //U+9F5C
0, //U+9F5D
0, //U+9F5E
0, //U+9F5F
0, //U+9F60
0, //U+9F61
0, //U+9F62
0, //U+9F63
0, //U+9F64
0, //U+9F65
0, //U+9F66
0, //U+9F67
0, //U+9F68
0, //U+9F69
0, //U+9F6A
0, //U+9F6B
0, //U+9F6C
0, //U+9F6D
0, //U+9F6E
0, //U+9F6F
0, //U+9F70
0, //U+9F71
0, //U+9F72
0, //U+9F73
0, //U+9F74
0, //U+9F75
0, //U+9F76
0, //U+9F77
0, //U+9F78
0, //U+9F79
0, //U+9F7A
0, //U+9F7B
0, //U+9F7C
0, //U+9F7D
0, //U+9F7E
0, //U+9F7F
0, //U+9F80
0, //U+9F81
0, //U+9F82
0, //U+9F83
0, //U+9F84
0, //U+9F85
0, //U+9F86
0, //U+9F87
0, //U+9F88
0, //U+9F89
0, //U+9F8A
0, //U+9F8B
0, //U+9F8C
0, //U+9F8D
0, //U+9F8E
0, //U+9F8F
0, //U+9F90
0, //U+9F91
0, //U+9F92
0, //U+9F93
0, //U+9F94
0, //U+9F95
0, //U+9F96
0, //U+9F97
0, //U+9F98
0, //U+9F99
0, //U+9F9A
0, //U+9F9B
0, //U+9F9C
0, //U+9F9D
0, //U+9F9E
0, //U+9F9F
0, //U+9FA0
0, //U+9FA1
0, //U+9FA2
0, //U+9FA3
0, //U+9FA4
0, //U+9FA5
0, //U+9FA6
0, //U+9FA7
0, //U+9FA8
0, //U+9FA9
0, //U+9FAA
0, //U+9FAB
0, //U+9FAC
0, //U+9FAD
0, //U+9FAE
0, //U+9FAF
0, //U+9FB0
0, //U+9FB1
0, //U+9FB2
0, //U+9FB3
0, //U+9FB4
0, //U+9FB5
0, //U+9FB6
0, //U+9FB7
0, //U+9FB8
0, //U+9FB9
0, //U+9FBA
0, //U+9FBB
0, //U+9FBC
0, //U+9FBD
0, //U+9FBE
0, //U+9FBF
0, //U+9FC0
0, //U+9FC1
0, //U+9FC2
0, //U+9FC3
0, //U+9FC4
0, //U+9FC5
0, //U+9FC6
0, //U+9FC7
0, //U+9FC8
0, //U+9FC9
0, //U+9FCA
UNICODE_Lo, //U+9FCB
0, //U+9FCC
0, //U+9FCD
0, //U+9FCE
0, //U+9FCF
0, //U+9FD0
0, //U+9FD1
0, //U+9FD2
0, //U+9FD3
0, //U+9FD4
0, //U+9FD5
0, //U+9FD6
0, //U+9FD7
0, //U+9FD8
0, //U+9FD9
0, //U+9FDA
0, //U+9FDB
0, //U+9FDC
0, //U+9FDD
0, //U+9FDE
0, //U+9FDF
0, //U+9FE0
0, //U+9FE1
0, //U+9FE2
0, //U+9FE3
0, //U+9FE4
0, //U+9FE5
0, //U+9FE6
0, //U+9FE7
0, //U+9FE8
0, //U+9FE9
0, //U+9FEA
0, //U+9FEB
0, //U+9FEC
0, //U+9FED
0, //U+9FEE
0, //U+9FEF
0, //U+9FF0
0, //U+9FF1
0, //U+9FF2
0, //U+9FF3
0, //U+9FF4
0, //U+9FF5
0, //U+9FF6
0, //U+9FF7
0, //U+9FF8
0, //U+9FF9
0, //U+9FFA
0, //U+9FFB
0, //U+9FFC
0, //U+9FFD
0, //U+9FFE
0, //U+9FFF
UNICODE_Lo, //U+A000
UNICODE_Lo, //U+A001
UNICODE_Lo, //U+A002
UNICODE_Lo, //U+A003
UNICODE_Lo, //U+A004
UNICODE_Lo, //U+A005
UNICODE_Lo, //U+A006
UNICODE_Lo, //U+A007
UNICODE_Lo, //U+A008
UNICODE_Lo, //U+A009
UNICODE_Lo, //U+A00A
UNICODE_Lo, //U+A00B
UNICODE_Lo, //U+A00C
UNICODE_Lo, //U+A00D
UNICODE_Lo, //U+A00E
UNICODE_Lo, //U+A00F
UNICODE_Lo, //U+A010
UNICODE_Lo, //U+A011
UNICODE_Lo, //U+A012
UNICODE_Lo, //U+A013
UNICODE_Lo, //U+A014
UNICODE_Lm, //U+A015
UNICODE_Lo, //U+A016
UNICODE_Lo, //U+A017
UNICODE_Lo, //U+A018
UNICODE_Lo, //U+A019
UNICODE_Lo, //U+A01A
UNICODE_Lo, //U+A01B
UNICODE_Lo, //U+A01C
UNICODE_Lo, //U+A01D
UNICODE_Lo, //U+A01E
UNICODE_Lo, //U+A01F
UNICODE_Lo, //U+A020
UNICODE_Lo, //U+A021
UNICODE_Lo, //U+A022
UNICODE_Lo, //U+A023
UNICODE_Lo, //U+A024
UNICODE_Lo, //U+A025
UNICODE_Lo, //U+A026
UNICODE_Lo, //U+A027
UNICODE_Lo, //U+A028
UNICODE_Lo, //U+A029
UNICODE_Lo, //U+A02A
UNICODE_Lo, //U+A02B
UNICODE_Lo, //U+A02C
UNICODE_Lo, //U+A02D
UNICODE_Lo, //U+A02E
UNICODE_Lo, //U+A02F
UNICODE_Lo, //U+A030
UNICODE_Lo, //U+A031
UNICODE_Lo, //U+A032
UNICODE_Lo, //U+A033
UNICODE_Lo, //U+A034
UNICODE_Lo, //U+A035
UNICODE_Lo, //U+A036
UNICODE_Lo, //U+A037
UNICODE_Lo, //U+A038
UNICODE_Lo, //U+A039
UNICODE_Lo, //U+A03A
UNICODE_Lo, //U+A03B
UNICODE_Lo, //U+A03C
UNICODE_Lo, //U+A03D
UNICODE_Lo, //U+A03E
UNICODE_Lo, //U+A03F
UNICODE_Lo, //U+A040
UNICODE_Lo, //U+A041
UNICODE_Lo, //U+A042
UNICODE_Lo, //U+A043
UNICODE_Lo, //U+A044
UNICODE_Lo, //U+A045
UNICODE_Lo, //U+A046
UNICODE_Lo, //U+A047
UNICODE_Lo, //U+A048
UNICODE_Lo, //U+A049
UNICODE_Lo, //U+A04A
UNICODE_Lo, //U+A04B
UNICODE_Lo, //U+A04C
UNICODE_Lo, //U+A04D
UNICODE_Lo, //U+A04E
UNICODE_Lo, //U+A04F
UNICODE_Lo, //U+A050
UNICODE_Lo, //U+A051
UNICODE_Lo, //U+A052
UNICODE_Lo, //U+A053
UNICODE_Lo, //U+A054
UNICODE_Lo, //U+A055
UNICODE_Lo, //U+A056
UNICODE_Lo, //U+A057
UNICODE_Lo, //U+A058
UNICODE_Lo, //U+A059
UNICODE_Lo, //U+A05A
UNICODE_Lo, //U+A05B
UNICODE_Lo, //U+A05C
UNICODE_Lo, //U+A05D
UNICODE_Lo, //U+A05E
UNICODE_Lo, //U+A05F
UNICODE_Lo, //U+A060
UNICODE_Lo, //U+A061
UNICODE_Lo, //U+A062
UNICODE_Lo, //U+A063
UNICODE_Lo, //U+A064
UNICODE_Lo, //U+A065
UNICODE_Lo, //U+A066
UNICODE_Lo, //U+A067
UNICODE_Lo, //U+A068
UNICODE_Lo, //U+A069
UNICODE_Lo, //U+A06A
UNICODE_Lo, //U+A06B
UNICODE_Lo, //U+A06C
UNICODE_Lo, //U+A06D
UNICODE_Lo, //U+A06E
UNICODE_Lo, //U+A06F
UNICODE_Lo, //U+A070
UNICODE_Lo, //U+A071
UNICODE_Lo, //U+A072
UNICODE_Lo, //U+A073
UNICODE_Lo, //U+A074
UNICODE_Lo, //U+A075
UNICODE_Lo, //U+A076
UNICODE_Lo, //U+A077
UNICODE_Lo, //U+A078
UNICODE_Lo, //U+A079
UNICODE_Lo, //U+A07A
UNICODE_Lo, //U+A07B
UNICODE_Lo, //U+A07C
UNICODE_Lo, //U+A07D
UNICODE_Lo, //U+A07E
UNICODE_Lo, //U+A07F
UNICODE_Lo, //U+A080
UNICODE_Lo, //U+A081
UNICODE_Lo, //U+A082
UNICODE_Lo, //U+A083
UNICODE_Lo, //U+A084
UNICODE_Lo, //U+A085
UNICODE_Lo, //U+A086
UNICODE_Lo, //U+A087
UNICODE_Lo, //U+A088
UNICODE_Lo, //U+A089
UNICODE_Lo, //U+A08A
UNICODE_Lo, //U+A08B
UNICODE_Lo, //U+A08C
UNICODE_Lo, //U+A08D
UNICODE_Lo, //U+A08E
UNICODE_Lo, //U+A08F
UNICODE_Lo, //U+A090
UNICODE_Lo, //U+A091
UNICODE_Lo, //U+A092
UNICODE_Lo, //U+A093
UNICODE_Lo, //U+A094
UNICODE_Lo, //U+A095
UNICODE_Lo, //U+A096
UNICODE_Lo, //U+A097
UNICODE_Lo, //U+A098
UNICODE_Lo, //U+A099
UNICODE_Lo, //U+A09A
UNICODE_Lo, //U+A09B
UNICODE_Lo, //U+A09C
UNICODE_Lo, //U+A09D
UNICODE_Lo, //U+A09E
UNICODE_Lo, //U+A09F
UNICODE_Lo, //U+A0A0
UNICODE_Lo, //U+A0A1
UNICODE_Lo, //U+A0A2
UNICODE_Lo, //U+A0A3
UNICODE_Lo, //U+A0A4
UNICODE_Lo, //U+A0A5
UNICODE_Lo, //U+A0A6
UNICODE_Lo, //U+A0A7
UNICODE_Lo, //U+A0A8
UNICODE_Lo, //U+A0A9
UNICODE_Lo, //U+A0AA
UNICODE_Lo, //U+A0AB
UNICODE_Lo, //U+A0AC
UNICODE_Lo, //U+A0AD
UNICODE_Lo, //U+A0AE
UNICODE_Lo, //U+A0AF
UNICODE_Lo, //U+A0B0
UNICODE_Lo, //U+A0B1
UNICODE_Lo, //U+A0B2
UNICODE_Lo, //U+A0B3
UNICODE_Lo, //U+A0B4
UNICODE_Lo, //U+A0B5
UNICODE_Lo, //U+A0B6
UNICODE_Lo, //U+A0B7
UNICODE_Lo, //U+A0B8
UNICODE_Lo, //U+A0B9
UNICODE_Lo, //U+A0BA
UNICODE_Lo, //U+A0BB
UNICODE_Lo, //U+A0BC
UNICODE_Lo, //U+A0BD
UNICODE_Lo, //U+A0BE
UNICODE_Lo, //U+A0BF
UNICODE_Lo, //U+A0C0
UNICODE_Lo, //U+A0C1
UNICODE_Lo, //U+A0C2
UNICODE_Lo, //U+A0C3
UNICODE_Lo, //U+A0C4
UNICODE_Lo, //U+A0C5
UNICODE_Lo, //U+A0C6
UNICODE_Lo, //U+A0C7
UNICODE_Lo, //U+A0C8
UNICODE_Lo, //U+A0C9
UNICODE_Lo, //U+A0CA
UNICODE_Lo, //U+A0CB
UNICODE_Lo, //U+A0CC
UNICODE_Lo, //U+A0CD
UNICODE_Lo, //U+A0CE
UNICODE_Lo, //U+A0CF
UNICODE_Lo, //U+A0D0
UNICODE_Lo, //U+A0D1
UNICODE_Lo, //U+A0D2
UNICODE_Lo, //U+A0D3
UNICODE_Lo, //U+A0D4
UNICODE_Lo, //U+A0D5
UNICODE_Lo, //U+A0D6
UNICODE_Lo, //U+A0D7
UNICODE_Lo, //U+A0D8
UNICODE_Lo, //U+A0D9
UNICODE_Lo, //U+A0DA
UNICODE_Lo, //U+A0DB
UNICODE_Lo, //U+A0DC
UNICODE_Lo, //U+A0DD
UNICODE_Lo, //U+A0DE
UNICODE_Lo, //U+A0DF
UNICODE_Lo, //U+A0E0
UNICODE_Lo, //U+A0E1
UNICODE_Lo, //U+A0E2
UNICODE_Lo, //U+A0E3
UNICODE_Lo, //U+A0E4
UNICODE_Lo, //U+A0E5
UNICODE_Lo, //U+A0E6
UNICODE_Lo, //U+A0E7
UNICODE_Lo, //U+A0E8
UNICODE_Lo, //U+A0E9
UNICODE_Lo, //U+A0EA
UNICODE_Lo, //U+A0EB
UNICODE_Lo, //U+A0EC
UNICODE_Lo, //U+A0ED
UNICODE_Lo, //U+A0EE
UNICODE_Lo, //U+A0EF
UNICODE_Lo, //U+A0F0
UNICODE_Lo, //U+A0F1
UNICODE_Lo, //U+A0F2
UNICODE_Lo, //U+A0F3
UNICODE_Lo, //U+A0F4
UNICODE_Lo, //U+A0F5
UNICODE_Lo, //U+A0F6
UNICODE_Lo, //U+A0F7
UNICODE_Lo, //U+A0F8
UNICODE_Lo, //U+A0F9
UNICODE_Lo, //U+A0FA
UNICODE_Lo, //U+A0FB
UNICODE_Lo, //U+A0FC
UNICODE_Lo, //U+A0FD
UNICODE_Lo, //U+A0FE
UNICODE_Lo, //U+A0FF
UNICODE_Lo, //U+A100
UNICODE_Lo, //U+A101
UNICODE_Lo, //U+A102
UNICODE_Lo, //U+A103
UNICODE_Lo, //U+A104
UNICODE_Lo, //U+A105
UNICODE_Lo, //U+A106
UNICODE_Lo, //U+A107
UNICODE_Lo, //U+A108
UNICODE_Lo, //U+A109
UNICODE_Lo, //U+A10A
UNICODE_Lo, //U+A10B
UNICODE_Lo, //U+A10C
UNICODE_Lo, //U+A10D
UNICODE_Lo, //U+A10E
UNICODE_Lo, //U+A10F
UNICODE_Lo, //U+A110
UNICODE_Lo, //U+A111
UNICODE_Lo, //U+A112
UNICODE_Lo, //U+A113
UNICODE_Lo, //U+A114
UNICODE_Lo, //U+A115
UNICODE_Lo, //U+A116
UNICODE_Lo, //U+A117
UNICODE_Lo, //U+A118
UNICODE_Lo, //U+A119
UNICODE_Lo, //U+A11A
UNICODE_Lo, //U+A11B
UNICODE_Lo, //U+A11C
UNICODE_Lo, //U+A11D
UNICODE_Lo, //U+A11E
UNICODE_Lo, //U+A11F
UNICODE_Lo, //U+A120
UNICODE_Lo, //U+A121
UNICODE_Lo, //U+A122
UNICODE_Lo, //U+A123
UNICODE_Lo, //U+A124
UNICODE_Lo, //U+A125
UNICODE_Lo, //U+A126
UNICODE_Lo, //U+A127
UNICODE_Lo, //U+A128
UNICODE_Lo, //U+A129
UNICODE_Lo, //U+A12A
UNICODE_Lo, //U+A12B
UNICODE_Lo, //U+A12C
UNICODE_Lo, //U+A12D
UNICODE_Lo, //U+A12E
UNICODE_Lo, //U+A12F
UNICODE_Lo, //U+A130
UNICODE_Lo, //U+A131
UNICODE_Lo, //U+A132
UNICODE_Lo, //U+A133
UNICODE_Lo, //U+A134
UNICODE_Lo, //U+A135
UNICODE_Lo, //U+A136
UNICODE_Lo, //U+A137
UNICODE_Lo, //U+A138
UNICODE_Lo, //U+A139
UNICODE_Lo, //U+A13A
UNICODE_Lo, //U+A13B
UNICODE_Lo, //U+A13C
UNICODE_Lo, //U+A13D
UNICODE_Lo, //U+A13E
UNICODE_Lo, //U+A13F
UNICODE_Lo, //U+A140
UNICODE_Lo, //U+A141
UNICODE_Lo, //U+A142
UNICODE_Lo, //U+A143
UNICODE_Lo, //U+A144
UNICODE_Lo, //U+A145
UNICODE_Lo, //U+A146
UNICODE_Lo, //U+A147
UNICODE_Lo, //U+A148
UNICODE_Lo, //U+A149
UNICODE_Lo, //U+A14A
UNICODE_Lo, //U+A14B
UNICODE_Lo, //U+A14C
UNICODE_Lo, //U+A14D
UNICODE_Lo, //U+A14E
UNICODE_Lo, //U+A14F
UNICODE_Lo, //U+A150
UNICODE_Lo, //U+A151
UNICODE_Lo, //U+A152
UNICODE_Lo, //U+A153
UNICODE_Lo, //U+A154
UNICODE_Lo, //U+A155
UNICODE_Lo, //U+A156
UNICODE_Lo, //U+A157
UNICODE_Lo, //U+A158
UNICODE_Lo, //U+A159
UNICODE_Lo, //U+A15A
UNICODE_Lo, //U+A15B
UNICODE_Lo, //U+A15C
UNICODE_Lo, //U+A15D
UNICODE_Lo, //U+A15E
UNICODE_Lo, //U+A15F
UNICODE_Lo, //U+A160
UNICODE_Lo, //U+A161
UNICODE_Lo, //U+A162
UNICODE_Lo, //U+A163
UNICODE_Lo, //U+A164
UNICODE_Lo, //U+A165
UNICODE_Lo, //U+A166
UNICODE_Lo, //U+A167
UNICODE_Lo, //U+A168
UNICODE_Lo, //U+A169
UNICODE_Lo, //U+A16A
UNICODE_Lo, //U+A16B
UNICODE_Lo, //U+A16C
UNICODE_Lo, //U+A16D
UNICODE_Lo, //U+A16E
UNICODE_Lo, //U+A16F
UNICODE_Lo, //U+A170
UNICODE_Lo, //U+A171
UNICODE_Lo, //U+A172
UNICODE_Lo, //U+A173
UNICODE_Lo, //U+A174
UNICODE_Lo, //U+A175
UNICODE_Lo, //U+A176
UNICODE_Lo, //U+A177
UNICODE_Lo, //U+A178
UNICODE_Lo, //U+A179
UNICODE_Lo, //U+A17A
UNICODE_Lo, //U+A17B
UNICODE_Lo, //U+A17C
UNICODE_Lo, //U+A17D
UNICODE_Lo, //U+A17E
UNICODE_Lo, //U+A17F
UNICODE_Lo, //U+A180
UNICODE_Lo, //U+A181
UNICODE_Lo, //U+A182
UNICODE_Lo, //U+A183
UNICODE_Lo, //U+A184
UNICODE_Lo, //U+A185
UNICODE_Lo, //U+A186
UNICODE_Lo, //U+A187
UNICODE_Lo, //U+A188
UNICODE_Lo, //U+A189
UNICODE_Lo, //U+A18A
UNICODE_Lo, //U+A18B
UNICODE_Lo, //U+A18C
UNICODE_Lo, //U+A18D
UNICODE_Lo, //U+A18E
UNICODE_Lo, //U+A18F
UNICODE_Lo, //U+A190
UNICODE_Lo, //U+A191
UNICODE_Lo, //U+A192
UNICODE_Lo, //U+A193
UNICODE_Lo, //U+A194
UNICODE_Lo, //U+A195
UNICODE_Lo, //U+A196
UNICODE_Lo, //U+A197
UNICODE_Lo, //U+A198
UNICODE_Lo, //U+A199
UNICODE_Lo, //U+A19A
UNICODE_Lo, //U+A19B
UNICODE_Lo, //U+A19C
UNICODE_Lo, //U+A19D
UNICODE_Lo, //U+A19E
UNICODE_Lo, //U+A19F
UNICODE_Lo, //U+A1A0
UNICODE_Lo, //U+A1A1
UNICODE_Lo, //U+A1A2
UNICODE_Lo, //U+A1A3
UNICODE_Lo, //U+A1A4
UNICODE_Lo, //U+A1A5
UNICODE_Lo, //U+A1A6
UNICODE_Lo, //U+A1A7
UNICODE_Lo, //U+A1A8
UNICODE_Lo, //U+A1A9
UNICODE_Lo, //U+A1AA
UNICODE_Lo, //U+A1AB
UNICODE_Lo, //U+A1AC
UNICODE_Lo, //U+A1AD
UNICODE_Lo, //U+A1AE
UNICODE_Lo, //U+A1AF
UNICODE_Lo, //U+A1B0
UNICODE_Lo, //U+A1B1
UNICODE_Lo, //U+A1B2
UNICODE_Lo, //U+A1B3
UNICODE_Lo, //U+A1B4
UNICODE_Lo, //U+A1B5
UNICODE_Lo, //U+A1B6
UNICODE_Lo, //U+A1B7
UNICODE_Lo, //U+A1B8
UNICODE_Lo, //U+A1B9
UNICODE_Lo, //U+A1BA
UNICODE_Lo, //U+A1BB
UNICODE_Lo, //U+A1BC
UNICODE_Lo, //U+A1BD
UNICODE_Lo, //U+A1BE
UNICODE_Lo, //U+A1BF
UNICODE_Lo, //U+A1C0
UNICODE_Lo, //U+A1C1
UNICODE_Lo, //U+A1C2
UNICODE_Lo, //U+A1C3
UNICODE_Lo, //U+A1C4
UNICODE_Lo, //U+A1C5
UNICODE_Lo, //U+A1C6
UNICODE_Lo, //U+A1C7
UNICODE_Lo, //U+A1C8
UNICODE_Lo, //U+A1C9
UNICODE_Lo, //U+A1CA
UNICODE_Lo, //U+A1CB
UNICODE_Lo, //U+A1CC
UNICODE_Lo, //U+A1CD
UNICODE_Lo, //U+A1CE
UNICODE_Lo, //U+A1CF
UNICODE_Lo, //U+A1D0
UNICODE_Lo, //U+A1D1
UNICODE_Lo, //U+A1D2
UNICODE_Lo, //U+A1D3
UNICODE_Lo, //U+A1D4
UNICODE_Lo, //U+A1D5
UNICODE_Lo, //U+A1D6
UNICODE_Lo, //U+A1D7
UNICODE_Lo, //U+A1D8
UNICODE_Lo, //U+A1D9
UNICODE_Lo, //U+A1DA
UNICODE_Lo, //U+A1DB
UNICODE_Lo, //U+A1DC
UNICODE_Lo, //U+A1DD
UNICODE_Lo, //U+A1DE
UNICODE_Lo, //U+A1DF
UNICODE_Lo, //U+A1E0
UNICODE_Lo, //U+A1E1
UNICODE_Lo, //U+A1E2
UNICODE_Lo, //U+A1E3
UNICODE_Lo, //U+A1E4
UNICODE_Lo, //U+A1E5
UNICODE_Lo, //U+A1E6
UNICODE_Lo, //U+A1E7
UNICODE_Lo, //U+A1E8
UNICODE_Lo, //U+A1E9
UNICODE_Lo, //U+A1EA
UNICODE_Lo, //U+A1EB
UNICODE_Lo, //U+A1EC
UNICODE_Lo, //U+A1ED
UNICODE_Lo, //U+A1EE
UNICODE_Lo, //U+A1EF
UNICODE_Lo, //U+A1F0
UNICODE_Lo, //U+A1F1
UNICODE_Lo, //U+A1F2
UNICODE_Lo, //U+A1F3
UNICODE_Lo, //U+A1F4
UNICODE_Lo, //U+A1F5
UNICODE_Lo, //U+A1F6
UNICODE_Lo, //U+A1F7
UNICODE_Lo, //U+A1F8
UNICODE_Lo, //U+A1F9
UNICODE_Lo, //U+A1FA
UNICODE_Lo, //U+A1FB
UNICODE_Lo, //U+A1FC
UNICODE_Lo, //U+A1FD
UNICODE_Lo, //U+A1FE
UNICODE_Lo, //U+A1FF
UNICODE_Lo, //U+A200
UNICODE_Lo, //U+A201
UNICODE_Lo, //U+A202
UNICODE_Lo, //U+A203
UNICODE_Lo, //U+A204
UNICODE_Lo, //U+A205
UNICODE_Lo, //U+A206
UNICODE_Lo, //U+A207
UNICODE_Lo, //U+A208
UNICODE_Lo, //U+A209
UNICODE_Lo, //U+A20A
UNICODE_Lo, //U+A20B
UNICODE_Lo, //U+A20C
UNICODE_Lo, //U+A20D
UNICODE_Lo, //U+A20E
UNICODE_Lo, //U+A20F
UNICODE_Lo, //U+A210
UNICODE_Lo, //U+A211
UNICODE_Lo, //U+A212
UNICODE_Lo, //U+A213
UNICODE_Lo, //U+A214
UNICODE_Lo, //U+A215
UNICODE_Lo, //U+A216
UNICODE_Lo, //U+A217
UNICODE_Lo, //U+A218
UNICODE_Lo, //U+A219
UNICODE_Lo, //U+A21A
UNICODE_Lo, //U+A21B
UNICODE_Lo, //U+A21C
UNICODE_Lo, //U+A21D
UNICODE_Lo, //U+A21E
UNICODE_Lo, //U+A21F
UNICODE_Lo, //U+A220
UNICODE_Lo, //U+A221
UNICODE_Lo, //U+A222
UNICODE_Lo, //U+A223
UNICODE_Lo, //U+A224
UNICODE_Lo, //U+A225
UNICODE_Lo, //U+A226
UNICODE_Lo, //U+A227
UNICODE_Lo, //U+A228
UNICODE_Lo, //U+A229
UNICODE_Lo, //U+A22A
UNICODE_Lo, //U+A22B
UNICODE_Lo, //U+A22C
UNICODE_Lo, //U+A22D
UNICODE_Lo, //U+A22E
UNICODE_Lo, //U+A22F
UNICODE_Lo, //U+A230
UNICODE_Lo, //U+A231
UNICODE_Lo, //U+A232
UNICODE_Lo, //U+A233
UNICODE_Lo, //U+A234
UNICODE_Lo, //U+A235
UNICODE_Lo, //U+A236
UNICODE_Lo, //U+A237
UNICODE_Lo, //U+A238
UNICODE_Lo, //U+A239
UNICODE_Lo, //U+A23A
UNICODE_Lo, //U+A23B
UNICODE_Lo, //U+A23C
UNICODE_Lo, //U+A23D
UNICODE_Lo, //U+A23E
UNICODE_Lo, //U+A23F
UNICODE_Lo, //U+A240
UNICODE_Lo, //U+A241
UNICODE_Lo, //U+A242
UNICODE_Lo, //U+A243
UNICODE_Lo, //U+A244
UNICODE_Lo, //U+A245
UNICODE_Lo, //U+A246
UNICODE_Lo, //U+A247
UNICODE_Lo, //U+A248
UNICODE_Lo, //U+A249
UNICODE_Lo, //U+A24A
UNICODE_Lo, //U+A24B
UNICODE_Lo, //U+A24C
UNICODE_Lo, //U+A24D
UNICODE_Lo, //U+A24E
UNICODE_Lo, //U+A24F
UNICODE_Lo, //U+A250
UNICODE_Lo, //U+A251
UNICODE_Lo, //U+A252
UNICODE_Lo, //U+A253
UNICODE_Lo, //U+A254
UNICODE_Lo, //U+A255
UNICODE_Lo, //U+A256
UNICODE_Lo, //U+A257
UNICODE_Lo, //U+A258
UNICODE_Lo, //U+A259
UNICODE_Lo, //U+A25A
UNICODE_Lo, //U+A25B
UNICODE_Lo, //U+A25C
UNICODE_Lo, //U+A25D
UNICODE_Lo, //U+A25E
UNICODE_Lo, //U+A25F
UNICODE_Lo, //U+A260
UNICODE_Lo, //U+A261
UNICODE_Lo, //U+A262
UNICODE_Lo, //U+A263
UNICODE_Lo, //U+A264
UNICODE_Lo, //U+A265
UNICODE_Lo, //U+A266
UNICODE_Lo, //U+A267
UNICODE_Lo, //U+A268
UNICODE_Lo, //U+A269
UNICODE_Lo, //U+A26A
UNICODE_Lo, //U+A26B
UNICODE_Lo, //U+A26C
UNICODE_Lo, //U+A26D
UNICODE_Lo, //U+A26E
UNICODE_Lo, //U+A26F
UNICODE_Lo, //U+A270
UNICODE_Lo, //U+A271
UNICODE_Lo, //U+A272
UNICODE_Lo, //U+A273
UNICODE_Lo, //U+A274
UNICODE_Lo, //U+A275
UNICODE_Lo, //U+A276
UNICODE_Lo, //U+A277
UNICODE_Lo, //U+A278
UNICODE_Lo, //U+A279
UNICODE_Lo, //U+A27A
UNICODE_Lo, //U+A27B
UNICODE_Lo, //U+A27C
UNICODE_Lo, //U+A27D
UNICODE_Lo, //U+A27E
UNICODE_Lo, //U+A27F
UNICODE_Lo, //U+A280
UNICODE_Lo, //U+A281
UNICODE_Lo, //U+A282
UNICODE_Lo, //U+A283
UNICODE_Lo, //U+A284
UNICODE_Lo, //U+A285
UNICODE_Lo, //U+A286
UNICODE_Lo, //U+A287
UNICODE_Lo, //U+A288
UNICODE_Lo, //U+A289
UNICODE_Lo, //U+A28A
UNICODE_Lo, //U+A28B
UNICODE_Lo, //U+A28C
UNICODE_Lo, //U+A28D
UNICODE_Lo, //U+A28E
UNICODE_Lo, //U+A28F
UNICODE_Lo, //U+A290
UNICODE_Lo, //U+A291
UNICODE_Lo, //U+A292
UNICODE_Lo, //U+A293
UNICODE_Lo, //U+A294
UNICODE_Lo, //U+A295
UNICODE_Lo, //U+A296
UNICODE_Lo, //U+A297
UNICODE_Lo, //U+A298
UNICODE_Lo, //U+A299
UNICODE_Lo, //U+A29A
UNICODE_Lo, //U+A29B
UNICODE_Lo, //U+A29C
UNICODE_Lo, //U+A29D
UNICODE_Lo, //U+A29E
UNICODE_Lo, //U+A29F
UNICODE_Lo, //U+A2A0
UNICODE_Lo, //U+A2A1
UNICODE_Lo, //U+A2A2
UNICODE_Lo, //U+A2A3
UNICODE_Lo, //U+A2A4
UNICODE_Lo, //U+A2A5
UNICODE_Lo, //U+A2A6
UNICODE_Lo, //U+A2A7
UNICODE_Lo, //U+A2A8
UNICODE_Lo, //U+A2A9
UNICODE_Lo, //U+A2AA
UNICODE_Lo, //U+A2AB
UNICODE_Lo, //U+A2AC
UNICODE_Lo, //U+A2AD
UNICODE_Lo, //U+A2AE
UNICODE_Lo, //U+A2AF
UNICODE_Lo, //U+A2B0
UNICODE_Lo, //U+A2B1
UNICODE_Lo, //U+A2B2
UNICODE_Lo, //U+A2B3
UNICODE_Lo, //U+A2B4
UNICODE_Lo, //U+A2B5
UNICODE_Lo, //U+A2B6
UNICODE_Lo, //U+A2B7
UNICODE_Lo, //U+A2B8
UNICODE_Lo, //U+A2B9
UNICODE_Lo, //U+A2BA
UNICODE_Lo, //U+A2BB
UNICODE_Lo, //U+A2BC
UNICODE_Lo, //U+A2BD
UNICODE_Lo, //U+A2BE
UNICODE_Lo, //U+A2BF
UNICODE_Lo, //U+A2C0
UNICODE_Lo, //U+A2C1
UNICODE_Lo, //U+A2C2
UNICODE_Lo, //U+A2C3
UNICODE_Lo, //U+A2C4
UNICODE_Lo, //U+A2C5
UNICODE_Lo, //U+A2C6
UNICODE_Lo, //U+A2C7
UNICODE_Lo, //U+A2C8
UNICODE_Lo, //U+A2C9
UNICODE_Lo, //U+A2CA
UNICODE_Lo, //U+A2CB
UNICODE_Lo, //U+A2CC
UNICODE_Lo, //U+A2CD
UNICODE_Lo, //U+A2CE
UNICODE_Lo, //U+A2CF
UNICODE_Lo, //U+A2D0
UNICODE_Lo, //U+A2D1
UNICODE_Lo, //U+A2D2
UNICODE_Lo, //U+A2D3
UNICODE_Lo, //U+A2D4
UNICODE_Lo, //U+A2D5
UNICODE_Lo, //U+A2D6
UNICODE_Lo, //U+A2D7
UNICODE_Lo, //U+A2D8
UNICODE_Lo, //U+A2D9
UNICODE_Lo, //U+A2DA
UNICODE_Lo, //U+A2DB
UNICODE_Lo, //U+A2DC
UNICODE_Lo, //U+A2DD
UNICODE_Lo, //U+A2DE
UNICODE_Lo, //U+A2DF
UNICODE_Lo, //U+A2E0
UNICODE_Lo, //U+A2E1
UNICODE_Lo, //U+A2E2
UNICODE_Lo, //U+A2E3
UNICODE_Lo, //U+A2E4
UNICODE_Lo, //U+A2E5
UNICODE_Lo, //U+A2E6
UNICODE_Lo, //U+A2E7
UNICODE_Lo, //U+A2E8
UNICODE_Lo, //U+A2E9
UNICODE_Lo, //U+A2EA
UNICODE_Lo, //U+A2EB
UNICODE_Lo, //U+A2EC
UNICODE_Lo, //U+A2ED
UNICODE_Lo, //U+A2EE
UNICODE_Lo, //U+A2EF
UNICODE_Lo, //U+A2F0
UNICODE_Lo, //U+A2F1
UNICODE_Lo, //U+A2F2
UNICODE_Lo, //U+A2F3
UNICODE_Lo, //U+A2F4
UNICODE_Lo, //U+A2F5
UNICODE_Lo, //U+A2F6
UNICODE_Lo, //U+A2F7
UNICODE_Lo, //U+A2F8
UNICODE_Lo, //U+A2F9
UNICODE_Lo, //U+A2FA
UNICODE_Lo, //U+A2FB
UNICODE_Lo, //U+A2FC
UNICODE_Lo, //U+A2FD
UNICODE_Lo, //U+A2FE
UNICODE_Lo, //U+A2FF
UNICODE_Lo, //U+A300
UNICODE_Lo, //U+A301
UNICODE_Lo, //U+A302
UNICODE_Lo, //U+A303
UNICODE_Lo, //U+A304
UNICODE_Lo, //U+A305
UNICODE_Lo, //U+A306
UNICODE_Lo, //U+A307
UNICODE_Lo, //U+A308
UNICODE_Lo, //U+A309
UNICODE_Lo, //U+A30A
UNICODE_Lo, //U+A30B
UNICODE_Lo, //U+A30C
UNICODE_Lo, //U+A30D
UNICODE_Lo, //U+A30E
UNICODE_Lo, //U+A30F
UNICODE_Lo, //U+A310
UNICODE_Lo, //U+A311
UNICODE_Lo, //U+A312
UNICODE_Lo, //U+A313
UNICODE_Lo, //U+A314
UNICODE_Lo, //U+A315
UNICODE_Lo, //U+A316
UNICODE_Lo, //U+A317
UNICODE_Lo, //U+A318
UNICODE_Lo, //U+A319
UNICODE_Lo, //U+A31A
UNICODE_Lo, //U+A31B
UNICODE_Lo, //U+A31C
UNICODE_Lo, //U+A31D
UNICODE_Lo, //U+A31E
UNICODE_Lo, //U+A31F
UNICODE_Lo, //U+A320
UNICODE_Lo, //U+A321
UNICODE_Lo, //U+A322
UNICODE_Lo, //U+A323
UNICODE_Lo, //U+A324
UNICODE_Lo, //U+A325
UNICODE_Lo, //U+A326
UNICODE_Lo, //U+A327
UNICODE_Lo, //U+A328
UNICODE_Lo, //U+A329
UNICODE_Lo, //U+A32A
UNICODE_Lo, //U+A32B
UNICODE_Lo, //U+A32C
UNICODE_Lo, //U+A32D
UNICODE_Lo, //U+A32E
UNICODE_Lo, //U+A32F
UNICODE_Lo, //U+A330
UNICODE_Lo, //U+A331
UNICODE_Lo, //U+A332
UNICODE_Lo, //U+A333
UNICODE_Lo, //U+A334
UNICODE_Lo, //U+A335
UNICODE_Lo, //U+A336
UNICODE_Lo, //U+A337
UNICODE_Lo, //U+A338
UNICODE_Lo, //U+A339
UNICODE_Lo, //U+A33A
UNICODE_Lo, //U+A33B
UNICODE_Lo, //U+A33C
UNICODE_Lo, //U+A33D
UNICODE_Lo, //U+A33E
UNICODE_Lo, //U+A33F
UNICODE_Lo, //U+A340
UNICODE_Lo, //U+A341
UNICODE_Lo, //U+A342
UNICODE_Lo, //U+A343
UNICODE_Lo, //U+A344
UNICODE_Lo, //U+A345
UNICODE_Lo, //U+A346
UNICODE_Lo, //U+A347
UNICODE_Lo, //U+A348
UNICODE_Lo, //U+A349
UNICODE_Lo, //U+A34A
UNICODE_Lo, //U+A34B
UNICODE_Lo, //U+A34C
UNICODE_Lo, //U+A34D
UNICODE_Lo, //U+A34E
UNICODE_Lo, //U+A34F
UNICODE_Lo, //U+A350
UNICODE_Lo, //U+A351
UNICODE_Lo, //U+A352
UNICODE_Lo, //U+A353
UNICODE_Lo, //U+A354
UNICODE_Lo, //U+A355
UNICODE_Lo, //U+A356
UNICODE_Lo, //U+A357
UNICODE_Lo, //U+A358
UNICODE_Lo, //U+A359
UNICODE_Lo, //U+A35A
UNICODE_Lo, //U+A35B
UNICODE_Lo, //U+A35C
UNICODE_Lo, //U+A35D
UNICODE_Lo, //U+A35E
UNICODE_Lo, //U+A35F
UNICODE_Lo, //U+A360
UNICODE_Lo, //U+A361
UNICODE_Lo, //U+A362
UNICODE_Lo, //U+A363
UNICODE_Lo, //U+A364
UNICODE_Lo, //U+A365
UNICODE_Lo, //U+A366
UNICODE_Lo, //U+A367
UNICODE_Lo, //U+A368
UNICODE_Lo, //U+A369
UNICODE_Lo, //U+A36A
UNICODE_Lo, //U+A36B
UNICODE_Lo, //U+A36C
UNICODE_Lo, //U+A36D
UNICODE_Lo, //U+A36E
UNICODE_Lo, //U+A36F
UNICODE_Lo, //U+A370
UNICODE_Lo, //U+A371
UNICODE_Lo, //U+A372
UNICODE_Lo, //U+A373
UNICODE_Lo, //U+A374
UNICODE_Lo, //U+A375
UNICODE_Lo, //U+A376
UNICODE_Lo, //U+A377
UNICODE_Lo, //U+A378
UNICODE_Lo, //U+A379
UNICODE_Lo, //U+A37A
UNICODE_Lo, //U+A37B
UNICODE_Lo, //U+A37C
UNICODE_Lo, //U+A37D
UNICODE_Lo, //U+A37E
UNICODE_Lo, //U+A37F
UNICODE_Lo, //U+A380
UNICODE_Lo, //U+A381
UNICODE_Lo, //U+A382
UNICODE_Lo, //U+A383
UNICODE_Lo, //U+A384
UNICODE_Lo, //U+A385
UNICODE_Lo, //U+A386
UNICODE_Lo, //U+A387
UNICODE_Lo, //U+A388
UNICODE_Lo, //U+A389
UNICODE_Lo, //U+A38A
UNICODE_Lo, //U+A38B
UNICODE_Lo, //U+A38C
UNICODE_Lo, //U+A38D
UNICODE_Lo, //U+A38E
UNICODE_Lo, //U+A38F
UNICODE_Lo, //U+A390
UNICODE_Lo, //U+A391
UNICODE_Lo, //U+A392
UNICODE_Lo, //U+A393
UNICODE_Lo, //U+A394
UNICODE_Lo, //U+A395
UNICODE_Lo, //U+A396
UNICODE_Lo, //U+A397
UNICODE_Lo, //U+A398
UNICODE_Lo, //U+A399
UNICODE_Lo, //U+A39A
UNICODE_Lo, //U+A39B
UNICODE_Lo, //U+A39C
UNICODE_Lo, //U+A39D
UNICODE_Lo, //U+A39E
UNICODE_Lo, //U+A39F
UNICODE_Lo, //U+A3A0
UNICODE_Lo, //U+A3A1
UNICODE_Lo, //U+A3A2
UNICODE_Lo, //U+A3A3
UNICODE_Lo, //U+A3A4
UNICODE_Lo, //U+A3A5
UNICODE_Lo, //U+A3A6
UNICODE_Lo, //U+A3A7
UNICODE_Lo, //U+A3A8
UNICODE_Lo, //U+A3A9
UNICODE_Lo, //U+A3AA
UNICODE_Lo, //U+A3AB
UNICODE_Lo, //U+A3AC
UNICODE_Lo, //U+A3AD
UNICODE_Lo, //U+A3AE
UNICODE_Lo, //U+A3AF
UNICODE_Lo, //U+A3B0
UNICODE_Lo, //U+A3B1
UNICODE_Lo, //U+A3B2
UNICODE_Lo, //U+A3B3
UNICODE_Lo, //U+A3B4
UNICODE_Lo, //U+A3B5
UNICODE_Lo, //U+A3B6
UNICODE_Lo, //U+A3B7
UNICODE_Lo, //U+A3B8
UNICODE_Lo, //U+A3B9
UNICODE_Lo, //U+A3BA
UNICODE_Lo, //U+A3BB
UNICODE_Lo, //U+A3BC
UNICODE_Lo, //U+A3BD
UNICODE_Lo, //U+A3BE
UNICODE_Lo, //U+A3BF
UNICODE_Lo, //U+A3C0
UNICODE_Lo, //U+A3C1
UNICODE_Lo, //U+A3C2
UNICODE_Lo, //U+A3C3
UNICODE_Lo, //U+A3C4
UNICODE_Lo, //U+A3C5
UNICODE_Lo, //U+A3C6
UNICODE_Lo, //U+A3C7
UNICODE_Lo, //U+A3C8
UNICODE_Lo, //U+A3C9
UNICODE_Lo, //U+A3CA
UNICODE_Lo, //U+A3CB
UNICODE_Lo, //U+A3CC
UNICODE_Lo, //U+A3CD
UNICODE_Lo, //U+A3CE
UNICODE_Lo, //U+A3CF
UNICODE_Lo, //U+A3D0
UNICODE_Lo, //U+A3D1
UNICODE_Lo, //U+A3D2
UNICODE_Lo, //U+A3D3
UNICODE_Lo, //U+A3D4
UNICODE_Lo, //U+A3D5
UNICODE_Lo, //U+A3D6
UNICODE_Lo, //U+A3D7
UNICODE_Lo, //U+A3D8
UNICODE_Lo, //U+A3D9
UNICODE_Lo, //U+A3DA
UNICODE_Lo, //U+A3DB
UNICODE_Lo, //U+A3DC
UNICODE_Lo, //U+A3DD
UNICODE_Lo, //U+A3DE
UNICODE_Lo, //U+A3DF
UNICODE_Lo, //U+A3E0
UNICODE_Lo, //U+A3E1
UNICODE_Lo, //U+A3E2
UNICODE_Lo, //U+A3E3
UNICODE_Lo, //U+A3E4
UNICODE_Lo, //U+A3E5
UNICODE_Lo, //U+A3E6
UNICODE_Lo, //U+A3E7
UNICODE_Lo, //U+A3E8
UNICODE_Lo, //U+A3E9
UNICODE_Lo, //U+A3EA
UNICODE_Lo, //U+A3EB
UNICODE_Lo, //U+A3EC
UNICODE_Lo, //U+A3ED
UNICODE_Lo, //U+A3EE
UNICODE_Lo, //U+A3EF
UNICODE_Lo, //U+A3F0
UNICODE_Lo, //U+A3F1
UNICODE_Lo, //U+A3F2
UNICODE_Lo, //U+A3F3
UNICODE_Lo, //U+A3F4
UNICODE_Lo, //U+A3F5
UNICODE_Lo, //U+A3F6
UNICODE_Lo, //U+A3F7
UNICODE_Lo, //U+A3F8
UNICODE_Lo, //U+A3F9
UNICODE_Lo, //U+A3FA
UNICODE_Lo, //U+A3FB
UNICODE_Lo, //U+A3FC
UNICODE_Lo, //U+A3FD
UNICODE_Lo, //U+A3FE
UNICODE_Lo, //U+A3FF
UNICODE_Lo, //U+A400
UNICODE_Lo, //U+A401
UNICODE_Lo, //U+A402
UNICODE_Lo, //U+A403
UNICODE_Lo, //U+A404
UNICODE_Lo, //U+A405
UNICODE_Lo, //U+A406
UNICODE_Lo, //U+A407
UNICODE_Lo, //U+A408
UNICODE_Lo, //U+A409
UNICODE_Lo, //U+A40A
UNICODE_Lo, //U+A40B
UNICODE_Lo, //U+A40C
UNICODE_Lo, //U+A40D
UNICODE_Lo, //U+A40E
UNICODE_Lo, //U+A40F
UNICODE_Lo, //U+A410
UNICODE_Lo, //U+A411
UNICODE_Lo, //U+A412
UNICODE_Lo, //U+A413
UNICODE_Lo, //U+A414
UNICODE_Lo, //U+A415
UNICODE_Lo, //U+A416
UNICODE_Lo, //U+A417
UNICODE_Lo, //U+A418
UNICODE_Lo, //U+A419
UNICODE_Lo, //U+A41A
UNICODE_Lo, //U+A41B
UNICODE_Lo, //U+A41C
UNICODE_Lo, //U+A41D
UNICODE_Lo, //U+A41E
UNICODE_Lo, //U+A41F
UNICODE_Lo, //U+A420
UNICODE_Lo, //U+A421
UNICODE_Lo, //U+A422
UNICODE_Lo, //U+A423
UNICODE_Lo, //U+A424
UNICODE_Lo, //U+A425
UNICODE_Lo, //U+A426
UNICODE_Lo, //U+A427
UNICODE_Lo, //U+A428
UNICODE_Lo, //U+A429
UNICODE_Lo, //U+A42A
UNICODE_Lo, //U+A42B
UNICODE_Lo, //U+A42C
UNICODE_Lo, //U+A42D
UNICODE_Lo, //U+A42E
UNICODE_Lo, //U+A42F
UNICODE_Lo, //U+A430
UNICODE_Lo, //U+A431
UNICODE_Lo, //U+A432
UNICODE_Lo, //U+A433
UNICODE_Lo, //U+A434
UNICODE_Lo, //U+A435
UNICODE_Lo, //U+A436
UNICODE_Lo, //U+A437
UNICODE_Lo, //U+A438
UNICODE_Lo, //U+A439
UNICODE_Lo, //U+A43A
UNICODE_Lo, //U+A43B
UNICODE_Lo, //U+A43C
UNICODE_Lo, //U+A43D
UNICODE_Lo, //U+A43E
UNICODE_Lo, //U+A43F
UNICODE_Lo, //U+A440
UNICODE_Lo, //U+A441
UNICODE_Lo, //U+A442
UNICODE_Lo, //U+A443
UNICODE_Lo, //U+A444
UNICODE_Lo, //U+A445
UNICODE_Lo, //U+A446
UNICODE_Lo, //U+A447
UNICODE_Lo, //U+A448
UNICODE_Lo, //U+A449
UNICODE_Lo, //U+A44A
UNICODE_Lo, //U+A44B
UNICODE_Lo, //U+A44C
UNICODE_Lo, //U+A44D
UNICODE_Lo, //U+A44E
UNICODE_Lo, //U+A44F
UNICODE_Lo, //U+A450
UNICODE_Lo, //U+A451
UNICODE_Lo, //U+A452
UNICODE_Lo, //U+A453
UNICODE_Lo, //U+A454
UNICODE_Lo, //U+A455
UNICODE_Lo, //U+A456
UNICODE_Lo, //U+A457
UNICODE_Lo, //U+A458
UNICODE_Lo, //U+A459
UNICODE_Lo, //U+A45A
UNICODE_Lo, //U+A45B
UNICODE_Lo, //U+A45C
UNICODE_Lo, //U+A45D
UNICODE_Lo, //U+A45E
UNICODE_Lo, //U+A45F
UNICODE_Lo, //U+A460
UNICODE_Lo, //U+A461
UNICODE_Lo, //U+A462
UNICODE_Lo, //U+A463
UNICODE_Lo, //U+A464
UNICODE_Lo, //U+A465
UNICODE_Lo, //U+A466
UNICODE_Lo, //U+A467
UNICODE_Lo, //U+A468
UNICODE_Lo, //U+A469
UNICODE_Lo, //U+A46A
UNICODE_Lo, //U+A46B
UNICODE_Lo, //U+A46C
UNICODE_Lo, //U+A46D
UNICODE_Lo, //U+A46E
UNICODE_Lo, //U+A46F
UNICODE_Lo, //U+A470
UNICODE_Lo, //U+A471
UNICODE_Lo, //U+A472
UNICODE_Lo, //U+A473
UNICODE_Lo, //U+A474
UNICODE_Lo, //U+A475
UNICODE_Lo, //U+A476
UNICODE_Lo, //U+A477
UNICODE_Lo, //U+A478
UNICODE_Lo, //U+A479
UNICODE_Lo, //U+A47A
UNICODE_Lo, //U+A47B
UNICODE_Lo, //U+A47C
UNICODE_Lo, //U+A47D
UNICODE_Lo, //U+A47E
UNICODE_Lo, //U+A47F
UNICODE_Lo, //U+A480
UNICODE_Lo, //U+A481
UNICODE_Lo, //U+A482
UNICODE_Lo, //U+A483
UNICODE_Lo, //U+A484
UNICODE_Lo, //U+A485
UNICODE_Lo, //U+A486
UNICODE_Lo, //U+A487
UNICODE_Lo, //U+A488
UNICODE_Lo, //U+A489
UNICODE_Lo, //U+A48A
UNICODE_Lo, //U+A48B
UNICODE_Lo, //U+A48C
0, //U+A48D
0, //U+A48E
0, //U+A48F
UNICODE_So, //U+A490
UNICODE_So, //U+A491
UNICODE_So, //U+A492
UNICODE_So, //U+A493
UNICODE_So, //U+A494
UNICODE_So, //U+A495
UNICODE_So, //U+A496
UNICODE_So, //U+A497
UNICODE_So, //U+A498
UNICODE_So, //U+A499
UNICODE_So, //U+A49A
UNICODE_So, //U+A49B
UNICODE_So, //U+A49C
UNICODE_So, //U+A49D
UNICODE_So, //U+A49E
UNICODE_So, //U+A49F
UNICODE_So, //U+A4A0
UNICODE_So, //U+A4A1
UNICODE_So, //U+A4A2
UNICODE_So, //U+A4A3
UNICODE_So, //U+A4A4
UNICODE_So, //U+A4A5
UNICODE_So, //U+A4A6
UNICODE_So, //U+A4A7
UNICODE_So, //U+A4A8
UNICODE_So, //U+A4A9
UNICODE_So, //U+A4AA
UNICODE_So, //U+A4AB
UNICODE_So, //U+A4AC
UNICODE_So, //U+A4AD
UNICODE_So, //U+A4AE
UNICODE_So, //U+A4AF
UNICODE_So, //U+A4B0
UNICODE_So, //U+A4B1
UNICODE_So, //U+A4B2
UNICODE_So, //U+A4B3
UNICODE_So, //U+A4B4
UNICODE_So, //U+A4B5
UNICODE_So, //U+A4B6
UNICODE_So, //U+A4B7
UNICODE_So, //U+A4B8
UNICODE_So, //U+A4B9
UNICODE_So, //U+A4BA
UNICODE_So, //U+A4BB
UNICODE_So, //U+A4BC
UNICODE_So, //U+A4BD
UNICODE_So, //U+A4BE
UNICODE_So, //U+A4BF
UNICODE_So, //U+A4C0
UNICODE_So, //U+A4C1
UNICODE_So, //U+A4C2
UNICODE_So, //U+A4C3
UNICODE_So, //U+A4C4
UNICODE_So, //U+A4C5
UNICODE_So, //U+A4C6
0, //U+A4C7
0, //U+A4C8
0, //U+A4C9
0, //U+A4CA
0, //U+A4CB
0, //U+A4CC
0, //U+A4CD
0, //U+A4CE
0, //U+A4CF
UNICODE_Lo, //U+A4D0
UNICODE_Lo, //U+A4D1
UNICODE_Lo, //U+A4D2
UNICODE_Lo, //U+A4D3
UNICODE_Lo, //U+A4D4
UNICODE_Lo, //U+A4D5
UNICODE_Lo, //U+A4D6
UNICODE_Lo, //U+A4D7
UNICODE_Lo, //U+A4D8
UNICODE_Lo, //U+A4D9
UNICODE_Lo, //U+A4DA
UNICODE_Lo, //U+A4DB
UNICODE_Lo, //U+A4DC
UNICODE_Lo, //U+A4DD
UNICODE_Lo, //U+A4DE
UNICODE_Lo, //U+A4DF
UNICODE_Lo, //U+A4E0
UNICODE_Lo, //U+A4E1
UNICODE_Lo, //U+A4E2
UNICODE_Lo, //U+A4E3
UNICODE_Lo, //U+A4E4
UNICODE_Lo, //U+A4E5
UNICODE_Lo, //U+A4E6
UNICODE_Lo, //U+A4E7
UNICODE_Lo, //U+A4E8
UNICODE_Lo, //U+A4E9
UNICODE_Lo, //U+A4EA
UNICODE_Lo, //U+A4EB
UNICODE_Lo, //U+A4EC
UNICODE_Lo, //U+A4ED
UNICODE_Lo, //U+A4EE
UNICODE_Lo, //U+A4EF
UNICODE_Lo, //U+A4F0
UNICODE_Lo, //U+A4F1
UNICODE_Lo, //U+A4F2
UNICODE_Lo, //U+A4F3
UNICODE_Lo, //U+A4F4
UNICODE_Lo, //U+A4F5
UNICODE_Lo, //U+A4F6
UNICODE_Lo, //U+A4F7
UNICODE_Lm, //U+A4F8
UNICODE_Lm, //U+A4F9
UNICODE_Lm, //U+A4FA
UNICODE_Lm, //U+A4FB
UNICODE_Lm, //U+A4FC
UNICODE_Lm, //U+A4FD
UNICODE_Po, //U+A4FE
UNICODE_Po, //U+A4FF
UNICODE_Lo, //U+A500
UNICODE_Lo, //U+A501
UNICODE_Lo, //U+A502
UNICODE_Lo, //U+A503
UNICODE_Lo, //U+A504
UNICODE_Lo, //U+A505
UNICODE_Lo, //U+A506
UNICODE_Lo, //U+A507
UNICODE_Lo, //U+A508
UNICODE_Lo, //U+A509
UNICODE_Lo, //U+A50A
UNICODE_Lo, //U+A50B
UNICODE_Lo, //U+A50C
UNICODE_Lo, //U+A50D
UNICODE_Lo, //U+A50E
UNICODE_Lo, //U+A50F
UNICODE_Lo, //U+A510
UNICODE_Lo, //U+A511
UNICODE_Lo, //U+A512
UNICODE_Lo, //U+A513
UNICODE_Lo, //U+A514
UNICODE_Lo, //U+A515
UNICODE_Lo, //U+A516
UNICODE_Lo, //U+A517
UNICODE_Lo, //U+A518
UNICODE_Lo, //U+A519
UNICODE_Lo, //U+A51A
UNICODE_Lo, //U+A51B
UNICODE_Lo, //U+A51C
UNICODE_Lo, //U+A51D
UNICODE_Lo, //U+A51E
UNICODE_Lo, //U+A51F
UNICODE_Lo, //U+A520
UNICODE_Lo, //U+A521
UNICODE_Lo, //U+A522
UNICODE_Lo, //U+A523
UNICODE_Lo, //U+A524
UNICODE_Lo, //U+A525
UNICODE_Lo, //U+A526
UNICODE_Lo, //U+A527
UNICODE_Lo, //U+A528
UNICODE_Lo, //U+A529
UNICODE_Lo, //U+A52A
UNICODE_Lo, //U+A52B
UNICODE_Lo, //U+A52C
UNICODE_Lo, //U+A52D
UNICODE_Lo, //U+A52E
UNICODE_Lo, //U+A52F
UNICODE_Lo, //U+A530
UNICODE_Lo, //U+A531
UNICODE_Lo, //U+A532
UNICODE_Lo, //U+A533
UNICODE_Lo, //U+A534
UNICODE_Lo, //U+A535
UNICODE_Lo, //U+A536
UNICODE_Lo, //U+A537
UNICODE_Lo, //U+A538
UNICODE_Lo, //U+A539
UNICODE_Lo, //U+A53A
UNICODE_Lo, //U+A53B
UNICODE_Lo, //U+A53C
UNICODE_Lo, //U+A53D
UNICODE_Lo, //U+A53E
UNICODE_Lo, //U+A53F
UNICODE_Lo, //U+A540
UNICODE_Lo, //U+A541
UNICODE_Lo, //U+A542
UNICODE_Lo, //U+A543
UNICODE_Lo, //U+A544
UNICODE_Lo, //U+A545
UNICODE_Lo, //U+A546
UNICODE_Lo, //U+A547
UNICODE_Lo, //U+A548
UNICODE_Lo, //U+A549
UNICODE_Lo, //U+A54A
UNICODE_Lo, //U+A54B
UNICODE_Lo, //U+A54C
UNICODE_Lo, //U+A54D
UNICODE_Lo, //U+A54E
UNICODE_Lo, //U+A54F
UNICODE_Lo, //U+A550
UNICODE_Lo, //U+A551
UNICODE_Lo, //U+A552
UNICODE_Lo, //U+A553
UNICODE_Lo, //U+A554
UNICODE_Lo, //U+A555
UNICODE_Lo, //U+A556
UNICODE_Lo, //U+A557
UNICODE_Lo, //U+A558
UNICODE_Lo, //U+A559
UNICODE_Lo, //U+A55A
UNICODE_Lo, //U+A55B
UNICODE_Lo, //U+A55C
UNICODE_Lo, //U+A55D
UNICODE_Lo, //U+A55E
UNICODE_Lo, //U+A55F
UNICODE_Lo, //U+A560
UNICODE_Lo, //U+A561
UNICODE_Lo, //U+A562
UNICODE_Lo, //U+A563
UNICODE_Lo, //U+A564
UNICODE_Lo, //U+A565
UNICODE_Lo, //U+A566
UNICODE_Lo, //U+A567
UNICODE_Lo, //U+A568
UNICODE_Lo, //U+A569
UNICODE_Lo, //U+A56A
UNICODE_Lo, //U+A56B
UNICODE_Lo, //U+A56C
UNICODE_Lo, //U+A56D
UNICODE_Lo, //U+A56E
UNICODE_Lo, //U+A56F
UNICODE_Lo, //U+A570
UNICODE_Lo, //U+A571
UNICODE_Lo, //U+A572
UNICODE_Lo, //U+A573
UNICODE_Lo, //U+A574
UNICODE_Lo, //U+A575
UNICODE_Lo, //U+A576
UNICODE_Lo, //U+A577
UNICODE_Lo, //U+A578
UNICODE_Lo, //U+A579
UNICODE_Lo, //U+A57A
UNICODE_Lo, //U+A57B
UNICODE_Lo, //U+A57C
UNICODE_Lo, //U+A57D
UNICODE_Lo, //U+A57E
UNICODE_Lo, //U+A57F
UNICODE_Lo, //U+A580
UNICODE_Lo, //U+A581
UNICODE_Lo, //U+A582
UNICODE_Lo, //U+A583
UNICODE_Lo, //U+A584
UNICODE_Lo, //U+A585
UNICODE_Lo, //U+A586
UNICODE_Lo, //U+A587
UNICODE_Lo, //U+A588
UNICODE_Lo, //U+A589
UNICODE_Lo, //U+A58A
UNICODE_Lo, //U+A58B
UNICODE_Lo, //U+A58C
UNICODE_Lo, //U+A58D
UNICODE_Lo, //U+A58E
UNICODE_Lo, //U+A58F
UNICODE_Lo, //U+A590
UNICODE_Lo, //U+A591
UNICODE_Lo, //U+A592
UNICODE_Lo, //U+A593
UNICODE_Lo, //U+A594
UNICODE_Lo, //U+A595
UNICODE_Lo, //U+A596
UNICODE_Lo, //U+A597
UNICODE_Lo, //U+A598
UNICODE_Lo, //U+A599
UNICODE_Lo, //U+A59A
UNICODE_Lo, //U+A59B
UNICODE_Lo, //U+A59C
UNICODE_Lo, //U+A59D
UNICODE_Lo, //U+A59E
UNICODE_Lo, //U+A59F
UNICODE_Lo, //U+A5A0
UNICODE_Lo, //U+A5A1
UNICODE_Lo, //U+A5A2
UNICODE_Lo, //U+A5A3
UNICODE_Lo, //U+A5A4
UNICODE_Lo, //U+A5A5
UNICODE_Lo, //U+A5A6
UNICODE_Lo, //U+A5A7
UNICODE_Lo, //U+A5A8
UNICODE_Lo, //U+A5A9
UNICODE_Lo, //U+A5AA
UNICODE_Lo, //U+A5AB
UNICODE_Lo, //U+A5AC
UNICODE_Lo, //U+A5AD
UNICODE_Lo, //U+A5AE
UNICODE_Lo, //U+A5AF
UNICODE_Lo, //U+A5B0
UNICODE_Lo, //U+A5B1
UNICODE_Lo, //U+A5B2
UNICODE_Lo, //U+A5B3
UNICODE_Lo, //U+A5B4
UNICODE_Lo, //U+A5B5
UNICODE_Lo, //U+A5B6
UNICODE_Lo, //U+A5B7
UNICODE_Lo, //U+A5B8
UNICODE_Lo, //U+A5B9
UNICODE_Lo, //U+A5BA
UNICODE_Lo, //U+A5BB
UNICODE_Lo, //U+A5BC
UNICODE_Lo, //U+A5BD
UNICODE_Lo, //U+A5BE
UNICODE_Lo, //U+A5BF
UNICODE_Lo, //U+A5C0
UNICODE_Lo, //U+A5C1
UNICODE_Lo, //U+A5C2
UNICODE_Lo, //U+A5C3
UNICODE_Lo, //U+A5C4
UNICODE_Lo, //U+A5C5
UNICODE_Lo, //U+A5C6
UNICODE_Lo, //U+A5C7
UNICODE_Lo, //U+A5C8
UNICODE_Lo, //U+A5C9
UNICODE_Lo, //U+A5CA
UNICODE_Lo, //U+A5CB
UNICODE_Lo, //U+A5CC
UNICODE_Lo, //U+A5CD
UNICODE_Lo, //U+A5CE
UNICODE_Lo, //U+A5CF
UNICODE_Lo, //U+A5D0
UNICODE_Lo, //U+A5D1
UNICODE_Lo, //U+A5D2
UNICODE_Lo, //U+A5D3
UNICODE_Lo, //U+A5D4
UNICODE_Lo, //U+A5D5
UNICODE_Lo, //U+A5D6
UNICODE_Lo, //U+A5D7
UNICODE_Lo, //U+A5D8
UNICODE_Lo, //U+A5D9
UNICODE_Lo, //U+A5DA
UNICODE_Lo, //U+A5DB
UNICODE_Lo, //U+A5DC
UNICODE_Lo, //U+A5DD
UNICODE_Lo, //U+A5DE
UNICODE_Lo, //U+A5DF
UNICODE_Lo, //U+A5E0
UNICODE_Lo, //U+A5E1
UNICODE_Lo, //U+A5E2
UNICODE_Lo, //U+A5E3
UNICODE_Lo, //U+A5E4
UNICODE_Lo, //U+A5E5
UNICODE_Lo, //U+A5E6
UNICODE_Lo, //U+A5E7
UNICODE_Lo, //U+A5E8
UNICODE_Lo, //U+A5E9
UNICODE_Lo, //U+A5EA
UNICODE_Lo, //U+A5EB
UNICODE_Lo, //U+A5EC
UNICODE_Lo, //U+A5ED
UNICODE_Lo, //U+A5EE
UNICODE_Lo, //U+A5EF
UNICODE_Lo, //U+A5F0
UNICODE_Lo, //U+A5F1
UNICODE_Lo, //U+A5F2
UNICODE_Lo, //U+A5F3
UNICODE_Lo, //U+A5F4
UNICODE_Lo, //U+A5F5
UNICODE_Lo, //U+A5F6
UNICODE_Lo, //U+A5F7
UNICODE_Lo, //U+A5F8
UNICODE_Lo, //U+A5F9
UNICODE_Lo, //U+A5FA
UNICODE_Lo, //U+A5FB
UNICODE_Lo, //U+A5FC
UNICODE_Lo, //U+A5FD
UNICODE_Lo, //U+A5FE
UNICODE_Lo, //U+A5FF
UNICODE_Lo, //U+A600
UNICODE_Lo, //U+A601
UNICODE_Lo, //U+A602
UNICODE_Lo, //U+A603
UNICODE_Lo, //U+A604
UNICODE_Lo, //U+A605
UNICODE_Lo, //U+A606
UNICODE_Lo, //U+A607
UNICODE_Lo, //U+A608
UNICODE_Lo, //U+A609
UNICODE_Lo, //U+A60A
UNICODE_Lo, //U+A60B
UNICODE_Lm, //U+A60C
UNICODE_Po, //U+A60D
UNICODE_Po, //U+A60E
UNICODE_Po, //U+A60F
UNICODE_Lo, //U+A610
UNICODE_Lo, //U+A611
UNICODE_Lo, //U+A612
UNICODE_Lo, //U+A613
UNICODE_Lo, //U+A614
UNICODE_Lo, //U+A615
UNICODE_Lo, //U+A616
UNICODE_Lo, //U+A617
UNICODE_Lo, //U+A618
UNICODE_Lo, //U+A619
UNICODE_Lo, //U+A61A
UNICODE_Lo, //U+A61B
UNICODE_Lo, //U+A61C
UNICODE_Lo, //U+A61D
UNICODE_Lo, //U+A61E
UNICODE_Lo, //U+A61F
UNICODE_Nd, //U+A620
UNICODE_Nd, //U+A621
UNICODE_Nd, //U+A622
UNICODE_Nd, //U+A623
UNICODE_Nd, //U+A624
UNICODE_Nd, //U+A625
UNICODE_Nd, //U+A626
UNICODE_Nd, //U+A627
UNICODE_Nd, //U+A628
UNICODE_Nd, //U+A629
UNICODE_Lo, //U+A62A
UNICODE_Lo, //U+A62B
0, //U+A62C
0, //U+A62D
0, //U+A62E
0, //U+A62F
0, //U+A630
0, //U+A631
0, //U+A632
0, //U+A633
0, //U+A634
0, //U+A635
0, //U+A636
0, //U+A637
0, //U+A638
0, //U+A639
0, //U+A63A
0, //U+A63B
0, //U+A63C
0, //U+A63D
0, //U+A63E
0, //U+A63F
UNICODE_Lu, //U+A640
UNICODE_Ll, //U+A641
UNICODE_Lu, //U+A642
UNICODE_Ll, //U+A643
UNICODE_Lu, //U+A644
UNICODE_Ll, //U+A645
UNICODE_Lu, //U+A646
UNICODE_Ll, //U+A647
UNICODE_Lu, //U+A648
UNICODE_Ll, //U+A649
UNICODE_Lu, //U+A64A
UNICODE_Ll, //U+A64B
UNICODE_Lu, //U+A64C
UNICODE_Ll, //U+A64D
UNICODE_Lu, //U+A64E
UNICODE_Ll, //U+A64F
UNICODE_Lu, //U+A650
UNICODE_Ll, //U+A651
UNICODE_Lu, //U+A652
UNICODE_Ll, //U+A653
UNICODE_Lu, //U+A654
UNICODE_Ll, //U+A655
UNICODE_Lu, //U+A656
UNICODE_Ll, //U+A657
UNICODE_Lu, //U+A658
UNICODE_Ll, //U+A659
UNICODE_Lu, //U+A65A
UNICODE_Ll, //U+A65B
UNICODE_Lu, //U+A65C
UNICODE_Ll, //U+A65D
UNICODE_Lu, //U+A65E
UNICODE_Ll, //U+A65F
UNICODE_Lu, //U+A660
UNICODE_Ll, //U+A661
UNICODE_Lu, //U+A662
UNICODE_Ll, //U+A663
UNICODE_Lu, //U+A664
UNICODE_Ll, //U+A665
UNICODE_Lu, //U+A666
UNICODE_Ll, //U+A667
UNICODE_Lu, //U+A668
UNICODE_Ll, //U+A669
UNICODE_Lu, //U+A66A
UNICODE_Ll, //U+A66B
UNICODE_Lu, //U+A66C
UNICODE_Ll, //U+A66D
UNICODE_Lo, //U+A66E
UNICODE_Mn, //U+A66F
UNICODE_Me, //U+A670
UNICODE_Me, //U+A671
UNICODE_Me, //U+A672
UNICODE_Po, //U+A673
0, //U+A674
0, //U+A675
0, //U+A676
0, //U+A677
0, //U+A678
0, //U+A679
0, //U+A67A
0, //U+A67B
UNICODE_Mn, //U+A67C
UNICODE_Mn, //U+A67D
UNICODE_Po, //U+A67E
UNICODE_Lm, //U+A67F
UNICODE_Lu, //U+A680
UNICODE_Ll, //U+A681
UNICODE_Lu, //U+A682
UNICODE_Ll, //U+A683
UNICODE_Lu, //U+A684
UNICODE_Ll, //U+A685
UNICODE_Lu, //U+A686
UNICODE_Ll, //U+A687
UNICODE_Lu, //U+A688
UNICODE_Ll, //U+A689
UNICODE_Lu, //U+A68A
UNICODE_Ll, //U+A68B
UNICODE_Lu, //U+A68C
UNICODE_Ll, //U+A68D
UNICODE_Lu, //U+A68E
UNICODE_Ll, //U+A68F
UNICODE_Lu, //U+A690
UNICODE_Ll, //U+A691
UNICODE_Lu, //U+A692
UNICODE_Ll, //U+A693
UNICODE_Lu, //U+A694
UNICODE_Ll, //U+A695
UNICODE_Lu, //U+A696
UNICODE_Ll, //U+A697
0, //U+A698
0, //U+A699
0, //U+A69A
0, //U+A69B
0, //U+A69C
0, //U+A69D
0, //U+A69E
0, //U+A69F
UNICODE_Lo, //U+A6A0
UNICODE_Lo, //U+A6A1
UNICODE_Lo, //U+A6A2
UNICODE_Lo, //U+A6A3
UNICODE_Lo, //U+A6A4
UNICODE_Lo, //U+A6A5
UNICODE_Lo, //U+A6A6
UNICODE_Lo, //U+A6A7
UNICODE_Lo, //U+A6A8
UNICODE_Lo, //U+A6A9
UNICODE_Lo, //U+A6AA
UNICODE_Lo, //U+A6AB
UNICODE_Lo, //U+A6AC
UNICODE_Lo, //U+A6AD
UNICODE_Lo, //U+A6AE
UNICODE_Lo, //U+A6AF
UNICODE_Lo, //U+A6B0
UNICODE_Lo, //U+A6B1
UNICODE_Lo, //U+A6B2
UNICODE_Lo, //U+A6B3
UNICODE_Lo, //U+A6B4
UNICODE_Lo, //U+A6B5
UNICODE_Lo, //U+A6B6
UNICODE_Lo, //U+A6B7
UNICODE_Lo, //U+A6B8
UNICODE_Lo, //U+A6B9
UNICODE_Lo, //U+A6BA
UNICODE_Lo, //U+A6BB
UNICODE_Lo, //U+A6BC
UNICODE_Lo, //U+A6BD
UNICODE_Lo, //U+A6BE
UNICODE_Lo, //U+A6BF
UNICODE_Lo, //U+A6C0
UNICODE_Lo, //U+A6C1
UNICODE_Lo, //U+A6C2
UNICODE_Lo, //U+A6C3
UNICODE_Lo, //U+A6C4
UNICODE_Lo, //U+A6C5
UNICODE_Lo, //U+A6C6
UNICODE_Lo, //U+A6C7
UNICODE_Lo, //U+A6C8
UNICODE_Lo, //U+A6C9
UNICODE_Lo, //U+A6CA
UNICODE_Lo, //U+A6CB
UNICODE_Lo, //U+A6CC
UNICODE_Lo, //U+A6CD
UNICODE_Lo, //U+A6CE
UNICODE_Lo, //U+A6CF
UNICODE_Lo, //U+A6D0
UNICODE_Lo, //U+A6D1
UNICODE_Lo, //U+A6D2
UNICODE_Lo, //U+A6D3
UNICODE_Lo, //U+A6D4
UNICODE_Lo, //U+A6D5
UNICODE_Lo, //U+A6D6
UNICODE_Lo, //U+A6D7
UNICODE_Lo, //U+A6D8
UNICODE_Lo, //U+A6D9
UNICODE_Lo, //U+A6DA
UNICODE_Lo, //U+A6DB
UNICODE_Lo, //U+A6DC
UNICODE_Lo, //U+A6DD
UNICODE_Lo, //U+A6DE
UNICODE_Lo, //U+A6DF
UNICODE_Lo, //U+A6E0
UNICODE_Lo, //U+A6E1
UNICODE_Lo, //U+A6E2
UNICODE_Lo, //U+A6E3
UNICODE_Lo, //U+A6E4
UNICODE_Lo, //U+A6E5
UNICODE_Nl, //U+A6E6
UNICODE_Nl, //U+A6E7
UNICODE_Nl, //U+A6E8
UNICODE_Nl, //U+A6E9
UNICODE_Nl, //U+A6EA
UNICODE_Nl, //U+A6EB
UNICODE_Nl, //U+A6EC
UNICODE_Nl, //U+A6ED
UNICODE_Nl, //U+A6EE
UNICODE_Nl, //U+A6EF
UNICODE_Mn, //U+A6F0
UNICODE_Mn, //U+A6F1
UNICODE_Po, //U+A6F2
UNICODE_Po, //U+A6F3
UNICODE_Po, //U+A6F4
UNICODE_Po, //U+A6F5
UNICODE_Po, //U+A6F6
UNICODE_Po, //U+A6F7
0, //U+A6F8
0, //U+A6F9
0, //U+A6FA
0, //U+A6FB
0, //U+A6FC
0, //U+A6FD
0, //U+A6FE
0, //U+A6FF
UNICODE_Sk, //U+A700
UNICODE_Sk, //U+A701
UNICODE_Sk, //U+A702
UNICODE_Sk, //U+A703
UNICODE_Sk, //U+A704
UNICODE_Sk, //U+A705
UNICODE_Sk, //U+A706
UNICODE_Sk, //U+A707
UNICODE_Sk, //U+A708
UNICODE_Sk, //U+A709
UNICODE_Sk, //U+A70A
UNICODE_Sk, //U+A70B
UNICODE_Sk, //U+A70C
UNICODE_Sk, //U+A70D
UNICODE_Sk, //U+A70E
UNICODE_Sk, //U+A70F
UNICODE_Sk, //U+A710
UNICODE_Sk, //U+A711
UNICODE_Sk, //U+A712
UNICODE_Sk, //U+A713
UNICODE_Sk, //U+A714
UNICODE_Sk, //U+A715
UNICODE_Sk, //U+A716
UNICODE_Lm, //U+A717
UNICODE_Lm, //U+A718
UNICODE_Lm, //U+A719
UNICODE_Lm, //U+A71A
UNICODE_Lm, //U+A71B
UNICODE_Lm, //U+A71C
UNICODE_Lm, //U+A71D
UNICODE_Lm, //U+A71E
UNICODE_Lm, //U+A71F
UNICODE_Sk, //U+A720
UNICODE_Sk, //U+A721
UNICODE_Lu, //U+A722
UNICODE_Ll, //U+A723
UNICODE_Lu, //U+A724
UNICODE_Ll, //U+A725
UNICODE_Lu, //U+A726
UNICODE_Ll, //U+A727
UNICODE_Lu, //U+A728
UNICODE_Ll, //U+A729
UNICODE_Lu, //U+A72A
UNICODE_Ll, //U+A72B
UNICODE_Lu, //U+A72C
UNICODE_Ll, //U+A72D
UNICODE_Lu, //U+A72E
UNICODE_Ll, //U+A72F
UNICODE_Ll, //U+A730
UNICODE_Ll, //U+A731
UNICODE_Lu, //U+A732
UNICODE_Ll, //U+A733
UNICODE_Lu, //U+A734
UNICODE_Ll, //U+A735
UNICODE_Lu, //U+A736
UNICODE_Ll, //U+A737
UNICODE_Lu, //U+A738
UNICODE_Ll, //U+A739
UNICODE_Lu, //U+A73A
UNICODE_Ll, //U+A73B
UNICODE_Lu, //U+A73C
UNICODE_Ll, //U+A73D
UNICODE_Lu, //U+A73E
UNICODE_Ll, //U+A73F
UNICODE_Lu, //U+A740
UNICODE_Ll, //U+A741
UNICODE_Lu, //U+A742
UNICODE_Ll, //U+A743
UNICODE_Lu, //U+A744
UNICODE_Ll, //U+A745
UNICODE_Lu, //U+A746
UNICODE_Ll, //U+A747
UNICODE_Lu, //U+A748
UNICODE_Ll, //U+A749
UNICODE_Lu, //U+A74A
UNICODE_Ll, //U+A74B
UNICODE_Lu, //U+A74C
UNICODE_Ll, //U+A74D
UNICODE_Lu, //U+A74E
UNICODE_Ll, //U+A74F
UNICODE_Lu, //U+A750
UNICODE_Ll, //U+A751
UNICODE_Lu, //U+A752
UNICODE_Ll, //U+A753
UNICODE_Lu, //U+A754
UNICODE_Ll, //U+A755
UNICODE_Lu, //U+A756
UNICODE_Ll, //U+A757
UNICODE_Lu, //U+A758
UNICODE_Ll, //U+A759
UNICODE_Lu, //U+A75A
UNICODE_Ll, //U+A75B
UNICODE_Lu, //U+A75C
UNICODE_Ll, //U+A75D
UNICODE_Lu, //U+A75E
UNICODE_Ll, //U+A75F
UNICODE_Lu, //U+A760
UNICODE_Ll, //U+A761
UNICODE_Lu, //U+A762
UNICODE_Ll, //U+A763
UNICODE_Lu, //U+A764
UNICODE_Ll, //U+A765
UNICODE_Lu, //U+A766
UNICODE_Ll, //U+A767
UNICODE_Lu, //U+A768
UNICODE_Ll, //U+A769
UNICODE_Lu, //U+A76A
UNICODE_Ll, //U+A76B
UNICODE_Lu, //U+A76C
UNICODE_Ll, //U+A76D
UNICODE_Lu, //U+A76E
UNICODE_Ll, //U+A76F
UNICODE_Lm, //U+A770
UNICODE_Ll, //U+A771
UNICODE_Ll, //U+A772
UNICODE_Ll, //U+A773
UNICODE_Ll, //U+A774
UNICODE_Ll, //U+A775
UNICODE_Ll, //U+A776
UNICODE_Ll, //U+A777
UNICODE_Ll, //U+A778
UNICODE_Lu, //U+A779
UNICODE_Ll, //U+A77A
UNICODE_Lu, //U+A77B
UNICODE_Ll, //U+A77C
UNICODE_Lu, //U+A77D
UNICODE_Lu, //U+A77E
UNICODE_Ll, //U+A77F
UNICODE_Lu, //U+A780
UNICODE_Ll, //U+A781
UNICODE_Lu, //U+A782
UNICODE_Ll, //U+A783
UNICODE_Lu, //U+A784
UNICODE_Ll, //U+A785
UNICODE_Lu, //U+A786
UNICODE_Ll, //U+A787
UNICODE_Lm, //U+A788
UNICODE_Sk, //U+A789
UNICODE_Sk, //U+A78A
UNICODE_Lu, //U+A78B
UNICODE_Ll, //U+A78C
UNICODE_Lu, //U+A78D
UNICODE_Ll, //U+A78E
0, //U+A78F
UNICODE_Lu, //U+A790
UNICODE_Ll, //U+A791
0, //U+A792
0, //U+A793
0, //U+A794
0, //U+A795
0, //U+A796
0, //U+A797
0, //U+A798
0, //U+A799
0, //U+A79A
0, //U+A79B
0, //U+A79C
0, //U+A79D
0, //U+A79E
0, //U+A79F
UNICODE_Lu, //U+A7A0
UNICODE_Ll, //U+A7A1
UNICODE_Lu, //U+A7A2
UNICODE_Ll, //U+A7A3
UNICODE_Lu, //U+A7A4
UNICODE_Ll, //U+A7A5
UNICODE_Lu, //U+A7A6
UNICODE_Ll, //U+A7A7
UNICODE_Lu, //U+A7A8
UNICODE_Ll, //U+A7A9
0, //U+A7AA
0, //U+A7AB
0, //U+A7AC
0, //U+A7AD
0, //U+A7AE
0, //U+A7AF
0, //U+A7B0
0, //U+A7B1
0, //U+A7B2
0, //U+A7B3
0, //U+A7B4
0, //U+A7B5
0, //U+A7B6
0, //U+A7B7
0, //U+A7B8
0, //U+A7B9
0, //U+A7BA
0, //U+A7BB
0, //U+A7BC
0, //U+A7BD
0, //U+A7BE
0, //U+A7BF
0, //U+A7C0
0, //U+A7C1
0, //U+A7C2
0, //U+A7C3
0, //U+A7C4
0, //U+A7C5
0, //U+A7C6
0, //U+A7C7
0, //U+A7C8
0, //U+A7C9
0, //U+A7CA
0, //U+A7CB
0, //U+A7CC
0, //U+A7CD
0, //U+A7CE
0, //U+A7CF
0, //U+A7D0
0, //U+A7D1
0, //U+A7D2
0, //U+A7D3
0, //U+A7D4
0, //U+A7D5
0, //U+A7D6
0, //U+A7D7
0, //U+A7D8
0, //U+A7D9
0, //U+A7DA
0, //U+A7DB
0, //U+A7DC
0, //U+A7DD
0, //U+A7DE
0, //U+A7DF
0, //U+A7E0
0, //U+A7E1
0, //U+A7E2
0, //U+A7E3
0, //U+A7E4
0, //U+A7E5
0, //U+A7E6
0, //U+A7E7
0, //U+A7E8
0, //U+A7E9
0, //U+A7EA
0, //U+A7EB
0, //U+A7EC
0, //U+A7ED
0, //U+A7EE
0, //U+A7EF
0, //U+A7F0
0, //U+A7F1
0, //U+A7F2
0, //U+A7F3
0, //U+A7F4
0, //U+A7F5
0, //U+A7F6
0, //U+A7F7
0, //U+A7F8
0, //U+A7F9
UNICODE_Ll, //U+A7FA
UNICODE_Lo, //U+A7FB
UNICODE_Lo, //U+A7FC
UNICODE_Lo, //U+A7FD
UNICODE_Lo, //U+A7FE
UNICODE_Lo, //U+A7FF
UNICODE_Lo, //U+A800
UNICODE_Lo, //U+A801
UNICODE_Mn, //U+A802
UNICODE_Lo, //U+A803
UNICODE_Lo, //U+A804
UNICODE_Lo, //U+A805
UNICODE_Mn, //U+A806
UNICODE_Lo, //U+A807
UNICODE_Lo, //U+A808
UNICODE_Lo, //U+A809
UNICODE_Lo, //U+A80A
UNICODE_Mn, //U+A80B
UNICODE_Lo, //U+A80C
UNICODE_Lo, //U+A80D
UNICODE_Lo, //U+A80E
UNICODE_Lo, //U+A80F
UNICODE_Lo, //U+A810
UNICODE_Lo, //U+A811
UNICODE_Lo, //U+A812
UNICODE_Lo, //U+A813
UNICODE_Lo, //U+A814
UNICODE_Lo, //U+A815
UNICODE_Lo, //U+A816
UNICODE_Lo, //U+A817
UNICODE_Lo, //U+A818
UNICODE_Lo, //U+A819
UNICODE_Lo, //U+A81A
UNICODE_Lo, //U+A81B
UNICODE_Lo, //U+A81C
UNICODE_Lo, //U+A81D
UNICODE_Lo, //U+A81E
UNICODE_Lo, //U+A81F
UNICODE_Lo, //U+A820
UNICODE_Lo, //U+A821
UNICODE_Lo, //U+A822
UNICODE_Mc, //U+A823
UNICODE_Mc, //U+A824
UNICODE_Mn, //U+A825
UNICODE_Mn, //U+A826
UNICODE_Mc, //U+A827
UNICODE_So, //U+A828
UNICODE_So, //U+A829
UNICODE_So, //U+A82A
UNICODE_So, //U+A82B
0, //U+A82C
0, //U+A82D
0, //U+A82E
0, //U+A82F
UNICODE_No, //U+A830
UNICODE_No, //U+A831
UNICODE_No, //U+A832
UNICODE_No, //U+A833
UNICODE_No, //U+A834
UNICODE_No, //U+A835
UNICODE_So, //U+A836
UNICODE_So, //U+A837
UNICODE_Sc, //U+A838
UNICODE_So, //U+A839
0, //U+A83A
0, //U+A83B
0, //U+A83C
0, //U+A83D
0, //U+A83E
0, //U+A83F
UNICODE_Lo, //U+A840
UNICODE_Lo, //U+A841
UNICODE_Lo, //U+A842
UNICODE_Lo, //U+A843
UNICODE_Lo, //U+A844
UNICODE_Lo, //U+A845
UNICODE_Lo, //U+A846
UNICODE_Lo, //U+A847
UNICODE_Lo, //U+A848
UNICODE_Lo, //U+A849
UNICODE_Lo, //U+A84A
UNICODE_Lo, //U+A84B
UNICODE_Lo, //U+A84C
UNICODE_Lo, //U+A84D
UNICODE_Lo, //U+A84E
UNICODE_Lo, //U+A84F
UNICODE_Lo, //U+A850
UNICODE_Lo, //U+A851
UNICODE_Lo, //U+A852
UNICODE_Lo, //U+A853
UNICODE_Lo, //U+A854
UNICODE_Lo, //U+A855
UNICODE_Lo, //U+A856
UNICODE_Lo, //U+A857
UNICODE_Lo, //U+A858
UNICODE_Lo, //U+A859
UNICODE_Lo, //U+A85A
UNICODE_Lo, //U+A85B
UNICODE_Lo, //U+A85C
UNICODE_Lo, //U+A85D
UNICODE_Lo, //U+A85E
UNICODE_Lo, //U+A85F
UNICODE_Lo, //U+A860
UNICODE_Lo, //U+A861
UNICODE_Lo, //U+A862
UNICODE_Lo, //U+A863
UNICODE_Lo, //U+A864
UNICODE_Lo, //U+A865
UNICODE_Lo, //U+A866
UNICODE_Lo, //U+A867
UNICODE_Lo, //U+A868
UNICODE_Lo, //U+A869
UNICODE_Lo, //U+A86A
UNICODE_Lo, //U+A86B
UNICODE_Lo, //U+A86C
UNICODE_Lo, //U+A86D
UNICODE_Lo, //U+A86E
UNICODE_Lo, //U+A86F
UNICODE_Lo, //U+A870
UNICODE_Lo, //U+A871
UNICODE_Lo, //U+A872
UNICODE_Lo, //U+A873
UNICODE_Po, //U+A874
UNICODE_Po, //U+A875
UNICODE_Po, //U+A876
UNICODE_Po, //U+A877
0, //U+A878
0, //U+A879
0, //U+A87A
0, //U+A87B
0, //U+A87C
0, //U+A87D
0, //U+A87E
0, //U+A87F
UNICODE_Mc, //U+A880
UNICODE_Mc, //U+A881
UNICODE_Lo, //U+A882
UNICODE_Lo, //U+A883
UNICODE_Lo, //U+A884
UNICODE_Lo, //U+A885
UNICODE_Lo, //U+A886
UNICODE_Lo, //U+A887
UNICODE_Lo, //U+A888
UNICODE_Lo, //U+A889
UNICODE_Lo, //U+A88A
UNICODE_Lo, //U+A88B
UNICODE_Lo, //U+A88C
UNICODE_Lo, //U+A88D
UNICODE_Lo, //U+A88E
UNICODE_Lo, //U+A88F
UNICODE_Lo, //U+A890
UNICODE_Lo, //U+A891
UNICODE_Lo, //U+A892
UNICODE_Lo, //U+A893
UNICODE_Lo, //U+A894
UNICODE_Lo, //U+A895
UNICODE_Lo, //U+A896
UNICODE_Lo, //U+A897
UNICODE_Lo, //U+A898
UNICODE_Lo, //U+A899
UNICODE_Lo, //U+A89A
UNICODE_Lo, //U+A89B
UNICODE_Lo, //U+A89C
UNICODE_Lo, //U+A89D
UNICODE_Lo, //U+A89E
UNICODE_Lo, //U+A89F
UNICODE_Lo, //U+A8A0
UNICODE_Lo, //U+A8A1
UNICODE_Lo, //U+A8A2
UNICODE_Lo, //U+A8A3
UNICODE_Lo, //U+A8A4
UNICODE_Lo, //U+A8A5
UNICODE_Lo, //U+A8A6
UNICODE_Lo, //U+A8A7
UNICODE_Lo, //U+A8A8
UNICODE_Lo, //U+A8A9
UNICODE_Lo, //U+A8AA
UNICODE_Lo, //U+A8AB
UNICODE_Lo, //U+A8AC
UNICODE_Lo, //U+A8AD
UNICODE_Lo, //U+A8AE
UNICODE_Lo, //U+A8AF
UNICODE_Lo, //U+A8B0
UNICODE_Lo, //U+A8B1
UNICODE_Lo, //U+A8B2
UNICODE_Lo, //U+A8B3
UNICODE_Mc, //U+A8B4
UNICODE_Mc, //U+A8B5
UNICODE_Mc, //U+A8B6
UNICODE_Mc, //U+A8B7
UNICODE_Mc, //U+A8B8
UNICODE_Mc, //U+A8B9
UNICODE_Mc, //U+A8BA
UNICODE_Mc, //U+A8BB
UNICODE_Mc, //U+A8BC
UNICODE_Mc, //U+A8BD
UNICODE_Mc, //U+A8BE
UNICODE_Mc, //U+A8BF
UNICODE_Mc, //U+A8C0
UNICODE_Mc, //U+A8C1
UNICODE_Mc, //U+A8C2
UNICODE_Mc, //U+A8C3
UNICODE_Mn, //U+A8C4
0, //U+A8C5
0, //U+A8C6
0, //U+A8C7
0, //U+A8C8
0, //U+A8C9
0, //U+A8CA
0, //U+A8CB
0, //U+A8CC
0, //U+A8CD
UNICODE_Po, //U+A8CE
UNICODE_Po, //U+A8CF
UNICODE_Nd, //U+A8D0
UNICODE_Nd, //U+A8D1
UNICODE_Nd, //U+A8D2
UNICODE_Nd, //U+A8D3
UNICODE_Nd, //U+A8D4
UNICODE_Nd, //U+A8D5
UNICODE_Nd, //U+A8D6
UNICODE_Nd, //U+A8D7
UNICODE_Nd, //U+A8D8
UNICODE_Nd, //U+A8D9
0, //U+A8DA
0, //U+A8DB
0, //U+A8DC
0, //U+A8DD
0, //U+A8DE
0, //U+A8DF
UNICODE_Mn, //U+A8E0
UNICODE_Mn, //U+A8E1
UNICODE_Mn, //U+A8E2
UNICODE_Mn, //U+A8E3
UNICODE_Mn, //U+A8E4
UNICODE_Mn, //U+A8E5
UNICODE_Mn, //U+A8E6
UNICODE_Mn, //U+A8E7
UNICODE_Mn, //U+A8E8
UNICODE_Mn, //U+A8E9
UNICODE_Mn, //U+A8EA
UNICODE_Mn, //U+A8EB
UNICODE_Mn, //U+A8EC
UNICODE_Mn, //U+A8ED
UNICODE_Mn, //U+A8EE
UNICODE_Mn, //U+A8EF
UNICODE_Mn, //U+A8F0
UNICODE_Mn, //U+A8F1
UNICODE_Lo, //U+A8F2
UNICODE_Lo, //U+A8F3
UNICODE_Lo, //U+A8F4
UNICODE_Lo, //U+A8F5
UNICODE_Lo, //U+A8F6
UNICODE_Lo, //U+A8F7
UNICODE_Po, //U+A8F8
UNICODE_Po, //U+A8F9
UNICODE_Po, //U+A8FA
UNICODE_Lo, //U+A8FB
0, //U+A8FC
0, //U+A8FD
0, //U+A8FE
0, //U+A8FF
UNICODE_Nd, //U+A900
UNICODE_Nd, //U+A901
UNICODE_Nd, //U+A902
UNICODE_Nd, //U+A903
UNICODE_Nd, //U+A904
UNICODE_Nd, //U+A905
UNICODE_Nd, //U+A906
UNICODE_Nd, //U+A907
UNICODE_Nd, //U+A908
UNICODE_Nd, //U+A909
UNICODE_Lo, //U+A90A
UNICODE_Lo, //U+A90B
UNICODE_Lo, //U+A90C
UNICODE_Lo, //U+A90D
UNICODE_Lo, //U+A90E
UNICODE_Lo, //U+A90F
UNICODE_Lo, //U+A910
UNICODE_Lo, //U+A911
UNICODE_Lo, //U+A912
UNICODE_Lo, //U+A913
UNICODE_Lo, //U+A914
UNICODE_Lo, //U+A915
UNICODE_Lo, //U+A916
UNICODE_Lo, //U+A917
UNICODE_Lo, //U+A918
UNICODE_Lo, //U+A919
UNICODE_Lo, //U+A91A
UNICODE_Lo, //U+A91B
UNICODE_Lo, //U+A91C
UNICODE_Lo, //U+A91D
UNICODE_Lo, //U+A91E
UNICODE_Lo, //U+A91F
UNICODE_Lo, //U+A920
UNICODE_Lo, //U+A921
UNICODE_Lo, //U+A922
UNICODE_Lo, //U+A923
UNICODE_Lo, //U+A924
UNICODE_Lo, //U+A925
UNICODE_Mn, //U+A926
UNICODE_Mn, //U+A927
UNICODE_Mn, //U+A928
UNICODE_Mn, //U+A929
UNICODE_Mn, //U+A92A
UNICODE_Mn, //U+A92B
UNICODE_Mn, //U+A92C
UNICODE_Mn, //U+A92D
UNICODE_Po, //U+A92E
UNICODE_Po, //U+A92F
UNICODE_Lo, //U+A930
UNICODE_Lo, //U+A931
UNICODE_Lo, //U+A932
UNICODE_Lo, //U+A933
UNICODE_Lo, //U+A934
UNICODE_Lo, //U+A935
UNICODE_Lo, //U+A936
UNICODE_Lo, //U+A937
UNICODE_Lo, //U+A938
UNICODE_Lo, //U+A939
UNICODE_Lo, //U+A93A
UNICODE_Lo, //U+A93B
UNICODE_Lo, //U+A93C
UNICODE_Lo, //U+A93D
UNICODE_Lo, //U+A93E
UNICODE_Lo, //U+A93F
UNICODE_Lo, //U+A940
UNICODE_Lo, //U+A941
UNICODE_Lo, //U+A942
UNICODE_Lo, //U+A943
UNICODE_Lo, //U+A944
UNICODE_Lo, //U+A945
UNICODE_Lo, //U+A946
UNICODE_Mn, //U+A947
UNICODE_Mn, //U+A948
UNICODE_Mn, //U+A949
UNICODE_Mn, //U+A94A
UNICODE_Mn, //U+A94B
UNICODE_Mn, //U+A94C
UNICODE_Mn, //U+A94D
UNICODE_Mn, //U+A94E
UNICODE_Mn, //U+A94F
UNICODE_Mn, //U+A950
UNICODE_Mn, //U+A951
UNICODE_Mc, //U+A952
UNICODE_Mc, //U+A953
0, //U+A954
0, //U+A955
0, //U+A956
0, //U+A957
0, //U+A958
0, //U+A959
0, //U+A95A
0, //U+A95B
0, //U+A95C
0, //U+A95D
0, //U+A95E
UNICODE_Po, //U+A95F
UNICODE_Lo, //U+A960
UNICODE_Lo, //U+A961
UNICODE_Lo, //U+A962
UNICODE_Lo, //U+A963
UNICODE_Lo, //U+A964
UNICODE_Lo, //U+A965
UNICODE_Lo, //U+A966
UNICODE_Lo, //U+A967
UNICODE_Lo, //U+A968
UNICODE_Lo, //U+A969
UNICODE_Lo, //U+A96A
UNICODE_Lo, //U+A96B
UNICODE_Lo, //U+A96C
UNICODE_Lo, //U+A96D
UNICODE_Lo, //U+A96E
UNICODE_Lo, //U+A96F
UNICODE_Lo, //U+A970
UNICODE_Lo, //U+A971
UNICODE_Lo, //U+A972
UNICODE_Lo, //U+A973
UNICODE_Lo, //U+A974
UNICODE_Lo, //U+A975
UNICODE_Lo, //U+A976
UNICODE_Lo, //U+A977
UNICODE_Lo, //U+A978
UNICODE_Lo, //U+A979
UNICODE_Lo, //U+A97A
UNICODE_Lo, //U+A97B
UNICODE_Lo, //U+A97C
0, //U+A97D
0, //U+A97E
0, //U+A97F
UNICODE_Mn, //U+A980
UNICODE_Mn, //U+A981
UNICODE_Mn, //U+A982
UNICODE_Mc, //U+A983
UNICODE_Lo, //U+A984
UNICODE_Lo, //U+A985
UNICODE_Lo, //U+A986
UNICODE_Lo, //U+A987
UNICODE_Lo, //U+A988
UNICODE_Lo, //U+A989
UNICODE_Lo, //U+A98A
UNICODE_Lo, //U+A98B
UNICODE_Lo, //U+A98C
UNICODE_Lo, //U+A98D
UNICODE_Lo, //U+A98E
UNICODE_Lo, //U+A98F
UNICODE_Lo, //U+A990
UNICODE_Lo, //U+A991
UNICODE_Lo, //U+A992
UNICODE_Lo, //U+A993
UNICODE_Lo, //U+A994
UNICODE_Lo, //U+A995
UNICODE_Lo, //U+A996
UNICODE_Lo, //U+A997
UNICODE_Lo, //U+A998
UNICODE_Lo, //U+A999
UNICODE_Lo, //U+A99A
UNICODE_Lo, //U+A99B
UNICODE_Lo, //U+A99C
UNICODE_Lo, //U+A99D
UNICODE_Lo, //U+A99E
UNICODE_Lo, //U+A99F
UNICODE_Lo, //U+A9A0
UNICODE_Lo, //U+A9A1
UNICODE_Lo, //U+A9A2
UNICODE_Lo, //U+A9A3
UNICODE_Lo, //U+A9A4
UNICODE_Lo, //U+A9A5
UNICODE_Lo, //U+A9A6
UNICODE_Lo, //U+A9A7
UNICODE_Lo, //U+A9A8
UNICODE_Lo, //U+A9A9
UNICODE_Lo, //U+A9AA
UNICODE_Lo, //U+A9AB
UNICODE_Lo, //U+A9AC
UNICODE_Lo, //U+A9AD
UNICODE_Lo, //U+A9AE
UNICODE_Lo, //U+A9AF
UNICODE_Lo, //U+A9B0
UNICODE_Lo, //U+A9B1
UNICODE_Lo, //U+A9B2
UNICODE_Mn, //U+A9B3
UNICODE_Mc, //U+A9B4
UNICODE_Mc, //U+A9B5
UNICODE_Mn, //U+A9B6
UNICODE_Mn, //U+A9B7
UNICODE_Mn, //U+A9B8
UNICODE_Mn, //U+A9B9
UNICODE_Mc, //U+A9BA
UNICODE_Mc, //U+A9BB
UNICODE_Mn, //U+A9BC
UNICODE_Mc, //U+A9BD
UNICODE_Mc, //U+A9BE
UNICODE_Mc, //U+A9BF
UNICODE_Mc, //U+A9C0
UNICODE_Po, //U+A9C1
UNICODE_Po, //U+A9C2
UNICODE_Po, //U+A9C3
UNICODE_Po, //U+A9C4
UNICODE_Po, //U+A9C5
UNICODE_Po, //U+A9C6
UNICODE_Po, //U+A9C7
UNICODE_Po, //U+A9C8
UNICODE_Po, //U+A9C9
UNICODE_Po, //U+A9CA
UNICODE_Po, //U+A9CB
UNICODE_Po, //U+A9CC
UNICODE_Po, //U+A9CD
0, //U+A9CE
UNICODE_Lm, //U+A9CF
UNICODE_Nd, //U+A9D0
UNICODE_Nd, //U+A9D1
UNICODE_Nd, //U+A9D2
UNICODE_Nd, //U+A9D3
UNICODE_Nd, //U+A9D4
UNICODE_Nd, //U+A9D5
UNICODE_Nd, //U+A9D6
UNICODE_Nd, //U+A9D7
UNICODE_Nd, //U+A9D8
UNICODE_Nd, //U+A9D9
0, //U+A9DA
0, //U+A9DB
0, //U+A9DC
0, //U+A9DD
UNICODE_Po, //U+A9DE
UNICODE_Po, //U+A9DF
0, //U+A9E0
0, //U+A9E1
0, //U+A9E2
0, //U+A9E3
0, //U+A9E4
0, //U+A9E5
0, //U+A9E6
0, //U+A9E7
0, //U+A9E8
0, //U+A9E9
0, //U+A9EA
0, //U+A9EB
0, //U+A9EC
0, //U+A9ED
0, //U+A9EE
0, //U+A9EF
0, //U+A9F0
0, //U+A9F1
0, //U+A9F2
0, //U+A9F3
0, //U+A9F4
0, //U+A9F5
0, //U+A9F6
0, //U+A9F7
0, //U+A9F8
0, //U+A9F9
0, //U+A9FA
0, //U+A9FB
0, //U+A9FC
0, //U+A9FD
0, //U+A9FE
0, //U+A9FF
UNICODE_Lo, //U+AA00
UNICODE_Lo, //U+AA01
UNICODE_Lo, //U+AA02
UNICODE_Lo, //U+AA03
UNICODE_Lo, //U+AA04
UNICODE_Lo, //U+AA05
UNICODE_Lo, //U+AA06
UNICODE_Lo, //U+AA07
UNICODE_Lo, //U+AA08
UNICODE_Lo, //U+AA09
UNICODE_Lo, //U+AA0A
UNICODE_Lo, //U+AA0B
UNICODE_Lo, //U+AA0C
UNICODE_Lo, //U+AA0D
UNICODE_Lo, //U+AA0E
UNICODE_Lo, //U+AA0F
UNICODE_Lo, //U+AA10
UNICODE_Lo, //U+AA11
UNICODE_Lo, //U+AA12
UNICODE_Lo, //U+AA13
UNICODE_Lo, //U+AA14
UNICODE_Lo, //U+AA15
UNICODE_Lo, //U+AA16
UNICODE_Lo, //U+AA17
UNICODE_Lo, //U+AA18
UNICODE_Lo, //U+AA19
UNICODE_Lo, //U+AA1A
UNICODE_Lo, //U+AA1B
UNICODE_Lo, //U+AA1C
UNICODE_Lo, //U+AA1D
UNICODE_Lo, //U+AA1E
UNICODE_Lo, //U+AA1F
UNICODE_Lo, //U+AA20
UNICODE_Lo, //U+AA21
UNICODE_Lo, //U+AA22
UNICODE_Lo, //U+AA23
UNICODE_Lo, //U+AA24
UNICODE_Lo, //U+AA25
UNICODE_Lo, //U+AA26
UNICODE_Lo, //U+AA27
UNICODE_Lo, //U+AA28
UNICODE_Mn, //U+AA29
UNICODE_Mn, //U+AA2A
UNICODE_Mn, //U+AA2B
UNICODE_Mn, //U+AA2C
UNICODE_Mn, //U+AA2D
UNICODE_Mn, //U+AA2E
UNICODE_Mc, //U+AA2F
UNICODE_Mc, //U+AA30
UNICODE_Mn, //U+AA31
UNICODE_Mn, //U+AA32
UNICODE_Mc, //U+AA33
UNICODE_Mc, //U+AA34
UNICODE_Mn, //U+AA35
UNICODE_Mn, //U+AA36
0, //U+AA37
0, //U+AA38
0, //U+AA39
0, //U+AA3A
0, //U+AA3B
0, //U+AA3C
0, //U+AA3D
0, //U+AA3E
0, //U+AA3F
UNICODE_Lo, //U+AA40
UNICODE_Lo, //U+AA41
UNICODE_Lo, //U+AA42
UNICODE_Mn, //U+AA43
UNICODE_Lo, //U+AA44
UNICODE_Lo, //U+AA45
UNICODE_Lo, //U+AA46
UNICODE_Lo, //U+AA47
UNICODE_Lo, //U+AA48
UNICODE_Lo, //U+AA49
UNICODE_Lo, //U+AA4A
UNICODE_Lo, //U+AA4B
UNICODE_Mn, //U+AA4C
UNICODE_Mc, //U+AA4D
0, //U+AA4E
0, //U+AA4F
UNICODE_Nd, //U+AA50
UNICODE_Nd, //U+AA51
UNICODE_Nd, //U+AA52
UNICODE_Nd, //U+AA53
UNICODE_Nd, //U+AA54
UNICODE_Nd, //U+AA55
UNICODE_Nd, //U+AA56
UNICODE_Nd, //U+AA57
UNICODE_Nd, //U+AA58
UNICODE_Nd, //U+AA59
0, //U+AA5A
0, //U+AA5B
UNICODE_Po, //U+AA5C
UNICODE_Po, //U+AA5D
UNICODE_Po, //U+AA5E
UNICODE_Po, //U+AA5F
UNICODE_Lo, //U+AA60
UNICODE_Lo, //U+AA61
UNICODE_Lo, //U+AA62
UNICODE_Lo, //U+AA63
UNICODE_Lo, //U+AA64
UNICODE_Lo, //U+AA65
UNICODE_Lo, //U+AA66
UNICODE_Lo, //U+AA67
UNICODE_Lo, //U+AA68
UNICODE_Lo, //U+AA69
UNICODE_Lo, //U+AA6A
UNICODE_Lo, //U+AA6B
UNICODE_Lo, //U+AA6C
UNICODE_Lo, //U+AA6D
UNICODE_Lo, //U+AA6E
UNICODE_Lo, //U+AA6F
UNICODE_Lm, //U+AA70
UNICODE_Lo, //U+AA71
UNICODE_Lo, //U+AA72
UNICODE_Lo, //U+AA73
UNICODE_Lo, //U+AA74
UNICODE_Lo, //U+AA75
UNICODE_Lo, //U+AA76
UNICODE_So, //U+AA77
UNICODE_So, //U+AA78
UNICODE_So, //U+AA79
UNICODE_Lo, //U+AA7A
UNICODE_Mc, //U+AA7B
0, //U+AA7C
0, //U+AA7D
0, //U+AA7E
0, //U+AA7F
UNICODE_Lo, //U+AA80
UNICODE_Lo, //U+AA81
UNICODE_Lo, //U+AA82
UNICODE_Lo, //U+AA83
UNICODE_Lo, //U+AA84
UNICODE_Lo, //U+AA85
UNICODE_Lo, //U+AA86
UNICODE_Lo, //U+AA87
UNICODE_Lo, //U+AA88
UNICODE_Lo, //U+AA89
UNICODE_Lo, //U+AA8A
UNICODE_Lo, //U+AA8B
UNICODE_Lo, //U+AA8C
UNICODE_Lo, //U+AA8D
UNICODE_Lo, //U+AA8E
UNICODE_Lo, //U+AA8F
UNICODE_Lo, //U+AA90
UNICODE_Lo, //U+AA91
UNICODE_Lo, //U+AA92
UNICODE_Lo, //U+AA93
UNICODE_Lo, //U+AA94
UNICODE_Lo, //U+AA95
UNICODE_Lo, //U+AA96
UNICODE_Lo, //U+AA97
UNICODE_Lo, //U+AA98
UNICODE_Lo, //U+AA99
UNICODE_Lo, //U+AA9A
UNICODE_Lo, //U+AA9B
UNICODE_Lo, //U+AA9C
UNICODE_Lo, //U+AA9D
UNICODE_Lo, //U+AA9E
UNICODE_Lo, //U+AA9F
UNICODE_Lo, //U+AAA0
UNICODE_Lo, //U+AAA1
UNICODE_Lo, //U+AAA2
UNICODE_Lo, //U+AAA3
UNICODE_Lo, //U+AAA4
UNICODE_Lo, //U+AAA5
UNICODE_Lo, //U+AAA6
UNICODE_Lo, //U+AAA7
UNICODE_Lo, //U+AAA8
UNICODE_Lo, //U+AAA9
UNICODE_Lo, //U+AAAA
UNICODE_Lo, //U+AAAB
UNICODE_Lo, //U+AAAC
UNICODE_Lo, //U+AAAD
UNICODE_Lo, //U+AAAE
UNICODE_Lo, //U+AAAF
UNICODE_Mn, //U+AAB0
UNICODE_Lo, //U+AAB1
UNICODE_Mn, //U+AAB2
UNICODE_Mn, //U+AAB3
UNICODE_Mn, //U+AAB4
UNICODE_Lo, //U+AAB5
UNICODE_Lo, //U+AAB6
UNICODE_Mn, //U+AAB7
UNICODE_Mn, //U+AAB8
UNICODE_Lo, //U+AAB9
UNICODE_Lo, //U+AABA
UNICODE_Lo, //U+AABB
UNICODE_Lo, //U+AABC
UNICODE_Lo, //U+AABD
UNICODE_Mn, //U+AABE
UNICODE_Mn, //U+AABF
UNICODE_Lo, //U+AAC0
UNICODE_Mn, //U+AAC1
UNICODE_Lo, //U+AAC2
0, //U+AAC3
0, //U+AAC4
0, //U+AAC5
0, //U+AAC6
0, //U+AAC7
0, //U+AAC8
0, //U+AAC9
0, //U+AACA
0, //U+AACB
0, //U+AACC
0, //U+AACD
0, //U+AACE
0, //U+AACF
0, //U+AAD0
0, //U+AAD1
0, //U+AAD2
0, //U+AAD3
0, //U+AAD4
0, //U+AAD5
0, //U+AAD6
0, //U+AAD7
0, //U+AAD8
0, //U+AAD9
0, //U+AADA
UNICODE_Lo, //U+AADB
UNICODE_Lo, //U+AADC
UNICODE_Lm, //U+AADD
UNICODE_Po, //U+AADE
UNICODE_Po, //U+AADF
0, //U+AAE0
0, //U+AAE1
0, //U+AAE2
0, //U+AAE3
0, //U+AAE4
0, //U+AAE5
0, //U+AAE6
0, //U+AAE7
0, //U+AAE8
0, //U+AAE9
0, //U+AAEA
0, //U+AAEB
0, //U+AAEC
0, //U+AAED
0, //U+AAEE
0, //U+AAEF
0, //U+AAF0
0, //U+AAF1
0, //U+AAF2
0, //U+AAF3
0, //U+AAF4
0, //U+AAF5
0, //U+AAF6
0, //U+AAF7
0, //U+AAF8
0, //U+AAF9
0, //U+AAFA
0, //U+AAFB
0, //U+AAFC
0, //U+AAFD
0, //U+AAFE
0, //U+AAFF
0, //U+AB00
UNICODE_Lo, //U+AB01
UNICODE_Lo, //U+AB02
UNICODE_Lo, //U+AB03
UNICODE_Lo, //U+AB04
UNICODE_Lo, //U+AB05
UNICODE_Lo, //U+AB06
0, //U+AB07
0, //U+AB08
UNICODE_Lo, //U+AB09
UNICODE_Lo, //U+AB0A
UNICODE_Lo, //U+AB0B
UNICODE_Lo, //U+AB0C
UNICODE_Lo, //U+AB0D
UNICODE_Lo, //U+AB0E
0, //U+AB0F
0, //U+AB10
UNICODE_Lo, //U+AB11
UNICODE_Lo, //U+AB12
UNICODE_Lo, //U+AB13
UNICODE_Lo, //U+AB14
UNICODE_Lo, //U+AB15
UNICODE_Lo, //U+AB16
0, //U+AB17
0, //U+AB18
0, //U+AB19
0, //U+AB1A
0, //U+AB1B
0, //U+AB1C
0, //U+AB1D
0, //U+AB1E
0, //U+AB1F
UNICODE_Lo, //U+AB20
UNICODE_Lo, //U+AB21
UNICODE_Lo, //U+AB22
UNICODE_Lo, //U+AB23
UNICODE_Lo, //U+AB24
UNICODE_Lo, //U+AB25
UNICODE_Lo, //U+AB26
0, //U+AB27
UNICODE_Lo, //U+AB28
UNICODE_Lo, //U+AB29
UNICODE_Lo, //U+AB2A
UNICODE_Lo, //U+AB2B
UNICODE_Lo, //U+AB2C
UNICODE_Lo, //U+AB2D
UNICODE_Lo, //U+AB2E
0, //U+AB2F
0, //U+AB30
0, //U+AB31
0, //U+AB32
0, //U+AB33
0, //U+AB34
0, //U+AB35
0, //U+AB36
0, //U+AB37
0, //U+AB38
0, //U+AB39
0, //U+AB3A
0, //U+AB3B
0, //U+AB3C
0, //U+AB3D
0, //U+AB3E
0, //U+AB3F
0, //U+AB40
0, //U+AB41
0, //U+AB42
0, //U+AB43
0, //U+AB44
0, //U+AB45
0, //U+AB46
0, //U+AB47
0, //U+AB48
0, //U+AB49
0, //U+AB4A
0, //U+AB4B
0, //U+AB4C
0, //U+AB4D
0, //U+AB4E
0, //U+AB4F
0, //U+AB50
0, //U+AB51
0, //U+AB52
0, //U+AB53
0, //U+AB54
0, //U+AB55
0, //U+AB56
0, //U+AB57
0, //U+AB58
0, //U+AB59
0, //U+AB5A
0, //U+AB5B
0, //U+AB5C
0, //U+AB5D
0, //U+AB5E
0, //U+AB5F
0, //U+AB60
0, //U+AB61
0, //U+AB62
0, //U+AB63
0, //U+AB64
0, //U+AB65
0, //U+AB66
0, //U+AB67
0, //U+AB68
0, //U+AB69
0, //U+AB6A
0, //U+AB6B
0, //U+AB6C
0, //U+AB6D
0, //U+AB6E
0, //U+AB6F
0, //U+AB70
0, //U+AB71
0, //U+AB72
0, //U+AB73
0, //U+AB74
0, //U+AB75
0, //U+AB76
0, //U+AB77
0, //U+AB78
0, //U+AB79
0, //U+AB7A
0, //U+AB7B
0, //U+AB7C
0, //U+AB7D
0, //U+AB7E
0, //U+AB7F
0, //U+AB80
0, //U+AB81
0, //U+AB82
0, //U+AB83
0, //U+AB84
0, //U+AB85
0, //U+AB86
0, //U+AB87
0, //U+AB88
0, //U+AB89
0, //U+AB8A
0, //U+AB8B
0, //U+AB8C
0, //U+AB8D
0, //U+AB8E
0, //U+AB8F
0, //U+AB90
0, //U+AB91
0, //U+AB92
0, //U+AB93
0, //U+AB94
0, //U+AB95
0, //U+AB96
0, //U+AB97
0, //U+AB98
0, //U+AB99
0, //U+AB9A
0, //U+AB9B
0, //U+AB9C
0, //U+AB9D
0, //U+AB9E
0, //U+AB9F
0, //U+ABA0
0, //U+ABA1
0, //U+ABA2
0, //U+ABA3
0, //U+ABA4
0, //U+ABA5
0, //U+ABA6
0, //U+ABA7
0, //U+ABA8
0, //U+ABA9
0, //U+ABAA
0, //U+ABAB
0, //U+ABAC
0, //U+ABAD
0, //U+ABAE
0, //U+ABAF
0, //U+ABB0
0, //U+ABB1
0, //U+ABB2
0, //U+ABB3
0, //U+ABB4
0, //U+ABB5
0, //U+ABB6
0, //U+ABB7
0, //U+ABB8
0, //U+ABB9
0, //U+ABBA
0, //U+ABBB
0, //U+ABBC
0, //U+ABBD
0, //U+ABBE
0, //U+ABBF
UNICODE_Lo, //U+ABC0
UNICODE_Lo, //U+ABC1
UNICODE_Lo, //U+ABC2
UNICODE_Lo, //U+ABC3
UNICODE_Lo, //U+ABC4
UNICODE_Lo, //U+ABC5
UNICODE_Lo, //U+ABC6
UNICODE_Lo, //U+ABC7
UNICODE_Lo, //U+ABC8
UNICODE_Lo, //U+ABC9
UNICODE_Lo, //U+ABCA
UNICODE_Lo, //U+ABCB
UNICODE_Lo, //U+ABCC
UNICODE_Lo, //U+ABCD
UNICODE_Lo, //U+ABCE
UNICODE_Lo, //U+ABCF
UNICODE_Lo, //U+ABD0
UNICODE_Lo, //U+ABD1
UNICODE_Lo, //U+ABD2
UNICODE_Lo, //U+ABD3
UNICODE_Lo, //U+ABD4
UNICODE_Lo, //U+ABD5
UNICODE_Lo, //U+ABD6
UNICODE_Lo, //U+ABD7
UNICODE_Lo, //U+ABD8
UNICODE_Lo, //U+ABD9
UNICODE_Lo, //U+ABDA
UNICODE_Lo, //U+ABDB
UNICODE_Lo, //U+ABDC
UNICODE_Lo, //U+ABDD
UNICODE_Lo, //U+ABDE
UNICODE_Lo, //U+ABDF
UNICODE_Lo, //U+ABE0
UNICODE_Lo, //U+ABE1
UNICODE_Lo, //U+ABE2
UNICODE_Mc, //U+ABE3
UNICODE_Mc, //U+ABE4
UNICODE_Mn, //U+ABE5
UNICODE_Mc, //U+ABE6
UNICODE_Mc, //U+ABE7
UNICODE_Mn, //U+ABE8
UNICODE_Mc, //U+ABE9
UNICODE_Mc, //U+ABEA
UNICODE_Po, //U+ABEB
UNICODE_Mc, //U+ABEC
UNICODE_Mn, //U+ABED
0, //U+ABEE
0, //U+ABEF
UNICODE_Nd, //U+ABF0
UNICODE_Nd, //U+ABF1
UNICODE_Nd, //U+ABF2
UNICODE_Nd, //U+ABF3
UNICODE_Nd, //U+ABF4
UNICODE_Nd, //U+ABF5
UNICODE_Nd, //U+ABF6
UNICODE_Nd, //U+ABF7
UNICODE_Nd, //U+ABF8
UNICODE_Nd, //U+ABF9
0, //U+ABFA
0, //U+ABFB
0, //U+ABFC
0, //U+ABFD
0, //U+ABFE
0, //U+ABFF
UNICODE_Lo, //U+AC00
0, //U+AC01
0, //U+AC02
0, //U+AC03
0, //U+AC04
0, //U+AC05
0, //U+AC06
0, //U+AC07
0, //U+AC08
0, //U+AC09
0, //U+AC0A
0, //U+AC0B
0, //U+AC0C
0, //U+AC0D
0, //U+AC0E
0, //U+AC0F
0, //U+AC10
0, //U+AC11
0, //U+AC12
0, //U+AC13
0, //U+AC14
0, //U+AC15
0, //U+AC16
0, //U+AC17
0, //U+AC18
0, //U+AC19
0, //U+AC1A
0, //U+AC1B
0, //U+AC1C
0, //U+AC1D
0, //U+AC1E
0, //U+AC1F
0, //U+AC20
0, //U+AC21
0, //U+AC22
0, //U+AC23
0, //U+AC24
0, //U+AC25
0, //U+AC26
0, //U+AC27
0, //U+AC28
0, //U+AC29
0, //U+AC2A
0, //U+AC2B
0, //U+AC2C
0, //U+AC2D
0, //U+AC2E
0, //U+AC2F
0, //U+AC30
0, //U+AC31
0, //U+AC32
0, //U+AC33
0, //U+AC34
0, //U+AC35
0, //U+AC36
0, //U+AC37
0, //U+AC38
0, //U+AC39
0, //U+AC3A
0, //U+AC3B
0, //U+AC3C
0, //U+AC3D
0, //U+AC3E
0, //U+AC3F
0, //U+AC40
0, //U+AC41
0, //U+AC42
0, //U+AC43
0, //U+AC44
0, //U+AC45
0, //U+AC46
0, //U+AC47
0, //U+AC48
0, //U+AC49
0, //U+AC4A
0, //U+AC4B
0, //U+AC4C
0, //U+AC4D
0, //U+AC4E
0, //U+AC4F
0, //U+AC50
0, //U+AC51
0, //U+AC52
0, //U+AC53
0, //U+AC54
0, //U+AC55
0, //U+AC56
0, //U+AC57
0, //U+AC58
0, //U+AC59
0, //U+AC5A
0, //U+AC5B
0, //U+AC5C
0, //U+AC5D
0, //U+AC5E
0, //U+AC5F
0, //U+AC60
0, //U+AC61
0, //U+AC62
0, //U+AC63
0, //U+AC64
0, //U+AC65
0, //U+AC66
0, //U+AC67
0, //U+AC68
0, //U+AC69
0, //U+AC6A
0, //U+AC6B
0, //U+AC6C
0, //U+AC6D
0, //U+AC6E
0, //U+AC6F
0, //U+AC70
0, //U+AC71
0, //U+AC72
0, //U+AC73
0, //U+AC74
0, //U+AC75
0, //U+AC76
0, //U+AC77
0, //U+AC78
0, //U+AC79
0, //U+AC7A
0, //U+AC7B
0, //U+AC7C
0, //U+AC7D
0, //U+AC7E
0, //U+AC7F
0, //U+AC80
0, //U+AC81
0, //U+AC82
0, //U+AC83
0, //U+AC84
0, //U+AC85
0, //U+AC86
0, //U+AC87
0, //U+AC88
0, //U+AC89
0, //U+AC8A
0, //U+AC8B
0, //U+AC8C
0, //U+AC8D
0, //U+AC8E
0, //U+AC8F
0, //U+AC90
0, //U+AC91
0, //U+AC92
0, //U+AC93
0, //U+AC94
0, //U+AC95
0, //U+AC96
0, //U+AC97
0, //U+AC98
0, //U+AC99
0, //U+AC9A
0, //U+AC9B
0, //U+AC9C
0, //U+AC9D
0, //U+AC9E
0, //U+AC9F
0, //U+ACA0
0, //U+ACA1
0, //U+ACA2
0, //U+ACA3
0, //U+ACA4
0, //U+ACA5
0, //U+ACA6
0, //U+ACA7
0, //U+ACA8
0, //U+ACA9
0, //U+ACAA
0, //U+ACAB
0, //U+ACAC
0, //U+ACAD
0, //U+ACAE
0, //U+ACAF
0, //U+ACB0
0, //U+ACB1
0, //U+ACB2
0, //U+ACB3
0, //U+ACB4
0, //U+ACB5
0, //U+ACB6
0, //U+ACB7
0, //U+ACB8
0, //U+ACB9
0, //U+ACBA
0, //U+ACBB
0, //U+ACBC
0, //U+ACBD
0, //U+ACBE
0, //U+ACBF
0, //U+ACC0
0, //U+ACC1
0, //U+ACC2
0, //U+ACC3
0, //U+ACC4
0, //U+ACC5
0, //U+ACC6
0, //U+ACC7
0, //U+ACC8
0, //U+ACC9
0, //U+ACCA
0, //U+ACCB
0, //U+ACCC
0, //U+ACCD
0, //U+ACCE
0, //U+ACCF
0, //U+ACD0
0, //U+ACD1
0, //U+ACD2
0, //U+ACD3
0, //U+ACD4
0, //U+ACD5
0, //U+ACD6
0, //U+ACD7
0, //U+ACD8
0, //U+ACD9
0, //U+ACDA
0, //U+ACDB
0, //U+ACDC
0, //U+ACDD
0, //U+ACDE
0, //U+ACDF
0, //U+ACE0
0, //U+ACE1
0, //U+ACE2
0, //U+ACE3
0, //U+ACE4
0, //U+ACE5
0, //U+ACE6
0, //U+ACE7
0, //U+ACE8
0, //U+ACE9
0, //U+ACEA
0, //U+ACEB
0, //U+ACEC
0, //U+ACED
0, //U+ACEE
0, //U+ACEF
0, //U+ACF0
0, //U+ACF1
0, //U+ACF2
0, //U+ACF3
0, //U+ACF4
0, //U+ACF5
0, //U+ACF6
0, //U+ACF7
0, //U+ACF8
0, //U+ACF9
0, //U+ACFA
0, //U+ACFB
0, //U+ACFC
0, //U+ACFD
0, //U+ACFE
0, //U+ACFF
0, //U+AD00
0, //U+AD01
0, //U+AD02
0, //U+AD03
0, //U+AD04
0, //U+AD05
0, //U+AD06
0, //U+AD07
0, //U+AD08
0, //U+AD09
0, //U+AD0A
0, //U+AD0B
0, //U+AD0C
0, //U+AD0D
0, //U+AD0E
0, //U+AD0F
0, //U+AD10
0, //U+AD11
0, //U+AD12
0, //U+AD13
0, //U+AD14
0, //U+AD15
0, //U+AD16
0, //U+AD17
0, //U+AD18
0, //U+AD19
0, //U+AD1A
0, //U+AD1B
0, //U+AD1C
0, //U+AD1D
0, //U+AD1E
0, //U+AD1F
0, //U+AD20
0, //U+AD21
0, //U+AD22
0, //U+AD23
0, //U+AD24
0, //U+AD25
0, //U+AD26
0, //U+AD27
0, //U+AD28
0, //U+AD29
0, //U+AD2A
0, //U+AD2B
0, //U+AD2C
0, //U+AD2D
0, //U+AD2E
0, //U+AD2F
0, //U+AD30
0, //U+AD31
0, //U+AD32
0, //U+AD33
0, //U+AD34
0, //U+AD35
0, //U+AD36
0, //U+AD37
0, //U+AD38
0, //U+AD39
0, //U+AD3A
0, //U+AD3B
0, //U+AD3C
0, //U+AD3D
0, //U+AD3E
0, //U+AD3F
0, //U+AD40
0, //U+AD41
0, //U+AD42
0, //U+AD43
0, //U+AD44
0, //U+AD45
0, //U+AD46
0, //U+AD47
0, //U+AD48
0, //U+AD49
0, //U+AD4A
0, //U+AD4B
0, //U+AD4C
0, //U+AD4D
0, //U+AD4E
0, //U+AD4F
0, //U+AD50
0, //U+AD51
0, //U+AD52
0, //U+AD53
0, //U+AD54
0, //U+AD55
0, //U+AD56
0, //U+AD57
0, //U+AD58
0, //U+AD59
0, //U+AD5A
0, //U+AD5B
0, //U+AD5C
0, //U+AD5D
0, //U+AD5E
0, //U+AD5F
0, //U+AD60
0, //U+AD61
0, //U+AD62
0, //U+AD63
0, //U+AD64
0, //U+AD65
0, //U+AD66
0, //U+AD67
0, //U+AD68
0, //U+AD69
0, //U+AD6A
0, //U+AD6B
0, //U+AD6C
0, //U+AD6D
0, //U+AD6E
0, //U+AD6F
0, //U+AD70
0, //U+AD71
0, //U+AD72
0, //U+AD73
0, //U+AD74
0, //U+AD75
0, //U+AD76
0, //U+AD77
0, //U+AD78
0, //U+AD79
0, //U+AD7A
0, //U+AD7B
0, //U+AD7C
0, //U+AD7D
0, //U+AD7E
0, //U+AD7F
0, //U+AD80
0, //U+AD81
0, //U+AD82
0, //U+AD83
0, //U+AD84
0, //U+AD85
0, //U+AD86
0, //U+AD87
0, //U+AD88
0, //U+AD89
0, //U+AD8A
0, //U+AD8B
0, //U+AD8C
0, //U+AD8D
0, //U+AD8E
0, //U+AD8F
0, //U+AD90
0, //U+AD91
0, //U+AD92
0, //U+AD93
0, //U+AD94
0, //U+AD95
0, //U+AD96
0, //U+AD97
0, //U+AD98
0, //U+AD99
0, //U+AD9A
0, //U+AD9B
0, //U+AD9C
0, //U+AD9D
0, //U+AD9E
0, //U+AD9F
0, //U+ADA0
0, //U+ADA1
0, //U+ADA2
0, //U+ADA3
0, //U+ADA4
0, //U+ADA5
0, //U+ADA6
0, //U+ADA7
0, //U+ADA8
0, //U+ADA9
0, //U+ADAA
0, //U+ADAB
0, //U+ADAC
0, //U+ADAD
0, //U+ADAE
0, //U+ADAF
0, //U+ADB0
0, //U+ADB1
0, //U+ADB2
0, //U+ADB3
0, //U+ADB4
0, //U+ADB5
0, //U+ADB6
0, //U+ADB7
0, //U+ADB8
0, //U+ADB9
0, //U+ADBA
0, //U+ADBB
0, //U+ADBC
0, //U+ADBD
0, //U+ADBE
0, //U+ADBF
0, //U+ADC0
0, //U+ADC1
0, //U+ADC2
0, //U+ADC3
0, //U+ADC4
0, //U+ADC5
0, //U+ADC6
0, //U+ADC7
0, //U+ADC8
0, //U+ADC9
0, //U+ADCA
0, //U+ADCB
0, //U+ADCC
0, //U+ADCD
0, //U+ADCE
0, //U+ADCF
0, //U+ADD0
0, //U+ADD1
0, //U+ADD2
0, //U+ADD3
0, //U+ADD4
0, //U+ADD5
0, //U+ADD6
0, //U+ADD7
0, //U+ADD8
0, //U+ADD9
0, //U+ADDA
0, //U+ADDB
0, //U+ADDC
0, //U+ADDD
0, //U+ADDE
0, //U+ADDF
0, //U+ADE0
0, //U+ADE1
0, //U+ADE2
0, //U+ADE3
0, //U+ADE4
0, //U+ADE5
0, //U+ADE6
0, //U+ADE7
0, //U+ADE8
0, //U+ADE9
0, //U+ADEA
0, //U+ADEB
0, //U+ADEC
0, //U+ADED
0, //U+ADEE
0, //U+ADEF
0, //U+ADF0
0, //U+ADF1
0, //U+ADF2
0, //U+ADF3
0, //U+ADF4
0, //U+ADF5
0, //U+ADF6
0, //U+ADF7
0, //U+ADF8
0, //U+ADF9
0, //U+ADFA
0, //U+ADFB
0, //U+ADFC
0, //U+ADFD
0, //U+ADFE
0, //U+ADFF
0, //U+AE00
0, //U+AE01
0, //U+AE02
0, //U+AE03
0, //U+AE04
0, //U+AE05
0, //U+AE06
0, //U+AE07
0, //U+AE08
0, //U+AE09
0, //U+AE0A
0, //U+AE0B
0, //U+AE0C
0, //U+AE0D
0, //U+AE0E
0, //U+AE0F
0, //U+AE10
0, //U+AE11
0, //U+AE12
0, //U+AE13
0, //U+AE14
0, //U+AE15
0, //U+AE16
0, //U+AE17
0, //U+AE18
0, //U+AE19
0, //U+AE1A
0, //U+AE1B
0, //U+AE1C
0, //U+AE1D
0, //U+AE1E
0, //U+AE1F
0, //U+AE20
0, //U+AE21
0, //U+AE22
0, //U+AE23
0, //U+AE24
0, //U+AE25
0, //U+AE26
0, //U+AE27
0, //U+AE28
0, //U+AE29
0, //U+AE2A
0, //U+AE2B
0, //U+AE2C
0, //U+AE2D
0, //U+AE2E
0, //U+AE2F
0, //U+AE30
0, //U+AE31
0, //U+AE32
0, //U+AE33
0, //U+AE34
0, //U+AE35
0, //U+AE36
0, //U+AE37
0, //U+AE38
0, //U+AE39
0, //U+AE3A
0, //U+AE3B
0, //U+AE3C
0, //U+AE3D
0, //U+AE3E
0, //U+AE3F
0, //U+AE40
0, //U+AE41
0, //U+AE42
0, //U+AE43
0, //U+AE44
0, //U+AE45
0, //U+AE46
0, //U+AE47
0, //U+AE48
0, //U+AE49
0, //U+AE4A
0, //U+AE4B
0, //U+AE4C
0, //U+AE4D
0, //U+AE4E
0, //U+AE4F
0, //U+AE50
0, //U+AE51
0, //U+AE52
0, //U+AE53
0, //U+AE54
0, //U+AE55
0, //U+AE56
0, //U+AE57
0, //U+AE58
0, //U+AE59
0, //U+AE5A
0, //U+AE5B
0, //U+AE5C
0, //U+AE5D
0, //U+AE5E
0, //U+AE5F
0, //U+AE60
0, //U+AE61
0, //U+AE62
0, //U+AE63
0, //U+AE64
0, //U+AE65
0, //U+AE66
0, //U+AE67
0, //U+AE68
0, //U+AE69
0, //U+AE6A
0, //U+AE6B
0, //U+AE6C
0, //U+AE6D
0, //U+AE6E
0, //U+AE6F
0, //U+AE70
0, //U+AE71
0, //U+AE72
0, //U+AE73
0, //U+AE74
0, //U+AE75
0, //U+AE76
0, //U+AE77
0, //U+AE78
0, //U+AE79
0, //U+AE7A
0, //U+AE7B
0, //U+AE7C
0, //U+AE7D
0, //U+AE7E
0, //U+AE7F
0, //U+AE80
0, //U+AE81
0, //U+AE82
0, //U+AE83
0, //U+AE84
0, //U+AE85
0, //U+AE86
0, //U+AE87
0, //U+AE88
0, //U+AE89
0, //U+AE8A
0, //U+AE8B
0, //U+AE8C
0, //U+AE8D
0, //U+AE8E
0, //U+AE8F
0, //U+AE90
0, //U+AE91
0, //U+AE92
0, //U+AE93
0, //U+AE94
0, //U+AE95
0, //U+AE96
0, //U+AE97
0, //U+AE98
0, //U+AE99
0, //U+AE9A
0, //U+AE9B
0, //U+AE9C
0, //U+AE9D
0, //U+AE9E
0, //U+AE9F
0, //U+AEA0
0, //U+AEA1
0, //U+AEA2
0, //U+AEA3
0, //U+AEA4
0, //U+AEA5
0, //U+AEA6
0, //U+AEA7
0, //U+AEA8
0, //U+AEA9
0, //U+AEAA
0, //U+AEAB
0, //U+AEAC
0, //U+AEAD
0, //U+AEAE
0, //U+AEAF
0, //U+AEB0
0, //U+AEB1
0, //U+AEB2
0, //U+AEB3
0, //U+AEB4
0, //U+AEB5
0, //U+AEB6
0, //U+AEB7
0, //U+AEB8
0, //U+AEB9
0, //U+AEBA
0, //U+AEBB
0, //U+AEBC
0, //U+AEBD
0, //U+AEBE
0, //U+AEBF
0, //U+AEC0
0, //U+AEC1
0, //U+AEC2
0, //U+AEC3
0, //U+AEC4
0, //U+AEC5
0, //U+AEC6
0, //U+AEC7
0, //U+AEC8
0, //U+AEC9
0, //U+AECA
0, //U+AECB
0, //U+AECC
0, //U+AECD
0, //U+AECE
0, //U+AECF
0, //U+AED0
0, //U+AED1
0, //U+AED2
0, //U+AED3
0, //U+AED4
0, //U+AED5
0, //U+AED6
0, //U+AED7
0, //U+AED8
0, //U+AED9
0, //U+AEDA
0, //U+AEDB
0, //U+AEDC
0, //U+AEDD
0, //U+AEDE
0, //U+AEDF
0, //U+AEE0
0, //U+AEE1
0, //U+AEE2
0, //U+AEE3
0, //U+AEE4
0, //U+AEE5
0, //U+AEE6
0, //U+AEE7
0, //U+AEE8
0, //U+AEE9
0, //U+AEEA
0, //U+AEEB
0, //U+AEEC
0, //U+AEED
0, //U+AEEE
0, //U+AEEF
0, //U+AEF0
0, //U+AEF1
0, //U+AEF2
0, //U+AEF3
0, //U+AEF4
0, //U+AEF5
0, //U+AEF6
0, //U+AEF7
0, //U+AEF8
0, //U+AEF9
0, //U+AEFA
0, //U+AEFB
0, //U+AEFC
0, //U+AEFD
0, //U+AEFE
0, //U+AEFF
0, //U+AF00
0, //U+AF01
0, //U+AF02
0, //U+AF03
0, //U+AF04
0, //U+AF05
0, //U+AF06
0, //U+AF07
0, //U+AF08
0, //U+AF09
0, //U+AF0A
0, //U+AF0B
0, //U+AF0C
0, //U+AF0D
0, //U+AF0E
0, //U+AF0F
0, //U+AF10
0, //U+AF11
0, //U+AF12
0, //U+AF13
0, //U+AF14
0, //U+AF15
0, //U+AF16
0, //U+AF17
0, //U+AF18
0, //U+AF19
0, //U+AF1A
0, //U+AF1B
0, //U+AF1C
0, //U+AF1D
0, //U+AF1E
0, //U+AF1F
0, //U+AF20
0, //U+AF21
0, //U+AF22
0, //U+AF23
0, //U+AF24
0, //U+AF25
0, //U+AF26
0, //U+AF27
0, //U+AF28
0, //U+AF29
0, //U+AF2A
0, //U+AF2B
0, //U+AF2C
0, //U+AF2D
0, //U+AF2E
0, //U+AF2F
0, //U+AF30
0, //U+AF31
0, //U+AF32
0, //U+AF33
0, //U+AF34
0, //U+AF35
0, //U+AF36
0, //U+AF37
0, //U+AF38
0, //U+AF39
0, //U+AF3A
0, //U+AF3B
0, //U+AF3C
0, //U+AF3D
0, //U+AF3E
0, //U+AF3F
0, //U+AF40
0, //U+AF41
0, //U+AF42
0, //U+AF43
0, //U+AF44
0, //U+AF45
0, //U+AF46
0, //U+AF47
0, //U+AF48
0, //U+AF49
0, //U+AF4A
0, //U+AF4B
0, //U+AF4C
0, //U+AF4D
0, //U+AF4E
0, //U+AF4F
0, //U+AF50
0, //U+AF51
0, //U+AF52
0, //U+AF53
0, //U+AF54
0, //U+AF55
0, //U+AF56
0, //U+AF57
0, //U+AF58
0, //U+AF59
0, //U+AF5A
0, //U+AF5B
0, //U+AF5C
0, //U+AF5D
0, //U+AF5E
0, //U+AF5F
0, //U+AF60
0, //U+AF61
0, //U+AF62
0, //U+AF63
0, //U+AF64
0, //U+AF65
0, //U+AF66
0, //U+AF67
0, //U+AF68
0, //U+AF69
0, //U+AF6A
0, //U+AF6B
0, //U+AF6C
0, //U+AF6D
0, //U+AF6E
0, //U+AF6F
0, //U+AF70
0, //U+AF71
0, //U+AF72
0, //U+AF73
0, //U+AF74
0, //U+AF75
0, //U+AF76
0, //U+AF77
0, //U+AF78
0, //U+AF79
0, //U+AF7A
0, //U+AF7B
0, //U+AF7C
0, //U+AF7D
0, //U+AF7E
0, //U+AF7F
0, //U+AF80
0, //U+AF81
0, //U+AF82
0, //U+AF83
0, //U+AF84
0, //U+AF85
0, //U+AF86
0, //U+AF87
0, //U+AF88
0, //U+AF89
0, //U+AF8A
0, //U+AF8B
0, //U+AF8C
0, //U+AF8D
0, //U+AF8E
0, //U+AF8F
0, //U+AF90
0, //U+AF91
0, //U+AF92
0, //U+AF93
0, //U+AF94
0, //U+AF95
0, //U+AF96
0, //U+AF97
0, //U+AF98
0, //U+AF99
0, //U+AF9A
0, //U+AF9B
0, //U+AF9C
0, //U+AF9D
0, //U+AF9E
0, //U+AF9F
0, //U+AFA0
0, //U+AFA1
0, //U+AFA2
0, //U+AFA3
0, //U+AFA4
0, //U+AFA5
0, //U+AFA6
0, //U+AFA7
0, //U+AFA8
0, //U+AFA9
0, //U+AFAA
0, //U+AFAB
0, //U+AFAC
0, //U+AFAD
0, //U+AFAE
0, //U+AFAF
0, //U+AFB0
0, //U+AFB1
0, //U+AFB2
0, //U+AFB3
0, //U+AFB4
0, //U+AFB5
0, //U+AFB6
0, //U+AFB7
0, //U+AFB8
0, //U+AFB9
0, //U+AFBA
0, //U+AFBB
0, //U+AFBC
0, //U+AFBD
0, //U+AFBE
0, //U+AFBF
0, //U+AFC0
0, //U+AFC1
0, //U+AFC2
0, //U+AFC3
0, //U+AFC4
0, //U+AFC5
0, //U+AFC6
0, //U+AFC7
0, //U+AFC8
0, //U+AFC9
0, //U+AFCA
0, //U+AFCB
0, //U+AFCC
0, //U+AFCD
0, //U+AFCE
0, //U+AFCF
0, //U+AFD0
0, //U+AFD1
0, //U+AFD2
0, //U+AFD3
0, //U+AFD4
0, //U+AFD5
0, //U+AFD6
0, //U+AFD7
0, //U+AFD8
0, //U+AFD9
0, //U+AFDA
0, //U+AFDB
0, //U+AFDC
0, //U+AFDD
0, //U+AFDE
0, //U+AFDF
0, //U+AFE0
0, //U+AFE1
0, //U+AFE2
0, //U+AFE3
0, //U+AFE4
0, //U+AFE5
0, //U+AFE6
0, //U+AFE7
0, //U+AFE8
0, //U+AFE9
0, //U+AFEA
0, //U+AFEB
0, //U+AFEC
0, //U+AFED
0, //U+AFEE
0, //U+AFEF
0, //U+AFF0
0, //U+AFF1
0, //U+AFF2
0, //U+AFF3
0, //U+AFF4
0, //U+AFF5
0, //U+AFF6
0, //U+AFF7
0, //U+AFF8
0, //U+AFF9
0, //U+AFFA
0, //U+AFFB
0, //U+AFFC
0, //U+AFFD
0, //U+AFFE
0, //U+AFFF
0, //U+B000
0, //U+B001
0, //U+B002
0, //U+B003
0, //U+B004
0, //U+B005
0, //U+B006
0, //U+B007
0, //U+B008
0, //U+B009
0, //U+B00A
0, //U+B00B
0, //U+B00C
0, //U+B00D
0, //U+B00E
0, //U+B00F
0, //U+B010
0, //U+B011
0, //U+B012
0, //U+B013
0, //U+B014
0, //U+B015
0, //U+B016
0, //U+B017
0, //U+B018
0, //U+B019
0, //U+B01A
0, //U+B01B
0, //U+B01C
0, //U+B01D
0, //U+B01E
0, //U+B01F
0, //U+B020
0, //U+B021
0, //U+B022
0, //U+B023
0, //U+B024
0, //U+B025
0, //U+B026
0, //U+B027
0, //U+B028
0, //U+B029
0, //U+B02A
0, //U+B02B
0, //U+B02C
0, //U+B02D
0, //U+B02E
0, //U+B02F
0, //U+B030
0, //U+B031
0, //U+B032
0, //U+B033
0, //U+B034
0, //U+B035
0, //U+B036
0, //U+B037
0, //U+B038
0, //U+B039
0, //U+B03A
0, //U+B03B
0, //U+B03C
0, //U+B03D
0, //U+B03E
0, //U+B03F
0, //U+B040
0, //U+B041
0, //U+B042
0, //U+B043
0, //U+B044
0, //U+B045
0, //U+B046
0, //U+B047
0, //U+B048
0, //U+B049
0, //U+B04A
0, //U+B04B
0, //U+B04C
0, //U+B04D
0, //U+B04E
0, //U+B04F
0, //U+B050
0, //U+B051
0, //U+B052
0, //U+B053
0, //U+B054
0, //U+B055
0, //U+B056
0, //U+B057
0, //U+B058
0, //U+B059
0, //U+B05A
0, //U+B05B
0, //U+B05C
0, //U+B05D
0, //U+B05E
0, //U+B05F
0, //U+B060
0, //U+B061
0, //U+B062
0, //U+B063
0, //U+B064
0, //U+B065
0, //U+B066
0, //U+B067
0, //U+B068
0, //U+B069
0, //U+B06A
0, //U+B06B
0, //U+B06C
0, //U+B06D
0, //U+B06E
0, //U+B06F
0, //U+B070
0, //U+B071
0, //U+B072
0, //U+B073
0, //U+B074
0, //U+B075
0, //U+B076
0, //U+B077
0, //U+B078
0, //U+B079
0, //U+B07A
0, //U+B07B
0, //U+B07C
0, //U+B07D
0, //U+B07E
0, //U+B07F
0, //U+B080
0, //U+B081
0, //U+B082
0, //U+B083
0, //U+B084
0, //U+B085
0, //U+B086
0, //U+B087
0, //U+B088
0, //U+B089
0, //U+B08A
0, //U+B08B
0, //U+B08C
0, //U+B08D
0, //U+B08E
0, //U+B08F
0, //U+B090
0, //U+B091
0, //U+B092
0, //U+B093
0, //U+B094
0, //U+B095
0, //U+B096
0, //U+B097
0, //U+B098
0, //U+B099
0, //U+B09A
0, //U+B09B
0, //U+B09C
0, //U+B09D
0, //U+B09E
0, //U+B09F
0, //U+B0A0
0, //U+B0A1
0, //U+B0A2
0, //U+B0A3
0, //U+B0A4
0, //U+B0A5
0, //U+B0A6
0, //U+B0A7
0, //U+B0A8
0, //U+B0A9
0, //U+B0AA
0, //U+B0AB
0, //U+B0AC
0, //U+B0AD
0, //U+B0AE
0, //U+B0AF
0, //U+B0B0
0, //U+B0B1
0, //U+B0B2
0, //U+B0B3
0, //U+B0B4
0, //U+B0B5
0, //U+B0B6
0, //U+B0B7
0, //U+B0B8
0, //U+B0B9
0, //U+B0BA
0, //U+B0BB
0, //U+B0BC
0, //U+B0BD
0, //U+B0BE
0, //U+B0BF
0, //U+B0C0
0, //U+B0C1
0, //U+B0C2
0, //U+B0C3
0, //U+B0C4
0, //U+B0C5
0, //U+B0C6
0, //U+B0C7
0, //U+B0C8
0, //U+B0C9
0, //U+B0CA
0, //U+B0CB
0, //U+B0CC
0, //U+B0CD
0, //U+B0CE
0, //U+B0CF
0, //U+B0D0
0, //U+B0D1
0, //U+B0D2
0, //U+B0D3
0, //U+B0D4
0, //U+B0D5
0, //U+B0D6
0, //U+B0D7
0, //U+B0D8
0, //U+B0D9
0, //U+B0DA
0, //U+B0DB
0, //U+B0DC
0, //U+B0DD
0, //U+B0DE
0, //U+B0DF
0, //U+B0E0
0, //U+B0E1
0, //U+B0E2
0, //U+B0E3
0, //U+B0E4
0, //U+B0E5
0, //U+B0E6
0, //U+B0E7
0, //U+B0E8
0, //U+B0E9
0, //U+B0EA
0, //U+B0EB
0, //U+B0EC
0, //U+B0ED
0, //U+B0EE
0, //U+B0EF
0, //U+B0F0
0, //U+B0F1
0, //U+B0F2
0, //U+B0F3
0, //U+B0F4
0, //U+B0F5
0, //U+B0F6
0, //U+B0F7
0, //U+B0F8
0, //U+B0F9
0, //U+B0FA
0, //U+B0FB
0, //U+B0FC
0, //U+B0FD
0, //U+B0FE
0, //U+B0FF
0, //U+B100
0, //U+B101
0, //U+B102
0, //U+B103
0, //U+B104
0, //U+B105
0, //U+B106
0, //U+B107
0, //U+B108
0, //U+B109
0, //U+B10A
0, //U+B10B
0, //U+B10C
0, //U+B10D
0, //U+B10E
0, //U+B10F
0, //U+B110
0, //U+B111
0, //U+B112
0, //U+B113
0, //U+B114
0, //U+B115
0, //U+B116
0, //U+B117
0, //U+B118
0, //U+B119
0, //U+B11A
0, //U+B11B
0, //U+B11C
0, //U+B11D
0, //U+B11E
0, //U+B11F
0, //U+B120
0, //U+B121
0, //U+B122
0, //U+B123
0, //U+B124
0, //U+B125
0, //U+B126
0, //U+B127
0, //U+B128
0, //U+B129
0, //U+B12A
0, //U+B12B
0, //U+B12C
0, //U+B12D
0, //U+B12E
0, //U+B12F
0, //U+B130
0, //U+B131
0, //U+B132
0, //U+B133
0, //U+B134
0, //U+B135
0, //U+B136
0, //U+B137
0, //U+B138
0, //U+B139
0, //U+B13A
0, //U+B13B
0, //U+B13C
0, //U+B13D
0, //U+B13E
0, //U+B13F
0, //U+B140
0, //U+B141
0, //U+B142
0, //U+B143
0, //U+B144
0, //U+B145
0, //U+B146
0, //U+B147
0, //U+B148
0, //U+B149
0, //U+B14A
0, //U+B14B
0, //U+B14C
0, //U+B14D
0, //U+B14E
0, //U+B14F
0, //U+B150
0, //U+B151
0, //U+B152
0, //U+B153
0, //U+B154
0, //U+B155
0, //U+B156
0, //U+B157
0, //U+B158
0, //U+B159
0, //U+B15A
0, //U+B15B
0, //U+B15C
0, //U+B15D
0, //U+B15E
0, //U+B15F
0, //U+B160
0, //U+B161
0, //U+B162
0, //U+B163
0, //U+B164
0, //U+B165
0, //U+B166
0, //U+B167
0, //U+B168
0, //U+B169
0, //U+B16A
0, //U+B16B
0, //U+B16C
0, //U+B16D
0, //U+B16E
0, //U+B16F
0, //U+B170
0, //U+B171
0, //U+B172
0, //U+B173
0, //U+B174
0, //U+B175
0, //U+B176
0, //U+B177
0, //U+B178
0, //U+B179
0, //U+B17A
0, //U+B17B
0, //U+B17C
0, //U+B17D
0, //U+B17E
0, //U+B17F
0, //U+B180
0, //U+B181
0, //U+B182
0, //U+B183
0, //U+B184
0, //U+B185
0, //U+B186
0, //U+B187
0, //U+B188
0, //U+B189
0, //U+B18A
0, //U+B18B
0, //U+B18C
0, //U+B18D
0, //U+B18E
0, //U+B18F
0, //U+B190
0, //U+B191
0, //U+B192
0, //U+B193
0, //U+B194
0, //U+B195
0, //U+B196
0, //U+B197
0, //U+B198
0, //U+B199
0, //U+B19A
0, //U+B19B
0, //U+B19C
0, //U+B19D
0, //U+B19E
0, //U+B19F
0, //U+B1A0
0, //U+B1A1
0, //U+B1A2
0, //U+B1A3
0, //U+B1A4
0, //U+B1A5
0, //U+B1A6
0, //U+B1A7
0, //U+B1A8
0, //U+B1A9
0, //U+B1AA
0, //U+B1AB
0, //U+B1AC
0, //U+B1AD
0, //U+B1AE
0, //U+B1AF
0, //U+B1B0
0, //U+B1B1
0, //U+B1B2
0, //U+B1B3
0, //U+B1B4
0, //U+B1B5
0, //U+B1B6
0, //U+B1B7
0, //U+B1B8
0, //U+B1B9
0, //U+B1BA
0, //U+B1BB
0, //U+B1BC
0, //U+B1BD
0, //U+B1BE
0, //U+B1BF
0, //U+B1C0
0, //U+B1C1
0, //U+B1C2
0, //U+B1C3
0, //U+B1C4
0, //U+B1C5
0, //U+B1C6
0, //U+B1C7
0, //U+B1C8
0, //U+B1C9
0, //U+B1CA
0, //U+B1CB
0, //U+B1CC
0, //U+B1CD
0, //U+B1CE
0, //U+B1CF
0, //U+B1D0
0, //U+B1D1
0, //U+B1D2
0, //U+B1D3
0, //U+B1D4
0, //U+B1D5
0, //U+B1D6
0, //U+B1D7
0, //U+B1D8
0, //U+B1D9
0, //U+B1DA
0, //U+B1DB
0, //U+B1DC
0, //U+B1DD
0, //U+B1DE
0, //U+B1DF
0, //U+B1E0
0, //U+B1E1
0, //U+B1E2
0, //U+B1E3
0, //U+B1E4
0, //U+B1E5
0, //U+B1E6
0, //U+B1E7
0, //U+B1E8
0, //U+B1E9
0, //U+B1EA
0, //U+B1EB
0, //U+B1EC
0, //U+B1ED
0, //U+B1EE
0, //U+B1EF
0, //U+B1F0
0, //U+B1F1
0, //U+B1F2
0, //U+B1F3
0, //U+B1F4
0, //U+B1F5
0, //U+B1F6
0, //U+B1F7
0, //U+B1F8
0, //U+B1F9
0, //U+B1FA
0, //U+B1FB
0, //U+B1FC
0, //U+B1FD
0, //U+B1FE
0, //U+B1FF
0, //U+B200
0, //U+B201
0, //U+B202
0, //U+B203
0, //U+B204
0, //U+B205
0, //U+B206
0, //U+B207
0, //U+B208
0, //U+B209
0, //U+B20A
0, //U+B20B
0, //U+B20C
0, //U+B20D
0, //U+B20E
0, //U+B20F
0, //U+B210
0, //U+B211
0, //U+B212
0, //U+B213
0, //U+B214
0, //U+B215
0, //U+B216
0, //U+B217
0, //U+B218
0, //U+B219
0, //U+B21A
0, //U+B21B
0, //U+B21C
0, //U+B21D
0, //U+B21E
0, //U+B21F
0, //U+B220
0, //U+B221
0, //U+B222
0, //U+B223
0, //U+B224
0, //U+B225
0, //U+B226
0, //U+B227
0, //U+B228
0, //U+B229
0, //U+B22A
0, //U+B22B
0, //U+B22C
0, //U+B22D
0, //U+B22E
0, //U+B22F
0, //U+B230
0, //U+B231
0, //U+B232
0, //U+B233
0, //U+B234
0, //U+B235
0, //U+B236
0, //U+B237
0, //U+B238
0, //U+B239
0, //U+B23A
0, //U+B23B
0, //U+B23C
0, //U+B23D
0, //U+B23E
0, //U+B23F
0, //U+B240
0, //U+B241
0, //U+B242
0, //U+B243
0, //U+B244
0, //U+B245
0, //U+B246
0, //U+B247
0, //U+B248
0, //U+B249
0, //U+B24A
0, //U+B24B
0, //U+B24C
0, //U+B24D
0, //U+B24E
0, //U+B24F
0, //U+B250
0, //U+B251
0, //U+B252
0, //U+B253
0, //U+B254
0, //U+B255
0, //U+B256
0, //U+B257
0, //U+B258
0, //U+B259
0, //U+B25A
0, //U+B25B
0, //U+B25C
0, //U+B25D
0, //U+B25E
0, //U+B25F
0, //U+B260
0, //U+B261
0, //U+B262
0, //U+B263
0, //U+B264
0, //U+B265
0, //U+B266
0, //U+B267
0, //U+B268
0, //U+B269
0, //U+B26A
0, //U+B26B
0, //U+B26C
0, //U+B26D
0, //U+B26E
0, //U+B26F
0, //U+B270
0, //U+B271
0, //U+B272
0, //U+B273
0, //U+B274
0, //U+B275
0, //U+B276
0, //U+B277
0, //U+B278
0, //U+B279
0, //U+B27A
0, //U+B27B
0, //U+B27C
0, //U+B27D
0, //U+B27E
0, //U+B27F
0, //U+B280
0, //U+B281
0, //U+B282
0, //U+B283
0, //U+B284
0, //U+B285
0, //U+B286
0, //U+B287
0, //U+B288
0, //U+B289
0, //U+B28A
0, //U+B28B
0, //U+B28C
0, //U+B28D
0, //U+B28E
0, //U+B28F
0, //U+B290
0, //U+B291
0, //U+B292
0, //U+B293
0, //U+B294
0, //U+B295
0, //U+B296
0, //U+B297
0, //U+B298
0, //U+B299
0, //U+B29A
0, //U+B29B
0, //U+B29C
0, //U+B29D
0, //U+B29E
0, //U+B29F
0, //U+B2A0
0, //U+B2A1
0, //U+B2A2
0, //U+B2A3
0, //U+B2A4
0, //U+B2A5
0, //U+B2A6
0, //U+B2A7
0, //U+B2A8
0, //U+B2A9
0, //U+B2AA
0, //U+B2AB
0, //U+B2AC
0, //U+B2AD
0, //U+B2AE
0, //U+B2AF
0, //U+B2B0
0, //U+B2B1
0, //U+B2B2
0, //U+B2B3
0, //U+B2B4
0, //U+B2B5
0, //U+B2B6
0, //U+B2B7
0, //U+B2B8
0, //U+B2B9
0, //U+B2BA
0, //U+B2BB
0, //U+B2BC
0, //U+B2BD
0, //U+B2BE
0, //U+B2BF
0, //U+B2C0
0, //U+B2C1
0, //U+B2C2
0, //U+B2C3
0, //U+B2C4
0, //U+B2C5
0, //U+B2C6
0, //U+B2C7
0, //U+B2C8
0, //U+B2C9
0, //U+B2CA
0, //U+B2CB
0, //U+B2CC
0, //U+B2CD
0, //U+B2CE
0, //U+B2CF
0, //U+B2D0
0, //U+B2D1
0, //U+B2D2
0, //U+B2D3
0, //U+B2D4
0, //U+B2D5
0, //U+B2D6
0, //U+B2D7
0, //U+B2D8
0, //U+B2D9
0, //U+B2DA
0, //U+B2DB
0, //U+B2DC
0, //U+B2DD
0, //U+B2DE
0, //U+B2DF
0, //U+B2E0
0, //U+B2E1
0, //U+B2E2
0, //U+B2E3
0, //U+B2E4
0, //U+B2E5
0, //U+B2E6
0, //U+B2E7
0, //U+B2E8
0, //U+B2E9
0, //U+B2EA
0, //U+B2EB
0, //U+B2EC
0, //U+B2ED
0, //U+B2EE
0, //U+B2EF
0, //U+B2F0
0, //U+B2F1
0, //U+B2F2
0, //U+B2F3
0, //U+B2F4
0, //U+B2F5
0, //U+B2F6
0, //U+B2F7
0, //U+B2F8
0, //U+B2F9
0, //U+B2FA
0, //U+B2FB
0, //U+B2FC
0, //U+B2FD
0, //U+B2FE
0, //U+B2FF
0, //U+B300
0, //U+B301
0, //U+B302
0, //U+B303
0, //U+B304
0, //U+B305
0, //U+B306
0, //U+B307
0, //U+B308
0, //U+B309
0, //U+B30A
0, //U+B30B
0, //U+B30C
0, //U+B30D
0, //U+B30E
0, //U+B30F
0, //U+B310
0, //U+B311
0, //U+B312
0, //U+B313
0, //U+B314
0, //U+B315
0, //U+B316
0, //U+B317
0, //U+B318
0, //U+B319
0, //U+B31A
0, //U+B31B
0, //U+B31C
0, //U+B31D
0, //U+B31E
0, //U+B31F
0, //U+B320
0, //U+B321
0, //U+B322
0, //U+B323
0, //U+B324
0, //U+B325
0, //U+B326
0, //U+B327
0, //U+B328
0, //U+B329
0, //U+B32A
0, //U+B32B
0, //U+B32C
0, //U+B32D
0, //U+B32E
0, //U+B32F
0, //U+B330
0, //U+B331
0, //U+B332
0, //U+B333
0, //U+B334
0, //U+B335
0, //U+B336
0, //U+B337
0, //U+B338
0, //U+B339
0, //U+B33A
0, //U+B33B
0, //U+B33C
0, //U+B33D
0, //U+B33E
0, //U+B33F
0, //U+B340
0, //U+B341
0, //U+B342
0, //U+B343
0, //U+B344
0, //U+B345
0, //U+B346
0, //U+B347
0, //U+B348
0, //U+B349
0, //U+B34A
0, //U+B34B
0, //U+B34C
0, //U+B34D
0, //U+B34E
0, //U+B34F
0, //U+B350
0, //U+B351
0, //U+B352
0, //U+B353
0, //U+B354
0, //U+B355
0, //U+B356
0, //U+B357
0, //U+B358
0, //U+B359
0, //U+B35A
0, //U+B35B
0, //U+B35C
0, //U+B35D
0, //U+B35E
0, //U+B35F
0, //U+B360
0, //U+B361
0, //U+B362
0, //U+B363
0, //U+B364
0, //U+B365
0, //U+B366
0, //U+B367
0, //U+B368
0, //U+B369
0, //U+B36A
0, //U+B36B
0, //U+B36C
0, //U+B36D
0, //U+B36E
0, //U+B36F
0, //U+B370
0, //U+B371
0, //U+B372
0, //U+B373
0, //U+B374
0, //U+B375
0, //U+B376
0, //U+B377
0, //U+B378
0, //U+B379
0, //U+B37A
0, //U+B37B
0, //U+B37C
0, //U+B37D
0, //U+B37E
0, //U+B37F
0, //U+B380
0, //U+B381
0, //U+B382
0, //U+B383
0, //U+B384
0, //U+B385
0, //U+B386
0, //U+B387
0, //U+B388
0, //U+B389
0, //U+B38A
0, //U+B38B
0, //U+B38C
0, //U+B38D
0, //U+B38E
0, //U+B38F
0, //U+B390
0, //U+B391
0, //U+B392
0, //U+B393
0, //U+B394
0, //U+B395
0, //U+B396
0, //U+B397
0, //U+B398
0, //U+B399
0, //U+B39A
0, //U+B39B
0, //U+B39C
0, //U+B39D
0, //U+B39E
0, //U+B39F
0, //U+B3A0
0, //U+B3A1
0, //U+B3A2
0, //U+B3A3
0, //U+B3A4
0, //U+B3A5
0, //U+B3A6
0, //U+B3A7
0, //U+B3A8
0, //U+B3A9
0, //U+B3AA
0, //U+B3AB
0, //U+B3AC
0, //U+B3AD
0, //U+B3AE
0, //U+B3AF
0, //U+B3B0
0, //U+B3B1
0, //U+B3B2
0, //U+B3B3
0, //U+B3B4
0, //U+B3B5
0, //U+B3B6
0, //U+B3B7
0, //U+B3B8
0, //U+B3B9
0, //U+B3BA
0, //U+B3BB
0, //U+B3BC
0, //U+B3BD
0, //U+B3BE
0, //U+B3BF
0, //U+B3C0
0, //U+B3C1
0, //U+B3C2
0, //U+B3C3
0, //U+B3C4
0, //U+B3C5
0, //U+B3C6
0, //U+B3C7
0, //U+B3C8
0, //U+B3C9
0, //U+B3CA
0, //U+B3CB
0, //U+B3CC
0, //U+B3CD
0, //U+B3CE
0, //U+B3CF
0, //U+B3D0
0, //U+B3D1
0, //U+B3D2
0, //U+B3D3
0, //U+B3D4
0, //U+B3D5
0, //U+B3D6
0, //U+B3D7
0, //U+B3D8
0, //U+B3D9
0, //U+B3DA
0, //U+B3DB
0, //U+B3DC
0, //U+B3DD
0, //U+B3DE
0, //U+B3DF
0, //U+B3E0
0, //U+B3E1
0, //U+B3E2
0, //U+B3E3
0, //U+B3E4
0, //U+B3E5
0, //U+B3E6
0, //U+B3E7
0, //U+B3E8
0, //U+B3E9
0, //U+B3EA
0, //U+B3EB
0, //U+B3EC
0, //U+B3ED
0, //U+B3EE
0, //U+B3EF
0, //U+B3F0
0, //U+B3F1
0, //U+B3F2
0, //U+B3F3
0, //U+B3F4
0, //U+B3F5
0, //U+B3F6
0, //U+B3F7
0, //U+B3F8
0, //U+B3F9
0, //U+B3FA
0, //U+B3FB
0, //U+B3FC
0, //U+B3FD
0, //U+B3FE
0, //U+B3FF
0, //U+B400
0, //U+B401
0, //U+B402
0, //U+B403
0, //U+B404
0, //U+B405
0, //U+B406
0, //U+B407
0, //U+B408
0, //U+B409
0, //U+B40A
0, //U+B40B
0, //U+B40C
0, //U+B40D
0, //U+B40E
0, //U+B40F
0, //U+B410
0, //U+B411
0, //U+B412
0, //U+B413
0, //U+B414
0, //U+B415
0, //U+B416
0, //U+B417
0, //U+B418
0, //U+B419
0, //U+B41A
0, //U+B41B
0, //U+B41C
0, //U+B41D
0, //U+B41E
0, //U+B41F
0, //U+B420
0, //U+B421
0, //U+B422
0, //U+B423
0, //U+B424
0, //U+B425
0, //U+B426
0, //U+B427
0, //U+B428
0, //U+B429
0, //U+B42A
0, //U+B42B
0, //U+B42C
0, //U+B42D
0, //U+B42E
0, //U+B42F
0, //U+B430
0, //U+B431
0, //U+B432
0, //U+B433
0, //U+B434
0, //U+B435
0, //U+B436
0, //U+B437
0, //U+B438
0, //U+B439
0, //U+B43A
0, //U+B43B
0, //U+B43C
0, //U+B43D
0, //U+B43E
0, //U+B43F
0, //U+B440
0, //U+B441
0, //U+B442
0, //U+B443
0, //U+B444
0, //U+B445
0, //U+B446
0, //U+B447
0, //U+B448
0, //U+B449
0, //U+B44A
0, //U+B44B
0, //U+B44C
0, //U+B44D
0, //U+B44E
0, //U+B44F
0, //U+B450
0, //U+B451
0, //U+B452
0, //U+B453
0, //U+B454
0, //U+B455
0, //U+B456
0, //U+B457
0, //U+B458
0, //U+B459
0, //U+B45A
0, //U+B45B
0, //U+B45C
0, //U+B45D
0, //U+B45E
0, //U+B45F
0, //U+B460
0, //U+B461
0, //U+B462
0, //U+B463
0, //U+B464
0, //U+B465
0, //U+B466
0, //U+B467
0, //U+B468
0, //U+B469
0, //U+B46A
0, //U+B46B
0, //U+B46C
0, //U+B46D
0, //U+B46E
0, //U+B46F
0, //U+B470
0, //U+B471
0, //U+B472
0, //U+B473
0, //U+B474
0, //U+B475
0, //U+B476
0, //U+B477
0, //U+B478
0, //U+B479
0, //U+B47A
0, //U+B47B
0, //U+B47C
0, //U+B47D
0, //U+B47E
0, //U+B47F
0, //U+B480
0, //U+B481
0, //U+B482
0, //U+B483
0, //U+B484
0, //U+B485
0, //U+B486
0, //U+B487
0, //U+B488
0, //U+B489
0, //U+B48A
0, //U+B48B
0, //U+B48C
0, //U+B48D
0, //U+B48E
0, //U+B48F
0, //U+B490
0, //U+B491
0, //U+B492
0, //U+B493
0, //U+B494
0, //U+B495
0, //U+B496
0, //U+B497
0, //U+B498
0, //U+B499
0, //U+B49A
0, //U+B49B
0, //U+B49C
0, //U+B49D
0, //U+B49E
0, //U+B49F
0, //U+B4A0
0, //U+B4A1
0, //U+B4A2
0, //U+B4A3
0, //U+B4A4
0, //U+B4A5
0, //U+B4A6
0, //U+B4A7
0, //U+B4A8
0, //U+B4A9
0, //U+B4AA
0, //U+B4AB
0, //U+B4AC
0, //U+B4AD
0, //U+B4AE
0, //U+B4AF
0, //U+B4B0
0, //U+B4B1
0, //U+B4B2
0, //U+B4B3
0, //U+B4B4
0, //U+B4B5
0, //U+B4B6
0, //U+B4B7
0, //U+B4B8
0, //U+B4B9
0, //U+B4BA
0, //U+B4BB
0, //U+B4BC
0, //U+B4BD
0, //U+B4BE
0, //U+B4BF
0, //U+B4C0
0, //U+B4C1
0, //U+B4C2
0, //U+B4C3
0, //U+B4C4
0, //U+B4C5
0, //U+B4C6
0, //U+B4C7
0, //U+B4C8
0, //U+B4C9
0, //U+B4CA
0, //U+B4CB
0, //U+B4CC
0, //U+B4CD
0, //U+B4CE
0, //U+B4CF
0, //U+B4D0
0, //U+B4D1
0, //U+B4D2
0, //U+B4D3
0, //U+B4D4
0, //U+B4D5
0, //U+B4D6
0, //U+B4D7
0, //U+B4D8
0, //U+B4D9
0, //U+B4DA
0, //U+B4DB
0, //U+B4DC
0, //U+B4DD
0, //U+B4DE
0, //U+B4DF
0, //U+B4E0
0, //U+B4E1
0, //U+B4E2
0, //U+B4E3
0, //U+B4E4
0, //U+B4E5
0, //U+B4E6
0, //U+B4E7
0, //U+B4E8
0, //U+B4E9
0, //U+B4EA
0, //U+B4EB
0, //U+B4EC
0, //U+B4ED
0, //U+B4EE
0, //U+B4EF
0, //U+B4F0
0, //U+B4F1
0, //U+B4F2
0, //U+B4F3
0, //U+B4F4
0, //U+B4F5
0, //U+B4F6
0, //U+B4F7
0, //U+B4F8
0, //U+B4F9
0, //U+B4FA
0, //U+B4FB
0, //U+B4FC
0, //U+B4FD
0, //U+B4FE
0, //U+B4FF
0, //U+B500
0, //U+B501
0, //U+B502
0, //U+B503
0, //U+B504
0, //U+B505
0, //U+B506
0, //U+B507
0, //U+B508
0, //U+B509
0, //U+B50A
0, //U+B50B
0, //U+B50C
0, //U+B50D
0, //U+B50E
0, //U+B50F
0, //U+B510
0, //U+B511
0, //U+B512
0, //U+B513
0, //U+B514
0, //U+B515
0, //U+B516
0, //U+B517
0, //U+B518
0, //U+B519
0, //U+B51A
0, //U+B51B
0, //U+B51C
0, //U+B51D
0, //U+B51E
0, //U+B51F
0, //U+B520
0, //U+B521
0, //U+B522
0, //U+B523
0, //U+B524
0, //U+B525
0, //U+B526
0, //U+B527
0, //U+B528
0, //U+B529
0, //U+B52A
0, //U+B52B
0, //U+B52C
0, //U+B52D
0, //U+B52E
0, //U+B52F
0, //U+B530
0, //U+B531
0, //U+B532
0, //U+B533
0, //U+B534
0, //U+B535
0, //U+B536
0, //U+B537
0, //U+B538
0, //U+B539
0, //U+B53A
0, //U+B53B
0, //U+B53C
0, //U+B53D
0, //U+B53E
0, //U+B53F
0, //U+B540
0, //U+B541
0, //U+B542
0, //U+B543
0, //U+B544
0, //U+B545
0, //U+B546
0, //U+B547
0, //U+B548
0, //U+B549
0, //U+B54A
0, //U+B54B
0, //U+B54C
0, //U+B54D
0, //U+B54E
0, //U+B54F
0, //U+B550
0, //U+B551
0, //U+B552
0, //U+B553
0, //U+B554
0, //U+B555
0, //U+B556
0, //U+B557
0, //U+B558
0, //U+B559
0, //U+B55A
0, //U+B55B
0, //U+B55C
0, //U+B55D
0, //U+B55E
0, //U+B55F
0, //U+B560
0, //U+B561
0, //U+B562
0, //U+B563
0, //U+B564
0, //U+B565
0, //U+B566
0, //U+B567
0, //U+B568
0, //U+B569
0, //U+B56A
0, //U+B56B
0, //U+B56C
0, //U+B56D
0, //U+B56E
0, //U+B56F
0, //U+B570
0, //U+B571
0, //U+B572
0, //U+B573
0, //U+B574
0, //U+B575
0, //U+B576
0, //U+B577
0, //U+B578
0, //U+B579
0, //U+B57A
0, //U+B57B
0, //U+B57C
0, //U+B57D
0, //U+B57E
0, //U+B57F
0, //U+B580
0, //U+B581
0, //U+B582
0, //U+B583
0, //U+B584
0, //U+B585
0, //U+B586
0, //U+B587
0, //U+B588
0, //U+B589
0, //U+B58A
0, //U+B58B
0, //U+B58C
0, //U+B58D
0, //U+B58E
0, //U+B58F
0, //U+B590
0, //U+B591
0, //U+B592
0, //U+B593
0, //U+B594
0, //U+B595
0, //U+B596
0, //U+B597
0, //U+B598
0, //U+B599
0, //U+B59A
0, //U+B59B
0, //U+B59C
0, //U+B59D
0, //U+B59E
0, //U+B59F
0, //U+B5A0
0, //U+B5A1
0, //U+B5A2
0, //U+B5A3
0, //U+B5A4
0, //U+B5A5
0, //U+B5A6
0, //U+B5A7
0, //U+B5A8
0, //U+B5A9
0, //U+B5AA
0, //U+B5AB
0, //U+B5AC
0, //U+B5AD
0, //U+B5AE
0, //U+B5AF
0, //U+B5B0
0, //U+B5B1
0, //U+B5B2
0, //U+B5B3
0, //U+B5B4
0, //U+B5B5
0, //U+B5B6
0, //U+B5B7
0, //U+B5B8
0, //U+B5B9
0, //U+B5BA
0, //U+B5BB
0, //U+B5BC
0, //U+B5BD
0, //U+B5BE
0, //U+B5BF
0, //U+B5C0
0, //U+B5C1
0, //U+B5C2
0, //U+B5C3
0, //U+B5C4
0, //U+B5C5
0, //U+B5C6
0, //U+B5C7
0, //U+B5C8
0, //U+B5C9
0, //U+B5CA
0, //U+B5CB
0, //U+B5CC
0, //U+B5CD
0, //U+B5CE
0, //U+B5CF
0, //U+B5D0
0, //U+B5D1
0, //U+B5D2
0, //U+B5D3
0, //U+B5D4
0, //U+B5D5
0, //U+B5D6
0, //U+B5D7
0, //U+B5D8
0, //U+B5D9
0, //U+B5DA
0, //U+B5DB
0, //U+B5DC
0, //U+B5DD
0, //U+B5DE
0, //U+B5DF
0, //U+B5E0
0, //U+B5E1
0, //U+B5E2
0, //U+B5E3
0, //U+B5E4
0, //U+B5E5
0, //U+B5E6
0, //U+B5E7
0, //U+B5E8
0, //U+B5E9
0, //U+B5EA
0, //U+B5EB
0, //U+B5EC
0, //U+B5ED
0, //U+B5EE
0, //U+B5EF
0, //U+B5F0
0, //U+B5F1
0, //U+B5F2
0, //U+B5F3
0, //U+B5F4
0, //U+B5F5
0, //U+B5F6
0, //U+B5F7
0, //U+B5F8
0, //U+B5F9
0, //U+B5FA
0, //U+B5FB
0, //U+B5FC
0, //U+B5FD
0, //U+B5FE
0, //U+B5FF
0, //U+B600
0, //U+B601
0, //U+B602
0, //U+B603
0, //U+B604
0, //U+B605
0, //U+B606
0, //U+B607
0, //U+B608
0, //U+B609
0, //U+B60A
0, //U+B60B
0, //U+B60C
0, //U+B60D
0, //U+B60E
0, //U+B60F
0, //U+B610
0, //U+B611
0, //U+B612
0, //U+B613
0, //U+B614
0, //U+B615
0, //U+B616
0, //U+B617
0, //U+B618
0, //U+B619
0, //U+B61A
0, //U+B61B
0, //U+B61C
0, //U+B61D
0, //U+B61E
0, //U+B61F
0, //U+B620
0, //U+B621
0, //U+B622
0, //U+B623
0, //U+B624
0, //U+B625
0, //U+B626
0, //U+B627
0, //U+B628
0, //U+B629
0, //U+B62A
0, //U+B62B
0, //U+B62C
0, //U+B62D
0, //U+B62E
0, //U+B62F
0, //U+B630
0, //U+B631
0, //U+B632
0, //U+B633
0, //U+B634
0, //U+B635
0, //U+B636
0, //U+B637
0, //U+B638
0, //U+B639
0, //U+B63A
0, //U+B63B
0, //U+B63C
0, //U+B63D
0, //U+B63E
0, //U+B63F
0, //U+B640
0, //U+B641
0, //U+B642
0, //U+B643
0, //U+B644
0, //U+B645
0, //U+B646
0, //U+B647
0, //U+B648
0, //U+B649
0, //U+B64A
0, //U+B64B
0, //U+B64C
0, //U+B64D
0, //U+B64E
0, //U+B64F
0, //U+B650
0, //U+B651
0, //U+B652
0, //U+B653
0, //U+B654
0, //U+B655
0, //U+B656
0, //U+B657
0, //U+B658
0, //U+B659
0, //U+B65A
0, //U+B65B
0, //U+B65C
0, //U+B65D
0, //U+B65E
0, //U+B65F
0, //U+B660
0, //U+B661
0, //U+B662
0, //U+B663
0, //U+B664
0, //U+B665
0, //U+B666
0, //U+B667
0, //U+B668
0, //U+B669
0, //U+B66A
0, //U+B66B
0, //U+B66C
0, //U+B66D
0, //U+B66E
0, //U+B66F
0, //U+B670
0, //U+B671
0, //U+B672
0, //U+B673
0, //U+B674
0, //U+B675
0, //U+B676
0, //U+B677
0, //U+B678
0, //U+B679
0, //U+B67A
0, //U+B67B
0, //U+B67C
0, //U+B67D
0, //U+B67E
0, //U+B67F
0, //U+B680
0, //U+B681
0, //U+B682
0, //U+B683
0, //U+B684
0, //U+B685
0, //U+B686
0, //U+B687
0, //U+B688
0, //U+B689
0, //U+B68A
0, //U+B68B
0, //U+B68C
0, //U+B68D
0, //U+B68E
0, //U+B68F
0, //U+B690
0, //U+B691
0, //U+B692
0, //U+B693
0, //U+B694
0, //U+B695
0, //U+B696
0, //U+B697
0, //U+B698
0, //U+B699
0, //U+B69A
0, //U+B69B
0, //U+B69C
0, //U+B69D
0, //U+B69E
0, //U+B69F
0, //U+B6A0
0, //U+B6A1
0, //U+B6A2
0, //U+B6A3
0, //U+B6A4
0, //U+B6A5
0, //U+B6A6
0, //U+B6A7
0, //U+B6A8
0, //U+B6A9
0, //U+B6AA
0, //U+B6AB
0, //U+B6AC
0, //U+B6AD
0, //U+B6AE
0, //U+B6AF
0, //U+B6B0
0, //U+B6B1
0, //U+B6B2
0, //U+B6B3
0, //U+B6B4
0, //U+B6B5
0, //U+B6B6
0, //U+B6B7
0, //U+B6B8
0, //U+B6B9
0, //U+B6BA
0, //U+B6BB
0, //U+B6BC
0, //U+B6BD
0, //U+B6BE
0, //U+B6BF
0, //U+B6C0
0, //U+B6C1
0, //U+B6C2
0, //U+B6C3
0, //U+B6C4
0, //U+B6C5
0, //U+B6C6
0, //U+B6C7
0, //U+B6C8
0, //U+B6C9
0, //U+B6CA
0, //U+B6CB
0, //U+B6CC
0, //U+B6CD
0, //U+B6CE
0, //U+B6CF
0, //U+B6D0
0, //U+B6D1
0, //U+B6D2
0, //U+B6D3
0, //U+B6D4
0, //U+B6D5
0, //U+B6D6
0, //U+B6D7
0, //U+B6D8
0, //U+B6D9
0, //U+B6DA
0, //U+B6DB
0, //U+B6DC
0, //U+B6DD
0, //U+B6DE
0, //U+B6DF
0, //U+B6E0
0, //U+B6E1
0, //U+B6E2
0, //U+B6E3
0, //U+B6E4
0, //U+B6E5
0, //U+B6E6
0, //U+B6E7
0, //U+B6E8
0, //U+B6E9
0, //U+B6EA
0, //U+B6EB
0, //U+B6EC
0, //U+B6ED
0, //U+B6EE
0, //U+B6EF
0, //U+B6F0
0, //U+B6F1
0, //U+B6F2
0, //U+B6F3
0, //U+B6F4
0, //U+B6F5
0, //U+B6F6
0, //U+B6F7
0, //U+B6F8
0, //U+B6F9
0, //U+B6FA
0, //U+B6FB
0, //U+B6FC
0, //U+B6FD
0, //U+B6FE
0, //U+B6FF
0, //U+B700
0, //U+B701
0, //U+B702
0, //U+B703
0, //U+B704
0, //U+B705
0, //U+B706
0, //U+B707
0, //U+B708
0, //U+B709
0, //U+B70A
0, //U+B70B
0, //U+B70C
0, //U+B70D
0, //U+B70E
0, //U+B70F
0, //U+B710
0, //U+B711
0, //U+B712
0, //U+B713
0, //U+B714
0, //U+B715
0, //U+B716
0, //U+B717
0, //U+B718
0, //U+B719
0, //U+B71A
0, //U+B71B
0, //U+B71C
0, //U+B71D
0, //U+B71E
0, //U+B71F
0, //U+B720
0, //U+B721
0, //U+B722
0, //U+B723
0, //U+B724
0, //U+B725
0, //U+B726
0, //U+B727
0, //U+B728
0, //U+B729
0, //U+B72A
0, //U+B72B
0, //U+B72C
0, //U+B72D
0, //U+B72E
0, //U+B72F
0, //U+B730
0, //U+B731
0, //U+B732
0, //U+B733
0, //U+B734
0, //U+B735
0, //U+B736
0, //U+B737
0, //U+B738
0, //U+B739
0, //U+B73A
0, //U+B73B
0, //U+B73C
0, //U+B73D
0, //U+B73E
0, //U+B73F
0, //U+B740
0, //U+B741
0, //U+B742
0, //U+B743
0, //U+B744
0, //U+B745
0, //U+B746
0, //U+B747
0, //U+B748
0, //U+B749
0, //U+B74A
0, //U+B74B
0, //U+B74C
0, //U+B74D
0, //U+B74E
0, //U+B74F
0, //U+B750
0, //U+B751
0, //U+B752
0, //U+B753
0, //U+B754
0, //U+B755
0, //U+B756
0, //U+B757
0, //U+B758
0, //U+B759
0, //U+B75A
0, //U+B75B
0, //U+B75C
0, //U+B75D
0, //U+B75E
0, //U+B75F
0, //U+B760
0, //U+B761
0, //U+B762
0, //U+B763
0, //U+B764
0, //U+B765
0, //U+B766
0, //U+B767
0, //U+B768
0, //U+B769
0, //U+B76A
0, //U+B76B
0, //U+B76C
0, //U+B76D
0, //U+B76E
0, //U+B76F
0, //U+B770
0, //U+B771
0, //U+B772
0, //U+B773
0, //U+B774
0, //U+B775
0, //U+B776
0, //U+B777
0, //U+B778
0, //U+B779
0, //U+B77A
0, //U+B77B
0, //U+B77C
0, //U+B77D
0, //U+B77E
0, //U+B77F
0, //U+B780
0, //U+B781
0, //U+B782
0, //U+B783
0, //U+B784
0, //U+B785
0, //U+B786
0, //U+B787
0, //U+B788
0, //U+B789
0, //U+B78A
0, //U+B78B
0, //U+B78C
0, //U+B78D
0, //U+B78E
0, //U+B78F
0, //U+B790
0, //U+B791
0, //U+B792
0, //U+B793
0, //U+B794
0, //U+B795
0, //U+B796
0, //U+B797
0, //U+B798
0, //U+B799
0, //U+B79A
0, //U+B79B
0, //U+B79C
0, //U+B79D
0, //U+B79E
0, //U+B79F
0, //U+B7A0
0, //U+B7A1
0, //U+B7A2
0, //U+B7A3
0, //U+B7A4
0, //U+B7A5
0, //U+B7A6
0, //U+B7A7
0, //U+B7A8
0, //U+B7A9
0, //U+B7AA
0, //U+B7AB
0, //U+B7AC
0, //U+B7AD
0, //U+B7AE
0, //U+B7AF
0, //U+B7B0
0, //U+B7B1
0, //U+B7B2
0, //U+B7B3
0, //U+B7B4
0, //U+B7B5
0, //U+B7B6
0, //U+B7B7
0, //U+B7B8
0, //U+B7B9
0, //U+B7BA
0, //U+B7BB
0, //U+B7BC
0, //U+B7BD
0, //U+B7BE
0, //U+B7BF
0, //U+B7C0
0, //U+B7C1
0, //U+B7C2
0, //U+B7C3
0, //U+B7C4
0, //U+B7C5
0, //U+B7C6
0, //U+B7C7
0, //U+B7C8
0, //U+B7C9
0, //U+B7CA
0, //U+B7CB
0, //U+B7CC
0, //U+B7CD
0, //U+B7CE
0, //U+B7CF
0, //U+B7D0
0, //U+B7D1
0, //U+B7D2
0, //U+B7D3
0, //U+B7D4
0, //U+B7D5
0, //U+B7D6
0, //U+B7D7
0, //U+B7D8
0, //U+B7D9
0, //U+B7DA
0, //U+B7DB
0, //U+B7DC
0, //U+B7DD
0, //U+B7DE
0, //U+B7DF
0, //U+B7E0
0, //U+B7E1
0, //U+B7E2
0, //U+B7E3
0, //U+B7E4
0, //U+B7E5
0, //U+B7E6
0, //U+B7E7
0, //U+B7E8
0, //U+B7E9
0, //U+B7EA
0, //U+B7EB
0, //U+B7EC
0, //U+B7ED
0, //U+B7EE
0, //U+B7EF
0, //U+B7F0
0, //U+B7F1
0, //U+B7F2
0, //U+B7F3
0, //U+B7F4
0, //U+B7F5
0, //U+B7F6
0, //U+B7F7
0, //U+B7F8
0, //U+B7F9
0, //U+B7FA
0, //U+B7FB
0, //U+B7FC
0, //U+B7FD
0, //U+B7FE
0, //U+B7FF
0, //U+B800
0, //U+B801
0, //U+B802
0, //U+B803
0, //U+B804
0, //U+B805
0, //U+B806
0, //U+B807
0, //U+B808
0, //U+B809
0, //U+B80A
0, //U+B80B
0, //U+B80C
0, //U+B80D
0, //U+B80E
0, //U+B80F
0, //U+B810
0, //U+B811
0, //U+B812
0, //U+B813
0, //U+B814
0, //U+B815
0, //U+B816
0, //U+B817
0, //U+B818
0, //U+B819
0, //U+B81A
0, //U+B81B
0, //U+B81C
0, //U+B81D
0, //U+B81E
0, //U+B81F
0, //U+B820
0, //U+B821
0, //U+B822
0, //U+B823
0, //U+B824
0, //U+B825
0, //U+B826
0, //U+B827
0, //U+B828
0, //U+B829
0, //U+B82A
0, //U+B82B
0, //U+B82C
0, //U+B82D
0, //U+B82E
0, //U+B82F
0, //U+B830
0, //U+B831
0, //U+B832
0, //U+B833
0, //U+B834
0, //U+B835
0, //U+B836
0, //U+B837
0, //U+B838
0, //U+B839
0, //U+B83A
0, //U+B83B
0, //U+B83C
0, //U+B83D
0, //U+B83E
0, //U+B83F
0, //U+B840
0, //U+B841
0, //U+B842
0, //U+B843
0, //U+B844
0, //U+B845
0, //U+B846
0, //U+B847
0, //U+B848
0, //U+B849
0, //U+B84A
0, //U+B84B
0, //U+B84C
0, //U+B84D
0, //U+B84E
0, //U+B84F
0, //U+B850
0, //U+B851
0, //U+B852
0, //U+B853
0, //U+B854
0, //U+B855
0, //U+B856
0, //U+B857
0, //U+B858
0, //U+B859
0, //U+B85A
0, //U+B85B
0, //U+B85C
0, //U+B85D
0, //U+B85E
0, //U+B85F
0, //U+B860
0, //U+B861
0, //U+B862
0, //U+B863
0, //U+B864
0, //U+B865
0, //U+B866
0, //U+B867
0, //U+B868
0, //U+B869
0, //U+B86A
0, //U+B86B
0, //U+B86C
0, //U+B86D
0, //U+B86E
0, //U+B86F
0, //U+B870
0, //U+B871
0, //U+B872
0, //U+B873
0, //U+B874
0, //U+B875
0, //U+B876
0, //U+B877
0, //U+B878
0, //U+B879
0, //U+B87A
0, //U+B87B
0, //U+B87C
0, //U+B87D
0, //U+B87E
0, //U+B87F
0, //U+B880
0, //U+B881
0, //U+B882
0, //U+B883
0, //U+B884
0, //U+B885
0, //U+B886
0, //U+B887
0, //U+B888
0, //U+B889
0, //U+B88A
0, //U+B88B
0, //U+B88C
0, //U+B88D
0, //U+B88E
0, //U+B88F
0, //U+B890
0, //U+B891
0, //U+B892
0, //U+B893
0, //U+B894
0, //U+B895
0, //U+B896
0, //U+B897
0, //U+B898
0, //U+B899
0, //U+B89A
0, //U+B89B
0, //U+B89C
0, //U+B89D
0, //U+B89E
0, //U+B89F
0, //U+B8A0
0, //U+B8A1
0, //U+B8A2
0, //U+B8A3
0, //U+B8A4
0, //U+B8A5
0, //U+B8A6
0, //U+B8A7
0, //U+B8A8
0, //U+B8A9
0, //U+B8AA
0, //U+B8AB
0, //U+B8AC
0, //U+B8AD
0, //U+B8AE
0, //U+B8AF
0, //U+B8B0
0, //U+B8B1
0, //U+B8B2
0, //U+B8B3
0, //U+B8B4
0, //U+B8B5
0, //U+B8B6
0, //U+B8B7
0, //U+B8B8
0, //U+B8B9
0, //U+B8BA
0, //U+B8BB
0, //U+B8BC
0, //U+B8BD
0, //U+B8BE
0, //U+B8BF
0, //U+B8C0
0, //U+B8C1
0, //U+B8C2
0, //U+B8C3
0, //U+B8C4
0, //U+B8C5
0, //U+B8C6
0, //U+B8C7
0, //U+B8C8
0, //U+B8C9
0, //U+B8CA
0, //U+B8CB
0, //U+B8CC
0, //U+B8CD
0, //U+B8CE
0, //U+B8CF
0, //U+B8D0
0, //U+B8D1
0, //U+B8D2
0, //U+B8D3
0, //U+B8D4
0, //U+B8D5
0, //U+B8D6
0, //U+B8D7
0, //U+B8D8
0, //U+B8D9
0, //U+B8DA
0, //U+B8DB
0, //U+B8DC
0, //U+B8DD
0, //U+B8DE
0, //U+B8DF
0, //U+B8E0
0, //U+B8E1
0, //U+B8E2
0, //U+B8E3
0, //U+B8E4
0, //U+B8E5
0, //U+B8E6
0, //U+B8E7
0, //U+B8E8
0, //U+B8E9
0, //U+B8EA
0, //U+B8EB
0, //U+B8EC
0, //U+B8ED
0, //U+B8EE
0, //U+B8EF
0, //U+B8F0
0, //U+B8F1
0, //U+B8F2
0, //U+B8F3
0, //U+B8F4
0, //U+B8F5
0, //U+B8F6
0, //U+B8F7
0, //U+B8F8
0, //U+B8F9
0, //U+B8FA
0, //U+B8FB
0, //U+B8FC
0, //U+B8FD
0, //U+B8FE
0, //U+B8FF
0, //U+B900
0, //U+B901
0, //U+B902
0, //U+B903
0, //U+B904
0, //U+B905
0, //U+B906
0, //U+B907
0, //U+B908
0, //U+B909
0, //U+B90A
0, //U+B90B
0, //U+B90C
0, //U+B90D
0, //U+B90E
0, //U+B90F
0, //U+B910
0, //U+B911
0, //U+B912
0, //U+B913
0, //U+B914
0, //U+B915
0, //U+B916
0, //U+B917
0, //U+B918
0, //U+B919
0, //U+B91A
0, //U+B91B
0, //U+B91C
0, //U+B91D
0, //U+B91E
0, //U+B91F
0, //U+B920
0, //U+B921
0, //U+B922
0, //U+B923
0, //U+B924
0, //U+B925
0, //U+B926
0, //U+B927
0, //U+B928
0, //U+B929
0, //U+B92A
0, //U+B92B
0, //U+B92C
0, //U+B92D
0, //U+B92E
0, //U+B92F
0, //U+B930
0, //U+B931
0, //U+B932
0, //U+B933
0, //U+B934
0, //U+B935
0, //U+B936
0, //U+B937
0, //U+B938
0, //U+B939
0, //U+B93A
0, //U+B93B
0, //U+B93C
0, //U+B93D
0, //U+B93E
0, //U+B93F
0, //U+B940
0, //U+B941
0, //U+B942
0, //U+B943
0, //U+B944
0, //U+B945
0, //U+B946
0, //U+B947
0, //U+B948
0, //U+B949
0, //U+B94A
0, //U+B94B
0, //U+B94C
0, //U+B94D
0, //U+B94E
0, //U+B94F
0, //U+B950
0, //U+B951
0, //U+B952
0, //U+B953
0, //U+B954
0, //U+B955
0, //U+B956
0, //U+B957
0, //U+B958
0, //U+B959
0, //U+B95A
0, //U+B95B
0, //U+B95C
0, //U+B95D
0, //U+B95E
0, //U+B95F
0, //U+B960
0, //U+B961
0, //U+B962
0, //U+B963
0, //U+B964
0, //U+B965
0, //U+B966
0, //U+B967
0, //U+B968
0, //U+B969
0, //U+B96A
0, //U+B96B
0, //U+B96C
0, //U+B96D
0, //U+B96E
0, //U+B96F
0, //U+B970
0, //U+B971
0, //U+B972
0, //U+B973
0, //U+B974
0, //U+B975
0, //U+B976
0, //U+B977
0, //U+B978
0, //U+B979
0, //U+B97A
0, //U+B97B
0, //U+B97C
0, //U+B97D
0, //U+B97E
0, //U+B97F
0, //U+B980
0, //U+B981
0, //U+B982
0, //U+B983
0, //U+B984
0, //U+B985
0, //U+B986
0, //U+B987
0, //U+B988
0, //U+B989
0, //U+B98A
0, //U+B98B
0, //U+B98C
0, //U+B98D
0, //U+B98E
0, //U+B98F
0, //U+B990
0, //U+B991
0, //U+B992
0, //U+B993
0, //U+B994
0, //U+B995
0, //U+B996
0, //U+B997
0, //U+B998
0, //U+B999
0, //U+B99A
0, //U+B99B
0, //U+B99C
0, //U+B99D
0, //U+B99E
0, //U+B99F
0, //U+B9A0
0, //U+B9A1
0, //U+B9A2
0, //U+B9A3
0, //U+B9A4
0, //U+B9A5
0, //U+B9A6
0, //U+B9A7
0, //U+B9A8
0, //U+B9A9
0, //U+B9AA
0, //U+B9AB
0, //U+B9AC
0, //U+B9AD
0, //U+B9AE
0, //U+B9AF
0, //U+B9B0
0, //U+B9B1
0, //U+B9B2
0, //U+B9B3
0, //U+B9B4
0, //U+B9B5
0, //U+B9B6
0, //U+B9B7
0, //U+B9B8
0, //U+B9B9
0, //U+B9BA
0, //U+B9BB
0, //U+B9BC
0, //U+B9BD
0, //U+B9BE
0, //U+B9BF
0, //U+B9C0
0, //U+B9C1
0, //U+B9C2
0, //U+B9C3
0, //U+B9C4
0, //U+B9C5
0, //U+B9C6
0, //U+B9C7
0, //U+B9C8
0, //U+B9C9
0, //U+B9CA
0, //U+B9CB
0, //U+B9CC
0, //U+B9CD
0, //U+B9CE
0, //U+B9CF
0, //U+B9D0
0, //U+B9D1
0, //U+B9D2
0, //U+B9D3
0, //U+B9D4
0, //U+B9D5
0, //U+B9D6
0, //U+B9D7
0, //U+B9D8
0, //U+B9D9
0, //U+B9DA
0, //U+B9DB
0, //U+B9DC
0, //U+B9DD
0, //U+B9DE
0, //U+B9DF
0, //U+B9E0
0, //U+B9E1
0, //U+B9E2
0, //U+B9E3
0, //U+B9E4
0, //U+B9E5
0, //U+B9E6
0, //U+B9E7
0, //U+B9E8
0, //U+B9E9
0, //U+B9EA
0, //U+B9EB
0, //U+B9EC
0, //U+B9ED
0, //U+B9EE
0, //U+B9EF
0, //U+B9F0
0, //U+B9F1
0, //U+B9F2
0, //U+B9F3
0, //U+B9F4
0, //U+B9F5
0, //U+B9F6
0, //U+B9F7
0, //U+B9F8
0, //U+B9F9
0, //U+B9FA
0, //U+B9FB
0, //U+B9FC
0, //U+B9FD
0, //U+B9FE
0, //U+B9FF
0, //U+BA00
0, //U+BA01
0, //U+BA02
0, //U+BA03
0, //U+BA04
0, //U+BA05
0, //U+BA06
0, //U+BA07
0, //U+BA08
0, //U+BA09
0, //U+BA0A
0, //U+BA0B
0, //U+BA0C
0, //U+BA0D
0, //U+BA0E
0, //U+BA0F
0, //U+BA10
0, //U+BA11
0, //U+BA12
0, //U+BA13
0, //U+BA14
0, //U+BA15
0, //U+BA16
0, //U+BA17
0, //U+BA18
0, //U+BA19
0, //U+BA1A
0, //U+BA1B
0, //U+BA1C
0, //U+BA1D
0, //U+BA1E
0, //U+BA1F
0, //U+BA20
0, //U+BA21
0, //U+BA22
0, //U+BA23
0, //U+BA24
0, //U+BA25
0, //U+BA26
0, //U+BA27
0, //U+BA28
0, //U+BA29
0, //U+BA2A
0, //U+BA2B
0, //U+BA2C
0, //U+BA2D
0, //U+BA2E
0, //U+BA2F
0, //U+BA30
0, //U+BA31
0, //U+BA32
0, //U+BA33
0, //U+BA34
0, //U+BA35
0, //U+BA36
0, //U+BA37
0, //U+BA38
0, //U+BA39
0, //U+BA3A
0, //U+BA3B
0, //U+BA3C
0, //U+BA3D
0, //U+BA3E
0, //U+BA3F
0, //U+BA40
0, //U+BA41
0, //U+BA42
0, //U+BA43
0, //U+BA44
0, //U+BA45
0, //U+BA46
0, //U+BA47
0, //U+BA48
0, //U+BA49
0, //U+BA4A
0, //U+BA4B
0, //U+BA4C
0, //U+BA4D
0, //U+BA4E
0, //U+BA4F
0, //U+BA50
0, //U+BA51
0, //U+BA52
0, //U+BA53
0, //U+BA54
0, //U+BA55
0, //U+BA56
0, //U+BA57
0, //U+BA58
0, //U+BA59
0, //U+BA5A
0, //U+BA5B
0, //U+BA5C
0, //U+BA5D
0, //U+BA5E
0, //U+BA5F
0, //U+BA60
0, //U+BA61
0, //U+BA62
0, //U+BA63
0, //U+BA64
0, //U+BA65
0, //U+BA66
0, //U+BA67
0, //U+BA68
0, //U+BA69
0, //U+BA6A
0, //U+BA6B
0, //U+BA6C
0, //U+BA6D
0, //U+BA6E
0, //U+BA6F
0, //U+BA70
0, //U+BA71
0, //U+BA72
0, //U+BA73
0, //U+BA74
0, //U+BA75
0, //U+BA76
0, //U+BA77
0, //U+BA78
0, //U+BA79
0, //U+BA7A
0, //U+BA7B
0, //U+BA7C
0, //U+BA7D
0, //U+BA7E
0, //U+BA7F
0, //U+BA80
0, //U+BA81
0, //U+BA82
0, //U+BA83
0, //U+BA84
0, //U+BA85
0, //U+BA86
0, //U+BA87
0, //U+BA88
0, //U+BA89
0, //U+BA8A
0, //U+BA8B
0, //U+BA8C
0, //U+BA8D
0, //U+BA8E
0, //U+BA8F
0, //U+BA90
0, //U+BA91
0, //U+BA92
0, //U+BA93
0, //U+BA94
0, //U+BA95
0, //U+BA96
0, //U+BA97
0, //U+BA98
0, //U+BA99
0, //U+BA9A
0, //U+BA9B
0, //U+BA9C
0, //U+BA9D
0, //U+BA9E
0, //U+BA9F
0, //U+BAA0
0, //U+BAA1
0, //U+BAA2
0, //U+BAA3
0, //U+BAA4
0, //U+BAA5
0, //U+BAA6
0, //U+BAA7
0, //U+BAA8
0, //U+BAA9
0, //U+BAAA
0, //U+BAAB
0, //U+BAAC
0, //U+BAAD
0, //U+BAAE
0, //U+BAAF
0, //U+BAB0
0, //U+BAB1
0, //U+BAB2
0, //U+BAB3
0, //U+BAB4
0, //U+BAB5
0, //U+BAB6
0, //U+BAB7
0, //U+BAB8
0, //U+BAB9
0, //U+BABA
0, //U+BABB
0, //U+BABC
0, //U+BABD
0, //U+BABE
0, //U+BABF
0, //U+BAC0
0, //U+BAC1
0, //U+BAC2
0, //U+BAC3
0, //U+BAC4
0, //U+BAC5
0, //U+BAC6
0, //U+BAC7
0, //U+BAC8
0, //U+BAC9
0, //U+BACA
0, //U+BACB
0, //U+BACC
0, //U+BACD
0, //U+BACE
0, //U+BACF
0, //U+BAD0
0, //U+BAD1
0, //U+BAD2
0, //U+BAD3
0, //U+BAD4
0, //U+BAD5
0, //U+BAD6
0, //U+BAD7
0, //U+BAD8
0, //U+BAD9
0, //U+BADA
0, //U+BADB
0, //U+BADC
0, //U+BADD
0, //U+BADE
0, //U+BADF
0, //U+BAE0
0, //U+BAE1
0, //U+BAE2
0, //U+BAE3
0, //U+BAE4
0, //U+BAE5
0, //U+BAE6
0, //U+BAE7
0, //U+BAE8
0, //U+BAE9
0, //U+BAEA
0, //U+BAEB
0, //U+BAEC
0, //U+BAED
0, //U+BAEE
0, //U+BAEF
0, //U+BAF0
0, //U+BAF1
0, //U+BAF2
0, //U+BAF3
0, //U+BAF4
0, //U+BAF5
0, //U+BAF6
0, //U+BAF7
0, //U+BAF8
0, //U+BAF9
0, //U+BAFA
0, //U+BAFB
0, //U+BAFC
0, //U+BAFD
0, //U+BAFE
0, //U+BAFF
0, //U+BB00
0, //U+BB01
0, //U+BB02
0, //U+BB03
0, //U+BB04
0, //U+BB05
0, //U+BB06
0, //U+BB07
0, //U+BB08
0, //U+BB09
0, //U+BB0A
0, //U+BB0B
0, //U+BB0C
0, //U+BB0D
0, //U+BB0E
0, //U+BB0F
0, //U+BB10
0, //U+BB11
0, //U+BB12
0, //U+BB13
0, //U+BB14
0, //U+BB15
0, //U+BB16
0, //U+BB17
0, //U+BB18
0, //U+BB19
0, //U+BB1A
0, //U+BB1B
0, //U+BB1C
0, //U+BB1D
0, //U+BB1E
0, //U+BB1F
0, //U+BB20
0, //U+BB21
0, //U+BB22
0, //U+BB23
0, //U+BB24
0, //U+BB25
0, //U+BB26
0, //U+BB27
0, //U+BB28
0, //U+BB29
0, //U+BB2A
0, //U+BB2B
0, //U+BB2C
0, //U+BB2D
0, //U+BB2E
0, //U+BB2F
0, //U+BB30
0, //U+BB31
0, //U+BB32
0, //U+BB33
0, //U+BB34
0, //U+BB35
0, //U+BB36
0, //U+BB37
0, //U+BB38
0, //U+BB39
0, //U+BB3A
0, //U+BB3B
0, //U+BB3C
0, //U+BB3D
0, //U+BB3E
0, //U+BB3F
0, //U+BB40
0, //U+BB41
0, //U+BB42
0, //U+BB43
0, //U+BB44
0, //U+BB45
0, //U+BB46
0, //U+BB47
0, //U+BB48
0, //U+BB49
0, //U+BB4A
0, //U+BB4B
0, //U+BB4C
0, //U+BB4D
0, //U+BB4E
0, //U+BB4F
0, //U+BB50
0, //U+BB51
0, //U+BB52
0, //U+BB53
0, //U+BB54
0, //U+BB55
0, //U+BB56
0, //U+BB57
0, //U+BB58
0, //U+BB59
0, //U+BB5A
0, //U+BB5B
0, //U+BB5C
0, //U+BB5D
0, //U+BB5E
0, //U+BB5F
0, //U+BB60
0, //U+BB61
0, //U+BB62
0, //U+BB63
0, //U+BB64
0, //U+BB65
0, //U+BB66
0, //U+BB67
0, //U+BB68
0, //U+BB69
0, //U+BB6A
0, //U+BB6B
0, //U+BB6C
0, //U+BB6D
0, //U+BB6E
0, //U+BB6F
0, //U+BB70
0, //U+BB71
0, //U+BB72
0, //U+BB73
0, //U+BB74
0, //U+BB75
0, //U+BB76
0, //U+BB77
0, //U+BB78
0, //U+BB79
0, //U+BB7A
0, //U+BB7B
0, //U+BB7C
0, //U+BB7D
0, //U+BB7E
0, //U+BB7F
0, //U+BB80
0, //U+BB81
0, //U+BB82
0, //U+BB83
0, //U+BB84
0, //U+BB85
0, //U+BB86
0, //U+BB87
0, //U+BB88
0, //U+BB89
0, //U+BB8A
0, //U+BB8B
0, //U+BB8C
0, //U+BB8D
0, //U+BB8E
0, //U+BB8F
0, //U+BB90
0, //U+BB91
0, //U+BB92
0, //U+BB93
0, //U+BB94
0, //U+BB95
0, //U+BB96
0, //U+BB97
0, //U+BB98
0, //U+BB99
0, //U+BB9A
0, //U+BB9B
0, //U+BB9C
0, //U+BB9D
0, //U+BB9E
0, //U+BB9F
0, //U+BBA0
0, //U+BBA1
0, //U+BBA2
0, //U+BBA3
0, //U+BBA4
0, //U+BBA5
0, //U+BBA6
0, //U+BBA7
0, //U+BBA8
0, //U+BBA9
0, //U+BBAA
0, //U+BBAB
0, //U+BBAC
0, //U+BBAD
0, //U+BBAE
0, //U+BBAF
0, //U+BBB0
0, //U+BBB1
0, //U+BBB2
0, //U+BBB3
0, //U+BBB4
0, //U+BBB5
0, //U+BBB6
0, //U+BBB7
0, //U+BBB8
0, //U+BBB9
0, //U+BBBA
0, //U+BBBB
0, //U+BBBC
0, //U+BBBD
0, //U+BBBE
0, //U+BBBF
0, //U+BBC0
0, //U+BBC1
0, //U+BBC2
0, //U+BBC3
0, //U+BBC4
0, //U+BBC5
0, //U+BBC6
0, //U+BBC7
0, //U+BBC8
0, //U+BBC9
0, //U+BBCA
0, //U+BBCB
0, //U+BBCC
0, //U+BBCD
0, //U+BBCE
0, //U+BBCF
0, //U+BBD0
0, //U+BBD1
0, //U+BBD2
0, //U+BBD3
0, //U+BBD4
0, //U+BBD5
0, //U+BBD6
0, //U+BBD7
0, //U+BBD8
0, //U+BBD9
0, //U+BBDA
0, //U+BBDB
0, //U+BBDC
0, //U+BBDD
0, //U+BBDE
0, //U+BBDF
0, //U+BBE0
0, //U+BBE1
0, //U+BBE2
0, //U+BBE3
0, //U+BBE4
0, //U+BBE5
0, //U+BBE6
0, //U+BBE7
0, //U+BBE8
0, //U+BBE9
0, //U+BBEA
0, //U+BBEB
0, //U+BBEC
0, //U+BBED
0, //U+BBEE
0, //U+BBEF
0, //U+BBF0
0, //U+BBF1
0, //U+BBF2
0, //U+BBF3
0, //U+BBF4
0, //U+BBF5
0, //U+BBF6
0, //U+BBF7
0, //U+BBF8
0, //U+BBF9
0, //U+BBFA
0, //U+BBFB
0, //U+BBFC
0, //U+BBFD
0, //U+BBFE
0, //U+BBFF
0, //U+BC00
0, //U+BC01
0, //U+BC02
0, //U+BC03
0, //U+BC04
0, //U+BC05
0, //U+BC06
0, //U+BC07
0, //U+BC08
0, //U+BC09
0, //U+BC0A
0, //U+BC0B
0, //U+BC0C
0, //U+BC0D
0, //U+BC0E
0, //U+BC0F
0, //U+BC10
0, //U+BC11
0, //U+BC12
0, //U+BC13
0, //U+BC14
0, //U+BC15
0, //U+BC16
0, //U+BC17
0, //U+BC18
0, //U+BC19
0, //U+BC1A
0, //U+BC1B
0, //U+BC1C
0, //U+BC1D
0, //U+BC1E
0, //U+BC1F
0, //U+BC20
0, //U+BC21
0, //U+BC22
0, //U+BC23
0, //U+BC24
0, //U+BC25
0, //U+BC26
0, //U+BC27
0, //U+BC28
0, //U+BC29
0, //U+BC2A
0, //U+BC2B
0, //U+BC2C
0, //U+BC2D
0, //U+BC2E
0, //U+BC2F
0, //U+BC30
0, //U+BC31
0, //U+BC32
0, //U+BC33
0, //U+BC34
0, //U+BC35
0, //U+BC36
0, //U+BC37
0, //U+BC38
0, //U+BC39
0, //U+BC3A
0, //U+BC3B
0, //U+BC3C
0, //U+BC3D
0, //U+BC3E
0, //U+BC3F
0, //U+BC40
0, //U+BC41
0, //U+BC42
0, //U+BC43
0, //U+BC44
0, //U+BC45
0, //U+BC46
0, //U+BC47
0, //U+BC48
0, //U+BC49
0, //U+BC4A
0, //U+BC4B
0, //U+BC4C
0, //U+BC4D
0, //U+BC4E
0, //U+BC4F
0, //U+BC50
0, //U+BC51
0, //U+BC52
0, //U+BC53
0, //U+BC54
0, //U+BC55
0, //U+BC56
0, //U+BC57
0, //U+BC58
0, //U+BC59
0, //U+BC5A
0, //U+BC5B
0, //U+BC5C
0, //U+BC5D
0, //U+BC5E
0, //U+BC5F
0, //U+BC60
0, //U+BC61
0, //U+BC62
0, //U+BC63
0, //U+BC64
0, //U+BC65
0, //U+BC66
0, //U+BC67
0, //U+BC68
0, //U+BC69
0, //U+BC6A
0, //U+BC6B
0, //U+BC6C
0, //U+BC6D
0, //U+BC6E
0, //U+BC6F
0, //U+BC70
0, //U+BC71
0, //U+BC72
0, //U+BC73
0, //U+BC74
0, //U+BC75
0, //U+BC76
0, //U+BC77
0, //U+BC78
0, //U+BC79
0, //U+BC7A
0, //U+BC7B
0, //U+BC7C
0, //U+BC7D
0, //U+BC7E
0, //U+BC7F
0, //U+BC80
0, //U+BC81
0, //U+BC82
0, //U+BC83
0, //U+BC84
0, //U+BC85
0, //U+BC86
0, //U+BC87
0, //U+BC88
0, //U+BC89
0, //U+BC8A
0, //U+BC8B
0, //U+BC8C
0, //U+BC8D
0, //U+BC8E
0, //U+BC8F
0, //U+BC90
0, //U+BC91
0, //U+BC92
0, //U+BC93
0, //U+BC94
0, //U+BC95
0, //U+BC96
0, //U+BC97
0, //U+BC98
0, //U+BC99
0, //U+BC9A
0, //U+BC9B
0, //U+BC9C
0, //U+BC9D
0, //U+BC9E
0, //U+BC9F
0, //U+BCA0
0, //U+BCA1
0, //U+BCA2
0, //U+BCA3
0, //U+BCA4
0, //U+BCA5
0, //U+BCA6
0, //U+BCA7
0, //U+BCA8
0, //U+BCA9
0, //U+BCAA
0, //U+BCAB
0, //U+BCAC
0, //U+BCAD
0, //U+BCAE
0, //U+BCAF
0, //U+BCB0
0, //U+BCB1
0, //U+BCB2
0, //U+BCB3
0, //U+BCB4
0, //U+BCB5
0, //U+BCB6
0, //U+BCB7
0, //U+BCB8
0, //U+BCB9
0, //U+BCBA
0, //U+BCBB
0, //U+BCBC
0, //U+BCBD
0, //U+BCBE
0, //U+BCBF
0, //U+BCC0
0, //U+BCC1
0, //U+BCC2
0, //U+BCC3
0, //U+BCC4
0, //U+BCC5
0, //U+BCC6
0, //U+BCC7
0, //U+BCC8
0, //U+BCC9
0, //U+BCCA
0, //U+BCCB
0, //U+BCCC
0, //U+BCCD
0, //U+BCCE
0, //U+BCCF
0, //U+BCD0
0, //U+BCD1
0, //U+BCD2
0, //U+BCD3
0, //U+BCD4
0, //U+BCD5
0, //U+BCD6
0, //U+BCD7
0, //U+BCD8
0, //U+BCD9
0, //U+BCDA
0, //U+BCDB
0, //U+BCDC
0, //U+BCDD
0, //U+BCDE
0, //U+BCDF
0, //U+BCE0
0, //U+BCE1
0, //U+BCE2
0, //U+BCE3
0, //U+BCE4
0, //U+BCE5
0, //U+BCE6
0, //U+BCE7
0, //U+BCE8
0, //U+BCE9
0, //U+BCEA
0, //U+BCEB
0, //U+BCEC
0, //U+BCED
0, //U+BCEE
0, //U+BCEF
0, //U+BCF0
0, //U+BCF1
0, //U+BCF2
0, //U+BCF3
0, //U+BCF4
0, //U+BCF5
0, //U+BCF6
0, //U+BCF7
0, //U+BCF8
0, //U+BCF9
0, //U+BCFA
0, //U+BCFB
0, //U+BCFC
0, //U+BCFD
0, //U+BCFE
0, //U+BCFF
0, //U+BD00
0, //U+BD01
0, //U+BD02
0, //U+BD03
0, //U+BD04
0, //U+BD05
0, //U+BD06
0, //U+BD07
0, //U+BD08
0, //U+BD09
0, //U+BD0A
0, //U+BD0B
0, //U+BD0C
0, //U+BD0D
0, //U+BD0E
0, //U+BD0F
0, //U+BD10
0, //U+BD11
0, //U+BD12
0, //U+BD13
0, //U+BD14
0, //U+BD15
0, //U+BD16
0, //U+BD17
0, //U+BD18
0, //U+BD19
0, //U+BD1A
0, //U+BD1B
0, //U+BD1C
0, //U+BD1D
0, //U+BD1E
0, //U+BD1F
0, //U+BD20
0, //U+BD21
0, //U+BD22
0, //U+BD23
0, //U+BD24
0, //U+BD25
0, //U+BD26
0, //U+BD27
0, //U+BD28
0, //U+BD29
0, //U+BD2A
0, //U+BD2B
0, //U+BD2C
0, //U+BD2D
0, //U+BD2E
0, //U+BD2F
0, //U+BD30
0, //U+BD31
0, //U+BD32
0, //U+BD33
0, //U+BD34
0, //U+BD35
0, //U+BD36
0, //U+BD37
0, //U+BD38
0, //U+BD39
0, //U+BD3A
0, //U+BD3B
0, //U+BD3C
0, //U+BD3D
0, //U+BD3E
0, //U+BD3F
0, //U+BD40
0, //U+BD41
0, //U+BD42
0, //U+BD43
0, //U+BD44
0, //U+BD45
0, //U+BD46
0, //U+BD47
0, //U+BD48
0, //U+BD49
0, //U+BD4A
0, //U+BD4B
0, //U+BD4C
0, //U+BD4D
0, //U+BD4E
0, //U+BD4F
0, //U+BD50
0, //U+BD51
0, //U+BD52
0, //U+BD53
0, //U+BD54
0, //U+BD55
0, //U+BD56
0, //U+BD57
0, //U+BD58
0, //U+BD59
0, //U+BD5A
0, //U+BD5B
0, //U+BD5C
0, //U+BD5D
0, //U+BD5E
0, //U+BD5F
0, //U+BD60
0, //U+BD61
0, //U+BD62
0, //U+BD63
0, //U+BD64
0, //U+BD65
0, //U+BD66
0, //U+BD67
0, //U+BD68
0, //U+BD69
0, //U+BD6A
0, //U+BD6B
0, //U+BD6C
0, //U+BD6D
0, //U+BD6E
0, //U+BD6F
0, //U+BD70
0, //U+BD71
0, //U+BD72
0, //U+BD73
0, //U+BD74
0, //U+BD75
0, //U+BD76
0, //U+BD77
0, //U+BD78
0, //U+BD79
0, //U+BD7A
0, //U+BD7B
0, //U+BD7C
0, //U+BD7D
0, //U+BD7E
0, //U+BD7F
0, //U+BD80
0, //U+BD81
0, //U+BD82
0, //U+BD83
0, //U+BD84
0, //U+BD85
0, //U+BD86
0, //U+BD87
0, //U+BD88
0, //U+BD89
0, //U+BD8A
0, //U+BD8B
0, //U+BD8C
0, //U+BD8D
0, //U+BD8E
0, //U+BD8F
0, //U+BD90
0, //U+BD91
0, //U+BD92
0, //U+BD93
0, //U+BD94
0, //U+BD95
0, //U+BD96
0, //U+BD97
0, //U+BD98
0, //U+BD99
0, //U+BD9A
0, //U+BD9B
0, //U+BD9C
0, //U+BD9D
0, //U+BD9E
0, //U+BD9F
0, //U+BDA0
0, //U+BDA1
0, //U+BDA2
0, //U+BDA3
0, //U+BDA4
0, //U+BDA5
0, //U+BDA6
0, //U+BDA7
0, //U+BDA8
0, //U+BDA9
0, //U+BDAA
0, //U+BDAB
0, //U+BDAC
0, //U+BDAD
0, //U+BDAE
0, //U+BDAF
0, //U+BDB0
0, //U+BDB1
0, //U+BDB2
0, //U+BDB3
0, //U+BDB4
0, //U+BDB5
0, //U+BDB6
0, //U+BDB7
0, //U+BDB8
0, //U+BDB9
0, //U+BDBA
0, //U+BDBB
0, //U+BDBC
0, //U+BDBD
0, //U+BDBE
0, //U+BDBF
0, //U+BDC0
0, //U+BDC1
0, //U+BDC2
0, //U+BDC3
0, //U+BDC4
0, //U+BDC5
0, //U+BDC6
0, //U+BDC7
0, //U+BDC8
0, //U+BDC9
0, //U+BDCA
0, //U+BDCB
0, //U+BDCC
0, //U+BDCD
0, //U+BDCE
0, //U+BDCF
0, //U+BDD0
0, //U+BDD1
0, //U+BDD2
0, //U+BDD3
0, //U+BDD4
0, //U+BDD5
0, //U+BDD6
0, //U+BDD7
0, //U+BDD8
0, //U+BDD9
0, //U+BDDA
0, //U+BDDB
0, //U+BDDC
0, //U+BDDD
0, //U+BDDE
0, //U+BDDF
0, //U+BDE0
0, //U+BDE1
0, //U+BDE2
0, //U+BDE3
0, //U+BDE4
0, //U+BDE5
0, //U+BDE6
0, //U+BDE7
0, //U+BDE8
0, //U+BDE9
0, //U+BDEA
0, //U+BDEB
0, //U+BDEC
0, //U+BDED
0, //U+BDEE
0, //U+BDEF
0, //U+BDF0
0, //U+BDF1
0, //U+BDF2
0, //U+BDF3
0, //U+BDF4
0, //U+BDF5
0, //U+BDF6
0, //U+BDF7
0, //U+BDF8
0, //U+BDF9
0, //U+BDFA
0, //U+BDFB
0, //U+BDFC
0, //U+BDFD
0, //U+BDFE
0, //U+BDFF
0, //U+BE00
0, //U+BE01
0, //U+BE02
0, //U+BE03
0, //U+BE04
0, //U+BE05
0, //U+BE06
0, //U+BE07
0, //U+BE08
0, //U+BE09
0, //U+BE0A
0, //U+BE0B
0, //U+BE0C
0, //U+BE0D
0, //U+BE0E
0, //U+BE0F
0, //U+BE10
0, //U+BE11
0, //U+BE12
0, //U+BE13
0, //U+BE14
0, //U+BE15
0, //U+BE16
0, //U+BE17
0, //U+BE18
0, //U+BE19
0, //U+BE1A
0, //U+BE1B
0, //U+BE1C
0, //U+BE1D
0, //U+BE1E
0, //U+BE1F
0, //U+BE20
0, //U+BE21
0, //U+BE22
0, //U+BE23
0, //U+BE24
0, //U+BE25
0, //U+BE26
0, //U+BE27
0, //U+BE28
0, //U+BE29
0, //U+BE2A
0, //U+BE2B
0, //U+BE2C
0, //U+BE2D
0, //U+BE2E
0, //U+BE2F
0, //U+BE30
0, //U+BE31
0, //U+BE32
0, //U+BE33
0, //U+BE34
0, //U+BE35
0, //U+BE36
0, //U+BE37
0, //U+BE38
0, //U+BE39
0, //U+BE3A
0, //U+BE3B
0, //U+BE3C
0, //U+BE3D
0, //U+BE3E
0, //U+BE3F
0, //U+BE40
0, //U+BE41
0, //U+BE42
0, //U+BE43
0, //U+BE44
0, //U+BE45
0, //U+BE46
0, //U+BE47
0, //U+BE48
0, //U+BE49
0, //U+BE4A
0, //U+BE4B
0, //U+BE4C
0, //U+BE4D
0, //U+BE4E
0, //U+BE4F
0, //U+BE50
0, //U+BE51
0, //U+BE52
0, //U+BE53
0, //U+BE54
0, //U+BE55
0, //U+BE56
0, //U+BE57
0, //U+BE58
0, //U+BE59
0, //U+BE5A
0, //U+BE5B
0, //U+BE5C
0, //U+BE5D
0, //U+BE5E
0, //U+BE5F
0, //U+BE60
0, //U+BE61
0, //U+BE62
0, //U+BE63
0, //U+BE64
0, //U+BE65
0, //U+BE66
0, //U+BE67
0, //U+BE68
0, //U+BE69
0, //U+BE6A
0, //U+BE6B
0, //U+BE6C
0, //U+BE6D
0, //U+BE6E
0, //U+BE6F
0, //U+BE70
0, //U+BE71
0, //U+BE72
0, //U+BE73
0, //U+BE74
0, //U+BE75
0, //U+BE76
0, //U+BE77
0, //U+BE78
0, //U+BE79
0, //U+BE7A
0, //U+BE7B
0, //U+BE7C
0, //U+BE7D
0, //U+BE7E
0, //U+BE7F
0, //U+BE80
0, //U+BE81
0, //U+BE82
0, //U+BE83
0, //U+BE84
0, //U+BE85
0, //U+BE86
0, //U+BE87
0, //U+BE88
0, //U+BE89
0, //U+BE8A
0, //U+BE8B
0, //U+BE8C
0, //U+BE8D
0, //U+BE8E
0, //U+BE8F
0, //U+BE90
0, //U+BE91
0, //U+BE92
0, //U+BE93
0, //U+BE94
0, //U+BE95
0, //U+BE96
0, //U+BE97
0, //U+BE98
0, //U+BE99
0, //U+BE9A
0, //U+BE9B
0, //U+BE9C
0, //U+BE9D
0, //U+BE9E
0, //U+BE9F
0, //U+BEA0
0, //U+BEA1
0, //U+BEA2
0, //U+BEA3
0, //U+BEA4
0, //U+BEA5
0, //U+BEA6
0, //U+BEA7
0, //U+BEA8
0, //U+BEA9
0, //U+BEAA
0, //U+BEAB
0, //U+BEAC
0, //U+BEAD
0, //U+BEAE
0, //U+BEAF
0, //U+BEB0
0, //U+BEB1
0, //U+BEB2
0, //U+BEB3
0, //U+BEB4
0, //U+BEB5
0, //U+BEB6
0, //U+BEB7
0, //U+BEB8
0, //U+BEB9
0, //U+BEBA
0, //U+BEBB
0, //U+BEBC
0, //U+BEBD
0, //U+BEBE
0, //U+BEBF
0, //U+BEC0
0, //U+BEC1
0, //U+BEC2
0, //U+BEC3
0, //U+BEC4
0, //U+BEC5
0, //U+BEC6
0, //U+BEC7
0, //U+BEC8
0, //U+BEC9
0, //U+BECA
0, //U+BECB
0, //U+BECC
0, //U+BECD
0, //U+BECE
0, //U+BECF
0, //U+BED0
0, //U+BED1
0, //U+BED2
0, //U+BED3
0, //U+BED4
0, //U+BED5
0, //U+BED6
0, //U+BED7
0, //U+BED8
0, //U+BED9
0, //U+BEDA
0, //U+BEDB
0, //U+BEDC
0, //U+BEDD
0, //U+BEDE
0, //U+BEDF
0, //U+BEE0
0, //U+BEE1
0, //U+BEE2
0, //U+BEE3
0, //U+BEE4
0, //U+BEE5
0, //U+BEE6
0, //U+BEE7
0, //U+BEE8
0, //U+BEE9
0, //U+BEEA
0, //U+BEEB
0, //U+BEEC
0, //U+BEED
0, //U+BEEE
0, //U+BEEF
0, //U+BEF0
0, //U+BEF1
0, //U+BEF2
0, //U+BEF3
0, //U+BEF4
0, //U+BEF5
0, //U+BEF6
0, //U+BEF7
0, //U+BEF8
0, //U+BEF9
0, //U+BEFA
0, //U+BEFB
0, //U+BEFC
0, //U+BEFD
0, //U+BEFE
0, //U+BEFF
0, //U+BF00
0, //U+BF01
0, //U+BF02
0, //U+BF03
0, //U+BF04
0, //U+BF05
0, //U+BF06
0, //U+BF07
0, //U+BF08
0, //U+BF09
0, //U+BF0A
0, //U+BF0B
0, //U+BF0C
0, //U+BF0D
0, //U+BF0E
0, //U+BF0F
0, //U+BF10
0, //U+BF11
0, //U+BF12
0, //U+BF13
0, //U+BF14
0, //U+BF15
0, //U+BF16
0, //U+BF17
0, //U+BF18
0, //U+BF19
0, //U+BF1A
0, //U+BF1B
0, //U+BF1C
0, //U+BF1D
0, //U+BF1E
0, //U+BF1F
0, //U+BF20
0, //U+BF21
0, //U+BF22
0, //U+BF23
0, //U+BF24
0, //U+BF25
0, //U+BF26
0, //U+BF27
0, //U+BF28
0, //U+BF29
0, //U+BF2A
0, //U+BF2B
0, //U+BF2C
0, //U+BF2D
0, //U+BF2E
0, //U+BF2F
0, //U+BF30
0, //U+BF31
0, //U+BF32
0, //U+BF33
0, //U+BF34
0, //U+BF35
0, //U+BF36
0, //U+BF37
0, //U+BF38
0, //U+BF39
0, //U+BF3A
0, //U+BF3B
0, //U+BF3C
0, //U+BF3D
0, //U+BF3E
0, //U+BF3F
0, //U+BF40
0, //U+BF41
0, //U+BF42
0, //U+BF43
0, //U+BF44
0, //U+BF45
0, //U+BF46
0, //U+BF47
0, //U+BF48
0, //U+BF49
0, //U+BF4A
0, //U+BF4B
0, //U+BF4C
0, //U+BF4D
0, //U+BF4E
0, //U+BF4F
0, //U+BF50
0, //U+BF51
0, //U+BF52
0, //U+BF53
0, //U+BF54
0, //U+BF55
0, //U+BF56
0, //U+BF57
0, //U+BF58
0, //U+BF59
0, //U+BF5A
0, //U+BF5B
0, //U+BF5C
0, //U+BF5D
0, //U+BF5E
0, //U+BF5F
0, //U+BF60
0, //U+BF61
0, //U+BF62
0, //U+BF63
0, //U+BF64
0, //U+BF65
0, //U+BF66
0, //U+BF67
0, //U+BF68
0, //U+BF69
0, //U+BF6A
0, //U+BF6B
0, //U+BF6C
0, //U+BF6D
0, //U+BF6E
0, //U+BF6F
0, //U+BF70
0, //U+BF71
0, //U+BF72
0, //U+BF73
0, //U+BF74
0, //U+BF75
0, //U+BF76
0, //U+BF77
0, //U+BF78
0, //U+BF79
0, //U+BF7A
0, //U+BF7B
0, //U+BF7C
0, //U+BF7D
0, //U+BF7E
0, //U+BF7F
0, //U+BF80
0, //U+BF81
0, //U+BF82
0, //U+BF83
0, //U+BF84
0, //U+BF85
0, //U+BF86
0, //U+BF87
0, //U+BF88
0, //U+BF89
0, //U+BF8A
0, //U+BF8B
0, //U+BF8C
0, //U+BF8D
0, //U+BF8E
0, //U+BF8F
0, //U+BF90
0, //U+BF91
0, //U+BF92
0, //U+BF93
0, //U+BF94
0, //U+BF95
0, //U+BF96
0, //U+BF97
0, //U+BF98
0, //U+BF99
0, //U+BF9A
0, //U+BF9B
0, //U+BF9C
0, //U+BF9D
0, //U+BF9E
0, //U+BF9F
0, //U+BFA0
0, //U+BFA1
0, //U+BFA2
0, //U+BFA3
0, //U+BFA4
0, //U+BFA5
0, //U+BFA6
0, //U+BFA7
0, //U+BFA8
0, //U+BFA9
0, //U+BFAA
0, //U+BFAB
0, //U+BFAC
0, //U+BFAD
0, //U+BFAE
0, //U+BFAF
0, //U+BFB0
0, //U+BFB1
0, //U+BFB2
0, //U+BFB3
0, //U+BFB4
0, //U+BFB5
0, //U+BFB6
0, //U+BFB7
0, //U+BFB8
0, //U+BFB9
0, //U+BFBA
0, //U+BFBB
0, //U+BFBC
0, //U+BFBD
0, //U+BFBE
0, //U+BFBF
0, //U+BFC0
0, //U+BFC1
0, //U+BFC2
0, //U+BFC3
0, //U+BFC4
0, //U+BFC5
0, //U+BFC6
0, //U+BFC7
0, //U+BFC8
0, //U+BFC9
0, //U+BFCA
0, //U+BFCB
0, //U+BFCC
0, //U+BFCD
0, //U+BFCE
0, //U+BFCF
0, //U+BFD0
0, //U+BFD1
0, //U+BFD2
0, //U+BFD3
0, //U+BFD4
0, //U+BFD5
0, //U+BFD6
0, //U+BFD7
0, //U+BFD8
0, //U+BFD9
0, //U+BFDA
0, //U+BFDB
0, //U+BFDC
0, //U+BFDD
0, //U+BFDE
0, //U+BFDF
0, //U+BFE0
0, //U+BFE1
0, //U+BFE2
0, //U+BFE3
0, //U+BFE4
0, //U+BFE5
0, //U+BFE6
0, //U+BFE7
0, //U+BFE8
0, //U+BFE9
0, //U+BFEA
0, //U+BFEB
0, //U+BFEC
0, //U+BFED
0, //U+BFEE
0, //U+BFEF
0, //U+BFF0
0, //U+BFF1
0, //U+BFF2
0, //U+BFF3
0, //U+BFF4
0, //U+BFF5
0, //U+BFF6
0, //U+BFF7
0, //U+BFF8
0, //U+BFF9
0, //U+BFFA
0, //U+BFFB
0, //U+BFFC
0, //U+BFFD
0, //U+BFFE
0, //U+BFFF
0, //U+C000
0, //U+C001
0, //U+C002
0, //U+C003
0, //U+C004
0, //U+C005
0, //U+C006
0, //U+C007
0, //U+C008
0, //U+C009
0, //U+C00A
0, //U+C00B
0, //U+C00C
0, //U+C00D
0, //U+C00E
0, //U+C00F
0, //U+C010
0, //U+C011
0, //U+C012
0, //U+C013
0, //U+C014
0, //U+C015
0, //U+C016
0, //U+C017
0, //U+C018
0, //U+C019
0, //U+C01A
0, //U+C01B
0, //U+C01C
0, //U+C01D
0, //U+C01E
0, //U+C01F
0, //U+C020
0, //U+C021
0, //U+C022
0, //U+C023
0, //U+C024
0, //U+C025
0, //U+C026
0, //U+C027
0, //U+C028
0, //U+C029
0, //U+C02A
0, //U+C02B
0, //U+C02C
0, //U+C02D
0, //U+C02E
0, //U+C02F
0, //U+C030
0, //U+C031
0, //U+C032
0, //U+C033
0, //U+C034
0, //U+C035
0, //U+C036
0, //U+C037
0, //U+C038
0, //U+C039
0, //U+C03A
0, //U+C03B
0, //U+C03C
0, //U+C03D
0, //U+C03E
0, //U+C03F
0, //U+C040
0, //U+C041
0, //U+C042
0, //U+C043
0, //U+C044
0, //U+C045
0, //U+C046
0, //U+C047
0, //U+C048
0, //U+C049
0, //U+C04A
0, //U+C04B
0, //U+C04C
0, //U+C04D
0, //U+C04E
0, //U+C04F
0, //U+C050
0, //U+C051
0, //U+C052
0, //U+C053
0, //U+C054
0, //U+C055
0, //U+C056
0, //U+C057
0, //U+C058
0, //U+C059
0, //U+C05A
0, //U+C05B
0, //U+C05C
0, //U+C05D
0, //U+C05E
0, //U+C05F
0, //U+C060
0, //U+C061
0, //U+C062
0, //U+C063
0, //U+C064
0, //U+C065
0, //U+C066
0, //U+C067
0, //U+C068
0, //U+C069
0, //U+C06A
0, //U+C06B
0, //U+C06C
0, //U+C06D
0, //U+C06E
0, //U+C06F
0, //U+C070
0, //U+C071
0, //U+C072
0, //U+C073
0, //U+C074
0, //U+C075
0, //U+C076
0, //U+C077
0, //U+C078
0, //U+C079
0, //U+C07A
0, //U+C07B
0, //U+C07C
0, //U+C07D
0, //U+C07E
0, //U+C07F
0, //U+C080
0, //U+C081
0, //U+C082
0, //U+C083
0, //U+C084
0, //U+C085
0, //U+C086
0, //U+C087
0, //U+C088
0, //U+C089
0, //U+C08A
0, //U+C08B
0, //U+C08C
0, //U+C08D
0, //U+C08E
0, //U+C08F
0, //U+C090
0, //U+C091
0, //U+C092
0, //U+C093
0, //U+C094
0, //U+C095
0, //U+C096
0, //U+C097
0, //U+C098
0, //U+C099
0, //U+C09A
0, //U+C09B
0, //U+C09C
0, //U+C09D
0, //U+C09E
0, //U+C09F
0, //U+C0A0
0, //U+C0A1
0, //U+C0A2
0, //U+C0A3
0, //U+C0A4
0, //U+C0A5
0, //U+C0A6
0, //U+C0A7
0, //U+C0A8
0, //U+C0A9
0, //U+C0AA
0, //U+C0AB
0, //U+C0AC
0, //U+C0AD
0, //U+C0AE
0, //U+C0AF
0, //U+C0B0
0, //U+C0B1
0, //U+C0B2
0, //U+C0B3
0, //U+C0B4
0, //U+C0B5
0, //U+C0B6
0, //U+C0B7
0, //U+C0B8
0, //U+C0B9
0, //U+C0BA
0, //U+C0BB
0, //U+C0BC
0, //U+C0BD
0, //U+C0BE
0, //U+C0BF
0, //U+C0C0
0, //U+C0C1
0, //U+C0C2
0, //U+C0C3
0, //U+C0C4
0, //U+C0C5
0, //U+C0C6
0, //U+C0C7
0, //U+C0C8
0, //U+C0C9
0, //U+C0CA
0, //U+C0CB
0, //U+C0CC
0, //U+C0CD
0, //U+C0CE
0, //U+C0CF
0, //U+C0D0
0, //U+C0D1
0, //U+C0D2
0, //U+C0D3
0, //U+C0D4
0, //U+C0D5
0, //U+C0D6
0, //U+C0D7
0, //U+C0D8
0, //U+C0D9
0, //U+C0DA
0, //U+C0DB
0, //U+C0DC
0, //U+C0DD
0, //U+C0DE
0, //U+C0DF
0, //U+C0E0
0, //U+C0E1
0, //U+C0E2
0, //U+C0E3
0, //U+C0E4
0, //U+C0E5
0, //U+C0E6
0, //U+C0E7
0, //U+C0E8
0, //U+C0E9
0, //U+C0EA
0, //U+C0EB
0, //U+C0EC
0, //U+C0ED
0, //U+C0EE
0, //U+C0EF
0, //U+C0F0
0, //U+C0F1
0, //U+C0F2
0, //U+C0F3
0, //U+C0F4
0, //U+C0F5
0, //U+C0F6
0, //U+C0F7
0, //U+C0F8
0, //U+C0F9
0, //U+C0FA
0, //U+C0FB
0, //U+C0FC
0, //U+C0FD
0, //U+C0FE
0, //U+C0FF
0, //U+C100
0, //U+C101
0, //U+C102
0, //U+C103
0, //U+C104
0, //U+C105
0, //U+C106
0, //U+C107
0, //U+C108
0, //U+C109
0, //U+C10A
0, //U+C10B
0, //U+C10C
0, //U+C10D
0, //U+C10E
0, //U+C10F
0, //U+C110
0, //U+C111
0, //U+C112
0, //U+C113
0, //U+C114
0, //U+C115
0, //U+C116
0, //U+C117
0, //U+C118
0, //U+C119
0, //U+C11A
0, //U+C11B
0, //U+C11C
0, //U+C11D
0, //U+C11E
0, //U+C11F
0, //U+C120
0, //U+C121
0, //U+C122
0, //U+C123
0, //U+C124
0, //U+C125
0, //U+C126
0, //U+C127
0, //U+C128
0, //U+C129
0, //U+C12A
0, //U+C12B
0, //U+C12C
0, //U+C12D
0, //U+C12E
0, //U+C12F
0, //U+C130
0, //U+C131
0, //U+C132
0, //U+C133
0, //U+C134
0, //U+C135
0, //U+C136
0, //U+C137
0, //U+C138
0, //U+C139
0, //U+C13A
0, //U+C13B
0, //U+C13C
0, //U+C13D
0, //U+C13E
0, //U+C13F
0, //U+C140
0, //U+C141
0, //U+C142
0, //U+C143
0, //U+C144
0, //U+C145
0, //U+C146
0, //U+C147
0, //U+C148
0, //U+C149
0, //U+C14A
0, //U+C14B
0, //U+C14C
0, //U+C14D
0, //U+C14E
0, //U+C14F
0, //U+C150
0, //U+C151
0, //U+C152
0, //U+C153
0, //U+C154
0, //U+C155
0, //U+C156
0, //U+C157
0, //U+C158
0, //U+C159
0, //U+C15A
0, //U+C15B
0, //U+C15C
0, //U+C15D
0, //U+C15E
0, //U+C15F
0, //U+C160
0, //U+C161
0, //U+C162
0, //U+C163
0, //U+C164
0, //U+C165
0, //U+C166
0, //U+C167
0, //U+C168
0, //U+C169
0, //U+C16A
0, //U+C16B
0, //U+C16C
0, //U+C16D
0, //U+C16E
0, //U+C16F
0, //U+C170
0, //U+C171
0, //U+C172
0, //U+C173
0, //U+C174
0, //U+C175
0, //U+C176
0, //U+C177
0, //U+C178
0, //U+C179
0, //U+C17A
0, //U+C17B
0, //U+C17C
0, //U+C17D
0, //U+C17E
0, //U+C17F
0, //U+C180
0, //U+C181
0, //U+C182
0, //U+C183
0, //U+C184
0, //U+C185
0, //U+C186
0, //U+C187
0, //U+C188
0, //U+C189
0, //U+C18A
0, //U+C18B
0, //U+C18C
0, //U+C18D
0, //U+C18E
0, //U+C18F
0, //U+C190
0, //U+C191
0, //U+C192
0, //U+C193
0, //U+C194
0, //U+C195
0, //U+C196
0, //U+C197
0, //U+C198
0, //U+C199
0, //U+C19A
0, //U+C19B
0, //U+C19C
0, //U+C19D
0, //U+C19E
0, //U+C19F
0, //U+C1A0
0, //U+C1A1
0, //U+C1A2
0, //U+C1A3
0, //U+C1A4
0, //U+C1A5
0, //U+C1A6
0, //U+C1A7
0, //U+C1A8
0, //U+C1A9
0, //U+C1AA
0, //U+C1AB
0, //U+C1AC
0, //U+C1AD
0, //U+C1AE
0, //U+C1AF
0, //U+C1B0
0, //U+C1B1
0, //U+C1B2
0, //U+C1B3
0, //U+C1B4
0, //U+C1B5
0, //U+C1B6
0, //U+C1B7
0, //U+C1B8
0, //U+C1B9
0, //U+C1BA
0, //U+C1BB
0, //U+C1BC
0, //U+C1BD
0, //U+C1BE
0, //U+C1BF
0, //U+C1C0
0, //U+C1C1
0, //U+C1C2
0, //U+C1C3
0, //U+C1C4
0, //U+C1C5
0, //U+C1C6
0, //U+C1C7
0, //U+C1C8
0, //U+C1C9
0, //U+C1CA
0, //U+C1CB
0, //U+C1CC
0, //U+C1CD
0, //U+C1CE
0, //U+C1CF
0, //U+C1D0
0, //U+C1D1
0, //U+C1D2
0, //U+C1D3
0, //U+C1D4
0, //U+C1D5
0, //U+C1D6
0, //U+C1D7
0, //U+C1D8
0, //U+C1D9
0, //U+C1DA
0, //U+C1DB
0, //U+C1DC
0, //U+C1DD
0, //U+C1DE
0, //U+C1DF
0, //U+C1E0
0, //U+C1E1
0, //U+C1E2
0, //U+C1E3
0, //U+C1E4
0, //U+C1E5
0, //U+C1E6
0, //U+C1E7
0, //U+C1E8
0, //U+C1E9
0, //U+C1EA
0, //U+C1EB
0, //U+C1EC
0, //U+C1ED
0, //U+C1EE
0, //U+C1EF
0, //U+C1F0
0, //U+C1F1
0, //U+C1F2
0, //U+C1F3
0, //U+C1F4
0, //U+C1F5
0, //U+C1F6
0, //U+C1F7
0, //U+C1F8
0, //U+C1F9
0, //U+C1FA
0, //U+C1FB
0, //U+C1FC
0, //U+C1FD
0, //U+C1FE
0, //U+C1FF
0, //U+C200
0, //U+C201
0, //U+C202
0, //U+C203
0, //U+C204
0, //U+C205
0, //U+C206
0, //U+C207
0, //U+C208
0, //U+C209
0, //U+C20A
0, //U+C20B
0, //U+C20C
0, //U+C20D
0, //U+C20E
0, //U+C20F
0, //U+C210
0, //U+C211
0, //U+C212
0, //U+C213
0, //U+C214
0, //U+C215
0, //U+C216
0, //U+C217
0, //U+C218
0, //U+C219
0, //U+C21A
0, //U+C21B
0, //U+C21C
0, //U+C21D
0, //U+C21E
0, //U+C21F
0, //U+C220
0, //U+C221
0, //U+C222
0, //U+C223
0, //U+C224
0, //U+C225
0, //U+C226
0, //U+C227
0, //U+C228
0, //U+C229
0, //U+C22A
0, //U+C22B
0, //U+C22C
0, //U+C22D
0, //U+C22E
0, //U+C22F
0, //U+C230
0, //U+C231
0, //U+C232
0, //U+C233
0, //U+C234
0, //U+C235
0, //U+C236
0, //U+C237
0, //U+C238
0, //U+C239
0, //U+C23A
0, //U+C23B
0, //U+C23C
0, //U+C23D
0, //U+C23E
0, //U+C23F
0, //U+C240
0, //U+C241
0, //U+C242
0, //U+C243
0, //U+C244
0, //U+C245
0, //U+C246
0, //U+C247
0, //U+C248
0, //U+C249
0, //U+C24A
0, //U+C24B
0, //U+C24C
0, //U+C24D
0, //U+C24E
0, //U+C24F
0, //U+C250
0, //U+C251
0, //U+C252
0, //U+C253
0, //U+C254
0, //U+C255
0, //U+C256
0, //U+C257
0, //U+C258
0, //U+C259
0, //U+C25A
0, //U+C25B
0, //U+C25C
0, //U+C25D
0, //U+C25E
0, //U+C25F
0, //U+C260
0, //U+C261
0, //U+C262
0, //U+C263
0, //U+C264
0, //U+C265
0, //U+C266
0, //U+C267
0, //U+C268
0, //U+C269
0, //U+C26A
0, //U+C26B
0, //U+C26C
0, //U+C26D
0, //U+C26E
0, //U+C26F
0, //U+C270
0, //U+C271
0, //U+C272
0, //U+C273
0, //U+C274
0, //U+C275
0, //U+C276
0, //U+C277
0, //U+C278
0, //U+C279
0, //U+C27A
0, //U+C27B
0, //U+C27C
0, //U+C27D
0, //U+C27E
0, //U+C27F
0, //U+C280
0, //U+C281
0, //U+C282
0, //U+C283
0, //U+C284
0, //U+C285
0, //U+C286
0, //U+C287
0, //U+C288
0, //U+C289
0, //U+C28A
0, //U+C28B
0, //U+C28C
0, //U+C28D
0, //U+C28E
0, //U+C28F
0, //U+C290
0, //U+C291
0, //U+C292
0, //U+C293
0, //U+C294
0, //U+C295
0, //U+C296
0, //U+C297
0, //U+C298
0, //U+C299
0, //U+C29A
0, //U+C29B
0, //U+C29C
0, //U+C29D
0, //U+C29E
0, //U+C29F
0, //U+C2A0
0, //U+C2A1
0, //U+C2A2
0, //U+C2A3
0, //U+C2A4
0, //U+C2A5
0, //U+C2A6
0, //U+C2A7
0, //U+C2A8
0, //U+C2A9
0, //U+C2AA
0, //U+C2AB
0, //U+C2AC
0, //U+C2AD
0, //U+C2AE
0, //U+C2AF
0, //U+C2B0
0, //U+C2B1
0, //U+C2B2
0, //U+C2B3
0, //U+C2B4
0, //U+C2B5
0, //U+C2B6
0, //U+C2B7
0, //U+C2B8
0, //U+C2B9
0, //U+C2BA
0, //U+C2BB
0, //U+C2BC
0, //U+C2BD
0, //U+C2BE
0, //U+C2BF
0, //U+C2C0
0, //U+C2C1
0, //U+C2C2
0, //U+C2C3
0, //U+C2C4
0, //U+C2C5
0, //U+C2C6
0, //U+C2C7
0, //U+C2C8
0, //U+C2C9
0, //U+C2CA
0, //U+C2CB
0, //U+C2CC
0, //U+C2CD
0, //U+C2CE
0, //U+C2CF
0, //U+C2D0
0, //U+C2D1
0, //U+C2D2
0, //U+C2D3
0, //U+C2D4
0, //U+C2D5
0, //U+C2D6
0, //U+C2D7
0, //U+C2D8
0, //U+C2D9
0, //U+C2DA
0, //U+C2DB
0, //U+C2DC
0, //U+C2DD
0, //U+C2DE
0, //U+C2DF
0, //U+C2E0
0, //U+C2E1
0, //U+C2E2
0, //U+C2E3
0, //U+C2E4
0, //U+C2E5
0, //U+C2E6
0, //U+C2E7
0, //U+C2E8
0, //U+C2E9
0, //U+C2EA
0, //U+C2EB
0, //U+C2EC
0, //U+C2ED
0, //U+C2EE
0, //U+C2EF
0, //U+C2F0
0, //U+C2F1
0, //U+C2F2
0, //U+C2F3
0, //U+C2F4
0, //U+C2F5
0, //U+C2F6
0, //U+C2F7
0, //U+C2F8
0, //U+C2F9
0, //U+C2FA
0, //U+C2FB
0, //U+C2FC
0, //U+C2FD
0, //U+C2FE
0, //U+C2FF
0, //U+C300
0, //U+C301
0, //U+C302
0, //U+C303
0, //U+C304
0, //U+C305
0, //U+C306
0, //U+C307
0, //U+C308
0, //U+C309
0, //U+C30A
0, //U+C30B
0, //U+C30C
0, //U+C30D
0, //U+C30E
0, //U+C30F
0, //U+C310
0, //U+C311
0, //U+C312
0, //U+C313
0, //U+C314
0, //U+C315
0, //U+C316
0, //U+C317
0, //U+C318
0, //U+C319
0, //U+C31A
0, //U+C31B
0, //U+C31C
0, //U+C31D
0, //U+C31E
0, //U+C31F
0, //U+C320
0, //U+C321
0, //U+C322
0, //U+C323
0, //U+C324
0, //U+C325
0, //U+C326
0, //U+C327
0, //U+C328
0, //U+C329
0, //U+C32A
0, //U+C32B
0, //U+C32C
0, //U+C32D
0, //U+C32E
0, //U+C32F
0, //U+C330
0, //U+C331
0, //U+C332
0, //U+C333
0, //U+C334
0, //U+C335
0, //U+C336
0, //U+C337
0, //U+C338
0, //U+C339
0, //U+C33A
0, //U+C33B
0, //U+C33C
0, //U+C33D
0, //U+C33E
0, //U+C33F
0, //U+C340
0, //U+C341
0, //U+C342
0, //U+C343
0, //U+C344
0, //U+C345
0, //U+C346
0, //U+C347
0, //U+C348
0, //U+C349
0, //U+C34A
0, //U+C34B
0, //U+C34C
0, //U+C34D
0, //U+C34E
0, //U+C34F
0, //U+C350
0, //U+C351
0, //U+C352
0, //U+C353
0, //U+C354
0, //U+C355
0, //U+C356
0, //U+C357
0, //U+C358
0, //U+C359
0, //U+C35A
0, //U+C35B
0, //U+C35C
0, //U+C35D
0, //U+C35E
0, //U+C35F
0, //U+C360
0, //U+C361
0, //U+C362
0, //U+C363
0, //U+C364
0, //U+C365
0, //U+C366
0, //U+C367
0, //U+C368
0, //U+C369
0, //U+C36A
0, //U+C36B
0, //U+C36C
0, //U+C36D
0, //U+C36E
0, //U+C36F
0, //U+C370
0, //U+C371
0, //U+C372
0, //U+C373
0, //U+C374
0, //U+C375
0, //U+C376
0, //U+C377
0, //U+C378
0, //U+C379
0, //U+C37A
0, //U+C37B
0, //U+C37C
0, //U+C37D
0, //U+C37E
0, //U+C37F
0, //U+C380
0, //U+C381
0, //U+C382
0, //U+C383
0, //U+C384
0, //U+C385
0, //U+C386
0, //U+C387
0, //U+C388
0, //U+C389
0, //U+C38A
0, //U+C38B
0, //U+C38C
0, //U+C38D
0, //U+C38E
0, //U+C38F
0, //U+C390
0, //U+C391
0, //U+C392
0, //U+C393
0, //U+C394
0, //U+C395
0, //U+C396
0, //U+C397
0, //U+C398
0, //U+C399
0, //U+C39A
0, //U+C39B
0, //U+C39C
0, //U+C39D
0, //U+C39E
0, //U+C39F
0, //U+C3A0
0, //U+C3A1
0, //U+C3A2
0, //U+C3A3
0, //U+C3A4
0, //U+C3A5
0, //U+C3A6
0, //U+C3A7
0, //U+C3A8
0, //U+C3A9
0, //U+C3AA
0, //U+C3AB
0, //U+C3AC
0, //U+C3AD
0, //U+C3AE
0, //U+C3AF
0, //U+C3B0
0, //U+C3B1
0, //U+C3B2
0, //U+C3B3
0, //U+C3B4
0, //U+C3B5
0, //U+C3B6
0, //U+C3B7
0, //U+C3B8
0, //U+C3B9
0, //U+C3BA
0, //U+C3BB
0, //U+C3BC
0, //U+C3BD
0, //U+C3BE
0, //U+C3BF
0, //U+C3C0
0, //U+C3C1
0, //U+C3C2
0, //U+C3C3
0, //U+C3C4
0, //U+C3C5
0, //U+C3C6
0, //U+C3C7
0, //U+C3C8
0, //U+C3C9
0, //U+C3CA
0, //U+C3CB
0, //U+C3CC
0, //U+C3CD
0, //U+C3CE
0, //U+C3CF
0, //U+C3D0
0, //U+C3D1
0, //U+C3D2
0, //U+C3D3
0, //U+C3D4
0, //U+C3D5
0, //U+C3D6
0, //U+C3D7
0, //U+C3D8
0, //U+C3D9
0, //U+C3DA
0, //U+C3DB
0, //U+C3DC
0, //U+C3DD
0, //U+C3DE
0, //U+C3DF
0, //U+C3E0
0, //U+C3E1
0, //U+C3E2
0, //U+C3E3
0, //U+C3E4
0, //U+C3E5
0, //U+C3E6
0, //U+C3E7
0, //U+C3E8
0, //U+C3E9
0, //U+C3EA
0, //U+C3EB
0, //U+C3EC
0, //U+C3ED
0, //U+C3EE
0, //U+C3EF
0, //U+C3F0
0, //U+C3F1
0, //U+C3F2
0, //U+C3F3
0, //U+C3F4
0, //U+C3F5
0, //U+C3F6
0, //U+C3F7
0, //U+C3F8
0, //U+C3F9
0, //U+C3FA
0, //U+C3FB
0, //U+C3FC
0, //U+C3FD
0, //U+C3FE
0, //U+C3FF
0, //U+C400
0, //U+C401
0, //U+C402
0, //U+C403
0, //U+C404
0, //U+C405
0, //U+C406
0, //U+C407
0, //U+C408
0, //U+C409
0, //U+C40A
0, //U+C40B
0, //U+C40C
0, //U+C40D
0, //U+C40E
0, //U+C40F
0, //U+C410
0, //U+C411
0, //U+C412
0, //U+C413
0, //U+C414
0, //U+C415
0, //U+C416
0, //U+C417
0, //U+C418
0, //U+C419
0, //U+C41A
0, //U+C41B
0, //U+C41C
0, //U+C41D
0, //U+C41E
0, //U+C41F
0, //U+C420
0, //U+C421
0, //U+C422
0, //U+C423
0, //U+C424
0, //U+C425
0, //U+C426
0, //U+C427
0, //U+C428
0, //U+C429
0, //U+C42A
0, //U+C42B
0, //U+C42C
0, //U+C42D
0, //U+C42E
0, //U+C42F
0, //U+C430
0, //U+C431
0, //U+C432
0, //U+C433
0, //U+C434
0, //U+C435
0, //U+C436
0, //U+C437
0, //U+C438
0, //U+C439
0, //U+C43A
0, //U+C43B
0, //U+C43C
0, //U+C43D
0, //U+C43E
0, //U+C43F
0, //U+C440
0, //U+C441
0, //U+C442
0, //U+C443
0, //U+C444
0, //U+C445
0, //U+C446
0, //U+C447
0, //U+C448
0, //U+C449
0, //U+C44A
0, //U+C44B
0, //U+C44C
0, //U+C44D
0, //U+C44E
0, //U+C44F
0, //U+C450
0, //U+C451
0, //U+C452
0, //U+C453
0, //U+C454
0, //U+C455
0, //U+C456
0, //U+C457
0, //U+C458
0, //U+C459
0, //U+C45A
0, //U+C45B
0, //U+C45C
0, //U+C45D
0, //U+C45E
0, //U+C45F
0, //U+C460
0, //U+C461
0, //U+C462
0, //U+C463
0, //U+C464
0, //U+C465
0, //U+C466
0, //U+C467
0, //U+C468
0, //U+C469
0, //U+C46A
0, //U+C46B
0, //U+C46C
0, //U+C46D
0, //U+C46E
0, //U+C46F
0, //U+C470
0, //U+C471
0, //U+C472
0, //U+C473
0, //U+C474
0, //U+C475
0, //U+C476
0, //U+C477
0, //U+C478
0, //U+C479
0, //U+C47A
0, //U+C47B
0, //U+C47C
0, //U+C47D
0, //U+C47E
0, //U+C47F
0, //U+C480
0, //U+C481
0, //U+C482
0, //U+C483
0, //U+C484
0, //U+C485
0, //U+C486
0, //U+C487
0, //U+C488
0, //U+C489
0, //U+C48A
0, //U+C48B
0, //U+C48C
0, //U+C48D
0, //U+C48E
0, //U+C48F
0, //U+C490
0, //U+C491
0, //U+C492
0, //U+C493
0, //U+C494
0, //U+C495
0, //U+C496
0, //U+C497
0, //U+C498
0, //U+C499
0, //U+C49A
0, //U+C49B
0, //U+C49C
0, //U+C49D
0, //U+C49E
0, //U+C49F
0, //U+C4A0
0, //U+C4A1
0, //U+C4A2
0, //U+C4A3
0, //U+C4A4
0, //U+C4A5
0, //U+C4A6
0, //U+C4A7
0, //U+C4A8
0, //U+C4A9
0, //U+C4AA
0, //U+C4AB
0, //U+C4AC
0, //U+C4AD
0, //U+C4AE
0, //U+C4AF
0, //U+C4B0
0, //U+C4B1
0, //U+C4B2
0, //U+C4B3
0, //U+C4B4
0, //U+C4B5
0, //U+C4B6
0, //U+C4B7
0, //U+C4B8
0, //U+C4B9
0, //U+C4BA
0, //U+C4BB
0, //U+C4BC
0, //U+C4BD
0, //U+C4BE
0, //U+C4BF
0, //U+C4C0
0, //U+C4C1
0, //U+C4C2
0, //U+C4C3
0, //U+C4C4
0, //U+C4C5
0, //U+C4C6
0, //U+C4C7
0, //U+C4C8
0, //U+C4C9
0, //U+C4CA
0, //U+C4CB
0, //U+C4CC
0, //U+C4CD
0, //U+C4CE
0, //U+C4CF
0, //U+C4D0
0, //U+C4D1
0, //U+C4D2
0, //U+C4D3
0, //U+C4D4
0, //U+C4D5
0, //U+C4D6
0, //U+C4D7
0, //U+C4D8
0, //U+C4D9
0, //U+C4DA
0, //U+C4DB
0, //U+C4DC
0, //U+C4DD
0, //U+C4DE
0, //U+C4DF
0, //U+C4E0
0, //U+C4E1
0, //U+C4E2
0, //U+C4E3
0, //U+C4E4
0, //U+C4E5
0, //U+C4E6
0, //U+C4E7
0, //U+C4E8
0, //U+C4E9
0, //U+C4EA
0, //U+C4EB
0, //U+C4EC
0, //U+C4ED
0, //U+C4EE
0, //U+C4EF
0, //U+C4F0
0, //U+C4F1
0, //U+C4F2
0, //U+C4F3
0, //U+C4F4
0, //U+C4F5
0, //U+C4F6
0, //U+C4F7
0, //U+C4F8
0, //U+C4F9
0, //U+C4FA
0, //U+C4FB
0, //U+C4FC
0, //U+C4FD
0, //U+C4FE
0, //U+C4FF
0, //U+C500
0, //U+C501
0, //U+C502
0, //U+C503
0, //U+C504
0, //U+C505
0, //U+C506
0, //U+C507
0, //U+C508
0, //U+C509
0, //U+C50A
0, //U+C50B
0, //U+C50C
0, //U+C50D
0, //U+C50E
0, //U+C50F
0, //U+C510
0, //U+C511
0, //U+C512
0, //U+C513
0, //U+C514
0, //U+C515
0, //U+C516
0, //U+C517
0, //U+C518
0, //U+C519
0, //U+C51A
0, //U+C51B
0, //U+C51C
0, //U+C51D
0, //U+C51E
0, //U+C51F
0, //U+C520
0, //U+C521
0, //U+C522
0, //U+C523
0, //U+C524
0, //U+C525
0, //U+C526
0, //U+C527
0, //U+C528
0, //U+C529
0, //U+C52A
0, //U+C52B
0, //U+C52C
0, //U+C52D
0, //U+C52E
0, //U+C52F
0, //U+C530
0, //U+C531
0, //U+C532
0, //U+C533
0, //U+C534
0, //U+C535
0, //U+C536
0, //U+C537
0, //U+C538
0, //U+C539
0, //U+C53A
0, //U+C53B
0, //U+C53C
0, //U+C53D
0, //U+C53E
0, //U+C53F
0, //U+C540
0, //U+C541
0, //U+C542
0, //U+C543
0, //U+C544
0, //U+C545
0, //U+C546
0, //U+C547
0, //U+C548
0, //U+C549
0, //U+C54A
0, //U+C54B
0, //U+C54C
0, //U+C54D
0, //U+C54E
0, //U+C54F
0, //U+C550
0, //U+C551
0, //U+C552
0, //U+C553
0, //U+C554
0, //U+C555
0, //U+C556
0, //U+C557
0, //U+C558
0, //U+C559
0, //U+C55A
0, //U+C55B
0, //U+C55C
0, //U+C55D
0, //U+C55E
0, //U+C55F
0, //U+C560
0, //U+C561
0, //U+C562
0, //U+C563
0, //U+C564
0, //U+C565
0, //U+C566
0, //U+C567
0, //U+C568
0, //U+C569
0, //U+C56A
0, //U+C56B
0, //U+C56C
0, //U+C56D
0, //U+C56E
0, //U+C56F
0, //U+C570
0, //U+C571
0, //U+C572
0, //U+C573
0, //U+C574
0, //U+C575
0, //U+C576
0, //U+C577
0, //U+C578
0, //U+C579
0, //U+C57A
0, //U+C57B
0, //U+C57C
0, //U+C57D
0, //U+C57E
0, //U+C57F
0, //U+C580
0, //U+C581
0, //U+C582
0, //U+C583
0, //U+C584
0, //U+C585
0, //U+C586
0, //U+C587
0, //U+C588
0, //U+C589
0, //U+C58A
0, //U+C58B
0, //U+C58C
0, //U+C58D
0, //U+C58E
0, //U+C58F
0, //U+C590
0, //U+C591
0, //U+C592
0, //U+C593
0, //U+C594
0, //U+C595
0, //U+C596
0, //U+C597
0, //U+C598
0, //U+C599
0, //U+C59A
0, //U+C59B
0, //U+C59C
0, //U+C59D
0, //U+C59E
0, //U+C59F
0, //U+C5A0
0, //U+C5A1
0, //U+C5A2
0, //U+C5A3
0, //U+C5A4
0, //U+C5A5
0, //U+C5A6
0, //U+C5A7
0, //U+C5A8
0, //U+C5A9
0, //U+C5AA
0, //U+C5AB
0, //U+C5AC
0, //U+C5AD
0, //U+C5AE
0, //U+C5AF
0, //U+C5B0
0, //U+C5B1
0, //U+C5B2
0, //U+C5B3
0, //U+C5B4
0, //U+C5B5
0, //U+C5B6
0, //U+C5B7
0, //U+C5B8
0, //U+C5B9
0, //U+C5BA
0, //U+C5BB
0, //U+C5BC
0, //U+C5BD
0, //U+C5BE
0, //U+C5BF
0, //U+C5C0
0, //U+C5C1
0, //U+C5C2
0, //U+C5C3
0, //U+C5C4
0, //U+C5C5
0, //U+C5C6
0, //U+C5C7
0, //U+C5C8
0, //U+C5C9
0, //U+C5CA
0, //U+C5CB
0, //U+C5CC
0, //U+C5CD
0, //U+C5CE
0, //U+C5CF
0, //U+C5D0
0, //U+C5D1
0, //U+C5D2
0, //U+C5D3
0, //U+C5D4
0, //U+C5D5
0, //U+C5D6
0, //U+C5D7
0, //U+C5D8
0, //U+C5D9
0, //U+C5DA
0, //U+C5DB
0, //U+C5DC
0, //U+C5DD
0, //U+C5DE
0, //U+C5DF
0, //U+C5E0
0, //U+C5E1
0, //U+C5E2
0, //U+C5E3
0, //U+C5E4
0, //U+C5E5
0, //U+C5E6
0, //U+C5E7
0, //U+C5E8
0, //U+C5E9
0, //U+C5EA
0, //U+C5EB
0, //U+C5EC
0, //U+C5ED
0, //U+C5EE
0, //U+C5EF
0, //U+C5F0
0, //U+C5F1
0, //U+C5F2
0, //U+C5F3
0, //U+C5F4
0, //U+C5F5
0, //U+C5F6
0, //U+C5F7
0, //U+C5F8
0, //U+C5F9
0, //U+C5FA
0, //U+C5FB
0, //U+C5FC
0, //U+C5FD
0, //U+C5FE
0, //U+C5FF
0, //U+C600
0, //U+C601
0, //U+C602
0, //U+C603
0, //U+C604
0, //U+C605
0, //U+C606
0, //U+C607
0, //U+C608
0, //U+C609
0, //U+C60A
0, //U+C60B
0, //U+C60C
0, //U+C60D
0, //U+C60E
0, //U+C60F
0, //U+C610
0, //U+C611
0, //U+C612
0, //U+C613
0, //U+C614
0, //U+C615
0, //U+C616
0, //U+C617
0, //U+C618
0, //U+C619
0, //U+C61A
0, //U+C61B
0, //U+C61C
0, //U+C61D
0, //U+C61E
0, //U+C61F
0, //U+C620
0, //U+C621
0, //U+C622
0, //U+C623
0, //U+C624
0, //U+C625
0, //U+C626
0, //U+C627
0, //U+C628
0, //U+C629
0, //U+C62A
0, //U+C62B
0, //U+C62C
0, //U+C62D
0, //U+C62E
0, //U+C62F
0, //U+C630
0, //U+C631
0, //U+C632
0, //U+C633
0, //U+C634
0, //U+C635
0, //U+C636
0, //U+C637
0, //U+C638
0, //U+C639
0, //U+C63A
0, //U+C63B
0, //U+C63C
0, //U+C63D
0, //U+C63E
0, //U+C63F
0, //U+C640
0, //U+C641
0, //U+C642
0, //U+C643
0, //U+C644
0, //U+C645
0, //U+C646
0, //U+C647
0, //U+C648
0, //U+C649
0, //U+C64A
0, //U+C64B
0, //U+C64C
0, //U+C64D
0, //U+C64E
0, //U+C64F
0, //U+C650
0, //U+C651
0, //U+C652
0, //U+C653
0, //U+C654
0, //U+C655
0, //U+C656
0, //U+C657
0, //U+C658
0, //U+C659
0, //U+C65A
0, //U+C65B
0, //U+C65C
0, //U+C65D
0, //U+C65E
0, //U+C65F
0, //U+C660
0, //U+C661
0, //U+C662
0, //U+C663
0, //U+C664
0, //U+C665
0, //U+C666
0, //U+C667
0, //U+C668
0, //U+C669
0, //U+C66A
0, //U+C66B
0, //U+C66C
0, //U+C66D
0, //U+C66E
0, //U+C66F
0, //U+C670
0, //U+C671
0, //U+C672
0, //U+C673
0, //U+C674
0, //U+C675
0, //U+C676
0, //U+C677
0, //U+C678
0, //U+C679
0, //U+C67A
0, //U+C67B
0, //U+C67C
0, //U+C67D
0, //U+C67E
0, //U+C67F
0, //U+C680
0, //U+C681
0, //U+C682
0, //U+C683
0, //U+C684
0, //U+C685
0, //U+C686
0, //U+C687
0, //U+C688
0, //U+C689
0, //U+C68A
0, //U+C68B
0, //U+C68C
0, //U+C68D
0, //U+C68E
0, //U+C68F
0, //U+C690
0, //U+C691
0, //U+C692
0, //U+C693
0, //U+C694
0, //U+C695
0, //U+C696
0, //U+C697
0, //U+C698
0, //U+C699
0, //U+C69A
0, //U+C69B
0, //U+C69C
0, //U+C69D
0, //U+C69E
0, //U+C69F
0, //U+C6A0
0, //U+C6A1
0, //U+C6A2
0, //U+C6A3
0, //U+C6A4
0, //U+C6A5
0, //U+C6A6
0, //U+C6A7
0, //U+C6A8
0, //U+C6A9
0, //U+C6AA
0, //U+C6AB
0, //U+C6AC
0, //U+C6AD
0, //U+C6AE
0, //U+C6AF
0, //U+C6B0
0, //U+C6B1
0, //U+C6B2
0, //U+C6B3
0, //U+C6B4
0, //U+C6B5
0, //U+C6B6
0, //U+C6B7
0, //U+C6B8
0, //U+C6B9
0, //U+C6BA
0, //U+C6BB
0, //U+C6BC
0, //U+C6BD
0, //U+C6BE
0, //U+C6BF
0, //U+C6C0
0, //U+C6C1
0, //U+C6C2
0, //U+C6C3
0, //U+C6C4
0, //U+C6C5
0, //U+C6C6
0, //U+C6C7
0, //U+C6C8
0, //U+C6C9
0, //U+C6CA
0, //U+C6CB
0, //U+C6CC
0, //U+C6CD
0, //U+C6CE
0, //U+C6CF
0, //U+C6D0
0, //U+C6D1
0, //U+C6D2
0, //U+C6D3
0, //U+C6D4
0, //U+C6D5
0, //U+C6D6
0, //U+C6D7
0, //U+C6D8
0, //U+C6D9
0, //U+C6DA
0, //U+C6DB
0, //U+C6DC
0, //U+C6DD
0, //U+C6DE
0, //U+C6DF
0, //U+C6E0
0, //U+C6E1
0, //U+C6E2
0, //U+C6E3
0, //U+C6E4
0, //U+C6E5
0, //U+C6E6
0, //U+C6E7
0, //U+C6E8
0, //U+C6E9
0, //U+C6EA
0, //U+C6EB
0, //U+C6EC
0, //U+C6ED
0, //U+C6EE
0, //U+C6EF
0, //U+C6F0
0, //U+C6F1
0, //U+C6F2
0, //U+C6F3
0, //U+C6F4
0, //U+C6F5
0, //U+C6F6
0, //U+C6F7
0, //U+C6F8
0, //U+C6F9
0, //U+C6FA
0, //U+C6FB
0, //U+C6FC
0, //U+C6FD
0, //U+C6FE
0, //U+C6FF
0, //U+C700
0, //U+C701
0, //U+C702
0, //U+C703
0, //U+C704
0, //U+C705
0, //U+C706
0, //U+C707
0, //U+C708
0, //U+C709
0, //U+C70A
0, //U+C70B
0, //U+C70C
0, //U+C70D
0, //U+C70E
0, //U+C70F
0, //U+C710
0, //U+C711
0, //U+C712
0, //U+C713
0, //U+C714
0, //U+C715
0, //U+C716
0, //U+C717
0, //U+C718
0, //U+C719
0, //U+C71A
0, //U+C71B
0, //U+C71C
0, //U+C71D
0, //U+C71E
0, //U+C71F
0, //U+C720
0, //U+C721
0, //U+C722
0, //U+C723
0, //U+C724
0, //U+C725
0, //U+C726
0, //U+C727
0, //U+C728
0, //U+C729
0, //U+C72A
0, //U+C72B
0, //U+C72C
0, //U+C72D
0, //U+C72E
0, //U+C72F
0, //U+C730
0, //U+C731
0, //U+C732
0, //U+C733
0, //U+C734
0, //U+C735
0, //U+C736
0, //U+C737
0, //U+C738
0, //U+C739
0, //U+C73A
0, //U+C73B
0, //U+C73C
0, //U+C73D
0, //U+C73E
0, //U+C73F
0, //U+C740
0, //U+C741
0, //U+C742
0, //U+C743
0, //U+C744
0, //U+C745
0, //U+C746
0, //U+C747
0, //U+C748
0, //U+C749
0, //U+C74A
0, //U+C74B
0, //U+C74C
0, //U+C74D
0, //U+C74E
0, //U+C74F
0, //U+C750
0, //U+C751
0, //U+C752
0, //U+C753
0, //U+C754
0, //U+C755
0, //U+C756
0, //U+C757
0, //U+C758
0, //U+C759
0, //U+C75A
0, //U+C75B
0, //U+C75C
0, //U+C75D
0, //U+C75E
0, //U+C75F
0, //U+C760
0, //U+C761
0, //U+C762
0, //U+C763
0, //U+C764
0, //U+C765
0, //U+C766
0, //U+C767
0, //U+C768
0, //U+C769
0, //U+C76A
0, //U+C76B
0, //U+C76C
0, //U+C76D
0, //U+C76E
0, //U+C76F
0, //U+C770
0, //U+C771
0, //U+C772
0, //U+C773
0, //U+C774
0, //U+C775
0, //U+C776
0, //U+C777
0, //U+C778
0, //U+C779
0, //U+C77A
0, //U+C77B
0, //U+C77C
0, //U+C77D
0, //U+C77E
0, //U+C77F
0, //U+C780
0, //U+C781
0, //U+C782
0, //U+C783
0, //U+C784
0, //U+C785
0, //U+C786
0, //U+C787
0, //U+C788
0, //U+C789
0, //U+C78A
0, //U+C78B
0, //U+C78C
0, //U+C78D
0, //U+C78E
0, //U+C78F
0, //U+C790
0, //U+C791
0, //U+C792
0, //U+C793
0, //U+C794
0, //U+C795
0, //U+C796
0, //U+C797
0, //U+C798
0, //U+C799
0, //U+C79A
0, //U+C79B
0, //U+C79C
0, //U+C79D
0, //U+C79E
0, //U+C79F
0, //U+C7A0
0, //U+C7A1
0, //U+C7A2
0, //U+C7A3
0, //U+C7A4
0, //U+C7A5
0, //U+C7A6
0, //U+C7A7
0, //U+C7A8
0, //U+C7A9
0, //U+C7AA
0, //U+C7AB
0, //U+C7AC
0, //U+C7AD
0, //U+C7AE
0, //U+C7AF
0, //U+C7B0
0, //U+C7B1
0, //U+C7B2
0, //U+C7B3
0, //U+C7B4
0, //U+C7B5
0, //U+C7B6
0, //U+C7B7
0, //U+C7B8
0, //U+C7B9
0, //U+C7BA
0, //U+C7BB
0, //U+C7BC
0, //U+C7BD
0, //U+C7BE
0, //U+C7BF
0, //U+C7C0
0, //U+C7C1
0, //U+C7C2
0, //U+C7C3
0, //U+C7C4
0, //U+C7C5
0, //U+C7C6
0, //U+C7C7
0, //U+C7C8
0, //U+C7C9
0, //U+C7CA
0, //U+C7CB
0, //U+C7CC
0, //U+C7CD
0, //U+C7CE
0, //U+C7CF
0, //U+C7D0
0, //U+C7D1
0, //U+C7D2
0, //U+C7D3
0, //U+C7D4
0, //U+C7D5
0, //U+C7D6
0, //U+C7D7
0, //U+C7D8
0, //U+C7D9
0, //U+C7DA
0, //U+C7DB
0, //U+C7DC
0, //U+C7DD
0, //U+C7DE
0, //U+C7DF
0, //U+C7E0
0, //U+C7E1
0, //U+C7E2
0, //U+C7E3
0, //U+C7E4
0, //U+C7E5
0, //U+C7E6
0, //U+C7E7
0, //U+C7E8
0, //U+C7E9
0, //U+C7EA
0, //U+C7EB
0, //U+C7EC
0, //U+C7ED
0, //U+C7EE
0, //U+C7EF
0, //U+C7F0
0, //U+C7F1
0, //U+C7F2
0, //U+C7F3
0, //U+C7F4
0, //U+C7F5
0, //U+C7F6
0, //U+C7F7
0, //U+C7F8
0, //U+C7F9
0, //U+C7FA
0, //U+C7FB
0, //U+C7FC
0, //U+C7FD
0, //U+C7FE
0, //U+C7FF
0, //U+C800
0, //U+C801
0, //U+C802
0, //U+C803
0, //U+C804
0, //U+C805
0, //U+C806
0, //U+C807
0, //U+C808
0, //U+C809
0, //U+C80A
0, //U+C80B
0, //U+C80C
0, //U+C80D
0, //U+C80E
0, //U+C80F
0, //U+C810
0, //U+C811
0, //U+C812
0, //U+C813
0, //U+C814
0, //U+C815
0, //U+C816
0, //U+C817
0, //U+C818
0, //U+C819
0, //U+C81A
0, //U+C81B
0, //U+C81C
0, //U+C81D
0, //U+C81E
0, //U+C81F
0, //U+C820
0, //U+C821
0, //U+C822
0, //U+C823
0, //U+C824
0, //U+C825
0, //U+C826
0, //U+C827
0, //U+C828
0, //U+C829
0, //U+C82A
0, //U+C82B
0, //U+C82C
0, //U+C82D
0, //U+C82E
0, //U+C82F
0, //U+C830
0, //U+C831
0, //U+C832
0, //U+C833
0, //U+C834
0, //U+C835
0, //U+C836
0, //U+C837
0, //U+C838
0, //U+C839
0, //U+C83A
0, //U+C83B
0, //U+C83C
0, //U+C83D
0, //U+C83E
0, //U+C83F
0, //U+C840
0, //U+C841
0, //U+C842
0, //U+C843
0, //U+C844
0, //U+C845
0, //U+C846
0, //U+C847
0, //U+C848
0, //U+C849
0, //U+C84A
0, //U+C84B
0, //U+C84C
0, //U+C84D
0, //U+C84E
0, //U+C84F
0, //U+C850
0, //U+C851
0, //U+C852
0, //U+C853
0, //U+C854
0, //U+C855
0, //U+C856
0, //U+C857
0, //U+C858
0, //U+C859
0, //U+C85A
0, //U+C85B
0, //U+C85C
0, //U+C85D
0, //U+C85E
0, //U+C85F
0, //U+C860
0, //U+C861
0, //U+C862
0, //U+C863
0, //U+C864
0, //U+C865
0, //U+C866
0, //U+C867
0, //U+C868
0, //U+C869
0, //U+C86A
0, //U+C86B
0, //U+C86C
0, //U+C86D
0, //U+C86E
0, //U+C86F
0, //U+C870
0, //U+C871
0, //U+C872
0, //U+C873
0, //U+C874
0, //U+C875
0, //U+C876
0, //U+C877
0, //U+C878
0, //U+C879
0, //U+C87A
0, //U+C87B
0, //U+C87C
0, //U+C87D
0, //U+C87E
0, //U+C87F
0, //U+C880
0, //U+C881
0, //U+C882
0, //U+C883
0, //U+C884
0, //U+C885
0, //U+C886
0, //U+C887
0, //U+C888
0, //U+C889
0, //U+C88A
0, //U+C88B
0, //U+C88C
0, //U+C88D
0, //U+C88E
0, //U+C88F
0, //U+C890
0, //U+C891
0, //U+C892
0, //U+C893
0, //U+C894
0, //U+C895
0, //U+C896
0, //U+C897
0, //U+C898
0, //U+C899
0, //U+C89A
0, //U+C89B
0, //U+C89C
0, //U+C89D
0, //U+C89E
0, //U+C89F
0, //U+C8A0
0, //U+C8A1
0, //U+C8A2
0, //U+C8A3
0, //U+C8A4
0, //U+C8A5
0, //U+C8A6
0, //U+C8A7
0, //U+C8A8
0, //U+C8A9
0, //U+C8AA
0, //U+C8AB
0, //U+C8AC
0, //U+C8AD
0, //U+C8AE
0, //U+C8AF
0, //U+C8B0
0, //U+C8B1
0, //U+C8B2
0, //U+C8B3
0, //U+C8B4
0, //U+C8B5
0, //U+C8B6
0, //U+C8B7
0, //U+C8B8
0, //U+C8B9
0, //U+C8BA
0, //U+C8BB
0, //U+C8BC
0, //U+C8BD
0, //U+C8BE
0, //U+C8BF
0, //U+C8C0
0, //U+C8C1
0, //U+C8C2
0, //U+C8C3
0, //U+C8C4
0, //U+C8C5
0, //U+C8C6
0, //U+C8C7
0, //U+C8C8
0, //U+C8C9
0, //U+C8CA
0, //U+C8CB
0, //U+C8CC
0, //U+C8CD
0, //U+C8CE
0, //U+C8CF
0, //U+C8D0
0, //U+C8D1
0, //U+C8D2
0, //U+C8D3
0, //U+C8D4
0, //U+C8D5
0, //U+C8D6
0, //U+C8D7
0, //U+C8D8
0, //U+C8D9
0, //U+C8DA
0, //U+C8DB
0, //U+C8DC
0, //U+C8DD
0, //U+C8DE
0, //U+C8DF
0, //U+C8E0
0, //U+C8E1
0, //U+C8E2
0, //U+C8E3
0, //U+C8E4
0, //U+C8E5
0, //U+C8E6
0, //U+C8E7
0, //U+C8E8
0, //U+C8E9
0, //U+C8EA
0, //U+C8EB
0, //U+C8EC
0, //U+C8ED
0, //U+C8EE
0, //U+C8EF
0, //U+C8F0
0, //U+C8F1
0, //U+C8F2
0, //U+C8F3
0, //U+C8F4
0, //U+C8F5
0, //U+C8F6
0, //U+C8F7
0, //U+C8F8
0, //U+C8F9
0, //U+C8FA
0, //U+C8FB
0, //U+C8FC
0, //U+C8FD
0, //U+C8FE
0, //U+C8FF
0, //U+C900
0, //U+C901
0, //U+C902
0, //U+C903
0, //U+C904
0, //U+C905
0, //U+C906
0, //U+C907
0, //U+C908
0, //U+C909
0, //U+C90A
0, //U+C90B
0, //U+C90C
0, //U+C90D
0, //U+C90E
0, //U+C90F
0, //U+C910
0, //U+C911
0, //U+C912
0, //U+C913
0, //U+C914
0, //U+C915
0, //U+C916
0, //U+C917
0, //U+C918
0, //U+C919
0, //U+C91A
0, //U+C91B
0, //U+C91C
0, //U+C91D
0, //U+C91E
0, //U+C91F
0, //U+C920
0, //U+C921
0, //U+C922
0, //U+C923
0, //U+C924
0, //U+C925
0, //U+C926
0, //U+C927
0, //U+C928
0, //U+C929
0, //U+C92A
0, //U+C92B
0, //U+C92C
0, //U+C92D
0, //U+C92E
0, //U+C92F
0, //U+C930
0, //U+C931
0, //U+C932
0, //U+C933
0, //U+C934
0, //U+C935
0, //U+C936
0, //U+C937
0, //U+C938
0, //U+C939
0, //U+C93A
0, //U+C93B
0, //U+C93C
0, //U+C93D
0, //U+C93E
0, //U+C93F
0, //U+C940
0, //U+C941
0, //U+C942
0, //U+C943
0, //U+C944
0, //U+C945
0, //U+C946
0, //U+C947
0, //U+C948
0, //U+C949
0, //U+C94A
0, //U+C94B
0, //U+C94C
0, //U+C94D
0, //U+C94E
0, //U+C94F
0, //U+C950
0, //U+C951
0, //U+C952
0, //U+C953
0, //U+C954
0, //U+C955
0, //U+C956
0, //U+C957
0, //U+C958
0, //U+C959
0, //U+C95A
0, //U+C95B
0, //U+C95C
0, //U+C95D
0, //U+C95E
0, //U+C95F
0, //U+C960
0, //U+C961
0, //U+C962
0, //U+C963
0, //U+C964
0, //U+C965
0, //U+C966
0, //U+C967
0, //U+C968
0, //U+C969
0, //U+C96A
0, //U+C96B
0, //U+C96C
0, //U+C96D
0, //U+C96E
0, //U+C96F
0, //U+C970
0, //U+C971
0, //U+C972
0, //U+C973
0, //U+C974
0, //U+C975
0, //U+C976
0, //U+C977
0, //U+C978
0, //U+C979
0, //U+C97A
0, //U+C97B
0, //U+C97C
0, //U+C97D
0, //U+C97E
0, //U+C97F
0, //U+C980
0, //U+C981
0, //U+C982
0, //U+C983
0, //U+C984
0, //U+C985
0, //U+C986
0, //U+C987
0, //U+C988
0, //U+C989
0, //U+C98A
0, //U+C98B
0, //U+C98C
0, //U+C98D
0, //U+C98E
0, //U+C98F
0, //U+C990
0, //U+C991
0, //U+C992
0, //U+C993
0, //U+C994
0, //U+C995
0, //U+C996
0, //U+C997
0, //U+C998
0, //U+C999
0, //U+C99A
0, //U+C99B
0, //U+C99C
0, //U+C99D
0, //U+C99E
0, //U+C99F
0, //U+C9A0
0, //U+C9A1
0, //U+C9A2
0, //U+C9A3
0, //U+C9A4
0, //U+C9A5
0, //U+C9A6
0, //U+C9A7
0, //U+C9A8
0, //U+C9A9
0, //U+C9AA
0, //U+C9AB
0, //U+C9AC
0, //U+C9AD
0, //U+C9AE
0, //U+C9AF
0, //U+C9B0
0, //U+C9B1
0, //U+C9B2
0, //U+C9B3
0, //U+C9B4
0, //U+C9B5
0, //U+C9B6
0, //U+C9B7
0, //U+C9B8
0, //U+C9B9
0, //U+C9BA
0, //U+C9BB
0, //U+C9BC
0, //U+C9BD
0, //U+C9BE
0, //U+C9BF
0, //U+C9C0
0, //U+C9C1
0, //U+C9C2
0, //U+C9C3
0, //U+C9C4
0, //U+C9C5
0, //U+C9C6
0, //U+C9C7
0, //U+C9C8
0, //U+C9C9
0, //U+C9CA
0, //U+C9CB
0, //U+C9CC
0, //U+C9CD
0, //U+C9CE
0, //U+C9CF
0, //U+C9D0
0, //U+C9D1
0, //U+C9D2
0, //U+C9D3
0, //U+C9D4
0, //U+C9D5
0, //U+C9D6
0, //U+C9D7
0, //U+C9D8
0, //U+C9D9
0, //U+C9DA
0, //U+C9DB
0, //U+C9DC
0, //U+C9DD
0, //U+C9DE
0, //U+C9DF
0, //U+C9E0
0, //U+C9E1
0, //U+C9E2
0, //U+C9E3
0, //U+C9E4
0, //U+C9E5
0, //U+C9E6
0, //U+C9E7
0, //U+C9E8
0, //U+C9E9
0, //U+C9EA
0, //U+C9EB
0, //U+C9EC
0, //U+C9ED
0, //U+C9EE
0, //U+C9EF
0, //U+C9F0
0, //U+C9F1
0, //U+C9F2
0, //U+C9F3
0, //U+C9F4
0, //U+C9F5
0, //U+C9F6
0, //U+C9F7
0, //U+C9F8
0, //U+C9F9
0, //U+C9FA
0, //U+C9FB
0, //U+C9FC
0, //U+C9FD
0, //U+C9FE
0, //U+C9FF
0, //U+CA00
0, //U+CA01
0, //U+CA02
0, //U+CA03
0, //U+CA04
0, //U+CA05
0, //U+CA06
0, //U+CA07
0, //U+CA08
0, //U+CA09
0, //U+CA0A
0, //U+CA0B
0, //U+CA0C
0, //U+CA0D
0, //U+CA0E
0, //U+CA0F
0, //U+CA10
0, //U+CA11
0, //U+CA12
0, //U+CA13
0, //U+CA14
0, //U+CA15
0, //U+CA16
0, //U+CA17
0, //U+CA18
0, //U+CA19
0, //U+CA1A
0, //U+CA1B
0, //U+CA1C
0, //U+CA1D
0, //U+CA1E
0, //U+CA1F
0, //U+CA20
0, //U+CA21
0, //U+CA22
0, //U+CA23
0, //U+CA24
0, //U+CA25
0, //U+CA26
0, //U+CA27
0, //U+CA28
0, //U+CA29
0, //U+CA2A
0, //U+CA2B
0, //U+CA2C
0, //U+CA2D
0, //U+CA2E
0, //U+CA2F
0, //U+CA30
0, //U+CA31
0, //U+CA32
0, //U+CA33
0, //U+CA34
0, //U+CA35
0, //U+CA36
0, //U+CA37
0, //U+CA38
0, //U+CA39
0, //U+CA3A
0, //U+CA3B
0, //U+CA3C
0, //U+CA3D
0, //U+CA3E
0, //U+CA3F
0, //U+CA40
0, //U+CA41
0, //U+CA42
0, //U+CA43
0, //U+CA44
0, //U+CA45
0, //U+CA46
0, //U+CA47
0, //U+CA48
0, //U+CA49
0, //U+CA4A
0, //U+CA4B
0, //U+CA4C
0, //U+CA4D
0, //U+CA4E
0, //U+CA4F
0, //U+CA50
0, //U+CA51
0, //U+CA52
0, //U+CA53
0, //U+CA54
0, //U+CA55
0, //U+CA56
0, //U+CA57
0, //U+CA58
0, //U+CA59
0, //U+CA5A
0, //U+CA5B
0, //U+CA5C
0, //U+CA5D
0, //U+CA5E
0, //U+CA5F
0, //U+CA60
0, //U+CA61
0, //U+CA62
0, //U+CA63
0, //U+CA64
0, //U+CA65
0, //U+CA66
0, //U+CA67
0, //U+CA68
0, //U+CA69
0, //U+CA6A
0, //U+CA6B
0, //U+CA6C
0, //U+CA6D
0, //U+CA6E
0, //U+CA6F
0, //U+CA70
0, //U+CA71
0, //U+CA72
0, //U+CA73
0, //U+CA74
0, //U+CA75
0, //U+CA76
0, //U+CA77
0, //U+CA78
0, //U+CA79
0, //U+CA7A
0, //U+CA7B
0, //U+CA7C
0, //U+CA7D
0, //U+CA7E
0, //U+CA7F
0, //U+CA80
0, //U+CA81
0, //U+CA82
0, //U+CA83
0, //U+CA84
0, //U+CA85
0, //U+CA86
0, //U+CA87
0, //U+CA88
0, //U+CA89
0, //U+CA8A
0, //U+CA8B
0, //U+CA8C
0, //U+CA8D
0, //U+CA8E
0, //U+CA8F
0, //U+CA90
0, //U+CA91
0, //U+CA92
0, //U+CA93
0, //U+CA94
0, //U+CA95
0, //U+CA96
0, //U+CA97
0, //U+CA98
0, //U+CA99
0, //U+CA9A
0, //U+CA9B
0, //U+CA9C
0, //U+CA9D
0, //U+CA9E
0, //U+CA9F
0, //U+CAA0
0, //U+CAA1
0, //U+CAA2
0, //U+CAA3
0, //U+CAA4
0, //U+CAA5
0, //U+CAA6
0, //U+CAA7
0, //U+CAA8
0, //U+CAA9
0, //U+CAAA
0, //U+CAAB
0, //U+CAAC
0, //U+CAAD
0, //U+CAAE
0, //U+CAAF
0, //U+CAB0
0, //U+CAB1
0, //U+CAB2
0, //U+CAB3
0, //U+CAB4
0, //U+CAB5
0, //U+CAB6
0, //U+CAB7
0, //U+CAB8
0, //U+CAB9
0, //U+CABA
0, //U+CABB
0, //U+CABC
0, //U+CABD
0, //U+CABE
0, //U+CABF
0, //U+CAC0
0, //U+CAC1
0, //U+CAC2
0, //U+CAC3
0, //U+CAC4
0, //U+CAC5
0, //U+CAC6
0, //U+CAC7
0, //U+CAC8
0, //U+CAC9
0, //U+CACA
0, //U+CACB
0, //U+CACC
0, //U+CACD
0, //U+CACE
0, //U+CACF
0, //U+CAD0
0, //U+CAD1
0, //U+CAD2
0, //U+CAD3
0, //U+CAD4
0, //U+CAD5
0, //U+CAD6
0, //U+CAD7
0, //U+CAD8
0, //U+CAD9
0, //U+CADA
0, //U+CADB
0, //U+CADC
0, //U+CADD
0, //U+CADE
0, //U+CADF
0, //U+CAE0
0, //U+CAE1
0, //U+CAE2
0, //U+CAE3
0, //U+CAE4
0, //U+CAE5
0, //U+CAE6
0, //U+CAE7
0, //U+CAE8
0, //U+CAE9
0, //U+CAEA
0, //U+CAEB
0, //U+CAEC
0, //U+CAED
0, //U+CAEE
0, //U+CAEF
0, //U+CAF0
0, //U+CAF1
0, //U+CAF2
0, //U+CAF3
0, //U+CAF4
0, //U+CAF5
0, //U+CAF6
0, //U+CAF7
0, //U+CAF8
0, //U+CAF9
0, //U+CAFA
0, //U+CAFB
0, //U+CAFC
0, //U+CAFD
0, //U+CAFE
0, //U+CAFF
0, //U+CB00
0, //U+CB01
0, //U+CB02
0, //U+CB03
0, //U+CB04
0, //U+CB05
0, //U+CB06
0, //U+CB07
0, //U+CB08
0, //U+CB09
0, //U+CB0A
0, //U+CB0B
0, //U+CB0C
0, //U+CB0D
0, //U+CB0E
0, //U+CB0F
0, //U+CB10
0, //U+CB11
0, //U+CB12
0, //U+CB13
0, //U+CB14
0, //U+CB15
0, //U+CB16
0, //U+CB17
0, //U+CB18
0, //U+CB19
0, //U+CB1A
0, //U+CB1B
0, //U+CB1C
0, //U+CB1D
0, //U+CB1E
0, //U+CB1F
0, //U+CB20
0, //U+CB21
0, //U+CB22
0, //U+CB23
0, //U+CB24
0, //U+CB25
0, //U+CB26
0, //U+CB27
0, //U+CB28
0, //U+CB29
0, //U+CB2A
0, //U+CB2B
0, //U+CB2C
0, //U+CB2D
0, //U+CB2E
0, //U+CB2F
0, //U+CB30
0, //U+CB31
0, //U+CB32
0, //U+CB33
0, //U+CB34
0, //U+CB35
0, //U+CB36
0, //U+CB37
0, //U+CB38
0, //U+CB39
0, //U+CB3A
0, //U+CB3B
0, //U+CB3C
0, //U+CB3D
0, //U+CB3E
0, //U+CB3F
0, //U+CB40
0, //U+CB41
0, //U+CB42
0, //U+CB43
0, //U+CB44
0, //U+CB45
0, //U+CB46
0, //U+CB47
0, //U+CB48
0, //U+CB49
0, //U+CB4A
0, //U+CB4B
0, //U+CB4C
0, //U+CB4D
0, //U+CB4E
0, //U+CB4F
0, //U+CB50
0, //U+CB51
0, //U+CB52
0, //U+CB53
0, //U+CB54
0, //U+CB55
0, //U+CB56
0, //U+CB57
0, //U+CB58
0, //U+CB59
0, //U+CB5A
0, //U+CB5B
0, //U+CB5C
0, //U+CB5D
0, //U+CB5E
0, //U+CB5F
0, //U+CB60
0, //U+CB61
0, //U+CB62
0, //U+CB63
0, //U+CB64
0, //U+CB65
0, //U+CB66
0, //U+CB67
0, //U+CB68
0, //U+CB69
0, //U+CB6A
0, //U+CB6B
0, //U+CB6C
0, //U+CB6D
0, //U+CB6E
0, //U+CB6F
0, //U+CB70
0, //U+CB71
0, //U+CB72
0, //U+CB73
0, //U+CB74
0, //U+CB75
0, //U+CB76
0, //U+CB77
0, //U+CB78
0, //U+CB79
0, //U+CB7A
0, //U+CB7B
0, //U+CB7C
0, //U+CB7D
0, //U+CB7E
0, //U+CB7F
0, //U+CB80
0, //U+CB81
0, //U+CB82
0, //U+CB83
0, //U+CB84
0, //U+CB85
0, //U+CB86
0, //U+CB87
0, //U+CB88
0, //U+CB89
0, //U+CB8A
0, //U+CB8B
0, //U+CB8C
0, //U+CB8D
0, //U+CB8E
0, //U+CB8F
0, //U+CB90
0, //U+CB91
0, //U+CB92
0, //U+CB93
0, //U+CB94
0, //U+CB95
0, //U+CB96
0, //U+CB97
0, //U+CB98
0, //U+CB99
0, //U+CB9A
0, //U+CB9B
0, //U+CB9C
0, //U+CB9D
0, //U+CB9E
0, //U+CB9F
0, //U+CBA0
0, //U+CBA1
0, //U+CBA2
0, //U+CBA3
0, //U+CBA4
0, //U+CBA5
0, //U+CBA6
0, //U+CBA7
0, //U+CBA8
0, //U+CBA9
0, //U+CBAA
0, //U+CBAB
0, //U+CBAC
0, //U+CBAD
0, //U+CBAE
0, //U+CBAF
0, //U+CBB0
0, //U+CBB1
0, //U+CBB2
0, //U+CBB3
0, //U+CBB4
0, //U+CBB5
0, //U+CBB6
0, //U+CBB7
0, //U+CBB8
0, //U+CBB9
0, //U+CBBA
0, //U+CBBB
0, //U+CBBC
0, //U+CBBD
0, //U+CBBE
0, //U+CBBF
0, //U+CBC0
0, //U+CBC1
0, //U+CBC2
0, //U+CBC3
0, //U+CBC4
0, //U+CBC5
0, //U+CBC6
0, //U+CBC7
0, //U+CBC8
0, //U+CBC9
0, //U+CBCA
0, //U+CBCB
0, //U+CBCC
0, //U+CBCD
0, //U+CBCE
0, //U+CBCF
0, //U+CBD0
0, //U+CBD1
0, //U+CBD2
0, //U+CBD3
0, //U+CBD4
0, //U+CBD5
0, //U+CBD6
0, //U+CBD7
0, //U+CBD8
0, //U+CBD9
0, //U+CBDA
0, //U+CBDB
0, //U+CBDC
0, //U+CBDD
0, //U+CBDE
0, //U+CBDF
0, //U+CBE0
0, //U+CBE1
0, //U+CBE2
0, //U+CBE3
0, //U+CBE4
0, //U+CBE5
0, //U+CBE6
0, //U+CBE7
0, //U+CBE8
0, //U+CBE9
0, //U+CBEA
0, //U+CBEB
0, //U+CBEC
0, //U+CBED
0, //U+CBEE
0, //U+CBEF
0, //U+CBF0
0, //U+CBF1
0, //U+CBF2
0, //U+CBF3
0, //U+CBF4
0, //U+CBF5
0, //U+CBF6
0, //U+CBF7
0, //U+CBF8
0, //U+CBF9
0, //U+CBFA
0, //U+CBFB
0, //U+CBFC
0, //U+CBFD
0, //U+CBFE
0, //U+CBFF
0, //U+CC00
0, //U+CC01
0, //U+CC02
0, //U+CC03
0, //U+CC04
0, //U+CC05
0, //U+CC06
0, //U+CC07
0, //U+CC08
0, //U+CC09
0, //U+CC0A
0, //U+CC0B
0, //U+CC0C
0, //U+CC0D
0, //U+CC0E
0, //U+CC0F
0, //U+CC10
0, //U+CC11
0, //U+CC12
0, //U+CC13
0, //U+CC14
0, //U+CC15
0, //U+CC16
0, //U+CC17
0, //U+CC18
0, //U+CC19
0, //U+CC1A
0, //U+CC1B
0, //U+CC1C
0, //U+CC1D
0, //U+CC1E
0, //U+CC1F
0, //U+CC20
0, //U+CC21
0, //U+CC22
0, //U+CC23
0, //U+CC24
0, //U+CC25
0, //U+CC26
0, //U+CC27
0, //U+CC28
0, //U+CC29
0, //U+CC2A
0, //U+CC2B
0, //U+CC2C
0, //U+CC2D
0, //U+CC2E
0, //U+CC2F
0, //U+CC30
0, //U+CC31
0, //U+CC32
0, //U+CC33
0, //U+CC34
0, //U+CC35
0, //U+CC36
0, //U+CC37
0, //U+CC38
0, //U+CC39
0, //U+CC3A
0, //U+CC3B
0, //U+CC3C
0, //U+CC3D
0, //U+CC3E
0, //U+CC3F
0, //U+CC40
0, //U+CC41
0, //U+CC42
0, //U+CC43
0, //U+CC44
0, //U+CC45
0, //U+CC46
0, //U+CC47
0, //U+CC48
0, //U+CC49
0, //U+CC4A
0, //U+CC4B
0, //U+CC4C
0, //U+CC4D
0, //U+CC4E
0, //U+CC4F
0, //U+CC50
0, //U+CC51
0, //U+CC52
0, //U+CC53
0, //U+CC54
0, //U+CC55
0, //U+CC56
0, //U+CC57
0, //U+CC58
0, //U+CC59
0, //U+CC5A
0, //U+CC5B
0, //U+CC5C
0, //U+CC5D
0, //U+CC5E
0, //U+CC5F
0, //U+CC60
0, //U+CC61
0, //U+CC62
0, //U+CC63
0, //U+CC64
0, //U+CC65
0, //U+CC66
0, //U+CC67
0, //U+CC68
0, //U+CC69
0, //U+CC6A
0, //U+CC6B
0, //U+CC6C
0, //U+CC6D
0, //U+CC6E
0, //U+CC6F
0, //U+CC70
0, //U+CC71
0, //U+CC72
0, //U+CC73
0, //U+CC74
0, //U+CC75
0, //U+CC76
0, //U+CC77
0, //U+CC78
0, //U+CC79
0, //U+CC7A
0, //U+CC7B
0, //U+CC7C
0, //U+CC7D
0, //U+CC7E
0, //U+CC7F
0, //U+CC80
0, //U+CC81
0, //U+CC82
0, //U+CC83
0, //U+CC84
0, //U+CC85
0, //U+CC86
0, //U+CC87
0, //U+CC88
0, //U+CC89
0, //U+CC8A
0, //U+CC8B
0, //U+CC8C
0, //U+CC8D
0, //U+CC8E
0, //U+CC8F
0, //U+CC90
0, //U+CC91
0, //U+CC92
0, //U+CC93
0, //U+CC94
0, //U+CC95
0, //U+CC96
0, //U+CC97
0, //U+CC98
0, //U+CC99
0, //U+CC9A
0, //U+CC9B
0, //U+CC9C
0, //U+CC9D
0, //U+CC9E
0, //U+CC9F
0, //U+CCA0
0, //U+CCA1
0, //U+CCA2
0, //U+CCA3
0, //U+CCA4
0, //U+CCA5
0, //U+CCA6
0, //U+CCA7
0, //U+CCA8
0, //U+CCA9
0, //U+CCAA
0, //U+CCAB
0, //U+CCAC
0, //U+CCAD
0, //U+CCAE
0, //U+CCAF
0, //U+CCB0
0, //U+CCB1
0, //U+CCB2
0, //U+CCB3
0, //U+CCB4
0, //U+CCB5
0, //U+CCB6
0, //U+CCB7
0, //U+CCB8
0, //U+CCB9
0, //U+CCBA
0, //U+CCBB
0, //U+CCBC
0, //U+CCBD
0, //U+CCBE
0, //U+CCBF
0, //U+CCC0
0, //U+CCC1
0, //U+CCC2
0, //U+CCC3
0, //U+CCC4
0, //U+CCC5
0, //U+CCC6
0, //U+CCC7
0, //U+CCC8
0, //U+CCC9
0, //U+CCCA
0, //U+CCCB
0, //U+CCCC
0, //U+CCCD
0, //U+CCCE
0, //U+CCCF
0, //U+CCD0
0, //U+CCD1
0, //U+CCD2
0, //U+CCD3
0, //U+CCD4
0, //U+CCD5
0, //U+CCD6
0, //U+CCD7
0, //U+CCD8
0, //U+CCD9
0, //U+CCDA
0, //U+CCDB
0, //U+CCDC
0, //U+CCDD
0, //U+CCDE
0, //U+CCDF
0, //U+CCE0
0, //U+CCE1
0, //U+CCE2
0, //U+CCE3
0, //U+CCE4
0, //U+CCE5
0, //U+CCE6
0, //U+CCE7
0, //U+CCE8
0, //U+CCE9
0, //U+CCEA
0, //U+CCEB
0, //U+CCEC
0, //U+CCED
0, //U+CCEE
0, //U+CCEF
0, //U+CCF0
0, //U+CCF1
0, //U+CCF2
0, //U+CCF3
0, //U+CCF4
0, //U+CCF5
0, //U+CCF6
0, //U+CCF7
0, //U+CCF8
0, //U+CCF9
0, //U+CCFA
0, //U+CCFB
0, //U+CCFC
0, //U+CCFD
0, //U+CCFE
0, //U+CCFF
0, //U+CD00
0, //U+CD01
0, //U+CD02
0, //U+CD03
0, //U+CD04
0, //U+CD05
0, //U+CD06
0, //U+CD07
0, //U+CD08
0, //U+CD09
0, //U+CD0A
0, //U+CD0B
0, //U+CD0C
0, //U+CD0D
0, //U+CD0E
0, //U+CD0F
0, //U+CD10
0, //U+CD11
0, //U+CD12
0, //U+CD13
0, //U+CD14
0, //U+CD15
0, //U+CD16
0, //U+CD17
0, //U+CD18
0, //U+CD19
0, //U+CD1A
0, //U+CD1B
0, //U+CD1C
0, //U+CD1D
0, //U+CD1E
0, //U+CD1F
0, //U+CD20
0, //U+CD21
0, //U+CD22
0, //U+CD23
0, //U+CD24
0, //U+CD25
0, //U+CD26
0, //U+CD27
0, //U+CD28
0, //U+CD29
0, //U+CD2A
0, //U+CD2B
0, //U+CD2C
0, //U+CD2D
0, //U+CD2E
0, //U+CD2F
0, //U+CD30
0, //U+CD31
0, //U+CD32
0, //U+CD33
0, //U+CD34
0, //U+CD35
0, //U+CD36
0, //U+CD37
0, //U+CD38
0, //U+CD39
0, //U+CD3A
0, //U+CD3B
0, //U+CD3C
0, //U+CD3D
0, //U+CD3E
0, //U+CD3F
0, //U+CD40
0, //U+CD41
0, //U+CD42
0, //U+CD43
0, //U+CD44
0, //U+CD45
0, //U+CD46
0, //U+CD47
0, //U+CD48
0, //U+CD49
0, //U+CD4A
0, //U+CD4B
0, //U+CD4C
0, //U+CD4D
0, //U+CD4E
0, //U+CD4F
0, //U+CD50
0, //U+CD51
0, //U+CD52
0, //U+CD53
0, //U+CD54
0, //U+CD55
0, //U+CD56
0, //U+CD57
0, //U+CD58
0, //U+CD59
0, //U+CD5A
0, //U+CD5B
0, //U+CD5C
0, //U+CD5D
0, //U+CD5E
0, //U+CD5F
0, //U+CD60
0, //U+CD61
0, //U+CD62
0, //U+CD63
0, //U+CD64
0, //U+CD65
0, //U+CD66
0, //U+CD67
0, //U+CD68
0, //U+CD69
0, //U+CD6A
0, //U+CD6B
0, //U+CD6C
0, //U+CD6D
0, //U+CD6E
0, //U+CD6F
0, //U+CD70
0, //U+CD71
0, //U+CD72
0, //U+CD73
0, //U+CD74
0, //U+CD75
0, //U+CD76
0, //U+CD77
0, //U+CD78
0, //U+CD79
0, //U+CD7A
0, //U+CD7B
0, //U+CD7C
0, //U+CD7D
0, //U+CD7E
0, //U+CD7F
0, //U+CD80
0, //U+CD81
0, //U+CD82
0, //U+CD83
0, //U+CD84
0, //U+CD85
0, //U+CD86
0, //U+CD87
0, //U+CD88
0, //U+CD89
0, //U+CD8A
0, //U+CD8B
0, //U+CD8C
0, //U+CD8D
0, //U+CD8E
0, //U+CD8F
0, //U+CD90
0, //U+CD91
0, //U+CD92
0, //U+CD93
0, //U+CD94
0, //U+CD95
0, //U+CD96
0, //U+CD97
0, //U+CD98
0, //U+CD99
0, //U+CD9A
0, //U+CD9B
0, //U+CD9C
0, //U+CD9D
0, //U+CD9E
0, //U+CD9F
0, //U+CDA0
0, //U+CDA1
0, //U+CDA2
0, //U+CDA3
0, //U+CDA4
0, //U+CDA5
0, //U+CDA6
0, //U+CDA7
0, //U+CDA8
0, //U+CDA9
0, //U+CDAA
0, //U+CDAB
0, //U+CDAC
0, //U+CDAD
0, //U+CDAE
0, //U+CDAF
0, //U+CDB0
0, //U+CDB1
0, //U+CDB2
0, //U+CDB3
0, //U+CDB4
0, //U+CDB5
0, //U+CDB6
0, //U+CDB7
0, //U+CDB8
0, //U+CDB9
0, //U+CDBA
0, //U+CDBB
0, //U+CDBC
0, //U+CDBD
0, //U+CDBE
0, //U+CDBF
0, //U+CDC0
0, //U+CDC1
0, //U+CDC2
0, //U+CDC3
0, //U+CDC4
0, //U+CDC5
0, //U+CDC6
0, //U+CDC7
0, //U+CDC8
0, //U+CDC9
0, //U+CDCA
0, //U+CDCB
0, //U+CDCC
0, //U+CDCD
0, //U+CDCE
0, //U+CDCF
0, //U+CDD0
0, //U+CDD1
0, //U+CDD2
0, //U+CDD3
0, //U+CDD4
0, //U+CDD5
0, //U+CDD6
0, //U+CDD7
0, //U+CDD8
0, //U+CDD9
0, //U+CDDA
0, //U+CDDB
0, //U+CDDC
0, //U+CDDD
0, //U+CDDE
0, //U+CDDF
0, //U+CDE0
0, //U+CDE1
0, //U+CDE2
0, //U+CDE3
0, //U+CDE4
0, //U+CDE5
0, //U+CDE6
0, //U+CDE7
0, //U+CDE8
0, //U+CDE9
0, //U+CDEA
0, //U+CDEB
0, //U+CDEC
0, //U+CDED
0, //U+CDEE
0, //U+CDEF
0, //U+CDF0
0, //U+CDF1
0, //U+CDF2
0, //U+CDF3
0, //U+CDF4
0, //U+CDF5
0, //U+CDF6
0, //U+CDF7
0, //U+CDF8
0, //U+CDF9
0, //U+CDFA
0, //U+CDFB
0, //U+CDFC
0, //U+CDFD
0, //U+CDFE
0, //U+CDFF
0, //U+CE00
0, //U+CE01
0, //U+CE02
0, //U+CE03
0, //U+CE04
0, //U+CE05
0, //U+CE06
0, //U+CE07
0, //U+CE08
0, //U+CE09
0, //U+CE0A
0, //U+CE0B
0, //U+CE0C
0, //U+CE0D
0, //U+CE0E
0, //U+CE0F
0, //U+CE10
0, //U+CE11
0, //U+CE12
0, //U+CE13
0, //U+CE14
0, //U+CE15
0, //U+CE16
0, //U+CE17
0, //U+CE18
0, //U+CE19
0, //U+CE1A
0, //U+CE1B
0, //U+CE1C
0, //U+CE1D
0, //U+CE1E
0, //U+CE1F
0, //U+CE20
0, //U+CE21
0, //U+CE22
0, //U+CE23
0, //U+CE24
0, //U+CE25
0, //U+CE26
0, //U+CE27
0, //U+CE28
0, //U+CE29
0, //U+CE2A
0, //U+CE2B
0, //U+CE2C
0, //U+CE2D
0, //U+CE2E
0, //U+CE2F
0, //U+CE30
0, //U+CE31
0, //U+CE32
0, //U+CE33
0, //U+CE34
0, //U+CE35
0, //U+CE36
0, //U+CE37
0, //U+CE38
0, //U+CE39
0, //U+CE3A
0, //U+CE3B
0, //U+CE3C
0, //U+CE3D
0, //U+CE3E
0, //U+CE3F
0, //U+CE40
0, //U+CE41
0, //U+CE42
0, //U+CE43
0, //U+CE44
0, //U+CE45
0, //U+CE46
0, //U+CE47
0, //U+CE48
0, //U+CE49
0, //U+CE4A
0, //U+CE4B
0, //U+CE4C
0, //U+CE4D
0, //U+CE4E
0, //U+CE4F
0, //U+CE50
0, //U+CE51
0, //U+CE52
0, //U+CE53
0, //U+CE54
0, //U+CE55
0, //U+CE56
0, //U+CE57
0, //U+CE58
0, //U+CE59
0, //U+CE5A
0, //U+CE5B
0, //U+CE5C
0, //U+CE5D
0, //U+CE5E
0, //U+CE5F
0, //U+CE60
0, //U+CE61
0, //U+CE62
0, //U+CE63
0, //U+CE64
0, //U+CE65
0, //U+CE66
0, //U+CE67
0, //U+CE68
0, //U+CE69
0, //U+CE6A
0, //U+CE6B
0, //U+CE6C
0, //U+CE6D
0, //U+CE6E
0, //U+CE6F
0, //U+CE70
0, //U+CE71
0, //U+CE72
0, //U+CE73
0, //U+CE74
0, //U+CE75
0, //U+CE76
0, //U+CE77
0, //U+CE78
0, //U+CE79
0, //U+CE7A
0, //U+CE7B
0, //U+CE7C
0, //U+CE7D
0, //U+CE7E
0, //U+CE7F
0, //U+CE80
0, //U+CE81
0, //U+CE82
0, //U+CE83
0, //U+CE84
0, //U+CE85
0, //U+CE86
0, //U+CE87
0, //U+CE88
0, //U+CE89
0, //U+CE8A
0, //U+CE8B
0, //U+CE8C
0, //U+CE8D
0, //U+CE8E
0, //U+CE8F
0, //U+CE90
0, //U+CE91
0, //U+CE92
0, //U+CE93
0, //U+CE94
0, //U+CE95
0, //U+CE96
0, //U+CE97
0, //U+CE98
0, //U+CE99
0, //U+CE9A
0, //U+CE9B
0, //U+CE9C
0, //U+CE9D
0, //U+CE9E
0, //U+CE9F
0, //U+CEA0
0, //U+CEA1
0, //U+CEA2
0, //U+CEA3
0, //U+CEA4
0, //U+CEA5
0, //U+CEA6
0, //U+CEA7
0, //U+CEA8
0, //U+CEA9
0, //U+CEAA
0, //U+CEAB
0, //U+CEAC
0, //U+CEAD
0, //U+CEAE
0, //U+CEAF
0, //U+CEB0
0, //U+CEB1
0, //U+CEB2
0, //U+CEB3
0, //U+CEB4
0, //U+CEB5
0, //U+CEB6
0, //U+CEB7
0, //U+CEB8
0, //U+CEB9
0, //U+CEBA
0, //U+CEBB
0, //U+CEBC
0, //U+CEBD
0, //U+CEBE
0, //U+CEBF
0, //U+CEC0
0, //U+CEC1
0, //U+CEC2
0, //U+CEC3
0, //U+CEC4
0, //U+CEC5
0, //U+CEC6
0, //U+CEC7
0, //U+CEC8
0, //U+CEC9
0, //U+CECA
0, //U+CECB
0, //U+CECC
0, //U+CECD
0, //U+CECE
0, //U+CECF
0, //U+CED0
0, //U+CED1
0, //U+CED2
0, //U+CED3
0, //U+CED4
0, //U+CED5
0, //U+CED6
0, //U+CED7
0, //U+CED8
0, //U+CED9
0, //U+CEDA
0, //U+CEDB
0, //U+CEDC
0, //U+CEDD
0, //U+CEDE
0, //U+CEDF
0, //U+CEE0
0, //U+CEE1
0, //U+CEE2
0, //U+CEE3
0, //U+CEE4
0, //U+CEE5
0, //U+CEE6
0, //U+CEE7
0, //U+CEE8
0, //U+CEE9
0, //U+CEEA
0, //U+CEEB
0, //U+CEEC
0, //U+CEED
0, //U+CEEE
0, //U+CEEF
0, //U+CEF0
0, //U+CEF1
0, //U+CEF2
0, //U+CEF3
0, //U+CEF4
0, //U+CEF5
0, //U+CEF6
0, //U+CEF7
0, //U+CEF8
0, //U+CEF9
0, //U+CEFA
0, //U+CEFB
0, //U+CEFC
0, //U+CEFD
0, //U+CEFE
0, //U+CEFF
0, //U+CF00
0, //U+CF01
0, //U+CF02
0, //U+CF03
0, //U+CF04
0, //U+CF05
0, //U+CF06
0, //U+CF07
0, //U+CF08
0, //U+CF09
0, //U+CF0A
0, //U+CF0B
0, //U+CF0C
0, //U+CF0D
0, //U+CF0E
0, //U+CF0F
0, //U+CF10
0, //U+CF11
0, //U+CF12
0, //U+CF13
0, //U+CF14
0, //U+CF15
0, //U+CF16
0, //U+CF17
0, //U+CF18
0, //U+CF19
0, //U+CF1A
0, //U+CF1B
0, //U+CF1C
0, //U+CF1D
0, //U+CF1E
0, //U+CF1F
0, //U+CF20
0, //U+CF21
0, //U+CF22
0, //U+CF23
0, //U+CF24
0, //U+CF25
0, //U+CF26
0, //U+CF27
0, //U+CF28
0, //U+CF29
0, //U+CF2A
0, //U+CF2B
0, //U+CF2C
0, //U+CF2D
0, //U+CF2E
0, //U+CF2F
0, //U+CF30
0, //U+CF31
0, //U+CF32
0, //U+CF33
0, //U+CF34
0, //U+CF35
0, //U+CF36
0, //U+CF37
0, //U+CF38
0, //U+CF39
0, //U+CF3A
0, //U+CF3B
0, //U+CF3C
0, //U+CF3D
0, //U+CF3E
0, //U+CF3F
0, //U+CF40
0, //U+CF41
0, //U+CF42
0, //U+CF43
0, //U+CF44
0, //U+CF45
0, //U+CF46
0, //U+CF47
0, //U+CF48
0, //U+CF49
0, //U+CF4A
0, //U+CF4B
0, //U+CF4C
0, //U+CF4D
0, //U+CF4E
0, //U+CF4F
0, //U+CF50
0, //U+CF51
0, //U+CF52
0, //U+CF53
0, //U+CF54
0, //U+CF55
0, //U+CF56
0, //U+CF57
0, //U+CF58
0, //U+CF59
0, //U+CF5A
0, //U+CF5B
0, //U+CF5C
0, //U+CF5D
0, //U+CF5E
0, //U+CF5F
0, //U+CF60
0, //U+CF61
0, //U+CF62
0, //U+CF63
0, //U+CF64
0, //U+CF65
0, //U+CF66
0, //U+CF67
0, //U+CF68
0, //U+CF69
0, //U+CF6A
0, //U+CF6B
0, //U+CF6C
0, //U+CF6D
0, //U+CF6E
0, //U+CF6F
0, //U+CF70
0, //U+CF71
0, //U+CF72
0, //U+CF73
0, //U+CF74
0, //U+CF75
0, //U+CF76
0, //U+CF77
0, //U+CF78
0, //U+CF79
0, //U+CF7A
0, //U+CF7B
0, //U+CF7C
0, //U+CF7D
0, //U+CF7E
0, //U+CF7F
0, //U+CF80
0, //U+CF81
0, //U+CF82
0, //U+CF83
0, //U+CF84
0, //U+CF85
0, //U+CF86
0, //U+CF87
0, //U+CF88
0, //U+CF89
0, //U+CF8A
0, //U+CF8B
0, //U+CF8C
0, //U+CF8D
0, //U+CF8E
0, //U+CF8F
0, //U+CF90
0, //U+CF91
0, //U+CF92
0, //U+CF93
0, //U+CF94
0, //U+CF95
0, //U+CF96
0, //U+CF97
0, //U+CF98
0, //U+CF99
0, //U+CF9A
0, //U+CF9B
0, //U+CF9C
0, //U+CF9D
0, //U+CF9E
0, //U+CF9F
0, //U+CFA0
0, //U+CFA1
0, //U+CFA2
0, //U+CFA3
0, //U+CFA4
0, //U+CFA5
0, //U+CFA6
0, //U+CFA7
0, //U+CFA8
0, //U+CFA9
0, //U+CFAA
0, //U+CFAB
0, //U+CFAC
0, //U+CFAD
0, //U+CFAE
0, //U+CFAF
0, //U+CFB0
0, //U+CFB1
0, //U+CFB2
0, //U+CFB3
0, //U+CFB4
0, //U+CFB5
0, //U+CFB6
0, //U+CFB7
0, //U+CFB8
0, //U+CFB9
0, //U+CFBA
0, //U+CFBB
0, //U+CFBC
0, //U+CFBD
0, //U+CFBE
0, //U+CFBF
0, //U+CFC0
0, //U+CFC1
0, //U+CFC2
0, //U+CFC3
0, //U+CFC4
0, //U+CFC5
0, //U+CFC6
0, //U+CFC7
0, //U+CFC8
0, //U+CFC9
0, //U+CFCA
0, //U+CFCB
0, //U+CFCC
0, //U+CFCD
0, //U+CFCE
0, //U+CFCF
0, //U+CFD0
0, //U+CFD1
0, //U+CFD2
0, //U+CFD3
0, //U+CFD4
0, //U+CFD5
0, //U+CFD6
0, //U+CFD7
0, //U+CFD8
0, //U+CFD9
0, //U+CFDA
0, //U+CFDB
0, //U+CFDC
0, //U+CFDD
0, //U+CFDE
0, //U+CFDF
0, //U+CFE0
0, //U+CFE1
0, //U+CFE2
0, //U+CFE3
0, //U+CFE4
0, //U+CFE5
0, //U+CFE6
0, //U+CFE7
0, //U+CFE8
0, //U+CFE9
0, //U+CFEA
0, //U+CFEB
0, //U+CFEC
0, //U+CFED
0, //U+CFEE
0, //U+CFEF
0, //U+CFF0
0, //U+CFF1
0, //U+CFF2
0, //U+CFF3
0, //U+CFF4
0, //U+CFF5
0, //U+CFF6
0, //U+CFF7
0, //U+CFF8
0, //U+CFF9
0, //U+CFFA
0, //U+CFFB
0, //U+CFFC
0, //U+CFFD
0, //U+CFFE
0, //U+CFFF
0, //U+D000
0, //U+D001
0, //U+D002
0, //U+D003
0, //U+D004
0, //U+D005
0, //U+D006
0, //U+D007
0, //U+D008
0, //U+D009
0, //U+D00A
0, //U+D00B
0, //U+D00C
0, //U+D00D
0, //U+D00E
0, //U+D00F
0, //U+D010
0, //U+D011
0, //U+D012
0, //U+D013
0, //U+D014
0, //U+D015
0, //U+D016
0, //U+D017
0, //U+D018
0, //U+D019
0, //U+D01A
0, //U+D01B
0, //U+D01C
0, //U+D01D
0, //U+D01E
0, //U+D01F
0, //U+D020
0, //U+D021
0, //U+D022
0, //U+D023
0, //U+D024
0, //U+D025
0, //U+D026
0, //U+D027
0, //U+D028
0, //U+D029
0, //U+D02A
0, //U+D02B
0, //U+D02C
0, //U+D02D
0, //U+D02E
0, //U+D02F
0, //U+D030
0, //U+D031
0, //U+D032
0, //U+D033
0, //U+D034
0, //U+D035
0, //U+D036
0, //U+D037
0, //U+D038
0, //U+D039
0, //U+D03A
0, //U+D03B
0, //U+D03C
0, //U+D03D
0, //U+D03E
0, //U+D03F
0, //U+D040
0, //U+D041
0, //U+D042
0, //U+D043
0, //U+D044
0, //U+D045
0, //U+D046
0, //U+D047
0, //U+D048
0, //U+D049
0, //U+D04A
0, //U+D04B
0, //U+D04C
0, //U+D04D
0, //U+D04E
0, //U+D04F
0, //U+D050
0, //U+D051
0, //U+D052
0, //U+D053
0, //U+D054
0, //U+D055
0, //U+D056
0, //U+D057
0, //U+D058
0, //U+D059
0, //U+D05A
0, //U+D05B
0, //U+D05C
0, //U+D05D
0, //U+D05E
0, //U+D05F
0, //U+D060
0, //U+D061
0, //U+D062
0, //U+D063
0, //U+D064
0, //U+D065
0, //U+D066
0, //U+D067
0, //U+D068
0, //U+D069
0, //U+D06A
0, //U+D06B
0, //U+D06C
0, //U+D06D
0, //U+D06E
0, //U+D06F
0, //U+D070
0, //U+D071
0, //U+D072
0, //U+D073
0, //U+D074
0, //U+D075
0, //U+D076
0, //U+D077
0, //U+D078
0, //U+D079
0, //U+D07A
0, //U+D07B
0, //U+D07C
0, //U+D07D
0, //U+D07E
0, //U+D07F
0, //U+D080
0, //U+D081
0, //U+D082
0, //U+D083
0, //U+D084
0, //U+D085
0, //U+D086
0, //U+D087
0, //U+D088
0, //U+D089
0, //U+D08A
0, //U+D08B
0, //U+D08C
0, //U+D08D
0, //U+D08E
0, //U+D08F
0, //U+D090
0, //U+D091
0, //U+D092
0, //U+D093
0, //U+D094
0, //U+D095
0, //U+D096
0, //U+D097
0, //U+D098
0, //U+D099
0, //U+D09A
0, //U+D09B
0, //U+D09C
0, //U+D09D
0, //U+D09E
0, //U+D09F
0, //U+D0A0
0, //U+D0A1
0, //U+D0A2
0, //U+D0A3
0, //U+D0A4
0, //U+D0A5
0, //U+D0A6
0, //U+D0A7
0, //U+D0A8
0, //U+D0A9
0, //U+D0AA
0, //U+D0AB
0, //U+D0AC
0, //U+D0AD
0, //U+D0AE
0, //U+D0AF
0, //U+D0B0
0, //U+D0B1
0, //U+D0B2
0, //U+D0B3
0, //U+D0B4
0, //U+D0B5
0, //U+D0B6
0, //U+D0B7
0, //U+D0B8
0, //U+D0B9
0, //U+D0BA
0, //U+D0BB
0, //U+D0BC
0, //U+D0BD
0, //U+D0BE
0, //U+D0BF
0, //U+D0C0
0, //U+D0C1
0, //U+D0C2
0, //U+D0C3
0, //U+D0C4
0, //U+D0C5
0, //U+D0C6
0, //U+D0C7
0, //U+D0C8
0, //U+D0C9
0, //U+D0CA
0, //U+D0CB
0, //U+D0CC
0, //U+D0CD
0, //U+D0CE
0, //U+D0CF
0, //U+D0D0
0, //U+D0D1
0, //U+D0D2
0, //U+D0D3
0, //U+D0D4
0, //U+D0D5
0, //U+D0D6
0, //U+D0D7
0, //U+D0D8
0, //U+D0D9
0, //U+D0DA
0, //U+D0DB
0, //U+D0DC
0, //U+D0DD
0, //U+D0DE
0, //U+D0DF
0, //U+D0E0
0, //U+D0E1
0, //U+D0E2
0, //U+D0E3
0, //U+D0E4
0, //U+D0E5
0, //U+D0E6
0, //U+D0E7
0, //U+D0E8
0, //U+D0E9
0, //U+D0EA
0, //U+D0EB
0, //U+D0EC
0, //U+D0ED
0, //U+D0EE
0, //U+D0EF
0, //U+D0F0
0, //U+D0F1
0, //U+D0F2
0, //U+D0F3
0, //U+D0F4
0, //U+D0F5
0, //U+D0F6
0, //U+D0F7
0, //U+D0F8
0, //U+D0F9
0, //U+D0FA
0, //U+D0FB
0, //U+D0FC
0, //U+D0FD
0, //U+D0FE
0, //U+D0FF
0, //U+D100
0, //U+D101
0, //U+D102
0, //U+D103
0, //U+D104
0, //U+D105
0, //U+D106
0, //U+D107
0, //U+D108
0, //U+D109
0, //U+D10A
0, //U+D10B
0, //U+D10C
0, //U+D10D
0, //U+D10E
0, //U+D10F
0, //U+D110
0, //U+D111
0, //U+D112
0, //U+D113
0, //U+D114
0, //U+D115
0, //U+D116
0, //U+D117
0, //U+D118
0, //U+D119
0, //U+D11A
0, //U+D11B
0, //U+D11C
0, //U+D11D
0, //U+D11E
0, //U+D11F
0, //U+D120
0, //U+D121
0, //U+D122
0, //U+D123
0, //U+D124
0, //U+D125
0, //U+D126
0, //U+D127
0, //U+D128
0, //U+D129
0, //U+D12A
0, //U+D12B
0, //U+D12C
0, //U+D12D
0, //U+D12E
0, //U+D12F
0, //U+D130
0, //U+D131
0, //U+D132
0, //U+D133
0, //U+D134
0, //U+D135
0, //U+D136
0, //U+D137
0, //U+D138
0, //U+D139
0, //U+D13A
0, //U+D13B
0, //U+D13C
0, //U+D13D
0, //U+D13E
0, //U+D13F
0, //U+D140
0, //U+D141
0, //U+D142
0, //U+D143
0, //U+D144
0, //U+D145
0, //U+D146
0, //U+D147
0, //U+D148
0, //U+D149
0, //U+D14A
0, //U+D14B
0, //U+D14C
0, //U+D14D
0, //U+D14E
0, //U+D14F
0, //U+D150
0, //U+D151
0, //U+D152
0, //U+D153
0, //U+D154
0, //U+D155
0, //U+D156
0, //U+D157
0, //U+D158
0, //U+D159
0, //U+D15A
0, //U+D15B
0, //U+D15C
0, //U+D15D
0, //U+D15E
0, //U+D15F
0, //U+D160
0, //U+D161
0, //U+D162
0, //U+D163
0, //U+D164
0, //U+D165
0, //U+D166
0, //U+D167
0, //U+D168
0, //U+D169
0, //U+D16A
0, //U+D16B
0, //U+D16C
0, //U+D16D
0, //U+D16E
0, //U+D16F
0, //U+D170
0, //U+D171
0, //U+D172
0, //U+D173
0, //U+D174
0, //U+D175
0, //U+D176
0, //U+D177
0, //U+D178
0, //U+D179
0, //U+D17A
0, //U+D17B
0, //U+D17C
0, //U+D17D
0, //U+D17E
0, //U+D17F
0, //U+D180
0, //U+D181
0, //U+D182
0, //U+D183
0, //U+D184
0, //U+D185
0, //U+D186
0, //U+D187
0, //U+D188
0, //U+D189
0, //U+D18A
0, //U+D18B
0, //U+D18C
0, //U+D18D
0, //U+D18E
0, //U+D18F
0, //U+D190
0, //U+D191
0, //U+D192
0, //U+D193
0, //U+D194
0, //U+D195
0, //U+D196
0, //U+D197
0, //U+D198
0, //U+D199
0, //U+D19A
0, //U+D19B
0, //U+D19C
0, //U+D19D
0, //U+D19E
0, //U+D19F
0, //U+D1A0
0, //U+D1A1
0, //U+D1A2
0, //U+D1A3
0, //U+D1A4
0, //U+D1A5
0, //U+D1A6
0, //U+D1A7
0, //U+D1A8
0, //U+D1A9
0, //U+D1AA
0, //U+D1AB
0, //U+D1AC
0, //U+D1AD
0, //U+D1AE
0, //U+D1AF
0, //U+D1B0
0, //U+D1B1
0, //U+D1B2
0, //U+D1B3
0, //U+D1B4
0, //U+D1B5
0, //U+D1B6
0, //U+D1B7
0, //U+D1B8
0, //U+D1B9
0, //U+D1BA
0, //U+D1BB
0, //U+D1BC
0, //U+D1BD
0, //U+D1BE
0, //U+D1BF
0, //U+D1C0
0, //U+D1C1
0, //U+D1C2
0, //U+D1C3
0, //U+D1C4
0, //U+D1C5
0, //U+D1C6
0, //U+D1C7
0, //U+D1C8
0, //U+D1C9
0, //U+D1CA
0, //U+D1CB
0, //U+D1CC
0, //U+D1CD
0, //U+D1CE
0, //U+D1CF
0, //U+D1D0
0, //U+D1D1
0, //U+D1D2
0, //U+D1D3
0, //U+D1D4
0, //U+D1D5
0, //U+D1D6
0, //U+D1D7
0, //U+D1D8
0, //U+D1D9
0, //U+D1DA
0, //U+D1DB
0, //U+D1DC
0, //U+D1DD
0, //U+D1DE
0, //U+D1DF
0, //U+D1E0
0, //U+D1E1
0, //U+D1E2
0, //U+D1E3
0, //U+D1E4
0, //U+D1E5
0, //U+D1E6
0, //U+D1E7
0, //U+D1E8
0, //U+D1E9
0, //U+D1EA
0, //U+D1EB
0, //U+D1EC
0, //U+D1ED
0, //U+D1EE
0, //U+D1EF
0, //U+D1F0
0, //U+D1F1
0, //U+D1F2
0, //U+D1F3
0, //U+D1F4
0, //U+D1F5
0, //U+D1F6
0, //U+D1F7
0, //U+D1F8
0, //U+D1F9
0, //U+D1FA
0, //U+D1FB
0, //U+D1FC
0, //U+D1FD
0, //U+D1FE
0, //U+D1FF
0, //U+D200
0, //U+D201
0, //U+D202
0, //U+D203
0, //U+D204
0, //U+D205
0, //U+D206
0, //U+D207
0, //U+D208
0, //U+D209
0, //U+D20A
0, //U+D20B
0, //U+D20C
0, //U+D20D
0, //U+D20E
0, //U+D20F
0, //U+D210
0, //U+D211
0, //U+D212
0, //U+D213
0, //U+D214
0, //U+D215
0, //U+D216
0, //U+D217
0, //U+D218
0, //U+D219
0, //U+D21A
0, //U+D21B
0, //U+D21C
0, //U+D21D
0, //U+D21E
0, //U+D21F
0, //U+D220
0, //U+D221
0, //U+D222
0, //U+D223
0, //U+D224
0, //U+D225
0, //U+D226
0, //U+D227
0, //U+D228
0, //U+D229
0, //U+D22A
0, //U+D22B
0, //U+D22C
0, //U+D22D
0, //U+D22E
0, //U+D22F
0, //U+D230
0, //U+D231
0, //U+D232
0, //U+D233
0, //U+D234
0, //U+D235
0, //U+D236
0, //U+D237
0, //U+D238
0, //U+D239
0, //U+D23A
0, //U+D23B
0, //U+D23C
0, //U+D23D
0, //U+D23E
0, //U+D23F
0, //U+D240
0, //U+D241
0, //U+D242
0, //U+D243
0, //U+D244
0, //U+D245
0, //U+D246
0, //U+D247
0, //U+D248
0, //U+D249
0, //U+D24A
0, //U+D24B
0, //U+D24C
0, //U+D24D
0, //U+D24E
0, //U+D24F
0, //U+D250
0, //U+D251
0, //U+D252
0, //U+D253
0, //U+D254
0, //U+D255
0, //U+D256
0, //U+D257
0, //U+D258
0, //U+D259
0, //U+D25A
0, //U+D25B
0, //U+D25C
0, //U+D25D
0, //U+D25E
0, //U+D25F
0, //U+D260
0, //U+D261
0, //U+D262
0, //U+D263
0, //U+D264
0, //U+D265
0, //U+D266
0, //U+D267
0, //U+D268
0, //U+D269
0, //U+D26A
0, //U+D26B
0, //U+D26C
0, //U+D26D
0, //U+D26E
0, //U+D26F
0, //U+D270
0, //U+D271
0, //U+D272
0, //U+D273
0, //U+D274
0, //U+D275
0, //U+D276
0, //U+D277
0, //U+D278
0, //U+D279
0, //U+D27A
0, //U+D27B
0, //U+D27C
0, //U+D27D
0, //U+D27E
0, //U+D27F
0, //U+D280
0, //U+D281
0, //U+D282
0, //U+D283
0, //U+D284
0, //U+D285
0, //U+D286
0, //U+D287
0, //U+D288
0, //U+D289
0, //U+D28A
0, //U+D28B
0, //U+D28C
0, //U+D28D
0, //U+D28E
0, //U+D28F
0, //U+D290
0, //U+D291
0, //U+D292
0, //U+D293
0, //U+D294
0, //U+D295
0, //U+D296
0, //U+D297
0, //U+D298
0, //U+D299
0, //U+D29A
0, //U+D29B
0, //U+D29C
0, //U+D29D
0, //U+D29E
0, //U+D29F
0, //U+D2A0
0, //U+D2A1
0, //U+D2A2
0, //U+D2A3
0, //U+D2A4
0, //U+D2A5
0, //U+D2A6
0, //U+D2A7
0, //U+D2A8
0, //U+D2A9
0, //U+D2AA
0, //U+D2AB
0, //U+D2AC
0, //U+D2AD
0, //U+D2AE
0, //U+D2AF
0, //U+D2B0
0, //U+D2B1
0, //U+D2B2
0, //U+D2B3
0, //U+D2B4
0, //U+D2B5
0, //U+D2B6
0, //U+D2B7
0, //U+D2B8
0, //U+D2B9
0, //U+D2BA
0, //U+D2BB
0, //U+D2BC
0, //U+D2BD
0, //U+D2BE
0, //U+D2BF
0, //U+D2C0
0, //U+D2C1
0, //U+D2C2
0, //U+D2C3
0, //U+D2C4
0, //U+D2C5
0, //U+D2C6
0, //U+D2C7
0, //U+D2C8
0, //U+D2C9
0, //U+D2CA
0, //U+D2CB
0, //U+D2CC
0, //U+D2CD
0, //U+D2CE
0, //U+D2CF
0, //U+D2D0
0, //U+D2D1
0, //U+D2D2
0, //U+D2D3
0, //U+D2D4
0, //U+D2D5
0, //U+D2D6
0, //U+D2D7
0, //U+D2D8
0, //U+D2D9
0, //U+D2DA
0, //U+D2DB
0, //U+D2DC
0, //U+D2DD
0, //U+D2DE
0, //U+D2DF
0, //U+D2E0
0, //U+D2E1
0, //U+D2E2
0, //U+D2E3
0, //U+D2E4
0, //U+D2E5
0, //U+D2E6
0, //U+D2E7
0, //U+D2E8
0, //U+D2E9
0, //U+D2EA
0, //U+D2EB
0, //U+D2EC
0, //U+D2ED
0, //U+D2EE
0, //U+D2EF
0, //U+D2F0
0, //U+D2F1
0, //U+D2F2
0, //U+D2F3
0, //U+D2F4
0, //U+D2F5
0, //U+D2F6
0, //U+D2F7
0, //U+D2F8
0, //U+D2F9
0, //U+D2FA
0, //U+D2FB
0, //U+D2FC
0, //U+D2FD
0, //U+D2FE
0, //U+D2FF
0, //U+D300
0, //U+D301
0, //U+D302
0, //U+D303
0, //U+D304
0, //U+D305
0, //U+D306
0, //U+D307
0, //U+D308
0, //U+D309
0, //U+D30A
0, //U+D30B
0, //U+D30C
0, //U+D30D
0, //U+D30E
0, //U+D30F
0, //U+D310
0, //U+D311
0, //U+D312
0, //U+D313
0, //U+D314
0, //U+D315
0, //U+D316
0, //U+D317
0, //U+D318
0, //U+D319
0, //U+D31A
0, //U+D31B
0, //U+D31C
0, //U+D31D
0, //U+D31E
0, //U+D31F
0, //U+D320
0, //U+D321
0, //U+D322
0, //U+D323
0, //U+D324
0, //U+D325
0, //U+D326
0, //U+D327
0, //U+D328
0, //U+D329
0, //U+D32A
0, //U+D32B
0, //U+D32C
0, //U+D32D
0, //U+D32E
0, //U+D32F
0, //U+D330
0, //U+D331
0, //U+D332
0, //U+D333
0, //U+D334
0, //U+D335
0, //U+D336
0, //U+D337
0, //U+D338
0, //U+D339
0, //U+D33A
0, //U+D33B
0, //U+D33C
0, //U+D33D
0, //U+D33E
0, //U+D33F
0, //U+D340
0, //U+D341
0, //U+D342
0, //U+D343
0, //U+D344
0, //U+D345
0, //U+D346
0, //U+D347
0, //U+D348
0, //U+D349
0, //U+D34A
0, //U+D34B
0, //U+D34C
0, //U+D34D
0, //U+D34E
0, //U+D34F
0, //U+D350
0, //U+D351
0, //U+D352
0, //U+D353
0, //U+D354
0, //U+D355
0, //U+D356
0, //U+D357
0, //U+D358
0, //U+D359
0, //U+D35A
0, //U+D35B
0, //U+D35C
0, //U+D35D
0, //U+D35E
0, //U+D35F
0, //U+D360
0, //U+D361
0, //U+D362
0, //U+D363
0, //U+D364
0, //U+D365
0, //U+D366
0, //U+D367
0, //U+D368
0, //U+D369
0, //U+D36A
0, //U+D36B
0, //U+D36C
0, //U+D36D
0, //U+D36E
0, //U+D36F
0, //U+D370
0, //U+D371
0, //U+D372
0, //U+D373
0, //U+D374
0, //U+D375
0, //U+D376
0, //U+D377
0, //U+D378
0, //U+D379
0, //U+D37A
0, //U+D37B
0, //U+D37C
0, //U+D37D
0, //U+D37E
0, //U+D37F
0, //U+D380
0, //U+D381
0, //U+D382
0, //U+D383
0, //U+D384
0, //U+D385
0, //U+D386
0, //U+D387
0, //U+D388
0, //U+D389
0, //U+D38A
0, //U+D38B
0, //U+D38C
0, //U+D38D
0, //U+D38E
0, //U+D38F
0, //U+D390
0, //U+D391
0, //U+D392
0, //U+D393
0, //U+D394
0, //U+D395
0, //U+D396
0, //U+D397
0, //U+D398
0, //U+D399
0, //U+D39A
0, //U+D39B
0, //U+D39C
0, //U+D39D
0, //U+D39E
0, //U+D39F
0, //U+D3A0
0, //U+D3A1
0, //U+D3A2
0, //U+D3A3
0, //U+D3A4
0, //U+D3A5
0, //U+D3A6
0, //U+D3A7
0, //U+D3A8
0, //U+D3A9
0, //U+D3AA
0, //U+D3AB
0, //U+D3AC
0, //U+D3AD
0, //U+D3AE
0, //U+D3AF
0, //U+D3B0
0, //U+D3B1
0, //U+D3B2
0, //U+D3B3
0, //U+D3B4
0, //U+D3B5
0, //U+D3B6
0, //U+D3B7
0, //U+D3B8
0, //U+D3B9
0, //U+D3BA
0, //U+D3BB
0, //U+D3BC
0, //U+D3BD
0, //U+D3BE
0, //U+D3BF
0, //U+D3C0
0, //U+D3C1
0, //U+D3C2
0, //U+D3C3
0, //U+D3C4
0, //U+D3C5
0, //U+D3C6
0, //U+D3C7
0, //U+D3C8
0, //U+D3C9
0, //U+D3CA
0, //U+D3CB
0, //U+D3CC
0, //U+D3CD
0, //U+D3CE
0, //U+D3CF
0, //U+D3D0
0, //U+D3D1
0, //U+D3D2
0, //U+D3D3
0, //U+D3D4
0, //U+D3D5
0, //U+D3D6
0, //U+D3D7
0, //U+D3D8
0, //U+D3D9
0, //U+D3DA
0, //U+D3DB
0, //U+D3DC
0, //U+D3DD
0, //U+D3DE
0, //U+D3DF
0, //U+D3E0
0, //U+D3E1
0, //U+D3E2
0, //U+D3E3
0, //U+D3E4
0, //U+D3E5
0, //U+D3E6
0, //U+D3E7
0, //U+D3E8
0, //U+D3E9
0, //U+D3EA
0, //U+D3EB
0, //U+D3EC
0, //U+D3ED
0, //U+D3EE
0, //U+D3EF
0, //U+D3F0
0, //U+D3F1
0, //U+D3F2
0, //U+D3F3
0, //U+D3F4
0, //U+D3F5
0, //U+D3F6
0, //U+D3F7
0, //U+D3F8
0, //U+D3F9
0, //U+D3FA
0, //U+D3FB
0, //U+D3FC
0, //U+D3FD
0, //U+D3FE
0, //U+D3FF
0, //U+D400
0, //U+D401
0, //U+D402
0, //U+D403
0, //U+D404
0, //U+D405
0, //U+D406
0, //U+D407
0, //U+D408
0, //U+D409
0, //U+D40A
0, //U+D40B
0, //U+D40C
0, //U+D40D
0, //U+D40E
0, //U+D40F
0, //U+D410
0, //U+D411
0, //U+D412
0, //U+D413
0, //U+D414
0, //U+D415
0, //U+D416
0, //U+D417
0, //U+D418
0, //U+D419
0, //U+D41A
0, //U+D41B
0, //U+D41C
0, //U+D41D
0, //U+D41E
0, //U+D41F
0, //U+D420
0, //U+D421
0, //U+D422
0, //U+D423
0, //U+D424
0, //U+D425
0, //U+D426
0, //U+D427
0, //U+D428
0, //U+D429
0, //U+D42A
0, //U+D42B
0, //U+D42C
0, //U+D42D
0, //U+D42E
0, //U+D42F
0, //U+D430
0, //U+D431
0, //U+D432
0, //U+D433
0, //U+D434
0, //U+D435
0, //U+D436
0, //U+D437
0, //U+D438
0, //U+D439
0, //U+D43A
0, //U+D43B
0, //U+D43C
0, //U+D43D
0, //U+D43E
0, //U+D43F
0, //U+D440
0, //U+D441
0, //U+D442
0, //U+D443
0, //U+D444
0, //U+D445
0, //U+D446
0, //U+D447
0, //U+D448
0, //U+D449
0, //U+D44A
0, //U+D44B
0, //U+D44C
0, //U+D44D
0, //U+D44E
0, //U+D44F
0, //U+D450
0, //U+D451
0, //U+D452
0, //U+D453
0, //U+D454
0, //U+D455
0, //U+D456
0, //U+D457
0, //U+D458
0, //U+D459
0, //U+D45A
0, //U+D45B
0, //U+D45C
0, //U+D45D
0, //U+D45E
0, //U+D45F
0, //U+D460
0, //U+D461
0, //U+D462
0, //U+D463
0, //U+D464
0, //U+D465
0, //U+D466
0, //U+D467
0, //U+D468
0, //U+D469
0, //U+D46A
0, //U+D46B
0, //U+D46C
0, //U+D46D
0, //U+D46E
0, //U+D46F
0, //U+D470
0, //U+D471
0, //U+D472
0, //U+D473
0, //U+D474
0, //U+D475
0, //U+D476
0, //U+D477
0, //U+D478
0, //U+D479
0, //U+D47A
0, //U+D47B
0, //U+D47C
0, //U+D47D
0, //U+D47E
0, //U+D47F
0, //U+D480
0, //U+D481
0, //U+D482
0, //U+D483
0, //U+D484
0, //U+D485
0, //U+D486
0, //U+D487
0, //U+D488
0, //U+D489
0, //U+D48A
0, //U+D48B
0, //U+D48C
0, //U+D48D
0, //U+D48E
0, //U+D48F
0, //U+D490
0, //U+D491
0, //U+D492
0, //U+D493
0, //U+D494
0, //U+D495
0, //U+D496
0, //U+D497
0, //U+D498
0, //U+D499
0, //U+D49A
0, //U+D49B
0, //U+D49C
0, //U+D49D
0, //U+D49E
0, //U+D49F
0, //U+D4A0
0, //U+D4A1
0, //U+D4A2
0, //U+D4A3
0, //U+D4A4
0, //U+D4A5
0, //U+D4A6
0, //U+D4A7
0, //U+D4A8
0, //U+D4A9
0, //U+D4AA
0, //U+D4AB
0, //U+D4AC
0, //U+D4AD
0, //U+D4AE
0, //U+D4AF
0, //U+D4B0
0, //U+D4B1
0, //U+D4B2
0, //U+D4B3
0, //U+D4B4
0, //U+D4B5
0, //U+D4B6
0, //U+D4B7
0, //U+D4B8
0, //U+D4B9
0, //U+D4BA
0, //U+D4BB
0, //U+D4BC
0, //U+D4BD
0, //U+D4BE
0, //U+D4BF
0, //U+D4C0
0, //U+D4C1
0, //U+D4C2
0, //U+D4C3
0, //U+D4C4
0, //U+D4C5
0, //U+D4C6
0, //U+D4C7
0, //U+D4C8
0, //U+D4C9
0, //U+D4CA
0, //U+D4CB
0, //U+D4CC
0, //U+D4CD
0, //U+D4CE
0, //U+D4CF
0, //U+D4D0
0, //U+D4D1
0, //U+D4D2
0, //U+D4D3
0, //U+D4D4
0, //U+D4D5
0, //U+D4D6
0, //U+D4D7
0, //U+D4D8
0, //U+D4D9
0, //U+D4DA
0, //U+D4DB
0, //U+D4DC
0, //U+D4DD
0, //U+D4DE
0, //U+D4DF
0, //U+D4E0
0, //U+D4E1
0, //U+D4E2
0, //U+D4E3
0, //U+D4E4
0, //U+D4E5
0, //U+D4E6
0, //U+D4E7
0, //U+D4E8
0, //U+D4E9
0, //U+D4EA
0, //U+D4EB
0, //U+D4EC
0, //U+D4ED
0, //U+D4EE
0, //U+D4EF
0, //U+D4F0
0, //U+D4F1
0, //U+D4F2
0, //U+D4F3
0, //U+D4F4
0, //U+D4F5
0, //U+D4F6
0, //U+D4F7
0, //U+D4F8
0, //U+D4F9
0, //U+D4FA
0, //U+D4FB
0, //U+D4FC
0, //U+D4FD
0, //U+D4FE
0, //U+D4FF
0, //U+D500
0, //U+D501
0, //U+D502
0, //U+D503
0, //U+D504
0, //U+D505
0, //U+D506
0, //U+D507
0, //U+D508
0, //U+D509
0, //U+D50A
0, //U+D50B
0, //U+D50C
0, //U+D50D
0, //U+D50E
0, //U+D50F
0, //U+D510
0, //U+D511
0, //U+D512
0, //U+D513
0, //U+D514
0, //U+D515
0, //U+D516
0, //U+D517
0, //U+D518
0, //U+D519
0, //U+D51A
0, //U+D51B
0, //U+D51C
0, //U+D51D
0, //U+D51E
0, //U+D51F
0, //U+D520
0, //U+D521
0, //U+D522
0, //U+D523
0, //U+D524
0, //U+D525
0, //U+D526
0, //U+D527
0, //U+D528
0, //U+D529
0, //U+D52A
0, //U+D52B
0, //U+D52C
0, //U+D52D
0, //U+D52E
0, //U+D52F
0, //U+D530
0, //U+D531
0, //U+D532
0, //U+D533
0, //U+D534
0, //U+D535
0, //U+D536
0, //U+D537
0, //U+D538
0, //U+D539
0, //U+D53A
0, //U+D53B
0, //U+D53C
0, //U+D53D
0, //U+D53E
0, //U+D53F
0, //U+D540
0, //U+D541
0, //U+D542
0, //U+D543
0, //U+D544
0, //U+D545
0, //U+D546
0, //U+D547
0, //U+D548
0, //U+D549
0, //U+D54A
0, //U+D54B
0, //U+D54C
0, //U+D54D
0, //U+D54E
0, //U+D54F
0, //U+D550
0, //U+D551
0, //U+D552
0, //U+D553
0, //U+D554
0, //U+D555
0, //U+D556
0, //U+D557
0, //U+D558
0, //U+D559
0, //U+D55A
0, //U+D55B
0, //U+D55C
0, //U+D55D
0, //U+D55E
0, //U+D55F
0, //U+D560
0, //U+D561
0, //U+D562
0, //U+D563
0, //U+D564
0, //U+D565
0, //U+D566
0, //U+D567
0, //U+D568
0, //U+D569
0, //U+D56A
0, //U+D56B
0, //U+D56C
0, //U+D56D
0, //U+D56E
0, //U+D56F
0, //U+D570
0, //U+D571
0, //U+D572
0, //U+D573
0, //U+D574
0, //U+D575
0, //U+D576
0, //U+D577
0, //U+D578
0, //U+D579
0, //U+D57A
0, //U+D57B
0, //U+D57C
0, //U+D57D
0, //U+D57E
0, //U+D57F
0, //U+D580
0, //U+D581
0, //U+D582
0, //U+D583
0, //U+D584
0, //U+D585
0, //U+D586
0, //U+D587
0, //U+D588
0, //U+D589
0, //U+D58A
0, //U+D58B
0, //U+D58C
0, //U+D58D
0, //U+D58E
0, //U+D58F
0, //U+D590
0, //U+D591
0, //U+D592
0, //U+D593
0, //U+D594
0, //U+D595
0, //U+D596
0, //U+D597
0, //U+D598
0, //U+D599
0, //U+D59A
0, //U+D59B
0, //U+D59C
0, //U+D59D
0, //U+D59E
0, //U+D59F
0, //U+D5A0
0, //U+D5A1
0, //U+D5A2
0, //U+D5A3
0, //U+D5A4
0, //U+D5A5
0, //U+D5A6
0, //U+D5A7
0, //U+D5A8
0, //U+D5A9
0, //U+D5AA
0, //U+D5AB
0, //U+D5AC
0, //U+D5AD
0, //U+D5AE
0, //U+D5AF
0, //U+D5B0
0, //U+D5B1
0, //U+D5B2
0, //U+D5B3
0, //U+D5B4
0, //U+D5B5
0, //U+D5B6
0, //U+D5B7
0, //U+D5B8
0, //U+D5B9
0, //U+D5BA
0, //U+D5BB
0, //U+D5BC
0, //U+D5BD
0, //U+D5BE
0, //U+D5BF
0, //U+D5C0
0, //U+D5C1
0, //U+D5C2
0, //U+D5C3
0, //U+D5C4
0, //U+D5C5
0, //U+D5C6
0, //U+D5C7
0, //U+D5C8
0, //U+D5C9
0, //U+D5CA
0, //U+D5CB
0, //U+D5CC
0, //U+D5CD
0, //U+D5CE
0, //U+D5CF
0, //U+D5D0
0, //U+D5D1
0, //U+D5D2
0, //U+D5D3
0, //U+D5D4
0, //U+D5D5
0, //U+D5D6
0, //U+D5D7
0, //U+D5D8
0, //U+D5D9
0, //U+D5DA
0, //U+D5DB
0, //U+D5DC
0, //U+D5DD
0, //U+D5DE
0, //U+D5DF
0, //U+D5E0
0, //U+D5E1
0, //U+D5E2
0, //U+D5E3
0, //U+D5E4
0, //U+D5E5
0, //U+D5E6
0, //U+D5E7
0, //U+D5E8
0, //U+D5E9
0, //U+D5EA
0, //U+D5EB
0, //U+D5EC
0, //U+D5ED
0, //U+D5EE
0, //U+D5EF
0, //U+D5F0
0, //U+D5F1
0, //U+D5F2
0, //U+D5F3
0, //U+D5F4
0, //U+D5F5
0, //U+D5F6
0, //U+D5F7
0, //U+D5F8
0, //U+D5F9
0, //U+D5FA
0, //U+D5FB
0, //U+D5FC
0, //U+D5FD
0, //U+D5FE
0, //U+D5FF
0, //U+D600
0, //U+D601
0, //U+D602
0, //U+D603
0, //U+D604
0, //U+D605
0, //U+D606
0, //U+D607
0, //U+D608
0, //U+D609
0, //U+D60A
0, //U+D60B
0, //U+D60C
0, //U+D60D
0, //U+D60E
0, //U+D60F
0, //U+D610
0, //U+D611
0, //U+D612
0, //U+D613
0, //U+D614
0, //U+D615
0, //U+D616
0, //U+D617
0, //U+D618
0, //U+D619
0, //U+D61A
0, //U+D61B
0, //U+D61C
0, //U+D61D
0, //U+D61E
0, //U+D61F
0, //U+D620
0, //U+D621
0, //U+D622
0, //U+D623
0, //U+D624
0, //U+D625
0, //U+D626
0, //U+D627
0, //U+D628
0, //U+D629
0, //U+D62A
0, //U+D62B
0, //U+D62C
0, //U+D62D
0, //U+D62E
0, //U+D62F
0, //U+D630
0, //U+D631
0, //U+D632
0, //U+D633
0, //U+D634
0, //U+D635
0, //U+D636
0, //U+D637
0, //U+D638
0, //U+D639
0, //U+D63A
0, //U+D63B
0, //U+D63C
0, //U+D63D
0, //U+D63E
0, //U+D63F
0, //U+D640
0, //U+D641
0, //U+D642
0, //U+D643
0, //U+D644
0, //U+D645
0, //U+D646
0, //U+D647
0, //U+D648
0, //U+D649
0, //U+D64A
0, //U+D64B
0, //U+D64C
0, //U+D64D
0, //U+D64E
0, //U+D64F
0, //U+D650
0, //U+D651
0, //U+D652
0, //U+D653
0, //U+D654
0, //U+D655
0, //U+D656
0, //U+D657
0, //U+D658
0, //U+D659
0, //U+D65A
0, //U+D65B
0, //U+D65C
0, //U+D65D
0, //U+D65E
0, //U+D65F
0, //U+D660
0, //U+D661
0, //U+D662
0, //U+D663
0, //U+D664
0, //U+D665
0, //U+D666
0, //U+D667
0, //U+D668
0, //U+D669
0, //U+D66A
0, //U+D66B
0, //U+D66C
0, //U+D66D
0, //U+D66E
0, //U+D66F
0, //U+D670
0, //U+D671
0, //U+D672
0, //U+D673
0, //U+D674
0, //U+D675
0, //U+D676
0, //U+D677
0, //U+D678
0, //U+D679
0, //U+D67A
0, //U+D67B
0, //U+D67C
0, //U+D67D
0, //U+D67E
0, //U+D67F
0, //U+D680
0, //U+D681
0, //U+D682
0, //U+D683
0, //U+D684
0, //U+D685
0, //U+D686
0, //U+D687
0, //U+D688
0, //U+D689
0, //U+D68A
0, //U+D68B
0, //U+D68C
0, //U+D68D
0, //U+D68E
0, //U+D68F
0, //U+D690
0, //U+D691
0, //U+D692
0, //U+D693
0, //U+D694
0, //U+D695
0, //U+D696
0, //U+D697
0, //U+D698
0, //U+D699
0, //U+D69A
0, //U+D69B
0, //U+D69C
0, //U+D69D
0, //U+D69E
0, //U+D69F
0, //U+D6A0
0, //U+D6A1
0, //U+D6A2
0, //U+D6A3
0, //U+D6A4
0, //U+D6A5
0, //U+D6A6
0, //U+D6A7
0, //U+D6A8
0, //U+D6A9
0, //U+D6AA
0, //U+D6AB
0, //U+D6AC
0, //U+D6AD
0, //U+D6AE
0, //U+D6AF
0, //U+D6B0
0, //U+D6B1
0, //U+D6B2
0, //U+D6B3
0, //U+D6B4
0, //U+D6B5
0, //U+D6B6
0, //U+D6B7
0, //U+D6B8
0, //U+D6B9
0, //U+D6BA
0, //U+D6BB
0, //U+D6BC
0, //U+D6BD
0, //U+D6BE
0, //U+D6BF
0, //U+D6C0
0, //U+D6C1
0, //U+D6C2
0, //U+D6C3
0, //U+D6C4
0, //U+D6C5
0, //U+D6C6
0, //U+D6C7
0, //U+D6C8
0, //U+D6C9
0, //U+D6CA
0, //U+D6CB
0, //U+D6CC
0, //U+D6CD
0, //U+D6CE
0, //U+D6CF
0, //U+D6D0
0, //U+D6D1
0, //U+D6D2
0, //U+D6D3
0, //U+D6D4
0, //U+D6D5
0, //U+D6D6
0, //U+D6D7
0, //U+D6D8
0, //U+D6D9
0, //U+D6DA
0, //U+D6DB
0, //U+D6DC
0, //U+D6DD
0, //U+D6DE
0, //U+D6DF
0, //U+D6E0
0, //U+D6E1
0, //U+D6E2
0, //U+D6E3
0, //U+D6E4
0, //U+D6E5
0, //U+D6E6
0, //U+D6E7
0, //U+D6E8
0, //U+D6E9
0, //U+D6EA
0, //U+D6EB
0, //U+D6EC
0, //U+D6ED
0, //U+D6EE
0, //U+D6EF
0, //U+D6F0
0, //U+D6F1
0, //U+D6F2
0, //U+D6F3
0, //U+D6F4
0, //U+D6F5
0, //U+D6F6
0, //U+D6F7
0, //U+D6F8
0, //U+D6F9
0, //U+D6FA
0, //U+D6FB
0, //U+D6FC
0, //U+D6FD
0, //U+D6FE
0, //U+D6FF
0, //U+D700
0, //U+D701
0, //U+D702
0, //U+D703
0, //U+D704
0, //U+D705
0, //U+D706
0, //U+D707
0, //U+D708
0, //U+D709
0, //U+D70A
0, //U+D70B
0, //U+D70C
0, //U+D70D
0, //U+D70E
0, //U+D70F
0, //U+D710
0, //U+D711
0, //U+D712
0, //U+D713
0, //U+D714
0, //U+D715
0, //U+D716
0, //U+D717
0, //U+D718
0, //U+D719
0, //U+D71A
0, //U+D71B
0, //U+D71C
0, //U+D71D
0, //U+D71E
0, //U+D71F
0, //U+D720
0, //U+D721
0, //U+D722
0, //U+D723
0, //U+D724
0, //U+D725
0, //U+D726
0, //U+D727
0, //U+D728
0, //U+D729
0, //U+D72A
0, //U+D72B
0, //U+D72C
0, //U+D72D
0, //U+D72E
0, //U+D72F
0, //U+D730
0, //U+D731
0, //U+D732
0, //U+D733
0, //U+D734
0, //U+D735
0, //U+D736
0, //U+D737
0, //U+D738
0, //U+D739
0, //U+D73A
0, //U+D73B
0, //U+D73C
0, //U+D73D
0, //U+D73E
0, //U+D73F
0, //U+D740
0, //U+D741
0, //U+D742
0, //U+D743
0, //U+D744
0, //U+D745
0, //U+D746
0, //U+D747
0, //U+D748
0, //U+D749
0, //U+D74A
0, //U+D74B
0, //U+D74C
0, //U+D74D
0, //U+D74E
0, //U+D74F
0, //U+D750
0, //U+D751
0, //U+D752
0, //U+D753
0, //U+D754
0, //U+D755
0, //U+D756
0, //U+D757
0, //U+D758
0, //U+D759
0, //U+D75A
0, //U+D75B
0, //U+D75C
0, //U+D75D
0, //U+D75E
0, //U+D75F
0, //U+D760
0, //U+D761
0, //U+D762
0, //U+D763
0, //U+D764
0, //U+D765
0, //U+D766
0, //U+D767
0, //U+D768
0, //U+D769
0, //U+D76A
0, //U+D76B
0, //U+D76C
0, //U+D76D
0, //U+D76E
0, //U+D76F
0, //U+D770
0, //U+D771
0, //U+D772
0, //U+D773
0, //U+D774
0, //U+D775
0, //U+D776
0, //U+D777
0, //U+D778
0, //U+D779
0, //U+D77A
0, //U+D77B
0, //U+D77C
0, //U+D77D
0, //U+D77E
0, //U+D77F
0, //U+D780
0, //U+D781
0, //U+D782
0, //U+D783
0, //U+D784
0, //U+D785
0, //U+D786
0, //U+D787
0, //U+D788
0, //U+D789
0, //U+D78A
0, //U+D78B
0, //U+D78C
0, //U+D78D
0, //U+D78E
0, //U+D78F
0, //U+D790
0, //U+D791
0, //U+D792
0, //U+D793
0, //U+D794
0, //U+D795
0, //U+D796
0, //U+D797
0, //U+D798
0, //U+D799
0, //U+D79A
0, //U+D79B
0, //U+D79C
0, //U+D79D
0, //U+D79E
0, //U+D79F
0, //U+D7A0
0, //U+D7A1
0, //U+D7A2
UNICODE_Lo, //U+D7A3
0, //U+D7A4
0, //U+D7A5
0, //U+D7A6
0, //U+D7A7
0, //U+D7A8
0, //U+D7A9
0, //U+D7AA
0, //U+D7AB
0, //U+D7AC
0, //U+D7AD
0, //U+D7AE
0, //U+D7AF
UNICODE_Lo, //U+D7B0
UNICODE_Lo, //U+D7B1
UNICODE_Lo, //U+D7B2
UNICODE_Lo, //U+D7B3
UNICODE_Lo, //U+D7B4
UNICODE_Lo, //U+D7B5
UNICODE_Lo, //U+D7B6
UNICODE_Lo, //U+D7B7
UNICODE_Lo, //U+D7B8
UNICODE_Lo, //U+D7B9
UNICODE_Lo, //U+D7BA
UNICODE_Lo, //U+D7BB
UNICODE_Lo, //U+D7BC
UNICODE_Lo, //U+D7BD
UNICODE_Lo, //U+D7BE
UNICODE_Lo, //U+D7BF
UNICODE_Lo, //U+D7C0
UNICODE_Lo, //U+D7C1
UNICODE_Lo, //U+D7C2
UNICODE_Lo, //U+D7C3
UNICODE_Lo, //U+D7C4
UNICODE_Lo, //U+D7C5
UNICODE_Lo, //U+D7C6
0, //U+D7C7
0, //U+D7C8
0, //U+D7C9
0, //U+D7CA
UNICODE_Lo, //U+D7CB
UNICODE_Lo, //U+D7CC
UNICODE_Lo, //U+D7CD
UNICODE_Lo, //U+D7CE
UNICODE_Lo, //U+D7CF
UNICODE_Lo, //U+D7D0
UNICODE_Lo, //U+D7D1
UNICODE_Lo, //U+D7D2
UNICODE_Lo, //U+D7D3
UNICODE_Lo, //U+D7D4
UNICODE_Lo, //U+D7D5
UNICODE_Lo, //U+D7D6
UNICODE_Lo, //U+D7D7
UNICODE_Lo, //U+D7D8
UNICODE_Lo, //U+D7D9
UNICODE_Lo, //U+D7DA
UNICODE_Lo, //U+D7DB
UNICODE_Lo, //U+D7DC
UNICODE_Lo, //U+D7DD
UNICODE_Lo, //U+D7DE
UNICODE_Lo, //U+D7DF
UNICODE_Lo, //U+D7E0
UNICODE_Lo, //U+D7E1
UNICODE_Lo, //U+D7E2
UNICODE_Lo, //U+D7E3
UNICODE_Lo, //U+D7E4
UNICODE_Lo, //U+D7E5
UNICODE_Lo, //U+D7E6
UNICODE_Lo, //U+D7E7
UNICODE_Lo, //U+D7E8
UNICODE_Lo, //U+D7E9
UNICODE_Lo, //U+D7EA
UNICODE_Lo, //U+D7EB
UNICODE_Lo, //U+D7EC
UNICODE_Lo, //U+D7ED
UNICODE_Lo, //U+D7EE
UNICODE_Lo, //U+D7EF
UNICODE_Lo, //U+D7F0
UNICODE_Lo, //U+D7F1
UNICODE_Lo, //U+D7F2
UNICODE_Lo, //U+D7F3
UNICODE_Lo, //U+D7F4
UNICODE_Lo, //U+D7F5
UNICODE_Lo, //U+D7F6
UNICODE_Lo, //U+D7F7
UNICODE_Lo, //U+D7F8
UNICODE_Lo, //U+D7F9
UNICODE_Lo, //U+D7FA
UNICODE_Lo, //U+D7FB
0, //U+D7FC
0, //U+D7FD
0, //U+D7FE
0, //U+D7FF
UNICODE_Cs, //U+D800
0, //U+D801
0, //U+D802
0, //U+D803
0, //U+D804
0, //U+D805
0, //U+D806
0, //U+D807
0, //U+D808
0, //U+D809
0, //U+D80A
0, //U+D80B
0, //U+D80C
0, //U+D80D
0, //U+D80E
0, //U+D80F
0, //U+D810
0, //U+D811
0, //U+D812
0, //U+D813
0, //U+D814
0, //U+D815
0, //U+D816
0, //U+D817
0, //U+D818
0, //U+D819
0, //U+D81A
0, //U+D81B
0, //U+D81C
0, //U+D81D
0, //U+D81E
0, //U+D81F
0, //U+D820
0, //U+D821
0, //U+D822
0, //U+D823
0, //U+D824
0, //U+D825
0, //U+D826
0, //U+D827
0, //U+D828
0, //U+D829
0, //U+D82A
0, //U+D82B
0, //U+D82C
0, //U+D82D
0, //U+D82E
0, //U+D82F
0, //U+D830
0, //U+D831
0, //U+D832
0, //U+D833
0, //U+D834
0, //U+D835
0, //U+D836
0, //U+D837
0, //U+D838
0, //U+D839
0, //U+D83A
0, //U+D83B
0, //U+D83C
0, //U+D83D
0, //U+D83E
0, //U+D83F
0, //U+D840
0, //U+D841
0, //U+D842
0, //U+D843
0, //U+D844
0, //U+D845
0, //U+D846
0, //U+D847
0, //U+D848
0, //U+D849
0, //U+D84A
0, //U+D84B
0, //U+D84C
0, //U+D84D
0, //U+D84E
0, //U+D84F
0, //U+D850
0, //U+D851
0, //U+D852
0, //U+D853
0, //U+D854
0, //U+D855
0, //U+D856
0, //U+D857
0, //U+D858
0, //U+D859
0, //U+D85A
0, //U+D85B
0, //U+D85C
0, //U+D85D
0, //U+D85E
0, //U+D85F
0, //U+D860
0, //U+D861
0, //U+D862
0, //U+D863
0, //U+D864
0, //U+D865
0, //U+D866
0, //U+D867
0, //U+D868
0, //U+D869
0, //U+D86A
0, //U+D86B
0, //U+D86C
0, //U+D86D
0, //U+D86E
0, //U+D86F
0, //U+D870
0, //U+D871
0, //U+D872
0, //U+D873
0, //U+D874
0, //U+D875
0, //U+D876
0, //U+D877
0, //U+D878
0, //U+D879
0, //U+D87A
0, //U+D87B
0, //U+D87C
0, //U+D87D
0, //U+D87E
0, //U+D87F
0, //U+D880
0, //U+D881
0, //U+D882
0, //U+D883
0, //U+D884
0, //U+D885
0, //U+D886
0, //U+D887
0, //U+D888
0, //U+D889
0, //U+D88A
0, //U+D88B
0, //U+D88C
0, //U+D88D
0, //U+D88E
0, //U+D88F
0, //U+D890
0, //U+D891
0, //U+D892
0, //U+D893
0, //U+D894
0, //U+D895
0, //U+D896
0, //U+D897
0, //U+D898
0, //U+D899
0, //U+D89A
0, //U+D89B
0, //U+D89C
0, //U+D89D
0, //U+D89E
0, //U+D89F
0, //U+D8A0
0, //U+D8A1
0, //U+D8A2
0, //U+D8A3
0, //U+D8A4
0, //U+D8A5
0, //U+D8A6
0, //U+D8A7
0, //U+D8A8
0, //U+D8A9
0, //U+D8AA
0, //U+D8AB
0, //U+D8AC
0, //U+D8AD
0, //U+D8AE
0, //U+D8AF
0, //U+D8B0
0, //U+D8B1
0, //U+D8B2
0, //U+D8B3
0, //U+D8B4
0, //U+D8B5
0, //U+D8B6
0, //U+D8B7
0, //U+D8B8
0, //U+D8B9
0, //U+D8BA
0, //U+D8BB
0, //U+D8BC
0, //U+D8BD
0, //U+D8BE
0, //U+D8BF
0, //U+D8C0
0, //U+D8C1
0, //U+D8C2
0, //U+D8C3
0, //U+D8C4
0, //U+D8C5
0, //U+D8C6
0, //U+D8C7
0, //U+D8C8
0, //U+D8C9
0, //U+D8CA
0, //U+D8CB
0, //U+D8CC
0, //U+D8CD
0, //U+D8CE
0, //U+D8CF
0, //U+D8D0
0, //U+D8D1
0, //U+D8D2
0, //U+D8D3
0, //U+D8D4
0, //U+D8D5
0, //U+D8D6
0, //U+D8D7
0, //U+D8D8
0, //U+D8D9
0, //U+D8DA
0, //U+D8DB
0, //U+D8DC
0, //U+D8DD
0, //U+D8DE
0, //U+D8DF
0, //U+D8E0
0, //U+D8E1
0, //U+D8E2
0, //U+D8E3
0, //U+D8E4
0, //U+D8E5
0, //U+D8E6
0, //U+D8E7
0, //U+D8E8
0, //U+D8E9
0, //U+D8EA
0, //U+D8EB
0, //U+D8EC
0, //U+D8ED
0, //U+D8EE
0, //U+D8EF
0, //U+D8F0
0, //U+D8F1
0, //U+D8F2
0, //U+D8F3
0, //U+D8F4
0, //U+D8F5
0, //U+D8F6
0, //U+D8F7
0, //U+D8F8
0, //U+D8F9
0, //U+D8FA
0, //U+D8FB
0, //U+D8FC
0, //U+D8FD
0, //U+D8FE
0, //U+D8FF
0, //U+D900
0, //U+D901
0, //U+D902
0, //U+D903
0, //U+D904
0, //U+D905
0, //U+D906
0, //U+D907
0, //U+D908
0, //U+D909
0, //U+D90A
0, //U+D90B
0, //U+D90C
0, //U+D90D
0, //U+D90E
0, //U+D90F
0, //U+D910
0, //U+D911
0, //U+D912
0, //U+D913
0, //U+D914
0, //U+D915
0, //U+D916
0, //U+D917
0, //U+D918
0, //U+D919
0, //U+D91A
0, //U+D91B
0, //U+D91C
0, //U+D91D
0, //U+D91E
0, //U+D91F
0, //U+D920
0, //U+D921
0, //U+D922
0, //U+D923
0, //U+D924
0, //U+D925
0, //U+D926
0, //U+D927
0, //U+D928
0, //U+D929
0, //U+D92A
0, //U+D92B
0, //U+D92C
0, //U+D92D
0, //U+D92E
0, //U+D92F
0, //U+D930
0, //U+D931
0, //U+D932
0, //U+D933
0, //U+D934
0, //U+D935
0, //U+D936
0, //U+D937
0, //U+D938
0, //U+D939
0, //U+D93A
0, //U+D93B
0, //U+D93C
0, //U+D93D
0, //U+D93E
0, //U+D93F
0, //U+D940
0, //U+D941
0, //U+D942
0, //U+D943
0, //U+D944
0, //U+D945
0, //U+D946
0, //U+D947
0, //U+D948
0, //U+D949
0, //U+D94A
0, //U+D94B
0, //U+D94C
0, //U+D94D
0, //U+D94E
0, //U+D94F
0, //U+D950
0, //U+D951
0, //U+D952
0, //U+D953
0, //U+D954
0, //U+D955
0, //U+D956
0, //U+D957
0, //U+D958
0, //U+D959
0, //U+D95A
0, //U+D95B
0, //U+D95C
0, //U+D95D
0, //U+D95E
0, //U+D95F
0, //U+D960
0, //U+D961
0, //U+D962
0, //U+D963
0, //U+D964
0, //U+D965
0, //U+D966
0, //U+D967
0, //U+D968
0, //U+D969
0, //U+D96A
0, //U+D96B
0, //U+D96C
0, //U+D96D
0, //U+D96E
0, //U+D96F
0, //U+D970
0, //U+D971
0, //U+D972
0, //U+D973
0, //U+D974
0, //U+D975
0, //U+D976
0, //U+D977
0, //U+D978
0, //U+D979
0, //U+D97A
0, //U+D97B
0, //U+D97C
0, //U+D97D
0, //U+D97E
0, //U+D97F
0, //U+D980
0, //U+D981
0, //U+D982
0, //U+D983
0, //U+D984
0, //U+D985
0, //U+D986
0, //U+D987
0, //U+D988
0, //U+D989
0, //U+D98A
0, //U+D98B
0, //U+D98C
0, //U+D98D
0, //U+D98E
0, //U+D98F
0, //U+D990
0, //U+D991
0, //U+D992
0, //U+D993
0, //U+D994
0, //U+D995
0, //U+D996
0, //U+D997
0, //U+D998
0, //U+D999
0, //U+D99A
0, //U+D99B
0, //U+D99C
0, //U+D99D
0, //U+D99E
0, //U+D99F
0, //U+D9A0
0, //U+D9A1
0, //U+D9A2
0, //U+D9A3
0, //U+D9A4
0, //U+D9A5
0, //U+D9A6
0, //U+D9A7
0, //U+D9A8
0, //U+D9A9
0, //U+D9AA
0, //U+D9AB
0, //U+D9AC
0, //U+D9AD
0, //U+D9AE
0, //U+D9AF
0, //U+D9B0
0, //U+D9B1
0, //U+D9B2
0, //U+D9B3
0, //U+D9B4
0, //U+D9B5
0, //U+D9B6
0, //U+D9B7
0, //U+D9B8
0, //U+D9B9
0, //U+D9BA
0, //U+D9BB
0, //U+D9BC
0, //U+D9BD
0, //U+D9BE
0, //U+D9BF
0, //U+D9C0
0, //U+D9C1
0, //U+D9C2
0, //U+D9C3
0, //U+D9C4
0, //U+D9C5
0, //U+D9C6
0, //U+D9C7
0, //U+D9C8
0, //U+D9C9
0, //U+D9CA
0, //U+D9CB
0, //U+D9CC
0, //U+D9CD
0, //U+D9CE
0, //U+D9CF
0, //U+D9D0
0, //U+D9D1
0, //U+D9D2
0, //U+D9D3
0, //U+D9D4
0, //U+D9D5
0, //U+D9D6
0, //U+D9D7
0, //U+D9D8
0, //U+D9D9
0, //U+D9DA
0, //U+D9DB
0, //U+D9DC
0, //U+D9DD
0, //U+D9DE
0, //U+D9DF
0, //U+D9E0
0, //U+D9E1
0, //U+D9E2
0, //U+D9E3
0, //U+D9E4
0, //U+D9E5
0, //U+D9E6
0, //U+D9E7
0, //U+D9E8
0, //U+D9E9
0, //U+D9EA
0, //U+D9EB
0, //U+D9EC
0, //U+D9ED
0, //U+D9EE
0, //U+D9EF
0, //U+D9F0
0, //U+D9F1
0, //U+D9F2
0, //U+D9F3
0, //U+D9F4
0, //U+D9F5
0, //U+D9F6
0, //U+D9F7
0, //U+D9F8
0, //U+D9F9
0, //U+D9FA
0, //U+D9FB
0, //U+D9FC
0, //U+D9FD
0, //U+D9FE
0, //U+D9FF
0, //U+DA00
0, //U+DA01
0, //U+DA02
0, //U+DA03
0, //U+DA04
0, //U+DA05
0, //U+DA06
0, //U+DA07
0, //U+DA08
0, //U+DA09
0, //U+DA0A
0, //U+DA0B
0, //U+DA0C
0, //U+DA0D
0, //U+DA0E
0, //U+DA0F
0, //U+DA10
0, //U+DA11
0, //U+DA12
0, //U+DA13
0, //U+DA14
0, //U+DA15
0, //U+DA16
0, //U+DA17
0, //U+DA18
0, //U+DA19
0, //U+DA1A
0, //U+DA1B
0, //U+DA1C
0, //U+DA1D
0, //U+DA1E
0, //U+DA1F
0, //U+DA20
0, //U+DA21
0, //U+DA22
0, //U+DA23
0, //U+DA24
0, //U+DA25
0, //U+DA26
0, //U+DA27
0, //U+DA28
0, //U+DA29
0, //U+DA2A
0, //U+DA2B
0, //U+DA2C
0, //U+DA2D
0, //U+DA2E
0, //U+DA2F
0, //U+DA30
0, //U+DA31
0, //U+DA32
0, //U+DA33
0, //U+DA34
0, //U+DA35
0, //U+DA36
0, //U+DA37
0, //U+DA38
0, //U+DA39
0, //U+DA3A
0, //U+DA3B
0, //U+DA3C
0, //U+DA3D
0, //U+DA3E
0, //U+DA3F
0, //U+DA40
0, //U+DA41
0, //U+DA42
0, //U+DA43
0, //U+DA44
0, //U+DA45
0, //U+DA46
0, //U+DA47
0, //U+DA48
0, //U+DA49
0, //U+DA4A
0, //U+DA4B
0, //U+DA4C
0, //U+DA4D
0, //U+DA4E
0, //U+DA4F
0, //U+DA50
0, //U+DA51
0, //U+DA52
0, //U+DA53
0, //U+DA54
0, //U+DA55
0, //U+DA56
0, //U+DA57
0, //U+DA58
0, //U+DA59
0, //U+DA5A
0, //U+DA5B
0, //U+DA5C
0, //U+DA5D
0, //U+DA5E
0, //U+DA5F
0, //U+DA60
0, //U+DA61
0, //U+DA62
0, //U+DA63
0, //U+DA64
0, //U+DA65
0, //U+DA66
0, //U+DA67
0, //U+DA68
0, //U+DA69
0, //U+DA6A
0, //U+DA6B
0, //U+DA6C
0, //U+DA6D
0, //U+DA6E
0, //U+DA6F
0, //U+DA70
0, //U+DA71
0, //U+DA72
0, //U+DA73
0, //U+DA74
0, //U+DA75
0, //U+DA76
0, //U+DA77
0, //U+DA78
0, //U+DA79
0, //U+DA7A
0, //U+DA7B
0, //U+DA7C
0, //U+DA7D
0, //U+DA7E
0, //U+DA7F
0, //U+DA80
0, //U+DA81
0, //U+DA82
0, //U+DA83
0, //U+DA84
0, //U+DA85
0, //U+DA86
0, //U+DA87
0, //U+DA88
0, //U+DA89
0, //U+DA8A
0, //U+DA8B
0, //U+DA8C
0, //U+DA8D
0, //U+DA8E
0, //U+DA8F
0, //U+DA90
0, //U+DA91
0, //U+DA92
0, //U+DA93
0, //U+DA94
0, //U+DA95
0, //U+DA96
0, //U+DA97
0, //U+DA98
0, //U+DA99
0, //U+DA9A
0, //U+DA9B
0, //U+DA9C
0, //U+DA9D
0, //U+DA9E
0, //U+DA9F
0, //U+DAA0
0, //U+DAA1
0, //U+DAA2
0, //U+DAA3
0, //U+DAA4
0, //U+DAA5
0, //U+DAA6
0, //U+DAA7
0, //U+DAA8
0, //U+DAA9
0, //U+DAAA
0, //U+DAAB
0, //U+DAAC
0, //U+DAAD
0, //U+DAAE
0, //U+DAAF
0, //U+DAB0
0, //U+DAB1
0, //U+DAB2
0, //U+DAB3
0, //U+DAB4
0, //U+DAB5
0, //U+DAB6
0, //U+DAB7
0, //U+DAB8
0, //U+DAB9
0, //U+DABA
0, //U+DABB
0, //U+DABC
0, //U+DABD
0, //U+DABE
0, //U+DABF
0, //U+DAC0
0, //U+DAC1
0, //U+DAC2
0, //U+DAC3
0, //U+DAC4
0, //U+DAC5
0, //U+DAC6
0, //U+DAC7
0, //U+DAC8
0, //U+DAC9
0, //U+DACA
0, //U+DACB
0, //U+DACC
0, //U+DACD
0, //U+DACE
0, //U+DACF
0, //U+DAD0
0, //U+DAD1
0, //U+DAD2
0, //U+DAD3
0, //U+DAD4
0, //U+DAD5
0, //U+DAD6
0, //U+DAD7
0, //U+DAD8
0, //U+DAD9
0, //U+DADA
0, //U+DADB
0, //U+DADC
0, //U+DADD
0, //U+DADE
0, //U+DADF
0, //U+DAE0
0, //U+DAE1
0, //U+DAE2
0, //U+DAE3
0, //U+DAE4
0, //U+DAE5
0, //U+DAE6
0, //U+DAE7
0, //U+DAE8
0, //U+DAE9
0, //U+DAEA
0, //U+DAEB
0, //U+DAEC
0, //U+DAED
0, //U+DAEE
0, //U+DAEF
0, //U+DAF0
0, //U+DAF1
0, //U+DAF2
0, //U+DAF3
0, //U+DAF4
0, //U+DAF5
0, //U+DAF6
0, //U+DAF7
0, //U+DAF8
0, //U+DAF9
0, //U+DAFA
0, //U+DAFB
0, //U+DAFC
0, //U+DAFD
0, //U+DAFE
0, //U+DAFF
0, //U+DB00
0, //U+DB01
0, //U+DB02
0, //U+DB03
0, //U+DB04
0, //U+DB05
0, //U+DB06
0, //U+DB07
0, //U+DB08
0, //U+DB09
0, //U+DB0A
0, //U+DB0B
0, //U+DB0C
0, //U+DB0D
0, //U+DB0E
0, //U+DB0F
0, //U+DB10
0, //U+DB11
0, //U+DB12
0, //U+DB13
0, //U+DB14
0, //U+DB15
0, //U+DB16
0, //U+DB17
0, //U+DB18
0, //U+DB19
0, //U+DB1A
0, //U+DB1B
0, //U+DB1C
0, //U+DB1D
0, //U+DB1E
0, //U+DB1F
0, //U+DB20
0, //U+DB21
0, //U+DB22
0, //U+DB23
0, //U+DB24
0, //U+DB25
0, //U+DB26
0, //U+DB27
0, //U+DB28
0, //U+DB29
0, //U+DB2A
0, //U+DB2B
0, //U+DB2C
0, //U+DB2D
0, //U+DB2E
0, //U+DB2F
0, //U+DB30
0, //U+DB31
0, //U+DB32
0, //U+DB33
0, //U+DB34
0, //U+DB35
0, //U+DB36
0, //U+DB37
0, //U+DB38
0, //U+DB39
0, //U+DB3A
0, //U+DB3B
0, //U+DB3C
0, //U+DB3D
0, //U+DB3E
0, //U+DB3F
0, //U+DB40
0, //U+DB41
0, //U+DB42
0, //U+DB43
0, //U+DB44
0, //U+DB45
0, //U+DB46
0, //U+DB47
0, //U+DB48
0, //U+DB49
0, //U+DB4A
0, //U+DB4B
0, //U+DB4C
0, //U+DB4D
0, //U+DB4E
0, //U+DB4F
0, //U+DB50
0, //U+DB51
0, //U+DB52
0, //U+DB53
0, //U+DB54
0, //U+DB55
0, //U+DB56
0, //U+DB57
0, //U+DB58
0, //U+DB59
0, //U+DB5A
0, //U+DB5B
0, //U+DB5C
0, //U+DB5D
0, //U+DB5E
0, //U+DB5F
0, //U+DB60
0, //U+DB61
0, //U+DB62
0, //U+DB63
0, //U+DB64
0, //U+DB65
0, //U+DB66
0, //U+DB67
0, //U+DB68
0, //U+DB69
0, //U+DB6A
0, //U+DB6B
0, //U+DB6C
0, //U+DB6D
0, //U+DB6E
0, //U+DB6F
0, //U+DB70
0, //U+DB71
0, //U+DB72
0, //U+DB73
0, //U+DB74
0, //U+DB75
0, //U+DB76
0, //U+DB77
0, //U+DB78
0, //U+DB79
0, //U+DB7A
0, //U+DB7B
0, //U+DB7C
0, //U+DB7D
0, //U+DB7E
UNICODE_Cs, //U+DB7F
UNICODE_Cs, //U+DB80
0, //U+DB81
0, //U+DB82
0, //U+DB83
0, //U+DB84
0, //U+DB85
0, //U+DB86
0, //U+DB87
0, //U+DB88
0, //U+DB89
0, //U+DB8A
0, //U+DB8B
0, //U+DB8C
0, //U+DB8D
0, //U+DB8E
0, //U+DB8F
0, //U+DB90
0, //U+DB91
0, //U+DB92
0, //U+DB93
0, //U+DB94
0, //U+DB95
0, //U+DB96
0, //U+DB97
0, //U+DB98
0, //U+DB99
0, //U+DB9A
0, //U+DB9B
0, //U+DB9C
0, //U+DB9D
0, //U+DB9E
0, //U+DB9F
0, //U+DBA0
0, //U+DBA1
0, //U+DBA2
0, //U+DBA3
0, //U+DBA4
0, //U+DBA5
0, //U+DBA6
0, //U+DBA7
0, //U+DBA8
0, //U+DBA9
0, //U+DBAA
0, //U+DBAB
0, //U+DBAC
0, //U+DBAD
0, //U+DBAE
0, //U+DBAF
0, //U+DBB0
0, //U+DBB1
0, //U+DBB2
0, //U+DBB3
0, //U+DBB4
0, //U+DBB5
0, //U+DBB6
0, //U+DBB7
0, //U+DBB8
0, //U+DBB9
0, //U+DBBA
0, //U+DBBB
0, //U+DBBC
0, //U+DBBD
0, //U+DBBE
0, //U+DBBF
0, //U+DBC0
0, //U+DBC1
0, //U+DBC2
0, //U+DBC3
0, //U+DBC4
0, //U+DBC5
0, //U+DBC6
0, //U+DBC7
0, //U+DBC8
0, //U+DBC9
0, //U+DBCA
0, //U+DBCB
0, //U+DBCC
0, //U+DBCD
0, //U+DBCE
0, //U+DBCF
0, //U+DBD0
0, //U+DBD1
0, //U+DBD2
0, //U+DBD3
0, //U+DBD4
0, //U+DBD5
0, //U+DBD6
0, //U+DBD7
0, //U+DBD8
0, //U+DBD9
0, //U+DBDA
0, //U+DBDB
0, //U+DBDC
0, //U+DBDD
0, //U+DBDE
0, //U+DBDF
0, //U+DBE0
0, //U+DBE1
0, //U+DBE2
0, //U+DBE3
0, //U+DBE4
0, //U+DBE5
0, //U+DBE6
0, //U+DBE7
0, //U+DBE8
0, //U+DBE9
0, //U+DBEA
0, //U+DBEB
0, //U+DBEC
0, //U+DBED
0, //U+DBEE
0, //U+DBEF
0, //U+DBF0
0, //U+DBF1
0, //U+DBF2
0, //U+DBF3
0, //U+DBF4
0, //U+DBF5
0, //U+DBF6
0, //U+DBF7
0, //U+DBF8
0, //U+DBF9
0, //U+DBFA
0, //U+DBFB
0, //U+DBFC
0, //U+DBFD
0, //U+DBFE
UNICODE_Cs, //U+DBFF
UNICODE_Cs, //U+DC00
0, //U+DC01
0, //U+DC02
0, //U+DC03
0, //U+DC04
0, //U+DC05
0, //U+DC06
0, //U+DC07
0, //U+DC08
0, //U+DC09
0, //U+DC0A
0, //U+DC0B
0, //U+DC0C
0, //U+DC0D
0, //U+DC0E
0, //U+DC0F
0, //U+DC10
0, //U+DC11
0, //U+DC12
0, //U+DC13
0, //U+DC14
0, //U+DC15
0, //U+DC16
0, //U+DC17
0, //U+DC18
0, //U+DC19
0, //U+DC1A
0, //U+DC1B
0, //U+DC1C
0, //U+DC1D
0, //U+DC1E
0, //U+DC1F
0, //U+DC20
0, //U+DC21
0, //U+DC22
0, //U+DC23
0, //U+DC24
0, //U+DC25
0, //U+DC26
0, //U+DC27
0, //U+DC28
0, //U+DC29
0, //U+DC2A
0, //U+DC2B
0, //U+DC2C
0, //U+DC2D
0, //U+DC2E
0, //U+DC2F
0, //U+DC30
0, //U+DC31
0, //U+DC32
0, //U+DC33
0, //U+DC34
0, //U+DC35
0, //U+DC36
0, //U+DC37
0, //U+DC38
0, //U+DC39
0, //U+DC3A
0, //U+DC3B
0, //U+DC3C
0, //U+DC3D
0, //U+DC3E
0, //U+DC3F
0, //U+DC40
0, //U+DC41
0, //U+DC42
0, //U+DC43
0, //U+DC44
0, //U+DC45
0, //U+DC46
0, //U+DC47
0, //U+DC48
0, //U+DC49
0, //U+DC4A
0, //U+DC4B
0, //U+DC4C
0, //U+DC4D
0, //U+DC4E
0, //U+DC4F
0, //U+DC50
0, //U+DC51
0, //U+DC52
0, //U+DC53
0, //U+DC54
0, //U+DC55
0, //U+DC56
0, //U+DC57
0, //U+DC58
0, //U+DC59
0, //U+DC5A
0, //U+DC5B
0, //U+DC5C
0, //U+DC5D
0, //U+DC5E
0, //U+DC5F
0, //U+DC60
0, //U+DC61
0, //U+DC62
0, //U+DC63
0, //U+DC64
0, //U+DC65
0, //U+DC66
0, //U+DC67
0, //U+DC68
0, //U+DC69
0, //U+DC6A
0, //U+DC6B
0, //U+DC6C
0, //U+DC6D
0, //U+DC6E
0, //U+DC6F
0, //U+DC70
0, //U+DC71
0, //U+DC72
0, //U+DC73
0, //U+DC74
0, //U+DC75
0, //U+DC76
0, //U+DC77
0, //U+DC78
0, //U+DC79
0, //U+DC7A
0, //U+DC7B
0, //U+DC7C
0, //U+DC7D
0, //U+DC7E
0, //U+DC7F
0, //U+DC80
0, //U+DC81
0, //U+DC82
0, //U+DC83
0, //U+DC84
0, //U+DC85
0, //U+DC86
0, //U+DC87
0, //U+DC88
0, //U+DC89
0, //U+DC8A
0, //U+DC8B
0, //U+DC8C
0, //U+DC8D
0, //U+DC8E
0, //U+DC8F
0, //U+DC90
0, //U+DC91
0, //U+DC92
0, //U+DC93
0, //U+DC94
0, //U+DC95
0, //U+DC96
0, //U+DC97
0, //U+DC98
0, //U+DC99
0, //U+DC9A
0, //U+DC9B
0, //U+DC9C
0, //U+DC9D
0, //U+DC9E
0, //U+DC9F
0, //U+DCA0
0, //U+DCA1
0, //U+DCA2
0, //U+DCA3
0, //U+DCA4
0, //U+DCA5
0, //U+DCA6
0, //U+DCA7
0, //U+DCA8
0, //U+DCA9
0, //U+DCAA
0, //U+DCAB
0, //U+DCAC
0, //U+DCAD
0, //U+DCAE
0, //U+DCAF
0, //U+DCB0
0, //U+DCB1
0, //U+DCB2
0, //U+DCB3
0, //U+DCB4
0, //U+DCB5
0, //U+DCB6
0, //U+DCB7
0, //U+DCB8
0, //U+DCB9
0, //U+DCBA
0, //U+DCBB
0, //U+DCBC
0, //U+DCBD
0, //U+DCBE
0, //U+DCBF
0, //U+DCC0
0, //U+DCC1
0, //U+DCC2
0, //U+DCC3
0, //U+DCC4
0, //U+DCC5
0, //U+DCC6
0, //U+DCC7
0, //U+DCC8
0, //U+DCC9
0, //U+DCCA
0, //U+DCCB
0, //U+DCCC
0, //U+DCCD
0, //U+DCCE
0, //U+DCCF
0, //U+DCD0
0, //U+DCD1
0, //U+DCD2
0, //U+DCD3
0, //U+DCD4
0, //U+DCD5
0, //U+DCD6
0, //U+DCD7
0, //U+DCD8
0, //U+DCD9
0, //U+DCDA
0, //U+DCDB
0, //U+DCDC
0, //U+DCDD
0, //U+DCDE
0, //U+DCDF
0, //U+DCE0
0, //U+DCE1
0, //U+DCE2
0, //U+DCE3
0, //U+DCE4
0, //U+DCE5
0, //U+DCE6
0, //U+DCE7
0, //U+DCE8
0, //U+DCE9
0, //U+DCEA
0, //U+DCEB
0, //U+DCEC
0, //U+DCED
0, //U+DCEE
0, //U+DCEF
0, //U+DCF0
0, //U+DCF1
0, //U+DCF2
0, //U+DCF3
0, //U+DCF4
0, //U+DCF5
0, //U+DCF6
0, //U+DCF7
0, //U+DCF8
0, //U+DCF9
0, //U+DCFA
0, //U+DCFB
0, //U+DCFC
0, //U+DCFD
0, //U+DCFE
0, //U+DCFF
0, //U+DD00
0, //U+DD01
0, //U+DD02
0, //U+DD03
0, //U+DD04
0, //U+DD05
0, //U+DD06
0, //U+DD07
0, //U+DD08
0, //U+DD09
0, //U+DD0A
0, //U+DD0B
0, //U+DD0C
0, //U+DD0D
0, //U+DD0E
0, //U+DD0F
0, //U+DD10
0, //U+DD11
0, //U+DD12
0, //U+DD13
0, //U+DD14
0, //U+DD15
0, //U+DD16
0, //U+DD17
0, //U+DD18
0, //U+DD19
0, //U+DD1A
0, //U+DD1B
0, //U+DD1C
0, //U+DD1D
0, //U+DD1E
0, //U+DD1F
0, //U+DD20
0, //U+DD21
0, //U+DD22
0, //U+DD23
0, //U+DD24
0, //U+DD25
0, //U+DD26
0, //U+DD27
0, //U+DD28
0, //U+DD29
0, //U+DD2A
0, //U+DD2B
0, //U+DD2C
0, //U+DD2D
0, //U+DD2E
0, //U+DD2F
0, //U+DD30
0, //U+DD31
0, //U+DD32
0, //U+DD33
0, //U+DD34
0, //U+DD35
0, //U+DD36
0, //U+DD37
0, //U+DD38
0, //U+DD39
0, //U+DD3A
0, //U+DD3B
0, //U+DD3C
0, //U+DD3D
0, //U+DD3E
0, //U+DD3F
0, //U+DD40
0, //U+DD41
0, //U+DD42
0, //U+DD43
0, //U+DD44
0, //U+DD45
0, //U+DD46
0, //U+DD47
0, //U+DD48
0, //U+DD49
0, //U+DD4A
0, //U+DD4B
0, //U+DD4C
0, //U+DD4D
0, //U+DD4E
0, //U+DD4F
0, //U+DD50
0, //U+DD51
0, //U+DD52
0, //U+DD53
0, //U+DD54
0, //U+DD55
0, //U+DD56
0, //U+DD57
0, //U+DD58
0, //U+DD59
0, //U+DD5A
0, //U+DD5B
0, //U+DD5C
0, //U+DD5D
0, //U+DD5E
0, //U+DD5F
0, //U+DD60
0, //U+DD61
0, //U+DD62
0, //U+DD63
0, //U+DD64
0, //U+DD65
0, //U+DD66
0, //U+DD67
0, //U+DD68
0, //U+DD69
0, //U+DD6A
0, //U+DD6B
0, //U+DD6C
0, //U+DD6D
0, //U+DD6E
0, //U+DD6F
0, //U+DD70
0, //U+DD71
0, //U+DD72
0, //U+DD73
0, //U+DD74
0, //U+DD75
0, //U+DD76
0, //U+DD77
0, //U+DD78
0, //U+DD79
0, //U+DD7A
0, //U+DD7B
0, //U+DD7C
0, //U+DD7D
0, //U+DD7E
0, //U+DD7F
0, //U+DD80
0, //U+DD81
0, //U+DD82
0, //U+DD83
0, //U+DD84
0, //U+DD85
0, //U+DD86
0, //U+DD87
0, //U+DD88
0, //U+DD89
0, //U+DD8A
0, //U+DD8B
0, //U+DD8C
0, //U+DD8D
0, //U+DD8E
0, //U+DD8F
0, //U+DD90
0, //U+DD91
0, //U+DD92
0, //U+DD93
0, //U+DD94
0, //U+DD95
0, //U+DD96
0, //U+DD97
0, //U+DD98
0, //U+DD99
0, //U+DD9A
0, //U+DD9B
0, //U+DD9C
0, //U+DD9D
0, //U+DD9E
0, //U+DD9F
0, //U+DDA0
0, //U+DDA1
0, //U+DDA2
0, //U+DDA3
0, //U+DDA4
0, //U+DDA5
0, //U+DDA6
0, //U+DDA7
0, //U+DDA8
0, //U+DDA9
0, //U+DDAA
0, //U+DDAB
0, //U+DDAC
0, //U+DDAD
0, //U+DDAE
0, //U+DDAF
0, //U+DDB0
0, //U+DDB1
0, //U+DDB2
0, //U+DDB3
0, //U+DDB4
0, //U+DDB5
0, //U+DDB6
0, //U+DDB7
0, //U+DDB8
0, //U+DDB9
0, //U+DDBA
0, //U+DDBB
0, //U+DDBC
0, //U+DDBD
0, //U+DDBE
0, //U+DDBF
0, //U+DDC0
0, //U+DDC1
0, //U+DDC2
0, //U+DDC3
0, //U+DDC4
0, //U+DDC5
0, //U+DDC6
0, //U+DDC7
0, //U+DDC8
0, //U+DDC9
0, //U+DDCA
0, //U+DDCB
0, //U+DDCC
0, //U+DDCD
0, //U+DDCE
0, //U+DDCF
0, //U+DDD0
0, //U+DDD1
0, //U+DDD2
0, //U+DDD3
0, //U+DDD4
0, //U+DDD5
0, //U+DDD6
0, //U+DDD7
0, //U+DDD8
0, //U+DDD9
0, //U+DDDA
0, //U+DDDB
0, //U+DDDC
0, //U+DDDD
0, //U+DDDE
0, //U+DDDF
0, //U+DDE0
0, //U+DDE1
0, //U+DDE2
0, //U+DDE3
0, //U+DDE4
0, //U+DDE5
0, //U+DDE6
0, //U+DDE7
0, //U+DDE8
0, //U+DDE9
0, //U+DDEA
0, //U+DDEB
0, //U+DDEC
0, //U+DDED
0, //U+DDEE
0, //U+DDEF
0, //U+DDF0
0, //U+DDF1
0, //U+DDF2
0, //U+DDF3
0, //U+DDF4
0, //U+DDF5
0, //U+DDF6
0, //U+DDF7
0, //U+DDF8
0, //U+DDF9
0, //U+DDFA
0, //U+DDFB
0, //U+DDFC
0, //U+DDFD
0, //U+DDFE
0, //U+DDFF
0, //U+DE00
0, //U+DE01
0, //U+DE02
0, //U+DE03
0, //U+DE04
0, //U+DE05
0, //U+DE06
0, //U+DE07
0, //U+DE08
0, //U+DE09
0, //U+DE0A
0, //U+DE0B
0, //U+DE0C
0, //U+DE0D
0, //U+DE0E
0, //U+DE0F
0, //U+DE10
0, //U+DE11
0, //U+DE12
0, //U+DE13
0, //U+DE14
0, //U+DE15
0, //U+DE16
0, //U+DE17
0, //U+DE18
0, //U+DE19
0, //U+DE1A
0, //U+DE1B
0, //U+DE1C
0, //U+DE1D
0, //U+DE1E
0, //U+DE1F
0, //U+DE20
0, //U+DE21
0, //U+DE22
0, //U+DE23
0, //U+DE24
0, //U+DE25
0, //U+DE26
0, //U+DE27
0, //U+DE28
0, //U+DE29
0, //U+DE2A
0, //U+DE2B
0, //U+DE2C
0, //U+DE2D
0, //U+DE2E
0, //U+DE2F
0, //U+DE30
0, //U+DE31
0, //U+DE32
0, //U+DE33
0, //U+DE34
0, //U+DE35
0, //U+DE36
0, //U+DE37
0, //U+DE38
0, //U+DE39
0, //U+DE3A
0, //U+DE3B
0, //U+DE3C
0, //U+DE3D
0, //U+DE3E
0, //U+DE3F
0, //U+DE40
0, //U+DE41
0, //U+DE42
0, //U+DE43
0, //U+DE44
0, //U+DE45
0, //U+DE46
0, //U+DE47
0, //U+DE48
0, //U+DE49
0, //U+DE4A
0, //U+DE4B
0, //U+DE4C
0, //U+DE4D
0, //U+DE4E
0, //U+DE4F
0, //U+DE50
0, //U+DE51
0, //U+DE52
0, //U+DE53
0, //U+DE54
0, //U+DE55
0, //U+DE56
0, //U+DE57
0, //U+DE58
0, //U+DE59
0, //U+DE5A
0, //U+DE5B
0, //U+DE5C
0, //U+DE5D
0, //U+DE5E
0, //U+DE5F
0, //U+DE60
0, //U+DE61
0, //U+DE62
0, //U+DE63
0, //U+DE64
0, //U+DE65
0, //U+DE66
0, //U+DE67
0, //U+DE68
0, //U+DE69
0, //U+DE6A
0, //U+DE6B
0, //U+DE6C
0, //U+DE6D
0, //U+DE6E
0, //U+DE6F
0, //U+DE70
0, //U+DE71
0, //U+DE72
0, //U+DE73
0, //U+DE74
0, //U+DE75
0, //U+DE76
0, //U+DE77
0, //U+DE78
0, //U+DE79
0, //U+DE7A
0, //U+DE7B
0, //U+DE7C
0, //U+DE7D
0, //U+DE7E
0, //U+DE7F
0, //U+DE80
0, //U+DE81
0, //U+DE82
0, //U+DE83
0, //U+DE84
0, //U+DE85
0, //U+DE86
0, //U+DE87
0, //U+DE88
0, //U+DE89
0, //U+DE8A
0, //U+DE8B
0, //U+DE8C
0, //U+DE8D
0, //U+DE8E
0, //U+DE8F
0, //U+DE90
0, //U+DE91
0, //U+DE92
0, //U+DE93
0, //U+DE94
0, //U+DE95
0, //U+DE96
0, //U+DE97
0, //U+DE98
0, //U+DE99
0, //U+DE9A
0, //U+DE9B
0, //U+DE9C
0, //U+DE9D
0, //U+DE9E
0, //U+DE9F
0, //U+DEA0
0, //U+DEA1
0, //U+DEA2
0, //U+DEA3
0, //U+DEA4
0, //U+DEA5
0, //U+DEA6
0, //U+DEA7
0, //U+DEA8
0, //U+DEA9
0, //U+DEAA
0, //U+DEAB
0, //U+DEAC
0, //U+DEAD
0, //U+DEAE
0, //U+DEAF
0, //U+DEB0
0, //U+DEB1
0, //U+DEB2
0, //U+DEB3
0, //U+DEB4
0, //U+DEB5
0, //U+DEB6
0, //U+DEB7
0, //U+DEB8
0, //U+DEB9
0, //U+DEBA
0, //U+DEBB
0, //U+DEBC
0, //U+DEBD
0, //U+DEBE
0, //U+DEBF
0, //U+DEC0
0, //U+DEC1
0, //U+DEC2
0, //U+DEC3
0, //U+DEC4
0, //U+DEC5
0, //U+DEC6
0, //U+DEC7
0, //U+DEC8
0, //U+DEC9
0, //U+DECA
0, //U+DECB
0, //U+DECC
0, //U+DECD
0, //U+DECE
0, //U+DECF
0, //U+DED0
0, //U+DED1
0, //U+DED2
0, //U+DED3
0, //U+DED4
0, //U+DED5
0, //U+DED6
0, //U+DED7
0, //U+DED8
0, //U+DED9
0, //U+DEDA
0, //U+DEDB
0, //U+DEDC
0, //U+DEDD
0, //U+DEDE
0, //U+DEDF
0, //U+DEE0
0, //U+DEE1
0, //U+DEE2
0, //U+DEE3
0, //U+DEE4
0, //U+DEE5
0, //U+DEE6
0, //U+DEE7
0, //U+DEE8
0, //U+DEE9
0, //U+DEEA
0, //U+DEEB
0, //U+DEEC
0, //U+DEED
0, //U+DEEE
0, //U+DEEF
0, //U+DEF0
0, //U+DEF1
0, //U+DEF2
0, //U+DEF3
0, //U+DEF4
0, //U+DEF5
0, //U+DEF6
0, //U+DEF7
0, //U+DEF8
0, //U+DEF9
0, //U+DEFA
0, //U+DEFB
0, //U+DEFC
0, //U+DEFD
0, //U+DEFE
0, //U+DEFF
0, //U+DF00
0, //U+DF01
0, //U+DF02
0, //U+DF03
0, //U+DF04
0, //U+DF05
0, //U+DF06
0, //U+DF07
0, //U+DF08
0, //U+DF09
0, //U+DF0A
0, //U+DF0B
0, //U+DF0C
0, //U+DF0D
0, //U+DF0E
0, //U+DF0F
0, //U+DF10
0, //U+DF11
0, //U+DF12
0, //U+DF13
0, //U+DF14
0, //U+DF15
0, //U+DF16
0, //U+DF17
0, //U+DF18
0, //U+DF19
0, //U+DF1A
0, //U+DF1B
0, //U+DF1C
0, //U+DF1D
0, //U+DF1E
0, //U+DF1F
0, //U+DF20
0, //U+DF21
0, //U+DF22
0, //U+DF23
0, //U+DF24
0, //U+DF25
0, //U+DF26
0, //U+DF27
0, //U+DF28
0, //U+DF29
0, //U+DF2A
0, //U+DF2B
0, //U+DF2C
0, //U+DF2D
0, //U+DF2E
0, //U+DF2F
0, //U+DF30
0, //U+DF31
0, //U+DF32
0, //U+DF33
0, //U+DF34
0, //U+DF35
0, //U+DF36
0, //U+DF37
0, //U+DF38
0, //U+DF39
0, //U+DF3A
0, //U+DF3B
0, //U+DF3C
0, //U+DF3D
0, //U+DF3E
0, //U+DF3F
0, //U+DF40
0, //U+DF41
0, //U+DF42
0, //U+DF43
0, //U+DF44
0, //U+DF45
0, //U+DF46
0, //U+DF47
0, //U+DF48
0, //U+DF49
0, //U+DF4A
0, //U+DF4B
0, //U+DF4C
0, //U+DF4D
0, //U+DF4E
0, //U+DF4F
0, //U+DF50
0, //U+DF51
0, //U+DF52
0, //U+DF53
0, //U+DF54
0, //U+DF55
0, //U+DF56
0, //U+DF57
0, //U+DF58
0, //U+DF59
0, //U+DF5A
0, //U+DF5B
0, //U+DF5C
0, //U+DF5D
0, //U+DF5E
0, //U+DF5F
0, //U+DF60
0, //U+DF61
0, //U+DF62
0, //U+DF63
0, //U+DF64
0, //U+DF65
0, //U+DF66
0, //U+DF67
0, //U+DF68
0, //U+DF69
0, //U+DF6A
0, //U+DF6B
0, //U+DF6C
0, //U+DF6D
0, //U+DF6E
0, //U+DF6F
0, //U+DF70
0, //U+DF71
0, //U+DF72
0, //U+DF73
0, //U+DF74
0, //U+DF75
0, //U+DF76
0, //U+DF77
0, //U+DF78
0, //U+DF79
0, //U+DF7A
0, //U+DF7B
0, //U+DF7C
0, //U+DF7D
0, //U+DF7E
0, //U+DF7F
0, //U+DF80
0, //U+DF81
0, //U+DF82
0, //U+DF83
0, //U+DF84
0, //U+DF85
0, //U+DF86
0, //U+DF87
0, //U+DF88
0, //U+DF89
0, //U+DF8A
0, //U+DF8B
0, //U+DF8C
0, //U+DF8D
0, //U+DF8E
0, //U+DF8F
0, //U+DF90
0, //U+DF91
0, //U+DF92
0, //U+DF93
0, //U+DF94
0, //U+DF95
0, //U+DF96
0, //U+DF97
0, //U+DF98
0, //U+DF99
0, //U+DF9A
0, //U+DF9B
0, //U+DF9C
0, //U+DF9D
0, //U+DF9E
0, //U+DF9F
0, //U+DFA0
0, //U+DFA1
0, //U+DFA2
0, //U+DFA3
0, //U+DFA4
0, //U+DFA5
0, //U+DFA6
0, //U+DFA7
0, //U+DFA8
0, //U+DFA9
0, //U+DFAA
0, //U+DFAB
0, //U+DFAC
0, //U+DFAD
0, //U+DFAE
0, //U+DFAF
0, //U+DFB0
0, //U+DFB1
0, //U+DFB2
0, //U+DFB3
0, //U+DFB4
0, //U+DFB5
0, //U+DFB6
0, //U+DFB7
0, //U+DFB8
0, //U+DFB9
0, //U+DFBA
0, //U+DFBB
0, //U+DFBC
0, //U+DFBD
0, //U+DFBE
0, //U+DFBF
0, //U+DFC0
0, //U+DFC1
0, //U+DFC2
0, //U+DFC3
0, //U+DFC4
0, //U+DFC5
0, //U+DFC6
0, //U+DFC7
0, //U+DFC8
0, //U+DFC9
0, //U+DFCA
0, //U+DFCB
0, //U+DFCC
0, //U+DFCD
0, //U+DFCE
0, //U+DFCF
0, //U+DFD0
0, //U+DFD1
0, //U+DFD2
0, //U+DFD3
0, //U+DFD4
0, //U+DFD5
0, //U+DFD6
0, //U+DFD7
0, //U+DFD8
0, //U+DFD9
0, //U+DFDA
0, //U+DFDB
0, //U+DFDC
0, //U+DFDD
0, //U+DFDE
0, //U+DFDF
0, //U+DFE0
0, //U+DFE1
0, //U+DFE2
0, //U+DFE3
0, //U+DFE4
0, //U+DFE5
0, //U+DFE6
0, //U+DFE7
0, //U+DFE8
0, //U+DFE9
0, //U+DFEA
0, //U+DFEB
0, //U+DFEC
0, //U+DFED
0, //U+DFEE
0, //U+DFEF
0, //U+DFF0
0, //U+DFF1
0, //U+DFF2
0, //U+DFF3
0, //U+DFF4
0, //U+DFF5
0, //U+DFF6
0, //U+DFF7
0, //U+DFF8
0, //U+DFF9
0, //U+DFFA
0, //U+DFFB
0, //U+DFFC
0, //U+DFFD
0, //U+DFFE
UNICODE_Cs, //U+DFFF
UNICODE_Co, //U+E000
0, //U+E001
0, //U+E002
0, //U+E003
0, //U+E004
0, //U+E005
0, //U+E006
0, //U+E007
0, //U+E008
0, //U+E009
0, //U+E00A
0, //U+E00B
0, //U+E00C
0, //U+E00D
0, //U+E00E
0, //U+E00F
0, //U+E010
0, //U+E011
0, //U+E012
0, //U+E013
0, //U+E014
0, //U+E015
0, //U+E016
0, //U+E017
0, //U+E018
0, //U+E019
0, //U+E01A
0, //U+E01B
0, //U+E01C
0, //U+E01D
0, //U+E01E
0, //U+E01F
0, //U+E020
0, //U+E021
0, //U+E022
0, //U+E023
0, //U+E024
0, //U+E025
0, //U+E026
0, //U+E027
0, //U+E028
0, //U+E029
0, //U+E02A
0, //U+E02B
0, //U+E02C
0, //U+E02D
0, //U+E02E
0, //U+E02F
0, //U+E030
0, //U+E031
0, //U+E032
0, //U+E033
0, //U+E034
0, //U+E035
0, //U+E036
0, //U+E037
0, //U+E038
0, //U+E039
0, //U+E03A
0, //U+E03B
0, //U+E03C
0, //U+E03D
0, //U+E03E
0, //U+E03F
0, //U+E040
0, //U+E041
0, //U+E042
0, //U+E043
0, //U+E044
0, //U+E045
0, //U+E046
0, //U+E047
0, //U+E048
0, //U+E049
0, //U+E04A
0, //U+E04B
0, //U+E04C
0, //U+E04D
0, //U+E04E
0, //U+E04F
0, //U+E050
0, //U+E051
0, //U+E052
0, //U+E053
0, //U+E054
0, //U+E055
0, //U+E056
0, //U+E057
0, //U+E058
0, //U+E059
0, //U+E05A
0, //U+E05B
0, //U+E05C
0, //U+E05D
0, //U+E05E
0, //U+E05F
0, //U+E060
0, //U+E061
0, //U+E062
0, //U+E063
0, //U+E064
0, //U+E065
0, //U+E066
0, //U+E067
0, //U+E068
0, //U+E069
0, //U+E06A
0, //U+E06B
0, //U+E06C
0, //U+E06D
0, //U+E06E
0, //U+E06F
0, //U+E070
0, //U+E071
0, //U+E072
0, //U+E073
0, //U+E074
0, //U+E075
0, //U+E076
0, //U+E077
0, //U+E078
0, //U+E079
0, //U+E07A
0, //U+E07B
0, //U+E07C
0, //U+E07D
0, //U+E07E
0, //U+E07F
0, //U+E080
0, //U+E081
0, //U+E082
0, //U+E083
0, //U+E084
0, //U+E085
0, //U+E086
0, //U+E087
0, //U+E088
0, //U+E089
0, //U+E08A
0, //U+E08B
0, //U+E08C
0, //U+E08D
0, //U+E08E
0, //U+E08F
0, //U+E090
0, //U+E091
0, //U+E092
0, //U+E093
0, //U+E094
0, //U+E095
0, //U+E096
0, //U+E097
0, //U+E098
0, //U+E099
0, //U+E09A
0, //U+E09B
0, //U+E09C
0, //U+E09D
0, //U+E09E
0, //U+E09F
0, //U+E0A0
0, //U+E0A1
0, //U+E0A2
0, //U+E0A3
0, //U+E0A4
0, //U+E0A5
0, //U+E0A6
0, //U+E0A7
0, //U+E0A8
0, //U+E0A9
0, //U+E0AA
0, //U+E0AB
0, //U+E0AC
0, //U+E0AD
0, //U+E0AE
0, //U+E0AF
0, //U+E0B0
0, //U+E0B1
0, //U+E0B2
0, //U+E0B3
0, //U+E0B4
0, //U+E0B5
0, //U+E0B6
0, //U+E0B7
0, //U+E0B8
0, //U+E0B9
0, //U+E0BA
0, //U+E0BB
0, //U+E0BC
0, //U+E0BD
0, //U+E0BE
0, //U+E0BF
0, //U+E0C0
0, //U+E0C1
0, //U+E0C2
0, //U+E0C3
0, //U+E0C4
0, //U+E0C5
0, //U+E0C6
0, //U+E0C7
0, //U+E0C8
0, //U+E0C9
0, //U+E0CA
0, //U+E0CB
0, //U+E0CC
0, //U+E0CD
0, //U+E0CE
0, //U+E0CF
0, //U+E0D0
0, //U+E0D1
0, //U+E0D2
0, //U+E0D3
0, //U+E0D4
0, //U+E0D5
0, //U+E0D6
0, //U+E0D7
0, //U+E0D8
0, //U+E0D9
0, //U+E0DA
0, //U+E0DB
0, //U+E0DC
0, //U+E0DD
0, //U+E0DE
0, //U+E0DF
0, //U+E0E0
0, //U+E0E1
0, //U+E0E2
0, //U+E0E3
0, //U+E0E4
0, //U+E0E5
0, //U+E0E6
0, //U+E0E7
0, //U+E0E8
0, //U+E0E9
0, //U+E0EA
0, //U+E0EB
0, //U+E0EC
0, //U+E0ED
0, //U+E0EE
0, //U+E0EF
0, //U+E0F0
0, //U+E0F1
0, //U+E0F2
0, //U+E0F3
0, //U+E0F4
0, //U+E0F5
0, //U+E0F6
0, //U+E0F7
0, //U+E0F8
0, //U+E0F9
0, //U+E0FA
0, //U+E0FB
0, //U+E0FC
0, //U+E0FD
0, //U+E0FE
0, //U+E0FF
0, //U+E100
0, //U+E101
0, //U+E102
0, //U+E103
0, //U+E104
0, //U+E105
0, //U+E106
0, //U+E107
0, //U+E108
0, //U+E109
0, //U+E10A
0, //U+E10B
0, //U+E10C
0, //U+E10D
0, //U+E10E
0, //U+E10F
0, //U+E110
0, //U+E111
0, //U+E112
0, //U+E113
0, //U+E114
0, //U+E115
0, //U+E116
0, //U+E117
0, //U+E118
0, //U+E119
0, //U+E11A
0, //U+E11B
0, //U+E11C
0, //U+E11D
0, //U+E11E
0, //U+E11F
0, //U+E120
0, //U+E121
0, //U+E122
0, //U+E123
0, //U+E124
0, //U+E125
0, //U+E126
0, //U+E127
0, //U+E128
0, //U+E129
0, //U+E12A
0, //U+E12B
0, //U+E12C
0, //U+E12D
0, //U+E12E
0, //U+E12F
0, //U+E130
0, //U+E131
0, //U+E132
0, //U+E133
0, //U+E134
0, //U+E135
0, //U+E136
0, //U+E137
0, //U+E138
0, //U+E139
0, //U+E13A
0, //U+E13B
0, //U+E13C
0, //U+E13D
0, //U+E13E
0, //U+E13F
0, //U+E140
0, //U+E141
0, //U+E142
0, //U+E143
0, //U+E144
0, //U+E145
0, //U+E146
0, //U+E147
0, //U+E148
0, //U+E149
0, //U+E14A
0, //U+E14B
0, //U+E14C
0, //U+E14D
0, //U+E14E
0, //U+E14F
0, //U+E150
0, //U+E151
0, //U+E152
0, //U+E153
0, //U+E154
0, //U+E155
0, //U+E156
0, //U+E157
0, //U+E158
0, //U+E159
0, //U+E15A
0, //U+E15B
0, //U+E15C
0, //U+E15D
0, //U+E15E
0, //U+E15F
0, //U+E160
0, //U+E161
0, //U+E162
0, //U+E163
0, //U+E164
0, //U+E165
0, //U+E166
0, //U+E167
0, //U+E168
0, //U+E169
0, //U+E16A
0, //U+E16B
0, //U+E16C
0, //U+E16D
0, //U+E16E
0, //U+E16F
0, //U+E170
0, //U+E171
0, //U+E172
0, //U+E173
0, //U+E174
0, //U+E175
0, //U+E176
0, //U+E177
0, //U+E178
0, //U+E179
0, //U+E17A
0, //U+E17B
0, //U+E17C
0, //U+E17D
0, //U+E17E
0, //U+E17F
0, //U+E180
0, //U+E181
0, //U+E182
0, //U+E183
0, //U+E184
0, //U+E185
0, //U+E186
0, //U+E187
0, //U+E188
0, //U+E189
0, //U+E18A
0, //U+E18B
0, //U+E18C
0, //U+E18D
0, //U+E18E
0, //U+E18F
0, //U+E190
0, //U+E191
0, //U+E192
0, //U+E193
0, //U+E194
0, //U+E195
0, //U+E196
0, //U+E197
0, //U+E198
0, //U+E199
0, //U+E19A
0, //U+E19B
0, //U+E19C
0, //U+E19D
0, //U+E19E
0, //U+E19F
0, //U+E1A0
0, //U+E1A1
0, //U+E1A2
0, //U+E1A3
0, //U+E1A4
0, //U+E1A5
0, //U+E1A6
0, //U+E1A7
0, //U+E1A8
0, //U+E1A9
0, //U+E1AA
0, //U+E1AB
0, //U+E1AC
0, //U+E1AD
0, //U+E1AE
0, //U+E1AF
0, //U+E1B0
0, //U+E1B1
0, //U+E1B2
0, //U+E1B3
0, //U+E1B4
0, //U+E1B5
0, //U+E1B6
0, //U+E1B7
0, //U+E1B8
0, //U+E1B9
0, //U+E1BA
0, //U+E1BB
0, //U+E1BC
0, //U+E1BD
0, //U+E1BE
0, //U+E1BF
0, //U+E1C0
0, //U+E1C1
0, //U+E1C2
0, //U+E1C3
0, //U+E1C4
0, //U+E1C5
0, //U+E1C6
0, //U+E1C7
0, //U+E1C8
0, //U+E1C9
0, //U+E1CA
0, //U+E1CB
0, //U+E1CC
0, //U+E1CD
0, //U+E1CE
0, //U+E1CF
0, //U+E1D0
0, //U+E1D1
0, //U+E1D2
0, //U+E1D3
0, //U+E1D4
0, //U+E1D5
0, //U+E1D6
0, //U+E1D7
0, //U+E1D8
0, //U+E1D9
0, //U+E1DA
0, //U+E1DB
0, //U+E1DC
0, //U+E1DD
0, //U+E1DE
0, //U+E1DF
0, //U+E1E0
0, //U+E1E1
0, //U+E1E2
0, //U+E1E3
0, //U+E1E4
0, //U+E1E5
0, //U+E1E6
0, //U+E1E7
0, //U+E1E8
0, //U+E1E9
0, //U+E1EA
0, //U+E1EB
0, //U+E1EC
0, //U+E1ED
0, //U+E1EE
0, //U+E1EF
0, //U+E1F0
0, //U+E1F1
0, //U+E1F2
0, //U+E1F3
0, //U+E1F4
0, //U+E1F5
0, //U+E1F6
0, //U+E1F7
0, //U+E1F8
0, //U+E1F9
0, //U+E1FA
0, //U+E1FB
0, //U+E1FC
0, //U+E1FD
0, //U+E1FE
0, //U+E1FF
0, //U+E200
0, //U+E201
0, //U+E202
0, //U+E203
0, //U+E204
0, //U+E205
0, //U+E206
0, //U+E207
0, //U+E208
0, //U+E209
0, //U+E20A
0, //U+E20B
0, //U+E20C
0, //U+E20D
0, //U+E20E
0, //U+E20F
0, //U+E210
0, //U+E211
0, //U+E212
0, //U+E213
0, //U+E214
0, //U+E215
0, //U+E216
0, //U+E217
0, //U+E218
0, //U+E219
0, //U+E21A
0, //U+E21B
0, //U+E21C
0, //U+E21D
0, //U+E21E
0, //U+E21F
0, //U+E220
0, //U+E221
0, //U+E222
0, //U+E223
0, //U+E224
0, //U+E225
0, //U+E226
0, //U+E227
0, //U+E228
0, //U+E229
0, //U+E22A
0, //U+E22B
0, //U+E22C
0, //U+E22D
0, //U+E22E
0, //U+E22F
0, //U+E230
0, //U+E231
0, //U+E232
0, //U+E233
0, //U+E234
0, //U+E235
0, //U+E236
0, //U+E237
0, //U+E238
0, //U+E239
0, //U+E23A
0, //U+E23B
0, //U+E23C
0, //U+E23D
0, //U+E23E
0, //U+E23F
0, //U+E240
0, //U+E241
0, //U+E242
0, //U+E243
0, //U+E244
0, //U+E245
0, //U+E246
0, //U+E247
0, //U+E248
0, //U+E249
0, //U+E24A
0, //U+E24B
0, //U+E24C
0, //U+E24D
0, //U+E24E
0, //U+E24F
0, //U+E250
0, //U+E251
0, //U+E252
0, //U+E253
0, //U+E254
0, //U+E255
0, //U+E256
0, //U+E257
0, //U+E258
0, //U+E259
0, //U+E25A
0, //U+E25B
0, //U+E25C
0, //U+E25D
0, //U+E25E
0, //U+E25F
0, //U+E260
0, //U+E261
0, //U+E262
0, //U+E263
0, //U+E264
0, //U+E265
0, //U+E266
0, //U+E267
0, //U+E268
0, //U+E269
0, //U+E26A
0, //U+E26B
0, //U+E26C
0, //U+E26D
0, //U+E26E
0, //U+E26F
0, //U+E270
0, //U+E271
0, //U+E272
0, //U+E273
0, //U+E274
0, //U+E275
0, //U+E276
0, //U+E277
0, //U+E278
0, //U+E279
0, //U+E27A
0, //U+E27B
0, //U+E27C
0, //U+E27D
0, //U+E27E
0, //U+E27F
0, //U+E280
0, //U+E281
0, //U+E282
0, //U+E283
0, //U+E284
0, //U+E285
0, //U+E286
0, //U+E287
0, //U+E288
0, //U+E289
0, //U+E28A
0, //U+E28B
0, //U+E28C
0, //U+E28D
0, //U+E28E
0, //U+E28F
0, //U+E290
0, //U+E291
0, //U+E292
0, //U+E293
0, //U+E294
0, //U+E295
0, //U+E296
0, //U+E297
0, //U+E298
0, //U+E299
0, //U+E29A
0, //U+E29B
0, //U+E29C
0, //U+E29D
0, //U+E29E
0, //U+E29F
0, //U+E2A0
0, //U+E2A1
0, //U+E2A2
0, //U+E2A3
0, //U+E2A4
0, //U+E2A5
0, //U+E2A6
0, //U+E2A7
0, //U+E2A8
0, //U+E2A9
0, //U+E2AA
0, //U+E2AB
0, //U+E2AC
0, //U+E2AD
0, //U+E2AE
0, //U+E2AF
0, //U+E2B0
0, //U+E2B1
0, //U+E2B2
0, //U+E2B3
0, //U+E2B4
0, //U+E2B5
0, //U+E2B6
0, //U+E2B7
0, //U+E2B8
0, //U+E2B9
0, //U+E2BA
0, //U+E2BB
0, //U+E2BC
0, //U+E2BD
0, //U+E2BE
0, //U+E2BF
0, //U+E2C0
0, //U+E2C1
0, //U+E2C2
0, //U+E2C3
0, //U+E2C4
0, //U+E2C5
0, //U+E2C6
0, //U+E2C7
0, //U+E2C8
0, //U+E2C9
0, //U+E2CA
0, //U+E2CB
0, //U+E2CC
0, //U+E2CD
0, //U+E2CE
0, //U+E2CF
0, //U+E2D0
0, //U+E2D1
0, //U+E2D2
0, //U+E2D3
0, //U+E2D4
0, //U+E2D5
0, //U+E2D6
0, //U+E2D7
0, //U+E2D8
0, //U+E2D9
0, //U+E2DA
0, //U+E2DB
0, //U+E2DC
0, //U+E2DD
0, //U+E2DE
0, //U+E2DF
0, //U+E2E0
0, //U+E2E1
0, //U+E2E2
0, //U+E2E3
0, //U+E2E4
0, //U+E2E5
0, //U+E2E6
0, //U+E2E7
0, //U+E2E8
0, //U+E2E9
0, //U+E2EA
0, //U+E2EB
0, //U+E2EC
0, //U+E2ED
0, //U+E2EE
0, //U+E2EF
0, //U+E2F0
0, //U+E2F1
0, //U+E2F2
0, //U+E2F3
0, //U+E2F4
0, //U+E2F5
0, //U+E2F6
0, //U+E2F7
0, //U+E2F8
0, //U+E2F9
0, //U+E2FA
0, //U+E2FB
0, //U+E2FC
0, //U+E2FD
0, //U+E2FE
0, //U+E2FF
0, //U+E300
0, //U+E301
0, //U+E302
0, //U+E303
0, //U+E304
0, //U+E305
0, //U+E306
0, //U+E307
0, //U+E308
0, //U+E309
0, //U+E30A
0, //U+E30B
0, //U+E30C
0, //U+E30D
0, //U+E30E
0, //U+E30F
0, //U+E310
0, //U+E311
0, //U+E312
0, //U+E313
0, //U+E314
0, //U+E315
0, //U+E316
0, //U+E317
0, //U+E318
0, //U+E319
0, //U+E31A
0, //U+E31B
0, //U+E31C
0, //U+E31D
0, //U+E31E
0, //U+E31F
0, //U+E320
0, //U+E321
0, //U+E322
0, //U+E323
0, //U+E324
0, //U+E325
0, //U+E326
0, //U+E327
0, //U+E328
0, //U+E329
0, //U+E32A
0, //U+E32B
0, //U+E32C
0, //U+E32D
0, //U+E32E
0, //U+E32F
0, //U+E330
0, //U+E331
0, //U+E332
0, //U+E333
0, //U+E334
0, //U+E335
0, //U+E336
0, //U+E337
0, //U+E338
0, //U+E339
0, //U+E33A
0, //U+E33B
0, //U+E33C
0, //U+E33D
0, //U+E33E
0, //U+E33F
0, //U+E340
0, //U+E341
0, //U+E342
0, //U+E343
0, //U+E344
0, //U+E345
0, //U+E346
0, //U+E347
0, //U+E348
0, //U+E349
0, //U+E34A
0, //U+E34B
0, //U+E34C
0, //U+E34D
0, //U+E34E
0, //U+E34F
0, //U+E350
0, //U+E351
0, //U+E352
0, //U+E353
0, //U+E354
0, //U+E355
0, //U+E356
0, //U+E357
0, //U+E358
0, //U+E359
0, //U+E35A
0, //U+E35B
0, //U+E35C
0, //U+E35D
0, //U+E35E
0, //U+E35F
0, //U+E360
0, //U+E361
0, //U+E362
0, //U+E363
0, //U+E364
0, //U+E365
0, //U+E366
0, //U+E367
0, //U+E368
0, //U+E369
0, //U+E36A
0, //U+E36B
0, //U+E36C
0, //U+E36D
0, //U+E36E
0, //U+E36F
0, //U+E370
0, //U+E371
0, //U+E372
0, //U+E373
0, //U+E374
0, //U+E375
0, //U+E376
0, //U+E377
0, //U+E378
0, //U+E379
0, //U+E37A
0, //U+E37B
0, //U+E37C
0, //U+E37D
0, //U+E37E
0, //U+E37F
0, //U+E380
0, //U+E381
0, //U+E382
0, //U+E383
0, //U+E384
0, //U+E385
0, //U+E386
0, //U+E387
0, //U+E388
0, //U+E389
0, //U+E38A
0, //U+E38B
0, //U+E38C
0, //U+E38D
0, //U+E38E
0, //U+E38F
0, //U+E390
0, //U+E391
0, //U+E392
0, //U+E393
0, //U+E394
0, //U+E395
0, //U+E396
0, //U+E397
0, //U+E398
0, //U+E399
0, //U+E39A
0, //U+E39B
0, //U+E39C
0, //U+E39D
0, //U+E39E
0, //U+E39F
0, //U+E3A0
0, //U+E3A1
0, //U+E3A2
0, //U+E3A3
0, //U+E3A4
0, //U+E3A5
0, //U+E3A6
0, //U+E3A7
0, //U+E3A8
0, //U+E3A9
0, //U+E3AA
0, //U+E3AB
0, //U+E3AC
0, //U+E3AD
0, //U+E3AE
0, //U+E3AF
0, //U+E3B0
0, //U+E3B1
0, //U+E3B2
0, //U+E3B3
0, //U+E3B4
0, //U+E3B5
0, //U+E3B6
0, //U+E3B7
0, //U+E3B8
0, //U+E3B9
0, //U+E3BA
0, //U+E3BB
0, //U+E3BC
0, //U+E3BD
0, //U+E3BE
0, //U+E3BF
0, //U+E3C0
0, //U+E3C1
0, //U+E3C2
0, //U+E3C3
0, //U+E3C4
0, //U+E3C5
0, //U+E3C6
0, //U+E3C7
0, //U+E3C8
0, //U+E3C9
0, //U+E3CA
0, //U+E3CB
0, //U+E3CC
0, //U+E3CD
0, //U+E3CE
0, //U+E3CF
0, //U+E3D0
0, //U+E3D1
0, //U+E3D2
0, //U+E3D3
0, //U+E3D4
0, //U+E3D5
0, //U+E3D6
0, //U+E3D7
0, //U+E3D8
0, //U+E3D9
0, //U+E3DA
0, //U+E3DB
0, //U+E3DC
0, //U+E3DD
0, //U+E3DE
0, //U+E3DF
0, //U+E3E0
0, //U+E3E1
0, //U+E3E2
0, //U+E3E3
0, //U+E3E4
0, //U+E3E5
0, //U+E3E6
0, //U+E3E7
0, //U+E3E8
0, //U+E3E9
0, //U+E3EA
0, //U+E3EB
0, //U+E3EC
0, //U+E3ED
0, //U+E3EE
0, //U+E3EF
0, //U+E3F0
0, //U+E3F1
0, //U+E3F2
0, //U+E3F3
0, //U+E3F4
0, //U+E3F5
0, //U+E3F6
0, //U+E3F7
0, //U+E3F8
0, //U+E3F9
0, //U+E3FA
0, //U+E3FB
0, //U+E3FC
0, //U+E3FD
0, //U+E3FE
0, //U+E3FF
0, //U+E400
0, //U+E401
0, //U+E402
0, //U+E403
0, //U+E404
0, //U+E405
0, //U+E406
0, //U+E407
0, //U+E408
0, //U+E409
0, //U+E40A
0, //U+E40B
0, //U+E40C
0, //U+E40D
0, //U+E40E
0, //U+E40F
0, //U+E410
0, //U+E411
0, //U+E412
0, //U+E413
0, //U+E414
0, //U+E415
0, //U+E416
0, //U+E417
0, //U+E418
0, //U+E419
0, //U+E41A
0, //U+E41B
0, //U+E41C
0, //U+E41D
0, //U+E41E
0, //U+E41F
0, //U+E420
0, //U+E421
0, //U+E422
0, //U+E423
0, //U+E424
0, //U+E425
0, //U+E426
0, //U+E427
0, //U+E428
0, //U+E429
0, //U+E42A
0, //U+E42B
0, //U+E42C
0, //U+E42D
0, //U+E42E
0, //U+E42F
0, //U+E430
0, //U+E431
0, //U+E432
0, //U+E433
0, //U+E434
0, //U+E435
0, //U+E436
0, //U+E437
0, //U+E438
0, //U+E439
0, //U+E43A
0, //U+E43B
0, //U+E43C
0, //U+E43D
0, //U+E43E
0, //U+E43F
0, //U+E440
0, //U+E441
0, //U+E442
0, //U+E443
0, //U+E444
0, //U+E445
0, //U+E446
0, //U+E447
0, //U+E448
0, //U+E449
0, //U+E44A
0, //U+E44B
0, //U+E44C
0, //U+E44D
0, //U+E44E
0, //U+E44F
0, //U+E450
0, //U+E451
0, //U+E452
0, //U+E453
0, //U+E454
0, //U+E455
0, //U+E456
0, //U+E457
0, //U+E458
0, //U+E459
0, //U+E45A
0, //U+E45B
0, //U+E45C
0, //U+E45D
0, //U+E45E
0, //U+E45F
0, //U+E460
0, //U+E461
0, //U+E462
0, //U+E463
0, //U+E464
0, //U+E465
0, //U+E466
0, //U+E467
0, //U+E468
0, //U+E469
0, //U+E46A
0, //U+E46B
0, //U+E46C
0, //U+E46D
0, //U+E46E
0, //U+E46F
0, //U+E470
0, //U+E471
0, //U+E472
0, //U+E473
0, //U+E474
0, //U+E475
0, //U+E476
0, //U+E477
0, //U+E478
0, //U+E479
0, //U+E47A
0, //U+E47B
0, //U+E47C
0, //U+E47D
0, //U+E47E
0, //U+E47F
0, //U+E480
0, //U+E481
0, //U+E482
0, //U+E483
0, //U+E484
0, //U+E485
0, //U+E486
0, //U+E487
0, //U+E488
0, //U+E489
0, //U+E48A
0, //U+E48B
0, //U+E48C
0, //U+E48D
0, //U+E48E
0, //U+E48F
0, //U+E490
0, //U+E491
0, //U+E492
0, //U+E493
0, //U+E494
0, //U+E495
0, //U+E496
0, //U+E497
0, //U+E498
0, //U+E499
0, //U+E49A
0, //U+E49B
0, //U+E49C
0, //U+E49D
0, //U+E49E
0, //U+E49F
0, //U+E4A0
0, //U+E4A1
0, //U+E4A2
0, //U+E4A3
0, //U+E4A4
0, //U+E4A5
0, //U+E4A6
0, //U+E4A7
0, //U+E4A8
0, //U+E4A9
0, //U+E4AA
0, //U+E4AB
0, //U+E4AC
0, //U+E4AD
0, //U+E4AE
0, //U+E4AF
0, //U+E4B0
0, //U+E4B1
0, //U+E4B2
0, //U+E4B3
0, //U+E4B4
0, //U+E4B5
0, //U+E4B6
0, //U+E4B7
0, //U+E4B8
0, //U+E4B9
0, //U+E4BA
0, //U+E4BB
0, //U+E4BC
0, //U+E4BD
0, //U+E4BE
0, //U+E4BF
0, //U+E4C0
0, //U+E4C1
0, //U+E4C2
0, //U+E4C3
0, //U+E4C4
0, //U+E4C5
0, //U+E4C6
0, //U+E4C7
0, //U+E4C8
0, //U+E4C9
0, //U+E4CA
0, //U+E4CB
0, //U+E4CC
0, //U+E4CD
0, //U+E4CE
0, //U+E4CF
0, //U+E4D0
0, //U+E4D1
0, //U+E4D2
0, //U+E4D3
0, //U+E4D4
0, //U+E4D5
0, //U+E4D6
0, //U+E4D7
0, //U+E4D8
0, //U+E4D9
0, //U+E4DA
0, //U+E4DB
0, //U+E4DC
0, //U+E4DD
0, //U+E4DE
0, //U+E4DF
0, //U+E4E0
0, //U+E4E1
0, //U+E4E2
0, //U+E4E3
0, //U+E4E4
0, //U+E4E5
0, //U+E4E6
0, //U+E4E7
0, //U+E4E8
0, //U+E4E9
0, //U+E4EA
0, //U+E4EB
0, //U+E4EC
0, //U+E4ED
0, //U+E4EE
0, //U+E4EF
0, //U+E4F0
0, //U+E4F1
0, //U+E4F2
0, //U+E4F3
0, //U+E4F4
0, //U+E4F5
0, //U+E4F6
0, //U+E4F7
0, //U+E4F8
0, //U+E4F9
0, //U+E4FA
0, //U+E4FB
0, //U+E4FC
0, //U+E4FD
0, //U+E4FE
0, //U+E4FF
0, //U+E500
0, //U+E501
0, //U+E502
0, //U+E503
0, //U+E504
0, //U+E505
0, //U+E506
0, //U+E507
0, //U+E508
0, //U+E509
0, //U+E50A
0, //U+E50B
0, //U+E50C
0, //U+E50D
0, //U+E50E
0, //U+E50F
0, //U+E510
0, //U+E511
0, //U+E512
0, //U+E513
0, //U+E514
0, //U+E515
0, //U+E516
0, //U+E517
0, //U+E518
0, //U+E519
0, //U+E51A
0, //U+E51B
0, //U+E51C
0, //U+E51D
0, //U+E51E
0, //U+E51F
0, //U+E520
0, //U+E521
0, //U+E522
0, //U+E523
0, //U+E524
0, //U+E525
0, //U+E526
0, //U+E527
0, //U+E528
0, //U+E529
0, //U+E52A
0, //U+E52B
0, //U+E52C
0, //U+E52D
0, //U+E52E
0, //U+E52F
0, //U+E530
0, //U+E531
0, //U+E532
0, //U+E533
0, //U+E534
0, //U+E535
0, //U+E536
0, //U+E537
0, //U+E538
0, //U+E539
0, //U+E53A
0, //U+E53B
0, //U+E53C
0, //U+E53D
0, //U+E53E
0, //U+E53F
0, //U+E540
0, //U+E541
0, //U+E542
0, //U+E543
0, //U+E544
0, //U+E545
0, //U+E546
0, //U+E547
0, //U+E548
0, //U+E549
0, //U+E54A
0, //U+E54B
0, //U+E54C
0, //U+E54D
0, //U+E54E
0, //U+E54F
0, //U+E550
0, //U+E551
0, //U+E552
0, //U+E553
0, //U+E554
0, //U+E555
0, //U+E556
0, //U+E557
0, //U+E558
0, //U+E559
0, //U+E55A
0, //U+E55B
0, //U+E55C
0, //U+E55D
0, //U+E55E
0, //U+E55F
0, //U+E560
0, //U+E561
0, //U+E562
0, //U+E563
0, //U+E564
0, //U+E565
0, //U+E566
0, //U+E567
0, //U+E568
0, //U+E569
0, //U+E56A
0, //U+E56B
0, //U+E56C
0, //U+E56D
0, //U+E56E
0, //U+E56F
0, //U+E570
0, //U+E571
0, //U+E572
0, //U+E573
0, //U+E574
0, //U+E575
0, //U+E576
0, //U+E577
0, //U+E578
0, //U+E579
0, //U+E57A
0, //U+E57B
0, //U+E57C
0, //U+E57D
0, //U+E57E
0, //U+E57F
0, //U+E580
0, //U+E581
0, //U+E582
0, //U+E583
0, //U+E584
0, //U+E585
0, //U+E586
0, //U+E587
0, //U+E588
0, //U+E589
0, //U+E58A
0, //U+E58B
0, //U+E58C
0, //U+E58D
0, //U+E58E
0, //U+E58F
0, //U+E590
0, //U+E591
0, //U+E592
0, //U+E593
0, //U+E594
0, //U+E595
0, //U+E596
0, //U+E597
0, //U+E598
0, //U+E599
0, //U+E59A
0, //U+E59B
0, //U+E59C
0, //U+E59D
0, //U+E59E
0, //U+E59F
0, //U+E5A0
0, //U+E5A1
0, //U+E5A2
0, //U+E5A3
0, //U+E5A4
0, //U+E5A5
0, //U+E5A6
0, //U+E5A7
0, //U+E5A8
0, //U+E5A9
0, //U+E5AA
0, //U+E5AB
0, //U+E5AC
0, //U+E5AD
0, //U+E5AE
0, //U+E5AF
0, //U+E5B0
0, //U+E5B1
0, //U+E5B2
0, //U+E5B3
0, //U+E5B4
0, //U+E5B5
0, //U+E5B6
0, //U+E5B7
0, //U+E5B8
0, //U+E5B9
0, //U+E5BA
0, //U+E5BB
0, //U+E5BC
0, //U+E5BD
0, //U+E5BE
0, //U+E5BF
0, //U+E5C0
0, //U+E5C1
0, //U+E5C2
0, //U+E5C3
0, //U+E5C4
0, //U+E5C5
0, //U+E5C6
0, //U+E5C7
0, //U+E5C8
0, //U+E5C9
0, //U+E5CA
0, //U+E5CB
0, //U+E5CC
0, //U+E5CD
0, //U+E5CE
0, //U+E5CF
0, //U+E5D0
0, //U+E5D1
0, //U+E5D2
0, //U+E5D3
0, //U+E5D4
0, //U+E5D5
0, //U+E5D6
0, //U+E5D7
0, //U+E5D8
0, //U+E5D9
0, //U+E5DA
0, //U+E5DB
0, //U+E5DC
0, //U+E5DD
0, //U+E5DE
0, //U+E5DF
0, //U+E5E0
0, //U+E5E1
0, //U+E5E2
0, //U+E5E3
0, //U+E5E4
0, //U+E5E5
0, //U+E5E6
0, //U+E5E7
0, //U+E5E8
0, //U+E5E9
0, //U+E5EA
0, //U+E5EB
0, //U+E5EC
0, //U+E5ED
0, //U+E5EE
0, //U+E5EF
0, //U+E5F0
0, //U+E5F1
0, //U+E5F2
0, //U+E5F3
0, //U+E5F4
0, //U+E5F5
0, //U+E5F6
0, //U+E5F7
0, //U+E5F8
0, //U+E5F9
0, //U+E5FA
0, //U+E5FB
0, //U+E5FC
0, //U+E5FD
0, //U+E5FE
0, //U+E5FF
0, //U+E600
0, //U+E601
0, //U+E602
0, //U+E603
0, //U+E604
0, //U+E605
0, //U+E606
0, //U+E607
0, //U+E608
0, //U+E609
0, //U+E60A
0, //U+E60B
0, //U+E60C
0, //U+E60D
0, //U+E60E
0, //U+E60F
0, //U+E610
0, //U+E611
0, //U+E612
0, //U+E613
0, //U+E614
0, //U+E615
0, //U+E616
0, //U+E617
0, //U+E618
0, //U+E619
0, //U+E61A
0, //U+E61B
0, //U+E61C
0, //U+E61D
0, //U+E61E
0, //U+E61F
0, //U+E620
0, //U+E621
0, //U+E622
0, //U+E623
0, //U+E624
0, //U+E625
0, //U+E626
0, //U+E627
0, //U+E628
0, //U+E629
0, //U+E62A
0, //U+E62B
0, //U+E62C
0, //U+E62D
0, //U+E62E
0, //U+E62F
0, //U+E630
0, //U+E631
0, //U+E632
0, //U+E633
0, //U+E634
0, //U+E635
0, //U+E636
0, //U+E637
0, //U+E638
0, //U+E639
0, //U+E63A
0, //U+E63B
0, //U+E63C
0, //U+E63D
0, //U+E63E
0, //U+E63F
0, //U+E640
0, //U+E641
0, //U+E642
0, //U+E643
0, //U+E644
0, //U+E645
0, //U+E646
0, //U+E647
0, //U+E648
0, //U+E649
0, //U+E64A
0, //U+E64B
0, //U+E64C
0, //U+E64D
0, //U+E64E
0, //U+E64F
0, //U+E650
0, //U+E651
0, //U+E652
0, //U+E653
0, //U+E654
0, //U+E655
0, //U+E656
0, //U+E657
0, //U+E658
0, //U+E659
0, //U+E65A
0, //U+E65B
0, //U+E65C
0, //U+E65D
0, //U+E65E
0, //U+E65F
0, //U+E660
0, //U+E661
0, //U+E662
0, //U+E663
0, //U+E664
0, //U+E665
0, //U+E666
0, //U+E667
0, //U+E668
0, //U+E669
0, //U+E66A
0, //U+E66B
0, //U+E66C
0, //U+E66D
0, //U+E66E
0, //U+E66F
0, //U+E670
0, //U+E671
0, //U+E672
0, //U+E673
0, //U+E674
0, //U+E675
0, //U+E676
0, //U+E677
0, //U+E678
0, //U+E679
0, //U+E67A
0, //U+E67B
0, //U+E67C
0, //U+E67D
0, //U+E67E
0, //U+E67F
0, //U+E680
0, //U+E681
0, //U+E682
0, //U+E683
0, //U+E684
0, //U+E685
0, //U+E686
0, //U+E687
0, //U+E688
0, //U+E689
0, //U+E68A
0, //U+E68B
0, //U+E68C
0, //U+E68D
0, //U+E68E
0, //U+E68F
0, //U+E690
0, //U+E691
0, //U+E692
0, //U+E693
0, //U+E694
0, //U+E695
0, //U+E696
0, //U+E697
0, //U+E698
0, //U+E699
0, //U+E69A
0, //U+E69B
0, //U+E69C
0, //U+E69D
0, //U+E69E
0, //U+E69F
0, //U+E6A0
0, //U+E6A1
0, //U+E6A2
0, //U+E6A3
0, //U+E6A4
0, //U+E6A5
0, //U+E6A6
0, //U+E6A7
0, //U+E6A8
0, //U+E6A9
0, //U+E6AA
0, //U+E6AB
0, //U+E6AC
0, //U+E6AD
0, //U+E6AE
0, //U+E6AF
0, //U+E6B0
0, //U+E6B1
0, //U+E6B2
0, //U+E6B3
0, //U+E6B4
0, //U+E6B5
0, //U+E6B6
0, //U+E6B7
0, //U+E6B8
0, //U+E6B9
0, //U+E6BA
0, //U+E6BB
0, //U+E6BC
0, //U+E6BD
0, //U+E6BE
0, //U+E6BF
0, //U+E6C0
0, //U+E6C1
0, //U+E6C2
0, //U+E6C3
0, //U+E6C4
0, //U+E6C5
0, //U+E6C6
0, //U+E6C7
0, //U+E6C8
0, //U+E6C9
0, //U+E6CA
0, //U+E6CB
0, //U+E6CC
0, //U+E6CD
0, //U+E6CE
0, //U+E6CF
0, //U+E6D0
0, //U+E6D1
0, //U+E6D2
0, //U+E6D3
0, //U+E6D4
0, //U+E6D5
0, //U+E6D6
0, //U+E6D7
0, //U+E6D8
0, //U+E6D9
0, //U+E6DA
0, //U+E6DB
0, //U+E6DC
0, //U+E6DD
0, //U+E6DE
0, //U+E6DF
0, //U+E6E0
0, //U+E6E1
0, //U+E6E2
0, //U+E6E3
0, //U+E6E4
0, //U+E6E5
0, //U+E6E6
0, //U+E6E7
0, //U+E6E8
0, //U+E6E9
0, //U+E6EA
0, //U+E6EB
0, //U+E6EC
0, //U+E6ED
0, //U+E6EE
0, //U+E6EF
0, //U+E6F0
0, //U+E6F1
0, //U+E6F2
0, //U+E6F3
0, //U+E6F4
0, //U+E6F5
0, //U+E6F6
0, //U+E6F7
0, //U+E6F8
0, //U+E6F9
0, //U+E6FA
0, //U+E6FB
0, //U+E6FC
0, //U+E6FD
0, //U+E6FE
0, //U+E6FF
0, //U+E700
0, //U+E701
0, //U+E702
0, //U+E703
0, //U+E704
0, //U+E705
0, //U+E706
0, //U+E707
0, //U+E708
0, //U+E709
0, //U+E70A
0, //U+E70B
0, //U+E70C
0, //U+E70D
0, //U+E70E
0, //U+E70F
0, //U+E710
0, //U+E711
0, //U+E712
0, //U+E713
0, //U+E714
0, //U+E715
0, //U+E716
0, //U+E717
0, //U+E718
0, //U+E719
0, //U+E71A
0, //U+E71B
0, //U+E71C
0, //U+E71D
0, //U+E71E
0, //U+E71F
0, //U+E720
0, //U+E721
0, //U+E722
0, //U+E723
0, //U+E724
0, //U+E725
0, //U+E726
0, //U+E727
0, //U+E728
0, //U+E729
0, //U+E72A
0, //U+E72B
0, //U+E72C
0, //U+E72D
0, //U+E72E
0, //U+E72F
0, //U+E730
0, //U+E731
0, //U+E732
0, //U+E733
0, //U+E734
0, //U+E735
0, //U+E736
0, //U+E737
0, //U+E738
0, //U+E739
0, //U+E73A
0, //U+E73B
0, //U+E73C
0, //U+E73D
0, //U+E73E
0, //U+E73F
0, //U+E740
0, //U+E741
0, //U+E742
0, //U+E743
0, //U+E744
0, //U+E745
0, //U+E746
0, //U+E747
0, //U+E748
0, //U+E749
0, //U+E74A
0, //U+E74B
0, //U+E74C
0, //U+E74D
0, //U+E74E
0, //U+E74F
0, //U+E750
0, //U+E751
0, //U+E752
0, //U+E753
0, //U+E754
0, //U+E755
0, //U+E756
0, //U+E757
0, //U+E758
0, //U+E759
0, //U+E75A
0, //U+E75B
0, //U+E75C
0, //U+E75D
0, //U+E75E
0, //U+E75F
0, //U+E760
0, //U+E761
0, //U+E762
0, //U+E763
0, //U+E764
0, //U+E765
0, //U+E766
0, //U+E767
0, //U+E768
0, //U+E769
0, //U+E76A
0, //U+E76B
0, //U+E76C
0, //U+E76D
0, //U+E76E
0, //U+E76F
0, //U+E770
0, //U+E771
0, //U+E772
0, //U+E773
0, //U+E774
0, //U+E775
0, //U+E776
0, //U+E777
0, //U+E778
0, //U+E779
0, //U+E77A
0, //U+E77B
0, //U+E77C
0, //U+E77D
0, //U+E77E
0, //U+E77F
0, //U+E780
0, //U+E781
0, //U+E782
0, //U+E783
0, //U+E784
0, //U+E785
0, //U+E786
0, //U+E787
0, //U+E788
0, //U+E789
0, //U+E78A
0, //U+E78B
0, //U+E78C
0, //U+E78D
0, //U+E78E
0, //U+E78F
0, //U+E790
0, //U+E791
0, //U+E792
0, //U+E793
0, //U+E794
0, //U+E795
0, //U+E796
0, //U+E797
0, //U+E798
0, //U+E799
0, //U+E79A
0, //U+E79B
0, //U+E79C
0, //U+E79D
0, //U+E79E
0, //U+E79F
0, //U+E7A0
0, //U+E7A1
0, //U+E7A2
0, //U+E7A3
0, //U+E7A4
0, //U+E7A5
0, //U+E7A6
0, //U+E7A7
0, //U+E7A8
0, //U+E7A9
0, //U+E7AA
0, //U+E7AB
0, //U+E7AC
0, //U+E7AD
0, //U+E7AE
0, //U+E7AF
0, //U+E7B0
0, //U+E7B1
0, //U+E7B2
0, //U+E7B3
0, //U+E7B4
0, //U+E7B5
0, //U+E7B6
0, //U+E7B7
0, //U+E7B8
0, //U+E7B9
0, //U+E7BA
0, //U+E7BB
0, //U+E7BC
0, //U+E7BD
0, //U+E7BE
0, //U+E7BF
0, //U+E7C0
0, //U+E7C1
0, //U+E7C2
0, //U+E7C3
0, //U+E7C4
0, //U+E7C5
0, //U+E7C6
0, //U+E7C7
0, //U+E7C8
0, //U+E7C9
0, //U+E7CA
0, //U+E7CB
0, //U+E7CC
0, //U+E7CD
0, //U+E7CE
0, //U+E7CF
0, //U+E7D0
0, //U+E7D1
0, //U+E7D2
0, //U+E7D3
0, //U+E7D4
0, //U+E7D5
0, //U+E7D6
0, //U+E7D7
0, //U+E7D8
0, //U+E7D9
0, //U+E7DA
0, //U+E7DB
0, //U+E7DC
0, //U+E7DD
0, //U+E7DE
0, //U+E7DF
0, //U+E7E0
0, //U+E7E1
0, //U+E7E2
0, //U+E7E3
0, //U+E7E4
0, //U+E7E5
0, //U+E7E6
0, //U+E7E7
0, //U+E7E8
0, //U+E7E9
0, //U+E7EA
0, //U+E7EB
0, //U+E7EC
0, //U+E7ED
0, //U+E7EE
0, //U+E7EF
0, //U+E7F0
0, //U+E7F1
0, //U+E7F2
0, //U+E7F3
0, //U+E7F4
0, //U+E7F5
0, //U+E7F6
0, //U+E7F7
0, //U+E7F8
0, //U+E7F9
0, //U+E7FA
0, //U+E7FB
0, //U+E7FC
0, //U+E7FD
0, //U+E7FE
0, //U+E7FF
0, //U+E800
0, //U+E801
0, //U+E802
0, //U+E803
0, //U+E804
0, //U+E805
0, //U+E806
0, //U+E807
0, //U+E808
0, //U+E809
0, //U+E80A
0, //U+E80B
0, //U+E80C
0, //U+E80D
0, //U+E80E
0, //U+E80F
0, //U+E810
0, //U+E811
0, //U+E812
0, //U+E813
0, //U+E814
0, //U+E815
0, //U+E816
0, //U+E817
0, //U+E818
0, //U+E819
0, //U+E81A
0, //U+E81B
0, //U+E81C
0, //U+E81D
0, //U+E81E
0, //U+E81F
0, //U+E820
0, //U+E821
0, //U+E822
0, //U+E823
0, //U+E824
0, //U+E825
0, //U+E826
0, //U+E827
0, //U+E828
0, //U+E829
0, //U+E82A
0, //U+E82B
0, //U+E82C
0, //U+E82D
0, //U+E82E
0, //U+E82F
0, //U+E830
0, //U+E831
0, //U+E832
0, //U+E833
0, //U+E834
0, //U+E835
0, //U+E836
0, //U+E837
0, //U+E838
0, //U+E839
0, //U+E83A
0, //U+E83B
0, //U+E83C
0, //U+E83D
0, //U+E83E
0, //U+E83F
0, //U+E840
0, //U+E841
0, //U+E842
0, //U+E843
0, //U+E844
0, //U+E845
0, //U+E846
0, //U+E847
0, //U+E848
0, //U+E849
0, //U+E84A
0, //U+E84B
0, //U+E84C
0, //U+E84D
0, //U+E84E
0, //U+E84F
0, //U+E850
0, //U+E851
0, //U+E852
0, //U+E853
0, //U+E854
0, //U+E855
0, //U+E856
0, //U+E857
0, //U+E858
0, //U+E859
0, //U+E85A
0, //U+E85B
0, //U+E85C
0, //U+E85D
0, //U+E85E
0, //U+E85F
0, //U+E860
0, //U+E861
0, //U+E862
0, //U+E863
0, //U+E864
0, //U+E865
0, //U+E866
0, //U+E867
0, //U+E868
0, //U+E869
0, //U+E86A
0, //U+E86B
0, //U+E86C
0, //U+E86D
0, //U+E86E
0, //U+E86F
0, //U+E870
0, //U+E871
0, //U+E872
0, //U+E873
0, //U+E874
0, //U+E875
0, //U+E876
0, //U+E877
0, //U+E878
0, //U+E879
0, //U+E87A
0, //U+E87B
0, //U+E87C
0, //U+E87D
0, //U+E87E
0, //U+E87F
0, //U+E880
0, //U+E881
0, //U+E882
0, //U+E883
0, //U+E884
0, //U+E885
0, //U+E886
0, //U+E887
0, //U+E888
0, //U+E889
0, //U+E88A
0, //U+E88B
0, //U+E88C
0, //U+E88D
0, //U+E88E
0, //U+E88F
0, //U+E890
0, //U+E891
0, //U+E892
0, //U+E893
0, //U+E894
0, //U+E895
0, //U+E896
0, //U+E897
0, //U+E898
0, //U+E899
0, //U+E89A
0, //U+E89B
0, //U+E89C
0, //U+E89D
0, //U+E89E
0, //U+E89F
0, //U+E8A0
0, //U+E8A1
0, //U+E8A2
0, //U+E8A3
0, //U+E8A4
0, //U+E8A5
0, //U+E8A6
0, //U+E8A7
0, //U+E8A8
0, //U+E8A9
0, //U+E8AA
0, //U+E8AB
0, //U+E8AC
0, //U+E8AD
0, //U+E8AE
0, //U+E8AF
0, //U+E8B0
0, //U+E8B1
0, //U+E8B2
0, //U+E8B3
0, //U+E8B4
0, //U+E8B5
0, //U+E8B6
0, //U+E8B7
0, //U+E8B8
0, //U+E8B9
0, //U+E8BA
0, //U+E8BB
0, //U+E8BC
0, //U+E8BD
0, //U+E8BE
0, //U+E8BF
0, //U+E8C0
0, //U+E8C1
0, //U+E8C2
0, //U+E8C3
0, //U+E8C4
0, //U+E8C5
0, //U+E8C6
0, //U+E8C7
0, //U+E8C8
0, //U+E8C9
0, //U+E8CA
0, //U+E8CB
0, //U+E8CC
0, //U+E8CD
0, //U+E8CE
0, //U+E8CF
0, //U+E8D0
0, //U+E8D1
0, //U+E8D2
0, //U+E8D3
0, //U+E8D4
0, //U+E8D5
0, //U+E8D6
0, //U+E8D7
0, //U+E8D8
0, //U+E8D9
0, //U+E8DA
0, //U+E8DB
0, //U+E8DC
0, //U+E8DD
0, //U+E8DE
0, //U+E8DF
0, //U+E8E0
0, //U+E8E1
0, //U+E8E2
0, //U+E8E3
0, //U+E8E4
0, //U+E8E5
0, //U+E8E6
0, //U+E8E7
0, //U+E8E8
0, //U+E8E9
0, //U+E8EA
0, //U+E8EB
0, //U+E8EC
0, //U+E8ED
0, //U+E8EE
0, //U+E8EF
0, //U+E8F0
0, //U+E8F1
0, //U+E8F2
0, //U+E8F3
0, //U+E8F4
0, //U+E8F5
0, //U+E8F6
0, //U+E8F7
0, //U+E8F8
0, //U+E8F9
0, //U+E8FA
0, //U+E8FB
0, //U+E8FC
0, //U+E8FD
0, //U+E8FE
0, //U+E8FF
0, //U+E900
0, //U+E901
0, //U+E902
0, //U+E903
0, //U+E904
0, //U+E905
0, //U+E906
0, //U+E907
0, //U+E908
0, //U+E909
0, //U+E90A
0, //U+E90B
0, //U+E90C
0, //U+E90D
0, //U+E90E
0, //U+E90F
0, //U+E910
0, //U+E911
0, //U+E912
0, //U+E913
0, //U+E914
0, //U+E915
0, //U+E916
0, //U+E917
0, //U+E918
0, //U+E919
0, //U+E91A
0, //U+E91B
0, //U+E91C
0, //U+E91D
0, //U+E91E
0, //U+E91F
0, //U+E920
0, //U+E921
0, //U+E922
0, //U+E923
0, //U+E924
0, //U+E925
0, //U+E926
0, //U+E927
0, //U+E928
0, //U+E929
0, //U+E92A
0, //U+E92B
0, //U+E92C
0, //U+E92D
0, //U+E92E
0, //U+E92F
0, //U+E930
0, //U+E931
0, //U+E932
0, //U+E933
0, //U+E934
0, //U+E935
0, //U+E936
0, //U+E937
0, //U+E938
0, //U+E939
0, //U+E93A
0, //U+E93B
0, //U+E93C
0, //U+E93D
0, //U+E93E
0, //U+E93F
0, //U+E940
0, //U+E941
0, //U+E942
0, //U+E943
0, //U+E944
0, //U+E945
0, //U+E946
0, //U+E947
0, //U+E948
0, //U+E949
0, //U+E94A
0, //U+E94B
0, //U+E94C
0, //U+E94D
0, //U+E94E
0, //U+E94F
0, //U+E950
0, //U+E951
0, //U+E952
0, //U+E953
0, //U+E954
0, //U+E955
0, //U+E956
0, //U+E957
0, //U+E958
0, //U+E959
0, //U+E95A
0, //U+E95B
0, //U+E95C
0, //U+E95D
0, //U+E95E
0, //U+E95F
0, //U+E960
0, //U+E961
0, //U+E962
0, //U+E963
0, //U+E964
0, //U+E965
0, //U+E966
0, //U+E967
0, //U+E968
0, //U+E969
0, //U+E96A
0, //U+E96B
0, //U+E96C
0, //U+E96D
0, //U+E96E
0, //U+E96F
0, //U+E970
0, //U+E971
0, //U+E972
0, //U+E973
0, //U+E974
0, //U+E975
0, //U+E976
0, //U+E977
0, //U+E978
0, //U+E979
0, //U+E97A
0, //U+E97B
0, //U+E97C
0, //U+E97D
0, //U+E97E
0, //U+E97F
0, //U+E980
0, //U+E981
0, //U+E982
0, //U+E983
0, //U+E984
0, //U+E985
0, //U+E986
0, //U+E987
0, //U+E988
0, //U+E989
0, //U+E98A
0, //U+E98B
0, //U+E98C
0, //U+E98D
0, //U+E98E
0, //U+E98F
0, //U+E990
0, //U+E991
0, //U+E992
0, //U+E993
0, //U+E994
0, //U+E995
0, //U+E996
0, //U+E997
0, //U+E998
0, //U+E999
0, //U+E99A
0, //U+E99B
0, //U+E99C
0, //U+E99D
0, //U+E99E
0, //U+E99F
0, //U+E9A0
0, //U+E9A1
0, //U+E9A2
0, //U+E9A3
0, //U+E9A4
0, //U+E9A5
0, //U+E9A6
0, //U+E9A7
0, //U+E9A8
0, //U+E9A9
0, //U+E9AA
0, //U+E9AB
0, //U+E9AC
0, //U+E9AD
0, //U+E9AE
0, //U+E9AF
0, //U+E9B0
0, //U+E9B1
0, //U+E9B2
0, //U+E9B3
0, //U+E9B4
0, //U+E9B5
0, //U+E9B6
0, //U+E9B7
0, //U+E9B8
0, //U+E9B9
0, //U+E9BA
0, //U+E9BB
0, //U+E9BC
0, //U+E9BD
0, //U+E9BE
0, //U+E9BF
0, //U+E9C0
0, //U+E9C1
0, //U+E9C2
0, //U+E9C3
0, //U+E9C4
0, //U+E9C5
0, //U+E9C6
0, //U+E9C7
0, //U+E9C8
0, //U+E9C9
0, //U+E9CA
0, //U+E9CB
0, //U+E9CC
0, //U+E9CD
0, //U+E9CE
0, //U+E9CF
0, //U+E9D0
0, //U+E9D1
0, //U+E9D2
0, //U+E9D3
0, //U+E9D4
0, //U+E9D5
0, //U+E9D6
0, //U+E9D7
0, //U+E9D8
0, //U+E9D9
0, //U+E9DA
0, //U+E9DB
0, //U+E9DC
0, //U+E9DD
0, //U+E9DE
0, //U+E9DF
0, //U+E9E0
0, //U+E9E1
0, //U+E9E2
0, //U+E9E3
0, //U+E9E4
0, //U+E9E5
0, //U+E9E6
0, //U+E9E7
0, //U+E9E8
0, //U+E9E9
0, //U+E9EA
0, //U+E9EB
0, //U+E9EC
0, //U+E9ED
0, //U+E9EE
0, //U+E9EF
0, //U+E9F0
0, //U+E9F1
0, //U+E9F2
0, //U+E9F3
0, //U+E9F4
0, //U+E9F5
0, //U+E9F6
0, //U+E9F7
0, //U+E9F8
0, //U+E9F9
0, //U+E9FA
0, //U+E9FB
0, //U+E9FC
0, //U+E9FD
0, //U+E9FE
0, //U+E9FF
0, //U+EA00
0, //U+EA01
0, //U+EA02
0, //U+EA03
0, //U+EA04
0, //U+EA05
0, //U+EA06
0, //U+EA07
0, //U+EA08
0, //U+EA09
0, //U+EA0A
0, //U+EA0B
0, //U+EA0C
0, //U+EA0D
0, //U+EA0E
0, //U+EA0F
0, //U+EA10
0, //U+EA11
0, //U+EA12
0, //U+EA13
0, //U+EA14
0, //U+EA15
0, //U+EA16
0, //U+EA17
0, //U+EA18
0, //U+EA19
0, //U+EA1A
0, //U+EA1B
0, //U+EA1C
0, //U+EA1D
0, //U+EA1E
0, //U+EA1F
0, //U+EA20
0, //U+EA21
0, //U+EA22
0, //U+EA23
0, //U+EA24
0, //U+EA25
0, //U+EA26
0, //U+EA27
0, //U+EA28
0, //U+EA29
0, //U+EA2A
0, //U+EA2B
0, //U+EA2C
0, //U+EA2D
0, //U+EA2E
0, //U+EA2F
0, //U+EA30
0, //U+EA31
0, //U+EA32
0, //U+EA33
0, //U+EA34
0, //U+EA35
0, //U+EA36
0, //U+EA37
0, //U+EA38
0, //U+EA39
0, //U+EA3A
0, //U+EA3B
0, //U+EA3C
0, //U+EA3D
0, //U+EA3E
0, //U+EA3F
0, //U+EA40
0, //U+EA41
0, //U+EA42
0, //U+EA43
0, //U+EA44
0, //U+EA45
0, //U+EA46
0, //U+EA47
0, //U+EA48
0, //U+EA49
0, //U+EA4A
0, //U+EA4B
0, //U+EA4C
0, //U+EA4D
0, //U+EA4E
0, //U+EA4F
0, //U+EA50
0, //U+EA51
0, //U+EA52
0, //U+EA53
0, //U+EA54
0, //U+EA55
0, //U+EA56
0, //U+EA57
0, //U+EA58
0, //U+EA59
0, //U+EA5A
0, //U+EA5B
0, //U+EA5C
0, //U+EA5D
0, //U+EA5E
0, //U+EA5F
0, //U+EA60
0, //U+EA61
0, //U+EA62
0, //U+EA63
0, //U+EA64
0, //U+EA65
0, //U+EA66
0, //U+EA67
0, //U+EA68
0, //U+EA69
0, //U+EA6A
0, //U+EA6B
0, //U+EA6C
0, //U+EA6D
0, //U+EA6E
0, //U+EA6F
0, //U+EA70
0, //U+EA71
0, //U+EA72
0, //U+EA73
0, //U+EA74
0, //U+EA75
0, //U+EA76
0, //U+EA77
0, //U+EA78
0, //U+EA79
0, //U+EA7A
0, //U+EA7B
0, //U+EA7C
0, //U+EA7D
0, //U+EA7E
0, //U+EA7F
0, //U+EA80
0, //U+EA81
0, //U+EA82
0, //U+EA83
0, //U+EA84
0, //U+EA85
0, //U+EA86
0, //U+EA87
0, //U+EA88
0, //U+EA89
0, //U+EA8A
0, //U+EA8B
0, //U+EA8C
0, //U+EA8D
0, //U+EA8E
0, //U+EA8F
0, //U+EA90
0, //U+EA91
0, //U+EA92
0, //U+EA93
0, //U+EA94
0, //U+EA95
0, //U+EA96
0, //U+EA97
0, //U+EA98
0, //U+EA99
0, //U+EA9A
0, //U+EA9B
0, //U+EA9C
0, //U+EA9D
0, //U+EA9E
0, //U+EA9F
0, //U+EAA0
0, //U+EAA1
0, //U+EAA2
0, //U+EAA3
0, //U+EAA4
0, //U+EAA5
0, //U+EAA6
0, //U+EAA7
0, //U+EAA8
0, //U+EAA9
0, //U+EAAA
0, //U+EAAB
0, //U+EAAC
0, //U+EAAD
0, //U+EAAE
0, //U+EAAF
0, //U+EAB0
0, //U+EAB1
0, //U+EAB2
0, //U+EAB3
0, //U+EAB4
0, //U+EAB5
0, //U+EAB6
0, //U+EAB7
0, //U+EAB8
0, //U+EAB9
0, //U+EABA
0, //U+EABB
0, //U+EABC
0, //U+EABD
0, //U+EABE
0, //U+EABF
0, //U+EAC0
0, //U+EAC1
0, //U+EAC2
0, //U+EAC3
0, //U+EAC4
0, //U+EAC5
0, //U+EAC6
0, //U+EAC7
0, //U+EAC8
0, //U+EAC9
0, //U+EACA
0, //U+EACB
0, //U+EACC
0, //U+EACD
0, //U+EACE
0, //U+EACF
0, //U+EAD0
0, //U+EAD1
0, //U+EAD2
0, //U+EAD3
0, //U+EAD4
0, //U+EAD5
0, //U+EAD6
0, //U+EAD7
0, //U+EAD8
0, //U+EAD9
0, //U+EADA
0, //U+EADB
0, //U+EADC
0, //U+EADD
0, //U+EADE
0, //U+EADF
0, //U+EAE0
0, //U+EAE1
0, //U+EAE2
0, //U+EAE3
0, //U+EAE4
0, //U+EAE5
0, //U+EAE6
0, //U+EAE7
0, //U+EAE8
0, //U+EAE9
0, //U+EAEA
0, //U+EAEB
0, //U+EAEC
0, //U+EAED
0, //U+EAEE
0, //U+EAEF
0, //U+EAF0
0, //U+EAF1
0, //U+EAF2
0, //U+EAF3
0, //U+EAF4
0, //U+EAF5
0, //U+EAF6
0, //U+EAF7
0, //U+EAF8
0, //U+EAF9
0, //U+EAFA
0, //U+EAFB
0, //U+EAFC
0, //U+EAFD
0, //U+EAFE
0, //U+EAFF
0, //U+EB00
0, //U+EB01
0, //U+EB02
0, //U+EB03
0, //U+EB04
0, //U+EB05
0, //U+EB06
0, //U+EB07
0, //U+EB08
0, //U+EB09
0, //U+EB0A
0, //U+EB0B
0, //U+EB0C
0, //U+EB0D
0, //U+EB0E
0, //U+EB0F
0, //U+EB10
0, //U+EB11
0, //U+EB12
0, //U+EB13
0, //U+EB14
0, //U+EB15
0, //U+EB16
0, //U+EB17
0, //U+EB18
0, //U+EB19
0, //U+EB1A
0, //U+EB1B
0, //U+EB1C
0, //U+EB1D
0, //U+EB1E
0, //U+EB1F
0, //U+EB20
0, //U+EB21
0, //U+EB22
0, //U+EB23
0, //U+EB24
0, //U+EB25
0, //U+EB26
0, //U+EB27
0, //U+EB28
0, //U+EB29
0, //U+EB2A
0, //U+EB2B
0, //U+EB2C
0, //U+EB2D
0, //U+EB2E
0, //U+EB2F
0, //U+EB30
0, //U+EB31
0, //U+EB32
0, //U+EB33
0, //U+EB34
0, //U+EB35
0, //U+EB36
0, //U+EB37
0, //U+EB38
0, //U+EB39
0, //U+EB3A
0, //U+EB3B
0, //U+EB3C
0, //U+EB3D
0, //U+EB3E
0, //U+EB3F
0, //U+EB40
0, //U+EB41
0, //U+EB42
0, //U+EB43
0, //U+EB44
0, //U+EB45
0, //U+EB46
0, //U+EB47
0, //U+EB48
0, //U+EB49
0, //U+EB4A
0, //U+EB4B
0, //U+EB4C
0, //U+EB4D
0, //U+EB4E
0, //U+EB4F
0, //U+EB50
0, //U+EB51
0, //U+EB52
0, //U+EB53
0, //U+EB54
0, //U+EB55
0, //U+EB56
0, //U+EB57
0, //U+EB58
0, //U+EB59
0, //U+EB5A
0, //U+EB5B
0, //U+EB5C
0, //U+EB5D
0, //U+EB5E
0, //U+EB5F
0, //U+EB60
0, //U+EB61
0, //U+EB62
0, //U+EB63
0, //U+EB64
0, //U+EB65
0, //U+EB66
0, //U+EB67
0, //U+EB68
0, //U+EB69
0, //U+EB6A
0, //U+EB6B
0, //U+EB6C
0, //U+EB6D
0, //U+EB6E
0, //U+EB6F
0, //U+EB70
0, //U+EB71
0, //U+EB72
0, //U+EB73
0, //U+EB74
0, //U+EB75
0, //U+EB76
0, //U+EB77
0, //U+EB78
0, //U+EB79
0, //U+EB7A
0, //U+EB7B
0, //U+EB7C
0, //U+EB7D
0, //U+EB7E
0, //U+EB7F
0, //U+EB80
0, //U+EB81
0, //U+EB82
0, //U+EB83
0, //U+EB84
0, //U+EB85
0, //U+EB86
0, //U+EB87
0, //U+EB88
0, //U+EB89
0, //U+EB8A
0, //U+EB8B
0, //U+EB8C
0, //U+EB8D
0, //U+EB8E
0, //U+EB8F
0, //U+EB90
0, //U+EB91
0, //U+EB92
0, //U+EB93
0, //U+EB94
0, //U+EB95
0, //U+EB96
0, //U+EB97
0, //U+EB98
0, //U+EB99
0, //U+EB9A
0, //U+EB9B
0, //U+EB9C
0, //U+EB9D
0, //U+EB9E
0, //U+EB9F
0, //U+EBA0
0, //U+EBA1
0, //U+EBA2
0, //U+EBA3
0, //U+EBA4
0, //U+EBA5
0, //U+EBA6
0, //U+EBA7
0, //U+EBA8
0, //U+EBA9
0, //U+EBAA
0, //U+EBAB
0, //U+EBAC
0, //U+EBAD
0, //U+EBAE
0, //U+EBAF
0, //U+EBB0
0, //U+EBB1
0, //U+EBB2
0, //U+EBB3
0, //U+EBB4
0, //U+EBB5
0, //U+EBB6
0, //U+EBB7
0, //U+EBB8
0, //U+EBB9
0, //U+EBBA
0, //U+EBBB
0, //U+EBBC
0, //U+EBBD
0, //U+EBBE
0, //U+EBBF
0, //U+EBC0
0, //U+EBC1
0, //U+EBC2
0, //U+EBC3
0, //U+EBC4
0, //U+EBC5
0, //U+EBC6
0, //U+EBC7
0, //U+EBC8
0, //U+EBC9
0, //U+EBCA
0, //U+EBCB
0, //U+EBCC
0, //U+EBCD
0, //U+EBCE
0, //U+EBCF
0, //U+EBD0
0, //U+EBD1
0, //U+EBD2
0, //U+EBD3
0, //U+EBD4
0, //U+EBD5
0, //U+EBD6
0, //U+EBD7
0, //U+EBD8
0, //U+EBD9
0, //U+EBDA
0, //U+EBDB
0, //U+EBDC
0, //U+EBDD
0, //U+EBDE
0, //U+EBDF
0, //U+EBE0
0, //U+EBE1
0, //U+EBE2
0, //U+EBE3
0, //U+EBE4
0, //U+EBE5
0, //U+EBE6
0, //U+EBE7
0, //U+EBE8
0, //U+EBE9
0, //U+EBEA
0, //U+EBEB
0, //U+EBEC
0, //U+EBED
0, //U+EBEE
0, //U+EBEF
0, //U+EBF0
0, //U+EBF1
0, //U+EBF2
0, //U+EBF3
0, //U+EBF4
0, //U+EBF5
0, //U+EBF6
0, //U+EBF7
0, //U+EBF8
0, //U+EBF9
0, //U+EBFA
0, //U+EBFB
0, //U+EBFC
0, //U+EBFD
0, //U+EBFE
0, //U+EBFF
0, //U+EC00
0, //U+EC01
0, //U+EC02
0, //U+EC03
0, //U+EC04
0, //U+EC05
0, //U+EC06
0, //U+EC07
0, //U+EC08
0, //U+EC09
0, //U+EC0A
0, //U+EC0B
0, //U+EC0C
0, //U+EC0D
0, //U+EC0E
0, //U+EC0F
0, //U+EC10
0, //U+EC11
0, //U+EC12
0, //U+EC13
0, //U+EC14
0, //U+EC15
0, //U+EC16
0, //U+EC17
0, //U+EC18
0, //U+EC19
0, //U+EC1A
0, //U+EC1B
0, //U+EC1C
0, //U+EC1D
0, //U+EC1E
0, //U+EC1F
0, //U+EC20
0, //U+EC21
0, //U+EC22
0, //U+EC23
0, //U+EC24
0, //U+EC25
0, //U+EC26
0, //U+EC27
0, //U+EC28
0, //U+EC29
0, //U+EC2A
0, //U+EC2B
0, //U+EC2C
0, //U+EC2D
0, //U+EC2E
0, //U+EC2F
0, //U+EC30
0, //U+EC31
0, //U+EC32
0, //U+EC33
0, //U+EC34
0, //U+EC35
0, //U+EC36
0, //U+EC37
0, //U+EC38
0, //U+EC39
0, //U+EC3A
0, //U+EC3B
0, //U+EC3C
0, //U+EC3D
0, //U+EC3E
0, //U+EC3F
0, //U+EC40
0, //U+EC41
0, //U+EC42
0, //U+EC43
0, //U+EC44
0, //U+EC45
0, //U+EC46
0, //U+EC47
0, //U+EC48
0, //U+EC49
0, //U+EC4A
0, //U+EC4B
0, //U+EC4C
0, //U+EC4D
0, //U+EC4E
0, //U+EC4F
0, //U+EC50
0, //U+EC51
0, //U+EC52
0, //U+EC53
0, //U+EC54
0, //U+EC55
0, //U+EC56
0, //U+EC57
0, //U+EC58
0, //U+EC59
0, //U+EC5A
0, //U+EC5B
0, //U+EC5C
0, //U+EC5D
0, //U+EC5E
0, //U+EC5F
0, //U+EC60
0, //U+EC61
0, //U+EC62
0, //U+EC63
0, //U+EC64
0, //U+EC65
0, //U+EC66
0, //U+EC67
0, //U+EC68
0, //U+EC69
0, //U+EC6A
0, //U+EC6B
0, //U+EC6C
0, //U+EC6D
0, //U+EC6E
0, //U+EC6F
0, //U+EC70
0, //U+EC71
0, //U+EC72
0, //U+EC73
0, //U+EC74
0, //U+EC75
0, //U+EC76
0, //U+EC77
0, //U+EC78
0, //U+EC79
0, //U+EC7A
0, //U+EC7B
0, //U+EC7C
0, //U+EC7D
0, //U+EC7E
0, //U+EC7F
0, //U+EC80
0, //U+EC81
0, //U+EC82
0, //U+EC83
0, //U+EC84
0, //U+EC85
0, //U+EC86
0, //U+EC87
0, //U+EC88
0, //U+EC89
0, //U+EC8A
0, //U+EC8B
0, //U+EC8C
0, //U+EC8D
0, //U+EC8E
0, //U+EC8F
0, //U+EC90
0, //U+EC91
0, //U+EC92
0, //U+EC93
0, //U+EC94
0, //U+EC95
0, //U+EC96
0, //U+EC97
0, //U+EC98
0, //U+EC99
0, //U+EC9A
0, //U+EC9B
0, //U+EC9C
0, //U+EC9D
0, //U+EC9E
0, //U+EC9F
0, //U+ECA0
0, //U+ECA1
0, //U+ECA2
0, //U+ECA3
0, //U+ECA4
0, //U+ECA5
0, //U+ECA6
0, //U+ECA7
0, //U+ECA8
0, //U+ECA9
0, //U+ECAA
0, //U+ECAB
0, //U+ECAC
0, //U+ECAD
0, //U+ECAE
0, //U+ECAF
0, //U+ECB0
0, //U+ECB1
0, //U+ECB2
0, //U+ECB3
0, //U+ECB4
0, //U+ECB5
0, //U+ECB6
0, //U+ECB7
0, //U+ECB8
0, //U+ECB9
0, //U+ECBA
0, //U+ECBB
0, //U+ECBC
0, //U+ECBD
0, //U+ECBE
0, //U+ECBF
0, //U+ECC0
0, //U+ECC1
0, //U+ECC2
0, //U+ECC3
0, //U+ECC4
0, //U+ECC5
0, //U+ECC6
0, //U+ECC7
0, //U+ECC8
0, //U+ECC9
0, //U+ECCA
0, //U+ECCB
0, //U+ECCC
0, //U+ECCD
0, //U+ECCE
0, //U+ECCF
0, //U+ECD0
0, //U+ECD1
0, //U+ECD2
0, //U+ECD3
0, //U+ECD4
0, //U+ECD5
0, //U+ECD6
0, //U+ECD7
0, //U+ECD8
0, //U+ECD9
0, //U+ECDA
0, //U+ECDB
0, //U+ECDC
0, //U+ECDD
0, //U+ECDE
0, //U+ECDF
0, //U+ECE0
0, //U+ECE1
0, //U+ECE2
0, //U+ECE3
0, //U+ECE4
0, //U+ECE5
0, //U+ECE6
0, //U+ECE7
0, //U+ECE8
0, //U+ECE9
0, //U+ECEA
0, //U+ECEB
0, //U+ECEC
0, //U+ECED
0, //U+ECEE
0, //U+ECEF
0, //U+ECF0
0, //U+ECF1
0, //U+ECF2
0, //U+ECF3
0, //U+ECF4
0, //U+ECF5
0, //U+ECF6
0, //U+ECF7
0, //U+ECF8
0, //U+ECF9
0, //U+ECFA
0, //U+ECFB
0, //U+ECFC
0, //U+ECFD
0, //U+ECFE
0, //U+ECFF
0, //U+ED00
0, //U+ED01
0, //U+ED02
0, //U+ED03
0, //U+ED04
0, //U+ED05
0, //U+ED06
0, //U+ED07
0, //U+ED08
0, //U+ED09
0, //U+ED0A
0, //U+ED0B
0, //U+ED0C
0, //U+ED0D
0, //U+ED0E
0, //U+ED0F
0, //U+ED10
0, //U+ED11
0, //U+ED12
0, //U+ED13
0, //U+ED14
0, //U+ED15
0, //U+ED16
0, //U+ED17
0, //U+ED18
0, //U+ED19
0, //U+ED1A
0, //U+ED1B
0, //U+ED1C
0, //U+ED1D
0, //U+ED1E
0, //U+ED1F
0, //U+ED20
0, //U+ED21
0, //U+ED22
0, //U+ED23
0, //U+ED24
0, //U+ED25
0, //U+ED26
0, //U+ED27
0, //U+ED28
0, //U+ED29
0, //U+ED2A
0, //U+ED2B
0, //U+ED2C
0, //U+ED2D
0, //U+ED2E
0, //U+ED2F
0, //U+ED30
0, //U+ED31
0, //U+ED32
0, //U+ED33
0, //U+ED34
0, //U+ED35
0, //U+ED36
0, //U+ED37
0, //U+ED38
0, //U+ED39
0, //U+ED3A
0, //U+ED3B
0, //U+ED3C
0, //U+ED3D
0, //U+ED3E
0, //U+ED3F
0, //U+ED40
0, //U+ED41
0, //U+ED42
0, //U+ED43
0, //U+ED44
0, //U+ED45
0, //U+ED46
0, //U+ED47
0, //U+ED48
0, //U+ED49
0, //U+ED4A
0, //U+ED4B
0, //U+ED4C
0, //U+ED4D
0, //U+ED4E
0, //U+ED4F
0, //U+ED50
0, //U+ED51
0, //U+ED52
0, //U+ED53
0, //U+ED54
0, //U+ED55
0, //U+ED56
0, //U+ED57
0, //U+ED58
0, //U+ED59
0, //U+ED5A
0, //U+ED5B
0, //U+ED5C
0, //U+ED5D
0, //U+ED5E
0, //U+ED5F
0, //U+ED60
0, //U+ED61
0, //U+ED62
0, //U+ED63
0, //U+ED64
0, //U+ED65
0, //U+ED66
0, //U+ED67
0, //U+ED68
0, //U+ED69
0, //U+ED6A
0, //U+ED6B
0, //U+ED6C
0, //U+ED6D
0, //U+ED6E
0, //U+ED6F
0, //U+ED70
0, //U+ED71
0, //U+ED72
0, //U+ED73
0, //U+ED74
0, //U+ED75
0, //U+ED76
0, //U+ED77
0, //U+ED78
0, //U+ED79
0, //U+ED7A
0, //U+ED7B
0, //U+ED7C
0, //U+ED7D
0, //U+ED7E
0, //U+ED7F
0, //U+ED80
0, //U+ED81
0, //U+ED82
0, //U+ED83
0, //U+ED84
0, //U+ED85
0, //U+ED86
0, //U+ED87
0, //U+ED88
0, //U+ED89
0, //U+ED8A
0, //U+ED8B
0, //U+ED8C
0, //U+ED8D
0, //U+ED8E
0, //U+ED8F
0, //U+ED90
0, //U+ED91
0, //U+ED92
0, //U+ED93
0, //U+ED94
0, //U+ED95
0, //U+ED96
0, //U+ED97
0, //U+ED98
0, //U+ED99
0, //U+ED9A
0, //U+ED9B
0, //U+ED9C
0, //U+ED9D
0, //U+ED9E
0, //U+ED9F
0, //U+EDA0
0, //U+EDA1
0, //U+EDA2
0, //U+EDA3
0, //U+EDA4
0, //U+EDA5
0, //U+EDA6
0, //U+EDA7
0, //U+EDA8
0, //U+EDA9
0, //U+EDAA
0, //U+EDAB
0, //U+EDAC
0, //U+EDAD
0, //U+EDAE
0, //U+EDAF
0, //U+EDB0
0, //U+EDB1
0, //U+EDB2
0, //U+EDB3
0, //U+EDB4
0, //U+EDB5
0, //U+EDB6
0, //U+EDB7
0, //U+EDB8
0, //U+EDB9
0, //U+EDBA
0, //U+EDBB
0, //U+EDBC
0, //U+EDBD
0, //U+EDBE
0, //U+EDBF
0, //U+EDC0
0, //U+EDC1
0, //U+EDC2
0, //U+EDC3
0, //U+EDC4
0, //U+EDC5
0, //U+EDC6
0, //U+EDC7
0, //U+EDC8
0, //U+EDC9
0, //U+EDCA
0, //U+EDCB
0, //U+EDCC
0, //U+EDCD
0, //U+EDCE
0, //U+EDCF
0, //U+EDD0
0, //U+EDD1
0, //U+EDD2
0, //U+EDD3
0, //U+EDD4
0, //U+EDD5
0, //U+EDD6
0, //U+EDD7
0, //U+EDD8
0, //U+EDD9
0, //U+EDDA
0, //U+EDDB
0, //U+EDDC
0, //U+EDDD
0, //U+EDDE
0, //U+EDDF
0, //U+EDE0
0, //U+EDE1
0, //U+EDE2
0, //U+EDE3
0, //U+EDE4
0, //U+EDE5
0, //U+EDE6
0, //U+EDE7
0, //U+EDE8
0, //U+EDE9
0, //U+EDEA
0, //U+EDEB
0, //U+EDEC
0, //U+EDED
0, //U+EDEE
0, //U+EDEF
0, //U+EDF0
0, //U+EDF1
0, //U+EDF2
0, //U+EDF3
0, //U+EDF4
0, //U+EDF5
0, //U+EDF6
0, //U+EDF7
0, //U+EDF8
0, //U+EDF9
0, //U+EDFA
0, //U+EDFB
0, //U+EDFC
0, //U+EDFD
0, //U+EDFE
0, //U+EDFF
0, //U+EE00
0, //U+EE01
0, //U+EE02
0, //U+EE03
0, //U+EE04
0, //U+EE05
0, //U+EE06
0, //U+EE07
0, //U+EE08
0, //U+EE09
0, //U+EE0A
0, //U+EE0B
0, //U+EE0C
0, //U+EE0D
0, //U+EE0E
0, //U+EE0F
0, //U+EE10
0, //U+EE11
0, //U+EE12
0, //U+EE13
0, //U+EE14
0, //U+EE15
0, //U+EE16
0, //U+EE17
0, //U+EE18
0, //U+EE19
0, //U+EE1A
0, //U+EE1B
0, //U+EE1C
0, //U+EE1D
0, //U+EE1E
0, //U+EE1F
0, //U+EE20
0, //U+EE21
0, //U+EE22
0, //U+EE23
0, //U+EE24
0, //U+EE25
0, //U+EE26
0, //U+EE27
0, //U+EE28
0, //U+EE29
0, //U+EE2A
0, //U+EE2B
0, //U+EE2C
0, //U+EE2D
0, //U+EE2E
0, //U+EE2F
0, //U+EE30
0, //U+EE31
0, //U+EE32
0, //U+EE33
0, //U+EE34
0, //U+EE35
0, //U+EE36
0, //U+EE37
0, //U+EE38
0, //U+EE39
0, //U+EE3A
0, //U+EE3B
0, //U+EE3C
0, //U+EE3D
0, //U+EE3E
0, //U+EE3F
0, //U+EE40
0, //U+EE41
0, //U+EE42
0, //U+EE43
0, //U+EE44
0, //U+EE45
0, //U+EE46
0, //U+EE47
0, //U+EE48
0, //U+EE49
0, //U+EE4A
0, //U+EE4B
0, //U+EE4C
0, //U+EE4D
0, //U+EE4E
0, //U+EE4F
0, //U+EE50
0, //U+EE51
0, //U+EE52
0, //U+EE53
0, //U+EE54
0, //U+EE55
0, //U+EE56
0, //U+EE57
0, //U+EE58
0, //U+EE59
0, //U+EE5A
0, //U+EE5B
0, //U+EE5C
0, //U+EE5D
0, //U+EE5E
0, //U+EE5F
0, //U+EE60
0, //U+EE61
0, //U+EE62
0, //U+EE63
0, //U+EE64
0, //U+EE65
0, //U+EE66
0, //U+EE67
0, //U+EE68
0, //U+EE69
0, //U+EE6A
0, //U+EE6B
0, //U+EE6C
0, //U+EE6D
0, //U+EE6E
0, //U+EE6F
0, //U+EE70
0, //U+EE71
0, //U+EE72
0, //U+EE73
0, //U+EE74
0, //U+EE75
0, //U+EE76
0, //U+EE77
0, //U+EE78
0, //U+EE79
0, //U+EE7A
0, //U+EE7B
0, //U+EE7C
0, //U+EE7D
0, //U+EE7E
0, //U+EE7F
0, //U+EE80
0, //U+EE81
0, //U+EE82
0, //U+EE83
0, //U+EE84
0, //U+EE85
0, //U+EE86
0, //U+EE87
0, //U+EE88
0, //U+EE89
0, //U+EE8A
0, //U+EE8B
0, //U+EE8C
0, //U+EE8D
0, //U+EE8E
0, //U+EE8F
0, //U+EE90
0, //U+EE91
0, //U+EE92
0, //U+EE93
0, //U+EE94
0, //U+EE95
0, //U+EE96
0, //U+EE97
0, //U+EE98
0, //U+EE99
0, //U+EE9A
0, //U+EE9B
0, //U+EE9C
0, //U+EE9D
0, //U+EE9E
0, //U+EE9F
0, //U+EEA0
0, //U+EEA1
0, //U+EEA2
0, //U+EEA3
0, //U+EEA4
0, //U+EEA5
0, //U+EEA6
0, //U+EEA7
0, //U+EEA8
0, //U+EEA9
0, //U+EEAA
0, //U+EEAB
0, //U+EEAC
0, //U+EEAD
0, //U+EEAE
0, //U+EEAF
0, //U+EEB0
0, //U+EEB1
0, //U+EEB2
0, //U+EEB3
0, //U+EEB4
0, //U+EEB5
0, //U+EEB6
0, //U+EEB7
0, //U+EEB8
0, //U+EEB9
0, //U+EEBA
0, //U+EEBB
0, //U+EEBC
0, //U+EEBD
0, //U+EEBE
0, //U+EEBF
0, //U+EEC0
0, //U+EEC1
0, //U+EEC2
0, //U+EEC3
0, //U+EEC4
0, //U+EEC5
0, //U+EEC6
0, //U+EEC7
0, //U+EEC8
0, //U+EEC9
0, //U+EECA
0, //U+EECB
0, //U+EECC
0, //U+EECD
0, //U+EECE
0, //U+EECF
0, //U+EED0
0, //U+EED1
0, //U+EED2
0, //U+EED3
0, //U+EED4
0, //U+EED5
0, //U+EED6
0, //U+EED7
0, //U+EED8
0, //U+EED9
0, //U+EEDA
0, //U+EEDB
0, //U+EEDC
0, //U+EEDD
0, //U+EEDE
0, //U+EEDF
0, //U+EEE0
0, //U+EEE1
0, //U+EEE2
0, //U+EEE3
0, //U+EEE4
0, //U+EEE5
0, //U+EEE6
0, //U+EEE7
0, //U+EEE8
0, //U+EEE9
0, //U+EEEA
0, //U+EEEB
0, //U+EEEC
0, //U+EEED
0, //U+EEEE
0, //U+EEEF
0, //U+EEF0
0, //U+EEF1
0, //U+EEF2
0, //U+EEF3
0, //U+EEF4
0, //U+EEF5
0, //U+EEF6
0, //U+EEF7
0, //U+EEF8
0, //U+EEF9
0, //U+EEFA
0, //U+EEFB
0, //U+EEFC
0, //U+EEFD
0, //U+EEFE
0, //U+EEFF
0, //U+EF00
0, //U+EF01
0, //U+EF02
0, //U+EF03
0, //U+EF04
0, //U+EF05
0, //U+EF06
0, //U+EF07
0, //U+EF08
0, //U+EF09
0, //U+EF0A
0, //U+EF0B
0, //U+EF0C
0, //U+EF0D
0, //U+EF0E
0, //U+EF0F
0, //U+EF10
0, //U+EF11
0, //U+EF12
0, //U+EF13
0, //U+EF14
0, //U+EF15
0, //U+EF16
0, //U+EF17
0, //U+EF18
0, //U+EF19
0, //U+EF1A
0, //U+EF1B
0, //U+EF1C
0, //U+EF1D
0, //U+EF1E
0, //U+EF1F
0, //U+EF20
0, //U+EF21
0, //U+EF22
0, //U+EF23
0, //U+EF24
0, //U+EF25
0, //U+EF26
0, //U+EF27
0, //U+EF28
0, //U+EF29
0, //U+EF2A
0, //U+EF2B
0, //U+EF2C
0, //U+EF2D
0, //U+EF2E
0, //U+EF2F
0, //U+EF30
0, //U+EF31
0, //U+EF32
0, //U+EF33
0, //U+EF34
0, //U+EF35
0, //U+EF36
0, //U+EF37
0, //U+EF38
0, //U+EF39
0, //U+EF3A
0, //U+EF3B
0, //U+EF3C
0, //U+EF3D
0, //U+EF3E
0, //U+EF3F
0, //U+EF40
0, //U+EF41
0, //U+EF42
0, //U+EF43
0, //U+EF44
0, //U+EF45
0, //U+EF46
0, //U+EF47
0, //U+EF48
0, //U+EF49
0, //U+EF4A
0, //U+EF4B
0, //U+EF4C
0, //U+EF4D
0, //U+EF4E
0, //U+EF4F
0, //U+EF50
0, //U+EF51
0, //U+EF52
0, //U+EF53
0, //U+EF54
0, //U+EF55
0, //U+EF56
0, //U+EF57
0, //U+EF58
0, //U+EF59
0, //U+EF5A
0, //U+EF5B
0, //U+EF5C
0, //U+EF5D
0, //U+EF5E
0, //U+EF5F
0, //U+EF60
0, //U+EF61
0, //U+EF62
0, //U+EF63
0, //U+EF64
0, //U+EF65
0, //U+EF66
0, //U+EF67
0, //U+EF68
0, //U+EF69
0, //U+EF6A
0, //U+EF6B
0, //U+EF6C
0, //U+EF6D
0, //U+EF6E
0, //U+EF6F
0, //U+EF70
0, //U+EF71
0, //U+EF72
0, //U+EF73
0, //U+EF74
0, //U+EF75
0, //U+EF76
0, //U+EF77
0, //U+EF78
0, //U+EF79
0, //U+EF7A
0, //U+EF7B
0, //U+EF7C
0, //U+EF7D
0, //U+EF7E
0, //U+EF7F
0, //U+EF80
0, //U+EF81
0, //U+EF82
0, //U+EF83
0, //U+EF84
0, //U+EF85
0, //U+EF86
0, //U+EF87
0, //U+EF88
0, //U+EF89
0, //U+EF8A
0, //U+EF8B
0, //U+EF8C
0, //U+EF8D
0, //U+EF8E
0, //U+EF8F
0, //U+EF90
0, //U+EF91
0, //U+EF92
0, //U+EF93
0, //U+EF94
0, //U+EF95
0, //U+EF96
0, //U+EF97
0, //U+EF98
0, //U+EF99
0, //U+EF9A
0, //U+EF9B
0, //U+EF9C
0, //U+EF9D
0, //U+EF9E
0, //U+EF9F
0, //U+EFA0
0, //U+EFA1
0, //U+EFA2
0, //U+EFA3
0, //U+EFA4
0, //U+EFA5
0, //U+EFA6
0, //U+EFA7
0, //U+EFA8
0, //U+EFA9
0, //U+EFAA
0, //U+EFAB
0, //U+EFAC
0, //U+EFAD
0, //U+EFAE
0, //U+EFAF
0, //U+EFB0
0, //U+EFB1
0, //U+EFB2
0, //U+EFB3
0, //U+EFB4
0, //U+EFB5
0, //U+EFB6
0, //U+EFB7
0, //U+EFB8
0, //U+EFB9
0, //U+EFBA
0, //U+EFBB
0, //U+EFBC
0, //U+EFBD
0, //U+EFBE
0, //U+EFBF
0, //U+EFC0
0, //U+EFC1
0, //U+EFC2
0, //U+EFC3
0, //U+EFC4
0, //U+EFC5
0, //U+EFC6
0, //U+EFC7
0, //U+EFC8
0, //U+EFC9
0, //U+EFCA
0, //U+EFCB
0, //U+EFCC
0, //U+EFCD
0, //U+EFCE
0, //U+EFCF
0, //U+EFD0
0, //U+EFD1
0, //U+EFD2
0, //U+EFD3
0, //U+EFD4
0, //U+EFD5
0, //U+EFD6
0, //U+EFD7
0, //U+EFD8
0, //U+EFD9
0, //U+EFDA
0, //U+EFDB
0, //U+EFDC
0, //U+EFDD
0, //U+EFDE
0, //U+EFDF
0, //U+EFE0
0, //U+EFE1
0, //U+EFE2
0, //U+EFE3
0, //U+EFE4
0, //U+EFE5
0, //U+EFE6
0, //U+EFE7
0, //U+EFE8
0, //U+EFE9
0, //U+EFEA
0, //U+EFEB
0, //U+EFEC
0, //U+EFED
0, //U+EFEE
0, //U+EFEF
0, //U+EFF0
0, //U+EFF1
0, //U+EFF2
0, //U+EFF3
0, //U+EFF4
0, //U+EFF5
0, //U+EFF6
0, //U+EFF7
0, //U+EFF8
0, //U+EFF9
0, //U+EFFA
0, //U+EFFB
0, //U+EFFC
0, //U+EFFD
0, //U+EFFE
0, //U+EFFF
0, //U+F000
0, //U+F001
0, //U+F002
0, //U+F003
0, //U+F004
0, //U+F005
0, //U+F006
0, //U+F007
0, //U+F008
0, //U+F009
0, //U+F00A
0, //U+F00B
0, //U+F00C
0, //U+F00D
0, //U+F00E
0, //U+F00F
0, //U+F010
0, //U+F011
0, //U+F012
0, //U+F013
0, //U+F014
0, //U+F015
0, //U+F016
0, //U+F017
0, //U+F018
0, //U+F019
0, //U+F01A
0, //U+F01B
0, //U+F01C
0, //U+F01D
0, //U+F01E
0, //U+F01F
0, //U+F020
0, //U+F021
0, //U+F022
0, //U+F023
0, //U+F024
0, //U+F025
0, //U+F026
0, //U+F027
0, //U+F028
0, //U+F029
0, //U+F02A
0, //U+F02B
0, //U+F02C
0, //U+F02D
0, //U+F02E
0, //U+F02F
0, //U+F030
0, //U+F031
0, //U+F032
0, //U+F033
0, //U+F034
0, //U+F035
0, //U+F036
0, //U+F037
0, //U+F038
0, //U+F039
0, //U+F03A
0, //U+F03B
0, //U+F03C
0, //U+F03D
0, //U+F03E
0, //U+F03F
0, //U+F040
0, //U+F041
0, //U+F042
0, //U+F043
0, //U+F044
0, //U+F045
0, //U+F046
0, //U+F047
0, //U+F048
0, //U+F049
0, //U+F04A
0, //U+F04B
0, //U+F04C
0, //U+F04D
0, //U+F04E
0, //U+F04F
0, //U+F050
0, //U+F051
0, //U+F052
0, //U+F053
0, //U+F054
0, //U+F055
0, //U+F056
0, //U+F057
0, //U+F058
0, //U+F059
0, //U+F05A
0, //U+F05B
0, //U+F05C
0, //U+F05D
0, //U+F05E
0, //U+F05F
0, //U+F060
0, //U+F061
0, //U+F062
0, //U+F063
0, //U+F064
0, //U+F065
0, //U+F066
0, //U+F067
0, //U+F068
0, //U+F069
0, //U+F06A
0, //U+F06B
0, //U+F06C
0, //U+F06D
0, //U+F06E
0, //U+F06F
0, //U+F070
0, //U+F071
0, //U+F072
0, //U+F073
0, //U+F074
0, //U+F075
0, //U+F076
0, //U+F077
0, //U+F078
0, //U+F079
0, //U+F07A
0, //U+F07B
0, //U+F07C
0, //U+F07D
0, //U+F07E
0, //U+F07F
0, //U+F080
0, //U+F081
0, //U+F082
0, //U+F083
0, //U+F084
0, //U+F085
0, //U+F086
0, //U+F087
0, //U+F088
0, //U+F089
0, //U+F08A
0, //U+F08B
0, //U+F08C
0, //U+F08D
0, //U+F08E
0, //U+F08F
0, //U+F090
0, //U+F091
0, //U+F092
0, //U+F093
0, //U+F094
0, //U+F095
0, //U+F096
0, //U+F097
0, //U+F098
0, //U+F099
0, //U+F09A
0, //U+F09B
0, //U+F09C
0, //U+F09D
0, //U+F09E
0, //U+F09F
0, //U+F0A0
0, //U+F0A1
0, //U+F0A2
0, //U+F0A3
0, //U+F0A4
0, //U+F0A5
0, //U+F0A6
0, //U+F0A7
0, //U+F0A8
0, //U+F0A9
0, //U+F0AA
0, //U+F0AB
0, //U+F0AC
0, //U+F0AD
0, //U+F0AE
0, //U+F0AF
0, //U+F0B0
0, //U+F0B1
0, //U+F0B2
0, //U+F0B3
0, //U+F0B4
0, //U+F0B5
0, //U+F0B6
0, //U+F0B7
0, //U+F0B8
0, //U+F0B9
0, //U+F0BA
0, //U+F0BB
0, //U+F0BC
0, //U+F0BD
0, //U+F0BE
0, //U+F0BF
0, //U+F0C0
0, //U+F0C1
0, //U+F0C2
0, //U+F0C3
0, //U+F0C4
0, //U+F0C5
0, //U+F0C6
0, //U+F0C7
0, //U+F0C8
0, //U+F0C9
0, //U+F0CA
0, //U+F0CB
0, //U+F0CC
0, //U+F0CD
0, //U+F0CE
0, //U+F0CF
0, //U+F0D0
0, //U+F0D1
0, //U+F0D2
0, //U+F0D3
0, //U+F0D4
0, //U+F0D5
0, //U+F0D6
0, //U+F0D7
0, //U+F0D8
0, //U+F0D9
0, //U+F0DA
0, //U+F0DB
0, //U+F0DC
0, //U+F0DD
0, //U+F0DE
0, //U+F0DF
0, //U+F0E0
0, //U+F0E1
0, //U+F0E2
0, //U+F0E3
0, //U+F0E4
0, //U+F0E5
0, //U+F0E6
0, //U+F0E7
0, //U+F0E8
0, //U+F0E9
0, //U+F0EA
0, //U+F0EB
0, //U+F0EC
0, //U+F0ED
0, //U+F0EE
0, //U+F0EF
0, //U+F0F0
0, //U+F0F1
0, //U+F0F2
0, //U+F0F3
0, //U+F0F4
0, //U+F0F5
0, //U+F0F6
0, //U+F0F7
0, //U+F0F8
0, //U+F0F9
0, //U+F0FA
0, //U+F0FB
0, //U+F0FC
0, //U+F0FD
0, //U+F0FE
0, //U+F0FF
0, //U+F100
0, //U+F101
0, //U+F102
0, //U+F103
0, //U+F104
0, //U+F105
0, //U+F106
0, //U+F107
0, //U+F108
0, //U+F109
0, //U+F10A
0, //U+F10B
0, //U+F10C
0, //U+F10D
0, //U+F10E
0, //U+F10F
0, //U+F110
0, //U+F111
0, //U+F112
0, //U+F113
0, //U+F114
0, //U+F115
0, //U+F116
0, //U+F117
0, //U+F118
0, //U+F119
0, //U+F11A
0, //U+F11B
0, //U+F11C
0, //U+F11D
0, //U+F11E
0, //U+F11F
0, //U+F120
0, //U+F121
0, //U+F122
0, //U+F123
0, //U+F124
0, //U+F125
0, //U+F126
0, //U+F127
0, //U+F128
0, //U+F129
0, //U+F12A
0, //U+F12B
0, //U+F12C
0, //U+F12D
0, //U+F12E
0, //U+F12F
0, //U+F130
0, //U+F131
0, //U+F132
0, //U+F133
0, //U+F134
0, //U+F135
0, //U+F136
0, //U+F137
0, //U+F138
0, //U+F139
0, //U+F13A
0, //U+F13B
0, //U+F13C
0, //U+F13D
0, //U+F13E
0, //U+F13F
0, //U+F140
0, //U+F141
0, //U+F142
0, //U+F143
0, //U+F144
0, //U+F145
0, //U+F146
0, //U+F147
0, //U+F148
0, //U+F149
0, //U+F14A
0, //U+F14B
0, //U+F14C
0, //U+F14D
0, //U+F14E
0, //U+F14F
0, //U+F150
0, //U+F151
0, //U+F152
0, //U+F153
0, //U+F154
0, //U+F155
0, //U+F156
0, //U+F157
0, //U+F158
0, //U+F159
0, //U+F15A
0, //U+F15B
0, //U+F15C
0, //U+F15D
0, //U+F15E
0, //U+F15F
0, //U+F160
0, //U+F161
0, //U+F162
0, //U+F163
0, //U+F164
0, //U+F165
0, //U+F166
0, //U+F167
0, //U+F168
0, //U+F169
0, //U+F16A
0, //U+F16B
0, //U+F16C
0, //U+F16D
0, //U+F16E
0, //U+F16F
0, //U+F170
0, //U+F171
0, //U+F172
0, //U+F173
0, //U+F174
0, //U+F175
0, //U+F176
0, //U+F177
0, //U+F178
0, //U+F179
0, //U+F17A
0, //U+F17B
0, //U+F17C
0, //U+F17D
0, //U+F17E
0, //U+F17F
0, //U+F180
0, //U+F181
0, //U+F182
0, //U+F183
0, //U+F184
0, //U+F185
0, //U+F186
0, //U+F187
0, //U+F188
0, //U+F189
0, //U+F18A
0, //U+F18B
0, //U+F18C
0, //U+F18D
0, //U+F18E
0, //U+F18F
0, //U+F190
0, //U+F191
0, //U+F192
0, //U+F193
0, //U+F194
0, //U+F195
0, //U+F196
0, //U+F197
0, //U+F198
0, //U+F199
0, //U+F19A
0, //U+F19B
0, //U+F19C
0, //U+F19D
0, //U+F19E
0, //U+F19F
0, //U+F1A0
0, //U+F1A1
0, //U+F1A2
0, //U+F1A3
0, //U+F1A4
0, //U+F1A5
0, //U+F1A6
0, //U+F1A7
0, //U+F1A8
0, //U+F1A9
0, //U+F1AA
0, //U+F1AB
0, //U+F1AC
0, //U+F1AD
0, //U+F1AE
0, //U+F1AF
0, //U+F1B0
0, //U+F1B1
0, //U+F1B2
0, //U+F1B3
0, //U+F1B4
0, //U+F1B5
0, //U+F1B6
0, //U+F1B7
0, //U+F1B8
0, //U+F1B9
0, //U+F1BA
0, //U+F1BB
0, //U+F1BC
0, //U+F1BD
0, //U+F1BE
0, //U+F1BF
0, //U+F1C0
0, //U+F1C1
0, //U+F1C2
0, //U+F1C3
0, //U+F1C4
0, //U+F1C5
0, //U+F1C6
0, //U+F1C7
0, //U+F1C8
0, //U+F1C9
0, //U+F1CA
0, //U+F1CB
0, //U+F1CC
0, //U+F1CD
0, //U+F1CE
0, //U+F1CF
0, //U+F1D0
0, //U+F1D1
0, //U+F1D2
0, //U+F1D3
0, //U+F1D4
0, //U+F1D5
0, //U+F1D6
0, //U+F1D7
0, //U+F1D8
0, //U+F1D9
0, //U+F1DA
0, //U+F1DB
0, //U+F1DC
0, //U+F1DD
0, //U+F1DE
0, //U+F1DF
0, //U+F1E0
0, //U+F1E1
0, //U+F1E2
0, //U+F1E3
0, //U+F1E4
0, //U+F1E5
0, //U+F1E6
0, //U+F1E7
0, //U+F1E8
0, //U+F1E9
0, //U+F1EA
0, //U+F1EB
0, //U+F1EC
0, //U+F1ED
0, //U+F1EE
0, //U+F1EF
0, //U+F1F0
0, //U+F1F1
0, //U+F1F2
0, //U+F1F3
0, //U+F1F4
0, //U+F1F5
0, //U+F1F6
0, //U+F1F7
0, //U+F1F8
0, //U+F1F9
0, //U+F1FA
0, //U+F1FB
0, //U+F1FC
0, //U+F1FD
0, //U+F1FE
0, //U+F1FF
0, //U+F200
0, //U+F201
0, //U+F202
0, //U+F203
0, //U+F204
0, //U+F205
0, //U+F206
0, //U+F207
0, //U+F208
0, //U+F209
0, //U+F20A
0, //U+F20B
0, //U+F20C
0, //U+F20D
0, //U+F20E
0, //U+F20F
0, //U+F210
0, //U+F211
0, //U+F212
0, //U+F213
0, //U+F214
0, //U+F215
0, //U+F216
0, //U+F217
0, //U+F218
0, //U+F219
0, //U+F21A
0, //U+F21B
0, //U+F21C
0, //U+F21D
0, //U+F21E
0, //U+F21F
0, //U+F220
0, //U+F221
0, //U+F222
0, //U+F223
0, //U+F224
0, //U+F225
0, //U+F226
0, //U+F227
0, //U+F228
0, //U+F229
0, //U+F22A
0, //U+F22B
0, //U+F22C
0, //U+F22D
0, //U+F22E
0, //U+F22F
0, //U+F230
0, //U+F231
0, //U+F232
0, //U+F233
0, //U+F234
0, //U+F235
0, //U+F236
0, //U+F237
0, //U+F238
0, //U+F239
0, //U+F23A
0, //U+F23B
0, //U+F23C
0, //U+F23D
0, //U+F23E
0, //U+F23F
0, //U+F240
0, //U+F241
0, //U+F242
0, //U+F243
0, //U+F244
0, //U+F245
0, //U+F246
0, //U+F247
0, //U+F248
0, //U+F249
0, //U+F24A
0, //U+F24B
0, //U+F24C
0, //U+F24D
0, //U+F24E
0, //U+F24F
0, //U+F250
0, //U+F251
0, //U+F252
0, //U+F253
0, //U+F254
0, //U+F255
0, //U+F256
0, //U+F257
0, //U+F258
0, //U+F259
0, //U+F25A
0, //U+F25B
0, //U+F25C
0, //U+F25D
0, //U+F25E
0, //U+F25F
0, //U+F260
0, //U+F261
0, //U+F262
0, //U+F263
0, //U+F264
0, //U+F265
0, //U+F266
0, //U+F267
0, //U+F268
0, //U+F269
0, //U+F26A
0, //U+F26B
0, //U+F26C
0, //U+F26D
0, //U+F26E
0, //U+F26F
0, //U+F270
0, //U+F271
0, //U+F272
0, //U+F273
0, //U+F274
0, //U+F275
0, //U+F276
0, //U+F277
0, //U+F278
0, //U+F279
0, //U+F27A
0, //U+F27B
0, //U+F27C
0, //U+F27D
0, //U+F27E
0, //U+F27F
0, //U+F280
0, //U+F281
0, //U+F282
0, //U+F283
0, //U+F284
0, //U+F285
0, //U+F286
0, //U+F287
0, //U+F288
0, //U+F289
0, //U+F28A
0, //U+F28B
0, //U+F28C
0, //U+F28D
0, //U+F28E
0, //U+F28F
0, //U+F290
0, //U+F291
0, //U+F292
0, //U+F293
0, //U+F294
0, //U+F295
0, //U+F296
0, //U+F297
0, //U+F298
0, //U+F299
0, //U+F29A
0, //U+F29B
0, //U+F29C
0, //U+F29D
0, //U+F29E
0, //U+F29F
0, //U+F2A0
0, //U+F2A1
0, //U+F2A2
0, //U+F2A3
0, //U+F2A4
0, //U+F2A5
0, //U+F2A6
0, //U+F2A7
0, //U+F2A8
0, //U+F2A9
0, //U+F2AA
0, //U+F2AB
0, //U+F2AC
0, //U+F2AD
0, //U+F2AE
0, //U+F2AF
0, //U+F2B0
0, //U+F2B1
0, //U+F2B2
0, //U+F2B3
0, //U+F2B4
0, //U+F2B5
0, //U+F2B6
0, //U+F2B7
0, //U+F2B8
0, //U+F2B9
0, //U+F2BA
0, //U+F2BB
0, //U+F2BC
0, //U+F2BD
0, //U+F2BE
0, //U+F2BF
0, //U+F2C0
0, //U+F2C1
0, //U+F2C2
0, //U+F2C3
0, //U+F2C4
0, //U+F2C5
0, //U+F2C6
0, //U+F2C7
0, //U+F2C8
0, //U+F2C9
0, //U+F2CA
0, //U+F2CB
0, //U+F2CC
0, //U+F2CD
0, //U+F2CE
0, //U+F2CF
0, //U+F2D0
0, //U+F2D1
0, //U+F2D2
0, //U+F2D3
0, //U+F2D4
0, //U+F2D5
0, //U+F2D6
0, //U+F2D7
0, //U+F2D8
0, //U+F2D9
0, //U+F2DA
0, //U+F2DB
0, //U+F2DC
0, //U+F2DD
0, //U+F2DE
0, //U+F2DF
0, //U+F2E0
0, //U+F2E1
0, //U+F2E2
0, //U+F2E3
0, //U+F2E4
0, //U+F2E5
0, //U+F2E6
0, //U+F2E7
0, //U+F2E8
0, //U+F2E9
0, //U+F2EA
0, //U+F2EB
0, //U+F2EC
0, //U+F2ED
0, //U+F2EE
0, //U+F2EF
0, //U+F2F0
0, //U+F2F1
0, //U+F2F2
0, //U+F2F3
0, //U+F2F4
0, //U+F2F5
0, //U+F2F6
0, //U+F2F7
0, //U+F2F8
0, //U+F2F9
0, //U+F2FA
0, //U+F2FB
0, //U+F2FC
0, //U+F2FD
0, //U+F2FE
0, //U+F2FF
0, //U+F300
0, //U+F301
0, //U+F302
0, //U+F303
0, //U+F304
0, //U+F305
0, //U+F306
0, //U+F307
0, //U+F308
0, //U+F309
0, //U+F30A
0, //U+F30B
0, //U+F30C
0, //U+F30D
0, //U+F30E
0, //U+F30F
0, //U+F310
0, //U+F311
0, //U+F312
0, //U+F313
0, //U+F314
0, //U+F315
0, //U+F316
0, //U+F317
0, //U+F318
0, //U+F319
0, //U+F31A
0, //U+F31B
0, //U+F31C
0, //U+F31D
0, //U+F31E
0, //U+F31F
0, //U+F320
0, //U+F321
0, //U+F322
0, //U+F323
0, //U+F324
0, //U+F325
0, //U+F326
0, //U+F327
0, //U+F328
0, //U+F329
0, //U+F32A
0, //U+F32B
0, //U+F32C
0, //U+F32D
0, //U+F32E
0, //U+F32F
0, //U+F330
0, //U+F331
0, //U+F332
0, //U+F333
0, //U+F334
0, //U+F335
0, //U+F336
0, //U+F337
0, //U+F338
0, //U+F339
0, //U+F33A
0, //U+F33B
0, //U+F33C
0, //U+F33D
0, //U+F33E
0, //U+F33F
0, //U+F340
0, //U+F341
0, //U+F342
0, //U+F343
0, //U+F344
0, //U+F345
0, //U+F346
0, //U+F347
0, //U+F348
0, //U+F349
0, //U+F34A
0, //U+F34B
0, //U+F34C
0, //U+F34D
0, //U+F34E
0, //U+F34F
0, //U+F350
0, //U+F351
0, //U+F352
0, //U+F353
0, //U+F354
0, //U+F355
0, //U+F356
0, //U+F357
0, //U+F358
0, //U+F359
0, //U+F35A
0, //U+F35B
0, //U+F35C
0, //U+F35D
0, //U+F35E
0, //U+F35F
0, //U+F360
0, //U+F361
0, //U+F362
0, //U+F363
0, //U+F364
0, //U+F365
0, //U+F366
0, //U+F367
0, //U+F368
0, //U+F369
0, //U+F36A
0, //U+F36B
0, //U+F36C
0, //U+F36D
0, //U+F36E
0, //U+F36F
0, //U+F370
0, //U+F371
0, //U+F372
0, //U+F373
0, //U+F374
0, //U+F375
0, //U+F376
0, //U+F377
0, //U+F378
0, //U+F379
0, //U+F37A
0, //U+F37B
0, //U+F37C
0, //U+F37D
0, //U+F37E
0, //U+F37F
0, //U+F380
0, //U+F381
0, //U+F382
0, //U+F383
0, //U+F384
0, //U+F385
0, //U+F386
0, //U+F387
0, //U+F388
0, //U+F389
0, //U+F38A
0, //U+F38B
0, //U+F38C
0, //U+F38D
0, //U+F38E
0, //U+F38F
0, //U+F390
0, //U+F391
0, //U+F392
0, //U+F393
0, //U+F394
0, //U+F395
0, //U+F396
0, //U+F397
0, //U+F398
0, //U+F399
0, //U+F39A
0, //U+F39B
0, //U+F39C
0, //U+F39D
0, //U+F39E
0, //U+F39F
0, //U+F3A0
0, //U+F3A1
0, //U+F3A2
0, //U+F3A3
0, //U+F3A4
0, //U+F3A5
0, //U+F3A6
0, //U+F3A7
0, //U+F3A8
0, //U+F3A9
0, //U+F3AA
0, //U+F3AB
0, //U+F3AC
0, //U+F3AD
0, //U+F3AE
0, //U+F3AF
0, //U+F3B0
0, //U+F3B1
0, //U+F3B2
0, //U+F3B3
0, //U+F3B4
0, //U+F3B5
0, //U+F3B6
0, //U+F3B7
0, //U+F3B8
0, //U+F3B9
0, //U+F3BA
0, //U+F3BB
0, //U+F3BC
0, //U+F3BD
0, //U+F3BE
0, //U+F3BF
0, //U+F3C0
0, //U+F3C1
0, //U+F3C2
0, //U+F3C3
0, //U+F3C4
0, //U+F3C5
0, //U+F3C6
0, //U+F3C7
0, //U+F3C8
0, //U+F3C9
0, //U+F3CA
0, //U+F3CB
0, //U+F3CC
0, //U+F3CD
0, //U+F3CE
0, //U+F3CF
0, //U+F3D0
0, //U+F3D1
0, //U+F3D2
0, //U+F3D3
0, //U+F3D4
0, //U+F3D5
0, //U+F3D6
0, //U+F3D7
0, //U+F3D8
0, //U+F3D9
0, //U+F3DA
0, //U+F3DB
0, //U+F3DC
0, //U+F3DD
0, //U+F3DE
0, //U+F3DF
0, //U+F3E0
0, //U+F3E1
0, //U+F3E2
0, //U+F3E3
0, //U+F3E4
0, //U+F3E5
0, //U+F3E6
0, //U+F3E7
0, //U+F3E8
0, //U+F3E9
0, //U+F3EA
0, //U+F3EB
0, //U+F3EC
0, //U+F3ED
0, //U+F3EE
0, //U+F3EF
0, //U+F3F0
0, //U+F3F1
0, //U+F3F2
0, //U+F3F3
0, //U+F3F4
0, //U+F3F5
0, //U+F3F6
0, //U+F3F7
0, //U+F3F8
0, //U+F3F9
0, //U+F3FA
0, //U+F3FB
0, //U+F3FC
0, //U+F3FD
0, //U+F3FE
0, //U+F3FF
0, //U+F400
0, //U+F401
0, //U+F402
0, //U+F403
0, //U+F404
0, //U+F405
0, //U+F406
0, //U+F407
0, //U+F408
0, //U+F409
0, //U+F40A
0, //U+F40B
0, //U+F40C
0, //U+F40D
0, //U+F40E
0, //U+F40F
0, //U+F410
0, //U+F411
0, //U+F412
0, //U+F413
0, //U+F414
0, //U+F415
0, //U+F416
0, //U+F417
0, //U+F418
0, //U+F419
0, //U+F41A
0, //U+F41B
0, //U+F41C
0, //U+F41D
0, //U+F41E
0, //U+F41F
0, //U+F420
0, //U+F421
0, //U+F422
0, //U+F423
0, //U+F424
0, //U+F425
0, //U+F426
0, //U+F427
0, //U+F428
0, //U+F429
0, //U+F42A
0, //U+F42B
0, //U+F42C
0, //U+F42D
0, //U+F42E
0, //U+F42F
0, //U+F430
0, //U+F431
0, //U+F432
0, //U+F433
0, //U+F434
0, //U+F435
0, //U+F436
0, //U+F437
0, //U+F438
0, //U+F439
0, //U+F43A
0, //U+F43B
0, //U+F43C
0, //U+F43D
0, //U+F43E
0, //U+F43F
0, //U+F440
0, //U+F441
0, //U+F442
0, //U+F443
0, //U+F444
0, //U+F445
0, //U+F446
0, //U+F447
0, //U+F448
0, //U+F449
0, //U+F44A
0, //U+F44B
0, //U+F44C
0, //U+F44D
0, //U+F44E
0, //U+F44F
0, //U+F450
0, //U+F451
0, //U+F452
0, //U+F453
0, //U+F454
0, //U+F455
0, //U+F456
0, //U+F457
0, //U+F458
0, //U+F459
0, //U+F45A
0, //U+F45B
0, //U+F45C
0, //U+F45D
0, //U+F45E
0, //U+F45F
0, //U+F460
0, //U+F461
0, //U+F462
0, //U+F463
0, //U+F464
0, //U+F465
0, //U+F466
0, //U+F467
0, //U+F468
0, //U+F469
0, //U+F46A
0, //U+F46B
0, //U+F46C
0, //U+F46D
0, //U+F46E
0, //U+F46F
0, //U+F470
0, //U+F471
0, //U+F472
0, //U+F473
0, //U+F474
0, //U+F475
0, //U+F476
0, //U+F477
0, //U+F478
0, //U+F479
0, //U+F47A
0, //U+F47B
0, //U+F47C
0, //U+F47D
0, //U+F47E
0, //U+F47F
0, //U+F480
0, //U+F481
0, //U+F482
0, //U+F483
0, //U+F484
0, //U+F485
0, //U+F486
0, //U+F487
0, //U+F488
0, //U+F489
0, //U+F48A
0, //U+F48B
0, //U+F48C
0, //U+F48D
0, //U+F48E
0, //U+F48F
0, //U+F490
0, //U+F491
0, //U+F492
0, //U+F493
0, //U+F494
0, //U+F495
0, //U+F496
0, //U+F497
0, //U+F498
0, //U+F499
0, //U+F49A
0, //U+F49B
0, //U+F49C
0, //U+F49D
0, //U+F49E
0, //U+F49F
0, //U+F4A0
0, //U+F4A1
0, //U+F4A2
0, //U+F4A3
0, //U+F4A4
0, //U+F4A5
0, //U+F4A6
0, //U+F4A7
0, //U+F4A8
0, //U+F4A9
0, //U+F4AA
0, //U+F4AB
0, //U+F4AC
0, //U+F4AD
0, //U+F4AE
0, //U+F4AF
0, //U+F4B0
0, //U+F4B1
0, //U+F4B2
0, //U+F4B3
0, //U+F4B4
0, //U+F4B5
0, //U+F4B6
0, //U+F4B7
0, //U+F4B8
0, //U+F4B9
0, //U+F4BA
0, //U+F4BB
0, //U+F4BC
0, //U+F4BD
0, //U+F4BE
0, //U+F4BF
0, //U+F4C0
0, //U+F4C1
0, //U+F4C2
0, //U+F4C3
0, //U+F4C4
0, //U+F4C5
0, //U+F4C6
0, //U+F4C7
0, //U+F4C8
0, //U+F4C9
0, //U+F4CA
0, //U+F4CB
0, //U+F4CC
0, //U+F4CD
0, //U+F4CE
0, //U+F4CF
0, //U+F4D0
0, //U+F4D1
0, //U+F4D2
0, //U+F4D3
0, //U+F4D4
0, //U+F4D5
0, //U+F4D6
0, //U+F4D7
0, //U+F4D8
0, //U+F4D9
0, //U+F4DA
0, //U+F4DB
0, //U+F4DC
0, //U+F4DD
0, //U+F4DE
0, //U+F4DF
0, //U+F4E0
0, //U+F4E1
0, //U+F4E2
0, //U+F4E3
0, //U+F4E4
0, //U+F4E5
0, //U+F4E6
0, //U+F4E7
0, //U+F4E8
0, //U+F4E9
0, //U+F4EA
0, //U+F4EB
0, //U+F4EC
0, //U+F4ED
0, //U+F4EE
0, //U+F4EF
0, //U+F4F0
0, //U+F4F1
0, //U+F4F2
0, //U+F4F3
0, //U+F4F4
0, //U+F4F5
0, //U+F4F6
0, //U+F4F7
0, //U+F4F8
0, //U+F4F9
0, //U+F4FA
0, //U+F4FB
0, //U+F4FC
0, //U+F4FD
0, //U+F4FE
0, //U+F4FF
0, //U+F500
0, //U+F501
0, //U+F502
0, //U+F503
0, //U+F504
0, //U+F505
0, //U+F506
0, //U+F507
0, //U+F508
0, //U+F509
0, //U+F50A
0, //U+F50B
0, //U+F50C
0, //U+F50D
0, //U+F50E
0, //U+F50F
0, //U+F510
0, //U+F511
0, //U+F512
0, //U+F513
0, //U+F514
0, //U+F515
0, //U+F516
0, //U+F517
0, //U+F518
0, //U+F519
0, //U+F51A
0, //U+F51B
0, //U+F51C
0, //U+F51D
0, //U+F51E
0, //U+F51F
0, //U+F520
0, //U+F521
0, //U+F522
0, //U+F523
0, //U+F524
0, //U+F525
0, //U+F526
0, //U+F527
0, //U+F528
0, //U+F529
0, //U+F52A
0, //U+F52B
0, //U+F52C
0, //U+F52D
0, //U+F52E
0, //U+F52F
0, //U+F530
0, //U+F531
0, //U+F532
0, //U+F533
0, //U+F534
0, //U+F535
0, //U+F536
0, //U+F537
0, //U+F538
0, //U+F539
0, //U+F53A
0, //U+F53B
0, //U+F53C
0, //U+F53D
0, //U+F53E
0, //U+F53F
0, //U+F540
0, //U+F541
0, //U+F542
0, //U+F543
0, //U+F544
0, //U+F545
0, //U+F546
0, //U+F547
0, //U+F548
0, //U+F549
0, //U+F54A
0, //U+F54B
0, //U+F54C
0, //U+F54D
0, //U+F54E
0, //U+F54F
0, //U+F550
0, //U+F551
0, //U+F552
0, //U+F553
0, //U+F554
0, //U+F555
0, //U+F556
0, //U+F557
0, //U+F558
0, //U+F559
0, //U+F55A
0, //U+F55B
0, //U+F55C
0, //U+F55D
0, //U+F55E
0, //U+F55F
0, //U+F560
0, //U+F561
0, //U+F562
0, //U+F563
0, //U+F564
0, //U+F565
0, //U+F566
0, //U+F567
0, //U+F568
0, //U+F569
0, //U+F56A
0, //U+F56B
0, //U+F56C
0, //U+F56D
0, //U+F56E
0, //U+F56F
0, //U+F570
0, //U+F571
0, //U+F572
0, //U+F573
0, //U+F574
0, //U+F575
0, //U+F576
0, //U+F577
0, //U+F578
0, //U+F579
0, //U+F57A
0, //U+F57B
0, //U+F57C
0, //U+F57D
0, //U+F57E
0, //U+F57F
0, //U+F580
0, //U+F581
0, //U+F582
0, //U+F583
0, //U+F584
0, //U+F585
0, //U+F586
0, //U+F587
0, //U+F588
0, //U+F589
0, //U+F58A
0, //U+F58B
0, //U+F58C
0, //U+F58D
0, //U+F58E
0, //U+F58F
0, //U+F590
0, //U+F591
0, //U+F592
0, //U+F593
0, //U+F594
0, //U+F595
0, //U+F596
0, //U+F597
0, //U+F598
0, //U+F599
0, //U+F59A
0, //U+F59B
0, //U+F59C
0, //U+F59D
0, //U+F59E
0, //U+F59F
0, //U+F5A0
0, //U+F5A1
0, //U+F5A2
0, //U+F5A3
0, //U+F5A4
0, //U+F5A5
0, //U+F5A6
0, //U+F5A7
0, //U+F5A8
0, //U+F5A9
0, //U+F5AA
0, //U+F5AB
0, //U+F5AC
0, //U+F5AD
0, //U+F5AE
0, //U+F5AF
0, //U+F5B0
0, //U+F5B1
0, //U+F5B2
0, //U+F5B3
0, //U+F5B4
0, //U+F5B5
0, //U+F5B6
0, //U+F5B7
0, //U+F5B8
0, //U+F5B9
0, //U+F5BA
0, //U+F5BB
0, //U+F5BC
0, //U+F5BD
0, //U+F5BE
0, //U+F5BF
0, //U+F5C0
0, //U+F5C1
0, //U+F5C2
0, //U+F5C3
0, //U+F5C4
0, //U+F5C5
0, //U+F5C6
0, //U+F5C7
0, //U+F5C8
0, //U+F5C9
0, //U+F5CA
0, //U+F5CB
0, //U+F5CC
0, //U+F5CD
0, //U+F5CE
0, //U+F5CF
0, //U+F5D0
0, //U+F5D1
0, //U+F5D2
0, //U+F5D3
0, //U+F5D4
0, //U+F5D5
0, //U+F5D6
0, //U+F5D7
0, //U+F5D8
0, //U+F5D9
0, //U+F5DA
0, //U+F5DB
0, //U+F5DC
0, //U+F5DD
0, //U+F5DE
0, //U+F5DF
0, //U+F5E0
0, //U+F5E1
0, //U+F5E2
0, //U+F5E3
0, //U+F5E4
0, //U+F5E5
0, //U+F5E6
0, //U+F5E7
0, //U+F5E8
0, //U+F5E9
0, //U+F5EA
0, //U+F5EB
0, //U+F5EC
0, //U+F5ED
0, //U+F5EE
0, //U+F5EF
0, //U+F5F0
0, //U+F5F1
0, //U+F5F2
0, //U+F5F3
0, //U+F5F4
0, //U+F5F5
0, //U+F5F6
0, //U+F5F7
0, //U+F5F8
0, //U+F5F9
0, //U+F5FA
0, //U+F5FB
0, //U+F5FC
0, //U+F5FD
0, //U+F5FE
0, //U+F5FF
0, //U+F600
0, //U+F601
0, //U+F602
0, //U+F603
0, //U+F604
0, //U+F605
0, //U+F606
0, //U+F607
0, //U+F608
0, //U+F609
0, //U+F60A
0, //U+F60B
0, //U+F60C
0, //U+F60D
0, //U+F60E
0, //U+F60F
0, //U+F610
0, //U+F611
0, //U+F612
0, //U+F613
0, //U+F614
0, //U+F615
0, //U+F616
0, //U+F617
0, //U+F618
0, //U+F619
0, //U+F61A
0, //U+F61B
0, //U+F61C
0, //U+F61D
0, //U+F61E
0, //U+F61F
0, //U+F620
0, //U+F621
0, //U+F622
0, //U+F623
0, //U+F624
0, //U+F625
0, //U+F626
0, //U+F627
0, //U+F628
0, //U+F629
0, //U+F62A
0, //U+F62B
0, //U+F62C
0, //U+F62D
0, //U+F62E
0, //U+F62F
0, //U+F630
0, //U+F631
0, //U+F632
0, //U+F633
0, //U+F634
0, //U+F635
0, //U+F636
0, //U+F637
0, //U+F638
0, //U+F639
0, //U+F63A
0, //U+F63B
0, //U+F63C
0, //U+F63D
0, //U+F63E
0, //U+F63F
0, //U+F640
0, //U+F641
0, //U+F642
0, //U+F643
0, //U+F644
0, //U+F645
0, //U+F646
0, //U+F647
0, //U+F648
0, //U+F649
0, //U+F64A
0, //U+F64B
0, //U+F64C
0, //U+F64D
0, //U+F64E
0, //U+F64F
0, //U+F650
0, //U+F651
0, //U+F652
0, //U+F653
0, //U+F654
0, //U+F655
0, //U+F656
0, //U+F657
0, //U+F658
0, //U+F659
0, //U+F65A
0, //U+F65B
0, //U+F65C
0, //U+F65D
0, //U+F65E
0, //U+F65F
0, //U+F660
0, //U+F661
0, //U+F662
0, //U+F663
0, //U+F664
0, //U+F665
0, //U+F666
0, //U+F667
0, //U+F668
0, //U+F669
0, //U+F66A
0, //U+F66B
0, //U+F66C
0, //U+F66D
0, //U+F66E
0, //U+F66F
0, //U+F670
0, //U+F671
0, //U+F672
0, //U+F673
0, //U+F674
0, //U+F675
0, //U+F676
0, //U+F677
0, //U+F678
0, //U+F679
0, //U+F67A
0, //U+F67B
0, //U+F67C
0, //U+F67D
0, //U+F67E
0, //U+F67F
0, //U+F680
0, //U+F681
0, //U+F682
0, //U+F683
0, //U+F684
0, //U+F685
0, //U+F686
0, //U+F687
0, //U+F688
0, //U+F689
0, //U+F68A
0, //U+F68B
0, //U+F68C
0, //U+F68D
0, //U+F68E
0, //U+F68F
0, //U+F690
0, //U+F691
0, //U+F692
0, //U+F693
0, //U+F694
0, //U+F695
0, //U+F696
0, //U+F697
0, //U+F698
0, //U+F699
0, //U+F69A
0, //U+F69B
0, //U+F69C
0, //U+F69D
0, //U+F69E
0, //U+F69F
0, //U+F6A0
0, //U+F6A1
0, //U+F6A2
0, //U+F6A3
0, //U+F6A4
0, //U+F6A5
0, //U+F6A6
0, //U+F6A7
0, //U+F6A8
0, //U+F6A9
0, //U+F6AA
0, //U+F6AB
0, //U+F6AC
0, //U+F6AD
0, //U+F6AE
0, //U+F6AF
0, //U+F6B0
0, //U+F6B1
0, //U+F6B2
0, //U+F6B3
0, //U+F6B4
0, //U+F6B5
0, //U+F6B6
0, //U+F6B7
0, //U+F6B8
0, //U+F6B9
0, //U+F6BA
0, //U+F6BB
0, //U+F6BC
0, //U+F6BD
0, //U+F6BE
0, //U+F6BF
0, //U+F6C0
0, //U+F6C1
0, //U+F6C2
0, //U+F6C3
0, //U+F6C4
0, //U+F6C5
0, //U+F6C6
0, //U+F6C7
0, //U+F6C8
0, //U+F6C9
0, //U+F6CA
0, //U+F6CB
0, //U+F6CC
0, //U+F6CD
0, //U+F6CE
0, //U+F6CF
0, //U+F6D0
0, //U+F6D1
0, //U+F6D2
0, //U+F6D3
0, //U+F6D4
0, //U+F6D5
0, //U+F6D6
0, //U+F6D7
0, //U+F6D8
0, //U+F6D9
0, //U+F6DA
0, //U+F6DB
0, //U+F6DC
0, //U+F6DD
0, //U+F6DE
0, //U+F6DF
0, //U+F6E0
0, //U+F6E1
0, //U+F6E2
0, //U+F6E3
0, //U+F6E4
0, //U+F6E5
0, //U+F6E6
0, //U+F6E7
0, //U+F6E8
0, //U+F6E9
0, //U+F6EA
0, //U+F6EB
0, //U+F6EC
0, //U+F6ED
0, //U+F6EE
0, //U+F6EF
0, //U+F6F0
0, //U+F6F1
0, //U+F6F2
0, //U+F6F3
0, //U+F6F4
0, //U+F6F5
0, //U+F6F6
0, //U+F6F7
0, //U+F6F8
0, //U+F6F9
0, //U+F6FA
0, //U+F6FB
0, //U+F6FC
0, //U+F6FD
0, //U+F6FE
0, //U+F6FF
0, //U+F700
0, //U+F701
0, //U+F702
0, //U+F703
0, //U+F704
0, //U+F705
0, //U+F706
0, //U+F707
0, //U+F708
0, //U+F709
0, //U+F70A
0, //U+F70B
0, //U+F70C
0, //U+F70D
0, //U+F70E
0, //U+F70F
0, //U+F710
0, //U+F711
0, //U+F712
0, //U+F713
0, //U+F714
0, //U+F715
0, //U+F716
0, //U+F717
0, //U+F718
0, //U+F719
0, //U+F71A
0, //U+F71B
0, //U+F71C
0, //U+F71D
0, //U+F71E
0, //U+F71F
0, //U+F720
0, //U+F721
0, //U+F722
0, //U+F723
0, //U+F724
0, //U+F725
0, //U+F726
0, //U+F727
0, //U+F728
0, //U+F729
0, //U+F72A
0, //U+F72B
0, //U+F72C
0, //U+F72D
0, //U+F72E
0, //U+F72F
0, //U+F730
0, //U+F731
0, //U+F732
0, //U+F733
0, //U+F734
0, //U+F735
0, //U+F736
0, //U+F737
0, //U+F738
0, //U+F739
0, //U+F73A
0, //U+F73B
0, //U+F73C
0, //U+F73D
0, //U+F73E
0, //U+F73F
0, //U+F740
0, //U+F741
0, //U+F742
0, //U+F743
0, //U+F744
0, //U+F745
0, //U+F746
0, //U+F747
0, //U+F748
0, //U+F749
0, //U+F74A
0, //U+F74B
0, //U+F74C
0, //U+F74D
0, //U+F74E
0, //U+F74F
0, //U+F750
0, //U+F751
0, //U+F752
0, //U+F753
0, //U+F754
0, //U+F755
0, //U+F756
0, //U+F757
0, //U+F758
0, //U+F759
0, //U+F75A
0, //U+F75B
0, //U+F75C
0, //U+F75D
0, //U+F75E
0, //U+F75F
0, //U+F760
0, //U+F761
0, //U+F762
0, //U+F763
0, //U+F764
0, //U+F765
0, //U+F766
0, //U+F767
0, //U+F768
0, //U+F769
0, //U+F76A
0, //U+F76B
0, //U+F76C
0, //U+F76D
0, //U+F76E
0, //U+F76F
0, //U+F770
0, //U+F771
0, //U+F772
0, //U+F773
0, //U+F774
0, //U+F775
0, //U+F776
0, //U+F777
0, //U+F778
0, //U+F779
0, //U+F77A
0, //U+F77B
0, //U+F77C
0, //U+F77D
0, //U+F77E
0, //U+F77F
0, //U+F780
0, //U+F781
0, //U+F782
0, //U+F783
0, //U+F784
0, //U+F785
0, //U+F786
0, //U+F787
0, //U+F788
0, //U+F789
0, //U+F78A
0, //U+F78B
0, //U+F78C
0, //U+F78D
0, //U+F78E
0, //U+F78F
0, //U+F790
0, //U+F791
0, //U+F792
0, //U+F793
0, //U+F794
0, //U+F795
0, //U+F796
0, //U+F797
0, //U+F798
0, //U+F799
0, //U+F79A
0, //U+F79B
0, //U+F79C
0, //U+F79D
0, //U+F79E
0, //U+F79F
0, //U+F7A0
0, //U+F7A1
0, //U+F7A2
0, //U+F7A3
0, //U+F7A4
0, //U+F7A5
0, //U+F7A6
0, //U+F7A7
0, //U+F7A8
0, //U+F7A9
0, //U+F7AA
0, //U+F7AB
0, //U+F7AC
0, //U+F7AD
0, //U+F7AE
0, //U+F7AF
0, //U+F7B0
0, //U+F7B1
0, //U+F7B2
0, //U+F7B3
0, //U+F7B4
0, //U+F7B5
0, //U+F7B6
0, //U+F7B7
0, //U+F7B8
0, //U+F7B9
0, //U+F7BA
0, //U+F7BB
0, //U+F7BC
0, //U+F7BD
0, //U+F7BE
0, //U+F7BF
0, //U+F7C0
0, //U+F7C1
0, //U+F7C2
0, //U+F7C3
0, //U+F7C4
0, //U+F7C5
0, //U+F7C6
0, //U+F7C7
0, //U+F7C8
0, //U+F7C9
0, //U+F7CA
0, //U+F7CB
0, //U+F7CC
0, //U+F7CD
0, //U+F7CE
0, //U+F7CF
0, //U+F7D0
0, //U+F7D1
0, //U+F7D2
0, //U+F7D3
0, //U+F7D4
0, //U+F7D5
0, //U+F7D6
0, //U+F7D7
0, //U+F7D8
0, //U+F7D9
0, //U+F7DA
0, //U+F7DB
0, //U+F7DC
0, //U+F7DD
0, //U+F7DE
0, //U+F7DF
0, //U+F7E0
0, //U+F7E1
0, //U+F7E2
0, //U+F7E3
0, //U+F7E4
0, //U+F7E5
0, //U+F7E6
0, //U+F7E7
0, //U+F7E8
0, //U+F7E9
0, //U+F7EA
0, //U+F7EB
0, //U+F7EC
0, //U+F7ED
0, //U+F7EE
0, //U+F7EF
0, //U+F7F0
0, //U+F7F1
0, //U+F7F2
0, //U+F7F3
0, //U+F7F4
0, //U+F7F5
0, //U+F7F6
0, //U+F7F7
0, //U+F7F8
0, //U+F7F9
0, //U+F7FA
0, //U+F7FB
0, //U+F7FC
0, //U+F7FD
0, //U+F7FE
0, //U+F7FF
0, //U+F800
0, //U+F801
0, //U+F802
0, //U+F803
0, //U+F804
0, //U+F805
0, //U+F806
0, //U+F807
0, //U+F808
0, //U+F809
0, //U+F80A
0, //U+F80B
0, //U+F80C
0, //U+F80D
0, //U+F80E
0, //U+F80F
0, //U+F810
0, //U+F811
0, //U+F812
0, //U+F813
0, //U+F814
0, //U+F815
0, //U+F816
0, //U+F817
0, //U+F818
0, //U+F819
0, //U+F81A
0, //U+F81B
0, //U+F81C
0, //U+F81D
0, //U+F81E
0, //U+F81F
0, //U+F820
0, //U+F821
0, //U+F822
0, //U+F823
0, //U+F824
0, //U+F825
0, //U+F826
0, //U+F827
0, //U+F828
0, //U+F829
0, //U+F82A
0, //U+F82B
0, //U+F82C
0, //U+F82D
0, //U+F82E
0, //U+F82F
0, //U+F830
0, //U+F831
0, //U+F832
0, //U+F833
0, //U+F834
0, //U+F835
0, //U+F836
0, //U+F837
0, //U+F838
0, //U+F839
0, //U+F83A
0, //U+F83B
0, //U+F83C
0, //U+F83D
0, //U+F83E
0, //U+F83F
0, //U+F840
0, //U+F841
0, //U+F842
0, //U+F843
0, //U+F844
0, //U+F845
0, //U+F846
0, //U+F847
0, //U+F848
0, //U+F849
0, //U+F84A
0, //U+F84B
0, //U+F84C
0, //U+F84D
0, //U+F84E
0, //U+F84F
0, //U+F850
0, //U+F851
0, //U+F852
0, //U+F853
0, //U+F854
0, //U+F855
0, //U+F856
0, //U+F857
0, //U+F858
0, //U+F859
0, //U+F85A
0, //U+F85B
0, //U+F85C
0, //U+F85D
0, //U+F85E
0, //U+F85F
0, //U+F860
0, //U+F861
0, //U+F862
0, //U+F863
0, //U+F864
0, //U+F865
0, //U+F866
0, //U+F867
0, //U+F868
0, //U+F869
0, //U+F86A
0, //U+F86B
0, //U+F86C
0, //U+F86D
0, //U+F86E
0, //U+F86F
0, //U+F870
0, //U+F871
0, //U+F872
0, //U+F873
0, //U+F874
0, //U+F875
0, //U+F876
0, //U+F877
0, //U+F878
0, //U+F879
0, //U+F87A
0, //U+F87B
0, //U+F87C
0, //U+F87D
0, //U+F87E
0, //U+F87F
0, //U+F880
0, //U+F881
0, //U+F882
0, //U+F883
0, //U+F884
0, //U+F885
0, //U+F886
0, //U+F887
0, //U+F888
0, //U+F889
0, //U+F88A
0, //U+F88B
0, //U+F88C
0, //U+F88D
0, //U+F88E
0, //U+F88F
0, //U+F890
0, //U+F891
0, //U+F892
0, //U+F893
0, //U+F894
0, //U+F895
0, //U+F896
0, //U+F897
0, //U+F898
0, //U+F899
0, //U+F89A
0, //U+F89B
0, //U+F89C
0, //U+F89D
0, //U+F89E
0, //U+F89F
0, //U+F8A0
0, //U+F8A1
0, //U+F8A2
0, //U+F8A3
0, //U+F8A4
0, //U+F8A5
0, //U+F8A6
0, //U+F8A7
0, //U+F8A8
0, //U+F8A9
0, //U+F8AA
0, //U+F8AB
0, //U+F8AC
0, //U+F8AD
0, //U+F8AE
0, //U+F8AF
0, //U+F8B0
0, //U+F8B1
0, //U+F8B2
0, //U+F8B3
0, //U+F8B4
0, //U+F8B5
0, //U+F8B6
0, //U+F8B7
0, //U+F8B8
0, //U+F8B9
0, //U+F8BA
0, //U+F8BB
0, //U+F8BC
0, //U+F8BD
0, //U+F8BE
0, //U+F8BF
0, //U+F8C0
0, //U+F8C1
0, //U+F8C2
0, //U+F8C3
0, //U+F8C4
0, //U+F8C5
0, //U+F8C6
0, //U+F8C7
0, //U+F8C8
0, //U+F8C9
0, //U+F8CA
0, //U+F8CB
0, //U+F8CC
0, //U+F8CD
0, //U+F8CE
0, //U+F8CF
0, //U+F8D0
0, //U+F8D1
0, //U+F8D2
0, //U+F8D3
0, //U+F8D4
0, //U+F8D5
0, //U+F8D6
0, //U+F8D7
0, //U+F8D8
0, //U+F8D9
0, //U+F8DA
0, //U+F8DB
0, //U+F8DC
0, //U+F8DD
0, //U+F8DE
0, //U+F8DF
0, //U+F8E0
0, //U+F8E1
0, //U+F8E2
0, //U+F8E3
0, //U+F8E4
0, //U+F8E5
0, //U+F8E6
0, //U+F8E7
0, //U+F8E8
0, //U+F8E9
0, //U+F8EA
0, //U+F8EB
0, //U+F8EC
0, //U+F8ED
0, //U+F8EE
0, //U+F8EF
0, //U+F8F0
0, //U+F8F1
0, //U+F8F2
0, //U+F8F3
0, //U+F8F4
0, //U+F8F5
0, //U+F8F6
0, //U+F8F7
0, //U+F8F8
0, //U+F8F9
0, //U+F8FA
0, //U+F8FB
0, //U+F8FC
0, //U+F8FD
0, //U+F8FE
UNICODE_Co, //U+F8FF
UNICODE_Lo, //U+F900
UNICODE_Lo, //U+F901
UNICODE_Lo, //U+F902
UNICODE_Lo, //U+F903
UNICODE_Lo, //U+F904
UNICODE_Lo, //U+F905
UNICODE_Lo, //U+F906
UNICODE_Lo, //U+F907
UNICODE_Lo, //U+F908
UNICODE_Lo, //U+F909
UNICODE_Lo, //U+F90A
UNICODE_Lo, //U+F90B
UNICODE_Lo, //U+F90C
UNICODE_Lo, //U+F90D
UNICODE_Lo, //U+F90E
UNICODE_Lo, //U+F90F
UNICODE_Lo, //U+F910
UNICODE_Lo, //U+F911
UNICODE_Lo, //U+F912
UNICODE_Lo, //U+F913
UNICODE_Lo, //U+F914
UNICODE_Lo, //U+F915
UNICODE_Lo, //U+F916
UNICODE_Lo, //U+F917
UNICODE_Lo, //U+F918
UNICODE_Lo, //U+F919
UNICODE_Lo, //U+F91A
UNICODE_Lo, //U+F91B
UNICODE_Lo, //U+F91C
UNICODE_Lo, //U+F91D
UNICODE_Lo, //U+F91E
UNICODE_Lo, //U+F91F
UNICODE_Lo, //U+F920
UNICODE_Lo, //U+F921
UNICODE_Lo, //U+F922
UNICODE_Lo, //U+F923
UNICODE_Lo, //U+F924
UNICODE_Lo, //U+F925
UNICODE_Lo, //U+F926
UNICODE_Lo, //U+F927
UNICODE_Lo, //U+F928
UNICODE_Lo, //U+F929
UNICODE_Lo, //U+F92A
UNICODE_Lo, //U+F92B
UNICODE_Lo, //U+F92C
UNICODE_Lo, //U+F92D
UNICODE_Lo, //U+F92E
UNICODE_Lo, //U+F92F
UNICODE_Lo, //U+F930
UNICODE_Lo, //U+F931
UNICODE_Lo, //U+F932
UNICODE_Lo, //U+F933
UNICODE_Lo, //U+F934
UNICODE_Lo, //U+F935
UNICODE_Lo, //U+F936
UNICODE_Lo, //U+F937
UNICODE_Lo, //U+F938
UNICODE_Lo, //U+F939
UNICODE_Lo, //U+F93A
UNICODE_Lo, //U+F93B
UNICODE_Lo, //U+F93C
UNICODE_Lo, //U+F93D
UNICODE_Lo, //U+F93E
UNICODE_Lo, //U+F93F
UNICODE_Lo, //U+F940
UNICODE_Lo, //U+F941
UNICODE_Lo, //U+F942
UNICODE_Lo, //U+F943
UNICODE_Lo, //U+F944
UNICODE_Lo, //U+F945
UNICODE_Lo, //U+F946
UNICODE_Lo, //U+F947
UNICODE_Lo, //U+F948
UNICODE_Lo, //U+F949
UNICODE_Lo, //U+F94A
UNICODE_Lo, //U+F94B
UNICODE_Lo, //U+F94C
UNICODE_Lo, //U+F94D
UNICODE_Lo, //U+F94E
UNICODE_Lo, //U+F94F
UNICODE_Lo, //U+F950
UNICODE_Lo, //U+F951
UNICODE_Lo, //U+F952
UNICODE_Lo, //U+F953
UNICODE_Lo, //U+F954
UNICODE_Lo, //U+F955
UNICODE_Lo, //U+F956
UNICODE_Lo, //U+F957
UNICODE_Lo, //U+F958
UNICODE_Lo, //U+F959
UNICODE_Lo, //U+F95A
UNICODE_Lo, //U+F95B
UNICODE_Lo, //U+F95C
UNICODE_Lo, //U+F95D
UNICODE_Lo, //U+F95E
UNICODE_Lo, //U+F95F
UNICODE_Lo, //U+F960
UNICODE_Lo, //U+F961
UNICODE_Lo, //U+F962
UNICODE_Lo, //U+F963
UNICODE_Lo, //U+F964
UNICODE_Lo, //U+F965
UNICODE_Lo, //U+F966
UNICODE_Lo, //U+F967
UNICODE_Lo, //U+F968
UNICODE_Lo, //U+F969
UNICODE_Lo, //U+F96A
UNICODE_Lo, //U+F96B
UNICODE_Lo, //U+F96C
UNICODE_Lo, //U+F96D
UNICODE_Lo, //U+F96E
UNICODE_Lo, //U+F96F
UNICODE_Lo, //U+F970
UNICODE_Lo, //U+F971
UNICODE_Lo, //U+F972
UNICODE_Lo, //U+F973
UNICODE_Lo, //U+F974
UNICODE_Lo, //U+F975
UNICODE_Lo, //U+F976
UNICODE_Lo, //U+F977
UNICODE_Lo, //U+F978
UNICODE_Lo, //U+F979
UNICODE_Lo, //U+F97A
UNICODE_Lo, //U+F97B
UNICODE_Lo, //U+F97C
UNICODE_Lo, //U+F97D
UNICODE_Lo, //U+F97E
UNICODE_Lo, //U+F97F
UNICODE_Lo, //U+F980
UNICODE_Lo, //U+F981
UNICODE_Lo, //U+F982
UNICODE_Lo, //U+F983
UNICODE_Lo, //U+F984
UNICODE_Lo, //U+F985
UNICODE_Lo, //U+F986
UNICODE_Lo, //U+F987
UNICODE_Lo, //U+F988
UNICODE_Lo, //U+F989
UNICODE_Lo, //U+F98A
UNICODE_Lo, //U+F98B
UNICODE_Lo, //U+F98C
UNICODE_Lo, //U+F98D
UNICODE_Lo, //U+F98E
UNICODE_Lo, //U+F98F
UNICODE_Lo, //U+F990
UNICODE_Lo, //U+F991
UNICODE_Lo, //U+F992
UNICODE_Lo, //U+F993
UNICODE_Lo, //U+F994
UNICODE_Lo, //U+F995
UNICODE_Lo, //U+F996
UNICODE_Lo, //U+F997
UNICODE_Lo, //U+F998
UNICODE_Lo, //U+F999
UNICODE_Lo, //U+F99A
UNICODE_Lo, //U+F99B
UNICODE_Lo, //U+F99C
UNICODE_Lo, //U+F99D
UNICODE_Lo, //U+F99E
UNICODE_Lo, //U+F99F
UNICODE_Lo, //U+F9A0
UNICODE_Lo, //U+F9A1
UNICODE_Lo, //U+F9A2
UNICODE_Lo, //U+F9A3
UNICODE_Lo, //U+F9A4
UNICODE_Lo, //U+F9A5
UNICODE_Lo, //U+F9A6
UNICODE_Lo, //U+F9A7
UNICODE_Lo, //U+F9A8
UNICODE_Lo, //U+F9A9
UNICODE_Lo, //U+F9AA
UNICODE_Lo, //U+F9AB
UNICODE_Lo, //U+F9AC
UNICODE_Lo, //U+F9AD
UNICODE_Lo, //U+F9AE
UNICODE_Lo, //U+F9AF
UNICODE_Lo, //U+F9B0
UNICODE_Lo, //U+F9B1
UNICODE_Lo, //U+F9B2
UNICODE_Lo, //U+F9B3
UNICODE_Lo, //U+F9B4
UNICODE_Lo, //U+F9B5
UNICODE_Lo, //U+F9B6
UNICODE_Lo, //U+F9B7
UNICODE_Lo, //U+F9B8
UNICODE_Lo, //U+F9B9
UNICODE_Lo, //U+F9BA
UNICODE_Lo, //U+F9BB
UNICODE_Lo, //U+F9BC
UNICODE_Lo, //U+F9BD
UNICODE_Lo, //U+F9BE
UNICODE_Lo, //U+F9BF
UNICODE_Lo, //U+F9C0
UNICODE_Lo, //U+F9C1
UNICODE_Lo, //U+F9C2
UNICODE_Lo, //U+F9C3
UNICODE_Lo, //U+F9C4
UNICODE_Lo, //U+F9C5
UNICODE_Lo, //U+F9C6
UNICODE_Lo, //U+F9C7
UNICODE_Lo, //U+F9C8
UNICODE_Lo, //U+F9C9
UNICODE_Lo, //U+F9CA
UNICODE_Lo, //U+F9CB
UNICODE_Lo, //U+F9CC
UNICODE_Lo, //U+F9CD
UNICODE_Lo, //U+F9CE
UNICODE_Lo, //U+F9CF
UNICODE_Lo, //U+F9D0
UNICODE_Lo, //U+F9D1
UNICODE_Lo, //U+F9D2
UNICODE_Lo, //U+F9D3
UNICODE_Lo, //U+F9D4
UNICODE_Lo, //U+F9D5
UNICODE_Lo, //U+F9D6
UNICODE_Lo, //U+F9D7
UNICODE_Lo, //U+F9D8
UNICODE_Lo, //U+F9D9
UNICODE_Lo, //U+F9DA
UNICODE_Lo, //U+F9DB
UNICODE_Lo, //U+F9DC
UNICODE_Lo, //U+F9DD
UNICODE_Lo, //U+F9DE
UNICODE_Lo, //U+F9DF
UNICODE_Lo, //U+F9E0
UNICODE_Lo, //U+F9E1
UNICODE_Lo, //U+F9E2
UNICODE_Lo, //U+F9E3
UNICODE_Lo, //U+F9E4
UNICODE_Lo, //U+F9E5
UNICODE_Lo, //U+F9E6
UNICODE_Lo, //U+F9E7
UNICODE_Lo, //U+F9E8
UNICODE_Lo, //U+F9E9
UNICODE_Lo, //U+F9EA
UNICODE_Lo, //U+F9EB
UNICODE_Lo, //U+F9EC
UNICODE_Lo, //U+F9ED
UNICODE_Lo, //U+F9EE
UNICODE_Lo, //U+F9EF
UNICODE_Lo, //U+F9F0
UNICODE_Lo, //U+F9F1
UNICODE_Lo, //U+F9F2
UNICODE_Lo, //U+F9F3
UNICODE_Lo, //U+F9F4
UNICODE_Lo, //U+F9F5
UNICODE_Lo, //U+F9F6
UNICODE_Lo, //U+F9F7
UNICODE_Lo, //U+F9F8
UNICODE_Lo, //U+F9F9
UNICODE_Lo, //U+F9FA
UNICODE_Lo, //U+F9FB
UNICODE_Lo, //U+F9FC
UNICODE_Lo, //U+F9FD
UNICODE_Lo, //U+F9FE
UNICODE_Lo, //U+F9FF
UNICODE_Lo, //U+FA00
UNICODE_Lo, //U+FA01
UNICODE_Lo, //U+FA02
UNICODE_Lo, //U+FA03
UNICODE_Lo, //U+FA04
UNICODE_Lo, //U+FA05
UNICODE_Lo, //U+FA06
UNICODE_Lo, //U+FA07
UNICODE_Lo, //U+FA08
UNICODE_Lo, //U+FA09
UNICODE_Lo, //U+FA0A
UNICODE_Lo, //U+FA0B
UNICODE_Lo, //U+FA0C
UNICODE_Lo, //U+FA0D
UNICODE_Lo, //U+FA0E
UNICODE_Lo, //U+FA0F
UNICODE_Lo, //U+FA10
UNICODE_Lo, //U+FA11
UNICODE_Lo, //U+FA12
UNICODE_Lo, //U+FA13
UNICODE_Lo, //U+FA14
UNICODE_Lo, //U+FA15
UNICODE_Lo, //U+FA16
UNICODE_Lo, //U+FA17
UNICODE_Lo, //U+FA18
UNICODE_Lo, //U+FA19
UNICODE_Lo, //U+FA1A
UNICODE_Lo, //U+FA1B
UNICODE_Lo, //U+FA1C
UNICODE_Lo, //U+FA1D
UNICODE_Lo, //U+FA1E
UNICODE_Lo, //U+FA1F
UNICODE_Lo, //U+FA20
UNICODE_Lo, //U+FA21
UNICODE_Lo, //U+FA22
UNICODE_Lo, //U+FA23
UNICODE_Lo, //U+FA24
UNICODE_Lo, //U+FA25
UNICODE_Lo, //U+FA26
UNICODE_Lo, //U+FA27
UNICODE_Lo, //U+FA28
UNICODE_Lo, //U+FA29
UNICODE_Lo, //U+FA2A
UNICODE_Lo, //U+FA2B
UNICODE_Lo, //U+FA2C
UNICODE_Lo, //U+FA2D
0, //U+FA2E
0, //U+FA2F
UNICODE_Lo, //U+FA30
UNICODE_Lo, //U+FA31
UNICODE_Lo, //U+FA32
UNICODE_Lo, //U+FA33
UNICODE_Lo, //U+FA34
UNICODE_Lo, //U+FA35
UNICODE_Lo, //U+FA36
UNICODE_Lo, //U+FA37
UNICODE_Lo, //U+FA38
UNICODE_Lo, //U+FA39
UNICODE_Lo, //U+FA3A
UNICODE_Lo, //U+FA3B
UNICODE_Lo, //U+FA3C
UNICODE_Lo, //U+FA3D
UNICODE_Lo, //U+FA3E
UNICODE_Lo, //U+FA3F
UNICODE_Lo, //U+FA40
UNICODE_Lo, //U+FA41
UNICODE_Lo, //U+FA42
UNICODE_Lo, //U+FA43
UNICODE_Lo, //U+FA44
UNICODE_Lo, //U+FA45
UNICODE_Lo, //U+FA46
UNICODE_Lo, //U+FA47
UNICODE_Lo, //U+FA48
UNICODE_Lo, //U+FA49
UNICODE_Lo, //U+FA4A
UNICODE_Lo, //U+FA4B
UNICODE_Lo, //U+FA4C
UNICODE_Lo, //U+FA4D
UNICODE_Lo, //U+FA4E
UNICODE_Lo, //U+FA4F
UNICODE_Lo, //U+FA50
UNICODE_Lo, //U+FA51
UNICODE_Lo, //U+FA52
UNICODE_Lo, //U+FA53
UNICODE_Lo, //U+FA54
UNICODE_Lo, //U+FA55
UNICODE_Lo, //U+FA56
UNICODE_Lo, //U+FA57
UNICODE_Lo, //U+FA58
UNICODE_Lo, //U+FA59
UNICODE_Lo, //U+FA5A
UNICODE_Lo, //U+FA5B
UNICODE_Lo, //U+FA5C
UNICODE_Lo, //U+FA5D
UNICODE_Lo, //U+FA5E
UNICODE_Lo, //U+FA5F
UNICODE_Lo, //U+FA60
UNICODE_Lo, //U+FA61
UNICODE_Lo, //U+FA62
UNICODE_Lo, //U+FA63
UNICODE_Lo, //U+FA64
UNICODE_Lo, //U+FA65
UNICODE_Lo, //U+FA66
UNICODE_Lo, //U+FA67
UNICODE_Lo, //U+FA68
UNICODE_Lo, //U+FA69
UNICODE_Lo, //U+FA6A
UNICODE_Lo, //U+FA6B
UNICODE_Lo, //U+FA6C
UNICODE_Lo, //U+FA6D
0, //U+FA6E
0, //U+FA6F
UNICODE_Lo, //U+FA70
UNICODE_Lo, //U+FA71
UNICODE_Lo, //U+FA72
UNICODE_Lo, //U+FA73
UNICODE_Lo, //U+FA74
UNICODE_Lo, //U+FA75
UNICODE_Lo, //U+FA76
UNICODE_Lo, //U+FA77
UNICODE_Lo, //U+FA78
UNICODE_Lo, //U+FA79
UNICODE_Lo, //U+FA7A
UNICODE_Lo, //U+FA7B
UNICODE_Lo, //U+FA7C
UNICODE_Lo, //U+FA7D
UNICODE_Lo, //U+FA7E
UNICODE_Lo, //U+FA7F
UNICODE_Lo, //U+FA80
UNICODE_Lo, //U+FA81
UNICODE_Lo, //U+FA82
UNICODE_Lo, //U+FA83
UNICODE_Lo, //U+FA84
UNICODE_Lo, //U+FA85
UNICODE_Lo, //U+FA86
UNICODE_Lo, //U+FA87
UNICODE_Lo, //U+FA88
UNICODE_Lo, //U+FA89
UNICODE_Lo, //U+FA8A
UNICODE_Lo, //U+FA8B
UNICODE_Lo, //U+FA8C
UNICODE_Lo, //U+FA8D
UNICODE_Lo, //U+FA8E
UNICODE_Lo, //U+FA8F
UNICODE_Lo, //U+FA90
UNICODE_Lo, //U+FA91
UNICODE_Lo, //U+FA92
UNICODE_Lo, //U+FA93
UNICODE_Lo, //U+FA94
UNICODE_Lo, //U+FA95
UNICODE_Lo, //U+FA96
UNICODE_Lo, //U+FA97
UNICODE_Lo, //U+FA98
UNICODE_Lo, //U+FA99
UNICODE_Lo, //U+FA9A
UNICODE_Lo, //U+FA9B
UNICODE_Lo, //U+FA9C
UNICODE_Lo, //U+FA9D
UNICODE_Lo, //U+FA9E
UNICODE_Lo, //U+FA9F
UNICODE_Lo, //U+FAA0
UNICODE_Lo, //U+FAA1
UNICODE_Lo, //U+FAA2
UNICODE_Lo, //U+FAA3
UNICODE_Lo, //U+FAA4
UNICODE_Lo, //U+FAA5
UNICODE_Lo, //U+FAA6
UNICODE_Lo, //U+FAA7
UNICODE_Lo, //U+FAA8
UNICODE_Lo, //U+FAA9
UNICODE_Lo, //U+FAAA
UNICODE_Lo, //U+FAAB
UNICODE_Lo, //U+FAAC
UNICODE_Lo, //U+FAAD
UNICODE_Lo, //U+FAAE
UNICODE_Lo, //U+FAAF
UNICODE_Lo, //U+FAB0
UNICODE_Lo, //U+FAB1
UNICODE_Lo, //U+FAB2
UNICODE_Lo, //U+FAB3
UNICODE_Lo, //U+FAB4
UNICODE_Lo, //U+FAB5
UNICODE_Lo, //U+FAB6
UNICODE_Lo, //U+FAB7
UNICODE_Lo, //U+FAB8
UNICODE_Lo, //U+FAB9
UNICODE_Lo, //U+FABA
UNICODE_Lo, //U+FABB
UNICODE_Lo, //U+FABC
UNICODE_Lo, //U+FABD
UNICODE_Lo, //U+FABE
UNICODE_Lo, //U+FABF
UNICODE_Lo, //U+FAC0
UNICODE_Lo, //U+FAC1
UNICODE_Lo, //U+FAC2
UNICODE_Lo, //U+FAC3
UNICODE_Lo, //U+FAC4
UNICODE_Lo, //U+FAC5
UNICODE_Lo, //U+FAC6
UNICODE_Lo, //U+FAC7
UNICODE_Lo, //U+FAC8
UNICODE_Lo, //U+FAC9
UNICODE_Lo, //U+FACA
UNICODE_Lo, //U+FACB
UNICODE_Lo, //U+FACC
UNICODE_Lo, //U+FACD
UNICODE_Lo, //U+FACE
UNICODE_Lo, //U+FACF
UNICODE_Lo, //U+FAD0
UNICODE_Lo, //U+FAD1
UNICODE_Lo, //U+FAD2
UNICODE_Lo, //U+FAD3
UNICODE_Lo, //U+FAD4
UNICODE_Lo, //U+FAD5
UNICODE_Lo, //U+FAD6
UNICODE_Lo, //U+FAD7
UNICODE_Lo, //U+FAD8
UNICODE_Lo, //U+FAD9
0, //U+FADA
0, //U+FADB
0, //U+FADC
0, //U+FADD
0, //U+FADE
0, //U+FADF
0, //U+FAE0
0, //U+FAE1
0, //U+FAE2
0, //U+FAE3
0, //U+FAE4
0, //U+FAE5
0, //U+FAE6
0, //U+FAE7
0, //U+FAE8
0, //U+FAE9
0, //U+FAEA
0, //U+FAEB
0, //U+FAEC
0, //U+FAED
0, //U+FAEE
0, //U+FAEF
0, //U+FAF0
0, //U+FAF1
0, //U+FAF2
0, //U+FAF3
0, //U+FAF4
0, //U+FAF5
0, //U+FAF6
0, //U+FAF7
0, //U+FAF8
0, //U+FAF9
0, //U+FAFA
0, //U+FAFB
0, //U+FAFC
0, //U+FAFD
0, //U+FAFE
0, //U+FAFF
UNICODE_Ll, //U+FB00
UNICODE_Ll, //U+FB01
UNICODE_Ll, //U+FB02
UNICODE_Ll, //U+FB03
UNICODE_Ll, //U+FB04
UNICODE_Ll, //U+FB05
UNICODE_Ll, //U+FB06
0, //U+FB07
0, //U+FB08
0, //U+FB09
0, //U+FB0A
0, //U+FB0B
0, //U+FB0C
0, //U+FB0D
0, //U+FB0E
0, //U+FB0F
0, //U+FB10
0, //U+FB11
0, //U+FB12
UNICODE_Ll, //U+FB13
UNICODE_Ll, //U+FB14
UNICODE_Ll, //U+FB15
UNICODE_Ll, //U+FB16
UNICODE_Ll, //U+FB17
0, //U+FB18
0, //U+FB19
0, //U+FB1A
0, //U+FB1B
0, //U+FB1C
UNICODE_Lo, //U+FB1D
UNICODE_Mn, //U+FB1E
UNICODE_Lo, //U+FB1F
UNICODE_Lo, //U+FB20
UNICODE_Lo, //U+FB21
UNICODE_Lo, //U+FB22
UNICODE_Lo, //U+FB23
UNICODE_Lo, //U+FB24
UNICODE_Lo, //U+FB25
UNICODE_Lo, //U+FB26
UNICODE_Lo, //U+FB27
UNICODE_Lo, //U+FB28
UNICODE_Sm, //U+FB29
UNICODE_Lo, //U+FB2A
UNICODE_Lo, //U+FB2B
UNICODE_Lo, //U+FB2C
UNICODE_Lo, //U+FB2D
UNICODE_Lo, //U+FB2E
UNICODE_Lo, //U+FB2F
UNICODE_Lo, //U+FB30
UNICODE_Lo, //U+FB31
UNICODE_Lo, //U+FB32
UNICODE_Lo, //U+FB33
UNICODE_Lo, //U+FB34
UNICODE_Lo, //U+FB35
UNICODE_Lo, //U+FB36
0, //U+FB37
UNICODE_Lo, //U+FB38
UNICODE_Lo, //U+FB39
UNICODE_Lo, //U+FB3A
UNICODE_Lo, //U+FB3B
UNICODE_Lo, //U+FB3C
0, //U+FB3D
UNICODE_Lo, //U+FB3E
0, //U+FB3F
UNICODE_Lo, //U+FB40
UNICODE_Lo, //U+FB41
0, //U+FB42
UNICODE_Lo, //U+FB43
UNICODE_Lo, //U+FB44
0, //U+FB45
UNICODE_Lo, //U+FB46
UNICODE_Lo, //U+FB47
UNICODE_Lo, //U+FB48
UNICODE_Lo, //U+FB49
UNICODE_Lo, //U+FB4A
UNICODE_Lo, //U+FB4B
UNICODE_Lo, //U+FB4C
UNICODE_Lo, //U+FB4D
UNICODE_Lo, //U+FB4E
UNICODE_Lo, //U+FB4F
UNICODE_Lo, //U+FB50
UNICODE_Lo, //U+FB51
UNICODE_Lo, //U+FB52
UNICODE_Lo, //U+FB53
UNICODE_Lo, //U+FB54
UNICODE_Lo, //U+FB55
UNICODE_Lo, //U+FB56
UNICODE_Lo, //U+FB57
UNICODE_Lo, //U+FB58
UNICODE_Lo, //U+FB59
UNICODE_Lo, //U+FB5A
UNICODE_Lo, //U+FB5B
UNICODE_Lo, //U+FB5C
UNICODE_Lo, //U+FB5D
UNICODE_Lo, //U+FB5E
UNICODE_Lo, //U+FB5F
UNICODE_Lo, //U+FB60
UNICODE_Lo, //U+FB61
UNICODE_Lo, //U+FB62
UNICODE_Lo, //U+FB63
UNICODE_Lo, //U+FB64
UNICODE_Lo, //U+FB65
UNICODE_Lo, //U+FB66
UNICODE_Lo, //U+FB67
UNICODE_Lo, //U+FB68
UNICODE_Lo, //U+FB69
UNICODE_Lo, //U+FB6A
UNICODE_Lo, //U+FB6B
UNICODE_Lo, //U+FB6C
UNICODE_Lo, //U+FB6D
UNICODE_Lo, //U+FB6E
UNICODE_Lo, //U+FB6F
UNICODE_Lo, //U+FB70
UNICODE_Lo, //U+FB71
UNICODE_Lo, //U+FB72
UNICODE_Lo, //U+FB73
UNICODE_Lo, //U+FB74
UNICODE_Lo, //U+FB75
UNICODE_Lo, //U+FB76
UNICODE_Lo, //U+FB77
UNICODE_Lo, //U+FB78
UNICODE_Lo, //U+FB79
UNICODE_Lo, //U+FB7A
UNICODE_Lo, //U+FB7B
UNICODE_Lo, //U+FB7C
UNICODE_Lo, //U+FB7D
UNICODE_Lo, //U+FB7E
UNICODE_Lo, //U+FB7F
UNICODE_Lo, //U+FB80
UNICODE_Lo, //U+FB81
UNICODE_Lo, //U+FB82
UNICODE_Lo, //U+FB83
UNICODE_Lo, //U+FB84
UNICODE_Lo, //U+FB85
UNICODE_Lo, //U+FB86
UNICODE_Lo, //U+FB87
UNICODE_Lo, //U+FB88
UNICODE_Lo, //U+FB89
UNICODE_Lo, //U+FB8A
UNICODE_Lo, //U+FB8B
UNICODE_Lo, //U+FB8C
UNICODE_Lo, //U+FB8D
UNICODE_Lo, //U+FB8E
UNICODE_Lo, //U+FB8F
UNICODE_Lo, //U+FB90
UNICODE_Lo, //U+FB91
UNICODE_Lo, //U+FB92
UNICODE_Lo, //U+FB93
UNICODE_Lo, //U+FB94
UNICODE_Lo, //U+FB95
UNICODE_Lo, //U+FB96
UNICODE_Lo, //U+FB97
UNICODE_Lo, //U+FB98
UNICODE_Lo, //U+FB99
UNICODE_Lo, //U+FB9A
UNICODE_Lo, //U+FB9B
UNICODE_Lo, //U+FB9C
UNICODE_Lo, //U+FB9D
UNICODE_Lo, //U+FB9E
UNICODE_Lo, //U+FB9F
UNICODE_Lo, //U+FBA0
UNICODE_Lo, //U+FBA1
UNICODE_Lo, //U+FBA2
UNICODE_Lo, //U+FBA3
UNICODE_Lo, //U+FBA4
UNICODE_Lo, //U+FBA5
UNICODE_Lo, //U+FBA6
UNICODE_Lo, //U+FBA7
UNICODE_Lo, //U+FBA8
UNICODE_Lo, //U+FBA9
UNICODE_Lo, //U+FBAA
UNICODE_Lo, //U+FBAB
UNICODE_Lo, //U+FBAC
UNICODE_Lo, //U+FBAD
UNICODE_Lo, //U+FBAE
UNICODE_Lo, //U+FBAF
UNICODE_Lo, //U+FBB0
UNICODE_Lo, //U+FBB1
UNICODE_Sk, //U+FBB2
UNICODE_Sk, //U+FBB3
UNICODE_Sk, //U+FBB4
UNICODE_Sk, //U+FBB5
UNICODE_Sk, //U+FBB6
UNICODE_Sk, //U+FBB7
UNICODE_Sk, //U+FBB8
UNICODE_Sk, //U+FBB9
UNICODE_Sk, //U+FBBA
UNICODE_Sk, //U+FBBB
UNICODE_Sk, //U+FBBC
UNICODE_Sk, //U+FBBD
UNICODE_Sk, //U+FBBE
UNICODE_Sk, //U+FBBF
UNICODE_Sk, //U+FBC0
UNICODE_Sk, //U+FBC1
0, //U+FBC2
0, //U+FBC3
0, //U+FBC4
0, //U+FBC5
0, //U+FBC6
0, //U+FBC7
0, //U+FBC8
0, //U+FBC9
0, //U+FBCA
0, //U+FBCB
0, //U+FBCC
0, //U+FBCD
0, //U+FBCE
0, //U+FBCF
0, //U+FBD0
0, //U+FBD1
0, //U+FBD2
UNICODE_Lo, //U+FBD3
UNICODE_Lo, //U+FBD4
UNICODE_Lo, //U+FBD5
UNICODE_Lo, //U+FBD6
UNICODE_Lo, //U+FBD7
UNICODE_Lo, //U+FBD8
UNICODE_Lo, //U+FBD9
UNICODE_Lo, //U+FBDA
UNICODE_Lo, //U+FBDB
UNICODE_Lo, //U+FBDC
UNICODE_Lo, //U+FBDD
UNICODE_Lo, //U+FBDE
UNICODE_Lo, //U+FBDF
UNICODE_Lo, //U+FBE0
UNICODE_Lo, //U+FBE1
UNICODE_Lo, //U+FBE2
UNICODE_Lo, //U+FBE3
UNICODE_Lo, //U+FBE4
UNICODE_Lo, //U+FBE5
UNICODE_Lo, //U+FBE6
UNICODE_Lo, //U+FBE7
UNICODE_Lo, //U+FBE8
UNICODE_Lo, //U+FBE9
UNICODE_Lo, //U+FBEA
UNICODE_Lo, //U+FBEB
UNICODE_Lo, //U+FBEC
UNICODE_Lo, //U+FBED
UNICODE_Lo, //U+FBEE
UNICODE_Lo, //U+FBEF
UNICODE_Lo, //U+FBF0
UNICODE_Lo, //U+FBF1
UNICODE_Lo, //U+FBF2
UNICODE_Lo, //U+FBF3
UNICODE_Lo, //U+FBF4
UNICODE_Lo, //U+FBF5
UNICODE_Lo, //U+FBF6
UNICODE_Lo, //U+FBF7
UNICODE_Lo, //U+FBF8
UNICODE_Lo, //U+FBF9
UNICODE_Lo, //U+FBFA
UNICODE_Lo, //U+FBFB
UNICODE_Lo, //U+FBFC
UNICODE_Lo, //U+FBFD
UNICODE_Lo, //U+FBFE
UNICODE_Lo, //U+FBFF
UNICODE_Lo, //U+FC00
UNICODE_Lo, //U+FC01
UNICODE_Lo, //U+FC02
UNICODE_Lo, //U+FC03
UNICODE_Lo, //U+FC04
UNICODE_Lo, //U+FC05
UNICODE_Lo, //U+FC06
UNICODE_Lo, //U+FC07
UNICODE_Lo, //U+FC08
UNICODE_Lo, //U+FC09
UNICODE_Lo, //U+FC0A
UNICODE_Lo, //U+FC0B
UNICODE_Lo, //U+FC0C
UNICODE_Lo, //U+FC0D
UNICODE_Lo, //U+FC0E
UNICODE_Lo, //U+FC0F
UNICODE_Lo, //U+FC10
UNICODE_Lo, //U+FC11
UNICODE_Lo, //U+FC12
UNICODE_Lo, //U+FC13
UNICODE_Lo, //U+FC14
UNICODE_Lo, //U+FC15
UNICODE_Lo, //U+FC16
UNICODE_Lo, //U+FC17
UNICODE_Lo, //U+FC18
UNICODE_Lo, //U+FC19
UNICODE_Lo, //U+FC1A
UNICODE_Lo, //U+FC1B
UNICODE_Lo, //U+FC1C
UNICODE_Lo, //U+FC1D
UNICODE_Lo, //U+FC1E
UNICODE_Lo, //U+FC1F
UNICODE_Lo, //U+FC20
UNICODE_Lo, //U+FC21
UNICODE_Lo, //U+FC22
UNICODE_Lo, //U+FC23
UNICODE_Lo, //U+FC24
UNICODE_Lo, //U+FC25
UNICODE_Lo, //U+FC26
UNICODE_Lo, //U+FC27
UNICODE_Lo, //U+FC28
UNICODE_Lo, //U+FC29
UNICODE_Lo, //U+FC2A
UNICODE_Lo, //U+FC2B
UNICODE_Lo, //U+FC2C
UNICODE_Lo, //U+FC2D
UNICODE_Lo, //U+FC2E
UNICODE_Lo, //U+FC2F
UNICODE_Lo, //U+FC30
UNICODE_Lo, //U+FC31
UNICODE_Lo, //U+FC32
UNICODE_Lo, //U+FC33
UNICODE_Lo, //U+FC34
UNICODE_Lo, //U+FC35
UNICODE_Lo, //U+FC36
UNICODE_Lo, //U+FC37
UNICODE_Lo, //U+FC38
UNICODE_Lo, //U+FC39
UNICODE_Lo, //U+FC3A
UNICODE_Lo, //U+FC3B
UNICODE_Lo, //U+FC3C
UNICODE_Lo, //U+FC3D
UNICODE_Lo, //U+FC3E
UNICODE_Lo, //U+FC3F
UNICODE_Lo, //U+FC40
UNICODE_Lo, //U+FC41
UNICODE_Lo, //U+FC42
UNICODE_Lo, //U+FC43
UNICODE_Lo, //U+FC44
UNICODE_Lo, //U+FC45
UNICODE_Lo, //U+FC46
UNICODE_Lo, //U+FC47
UNICODE_Lo, //U+FC48
UNICODE_Lo, //U+FC49
UNICODE_Lo, //U+FC4A
UNICODE_Lo, //U+FC4B
UNICODE_Lo, //U+FC4C
UNICODE_Lo, //U+FC4D
UNICODE_Lo, //U+FC4E
UNICODE_Lo, //U+FC4F
UNICODE_Lo, //U+FC50
UNICODE_Lo, //U+FC51
UNICODE_Lo, //U+FC52
UNICODE_Lo, //U+FC53
UNICODE_Lo, //U+FC54
UNICODE_Lo, //U+FC55
UNICODE_Lo, //U+FC56
UNICODE_Lo, //U+FC57
UNICODE_Lo, //U+FC58
UNICODE_Lo, //U+FC59
UNICODE_Lo, //U+FC5A
UNICODE_Lo, //U+FC5B
UNICODE_Lo, //U+FC5C
UNICODE_Lo, //U+FC5D
UNICODE_Lo, //U+FC5E
UNICODE_Lo, //U+FC5F
UNICODE_Lo, //U+FC60
UNICODE_Lo, //U+FC61
UNICODE_Lo, //U+FC62
UNICODE_Lo, //U+FC63
UNICODE_Lo, //U+FC64
UNICODE_Lo, //U+FC65
UNICODE_Lo, //U+FC66
UNICODE_Lo, //U+FC67
UNICODE_Lo, //U+FC68
UNICODE_Lo, //U+FC69
UNICODE_Lo, //U+FC6A
UNICODE_Lo, //U+FC6B
UNICODE_Lo, //U+FC6C
UNICODE_Lo, //U+FC6D
UNICODE_Lo, //U+FC6E
UNICODE_Lo, //U+FC6F
UNICODE_Lo, //U+FC70
UNICODE_Lo, //U+FC71
UNICODE_Lo, //U+FC72
UNICODE_Lo, //U+FC73
UNICODE_Lo, //U+FC74
UNICODE_Lo, //U+FC75
UNICODE_Lo, //U+FC76
UNICODE_Lo, //U+FC77
UNICODE_Lo, //U+FC78
UNICODE_Lo, //U+FC79
UNICODE_Lo, //U+FC7A
UNICODE_Lo, //U+FC7B
UNICODE_Lo, //U+FC7C
UNICODE_Lo, //U+FC7D
UNICODE_Lo, //U+FC7E
UNICODE_Lo, //U+FC7F
UNICODE_Lo, //U+FC80
UNICODE_Lo, //U+FC81
UNICODE_Lo, //U+FC82
UNICODE_Lo, //U+FC83
UNICODE_Lo, //U+FC84
UNICODE_Lo, //U+FC85
UNICODE_Lo, //U+FC86
UNICODE_Lo, //U+FC87
UNICODE_Lo, //U+FC88
UNICODE_Lo, //U+FC89
UNICODE_Lo, //U+FC8A
UNICODE_Lo, //U+FC8B
UNICODE_Lo, //U+FC8C
UNICODE_Lo, //U+FC8D
UNICODE_Lo, //U+FC8E
UNICODE_Lo, //U+FC8F
UNICODE_Lo, //U+FC90
UNICODE_Lo, //U+FC91
UNICODE_Lo, //U+FC92
UNICODE_Lo, //U+FC93
UNICODE_Lo, //U+FC94
UNICODE_Lo, //U+FC95
UNICODE_Lo, //U+FC96
UNICODE_Lo, //U+FC97
UNICODE_Lo, //U+FC98
UNICODE_Lo, //U+FC99
UNICODE_Lo, //U+FC9A
UNICODE_Lo, //U+FC9B
UNICODE_Lo, //U+FC9C
UNICODE_Lo, //U+FC9D
UNICODE_Lo, //U+FC9E
UNICODE_Lo, //U+FC9F
UNICODE_Lo, //U+FCA0
UNICODE_Lo, //U+FCA1
UNICODE_Lo, //U+FCA2
UNICODE_Lo, //U+FCA3
UNICODE_Lo, //U+FCA4
UNICODE_Lo, //U+FCA5
UNICODE_Lo, //U+FCA6
UNICODE_Lo, //U+FCA7
UNICODE_Lo, //U+FCA8
UNICODE_Lo, //U+FCA9
UNICODE_Lo, //U+FCAA
UNICODE_Lo, //U+FCAB
UNICODE_Lo, //U+FCAC
UNICODE_Lo, //U+FCAD
UNICODE_Lo, //U+FCAE
UNICODE_Lo, //U+FCAF
UNICODE_Lo, //U+FCB0
UNICODE_Lo, //U+FCB1
UNICODE_Lo, //U+FCB2
UNICODE_Lo, //U+FCB3
UNICODE_Lo, //U+FCB4
UNICODE_Lo, //U+FCB5
UNICODE_Lo, //U+FCB6
UNICODE_Lo, //U+FCB7
UNICODE_Lo, //U+FCB8
UNICODE_Lo, //U+FCB9
UNICODE_Lo, //U+FCBA
UNICODE_Lo, //U+FCBB
UNICODE_Lo, //U+FCBC
UNICODE_Lo, //U+FCBD
UNICODE_Lo, //U+FCBE
UNICODE_Lo, //U+FCBF
UNICODE_Lo, //U+FCC0
UNICODE_Lo, //U+FCC1
UNICODE_Lo, //U+FCC2
UNICODE_Lo, //U+FCC3
UNICODE_Lo, //U+FCC4
UNICODE_Lo, //U+FCC5
UNICODE_Lo, //U+FCC6
UNICODE_Lo, //U+FCC7
UNICODE_Lo, //U+FCC8
UNICODE_Lo, //U+FCC9
UNICODE_Lo, //U+FCCA
UNICODE_Lo, //U+FCCB
UNICODE_Lo, //U+FCCC
UNICODE_Lo, //U+FCCD
UNICODE_Lo, //U+FCCE
UNICODE_Lo, //U+FCCF
UNICODE_Lo, //U+FCD0
UNICODE_Lo, //U+FCD1
UNICODE_Lo, //U+FCD2
UNICODE_Lo, //U+FCD3
UNICODE_Lo, //U+FCD4
UNICODE_Lo, //U+FCD5
UNICODE_Lo, //U+FCD6
UNICODE_Lo, //U+FCD7
UNICODE_Lo, //U+FCD8
UNICODE_Lo, //U+FCD9
UNICODE_Lo, //U+FCDA
UNICODE_Lo, //U+FCDB
UNICODE_Lo, //U+FCDC
UNICODE_Lo, //U+FCDD
UNICODE_Lo, //U+FCDE
UNICODE_Lo, //U+FCDF
UNICODE_Lo, //U+FCE0
UNICODE_Lo, //U+FCE1
UNICODE_Lo, //U+FCE2
UNICODE_Lo, //U+FCE3
UNICODE_Lo, //U+FCE4
UNICODE_Lo, //U+FCE5
UNICODE_Lo, //U+FCE6
UNICODE_Lo, //U+FCE7
UNICODE_Lo, //U+FCE8
UNICODE_Lo, //U+FCE9
UNICODE_Lo, //U+FCEA
UNICODE_Lo, //U+FCEB
UNICODE_Lo, //U+FCEC
UNICODE_Lo, //U+FCED
UNICODE_Lo, //U+FCEE
UNICODE_Lo, //U+FCEF
UNICODE_Lo, //U+FCF0
UNICODE_Lo, //U+FCF1
UNICODE_Lo, //U+FCF2
UNICODE_Lo, //U+FCF3
UNICODE_Lo, //U+FCF4
UNICODE_Lo, //U+FCF5
UNICODE_Lo, //U+FCF6
UNICODE_Lo, //U+FCF7
UNICODE_Lo, //U+FCF8
UNICODE_Lo, //U+FCF9
UNICODE_Lo, //U+FCFA
UNICODE_Lo, //U+FCFB
UNICODE_Lo, //U+FCFC
UNICODE_Lo, //U+FCFD
UNICODE_Lo, //U+FCFE
UNICODE_Lo, //U+FCFF
UNICODE_Lo, //U+FD00
UNICODE_Lo, //U+FD01
UNICODE_Lo, //U+FD02
UNICODE_Lo, //U+FD03
UNICODE_Lo, //U+FD04
UNICODE_Lo, //U+FD05
UNICODE_Lo, //U+FD06
UNICODE_Lo, //U+FD07
UNICODE_Lo, //U+FD08
UNICODE_Lo, //U+FD09
UNICODE_Lo, //U+FD0A
UNICODE_Lo, //U+FD0B
UNICODE_Lo, //U+FD0C
UNICODE_Lo, //U+FD0D
UNICODE_Lo, //U+FD0E
UNICODE_Lo, //U+FD0F
UNICODE_Lo, //U+FD10
UNICODE_Lo, //U+FD11
UNICODE_Lo, //U+FD12
UNICODE_Lo, //U+FD13
UNICODE_Lo, //U+FD14
UNICODE_Lo, //U+FD15
UNICODE_Lo, //U+FD16
UNICODE_Lo, //U+FD17
UNICODE_Lo, //U+FD18
UNICODE_Lo, //U+FD19
UNICODE_Lo, //U+FD1A
UNICODE_Lo, //U+FD1B
UNICODE_Lo, //U+FD1C
UNICODE_Lo, //U+FD1D
UNICODE_Lo, //U+FD1E
UNICODE_Lo, //U+FD1F
UNICODE_Lo, //U+FD20
UNICODE_Lo, //U+FD21
UNICODE_Lo, //U+FD22
UNICODE_Lo, //U+FD23
UNICODE_Lo, //U+FD24
UNICODE_Lo, //U+FD25
UNICODE_Lo, //U+FD26
UNICODE_Lo, //U+FD27
UNICODE_Lo, //U+FD28
UNICODE_Lo, //U+FD29
UNICODE_Lo, //U+FD2A
UNICODE_Lo, //U+FD2B
UNICODE_Lo, //U+FD2C
UNICODE_Lo, //U+FD2D
UNICODE_Lo, //U+FD2E
UNICODE_Lo, //U+FD2F
UNICODE_Lo, //U+FD30
UNICODE_Lo, //U+FD31
UNICODE_Lo, //U+FD32
UNICODE_Lo, //U+FD33
UNICODE_Lo, //U+FD34
UNICODE_Lo, //U+FD35
UNICODE_Lo, //U+FD36
UNICODE_Lo, //U+FD37
UNICODE_Lo, //U+FD38
UNICODE_Lo, //U+FD39
UNICODE_Lo, //U+FD3A
UNICODE_Lo, //U+FD3B
UNICODE_Lo, //U+FD3C
UNICODE_Lo, //U+FD3D
UNICODE_Ps, //U+FD3E
UNICODE_Pe, //U+FD3F
0, //U+FD40
0, //U+FD41
0, //U+FD42
0, //U+FD43
0, //U+FD44
0, //U+FD45
0, //U+FD46
0, //U+FD47
0, //U+FD48
0, //U+FD49
0, //U+FD4A
0, //U+FD4B
0, //U+FD4C
0, //U+FD4D
0, //U+FD4E
0, //U+FD4F
UNICODE_Lo, //U+FD50
UNICODE_Lo, //U+FD51
UNICODE_Lo, //U+FD52
UNICODE_Lo, //U+FD53
UNICODE_Lo, //U+FD54
UNICODE_Lo, //U+FD55
UNICODE_Lo, //U+FD56
UNICODE_Lo, //U+FD57
UNICODE_Lo, //U+FD58
UNICODE_Lo, //U+FD59
UNICODE_Lo, //U+FD5A
UNICODE_Lo, //U+FD5B
UNICODE_Lo, //U+FD5C
UNICODE_Lo, //U+FD5D
UNICODE_Lo, //U+FD5E
UNICODE_Lo, //U+FD5F
UNICODE_Lo, //U+FD60
UNICODE_Lo, //U+FD61
UNICODE_Lo, //U+FD62
UNICODE_Lo, //U+FD63
UNICODE_Lo, //U+FD64
UNICODE_Lo, //U+FD65
UNICODE_Lo, //U+FD66
UNICODE_Lo, //U+FD67
UNICODE_Lo, //U+FD68
UNICODE_Lo, //U+FD69
UNICODE_Lo, //U+FD6A
UNICODE_Lo, //U+FD6B
UNICODE_Lo, //U+FD6C
UNICODE_Lo, //U+FD6D
UNICODE_Lo, //U+FD6E
UNICODE_Lo, //U+FD6F
UNICODE_Lo, //U+FD70
UNICODE_Lo, //U+FD71
UNICODE_Lo, //U+FD72
UNICODE_Lo, //U+FD73
UNICODE_Lo, //U+FD74
UNICODE_Lo, //U+FD75
UNICODE_Lo, //U+FD76
UNICODE_Lo, //U+FD77
UNICODE_Lo, //U+FD78
UNICODE_Lo, //U+FD79
UNICODE_Lo, //U+FD7A
UNICODE_Lo, //U+FD7B
UNICODE_Lo, //U+FD7C
UNICODE_Lo, //U+FD7D
UNICODE_Lo, //U+FD7E
UNICODE_Lo, //U+FD7F
UNICODE_Lo, //U+FD80
UNICODE_Lo, //U+FD81
UNICODE_Lo, //U+FD82
UNICODE_Lo, //U+FD83
UNICODE_Lo, //U+FD84
UNICODE_Lo, //U+FD85
UNICODE_Lo, //U+FD86
UNICODE_Lo, //U+FD87
UNICODE_Lo, //U+FD88
UNICODE_Lo, //U+FD89
UNICODE_Lo, //U+FD8A
UNICODE_Lo, //U+FD8B
UNICODE_Lo, //U+FD8C
UNICODE_Lo, //U+FD8D
UNICODE_Lo, //U+FD8E
UNICODE_Lo, //U+FD8F
0, //U+FD90
0, //U+FD91
UNICODE_Lo, //U+FD92
UNICODE_Lo, //U+FD93
UNICODE_Lo, //U+FD94
UNICODE_Lo, //U+FD95
UNICODE_Lo, //U+FD96
UNICODE_Lo, //U+FD97
UNICODE_Lo, //U+FD98
UNICODE_Lo, //U+FD99
UNICODE_Lo, //U+FD9A
UNICODE_Lo, //U+FD9B
UNICODE_Lo, //U+FD9C
UNICODE_Lo, //U+FD9D
UNICODE_Lo, //U+FD9E
UNICODE_Lo, //U+FD9F
UNICODE_Lo, //U+FDA0
UNICODE_Lo, //U+FDA1
UNICODE_Lo, //U+FDA2
UNICODE_Lo, //U+FDA3
UNICODE_Lo, //U+FDA4
UNICODE_Lo, //U+FDA5
UNICODE_Lo, //U+FDA6
UNICODE_Lo, //U+FDA7
UNICODE_Lo, //U+FDA8
UNICODE_Lo, //U+FDA9
UNICODE_Lo, //U+FDAA
UNICODE_Lo, //U+FDAB
UNICODE_Lo, //U+FDAC
UNICODE_Lo, //U+FDAD
UNICODE_Lo, //U+FDAE
UNICODE_Lo, //U+FDAF
UNICODE_Lo, //U+FDB0
UNICODE_Lo, //U+FDB1
UNICODE_Lo, //U+FDB2
UNICODE_Lo, //U+FDB3
UNICODE_Lo, //U+FDB4
UNICODE_Lo, //U+FDB5
UNICODE_Lo, //U+FDB6
UNICODE_Lo, //U+FDB7
UNICODE_Lo, //U+FDB8
UNICODE_Lo, //U+FDB9
UNICODE_Lo, //U+FDBA
UNICODE_Lo, //U+FDBB
UNICODE_Lo, //U+FDBC
UNICODE_Lo, //U+FDBD
UNICODE_Lo, //U+FDBE
UNICODE_Lo, //U+FDBF
UNICODE_Lo, //U+FDC0
UNICODE_Lo, //U+FDC1
UNICODE_Lo, //U+FDC2
UNICODE_Lo, //U+FDC3
UNICODE_Lo, //U+FDC4
UNICODE_Lo, //U+FDC5
UNICODE_Lo, //U+FDC6
UNICODE_Lo, //U+FDC7
0, //U+FDC8
0, //U+FDC9
0, //U+FDCA
0, //U+FDCB
0, //U+FDCC
0, //U+FDCD
0, //U+FDCE
0, //U+FDCF
0, //U+FDD0
0, //U+FDD1
0, //U+FDD2
0, //U+FDD3
0, //U+FDD4
0, //U+FDD5
0, //U+FDD6
0, //U+FDD7
0, //U+FDD8
0, //U+FDD9
0, //U+FDDA
0, //U+FDDB
0, //U+FDDC
0, //U+FDDD
0, //U+FDDE
0, //U+FDDF
0, //U+FDE0
0, //U+FDE1
0, //U+FDE2
0, //U+FDE3
0, //U+FDE4
0, //U+FDE5
0, //U+FDE6
0, //U+FDE7
0, //U+FDE8
0, //U+FDE9
0, //U+FDEA
0, //U+FDEB
0, //U+FDEC
0, //U+FDED
0, //U+FDEE
0, //U+FDEF
UNICODE_Lo, //U+FDF0
UNICODE_Lo, //U+FDF1
UNICODE_Lo, //U+FDF2
UNICODE_Lo, //U+FDF3
UNICODE_Lo, //U+FDF4
UNICODE_Lo, //U+FDF5
UNICODE_Lo, //U+FDF6
UNICODE_Lo, //U+FDF7
UNICODE_Lo, //U+FDF8
UNICODE_Lo, //U+FDF9
UNICODE_Lo, //U+FDFA
UNICODE_Lo, //U+FDFB
UNICODE_Sc, //U+FDFC
UNICODE_So, //U+FDFD
0, //U+FDFE
0, //U+FDFF
UNICODE_Mn, //U+FE00
UNICODE_Mn, //U+FE01
UNICODE_Mn, //U+FE02
UNICODE_Mn, //U+FE03
UNICODE_Mn, //U+FE04
UNICODE_Mn, //U+FE05
UNICODE_Mn, //U+FE06
UNICODE_Mn, //U+FE07
UNICODE_Mn, //U+FE08
UNICODE_Mn, //U+FE09
UNICODE_Mn, //U+FE0A
UNICODE_Mn, //U+FE0B
UNICODE_Mn, //U+FE0C
UNICODE_Mn, //U+FE0D
UNICODE_Mn, //U+FE0E
UNICODE_Mn, //U+FE0F
UNICODE_Po, //U+FE10
UNICODE_Po, //U+FE11
UNICODE_Po, //U+FE12
UNICODE_Po, //U+FE13
UNICODE_Po, //U+FE14
UNICODE_Po, //U+FE15
UNICODE_Po, //U+FE16
UNICODE_Ps, //U+FE17
UNICODE_Pe, //U+FE18
UNICODE_Po, //U+FE19
0, //U+FE1A
0, //U+FE1B
0, //U+FE1C
0, //U+FE1D
0, //U+FE1E
0, //U+FE1F
UNICODE_Mn, //U+FE20
UNICODE_Mn, //U+FE21
UNICODE_Mn, //U+FE22
UNICODE_Mn, //U+FE23
UNICODE_Mn, //U+FE24
UNICODE_Mn, //U+FE25
UNICODE_Mn, //U+FE26
0, //U+FE27
0, //U+FE28
0, //U+FE29
0, //U+FE2A
0, //U+FE2B
0, //U+FE2C
0, //U+FE2D
0, //U+FE2E
0, //U+FE2F
UNICODE_Po, //U+FE30
UNICODE_Pd, //U+FE31
UNICODE_Pd, //U+FE32
UNICODE_Pc, //U+FE33
UNICODE_Pc, //U+FE34
UNICODE_Ps, //U+FE35
UNICODE_Pe, //U+FE36
UNICODE_Ps, //U+FE37
UNICODE_Pe, //U+FE38
UNICODE_Ps, //U+FE39
UNICODE_Pe, //U+FE3A
UNICODE_Ps, //U+FE3B
UNICODE_Pe, //U+FE3C
UNICODE_Ps, //U+FE3D
UNICODE_Pe, //U+FE3E
UNICODE_Ps, //U+FE3F
UNICODE_Pe, //U+FE40
UNICODE_Ps, //U+FE41
UNICODE_Pe, //U+FE42
UNICODE_Ps, //U+FE43
UNICODE_Pe, //U+FE44
UNICODE_Po, //U+FE45
UNICODE_Po, //U+FE46
UNICODE_Ps, //U+FE47
UNICODE_Pe, //U+FE48
UNICODE_Po, //U+FE49
UNICODE_Po, //U+FE4A
UNICODE_Po, //U+FE4B
UNICODE_Po, //U+FE4C
UNICODE_Pc, //U+FE4D
UNICODE_Pc, //U+FE4E
UNICODE_Pc, //U+FE4F
UNICODE_Po, //U+FE50
UNICODE_Po, //U+FE51
UNICODE_Po, //U+FE52
0, //U+FE53
UNICODE_Po, //U+FE54
UNICODE_Po, //U+FE55
UNICODE_Po, //U+FE56
UNICODE_Po, //U+FE57
UNICODE_Pd, //U+FE58
UNICODE_Ps, //U+FE59
UNICODE_Pe, //U+FE5A
UNICODE_Ps, //U+FE5B
UNICODE_Pe, //U+FE5C
UNICODE_Ps, //U+FE5D
UNICODE_Pe, //U+FE5E
UNICODE_Po, //U+FE5F
UNICODE_Po, //U+FE60
UNICODE_Po, //U+FE61
UNICODE_Sm, //U+FE62
UNICODE_Pd, //U+FE63
UNICODE_Sm, //U+FE64
UNICODE_Sm, //U+FE65
UNICODE_Sm, //U+FE66
0, //U+FE67
UNICODE_Po, //U+FE68
UNICODE_Sc, //U+FE69
UNICODE_Po, //U+FE6A
UNICODE_Po, //U+FE6B
0, //U+FE6C
0, //U+FE6D
0, //U+FE6E
0, //U+FE6F
UNICODE_Lo, //U+FE70
UNICODE_Lo, //U+FE71
UNICODE_Lo, //U+FE72
UNICODE_Lo, //U+FE73
UNICODE_Lo, //U+FE74
0, //U+FE75
UNICODE_Lo, //U+FE76
UNICODE_Lo, //U+FE77
UNICODE_Lo, //U+FE78
UNICODE_Lo, //U+FE79
UNICODE_Lo, //U+FE7A
UNICODE_Lo, //U+FE7B
UNICODE_Lo, //U+FE7C
UNICODE_Lo, //U+FE7D
UNICODE_Lo, //U+FE7E
UNICODE_Lo, //U+FE7F
UNICODE_Lo, //U+FE80
UNICODE_Lo, //U+FE81
UNICODE_Lo, //U+FE82
UNICODE_Lo, //U+FE83
UNICODE_Lo, //U+FE84
UNICODE_Lo, //U+FE85
UNICODE_Lo, //U+FE86
UNICODE_Lo, //U+FE87
UNICODE_Lo, //U+FE88
UNICODE_Lo, //U+FE89
UNICODE_Lo, //U+FE8A
UNICODE_Lo, //U+FE8B
UNICODE_Lo, //U+FE8C
UNICODE_Lo, //U+FE8D
UNICODE_Lo, //U+FE8E
UNICODE_Lo, //U+FE8F
UNICODE_Lo, //U+FE90
UNICODE_Lo, //U+FE91
UNICODE_Lo, //U+FE92
UNICODE_Lo, //U+FE93
UNICODE_Lo, //U+FE94
UNICODE_Lo, //U+FE95
UNICODE_Lo, //U+FE96
UNICODE_Lo, //U+FE97
UNICODE_Lo, //U+FE98
UNICODE_Lo, //U+FE99
UNICODE_Lo, //U+FE9A
UNICODE_Lo, //U+FE9B
UNICODE_Lo, //U+FE9C
UNICODE_Lo, //U+FE9D
UNICODE_Lo, //U+FE9E
UNICODE_Lo, //U+FE9F
UNICODE_Lo, //U+FEA0
UNICODE_Lo, //U+FEA1
UNICODE_Lo, //U+FEA2
UNICODE_Lo, //U+FEA3
UNICODE_Lo, //U+FEA4
UNICODE_Lo, //U+FEA5
UNICODE_Lo, //U+FEA6
UNICODE_Lo, //U+FEA7
UNICODE_Lo, //U+FEA8
UNICODE_Lo, //U+FEA9
UNICODE_Lo, //U+FEAA
UNICODE_Lo, //U+FEAB
UNICODE_Lo, //U+FEAC
UNICODE_Lo, //U+FEAD
UNICODE_Lo, //U+FEAE
UNICODE_Lo, //U+FEAF
UNICODE_Lo, //U+FEB0
UNICODE_Lo, //U+FEB1
UNICODE_Lo, //U+FEB2
UNICODE_Lo, //U+FEB3
UNICODE_Lo, //U+FEB4
UNICODE_Lo, //U+FEB5
UNICODE_Lo, //U+FEB6
UNICODE_Lo, //U+FEB7
UNICODE_Lo, //U+FEB8
UNICODE_Lo, //U+FEB9
UNICODE_Lo, //U+FEBA
UNICODE_Lo, //U+FEBB
UNICODE_Lo, //U+FEBC
UNICODE_Lo, //U+FEBD
UNICODE_Lo, //U+FEBE
UNICODE_Lo, //U+FEBF
UNICODE_Lo, //U+FEC0
UNICODE_Lo, //U+FEC1
UNICODE_Lo, //U+FEC2
UNICODE_Lo, //U+FEC3
UNICODE_Lo, //U+FEC4
UNICODE_Lo, //U+FEC5
UNICODE_Lo, //U+FEC6
UNICODE_Lo, //U+FEC7
UNICODE_Lo, //U+FEC8
UNICODE_Lo, //U+FEC9
UNICODE_Lo, //U+FECA
UNICODE_Lo, //U+FECB
UNICODE_Lo, //U+FECC
UNICODE_Lo, //U+FECD
UNICODE_Lo, //U+FECE
UNICODE_Lo, //U+FECF
UNICODE_Lo, //U+FED0
UNICODE_Lo, //U+FED1
UNICODE_Lo, //U+FED2
UNICODE_Lo, //U+FED3
UNICODE_Lo, //U+FED4
UNICODE_Lo, //U+FED5
UNICODE_Lo, //U+FED6
UNICODE_Lo, //U+FED7
UNICODE_Lo, //U+FED8
UNICODE_Lo, //U+FED9
UNICODE_Lo, //U+FEDA
UNICODE_Lo, //U+FEDB
UNICODE_Lo, //U+FEDC
UNICODE_Lo, //U+FEDD
UNICODE_Lo, //U+FEDE
UNICODE_Lo, //U+FEDF
UNICODE_Lo, //U+FEE0
UNICODE_Lo, //U+FEE1
UNICODE_Lo, //U+FEE2
UNICODE_Lo, //U+FEE3
UNICODE_Lo, //U+FEE4
UNICODE_Lo, //U+FEE5
UNICODE_Lo, //U+FEE6
UNICODE_Lo, //U+FEE7
UNICODE_Lo, //U+FEE8
UNICODE_Lo, //U+FEE9
UNICODE_Lo, //U+FEEA
UNICODE_Lo, //U+FEEB
UNICODE_Lo, //U+FEEC
UNICODE_Lo, //U+FEED
UNICODE_Lo, //U+FEEE
UNICODE_Lo, //U+FEEF
UNICODE_Lo, //U+FEF0
UNICODE_Lo, //U+FEF1
UNICODE_Lo, //U+FEF2
UNICODE_Lo, //U+FEF3
UNICODE_Lo, //U+FEF4
UNICODE_Lo, //U+FEF5
UNICODE_Lo, //U+FEF6
UNICODE_Lo, //U+FEF7
UNICODE_Lo, //U+FEF8
UNICODE_Lo, //U+FEF9
UNICODE_Lo, //U+FEFA
UNICODE_Lo, //U+FEFB
UNICODE_Lo, //U+FEFC
0, //U+FEFD
0, //U+FEFE
UNICODE_Cf, //U+FEFF
0, //U+FF00
UNICODE_Po, //U+FF01
UNICODE_Po, //U+FF02
UNICODE_Po, //U+FF03
UNICODE_Sc, //U+FF04
UNICODE_Po, //U+FF05
UNICODE_Po, //U+FF06
UNICODE_Po, //U+FF07
UNICODE_Ps, //U+FF08
UNICODE_Pe, //U+FF09
UNICODE_Po, //U+FF0A
UNICODE_Sm, //U+FF0B
UNICODE_Po, //U+FF0C
UNICODE_Pd, //U+FF0D
UNICODE_Po, //U+FF0E
UNICODE_Po, //U+FF0F
UNICODE_Nd, //U+FF10
UNICODE_Nd, //U+FF11
UNICODE_Nd, //U+FF12
UNICODE_Nd, //U+FF13
UNICODE_Nd, //U+FF14
UNICODE_Nd, //U+FF15
UNICODE_Nd, //U+FF16
UNICODE_Nd, //U+FF17
UNICODE_Nd, //U+FF18
UNICODE_Nd, //U+FF19
UNICODE_Po, //U+FF1A
UNICODE_Po, //U+FF1B
UNICODE_Sm, //U+FF1C
UNICODE_Sm, //U+FF1D
UNICODE_Sm, //U+FF1E
UNICODE_Po, //U+FF1F
UNICODE_Po, //U+FF20
UNICODE_Lu, //U+FF21
UNICODE_Lu, //U+FF22
UNICODE_Lu, //U+FF23
UNICODE_Lu, //U+FF24
UNICODE_Lu, //U+FF25
UNICODE_Lu, //U+FF26
UNICODE_Lu, //U+FF27
UNICODE_Lu, //U+FF28
UNICODE_Lu, //U+FF29
UNICODE_Lu, //U+FF2A
UNICODE_Lu, //U+FF2B
UNICODE_Lu, //U+FF2C
UNICODE_Lu, //U+FF2D
UNICODE_Lu, //U+FF2E
UNICODE_Lu, //U+FF2F
UNICODE_Lu, //U+FF30
UNICODE_Lu, //U+FF31
UNICODE_Lu, //U+FF32
UNICODE_Lu, //U+FF33
UNICODE_Lu, //U+FF34
UNICODE_Lu, //U+FF35
UNICODE_Lu, //U+FF36
UNICODE_Lu, //U+FF37
UNICODE_Lu, //U+FF38
UNICODE_Lu, //U+FF39
UNICODE_Lu, //U+FF3A
UNICODE_Ps, //U+FF3B
UNICODE_Po, //U+FF3C
UNICODE_Pe, //U+FF3D
UNICODE_Sk, //U+FF3E
UNICODE_Pc, //U+FF3F
UNICODE_Sk, //U+FF40
UNICODE_Ll, //U+FF41
UNICODE_Ll, //U+FF42
UNICODE_Ll, //U+FF43
UNICODE_Ll, //U+FF44
UNICODE_Ll, //U+FF45
UNICODE_Ll, //U+FF46
UNICODE_Ll, //U+FF47
UNICODE_Ll, //U+FF48
UNICODE_Ll, //U+FF49
UNICODE_Ll, //U+FF4A
UNICODE_Ll, //U+FF4B
UNICODE_Ll, //U+FF4C
UNICODE_Ll, //U+FF4D
UNICODE_Ll, //U+FF4E
UNICODE_Ll, //U+FF4F
UNICODE_Ll, //U+FF50
UNICODE_Ll, //U+FF51
UNICODE_Ll, //U+FF52
UNICODE_Ll, //U+FF53
UNICODE_Ll, //U+FF54
UNICODE_Ll, //U+FF55
UNICODE_Ll, //U+FF56
UNICODE_Ll, //U+FF57
UNICODE_Ll, //U+FF58
UNICODE_Ll, //U+FF59
UNICODE_Ll, //U+FF5A
UNICODE_Ps, //U+FF5B
UNICODE_Sm, //U+FF5C
UNICODE_Pe, //U+FF5D
UNICODE_Sm, //U+FF5E
UNICODE_Ps, //U+FF5F
UNICODE_Pe, //U+FF60
UNICODE_Po, //U+FF61
UNICODE_Ps, //U+FF62
UNICODE_Pe, //U+FF63
UNICODE_Po, //U+FF64
UNICODE_Po, //U+FF65
UNICODE_Lo, //U+FF66
UNICODE_Lo, //U+FF67
UNICODE_Lo, //U+FF68
UNICODE_Lo, //U+FF69
UNICODE_Lo, //U+FF6A
UNICODE_Lo, //U+FF6B
UNICODE_Lo, //U+FF6C
UNICODE_Lo, //U+FF6D
UNICODE_Lo, //U+FF6E
UNICODE_Lo, //U+FF6F
UNICODE_Lm, //U+FF70
UNICODE_Lo, //U+FF71
UNICODE_Lo, //U+FF72
UNICODE_Lo, //U+FF73
UNICODE_Lo, //U+FF74
UNICODE_Lo, //U+FF75
UNICODE_Lo, //U+FF76
UNICODE_Lo, //U+FF77
UNICODE_Lo, //U+FF78
UNICODE_Lo, //U+FF79
UNICODE_Lo, //U+FF7A
UNICODE_Lo, //U+FF7B
UNICODE_Lo, //U+FF7C
UNICODE_Lo, //U+FF7D
UNICODE_Lo, //U+FF7E
UNICODE_Lo, //U+FF7F
UNICODE_Lo, //U+FF80
UNICODE_Lo, //U+FF81
UNICODE_Lo, //U+FF82
UNICODE_Lo, //U+FF83
UNICODE_Lo, //U+FF84
UNICODE_Lo, //U+FF85
UNICODE_Lo, //U+FF86
UNICODE_Lo, //U+FF87
UNICODE_Lo, //U+FF88
UNICODE_Lo, //U+FF89
UNICODE_Lo, //U+FF8A
UNICODE_Lo, //U+FF8B
UNICODE_Lo, //U+FF8C
UNICODE_Lo, //U+FF8D
UNICODE_Lo, //U+FF8E
UNICODE_Lo, //U+FF8F
UNICODE_Lo, //U+FF90
UNICODE_Lo, //U+FF91
UNICODE_Lo, //U+FF92
UNICODE_Lo, //U+FF93
UNICODE_Lo, //U+FF94
UNICODE_Lo, //U+FF95
UNICODE_Lo, //U+FF96
UNICODE_Lo, //U+FF97
UNICODE_Lo, //U+FF98
UNICODE_Lo, //U+FF99
UNICODE_Lo, //U+FF9A
UNICODE_Lo, //U+FF9B
UNICODE_Lo, //U+FF9C
UNICODE_Lo, //U+FF9D
UNICODE_Lm, //U+FF9E
UNICODE_Lm, //U+FF9F
UNICODE_Lo, //U+FFA0
UNICODE_Lo, //U+FFA1
UNICODE_Lo, //U+FFA2
UNICODE_Lo, //U+FFA3
UNICODE_Lo, //U+FFA4
UNICODE_Lo, //U+FFA5
UNICODE_Lo, //U+FFA6
UNICODE_Lo, //U+FFA7
UNICODE_Lo, //U+FFA8
UNICODE_Lo, //U+FFA9
UNICODE_Lo, //U+FFAA
UNICODE_Lo, //U+FFAB
UNICODE_Lo, //U+FFAC
UNICODE_Lo, //U+FFAD
UNICODE_Lo, //U+FFAE
UNICODE_Lo, //U+FFAF
UNICODE_Lo, //U+FFB0
UNICODE_Lo, //U+FFB1
UNICODE_Lo, //U+FFB2
UNICODE_Lo, //U+FFB3
UNICODE_Lo, //U+FFB4
UNICODE_Lo, //U+FFB5
UNICODE_Lo, //U+FFB6
UNICODE_Lo, //U+FFB7
UNICODE_Lo, //U+FFB8
UNICODE_Lo, //U+FFB9
UNICODE_Lo, //U+FFBA
UNICODE_Lo, //U+FFBB
UNICODE_Lo, //U+FFBC
UNICODE_Lo, //U+FFBD
UNICODE_Lo, //U+FFBE
0, //U+FFBF
0, //U+FFC0
0, //U+FFC1
UNICODE_Lo, //U+FFC2
UNICODE_Lo, //U+FFC3
UNICODE_Lo, //U+FFC4
UNICODE_Lo, //U+FFC5
UNICODE_Lo, //U+FFC6
UNICODE_Lo, //U+FFC7
0, //U+FFC8
0, //U+FFC9
UNICODE_Lo, //U+FFCA
UNICODE_Lo, //U+FFCB
UNICODE_Lo, //U+FFCC
UNICODE_Lo, //U+FFCD
UNICODE_Lo, //U+FFCE
UNICODE_Lo, //U+FFCF
0, //U+FFD0
0, //U+FFD1
UNICODE_Lo, //U+FFD2
UNICODE_Lo, //U+FFD3
UNICODE_Lo, //U+FFD4
UNICODE_Lo, //U+FFD5
UNICODE_Lo, //U+FFD6
UNICODE_Lo, //U+FFD7
0, //U+FFD8
0, //U+FFD9
UNICODE_Lo, //U+FFDA
UNICODE_Lo, //U+FFDB
UNICODE_Lo, //U+FFDC
0, //U+FFDD
0, //U+FFDE
0, //U+FFDF
UNICODE_Sc, //U+FFE0
UNICODE_Sc, //U+FFE1
UNICODE_Sm, //U+FFE2
UNICODE_Sk, //U+FFE3
UNICODE_So, //U+FFE4
UNICODE_Sc, //U+FFE5
UNICODE_Sc, //U+FFE6
0, //U+FFE7
UNICODE_So, //U+FFE8
UNICODE_Sm, //U+FFE9
UNICODE_Sm, //U+FFEA
UNICODE_Sm, //U+FFEB
UNICODE_Sm, //U+FFEC
UNICODE_So, //U+FFED
UNICODE_So, //U+FFEE
0, //U+FFEF
0, //U+FFF0
0, //U+FFF1
0, //U+FFF2
0, //U+FFF3
0, //U+FFF4
0, //U+FFF5
0, //U+FFF6
0, //U+FFF7
0, //U+FFF8
UNICODE_Cf, //U+FFF9
UNICODE_Cf, //U+FFFA
UNICODE_Cf, //U+FFFB
UNICODE_So, //U+FFFC
UNICODE_So, //U+FFFD
0, //U+FFFE
0 //U+FFFF
};


typedef struct {
  code_point cp1;
  code_point cp2;
  unsigned char category;
}codepoint_range_category;


static const codepoint_range_category codepoints_categories2[] = {
  { 0x10000, 0x1000B, UNICODE_Lo},
  { 0x1000D, 0x10026, UNICODE_Lo},
  { 0x10028, 0x1003A, UNICODE_Lo},
  { 0x1003C, 0x1003D, UNICODE_Lo},
  { 0x1003F, 0x1004D, UNICODE_Lo},
  { 0x10050, 0x1005D, UNICODE_Lo},
  { 0x10080, 0x100FA, UNICODE_Lo},
  { 0x10100, 0x10101, UNICODE_Po},
  { 0x10102, 0x10102, UNICODE_So},
  { 0x10107, 0x10133, UNICODE_No},
  { 0x10137, 0x1013F, UNICODE_So},
  { 0x10140, 0x10174, UNICODE_Nl},
  { 0x10175, 0x10178, UNICODE_No},
  { 0x10179, 0x10189, UNICODE_So},
  { 0x1018A, 0x1018A, UNICODE_No},
  { 0x10190, 0x1019B, UNICODE_So},
  { 0x101D0, 0x101FC, UNICODE_So},
  { 0x101FD, 0x101FD, UNICODE_Mn},
  { 0x10280, 0x1029C, UNICODE_Lo},
  { 0x102A0, 0x102D0, UNICODE_Lo},
  { 0x10300, 0x1031E, UNICODE_Lo},
  { 0x10320, 0x10323, UNICODE_No},
  { 0x10330, 0x10340, UNICODE_Lo},
  { 0x10341, 0x10341, UNICODE_Nl},
  { 0x10342, 0x10349, UNICODE_Lo},
  { 0x1034A, 0x1034A, UNICODE_Nl},
  { 0x10380, 0x1039D, UNICODE_Lo},
  { 0x1039F, 0x1039F, UNICODE_Po},
  { 0x103A0, 0x103C3, UNICODE_Lo},
  { 0x103C8, 0x103CF, UNICODE_Lo},
  { 0x103D0, 0x103D0, UNICODE_Po},
  { 0x103D1, 0x103D5, UNICODE_Nl},
  { 0x10400, 0x10427, UNICODE_Lu},
  { 0x10428, 0x1044F, UNICODE_Ll},
  { 0x10450, 0x1049D, UNICODE_Lo},
  { 0x104A0, 0x104A9, UNICODE_Nd},
  { 0x10800, 0x10805, UNICODE_Lo},
  { 0x10808, 0x10808, UNICODE_Lo},
  { 0x1080A, 0x10835, UNICODE_Lo},
  { 0x10837, 0x10838, UNICODE_Lo},
  { 0x1083C, 0x1083C, UNICODE_Lo},
  { 0x1083F, 0x10855, UNICODE_Lo},
  { 0x10857, 0x10857, UNICODE_Po},
  { 0x10858, 0x1085F, UNICODE_No},
  { 0x10900, 0x10915, UNICODE_Lo},
  { 0x10916, 0x1091B, UNICODE_No},
  { 0x1091F, 0x1091F, UNICODE_Po},
  { 0x10920, 0x10939, UNICODE_Lo},
  { 0x1093F, 0x1093F, UNICODE_Po},
  { 0x10A00, 0x10A00, UNICODE_Lo},
  { 0x10A01, 0x10A03, UNICODE_Mn},
  { 0x10A05, 0x10A06, UNICODE_Mn},
  { 0x10A0C, 0x10A0F, UNICODE_Mn},
  { 0x10A10, 0x10A13, UNICODE_Lo},
  { 0x10A15, 0x10A17, UNICODE_Lo},
  { 0x10A19, 0x10A33, UNICODE_Lo},
  { 0x10A38, 0x10A3A, UNICODE_Mn},
  { 0x10A3F, 0x10A3F, UNICODE_Mn},
  { 0x10A40, 0x10A47, UNICODE_No},
  { 0x10A50, 0x10A58, UNICODE_Po},
  { 0x10A60, 0x10A7C, UNICODE_Lo},
  { 0x10A7D, 0x10A7E, UNICODE_No},
  { 0x10A7F, 0x10A7F, UNICODE_Po},
  { 0x10B00, 0x10B35, UNICODE_Lo},
  { 0x10B39, 0x10B3F, UNICODE_Po},
  { 0x10B40, 0x10B55, UNICODE_Lo},
  { 0x10B58, 0x10B5F, UNICODE_No},
  { 0x10B60, 0x10B72, UNICODE_Lo},
  { 0x10B78, 0x10B7F, UNICODE_No},
  { 0x10C00, 0x10C48, UNICODE_Lo},
  { 0x10E60, 0x10E7E, UNICODE_No},
  { 0x11000, 0x11000, UNICODE_Mc},
  { 0x11001, 0x11001, UNICODE_Mn},
  { 0x11002, 0x11002, UNICODE_Mc},
  { 0x11003, 0x11037, UNICODE_Lo},
  { 0x11038, 0x11046, UNICODE_Mn},
  { 0x11047, 0x1104D, UNICODE_Po},
  { 0x11052, 0x11065, UNICODE_No},
  { 0x11066, 0x1106F, UNICODE_Nd},
  { 0x11080, 0x11081, UNICODE_Mn},
  { 0x11082, 0x11082, UNICODE_Mc},
  { 0x11083, 0x110AF, UNICODE_Lo},
  { 0x110B0, 0x110B2, UNICODE_Mc},
  { 0x110B3, 0x110B6, UNICODE_Mn},
  { 0x110B7, 0x110B8, UNICODE_Mc},
  { 0x110B9, 0x110BA, UNICODE_Mn},
  { 0x110BB, 0x110BC, UNICODE_Po},
  { 0x110BD, 0x110BD, UNICODE_Cf},
  { 0x110BE, 0x110C1, UNICODE_Po},
  { 0x12000, 0x1236E, UNICODE_Lo},
  { 0x12400, 0x12462, UNICODE_Nl},
  { 0x12470, 0x12473, UNICODE_Po},
  { 0x13000, 0x1342E, UNICODE_Lo},
  { 0x16800, 0x16A38, UNICODE_Lo},
  { 0x1B000, 0x1B001, UNICODE_Lo},
  { 0x1D000, 0x1D0F5, UNICODE_So},
  { 0x1D100, 0x1D126, UNICODE_So},
  { 0x1D129, 0x1D164, UNICODE_So},
  { 0x1D165, 0x1D166, UNICODE_Mc},
  { 0x1D167, 0x1D169, UNICODE_Mn},
  { 0x1D16A, 0x1D16C, UNICODE_So},
  { 0x1D16D, 0x1D172, UNICODE_Mc},
  { 0x1D173, 0x1D17A, UNICODE_Cf},
  { 0x1D17B, 0x1D182, UNICODE_Mn},
  { 0x1D183, 0x1D184, UNICODE_So},
  { 0x1D185, 0x1D18B, UNICODE_Mn},
  { 0x1D18C, 0x1D1A9, UNICODE_So},
  { 0x1D1AA, 0x1D1AD, UNICODE_Mn},
  { 0x1D1AE, 0x1D1DD, UNICODE_So},
  { 0x1D200, 0x1D241, UNICODE_So},
  { 0x1D242, 0x1D244, UNICODE_Mn},
  { 0x1D245, 0x1D245, UNICODE_So},
  { 0x1D300, 0x1D356, UNICODE_So},
  { 0x1D360, 0x1D371, UNICODE_No},
  { 0x1D400, 0x1D419, UNICODE_Lu},
  { 0x1D41A, 0x1D433, UNICODE_Ll},
  { 0x1D434, 0x1D44D, UNICODE_Lu},
  { 0x1D44E, 0x1D454, UNICODE_Ll},
  { 0x1D456, 0x1D467, UNICODE_Ll},
  { 0x1D468, 0x1D481, UNICODE_Lu},
  { 0x1D482, 0x1D49B, UNICODE_Ll},
  { 0x1D49C, 0x1D49C, UNICODE_Lu},
  { 0x1D49E, 0x1D49F, UNICODE_Lu},
  { 0x1D4A2, 0x1D4A2, UNICODE_Lu},
  { 0x1D4A5, 0x1D4A6, UNICODE_Lu},
  { 0x1D4A9, 0x1D4AC, UNICODE_Lu},
  { 0x1D4AE, 0x1D4B5, UNICODE_Lu},
  { 0x1D4B6, 0x1D4B9, UNICODE_Ll},
  { 0x1D4BB, 0x1D4BB, UNICODE_Ll},
  { 0x1D4BD, 0x1D4C3, UNICODE_Ll},
  { 0x1D4C5, 0x1D4CF, UNICODE_Ll},
  { 0x1D4D0, 0x1D4E9, UNICODE_Lu},
  { 0x1D4EA, 0x1D503, UNICODE_Ll},
  { 0x1D504, 0x1D505, UNICODE_Lu},
  { 0x1D507, 0x1D50A, UNICODE_Lu},
  { 0x1D50D, 0x1D514, UNICODE_Lu},
  { 0x1D516, 0x1D51C, UNICODE_Lu},
  { 0x1D51E, 0x1D537, UNICODE_Ll},
  { 0x1D538, 0x1D539, UNICODE_Lu},
  { 0x1D53B, 0x1D53E, UNICODE_Lu},
  { 0x1D540, 0x1D544, UNICODE_Lu},
  { 0x1D546, 0x1D546, UNICODE_Lu},
  { 0x1D54A, 0x1D550, UNICODE_Lu},
  { 0x1D552, 0x1D56B, UNICODE_Ll},
  { 0x1D56C, 0x1D585, UNICODE_Lu},
  { 0x1D586, 0x1D59F, UNICODE_Ll},
  { 0x1D5A0, 0x1D5B9, UNICODE_Lu},
  { 0x1D5BA, 0x1D5D3, UNICODE_Ll},
  { 0x1D5D4, 0x1D5ED, UNICODE_Lu},
  { 0x1D5EE, 0x1D607, UNICODE_Ll},
  { 0x1D608, 0x1D621, UNICODE_Lu},
  { 0x1D622, 0x1D63B, UNICODE_Ll},
  { 0x1D63C, 0x1D655, UNICODE_Lu},
  { 0x1D656, 0x1D66F, UNICODE_Ll},
  { 0x1D670, 0x1D689, UNICODE_Lu},
  { 0x1D68A, 0x1D6A5, UNICODE_Ll},
  { 0x1D6A8, 0x1D6C0, UNICODE_Lu},
  { 0x1D6C1, 0x1D6C1, UNICODE_Sm},
  { 0x1D6C2, 0x1D6DA, UNICODE_Ll},
  { 0x1D6DB, 0x1D6DB, UNICODE_Sm},
  { 0x1D6DC, 0x1D6E1, UNICODE_Ll},
  { 0x1D6E2, 0x1D6FA, UNICODE_Lu},
  { 0x1D6FB, 0x1D6FB, UNICODE_Sm},
  { 0x1D6FC, 0x1D714, UNICODE_Ll},
  { 0x1D715, 0x1D715, UNICODE_Sm},
  { 0x1D716, 0x1D71B, UNICODE_Ll},
  { 0x1D71C, 0x1D734, UNICODE_Lu},
  { 0x1D735, 0x1D735, UNICODE_Sm},
  { 0x1D736, 0x1D74E, UNICODE_Ll},
  { 0x1D74F, 0x1D74F, UNICODE_Sm},
  { 0x1D750, 0x1D755, UNICODE_Ll},
  { 0x1D756, 0x1D76E, UNICODE_Lu},
  { 0x1D76F, 0x1D76F, UNICODE_Sm},
  { 0x1D770, 0x1D788, UNICODE_Ll},
  { 0x1D789, 0x1D789, UNICODE_Sm},
  { 0x1D78A, 0x1D78F, UNICODE_Ll},
  { 0x1D790, 0x1D7A8, UNICODE_Lu},
  { 0x1D7A9, 0x1D7A9, UNICODE_Sm},
  { 0x1D7AA, 0x1D7C2, UNICODE_Ll},
  { 0x1D7C3, 0x1D7C3, UNICODE_Sm},
  { 0x1D7C4, 0x1D7C9, UNICODE_Ll},
  { 0x1D7CA, 0x1D7CA, UNICODE_Lu},
  { 0x1D7CB, 0x1D7CB, UNICODE_Ll},
  { 0x1D7CE, 0x1D7FF, UNICODE_Nd},
  { 0x1F000, 0x1F02B, UNICODE_So},
  { 0x1F030, 0x1F093, UNICODE_So},
  { 0x1F0A0, 0x1F0AE, UNICODE_So},
  { 0x1F0B1, 0x1F0BE, UNICODE_So},
  { 0x1F0C1, 0x1F0CF, UNICODE_So},
  { 0x1F0D1, 0x1F0DF, UNICODE_So},
  { 0x1F100, 0x1F10A, UNICODE_No},
  { 0x1F110, 0x1F12E, UNICODE_So},
  { 0x1F130, 0x1F169, UNICODE_So},
  { 0x1F170, 0x1F19A, UNICODE_So},
  { 0x1F1E6, 0x1F202, UNICODE_So},
  { 0x1F210, 0x1F23A, UNICODE_So},
  { 0x1F240, 0x1F248, UNICODE_So},
  { 0x1F250, 0x1F251, UNICODE_So},
  { 0x1F300, 0x1F320, UNICODE_So},
  { 0x1F330, 0x1F335, UNICODE_So},
  { 0x1F337, 0x1F37C, UNICODE_So},
  { 0x1F380, 0x1F393, UNICODE_So},
  { 0x1F3A0, 0x1F3C4, UNICODE_So},
  { 0x1F3C6, 0x1F3CA, UNICODE_So},
  { 0x1F3E0, 0x1F3F0, UNICODE_So},
  { 0x1F400, 0x1F43E, UNICODE_So},
  { 0x1F440, 0x1F440, UNICODE_So},
  { 0x1F442, 0x1F4F7, UNICODE_So},
  { 0x1F4F9, 0x1F4FC, UNICODE_So},
  { 0x1F500, 0x1F53D, UNICODE_So},
  { 0x1F550, 0x1F567, UNICODE_So},
  { 0x1F5FB, 0x1F5FF, UNICODE_So},
  { 0x1F601, 0x1F610, UNICODE_So},
  { 0x1F612, 0x1F614, UNICODE_So},
  { 0x1F616, 0x1F616, UNICODE_So},
  { 0x1F618, 0x1F618, UNICODE_So},
  { 0x1F61A, 0x1F61A, UNICODE_So},
  { 0x1F61C, 0x1F61E, UNICODE_So},
  { 0x1F620, 0x1F625, UNICODE_So},
  { 0x1F628, 0x1F62B, UNICODE_So},
  { 0x1F62D, 0x1F62D, UNICODE_So},
  { 0x1F630, 0x1F633, UNICODE_So},
  { 0x1F635, 0x1F640, UNICODE_So},
  { 0x1F645, 0x1F64F, UNICODE_So},
  { 0x1F680, 0x1F6C5, UNICODE_So},
  { 0x1F700, 0x1F773, UNICODE_So},
  { 0x20000, 0x20000, UNICODE_Lo},
  { 0x2A6D6, 0x2A6D6, UNICODE_Lo},
  { 0x2A700, 0x2A700, UNICODE_Lo},
  { 0x2B734, 0x2B734, UNICODE_Lo},
  { 0x2B740, 0x2B740, UNICODE_Lo},
  { 0x2B81D, 0x2B81D, UNICODE_Lo},
  { 0x2F800, 0x2FA1D, UNICODE_Lo},
  { 0xE0001, 0xE0001, UNICODE_Cf},
  { 0xE0020, 0xE007F, UNICODE_Cf},
  { 0xE0100, 0xE01EF, UNICODE_Mn},
  { 0xF0000, 0xF0000, UNICODE_Co},
  { 0xFFFFD, 0xFFFFD, UNICODE_Co},
  { 0x100000, 0x100000, UNICODE_Co},
};

bool check_codepoint_category(code_point cp, UnicodeCategoriesEnum categ)
{
  if(cp < 0x10000)
    return codepoints_categories[cp] == categ;
  unsigned int categ2_size = sizeof(codepoints_categories2)/sizeof(codepoint_range_category);
  for(unsigned int i=0;i<categ2_size;i++)
  {
    if(cp <= codepoints_categories2[i].cp2)
    {
      if(cp >= codepoints_categories2[i].cp1)
        return codepoints_categories2[i].category == categ;
      else
        return false;
    }
  }
  return false;
}

/*
typedef struct{
  code_point  cp1;
  code_point  cp2;
}codepoint_range_t;

typedef struct{
  code_point  cp1;
  code_point  cp2;
  unsigned char step;
}codepoint_range_steps_t;
*/
//Nd category, reference http://www.fileformat.info/info/unicode/category/Nd/list.htm
static code_point   unicode_Nd_zeros[] = 
{
  0x0030, //normal digit 0 - 9
  0x0660, //ARABIC-INDIC DIGIT ZERO
  0x06F0, //EXTENDED ARABIC-INDIC DIGIT ZERO
  0x07C0, //NKO DIGIT ZERO
  0x0966, //DEVANAGARI DIGIT ZERO
  0x09E6, //BENGALI DIGIT ZERO
  0x0A66, //GURMUKHI DIGIT ZERO
  0x0AE6, //GUJARATI DIGIT ZERO
  0x0B66, //ORIYA DIGIT ZERO
  0x0BE6, //TAMIL DIGIT ZERO
  0x0C66, //TELUGU DIGIT ZERO
  0x0CE6, //KANNADA DIGIT ZERO
  0x0D66, //MALAYALAM DIGIT ZERO
  0x0E50, //THAI DIGIT ZERO
  0x0ED0, //LAO DIGIT ZERO
  0x0F20, //TIBETAN DIGIT ZERO
  0x1040, //MYANMAR DIGIT ZERO
  0x1090, //MYANMAR SHAN DIGIT ZERO
  0x17E0, //KHMER DIGIT ZERO
  0x1810, //MONGOLIAN DIGIT ZERO
  0x1946, //LIMBU DIGIT ZERO
  0x19D0, //NEW TAI LUE DIGIT ZERO
  0x1A80, //TAI THAM HORA DIGIT ZERO
  0x1A90, //TAI THAM THAM DIGIT ZERO
  0x1B50, //BALINESE DIGIT ZERO
  0x1BB0, //SUNDANESE DIGIT ZERO
  0x1C40, //LEPCHA DIGIT ZERO
  0x1C50, //OL CHIKI DIGIT ZERO
  0xA620, //VAI DIGIT ZERO
  0xA8D0, //SAURASHTRA DIGIT ZERO
  0xA900, //KAYAH LI DIGIT ZERO
  0xA9D0, //JAVANESE DIGIT ZERO
  0xAA50, //CHAM DIGIT ZERO
  0xABF0, //MEETEI MAYEK DIGIT ZERO
  0xFF10, //FULLWIDTH DIGIT ZERO
  0x104A0, //OSMANYA DIGIT ZERO
  0x11066, //BRAHMI DIGIT ZERO
  0x1D7CE, //MATHEMATICAL BOLD DIGIT ZERO
  0x1D7D8, //MATHEMATICAL DOUBLE-STRUCK DIGIT ZERO
  0x1D7E2, //MATHEMATICAL SANS-SERIF DIGIT ZERO
  0x1D7EC, //MATHEMATICAL SANS-SERIF BOLD DIGIT ZERO
  0x1D7F6  //MATHEMATICAL MONOSPACE DIGIT ZERO
};

//check if code_point is a Unicode digit character
//if it is Nd, then ret_zero will point to the zero character for that digit range
bool is_UnicodeNd(code_point cp, code_point *ret_zero)
{
  int unicode_Nd_zeros_size = sizeof(unicode_Nd_zeros)/sizeof(code_point);
  for(int i=0;i<unicode_Nd_zeros_size;i++)
  {
    if(cp <= (unicode_Nd_zeros[i] + 9))
    {
      if(cp >= unicode_Nd_zeros[i])
      {
        if(ret_zero)
          *ret_zero = unicode_Nd_zeros[i];
        return true;
      }
      else
        return false;
    }
  }
  return false;
}

/*
//Nl category: Number letter, reference http://www.fileformat.info/info/unicode/category/Nl/list.htm
static codepoint_range_t    unicode_Nl[] =
{
  {0x16EE, 0x16F0}, //RUNIC SYMBOL
  {0x2160, 0x216F}, //ROMAN NUMERAL
  {0x2170, 0x217F}, //SMALL ROMAN NUMERAL 
  {0x2180, 0x2188}, //ROMAN NUMERAL 
  {0x3007, 0x3007}, //IDEOGRAPHIC NUMBER ZERO
  {0x3021, 0x303A}, //HANGZHOU NUMERAL 
  {0xA6E6, 0xA6EF}, //BAMUM LETTER 
  {0x10140, 0x10174}, //GREEK ACROPHONIC 
  {0x10341, 0x10341}, //GOTHIC LETTER NINETY
  {0x1034A, 0x1034A}, //GOTHIC LETTER NINE HUNDRED
  {0x103D1, 0x103D5}, //OLD PERSIAN NUMBER 
  {0x12400, 0x12462}  //CUNEIFORM NUMERIC SIGN 
};

//check if code_point is a Unicode numeric letter
bool is_UnicodeNl(code_point cp)
{
  int unicode_Nl_size = sizeof(unicode_Nl)/sizeof(codepoint_range_t);
  for(int i=0;i<unicode_Nl_size;i++)
  {
    if(cp <= unicode_Nl[i].cp2)
    {
      if(cp >= unicode_Nl[i].cp1)
        return true;
      else
        return false;
    }
  }
  return false;
}

//No category: Number Other, reference http://www.fileformat.info/info/unicode/category/No/list.htm
static codepoint_range_t    unicode_No[] = 
{
  {0x00B2, 0x00B3}, //SUPERSCRIPT TWO, THREE
  {0x00B9, 0x00B9}, //SUPERSCRIPT ONE
  {0x00BC, 0x00BE}, //VULGAR FRACTION 
  {0x09F4, 0x09F9}, //BENGALI CURRENCY NUMERATOR 
  {0x0B72, 0x0B77}, //ORIYA FRACTION 
  {0x0BF0, 0x0BF2}, //TAMIL NUMBER 
  {0x0C78, 0x0C7E}, //TELUGU FRACTION DIGIT 
  {0x0D70, 0x0D75}, //MALAYALAM FRACTION
  {0x0F2A, 0x0F33}, //TIBETAN DIGIT HALF 
  {0x1369, 0x137C}, //ETHIOPIC NUMBER 
  {0x17F0, 0x17F9}, //KHMER SYMBOL 
  {0x19DA, 0x19DA}, //NEW TAI LUE THAM DIGIT ONE
  {0x2070, 0x2070}, //SUPERSCRIPT ZERO
  {0x2074, 0x2079}, //SUPERSCRIPT
  {0x2080, 0x2089}, //SUBSCRIPT
  {0x2150, 0x215E}, //VULGAR FRACTION 
  {0x215F, 0x215F}, //FRACTION NUMERATOR ONE
  {0x2189, 0x2189}, //VULGAR FRACTION ZERO THIRDS
  {0x2460, 0x2473}, //CIRCLED NUMBER 
  {0x2474, 0x2487}, //PARENTHESIZED NUMBER 
  {0x2488, 0x249B}, //DIGIT NUMBER FULL STOP
  {0x24EA, 0x24EA}, //CIRCLED DIGIT ZERO
  {0x24EB, 0x24F4}, //NEGATIVE CIRCLED NUMBER 
  {0x24F5, 0x24FE}, //DOUBLE CIRCLED DIGIT 
  {0x24FF, 0x24FF}, //NEGATIVE CIRCLED DIGIT ZERO
  {0x2776, 0x2793}, //DINGBAT NEGATIVE CIRCLED SANS-SERIF NUMBER 
  {0x2CFD, 0x2CFD}, //COPTIC FRACTION ONE HALF
  {0x3192, 0x3195}, //IDEOGRAPHIC ANNOTATION MARK
  {0x3220, 0x3229}, //PARENTHESIZED IDEOGRAPH 
  {0x3251, 0x325F}, //CIRCLED NUMBER 
  {0x3280, 0x3289}, //CIRCLED IDEOGRAPH 
  {0x32B1, 0x32BF}, //CIRCLED NUMBER 
  {0xA830, 0xA835}, //NORTH INDIC FRACTION 
  {0x10107, 0x10133}, //AEGEAN NUMBER 
  {0x10175, 0x10178}, //GREEK SIGN
  {0x1018A, 0x1018A}, //GREEK ZERO SIGN
  {0x10320, 0x10323}, //OLD ITALIC NUMERAL 
  {0x10858, 0x1085F}, //IMPERIAL ARAMAIC NUMBER 
  {0x10916, 0x1091B}, //PHOENICIAN NUMBER 
  {0x10A40, 0x10A47}, //KHAROSHTHI DIGIT 
  {0x10A7D, 0x10A7E}, //OLD SOUTH ARABIAN NUMBER 
  {0x10B58, 0x10B5F}, //INSCRIPTIONAL PARTHIAN NUMBER 
  {0x10B78, 0x10B7F}, //INSCRIPTIONAL PAHLAVI NUMBER 
  {0x10E60, 0x10E7A}, //RUMI NUMBER 
  {0x10E7B, 0x10E7E}, //RUMI FRACTION 
  {0x11052, 0x11065}, //BRAHMI NUMBER 
  {0x1D360, 0x1D371}, //COUNTING ROD
  {0x1F100, 0x1F100}, //DIGIT ZERO FULL STOP
  {0x1F101, 0x1F10A}  //DIGIT COMMA
};

//check if code_point is a Unicode Numeric Other (No)
bool is_UnicodeNo(code_point cp)
{
  int unicode_No_size = sizeof(unicode_No)/sizeof(codepoint_range_t);
  for(int i=0;i<unicode_No_size;i++)
  {
    if(cp <= unicode_No[i].cp2)
    {
      if(cp >= unicode_No[i].cp1)
        return true;
      else
        return false;
    }
  }
  return false;
}

*/

}//end ns unicode
}//end ns zorba

/* vim:set et sw=2 ts=2: */
